@@ -57,11 +57,11 @@ def unpackData(fmt, data):
 	# If the format needs post-processing such as normalisation, do that now
 	if fmt.compType == rd.CompType.UNorm:
 		divisor = float((1 << fmt.compByteWidth) - 1)
-		value = tuple(float(value[i]) / divisor for i in value)
+		value = tuple(float(i) / divisor for i in value)
 	elif fmt.compType == rd.CompType.SNorm:
 		maxNeg = -(1 << (fmt.compByteWidth - 1))
 		divisor = float(-(maxNeg-1))
-		value = tuple((float(value[i]) if (value[i] == maxNeg) else (float(value[i]) / divisor)) for i in value)
+		value = tuple((float(i) if (i == maxNeg) else (float(i) / divisor)) for i in value)
 
 	# If the format is BGRA, swap the two components
 	if fmt.BGRAOrder():
@@ -265,7 +265,7 @@ def loadCapture(filename):
 		raise RuntimeError("Capture cannot be replayed")
 
 	# Initialise the replay
-	status,controller = cap.OpenCapture(None)
+	status,controller = cap.OpenCapture(rd.ReplayOptions(), None)
 
 	if status != rd.ReplayStatus.Succeeded:
 		raise RuntimeError("Couldn't initialise replay: " + str(status))
@@ -275,7 +275,11 @@ def loadCapture(filename):
 if 'pyrenderdoc' in globals():
 	pyrenderdoc.Replay().BlockInvoke(sampleCode)
 else:
-	cap,controller = loadCapture('test.rdc')
+	if len(sys.argv) <= 1:
+		print('Usage: python3 {} filename.rdc'.format(sys.argv[0]))
+		sys.exit(0)
+
+	cap,controller = loadCapture(sys.argv[1])
 
 	sampleCode(controller)
 
