@@ -50,7 +50,7 @@ def sampleCode(controller):
 	print("Pixel shader:")
 	print(controller.DisassembleShader(pipe, ps, target))
 
-	cbufferVars = controller.GetCBufferVariableContents(ps.resourceId, entry, 0, cb.resourceId, 0)
+	cbufferVars = controller.GetCBufferVariableContents(pipe, ps.resourceId, entry, 0, cb.resourceId, 0)
 
 	for v in cbufferVars:
 		printVar(v)
@@ -71,7 +71,7 @@ def loadCapture(filename):
 		raise RuntimeError("Capture cannot be replayed")
 
 	# Initialise the replay
-	status,controller = cap.OpenCapture(None)
+	status,controller = cap.OpenCapture(rd.ReplayOptions(), None)
 
 	if status != rd.ReplayStatus.Succeeded:
 		raise RuntimeError("Couldn't initialise replay: " + str(status))
@@ -81,7 +81,11 @@ def loadCapture(filename):
 if 'pyrenderdoc' in globals():
 	pyrenderdoc.Replay().BlockInvoke(sampleCode)
 else:
-	cap,controller = loadCapture('test.rdc')
+	if len(sys.argv) <= 1:
+		print('Usage: python3 {} filename.rdc'.format(sys.argv[0]))
+		sys.exit(0)
+
+	cap,controller = loadCapture(sys.argv[1])
 
 	sampleCode(controller)
 
