@@ -74,7 +74,7 @@ def loadCapture(filename):
 		raise RuntimeError("Capture cannot be replayed")
 
 	# Initialise the replay
-	status,controller = cap.OpenCapture(None)
+	status,controller = cap.OpenCapture(rd.ReplayOptions(), None)
 
 	if status != rd.ReplayStatus.Succeeded:
 		raise RuntimeError("Couldn't initialise replay: " + str(status))
@@ -84,7 +84,11 @@ def loadCapture(filename):
 if 'pyrenderdoc' in globals():
 	pyrenderdoc.Replay().BlockInvoke(sampleCode)
 else:
-	cap,controller = loadCapture('test.rdc')
+	if len(sys.argv) <= 1:
+		print('Usage: python3 {} filename.rdc'.format(sys.argv[0]))
+		sys.exit(0)
+
+	cap,controller = loadCapture(sys.argv[1])
 
 	sampleCode(controller)
 
