@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -46,6 +46,7 @@
 #include "Windows/Dialogs/LiveCapture.h"
 #include "Windows/Dialogs/SettingsDialog.h"
 #include "Windows/EventBrowser.h"
+#include "Windows/LogView.h"
 #include "Windows/MainWindow.h"
 #include "Windows/PerformanceCounterViewer.h"
 #include "Windows/PipelineState/PipelineStateViewer.h"
@@ -680,11 +681,13 @@ void CaptureContext::CleanMenu(QAction *action)
     delete menu;
 }
 
-void CaptureContext::LoadCapture(const rdcstr &captureFile, const rdcstr &origFilename,
-                                 bool temporary, bool local)
+void CaptureContext::LoadCapture(const rdcstr &captureFile, const ReplayOptions &opts,
+                                 const rdcstr &origFilename, bool temporary, bool local)
 {
   CloseCapture();
 
+  PointerTypeRegistry::Init();
+
   m_LoadInProgress = true;
 
   if(local)
@@ -695,9 +698,11 @@ void CaptureContext::LoadCapture(const rdcstr &captureFile, const rdcstr &origFi
 
   bool newCapture = (!temporary && !Config().RecentCaptureFiles.contains(origFilename));
 
-  LambdaThread *thread = new LambdaThread([this, captureFile, origFilename, temporary, local]() {
-    LoadCaptureThreaded(captureFile, origFilename, temporary, local);
-  });
+  LambdaThread *thread =
+      new LambdaThread([this, captureFile, opts, origFilename, temporary, local]() {
+        LoadCaptureThreaded(captureFile, opts, origFilename, temporary, local);
+      });
+  thread->setName(lit("LoadCapture"));
   thread->selfDelete(true);
   thread->start();
 
@@ -712,11 +717,16 @@ void CaptureContext::LoadCapture(const rdcstr &captureFile, const rdcstr &origFi
   ANALYTIC_ADDAVG(Performance.LoadTime, double(loadTimer.nsecsElapsed() * 1.0e-9));
 #endif
 
-  ANALYTIC_SET(CaptureFeatures.ShaderLinkage, m_APIProps.ShaderLinkage);
-  ANALYTIC_SET(CaptureFeatures.YUVTextures, m_APIProps.YUVTextures);
-  ANALYTIC_SET(CaptureFeatures.SparseResources, m_APIProps.SparseResources);
-  ANALYTIC_SET(CaptureFeatures.MultiGPU, m_APIProps.MultiGPU);
-  ANALYTIC_SET(CaptureFeatures.D3D12Bundle, m_APIProps.D3D12Bundle);
+  if(m_APIProps.ShaderLinkage)
+    ANALYTIC_SET(CaptureFeatures.ShaderLinkage, true);
+  if(m_APIProps.YUVTextures)
+    ANALYTIC_SET(CaptureFeatures.YUVTextures, true);
+  if(m_APIProps.SparseResources)
+    ANALYTIC_SET(CaptureFeatures.SparseResources, true);
+  if(m_APIProps.MultiGPU)
+    ANALYTIC_SET(CaptureFeatures.MultiGPU, true);
+  if(m_APIProps.D3D12Bundle)
+    ANALYTIC_SET(CaptureFeatures.D3D12Bundle, true);
 
   if(m_APIProps.vendor != GPUVendor::Unknown)
   {
@@ -729,6 +739,8 @@ void CaptureContext::LoadCapture(const rdcstr &captureFile, const rdcstr &origFi
   {
     m_CaptureTemporary = temporary;
 
+    BufferFormatter::Init(m_APIProps.pipelineType);
+
     m_CaptureMods = CaptureModifications::NoModifications;
 
     rdcarray<ICaptureViewer *> viewers(m_CaptureViewers);
@@ -769,8 +781,8 @@ float CaptureContext::UpdateLoadProgress()
   return val;
 }
 
-void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const QString &origFilename,
-                                         bool temporary, bool local)
+void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const ReplayOptions &opts,
+                                         const QString &origFilename, bool temporary, bool local)
 {
   m_CaptureFile = origFilename;
 
@@ -782,7 +794,7 @@ void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const QStri
   m_PostloadProgress = 0.0f;
 
   // this function call will block until the capture is either loaded, or there's some failure
-  m_Replay.OpenCapture(captureFile, [this](float p) { m_LoadProgress = p; });
+  m_Replay.OpenCapture(captureFile, opts, [this](float p) { m_LoadProgress = p; });
 
   // if the renderer isn't running, we hit a failure case so display an error message
   if(!m_Replay.IsRunning())
@@ -802,7 +814,7 @@ void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const QStri
 
   if(!temporary)
   {
-    AddRecentFile(Config().RecentCaptureFiles, origFilename, 10);
+    AddRecentFile(Config().RecentCaptureFiles, origFilename);
 
     Config().Save();
   }
@@ -820,6 +832,7 @@ void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const QStri
 
     m_APIProps = r->GetAPIProperties();
 
+    m_CustomEncodings = r->GetCustomShaderEncodings();
     m_TargetEncodings = r->GetTargetShaderEncodings();
 
     m_PostloadProgress = 0.2f;
@@ -841,22 +854,48 @@ void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const QStri
 #if defined(RENDERDOC_PLATFORM_WIN32)
     m_CurWinSystem = WindowingSystem::Win32;
 #elif defined(RENDERDOC_PLATFORM_LINUX)
-    m_CurWinSystem = WindowingSystem::Xlib;
+    m_CurWinSystem = WindowingSystem::Unknown;
 
-    // prefer XCB, if supported
-    for(WindowingSystem sys : m_WinSystems)
+    if(QGuiApplication::platformName() == lit("wayland"))
     {
-      if(sys == WindowingSystem::XCB)
+      // if we're using Wayland we must use wayland since we can't get any other surface type
+      for(WindowingSystem sys : m_WinSystems)
       {
-        m_CurWinSystem = WindowingSystem::XCB;
-        break;
+        if(sys == WindowingSystem::Wayland)
+        {
+          m_CurWinSystem = WindowingSystem::Wayland;
+          m_WaylandDisplay = (wl_display *)AccessWaylandPlatformInterface("display", NULL);
+          break;
+        }
+      }
+
+      if(m_CurWinSystem == WindowingSystem::Unknown)
+      {
+        RDDialog::critical(NULL, tr("No wayland support"),
+                           tr("Replay doesn't support Wayland surfaces - check you compiled this "
+                              "build of RenderDoc with Wayland support enabled."));
       }
     }
-
-    if(m_CurWinSystem == WindowingSystem::XCB)
-      m_XCBConnection = QX11Info::connection();
     else
-      m_X11Display = QX11Info::display();
+    {
+      m_CurWinSystem = WindowingSystem::Xlib;
+
+      // prefer XCB, if supported
+      for(WindowingSystem sys : m_WinSystems)
+      {
+        if(sys == WindowingSystem::XCB)
+        {
+          m_CurWinSystem = WindowingSystem::XCB;
+          break;
+        }
+      }
+
+      if(m_CurWinSystem == WindowingSystem::XCB)
+        m_XCBConnection = QX11Info::connection();
+      else
+        m_X11Display = QX11Info::display();
+    }
+
 #elif defined(RENDERDOC_PLATFORM_APPLE)
     m_CurWinSystem = WindowingSystem::MacOS;
 #endif
@@ -933,6 +972,15 @@ void CaptureContext::LoadCaptureThreaded(const QString &captureFile, const QStri
       bytebuf buf = access->GetSectionContents(idx);
       LoadNotes(QString::fromUtf8((const char *)buf.data(), buf.count()));
     }
+    QString driver = access->DriverName();
+    if(driver == lit("Image"))
+    {
+      ANALYTIC_SET(UIFeatures.ImageViewer, true);
+    }
+    else if(!driver.isEmpty())
+    {
+      ANALYTIC_ADDUNIQ(APIs, driver);
+    }
   }
 
   m_LoadInProgress = false;
@@ -957,6 +1005,8 @@ void CaptureContext::RecompressCapture()
   QString destFilename = GetCaptureFilename();
   QString tempFilename;
 
+  qint64 oldSize = QFile(destFilename).size();
+
   ICaptureFile *cap = NULL;
   ICaptureFile *tempCap = NULL;
 
@@ -1033,7 +1083,10 @@ void CaptureContext::RecompressCapture()
     if(tempCap)
       tempCap->Shutdown();
     if(!tempFilename.isEmpty())
+    {
+      m_MainWindow->RemoveRecentCapture(tempFilename);
       QFile::remove(tempFilename);
+    }
     return;
   }
 
@@ -1043,6 +1096,7 @@ void CaptureContext::RecompressCapture()
   LambdaThread *th = new LambdaThread([cap, destFilename, &progress]() {
     cap->Convert(destFilename.toUtf8().data(), "rdc", NULL, [&progress](float p) { progress = p; });
   });
+  th->setName(lit("RecompressCapture"));
   th->start();
   // wait a few ms before popping up a progress bar
   th->wait(500);
@@ -1091,7 +1145,19 @@ void CaptureContext::RecompressCapture()
   if(tempCap)
     tempCap->Shutdown();
   if(!tempFilename.isEmpty())
+  {
+    m_MainWindow->RemoveRecentCapture(tempFilename);
     QFile::remove(tempFilename);
+  }
+
+  qint64 newSize = QFile(m_CaptureFile).size();
+
+  RDDialog::information(
+      m_MainWindow, tr("Capture recompressed"),
+      tr("Compression successful, capture is %1% of the original size: %2 MB -> %3 MB")
+          .arg(double(newSize * 100) / double(oldSize), 0, 'f', 2)
+          .arg(double(oldSize) / (1024.0 * 1024.0), 0, 'f', 1)
+          .arg(double(newSize) / (1024.0 * 1024.0), 0, 'f', 1));
 }
 
 bool CaptureContext::SaveCaptureTo(const rdcstr &captureFile)
@@ -1152,7 +1218,10 @@ bool CaptureContext::SaveCaptureTo(const rdcstr &captureFile)
 
   // if it was a temporary capture, remove the old instnace
   if(m_CaptureTemporary)
+  {
+    m_MainWindow->RemoveRecentCapture(m_CaptureFile);
     QFile::remove(m_CaptureFile);
+  }
 
   // Update the filename, and mark that it's local and not temporary now.
   m_CaptureFile = captureFile;
@@ -1252,6 +1321,7 @@ bool CaptureContext::ImportCapture(const CaptureFileFormat &fmt, const rdcstr &i
                            [&progress](float p) { progress = 0.5f + p * 0.5f; });
     file->Shutdown();
   });
+  th->setName(lit("ImportCapture"));
   th->start();
   // wait a few ms before popping up a progress bar
   th->wait(500);
@@ -1327,6 +1397,7 @@ void CaptureContext::ExportCapture(const CaptureFileFormat &fmt, const rdcstr &e
     status = file->Convert(exportfile.c_str(), ext.toUtf8().data(), sdfile,
                            [&progress](float p) { progress = p; });
   });
+  th->setName(lit("ExportCapture"));
   th->start();
   // wait a few ms before popping up a progress bar
   th->wait(500);
@@ -1385,6 +1456,15 @@ void CaptureContext::SetRemoteHost(int hostIdx)
 void CaptureContext::RefreshUIStatus(const rdcarray<ICaptureViewer *> &exclude,
                                      bool updateSelectedEvent, bool updateEvent)
 {
+  // cache and assign pointer type IDs for any known pointer types in current shaders
+  for(ShaderStage stage : {ShaderStage::Vertex, ShaderStage::Hull, ShaderStage::Domain,
+                           ShaderStage::Geometry, ShaderStage::Pixel, ShaderStage::Compute})
+  {
+    const ShaderReflection *refl = m_CurPipelineState->GetShaderReflection(stage);
+    if(refl)
+      PointerTypeRegistry::CacheShader(refl);
+  }
+
   for(ICaptureViewer *viewer : m_CaptureViewers)
   {
     if(exclude.contains(viewer))
@@ -1760,10 +1840,26 @@ WindowingData CaptureContext::CreateWindowingData(QWidget *window)
 
 #elif defined(RENDERDOC_PLATFORM_LINUX)
 
-  if(m_CurWinSystem == WindowingSystem::XCB)
+  if(m_CurWinSystem == WindowingSystem::Wayland)
+  {
+    // we don't need this, we just need to force creation of a window handle
+    window->winId();
+    wl_surface *surface =
+        (wl_surface *)AccessWaylandPlatformInterface("surface", window->windowHandle());
+    return CreateWaylandWindowingData(m_WaylandDisplay, surface);
+  }
+  else if(m_CurWinSystem == WindowingSystem::XCB)
+  {
     return CreateXCBWindowingData(m_XCBConnection, (xcb_window_t)window->winId());
-  else
+  }
+  else if(m_CurWinSystem == WindowingSystem::Xlib)
+  {
     return CreateXlibWindowingData(m_X11Display, (Drawable)window->winId());
+  }
+  else
+  {
+    return CreateHeadlessWindowingData(1, 1);
+  }
 
 #elif defined(RENDERDOC_PLATFORM_APPLE)
 
@@ -1885,6 +1981,18 @@ IDebugMessageView *CaptureContext::GetDebugMessageView()
   return m_DebugMessageView;
 }
 
+IDiagnosticLogView *CaptureContext::GetDiagnosticLogView()
+{
+  if(m_DiagnosticLogView)
+    return m_DiagnosticLogView;
+
+  m_DiagnosticLogView = new LogView(*this, m_MainWindow);
+  m_DiagnosticLogView->setObjectName(lit("diagnosticLogView"));
+  setupDockWindow(m_DiagnosticLogView);
+
+  return m_DiagnosticLogView;
+}
+
 ICommentView *CaptureContext::GetCommentView()
 {
   if(m_CommentView)
@@ -1992,6 +2100,11 @@ void CaptureContext::ShowDebugMessageView()
   m_MainWindow->showDebugMessageView();
 }
 
+void CaptureContext::ShowDiagnosticLogView()
+{
+  m_MainWindow->showDiagnosticLogView();
+}
+
 void CaptureContext::ShowCommentView()
 {
   m_MainWindow->showCommentView();
@@ -2055,12 +2168,12 @@ IBufferViewer *CaptureContext::ViewBuffer(uint64_t byteOffset, uint64_t byteSize
   return viewer;
 }
 
-IBufferViewer *CaptureContext::ViewTextureAsBuffer(uint32_t arrayIdx, uint32_t mip, ResourceId id,
+IBufferViewer *CaptureContext::ViewTextureAsBuffer(ResourceId id, const Subresource &sub,
                                                    const rdcstr &format)
 {
   BufferViewer *viewer = new BufferViewer(*this, false, m_MainWindow);
 
-  viewer->ViewTexture(arrayIdx, mip, id, format);
+  viewer->ViewTexture(id, sub, format);
 
   return viewer;
 }
@@ -2111,6 +2224,10 @@ QWidget *CaptureContext::CreateBuiltinWindow(const rdcstr &objectName)
   {
     return GetDebugMessageView()->Widget();
   }
+  else if(objectName == "diagnosticLogView")
+  {
+    return GetDiagnosticLogView()->Widget();
+  }
   else if(objectName == "commentView")
   {
     return GetCommentView()->Widget();
@@ -2155,6 +2272,8 @@ void CaptureContext::BuiltinWindowClosed(QWidget *window)
     m_MeshPreview = NULL;
   else if(m_DebugMessageView && m_DebugMessageView->Widget() == window)
     m_DebugMessageView = NULL;
+  else if(m_DiagnosticLogView && m_DiagnosticLogView->Widget() == window)
+    m_DiagnosticLogView = NULL;
   else if(m_CommentView && m_CommentView->Widget() == window)
     m_CommentView = NULL;
   else if(m_StatisticsViewer && m_StatisticsViewer->Widget() == window)
