@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,7 @@
 #ifdef RENDERDOC_QT_COMPAT
 
 typedef rdcarray<rdcpair<rdcstr, QVariant> > ExtensionCallbackData;
-#define make_pyarg make_rdcpair<rdcstr, QVariant>
+#define make_pyarg rdcpair<rdcstr, QVariant>
 
 #endif
 
