@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,13 +25,14 @@
 #include <QApplication>
 #include <QDebug>
 #include <QDir>
+#include <QMutexLocker>
 #include <QStandardPaths>
 #include "Code/QRDUtils.h"
 #include "Styles/StyleData.h"
 #include "QRDInterface.h"
 
 template <>
-std::string DoStringise(const TimeUnit &el)
+rdcstr DoStringise(const TimeUnit &el)
 {
   BEGIN_ENUM_STRINGISE(TimeUnit)
   {
@@ -102,7 +103,7 @@ rdcstrpairs convertFromVariant(const QVariantMap &val)
   rdcstrpairs ret;
   for(const QString &k : val.keys())
   {
-    ret.push_back(make_rdcpair<rdcstr, rdcstr>(k, val[k].toString()));
+    ret.push_back({k, val[k].toString()});
   }
   return ret;
 }
@@ -194,35 +195,72 @@ void PersistantConfig::applyValues(const QVariantMap &values)
   RENAMED_SETTING(QVariantList, SPIRVDisassemblers, ShaderProcessors);
 }
 
-int PersistantConfig::RemoteHostCount()
+static QMutex RemoteHostLock;
+
+rdcarray<RemoteHost> PersistantConfig::GetRemoteHosts()
 {
-  return RemoteHosts.count();
+  QMutexLocker autolock(&RemoteHostLock);
+  return RemoteHostList;
 }
 
-RemoteHost *PersistantConfig::GetRemoteHost(int index)
+RemoteHost PersistantConfig::GetRemoteHost(const rdcstr &hostname)
 {
-  if(index < 0 || index >= RemoteHostCount())
-    return NULL;
-  return RemoteHosts[index];
+  RemoteHost ret;
+
+  {
+    QMutexLocker autolock(&RemoteHostLock);
+    for(size_t i = 0; i < RemoteHostList.size(); i++)
+    {
+      if(RemoteHostList[i].Hostname() == hostname)
+      {
+        ret = RemoteHostList[i];
+        break;
+      }
+    }
+  }
+
+  return ret;
 }
 
 void PersistantConfig::AddRemoteHost(RemoteHost host)
 {
-  RemoteHosts.push_back(new RemoteHost(host));
-}
+  if(!host.IsValid())
+    return;
 
-void PersistantConfig::AddAndroidHosts()
-{
-  QMap<rdcstr, RemoteHost *> oldHosts;
-  for(int i = RemoteHosts.count() - 1; i >= 0; i--)
+  QMutexLocker autolock(&RemoteHostLock);
+
+  // don't add duplicates
+  for(size_t i = 0; i < RemoteHostList.size(); i++)
   {
-    if(RemoteHosts[i]->IsADB())
+    if(RemoteHostList[i] == host)
     {
-      RemoteHost *host = RemoteHosts.takeAt(i);
-      oldHosts[host->hostname] = host;
+      RemoteHostList[i] = host;
+      return;
     }
   }
 
+  RemoteHostList.push_back(host);
+}
+
+void PersistantConfig::RemoveRemoteHost(RemoteHost host)
+{
+  if(!host.IsValid())
+    return;
+
+  QMutexLocker autolock(&RemoteHostLock);
+
+  for(size_t i = 0; i < RemoteHostList.size(); i++)
+  {
+    if(RemoteHostList[i] == host)
+    {
+      RemoteHostList.takeAt(i);
+      break;
+    }
+  }
+}
+
+void PersistantConfig::UpdateEnumeratedProtocolDevices()
+{
   QString androidSDKPath = (!Android_SDKPath.isEmpty() && QFile::exists(Android_SDKPath))
                                ? QString(Android_SDKPath)
                                : QString();
@@ -237,33 +275,55 @@ void PersistantConfig::AddAndroidHosts()
 
   SetConfigSetting("MaxConnectTimeout", QString::number(Android_MaxConnectTimeout));
 
-  rdcstr androidHosts;
-  RENDERDOC_EnumerateAndroidDevices(&androidHosts);
-  for(const QString &hostName :
-      QString(androidHosts).split(QLatin1Char(','), QString::SkipEmptyParts))
+  rdcarray<RemoteHost> enumeratedDevices;
+
+  rdcarray<rdcstr> protocols;
+  RENDERDOC_GetSupportedDeviceProtocols(&protocols);
+
+  for(const rdcstr &p : protocols)
   {
-    RemoteHost *host = NULL;
+    IDeviceProtocolController *protocol = RENDERDOC_GetDeviceProtocolController(p);
 
-    if(oldHosts.contains(hostName))
-      host = oldHosts.take(hostName);
-    else
-      host = new RemoteHost();
+    rdcarray<rdcstr> devices = protocol->GetDevices();
 
-    host->hostname = hostName;
-    rdcstr friendly;
-    RENDERDOC_GetAndroidFriendlyName(hostName.toUtf8().data(), friendly);
-    host->friendlyName = friendly;
+    for(const rdcstr &d : devices)
+    {
+      RemoteHost newhost(protocol->GetProtocolName() + "://" + d);
+      enumeratedDevices.push_back(newhost);
+    }
+  }
+
+  QMutexLocker autolock(&RemoteHostLock);
+
+  QMap<rdcstr, RemoteHost> oldHosts;
+
+  for(int i = RemoteHostList.count() - 1; i >= 0; i--)
+  {
+    if(RemoteHostList[i].Protocol())
+    {
+      RemoteHost host = RemoteHostList.takeAt(i);
+      oldHosts[host.Hostname()] = host;
+    }
+  }
+
+  for(RemoteHost host : enumeratedDevices)
+  {
+    // if we already had this host, use that one.
+    if(oldHosts.contains(host.Hostname()))
+      host = oldHosts.take(host.Hostname());
+
+    host.SetFriendlyName(host.Protocol()->GetFriendlyName(host.Hostname()));
     // Just a command to display in the GUI and allow Launch() to be called.
-    host->runCommand = lit("Automatically handled");
-    RemoteHosts.push_back(host);
+    host.SetRunCommand("Automatically handled");
+    RemoteHostList.push_back(host);
   }
 
   // delete any leftovers
-  QMapIterator<rdcstr, RemoteHost *> i(oldHosts);
+  QMutableMapIterator<rdcstr, RemoteHost> i(oldHosts);
   while(i.hasNext())
   {
     i.next();
-    delete i.value();
+    i.value().SetShutdown();
   }
 }
 
@@ -288,8 +348,6 @@ bool PersistantConfig::SetStyle()
 
 PersistantConfig::~PersistantConfig()
 {
-  for(RemoteHost *h : RemoteHosts)
-    delete h;
 }
 
 bool PersistantConfig::Load(const rdcstr &filename)
@@ -309,24 +367,36 @@ bool PersistantConfig::Load(const rdcstr &filename)
   RDDialog::DefaultBrowsePath = LastFileBrowsePath;
 
   // localhost should always be available as a remote host
-  bool foundLocalhost = false;
-
-  for(RemoteHost host : RemoteHostList)
   {
-    if(host.hostname.isEmpty())
-      continue;
+    QMutexLocker autolock(&RemoteHostLock);
 
-    RemoteHosts.push_back(new RemoteHost(host));
+    bool foundLocalhost = false;
 
-    if(host.IsLocalhost())
-      foundLocalhost = true;
-  }
+    rdcarray<RemoteHost> hosts;
+    hosts.swap(RemoteHostList);
 
-  if(!foundLocalhost)
-  {
-    RemoteHost *host = new RemoteHost();
-    host->hostname = "localhost";
-    RemoteHosts.insert(0, host);
+    for(RemoteHost host : hosts)
+    {
+      // skip invalid hosts
+      if(!host.IsValid())
+        continue;
+
+      // backwards compatibility - skip old adb hosts that were adb:
+      if(host.Hostname().find("adb:") >= 0 && host.Protocol() == NULL)
+        continue;
+
+      RemoteHostList.push_back(host);
+
+      if(host.IsLocalhost())
+        foundLocalhost = true;
+    }
+
+    if(!foundLocalhost)
+    {
+      RemoteHost host;
+      host.m_hostname = "localhost";
+      RemoteHostList.insert(0, host);
+    }
   }
 
   bool tools[arraydim<KnownShaderTool>()] = {};
@@ -427,17 +497,28 @@ bool PersistantConfig::Save()
   if(m_Filename.isEmpty())
     return true;
 
-  // update serialize list
-  RemoteHostList.clear();
-  for(RemoteHost *host : RemoteHosts)
-    RemoteHostList.push_back(*host);
-
   RENDERDOC_SetConfigSetting("Disassembly_FriendlyNaming", ShaderViewer_FriendlyNaming ? "1" : "0");
   RENDERDOC_SetConfigSetting("ExternalTool_RGPIntegration", ExternalTool_RGPIntegration ? "1" : "0");
 
   LastFileBrowsePath = RDDialog::DefaultBrowsePath;
 
-  return Serialize(m_Filename);
+  // truncate the lists to a maximum of 9 items, allow more to exist in memory
+  rdcarray<rdcstr> capFiles = RecentCaptureFiles;
+  rdcarray<rdcstr> capSettings = RecentCaptureSettings;
+
+  // the oldest items are first, so remove from there
+  while(RecentCaptureFiles.count() >= 10)
+    RecentCaptureFiles.erase(0);
+  while(RecentCaptureSettings.count() >= 10)
+    RecentCaptureSettings.erase(0);
+
+  bool ret = Serialize(m_Filename);
+
+  // restore lists
+  RecentCaptureFiles = capFiles;
+  RecentCaptureSettings = capSettings;
+
+  return ret;
 }
 
 void PersistantConfig::Close()
@@ -450,7 +531,12 @@ void PersistantConfig::SetupFormatting()
   Formatter::setParams(*this);
 }
 
-void AddRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file, int maxItems)
+void RemoveRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file)
+{
+  recentList.removeOne(QDir::cleanPath(file));
+}
+
+void AddRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file)
 {
   QDir dir(file);
   QString path = dir.canonicalPath();
@@ -461,17 +547,10 @@ void AddRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file, int maxItem
     return;
   }
 
-  if(!recentList.contains(path))
-  {
-    recentList.push_back(path);
-    if(recentList.count() >= maxItems)
-      recentList.erase(0);
-  }
-  else
-  {
+  if(recentList.contains(path))
     recentList.removeOne(path);
-    recentList.push_back(path);
-  }
+
+  recentList.push_back(path);
 }
 
 void PersistantConfig::SetConfigSetting(const rdcstr &name, const rdcstr &value)
@@ -557,6 +636,8 @@ rdcstr ShaderProcessingTool::DefaultArguments() const
     return "-D -g -V -o {output_file} {input_file} -S {glsl_stage4} -e {entry_point}";
   else if(tool == KnownShaderTool::spirv_as)
     return "-o {output_file} {input_file}";
+  else if(tool == KnownShaderTool::dxc)
+    return "-T {hlsl_stage2}_6_0 -E {entry_point} -Fo {output_file} {input_file} -spirv";
 
   return args;
 }
@@ -604,3 +685,32 @@ BugReport::operator QVariant() const
 
   return map;
 }
+
+ReplayOptions::ReplayOptions(const QVariant &var)
+{
+  QVariantMap map = var.toMap();
+
+  if(map.contains(lit("apiValidation")))
+    apiValidation = map[lit("apiValidation")].toBool();
+  if(map.contains(lit("forceGPUVendor")))
+    forceGPUVendor = (GPUVendor)map[lit("forceGPUVendor")].toUInt();
+  if(map.contains(lit("forceGPUDeviceID")))
+    forceGPUDeviceID = map[lit("forceGPUDeviceID")].toUInt();
+  if(map.contains(lit("forceGPUDriverName")))
+    forceGPUDriverName = map[lit("forceGPUDriverName")].toString();
+  if(map.contains(lit("optimisation")))
+    optimisation = (ReplayOptimisationLevel)map[lit("optimisation")].toUInt();
+}
+
+ReplayOptions::operator QVariant() const
+{
+  QVariantMap map;
+
+  map[lit("apiValidation")] = apiValidation;
+  map[lit("forceGPUVendor")] = (uint32_t)forceGPUVendor;
+  map[lit("forceGPUDeviceID")] = forceGPUDeviceID;
+  map[lit("forceGPUDriverName")] = forceGPUDriverName;
+  map[lit("optimisation")] = (uint32_t)optimisation;
+
+  return map;
+}
