@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,6 +28,8 @@
 #include "QRDInterface.h"
 #include "RemoteHost.h"
 
+class QMutex;
+
 DOCUMENT(R"(Identifies a particular known tool used for shader processing.
 
 .. data:: Unknown
@@ -54,6 +56,10 @@ DOCUMENT(R"(Identifies a particular known tool used for shader processing.
 
   `spirv-as from SPIRV-Tools <https://github.com/KhronosGroup/SPIRV-Tools>`_.
 
+.. data:: dxc
+
+  `DirectX Shader Compiler <https://github.com/microsoft/DirectXShaderCompiler>`_.
+
 )");
 enum class KnownShaderTool : uint32_t
 {
@@ -64,6 +70,7 @@ enum class KnownShaderTool : uint32_t
   glslangValidatorGLSL,
   glslangValidatorHLSL,
   spirv_as,
+  dxc,
   Count,
 };
 
@@ -81,6 +88,8 @@ inline rdcstr ToolExecutable(KnownShaderTool tool)
     return "glslangValidator";
   else if(tool == KnownShaderTool::spirv_as)
     return "spirv-as";
+  else if(tool == KnownShaderTool::dxc)
+    return "dxc";
 
   return "";
 }
@@ -95,6 +104,8 @@ inline ShaderEncoding ToolInput(KnownShaderTool tool)
     return ShaderEncoding::HLSL;
   else if(tool == KnownShaderTool::spirv_as)
     return ShaderEncoding::SPIRVAsm;
+  else if(tool == KnownShaderTool::dxc)
+    return ShaderEncoding::HLSL;
 
   return ShaderEncoding::Unknown;
 }
@@ -106,7 +117,8 @@ inline ShaderEncoding ToolOutput(KnownShaderTool tool)
   else if(tool == KnownShaderTool::spirv_dis)
     return ShaderEncoding::SPIRVAsm;
   else if(tool == KnownShaderTool::glslangValidatorGLSL ||
-          tool == KnownShaderTool::glslangValidatorHLSL || tool == KnownShaderTool::spirv_as)
+          tool == KnownShaderTool::glslangValidatorHLSL || tool == KnownShaderTool::spirv_as ||
+          tool == KnownShaderTool::dxc)
     return ShaderEncoding::SPIRV;
 
   return ShaderEncoding::Unknown;
@@ -282,6 +294,8 @@ DECLARE_REFLECTION_STRUCT(BugReport);
                                                                                            \
   CONFIG_SETTING_VAL(public, QString, rdcstr, DefaultCaptureSaveDirectory, "")             \
                                                                                            \
+  CONFIG_SETTING(public, QVariant, ReplayOptions, DefaultReplayOptions)                    \
+                                                                                           \
   CONFIG_SETTING_VAL(public, bool, bool, TextureViewer_ResetRange, false)                  \
                                                                                            \
   CONFIG_SETTING_VAL(public, bool, bool, TextureViewer_PerTexSettings, true)               \
@@ -294,6 +308,8 @@ DECLARE_REFLECTION_STRUCT(BugReport);
                                                                                            \
   CONFIG_SETTING_VAL(public, int, int, LocalProxyAPI, -1)                                  \
                                                                                            \
+  CONFIG_SETTING_VAL(public, bool, bool, BufferFormatter_ShowHelp, true)                   \
+                                                                                           \
   CONFIG_SETTING_VAL(public, int, TimeUnit, EventBrowser_TimeUnit, TimeUnit::Microseconds) \
                                                                                            \
   CONFIG_SETTING_VAL(public, bool, bool, EventBrowser_AddFake, true)                       \
@@ -331,6 +347,8 @@ DECLARE_REFLECTION_STRUCT(BugReport);
                                                                                            \
   CONFIG_SETTING_VAL(public, bool, bool, CheckUpdate_UpdateAvailable, false)               \
                                                                                            \
+  CONFIG_SETTING_VAL(public, QString, rdcstr, CheckUpdate_CurrentVersion, "")              \
+                                                                                           \
   CONFIG_SETTING_VAL(public, QString, rdcstr, CheckUpdate_UpdateResponse, "")              \
                                                                                            \
   CONFIG_SETTING_VAL(public, QDateTime, rdcdatetime, CheckUpdate_LastUpdate,               \
@@ -419,16 +437,24 @@ inline rdcstr UnitSuffix(TimeUnit unit)
 }
 
 DOCUMENT(R"(Checks if a given file is in a list. If it is, then it's shuffled to the end. If it's
-not then it's added to the end and an item is dropped from the front of the list if necessary to
-stay within a given maximum
+not then it's added to the end
 
 As the name suggests, this is used for tracking a 'recent file' list.
 
 :param list recentList: A ``list`` of ``str`` that is mutated by the function.
 :param str file: The file to add to the list.
-:param int maxItems: The maximum allowed length of the list.
 )");
-void AddRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file, int maxItems);
+void AddRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file);
+
+DOCUMENT(R"(Removes a given file from the list, after normalising the path. If the path isn't
+present then the list is not modified.
+
+As the name suggests, this is used for tracking a 'recent file' list.
+
+:param list recentList: A ``list`` of ``str`` that is mutated by the function.
+:param str file: The file to remove from the list.
+)");
+void RemoveRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file);
 
 DOCUMENT2(R"(A persistant config file that is automatically loaded and saved, which contains any
 settings and information that needs to be preserved from one run to the next.
@@ -477,6 +503,11 @@ For more information about some of these settings that are user-facing see
 
   The default path to save captures in, when browsing to save a temporary capture somewhere.
 
+.. data:: DefaultReplayOptions
+
+  A :class:`ReplayOptions` containing the configured default replay options to use in most scenarios
+  when no specific options are given.
+
 .. data:: TextureViewer_ResetRange
 
   ``True`` if the :class:`TextureViewer` should reset the visible range when a new texture is
@@ -522,6 +553,12 @@ For more information about some of these settings that are user-facing see
 
   Defaults to ``-1``.
 
+.. data:: BufferFormatter_ShowHelp
+
+  ``True`` if the buffer formatter's help section should be shown.
+
+  Defaults to ``True``.
+
 .. data:: EventBrowser_TimeUnit
 
   The :class:`TimeUnit` to use to display the duration column in the :class:`EventBrowser`.
@@ -655,6 +692,11 @@ For more information about some of these settings that are user-facing see
 
   Defaults to ``False``.
 
+.. data:: CheckUpdate_CurrentVersion
+
+  The current version at the time of update checks. Used to determine if a cached pending update is
+  no longer valid because we got updated through some other method.
+
 .. data:: CheckUpdate_UpdateResponse
 
   Contains the response from the update server from the last update check, with any release notes
@@ -749,40 +791,31 @@ For more information about some of these settings that are user-facing see
 class PersistantConfig
 {
 public:
-// don't allow SWIG direct access to the RemoteHosts since they're an array of references and our
-// bindings can't handle that properly
-#if !defined(SWIG)
-  // Runtime list of dynamically allocated hosts.
-  // Saved to/from private RemoteHostList in CONFIG_SETTINGS()
-  // This is documented above in the docstring, similar to the values in CONFIG_SETTINGS()
-  // This must only be accessed on the UI thread to prevent races. For access on other threads (e.g.
-  // a background/asynchronous update), take a copy on the UI thread, update it in the background,
-  // then apply the updates.
-  DOCUMENT("");
-  rdcarray<RemoteHost *> RemoteHosts;
-#endif
+  DOCUMENT(R"(Returns a list of all remote hosts.
 
-  DOCUMENT(R"(Returns the number of remote hosts currently registered.
-
-:return: The number of remote hosts.
-:rtype: ``int``
+:return: The remote host list
+:rtype: ``list`` of ``RemoteHost``
 )");
-  int RemoteHostCount();
-  DOCUMENT(R"(Returns a given remote host at an index.
+  rdcarray<RemoteHost> GetRemoteHosts();
+  DOCUMENT(R"(Look up a remote host by hostname.
 
-:param int index: The index of the remote host to retrieve
-:return: The remote host specified, or ``None`` if an invalid index was passed
+:return: The remote host for the given hostname, or an invalid ``RemoteHost`` if no such exists.
 :rtype: ``RemoteHost``
 )");
-  RemoteHost *GetRemoteHost(int index);
+  RemoteHost GetRemoteHost(const rdcstr &hostname);
 
   DOCUMENT(R"(Adds a new remote host.
 
 :param RemoteHost host: The remote host to add.
-R)");
+)");
   void AddRemoteHost(RemoteHost host);
-  DOCUMENT("If configured, queries ``adb`` to add android hosts to :data:`RemoteHosts`.");
-  void AddAndroidHosts();
+  DOCUMENT(R"(Removes an existing remote host.
+
+:param RemoteHost host: The remote host to remove.
+)");
+  void RemoveRemoteHost(RemoteHost host);
+  DOCUMENT("If configured, queries available device protocols to update auto-configured hosts.");
+  void UpdateEnumeratedProtocolDevices();
 
   DOCUMENT("");
   CONFIG_SETTINGS()
