@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -85,6 +85,9 @@ class QMenu;
 // we depend on the internal RenderDoc API, but the bindings for that are imported entirely
 #include "renderdoc_replay.h"
 
+typedef rdcpair<rdcstr, rdcstr> rdcstrpair;
+typedef rdcarray<rdcstrpair> rdcstrpairs;
+
 struct ICaptureContext;
 
 #include "Analytics.h"
@@ -295,13 +298,11 @@ struct IBufferViewer
                           const rdcstr &format = "") = 0;
   DOCUMENT(R"(In a raw buffer viewer, load the contents from a particular texture resource.
 
-:param int arrayIdx: The array slice to load from.
-:param int mip: The mip level to load from.
 :param ~renderdoc.ResourceId id: The ID of the texture itself.
+:param Subresource sub: The subresource within this texture to use.
 :param str format: Optionally a HLSL/GLSL style formatting string.
 )");
-  virtual void ViewTexture(uint32_t arrayIdx, uint32_t mip, ResourceId id,
-                           const rdcstr &format = "") = 0;
+  virtual void ViewTexture(ResourceId id, const Subresource &sub, const rdcstr &format = "") = 0;
 
 protected:
   IBufferViewer() = default;
@@ -441,6 +442,22 @@ protected:
 
 DECLARE_REFLECTION_STRUCT(IDebugMessageView);
 
+DOCUMENT("The diagnostic log viewing window.");
+struct IDiagnosticLogView
+{
+  DOCUMENT(
+      "Retrieves the QWidget for this :class:`DiagnosticLogView` if PySide2 is available, or "
+      "otherwise unique opaque pointer that can be passed to RenderDoc functions expecting a "
+      "QWidget.");
+  virtual QWidget *Widget() = 0;
+
+protected:
+  IDiagnosticLogView() = default;
+  ~IDiagnosticLogView() = default;
+};
+
+DECLARE_REFLECTION_STRUCT(IDiagnosticLogView);
+
 DOCUMENT("The capture comments window.");
 struct ICommentView
 {
@@ -532,7 +549,7 @@ DECLARE_REFLECTION_STRUCT(IPythonShell);
 
 DOCUMENT(R"(A shader window used for viewing, editing, or debugging.
 
-.. function:: SaveCallback(context, viewer, files)
+.. function:: SaveCallback(context, viewer, encoding, flags, entry, compiled)
 
   Not a member function - the signature for any ``SaveCallback`` callbacks.
 
@@ -719,7 +736,7 @@ struct IReplayManager
 :return: Whether or not the connection was successful.
 :rtype: ~renderdoc.ReplayStatus
 )");
-  virtual ReplayStatus ConnectToRemoteServer(RemoteHost *host) = 0;
+  virtual ReplayStatus ConnectToRemoteServer(RemoteHost host) = 0;
 
   DOCUMENT("Disconnect from the server the manager is currently connected to.");
   virtual void DisconnectFromRemoteServer() = 0;
@@ -735,10 +752,10 @@ struct IReplayManager
 
   DOCUMENT(R"(Retrieves the host that the manager is currently connected to.
 
-:return: The host connected to, or ``None`` if no connection is active.
+:return: The host connected to, or an invalid RemoteHost if no connection is active.
 :rtype: RemoteHost
 )");
-  virtual RemoteHost *CurrentRemote() = 0;
+  virtual RemoteHost CurrentRemote() = 0;
 
   DOCUMENT(R"(Retrieves the capture file handle for the currently open file.
 
@@ -1067,13 +1084,14 @@ data.
   DOCUMENT(R"(Open a capture file for replay.
 
 :param str captureFile: The actual path to the capture file.
+:param ReplayOptions opts: The options controlling how the capture should be replayed.
 :param str origFilename: The original filename, if the capture was copied remotely for replay.
 :param bool temporary: ``True`` if this is a temporary capture which should prompt the user for
   either save or delete on close.
 :param bool local: ``True`` if ``captureFile`` refers to a file on the local machine.
 )");
-  virtual void LoadCapture(const rdcstr &captureFile, const rdcstr &origFilename, bool temporary,
-                           bool local) = 0;
+  virtual void LoadCapture(const rdcstr &captureFile, const ReplayOptions &opts,
+                           const rdcstr &origFilename, bool temporary, bool local) = 0;
 
   DOCUMENT(R"(Saves the current capture file to a given path.
 
@@ -1228,6 +1246,15 @@ building target shaders for the currently loaded capture. See
 )");
   virtual rdcarray<ShaderEncoding> TargetShaderEncodings() = 0;
 
+  DOCUMENT(R"(Retrieve the list of :class:`~renderdoc.ShaderEncoding` that are available for
+building custom shaders for the currently loaded capture. See
+:meth:`~renderdoc.ReplayController.BuildCustomShader`.
+
+:return: The available encodings.
+:rtype: ``list`` of :class:`~renderdoc.ShaderEncoding`
+)");
+  virtual rdcarray<ShaderEncoding> CustomShaderEncodings() = 0;
+
   DOCUMENT(R"(Retrieve the currently selected :data:`eventId <renderdoc.APIEvent.eventId>`.
 
 In most cases, prefer using :meth:`CurEvent`. See :meth:`CaptureViewer.OnSelectedEventChanged` for more
@@ -1588,6 +1615,13 @@ If no bookmark exists, this function will do nothing.
 )");
   virtual IDebugMessageView *GetDebugMessageView() = 0;
 
+  DOCUMENT(R"(Retrieve the current singleton :class:`LogView`.
+
+:return: The current window, which is created (but not shown) it there wasn't one open.
+:rtype: LogView
+)");
+  virtual IDiagnosticLogView *GetDiagnosticLogView() = 0;
+
   DOCUMENT(R"(Retrieve the current singleton :class:`CommentView`.
 
 :return: The current window, which is created (but not shown) it there wasn't one open.
@@ -1679,6 +1713,13 @@ If no bookmark exists, this function will do nothing.
 )");
   virtual bool HasDebugMessageView() = 0;
 
+  DOCUMENT(R"(Check if there is a current :class:`DiagnosticLogView` open.
+
+:return: ``True`` if there is a window open.
+:rtype: ``bool``
+)");
+  virtual bool HasDiagnosticLogView() = 0;
+
   DOCUMENT(R"(Check if there is a current :class:`CommentView` open.
 
 :return: ``True`` if there is a window open.
@@ -1739,6 +1780,9 @@ place if needed.
   DOCUMENT(
       "Raise the current :class:`DebugMessageView`, showing it in the default place if needed.");
   virtual void ShowDebugMessageView() = 0;
+  DOCUMENT(
+      "Raise the current :class:`DiagnosticLogView`, showing it in the default place if needed.");
+  virtual void ShowDiagnosticLogView() = 0;
   DOCUMENT("Raise the current :class:`CommentView`, showing it in the default place if needed.");
   virtual void ShowCommentView() = 0;
   DOCUMENT(
@@ -1820,14 +1864,13 @@ through the execution of a given shader.
   DOCUMENT(R"(Show a new :class:`BufferViewer` window, showing a read-only view of a texture's raw
 bytes.
 
-:param int arrayIdx: The array slice to load from.
-:param int mip: The mip level to load from.
 :param ~renderdoc.ResourceId id: The ID of the texture itself.
+:param Subresource sub: The subresource within this texture to use.
 :param str format: Optionally a HLSL/GLSL style formatting string.
 :return: The new :class:`BufferViewer` window opened, but not shown.
 :rtype: BufferViewer
 )");
-  virtual IBufferViewer *ViewTextureAsBuffer(uint32_t arrayIdx, uint32_t mip, ResourceId id,
+  virtual IBufferViewer *ViewTextureAsBuffer(ResourceId id, const Subresource &sub,
                                              const rdcstr &format = "") = 0;
 
   DOCUMENT(R"(Show a new :class:`ConstantBufferPreviewer` window, showing a read-only view of a the
