@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,16 +28,16 @@
 #include "Code/QRDUtils.h"
 #include "QRDInterface.h"
 
-static const QString glsl_stage4[ENUM_ARRAY_SIZE(ShaderStage)] = {
+static const QString glsl_stage4[arraydim<ShaderStage>()] = {
     lit("vert"), lit("tesc"), lit("tese"), lit("geom"), lit("frag"), lit("comp"),
 };
 
-static const QString hlsl_stage2[ENUM_ARRAY_SIZE(ShaderStage)] = {
+static const QString hlsl_stage2[arraydim<ShaderStage>()] = {
     lit("vs"), lit("hs"), lit("ds"), lit("gs"), lit("ps"), lit("cs"),
 };
 
 template <>
-std::string DoStringise(const KnownShaderTool &el)
+rdcstr DoStringise(const KnownShaderTool &el)
 {
   BEGIN_ENUM_STRINGISE(KnownShaderTool);
   {
@@ -47,6 +47,7 @@ std::string DoStringise(const KnownShaderTool &el)
     STRINGISE_ENUM_CLASS_NAMED(glslangValidatorGLSL, "glslang (GLSL)");
     STRINGISE_ENUM_CLASS_NAMED(glslangValidatorHLSL, "glslang (HLSL)");
     STRINGISE_ENUM_CLASS_NAMED(spirv_as, "spirv-as");
+    STRINGISE_ENUM_CLASS_NAMED(dxc, "dxc");
   }
   END_ENUM_STRINGISE();
 }
@@ -157,6 +158,7 @@ static ShaderToolOutput RunTool(const ShaderProcessingTool &tool, QWidget *windo
     process.moveToThread(mainThread);
   });
 
+  thread->setName(lit("ShaderProcessingTool %s").arg(tool.name));
   thread->moveObjectToThread(&process);
 
   thread->start();
@@ -211,10 +213,12 @@ ShaderToolOutput ShaderProcessingTool::DisassembleShader(QWidget *window,
       arg = input_file;
     if(arg == lit("{output_file}"))
       arg = output_file = tmpPath(lit("shader_output"));
-    if(arg == lit("{glsl_stage4}"))
-      arg = glsl_stage4[int(shaderDetails->stage)];
-    if(arg == lit("{hlsl_stage2}"))
-      arg = hlsl_stage2[int(shaderDetails->stage)];
+
+    // allow substring matches from the left, to enable e.g. {hlsl_stage2}_6_0
+    if(arg.left(13) == lit("{glsl_stage4}"))
+      arg.replace(0, 13, glsl_stage4[int(shaderDetails->stage)]);
+    if(arg.left(13) == lit("{hlsl_stage2}"))
+      arg.replace(0, 13, hlsl_stage2[int(shaderDetails->stage)]);
   }
 
   QFile binHandle(input_file);
@@ -257,10 +261,12 @@ ShaderToolOutput ShaderProcessingTool::CompileShader(QWidget *window, rdcstr sou
       arg = output_file = tmpPath(lit("shader_output"));
     if(arg == lit("{entry_point}"))
       arg = entryPoint;
-    if(arg == lit("{glsl_stage4}"))
-      arg = glsl_stage4[int(stage)];
-    if(arg == lit("{hlsl_stage2}"))
-      arg = hlsl_stage2[int(stage)];
+
+    // allow substring matches from the left, to enable e.g. {hlsl_stage2}_6_0
+    if(arg.left(13) == lit("{glsl_stage4}"))
+      arg.replace(0, 13, glsl_stage4[int(stage)]);
+    if(arg.left(13) == lit("{hlsl_stage2}"))
+      arg.replace(0, 13, hlsl_stage2[int(stage)]);
   }
 
   QFile binHandle(input_file);
