@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -55,18 +55,108 @@
 // so we implement it here using QString. It's inefficient, but this is a very uncommon path -
 // either for invalid values or for when a new enum is added and the code isn't updated
 template <>
-std::string DoStringise(const uint32_t &el)
+rdcstr DoStringise(const uint32_t &el)
 {
-  return QString::number(el).toStdString();
+  return QString::number(el);
 }
 
-// this one we do by hand as it requires formatting
+// these ones we do by hand as it requires formatting
 template <>
-std::string DoStringise(const ResourceId &el)
+rdcstr DoStringise(const ResourceId &el)
 {
   uint64_t num;
   memcpy(&num, &el, sizeof(num));
-  return lit("ResourceId::%1").arg(num).toStdString();
+  return lit("ResourceId::%1").arg(num);
+}
+
+QMap<QPair<ResourceId, uint32_t>, uint32_t> PointerTypeRegistry::typeMapping;
+rdcarray<ShaderVariableType> PointerTypeRegistry::typeDescriptions;
+
+static const uint32_t TypeIDBit = 0x80000000;
+
+void PointerTypeRegistry::Init()
+{
+  typeMapping.clear();
+
+  // type ID 0 is reserved as a NULL/empty descriptor
+  typeDescriptions.resize(1);
+  typeDescriptions[0].descriptor.name = "<Unknown>";
+}
+
+uint32_t PointerTypeRegistry::GetTypeID(ResourceId shader, uint32_t pointerTypeId)
+{
+  return typeMapping[qMakePair(shader, pointerTypeId)];
+}
+
+uint32_t PointerTypeRegistry::GetTypeID(const ShaderVariableType &structDef)
+{
+  // see if the type is already registered, return its existing ID
+  for(uint32_t i = 1; i < typeDescriptions.size(); i++)
+  {
+    if(structDef == typeDescriptions[i])
+      return TypeIDBit | i;
+  }
+
+  uint32_t id = TypeIDBit | (uint32_t)typeDescriptions.size();
+
+  // otherwise register the new type
+  typeDescriptions.push_back(structDef);
+  typeMapping[qMakePair(ResourceId(), id)] = id;
+
+  return id;
+}
+
+const ShaderVariableType &PointerTypeRegistry::GetTypeDescriptor(uint32_t typeId)
+{
+  return typeDescriptions[typeId & ~TypeIDBit];
+}
+
+void PointerTypeRegistry::CacheSubTypes(const ShaderReflection *reflection,
+                                        ShaderVariableType &structDef)
+{
+  if((structDef.descriptor.pointerTypeID & TypeIDBit) == 0)
+    structDef.descriptor.pointerTypeID =
+        PointerTypeRegistry::GetTypeID(reflection->pointerTypes[structDef.descriptor.pointerTypeID]);
+
+  for(ShaderConstant &member : structDef.members)
+    CacheSubTypes(reflection, member.type);
+}
+
+void PointerTypeRegistry::CacheShader(const ShaderReflection *reflection)
+{
+  // nothing to do if there are no pointer types
+  if(reflection->pointerTypes.isEmpty())
+    return;
+
+  // check if we've already cached this shader (we know there's at least one pointer type)
+  if(typeMapping.contains(qMakePair(reflection->resourceId, 0)))
+    return;
+
+  for(uint32_t i = 0; i < reflection->pointerTypes.size(); i++)
+  {
+    ShaderVariableType typeDesc = reflection->pointerTypes[i];
+
+    // first recursively cache all subtypes needed by the root struct types
+    CacheSubTypes(reflection, typeDesc);
+
+    // then look up the Type ID for this struct
+    typeMapping[qMakePair(reflection->resourceId, i)] = GetTypeID(typeDesc);
+  }
+}
+
+template <>
+rdcstr DoStringise(const PointerVal &el)
+{
+  if(el.pointerTypeID != ~0U)
+  {
+    uint32_t ptrTypeId = PointerTypeRegistry::GetTypeID(el.shader, el.pointerTypeID);
+
+    return QFormatStr("GPUAddress::%1::%2").arg(el.pointer).arg(ptrTypeId);
+  }
+  else
+  {
+    return QFormatStr("GPUAddress::%1").arg(el.pointer);
+  }
 }
 
 // this is an opaque struct that contains the data to render, hit-test, etc for some text that
@@ -124,7 +214,8 @@ struct RichResourceText
       if(v.userType() == qMetaTypeId<ResourceId>())
       {
         QString resname = QString(ctx.GetResourceName(v.value<ResourceId>())).toHtmlEscaped();
-        html += lit("<td><b>%1</b></td><td><img width=\"16\" src=':/link%3.png'></td>")
+        html += lit("<td valign=\"middle\"><b>%1</b></td>"
+                    "<td><img width=\"16\" src=':/link%3.png'></td>")
                     .arg(resname)
                     .arg(highdpi ? lit("@2x") : QString());
         text += resname;
@@ -135,7 +226,7 @@ struct RichResourceText
       }
       else
       {
-        html += lit("<td>%1</td>").arg(v.toString().toHtmlEscaped());
+        html += lit("<td valign=\"middle\">%1</td>").arg(v.toString().toHtmlEscaped());
         text += v.toString();
 
         // this only generates one block
@@ -152,6 +243,7 @@ struct RichResourceText
 
     doc.setDocumentMargin(0);
     doc.setHtml(html);
+    doc.setDefaultFont(widget->font());
 
     if(doc.blockCount() != fragmentIndexFromBlockIndex.count())
     {
@@ -181,6 +273,37 @@ typedef QSharedPointer<RichResourceText> RichResourceTextPtr;
 
 Q_DECLARE_METATYPE(RichResourceTextPtr);
 
+void GPUAddress::cacheAddress(const QWidget *widget)
+{
+  if(!ctxptr)
+    ctxptr = getCaptureContext(widget);
+
+  // bail out if we don't have a context
+  if(!ctxptr)
+    return;
+
+  // bail if we're already cached
+  if(base != ResourceId())
+    return;
+
+  // find the first matching buffer
+  for(const BufferDescription &b : ctxptr->GetBuffers())
+  {
+    if(b.gpuAddress && b.gpuAddress <= val.pointer && b.gpuAddress + b.length > val.pointer)
+    {
+      base = b.resourceId;
+      offset = val.pointer - b.gpuAddress;
+      return;
+    }
+  }
+}
+
+// for the same reason as above we use a shared pointer for GPU addresses too. This ensures the
+// cached data doesn't keep getting re-cached in copies.
+typedef QSharedPointer<GPUAddress> GPUAddressPtr;
+
+Q_DECLARE_METATYPE(GPUAddressPtr);
+
 QString ResIdTextToString(RichResourceTextPtr ptr)
 {
   return ptr->text;
@@ -191,10 +314,19 @@ QString ResIdToString(ResourceId ptr)
   return ToQStr(ptr);
 }
 
+QString GPUAddressToString(GPUAddressPtr addr)
+{
+  if(addr->base != ResourceId())
+    return QFormatStr("%1+%2").arg(ToQStr(addr->base)).arg(addr->offset);
+  else
+    return QFormatStr("0x%1").arg(addr->val.pointer, 0, 16);
+}
+
 void RegisterMetatypeConversions()
 {
   QMetaType::registerConverter<RichResourceTextPtr, QString>(&ResIdTextToString);
   QMetaType::registerConverter<ResourceId, QString>(&ResIdToString);
+  QMetaType::registerConverter<GPUAddressPtr, QString>(&GPUAddressToString);
 }
 
 void RichResourceTextInitialise(QVariant &var)
@@ -209,14 +341,40 @@ void RichResourceTextInitialise(QVariant &var)
   QString text = var.toString().trimmed();
 
   // do a simple string search first before using regular expressions
-  if(!text.contains(lit("ResourceId::")))
+  if(!text.contains(lit("ResourceId::")) && !text.contains(lit("GPUAddress::")))
     return;
 
+  // two forms: GPUAddress::012345        - typeless
+  //            GPUAddress::012345::991   - using type 991 from PointerTypeRegistry
+  static QRegularExpression addrRE(lit("GPUAddress::([0-9]*)(::([0-9]*))?"));
+
+  QRegularExpressionMatch match = addrRE.match(text);
+
+  if(match.hasMatch())
+  {
+    // don't support mixed text & addresses. Only do the replacement if we matched the whole string
+    if(match.capturedStart(0) == 0 && match.capturedLength(0) == text.length())
+    {
+      GPUAddressPtr addr(new GPUAddress);
+      addr->val.pointer = match.captured(1).toULongLong();
+
+      // we deliberately set this to ResourceId() to indicate that we're using an ID from the
+      // registry, not a shader-relative index
+      addr->val.shader = ResourceId();
+      addr->val.pointerTypeID = match.captured(3).toULong();
+
+      var = QVariant::fromValue(addr);
+      return;
+    }
+
+    return;
+  }
+
   // use regexp to split up into fragments of text and resourceid. The resourceid is then
   // formatted on the fly in RichResourceText::cacheDocument
-  static QRegularExpression re(lit("(ResourceId::)([0-9]*)"));
+  static QRegularExpression resRE(lit("(ResourceId::)([0-9]*)"));
 
-  QRegularExpressionMatch match = re.match(text);
+  match = resRE.match(text);
 
   if(match.hasMatch())
   {
@@ -248,7 +406,7 @@ void RichResourceTextInitialise(QVariant &var)
 
       linkedText->fragments.push_back(id);
 
-      match = re.match(text);
+      match = resRE.match(text);
     }
 
     if(!text.isEmpty())
@@ -263,6 +421,7 @@ void RichResourceTextInitialise(QVariant &var)
 bool RichResourceTextCheck(const QVariant &var)
 {
   return var.userType() == qMetaTypeId<RichResourceTextPtr>() ||
+         var.userType() == qMetaTypeId<GPUAddressPtr>() ||
          var.userType() == qMetaTypeId<ResourceId>();
 }
 
@@ -273,11 +432,12 @@ static const int RichResourceTextMargin = 2;
 void RichResourceTextPaint(const QWidget *owner, QPainter *painter, QRect rect, QFont font,
                            QPalette palette, bool mouseOver, QPoint mousePos, const QVariant &var)
 {
-  // special case handling for ResourceId
-  if(var.userType() == qMetaTypeId<ResourceId>())
+  // special case handling for ResourceId/GPUAddress on its own
+  if(var.userType() == qMetaTypeId<ResourceId>() || var.userType() == qMetaTypeId<GPUAddressPtr>())
   {
-    QFont origfont = painter->font();
-    QFont f = origfont;
+    painter->save();
+
+    QFont f = painter->font();
     f.setBold(true);
     painter->setFont(f);
 
@@ -287,14 +447,46 @@ void RichResourceTextPaint(const QWidget *owner, QPainter *painter, QRect rect,
 
     QString name;
 
-    ICaptureContext *ctxptr = getCaptureContext(owner);
+    bool valid = false;
 
-    ResourceId id = var.value<ResourceId>();
+    if(var.userType() == qMetaTypeId<ResourceId>())
+    {
+      ICaptureContext *ctxptr = getCaptureContext(owner);
 
-    if(ctxptr)
-      name = ctxptr->GetResourceName(id);
+      ResourceId id = var.value<ResourceId>();
+
+      valid = (id != ResourceId());
+
+      if(ctxptr)
+        name = ctxptr->GetResourceName(id);
+      else
+        name = ToQStr(id);
+    }
     else
-      name = ToQStr(id);
+    {
+      GPUAddressPtr ptr = var.value<GPUAddressPtr>();
+
+      ptr->cacheAddress(owner);
+
+      valid = (ptr->val.pointer != 0);
+
+      if(valid)
+      {
+        if(ptr->base != ResourceId())
+        {
+          name = QFormatStr("%1+%2").arg(ptr->ctxptr->GetResourceName(ptr->base)).arg(ptr->offset);
+        }
+        else
+        {
+          name = QFormatStr("Unknown 0x%1").arg(ptr->val.pointer, 16, 16);
+          valid = false;
+        }
+      }
+      else
+      {
+        name = lit("NULL");
+      }
+    }
 
     painter->drawText(rect, Qt::AlignLeft | Qt::AlignVCenter, name);
 
@@ -303,25 +495,27 @@ void RichResourceTextPaint(const QWidget *owner, QPainter *painter, QRect rect,
 
     const QPixmap &px = Pixmaps::link(owner->devicePixelRatio());
 
-    rect.setTop(textRect.top());
-    rect.setWidth(textRect.width() + margin + px.width());
-    rect.setHeight(qMax(textRect.height(), px.height()));
+    painter->setClipRect(rect);
+
+    textRect.setLeft(rect.left());
+    textRect.setWidth(textRect.width() + margin + px.width());
+    textRect.setHeight(qMax(textRect.height(), px.height()));
 
     QPoint pos;
-    pos.setX(rect.right() - px.width() + 1);
-    pos.setY(rect.center().y() - px.height() / 2);
+    pos.setX(textRect.right() - px.width() + 1);
+    pos.setY(textRect.center().y() - px.height() / 2);
 
     painter->drawPixmap(pos, px, px.rect());
 
-    if(mouseOver && rect.contains(mousePos) && id != ResourceId())
+    if(mouseOver && textRect.contains(mousePos) && valid)
     {
       int underline_y = textRect.bottom() + margin;
 
       painter->setPen(QPen(palette.brush(QPalette::WindowText), 1.0));
-      painter->drawLine(QPoint(rect.left(), underline_y), QPoint(rect.right(), underline_y));
+      painter->drawLine(QPoint(textRect.left(), underline_y), QPoint(textRect.right(), underline_y));
     }
 
-    painter->setFont(origfont);
+    painter->restore();
 
     return;
   }
@@ -339,9 +533,11 @@ void RichResourceTextPaint(const QWidget *owner, QPainter *painter, QRect rect,
   int diff = rect.height() - linkedText->doc.size().height();
 
   if(diff > 0)
-    painter->translate(0, diff / 2);
+    painter->translate(1, diff / 2);
+  else
+    painter->translate(1, 0);
 
-  linkedText->doc.drawContents(painter, QRectF(0, 0, rect.width(), rect.height()));
+  linkedText->doc.drawContents(painter, QRectF(0, 0, rect.width() - 1, rect.height()));
 
   if(mouseOver)
   {
@@ -351,7 +547,9 @@ void RichResourceTextPaint(const QWidget *owner, QPainter *painter, QRect rect,
 
     QPoint p = mousePos - rect.topLeft();
     if(diff > 0)
-      p -= QPoint(0, diff / 2);
+      p -= QPoint(1, diff / 2);
+    else
+      p -= QPoint(1, 0);
 
     int pos = layout->hitTest(p, Qt::FuzzyHit);
 
@@ -391,8 +589,8 @@ void RichResourceTextPaint(const QWidget *owner, QPainter *painter, QRect rect,
 
 int RichResourceTextWidthHint(const QWidget *owner, const QFont &font, const QVariant &var)
 {
-  // special case handling for ResourceId
-  if(var.userType() == qMetaTypeId<ResourceId>())
+  // special case handling for ResourceId/GPUAddress on its own
+  if(var.userType() == qMetaTypeId<ResourceId>() || var.userType() == qMetaTypeId<GPUAddressPtr>())
   {
     QFont f = font;
     f.setBold(true);
@@ -403,12 +601,28 @@ int RichResourceTextWidthHint(const QWidget *owner, const QFont &font, const QVa
 
     QString name;
 
-    ICaptureContext *ctxptr = getCaptureContext(owner);
+    if(var.userType() == qMetaTypeId<ResourceId>())
+    {
+      ICaptureContext *ctxptr = getCaptureContext(owner);
 
-    if(ctxptr)
-      name = ctxptr->GetResourceName(var.value<ResourceId>());
+      ResourceId id = var.value<ResourceId>();
+
+      if(ctxptr)
+        name = ctxptr->GetResourceName(id);
+      else
+        name = ToQStr(id);
+    }
     else
-      name = ToQStr(var.value<ResourceId>());
+    {
+      GPUAddressPtr ptr = var.value<GPUAddressPtr>();
+
+      ptr->cacheAddress(owner);
+
+      if(ptr->val.pointer != 0)
+        name = QFormatStr("%1+%2").arg(ptr->ctxptr->GetResourceName(ptr->base)).arg(ptr->offset);
+      else
+        name = lit("NULL");
+    }
 
     const QPixmap &px = Pixmaps::link(owner->devicePixelRatio());
 
@@ -430,14 +644,32 @@ bool RichResourceTextMouseEvent(const QWidget *owner, const QVariant &var, QRect
   if(event->type() != QEvent::MouseButtonRelease && event->type() != QEvent::MouseMove)
     return false;
 
-  // special case handling for ResourceId
-  if(var.userType() == qMetaTypeId<ResourceId>())
+  // special case handling for ResourceId/GPUAddress on its own
+  if(var.userType() == qMetaTypeId<ResourceId>() || var.userType() == qMetaTypeId<GPUAddressPtr>())
   {
-    ResourceId id = var.value<ResourceId>();
+    ResourceId id;
+    GPUAddressPtr ptr;
+    ICaptureContext *ctxptr = NULL;
 
-    // empty resource ids are not clickable or hover-highlighted.
-    if(id == ResourceId())
-      return false;
+    if(var.userType() == qMetaTypeId<ResourceId>())
+    {
+      id = var.value<ResourceId>();
+
+      // empty resource ids are not clickable or hover-highlighted.
+      if(id == ResourceId())
+        return false;
+    }
+
+    if(var.userType() == qMetaTypeId<GPUAddressPtr>())
+    {
+      ptr = var.value<GPUAddressPtr>();
+
+      ptr->cacheAddress(owner);
+
+      // NULL or unknown addresses also are not clickable
+      if(ptr->val.pointer == 0 || ptr->base == ResourceId())
+        return false;
+    }
 
     QFont f = font;
     f.setBold(true);
@@ -448,12 +680,21 @@ bool RichResourceTextMouseEvent(const QWidget *owner, const QVariant &var, QRect
 
     QString name;
 
-    ICaptureContext *ctxptr = getCaptureContext(owner);
+    if(var.userType() == qMetaTypeId<ResourceId>())
+    {
+      ctxptr = getCaptureContext(owner);
 
-    if(ctxptr)
-      name = ctxptr->GetResourceName(id);
+      if(ctxptr)
+        name = ctxptr->GetResourceName(id);
+      else
+        name = ToQStr(id);
+    }
     else
-      name = ToQStr(id);
+    {
+      ctxptr = ptr->ctxptr;
+
+      name = QFormatStr("%1+%2").arg(ptr->ctxptr->GetResourceName(ptr->base)).arg(ptr->offset);
+    }
 
     QRect textRect = QFontMetrics(f).boundingRect(rect, Qt::AlignLeft | Qt::AlignVCenter, name);
 
@@ -463,21 +704,45 @@ bool RichResourceTextMouseEvent(const QWidget *owner, const QVariant &var, QRect
     rect.setWidth(textRect.width() + margin + px.width());
     rect.setHeight(qMax(textRect.height(), px.height()));
 
-    if(rect.contains(event->pos()) && id != ResourceId())
+    if(rect.contains(event->pos()))
     {
-      if(event->type() == QEvent::MouseButtonRelease && ctxptr)
+      if(var.userType() == qMetaTypeId<ResourceId>())
       {
-        ICaptureContext &ctx = *(ICaptureContext *)ctxptr;
+        if(event->type() == QEvent::MouseButtonRelease && ctxptr)
+        {
+          ICaptureContext &ctx = *(ICaptureContext *)ctxptr;
 
-        if(!ctx.HasResourceInspector())
-          ctx.ShowResourceInspector();
+          if(!ctx.HasResourceInspector())
+            ctx.ShowResourceInspector();
 
-        ctx.GetResourceInspector()->Inspect(id);
+          ctx.GetResourceInspector()->Inspect(id);
 
-        ctx.RaiseDockWindow(ctx.GetResourceInspector()->Widget());
+          ctx.RaiseDockWindow(ctx.GetResourceInspector()->Widget());
+        }
+
+        return true;
       }
+      else if(var.userType() == qMetaTypeId<GPUAddressPtr>())
+      {
+        if(event->type() == QEvent::MouseButtonRelease && ctxptr)
+        {
+          ICaptureContext &ctx = *(ICaptureContext *)ctxptr;
 
-      return true;
+          const ShaderVariableType &ptrType = PointerTypeRegistry::GetTypeDescriptor(ptr->val);
+
+          QString formatter;
+
+          if(!ptrType.members.isEmpty())
+            formatter = BufferFormatter::DeclareStruct(ptrType.descriptor.name, ptrType.members,
+                                                       ptrType.descriptor.arrayByteStride);
+
+          IBufferViewer *view = ctx.ViewBuffer(ptr->offset, 0, ptr->base, formatter);
+
+          ctx.AddDockWindow(view->Widget(), DockReference::MainToolArea, NULL);
+        }
+
+        return true;
+      }
     }
 
     return false;
@@ -494,7 +759,9 @@ bool RichResourceTextMouseEvent(const QWidget *owner, const QVariant &var, QRect
 
   QPoint p = event->pos() - rect.topLeft();
   if(diff > 0)
-    p -= QPoint(0, diff / 2);
+    p -= QPoint(1, diff / 2);
+  else
+    p -= QPoint(1, 0);
 
   int pos = layout->hitTest(p, Qt::FuzzyHit);
 
@@ -807,6 +1074,36 @@ QString ToQStr(const ShaderStage stage, const GraphicsAPI apitype)
   return lit("Unknown");
 }
 
+QString ToQStr(const AddressMode addr, const GraphicsAPI apitype)
+{
+  if(IsD3D(apitype))
+  {
+    switch(addr)
+    {
+      case AddressMode::Wrap: return lit("Wrap");
+      case AddressMode::Mirror: return lit("Mirror");
+      case AddressMode::MirrorOnce: return lit("MirrorOnce");
+      case AddressMode::ClampEdge: return lit("ClampEdge");
+      case AddressMode::ClampBorder: return lit("ClampBorder");
+      default: break;
+    }
+  }
+  else
+  {
+    switch(addr)
+    {
+      case AddressMode::Repeat: return lit("Repeat");
+      case AddressMode::MirrorRepeat: return lit("MirrorRepeat");
+      case AddressMode::MirrorClamp: return lit("MirrorClamp");
+      case AddressMode::ClampEdge: return lit("ClampEdge");
+      case AddressMode::ClampBorder: return lit("ClampBorder");
+      default: break;
+    }
+  }
+
+  return lit("Unknown");
+}
+
 QString TypeString(const SigParameter &sig)
 {
   QString ret = lit("");
@@ -823,6 +1120,8 @@ QString TypeString(const SigParameter &sig)
     ret += lit("snorm float");
   else if(sig.compType == CompType::Depth)
     ret += lit("float");
+  else if(sig.compType == CompType::Double)
+    ret += lit("double");
 
   if(sig.compCount > 1)
     ret += QString::number(sig.compCount);
@@ -835,7 +1134,7 @@ QString D3DSemanticString(const SigParameter &sig)
   if(sig.systemValue == ShaderBuiltin::Undefined)
     return sig.semanticIdxName;
 
-  QString sysValues[ENUM_ARRAY_SIZE(ShaderBuiltin)] = {
+  QString sysValues[arraydim<ShaderBuiltin>()] = {
       lit("SV_Undefined"),
       lit("SV_Position"),
       lit("Unsupported (PointSize)"),
@@ -1749,7 +2048,7 @@ bool RunProcessAsAdmin(const QString &fullExecutablePath, const QStringList &par
 #else
   // try to find a way to run the application elevated.
   const QString graphicalSudo[] = {
-      lit("pkexec"), lit("kdesudo"), lit("gksudo"), lit("beesu"),
+      lit("kdesudo"), lit("gksudo"), lit("beesu"),
   };
 
   // if none of the graphical options, then look for sudo and either
@@ -1782,7 +2081,7 @@ bool RunProcessAsAdmin(const QString &fullExecutablePath, const QStringList &par
     process->start(sudo, sudoParams);
 
     // when the process exits, call the callback and delete
-    QObject::connect(process, OverloadedSlot<int>::of(&QProcess::finished),
+    QObject::connect(process, OverloadedSlot<int, QProcess::ExitStatus>::of(&QProcess::finished),
                      [parent, process, finishedCallback](int exitCode) {
                        process->deleteLater();
                        GUIInvoke::call(parent, finishedCallback);
@@ -1812,14 +2111,15 @@ bool RunProcessAsAdmin(const QString &fullExecutablePath, const QStringList &par
 
     // run terminal sudo with emulator
     QStringList termParams;
-    termParams
-        << lit("-e")
-        << lit("bash -c 'sudo %1 %2'").arg(fullExecutablePath).arg(params.join(QLatin1Char(' ')));
+    termParams << lit("-e")
+               << lit("bash -c 'echo Running \"%1 %2\" as root.;echo;sudo %1 %2'")
+                      .arg(fullExecutablePath)
+                      .arg(params.join(QLatin1Char(' ')));
 
     process->start(term, termParams);
 
     // when the process exits, call the callback and delete
-    QObject::connect(process, OverloadedSlot<int>::of(&QProcess::finished),
+    QObject::connect(process, OverloadedSlot<int, QProcess::ExitStatus>::of(&QProcess::finished),
                      [parent, process, finishedCallback](int exitCode) {
                        process->deleteLater();
                        GUIInvoke::call(parent, finishedCallback);
@@ -1867,7 +2167,7 @@ QStringList ParseArgsList(const QString &args)
 
   LocalFree(argv);
 #else
-  std::string argString = args.toStdString();
+  rdcstr argString = args;
 
   // perform some kind of sane parsing
   bool dquot = false, squot = false;    // are we inside ''s or ""s
@@ -1876,14 +2176,14 @@ QStringList ParseArgsList(const QString &args)
   char *c = &argString[0];
 
   // current argument we're building
-  std::string a;
+  rdcstr a;
 
   while(*c)
   {
     if(!dquot && !squot && (*c == ' ' || *c == '\t'))
     {
       if(!a.empty())
-        ret << QString::fromStdString(a);
+        ret << QString(a);
 
       a = "";
     }
@@ -1939,7 +2239,7 @@ QStringList ParseArgsList(const QString &args)
 
   // if we were building an argument when we hit the end of the string
   if(!a.empty())
-    ret << QString::fromStdString(a);
+    ret << QString(a);
 #endif
 
   return ret;
@@ -1975,6 +2275,7 @@ void ShowProgressDialog(QWidget *window, const QString &labelText, ProgressFinis
 
     GUIInvoke::call(&dialog, [&dialog]() { dialog.closeAndReset(); });
   });
+  progressTickerThread.setName(lit("Progress Dialog"));
   progressTickerThread.start();
 
   // show the dialog
@@ -2123,3 +2424,90 @@ QColor contrastingColor(const QColor &col, const QColor &defaultCol)
   else
     return QColor(Qt::black);
 }
+
+// we declare this partial class to get the accessors. THIS IS DANGEROUS as the ABI is unstable and
+// this is a private class. The first few functions have been stable for a while so we hope that it
+// will remain so. If a stable interface is added in future like QX11Info we should definitely use
+// it instead.
+//
+// Unfortunately we need this for Wayland, so we only ever use it when we are absolutely forced to
+// because we're running under the Wayland Qt platform.
+class QOpenGLContext;
+
+class Q_GUI_EXPORT QPlatformNativeInterface : public QObject
+{
+  Q_OBJECT
+public:
+  virtual void *nativeResourceForIntegration(const QByteArray &resource);
+  virtual void *nativeResourceForContext(const QByteArray &resource, QOpenGLContext *context);
+  virtual void *nativeResourceForScreen(const QByteArray &resource, QScreen *screen);
+  virtual void *nativeResourceForWindow(const QByteArray &resource, QWindow *window);
+};
+
+void *AccessWaylandPlatformInterface(const QByteArray &resource, QWindow *window)
+{
+  QPlatformNativeInterface *native = QGuiApplication::platformNativeInterface();
+  return native->nativeResourceForWindow(resource, window);
+}
+
+// Default Qt doesn't do this in release Qt builds, which is all we use
+#if defined(Q_OS_WIN32)
+
+#include <windows.h>
+
+typedef HRESULT(WINAPI *PFN_SetThreadDescription)(HANDLE hThread, PCWSTR lpThreadDescription);
+
+const DWORD MS_VC_EXCEPTION = 0x406D1388;
+#pragma pack(push, 8)
+typedef struct tagTHREADNAME_INFO
+{
+  DWORD dwType;        // Must be 0x1000.
+  LPCSTR szName;       // Pointer to name (in user addr space).
+  DWORD dwThreadID;    // Thread ID (-1=caller thread).
+  DWORD dwFlags;       // Reserved for future use, must be zero.
+} THREADNAME_INFO;
+#pragma pack(pop)
+
+static void SetThreadNameWithException(const char *name)
+{
+  THREADNAME_INFO info;
+  info.dwType = 0x1000;
+  info.szName = name;
+  info.dwThreadID = GetCurrentThreadId();
+  info.dwFlags = 0;
+  __try
+  {
+    RaiseException(MS_VC_EXCEPTION, 0, sizeof(info) / sizeof(ULONG_PTR), (ULONG_PTR *)(&info));
+  }
+  __except(EXCEPTION_CONTINUE_EXECUTION)
+  {
+  }
+}
+
+void LambdaThread::windowsSetName()
+{
+  // try to use the fancy modern API
+  static PFN_SetThreadDescription setThreadDesc = (PFN_SetThreadDescription)GetProcAddress(
+      GetModuleHandleA("kernel32.dll"), "SetThreadDescription");
+
+  if(setThreadDesc)
+  {
+    setThreadDesc(GetCurrentThread(), m_Name.toStdWString().c_str());
+  }
+  else
+  {
+    // don't throw the exception if there's no debugger present
+    if(!IsDebuggerPresent())
+      return;
+
+    SetThreadNameWithException(m_Name.toStdString().c_str());
+  }
+}
+
+#else
+
+void LambdaThread::windowsSetName()
+{
+}
+
+#endif
