@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -42,7 +42,8 @@ ReplayManager::~ReplayManager()
   RENDERDOC_UnregisterMemoryRegion(this);
 }
 
-void ReplayManager::OpenCapture(const QString &capturefile, RENDERDOC_ProgressCallback progress)
+void ReplayManager::OpenCapture(const QString &capturefile, const ReplayOptions &opts,
+                                RENDERDOC_ProgressCallback progress)
 {
   if(m_Running)
     return;
@@ -50,9 +51,10 @@ void ReplayManager::OpenCapture(const QString &capturefile, RENDERDOC_ProgressCa
   // TODO maybe we could expose this choice to the user?
   int proxyRenderer = -1;
 
-  m_Thread = new LambdaThread([this, proxyRenderer, capturefile, progress]() {
-    run(proxyRenderer, capturefile, progress);
+  m_Thread = new LambdaThread([this, proxyRenderer, capturefile, opts, progress]() {
+    run(proxyRenderer, capturefile, opts, progress);
   });
+  m_Thread->setName(lit("ReplayManager"));
   m_Thread->start(QThread::HighestPriority);
 
   while(m_Thread->isRunning() && !m_Running)
@@ -63,7 +65,7 @@ void ReplayManager::OpenCapture(const QString &capturefile, RENDERDOC_ProgressCa
 
 void ReplayManager::DeleteCapture(const rdcstr &capture, bool local)
 {
-  if(IsRunning())
+  if(IsRunning() && !m_Thread->isCurrentThread())
   {
     AsyncInvoke([this, capture, local](IReplayController *) { DeleteCapture(capture, local); });
     return;
@@ -184,6 +186,7 @@ rdcstr ReplayManager::CopyCaptureToRemote(const rdcstr &localpath, QWidget *wind
   {
     LambdaThread *thread = new LambdaThread([&lambda]() { lambda(NULL); });
     thread->selfDelete(true);
+    thread->setName(lit("CopyCaptureToRemote"));
     thread->start();
   }
 
@@ -218,6 +221,7 @@ void ReplayManager::CopyCaptureFromRemote(const rdcstr &remotepath, const rdcstr
   {
     LambdaThread *thread = new LambdaThread([&lambda]() { lambda(NULL); });
     thread->selfDelete(true);
+    thread->setName(lit("CopyCaptureFromRemote"));
     thread->start();
   }
 
@@ -305,11 +309,11 @@ void ReplayManager::CloseThread()
   m_Thread = NULL;
 }
 
-ReplayStatus ReplayManager::ConnectToRemoteServer(RemoteHost *host)
+ReplayStatus ReplayManager::ConnectToRemoteServer(RemoteHost host)
 {
-  ReplayStatus status = RENDERDOC_CreateRemoteServerConnection(host->hostname.c_str(), 0, &m_Remote);
+  ReplayStatus status = RENDERDOC_CreateRemoteServerConnection(host.Hostname().c_str(), &m_Remote);
 
-  if(host->IsADB())
+  if(host.Protocol() && host.Protocol()->GetProtocolName() == "adb")
   {
     ANALYTIC_SET(UIFeatures.AndroidRemoteReplay, true);
   }
@@ -322,7 +326,7 @@ ReplayStatus ReplayManager::ConnectToRemoteServer(RemoteHost *host)
 
   if(status == ReplayStatus::Succeeded)
   {
-    m_RemoteHost->connected = true;
+    m_RemoteHost.SetConnected(true);
     return status;
   }
 
@@ -331,8 +335,7 @@ ReplayStatus ReplayManager::ConnectToRemoteServer(RemoteHost *host)
 
 void ReplayManager::DisconnectFromRemoteServer()
 {
-  if(m_RemoteHost)
-    m_RemoteHost->connected = false;
+  m_RemoteHost.SetConnected(false);
 
   if(m_Remote)
   {
@@ -340,12 +343,14 @@ void ReplayManager::DisconnectFromRemoteServer()
     m_Remote->ShutdownConnection();
   }
 
-  m_RemoteHost = NULL;
+  m_RemoteHost = RemoteHost();
   m_Remote = NULL;
 }
 
 void ReplayManager::ShutdownServer()
 {
+  m_RemoteHost.SetShutdown();
+
   if(m_Remote)
   {
     QMutexLocker autolock(&m_RemoteLock);
@@ -365,7 +370,7 @@ void ReplayManager::PingRemote()
     if(!IsRunning() || m_Thread->isCurrentThread())
     {
       if(!m_Remote->Ping())
-        m_RemoteHost->serverRunning = false;
+        m_RemoteHost.SetShutdown();
     }
     m_RemoteLock.unlock();
   }
@@ -415,15 +420,15 @@ void ReplayManager::PushInvoke(ReplayManager::InvokeHandle *cmd)
   m_RenderCondition.wakeAll();
 }
 
-void ReplayManager::run(int proxyRenderer, const QString &capturefile,
+void ReplayManager::run(int proxyRenderer, const QString &capturefile, const ReplayOptions &opts,
                         RENDERDOC_ProgressCallback progress)
 {
   m_Renderer = NULL;
 
   if(m_Remote)
   {
-    std::tie(m_CreateStatus, m_Renderer) =
-        m_Remote->OpenCapture(proxyRenderer, capturefile.toUtf8().data(), progress);
+    rdctie(m_CreateStatus, m_Renderer) =
+        m_Remote->OpenCapture(proxyRenderer, capturefile.toUtf8().data(), opts, progress);
   }
   else
   {
@@ -432,7 +437,7 @@ void ReplayManager::run(int proxyRenderer, const QString &capturefile,
     m_CreateStatus = m_CaptureFile->OpenFile(capturefile.toUtf8().data(), "rdc", NULL);
 
     if(m_CreateStatus == ReplayStatus::Succeeded)
-      std::tie(m_CreateStatus, m_Renderer) = m_CaptureFile->OpenCapture(progress);
+      rdctie(m_CreateStatus, m_Renderer) = m_CaptureFile->OpenCapture(opts, progress);
   }
 
   if(m_Renderer == NULL)
