@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -314,6 +314,28 @@ void PythonContext::GlobalInit()
   }
 #endif
 
+#if RENDERDOC_STABLE_BUILD == 0
+  // if we're running in the git checkout and we can find the test scripts, add that location to the
+  // path
+  {
+    QDir bin = QFileInfo(QCoreApplication::applicationFilePath()).absoluteDir();
+
+    QString testpath = QDir::cleanPath(bin.absoluteFilePath(lit("../../util/test")));
+
+    if(QDir(testpath).exists(lit("run_tests.py")))
+    {
+      PyObject *syspath = PyObject_GetAttrString(sysobj, "path");
+
+      PyObject *str = PyUnicode_FromString(testpath.toUtf8().data());
+
+      PyList_Append(syspath, str);
+
+      Py_DecRef(str);
+      Py_DecRef(syspath);
+    }
+  }
+#endif
+
 // set up PySide
 #if PYSIDE2_ENABLED
   {
@@ -590,9 +612,8 @@ bool PythonContext::LoadExtension(ICaptureContext &ctx, const rdcstr &extension)
 
     if(register_func)
     {
-      QByteArray baseTypeName = TypeName<ICaptureContext>();
-      baseTypeName += " *";
-      PyObject *pyctx = PassObjectToPython(baseTypeName.data(), &ctx);
+      PyObject *pyctx =
+          PassObjectToPython((rdcstr(TypeName<ICaptureContext>()) + " *").c_str(), &ctx);
 
       PyObject *retval = NULL;
       if(pyctx)
@@ -1117,7 +1138,7 @@ PyObject *PythonContext::outstream_write(PyObject *self, PyObject *args)
       _frame *frame = PyEval_GetFrame();
 
       while(message.back() == '\n' || message.back() == '\r')
-        message.erase(message.size() - 1);
+        message.pop_back();
 
       QString filename = lit("unknown");
       int line = 0;
@@ -1201,16 +1222,23 @@ extern "C" void HandleException(PyObject *global_handle)
   else if(redirector && !redirector->context)
   {
     // if still NULL we're running in the extension context
-    std::string exString;
+    rdcstr exString;
 
     if(!frames.isEmpty())
     {
       exString += "Traceback (most recent call last):\n";
       for(const QString &f : frames)
-        exString += "  " + f.toUtf8().toStdString() + "\n";
+      {
+        exString += "  ";
+        exString += f;
+        exString += "\n";
+      }
     }
 
-    exString += typeStr.toUtf8().toStdString() + ": " + valueStr.toUtf8().toStdString() + "\n";
+    exString += typeStr;
+    exString += ": ";
+    exString += valueStr;
+    exString += "\n";
 
     _frame *frame = PyEval_GetFrame();
 
@@ -1250,4 +1278,4 @@ extern "C" QWidget *QWidgetFromPy(PyObject *widget)
 extern "C" PyObject *QWidgetToPy(QWidget *widget)
 {
   return PythonContext::QWidgetToPy(widget);
-}
\ No newline at end of file
+}
