@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -125,21 +125,25 @@ void PersistantConfig::Close()
 {
 }
 
-int PersistantConfig::RemoteHostCount()
+rdcarray<RemoteHost> PersistantConfig::GetRemoteHosts()
 {
-  return 0;
+  return {};
 }
 
-RemoteHost *PersistantConfig::GetRemoteHost(int index)
+RemoteHost PersistantConfig::GetRemoteHost(const rdcstr &)
 {
-  return NULL;
+  return RemoteHost();
 }
 
 void PersistantConfig::AddRemoteHost(RemoteHost host)
 {
 }
 
-void PersistantConfig::AddAndroidHosts()
+void PersistantConfig::RemoveRemoteHost(RemoteHost host)
+{
+}
+
+void PersistantConfig::UpdateEnumeratedProtocolDevices()
 {
 }
 
@@ -147,7 +151,11 @@ void PersistantConfig::SetupFormatting()
 {
 }
 
-void AddRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file, int maxItems)
+void AddRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file)
+{
+}
+
+void RemoveRecentFile(rdcarray<rdcstr> &recentList, const rdcstr &file)
 {
 }
 
@@ -166,7 +174,23 @@ rdcstr PersistantConfig::GetConfigSetting(const rdcstr &name)
 
 RemoteHost::RemoteHost()
 {
-  serverRunning = connected = busy = versionMismatch = false;
+}
+
+RemoteHost::RemoteHost(const rdcstr &host)
+{
+}
+
+RemoteHost::RemoteHost(const RemoteHost &o)
+{
+}
+
+RemoteHost &RemoteHost::operator=(const RemoteHost &o)
+{
+  return *this;
+}
+
+RemoteHost::~RemoteHost()
+{
 }
 
 void RemoteHost::CheckStatus()
@@ -177,3 +201,58 @@ ReplayStatus RemoteHost::Launch()
 {
   return ReplayStatus::Succeeded;
 }
+
+bool RemoteHost::IsServerRunning() const
+{
+  return false;
+}
+
+bool RemoteHost::IsConnected() const
+{
+  return false;
+}
+
+bool RemoteHost::IsBusy() const
+{
+  return false;
+}
+
+bool RemoteHost::IsVersionMismatch() const
+{
+  return false;
+}
+
+rdcstr RemoteHost::FriendlyName() const
+{
+  return rdcstr();
+}
+
+void RemoteHost::SetFriendlyName(const rdcstr &name)
+{
+}
+
+rdcstr RemoteHost::RunCommand() const
+{
+  return rdcstr();
+}
+
+void RemoteHost::SetRunCommand(const rdcstr &cmd)
+{
+}
+
+rdcstr RemoteHost::LastCapturePath() const
+{
+  return rdcstr();
+}
+
+void RemoteHost::SetLastCapturePath(const rdcstr &path)
+{
+}
+
+void RemoteHost::SetConnected(bool connected)
+{
+}
+
+void RemoteHost::SetShutdown()
+{
+}
