@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -87,6 +87,9 @@ int main(int argc, char *argv[])
 
   qInfo() << "QRenderDoc initialising.";
 
+  if(IsRunningAsAdmin())
+    qInfo() << "Running as administrator";
+
   QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
   QGuiApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
 
@@ -180,7 +183,7 @@ int main(int argc, char *argv[])
 
   if(parser.isSet(versionOption))
   {
-    printf("QRenderDoc v%s (%s)\n", MAJOR_MINOR_VERSION_STRING, GitVersionHash);
+    printf("QRenderDoc v%s (%s)\n", MAJOR_MINOR_VERSION_STRING, RENDERDOC_GetCommitHash());
 #if defined(DISTRIBUTION_VERSION)
     printf("Packaged for %s - %s\n", DISTRIBUTION_NAME, DISTRIBUTION_CONTACT);
 #endif
@@ -309,10 +312,11 @@ int main(int argc, char *argv[])
     int replayHostIndex = -1;
     if(parser.isSet(replayhost))
     {
-      QString replayHost = parser.value(replayhost);
-      for(int i = 0; i < config.RemoteHosts.count(); i++)
+      rdcstr replayHost = parser.value(replayhost);
+      rdcarray<RemoteHost> hosts = config.GetRemoteHosts();
+      for(int i = 0; i < hosts.count(); i++)
       {
-        if(QString(config.RemoteHosts[i]->hostname) == replayHost)
+        if(hosts[i].Hostname() == replayHost)
         {
           replayHostIndex = i;
           break;
@@ -354,6 +358,20 @@ int main(int argc, char *argv[])
       GlobalEnvironment env;
 #if defined(RENDERDOC_PLATFORM_LINUX)
       env.xlibDisplay = QX11Info::display();
+      if(QGuiApplication::platformName() == lit("wayland"))
+      {
+        env.waylandDisplay = (wl_display *)AccessWaylandPlatformInterface("display", NULL);
+
+        QString warning =
+            tr("Running directly on Wayland is NOT SUPPORTED and is likely to crash, hang, or "
+               "fail to render.");
+
+        qInfo() << "------ !!!! WARNING !!!! ------";
+        qInfo() << warning;
+        qInfo() << "------ !!!! WARNING !!!! ------";
+
+        RDDialog::critical(NULL, tr("Wayland Qt platform not supported"), warning);
+      }
 #endif
       rdcarray<rdcstr> coreargs;
       if(!crashReportPath.isEmpty())
@@ -365,18 +383,20 @@ int main(int argc, char *argv[])
 
     if(!crashReportPath.isEmpty())
     {
-      QFile f(crashReportPath);
+      QVariantMap json;
 
-      if(f.exists() && f.open(QIODevice::ReadOnly | QIODevice::Text))
       {
-        QVariantMap json = JSONToVariant(QString::fromUtf8(f.readAll()));
+        QFile f(crashReportPath);
 
-        if(json.contains(lit("report")))
-        {
-          CrashDialog dialog(config, json);
+        if(f.exists() && f.open(QIODevice::ReadOnly | QIODevice::Text))
+          json = JSONToVariant(QString::fromUtf8(f.readAll()));
+      }
 
-          RDDialog::show(&dialog);
-        }
+      if(json.contains(lit("report")))
+      {
+        CrashDialog dialog(config, json);
+
+        RDDialog::show(&dialog);
       }
     }
     else
@@ -470,8 +490,6 @@ int main(int argc, char *argv[])
       config.Save();
     }
 
-    RENDERDOC_AndroidShutdown();
-
     PythonContext::GlobalShutdown();
 
     Formatter::shutdown();
