@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -79,11 +79,15 @@ void RDHeaderView::setModel(QAbstractItemModel *model)
 
   QHeaderView::setModel(model);
 
-  QObject::connect(model, &QAbstractItemModel::headerDataChanged, this,
-                   &RDHeaderView::headerDataChanged);
-  QObject::connect(model, &QAbstractItemModel::columnsInserted, this, &RDHeaderView::columnsInserted);
-  QObject::connect(model, &QAbstractItemModel::rowsInserted, this, &RDHeaderView::rowsChanged);
-  QObject::connect(model, &QAbstractItemModel::rowsRemoved, this, &RDHeaderView::rowsChanged);
+  if(model)
+  {
+    QObject::connect(model, &QAbstractItemModel::headerDataChanged, this,
+                     &RDHeaderView::headerDataChanged);
+    QObject::connect(model, &QAbstractItemModel::columnsInserted, this,
+                     &RDHeaderView::columnsInserted);
+    QObject::connect(model, &QAbstractItemModel::rowsInserted, this, &RDHeaderView::rowsChanged);
+    QObject::connect(model, &QAbstractItemModel::rowsRemoved, this, &RDHeaderView::rowsChanged);
+  }
 }
 
 void RDHeaderView::reset()
@@ -383,6 +387,9 @@ void RDHeaderView::resizeSectionsWithHints()
     minsizesTotal += sizes[i];
   }
 
+  if(stretchHintTotal == 0)
+    return;
+
   int available = 0;
 
   if(orientation() == Qt::Horizontal)
