@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -596,6 +596,8 @@ RDTreeWidget::RDTreeWidget(QWidget *parent) : RDTreeView(parent)
 
 RDTreeWidget::~RDTreeWidget()
 {
+  RDTreeView::setModel(NULL);
+
   delete m_root;
   delete m_model;
 }
@@ -807,6 +809,8 @@ void RDTreeWidget::mouseMoveEvent(QMouseEvent *e)
     }
   }
 
+  emit hoverItemChanged(newHover);
+
   emit mouseMove(e);
 
   RDTreeView::mouseMoveEvent(e);
@@ -836,6 +840,8 @@ void RDTreeWidget::leaveEvent(QEvent *e)
     m_model->itemChanged(item, {Qt::DecorationRole, Qt::BackgroundRole, Qt::ForegroundRole});
   }
 
+  emit hoverItemChanged(NULL);
+
   RDTreeView::leaveEvent(e);
 }
 
@@ -1076,4 +1082,4 @@ void RDTreeWidget::endInsertChild(RDTreeWidgetItem *item, int index)
   // work is all done in beginInsertChild
   if(!m_queueUpdates)
     m_model->endInsertChild(item);
-}
\ No newline at end of file
+}
