@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -43,7 +43,7 @@ ConstantBufferPreviewer::ConstantBufferPreviewer(ICaptureContext &ctx, const Sha
   QObject::connect(ui->formatSpecifier, &BufferFormatSpecifier::processFormat, this,
                    &ConstantBufferPreviewer::processFormat);
 
-  ui->formatSpecifier->showHelp(false);
+  ui->formatSpecifier->setContext(&m_Ctx);
 
   ui->splitter->setCollapsible(1, true);
   ui->splitter->setSizes({1, 0});
@@ -82,17 +82,22 @@ ConstantBufferPreviewer *ConstantBufferPreviewer::has(ShaderStage stage, uint32_
 
 void ConstantBufferPreviewer::OnCaptureLoaded()
 {
-  OnCaptureClosed();
+  Reset();
 }
 
 void ConstantBufferPreviewer::OnCaptureClosed()
+{
+  Reset();
+
+  ToolWindowManager::closeToolWindow(this);
+}
+
+void ConstantBufferPreviewer::Reset()
 {
   ui->variables->clear();
   ui->variables->clearInternalExpansions();
 
   ui->saveCSV->setEnabled(false);
-
-  ToolWindowManager::closeToolWindow(this);
 }
 
 void ConstantBufferPreviewer::OnEventChanged(uint32_t eventId)
@@ -104,6 +109,8 @@ void ConstantBufferPreviewer::OnEventChanged(uint32_t eventId)
 
   ResourceId prevShader = m_shader;
 
+  m_pipe = m_stage == ShaderStage::Compute ? m_Ctx.CurPipelineState().GetComputePipelineObject()
+                                           : m_Ctx.CurPipelineState().GetGraphicsPipelineObject();
   m_shader = m_Ctx.CurPipelineState().GetShader(m_stage);
   QString entryPoint = m_Ctx.CurPipelineState().GetShaderEntryPoint(m_stage);
   const ShaderReflection *reflection = m_Ctx.CurPipelineState().GetShaderReflection(m_stage);
@@ -115,7 +122,7 @@ void ConstantBufferPreviewer::OnEventChanged(uint32_t eventId)
   if(reflection == NULL || m_slot >= reflection->constantBlocks.size())
   {
     // save expansion before clearing
-    if(m_formatOverride.empty())
+    if(m_formatOverride.type.members.empty())
     {
       RDTreeViewExpansionState &prevShaderExpansionState =
           ui->variables->getInternalExpansion(qHash(ToQStr(prevShader)));
@@ -126,7 +133,7 @@ void ConstantBufferPreviewer::OnEventChanged(uint32_t eventId)
     return;
   }
 
-  if(!m_formatOverride.empty())
+  if(!m_formatOverride.type.members.empty())
   {
     m_Ctx.Replay().AsyncInvoke([this, offs, size, wasEmpty](IReplayController *r) {
       bytebuf data = r->GetBufferData(m_cbuffer, offs, size);
@@ -148,7 +155,7 @@ void ConstantBufferPreviewer::OnEventChanged(uint32_t eventId)
   {
     m_Ctx.Replay().AsyncInvoke([this, prevShader, entryPoint, offs, wasEmpty](IReplayController *r) {
       rdcarray<ShaderVariable> vars = r->GetCBufferVariableContents(
-          m_shader, entryPoint.toUtf8().data(), m_slot, m_cbuffer, offs);
+          m_pipe, m_shader, entryPoint.toUtf8().data(), m_slot, m_cbuffer, offs);
       GUIInvoke::call(this, [this, prevShader, vars, wasEmpty] {
 
         RDTreeViewExpansionState &prevShaderExpansionState =
@@ -258,14 +265,14 @@ void ConstantBufferPreviewer::processFormat(const QString &format)
 {
   if(format.isEmpty())
   {
-    m_formatOverride.clear();
+    m_formatOverride = ShaderConstant();
     ui->formatSpecifier->setErrors(QString());
   }
   else
   {
     QString errors;
 
-    m_formatOverride = FormatElement::ParseFormatString(format, 0, false, errors);
+    m_formatOverride = BufferFormatter::ParseFormatString(format, 0, false, errors);
     ui->formatSpecifier->setErrors(errors);
   }
 
@@ -341,17 +348,7 @@ void ConstantBufferPreviewer::updateLabels()
 
 rdcarray<ShaderVariable> ConstantBufferPreviewer::applyFormatOverride(const bytebuf &bytes)
 {
-  QVector<ShaderVariable> variables;
+  ShaderVariable var = InterpretShaderVar(m_formatOverride, bytes.begin(), bytes.end());
 
-  variables.resize(m_formatOverride.length());
-
-  for(int i = 0; i < m_formatOverride.length(); i++)
-  {
-    const byte *b = bytes.begin() + m_formatOverride[i].offset;
-    variables[i] = m_formatOverride[i].GetShaderVar(b, bytes.end());
-  }
-
-  rdcarray<ShaderVariable> ret;
-  ret = variables.toStdVector();
-  return ret;
+  return var.members;
 }
