@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -34,7 +34,7 @@ AboutDialog::AboutDialog(QWidget *parent) : QDialog(parent), ui(new Ui::AboutDia
 {
   ui->setupUi(this);
 
-  QString hash = QString::fromLatin1(GitVersionHash);
+  QString hash = QString::fromLatin1(RENDERDOC_GetCommitHash());
 
   if(hash[0] == QLatin1Char('N') && hash[1] == QLatin1Char('O'))
   {
