@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -108,12 +108,30 @@ void CaptureDialog::PopulateMostRecent()
     if(!settings.executable.isEmpty())
     {
       ui->loadLastCapture->setEnabled(true);
-      if(settings.executable.indexOf('/') < 0 && settings.executable.indexOf('\\') < 0)
-        ui->loadLastCapture->setText(tr("Load Last Settings - %1").arg(QString(settings.executable)));
+
+      QString exe = settings.executable;
+
+      // if the executable isn't a path, display the full name
+      bool fullName = (exe.indexOf(QLatin1Char('/')) < 0 && exe.indexOf(QLatin1Char('\\')) < 0);
+
+      // if it's not a windows path and only contains one '/', it's an android package so also
+      // display the full name
+      fullName |= (exe.indexOf(QLatin1Char('\\')) < 0 && exe.count(QLatin1Char('/')) == 1);
+
+      if(fullName)
+      {
+        ui->loadLastCapture->setText(tr("Load Last Settings - %1").arg(exe));
+      }
       else
-        ui->loadLastCapture->setText(
-            tr("Load Last Settings - %1")
-                .arg(QFileInfo(QString(settings.executable)).completeBaseName()));
+      {
+        int offs = exe.lastIndexOf(QLatin1Char('\\'));
+        if(offs > 0)
+          exe.remove(0, offs + 1);
+        offs = exe.lastIndexOf(QLatin1Char('/'));
+        if(offs > 0)
+          exe.remove(0, offs + 1);
+        ui->loadLastCapture->setText(tr("Load Last Settings - %1").arg(exe));
+      }
       return;
     }
   }
@@ -292,7 +310,7 @@ void CaptureDialog::on_exePath_textChanged(const QString &text)
   {
     QString path = dir.absolutePath();
 
-    if(!m_Ctx.Replay().CurrentRemote())
+    if(!m_Ctx.Replay().CurrentRemote().IsValid())
       path = QDir::toNativeSeparators(path);
 
     // match the path separators from the path
@@ -322,15 +340,23 @@ void CaptureDialog::vulkanLayerWarn_mouseClick()
   const bool hasOtherJSON = bool(info.flags & VulkanLayerFlags::OtherInstallsRegistered);
   const bool thisRegistered = bool(info.flags & VulkanLayerFlags::ThisInstallRegistered);
   const bool needElevation = bool(info.flags & VulkanLayerFlags::NeedElevation);
-  const bool couldElevate = bool(info.flags & VulkanLayerFlags::CouldElevate);
+  const bool userRegisterable = bool(info.flags & VulkanLayerFlags::UserRegisterable);
   const bool registerAll = bool(info.flags & VulkanLayerFlags::RegisterAll);
   const bool updateAllowed = bool(info.flags & VulkanLayerFlags::UpdateAllowed);
 
   if(info.flags & VulkanLayerFlags::Unfixable)
   {
     QString msg =
-        tr("There is an unfixable problem with your vulkan layer configuration. Please consult the "
-           "RenderDoc documentation, or package/distribution documentation on linux\n\n");
+        tr("There is an unfixable problem with your vulkan layer configuration.\n\n"
+           "This is most commonly caused by having a distribution-provided package of RenderDoc "
+           "installed, which cannot be modified by another build of RenderDoc.\n\n"
+           "Please consult the RenderDoc documentation, or package/distribution documentation on "
+           "linux. ");
+
+    if(info.otherJSONs.size() > 1)
+      msg += tr("Conflicting manifests:\n\n");
+    else
+      msg += tr("Conflicting manifest:\n\n");
 
     for(const rdcstr &j : info.otherJSONs)
       msg += j + lit("\n");
@@ -346,12 +372,12 @@ void CaptureDialog::vulkanLayerWarn_mouseClick()
   {
     if(info.otherJSONs.size() > 1)
       msg +=
-          tr("there are other RenderDoc builds registered already. They must be disabled so that "
-             "capture can happen without nasty clashes.");
+          tr("there are other conflicting RenderDoc builds registered already. They must be "
+             "disabled so that vulkan programs can be captured without crashes.");
     else
       msg +=
-          tr("there is another RenderDoc build registered already. It must be disabled so that "
-             "capture can happen without nasty clashes.");
+          tr("there is another conflicting RenderDoc build registered already. It must be disabled "
+             "so that vulkan programs can be captured without crashes.");
 
     if(!thisRegistered)
       msg += tr(" Also ");
@@ -392,44 +418,69 @@ void CaptureDialog::vulkanLayerWarn_mouseClick()
     msg += lit("\n");
   }
 
+  if(needElevation)
+    msg +=
+        tr("Due to some builds being in privileged locations, RenderDoc must elevate permissions "
+           "to update them.\n\n");
+
   msg += tr("This is a one-off change, it won't be needed again unless the installation moves.");
 
   QMessageBox::StandardButton install = RDDialog::question(this, caption, msg, RDDialog::YesNoCancel);
 
   if(install == QMessageBox::Yes)
   {
-    bool run = false;
-    bool admin = false;
+    bool admin = needElevation;
+    bool system = true;    // default to system-wide install
+    bool run = true;       // default to running
 
-    // if we need to elevate, just try it.
-    if(needElevation)
+    // if we could install user-local, ask the user if that's what they want.
+    if(userRegisterable)
     {
-      admin = run = true;
-    }
-    // if we could elevate, ask the user
-    else if(couldElevate)
-    {
-      QMessageBox::StandardButton elevate = RDDialog::question(
-          this, tr("System layer install"),
-          tr("Do you want to elevate permissions to install the layer at a system level?\n\n"
-             "If you click 'No', the layer will be installed at a per-user level."),
-          RDDialog::YesNoCancel);
+      msg =
+          tr("Do you want to install the layer at a system level?\n\n"
+             "If you click 'No', the layer will be installed at a per-user level.");
+
+      if(needElevation)
+        msg +=
+            tr("\n\nNote that RenderDoc needs to elevate permissions to update the registration "
+               "regardless.");
+      else
+        msg +=
+            tr("\n\nNote that RenderDoc will need to elevate permissions to register at system "
+               "level.");
+
+      QMessageBox::StandardButton elevate =
+          RDDialog::question(this, tr("Install at system level"), msg, RDDialog::YesNoCancel);
 
       if(elevate == QMessageBox::Yes)
-        admin = true;
+        admin = system = true;
       else if(elevate == QMessageBox::No)
-        admin = false;
+        system = false;
 
       run = (elevate != QMessageBox::Cancel);
     }
-    // otherwise run non-elevated
-    else
-    {
-      run = true;
-    }
 
     if(run)
     {
+      auto regComplete = [this, admin]() {
+        bool needReg = RENDERDOC_NeedVulkanLayerRegistration(NULL);
+        ui->vulkanLayerWarn->setVisible(needReg);
+
+#if !defined(Q_OS_LINUX)
+        // can't alert the user on linux because the command might still be running - there's
+        // seemingly no portable way to wait for the command to finish.
+        if(needReg)
+        {
+          QString err = tr("Vulkan layer registration failed for unknown reasons.");
+
+          if(admin)
+            err += tr(" Ensure that the elevation to admin permissions succeeded.");
+
+          RDDialog::critical(this, tr("Layer registration failed"), err);
+        }
+#endif
+      };
+
       if(admin)
       {
 // linux sometimes can't run GUI apps as root, so we have to run renderdoccmd. Check that it's
@@ -462,25 +513,40 @@ void CaptureDialog::vulkanLayerWarn_mouseClick()
           // it's in the path, we can continue
         }
 
-        RunProcessAsAdmin(cmd, QStringList() << lit("vulkanregister") << lit("--system"), this,
-                          true, [this]() { ui->vulkanLayerWarn->setVisible(false); });
+        QStringList renderdoccmdParams;
+
+        renderdoccmdParams << lit("vulkanregister");
+        if(system)
+          renderdoccmdParams << lit("--system");
+
+        if(!RunProcessAsAdmin(cmd, renderdoccmdParams, this, true, regComplete))
+          regComplete();
 #else
-        RunProcessAsAdmin(qApp->applicationFilePath(),
-                          QStringList() << lit("--install_vulkan_layer") << lit("root"), this,
-                          false, [this]() { ui->vulkanLayerWarn->setVisible(false); });
+        QStringList qrenderdocParams;
+
+        qrenderdocParams << lit("--install_vulkan_layer");
+        if(system)
+          qrenderdocParams << lit("root");
+
+        if(!RunProcessAsAdmin(qApp->applicationFilePath(), qrenderdocParams, this, false, regComplete))
+          regComplete();
 #endif
         return;
       }
       else
       {
         QProcess *process = new QProcess;
-        process->start(qApp->applicationFilePath(), QStringList() << lit("--install_vulkan_layer")
-                                                                  << lit("user"));
+        process->start(qApp->applicationFilePath(), QStringList()
+                                                        << lit("--install_vulkan_layer")
+                                                        << (system ? lit("root") : lit("user")));
         process->waitForFinished(300);
 
         // when the process exits, delete
-        QObject::connect(process, OverloadedSlot<int>::of(&QProcess::finished),
-                         [process](int exitCode) { process->deleteLater(); });
+        QObject::connect(process, OverloadedSlot<int, QProcess::ExitStatus>::of(&QProcess::finished),
+                         [regComplete, process](int exitCode, QProcess::ExitStatus) {
+                           process->deleteLater();
+                           regComplete();
+                         });
       }
     }
 
@@ -495,7 +561,7 @@ void CaptureDialog::CheckAndroidSetup(QString &filename)
 
   LambdaThread *scan = new LambdaThread([this, filename]() {
 
-    rdcstr host = m_Ctx.Replay().CurrentRemote()->hostname;
+    rdcstr host = m_Ctx.Replay().CurrentRemote().Hostname();
     RENDERDOC_CheckAndroidPackage(host.c_str(), filename.toUtf8().data(), &m_AndroidFlags);
 
     const bool debuggable = bool(m_AndroidFlags & AndroidFlags::Debuggable);
@@ -519,6 +585,7 @@ void CaptureDialog::CheckAndroidSetup(QString &filename)
     }
   });
 
+  scan->setName(lit("CheckAndroidSetup"));
   scan->start();
   scan->deleteLater();
 }
@@ -527,9 +594,9 @@ void CaptureDialog::androidWarn_mouseClick()
 {
   QString exe = ui->exePath->text();
 
-  const RemoteHost *remote = m_Ctx.Replay().CurrentRemote();
+  RemoteHost remote = m_Ctx.Replay().CurrentRemote();
 
-  if(!remote)
+  if(!remote.IsValid())
   {
     RDDialog::critical(this, tr("Android server disconnected"),
                        tr("You've been disconnected from the android server.\n\n"
@@ -537,7 +604,7 @@ void CaptureDialog::androidWarn_mouseClick()
     return;
   }
 
-  rdcstr host = remote->hostname;
+  rdcstr host = remote.Hostname();
 
   QString caption = tr("Application is not debuggable");
 
@@ -597,6 +664,7 @@ Would you like RenderDoc to try patching your package?
       }
     });
 
+    patch->setName(lit("Android patch"));
     patch->start();
     // wait a few ms before popping up a progress bar
     patch->wait(500);
@@ -642,9 +710,9 @@ void CaptureDialog::on_exePathBrowse_clicked()
     initDir = dir.absolutePath();
     initExe = f.fileName();
   }
-  else if(m_Ctx.Replay().CurrentRemote())
+  else if(m_Ctx.Replay().CurrentRemote().IsValid())
   {
-    initDir = m_Ctx.Replay().CurrentRemote()->lastCapturePath;
+    initDir = m_Ctx.Replay().CurrentRemote().LastCapturePath();
   }
   else if(!m_Ctx.Config().LastCapturePath.isEmpty())
   {
@@ -655,7 +723,7 @@ void CaptureDialog::on_exePathBrowse_clicked()
 
   QString filename;
 
-  if(m_Ctx.Replay().CurrentRemote())
+  if(m_Ctx.Replay().CurrentRemote().IsValid())
   {
     VirtualFileDialog vfd(m_Ctx, initDir, this);
     RDDialog::show(&vfd);
@@ -670,7 +738,8 @@ void CaptureDialog::on_exePathBrowse_clicked()
   {
     SetExecutableFilename(filename);
 
-    if(m_Ctx.Replay().CurrentRemote() && m_Ctx.Replay().CurrentRemote()->IsADB())
+    if(m_Ctx.Replay().CurrentRemote().Protocol() &&
+       m_Ctx.Replay().CurrentRemote().Protocol()->GetProtocolName() == "adb")
     {
       CheckAndroidSetup(filename);
     }
@@ -683,9 +752,9 @@ void CaptureDialog::on_workDirBrowse_clicked()
 
   if(initDir.isEmpty())
   {
-    if(m_Ctx.Replay().CurrentRemote())
+    if(m_Ctx.Replay().CurrentRemote().IsValid())
     {
-      initDir = m_Ctx.Replay().CurrentRemote()->lastCapturePath;
+      initDir = m_Ctx.Replay().CurrentRemote().LastCapturePath();
     }
     else if(!QDir(initDir).exists())
     {
@@ -701,7 +770,7 @@ void CaptureDialog::on_workDirBrowse_clicked()
 
   QString dir;
 
-  if(m_Ctx.Replay().CurrentRemote())
+  if(m_Ctx.Replay().CurrentRemote().IsValid())
   {
     VirtualFileDialog vfd(m_Ctx, initDir, this);
     vfd.setDirBrowse();
@@ -854,7 +923,7 @@ void CaptureDialog::on_saveSettings_clicked()
     if(dirinfo.exists())
     {
       SaveSettings(filename);
-      AddRecentFile(m_Ctx.Config().RecentCaptureSettings, filename, 10);
+      AddRecentFile(m_Ctx.Config().RecentCaptureSettings, filename);
       m_Main->PopulateRecentCaptureSettings();
     }
   }
@@ -868,7 +937,7 @@ void CaptureDialog::on_loadSettings_clicked()
   if(!filename.isEmpty() && QFileInfo::exists(filename))
   {
     LoadSettings(filename);
-    AddRecentFile(m_Ctx.Config().RecentCaptureSettings, filename, 10);
+    AddRecentFile(m_Ctx.Config().RecentCaptureSettings, filename);
   }
 }
 
@@ -1013,12 +1082,12 @@ void CaptureDialog::SetExecutableFilename(const rdcstr &filename)
 {
   QString fn = filename;
 
-  if(!m_Ctx.Replay().CurrentRemote())
+  if(!m_Ctx.Replay().CurrentRemote().IsValid())
     fn = QDir::toNativeSeparators(QFileInfo(fn).absoluteFilePath());
 
   ui->exePath->setText(fn);
 
-  if(m_Ctx.Replay().CurrentRemote())
+  if(m_Ctx.Replay().CurrentRemote().IsValid())
   {
     // remove the filename itself before setting the last capture path. Try /, then \ as path
     // separator. If no separators are found, there is no path to set.
@@ -1037,7 +1106,7 @@ void CaptureDialog::SetExecutableFilename(const rdcstr &filename)
         fn = QString();
     }
 
-    m_Ctx.Replay().CurrentRemote()->lastCapturePath = fn;
+    m_Ctx.Replay().CurrentRemote().SetLastCapturePath(fn);
   }
   else
   {
@@ -1114,9 +1183,8 @@ void CaptureDialog::UpdateGlobalHook()
 
 void CaptureDialog::UpdateRemoteHost()
 {
-  const RemoteHost *host = m_Ctx.Replay().CurrentRemote();
-
-  if(host && host->IsADB())
+  if(m_Ctx.Replay().CurrentRemote().Protocol() &&
+     m_Ctx.Replay().CurrentRemote().Protocol()->GetProtocolName() == "adb")
     ui->cmdLineLabel->setText(tr("Intent Arguments"));
   else
     ui->cmdLineLabel->setText(tr("Command-line Arguments"));
@@ -1180,7 +1248,7 @@ void CaptureDialog::TriggerCapture()
     }
 
     // for non-remote captures, check the executable locally
-    if(!m_Ctx.Replay().CurrentRemote())
+    if(!m_Ctx.Replay().CurrentRemote().IsValid())
     {
       if(!QFileInfo::exists(exe))
       {
@@ -1193,7 +1261,7 @@ void CaptureDialog::TriggerCapture()
     QString workingDir;
 
     // for non-remote captures, check the directory locally
-    if(m_Ctx.Replay().CurrentRemote())
+    if(m_Ctx.Replay().CurrentRemote().IsValid())
     {
       workingDir = ui->workDirPath->text();
     }
@@ -1209,6 +1277,22 @@ void CaptureDialog::TriggerCapture()
 
     PopulateMostRecent();
 
+    if(m_Ctx.Replay().CurrentRemote().Protocol() &&
+       m_Ctx.Replay().CurrentRemote().Protocol()->GetProtocolName() == "adb")
+    {
+      cmdLine = cmdLine.trimmed();
+
+      if(!cmdLine.isEmpty() && cmdLine[0] != QLatin1Char('-'))
+      {
+        RDDialog::critical(this, tr("Invalid intent arguments"),
+                           tr("Invalid intent arguments: %1\n"
+                              "The intent arguments must include the full parameters e.g. "
+                              "--es args \"my arguments\"")
+                               .arg(cmdLine));
+        return;
+      }
+    }
+
     m_CaptureCallback(exe, workingDir, cmdLine, Settings().environment, Settings().options,
                       [this](LiveCapture *live) {
                         if(ui->queueFrameCap->isChecked())
