@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -193,7 +193,7 @@ void LiveCapture::QueueCapture(int frameNumber, int numFrames)
 {
   m_QueueCaptureFrameNum = frameNumber;
   m_CaptureNumFrames = numFrames;
-  m_QueueCapture = true;
+  m_QueueCapture.release();
 }
 
 void LiveCapture::showEvent(QShowEvent *event)
@@ -275,13 +275,13 @@ void LiveCapture::on_queueCap_clicked()
 {
   m_CaptureNumFrames = (int)ui->numFrames->value();
   m_QueueCaptureFrameNum = (int)ui->captureFrame->value();
-  m_QueueCapture = true;
+  m_QueueCapture.release();
 }
 
 void LiveCapture::on_triggerImmediateCapture_clicked()
 {
-  m_TriggerCapture = true;
   m_CaptureNumFrames = (int)ui->numFrames->value();
+  m_TriggerCapture.release();
 }
 
 void LiveCapture::on_cycleActiveWindow_clicked()
@@ -355,10 +355,30 @@ void LiveCapture::saveCapture_triggered()
       if(path.endsWith(lit(".rdc")))
         path.chop(4);
 
+      // don't save duplicates if we have multiple captures from the same frame (possible if the
+      // application is not presenting at all and using the API to capture)
+      QMap<uint32_t, uint32_t> existingFiles;
+
       for(QListWidgetItem *item : ui->captures->selectedItems())
       {
         Capture *cap = GetCapture(item);
-        saveCapture(cap, QFormatStr("%1-frame%2.rdc").arg(path).arg(cap->frameNumber));
+
+        QString filename = QFormatStr("%1-frame%2").arg(path).arg(cap->frameNumber);
+        if(cap->frameNumber == ~0U)
+          filename = QFormatStr("%1-capture").arg(path);
+
+        if(existingFiles.contains(cap->frameNumber))
+        {
+          filename += QFormatStr("_%1").arg(existingFiles[cap->frameNumber]);
+          existingFiles[cap->frameNumber]++;
+        }
+        else
+        {
+          // start on 2 next time
+          existingFiles[cap->frameNumber] = 2;
+        }
+
+        saveCapture(cap, QFormatStr("%1.rdc").arg(filename));
       }
     }
   }
@@ -396,6 +416,11 @@ void LiveCapture::deleteCapture_triggered()
         {
           m_Ctx.Replay().DeleteCapture(cap->path, cap->local);
         }
+
+        if(cap->saved || cap->local)
+        {
+          m_Main->RemoveRecentCapture(cap->path);
+        }
       }
     }
 
@@ -508,10 +533,10 @@ void LiveCapture::captureCountdownTick()
 
   if(m_CaptureCounter == 0)
   {
-    m_TriggerCapture = true;
     m_CaptureNumFrames = (int)ui->numFrames->value();
     ui->triggerDelayedCapture->setEnabled(true);
     ui->triggerDelayedCapture->setText(tr("Trigger After Delay"));
+    m_TriggerCapture.release();
   }
   else
   {
@@ -647,8 +672,12 @@ QString LiveCapture::MakeText(Capture *cap)
     text += tr(" (Remote)");
 
   text += lit("\n") + cap->api;
-  text += tr("\nFrame #%1 ").arg(cap->frameNumber) +
-          cap->timestamp.toString(lit("yyyy-MM-dd HH:mm:ss"));
+  if(cap->frameNumber == ~0U)
+    text += tr("\nUser-defined Capture ");
+  else
+    text += tr("\nFrame #%1 ").arg(cap->frameNumber);
+
+  text += cap->timestamp.toString(lit("yyyy-MM-dd HH:mm:ss"));
 
   return text;
 }
@@ -682,10 +711,14 @@ bool LiveCapture::checkAllowClose()
 
     if(!suppressRemoteWarning && !notoall)
     {
+      QString frameName = tr("Frame #%1").arg(cap->frameNumber);
+      if(cap->frameNumber == ~0U)
+        frameName = tr("User-defined Capture");
+
       res = RDDialog::question(this, tr("Unsaved capture"),
-                               tr("Save this capture '%1 Frame #%2' at %3?")
+                               tr("Save this capture '%1 %2' at %3?")
                                    .arg(cap->name)
-                                   .arg(cap->frameNumber)
+                                   .arg(frameName)
                                    .arg(cap->timestamp.toString(lit("HH:mm:ss"))),
                                msgFlags);
 
@@ -705,9 +738,7 @@ bool LiveCapture::checkAllowClose()
     // we either have to save or delete the capture. Make sure that if it's remote that we are able
     // to by having an active connection or replay context on that host.
     if(suppressRemoteWarning == false && (!m_Connection || !m_Connection->Connected()) &&
-       !cap->local && (!m_Ctx.Replay().CurrentRemote() ||
-                       QString(m_Ctx.Replay().CurrentRemote()->hostname) != m_Hostname ||
-                       !m_Ctx.Replay().CurrentRemote()->connected))
+       !cap->local && m_Ctx.Replay().CurrentRemote().Hostname() != rdcstr(m_Hostname))
     {
       QMessageBox::StandardButton res2 = RDDialog::question(
           this, tr("No active replay context"),
@@ -752,9 +783,7 @@ void LiveCapture::openCapture(Capture *cap)
 {
   cap->opened = true;
 
-  if(!cap->local && (!m_Ctx.Replay().CurrentRemote() ||
-                     QString(m_Ctx.Replay().CurrentRemote()->hostname) != m_Hostname ||
-                     !m_Ctx.Replay().CurrentRemote()->connected))
+  if(!cap->local && m_Ctx.Replay().CurrentRemote().Hostname() != rdcstr(m_Hostname))
   {
     RDDialog::critical(
         this, tr("No active replay context"),
@@ -764,14 +793,19 @@ void LiveCapture::openCapture(Capture *cap)
     return;
   }
 
-  m_Main->LoadCapture(cap->path, !cap->saved, cap->local);
+  m_Main->LoadCapture(cap->path, m_Ctx.Config().DefaultReplayOptions, !cap->saved, cap->local);
 }
 
 bool LiveCapture::saveCapture(Capture *cap, QString path)
 {
   if(path.isEmpty())
+  {
     path = m_Main->GetSavePath();
 
+    if(path.isEmpty())
+      return false;
+  }
+
   if(QString(m_Ctx.GetCaptureFilename()) == path)
   {
     RDDialog::critical(this, tr("Cannot save"), tr("Can't overwrite currently open capture at %1\n"
@@ -783,76 +817,72 @@ bool LiveCapture::saveCapture(Capture *cap, QString path)
   // we copy the temp capture to the desired path, but the capture item remains referring to the
   // temp path.
   // This ensures that if the user deletes the saved path we can still open or re-save it.
-  if(!path.isEmpty())
+  if(cap->local)
   {
-    if(cap->local)
+    QFile src(cap->path);
+    QFile dst(path);
+
+    // remove any existing file, the user was already prompted to overwrite
+    if(dst.exists())
     {
-      QFile src(cap->path);
-      QFile dst(path);
-
-      // remove any existing file, the user was already prompted to overwrite
-      if(dst.exists())
-      {
-        if(!dst.remove())
-        {
-          RDDialog::critical(this, tr("Cannot save"),
-                             tr("Couldn't remove file at %1\n%2").arg(path).arg(dst.errorString()));
-          return false;
-        }
-      }
-
-      if(!src.copy(path))
+      if(!dst.remove())
       {
         RDDialog::critical(this, tr("Cannot save"),
-                           tr("Couldn't copy file to %1\n%2").arg(path).arg(src.errorString()));
+                           tr("Couldn't remove file at %1\n%2").arg(path).arg(dst.errorString()));
         return false;
       }
     }
-    else if(m_Connection && m_Connection->Connected())
+
+    if(!src.copy(path))
     {
-      // if we have a current live connection, prefer using it
-      m_CopyCaptureLocalPath = path;
-      m_CopyCaptureID = cap->remoteID;
+      RDDialog::critical(this, tr("Cannot save"),
+                         tr("Couldn't copy file to %1\n%2").arg(path).arg(src.errorString()));
+      return false;
     }
-    else
+  }
+  else if(m_Connection && m_Connection->Connected())
+  {
+    // if we have a current live connection, prefer using it
+    m_CopyCaptureLocalPath = path;
+    m_CopyCaptureID = cap->remoteID;
+
+    m_CopyCapture.release();
+  }
+  else
+  {
+    if(m_Ctx.Replay().CurrentRemote().Hostname() != rdcstr(m_Hostname))
     {
-      if(!m_Ctx.Replay().CurrentRemote() ||
-         QString(m_Ctx.Replay().CurrentRemote()->hostname) != m_Hostname ||
-         !m_Ctx.Replay().CurrentRemote()->connected)
-      {
-        RDDialog::critical(this, tr("No active replay context"),
-                           tr("This capture is on remote host %1 and there is no active replay "
-                              "context on that host.\n") +
-                               tr("Without an active replay context the capture cannot be saved, "
-                                  "try switching to a replay context on %1.")
-                                   .arg(m_Hostname));
-        return false;
-      }
-
-      m_Ctx.Replay().CopyCaptureFromRemote(cap->path, path, this);
-
-      if(!QFile::exists(path))
-      {
-        RDDialog::critical(this, tr("Cannot save"),
-                           tr("File couldn't be transferred from remote host"));
-        return false;
-      }
-
-      m_Ctx.Replay().DeleteCapture(cap->path, false);
+      RDDialog::critical(this, tr("No active replay context"),
+                         tr("This capture is on remote host %1 and there is no active replay "
+                            "context on that host.\n") +
+                             tr("Without an active replay context the capture cannot be saved, "
+                                "try switching to a replay context on %1.")
+                                 .arg(m_Hostname));
+      return false;
     }
 
-    // delete the temporary copy
-    if(!cap->saved)
-      m_Ctx.Replay().DeleteCapture(cap->path, cap->local);
+    m_Ctx.Replay().CopyCaptureFromRemote(cap->path, path, this);
 
-    cap->saved = true;
-    cap->path = path;
-    AddRecentFile(m_Ctx.Config().RecentCaptureFiles, path, 10);
-    m_Main->PopulateRecentCaptureFiles();
-    return true;
+    if(!QFile::exists(path))
+    {
+      RDDialog::critical(this, tr("Cannot save"),
+                         tr("File couldn't be transferred from remote host"));
+      return false;
+    }
+
+    m_Ctx.Replay().DeleteCapture(cap->path, false);
   }
 
-  return false;
+  // delete the temporary copy
+  if(!cap->saved)
+    m_Ctx.Replay().DeleteCapture(cap->path, cap->local);
+
+  m_Main->RemoveRecentCapture(cap->path);
+  cap->saved = true;
+  cap->path = path;
+  AddRecentFile(m_Ctx.Config().RecentCaptureFiles, path);
+  m_Main->PopulateRecentCaptureFiles();
+  return true;
 }
 
 void LiveCapture::cleanItems()
@@ -879,6 +909,11 @@ void LiveCapture::cleanItems()
         {
           m_Ctx.Replay().DeleteCapture(cap->path, cap->local);
         }
+
+        if(cap->saved || cap->local)
+        {
+          m_Main->RemoveRecentCapture(cap->path);
+        }
       }
     }
 
@@ -1009,7 +1044,7 @@ void LiveCapture::captureAdded(const NewCaptureData &newCapture)
     cap->api = QString::fromUtf8(m_Connection->GetAPI());
 
   cap->timestamp =
-      QDateTime(QDate(1970, 1, 1), QTime(0, 0, 0)).addSecs(newCapture.timestamp).toLocalTime();
+      QDateTime(QDate(1970, 1, 1), QTime(0, 0, 0), Qt::UTC).addSecs(newCapture.timestamp).toLocalTime();
 
   cap->thumb = QImage(newCapture.thumbnail.data(), newCapture.thumbWidth, newCapture.thumbHeight,
                       newCapture.thumbWidth * 3, QImage::Format_RGB888)
@@ -1052,12 +1087,14 @@ void LiveCapture::connectionClosed()
       // to this machine as a remote context
       if(!cap->local)
       {
-        if(!m_Ctx.Replay().CurrentRemote() ||
-           QString(m_Ctx.Replay().CurrentRemote()->hostname) != m_Hostname ||
-           !m_Ctx.Replay().CurrentRemote()->connected)
+        if(m_Ctx.Replay().CurrentRemote().Hostname() != rdcstr(m_Hostname))
           return;
       }
 
+      // don't close if a dialog is open
+      if(QApplication::activeModalWidget() || QApplication::activePopupWidget())
+        return;
+
       if(cap->opened)
         return;
 
@@ -1147,22 +1184,20 @@ void LiveCapture::connectionThreadEntry()
 
   while(m_Connection && m_Connection->Connected())
   {
-    if(m_TriggerCapture)
+    if(m_TriggerCapture.tryAcquire())
     {
       m_Connection->TriggerCapture((uint)m_CaptureNumFrames);
-      m_TriggerCapture = false;
       m_CaptureNumFrames = 1;
     }
 
-    if(m_QueueCapture)
+    if(m_QueueCapture.tryAcquire())
     {
       m_Connection->QueueCapture((uint32_t)m_QueueCaptureFrameNum, (uint32_t)m_CaptureNumFrames);
-      m_QueueCapture = false;
       m_QueueCaptureFrameNum = 0;
       m_CaptureNumFrames = 1;
     }
 
-    if(!m_CopyCaptureLocalPath.isEmpty())
+    if(m_CopyCapture.tryAcquire())
     {
       m_Connection->CopyCapture(m_CopyCaptureID, m_CopyCaptureLocalPath.toUtf8().data());
       m_CopyCaptureLocalPath = QString();
@@ -1280,6 +1315,12 @@ void LiveCapture::connectionThreadEntry()
     }
   }
 
+  if(m_Connection)
+  {
+    m_Connection->Shutdown();
+    m_Connection = NULL;
+  }
+
   GUIInvoke::call(this, [this]() {
     ui->connectionStatus->setText(tr("Closed"));
     ui->connectionIcon->setPixmap(Pixmaps::disconnect(ui->connectionIcon));
