@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -50,6 +50,7 @@ enum class CounterFamily
   AMD,
   Intel,
   NVIDIA,
+  VulkanExtended,
 };
 
 CounterFamily GetCounterFamily(GPUCounter counter)
@@ -66,6 +67,10 @@ CounterFamily GetCounterFamily(GPUCounter counter)
   {
     return CounterFamily::NVIDIA;
   }
+  else if(IsVulkanExtendedCounter(counter))
+  {
+    return CounterFamily::VulkanExtended;
+  }
 
   return CounterFamily::Generic;
 }
@@ -78,12 +83,13 @@ QString ToString(CounterFamily family)
     case CounterFamily::Generic: return lit("Generic");
     case CounterFamily::Intel: return lit("Intel");
     case CounterFamily::NVIDIA: return lit("NVIDIA");
+    case CounterFamily::VulkanExtended: return lit("Vulkan Extended");
     case CounterFamily::Unknown: return lit("Unknown");
   }
 
   return QString();
 }
-}
+}    // namespace
 
 const int PerformanceCounterSelection::CounterDescriptionRole = Qt::UserRole + 1;
 const int PerformanceCounterSelection::CounterIdRole = Qt::UserRole + 2;
@@ -176,10 +182,10 @@ PerformanceCounterSelection::PerformanceCounterSelection(ICaptureContext &ctx,
             }
           });
 
-  connect(ui->save, &QPushButton::pressed, this, &PerformanceCounterSelection::Save);
-  connect(ui->load, &QPushButton::pressed, this, &PerformanceCounterSelection::Load);
-  connect(ui->sampleCounters, &QPushButton::pressed, this, &PerformanceCounterSelection::accept);
-  connect(ui->cancel, &QPushButton::pressed, this, &PerformanceCounterSelection::reject);
+  connect(ui->save, &QPushButton::clicked, this, &PerformanceCounterSelection::Save);
+  connect(ui->load, &QPushButton::clicked, this, &PerformanceCounterSelection::Load);
+  connect(ui->sampleCounters, &QPushButton::clicked, this, &PerformanceCounterSelection::accept);
+  connect(ui->cancel, &QPushButton::clicked, this, &PerformanceCounterSelection::reject);
 
   connect(ui->counterTree, &RDTreeWidget::itemChanged, [this](RDTreeWidgetItem *item, int) -> void {
     const QVariant d = item->data(0, CounterIdRole);
@@ -272,7 +278,7 @@ void PerformanceCounterSelection::SetCounters(const QVector<CounterDescription>
   RDTreeWidgetItem *currentRoot = NULL;
   CounterFamily currentFamily = CounterFamily::Unknown;
 
-  std::unordered_map<std::string, RDTreeWidgetItem *> categories;
+  QMap<rdcstr, RDTreeWidgetItem *> categories;
 
   for(const CounterDescription &desc : descriptions)
   {
@@ -295,7 +301,7 @@ void PerformanceCounterSelection::SetCounters(const QVector<CounterDescription>
 
     RDTreeWidgetItem *categoryItem = NULL;
 
-    const std::string category = desc.category;
+    const rdcstr category = desc.category;
     auto categoryIterator = categories.find(category);
 
     if(categoryIterator == categories.end())
@@ -311,7 +317,7 @@ void PerformanceCounterSelection::SetCounters(const QVector<CounterDescription>
     }
     else
     {
-      categoryItem = categoryIterator->second;
+      categoryItem = categoryIterator.value();
     }
 
     RDTreeWidgetItem *counterItem = new RDTreeWidgetItem();
