@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,7 @@
 #include "Code/QRDUtils.h"
 #include "Styles/StyleData.h"
 #include "Widgets/OrderedListEditor.h"
+#include "Widgets/ReplayOptionsSelector.h"
 #include "CaptureDialog.h"
 #include "ui_SettingsDialog.h"
 
@@ -38,6 +39,10 @@ SettingsDialog::SettingsDialog(ICaptureContext &ctx, QWidget *parent)
 {
   ui->setupUi(this);
 
+  m_ReplayOptions = new ReplayOptionsSelector(m_Ctx, false, this);
+
+  ui->replayOptionsLayout->insertWidget(0, m_ReplayOptions);
+
   QString styleChooseTooltip = ui->UIStyle->toolTip();
 
   for(int i = 0; i < StyleData::numAvailable; i++)
@@ -64,7 +69,7 @@ SettingsDialog::SettingsDialog(ICaptureContext &ctx, QWidget *parent)
   }
 
   ui->pages->clearSelection();
-  ui->pages->setItemSelected(ui->pages->item(0), true);
+  ui->pages->item(0)->setSelected(true);
   ui->tabWidget->setCurrentIndex(0);
 
   ui->pages->setMinimumWidth(ui->pages->sizeHintForColumn(0));
@@ -192,6 +197,9 @@ SettingsDialog::SettingsDialog(ICaptureContext &ctx, QWidget *parent)
 
 SettingsDialog::~SettingsDialog()
 {
+  m_Ctx.Config().DefaultReplayOptions = m_ReplayOptions->options();
+  m_Ctx.Config().Save();
+
   delete ui;
 }
 
@@ -218,7 +226,7 @@ void SettingsDialog::on_pages_itemSelectionChanged()
 
   if(sel.empty())
   {
-    ui->pages->setItemSelected(ui->pages->item(ui->tabWidget->currentIndex()), true);
+    ui->pages->item(ui->tabWidget->currentIndex())->setSelected(true);
   }
   else
   {
@@ -972,4 +980,4 @@ void SettingsDialog::on_UIStyle_currentIndexChanged(int index)
   }
 
   m_Ctx.Config().Save();
-}
\ No newline at end of file
+}
