@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -125,6 +125,10 @@ public:
         }
       }
     }
+    else if(roots.size() == 1 && roots[0]->file.filename == "")
+    {
+      normPath.insert(0, QLatin1Char('/'));
+    }
 
     // normPath is now of the form /subdir1/subdir2/subdir3/...
     // with ret pointing to the root directory (trivial on unix)
@@ -323,7 +327,7 @@ public:
             {
               if(node->file.lastmod == 0)
                 return QVariant();
-              return QDateTime::fromTime_t(node->file.lastmod);
+              return QDateTime(QDate(1970, 1, 1), QTime(0, 0, 0), Qt::UTC).addSecs(node->file.lastmod);
             }
             default: break;
           }
@@ -448,37 +452,38 @@ private:
     if(!(node->file.flags & PathProperty::Directory))
       return;
 
-    Renderer.ListFolder(
-        makePath(node), true, [node](const rdcstr &path, const rdcarray<PathEntry> &files) {
+    Renderer.ListFolder(makePath(node), true, [node](const rdcstr &path,
+                                                     const rdcarray<PathEntry> &files) {
 
-          if(files.count() == 1 && (files[0].flags & PathProperty::ErrorAccessDenied))
-          {
-            node->file.flags |= PathProperty::ErrorAccessDenied;
-            return;
-          }
+      if(files.count() == 1 && (files[0].flags & PathProperty::ErrorAccessDenied))
+      {
+        node->file.flags |= PathProperty::ErrorAccessDenied;
+        return;
+      }
 
-          QVector<PathEntry> sortedFiles;
-          sortedFiles.reserve(files.count());
-          for(const PathEntry &f : files)
-            sortedFiles.push_back(f);
+      QVector<PathEntry> sortedFiles;
+      sortedFiles.reserve(files.count());
+      for(const PathEntry &f : files)
+        sortedFiles.push_back(f);
 
-          qSort(sortedFiles.begin(), sortedFiles.end(), [](const PathEntry &a, const PathEntry &b) {
-            // sort greater than so that files with the flag are sorted before those without
-            if((a.flags & PathProperty::Directory) != (b.flags & PathProperty::Directory))
-              return (a.flags & PathProperty::Directory) > (b.flags & PathProperty::Directory);
+      std::sort(sortedFiles.begin(), sortedFiles.end(), [](const PathEntry &a, const PathEntry &b) {
+        // sort greater than so that files with the flag are sorted before those without
+        if((a.flags & PathProperty::Directory) != (b.flags & PathProperty::Directory))
+          return (a.flags & PathProperty::Directory) > (b.flags & PathProperty::Directory);
 
-            return strcmp(a.filename.c_str(), b.filename.c_str()) < 0;
-          });
+        return strcmp(a.filename.c_str(), b.filename.c_str()) < 0;
+      });
 
-          for(int i = 0; i < sortedFiles.count(); i++)
-          {
-            FSNode *child = new FSNode();
-            child->parent = node;
-            child->parentIndex = i;
-            child->file = sortedFiles[i];
-            node->children.push_back(child);
-          }
-        });
+      for(int i = 0; i < sortedFiles.count(); i++)
+      {
+        FSNode *child = new FSNode();
+        child->parent = node;
+        child->parentIndex = i;
+        child->file = sortedFiles[i];
+        child->populated = !(child->file.flags & PathProperty::Directory);
+        node->children.push_back(child);
+      }
+    });
   }
 };
 
@@ -491,7 +496,7 @@ public:
     return qMin(maxColCount, sourceModel()->columnCount(parent));
   }
 
-  void refresh() { QSortFilterProxyModel::filterChanged(); }
+  void refresh() { QSortFilterProxyModel::invalidateFilter(); }
   int maxColCount = INT_MAX;
   bool showFiles = true;
   bool showDirs = true;
@@ -604,6 +609,8 @@ VirtualFileDialog::VirtualFileDialog(ICaptureContext &ctx, QString initialDirect
 
   QObject::connect(ui->fileList->selectionModel(), &QItemSelectionModel::selectionChanged, this,
                    &VirtualFileDialog::fileList_selectionChanged);
+  QObject::connect(ui->dirList->selectionModel(), &QItemSelectionModel::selectionChanged, this,
+                   &VirtualFileDialog::dirList_selectionChanged);
 }
 
 VirtualFileDialog::~VirtualFileDialog()
@@ -712,6 +719,14 @@ void VirtualFileDialog::on_dirList_clicked(const QModelIndex &index)
   changeCurrentDir(m_DirProxy->mapToSource(index));
 }
 
+void VirtualFileDialog::dirList_selectionChanged(const QItemSelection &selected,
+                                                 const QItemSelection &deselected)
+{
+  QModelIndexList indices = selected.indexes();
+  if(indices.count() >= 1)
+    on_dirList_clicked(indices[0]);
+}
+
 void VirtualFileDialog::on_fileList_doubleClicked(const QModelIndex &index)
 {
   bool isDir = m_FileProxy->data(index, RemoteFileModel::FileIsDirRole).toBool();
