@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -238,11 +238,15 @@ EventBrowser::~EventBrowser()
 
 void EventBrowser::OnCaptureLoaded()
 {
-  RDTreeWidgetItem *frame = new RDTreeWidgetItem(
-      {QFormatStr("Frame #%1").arg(m_Ctx.FrameInfo().frameNumber), QString(), QString(), QString()});
+  uint32_t frameNumber = m_Ctx.FrameInfo().frameNumber;
+
+  QString rootName =
+      frameNumber == ~0U ? tr("User-defined Capture") : tr("Frame #%1").arg(frameNumber);
+
+  RDTreeWidgetItem *frame = new RDTreeWidgetItem({rootName, QString(), QString(), QString()});
 
   RDTreeWidgetItem *framestart =
-      new RDTreeWidgetItem({tr("Frame Start"), lit("0"), lit("0"), QString()});
+      new RDTreeWidgetItem({tr("Capture Start"), lit("0"), lit("0"), QString()});
   framestart->setTag(QVariant::fromValue(EventItemTag(0, 0)));
 
   frame->addChild(framestart);
@@ -270,6 +274,8 @@ void EventBrowser::OnCaptureClosed()
 {
   clearBookmarks();
 
+  on_HideFindJump();
+
   ui->events->clear();
 
   ui->find->setEnabled(false);
@@ -1244,7 +1250,7 @@ void EventBrowser::ClearFindIcons(RDTreeWidgetItem *parent)
 
 void EventBrowser::ClearFindIcons()
 {
-  if(m_Ctx.IsCaptureLoaded())
+  if(m_Ctx.IsCaptureLoaded() && ui->events->topLevelItemCount() > 0)
     ClearFindIcons(ui->events->topLevelItem(0));
 }
 
@@ -1276,7 +1282,7 @@ int EventBrowser::SetFindIcons(RDTreeWidgetItem *parent, QString filter)
 
 int EventBrowser::SetFindIcons(QString filter)
 {
-  if(filter.isEmpty())
+  if(filter.isEmpty() || !m_Ctx.IsCaptureLoaded())
     return 0;
 
   return SetFindIcons(ui->events->topLevelItem(0), filter);
