@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,6 +39,7 @@
 #include "Code/QRDUtils.h"
 #include "Code/Resources.h"
 #include "Widgets/Extended/RDLabel.h"
+#include "Widgets/ReplayOptionsSelector.h"
 #include "Windows/Dialogs/AboutDialog.h"
 #include "Windows/Dialogs/CaptureDialog.h"
 #include "Windows/Dialogs/CrashDialog.h"
@@ -160,11 +161,15 @@ MainWindow::MainWindow(ICaptureContext &ctx) : QMainWindow(NULL), ui(new Ui::Mai
 
   m_RemoteProbeSemaphore.release();
   m_RemoteProbe = new LambdaThread([this]() {
-    RENDERDOC_AndroidInitialise();
+    // fetch all device protocols to start them processing
+    rdcarray<rdcstr> protocols;
+    RENDERDOC_GetSupportedDeviceProtocols(&protocols);
+    for(const rdcstr &p : protocols)
+      RENDERDOC_GetDeviceProtocolController(p);
 
     while(m_RemoteProbeSemaphore.available())
     {
-      // do a remoteProbe immediately to populate the android hosts list on startup.
+      // do a remoteProbe immediately to populate the device list on startup.
       remoteProbe();
 
       // do several small sleeps so we can respond quicker when we need to shut down
@@ -176,6 +181,7 @@ MainWindow::MainWindow(ICaptureContext &ctx) : QMainWindow(NULL), ui(new Ui::Mai
       }
     }
   });
+  m_RemoteProbe->setName(lit("Remote Probe"));
   m_RemoteProbe->start();
 
   SetTitle();
@@ -441,6 +447,47 @@ void MainWindow::on_action_Open_Capture_triggered()
     LoadFromFilename(filename, false);
 }
 
+void MainWindow::on_action_Open_Capture_with_Options_triggered()
+{
+  if(!PromptCloseCapture())
+    return;
+
+  ReplayOptionsSelector *replayOptions = new ReplayOptionsSelector(m_Ctx, true, this);
+
+  QDialog *openWithOptions = new QDialog(this);
+  openWithOptions->setWindowFlags(openWithOptions->windowFlags() & ~Qt::WindowContextHelpButtonHint);
+  openWithOptions->setWindowIcon(windowIcon());
+  openWithOptions->setWindowTitle(tr("Open Capture with Options"));
+  openWithOptions->setSizeGripEnabled(false);
+  openWithOptions->setModal(true);
+
+  QVBoxLayout l;
+  l.addWidget(replayOptions);
+  l.setMargin(3);
+  l.setSizeConstraint(QLayout::SetFixedSize);
+
+  openWithOptions->setLayout(&l);
+
+  QObject::connect(replayOptions, &ReplayOptionsSelector::canceled, openWithOptions,
+                   &QDialog::reject);
+  QObject::connect(replayOptions, &ReplayOptionsSelector::opened, openWithOptions, &QDialog::accept);
+
+  if(RDDialog::show(openWithOptions) != QDialog::Accepted)
+  {
+    openWithOptions->deleteLater();
+    return;
+  }
+
+  QString filename = replayOptions->filename();
+
+  openWithOptions->deleteLater();
+
+  if(filename.isEmpty())
+    return;
+
+  LoadCapture(filename, replayOptions->options(), false, true);
+}
+
 void MainWindow::importCapture(const CaptureFileFormat &fmt)
 {
   if(!PromptCloseCapture())
@@ -483,7 +530,7 @@ void MainWindow::LoadFromFilename(const QString &filename, bool temporary)
 
   if(ext == lit("rdc"))
   {
-    LoadCapture(filename, temporary, true);
+    LoadCapture(filename, m_Ctx.Config().DefaultReplayOptions, temporary, true);
   }
   else if(ext == lit("cap"))
   {
@@ -496,7 +543,7 @@ void MainWindow::LoadFromFilename(const QString &filename, bool temporary)
   else
   {
     // not a recognised filetype, see if we can load it anyway
-    LoadCapture(filename, temporary, true);
+    LoadCapture(filename, m_Ctx.Config().DefaultReplayOptions, temporary, true);
   }
 }
 
@@ -510,7 +557,7 @@ void MainWindow::OnCaptureTrigger(const QString &exe, const QString &workingDir,
 
   LambdaThread *th = new LambdaThread([this, exe, workingDir, cmdLine, env, opts, callback]() {
 
-    if(isCapturableAppRunningOnAndroid())
+    if(isUnshareableDeviceInUse())
     {
       RDDialog::warning(this, tr("RenderDoc is already capturing an app on this device"),
                         tr("A running app on this device is already being captured with RenderDoc. "
@@ -533,7 +580,8 @@ void MainWindow::OnCaptureTrigger(const QString &exe, const QString &workingDir,
             tr("<html>Error launching %1 for capture.\n\n"
                "Something went wrong connecting to the debugger on the Android device.\n\n"
                "This can happen if the package is not "
-               "marked as debuggable, or if another android tool such as Android Studio "
+               "marked as debuggable, if the intent arguments are badly specified, "
+               "or if another android tool such as Android Studio "
                "is interfering with the debug connection.\n\n"
                "Close <b>all</b> instances of Android Studio or other Android programs "
                "and try again.</html>")
@@ -552,13 +600,15 @@ void MainWindow::OnCaptureTrigger(const QString &exe, const QString &workingDir,
       }
 
       LiveCapture *live = new LiveCapture(
-          m_Ctx, m_Ctx.Replay().CurrentRemote() ? m_Ctx.Replay().CurrentRemote()->hostname : "",
-          m_Ctx.Replay().CurrentRemote() ? m_Ctx.Replay().CurrentRemote()->Name() : "", ret.ident,
-          this, this);
+          m_Ctx,
+          m_Ctx.Replay().CurrentRemote().IsValid() ? m_Ctx.Replay().CurrentRemote().Hostname() : "",
+          m_Ctx.Replay().CurrentRemote().IsValid() ? m_Ctx.Replay().CurrentRemote().Name() : "",
+          ret.ident, this, this);
       ShowLiveCapture(live);
       callback(live);
     });
   });
+  th->setName(lit("ExecuteAndInject"));
   th->start();
   // wait a few ms before popping up a progress bar
   th->wait(500);
@@ -614,6 +664,7 @@ void MainWindow::OnInjectTrigger(uint32_t PID, const rdcarray<EnvironmentModific
 
       LiveCapture *live = new LiveCapture(m_Ctx, QString(), QString(), ret.ident, this, this);
       ShowLiveCapture(live);
+      callback(live);
     });
   });
   th->start();
@@ -627,7 +678,8 @@ void MainWindow::OnInjectTrigger(uint32_t PID, const rdcarray<EnvironmentModific
   th->deleteLater();
 }
 
-void MainWindow::LoadCapture(const QString &filename, bool temporary, bool local)
+void MainWindow::LoadCapture(const QString &filename, const ReplayOptions &opts, bool temporary,
+                             bool local)
 {
   if(PromptCloseCapture())
   {
@@ -638,8 +690,7 @@ void MainWindow::LoadCapture(const QString &filename, bool temporary, bool local
     QString machineIdent;
     ReplaySupport support = ReplaySupport::Unsupported;
 
-    bool remoteReplay =
-        !local || (m_Ctx.Replay().CurrentRemote() && m_Ctx.Replay().CurrentRemote()->connected);
+    bool remoteReplay = !local || m_Ctx.Replay().CurrentRemote().IsConnected();
 
     if(local)
     {
@@ -662,7 +713,7 @@ void MainWindow::LoadCapture(const QString &filename, bool temporary, bool local
         return;
       }
 
-      driver = QString::fromUtf8(file->DriverName());
+      driver = file->DriverName();
       machineIdent = QString::fromUtf8(file->RecordedMachineIdent());
       support = file->LocalReplaySupport();
 
@@ -700,8 +751,7 @@ void MainWindow::LoadCapture(const QString &filename, bool temporary, bool local
                                [this]() { return contextChooser->isEnabled(); });
           }
 
-          remoteReplay =
-              (m_Ctx.Replay().CurrentRemote() && m_Ctx.Replay().CurrentRemote()->connected);
+          remoteReplay = m_Ctx.Replay().CurrentRemote().IsConnected();
 
           if(!remoteReplay)
           {
@@ -752,7 +802,7 @@ void MainWindow::LoadCapture(const QString &filename, bool temporary, bool local
         QString remoteMessage =
             tr("This capture was captured with %1 and cannot be replayed on %2.\n\n")
                 .arg(driver)
-                .arg(m_Ctx.Replay().CurrentRemote()->Name());
+                .arg(m_Ctx.Replay().CurrentRemote().Name());
 
         remoteMessage += tr("Try selecting a different remote context in the status bar.");
 
@@ -795,15 +845,11 @@ void MainWindow::LoadCapture(const QString &filename, bool temporary, bool local
       {
         ANALYTIC_SET(UIFeatures.ImageViewer, true);
       }
-      else if(!driver.isEmpty())
-      {
-        ANALYTIC_ADDUNIQ(APIs, driver);
-      }
 
-      m_Ctx.LoadCapture(fileToLoad, origFilename, temporary, local);
+      m_Ctx.LoadCapture(fileToLoad, opts, origFilename, temporary, local);
     }
 
-    if(!remoteReplay)
+    if(local && !temporary)
     {
       m_Ctx.Config().LastCaptureFilePath = QFileInfo(filename).absolutePath();
     }
@@ -868,7 +914,7 @@ bool MainWindow::PromptSaveCaptureAs()
     if(!success)
       return false;
 
-    AddRecentFile(m_Ctx.Config().RecentCaptureFiles, saveFilename, 10);
+    AddRecentFile(m_Ctx.Config().RecentCaptureFiles, saveFilename);
     PopulateRecentCaptureFiles();
     SetTitle(saveFilename);
 
@@ -982,15 +1028,29 @@ bool MainWindow::PromptCloseCapture()
   CloseCapture();
 
   if(!deletepath.isEmpty())
+  {
     m_Ctx.Replay().DeleteCapture(deletepath, caplocal);
+    RemoveRecentCapture(deletepath);
+  }
 
   return true;
 }
 
 void MainWindow::CloseCapture()
 {
+  QString path = m_Ctx.GetCaptureFilename();
+  bool local = m_Ctx.IsCaptureLocal();
+  bool temp = m_Ctx.IsCaptureTemporary();
+
   m_Ctx.CloseCapture();
 
+  if(m_OwnTempCapture && temp)
+  {
+    m_Ctx.Replay().DeleteCapture(path, local);
+    RemoveRecentCapture(path);
+    m_OwnTempCapture = false;
+  }
+
   ui->action_Save_Capture_Inplace->setEnabled(false);
   ui->action_Save_Capture_As->setEnabled(false);
   ui->menu_Export_As->setEnabled(false);
@@ -1008,8 +1068,8 @@ void MainWindow::SetTitle(const QString &filename)
     prefix += lit(" - ");
   }
 
-  if(m_Ctx.Replay().CurrentRemote())
-    prefix += tr("Remote: %1 - ").arg(m_Ctx.Replay().CurrentRemote()->Name());
+  if(m_Ctx.Replay().CurrentRemote().IsValid())
+    prefix += tr("Remote: %1 - ").arg(m_Ctx.Replay().CurrentRemote().Name());
 
   QString text = prefix + lit("RenderDoc ");
 
@@ -1018,7 +1078,7 @@ void MainWindow::SetTitle(const QString &filename)
   else
     text += tr("Unstable release (%1 - %2)")
                 .arg(lit(FULL_VERSION_STRING))
-                .arg(QString::fromLatin1(GitVersionHash));
+                .arg(QString::fromLatin1(RENDERDOC_GetCommitHash()));
 
   if(IsRunningAsAdmin())
     text += tr(" (Administrator)");
@@ -1098,6 +1158,10 @@ void MainWindow::PopulateRecentCaptureFiles()
     idx++;
 
     ui->menu_Recent_Capture_Files->setEnabled(true);
+
+    // only populate the 9 most recent, even if more exist in memory
+    if(idx == 10)
+      break;
   }
 
   ui->menu_Recent_Capture_Files->addSeparator();
@@ -1127,6 +1191,10 @@ void MainWindow::PopulateRecentCaptureSettings()
     idx++;
 
     ui->menu_Recent_Capture_Settings->setEnabled(true);
+
+    // only populate the 9 most recent, even if more exist in memory
+    if(idx == 10)
+      break;
   }
 
   ui->menu_Recent_Capture_Settings->addSeparator();
@@ -1214,6 +1282,15 @@ void MainWindow::CheckUpdates(bool forceCheck, UpdateResultMethod callback)
   }
 
 #if RENDERDOC_OFFICIAL_BUILD
+
+  // if the current version isn't the one we expected, clear any cached update state
+  if(m_Ctx.Config().CheckUpdate_CurrentVersion != MAJOR_MINOR_VERSION_STRING)
+  {
+    m_Ctx.Config().CheckUpdate_UpdateAvailable = false;
+    m_Ctx.Config().CheckUpdate_UpdateResponse = "";
+    m_Ctx.Config().CheckUpdate_CurrentVersion = MAJOR_MINOR_VERSION_STRING;
+  }
+
   QDateTime today = QDateTime::currentDateTime();
 
   // check by default every 2 days
@@ -1278,18 +1355,19 @@ void MainWindow::CheckUpdates(bool forceCheck, UpdateResultMethod callback)
                    });
 
   QObject::connect(req, &QNetworkReply::finished, [this, req, callback]() {
+    statusText->setText(QString());
+    statusProgress->setVisible(false);
+
     if(req->error() != QNetworkReply::NoError)
       return;
 
     QString response = QString::fromUtf8(req->readAll());
 
-    statusText->setText(QString());
-    statusProgress->setVisible(false);
-
     if(response.isEmpty())
     {
       m_Ctx.Config().CheckUpdate_UpdateAvailable = false;
       m_Ctx.Config().CheckUpdate_UpdateResponse = "";
+      m_Ctx.Config().CheckUpdate_CurrentVersion = lit(MAJOR_MINOR_VERSION_STRING);
       m_Ctx.Config().Save();
       SetNoUpdate();
 
@@ -1301,6 +1379,7 @@ void MainWindow::CheckUpdates(bool forceCheck, UpdateResultMethod callback)
 
     m_Ctx.Config().CheckUpdate_UpdateAvailable = true;
     m_Ctx.Config().CheckUpdate_UpdateResponse = response;
+    m_Ctx.Config().CheckUpdate_CurrentVersion = lit(MAJOR_MINOR_VERSION_STRING);
     m_Ctx.Config().Save();
     SetUpdateAvailable();
     UpdatePopup();
@@ -1491,11 +1570,18 @@ void MainWindow::show()
   QMainWindow::show();
 }
 
+void MainWindow::RemoveRecentCapture(const QString &filename)
+{
+  RemoveRecentFile(m_Ctx.Config().RecentCaptureFiles, filename);
+
+  PopulateRecentCaptureFiles();
+}
+
 void MainWindow::recentCaptureFile(const QString &filename)
 {
   if(QFileInfo::exists(filename))
   {
-    LoadCapture(filename, false, true);
+    LoadCapture(filename, m_Ctx.Config().DefaultReplayOptions, false, true);
   }
   else
   {
@@ -1505,9 +1591,7 @@ void MainWindow::recentCaptureFile(const QString &filename)
 
     if(res == QMessageBox::Yes)
     {
-      m_Ctx.Config().RecentCaptureFiles.removeOne(filename);
-
-      PopulateRecentCaptureFiles();
+      RemoveRecentCapture(filename);
     }
   }
 }
@@ -1576,61 +1660,25 @@ void MainWindow::remoteProbe()
 {
   if(!m_Ctx.IsCaptureLoaded() && !m_Ctx.IsCaptureLoading())
   {
-    GUIInvoke::call(this, [this] {
-      m_Ctx.Config().AddAndroidHosts();
-
-      // update the latest list by copy. Note this lock only protects m_ProbeRemoteHosts, not the
-      // actual RemoteHosts list itself - that is only accessed on the UI thread so is not locked.
-      {
-        QMutexLocker lock(&m_ProbeRemoteHostsLock);
-        m_ProbeRemoteHosts.clear();
-        for(RemoteHost *host : m_Ctx.Config().RemoteHosts)
-          m_ProbeRemoteHosts.push_back(*host);
-      }
-    });
+    m_Ctx.Config().UpdateEnumeratedProtocolDevices();
 
     // fetch the latest list
-    rdcarray<RemoteHost> hosts;
-    {
-      QMutexLocker lock(&m_ProbeRemoteHostsLock);
-      hosts = m_ProbeRemoteHosts;
-    }
-
-    QList<QPair<RemoteHost, RemoteHost>> updates;
+    rdcarray<RemoteHost> hosts = m_Ctx.Config().GetRemoteHosts();
 
     for(RemoteHost &host : hosts)
     {
       // don't mess with a host we're connected to - this is handled anyway
-      if(host.connected)
+      if(host.IsConnected())
         continue;
 
-      RemoteHost updated = host;
-
-      updated.CheckStatus();
+      // this will do the bulk of the status checking on this thread without holding any lock, then
+      // grab the remote host lock and update the config's host (if it's still there)
+      host.CheckStatus();
 
       // bail as soon as we notice that we're done
       if(!m_RemoteProbeSemaphore.available())
         return;
-
-      if(updated != host)
-        updates.push_back(qMakePair(host, updated));
     }
-
-    GUIInvoke::call(this, [this, updates] {
-      // this is a bit naive - it suffers from the A-B-A problem where the host could be updated
-      // then restored to its original state before we notice. In this case though we don't care
-      // about that, we just don't want to trash some important changes like the connected status
-      // changing or something.
-
-      for(const QPair<RemoteHost, RemoteHost> &u : updates)
-      {
-        for(RemoteHost *host : m_Ctx.Config().RemoteHosts)
-        {
-          if(*host == u.first)
-            *host = u.second;
-        }
-      }
-    });
   }
 }
 
@@ -1653,13 +1701,13 @@ void MainWindow::messageCheck()
 
       bool disconnected = false;
 
-      if(m_Ctx.Replay().CurrentRemote())
+      if(m_Ctx.Replay().CurrentRemote().IsValid())
       {
-        bool wasRunning = m_Ctx.Replay().CurrentRemote()->serverRunning;
+        bool wasRunning = m_Ctx.Replay().CurrentRemote().IsServerRunning();
 
         m_Ctx.Replay().PingRemote();
 
-        if(wasRunning != m_Ctx.Replay().CurrentRemote()->serverRunning)
+        if(wasRunning != m_Ctx.Replay().CurrentRemote().IsServerRunning())
         {
           qCritical() << "Remote server disconnected";
           disconnected = true;
@@ -1684,7 +1732,8 @@ void MainWindow::messageCheck()
                                 "RenderDoc to reconnect and load the capture again"));
         }
 
-        if(m_Ctx.Replay().CurrentRemote() && !m_Ctx.Replay().CurrentRemote()->serverRunning)
+        if(m_Ctx.Replay().CurrentRemote().IsValid() &&
+           !m_Ctx.Replay().CurrentRemote().IsServerRunning())
           contextChooser->setIcon(Icons::cross());
 
         if(!msgs.empty())
@@ -1703,11 +1752,11 @@ void MainWindow::messageCheck()
   }
   else if(!m_Ctx.IsCaptureLoaded() && !m_Ctx.IsCaptureLoading())
   {
-    if(m_Ctx.Replay().CurrentRemote())
+    if(m_Ctx.Replay().CurrentRemote().IsValid())
       m_Ctx.Replay().PingRemote();
 
     GUIInvoke::call(this, [this]() {
-      if(m_Ctx.Replay().CurrentRemote() && !m_Ctx.Replay().CurrentRemote()->serverRunning)
+      if(m_Ctx.Replay().CurrentRemote().IsValid() && !m_Ctx.Replay().CurrentRemote().IsServerRunning())
       {
         contextChooser->setIcon(Icons::cross());
         contextChooser->setText(tr("Replay Context: %1").arg(tr("Local")));
@@ -1727,32 +1776,35 @@ void MainWindow::FillRemotesMenu(QMenu *menu, bool includeLocalhost)
 {
   menu->clear();
 
-  for(int i = 0; i < m_Ctx.Config().RemoteHosts.count(); i++)
-  {
-    RemoteHost *host = m_Ctx.Config().RemoteHosts[i];
+  rdcarray<RemoteHost> hosts = m_Ctx.Config().GetRemoteHosts();
 
-    // add localhost at the end
-    if(host->IsLocalhost())
+  for(int i = 0; i < hosts.count(); i++)
+  {
+    RemoteHost host = hosts[i];
+
+    // add localhost at the end, skip invalid hosts
+    if(host.IsLocalhost() || !host.IsValid())
       continue;
 
     QAction *action = new QAction(menu);
 
-    action->setIcon(host->serverRunning && !host->versionMismatch ? Icons::tick() : Icons::cross());
-    if(host->connected)
-      action->setText(tr("%1 (Connected)").arg(host->Name()));
-    else if(host->serverRunning && host->versionMismatch)
-      action->setText(tr("%1 (Bad Version)").arg(host->Name()));
-    else if(host->serverRunning && host->busy)
-      action->setText(tr("%1 (Busy)").arg(host->Name()));
-    else if(host->serverRunning)
-      action->setText(tr("%1 (Online)").arg(host->Name()));
+    action->setIcon(host.IsServerRunning() && !host.IsVersionMismatch() ? Icons::tick()
+                                                                        : Icons::cross());
+    if(host.IsConnected())
+      action->setText(tr("%1 (Connected)").arg(host.Name()));
+    else if(host.IsServerRunning() && host.IsVersionMismatch())
+      action->setText(tr("%1 (Bad Version)").arg(host.Name()));
+    else if(host.IsServerRunning() && host.IsBusy())
+      action->setText(tr("%1 (Busy)").arg(host.Name()));
+    else if(host.IsServerRunning())
+      action->setText(tr("%1 (Online)").arg(host.Name()));
     else
-      action->setText(tr("%1 (Offline)").arg(host->Name()));
+      action->setText(tr("%1 (Offline)").arg(host.Name()));
     QObject::connect(action, &QAction::triggered, this, &MainWindow::switchContext);
     action->setData(i);
 
     // don't allow switching to the connected host
-    if(host->connected)
+    if(host.IsConnected())
       action->setEnabled(false);
 
     menu->addAction(action);
@@ -1774,18 +1826,18 @@ void MainWindow::FillRemotesMenu(QMenu *menu, bool includeLocalhost)
 
 void MainWindow::setRemoteHost(int hostIdx)
 {
-  RemoteHost *host = NULL;
-  if(hostIdx >= 0 && hostIdx < m_Ctx.Config().RemoteHosts.count())
-  {
-    host = m_Ctx.Config().RemoteHosts[hostIdx];
-  }
+  rdcarray<RemoteHost> hosts = m_Ctx.Config().GetRemoteHosts();
+
+  RemoteHost host;
+  if(hostIdx >= 0 && hostIdx < hosts.count())
+    host = hosts[hostIdx];
 
   for(LiveCapture *live : m_LiveCaptures)
   {
     // allow live captures to this host to stay open, that way
     // we can connect to a live capture, then switch into that
     // context
-    if(host && live->hostname() == host->hostname)
+    if(host.IsValid() && live->hostname() == host.Hostname())
       continue;
 
     if(!live->checkAllowClose())
@@ -1800,7 +1852,7 @@ void MainWindow::setRemoteHost(int hostIdx)
     // allow live captures to this host to stay open, that way
     // we can connect to a live capture, then switch into that
     // context
-    if(host && live->hostname() == host->hostname)
+    if(host.IsValid() && live->hostname() == host.Hostname())
       continue;
 
     live->cleanItems();
@@ -1809,7 +1861,7 @@ void MainWindow::setRemoteHost(int hostIdx)
 
   m_Ctx.Replay().DisconnectFromRemoteServer();
 
-  if(!host)
+  if(!host.IsValid())
   {
     contextChooser->setIcon(Icons::house());
     contextChooser->setText(tr("Replay Context: %1").arg(tr("Local")));
@@ -1825,8 +1877,8 @@ void MainWindow::setRemoteHost(int hostIdx)
   }
   else
   {
-    contextChooser->setText(tr("Replay Context: %1").arg(host->Name()));
-    contextChooser->setIcon(host->serverRunning ? Icons::connect() : Icons::disconnect());
+    contextChooser->setText(tr("Replay Context: %1").arg(host.Name()));
+    contextChooser->setIcon(host.IsServerRunning() ? Icons::connect() : Icons::disconnect());
 
     // disable until checking is done
     contextChooser->setEnabled(false);
@@ -1837,31 +1889,74 @@ void MainWindow::setRemoteHost(int hostIdx)
 
     statusText->setText(tr("Checking remote server status..."));
 
-    LambdaThread *th = new LambdaThread([this, host]() {
-      // see if the server is up
-      host->CheckStatus();
+    LambdaThread *th = new LambdaThread([ this, h = host ]() {
+      // make a mutable copy and see if the server is up
+      RemoteHost host = h;
+      host.CheckStatus();
 
-      if(host->IsADB() && !RENDERDOC_IsAndroidSupported(host->hostname.c_str()))
+      if(host.Protocol() && !host.Protocol()->IsSupported(host.Hostname()))
       {
         // check to see if we should warn the user about this unsupported android version.
-        GUIInvoke::call(this, [this]() {
+        GUIInvoke::call(this, [this, host]() {
           QDateTime today = QDateTime::currentDateTimeUtc();
           QDateTime compare = today.addDays(-21);
 
-          if(compare > m_Ctx.Config().UnsupportedAndroid_LastUpdate)
+          if(host.Protocol()->GetProtocolName() == "adb")
+          {
+            if(compare > m_Ctx.Config().UnsupportedAndroid_LastUpdate)
+            {
+              RDDialog::critical(
+                  this, tr("Unsupported Device Android Version"),
+                  tr("This device is older than Android 6.0, the minimum required version for "
+                     "RenderDoc.\n\nThis may break or cause unknown problems - use at your own "
+                     "risk."));
+            }
+
+            m_Ctx.Config().UnsupportedAndroid_LastUpdate = today;
+          }
+          else
           {
             RDDialog::critical(
-                this, tr("Unsupported Device Android Version"),
-                tr("This device is older than Android 6.0, the minimum required version for "
-                   "RenderDoc.\n\nThis may break or cause unknown problems - use at your own "
-                   "risk."));
+                this, tr("Unsupported Device"),
+                tr("This device is not able to support RenderDoc. Please consult the documentation "
+                   "for this type of device to see what the problem may be."));
           }
-
-          m_Ctx.Config().UnsupportedAndroid_LastUpdate = today;
         });
       }
 
-      if(!host->serverRunning && !host->runCommand.isEmpty())
+      if(host.Protocol() && host.IsVersionMismatch())
+      {
+        GUIInvoke::blockcall(this, [this, &host]() {
+
+          QMessageBox::StandardButton res =
+              RDDialog::question(this, tr("Unsupported version"),
+                                 tr("Remote server on %1 has an incompatible version.\n"
+                                    "Would you like to try to reinstall the version %2?")
+                                     .arg(host.Name())
+                                     .arg(lit(FULL_VERSION_STRING)),
+                                 QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
+
+          if(res == QMessageBox::Yes)
+          {
+            LambdaThread *launchthread = new LambdaThread([&host]() {
+              // since we have a protocol, try to force-launch which should attempt to reinstall.
+              host.Launch();
+
+              // update status
+              host.CheckStatus();
+            });
+            launchthread->setName(lit("Remote host launch"));
+            launchthread->start();
+
+            ShowProgressDialog(this, tr("Attempting to update remote server, please wait..."),
+                               [launchthread]() { return !launchthread->isRunning(); });
+
+            launchthread->deleteLater();
+          }
+        });
+      }
+
+      if(!host.IsServerRunning() && !host.RunCommand().isEmpty())
       {
         GUIInvoke::call(this, [this]() {
           statusText->setText(tr("Running remote server command..."));
@@ -1869,28 +1964,28 @@ void MainWindow::setRemoteHost(int hostIdx)
           statusProgress->setMaximum(0);
         });
 
-        ReplayStatus launchStatus = host->Launch();
+        ReplayStatus launchStatus = host.Launch();
         if(launchStatus != ReplayStatus::Succeeded)
         {
           showLaunchError(launchStatus);
         }
 
         // check if it's running now
-        host->CheckStatus();
+        host.CheckStatus();
 
         GUIInvoke::call(this, [this]() { statusProgress->setVisible(false); });
       }
 
       ReplayStatus status = ReplayStatus::Succeeded;
 
-      if(host->serverRunning && !host->busy)
+      if(host.IsServerRunning() && !host.IsBusy())
       {
         status = m_Ctx.Replay().ConnectToRemoteServer(host);
       }
 
       GUIInvoke::call(this, [this, host, status]() {
-        contextChooser->setIcon(host->serverRunning && !host->busy ? Icons::connect()
-                                                                   : Icons::disconnect());
+        contextChooser->setIcon(host.IsServerRunning() && !host.IsBusy() ? Icons::connect()
+                                                                         : Icons::disconnect());
 
         if(status != ReplayStatus::Succeeded)
         {
@@ -1898,22 +1993,22 @@ void MainWindow::setRemoteHost(int hostIdx)
           contextChooser->setText(tr("Replay Context: %1").arg(tr("Local")));
           statusText->setText(tr("Connection failed: %1").arg(ToQStr(status)));
         }
-        else if(host->versionMismatch)
+        else if(host.IsVersionMismatch())
         {
           statusText->setText(
               tr("Remote server is not running RenderDoc %1").arg(lit(FULL_VERSION_STRING)));
         }
-        else if(host->busy)
+        else if(host.IsBusy())
         {
           statusText->setText(tr("Remote server in use elsewhere"));
         }
-        else if(host->serverRunning)
+        else if(host.IsServerRunning())
         {
           statusText->setText(tr("Remote server ready"));
         }
         else
         {
-          if(!host->runCommand.isEmpty())
+          if(!host.RunCommand().isEmpty())
             statusText->setText(tr("Remote server not running or failed to start"));
           else
             statusText->setText(tr("Remote server not running - no start command configured"));
@@ -1925,6 +2020,7 @@ void MainWindow::setRemoteHost(int hostIdx)
           m_Ctx.GetCaptureDialog()->UpdateRemoteHost();
       });
     });
+    th->setName(lit("Remote host check"));
     th->selfDelete(true);
     th->start();
   }
@@ -1994,7 +2090,8 @@ void MainWindow::OnCaptureLoaded()
 
   PopulateRecentCaptureFiles();
 
-  ToolWindowManager::raiseToolWindow(m_Ctx.GetEventBrowser()->Widget());
+  if(m_Ctx.HasEventBrowser())
+    ToolWindowManager::raiseToolWindow(m_Ctx.GetEventBrowser()->Widget());
 }
 
 void MainWindow::OnCaptureClosed()
@@ -2022,7 +2119,7 @@ void MainWindow::OnCaptureClosed()
   SetTitle();
 
   // if the remote sever disconnected during capture replay, resort back to a 'disconnected' state
-  if(m_Ctx.Replay().CurrentRemote() && !m_Ctx.Replay().CurrentRemote()->serverRunning)
+  if(m_Ctx.Replay().CurrentRemote().IsValid() && !m_Ctx.Replay().CurrentRemote().IsServerRunning())
   {
     statusText->setText(
         tr("Remote server disconnected. To attempt to reconnect please select it again."));
@@ -2344,8 +2441,8 @@ void MainWindow::on_action_Resolve_Symbols_triggered()
   bool finished = false;
 
   m_Ctx.Replay().AsyncInvoke([this, &progress, &finished](IReplayController *) {
-    bool success =
-        m_Ctx.Replay().GetCaptureAccess()->InitResolver([&progress](float p) { progress = p; });
+    bool success = m_Ctx.Replay().GetCaptureAccess()->InitResolver(
+        true, [&progress](float p) { progress = p; });
 
     if(!success)
     {
@@ -2548,11 +2645,25 @@ void MainWindow::on_action_Manage_Extensions_triggered()
 
 void MainWindow::on_action_Manage_Remote_Servers_triggered()
 {
-  RemoteManager *rm = new RemoteManager(m_Ctx, this);
-  RDDialog::show(rm);
-  // now that we're done with it, the manager deletes itself when all lookups terminate (or
-  // immediately if there are no lookups ongoing).
-  rm->closeWhenFinished();
+  LambdaThread *th = new LambdaThread([this]() {
+    m_Ctx.Config().UpdateEnumeratedProtocolDevices();
+
+    GUIInvoke::call(this, [this]() {
+      RemoteManager *rm = new RemoteManager(m_Ctx, this);
+      RDDialog::show(rm);
+      // now that we're done with it, the manager deletes itself when all lookups terminate (or
+      // immediately if there are no lookups ongoing).
+      rm->closeWhenFinished();
+    });
+  });
+  th->start();
+  th->wait(500);
+  if(th->isRunning())
+  {
+    ShowProgressDialog(this, tr("Updating available devices, please wait..."),
+                       [th]() { return !th->isRunning(); });
+  }
+  th->deleteLater();
 }
 
 void MainWindow::on_action_Settings_triggered()
@@ -2572,13 +2683,6 @@ void MainWindow::on_action_View_Documentation_triggered()
     QDesktopServices::openUrl(QUrl::fromUserInput(lit("https://renderdoc.org/docs")));
 }
 
-void MainWindow::on_action_View_Diagnostic_Log_File_triggered()
-{
-  QString logPath = QString::fromUtf8(RENDERDOC_GetLogFile());
-  if(QFileInfo::exists(logPath))
-    QDesktopServices::openUrl(QUrl::fromLocalFile(logPath));
-}
-
 void MainWindow::on_action_Source_on_GitHub_triggered()
 {
   QDesktopServices::openUrl(QUrl::fromUserInput(lit("https://github.com/baldurk/renderdoc")));
@@ -2595,6 +2699,16 @@ void MainWindow::on_action_Show_Tips_triggered()
   RDDialog::show(&tipsDialog);
 }
 
+void MainWindow::on_action_View_Diagnostic_Log_File_triggered()
+{
+  QWidget *logView = m_Ctx.GetDiagnosticLogView()->Widget();
+
+  if(ui->toolWindowManager->toolWindows().contains(logView))
+    ToolWindowManager::raiseToolWindow(logView);
+  else
+    ui->toolWindowManager->addToolWindow(logView, mainToolArea());
+}
+
 void MainWindow::on_action_Counter_Viewer_triggered()
 {
   QWidget *performanceCounterViewer = m_Ctx.GetPerformanceCounterViewer()->Widget();
@@ -2623,8 +2737,9 @@ void MainWindow::on_action_Send_Error_Report_triggered()
   QVariantMap json;
 
   json[lit("version")] = lit(FULL_VERSION_STRING);
-  json[lit("gitcommit")] = QString::fromLatin1(GitVersionHash);
+  json[lit("gitcommit")] = QString::fromLatin1(RENDERDOC_GetCommitHash());
   json[lit("replaycrash")] = 1;
+  json[lit("manual")] = 1;
   json[lit("report")] = (QString)report;
 
   CrashDialog crash(m_Ctx.Config(), json, this);
@@ -2867,39 +2982,51 @@ bool MainWindow::LoadLayout(int layout)
 
 void MainWindow::showLaunchError(ReplayStatus status)
 {
-  QString title;
   QString message;
   switch(status)
   {
     case ReplayStatus::AndroidGrantPermissionsFailed:
-      title = tr("Permission is required");
-      message = tr("Enable RenderDocCmd to access storage on your device.");
+      message =
+          tr("Failed to automatically grant Android permissions to installed server.\n\n"
+             "Please manually allow the RenderDocCmd program storage permissions on your device "
+             "to ensure correct functionality.");
       break;
     case ReplayStatus::AndroidABINotFound:
-      title = tr("Failed to install RenderDoc server");
-      message = tr("Couldn't determine supported ABIs.");
+      message =
+          tr("Couldn't determine supported ABIs for your device, please check device connection "
+             "and status.");
       break;
     case ReplayStatus::AndroidAPKFolderNotFound:
-      title = tr("Failed to install RenderDoc server");
-      message = tr("APK folder missing.");
+      message = tr("Couldn't find APK folder, please check that your installation is complete.");
       break;
     case ReplayStatus::AndroidAPKInstallFailed:
-      title = tr("Failed to install RenderDoc server");
-      message = tr("Couldn't find any installed APKs.");
-    default:
-      title = tr("Failed to install RenderDoc server");
-      message = tr("Unknown error.");
+      message =
+          tr("Couldn't install APK, please check that your device is connected and accessible to "
+             "adb, and that installing APKs over USB is allowed.");
       break;
+    case ReplayStatus::AndroidAPKVerifyFailed:
+      message =
+          tr("Couldn't correctly verify installed APK version.\n\n"
+             "Please check your installation is not corrupted, or if this is a custom build check "
+             "that all ABIs are built at the same version as this program.");
+      break;
+    default: message = tr("Unexpected error: %1.").arg(ToQStr(status)); break;
   }
-  GUIInvoke::call(this, [this, title, message]() { RDDialog::warning(this, title, message); });
+  GUIInvoke::call(this, [this, message]() {
+    RDDialog::warning(this, tr("Problems installing RenderDoc server"), message);
+  });
 }
 
-bool MainWindow::isCapturableAppRunningOnAndroid()
+bool MainWindow::isUnshareableDeviceInUse()
 {
-  if(!m_Ctx.Replay().CurrentRemote() || !m_Ctx.Replay().CurrentRemote()->IsADB())
+  if(!m_Ctx.Replay().CurrentRemote().Protocol())
+    return false;
+
+  rdcstr host = m_Ctx.Replay().CurrentRemote().Hostname();
+
+  if(m_Ctx.Replay().CurrentRemote().Protocol()->SupportsMultiplePrograms(host))
     return false;
 
-  rdcstr host = m_Ctx.Replay().CurrentRemote()->hostname;
   uint32_t ident = RENDERDOC_EnumerateRemoteTargets(host.c_str(), 0);
   return ident != 0;
 }
