@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -77,7 +77,7 @@ public:
   void takeCaptureOwnership() { m_OwnTempCapture = true; }
   void captureModified();
   void LoadFromFilename(const QString &filename, bool temporary);
-  void LoadCapture(const QString &filename, bool temporary, bool local);
+  void LoadCapture(const QString &filename, const ReplayOptions &opts, bool temporary, bool local);
   void CloseCapture();
   QString GetSavePath(QString title = QString(), QString filter = QString());
 
@@ -91,6 +91,8 @@ public:
   void ShowLiveCapture(LiveCapture *live);
   void LiveCaptureClosed(LiveCapture *live);
 
+  void RemoveRecentCapture(const QString &filename);
+
   QMenu *GetBaseMenu(WindowMenu base, rdcstr name);
   QList<QAction *> GetMenuActions();
 
@@ -101,6 +103,7 @@ public:
   void showPipelineViewer() { on_action_Pipeline_State_triggered(); }
   void showCaptureDialog() { on_action_Launch_Application_triggered(); }
   void showDebugMessageView() { on_action_Errors_and_Warnings_triggered(); }
+  void showDiagnosticLogView() { on_action_View_Diagnostic_Log_File_triggered(); }
   void showCommentView() { on_action_Comments_triggered(); }
   void showStatisticsViewer() { on_action_Statistics_Viewer_triggered(); }
   void showTimelineBar() { on_action_Timeline_triggered(); }
@@ -116,6 +119,7 @@ private slots:
   void on_action_Exit_triggered();
   void on_action_About_triggered();
   void on_action_Open_Capture_triggered();
+  void on_action_Open_Capture_with_Options_triggered();
   void on_action_Save_Capture_Inplace_triggered();
   void on_action_Save_Capture_As_triggered();
   void on_action_Close_Capture_triggered();
@@ -142,6 +146,7 @@ private slots:
   void on_action_Settings_triggered();
   void on_action_View_Documentation_triggered();
   void on_action_View_Diagnostic_Log_File_triggered();
+  void on_action_Diagnostic_Log_triggered() { on_action_View_Diagnostic_Log_File_triggered(); }
   void on_action_Source_on_GitHub_triggered();
   void on_action_Build_Release_Downloads_triggered();
   void on_action_Show_Tips_triggered();
@@ -254,5 +259,5 @@ private:
 
   void showLaunchError(ReplayStatus status);
 
-  bool isCapturableAppRunningOnAndroid();
+  bool isUnshareableDeviceInUse();
 };
