@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -281,6 +281,24 @@ void PerformanceCounterViewer::OnCaptureLoaded()
   ui->saveCSV->setEnabled(true);
 }
 
+void PerformanceCounterViewer::OnEventChanged(uint32_t eventId)
+{
+  if(ui->syncViews->isChecked())
+  {
+    const int numItems = (int)ui->counterResults->rowCount();
+    for(int i = 0; i < numItems; ++i)
+    {
+      QTableWidgetItem *item = ui->counterResults->item(i, 0);
+      if(item->data(Qt::UserRole).toUInt() == eventId)
+      {
+        ui->counterResults->setCurrentItem(item);
+        ui->counterResults->scrollToItem(item);
+        break;
+      }
+    }
+  }
+}
+
 void PerformanceCounterViewer::on_counterResults_doubleClicked(const QModelIndex &index)
 {
   QTableWidgetItem *item = ui->counterResults->item(index.row(), 0);
