@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -56,7 +56,7 @@ public:
 
 private slots:
   // automatic slots
-  void on_showDisabled_toggled(bool checked);
+  void on_showUnused_toggled(bool checked);
   void on_showEmpty_toggled(bool checked);
   void on_exportHTML_clicked();
   void on_meshView_clicked();
@@ -106,7 +106,7 @@ private:
   bool showNode(bool usedSlot, bool filledSlot);
 
   void setViewDetails(RDTreeWidgetItem *node, TextureDescription *tex, uint32_t firstMip,
-                      uint32_t numMips);
+                      uint32_t numMips, const rdcstr &completeStatus = rdcstr());
 
   void exportHTML(QXmlStreamWriter &xml, const GLPipe::VertexInput &vtx);
   void exportHTML(QXmlStreamWriter &xml, const GLPipe::Shader &sh);
