@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -70,6 +70,7 @@ static uint32_t byteSize(const ResourceFormat &fmt)
       case ResourceFormatType::YUV12:
       case ResourceFormatType::YUV16:
       case ResourceFormatType::PVRTC: return ~0U;
+      case ResourceFormatType::A8: return 1;
       case ResourceFormatType::R10G10B10A2:
       case ResourceFormatType::R11G11B10: return 4;
     }
@@ -78,32 +79,6 @@ static uint32_t byteSize(const ResourceFormat &fmt)
   return fmt.compByteWidth * fmt.compCount;
 }
 
-static QString padding(uint32_t bytes)
-{
-  if(bytes == 0)
-    return QString();
-
-  QString ret;
-
-  if(bytes > 4)
-  {
-    ret += lit("xint pad[%1];").arg(bytes / 4);
-
-    bytes = bytes % 4;
-  }
-
-  if(bytes == 4)
-    ret += lit("xint pad;");
-  else if(bytes == 3)
-    ret += lit("xshort pad; xbyte pad;");
-  else if(bytes == 2)
-    ret += lit("xshort pad;");
-  else if(bytes == 1)
-    ret += lit("xbyte pad;");
-
-  return ret + lit("\n");
-}
-
 PipelineStateViewer::PipelineStateViewer(ICaptureContext &ctx, QWidget *parent)
     : QFrame(parent), ui(new Ui::PipelineStateViewer), m_Ctx(ctx)
 {
@@ -368,7 +343,9 @@ div.stage table tr td { border-right: 1px solid #AAAAAA; background-color: #EEEE
 
           xml.writeStartElement(lit("h3"));
           {
-            QString context = tr("Frame %1").arg(m_Ctx.FrameInfo().frameNumber);
+            uint32_t frameNumber = m_Ctx.FrameInfo().frameNumber;
+
+            QString context = frameNumber == ~0U ? tr("Capture") : tr("Frame %1").arg(frameNumber);
 
             const DrawcallDescription *draw = m_Ctx.CurDrawcall();
 
@@ -614,246 +591,6 @@ void PipelineStateViewer::setMeshViewPixmap(RDLabel *meshView)
   });
 }
 
-QString PipelineStateViewer::declareStruct(QList<QString> &declaredStructs, const QString &name,
-                                           const rdcarray<ShaderConstant> &members,
-                                           uint32_t requiredByteStride)
-{
-  QString ret;
-
-  ret = lit("struct %1\n{\n").arg(name);
-
-  for(int i = 0; i < members.count(); i++)
-  {
-    QString arraySize;
-    if(members[i].type.descriptor.elements > 1)
-      arraySize = QFormatStr("[%1]").arg(members[i].type.descriptor.elements);
-
-    QString varTypeName = members[i].type.descriptor.name;
-
-    if(!members[i].type.members.isEmpty())
-    {
-      // GL structs don't give us typenames (boo!) so give them unique names. This will mean some
-      // structs get duplicated if they're used in multiple places, but not much we can do about
-      // that.
-      if(varTypeName.isEmpty() || varTypeName == lit("struct"))
-        varTypeName = lit("anon%1").arg(declaredStructs.size());
-
-      if(!declaredStructs.contains(varTypeName))
-      {
-        declaredStructs.push_back(varTypeName);
-        ret = declareStruct(declaredStructs, varTypeName, members[i].type.members,
-                            members[i].type.descriptor.arrayByteStride) +
-              lit("\n") + ret;
-      }
-    }
-
-    ret += QFormatStr("    %1 %2%3;\n").arg(varTypeName).arg(members[i].name).arg(arraySize);
-  }
-
-  if(requiredByteStride > 0)
-  {
-    uint32_t structStart = 0;
-
-    const ShaderConstant *lastChild = &members.back();
-
-    structStart += lastChild->byteOffset;
-    while(!lastChild->type.members.isEmpty())
-    {
-      lastChild = &lastChild->type.members.back();
-      structStart += lastChild->byteOffset;
-    }
-
-    uint32_t size = lastChild->type.descriptor.rows * lastChild->type.descriptor.columns;
-    if(lastChild->type.descriptor.type == VarType::Double ||
-       lastChild->type.descriptor.type == VarType::ULong ||
-       lastChild->type.descriptor.type == VarType::SLong)
-      size *= 8;
-    else if(lastChild->type.descriptor.type == VarType::Float ||
-            lastChild->type.descriptor.type == VarType::UInt ||
-            lastChild->type.descriptor.type == VarType::SInt)
-      size *= 4;
-    else if(lastChild->type.descriptor.type == VarType::Half ||
-            lastChild->type.descriptor.type == VarType::UShort ||
-            lastChild->type.descriptor.type == VarType::SShort)
-      size *= 2;
-
-    if(lastChild->type.descriptor.elements > 1)
-      size *= lastChild->type.descriptor.elements;
-
-    uint32_t padBytes = requiredByteStride - (lastChild->byteOffset + size);
-
-    if(padBytes > 0)
-      ret += lit("    ") + padding(padBytes);
-  }
-
-  ret += lit("}\n");
-
-  return ret;
-}
-
-QString PipelineStateViewer::GenerateBufferFormatter(const ShaderResource &res,
-                                                     const ResourceFormat &viewFormat,
-                                                     uint64_t &baseByteOffset)
-{
-  QString format;
-
-  if(!res.variableType.members.empty())
-  {
-    if(m_Ctx.APIProps().pipelineType == GraphicsAPI::Vulkan ||
-       m_Ctx.APIProps().pipelineType == GraphicsAPI::OpenGL)
-    {
-      const rdcarray<ShaderConstant> &members = res.variableType.members;
-
-      format += QFormatStr("struct %1\n{\n").arg(res.name);
-
-      // GL/Vulkan allow fixed-sized members before the array-of-structs. This can't be
-      // represented in a buffer format so we skip it
-      if(members.count() > 1)
-      {
-        format += tr("    // members skipped as they are fixed size:\n");
-        baseByteOffset += members.back().byteOffset;
-      }
-
-      QString varTypeName;
-      QString comment = lit("// ");
-      for(int i = 0; i < members.count(); i++)
-      {
-        QString arraySize;
-        if(members[i].type.descriptor.elements > 1)
-          arraySize = QFormatStr("[%1]").arg(members[i].type.descriptor.elements);
-
-        varTypeName = members[i].type.descriptor.name;
-
-        if(i + 1 == members.count())
-        {
-          comment = arraySize = QString();
-
-          if(members.count() > 1)
-            format +=
-                lit("    // final array struct @ byte offset %1\n").arg(members.back().byteOffset);
-
-          // give GL nameless structs a better name
-          if(varTypeName.isEmpty() || varTypeName == lit("struct"))
-            varTypeName = lit("root_struct");
-        }
-
-        format +=
-            QFormatStr("    %1%2 %3%4;\n").arg(comment).arg(varTypeName).arg(members[i].name).arg(arraySize);
-      }
-
-      format += lit("}");
-
-      // if the last member is a struct, declare it
-      if(!members.back().type.members.isEmpty())
-      {
-        QList<QString> declaredStructs;
-        format = declareStruct(declaredStructs, varTypeName, members.back().type.members,
-                               members.back().type.descriptor.arrayByteStride) +
-                 lit("\n") + format;
-      }
-    }
-    else
-    {
-      QList<QString> declaredStructs;
-      format = declareStruct(declaredStructs, res.variableType.descriptor.name,
-                             res.variableType.members, 0);
-    }
-  }
-  else
-  {
-    const auto &desc = res.variableType.descriptor;
-
-    if(viewFormat.type == ResourceFormatType::Undefined)
-    {
-      if(desc.type == VarType::Unknown)
-      {
-        format = desc.name;
-      }
-      else
-      {
-        if(desc.rowMajorStorage)
-          format += lit("row_major ");
-
-        format += ToQStr(desc.type);
-        if(desc.rows > 1 && desc.columns > 1)
-          format += QFormatStr("%1x%2").arg(desc.rows).arg(desc.columns);
-        else if(desc.columns > 1)
-          format += QString::number(desc.columns);
-
-        if(!desc.name.empty())
-          format += lit(" ") + desc.name;
-
-        if(desc.elements > 1)
-          format += QFormatStr("[%1]").arg(desc.elements);
-      }
-    }
-    else
-    {
-      if(viewFormat.type == ResourceFormatType::R10G10B10A2)
-      {
-        if(viewFormat.compType == CompType::UInt)
-          format = lit("uintten");
-        if(viewFormat.compType == CompType::UNorm)
-          format = lit("unormten");
-      }
-      else if(viewFormat.type == ResourceFormatType::R11G11B10)
-      {
-        format = lit("floateleven");
-      }
-      else
-      {
-        switch(viewFormat.compByteWidth)
-        {
-          case 1:
-          {
-            if(viewFormat.compType == CompType::UNorm || viewFormat.compType == CompType::UNormSRGB)
-              format = lit("unormb");
-            if(viewFormat.compType == CompType::SNorm)
-              format = lit("snormb");
-            if(viewFormat.compType == CompType::UInt)
-              format = lit("ubyte");
-            if(viewFormat.compType == CompType::SInt)
-              format = lit("byte");
-            break;
-          }
-          case 2:
-          {
-            if(viewFormat.compType == CompType::UNorm || viewFormat.compType == CompType::UNormSRGB)
-              format = lit("unormh");
-            if(viewFormat.compType == CompType::SNorm)
-              format = lit("snormh");
-            if(viewFormat.compType == CompType::UInt)
-              format = lit("ushort");
-            if(viewFormat.compType == CompType::SInt)
-              format = lit("short");
-            if(viewFormat.compType == CompType::Float)
-              format = lit("half");
-            break;
-          }
-          case 4:
-          {
-            if(viewFormat.compType == CompType::UNorm || viewFormat.compType == CompType::UNormSRGB)
-              format = lit("unormf");
-            if(viewFormat.compType == CompType::SNorm)
-              format = lit("snormf");
-            if(viewFormat.compType == CompType::UInt)
-              format = lit("uint");
-            if(viewFormat.compType == CompType::SInt)
-              format = lit("int");
-            if(viewFormat.compType == CompType::Float)
-              format = lit("float");
-            break;
-          }
-        }
-
-        format += QString::number(viewFormat.compCount);
-      }
-    }
-  }
-
-  return format;
-}
-
 void PipelineStateViewer::MakeShaderVariablesHLSL(bool cbufferContents,
                                                   const rdcarray<ShaderConstant> &vars,
                                                   QString &struct_contents, QString &struct_defs)
@@ -900,33 +637,52 @@ void PipelineStateViewer::MakeShaderVariablesHLSL(bool cbufferContents,
   }
 }
 
-QString PipelineStateViewer::GenerateHLSLStub(const ShaderReflection *shaderDetails,
+QString PipelineStateViewer::GenerateHLSLStub(const ShaderBindpointMapping &bindpointMapping,
+                                              const ShaderReflection *shaderDetails,
                                               const QString &entryFunc)
 {
   QString hlsl = lit("// HLSL function stub generated\n\n");
 
-  const QString textureDim[ENUM_ARRAY_SIZE(TextureType)] = {
+  const QString textureDim[arraydim<TextureType>()] = {
       lit("Unknown"),          lit("Buffer"),      lit("Texture1D"),      lit("Texture1DArray"),
       lit("Texture2D"),        lit("TextureRect"), lit("Texture2DArray"), lit("Texture2DMS"),
       lit("Texture2DMSArray"), lit("Texture3D"),   lit("TextureCube"),    lit("TextureCubeArray"),
   };
 
+  // use bindpoint mapping
+
   for(const ShaderSampler &samp : shaderDetails->samplers)
   {
-    hlsl += lit("//SamplerComparisonState %1 : register(s%2); // can't disambiguate\n"
-                "SamplerState %1 : register(s%2); // can't disambiguate\n")
+    uint32_t reg = ~0U;
+
+    if(samp.bindPoint < bindpointMapping.samplers.count())
+      reg = bindpointMapping.samplers[samp.bindPoint].bind;
+    else
+      hlsl += lit("//");
+
+    hlsl += lit("SamplerState %1 : register(s%2); // can't disambiguate\n"
+                "//SamplerComparisonState %1 : register(s%2); // can't disambiguate\n")
                 .arg(samp.name)
-                .arg(samp.bindPoint);
+                .arg(reg);
   }
 
   for(int i = 0; i < 2; i++)
   {
+    const rdcarray<Bindpoint> &binds =
+        (i == 0 ? bindpointMapping.readOnlyResources : bindpointMapping.readWriteResources);
     const rdcarray<ShaderResource> &resources =
         (i == 0 ? shaderDetails->readOnlyResources : shaderDetails->readWriteResources);
     for(const ShaderResource &res : resources)
     {
       char regChar = 't';
 
+      uint32_t reg = ~0U;
+
+      if(res.bindPoint < binds.count())
+        reg = binds[res.bindPoint].bind;
+      else
+        hlsl += lit("//");
+
       if(i == 1)
       {
         hlsl += lit("RW");
@@ -940,7 +696,7 @@ QString PipelineStateViewer::GenerateHLSLStub(const ShaderReflection *shaderDeta
                     .arg(res.variableType.descriptor.name)
                     .arg(res.name)
                     .arg(QLatin1Char(regChar))
-                    .arg(res.bindPoint);
+                    .arg(reg);
       }
       else
       {
@@ -951,7 +707,7 @@ QString PipelineStateViewer::GenerateHLSLStub(const ShaderReflection *shaderDeta
                     .arg(res.variableType.descriptor.name)
                     .arg(res.name)
                     .arg(QLatin1Char(regChar))
-                    .arg(res.bindPoint);
+                    .arg(reg);
       }
     }
   }
@@ -965,12 +721,22 @@ QString PipelineStateViewer::GenerateHLSLStub(const ShaderReflection *shaderDeta
   {
     if(!cbuf.name.isEmpty() && !cbuf.variables.isEmpty())
     {
+      uint32_t reg = ~0U;
+
+      if(cbuf.bindPoint < bindpointMapping.constantBlocks.count())
+        reg = bindpointMapping.constantBlocks[cbuf.bindPoint].bind;
+      else
+        hlsl += lit("/*\n");
+
       QString cbufName = cbuf.name;
       if(cbufName == lit("$Globals"))
         cbufName = lit("_Globals");
       cbuffers += lit("cbuffer %1 : register(b%2) {\n").arg(cbufName).arg(cbuf.bindPoint);
       MakeShaderVariablesHLSL(true, cbuf.variables, cbuffers, hlsl);
       cbuffers += lit("};\n\n");
+
+      if(reg == ~0U)
+        hlsl += lit("*/\n");
     }
     cbufIdx++;
   }
@@ -981,18 +747,27 @@ QString PipelineStateViewer::GenerateHLSLStub(const ShaderReflection *shaderDeta
 
   hlsl += lit("struct InputStruct {\n");
   for(const SigParameter &sig : shaderDetails->inputSignature)
-    hlsl += lit("\t%1 %2 : %3;\n")
-                .arg(TypeString(sig))
-                .arg(!sig.varName.isEmpty() ? QString(sig.varName) : lit("param%1").arg(sig.regIndex))
-                .arg(D3DSemanticString(sig));
+  {
+    QString name = !sig.varName.isEmpty() ? QString(sig.varName) : lit("param%1").arg(sig.regIndex);
+
+    if(sig.varName.isEmpty() && sig.systemValue != ShaderBuiltin::Undefined)
+      name = D3DSemanticString(sig).replace(lit("SV_"), QString());
+
+    hlsl += lit("\t%1 %2 : %3;\n").arg(TypeString(sig)).arg(name).arg(D3DSemanticString(sig));
+  }
   hlsl += lit("};\n\n");
 
   hlsl += lit("struct OutputStruct {\n");
   for(const SigParameter &sig : shaderDetails->outputSignature)
-    hlsl += lit("\t%1 %2 : %3;\n")
-                .arg(TypeString(sig))
-                .arg(!sig.varName.isEmpty() ? QString(sig.varName) : lit("param%1").arg(sig.regIndex))
-                .arg(D3DSemanticString(sig));
+  {
+    QString name = !sig.varName.isEmpty() ? QString(sig.varName) : lit("param%1").arg(sig.regIndex);
+
+    if(sig.varName.isEmpty() && sig.systemValue != ShaderBuiltin::Undefined)
+      name = D3DSemanticString(sig).replace(lit("SV_"), QString());
+
+    hlsl += lit("\t%1 %2 : %3;\n").arg(TypeString(sig)).arg(name).arg(D3DSemanticString(sig));
+  }
+
   hlsl += lit("};\n\n");
 
   hlsl += lit("OutputStruct %1(in InputStruct IN)\n"
@@ -1032,28 +807,33 @@ IShaderViewer *PipelineStateViewer::EditShader(ResourceId id, ShaderStage shader
 
     ANALYTIC_SET(UIFeatures.ShaderEditing, true);
 
+    QPointer<QObject> ptr(viewer->Widget());
+
     // invoke off to the ReplayController to replace the capture's shader
     // with our edited one
     ctx->Replay().AsyncInvoke([ctx, entryFunc, shaderBytes, shaderEncoding, flags, shaderType, id,
-                               viewer](IReplayController *r) {
+                               ptr, viewer](IReplayController *r) {
       rdcstr errs;
 
       ResourceId from = id;
       ResourceId to;
 
-      std::tie(to, errs) =
+      rdctie(to, errs) =
           r->BuildTargetShader(entryFunc.c_str(), shaderEncoding, shaderBytes, flags, shaderType);
 
-      GUIInvoke::call(viewer->Widget(), [viewer, errs]() { viewer->ShowErrors(errs); });
+      if(ptr)
+        GUIInvoke::call(ptr, [viewer, errs]() { viewer->ShowErrors(errs); });
       if(to == ResourceId())
       {
         r->RemoveReplacement(from);
-        GUIInvoke::call(viewer->Widget(), [ctx]() { ctx->RefreshStatus(); });
+        if(ptr)
+          GUIInvoke::call(ptr, [ctx]() { ctx->RefreshStatus(); });
       }
       else
       {
         r->ReplaceResource(from, to);
-        GUIInvoke::call(viewer->Widget(), [ctx]() { ctx->RefreshStatus(); });
+        if(ptr)
+          GUIInvoke::call(ptr, [ctx]() { ctx->RefreshStatus(); });
       }
     });
   };
@@ -1111,7 +891,7 @@ IShaderViewer *PipelineStateViewer::EditDecompiledSource(const ShaderProcessingT
   source.assign((const char *)out.result.data(), out.result.size());
 
   rdcstrpairs files;
-  files.push_back(make_rdcpair<rdcstr, rdcstr>("decompiled", source));
+  files.push_back(rdcpair<rdcstr, rdcstr>("decompiled", source));
 
   IShaderViewer *sv = EditShader(id, shaderDetails->stage, shaderDetails->entryPoint,
                                  shaderDetails->debugInfo.compileFlags, tool.output, files);
@@ -1123,6 +903,7 @@ IShaderViewer *PipelineStateViewer::EditDecompiledSource(const ShaderProcessingT
 
 void PipelineStateViewer::SetupShaderEditButton(QToolButton *button, ResourceId pipelineId,
                                                 ResourceId shaderId,
+                                                const ShaderBindpointMapping &bindpointMapping,
                                                 const ShaderReflection *shaderDetails)
 {
   if(!shaderDetails || !button->isEnabled() || button->popupMode() != QToolButton::MenuButtonPopup)
@@ -1181,7 +962,8 @@ void PipelineStateViewer::SetupShaderEditButton(QToolButton *button, ResourceId
     QAction *action = new QAction(label, menu);
     action->setIcon(Icons::page_white_edit());
 
-    QObject::connect(action, &QAction::triggered, [this, pipelineId, shaderId, shaderDetails]() {
+    QObject::connect(action, &QAction::triggered, [this, pipelineId, shaderId, bindpointMapping,
+                                                   shaderDetails]() {
       QString entry;
       QString src;
 
@@ -1198,7 +980,7 @@ void PipelineStateViewer::SetupShaderEditButton(QToolButton *button, ResourceId
 
           GUIInvoke::call(this, [this, shaderId, shaderDetails, editeddisasm]() {
             rdcstrpairs files;
-            files.push_back(make_rdcpair<rdcstr, rdcstr>("pseudocode", editeddisasm));
+            files.push_back(rdcpair<rdcstr, rdcstr>("pseudocode", editeddisasm));
 
             EditShader(shaderId, shaderDetails->stage, shaderDetails->entryPoint,
                        ShaderCompileFlags(), ShaderEncoding::Unknown, files);
@@ -1210,8 +992,8 @@ void PipelineStateViewer::SetupShaderEditButton(QToolButton *button, ResourceId
         entry = lit("EditedShader%1S").arg(ToQStr(shaderDetails->stage, GraphicsAPI::D3D11)[0]);
 
         rdcstrpairs files;
-        files.push_back(make_rdcpair<rdcstr, rdcstr>("decompiled_stub.hlsl",
-                                                     GenerateHLSLStub(shaderDetails, entry)));
+        files.push_back(rdcpair<rdcstr, rdcstr>(
+            "decompiled_stub.hlsl", GenerateHLSLStub(bindpointMapping, shaderDetails, entry)));
 
         EditShader(shaderId, shaderDetails->stage, entry, ShaderCompileFlags(),
                    ShaderEncoding::HLSL, files);
@@ -1236,19 +1018,9 @@ QString PipelineStateViewer::GetVBufferFormatString(uint32_t slot)
   uint32_t stride = vbs[slot].byteStride;
 
   // filter attributes to only the ones enabled and using this slot
-  for(size_t i = 0; i < attrs.size();)
-  {
-    if(!attrs[i].used || attrs[i].vertexBuffer != (int)slot)
-    {
-      attrs.erase(i);
-      // continue with same i
-    }
-    else
-    {
-      // move to next i
-      i++;
-    }
-  }
+  attrs.removeIf([slot](const VertexInputAttribute &attr) {
+    return (!attr.used || attr.vertexBuffer != (int)slot);
+  });
 
   // we now have all attributes in this buffer. Sort by offset
   std::sort(attrs.begin(), attrs.end(),
@@ -1296,11 +1068,11 @@ QString PipelineStateViewer::GetVBufferFormatString(uint32_t slot)
       continue;
 
     // declare any padding from previous element to this one
-    format += padding(attrs[i].byteOffset - offset);
+    format += BufferFormatter::DeclarePaddingBytes(attrs[i].byteOffset - offset);
 
     const ResourceFormat &fmt = attrs[i].format;
 
-    offset += byteSize(fmt);
+    offset = attrs[i].byteOffset + byteSize(fmt);
 
     if(fmt.Special())
     {
@@ -1376,7 +1148,7 @@ QString PipelineStateViewer::GetVBufferFormatString(uint32_t slot)
   }
 
   if(stride > 0)
-    format += padding(stride - offset);
+    format += BufferFormatter::DeclarePaddingBytes(stride - offset);
 
   return format;
 }
