@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -42,8 +42,8 @@ class PixelHistoryItemModel : public QAbstractItemModel
 {
 public:
   PixelHistoryItemModel(ICaptureContext &ctx, ResourceId tex, const TextureDisplay &display,
-                        QObject *parent)
-      : QAbstractItemModel(parent), m_Ctx(ctx)
+                        const QPalette &palette, QObject *parent)
+      : QAbstractItemModel(parent), m_Ctx(ctx), m_Palette(palette)
   {
     m_Tex = m_Ctx.GetTexture(tex);
     m_Display = display;
@@ -51,7 +51,7 @@ public:
     CompType compType = m_Tex->format.compType;
 
     if(compType == CompType::Typeless)
-      compType = display.typeHint;
+      compType = display.typeCast;
 
     m_IsUint = (compType == CompType::UInt);
     m_IsSint = (compType == CompType::SInt);
@@ -320,9 +320,16 @@ public:
         if(col == 2)
         {
           if(isEvent(index))
+          {
             return backgroundBrush(getMods(index).first().preMod);
+          }
           else
-            return backgroundBrush(getMod(index).preMod);
+          {
+            const PixelModification &mod = getMod(index);
+            if(mod.directShaderWrite)
+              return backgroundBrush(mod.preMod);
+            return backgroundBrush(mod.shaderOut);
+          }
         }
         else if(col == 4)
         {
@@ -360,6 +367,23 @@ public:
         }
       }
 
+      // Since we change the background color for some cells, also change the foreground color to
+      // ensure contrast with all UI themes
+      if(role == Qt::ForegroundRole && (col == 0 || col == 1 || col == 3))
+      {
+        QColor textColor =
+            contrastingColor(QColor::fromRgb(235, 235, 235), m_Palette.color(QPalette::Text));
+        if(isEvent(index))
+        {
+          return QBrush(textColor);
+        }
+        else
+        {
+          if(getMod(index).shaderDiscarded)
+            return QBrush(textColor);
+        }
+      }
+
       if(role == Qt::UserRole)
       {
         EventTag tag;
@@ -397,6 +421,8 @@ private:
   QVector<QList<PixelModification>> m_History;
   QVector<PixelModification> m_ModList;
 
+  const QPalette &m_Palette;
+
   // mask for top bit of quintptr
   static const quintptr eventTagMask = 1ULL << (Q_PROCESSOR_WORDSIZE * 8 - 1);
 
@@ -476,13 +502,16 @@ private:
     if(m_IsDepth)
       r = g = b = qBound(0.0f, (val.depth - m_Display.rangeMin) / rangesize, 1.0f);
 
+    // Convert from linear color to sRGB
     {
-      r = (float)powf(r, 1.0f / 2.2f);
-      g = (float)powf(g, 1.0f / 2.2f);
-      b = (float)powf(b, 1.0f / 2.2f);
+      r = (r <= 0.0031308f) ? r * 12.92f : 1.055f * (float)powf(r, 1.0f / 2.4f) - 0.055f;
+      g = (g <= 0.0031308f) ? g * 12.92f : 1.055f * (float)powf(g, 1.0f / 2.4f) - 0.055f;
+      b = (b <= 0.0031308f) ? b * 12.92f : 1.055f * (float)powf(b, 1.0f / 2.4f) - 0.055f;
     }
 
-    return QBrush(QColor::fromRgb((int)(255.0f * r), (int)(255.0f * g), (int)(255.0f * b)));
+    // Round to nearest value in [0,255]
+    return QBrush(QColor::fromRgb((int)(255.0f * r + 0.5f), (int)(255.0f * g + 0.5f),
+                                  (int)(255.0f * b + 0.5f)));
   }
 
   QString modString(const ModificationValue &val, int forceComps = 0) const
@@ -600,7 +629,7 @@ PixelHistoryView::PixelHistoryView(ICaptureContext &ctx, ResourceId id, QPoint p
 
   ui->eventsHidden->setVisible(false);
 
-  m_Model = new PixelHistoryItemModel(ctx, id, display, this);
+  m_Model = new PixelHistoryItemModel(ctx, id, display, palette(), this);
   ui->events->setModel(m_Model);
 
   ui->events->hideBranches();
@@ -623,7 +652,7 @@ void PixelHistoryView::updateWindowTitle()
 
   TextureDescription *tex = m_Ctx.GetTexture(m_ID);
   if(tex->msSamp > 1)
-    title += tr(" @ Sample %1").arg(m_Display.sampleIdx);
+    title += tr(" @ Sample %1").arg(m_Display.subresource.sample);
 
   setWindowTitle(title);
 }
@@ -688,8 +717,8 @@ void PixelHistoryView::startDebug(EventTag tag)
   ShaderDebugTrace *trace = NULL;
 
   m_Ctx.Replay().AsyncInvoke([this, &trace, &done, tag](IReplayController *r) {
-    trace = r->DebugPixel((uint32_t)m_Pixel.x(), (uint32_t)m_Pixel.y(), m_Display.sampleIdx,
-                          tag.primitive);
+    trace = r->DebugPixel((uint32_t)m_Pixel.x(), (uint32_t)m_Pixel.y(),
+                          m_Display.subresource.sample, tag.primitive);
 
     if(trace->states.isEmpty())
     {
