@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -69,6 +69,10 @@ struct CaptureContextInvoker : ICaptureContext
   {
     return m_Ctx.TargetShaderEncodings();
   }
+  virtual rdcarray<ShaderEncoding> CustomShaderEncodings() override
+  {
+    return m_Ctx.CustomShaderEncodings();
+  }
   virtual uint32_t CurSelectedEvent() override { return m_Ctx.CurSelectedEvent(); }
   virtual uint32_t CurEvent() override { return m_Ctx.CurEvent(); }
   virtual const DrawcallDescription *CurSelectedDrawcall() override
@@ -169,10 +173,10 @@ struct CaptureContextInvoker : ICaptureContext
   {
     return InvokeRetFunction<WindowingData>(&ICaptureContext::CreateWindowingData, window);
   }
-  virtual void LoadCapture(const rdcstr &capture, const rdcstr &origFilename, bool temporary,
-                           bool local) override
+  virtual void LoadCapture(const rdcstr &capture, const ReplayOptions &opts,
+                           const rdcstr &origFilename, bool temporary, bool local) override
   {
-    InvokeVoidFunction(&ICaptureContext::LoadCapture, capture, origFilename, temporary, local);
+    InvokeVoidFunction(&ICaptureContext::LoadCapture, capture, opts, origFilename, temporary, local);
   }
   virtual bool SaveCaptureTo(const rdcstr &capture) override
   {
@@ -259,6 +263,10 @@ struct CaptureContextInvoker : ICaptureContext
   {
     return InvokeRetFunction<IDebugMessageView *>(&ICaptureContext::GetDebugMessageView);
   }
+  virtual IDiagnosticLogView *GetDiagnosticLogView() override
+  {
+    return InvokeRetFunction<IDiagnosticLogView *>(&ICaptureContext::GetDiagnosticLogView);
+  }
   virtual ICommentView *GetCommentView() override
   {
     return InvokeRetFunction<ICommentView *>(&ICaptureContext::GetCommentView);
@@ -312,6 +320,10 @@ struct CaptureContextInvoker : ICaptureContext
   {
     return InvokeRetFunction<bool>(&ICaptureContext::HasDebugMessageView);
   }
+  virtual bool HasDiagnosticLogView() override
+  {
+    return InvokeRetFunction<bool>(&ICaptureContext::HasDiagnosticLogView);
+  }
   virtual bool HasCommentView() override
   {
     return InvokeRetFunction<bool>(&ICaptureContext::HasCommentView);
@@ -362,6 +374,10 @@ struct CaptureContextInvoker : ICaptureContext
   {
     InvokeVoidFunction(&ICaptureContext::ShowDebugMessageView);
   }
+  virtual void ShowDiagnosticLogView() override
+  {
+    InvokeVoidFunction(&ICaptureContext::ShowDiagnosticLogView);
+  }
   virtual void ShowCommentView() override { InvokeVoidFunction(&ICaptureContext::ShowCommentView); }
   virtual void ShowPerformanceCounterViewer() override
   {
@@ -408,11 +424,10 @@ struct CaptureContextInvoker : ICaptureContext
                                               id, format);
   }
 
-  virtual IBufferViewer *ViewTextureAsBuffer(uint32_t arrayIdx, uint32_t mip, ResourceId id,
+  virtual IBufferViewer *ViewTextureAsBuffer(ResourceId id, const Subresource &sub,
                                              const rdcstr &format = "") override
   {
-    return InvokeRetFunction<IBufferViewer *>(&ICaptureContext::ViewTextureAsBuffer, arrayIdx, mip,
-                                              id, format);
+    return InvokeRetFunction<IBufferViewer *>(&ICaptureContext::ViewTextureAsBuffer, id, sub, format);
   }
 
   virtual IConstantBufferPreviewer *ViewConstantBuffer(ShaderStage stage, uint32_t slot,
@@ -653,7 +668,9 @@ void PythonShell::on_saveScript_clicked()
       QFile f(filename);
       if(f.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text))
       {
-        f.write(scriptEditor->getText(scriptEditor->textLength() + 1));
+        QString text = QString::fromUtf8(scriptEditor->getText(scriptEditor->textLength() + 1));
+        text.remove(QLatin1Char('\r'));
+        f.write(text.toUtf8());
       }
       else
       {
@@ -696,6 +713,7 @@ void PythonShell::on_runScript_clicked()
     });
   });
 
+  thread->setName(lit("Python script"));
   thread->selfDelete(true);
   thread->start();
 }
