@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -143,7 +143,7 @@ void TimelineBar::HighlightResourceUsage(ResourceId id)
     GUIInvoke::call(this, [this, usage]() {
       for(const EventUsage &u : usage)
         m_UsageEvents << u;
-      qSort(m_UsageEvents);
+      std::sort(m_UsageEvents.begin(), m_UsageEvents.end());
       viewport()->update();
     });
   });
@@ -163,7 +163,7 @@ void TimelineBar::HighlightHistory(ResourceId id, const rdcarray<PixelModificati
 
     for(const PixelModification &mod : history)
       m_HistoryEvents << mod;
-    qSort(m_HistoryEvents);
+    std::sort(m_HistoryEvents.begin(), m_HistoryEvents.end());
   }
 
   viewport()->update();
@@ -187,7 +187,12 @@ void TimelineBar::OnCaptureClosed()
 
 void TimelineBar::OnCaptureLoaded()
 {
-  setWindowTitle(tr("Timeline - Frame #%1").arg(m_Ctx.FrameInfo().frameNumber));
+  uint32_t frameNumber = m_Ctx.FrameInfo().frameNumber;
+
+  if(frameNumber != ~0U)
+    setWindowTitle(tr("Timeline - Frame #%1").arg(frameNumber));
+  else
+    setWindowTitle(tr("Timeline - Capture"));
 
   processDraws(m_RootMarkers, m_RootDraws, m_Ctx.CurDrawcalls());
 
@@ -227,7 +232,7 @@ void TimelineBar::layout()
   // the area of everything
   m_area = QRectF(viewport()->rect()).marginsRemoved(uniformMargins(borderWidth + margin));
 
-  m_titleWidth = fm.width(eidAxisTitle) + fm.height();
+  m_titleWidth = fm.boundingRect(eidAxisTitle).width() + fm.height();
 
   m_dataArea = m_area;
   m_dataArea.setLeft(m_dataArea.left() + m_titleWidth);
@@ -239,7 +244,7 @@ void TimelineBar::layout()
   m_markerRect.setTop(m_eidAxisRect.bottom() + margin);
 
   m_highlightingRect = m_area;
-  m_highlightingRect.setHeight(qMax(fm.height(), dataBarHeight) + highlightingExtra);
+  m_highlightingRect.setHeight(qMax(fm.height(), dataBarHeight) * 2 + highlightingMargin);
   m_highlightingRect.moveTop(m_markerRect.bottom() - m_highlightingRect.height());
 
   m_markerRect.setBottom(m_highlightingRect.top());
@@ -249,7 +254,7 @@ void TimelineBar::layout()
   int stepSize = 1;
   int stepMagnitude = 1;
 
-  m_eidAxisLabelTextWidth = fm.width(QString::number(maxEID));
+  m_eidAxisLabelTextWidth = fm.boundingRect(QString::number(maxEID)).width();
   m_eidAxisLabelWidth = m_eidAxisLabelTextWidth + fm.height();
   m_eidAxisLabelStep = stepSize * stepMagnitude;
 
@@ -473,8 +478,6 @@ void TimelineBar::paintEvent(QPaintEvent *e)
   to.setWrapMode(QTextOption::NoWrap);
   to.setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
 
-  QFontMetrics fm = p.fontMetrics();
-
   {
     QRectF titleRect = m_eidAxisRect;
     titleRect.setLeft(titleRect.left() - m_titleWidth);
@@ -508,6 +511,8 @@ void TimelineBar::paintEvent(QPaintEvent *e)
 
   p.setFont(Formatter::PreferredFont());
 
+  QFontMetrics fm = p.fontMetrics();
+
   QRectF hoverRect = eidAxisRect;
 
   // clip labels to the visible section
@@ -655,7 +660,7 @@ void TimelineBar::paintEvent(QPaintEvent *e)
     if(!m_HistoryTarget.isEmpty())
       text = tr("Pixel history for %1").arg(m_HistoryTarget);
     else
-      text = tr("Usage for %1:").arg(m_UsageTarget);
+      text = tr("Usage for %1: ").arg(m_UsageTarget);
 
     p.drawText(highlightLabel, text, to);
 
@@ -706,11 +711,11 @@ void TimelineBar::paintEvent(QPaintEvent *e)
     if(m_HistoryTarget.isEmpty())
     {
       // advance past the first text to draw the key
-      highlightLabel.setLeft(highlightLabel.left() + fm.width(text));
+      highlightLabel.setLeft(highlightLabel.left() + fm.boundingRect(text).width());
 
       text = lit(" Reads ( ");
       p.drawText(highlightLabel, text, to);
-      highlightLabel.setLeft(highlightLabel.left() + fm.width(text));
+      highlightLabel.setLeft(highlightLabel.left() + fm.boundingRect(text).width());
 
       QPainterPath path = triangle.translated(aliasAlign(highlightLabel.topLeft()));
       p.fillPath(path, colors[ReadUsage]);
@@ -719,7 +724,7 @@ void TimelineBar::paintEvent(QPaintEvent *e)
 
       text = lit(" ), Writes ( ");
       p.drawText(highlightLabel, text, to);
-      highlightLabel.setLeft(highlightLabel.left() + fm.width(text));
+      highlightLabel.setLeft(highlightLabel.left() + fm.boundingRect(text).width());
 
       path = triangle.translated(aliasAlign(highlightLabel.topLeft()));
       p.fillPath(path, colors[WriteUsage]);
@@ -728,7 +733,7 @@ void TimelineBar::paintEvent(QPaintEvent *e)
 
       text = lit(" ), Read/Write ( ");
       p.drawText(highlightLabel, text, to);
-      highlightLabel.setLeft(highlightLabel.left() + fm.width(text));
+      highlightLabel.setLeft(highlightLabel.left() + fm.boundingRect(text).width());
 
       path = triangle.translated(aliasAlign(highlightLabel.topLeft()));
       p.fillPath(path, colors[ReadWriteUsage]);
@@ -739,7 +744,7 @@ void TimelineBar::paintEvent(QPaintEvent *e)
       {
         text = lit(" ) Barriers ( ");
         p.drawText(highlightLabel, text, to);
-        highlightLabel.setLeft(highlightLabel.left() + fm.width(text));
+        highlightLabel.setLeft(highlightLabel.left() + fm.boundingRect(text).width());
 
         path = triangle.translated(aliasAlign(highlightLabel.topLeft()));
         p.fillPath(path, colors[BarrierUsage]);
@@ -749,7 +754,7 @@ void TimelineBar::paintEvent(QPaintEvent *e)
 
       text = lit(" ), and Clears ( ");
       p.drawText(highlightLabel, text, to);
-      highlightLabel.setLeft(highlightLabel.left() + fm.width(text));
+      highlightLabel.setLeft(highlightLabel.left() + fm.boundingRect(text).width());
 
       path = triangle.translated(aliasAlign(highlightLabel.topLeft()));
       p.fillPath(path, colors[ClearUsage]);
@@ -920,7 +925,7 @@ void TimelineBar::paintMarkers(QPainter &p, const QVector<Marker> &markers,
 
     r.setLeft(r.left() + margin);
 
-    int plusWidth = fm.width(QLatin1Char('+'));
+    int plusWidth = fm.boundingRect(QLatin1Char('+')).width();
     if(r.width() > plusWidth)
     {
       QRectF plusRect = r;
