@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,49 +22,40 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include <sstream>
 #include "3rdparty/miniz/miniz.h"
 #include "api/replay/version.h"
 #include "core/core.h"
+#include "replay/replay_driver.h"
 #include "strings/string_utils.h"
 #include "android_utils.h"
 
-// we use GIT_COMMIT_HASH here instead of GitVersionHash since the value is actually only used on
-// android - where GIT_COMMIT_HASH is available globally. We need to have it available in a static
-// initializer at compile time, which wouldn't be possible with GitVersionHash.
-#if !defined(GIT_COMMIT_HASH)
-#define GIT_COMMIT_HASH "NO_GIT_COMMIT_HASH_DEFINED_AT_BUILD_TIME"
-#endif
-
-extern "C" RENDERDOC_API const char RENDERDOC_Version_Tag_String[] =
-    "RenderDoc_build_version: " FULL_VERSION_STRING " from git commit " GIT_COMMIT_HASH;
-
 static const char keystoreName[] = "renderdoc.keystore";
 
 namespace Android
 {
-bool RemoveAPKSignature(const string &apk)
+bool RemoveAPKSignature(const rdcstr &apk)
 {
   RDCLOG("Checking for existing signature");
 
-  std::string aapt = getToolPath(ToolDir::BuildTools, "aapt", false);
+  rdcstr aapt = getToolPath(ToolDir::BuildTools, "aapt", false);
 
   // Get the list of files in META-INF
-  string fileList = execCommand(aapt, "list \"" + apk + "\"").strStdout;
+  rdcstr fileList = execCommand(aapt, "list \"" + apk + "\"").strStdout;
   if(fileList.empty())
     return false;
 
   // Walk through the output.  If it starts with META-INF, remove it.
   uint32_t fileCount = 0;
   uint32_t matchCount = 0;
-  std::istringstream contents(fileList);
-  string line;
-  string prefix("META-INF");
-  while(std::getline(contents, line))
+
+  rdcarray<rdcstr> lines;
+  split(fileList, lines, '\n');
+
+  for(rdcstr &line : lines)
   {
-    line = trim(line);
+    line.trim();
     fileCount++;
-    if(line.compare(0, prefix.size(), prefix) == 0)
+    if(line.beginsWith("META-INF"))
     {
       RDCDEBUG("Match found, removing  %s", line.c_str());
       execCommand(aapt, "remove \"" + apk + "\" " + line);
@@ -76,10 +67,11 @@ bool RemoveAPKSignature(const string &apk)
   // Ensure no hits on second pass through
   RDCDEBUG("Walk through file list again, ensure signature removed");
   fileList = execCommand(aapt, "list \"" + apk + "\"").strStdout;
-  std::istringstream recheck(fileList);
-  while(std::getline(recheck, line))
+  split(fileList, lines, '\n');
+  for(rdcstr &line : lines)
   {
-    if(line.compare(0, prefix.size(), prefix) == 0)
+    line.trim();
+    if(line.beginsWith("META-INF"))
     {
       RDCERR("Match found, that means removal failed! %s", line.c_str());
       return false;
@@ -88,7 +80,7 @@ bool RemoveAPKSignature(const string &apk)
   return true;
 }
 
-bool ExtractAndRemoveManifest(const std::string &apk, std::vector<byte> &manifest)
+bool ExtractAndRemoveManifest(const rdcstr &apk, bytebuf &manifest)
 {
   // pull out the manifest with miniz
   mz_zip_archive zip;
@@ -112,7 +104,8 @@ bool ExtractAndRemoveManifest(const std::string &apk, std::vector<byte> &manifes
 
         RDCLOG("Got manifest of %zu bytes", sz);
 
-        manifest.insert(manifest.begin(), buf, buf + sz);
+        manifest = bytebuf(buf, sz);
+        break;
       }
     }
   }
@@ -126,18 +119,19 @@ bool ExtractAndRemoveManifest(const std::string &apk, std::vector<byte> &manifes
   if(manifest.empty())
     return false;
 
-  std::string aapt = getToolPath(ToolDir::BuildTools, "aapt", false);
+  rdcstr aapt = getToolPath(ToolDir::BuildTools, "aapt", false);
 
   RDCDEBUG("Removing AndroidManifest.xml");
   execCommand(aapt, "remove \"" + apk + "\" AndroidManifest.xml");
 
-  std::string fileList = execCommand(aapt, "list \"" + apk + "\"").strStdout;
-  std::vector<std::string> files;
+  rdcstr fileList = execCommand(aapt, "list \"" + apk + "\"").strStdout;
+  rdcarray<rdcstr> files;
   split(fileList, files, ' ');
 
-  for(const std::string &f : files)
+  for(rdcstr &f : files)
   {
-    if(trim(f) == "AndroidManifest.xml")
+    f.trim();
+    if(f == "AndroidManifest.xml")
     {
       RDCERR("AndroidManifest.xml found, that means removal failed!");
       return false;
@@ -147,13 +141,12 @@ bool ExtractAndRemoveManifest(const std::string &apk, std::vector<byte> &manifes
   return true;
 }
 
-bool AddManifestToAPK(const std::string &apk, const std::string &tmpDir,
-                      const std::vector<byte> &manifest)
+bool AddManifestToAPK(const rdcstr &apk, const rdcstr &tmpDir, const bytebuf &manifest)
 {
-  std::string aapt = getToolPath(ToolDir::BuildTools, "aapt", false);
+  rdcstr aapt = getToolPath(ToolDir::BuildTools, "aapt", false);
 
   // write the manifest to disk
-  FileIO::dump((tmpDir + "AndroidManifest.xml").c_str(), manifest.data(), manifest.size());
+  FileIO::WriteAll((tmpDir + "AndroidManifest.xml").c_str(), manifest);
 
   // run aapt to add the manifest
   Process::ProcessResult result =
@@ -168,13 +161,13 @@ bool AddManifestToAPK(const std::string &apk, const std::string &tmpDir,
   return true;
 }
 
-bool RealignAPK(const string &apk, string &alignedAPK, const string &tmpDir)
+bool RealignAPK(const rdcstr &apk, rdcstr &alignedAPK, const rdcstr &tmpDir)
 {
-  std::string zipalign = getToolPath(ToolDir::BuildTools, "zipalign", false);
+  rdcstr zipalign = getToolPath(ToolDir::BuildTools, "zipalign", false);
 
   // Re-align the APK for performance
   RDCLOG("Realigning APK");
-  string errOut =
+  rdcstr errOut =
       execCommand(zipalign, "-f 4 \"" + apk + "\" \"" + alignedAPK + "\"", tmpDir).strStderror;
 
   if(!errOut.empty())
@@ -199,59 +192,78 @@ bool RealignAPK(const string &apk, string &alignedAPK, const string &tmpDir)
   return false;
 }
 
-string GetAndroidDebugKey()
+rdcstr GetAndroidDebugKey()
 {
-  std::string keystore = getToolPath(ToolDir::None, keystoreName, false);
+  rdcstr keystore = getToolPath(ToolDir::None, keystoreName, false);
 
   // if we found the keystore, use that.
   if(FileIO::exists(keystore.c_str()))
     return keystore;
 
-  // otherwise, see if we generated a temporary one
-  string key = FileIO::GetTempFolderFilename() + keystoreName;
+  // otherwise, generate a temporary one
+  rdcstr key = FileIO::GetTempFolderFilename() + keystoreName;
 
-  if(FileIO::exists(key.c_str()))
-    return key;
+  FileIO::Delete(key.c_str());
 
   // locate keytool and use it to generate a keystore
-  string create;
+  rdcstr create;
   create += " -genkey";
   create += " -keystore \"" + key + "\"";
   create += " -storepass android";
-  create += " -alias androiddebugkey";
+  create += " -alias rdocandroidkey";
   create += " -keypass android";
   create += " -keyalg RSA";
   create += " -keysize 2048";
   create += " -validity 10000";
   create += " -dname \"CN=, OU=, O=, L=, S=, C=\"";
 
-  std::string keytool = getToolPath(ToolDir::Java, "keytool", false);
+  rdcstr keytool = getToolPath(ToolDir::Java, "keytool", false);
 
-  Process::ProcessResult result = execCommand(keytool, create);
+  Process::ProcessResult createResult = execCommand(keytool, create);
 
-  if(!result.strStderror.empty())
-    RDCERR("Failed to create debug key");
+  Process::ProcessResult verifyResult;
 
-  return key;
+  // if the keystore was created, check that the key we expect to be in it is there
+  if(FileIO::exists(key.c_str()))
+  {
+    rdcstr verify;
+    verify += " -list";
+    verify += " -keystore \"" + key + "\"";
+    verify += " -storepass android";
+
+    verifyResult = execCommand(keytool, verify);
+
+    if(verifyResult.strStdout.contains("rdocandroidkey"))
+      return key;
+  }
+
+  RDCERR("Failed to create debug key: %s\n%s\n%s\n%s", createResult.strStdout.c_str(),
+         createResult.strStderror.c_str(), verifyResult.strStdout.c_str(),
+         verifyResult.strStderror.c_str());
+  return "";
 }
-bool DebugSignAPK(const string &apk, const string &workDir)
+
+bool DebugSignAPK(const rdcstr &apk, const rdcstr &workDir)
 {
   RDCLOG("Signing with debug key");
 
-  std::string aapt = getToolPath(ToolDir::BuildTools, "aapt", false);
-  std::string apksigner = getToolPath(ToolDir::BuildToolsLib, "apksigner.jar", false);
+  rdcstr aapt = getToolPath(ToolDir::BuildTools, "aapt", false);
+  rdcstr apksigner = getToolPath(ToolDir::BuildToolsLib, "apksigner.jar", false);
 
-  string debugKey = GetAndroidDebugKey();
+  rdcstr debugKey = GetAndroidDebugKey();
 
-  string args;
+  if(debugKey.empty())
+    return false;
+
+  rdcstr args;
   args += " sign ";
   args += " --ks \"" + debugKey + "\" ";
   args += " --ks-pass pass:android ";
   args += " --key-pass pass:android ";
-  args += " --ks-key-alias androiddebugkey ";
+  args += " --ks-key-alias rdocandroidkey ";
   args += "\"" + apk + "\"";
 
-  if(apksigner.find(".jar") == std::string::npos)
+  if(!apksigner.contains(".jar"))
   {
     // if we found the non-jar version, then the jar wasn't located and we found the wrapper script
     // in PATH. Execute it as a script
@@ -260,11 +272,11 @@ bool DebugSignAPK(const string &apk, const string &workDir)
   else
   {
     // otherwise, find and invoke java on the .jar
-    std::string java = getToolPath(ToolDir::Java, "java", false);
+    rdcstr java = getToolPath(ToolDir::Java, "java", false);
 
-    std::string signerdir = get_dirname(FileIO::GetFullPathname(apksigner));
+    rdcstr signerdir = get_dirname(FileIO::GetFullPathname(apksigner));
 
-    std::string javaargs;
+    rdcstr javaargs;
     javaargs += " \"-Djava.ext.dirs=" + signerdir + "\"";
     javaargs += " -jar \"" + apksigner + "\"";
     javaargs += args;
@@ -273,33 +285,28 @@ bool DebugSignAPK(const string &apk, const string &workDir)
   }
 
   // Check for signature
-  string list = execCommand(aapt, "list \"" + apk + "\"").strStdout;
+  rdcstr list = execCommand(aapt, "list \"" + apk + "\"").strStdout;
 
-  // Walk through the output.  If it starts with META-INF, we're good
-  std::istringstream contents(list);
-  string line;
-  string prefix("META-INF");
-  while(std::getline(contents, line))
+  list.insert(0, '\n');
+
+  if(list.find("\nMETA-INF") >= 0)
   {
-    if(line.compare(0, prefix.size(), prefix) == 0)
-    {
-      RDCLOG("Signature found, continuing...");
-      return true;
-    }
+    RDCLOG("Signature found, continuing...");
+    return true;
   }
 
   RDCERR("re-sign of APK failed!");
   return false;
 }
 
-bool UninstallOriginalAPK(const string &deviceID, const string &packageName, const string &workDir)
+bool UninstallOriginalAPK(const rdcstr &deviceID, const rdcstr &packageName, const rdcstr &workDir)
 {
   RDCLOG("Uninstalling previous version of application");
 
   adbExecCommand(deviceID, "uninstall " + packageName, workDir);
 
   // Wait until uninstall completes
-  string uninstallResult;
+  rdcstr uninstallResult;
   uint32_t elapsed = 0;
   uint32_t timeout = 10000;    // 10 seconds
   while(elapsed < timeout)
@@ -319,8 +326,8 @@ bool UninstallOriginalAPK(const string &deviceID, const string &packageName, con
   return false;
 }
 
-bool ReinstallPatchedAPK(const string &deviceID, const string &apk, const string &abi,
-                         const string &packageName, const string &workDir)
+bool ReinstallPatchedAPK(const rdcstr &deviceID, const rdcstr &apk, const rdcstr &abi,
+                         const rdcstr &packageName, const rdcstr &workDir)
 {
   RDCLOG("Reinstalling APK");
 
@@ -330,7 +337,7 @@ bool ReinstallPatchedAPK(const string &deviceID, const string &apk, const string
     adbExecCommand(deviceID, "install --abi " + abi + " \"" + apk + "\"", workDir);
 
   // Wait until re-install completes
-  string reinstallResult;
+  rdcstr reinstallResult;
   uint32_t elapsed = 0;
   uint32_t timeout = 10000;    // 10 seconds
   while(elapsed < timeout)
@@ -353,16 +360,16 @@ bool ReinstallPatchedAPK(const string &deviceID, const string &apk, const string
 bool CheckPatchingRequirements()
 {
   // check for required tools for patching
-  std::vector<std::pair<ToolDir, std::string>> requirements;
-  std::vector<std::string> missingTools;
-  requirements.push_back(std::make_pair(ToolDir::BuildTools, "aapt"));
-  requirements.push_back(std::make_pair(ToolDir::BuildTools, "zipalign"));
-  requirements.push_back(std::make_pair(ToolDir::BuildToolsLib, "apksigner.jar"));
-  requirements.push_back(std::make_pair(ToolDir::Java, "java"));
+  rdcarray<rdcpair<ToolDir, rdcstr>> requirements;
+  rdcarray<rdcstr> missingTools;
+  requirements.push_back({ToolDir::BuildTools, "aapt"});
+  requirements.push_back({ToolDir::BuildTools, "zipalign"});
+  requirements.push_back({ToolDir::BuildToolsLib, "apksigner.jar"});
+  requirements.push_back({ToolDir::Java, "java"});
 
   for(uint32_t i = 0; i < requirements.size(); i++)
   {
-    std::string tool = getToolPath(requirements[i].first, requirements[i].second, true);
+    rdcstr tool = getToolPath(requirements[i].first, requirements[i].second, true);
 
     // if we located the tool, we're fine.
     if(toolExists(tool))
@@ -374,12 +381,12 @@ bool CheckPatchingRequirements()
 
   // keytool is special - we look for a debug key first
   {
-    std::string key = getToolPath(ToolDir::None, keystoreName, true);
+    rdcstr key = getToolPath(ToolDir::None, keystoreName, true);
 
     if(key.empty())
     {
       // if we don't have the debug key, check that we can find keytool. First in our normal search
-      std::string tool = getToolPath(ToolDir::Java, "keytool", true);
+      rdcstr tool = getToolPath(ToolDir::Java, "keytool", true);
 
       if(tool.empty())
       {
@@ -399,38 +406,7 @@ bool CheckPatchingRequirements()
   return true;
 }
 
-std::string DetermineInstalledABI(const std::string &deviceID, const std::string &packageName)
-{
-  RDCLOG("Checking installed ABI for %s", packageName.c_str());
-  string abi;
-
-  string dump = adbExecCommand(deviceID, "shell pm dump " + packageName).strStdout;
-  if(dump.empty())
-    RDCERR("Unable to pm dump %s", packageName.c_str());
-
-  // Walk through the output and look for primaryCpuAbi
-  std::istringstream contents(dump);
-  string line;
-  string prefix("primaryCpuAbi=");
-  while(std::getline(contents, line))
-  {
-    line = trim(line);
-    if(line.compare(0, prefix.size(), prefix) == 0)
-    {
-      // Extract the abi
-      abi = line.substr(line.find_last_of("=") + 1);
-      RDCLOG("primaryCpuAbi found: %s", abi.c_str());
-      break;
-    }
-  }
-
-  if(abi.empty())
-    RDCERR("Unable to determine installed abi for: %s", packageName.c_str());
-
-  return abi;
-}
-
-bool PullAPK(const string &deviceID, const string &pkgPath, const string &apk)
+bool PullAPK(const rdcstr &deviceID, const rdcstr &pkgPath, const rdcstr &apk)
 {
   RDCLOG("Pulling APK to patch");
 
@@ -455,19 +431,19 @@ bool PullAPK(const string &deviceID, const string &pkgPath, const string &apk)
   return false;
 }
 
-void CopyAPK(const string &deviceID, const string &pkgPath, const string &copyPath)
+void CopyAPK(const rdcstr &deviceID, const rdcstr &pkgPath, const rdcstr &copyPath)
 {
   RDCLOG("Copying APK to %s", copyPath.c_str());
   adbExecCommand(deviceID, "shell cp " + pkgPath + " " + copyPath);
 }
 
-void RemoveAPK(const string &deviceID, const string &path)
+void RemoveAPK(const rdcstr &deviceID, const rdcstr &path)
 {
   RDCLOG("Removing APK from %s", path.c_str());
   adbExecCommand(deviceID, "shell rm -f " + path);
 }
 
-bool HasRootAccess(const std::string &deviceID)
+bool HasRootAccess(const rdcstr &deviceID)
 {
   RDCLOG("Checking for root access on %s", deviceID.c_str());
 
@@ -479,38 +455,37 @@ bool HasRootAccess(const std::string &deviceID)
 
   result = adbExecCommand(deviceID, "root");
 
-  std::string whoami = trim(adbExecCommand(deviceID, "shell whoami").strStdout);
+  rdcstr whoami = adbExecCommand(deviceID, "shell whoami").strStdout.trimmed();
   if(whoami == "root")
     return true;
 
-  std::string checksu =
-      trim(adbExecCommand(deviceID, "shell test -e /system/xbin/su && echo found").strStdout);
+  rdcstr checksu =
+      adbExecCommand(deviceID, "shell test -e /system/xbin/su && echo found").strStdout.trimmed();
   if(checksu == "found")
     return true;
 
   return false;
 }
 
-std::string GetFirstMatchingLine(const std::string &haystack, const std::string &needle)
+rdcstr GetFirstMatchingLine(const rdcstr &haystack, const rdcstr &needle)
 {
-  size_t needleOffset = haystack.find(needle);
+  int needleOffset = haystack.find(needle);
 
-  if(needleOffset == std::string::npos)
-    return "";
+  if(needleOffset == -1)
+    return rdcstr();
 
-  size_t nextLine = haystack.find('\n', needleOffset + 1);
+  int nextLine = haystack.find('\n', needleOffset + 1);
 
-  return haystack.substr(needleOffset,
-                         nextLine == std::string::npos ? nextLine : nextLine - needleOffset);
+  return haystack.substr(needleOffset, nextLine == -1 ? ~0U : size_t(nextLine - needleOffset));
 }
 
-bool IsDebuggable(const std::string &deviceID, const std::string &packageName)
+bool IsDebuggable(const rdcstr &deviceID, const rdcstr &packageName)
 {
   RDCLOG("Checking that APK is debuggable");
 
-  std::string info = adbExecCommand(deviceID, "shell dumpsys package " + packageName).strStdout;
+  rdcstr info = adbExecCommand(deviceID, "shell dumpsys package " + packageName).strStdout;
 
-  std::string pkgFlags = GetFirstMatchingLine(info, "pkgFlags=[");
+  rdcstr pkgFlags = GetFirstMatchingLine(info, "pkgFlags=[");
 
   if(pkgFlags == "")
   {
@@ -518,28 +493,27 @@ bool IsDebuggable(const std::string &deviceID, const std::string &packageName)
     return false;
   }
 
-  return pkgFlags.find("DEBUGGABLE") != std::string::npos;
+  return pkgFlags.contains("DEBUGGABLE");
 }
 };
 
-extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_CheckAndroidPackage(const char *hostname,
-                                                                         const char *packageName,
-                                                                         AndroidFlags *flags)
+extern "C" RENDERDOC_API void RENDERDOC_CC
+RENDERDOC_CheckAndroidPackage(const char *URL, const char *packageAndActivity, AndroidFlags *flags)
 {
-  int index = 0;
-  std::string deviceID;
-  Android::ExtractDeviceIDAndIndex(hostname, index, deviceID);
+  IDeviceProtocolHandler *adb = RenderDoc::Inst().GetDeviceProtocol("adb");
+
+  rdcstr deviceID = adb->GetDeviceID(URL);
 
   // Reset the flags each time we check
   *flags = AndroidFlags::NoFlags;
 
-  if(Android::IsDebuggable(deviceID, get_basename(std::string(packageName))))
+  if(Android::IsDebuggable(deviceID, Android::GetPackageName(packageAndActivity)))
   {
     *flags |= AndroidFlags::Debuggable;
   }
   else
   {
-    RDCLOG("%s is not debuggable", packageName);
+    RDCLOG("%s is not debuggable", packageAndActivity);
   }
 
   if(Android::HasRootAccess(deviceID))
@@ -552,14 +526,15 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_CheckAndroidPackage(const c
 }
 
 extern "C" RENDERDOC_API AndroidFlags RENDERDOC_CC RENDERDOC_MakeDebuggablePackage(
-    const char *hostname, const char *packageName, RENDERDOC_ProgressCallback progress)
+    const char *URL, const char *packageAndActivity, RENDERDOC_ProgressCallback progress)
 {
-  Process::ProcessResult result = {};
-  std::string package(get_basename(std::string(packageName)));
+  rdcstr package = Android::GetPackageName(packageAndActivity);
 
-  int index = 0;
-  std::string deviceID;
-  Android::ExtractDeviceIDAndIndex(hostname, index, deviceID);
+  Process::ProcessResult result = {};
+
+  IDeviceProtocolHandler *adb = RenderDoc::Inst().GetDeviceProtocol("adb");
+
+  rdcstr deviceID = adb->GetDeviceID(URL);
 
   // make sure progress is valid so we don't have to check it everywhere
   if(!progress)
@@ -572,21 +547,21 @@ extern "C" RENDERDOC_API AndroidFlags RENDERDOC_CC RENDERDOC_MakeDebuggablePacka
 
   progress(0.02f);
 
-  std::string abi = Android::DetermineInstalledABI(deviceID, package);
+  rdcstr abi = Android::DetermineInstalledABI(deviceID, package);
 
   // Find the APK on the device
-  std::string pkgPath = Android::GetPathForPackage(deviceID, package) + "base.apk";
+  rdcstr pkgPath = Android::GetPathForPackage(deviceID, package) + "base.apk";
 
-  std::string tmpDir = FileIO::GetTempFolderFilename();
-  std::string origAPK(tmpDir + package + ".orig.apk");
-  std::string alignedAPK(origAPK + ".aligned.apk");
-  std::vector<byte> manifest;
+  rdcstr tmpDir = FileIO::GetTempFolderFilename();
+  rdcstr origAPK(tmpDir + package + ".orig.apk");
+  rdcstr alignedAPK(origAPK + ".aligned.apk");
+  bytebuf manifest;
 
   // Try the following steps, bailing if anything fails
   if(!Android::PullAPK(deviceID, pkgPath, origAPK))
   {
     // Copy the APK to public storage, then try to pull again
-    std::string copyPath = "/sdcard/" + package + ".copy.apk";
+    rdcstr copyPath = "/sdcard/" + package + ".copy.apk";
     Android::CopyAPK(deviceID, pkgPath, copyPath);
     bool success = Android::PullAPK(deviceID, copyPath, origAPK);
     Android::RemoveAPK(deviceID, copyPath);
@@ -628,17 +603,17 @@ extern "C" RENDERDOC_API AndroidFlags RENDERDOC_CC RENDERDOC_MakeDebuggablePacka
 
   progress(0.525f);
 
-  if(!Android::UninstallOriginalAPK(deviceID, packageName, tmpDir))
+  if(!Android::UninstallOriginalAPK(deviceID, package, tmpDir))
     return AndroidFlags::RepackagingAPKFailure;
 
   progress(0.6f);
 
-  if(!Android::ReinstallPatchedAPK(deviceID, alignedAPK, abi, packageName, tmpDir))
+  if(!Android::ReinstallPatchedAPK(deviceID, alignedAPK, abi, package, tmpDir))
     return AndroidFlags::RepackagingAPKFailure;
 
   progress(0.95f);
 
-  if(!Android::IsDebuggable(deviceID, packageName))
+  if(!Android::IsDebuggable(deviceID, package))
     return AndroidFlags::ManifestPatchFailure;
 
   progress(1.0f);
