@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,46 +25,36 @@
 
 #pragma once
 
+#include <functional>
+#include "apidefs.h"
 #include "data_types.h"
+#include "rdcarray.h"
 #include "replay_enums.h"
 
-DOCUMENT(R"(
-Contains the details of a single element of data (such as position or texture
-co-ordinates) within a mesh.)");
+DOCUMENT(R"(Contains the details of a single element of data (such as position or texture
+co-ordinates) within a mesh.
+)");
 struct MeshFormat
 {
-  MeshFormat()
-  {
-    indexByteOffset = 0;
-    indexByteStride = 0;
-    baseVertex = 0;
-    vertexByteOffset = 0;
-    vertexByteStride = 0;
-    instStepRate = 1;
-    showAlpha = false;
-    topology = Topology::Unknown;
-    numIndices = 0;
-    unproject = false;
-    instanced = false;
-    nearPlane = farPlane = 0.0f;
-  }
+  MeshFormat() = default;
   MeshFormat(const MeshFormat &o) = default;
+  MeshFormat &operator=(const MeshFormat &) = default;
 
   DOCUMENT("The :class:`ResourceId` of the index buffer that goes with this mesh element.");
   ResourceId indexResourceId;
   DOCUMENT("The offset in bytes where the indices start in idxbuf.");
-  uint64_t indexByteOffset;
+  uint64_t indexByteOffset = 0;
   DOCUMENT("The width in bytes of each index. Valid values are 1 (depending on API), 2 or 4.");
-  uint32_t indexByteStride;
+  uint32_t indexByteStride = 0;
   DOCUMENT("For indexed meshes, a value added to each index before using it to read the vertex.");
-  int32_t baseVertex;
+  int32_t baseVertex = 0;
 
   DOCUMENT("The :class:`ResourceId` of the vertex buffer containing this mesh element.");
   ResourceId vertexResourceId;
   DOCUMENT("The offset in bytes to the start of the vertex data.");
-  uint64_t vertexByteOffset;
+  uint64_t vertexByteOffset = 0;
   DOCUMENT("The stride in bytes between the start of one vertex and the start of another.");
-  uint32_t vertexByteStride;
+  uint32_t vertexByteStride = 0;
 
   DOCUMENT("The :class:`ResourceFormat` describing this mesh component.");
   ResourceFormat format;
@@ -75,24 +65,29 @@ struct MeshFormat
   FloatVector meshColor;
 
   DOCUMENT("The :class:`Topology` that describes the primitives in this mesh.");
-  Topology topology;
+  Topology topology = Topology::Unknown;
   DOCUMENT("The number of vertices in the mesh.");
-  uint32_t numIndices;
+  uint32_t numIndices = 0;
   DOCUMENT("The number of instances to render with the same value. See :data:`instanced`.");
-  uint32_t instStepRate;
+  uint32_t instStepRate = 1;
+  DOCUMENT("The primitive restart index to use, if possible. See :data:`allowRestart`.");
+  uint32_t restartIndex = 0xffffffff;
 
   DOCUMENT("The near plane for the projection matrix.");
-  float nearPlane;
+  float nearPlane = 0.1f;
   DOCUMENT("The far plane for the projection matrix.");
-  float farPlane;
+  float farPlane = 100.0f;
   DOCUMENT("``True`` if this mesh element contains post-projection positional data.");
-  bool unproject;
+  bool unproject = false;
 
   DOCUMENT("``True`` if this mesh element comes from instanced data. See :data:`instStepRate`.");
-  bool instanced;
+  bool instanced = false;
 
   DOCUMENT("``True`` if the alpha component of this element should be used.");
-  bool showAlpha;
+  bool showAlpha = false;
+
+  DOCUMENT("``True`` if the primitive restart index feature should be used.");
+  bool allowRestart = true;
 };
 
 DECLARE_REFLECTION_STRUCT(MeshFormat);
@@ -112,38 +107,39 @@ struct MeshDisplay
   DOCUMENT("");
   MeshDisplay() = default;
   MeshDisplay(const MeshDisplay &) = default;
+  MeshDisplay &operator=(const MeshDisplay &) = default;
 
   DOCUMENT("The :class:`MeshDataStage` where this mesh data comes from.");
-  MeshDataStage type;
+  MeshDataStage type = MeshDataStage::Unknown;
 
   DOCUMENT("The :class:`Camera` to use when rendering all of the meshes.");
-  ICamera *cam;
+  ICamera *cam = NULL;
 
   DOCUMENT(
       "``True`` if the projection matrix to use when unprojecting vertex positions is "
       "orthographic.");
-  bool ortho;
+  bool ortho = false;
   DOCUMENT("The field of view to use when calculating a perspective projection matrix.");
-  float fov;
+  float fov = 90.0f;
   DOCUMENT("The aspect ratio to use when calculating a perspective projection matrix.");
-  float aspect;
+  float aspect = 1.0f;
 
   DOCUMENT(
       "``True`` if all previous instances in the drawcall should be drawn as secondary meshes.");
-  bool showPrevInstances;
+  bool showPrevInstances = false;
   DOCUMENT("``True`` if all instances in the drawcall should be drawn as secondary meshes.");
-  bool showAllInstances;
+  bool showAllInstances = false;
   DOCUMENT(
       "``True`` if all draws in the current pass up to the current draw should be drawn as "
       "secondary meshes.");
-  bool showWholePass;
+  bool showWholePass = false;
   DOCUMENT("The index of the currently selected instance in the drawcall.");
-  uint32_t curInstance;
+  uint32_t curInstance = 0;
   DOCUMENT("The index of the currently selected multiview view in the drawcall.");
-  uint32_t curView;
+  uint32_t curView = 0;
 
   DOCUMENT("The index of the vertex to highlight, or :data:`NoHighlight` to select no vertex.");
-  uint32_t highlightVert;
+  uint32_t highlightVert = ~0U;
   DOCUMENT("The :class:`MeshFormat` of the position data for the mesh.");
   MeshFormat position;
   DOCUMENT(
@@ -155,12 +151,12 @@ struct MeshDisplay
   DOCUMENT("The maximum co-ordinates in each axis of the mesh bounding box.");
   FloatVector maxBounds;
   DOCUMENT("``True`` if the bounding box around the mesh should be rendered.");
-  bool showBBox;
+  bool showBBox = false;
 
   DOCUMENT("The :class:`solid shading mode <SolidShade>` to use when rendering the current mesh.");
-  SolidShade solidShadeMode;
+  SolidShade solidShadeMode = SolidShade::NoSolid;
   DOCUMENT("``True`` if the wireframe of the mesh should be rendered as well as solid shading.");
-  bool wireframeDraw;
+  bool wireframeDraw = true;
 
   static const uint32_t NoHighlight = ~0U;
 };
@@ -172,6 +168,12 @@ Describes how to render a texture preview of an image. Describes the zoom and pa
 texture when rendering on a particular output, as well as the modification and selection of a
 particular subresource (such as array slice, mip or multi-sampled sample).
 
+.. note::
+  X and Y co-ordinates are always considered to be top-left, even on GL, for consistency between
+  APIs and preventing the need for API-specific code in most cases. This means if co-ordinates are
+  fetched from e.g. viewport or scissor data or other GL pipeline state which is perhaps in
+  bottom-left co-ordinates, care must be taken to translate them.
+
 .. data:: ResolveSamples
 
   Value for :data:`sampleIdx` if the samples should be averaged.
@@ -181,12 +183,18 @@ struct TextureDisplay
   DOCUMENT("");
   TextureDisplay() = default;
   TextureDisplay(const TextureDisplay &) = default;
+  TextureDisplay &operator=(const TextureDisplay &) = default;
 
   DOCUMENT("The :class:`ResourceId` of the texture to display.");
   ResourceId resourceId;
 
-  DOCUMENT("An optional :class:`CompType` hint to use when displaying a typeless texture.");
-  CompType typeHint = CompType::Typeless;
+  DOCUMENT(R"(If possible interpret the texture with this type instead of its normal type.
+
+If set to :data:`CompType.Typeless` then no cast is applied, otherwise where allowed the texture
+data will be reinterpreted - e.g. from unsigned integers to floats, or to unsigned normalised
+values.
+)");
+  CompType typeCast = CompType::Typeless;
 
   DOCUMENT("The value in each channel to map to the black point.");
   float rangeMin = 0.0f;
@@ -246,18 +254,12 @@ See :meth:`ReplayController.BuildCustomShader` for creating an appropriate custo
 )");
   ResourceId customShaderId;
 
-  DOCUMENT("Select the mip of the texture to display.");
-  uint32_t mip = 0;
+  DOCUMENT(R"(The subresource of the texture to display.
 
-  DOCUMENT("Select the slice or face of the texture to display if it's an array, 3D, or cube tex.");
-  uint32_t sliceFace = 0;
-
-  DOCUMENT(R"(Select the sample of the texture to display if it's a multi-sampled texture.
-
-If this is set to :data:`ResolveSamples` then a default resolve will be performed that averages all
-samples.
+If the :data:`Subresource.sample` member is set to :data:`ResolveSamples` then a default resolve
+will be performed that averages all samples.
 )");
-  uint32_t sampleIdx = 0;
+  Subresource subresource = {0, 0, 0};
 
   DOCUMENT(R"(``True`` if the rendered image should be as close as possible in value to the input.
 
@@ -293,6 +295,7 @@ struct TextureComponentMapping
   DOCUMENT("");
   TextureComponentMapping() = default;
   TextureComponentMapping(const TextureComponentMapping &) = default;
+  TextureComponentMapping &operator=(const TextureComponentMapping &) = default;
 
   DOCUMENT("The value that should be mapped to ``0``");
   float blackPoint = 0.0f;
@@ -313,6 +316,7 @@ struct TextureSampleMapping
   DOCUMENT("");
   TextureSampleMapping() = default;
   TextureSampleMapping(const TextureSampleMapping &) = default;
+  TextureSampleMapping &operator=(const TextureSampleMapping &) = default;
 
   DOCUMENT(R"(
 ``True`` if the samples should be mapped to array slices. A multisampled array expands each slice
@@ -344,6 +348,7 @@ struct TextureSliceMapping
   DOCUMENT("");
   TextureSliceMapping() = default;
   TextureSliceMapping(const TextureSliceMapping &) = default;
+  TextureSliceMapping &operator=(const TextureSliceMapping &) = default;
 
   DOCUMENT(R"(
 Selects the (depth/array) slice to save.
@@ -390,12 +395,18 @@ struct TextureSave
   DOCUMENT("");
   TextureSave() = default;
   TextureSave(const TextureSave &) = default;
+  TextureSave &operator=(const TextureSave &) = default;
 
   DOCUMENT("The :class:`ResourceId` of the texture to save.");
   ResourceId resourceId;
 
-  DOCUMENT("An optional :class:`CompType` hint to use when saving a typeless texture.");
-  CompType typeHint = CompType::Typeless;
+  DOCUMENT(R"(If possible interpret the texture with this type instead of its normal type.
+
+If set to :data:`CompType.Typeless` then no cast is applied, otherwise where allowed the texture
+data will be reinterpreted - e.g. from unsigned integers to floats, or to unsigned normalised
+values.
+)");
+  CompType typeCast = CompType::Typeless;
 
   DOCUMENT("The :class:`FileType` to use when saving to the destination file.");
   FileType destType = FileType::DDS;
@@ -449,6 +460,7 @@ struct NewCaptureData
   DOCUMENT("");
   NewCaptureData() = default;
   NewCaptureData(const NewCaptureData &) = default;
+  NewCaptureData &operator=(const NewCaptureData &) = default;
 
   DOCUMENT("An identifier to use to refer to this capture.");
   uint32_t captureId = 0;
@@ -482,6 +494,7 @@ struct APIUseData
   DOCUMENT("");
   APIUseData() = default;
   APIUseData(const APIUseData &) = default;
+  APIUseData &operator=(const APIUseData &) = default;
 
   DOCUMENT("The name of the API.");
   rdcstr name;
@@ -501,6 +514,7 @@ struct BusyData
   DOCUMENT("");
   BusyData() = default;
   BusyData(const BusyData &) = default;
+  BusyData &operator=(const BusyData &) = default;
 
   DOCUMENT("The name of the client currently connected to the target.");
   rdcstr clientName;
@@ -514,6 +528,7 @@ struct NewChildData
   DOCUMENT("");
   NewChildData() = default;
   NewChildData(const NewChildData &) = default;
+  NewChildData &operator=(const NewChildData &) = default;
 
   DOCUMENT("The PID (Process ID) of the new child.");
   uint32_t processId = 0;
@@ -529,6 +544,7 @@ struct TargetControlMessage
   DOCUMENT("");
   TargetControlMessage() = default;
   TargetControlMessage(const TargetControlMessage &) = default;
+  TargetControlMessage &operator=(const TargetControlMessage &) = default;
 
   DOCUMENT("The :class:`type <TargetControlMessageType>` of message received");
   TargetControlMessageType type = TargetControlMessageType::Unknown;
@@ -564,6 +580,7 @@ struct EnvironmentModification
       : mod(m), sep(s), name(n), value(v)
   {
   }
+  EnvironmentModification &operator=(const EnvironmentModification &) = default;
 
   bool operator==(const EnvironmentModification &o) const
   {
@@ -599,6 +616,7 @@ struct CaptureFileFormat
   DOCUMENT("");
   CaptureFileFormat() = default;
   CaptureFileFormat(const CaptureFileFormat &) = default;
+  CaptureFileFormat &operator=(const CaptureFileFormat &) = default;
 
   bool operator==(const CaptureFileFormat &o) const
   {
@@ -647,3 +665,247 @@ structured data.
 };
 
 DECLARE_REFLECTION_STRUCT(CaptureFileFormat);
+
+DOCUMENT("Describes a single GPU at replay time.");
+struct GPUDevice
+{
+  DOCUMENT("");
+  GPUDevice() = default;
+  GPUDevice(const GPUDevice &) = default;
+  GPUDevice &operator=(const GPUDevice &) = default;
+
+  bool operator==(const GPUDevice &o) const
+  {
+    // deliberately don't compare name or APIs - only this triple counts for equality
+    return vendor == o.vendor && deviceID == o.deviceID && driver == o.driver;
+  }
+  bool operator<(const GPUDevice &o) const
+  {
+    if(!(vendor == o.vendor))
+      return vendor < o.vendor;
+    if(!(deviceID == o.deviceID))
+      return deviceID < o.deviceID;
+    if(!(driver == o.driver))
+      return driver < o.driver;
+    return false;
+  }
+  DOCUMENT("The :class:`GPUVendor` of this GPU.");
+  GPUVendor vendor = GPUVendor::Unknown;
+  DOCUMENT("The PCI deviceID of this GPU.");
+  uint32_t deviceID = 0;
+  DOCUMENT("The name of the driver of this GPU, if multiple drivers are available for it.");
+  rdcstr driver;
+
+  DOCUMENT("The human-readable name of this GPU.");
+  rdcstr name;
+
+  DOCUMENT("The list of APIs that this device supports.");
+  rdcarray<GraphicsAPI> apis;
+};
+
+DECLARE_REFLECTION_STRUCT(GPUDevice);
+
+DOCUMENT("The options controlling how replay of a capture should be performed");
+struct ReplayOptions
+{
+  DOCUMENT("");
+  ReplayOptions() = default;
+  ReplayOptions(const ReplayOptions &) = default;
+  ReplayOptions &operator=(const ReplayOptions &) = default;
+
+  DOCUMENT(R"(Replay with API validation enabled and use debug messages from there, ignoring any
+that may be contained in the capture.
+
+The default is not to do any validation.
+
+.. note:: RenderDoc does not handle invalid API use in the general case so validation should still
+  be performed at runtime in your program for ground truth results.
+)");
+  bool apiValidation = false;
+
+  DOCUMENT(R"(Force the selection of a GPU by vendor ID. This allows overriding which GPU is used to
+replay on, even if a different GPU would be the best match for the capture.
+
+When set to :data:`GPUVendor.Unknown`, specifies no particular vendor.
+
+See also :data:`forceGPUDeviceID` and :data:`forceGPUDriverName`. Available GPUs can be enumerated
+using :meth:`CaptureAccess.GetAvailableGPUs`.
+
+The default is not to do any override. The capture contains information about what GPU was used, and
+the closest matching GPU is used on replay.
+
+.. note::
+  If a GPU is forced that is not available or not supported for a given capture, such as when GPUs
+  are only available for some APIs and not others, the default GPU selection will be used. If a GPU
+  is available for a capture but fails to open however then there is no fallback to a default GPU.
+
+  OpenGL does not support GPU selection so the default method (which effectively does nothing) will
+  always be used.
+)");
+  GPUVendor forceGPUVendor = GPUVendor::Unknown;
+
+  DOCUMENT(R"(Force the selection of a GPU by device ID. This allows overriding which GPU is used to
+replay on.
+
+When set to 0, specifies no particular device.
+
+See :data:`forceGPUDeviceID` for a full explanation of GPU selection override.
+)");
+  uint32_t forceGPUDeviceID = 0;
+
+  DOCUMENT(R"(Force the selection of a GPU by driver name. This allows overriding which GPU is used
+to replay on.
+
+When set to an empty string, specifies no particular driver.
+
+See :data:`forceGPUDeviceID` for a full explanation of GPU selection override.
+)");
+  rdcstr forceGPUDriverName;
+
+  DOCUMENT(R"(How much optimisation should be done, potentially at the cost of correctness.
+
+The default is :data:`ReplayOptimisationLevel.Balanced`.
+)");
+  ReplayOptimisationLevel optimisation = ReplayOptimisationLevel::Balanced;
+
+// helpers for Qt, define constructor and cast. These will be defined in Qt code
+#if defined(RENDERDOC_QT_COMPAT)
+  ReplayOptions(const QVariant &var);
+  operator QVariant() const;
+#endif
+};
+
+DECLARE_REFLECTION_STRUCT(ReplayOptions);
+
+// typedef the window data structs so this will compile on all platforms without system headers. We
+// only actually need the real definitions when we're using the data, otherwise it's mostly opaque
+// pointers or integers.
+
+// Win32
+typedef struct HWND__ *HWND;
+
+// xlib
+typedef struct _XDisplay Display;
+typedef unsigned long Drawable;
+
+// xcb
+struct xcb_connection_t;
+typedef uint32_t xcb_window_t;
+
+// wayland
+struct wl_display;
+struct wl_surface;
+
+// android
+struct ANativeWindow;
+
+// for swig bindings treat the windowing data struct as completely opaque
+#if defined(SWIG)
+
+DOCUMENT("An opaque structure created to hold windowing setup data");
+struct WindowingData
+{
+};
+
+#else
+
+struct WindowingData
+{
+  WindowingSystem system;
+
+  union
+  {
+    struct
+    {
+      int32_t width, height;
+    } headless;
+
+    struct
+    {
+      HWND window;
+    } win32;
+
+    struct
+    {
+      Display *display;
+      Drawable window;
+    } xlib;
+
+    struct
+    {
+      xcb_connection_t *connection;
+      xcb_window_t window;
+    } xcb;
+
+    struct
+    {
+      wl_display *display;
+      wl_surface *window;
+    } wayland;
+
+    struct
+    {
+      ANativeWindow *window;
+    } android;
+
+    struct
+    {
+      void *view;
+      void *layer;
+    } macOS;
+  };
+};
+
+DECLARE_STRINGISE_TYPE(WindowingData);
+
+#endif
+
+DOCUMENT(R"(Internal structure used for initialising environment in a replay application.)");
+struct GlobalEnvironment
+{
+  DOCUMENT("");
+  GlobalEnvironment() = default;
+  GlobalEnvironment(const GlobalEnvironment &) = default;
+  GlobalEnvironment &operator=(const GlobalEnvironment &) = default;
+
+  DOCUMENT("The handle to the X display to use internally. If left ``NULL``, one will be opened.");
+  Display *xlibDisplay = NULL;
+
+  DOCUMENT(
+      "The handle to the X display to use internally. If left ``NULL``, wayland cannot be used.");
+  wl_display *waylandDisplay = NULL;
+
+  DOCUMENT(R"(Whether to enumerate available GPUs. If the replay program is only being used for
+internal operation where enumerating GPUs would be too expensive or problematic, it can be disabled
+here.
+)");
+  bool enumerateGPUs = true;
+};
+
+DECLARE_REFLECTION_STRUCT(GlobalEnvironment);
+
+DOCUMENT("The result of executing or injecting into a program.")
+struct ExecuteResult
+{
+  DOCUMENT("");
+  ExecuteResult() = default;
+  ExecuteResult(const ExecuteResult &) = default;
+  ExecuteResult &operator=(const ExecuteResult &) = default;
+
+  DOCUMENT(
+      "The :class:`ReplayStatus` resulting from the operation, indicating success or failure.");
+  ReplayStatus status;
+  DOCUMENT(R"(The ident where the new application is listening for target control, or 0 if something
+went wrong.
+)");
+  uint32_t ident;
+};
+
+DECLARE_REFLECTION_STRUCT(ExecuteResult);
+
+// there's not a good way to document a callback, so for lack of a better place we declare these
+// here and document them in the main IReplayController. They can be linked to from anywhere by
+// name.
+typedef std::function<bool()> RENDERDOC_KillCallback;
+typedef std::function<void(float)> RENDERDOC_ProgressCallback;
+typedef std::function<WindowingData(bool, const rdcarray<WindowingSystem> &)> RENDERDOC_PreviewWindowCallback;
