@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,7 +25,11 @@
 
 #pragma once
 
+#include "apidefs.h"
+#include "rdcarray.h"
 #include "replay_enums.h"
+#include "resourceid.h"
+#include "stringise.h"
 
 DOCUMENT("A floating point four-component vector");
 struct FloatVector
@@ -34,6 +38,7 @@ struct FloatVector
   FloatVector() : x(0.0f), y(0.0f), z(0.0f), w(0.0f) {}
   FloatVector(const FloatVector &) = default;
   FloatVector(float X, float Y, float Z, float W) : x(X), y(Y), z(Z), w(W) {}
+  FloatVector &operator=(const FloatVector &) = default;
 #if defined(RENDERDOC_QT_COMPAT)
   FloatVector(const QColor &col) : x(col.redF()), y(col.greenF()), z(col.blueF()), w(col.alphaF())
   {
@@ -43,6 +48,7 @@ struct FloatVector
   {
     return x == o.x && y == o.y && z == o.z && w == o.w;
   }
+  bool operator!=(const FloatVector &o) const { return !(*this == o); }
   bool operator<(const FloatVector &o) const
   {
     if(!(x == o.x))
@@ -74,6 +80,7 @@ struct PathEntry
   PathEntry() : flags(PathProperty::NoFlags), lastmod(0), size(0) {}
   PathEntry(const PathEntry &) = default;
   PathEntry(const char *fn, PathProperty f) : filename(fn), flags(f), lastmod(0), size(0) {}
+  PathEntry &operator=(const PathEntry &) = default;
   bool operator==(const PathEntry &o) const
   {
     return filename == o.filename && flags == o.flags && lastmod == o.lastmod && size == o.size;
@@ -111,6 +118,7 @@ struct SectionProperties
   DOCUMENT("");
   SectionProperties() = default;
   SectionProperties(const SectionProperties &) = default;
+  SectionProperties &operator=(const SectionProperties &) = default;
 
   DOCUMENT("The name of this section.");
   rdcstr name;
@@ -153,6 +161,7 @@ struct ResourceFormat
     flags = 0;
   }
   ResourceFormat(const ResourceFormat &) = default;
+  ResourceFormat &operator=(const ResourceFormat &) = default;
 
   bool operator==(const ResourceFormat &r) const
   {
@@ -283,6 +292,70 @@ Invalid values will result in 1 being set.
       flags |= ResourceFormat_3Planes;
   }
 
+  DOCUMENT(R"(Return the size of a single element in this format, usually a pixel. For regular sized
+formats this is just :data:`compByteWidth` times :data:`compCount`, for special packed formats it's
+the tightly packed size of a single element, with no padding.
+
+Block-compressed formats define an 'element' as a whole block of texels.
+
+YUV formats where texel size varies depending on subsampling will return the size of a decompressed
+texel.
+
+:return: The size of an element
+:rtype: int
+)");
+  uint32_t ElementSize() const
+  {
+    switch(type)
+    {
+      case ResourceFormatType::Undefined: break;
+      case ResourceFormatType::Regular: return compByteWidth * compCount;
+      case ResourceFormatType::BC1:
+      case ResourceFormatType::BC4:
+        return 8;    // 8 bytes for 4x4 block
+      case ResourceFormatType::BC2:
+      case ResourceFormatType::BC3:
+      case ResourceFormatType::BC5:
+      case ResourceFormatType::BC6:
+      case ResourceFormatType::BC7:
+        return 16;    // 16 bytes for 4x4 block
+      case ResourceFormatType::ETC2: return 8;
+      case ResourceFormatType::EAC:
+        if(compCount == 1)
+          return 8;    // single channel R11 EAC
+        else if(compCount == 2)
+          return 16;    // two channel RG11 EAC
+        else
+          return 16;    // RGBA8 EAC
+      case ResourceFormatType::ASTC:
+        return 16;    // ASTC is always 128 bits per block
+      case ResourceFormatType::R10G10B10A2:
+      case ResourceFormatType::R11G11B10:
+      case ResourceFormatType::R9G9B9E5: return 4;
+      case ResourceFormatType::R5G6B5:
+      case ResourceFormatType::R5G5B5A1:
+      case ResourceFormatType::R4G4B4A4: return 2;
+      case ResourceFormatType::R4G4: return 1;
+      case ResourceFormatType::D16S8:
+        return 3;    // we define the size as tightly packed, so 3 bytes.
+      case ResourceFormatType::D24S8: return 4;
+      case ResourceFormatType::D32S8:
+        return 5;    // we define the size as tightly packed, so 5 bytes.
+      case ResourceFormatType::S8:
+      case ResourceFormatType::A8:
+        return 1;
+      // can't give a sensible answer for YUV formats as the texel varies.
+      case ResourceFormatType::YUV8: return compCount;
+      case ResourceFormatType::YUV10:
+      case ResourceFormatType::YUV12:
+      case ResourceFormatType::YUV16: return compCount * 2;
+      case ResourceFormatType::PVRTC:
+        return 8;    // our representation can't differentiate 2bpp from 4bpp, so guess
+    }
+
+    return 0;
+  }
+
   ResourceFormatType type;
 
   DOCUMENT("The :class:`type <CompType>` of each component.");
@@ -321,6 +394,7 @@ struct TextureFilter
   DOCUMENT("");
   TextureFilter() = default;
   TextureFilter(const TextureFilter &) = default;
+  TextureFilter &operator=(const TextureFilter &) = default;
 
   bool operator==(const TextureFilter &o) const
   {
@@ -356,6 +430,7 @@ struct ResourceDescription
   DOCUMENT("");
   ResourceDescription() = default;
   ResourceDescription(const ResourceDescription &) = default;
+  ResourceDescription &operator=(const ResourceDescription &) = default;
 
   bool operator==(const ResourceDescription &o) const { return resourceId == o.resourceId; }
   bool operator<(const ResourceDescription &o) const { return resourceId < o.resourceId; }
@@ -416,10 +491,12 @@ struct BufferDescription
   DOCUMENT("");
   BufferDescription() = default;
   BufferDescription(const BufferDescription &) = default;
+  BufferDescription &operator=(const BufferDescription &) = default;
 
   bool operator==(const BufferDescription &o) const
   {
-    return resourceId == o.resourceId && creationFlags == o.creationFlags && length == o.length;
+    return resourceId == o.resourceId && creationFlags == o.creationFlags &&
+           gpuAddress == o.gpuAddress && length == o.length;
   }
   bool operator<(const BufferDescription &o) const
   {
@@ -427,6 +504,8 @@ struct BufferDescription
       return resourceId < o.resourceId;
     if(!(creationFlags == o.creationFlags))
       return creationFlags < o.creationFlags;
+    if(!(gpuAddress == o.gpuAddress))
+      return gpuAddress < o.gpuAddress;
     if(!(length == o.length))
       return length < o.length;
     return false;
@@ -435,10 +514,13 @@ struct BufferDescription
   ResourceId resourceId;
 
   DOCUMENT("The way this buffer will be used in the pipeline.");
-  BufferCategory creationFlags;
+  BufferCategory creationFlags = BufferCategory::NoFlags;
+
+  DOCUMENT("The known base GPU Address of this buffer. 0 if not applicable or available.");
+  uint64_t gpuAddress = 0;
 
   DOCUMENT("The byte length of the buffer.");
-  uint64_t length;
+  uint64_t length = 0;
 };
 
 DECLARE_REFLECTION_STRUCT(BufferDescription);
@@ -449,6 +531,7 @@ struct TextureDescription
   DOCUMENT("");
   TextureDescription() = default;
   TextureDescription(const TextureDescription &) = default;
+  TextureDescription &operator=(const TextureDescription &) = default;
 
   bool operator==(const TextureDescription &o) const
   {
@@ -541,6 +624,7 @@ struct APIEvent
   DOCUMENT("");
   APIEvent() = default;
   APIEvent(const APIEvent &) = default;
+  APIEvent &operator=(const APIEvent &) = default;
 
   bool operator==(const APIEvent &o) const { return eventId == o.eventId; }
   bool operator<(const APIEvent &o) const { return eventId < o.eventId; }
@@ -558,20 +642,20 @@ Also eventIds may not correspond directly to an actual function call - sometimes
 a multi draw indirect will be one function call that expands to multiple events to allow inspection
 of results part way through the multi draw.
 )");
-  uint32_t eventId;
+  uint32_t eventId = 0;
 
   DOCUMENT("A list of addresses in the CPU callstack where this function was called.");
   rdcarray<uint64_t> callstack;
 
   DOCUMENT("The chunk index for this function call in the structured file.");
-  uint32_t chunkIndex;
+  uint32_t chunkIndex = 0;
 
   DOCUMENT(R"(A byte offset in the data stream where this event happens.
 
 .. note:: This should only be used as a relative measure, it is not a literal number of bytes from
   the start of the file on disk.
 )");
-  uint64_t fileOffset;
+  uint64_t fileOffset = 0;
 };
 
 DECLARE_REFLECTION_STRUCT(APIEvent);
@@ -582,6 +666,7 @@ struct DebugMessage
   DOCUMENT("");
   DebugMessage() = default;
   DebugMessage(const DebugMessage &) = default;
+  DebugMessage &operator=(const DebugMessage &) = default;
 
   bool operator==(const DebugMessage &o) const
   {
@@ -659,6 +744,7 @@ struct ConstantBindStats
   DOCUMENT("");
   ConstantBindStats() = default;
   ConstantBindStats(const ConstantBindStats &) = default;
+  ConstantBindStats &operator=(const ConstantBindStats &) = default;
 
   static const BucketRecordType BucketType = BucketRecordType::Pow2;
   static const size_t BucketCount = 31;
@@ -687,6 +773,7 @@ struct SamplerBindStats
   DOCUMENT("");
   SamplerBindStats() = default;
   SamplerBindStats(const SamplerBindStats &) = default;
+  SamplerBindStats &operator=(const SamplerBindStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -709,6 +796,7 @@ struct ResourceBindStats
   DOCUMENT("");
   ResourceBindStats() = default;
   ResourceBindStats(const ResourceBindStats &) = default;
+  ResourceBindStats &operator=(const ResourceBindStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -746,6 +834,7 @@ struct ResourceUpdateStats
   DOCUMENT("");
   ResourceUpdateStats() = default;
   ResourceUpdateStats(const ResourceUpdateStats &) = default;
+  ResourceUpdateStats &operator=(const ResourceUpdateStats &) = default;
 
   static const BucketRecordType BucketType = BucketRecordType::Pow2;
   static const size_t BucketCount = 31;
@@ -790,6 +879,7 @@ struct DrawcallStats
   DOCUMENT("");
   DrawcallStats() = default;
   DrawcallStats(const DrawcallStats &) = default;
+  DrawcallStats &operator=(const DrawcallStats &) = default;
 
   static const BucketRecordType BucketType = BucketRecordType::Linear;
   static const size_t BucketSize = 1;
@@ -814,6 +904,7 @@ struct DispatchStats
   DOCUMENT("");
   DispatchStats() = default;
   DispatchStats(const DispatchStats &) = default;
+  DispatchStats &operator=(const DispatchStats &) = default;
 
   DOCUMENT("How many dispatch calls were made.");
   uint32_t calls;
@@ -830,6 +921,7 @@ struct IndexBindStats
   DOCUMENT("");
   IndexBindStats() = default;
   IndexBindStats(const IndexBindStats &) = default;
+  IndexBindStats &operator=(const IndexBindStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -849,6 +941,7 @@ struct VertexBindStats
   DOCUMENT("");
   VertexBindStats() = default;
   VertexBindStats(const VertexBindStats &) = default;
+  VertexBindStats &operator=(const VertexBindStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -872,6 +965,7 @@ struct LayoutBindStats
   DOCUMENT("");
   LayoutBindStats() = default;
   LayoutBindStats(const LayoutBindStats &) = default;
+  LayoutBindStats &operator=(const LayoutBindStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -891,6 +985,7 @@ struct ShaderChangeStats
   DOCUMENT("");
   ShaderChangeStats() = default;
   ShaderChangeStats(const ShaderChangeStats &) = default;
+  ShaderChangeStats &operator=(const ShaderChangeStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -913,6 +1008,7 @@ struct BlendStats
   DOCUMENT("");
   BlendStats() = default;
   BlendStats(const BlendStats &) = default;
+  BlendStats &operator=(const BlendStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -935,6 +1031,7 @@ struct DepthStencilStats
   DOCUMENT("");
   DepthStencilStats() = default;
   DepthStencilStats(const DepthStencilStats &) = default;
+  DepthStencilStats &operator=(const DepthStencilStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -957,6 +1054,7 @@ struct RasterizationStats
   DOCUMENT("");
   RasterizationStats() = default;
   RasterizationStats(const RasterizationStats &) = default;
+  RasterizationStats &operator=(const RasterizationStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -985,6 +1083,7 @@ struct OutputTargetStats
   DOCUMENT("");
   OutputTargetStats() = default;
   OutputTargetStats(const OutputTargetStats &) = default;
+  OutputTargetStats &operator=(const OutputTargetStats &) = default;
 
   DOCUMENT("How many function calls were made.");
   uint32_t calls;
@@ -1010,6 +1109,7 @@ struct FrameStatistics
   DOCUMENT("");
   FrameStatistics() = default;
   FrameStatistics(const FrameStatistics &) = default;
+  FrameStatistics &operator=(const FrameStatistics &) = default;
 
   DOCUMENT("``True`` if the statistics in this structure are valid.");
   bool recorded;
@@ -1059,7 +1159,12 @@ struct FrameStatistics
 
 DECLARE_REFLECTION_STRUCT(FrameStatistics);
 
-DOCUMENT("Contains frame-level global information");
+DOCUMENT(R"(Contains frame-level global information
+
+.. data:: NoFrameNumber
+
+  No frame number is available.
+)");
 struct FrameDescription
 {
   DOCUMENT("");
@@ -1074,12 +1179,14 @@ struct FrameDescription
   {
   }
   FrameDescription(const FrameDescription &) = default;
+  FrameDescription &operator=(const FrameDescription &) = default;
 
   DOCUMENT(R"(Starting from frame #1 defined as the time from application startup to first present,
 this counts the frame number when the capture was made.
 
 .. note:: This value is only accurate if the capture was triggered through the default mechanism, if
-  it was triggered from the application API it doesn't correspond to anything.
+  it was triggered from the application API it doesn't correspond to anything and will be set to
+  :data:`NoFrameNumber`.
 )");
   uint32_t frameNumber;
 
@@ -1110,6 +1217,8 @@ this counts the frame number when the capture was made.
 
   DOCUMENT("A list of debug messages that are not associated with any particular event.");
   rdcarray<DebugMessage> debugMessages;
+
+  static const uint32_t NoFrameNumber = ~0U;
 };
 
 DECLARE_REFLECTION_STRUCT(FrameDescription);
@@ -1123,6 +1232,7 @@ struct EventUsage
   EventUsage(const EventUsage &) = default;
   EventUsage(uint32_t e, ResourceUsage u) : eventId(e), usage(u) {}
   EventUsage(uint32_t e, ResourceUsage u, ResourceId v) : eventId(e), usage(u), view(v) {}
+  EventUsage &operator=(const EventUsage &) = default;
   bool operator<(const EventUsage &o) const
   {
     if(!(eventId == o.eventId))
@@ -1149,6 +1259,7 @@ struct DrawcallDescription
   DOCUMENT("");
   DrawcallDescription() { Reset(); }
   DrawcallDescription(const DrawcallDescription &) = default;
+  DrawcallDescription &operator=(const DrawcallDescription &) = default;
 
   DOCUMENT("Resets the drawcall back to a default/empty state.");
   void Reset()
@@ -1288,6 +1399,7 @@ struct APIProperties
   DOCUMENT("");
   APIProperties() = default;
   APIProperties(const APIProperties &) = default;
+  APIProperties &operator=(const APIProperties &) = default;
 
   DOCUMENT("The :class:`GraphicsAPI` of the actual log/capture.");
   GraphicsAPI pipelineType = GraphicsAPI::D3D11;
@@ -1329,6 +1441,11 @@ DECLARE_REFLECTION_STRUCT(APIProperties);
 DOCUMENT("Gives information about the driver for this API.");
 struct DriverInformation
 {
+  DOCUMENT("");
+  DriverInformation() = default;
+  DriverInformation(const DriverInformation &) = default;
+  DriverInformation &operator=(const DriverInformation &) = default;
+
   DOCUMENT("The :class:`GPUVendor` that provides this driver");
   GPUVendor vendor;
 
@@ -1352,6 +1469,7 @@ struct Uuid
 
   Uuid() { words[0] = words[1] = words[2] = words[3] = 0; }
   Uuid(const Uuid &) = default;
+  Uuid &operator=(const Uuid &) = default;
 
   DOCUMENT("Compares two ``Uuid`` objects for less-than.");
   bool operator<(const Uuid &rhs) const
@@ -1373,6 +1491,7 @@ struct CounterDescription
   DOCUMENT("");
   CounterDescription() = default;
   CounterDescription(const CounterDescription &) = default;
+  CounterDescription &operator=(const CounterDescription &) = default;
 
   DOCUMENT(R"(The :class:`GPUCounter` this counter represents.
 
@@ -1464,6 +1583,7 @@ struct CounterResult
     value.u64 = data;
   }
 #endif
+  CounterResult &operator=(const CounterResult &) = default;
 
   DOCUMENT("Compares two ``CounterResult`` objects for less-than.");
   bool operator<(const CounterResult &o) const
@@ -1519,12 +1639,57 @@ union PixelValue
 
 DECLARE_REFLECTION_STRUCT(PixelValue);
 
+DOCUMENT("Specifies a subresource within a texture.");
+struct Subresource
+{
+  DOCUMENT("");
+  Subresource(uint32_t mip = 0, uint32_t slice = 0, uint32_t sample = 0)
+      : mip(mip), slice(slice), sample(sample)
+  {
+  }
+  Subresource(const Subresource &) = default;
+  Subresource &operator=(const Subresource &) = default;
+
+  bool operator==(const Subresource &o) const
+  {
+    return mip == o.mip && slice == o.slice && sample == o.sample;
+  }
+  bool operator!=(const Subresource &o) const { return !(*this == o); }
+  bool operator<(const Subresource &o) const
+  {
+    if(!(mip == o.mip))
+      return mip < o.mip;
+    if(!(slice == o.slice))
+      return slice < o.slice;
+    if(!(sample == o.sample))
+      return sample < o.sample;
+    return false;
+  }
+
+  DOCUMENT("The mip level in the texture.");
+  uint32_t mip;
+  DOCUMENT(R"(The slice within the texture. For 3D textures this is a depth slice, for arrays it is
+an array slice.
+
+.. note::
+  Cubemaps are simply 2D array textures with a special meaning, so the faces of a cubemap are the 2D
+  array slices in the standard order: X+, X-, Y+, Y-, Z+, Z-. Cubemap arrays are 2D arrays with
+  ``6 * N`` faces, where each cubemap within the array takes up 6 slices in the above order.
+)");
+  uint32_t slice;
+  DOCUMENT("The sample in a multisampled texture.");
+  uint32_t sample;
+};
+
+DECLARE_REFLECTION_STRUCT(Subresource);
+
 DOCUMENT("The value of pixel output at a particular event.");
 struct ModificationValue
 {
   DOCUMENT("");
   ModificationValue() = default;
   ModificationValue(const ModificationValue &) = default;
+  ModificationValue &operator=(const ModificationValue &) = default;
 
   bool operator==(const ModificationValue &o) const
   {
@@ -1558,6 +1723,7 @@ struct PixelModification
   DOCUMENT("");
   PixelModification() = default;
   PixelModification(const PixelModification &) = default;
+  PixelModification &operator=(const PixelModification &) = default;
 
   bool operator==(const PixelModification &o) const
   {
@@ -1672,6 +1838,7 @@ struct Thumbnail
   DOCUMENT("");
   Thumbnail() = default;
   Thumbnail(const Thumbnail &) = default;
+  Thumbnail &operator=(const Thumbnail &) = default;
 
   DOCUMENT("The :class:`FileType` of the data in the thumbnail.");
   FileType type = FileType::Raw;
