@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,6 +25,9 @@
 
 #pragma once
 
+#include "apidefs.h"
+#include "stringise.h"
+
 DOCUMENT(R"(The types of several pre-defined and known sections. This allows consumers of the API
 to recognise and understand the contents of the section.
 
@@ -186,7 +189,7 @@ DOCUMENT(R"(Represents the base type of a shader variable in debugging or consta
 
   An unknown type.
 )");
-enum class VarType : uint32_t
+enum class VarType : uint8_t
 {
   Float = 0,
   Double,
@@ -199,11 +202,30 @@ enum class VarType : uint32_t
   ULong,
   SByte,
   UByte,
-  Unknown = ~0U,
+  Unknown = 0xFF,
 };
 
 DECLARE_REFLECTION_ENUM(VarType);
 
+DOCUMENT(R"(Get the byte size of a variable type.
+
+:param VarType type: The variable type
+:return: The size in bytes of this type
+:rtype: int
+)");
+constexpr uint32_t VarTypeByteSize(VarType type)
+{
+  // temporarily disable clang-format to make this more readable.
+  // Ideally we'd use a simple switch() but VS2015 doesn't support that :(.
+  // clang-format off
+  return (type == VarType::UByte  || type == VarType::SByte) ? 1
+       : (type == VarType::Half   || type == VarType::UShort || type == VarType::SShort) ? 2
+       : (type == VarType::Float  || type == VarType::UInt   || type == VarType::SInt  ) ? 4
+       : (type == VarType::Double || type == VarType::ULong  || type == VarType::SLong ) ? 8
+       : 0;
+  // clang-format on
+}
+
 DOCUMENT(R"(Represents the component type of a channel in a texture or element in a structure.
 
 .. data:: Typeless
@@ -327,15 +349,27 @@ DOCUMENT(R"(A texture addressing mode in a single direction (U,V or W).
 
   The texture is tiled at every multiple of 1.0.
 
+.. data:: Repeat
+
+  Alias of :data:`Wrap`.
+
 .. data:: Mirror
 
   The texture is tiled as with :data:`Wrap`, but with the absolute value of the texture co-ordinate.
 
+.. data:: MirrorRepeat
+
+  Alias of :data:`Mirror`.
+
 .. data:: MirrorOnce
 
   The texture is mirrored with :data:`Mirror`, but the texture does not tile as with
   :data:`ClampEdge`.
 
+.. data:: MirrorClamp
+
+  Alias of :data:`MirrorOnce`.
+
 .. data:: ClampEdge
 
   The texture is clamped to the range of ``[0.0, 1.0]`` and the texture value at each end used.
@@ -348,8 +382,11 @@ DOCUMENT(R"(A texture addressing mode in a single direction (U,V or W).
 enum class AddressMode : uint32_t
 {
   Wrap,
+  Repeat = Wrap,
   Mirror,
+  MirrorRepeat = Mirror,
   MirrorOnce,
+  MirrorClamp = MirrorOnce,
   ClampEdge,
   ClampBorder,
 };
@@ -828,7 +865,69 @@ to apply to multiple related things - see :data:`ClipDistance`, :data:`CullDista
 .. data:: StencilReference
 
   The stencil reference to be used for stenciling operations on this fragment.
+  
+.. data:: PointCoord
 
+  The fragments co-ordinates within a point primitive being rasterized.
+
+.. data:: IsHelper
+
+  Indicates if the current invocation is a helper invocation.
+
+.. data:: SubgroupSize
+
+  The number of invocations in a subgroup.
+
+.. data:: NumSubgroups
+
+  The number of subgroups in the local workgroup.
+
+.. data:: SubgroupIndexInWorkgroup
+
+  The index of the current subgroup within all subgroups in the workgroup, up to
+  :data:`NumSubgroups` - 1.
+
+.. data:: IndexInSubgroup
+
+  The index of the current thread in the current subgroup, up to :data:`SubgroupSize` - 1.
+
+.. data:: SubgroupEqualMask
+
+  A bitmask where the bit corresponding to :data:`IndexInSubgroup` is set.
+
+.. data:: SubgroupGreaterEqualMask
+
+  A bitmask where all bits greater or equal to the one corresponding to :data:`IndexInSubgroup` are
+  set.
+
+.. data:: SubgroupGreaterMask
+
+  A bitmask where all bits greater than the one corresponding to :data:`IndexInSubgroup` are set.
+
+.. data:: SubgroupLessEqualMask
+
+  A bitmask where all bits less or equal to the one corresponding to :data:`IndexInSubgroup` are
+  set.
+
+.. data:: SubgroupLessMask
+
+  A bitmask where all bits less than the one corresponding to :data:`IndexInSubgroup` are set.
+
+.. data:: DeviceIndex
+
+  The device index executing the shader, relative to the current device group.
+
+.. data:: IsFullyCovered
+
+  Indicates if the current fragment area is fully covered by the generating primitive.
+
+.. data:: FragAreaSize
+
+  Gives the dimensions of the area that the fragment covers.
+
+.. data:: FragInvocationCount
+
+  Gives the maximum number of invocations for the fragment being covered.
 )");
 enum class ShaderBuiltin : uint32_t
 {
@@ -866,6 +965,21 @@ enum class ShaderBuiltin : uint32_t
   BaseInstance,
   DrawIndex,
   StencilReference,
+  PointCoord,
+  IsHelper,
+  SubgroupSize,
+  NumSubgroups,
+  SubgroupIndexInWorkgroup,
+  IndexInSubgroup,
+  SubgroupEqualMask,
+  SubgroupGreaterEqualMask,
+  SubgroupGreaterMask,
+  SubgroupLessEqualMask,
+  SubgroupLessMask,
+  DeviceIndex,
+  IsFullyCovered,
+  FragAreaSize,
+  FragInvocationCount,
   Count,
 };
 
@@ -1140,7 +1254,7 @@ enum class AlphaMapping : uint32_t
 ITERABLE_OPERATORS(AlphaMapping);
 DECLARE_REFLECTION_ENUM(AlphaMapping);
 
-DOCUMENT(R"(A resource format's particular type. This accounts for either block-compressed textures
+DOCUMENT2(R"(A resource format's particular type. This accounts for either block-compressed textures
 or formats that don't have equal byte-multiple sizes for each channel.
 
 .. data:: Regular
@@ -1201,12 +1315,22 @@ or formats that don't have equal byte-multiple sizes for each channel.
 
   Commonly used on mobile or embedded platforms.
 
+  Note that the mode added in ``EAC`` with 1 byte per pixel and full 8-bit alpha is
+  grouped as ``EAC``, with a component count of 4. See :data:`EAC`.
+)",
+          R"(
 .. data:: EAC
 
-  A block-compressed texture in ``EAC`` format (Single channel 11-bit or RGBA, 0.5 bytes per pixel)
+  A block-compressed texture in ``EAC`` format, expanded from ``ETC2``.
 
   Commonly used on mobile or embedded platforms.
 
+  The single and dual channel formats encode 11-bit data with 0.5 bytes per channel (so
+  the single channel format is 0.5 bytes per pixel total, and the dual channel format is 1 byte per
+  pixel total). The four channel format is encoded similarly to ETC2 for the base RGB data and
+  similarly to the single channel format for the alpha, giving 1 byte per pixel total.
+  See :data:`ETC2`.
+
 .. data:: ASTC
 
   A block-compressed texture in ``ASTC`` format (Representation varies a lot)
@@ -1297,6 +1421,10 @@ or formats that don't have equal byte-multiple sizes for each channel.
 .. data:: PVRTC
 
   PowerVR properitary texture compression format.
+
+.. data:: A8
+
+  8-bit unsigned normalised alpha - equivalent to standard R8 with a pre-baked swizzle.
 )");
 enum class ResourceFormatType : uint8_t
 {
@@ -1328,6 +1456,7 @@ enum class ResourceFormatType : uint8_t
   YUV12,
   YUV16,
   PVRTC,
+  A8,
 };
 
 DECLARE_REFLECTION_ENUM(ResourceFormatType);
@@ -1432,6 +1561,8 @@ constexpr GPUVendor GPUVendorFromPCIVendor(uint32_t vendorID)
        : vendorID == 0x8086 ? GPUVendor::Intel
        : vendorID == 0x10DE ? GPUVendor::nVidia
        : vendorID == 0x5143 ? GPUVendor::Qualcomm
+       : vendorID == 0x1AE0 ? GPUVendor::Software   // Google Swiftshader
+       : vendorID == 0x1414 ? GPUVendor::Software   // Microsoft WARP
        : GPUVendor::Unknown;
   // clang-format on
 }
@@ -1792,6 +1923,22 @@ constexpr inline uint32_t PatchList_Count(Topology topology)
              : uint32_t(topology) - uint32_t(Topology::PatchList_1CPs) + 1;
 }
 
+DOCUMENT(R"(Check whether or not this topology supports primitive restart.
+
+.. note:: This is almost but not quite the same as being a line/triangle strip - triangle fans
+  also support restart. See also :func:`IsStrip`.
+
+:param Topology t: The topology to check.
+:return: ``True`` if it describes a topology that allows restart, ``False`` for a list.
+:rtype: ``bool``
+)");
+constexpr inline bool SupportsRestart(Topology topology)
+{
+  return topology == Topology::LineStrip || topology == Topology::TriangleStrip ||
+         topology == Topology::LineStrip_Adj || topology == Topology::TriangleStrip_Adj ||
+         topology == Topology::TriangleFan;
+}
+
 DOCUMENT(R"(Check whether or not this is a strip-type topology.
 
 :param Topology t: The topology to check.
@@ -2419,6 +2566,34 @@ enum class ConservativeRaster : uint32_t
 
 DECLARE_REFLECTION_ENUM(ConservativeRaster);
 
+DOCUMENT(R"(The line rasterization mode.
+
+.. data:: Default
+
+  Default line rasterization mode as defined by the API specification.
+
+.. data:: Rectangular
+
+  Lines are rasterized as rectangles extruded from the line.
+
+.. data:: Bresenham
+
+  Lines are rasterized according to the bresenham line algorithm.
+
+.. data:: RectangularSmooth
+
+  Lines are rasterized as rectangles extruded from the line with coverage falloff.
+)");
+enum class LineRaster : uint32_t
+{
+  Default = 0,
+  Rectangular,
+  Bresenham,
+  RectangularSmooth,
+};
+
+DECLARE_REFLECTION_ENUM(LineRaster);
+
 DOCUMENT(R"(The texture filtering mode for a given direction (minification, magnification, or
 between mips).
 
@@ -2964,7 +3139,10 @@ enum class GPUCounter : uint32_t
   FirstNvidia = 3000000,
   LastIntel = FirstNvidia - 1,
 
-  LastNvidia = 4000000,
+  FirstVulkanExtended = 4000000,
+  LastNvidia = FirstVulkanExtended - 1,
+
+  LastVulkanExtended = 5000000,
 };
 
 ITERABLE_OPERATORS(GPUCounter);
@@ -3014,6 +3192,17 @@ inline constexpr bool IsNvidiaCounter(GPUCounter c)
   return c >= GPUCounter::FirstNvidia && c <= GPUCounter::LastNvidia;
 }
 
+DOCUMENT(R"(Check whether or not this is a KHR counter.
+
+:param GPUCounter c: The counter.
+:return: ``True`` if it is a Vulkan counter reported through the VK_KHR_performance_query extension, ``False`` if it's not.
+:rtype: ``bool``
+)");
+inline constexpr bool IsVulkanExtendedCounter(GPUCounter c)
+{
+  return c >= GPUCounter::FirstVulkanExtended && c <= GPUCounter::LastVulkanExtended;
+}
+
 DOCUMENT(R"(The unit that GPU counter data is returned in.
 
 .. data:: Absolute
@@ -3184,10 +3373,14 @@ a remote server.
 
 .. data:: AndroidAPKFolderNotFound
 
-  Couldn't find the build-android folder which contains the Android remote server APK.
+  Couldn't find the folder which contains the Android remote server APK.
 
 .. data:: AndroidAPKInstallFailed
 
+  Failed to install Android remote server for unknown reasons.
+
+.. data:: AndroidAPKVerifyFailed
+
   Failed to install Android remote server.
 )");
 enum class ReplayStatus : uint32_t
@@ -3215,7 +3408,8 @@ enum class ReplayStatus : uint32_t
   AndroidGrantPermissionsFailed,
   AndroidABINotFound,
   AndroidAPKFolderNotFound,
-  AndroidAPKInstallFailed
+  AndroidAPKInstallFailed,
+  AndroidAPKVerifyFailed,
 };
 
 DECLARE_REFLECTION_ENUM(ReplayStatus);
@@ -3328,6 +3522,11 @@ enum class EnvSep : uint32_t
 
 DECLARE_REFLECTION_ENUM(EnvSep);
 
+// see comment in common.h
+#if !defined(LOGTYPE_DEFINED)
+
+#define LOGTYPE_DEFINED
+
 DOCUMENT(R"(The type of a log message
 
 .. data:: Debug
@@ -3350,9 +3549,10 @@ DOCUMENT(R"(The type of a log message
 
   The log message indicates a fatal error occurred which is impossible to recover from.
 )");
-enum class LogType : int32_t
+enum class LogType : uint32_t
 {
   Debug,
+  First = Debug,
   Comment,
   Warning,
   Error,
@@ -3360,8 +3560,101 @@ enum class LogType : int32_t
   Count,
 };
 
+#endif
+
+// this is OUTSIDE the #endif because we don't declare these in common.h, so in case they're needed
+// we define them here
 DECLARE_REFLECTION_ENUM(LogType);
 
+ITERABLE_OPERATORS(LogType);
+
+DOCUMENT(R"(The level of optimisation used in
+
+.. data:: NoOptimisation
+
+  Completely disabled, no optimisation will be used at all.
+
+.. data:: Conservative
+
+  Optimisation is used when it doesn't interfere with replay correctness.
+
+.. data:: Balanced
+
+  Optimisation is used when it has minimal impact on replay correctness. This could include e.g.
+  resources appearing cleared instead of containing contents from prior frames where those resources
+  are written to before being read.
+
+.. data:: Fastest
+
+  All possible optimisations are enabled as long as they do not cause invalid/incorrect replay.
+  This could result in side-effects like data from one replay being visible early in another replay,
+  if it's known that the data will be overwritten before being used.
+)");
+enum class ReplayOptimisationLevel : uint32_t
+{
+  NoOptimisation,
+  First = NoOptimisation,
+  Conservative,
+  Balanced,
+  Fastest,
+  Count,
+};
+
+DECLARE_REFLECTION_ENUM(ReplayOptimisationLevel);
+ITERABLE_OPERATORS(ReplayOptimisationLevel);
+
+DOCUMENT(R"(Specifies a windowing system to use for creating an output window.
+
+.. data:: Unknown
+
+  Unknown window type, no windowing data is passed and no native window is described.
+
+.. data:: Headless
+
+  The windowing data doesn't describe a real window but a virtual area, allowing all normal output
+  rendering to happen off-screen.
+  See :func:`CreateHeadlessWindowingData`.
+
+.. data:: Win32
+
+  The windowing data refers to a Win32 window. See :func:`CreateWin32WindowingData`.
+
+.. data:: Xlib
+
+  The windowing data refers to an Xlib window. See :func:`CreateXLibWindowingData`.
+
+.. data:: XCB
+
+  The windowing data refers to an XCB window. See :func:`CreateXCBWindowingData`.
+
+.. data:: Wayland
+
+  The windowing data refers to an Wayland window. See :func:`CreateWaylandWindowingData`.
+
+.. data:: Android
+
+  The windowing data refers to an Android window. See :func:`CreateAndroidWindowingData`.
+
+.. data:: MacOS
+
+  The windowing data refers to a MacOS / OS X NSView & CALayer that is Metal/GL compatible.
+  See :func:`CreateMacOSWindowingData`.
+)");
+enum class WindowingSystem : uint32_t
+{
+  Unknown,
+  Headless,
+  Win32,
+  Xlib,
+  XCB,
+  Android,
+  MacOS,
+  GGP,
+  Wayland,
+};
+
+DECLARE_REFLECTION_ENUM(WindowingSystem);
+
 #if defined(ENABLE_PYTHON_FLAG_ENUMS)
 
 ENABLE_PYTHON_FLAG_ENUMS;
@@ -3807,10 +4100,18 @@ DOCUMENT(R"(A set of flags giving details of the current status of vulkan layer
 
   Fixing any issues will require elevation to system administrator privileges.
 
-.. data:: CouldElevate
+.. data:: UserRegisterable
 
-  Fixing issues could be done purely as a user, but can optionally be done at system level with
-  system administrator privileges.
+  This layer can be registered as user-local, as well as system-wide. If :data:`NeedElevation` isn't
+  also set then the entire process can be done un-elevated if user-local is desired.
+
+  .. note::
+
+    If the :data:`NeedElevation` flag is set then elevation is required to fix the layer
+    registration, even if a user-local registration is desired.
+
+    Most commonly this situation arises if there is no other registration, or the existing one is
+    already user-local.
 
 .. data:: RegisterAll
 
@@ -3832,7 +4133,7 @@ enum class VulkanLayerFlags : uint32_t
   OtherInstallsRegistered = 0x1,
   ThisInstallRegistered = 0x2,
   NeedElevation = 0x4,
-  CouldElevate = 0x8,
+  UserRegisterable = 0x8,
   RegisterAll = 0x10,
   UpdateAllowed = 0x20,
   Unfixable = 0x40,
@@ -3882,4 +4183,4 @@ DECLARE_REFLECTION_ENUM(AndroidFlags);
 
 #if defined(DISABLE_PYTHON_FLAG_ENUMS)
 DISABLE_PYTHON_FLAG_ENUMS;
-#endif
\ No newline at end of file
+#endif
