@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -26,8 +26,12 @@
 #pragma once
 
 #include <stdint.h>
-#include "basic_types.h"
+#include "apidefs.h"
+#include "rdcarray.h"
+#include "rdcstr.h"
 #include "replay_enums.h"
+#include "resourceid.h"
+#include "stringise.h"
 
 DOCUMENT("A ``float`` 4 component vector.")
 struct FloatVecVal
@@ -35,6 +39,7 @@ struct FloatVecVal
   DOCUMENT("");
   FloatVecVal() = default;
   FloatVecVal(const FloatVecVal &) = default;
+  FloatVecVal &operator=(const FloatVecVal &) = default;
 
   DOCUMENT("The x component.");
   float x;
@@ -52,6 +57,7 @@ struct DoubleVecVal
   DOCUMENT("");
   DoubleVecVal() = default;
   DoubleVecVal(const DoubleVecVal &) = default;
+  DoubleVecVal &operator=(const DoubleVecVal &) = default;
 
   DOCUMENT("The x component.");
   double x;
@@ -69,6 +75,7 @@ struct IntVecVal
   DOCUMENT("");
   IntVecVal() = default;
   IntVecVal(const IntVecVal &) = default;
+  IntVecVal &operator=(const IntVecVal &) = default;
 
   DOCUMENT("The x component.");
   int32_t x;
@@ -86,6 +93,7 @@ struct UIntVecVal
   DOCUMENT("");
   UIntVecVal() = default;
   UIntVecVal(const UIntVecVal &) = default;
+  UIntVecVal &operator=(const UIntVecVal &) = default;
 
   DOCUMENT("The x component.");
   uint32_t x;
@@ -97,6 +105,26 @@ struct UIntVecVal
   uint32_t w;
 };
 
+DOCUMENT("A 64-bit pointer value with optional type information.")
+struct PointerVal
+{
+  DOCUMENT("");
+  PointerVal() = default;
+  PointerVal(const PointerVal &) = default;
+  PointerVal &operator=(const PointerVal &) = default;
+
+  DOCUMENT("The actual pointer value itself.");
+  uint64_t pointer;
+
+  DOCUMENT("An optional :class:`ResourceId` identifying the shader containing the type info.");
+  ResourceId shader;
+
+  DOCUMENT("The index into :data:`ShaderReflection.pointerTypes` of the pointed type.");
+  uint32_t pointerTypeID;
+};
+
+DECLARE_STRINGISE_TYPE(PointerVal);
+
 DOCUMENT("A C union that holds 16 values, with each different basic variable type.");
 union ShaderValue
 {
@@ -141,18 +169,19 @@ struct ShaderVariable
   {
     name = "";
     rows = columns = 0;
-    displayAsHex = isStruct = rowMajor = false;
+    displayAsHex = isStruct = rowMajor = isPointer = false;
     type = VarType::Float;
     for(int i = 0; i < 16; i++)
       value.uv[i] = 0;
   }
   ShaderVariable(const ShaderVariable &) = default;
-  ShaderVariable(const char *n, float x, float y, float z, float w)
+  ShaderVariable &operator=(const ShaderVariable &) = default;
+  ShaderVariable(const rdcstr &n, float x, float y, float z, float w)
   {
     name = n;
     rows = 1;
     columns = 4;
-    displayAsHex = isStruct = rowMajor = false;
+    displayAsHex = isStruct = rowMajor = isPointer = false;
     for(int i = 0; i < 16; i++)
       value.uv[i] = 0;
     type = VarType::Float;
@@ -161,12 +190,12 @@ struct ShaderVariable
     value.f.z = z;
     value.f.w = w;
   }
-  ShaderVariable(const char *n, int x, int y, int z, int w)
+  ShaderVariable(const rdcstr &n, int x, int y, int z, int w)
   {
     name = n;
     rows = 1;
     columns = 4;
-    displayAsHex = isStruct = rowMajor = false;
+    displayAsHex = isStruct = rowMajor = isPointer = false;
     for(int i = 0; i < 16; i++)
       value.uv[i] = 0;
     type = VarType::SInt;
@@ -175,12 +204,12 @@ struct ShaderVariable
     value.i.z = z;
     value.i.w = w;
   }
-  ShaderVariable(const char *n, uint32_t x, uint32_t y, uint32_t z, uint32_t w)
+  ShaderVariable(const rdcstr &n, uint32_t x, uint32_t y, uint32_t z, uint32_t w)
   {
     name = n;
     rows = 1;
     columns = 4;
-    displayAsHex = isStruct = rowMajor = false;
+    displayAsHex = isStruct = rowMajor = isPointer = false;
     for(int i = 0; i < 16; i++)
       value.uv[i] = 0;
     type = VarType::UInt;
@@ -193,7 +222,8 @@ struct ShaderVariable
   {
     return rows == o.rows && columns == o.columns && name == o.name && type == o.type &&
            displayAsHex == o.displayAsHex && !memcmp(&value, &o.value, sizeof(value)) &&
-           isStruct == o.isStruct && members == o.members;
+           isStruct == o.isStruct && rowMajor == o.rowMajor && isPointer == o.isPointer &&
+           members == o.members;
   }
   bool operator<(const ShaderVariable &o) const
   {
@@ -211,6 +241,8 @@ struct ShaderVariable
       return isStruct < o.isStruct;
     if(!(rowMajor == o.rowMajor))
       return rowMajor < o.rowMajor;
+    if(!(isPointer == o.isPointer))
+      return isPointer < o.isPointer;
     if(memcmp(&value, &o.value, sizeof(value)) < 0)
       return true;
     if(!(members == o.members))
@@ -218,18 +250,16 @@ struct ShaderVariable
     return false;
   }
 
-  DOCUMENT("The number of rows in this matrix.");
-  uint32_t rows;
-  DOCUMENT("The number of columns in this matrix.");
-  uint32_t columns;
   DOCUMENT("The name of this variable.");
   rdcstr name;
 
-  DOCUMENT("The :class:`basic type <VarType>` of this variable.");
-  VarType type;
+  DOCUMENT("The number of rows in this matrix.");
+  uint8_t rows;
+  DOCUMENT("The number of columns in this matrix.");
+  uint8_t columns;
 
-  DOCUMENT("The :class:`contents <ShaderValue>` of this variable if it has no members.");
-  ShaderValue value;
+  DOCUMENT("``True`` if this variable is a pointer.");
+  bool isPointer;
 
   DOCUMENT("``True`` if the contents of this variable should be displayed as hex.");
   bool displayAsHex;
@@ -240,8 +270,56 @@ struct ShaderVariable
   DOCUMENT("``True`` if this variable is stored in rows in memory. Only relevant for matrices.");
   bool rowMajor;
 
+  DOCUMENT("The :class:`basic type <VarType>` of this variable.");
+  VarType type;
+
+  DOCUMENT("The :class:`contents <ShaderValue>` of this variable if it has no members.");
+  ShaderValue value;
+
   DOCUMENT("The members of this variable as a list of :class:`ShaderVariable`.");
   rdcarray<ShaderVariable> members;
+
+  DOCUMENT(R"(Utility function for setting a pointer value with no type information.
+
+:param int pointer: The actual pointer value.
+)");
+  inline void SetTypelessPointer(uint64_t pointer)
+  {
+    isPointer = true;
+    value.u64v[0] = pointer;
+  }
+  DOCUMENT(R"(Utility function for setting a pointer value with type information.
+
+:param int pointer: The actual pointer value.
+:param ResourceId shader: The shader containing the type information.
+:param int pointerTypeID: The type's index in the shader's :data:`ShaderReflection.pointerTypes`
+  list.
+)");
+  inline void SetTypedPointer(uint64_t pointer, ResourceId shader, uint32_t pointerTypeID)
+  {
+    isPointer = true;
+    value.u64v[0] = pointer;
+    value.u64v[1] = pointerTypeID;
+    pointerShader = shader;
+  }
+
+  DOCUMENT(R"(Utility function for getting a pointer value, with optional type information.
+
+:return: A :class:`PointerVal` with the pointer value.
+:rtype: PointerVal
+)");
+  inline PointerVal GetPointer() const
+  {
+    return {value.u64v[0], pointerShader, uint32_t(value.u64v[1] & 0xFFFFFFFF)};
+  }
+
+private:
+  DOCUMENT("");
+  ResourceId pointerShader;
+
+  // make DoSerialise a friend so it can serialise pointerShader
+  template <typename SerialiserType>
+  friend void DoSerialise(SerialiserType &ser, ShaderVariable &el);
 };
 
 DECLARE_REFLECTION_STRUCT(ShaderVariable);
@@ -253,6 +331,7 @@ struct RegisterRange
   DOCUMENT("");
   RegisterRange() = default;
   RegisterRange(const RegisterRange &) = default;
+  RegisterRange &operator=(const RegisterRange &) = default;
 
   bool operator==(const RegisterRange &o) const
   {
@@ -296,6 +375,7 @@ struct LocalVariableMapping
   DOCUMENT("");
   LocalVariableMapping() = default;
   LocalVariableMapping(const LocalVariableMapping &) = default;
+  LocalVariableMapping &operator=(const LocalVariableMapping &) = default;
 
   bool operator==(const LocalVariableMapping &o) const
   {
@@ -355,6 +435,7 @@ struct LineColumnInfo
   DOCUMENT("");
   LineColumnInfo() = default;
   LineColumnInfo(const LineColumnInfo &) = default;
+  LineColumnInfo &operator=(const LineColumnInfo &) = default;
 
   bool operator==(const LineColumnInfo &o) const
   {
@@ -415,6 +496,7 @@ struct ShaderDebugState
   DOCUMENT("");
   ShaderDebugState() = default;
   ShaderDebugState(const ShaderDebugState &) = default;
+  ShaderDebugState &operator=(const ShaderDebugState &) = default;
 
   bool operator==(const ShaderDebugState &o) const
   {
@@ -473,6 +555,7 @@ struct ShaderDebugTrace
   DOCUMENT("");
   ShaderDebugTrace() = default;
   ShaderDebugTrace(const ShaderDebugTrace &) = default;
+  ShaderDebugTrace &operator=(const ShaderDebugTrace &) = default;
 
   DOCUMENT("The input variables for this shader as a list of :class:`ShaderValue`.");
   rdcarray<ShaderVariable> inputs;
@@ -511,6 +594,7 @@ struct SigParameter
   DOCUMENT("");
   SigParameter() = default;
   SigParameter(const SigParameter &) = default;
+  SigParameter &operator=(const SigParameter &) = default;
 
   bool operator==(const SigParameter &o) const
   {
@@ -607,13 +691,14 @@ struct ShaderVariableDescriptor
   DOCUMENT("");
   ShaderVariableDescriptor() = default;
   ShaderVariableDescriptor(const ShaderVariableDescriptor &) = default;
+  ShaderVariableDescriptor &operator=(const ShaderVariableDescriptor &) = default;
 
   bool operator==(const ShaderVariableDescriptor &o) const
   {
     return type == o.type && rows == o.rows && columns == o.columns &&
            rowMajorStorage == o.rowMajorStorage && elements == o.elements &&
            arrayByteStride == o.arrayByteStride && matrixByteStride == o.matrixByteStride &&
-           name == o.name;
+           pointerTypeID == o.pointerTypeID && name == o.name;
   }
   bool operator<(const ShaderVariableDescriptor &o) const
   {
@@ -635,6 +720,14 @@ struct ShaderVariableDescriptor
       return name < o.name;
     return false;
   }
+  DOCUMENT("The name of the type of this constant, e.g. a ``struct`` name.");
+  rdcstr name;
+  DOCUMENT("The index in :data:`ShaderReflection.pointerTypes` of the pointee type.");
+  uint32_t pointerTypeID = ~0U;
+  DOCUMENT("The number of elements in the array, or 1 if it's not an array.");
+  uint32_t elements = 1;
+  DOCUMENT("The number of bytes between the start of one element in the array and the next.");
+  uint32_t arrayByteStride = 0;
   DOCUMENT("The :class:`VarType` that this basic constant stores.");
   VarType type = VarType::Unknown;
   DOCUMENT("The number of rows in this matrix.");
@@ -645,12 +738,12 @@ struct ShaderVariableDescriptor
   uint8_t matrixByteStride = 0;
   DOCUMENT("``True`` if the matrix is stored as row major instead of column major.");
   bool rowMajorStorage = false;
-  DOCUMENT("The number of elements in the array, or 1 if it's not an array.");
-  uint32_t elements = 1;
-  DOCUMENT("The number of bytes between the start of one element in the array and the next.");
-  uint32_t arrayByteStride = 0;
-  DOCUMENT("The name of the type of this constant, e.g. a ``struct`` name.");
-  rdcstr name;
+  DOCUMENT("``True`` if the contents of this variable should be displayed as hex.");
+  bool displayAsHex = false;
+  DOCUMENT(R"(``True`` if the contents of this variable should be displayed as RGB color (where
+ possible).
+ )");
+  bool displayAsRGB = false;
 };
 
 DECLARE_REFLECTION_STRUCT(ShaderVariableDescriptor);
@@ -661,6 +754,7 @@ struct ShaderVariableType
   DOCUMENT("");
   ShaderVariableType() = default;
   ShaderVariableType(const ShaderVariableType &) = default;
+  ShaderVariableType &operator=(const ShaderVariableType &) = default;
 
   bool operator==(const ShaderVariableType &o) const
   {
@@ -689,6 +783,7 @@ struct ShaderConstant
   DOCUMENT("");
   ShaderConstant() = default;
   ShaderConstant(const ShaderConstant &) = default;
+  ShaderConstant &operator=(const ShaderConstant &) = default;
 
   bool operator==(const ShaderConstant &o) const
   {
@@ -697,10 +792,10 @@ struct ShaderConstant
   }
   bool operator<(const ShaderConstant &o) const
   {
-    if(!(name == o.name))
-      return name < o.name;
     if(!(byteOffset == o.byteOffset))
       return byteOffset < o.byteOffset;
+    if(!(name == o.name))
+      return name < o.name;
     if(!(defaultValue == o.defaultValue))
       return defaultValue < o.defaultValue;
     if(!(type == o.type))
@@ -709,7 +804,7 @@ struct ShaderConstant
   }
   DOCUMENT("The name of this constant");
   rdcstr name;
-  DOCUMENT("The byte offset of this constant relative to the start of the block");
+  DOCUMENT("The byte offset of this constant relative to the parent structure");
   uint32_t byteOffset;
   DOCUMENT("If this constant is no larger than a 64-bit constant, gives a default value for it.");
   uint64_t defaultValue;
@@ -729,6 +824,7 @@ struct ConstantBlock
   DOCUMENT("");
   ConstantBlock() = default;
   ConstantBlock(const ConstantBlock &) = default;
+  ConstantBlock &operator=(const ConstantBlock &) = default;
 
   bool operator==(const ConstantBlock &o) const
   {
@@ -779,6 +875,7 @@ struct ShaderSampler
   DOCUMENT("");
   ShaderSampler() = default;
   ShaderSampler(const ShaderSampler &) = default;
+  ShaderSampler &operator=(const ShaderSampler &) = default;
 
   bool operator==(const ShaderSampler &o) const
   {
@@ -814,6 +911,7 @@ struct ShaderResource
   DOCUMENT("");
   ShaderResource() = default;
   ShaderResource(const ShaderResource &) = default;
+  ShaderResource &operator=(const ShaderResource &) = default;
 
   bool operator==(const ShaderResource &o) const
   {
@@ -866,6 +964,7 @@ struct ShaderEntryPoint
 {
   ShaderEntryPoint() = default;
   ShaderEntryPoint(const ShaderEntryPoint &) = default;
+  ShaderEntryPoint &operator=(const ShaderEntryPoint &) = default;
   ShaderEntryPoint(const rdcstr &n, ShaderStage s) : name(n), stage(s) {}
   DOCUMENT("");
   bool operator==(const ShaderEntryPoint &o) const { return name == o.name && stage == o.stage; }
@@ -892,6 +991,7 @@ struct ShaderCompileFlag
   DOCUMENT("");
   ShaderCompileFlag() = default;
   ShaderCompileFlag(const ShaderCompileFlag &) = default;
+  ShaderCompileFlag &operator=(const ShaderCompileFlag &) = default;
 
   bool operator==(const ShaderCompileFlag &o) const { return name == o.name && value == o.value; }
   bool operator<(const ShaderCompileFlag &o) const
@@ -917,6 +1017,7 @@ struct ShaderCompileFlags
   DOCUMENT("");
   ShaderCompileFlags() = default;
   ShaderCompileFlags(const ShaderCompileFlags &) = default;
+  ShaderCompileFlags &operator=(const ShaderCompileFlags &) = default;
 
   DOCUMENT(R"(A list of :class:`ShaderCompileFlag`.
 
@@ -933,6 +1034,7 @@ struct ShaderSourceFile
   DOCUMENT("");
   ShaderSourceFile() = default;
   ShaderSourceFile(const ShaderSourceFile &) = default;
+  ShaderSourceFile &operator=(const ShaderSourceFile &) = default;
 
   bool operator==(const ShaderSourceFile &o) const
   {
@@ -964,6 +1066,7 @@ struct ShaderDebugInfo
 {
   ShaderDebugInfo() {}
   ShaderDebugInfo(const ShaderDebugInfo &) = default;
+  ShaderDebugInfo &operator=(const ShaderDebugInfo &) = default;
 
   DOCUMENT("A :class:`ShaderCompileFlags` containing the flags used to compile this shader.");
   ShaderCompileFlags compileFlags;
@@ -991,6 +1094,7 @@ struct ShaderReflection
   DOCUMENT("");
   ShaderReflection() = default;
   ShaderReflection(const ShaderReflection &) = default;
+  ShaderReflection &operator=(const ShaderReflection &) = default;
 
   DOCUMENT("The :class:`ResourceId` of this shader.");
   ResourceId resourceId;
@@ -1007,7 +1111,7 @@ struct ShaderReflection
   ShaderDebugInfo debugInfo;
 
   DOCUMENT("The :class:`ShaderEncoding` of this shader. See :data:`rawBytes`.");
-  ShaderEncoding encoding;
+  ShaderEncoding encoding = ShaderEncoding::Unknown;
 
   DOCUMENT(R"(A raw ``bytes`` dump of the original shader, encoded in the form denoted by
 :data:`encoding`.
@@ -1036,6 +1140,9 @@ struct ShaderReflection
   // TODO expand this to encompass shader subroutines.
   DOCUMENT("A list of strings with the shader's interfaces. Largely an unused API feature.");
   rdcarray<rdcstr> interfaces;
+
+  DOCUMENT("A list of :class:`ShaderVariableType` with the shader's pointer types.");
+  rdcarray<ShaderVariableType> pointerTypes;
 };
 
 DECLARE_REFLECTION_STRUCT(ShaderReflection);
@@ -1055,6 +1162,7 @@ struct Bindpoint
     arraySize = 1;
   }
   Bindpoint(const Bindpoint &) = default;
+  Bindpoint &operator=(const Bindpoint &) = default;
 
   Bindpoint(int32_t s, int32_t b)
   {
@@ -1127,6 +1235,7 @@ struct ShaderBindpointMapping
   DOCUMENT("");
   ShaderBindpointMapping() = default;
   ShaderBindpointMapping(const ShaderBindpointMapping &) = default;
+  ShaderBindpointMapping &operator=(const ShaderBindpointMapping &) = default;
 
   DOCUMENT(R"(This maps input attributes as a simple swizzle on the
 :data:`ShaderReflection.inputSignature` indices for APIs where this mapping is mutable at runtime.
