@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,10 @@
 #pragma once
 
 #include <stdint.h>
+#include "apidefs.h"
+#include "rdcarray.h"
+#include "rdcstr.h"
+#include "resourceid.h"
 #include "stringise.h"
 
 DOCUMENT(R"(The basic irreducible type of an object. Every other more complex type is built on these.
@@ -161,7 +165,7 @@ struct SDChunk;
 DOCUMENT("Details the name and properties of a structured type");
 struct SDType
 {
-  SDType(const char *n)
+  SDType(const rdcstr &n)
       : name(n), basetype(SDBasic::Struct), flags(SDTypeFlags::NoFlags), byteSize(0)
   {
   }
@@ -192,6 +196,10 @@ protected:
   SDType() = default;
   SDType(const SDType &) = default;
   SDType &operator=(const SDType &) = default;
+  void *operator new(size_t count) = delete;
+  void *operator new[](size_t count) = delete;
+  void operator delete(void *p) = delete;
+  void operator delete[](void *p) = delete;
 };
 
 DECLARE_REFLECTION_STRUCT(SDType);
@@ -228,6 +236,7 @@ struct SDChunkMetaData
   DOCUMENT("");
   SDChunkMetaData() = default;
   SDChunkMetaData(const SDChunkMetaData &) = default;
+  SDChunkMetaData &operator=(const SDChunkMetaData &) = default;
 
   DOCUMENT("The internal chunk ID - unique given a particular driver in use.");
   uint32_t chunkID = 0;
@@ -238,7 +247,7 @@ struct SDChunkMetaData
   DOCUMENT(R"(The length in bytes of this chunk - may be longer than the actual sum of the data if a
 conservative size estimate was used on creation to avoid seeking to fix-up the stored length.
 )");
-  uint32_t length = 0;
+  uint64_t length = 0;
 
   DOCUMENT("The ID of the thread where this chunk was recorded.");
   uint64_t threadID = 0;
@@ -254,6 +263,12 @@ Since 0 is a possible value for this (for extremely fast calls), -1 is the inval
 
   DOCUMENT("The frames of the CPU-side callstack leading up to the chunk.");
   rdcarray<uint64_t> callstack;
+
+private:
+  void *operator new(size_t count) = delete;
+  void *operator new[](size_t count) = delete;
+  void operator delete(void *p) = delete;
+  void operator delete[](void *p) = delete;
 };
 
 DECLARE_REFLECTION_STRUCT(SDChunkMetaData);
@@ -287,6 +302,11 @@ union SDObjectPODData
   uint64_t numChildren;
 
   SDObjectPODData() : u(0) {}
+private:
+  void *operator new(size_t count) = delete;
+  void *operator new[](size_t count) = delete;
+  void operator delete(void *p) = delete;
+  void operator delete[](void *p) = delete;
 };
 
 DECLARE_REFLECTION_STRUCT(SDObjectPODData);
@@ -311,11 +331,19 @@ struct StructuredObjectList : public rdcarray<SDObject *>
 #else
   StructuredObjectList &operator=(const StructuredObjectList &other) = delete;
 #endif
+  // allow placement new for swig
+  void *operator new(size_t, void *ptr) { return ptr; }
+  void operator delete(void *p, void *) {}
+  void operator delete(void *p) {}
+private:
+  void *operator new(size_t count) = delete;
+  void *operator new[](size_t count) = delete;
+  void operator delete[](void *p) = delete;
 };
 
 DECLARE_REFLECTION_STRUCT(StructuredObjectList);
 
-DOCUMENT("The data inside an class:`SDObject`, whether it's plain old data or complex children.");
+DOCUMENT("The data inside an :class:`SDObject` whether it's plain old data or complex children.");
 struct SDObjectData
 {
   DOCUMENT("");
@@ -327,11 +355,17 @@ struct SDObjectData
   DOCUMENT("The string contents of the object.");
   rdcstr str;
 
-  DOCUMENT("A ``list`` of class:`SDObject` containing the children of this class:`SDObject`.");
+  DOCUMENT("A list of :class:`SDObject` containing the children of this :class:`SDObject`.");
   StructuredObjectList children;
 
   SDObjectData(const SDObjectData &) = delete;
   SDObjectData &operator=(const SDObjectData &other) = delete;
+
+private:
+  void *operator new(size_t count) = delete;
+  void *operator new[](size_t count) = delete;
+  void operator delete(void *p) = delete;
+  void operator delete[](void *p) = delete;
 };
 
 DECLARE_REFLECTION_STRUCT(SDObjectData);
@@ -339,7 +373,32 @@ DECLARE_REFLECTION_STRUCT(SDObjectData);
 DOCUMENT("Defines a single structured object.");
 struct SDObject
 {
-  SDObject(const char *n, const char *t) : type(t)
+  /////////////////////////////////////////////////////////////////
+  // memory management, in a dll safe way
+  void *operator new(size_t sz)
+  {
+    void *ret = NULL;
+#ifdef RENDERDOC_EXPORTS
+    ret = malloc(sz);
+    if(ret == NULL)
+      RENDERDOC_OutOfMemory(sz);
+#else
+    ret = RENDERDOC_AllocArrayMem(sz);
+#endif
+    return ret;
+  }
+  void operator delete(void *p)
+  {
+#ifdef RENDERDOC_EXPORTS
+    free(p);
+#else
+    RENDERDOC_FreeArrayMem(p);
+#endif
+  }
+  void *operator new[](size_t count) = delete;
+  void operator delete[](void *p) = delete;
+
+  SDObject(const rdcstr &n, const rdcstr &t) : type(t)
   {
     name = n;
     data.basic.u = 0;
@@ -420,7 +479,7 @@ struct SDObject
   inline double AsDouble() const { return data.basic.d; }
   inline float AsFloat() const { return (float)data.basic.d; }
   inline char AsChar() const { return data.basic.c; }
-  inline std::string AsString() const { return data.str; }
+  inline rdcstr AsString() const { return data.str; }
   inline uint64_t AsUInt64() const { return (uint64_t)data.basic.u; }
   inline int64_t AsInt64() const { return (int64_t)data.basic.i; }
   inline uint32_t AsUInt32() const { return (uint32_t)data.basic.u; }
@@ -553,7 +612,7 @@ DECLARE_REFLECTION_STRUCT(SDObject);
 #if defined(RENDERDOC_QT_COMPAT)
 inline SDObject *makeSDObject(const char *name, QVariant val)
 {
-  SDObject *ret = new SDObject(name, "QVariant");
+  SDObject *ret = new SDObject(name, "QVariant"_lit);
   ret->type.basetype = SDBasic::Null;
 
   // coverity[mixed_enums]
@@ -562,57 +621,57 @@ inline SDObject *makeSDObject(const char *name, QVariant val)
   switch(type)
   {
     case QMetaType::Bool:
-      ret->type.name = "bool";
+      ret->type.name = "bool"_lit;
       ret->type.basetype = SDBasic::Boolean;
       ret->type.byteSize = 1;
       ret->data.basic.b = val.toBool();
       break;
     case QMetaType::Short:
-      ret->type.name = "int16_t";
+      ret->type.name = "int16_t"_lit;
       ret->type.basetype = SDBasic::SignedInteger;
       ret->type.byteSize = 2;
       ret->data.basic.i = val.toInt();
       break;
     case QMetaType::UShort:
-      ret->type.name = "uint16_t";
+      ret->type.name = "uint16_t"_lit;
       ret->type.basetype = SDBasic::UnsignedInteger;
       ret->type.byteSize = 2;
       ret->data.basic.u = val.toUInt();
       break;
     case QMetaType::Long:
     case QMetaType::Int:
-      ret->type.name = "int32_t";
+      ret->type.name = "int32_t"_lit;
       ret->type.basetype = SDBasic::SignedInteger;
       ret->type.byteSize = 4;
       ret->data.basic.i = val.toInt();
       break;
     case QMetaType::ULong:
     case QMetaType::UInt:
-      ret->type.name = "uint32_t";
+      ret->type.name = "uint32_t"_lit;
       ret->type.basetype = SDBasic::UnsignedInteger;
       ret->type.byteSize = 4;
       ret->data.basic.u = val.toUInt();
       break;
     case QMetaType::LongLong:
-      ret->type.name = "int64_t";
+      ret->type.name = "int64_t"_lit;
       ret->type.basetype = SDBasic::SignedInteger;
       ret->type.byteSize = 8;
       ret->data.basic.i = val.toLongLong();
       break;
     case QMetaType::ULongLong:
-      ret->type.name = "uint64_t";
+      ret->type.name = "uint64_t"_lit;
       ret->type.basetype = SDBasic::UnsignedInteger;
       ret->type.byteSize = 8;
       ret->data.basic.u = val.toULongLong();
       break;
     case QMetaType::Float:
-      ret->type.name = "float";
+      ret->type.name = "float"_lit;
       ret->type.basetype = SDBasic::Float;
       ret->type.byteSize = 4;
       ret->data.basic.d = val.toFloat();
       break;
     case QMetaType::Double:
-      ret->type.name = "double";
+      ret->type.name = "double"_lit;
       ret->type.basetype = SDBasic::Float;
       ret->type.byteSize = 8;
       ret->data.basic.d = val.toDouble();
@@ -620,17 +679,39 @@ inline SDObject *makeSDObject(const char *name, QVariant val)
     case QMetaType::UChar:
     case QMetaType::Char:
     case QMetaType::QChar:
-      ret->type.name = "char";
+      ret->type.name = "char"_lit;
       ret->type.basetype = SDBasic::Character;
       ret->type.byteSize = 1;
       ret->data.basic.c = val.toChar().toLatin1();
       break;
     case QMetaType::QString:
-      ret->type.name = "string";
+      ret->type.name = "string"_lit;
       ret->type.basetype = SDBasic::String;
       ret->data.str = val.toString().toUtf8().data();
       ret->type.byteSize = ret->data.str.size();
       break;
+    case QMetaType::QVariantList:
+    {
+      QVariantList list = val.toList();
+      ret->type.name = "array"_lit;
+      ret->type.basetype = SDBasic::Array;
+      ret->data.children.reserve(list.size());
+      for(int i = 0; i < list.size(); i++)
+        ret->data.children.push_back(makeSDObject("[]", list.at(i)));
+      ret->type.byteSize = list.size();
+      break;
+    }
+    case QMetaType::QVariantMap:
+    {
+      QVariantMap map = val.toMap();
+      ret->type.name = "struct"_lit;
+      ret->type.basetype = SDBasic::Struct;
+      ret->data.children.reserve(map.size());
+      for(const QString &str : map.keys())
+        ret->data.children.push_back(makeSDObject(str.toUtf8().data(), map[str]));
+      ret->type.byteSize = map.size();
+      break;
+    }
     default: break;
   }
 
@@ -641,7 +722,7 @@ inline SDObject *makeSDObject(const char *name, QVariant val)
 DOCUMENT("Make a structured object out of a signed integer");
 inline SDObject *makeSDInt64(const char *name, int64_t val)
 {
-  SDObject *ret = new SDObject(name, "int64_t");
+  SDObject *ret = new SDObject(name, "int64_t"_lit);
   ret->type.basetype = SDBasic::SignedInteger;
   ret->type.byteSize = 8;
   ret->data.basic.i = val;
@@ -651,7 +732,7 @@ inline SDObject *makeSDInt64(const char *name, int64_t val)
 DOCUMENT("Make a structured object out of an unsigned integer");
 inline SDObject *makeSDUInt64(const char *name, uint64_t val)
 {
-  SDObject *ret = new SDObject(name, "uint64_t");
+  SDObject *ret = new SDObject(name, "uint64_t"_lit);
   ret->type.basetype = SDBasic::UnsignedInteger;
   ret->type.byteSize = 8;
   ret->data.basic.u = val;
@@ -661,17 +742,17 @@ inline SDObject *makeSDUInt64(const char *name, uint64_t val)
 DOCUMENT("Make a structured object out of a integer, stored as signed 32-bits");
 inline SDObject *makeSDInt32(const char *name, int32_t val)
 {
-  SDObject *ret = new SDObject(name, "int32_t");
+  SDObject *ret = new SDObject(name, "int32_t"_lit);
   ret->type.basetype = SDBasic::SignedInteger;
   ret->type.byteSize = 4;
-  ret->data.basic.u = val;
+  ret->data.basic.i = val;
   return ret;
 }
 
 DOCUMENT("Make a structured object out of a integer, stored as unsigned 32-bits");
 inline SDObject *makeSDUInt32(const char *name, uint32_t val)
 {
-  SDObject *ret = new SDObject(name, "uint32_t");
+  SDObject *ret = new SDObject(name, "uint32_t"_lit);
   ret->type.basetype = SDBasic::UnsignedInteger;
   ret->type.byteSize = 4;
   ret->data.basic.u = val;
@@ -681,7 +762,7 @@ inline SDObject *makeSDUInt32(const char *name, uint32_t val)
 DOCUMENT("Make a structured object out of a floating point value");
 inline SDObject *makeSDFloat(const char *name, float val)
 {
-  SDObject *ret = new SDObject(name, "float");
+  SDObject *ret = new SDObject(name, "float"_lit);
   ret->type.basetype = SDBasic::Float;
   ret->type.byteSize = 4;
   ret->data.basic.d = val;
@@ -691,7 +772,7 @@ inline SDObject *makeSDFloat(const char *name, float val)
 DOCUMENT("Make a structured object out of a boolean value");
 inline SDObject *makeSDBool(const char *name, bool val)
 {
-  SDObject *ret = new SDObject(name, "bool");
+  SDObject *ret = new SDObject(name, "bool"_lit);
   ret->type.basetype = SDBasic::Boolean;
   ret->type.byteSize = 1;
   ret->data.basic.b = val;
@@ -701,7 +782,7 @@ inline SDObject *makeSDBool(const char *name, bool val)
 DOCUMENT("Make a structured object out of a string");
 inline SDObject *makeSDString(const char *name, const char *val)
 {
-  SDObject *ret = new SDObject(name, "string");
+  SDObject *ret = new SDObject(name, "string"_lit);
   ret->type.basetype = SDBasic::String;
   ret->type.byteSize = strlen(val);
   ret->data.str = val;
@@ -711,7 +792,7 @@ inline SDObject *makeSDString(const char *name, const char *val)
 DOCUMENT("Make a structured object out of a ResourceId");
 inline SDObject *makeSDResourceId(const char *name, ResourceId val)
 {
-  SDObject *ret = new SDObject(name, "ResourceId");
+  SDObject *ret = new SDObject(name, "ResourceId"_lit);
   ret->type.basetype = SDBasic::Resource;
   ret->type.byteSize = 8;
   ret->data.basic.id = val;
@@ -721,7 +802,7 @@ inline SDObject *makeSDResourceId(const char *name, ResourceId val)
 DOCUMENT("Make a structured object out of an enumeration value");
 inline SDObject *makeSDEnum(const char *name, uint32_t val)
 {
-  SDObject *ret = new SDObject(name, "enum");
+  SDObject *ret = new SDObject(name, "enum"_lit);
   ret->type.basetype = SDBasic::Enum;
   ret->type.byteSize = 4;
   ret->data.basic.u = val;
@@ -731,7 +812,7 @@ inline SDObject *makeSDEnum(const char *name, uint32_t val)
 DOCUMENT("Make an array-type structured object");
 inline SDObject *makeSDArray(const char *name)
 {
-  SDObject *ret = new SDObject(name, "array");
+  SDObject *ret = new SDObject(name, "array"_lit);
   ret->type.basetype = SDBasic::Array;
   return ret;
 }
@@ -776,7 +857,32 @@ SDOBJECT_MAKER(ResourceId, makeSDResourceId);
 DOCUMENT("Defines a single structured chunk, which is a :class:`SDObject`.");
 struct SDChunk : public SDObject
 {
-  SDChunk(const char *name) : SDObject(name, "Chunk") { type.basetype = SDBasic::Chunk; }
+  /////////////////////////////////////////////////////////////////
+  // memory management, in a dll safe way
+  void *operator new(size_t sz)
+  {
+    void *ret = NULL;
+#ifdef RENDERDOC_EXPORTS
+    ret = malloc(sz);
+    if(ret == NULL)
+      RENDERDOC_OutOfMemory(sz);
+#else
+    ret = RENDERDOC_AllocArrayMem(sz);
+#endif
+    return ret;
+  }
+  void operator delete(void *p)
+  {
+#ifdef RENDERDOC_EXPORTS
+    free(p);
+#else
+    RENDERDOC_FreeArrayMem(p);
+#endif
+  }
+  void *operator new[](size_t count) = delete;
+  void operator delete[](void *p) = delete;
+
+  SDChunk(const char *name) : SDObject(name, "Chunk"_lit) { type.basetype = SDBasic::Chunk; }
   DOCUMENT("The :class:`SDChunkMetaData` with the metadata for this chunk.");
   SDChunkMetaData metadata;
 
@@ -827,6 +933,15 @@ struct StructuredChunkList : public rdcarray<SDChunk *>
 #else
   StructuredChunkList &operator=(const StructuredChunkList &other) = delete;
 #endif
+
+  // allow placement new for swig
+  void *operator new(size_t, void *ptr) { return ptr; }
+  void operator delete(void *p, void *) {}
+  void operator delete(void *p) {}
+private:
+  void *operator new(size_t count) = delete;
+  void *operator new[](size_t count) = delete;
+  void operator delete[](void *p) = delete;
 };
 
 DECLARE_REFLECTION_STRUCT(StructuredChunkList);
@@ -855,6 +970,15 @@ struct StructuredBufferList : public rdcarray<bytebuf *>
 #else
   StructuredBufferList &operator=(const StructuredBufferList &other) = delete;
 #endif
+
+  // allow placement new for swig
+  void *operator new(size_t, void *ptr) { return ptr; }
+  void operator delete(void *p, void *) {}
+  void operator delete(void *p) {}
+private:
+  void *operator new(size_t count) = delete;
+  void *operator new[](size_t count) = delete;
+  void operator delete[](void *p) = delete;
 };
 
 DECLARE_REFLECTION_STRUCT(StructuredBufferList);
@@ -862,6 +986,37 @@ DECLARE_REFLECTION_STRUCT(StructuredBufferList);
 DOCUMENT("Contains the structured information in a file. Owns the buffers and chunks.");
 struct SDFile
 {
+private:
+  /////////////////////////////////////////////////////////////////
+  // memory management, in a dll safe way
+  static void *allocate(size_t count)
+  {
+    const size_t sz = count * sizeof(SDFile);
+    void *ret = NULL;
+#ifdef RENDERDOC_EXPORTS
+    ret = malloc(sz);
+    if(ret == NULL)
+      RENDERDOC_OutOfMemory(sz);
+#else
+    ret = RENDERDOC_AllocArrayMem(sz);
+#endif
+    return ret;
+  }
+  static void deallocate(void *p)
+  {
+#ifdef RENDERDOC_EXPORTS
+    free(p);
+#else
+    RENDERDOC_FreeArrayMem(p);
+#endif
+  }
+
+  void *operator new[](size_t count) = delete;
+  void operator delete[](void *p) = delete;
+
+public:
+  void *operator new(size_t count) { return allocate(count); }
+  void operator delete(void *p) { return deallocate(p); };
   SDFile() {}
   ~SDFile()
   {
