@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2014-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -41,12 +41,14 @@
 //
 
 // To prevent a project rebuild cascading when the git commit changes, we declare a char array
-// that's implemented in version.inl, which can be included in each module that uses the version.
+// that's implemented in version.cpp, which is linked into the core module.
 // It's 41 characters to allow 40 characters of commit hash plus trailing NULL.
-// Then the .cpp that includes version.inl is the only one that actually needs to have the hash
-// defined properly.
-// This replaces the previous GIT_COMMIT_HASH define here.
+// Then version.cpp is the only thing that needs to be rebuilt when the git commit changes
+//
+// Only available internally, external users should use RENDERDOC_GetCommitHash()
+#if defined(RENDERDOC_EXPORTS)
 extern "C" const char GitVersionHash[41];
+#endif
 
 // If this variable is set to 1, then this build is considered a stable version - based on a tagged
 // version number upstream, possibly with some patches applied as necessary.
@@ -92,7 +94,7 @@ extern "C" const char GitVersionHash[41];
 // upstream and should not be modified downstream. You can set DISTRIBUTION_VERSION to include any
 // arbitrary release marker or package version you wish.
 #define RENDERDOC_VERSION_MAJOR 1
-#define RENDERDOC_VERSION_MINOR 4
+#define RENDERDOC_VERSION_MINOR 6
 
 #define RDOC_INTERNAL_VERSION_STRINGIZE2(a) #a
 #define RDOC_INTERNAL_VERSION_STRINGIZE(a) RDOC_INTERNAL_VERSION_STRINGIZE2(a)
