@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -26,63 +26,17 @@
 #include "common.h"
 #include <stdarg.h>
 #include <string.h>
-#include <string>
 #include "common/threading.h"
 #include "os/os_specific.h"
 #include "strings/string_utils.h"
 
-using std::string;
-
-//	for(int i=0; i < 256; i++)
-//	{
-//		uint8_t comp = i&0xff;
-//		float srgbF = float(comp)/255.0f;
-//
-//		if(srgbF <= 0.04045f)
-//		  SRGB8_lookuptable[comp] = srgbF/12.92f;
-//		else
-//		  SRGB8_lookuptable[comp] = powf((0.055f + srgbF) / 1.055f, 2.4f);
-//	}
-
-float SRGB8_lookuptable[256] = {
-    0.000000f, 0.000304f, 0.000607f, 0.000911f, 0.001214f, 0.001518f, 0.001821f, 0.002125f,
-    0.002428f, 0.002732f, 0.003035f, 0.003347f, 0.003677f, 0.004025f, 0.004391f, 0.004777f,
-    0.005182f, 0.005605f, 0.006049f, 0.006512f, 0.006995f, 0.007499f, 0.008023f, 0.008568f,
-    0.009134f, 0.009721f, 0.010330f, 0.010960f, 0.011612f, 0.012286f, 0.012983f, 0.013702f,
-    0.014444f, 0.015209f, 0.015996f, 0.016807f, 0.017642f, 0.018500f, 0.019382f, 0.020289f,
-    0.021219f, 0.022174f, 0.023153f, 0.024158f, 0.025187f, 0.026241f, 0.027321f, 0.028426f,
-    0.029557f, 0.030713f, 0.031896f, 0.033105f, 0.034340f, 0.035601f, 0.036889f, 0.038204f,
-    0.039546f, 0.040915f, 0.042311f, 0.043735f, 0.045186f, 0.046665f, 0.048172f, 0.049707f,
-    0.051269f, 0.052861f, 0.054480f, 0.056128f, 0.057805f, 0.059511f, 0.061246f, 0.063010f,
-    0.064803f, 0.066626f, 0.068478f, 0.070360f, 0.072272f, 0.074214f, 0.076185f, 0.078187f,
-    0.080220f, 0.082283f, 0.084376f, 0.086500f, 0.088656f, 0.090842f, 0.093059f, 0.095307f,
-    0.097587f, 0.099899f, 0.102242f, 0.104616f, 0.107023f, 0.109462f, 0.111932f, 0.114435f,
-    0.116971f, 0.119538f, 0.122139f, 0.124772f, 0.127438f, 0.130136f, 0.132868f, 0.135633f,
-    0.138432f, 0.141263f, 0.144128f, 0.147027f, 0.149960f, 0.152926f, 0.155926f, 0.158961f,
-    0.162029f, 0.165132f, 0.168269f, 0.171441f, 0.174647f, 0.177888f, 0.181164f, 0.184475f,
-    0.187821f, 0.191202f, 0.194618f, 0.198069f, 0.201556f, 0.205079f, 0.208637f, 0.212231f,
-    0.215861f, 0.219526f, 0.223228f, 0.226966f, 0.230740f, 0.234551f, 0.238398f, 0.242281f,
-    0.246201f, 0.250158f, 0.254152f, 0.258183f, 0.262251f, 0.266356f, 0.270498f, 0.274677f,
-    0.278894f, 0.283149f, 0.287441f, 0.291771f, 0.296138f, 0.300544f, 0.304987f, 0.309469f,
-    0.313989f, 0.318547f, 0.323143f, 0.327778f, 0.332452f, 0.337164f, 0.341914f, 0.346704f,
-    0.351533f, 0.356400f, 0.361307f, 0.366253f, 0.371238f, 0.376262f, 0.381326f, 0.386430f,
-    0.391573f, 0.396755f, 0.401978f, 0.407240f, 0.412543f, 0.417885f, 0.423268f, 0.428691f,
-    0.434154f, 0.439657f, 0.445201f, 0.450786f, 0.456411f, 0.462077f, 0.467784f, 0.473532f,
-    0.479320f, 0.485150f, 0.491021f, 0.496933f, 0.502887f, 0.508881f, 0.514918f, 0.520996f,
-    0.527115f, 0.533276f, 0.539480f, 0.545725f, 0.552011f, 0.558340f, 0.564712f, 0.571125f,
-    0.577581f, 0.584078f, 0.590619f, 0.597202f, 0.603827f, 0.610496f, 0.617207f, 0.623960f,
-    0.630757f, 0.637597f, 0.644480f, 0.651406f, 0.658375f, 0.665387f, 0.672443f, 0.679543f,
-    0.686685f, 0.693872f, 0.701102f, 0.708376f, 0.715694f, 0.723055f, 0.730461f, 0.737911f,
-    0.745404f, 0.752942f, 0.760525f, 0.768151f, 0.775822f, 0.783538f, 0.791298f, 0.799103f,
-    0.806952f, 0.814847f, 0.822786f, 0.830770f, 0.838799f, 0.846873f, 0.854993f, 0.863157f,
-    0.871367f, 0.879622f, 0.887923f, 0.896269f, 0.904661f, 0.913099f, 0.921582f, 0.930111f,
-    0.938686f, 0.947307f, 0.955974f, 0.964686f, 0.973445f, 0.982251f, 0.991102f, 1.000000f,
-};
+int utf8printv(char *buf, size_t bufsize, const char *fmt, va_list args);
+int utf8printf(char *str, size_t bufSize, const char *fmt, ...);
 
 void rdcassert(const char *msg, const char *file, unsigned int line, const char *func)
 {
-  rdclog_direct(Timing::GetUTCTime(), Process::GetCurrentPID(), LogType::Error, RDCLOG_PROJECT,
-                file, line, "Assertion failed: %s", msg);
+  rdclog_direct(FILL_AUTO_VALUE, FILL_AUTO_VALUE, LogType::Error, RDCLOG_PROJECT, file, line,
+                "Assertion failed: %s", msg);
 }
 
 #if 0
@@ -294,8 +248,8 @@ uint64_t Log2Floor(uint64_t value)
 }
 #endif
 
-static string logfile;
-static bool logfileOpened = false;
+static rdcstr logfile;
+static FileIO::LogFileHandle *logfileHandle = NULL;
 
 const char *rdclog_getfilename()
 {
@@ -304,27 +258,27 @@ const char *rdclog_getfilename()
 
 void rdclog_filename(const char *filename)
 {
-  string previous = logfile;
+  rdcstr previous = logfile;
 
   logfile = "";
   if(filename && filename[0])
     logfile = filename;
 
-  FileIO::logfile_close(NULL);
+  FileIO::logfile_close(logfileHandle, NULL);
 
-  logfileOpened = false;
+  logfileHandle = NULL;
 
   if(!logfile.empty())
   {
-    logfileOpened = FileIO::logfile_open(logfile.c_str());
+    logfileHandle = FileIO::logfile_open(logfile.c_str());
 
-    if(logfileOpened && previous.c_str())
+    if(logfileHandle && previous.c_str())
     {
-      vector<unsigned char> previousContents;
-      FileIO::slurp(previous.c_str(), previousContents);
+      rdcstr previousContents;
+      FileIO::ReadAll(previous.c_str(), previousContents);
 
       if(!previousContents.empty())
-        FileIO::logfile_append((const char *)&previousContents[0], previousContents.size());
+        FileIO::logfile_append(logfileHandle, previousContents.c_str(), previousContents.size());
 
       FileIO::Delete(previous.c_str());
     }
@@ -341,7 +295,7 @@ void rdclog_enableoutput()
 void rdclog_closelog(const char *filename)
 {
   log_output_enabled = false;
-  FileIO::logfile_close(filename);
+  FileIO::logfile_close(logfileHandle, filename);
 }
 
 void rdclog_flush()
@@ -350,9 +304,9 @@ void rdclog_flush()
 
 void rdclogprint_int(LogType type, const char *fullMsg, const char *msg)
 {
-  static Threading::CriticalSection lock;
+  static Threading::CriticalSection *lock = new Threading::CriticalSection();
 
-  SCOPED_LOCK(lock);
+  SCOPED_LOCK(*lock);
 
 #if ENABLED(OUTPUT_LOG_TO_DEBUG_OUT)
   OSUtility::WriteOutput(OSUtility::Output_DebugMon, fullMsg);
@@ -368,10 +322,10 @@ void rdclogprint_int(LogType type, const char *fullMsg, const char *msg)
     OSUtility::WriteOutput(OSUtility::Output_StdErr, msg);
 #endif
 #if ENABLED(OUTPUT_LOG_TO_DISK)
-  if(logfileOpened)
+  if(logfileHandle)
   {
     // strlen used as byte length - str is UTF-8 so this is NOT number of characters
-    FileIO::logfile_append(fullMsg, strlen(fullMsg));
+    FileIO::logfile_append(logfileHandle, fullMsg, strlen(fullMsg));
   }
 #endif
 }
@@ -391,6 +345,14 @@ static void write_newline(char *output)
 void rdclog_direct(time_t utcTime, uint32_t pid, LogType type, const char *project,
                    const char *file, unsigned int line, const char *fmt, ...)
 {
+  if(utcTime == FILL_AUTO_VALUE)
+    utcTime = Timing::GetUTCTime();
+
+  static uint32_t curpid = Process::GetCurrentPID();
+
+  if(pid == FILL_AUTO_VALUE)
+    pid = curpid;
+
   va_list args;
   va_start(args, fmt);
 
@@ -398,25 +360,25 @@ void rdclog_direct(time_t utcTime, uint32_t pid, LogType type, const char *proje
   va_list args2;
   va_copy(args2, args);
 
-  char timestamp[64] = {0};
+  rdcstr timestamp;
 #if ENABLED(INCLUDE_TIMESTAMP_IN_LOG)
-  StringFormat::sntimef(utcTime, timestamp, 63, "[%H:%M:%S] ");
+  timestamp = StringFormat::sntimef(utcTime, "[%H:%M:%S] ");
 #endif
 
   char location[64] = {0};
 #if ENABLED(INCLUDE_LOCATION_IN_LOG)
-  string loc;
-  loc = get_basename(string(file));
-  StringFormat::snprintf(location, 63, "% 20s(%4d) - ", loc.c_str(), line);
+  rdcstr loc;
+  loc = get_basename(file);
+  utf8printf(location, 63, "% 20s(%4d) - ", loc.c_str(), line);
 #endif
 
   const char *typestr[(uint32_t)LogType::Count] = {
       "Debug  ", "Log    ", "Warning", "Error  ", "Fatal  ",
   };
 
-  static Threading::CriticalSection lock;
+  static Threading::CriticalSection *lock = new Threading::CriticalSection();
 
-  SCOPED_LOCK(lock);
+  SCOPED_LOCK(*lock);
 
   rdclog_outputBuffer[rdclog_outBufSize] = rdclog_outputBuffer[0] = 0;
 
@@ -425,8 +387,8 @@ void rdclog_direct(time_t utcTime, uint32_t pid, LogType type, const char *proje
 
   char *base = output;
 
-  int numWritten = StringFormat::snprintf(output, available, "% 4s %06u: %s%s%s - ", project, pid,
-                                          timestamp, location, typestr[(uint32_t)type]);
+  int numWritten = utf8printf(output, available, "% 4s %06u: %s%s%s - ", project, pid,
+                              timestamp.c_str(), location, typestr[(uint32_t)type]);
 
   if(numWritten < 0)
   {
@@ -444,7 +406,7 @@ void rdclog_direct(time_t utcTime, uint32_t pid, LogType type, const char *proje
 
   int totalWritten = numWritten;
 
-  numWritten = StringFormat::vsnprintf(output, available, fmt, args);
+  numWritten = utf8printv(output, available, fmt, args);
 
   totalWritten += numWritten;
 
@@ -467,9 +429,9 @@ void rdclog_direct(time_t utcTime, uint32_t pid, LogType type, const char *proje
     oversizedBuffer = output = new char[available + 3];
     base = output;
 
-    numWritten = StringFormat::snprintf(output, available, "% 4s %06u: %s%s%s - ", project,
-                                        Process::GetCurrentPID(), timestamp, location,
-                                        typestr[(uint32_t)type]);
+    numWritten =
+        utf8printf(output, available, "% 4s %06u: %s%s%s - ", project, Process::GetCurrentPID(),
+                   timestamp.c_str(), location, typestr[(uint32_t)type]);
 
     output += numWritten;
     available -= numWritten;
@@ -478,7 +440,7 @@ void rdclog_direct(time_t utcTime, uint32_t pid, LogType type, const char *proje
 
     noPrefixOutput = (output - 3 - (sizeof(typestr[(uint32_t)type]) - 1));
 
-    numWritten = StringFormat::vsnprintf(output, available, fmt, args2);
+    numWritten = utf8printv(output, available, fmt, args2);
 
     output += numWritten;
 
@@ -498,7 +460,7 @@ void rdclog_direct(time_t utcTime, uint32_t pid, LogType type, const char *proje
   {
     char backup[2];
 
-    std::string prefixText(base, size_t(prefixEnd - base));
+    rdcstr prefixText(base, size_t(prefixEnd - base));
 
     bool first = true;
 
