@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -105,25 +105,25 @@
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-#define RDCASSERTMSG(msg, ...)                                                            \
-  do                                                                                      \
-  {                                                                                       \
-    if(!(RDCASSERT_IFCOND(__VA_ARGS__)))                                                  \
-    {                                                                                     \
-      const char custommsg[] = msg;                                                       \
-      (void)custommsg;                                                                    \
-      std::string assertmsg = "'" STRINGIZE(RDCASSERT_GETCOND(__VA_ARGS__)) "' ";         \
-      assertmsg += (sizeof(custommsg) > 1) ? msg " " : "";                                \
-      std::string failmsg;                                                                \
-      RDCASSERT_FAILMSG(__VA_ARGS__);                                                     \
-      if(!failmsg.empty())                                                                \
-      {                                                                                   \
-        failmsg.pop_back();                                                               \
-        failmsg.pop_back();                                                               \
-      }                                                                                   \
-      std::string combinedmsg = assertmsg + (failmsg.empty() ? "" : "(" + failmsg + ")"); \
-      rdcassert(combinedmsg.c_str(), __FILE__, __LINE__, __PRETTY_FUNCTION_SIGNATURE__);  \
-      rdclog_flush();                                                                     \
-      RDCBREAK();                                                                         \
-    }                                                                                     \
+#define RDCASSERTMSG(msg, ...)                                                           \
+  do                                                                                     \
+  {                                                                                      \
+    if(!(RDCASSERT_IFCOND(__VA_ARGS__)))                                                 \
+    {                                                                                    \
+      const char custommsg[] = msg;                                                      \
+      (void)custommsg;                                                                   \
+      rdcstr assertmsg = "'" STRINGIZE(RDCASSERT_GETCOND(__VA_ARGS__)) "' ";             \
+      assertmsg += (sizeof(custommsg) > 1) ? msg " " : "";                               \
+      rdcstr failmsg;                                                                    \
+      RDCASSERT_FAILMSG(__VA_ARGS__);                                                    \
+      if(!failmsg.empty())                                                               \
+      {                                                                                  \
+        failmsg.pop_back();                                                              \
+        failmsg.pop_back();                                                              \
+      }                                                                                  \
+      rdcstr combinedmsg = assertmsg + (failmsg.empty() ? "" : "(" + failmsg + ")");     \
+      rdcassert(combinedmsg.c_str(), __FILE__, __LINE__, __PRETTY_FUNCTION_SIGNATURE__); \
+      rdclog_flush();                                                                    \
+      RDCBREAK();                                                                        \
+    }                                                                                    \
   } while((void)0, 0)
