@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2014-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,7 @@
 #include "dds_readwrite.h"
 #include <stdint.h>
 #include "common/common.h"
+#include "os/os_specific.h"
 
 static const uint32_t dds_fourcc = MAKE_FOURCC('D', 'D', 'S', ' ');
 
@@ -295,22 +296,36 @@ ResourceFormat DXGIFormat2ResourceFormat(DXGI_FORMAT format)
       return special;
     case DXGI_FORMAT_R11G11B10_FLOAT: special.type = ResourceFormatType::R11G11B10; return special;
     case DXGI_FORMAT_B5G6R5_UNORM:
-      fmt8.SetBGRAOrder(true);
+      special.SetBGRAOrder(true);
       special.type = ResourceFormatType::R5G6B5;
       return special;
     case DXGI_FORMAT_B5G5R5A1_UNORM:
-      fmt8.SetBGRAOrder(true);
+      special.SetBGRAOrder(true);
       special.type = ResourceFormatType::R5G5B5A1;
       return special;
     case DXGI_FORMAT_R9G9B9E5_SHAREDEXP:
       special.type = ResourceFormatType::R9G9B9E5;
       return special;
+    case DXGI_FORMAT_A8_UNORM: special.type = ResourceFormatType::A8; return special;
     case DXGI_FORMAT_B4G4R4A4_UNORM:
-      fmt8.SetBGRAOrder(true);
+      special.SetBGRAOrder(true);
       special.type = ResourceFormatType::R4G4B4A4;
       return special;
-    case DXGI_FORMAT_D24_UNORM_S8_UINT: special.type = ResourceFormatType::D24S8; return special;
-    case DXGI_FORMAT_D32_FLOAT_S8X24_UINT: special.type = ResourceFormatType::D32S8; return special;
+    case DXGI_FORMAT_D24_UNORM_S8_UINT:
+      special.compType = CompType::Depth;
+      special.type = ResourceFormatType::D24S8;
+      return special;
+    case DXGI_FORMAT_D32_FLOAT_S8X24_UINT:
+      special.compType = CompType::Depth;
+      special.type = ResourceFormatType::D32S8;
+      return special;
+    case DXGI_FORMAT_YUY2:
+      special.type = ResourceFormatType::YUV8;
+      special.compByteWidth = 1;
+      special.compCount = 3;
+      special.compType = CompType::UNorm;
+      special.SetYUVSubsampling(422);
+      return special;
 
     case DXGI_FORMAT_R32G32B32A32_UINT:
       fmt32.compType = CompType::UInt;
@@ -344,6 +359,7 @@ ResourceFormat DXGIFormat2ResourceFormat(DXGI_FORMAT format)
 
     case DXGI_FORMAT_R32_UINT: fmt32.compType = CompType::UInt; return fmt32;
     case DXGI_FORMAT_R32_SINT: fmt32.compType = CompType::SInt; return fmt32;
+    case DXGI_FORMAT_D32_FLOAT: fmt32.compType = CompType::Depth; return fmt32;
     case DXGI_FORMAT_R32_FLOAT: return fmt32;
 
     case DXGI_FORMAT_R16G16B16A16_UINT:
@@ -382,6 +398,7 @@ ResourceFormat DXGIFormat2ResourceFormat(DXGI_FORMAT format)
       return fmt16;
     case DXGI_FORMAT_R16G16_FLOAT: fmt16.compCount = 2; return fmt16;
 
+    case DXGI_FORMAT_D16_UNORM: fmt16.compType = CompType::Depth; return fmt16;
     case DXGI_FORMAT_R16_UINT: fmt16.compType = CompType::UInt; return fmt16;
     case DXGI_FORMAT_R16_SINT: fmt16.compType = CompType::SInt; return fmt16;
     case DXGI_FORMAT_R16_UNORM: fmt16.compType = CompType::UNorm; return fmt16;
@@ -451,30 +468,36 @@ DXGI_FORMAT ResourceFormat2DXGIFormat(ResourceFormat format)
       case ResourceFormatType::BC3:
         return format.SRGBCorrected() ? DXGI_FORMAT_BC3_UNORM_SRGB : DXGI_FORMAT_BC3_UNORM;
       case ResourceFormatType::BC4:
-        return format.compType == CompType::UNorm ? DXGI_FORMAT_BC4_UNORM : DXGI_FORMAT_BC4_SNORM;
+        return format.compType == CompType::SNorm ? DXGI_FORMAT_BC4_SNORM : DXGI_FORMAT_BC4_UNORM;
       case ResourceFormatType::BC5:
-        return format.compType == CompType::UNorm ? DXGI_FORMAT_BC5_UNORM : DXGI_FORMAT_BC5_SNORM;
+        return format.compType == CompType::SNorm ? DXGI_FORMAT_BC5_SNORM : DXGI_FORMAT_BC5_UNORM;
       case ResourceFormatType::BC6:
-        return format.compType == CompType::UNorm ? DXGI_FORMAT_BC6H_UF16 : DXGI_FORMAT_BC6H_SF16;
+        return format.compType == CompType::SNorm ? DXGI_FORMAT_BC6H_SF16 : DXGI_FORMAT_BC6H_UF16;
       case ResourceFormatType::BC7:
         return format.SRGBCorrected() ? DXGI_FORMAT_BC7_UNORM_SRGB : DXGI_FORMAT_BC7_UNORM;
       case ResourceFormatType::R10G10B10A2:
-        return format.compType == CompType::UNorm ? DXGI_FORMAT_R10G10B10A2_UNORM
-                                                  : DXGI_FORMAT_R10G10B10A2_UINT;
+        if(format.BGRAOrder())
+          return DXGI_FORMAT_UNKNOWN;
+        return format.compType == CompType::UInt ? DXGI_FORMAT_R10G10B10A2_UINT
+                                                 : DXGI_FORMAT_R10G10B10A2_UNORM;
       case ResourceFormatType::R11G11B10: return DXGI_FORMAT_R11G11B10_FLOAT;
       case ResourceFormatType::R5G6B5:
-        RDCASSERT(format.BGRAOrder());
+        if(!format.BGRAOrder())
+          return DXGI_FORMAT_UNKNOWN;
         return DXGI_FORMAT_B5G6R5_UNORM;
       case ResourceFormatType::R5G5B5A1:
-        RDCASSERT(format.BGRAOrder());
+        if(!format.BGRAOrder())
+          return DXGI_FORMAT_UNKNOWN;
         return DXGI_FORMAT_B5G5R5A1_UNORM;
       case ResourceFormatType::R9G9B9E5: return DXGI_FORMAT_R9G9B9E5_SHAREDEXP;
       case ResourceFormatType::R4G4B4A4:
-        RDCASSERT(format.BGRAOrder());
+        if(!format.BGRAOrder())
+          return DXGI_FORMAT_UNKNOWN;
         return DXGI_FORMAT_B4G4R4A4_UNORM;
       case ResourceFormatType::D24S8: return DXGI_FORMAT_D24_UNORM_S8_UINT;
       case ResourceFormatType::D32S8: return DXGI_FORMAT_D32_FLOAT_S8X24_UINT;
       case ResourceFormatType::S8: return DXGI_FORMAT_R8_UINT;
+      case ResourceFormatType::A8: return DXGI_FORMAT_A8_UNORM;
       default:
       case ResourceFormatType::R4G4:
       case ResourceFormatType::D16S8:
@@ -605,6 +628,7 @@ DXGI_FORMAT ResourceFormat2DXGIFormat(ResourceFormat format)
     {
       switch(format.compType)
       {
+        case CompType::Depth: return DXGI_FORMAT_D16_UNORM;
         case CompType::UInt: return DXGI_FORMAT_R16_UINT;
         case CompType::SInt: return DXGI_FORMAT_R16_SINT;
         case CompType::UNorm: return DXGI_FORMAT_R16_UNORM;
@@ -733,7 +757,8 @@ bool write_dds_to_file(FILE *f, const dds_data &data)
   {
     switch(data.format.type)
     {
-      case ResourceFormatType::S8: bytesPerPixel = 1; break;
+      case ResourceFormatType::S8:
+      case ResourceFormatType::A8: bytesPerPixel = 1; break;
       case ResourceFormatType::R10G10B10A2:
       case ResourceFormatType::R9G9B9E5:
       case ResourceFormatType::R11G11B10:
@@ -759,6 +784,7 @@ bool write_dds_to_file(FILE *f, const dds_data &data)
   // special case a couple of formats to write out non-DX10 style, for
   // backwards compatibility
   if(data.format.compByteWidth == 1 && data.format.compCount == 4 &&
+     data.format.type == ResourceFormatType::Regular &&
      (data.format.compType == CompType::UNorm || data.format.compType == CompType::UNormSRGB))
   {
     header.ddspf.dwFlags = DDPF_RGBA;
@@ -895,7 +921,7 @@ dds_data load_dds_from_file(FILE *f)
   bool dx10Header = false;
   DDS_HEADER_DXT10 headerDXT10 = {};
 
-  if(header.ddspf.dwFlags == DDPF_FOURCC && header.ddspf.dwFourCC == MAKE_FOURCC('D', 'X', '1', '0'))
+  if(header.ddspf.dwFlags & DDPF_FOURCC && header.ddspf.dwFourCC == MAKE_FOURCC('D', 'X', '1', '0'))
   {
     FileIO::fread(&headerDXT10, sizeof(headerDXT10), 1, f);
     dx10Header = true;
@@ -918,6 +944,8 @@ dds_data load_dds_from_file(FILE *f)
   if(ret.cubemap)
     ret.slices *= 6;
 
+  bool bgrSwap = false;
+
   if(dx10Header)
   {
     ret.format = DXGIFormat2ResourceFormat(headerDXT10.dxgiFormat);
@@ -934,9 +962,11 @@ dds_data load_dds_from_file(FILE *f)
       case MAKE_FOURCC('D', 'X', 'T', '1'):
         ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_BC1_UNORM);
         break;
+      case MAKE_FOURCC('D', 'X', 'T', '2'):
       case MAKE_FOURCC('D', 'X', 'T', '3'):
         ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_BC2_UNORM);
         break;
+      case MAKE_FOURCC('D', 'X', 'T', '4'):
       case MAKE_FOURCC('D', 'X', 'T', '5'):
         ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_BC3_UNORM);
         break;
@@ -960,7 +990,12 @@ dds_data load_dds_from_file(FILE *f)
       case MAKE_FOURCC('G', 'R', 'G', 'B'):
         ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_G8R8_G8B8_UNORM);
         break;
+      case MAKE_FOURCC('U', 'Y', 'V', 'Y'): bgrSwap = true; DELIBERATE_FALLTHROUGH();
+      case MAKE_FOURCC('Y', 'U', 'Y', '2'):
+        ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_YUY2);
+        break;
       case 36: ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_R16G16B16A16_UNORM); break;
+      case 82: ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_D32_FLOAT); break;
       case 110: ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_R16G16B16A16_SNORM); break;
       case 111: ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_R16_FLOAT); break;
       case 112: ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_R16G16_FLOAT); break;
@@ -968,6 +1003,7 @@ dds_data load_dds_from_file(FILE *f)
       case 114: ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_R32_FLOAT); break;
       case 115: ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_R32G32_FLOAT); break;
       case 116: ret.format = DXGIFormat2ResourceFormat(DXGI_FORMAT_R32G32B32A32_FLOAT); break;
+      case 117: RDCERR("Legacy CxV8U8 format is unsupported"); return error;
       default: RDCWARN("Unsupported FourCC: %08x", header.ddspf.dwFourCC); return error;
     }
   }
@@ -990,9 +1026,11 @@ dds_data load_dds_from_file(FILE *f)
   }
 
   uint32_t bytesPerPixel = 1;
+  int subsamplePacking = 1;
   switch(ret.format.type)
   {
-    case ResourceFormatType::S8: bytesPerPixel = 1; break;
+    case ResourceFormatType::S8:
+    case ResourceFormatType::A8: bytesPerPixel = 1; break;
     case ResourceFormatType::R10G10B10A2:
     case ResourceFormatType::R9G9B9E5:
     case ResourceFormatType::R11G11B10:
@@ -1001,11 +1039,18 @@ dds_data load_dds_from_file(FILE *f)
     case ResourceFormatType::R5G5B5A1:
     case ResourceFormatType::R4G4B4A4: bytesPerPixel = 2; break;
     case ResourceFormatType::D32S8: bytesPerPixel = 8; break;
-    case ResourceFormatType::D16S8:
     case ResourceFormatType::YUV8:
+      if(ret.format.YUVPlaneCount() == 1 && ret.format.YUVSubsampling() == 422)
+      {
+        subsamplePacking = 2;
+        bytesPerPixel = 2;
+        break;
+      }
+      DELIBERATE_FALLTHROUGH();
     case ResourceFormatType::YUV10:
     case ResourceFormatType::YUV12:
     case ResourceFormatType::YUV16:
+    case ResourceFormatType::D16S8:
     case ResourceFormatType::R4G4:
       RDCERR("Unsupported file format %u", ret.format.type);
       return error;
@@ -1028,10 +1073,6 @@ dds_data load_dds_from_file(FILE *f)
       case ResourceFormatType::ETC2:
       case ResourceFormatType::EAC:
       case ResourceFormatType::ASTC:
-      case ResourceFormatType::YUV8:
-      case ResourceFormatType::YUV10:
-      case ResourceFormatType::YUV12:
-      case ResourceFormatType::YUV16:
         RDCERR("Unsupported file format, %u", ret.format.type);
         return error;
       default: break;
@@ -1047,6 +1088,7 @@ dds_data load_dds_from_file(FILE *f)
     for(int mip = 0; mip < ret.mips; mip++)
     {
       int rowlen = RDCMAX(1, ret.width >> mip);
+      rowlen = AlignUp(rowlen, subsamplePacking);
       int numRows = RDCMAX(1, ret.height >> mip);
       int numdepths = RDCMAX(1, ret.depth >> mip);
       int pitch = RDCMAX(1U, rowlen * bytesPerPixel);
@@ -1074,6 +1116,28 @@ dds_data load_dds_from_file(FILE *f)
         {
           FileIO::fread(bytedata, 1, pitch, f);
 
+          if(bgrSwap)
+          {
+            byte *rgba = bytedata;
+
+            if(bytesPerPixel >= 3)
+            {
+              for(int p = 0; p < rowlen; p++)
+              {
+                std::swap(rgba[0], rgba[2]);
+                rgba += bytesPerPixel;
+              }
+            }
+            else
+            {
+              for(int p = 0; p < rowlen; p++)
+              {
+                std::swap(rgba[0], rgba[1]);
+                rgba += bytesPerPixel;
+              }
+            }
+          }
+
           bytedata += pitch;
         }
       }
