@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -129,6 +129,12 @@
 #define RDOC_XCB OPTION_OFF
 #endif
 
+#if defined(RENDERDOC_WINDOWING_WAYLAND)
+#define RDOC_WAYLAND OPTION_ON
+#else
+#define RDOC_WAYLAND OPTION_OFF
+#endif
+
 /////////////////////////////////////////////////
 // Global constants
 enum
@@ -136,7 +142,11 @@ enum
   RenderDoc_FirstTargetControlPort = 38920,
   RenderDoc_LastTargetControlPort = RenderDoc_FirstTargetControlPort + 7,
   RenderDoc_RemoteServerPort = 39920,
-  RenderDoc_AndroidPortOffset = 50,
+
+  RenderDoc_ForwardPortBase = 38950,
+  RenderDoc_ForwardTargetControlOffset = 0,
+  RenderDoc_ForwardRemoteServerOffset = 9,
+  RenderDoc_ForwardPortStride = 10,
 };
 
 #define RENDERDOC_VULKAN_LAYER_NAME "VK_LAYER_RENDERDOC_Capture"
@@ -182,7 +192,7 @@ enum
 
 #else
 
-#define OUTPUT_LOG_TO_STDOUT OPTION_ON
+#define OUTPUT_LOG_TO_STDOUT OPTION_OFF
 #define OUTPUT_LOG_TO_STDERR OPTION_OFF
 
 #endif
@@ -209,4 +219,4 @@ enum
 // otherwise, enable them in development builds
 #define ENABLE_UNIT_TESTS RDOC_DEVEL
 
-#endif
\ No newline at end of file
+#endif
