@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,13 +24,15 @@
 
 #pragma once
 
+#include <map>
+#include "common/common.h"
 #include "os/os_specific.h"
 
 template <typename ResultType, typename ShaderCallbacks>
 bool LoadShaderCache(const char *filename, const uint32_t magicNumber, const uint32_t versionNumber,
                      std::map<uint32_t, ResultType> &resultCache, const ShaderCallbacks &callbacks)
 {
-  string shadercache = FileIO::GetAppFolderFilename(filename);
+  rdcstr shadercache = FileIO::GetAppFolderFilename(filename);
 
   FILE *f = FileIO::fopen(shadercache.c_str(), "rb");
 
@@ -152,7 +154,7 @@ template <typename ResultType, typename ShaderCallbacks>
 void SaveShaderCache(const char *filename, uint32_t magicNumber, uint32_t versionNumber,
                      const std::map<uint32_t, ResultType> &cache, const ShaderCallbacks &callbacks)
 {
-  string shadercache = FileIO::GetAppFolderFilename(filename);
+  rdcstr shadercache = FileIO::GetAppFolderFilename(filename);
 
   FILE *f = FileIO::fopen(shadercache.c_str(), "wb");
 
