@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -28,6 +28,7 @@
 #include <algorithm>
 #include "api/replay/version.h"
 #include "common/common.h"
+#include "common/threading.h"
 #include "hooks/hooks.h"
 #include "maths/formatpacking.h"
 #include "replay/replay_driver.h"
@@ -43,9 +44,28 @@
 
 #include "replay/renderdoc_serialise.inl"
 
-// this one is done by hand as we format it
+void LogReplayOptions(const ReplayOptions &opts)
+{
+  RDCLOG("%s API validation during replay", (opts.apiValidation ? "Enabling" : "Not enabling"));
+
+  if(opts.forceGPUVendor == GPUVendor::Unknown && opts.forceGPUDeviceID == 0 &&
+     opts.forceGPUDriverName.empty())
+  {
+    RDCLOG("Using default GPU replay selection algorithm");
+  }
+  else
+  {
+    RDCLOG("Overriding GPU replay selection:");
+    RDCLOG("  Vendor %s, device %u, driver \"%s\"", ToStr(opts.forceGPUVendor).c_str(),
+           opts.forceGPUDeviceID, opts.forceGPUDriverName.c_str());
+  }
+
+  RDCLOG("Replay optimisation level: %s", ToStr(opts.optimisation).c_str());
+}
+
+// these one is done by hand as we format it
 template <>
-std::string DoStringise(const ResourceId &el)
+rdcstr DoStringise(const ResourceId &el)
 {
   RDCCOMPILE_ASSERT(sizeof(el) == sizeof(uint64_t), "ResourceId is no longer 1:1 with uint64_t");
 
@@ -86,10 +106,29 @@ std::string DoStringise(const ResourceId &el)
   for(size_t i = 0; i <= prefixlast; i++)
     *(c--) = PREFIX[prefixlast - i];
 
+#undef PREFIX
+
   // the loop will have stepped us to the first NULL before our string, so return c+1
   return c + 1;
 }
 
+template <>
+rdcstr DoStringise(const PointerVal &el)
+{
+  if(el.shader != ResourceId())
+  {
+    // we don't want to format as an ID, we need to encode the raw value
+    uint64_t num;
+    memcpy(&num, &el.shader, sizeof(num));
+
+    return StringFormat::Fmt("GPUAddress::%llu::%llu::%u", el.pointer, num, el.pointerTypeID);
+  }
+  else
+  {
+    return StringFormat::Fmt("GPUAddress::%llu", el.pointer);
+  }
+}
+
 BASIC_TYPE_SERIALISE_STRINGIFY(ResourceId, (uint64_t &)el, SDBasic::Resource, 8);
 
 INSTANTIATE_SERIALISE_TYPE(ResourceId);
@@ -102,7 +141,7 @@ INSTANTIATE_SERIALISE_TYPE(ResourceId);
 ReplayStatus IMG_CreateReplayDevice(RDCFile *rdc, IReplayDriver **driver);
 
 template <>
-std::string DoStringise(const RDCDriver &el)
+rdcstr DoStringise(const RDCDriver &el)
 {
   BEGIN_ENUM_STRINGISE(RDCDriver);
   {
@@ -122,7 +161,7 @@ std::string DoStringise(const RDCDriver &el)
 }
 
 template <>
-std::string DoStringise(const ReplayLogType &el)
+rdcstr DoStringise(const ReplayLogType &el)
 {
   BEGIN_ENUM_STRINGISE(ReplayLogType);
   {
@@ -134,7 +173,7 @@ std::string DoStringise(const ReplayLogType &el)
 }
 
 template <>
-std::string DoStringise(const VendorExtensions &el)
+rdcstr DoStringise(const VendorExtensions &el)
 {
   BEGIN_ENUM_STRINGISE(VendorExtensions);
   {
@@ -146,7 +185,7 @@ std::string DoStringise(const VendorExtensions &el)
 }
 
 template <>
-std::string DoStringise(const RENDERDOC_InputButton &el)
+rdcstr DoStringise(const RENDERDOC_InputButton &el)
 {
   char alphanumericbuf[2] = {'A', 0};
 
@@ -194,16 +233,16 @@ std::string DoStringise(const RENDERDOC_InputButton &el)
 }
 
 template <>
-std::string DoStringise(const SystemChunk &el)
+rdcstr DoStringise(const SystemChunk &el)
 {
   BEGIN_ENUM_STRINGISE(SystemChunk);
   {
-    STRINGISE_ENUM_CLASS_NAMED(DriverInit, "Driver Initialisation Parameters");
-    STRINGISE_ENUM_CLASS_NAMED(InitialContentsList, "List of Initial Contents Resources");
-    STRINGISE_ENUM_CLASS_NAMED(InitialContents, "Initial Contents");
-    STRINGISE_ENUM_CLASS_NAMED(CaptureBegin, "Beginning of Capture");
-    STRINGISE_ENUM_CLASS_NAMED(CaptureScope, "Frame Metadata");
-    STRINGISE_ENUM_CLASS_NAMED(CaptureEnd, "End of Capture");
+    STRINGISE_ENUM_CLASS_NAMED(DriverInit, "Internal: Driver Initialisation Parameters");
+    STRINGISE_ENUM_CLASS_NAMED(InitialContentsList, "Internal: List of Initial Contents Resources");
+    STRINGISE_ENUM_CLASS_NAMED(InitialContents, "Internal: Initial Contents");
+    STRINGISE_ENUM_CLASS_NAMED(CaptureBegin, "Internal: Beginning of Capture");
+    STRINGISE_ENUM_CLASS_NAMED(CaptureScope, "Internal: Frame Metadata");
+    STRINGISE_ENUM_CLASS_NAMED(CaptureEnd, "Internal: End of Capture");
   }
   END_ENUM_STRINGISE();
 }
@@ -220,6 +259,37 @@ RenderDoc &RenderDoc::Inst()
 void RenderDoc::RecreateCrashHandler()
 {
 #if ENABLED(RDOC_CRASH_HANDLER)
+
+#if ENABLED(RDOC_WIN32)
+  // there are way too many invalid reports coming from chrome, completely disable the crash handler
+  // in that case.
+  rdcstr exename;
+  FileIO::GetExecutableFilename(exename);
+  exename = strlower(exename);
+
+  if(exename.find("chrome.exe") &&
+     (GetModuleHandleA("chrome_elf.dll") || GetModuleHandleA("chrome_child.dll")))
+  {
+    RDCWARN("Disabling crash handling server due to detected chrome.");
+    return;
+  }
+
+  // some people use vivaldi which is just chrome
+  if(exename.find("vivaldi.exe") &&
+     (GetModuleHandleA("vivaldi_elf.dll") || GetModuleHandleA("vivaldi_child.dll")))
+  {
+    RDCWARN("Disabling crash handling server due to detected chrome.");
+    return;
+  }
+
+  // ditto opera
+  if(exename.find("opera.exe") && GetModuleHandleA("opera_browser.dll"))
+  {
+    RDCWARN("Disabling crash handling server due to detected chrome.");
+    return;
+  }
+#endif
+
   m_ExHandler = new CrashHandler(m_ExHandler);
 #endif
 
@@ -316,7 +386,7 @@ void RenderDoc::Initialise()
   // set default capture log - useful for when hooks aren't setup
   // through the UI (and a log file isn't set manually)
   {
-    string capture_filename;
+    rdcstr capture_filename;
 
     const char *base = "RenderDoc_app";
     if(IsReplayApp())
@@ -354,6 +424,10 @@ void RenderDoc::Initialise()
   RDCLOG("Packaged for %s (%s) - %s", DISTRIBUTION_NAME, DISTRIBUTION_VERSION, DISTRIBUTION_CONTACT);
 #endif
 
+#if defined(RENDERDOC_HOOK_DLSYM)
+  RDCWARN("dlsym() hooking enabled!");
+#endif
+
   Keyboard::Init();
 
   m_FrameTimer.InitTimers();
@@ -361,14 +435,14 @@ void RenderDoc::Initialise()
   m_ExHandler = NULL;
 
   {
-    string curFile;
+    rdcstr curFile;
     FileIO::GetExecutableFilename(curFile);
 
-    string f = strlower(curFile);
+    rdcstr f = strlower(curFile);
 
     // only create crash handler when we're not in renderdoccmd.exe (to prevent infinite loop as
     // the crash handler itself launches renderdoccmd.exe)
-    if(f.find("renderdoccmd.exe") == string::npos)
+    if(!f.contains("renderdoccmd.exe"))
     {
       RecreateCrashHandler();
     }
@@ -446,7 +520,7 @@ void RenderDoc::Shutdown()
   }
 }
 
-void RenderDoc::ProcessGlobalEnvironment(GlobalEnvironment env, const std::vector<std::string> &args)
+void RenderDoc::ProcessGlobalEnvironment(GlobalEnvironment env, const rdcarray<rdcstr> &args)
 {
   m_GlobalEnv = env;
 
@@ -461,6 +535,122 @@ void RenderDoc::ProcessGlobalEnvironment(GlobalEnvironment env, const std::vecto
     for(size_t i = 0; i < args.size(); i++)
       RDCDEBUG("[%u]: %s", (uint32_t)i, args[i].c_str());
   }
+
+  if(env.enumerateGPUs)
+  {
+    m_AvailableGPUThread = Threading::CreateThread([this]() {
+      for(GraphicsAPI api : {GraphicsAPI::D3D11, GraphicsAPI::D3D12, GraphicsAPI::Vulkan})
+      {
+        RDCDriver driverType = RDCDriver::Unknown;
+
+        switch(api)
+        {
+          case GraphicsAPI::D3D11: driverType = RDCDriver::D3D11; break;
+          case GraphicsAPI::D3D12: driverType = RDCDriver::D3D12; break;
+          case GraphicsAPI::OpenGL: break;
+          case GraphicsAPI::Vulkan: driverType = RDCDriver::Vulkan; break;
+        }
+
+        if(driverType == RDCDriver::Unknown || !HasReplayDriver(driverType))
+          continue;
+
+        IReplayDriver *driver = NULL;
+        ReplayStatus status = m_ReplayDriverProviders[driverType](NULL, ReplayOptions(), &driver);
+
+        if(status == ReplayStatus::Succeeded)
+        {
+          rdcarray<GPUDevice> gpus = driver->GetAvailableGPUs();
+
+          for(const GPUDevice &newgpu : gpus)
+          {
+            bool addnew = true;
+
+            for(GPUDevice &oldgpu : m_AvailableGPUs)
+            {
+              // if we have this GPU listed already, just add its API to the previous list
+              if(oldgpu == newgpu)
+              {
+                oldgpu.apis.push_back(api);
+                addnew = false;
+              }
+            }
+
+            if(addnew)
+              m_AvailableGPUs.push_back(newgpu);
+          }
+        }
+        else
+        {
+          RDCWARN("Couldn't create proxy replay driver for %s: %s", ToStr(driverType).c_str(),
+                  ToStr(status).c_str());
+        }
+
+        if(driver)
+          driver->Shutdown();
+      }
+
+      // we now have a list of GPUs, however we might have some duplicates if some APIs have
+      // multiple drivers for a single device. To compact this list, for each GPU with no driver
+      // we find all matching multi-drive GPUs and merge it into all matching copies.
+      bool hasDriverNames = false;
+      for(size_t i = 0; i < m_AvailableGPUs.size(); i++)
+        hasDriverNames |= !m_AvailableGPUs[i].driver.empty();
+
+      if(hasDriverNames)
+      {
+        for(size_t i = 0; i < m_AvailableGPUs.size();)
+        {
+          bool applied = false;
+
+          if(!m_AvailableGPUs[i].driver.empty())
+          {
+            i++;
+            continue;
+          }
+
+          // scan all subsequent GPUs, if we find a duplicate, merge the APIs
+          for(size_t j = i + 1; j < m_AvailableGPUs.size(); j++)
+          {
+            if(m_AvailableGPUs[i].vendor == m_AvailableGPUs[j].vendor &&
+               m_AvailableGPUs[i].deviceID == m_AvailableGPUs[j].deviceID)
+            {
+              RDCASSERT(!m_AvailableGPUs[j].driver.empty());
+              for(GraphicsAPI a : m_AvailableGPUs[i].apis)
+              {
+                if(m_AvailableGPUs[j].apis.indexOf(a) == -1)
+                  m_AvailableGPUs[j].apis.push_back(a);
+              }
+              applied = true;
+            }
+          }
+
+          // we "applied" this GPU to all its driver-based duplicates, so we can remove it now
+          if(applied)
+          {
+            m_AvailableGPUs.erase(i);
+          }
+          else
+          {
+            i++;
+          }
+        }
+      }
+
+      // sort the APIs list in each GPU, and sort the GPUs
+      std::sort(m_AvailableGPUs.begin(), m_AvailableGPUs.end());
+      for(GPUDevice &dev : m_AvailableGPUs)
+      {
+        std::sort(dev.apis.begin(), dev.apis.end());
+      }
+    });
+  }
+}
+
+void RenderDoc::RegisterShutdownFunction(ShutdownFunction func)
+{
+  auto it = std::lower_bound(m_ShutdownFunctions.begin(), m_ShutdownFunctions.end(), func);
+  if(it == m_ShutdownFunctions.end() || *it != func)
+    m_ShutdownFunctions.insert(it - m_ShutdownFunctions.begin(), func);
 }
 
 bool RenderDoc::MatchClosestWindow(void *&dev, void *&wnd)
@@ -515,7 +705,10 @@ IFrameCapturer *RenderDoc::MatchFrameCapturer(void *dev, void *wnd)
         return defaultit->second;
     }
 
-    RDCERR("Couldn't find matching frame capturer for device %p window %p", dev, wnd);
+    RDCERR(
+        "Couldn't find matching frame capturer for device %p window %p "
+        "from %zu device frame capturers and %zu frame capturers",
+        dev, wnd, m_DeviceFrameCapturers.size(), m_WindowFrameCapturers.size());
     return NULL;
   }
 
@@ -584,7 +777,7 @@ bool RenderDoc::IsTargetControlConnected()
   return !RenderDoc::Inst().m_SingleClientName.empty();
 }
 
-string RenderDoc::GetTargetControlUsername()
+rdcstr RenderDoc::GetTargetControlUsername()
 {
   SCOPED_LOCK(RenderDoc::Inst().m_SingleClientLock);
   return RenderDoc::Inst().m_SingleClientName;
@@ -643,18 +836,18 @@ void RenderDoc::CycleActiveWindow()
   }
 }
 
-string RenderDoc::GetOverlayText(RDCDriver driver, uint32_t frameNumber, int flags)
+rdcstr RenderDoc::GetOverlayText(RDCDriver driver, uint32_t frameNumber, int flags)
 {
   const bool activeWindow = (flags & eOverlay_ActiveWindow);
   const bool capturesEnabled = (flags & eOverlay_CaptureDisabled) == 0;
 
   uint32_t overlay = GetOverlayBits();
 
-  std::string overlayText = ToStr(driver) + ". ";
+  rdcstr overlayText = ToStr(driver) + ". ";
 
   if(activeWindow)
   {
-    vector<RENDERDOC_InputButton> keys = GetCaptureKeys();
+    rdcarray<RENDERDOC_InputButton> keys = GetCaptureKeys();
 
     if(capturesEnabled)
     {
@@ -704,7 +897,10 @@ string RenderDoc::GetOverlayText(RDCDriver driver, uint32_t frameNumber, int fla
       {
         if(now - m_Captures[i].timestamp < 20)
         {
-          overlayText += StringFormat::Fmt("Captured frame %d.\n", m_Captures[i].frameNumber);
+          if(m_Captures[i].frameNumber == ~0U)
+            overlayText += "Captured user-defined capture.\n";
+          else
+            overlayText += StringFormat::Fmt("Captured frame %d.\n", m_Captures[i].frameNumber);
         }
       }
     }
@@ -716,7 +912,7 @@ string RenderDoc::GetOverlayText(RDCDriver driver, uint32_t frameNumber, int fla
   }
   else if(capturesEnabled)
   {
-    vector<RENDERDOC_InputButton> keys = GetFocusKeys();
+    rdcarray<RENDERDOC_InputButton> keys = GetFocusKeys();
 
     overlayText += "Inactive window.";
 
@@ -739,6 +935,13 @@ string RenderDoc::GetOverlayText(RDCDriver driver, uint32_t frameNumber, int fla
   return overlayText;
 }
 
+void RenderDoc::QueueCapture(uint32_t frameNumber)
+{
+  auto it = std::lower_bound(m_QueuedFrameCaptures.begin(), m_QueuedFrameCaptures.end(), frameNumber);
+  if(it == m_QueuedFrameCaptures.end() || *it != frameNumber)
+    m_QueuedFrameCaptures.insert(it - m_QueuedFrameCaptures.begin(), frameNumber);
+}
+
 bool RenderDoc::ShouldTriggerCapture(uint32_t frameNumber)
 {
   bool ret = m_Cap > 0;
@@ -746,7 +949,7 @@ bool RenderDoc::ShouldTriggerCapture(uint32_t frameNumber)
   if(m_Cap > 0)
     m_Cap--;
 
-  set<uint32_t> frames;
+  rdcarray<uint32_t> frames;
   frames.swap(m_QueuedFrameCaptures);
   for(auto it = frames.begin(); it != frames.end(); ++it)
   {
@@ -762,7 +965,7 @@ bool RenderDoc::ShouldTriggerCapture(uint32_t frameNumber)
     else
     {
       // not hit this yet, keep it around
-      m_QueuedFrameCaptures.insert(*it);
+      m_QueuedFrameCaptures.push_back(*it);
     }
   }
 
@@ -771,6 +974,12 @@ bool RenderDoc::ShouldTriggerCapture(uint32_t frameNumber)
 
 void RenderDoc::ResamplePixels(const FramePixels &in, RDCThumb &out)
 {
+  if(in.width == 0 || in.height == 0)
+  {
+    out = RDCThumb();
+    return;
+  }
+
   // code below assumes pitch_requirement is a power of 2 number
   RDCASSERT((in.pitch_requirement & (in.pitch_requirement - 1)) == 0);
 
@@ -830,20 +1039,9 @@ void RenderDoc::ResamplePixels(const FramePixels &in, RDCThumb &out)
         float linearG = RDCCLAMP(ConvertFromHalf(src16[1]), 0.0f, 1.0f);
         float linearB = RDCCLAMP(ConvertFromHalf(src16[2]), 0.0f, 1.0f);
 
-        if(linearR < 0.0031308f)
-          dst[0] = byte(255.0f * (12.92f * linearR));
-        else
-          dst[0] = byte(255.0f * (1.055f * powf(linearR, 1.0f / 2.4f) - 0.055f));
-
-        if(linearG < 0.0031308f)
-          dst[1] = byte(255.0f * (12.92f * linearG));
-        else
-          dst[1] = byte(255.0f * (1.055f * powf(linearG, 1.0f / 2.4f) - 0.055f));
-
-        if(linearB < 0.0031308f)
-          dst[2] = byte(255.0f * (12.92f * linearB));
-        else
-          dst[2] = byte(255.0f * (1.055f * powf(linearB, 1.0f / 2.4f) - 0.055f));
+        dst[0] = byte(255.0f * ConvertLinearToSRGB(linearR));
+        dst[1] = byte(255.0f * ConvertLinearToSRGB(linearG));
+        dst[2] = byte(255.0f * ConvertLinearToSRGB(linearB));
       }
       else
       {
@@ -879,17 +1077,23 @@ void RenderDoc::ResamplePixels(const FramePixels &in, RDCThumb &out)
     }
   }
 }
+
 void RenderDoc::EncodePixelsPNG(const RDCThumb &in, RDCThumb &out)
 {
+  if(in.width == 0 || in.height == 0)
+  {
+    out = RDCThumb();
+    return;
+  }
+
   struct WriteCallbackData
   {
-    std::vector<byte> buffer;
+    bytebuf buffer;
 
     static void writeData(void *context, void *data, int size)
     {
       WriteCallbackData *pThis = (WriteCallbackData *)context;
-      const byte *start = (const byte *)data;
-      pThis->buffer.insert(pThis->buffer.end(), start, start + size);
+      pThis->buffer.append((const byte *)data, size);
     }
   };
 
@@ -908,7 +1112,12 @@ RDCFile *RenderDoc::CreateRDC(RDCDriver driver, uint32_t frameNum, const FramePi
 {
   RDCFile *ret = new RDCFile;
 
-  m_CurrentLogFile = StringFormat::Fmt("%s_frame%u.rdc", m_CaptureFileTemplate.c_str(), frameNum);
+  rdcstr suffix = StringFormat::Fmt("_frame%u", frameNum);
+
+  if(frameNum == ~0U)
+    suffix = "_capture";
+
+  m_CurrentLogFile = StringFormat::Fmt("%s%s.rdc", m_CaptureFileTemplate.c_str(), suffix.c_str());
 
   // make sure we don't stomp another capture if we make multiple captures in the same frame.
   {
@@ -919,7 +1128,7 @@ RDCFile *RenderDoc::CreateRDC(RDCDriver driver, uint32_t frameNum, const FramePi
           }) != m_Captures.end())
     {
       m_CurrentLogFile =
-          StringFormat::Fmt("%s_frame%u_%d.rdc", m_CaptureFileTemplate.c_str(), frameNum, altnum);
+          StringFormat::Fmt("%s%s_%d.rdc", m_CaptureFileTemplate.c_str(), suffix.c_str(), altnum);
       altnum++;
     }
   }
@@ -932,8 +1141,6 @@ RDCFile *RenderDoc::CreateRDC(RDCDriver driver, uint32_t frameNum, const FramePi
     EncodePixelsPNG(outRaw, outPng);
   }
 
-  RDCASSERT(outPng.pixels != NULL);
-
   ret->SetData(driver, ToStr(driver).c_str(), OSUtility::GetMachineIdent(), &outPng);
 
   FileIO::CreateParentDirectory(m_CurrentLogFile);
@@ -998,7 +1205,7 @@ void RenderDoc::RegisterStructuredProcessor(RDCDriver driver, StructuredProcesso
 
 void RenderDoc::RegisterCaptureExporter(CaptureExporter exporter, CaptureFileFormat description)
 {
-  std::string filetype = description.extension;
+  rdcstr filetype = description.extension;
 
   for(const CaptureFileFormat &fmt : m_ImportExportFormats)
   {
@@ -1020,7 +1227,7 @@ void RenderDoc::RegisterCaptureExporter(CaptureExporter exporter, CaptureFileFor
 void RenderDoc::RegisterCaptureImportExporter(CaptureImporter importer, CaptureExporter exporter,
                                               CaptureFileFormat description)
 {
-  std::string filetype = description.extension;
+  rdcstr filetype = description.extension;
 
   for(const CaptureFileFormat &fmt : m_ImportExportFormats)
   {
@@ -1040,6 +1247,16 @@ void RenderDoc::RegisterCaptureImportExporter(CaptureImporter importer, CaptureE
   m_Exporters[filetype] = exporter;
 }
 
+void RenderDoc::RegisterDeviceProtocol(const rdcstr &protocol, ProtocolHandler handler)
+{
+  if(m_Protocols[protocol] != NULL)
+  {
+    RDCERR("Duplicate protocol registration: %s", protocol.c_str());
+    return;
+  }
+  m_Protocols[protocol] = handler;
+}
+
 StructuredProcessor RenderDoc::GetStructuredProcessor(RDCDriver driver)
 {
   auto it = m_StructProcesssors.find(driver);
@@ -1076,9 +1293,36 @@ CaptureImporter RenderDoc::GetCaptureImporter(const char *filetype)
   return it->second;
 }
 
-std::vector<CaptureFileFormat> RenderDoc::GetCaptureFileFormats()
+rdcarray<rdcstr> RenderDoc::GetSupportedDeviceProtocols()
 {
-  std::vector<CaptureFileFormat> ret = m_ImportExportFormats;
+  rdcarray<rdcstr> ret;
+
+  for(auto it = m_Protocols.begin(); it != m_Protocols.end(); ++it)
+    ret.push_back(it->first);
+
+  return ret;
+}
+
+IDeviceProtocolHandler *RenderDoc::GetDeviceProtocol(const rdcstr &protocol)
+{
+  rdcstr p = protocol;
+
+  // allow passing in an URL with ://
+  int32_t offs = p.find("://");
+  if(offs >= 0)
+    p.erase(offs, p.size() - offs);
+
+  auto it = m_Protocols.find(p);
+
+  if(it != m_Protocols.end())
+    return it->second();
+
+  return NULL;
+}
+
+rdcarray<CaptureFileFormat> RenderDoc::GetCaptureFileFormats()
+{
+  rdcarray<CaptureFileFormat> ret = m_ImportExportFormats;
 
   std::sort(ret.begin(), ret.end());
 
@@ -1090,12 +1334,29 @@ std::vector<CaptureFileFormat> RenderDoc::GetCaptureFileFormats()
     rdc.openSupported = true;
     rdc.convertSupported = true;
 
-    ret.insert(ret.begin(), rdc);
+    ret.insert(0, rdc);
   }
 
   return ret;
 }
 
+rdcarray<GPUDevice> RenderDoc::GetAvailableGPUs()
+{
+  SyncAvailableGPUThread();
+
+  return m_AvailableGPUs;
+}
+
+void RenderDoc::SyncAvailableGPUThread()
+{
+  if(m_AvailableGPUThread)
+  {
+    Threading::JoinThread(m_AvailableGPUThread);
+    Threading::CloseThread(m_AvailableGPUThread);
+    m_AvailableGPUThread = 0;
+  }
+}
+
 bool RenderDoc::HasReplaySupport(RDCDriver driverType)
 {
   if(driverType == RDCDriver::Image)
@@ -1109,30 +1370,35 @@ bool RenderDoc::HasReplaySupport(RDCDriver driverType)
 
 ReplayStatus RenderDoc::CreateProxyReplayDriver(RDCDriver proxyDriver, IReplayDriver **driver)
 {
+  SyncAvailableGPUThread();
+
   // passing RDCDriver::Unknown means 'I don't care, give me a proxy driver of any type'
   if(proxyDriver == RDCDriver::Unknown)
   {
     if(!m_ReplayDriverProviders.empty())
-      return m_ReplayDriverProviders.begin()->second(NULL, driver);
+      return m_ReplayDriverProviders.begin()->second(NULL, ReplayOptions(), driver);
   }
 
   if(m_ReplayDriverProviders.find(proxyDriver) != m_ReplayDriverProviders.end())
-    return m_ReplayDriverProviders[proxyDriver](NULL, driver);
+    return m_ReplayDriverProviders[proxyDriver](NULL, ReplayOptions(), driver);
 
   RDCERR("Unsupported replay driver requested: %s", ToStr(proxyDriver).c_str());
   return ReplayStatus::APIUnsupported;
 }
 
-ReplayStatus RenderDoc::CreateReplayDriver(RDCFile *rdc, IReplayDriver **driver)
+ReplayStatus RenderDoc::CreateReplayDriver(RDCFile *rdc, const ReplayOptions &opts,
+                                           IReplayDriver **driver)
 {
   if(driver == NULL)
     return ReplayStatus::InternalError;
 
+  SyncAvailableGPUThread();
+
   // allows passing NULL rdcfile as 'I don't care, give me a proxy driver of any type'
   if(rdc == NULL)
   {
     if(!m_ReplayDriverProviders.empty())
-      return m_ReplayDriverProviders.begin()->second(NULL, driver);
+      return m_ReplayDriverProviders.begin()->second(NULL, opts, driver);
 
     RDCERR("Request for proxy replay device, but no replay providers are available.");
     return ReplayStatus::InternalError;
@@ -1145,27 +1411,30 @@ ReplayStatus RenderDoc::CreateReplayDriver(RDCFile *rdc, IReplayDriver **driver)
     return IMG_CreateReplayDevice(rdc, driver);
 
   if(m_ReplayDriverProviders.find(driverType) != m_ReplayDriverProviders.end())
-    return m_ReplayDriverProviders[driverType](rdc, driver);
+    return m_ReplayDriverProviders[driverType](rdc, opts, driver);
 
   RDCERR("Unsupported replay driver requested: %s", ToStr(driverType).c_str());
   return ReplayStatus::APIUnsupported;
 }
 
-ReplayStatus RenderDoc::CreateRemoteDriver(RDCFile *rdc, IRemoteDriver **driver)
+ReplayStatus RenderDoc::CreateRemoteDriver(RDCFile *rdc, const ReplayOptions &opts,
+                                           IRemoteDriver **driver)
 {
   if(rdc == NULL || driver == NULL)
     return ReplayStatus::InternalError;
 
+  SyncAvailableGPUThread();
+
   RDCDriver driverType = rdc->GetDriver();
 
   if(m_RemoteDriverProviders.find(driverType) != m_RemoteDriverProviders.end())
-    return m_RemoteDriverProviders[driverType](rdc, driver);
+    return m_RemoteDriverProviders[driverType](rdc, opts, driver);
 
   // replay drivers are remote drivers, fall back and try them
   if(m_ReplayDriverProviders.find(driverType) != m_ReplayDriverProviders.end())
   {
     IReplayDriver *dr = NULL;
-    ReplayStatus status = m_ReplayDriverProviders[driverType](rdc, &dr);
+    ReplayStatus status = m_ReplayDriverProviders[driverType](rdc, opts, &dr);
 
     if(status == ReplayStatus::Succeeded)
       *driver = (IRemoteDriver *)dr;
@@ -1222,17 +1491,17 @@ std::map<RDCDriver, bool> RenderDoc::GetActiveDrivers()
   return ret;
 }
 
-map<RDCDriver, string> RenderDoc::GetReplayDrivers()
+std::map<RDCDriver, rdcstr> RenderDoc::GetReplayDrivers()
 {
-  map<RDCDriver, string> ret;
+  std::map<RDCDriver, rdcstr> ret;
   for(auto it = m_ReplayDriverProviders.begin(); it != m_ReplayDriverProviders.end(); ++it)
     ret[it->first] = ToStr(it->first);
   return ret;
 }
 
-map<RDCDriver, string> RenderDoc::GetRemoteDrivers()
+std::map<RDCDriver, rdcstr> RenderDoc::GetRemoteDrivers()
 {
-  map<RDCDriver, string> ret;
+  std::map<RDCDriver, rdcstr> ret;
 
   for(auto it = m_RemoteDriverProviders.begin(); it != m_RemoteDriverProviders.end(); ++it)
     ret[it->first] = ToStr(it->first);
@@ -1381,36 +1650,79 @@ void RenderDoc::FinishCaptureWriting(RDCFile *rdc, uint32_t frameNumber)
   RenderDoc::Inst().SetProgress(CaptureProgress::FileWriting, 1.0f);
 }
 
+void RenderDoc::AddChildProcess(uint32_t pid, uint32_t ident)
+{
+  SCOPED_LOCK(m_ChildLock);
+  m_Children.push_back(make_rdcpair(pid, ident));
+}
+
+rdcarray<rdcpair<uint32_t, uint32_t> > RenderDoc::GetChildProcesses()
+{
+  SCOPED_LOCK(m_ChildLock);
+  return m_Children;
+}
+
+rdcarray<CaptureData> RenderDoc::GetCaptures()
+{
+  SCOPED_LOCK(m_CaptureLock);
+  return m_Captures;
+}
+
+void RenderDoc::MarkCaptureRetrieved(uint32_t idx)
+{
+  SCOPED_LOCK(m_CaptureLock);
+  if(idx < m_Captures.size())
+  {
+    m_Captures[idx].retrieved = true;
+  }
+}
+
 void RenderDoc::AddDeviceFrameCapturer(void *dev, IFrameCapturer *cap)
 {
+  if(IsReplayApp())
+    return;
+
   if(dev == NULL || cap == NULL)
   {
-    RDCERR("Invalid FrameCapturer combination: %#p / %#p", dev, cap);
+    RDCERR("Invalid FrameCapturer %#p for device: %#p", cap, dev);
     return;
   }
 
+  RDCLOG("Adding %s device frame capturer for %#p", ToStr(cap->GetFrameCaptureDriver()).c_str(), dev);
+
   m_DeviceFrameCapturers[dev] = cap;
 }
 
 void RenderDoc::RemoveDeviceFrameCapturer(void *dev)
 {
+  if(IsReplayApp())
+    return;
+
   if(dev == NULL)
   {
     RDCERR("Invalid device pointer: %#p", dev);
     return;
   }
 
+  RDCLOG("Removing device frame capturer for %#p", dev);
+
   m_DeviceFrameCapturers.erase(dev);
 }
 
 void RenderDoc::AddFrameCapturer(void *dev, void *wnd, IFrameCapturer *cap)
 {
+  if(IsReplayApp())
+    return;
+
   if(dev == NULL || wnd == NULL || cap == NULL)
   {
-    RDCERR("Invalid FrameCapturer combination: %#p / %#p", wnd, cap);
+    RDCERR("Invalid FrameCapturer %#p for combination: %#p / %#p", cap, dev, wnd);
     return;
   }
 
+  RDCLOG("Adding %s frame capturer for %#p / %#p", ToStr(cap->GetFrameCaptureDriver()).c_str(), dev,
+         wnd);
+
   DeviceWnd dw(dev, wnd);
 
   auto it = m_WindowFrameCapturers.find(dw);
@@ -1433,8 +1745,13 @@ void RenderDoc::AddFrameCapturer(void *dev, void *wnd, IFrameCapturer *cap)
 
 void RenderDoc::RemoveFrameCapturer(void *dev, void *wnd)
 {
+  if(IsReplayApp())
+    return;
+
   DeviceWnd dw(dev, wnd);
 
+  RDCLOG("Removing frame capturer for %#p / %#p", dev, wnd);
+
   auto it = m_WindowFrameCapturers.find(dw);
   if(it != m_WindowFrameCapturers.end())
   {
@@ -1516,4 +1833,4 @@ TEST_CASE("Check ResourceId tostr", "[tostr]")
   CHECK(ToStr(*u.id) == "ResourceId::1311768465173141112");
 }
 
-#endif
\ No newline at end of file
+#endif
