@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,28 +27,36 @@
 
 #include <stdint.h>
 #include <map>
-#include <set>
-#include <string>
-#include <utility>
-#include <vector>
 #include "api/app/renderdoc_app.h"
-#include "api/replay/renderdoc_replay.h"
-#include "common/threading.h"
+#include "api/replay/apidefs.h"
+#include "api/replay/capture_options.h"
+#include "api/replay/control_types.h"
+#include "api/replay/stringise.h"
 #include "common/timing.h"
-#include "maths/vec.h"
 #include "os/os_specific.h"
 
-using std::string;
-using std::vector;
-using std::map;
-using std::pair;
-using std::set;
-
 class Chunk;
 struct RDCThumb;
+struct ReplayOptions;
 
 // not provided by tinyexr, just do by hand
 bool is_exr_file(FILE *f);
+void LogReplayOptions(const ReplayOptions &opts);
+
+enum class RDCDriver : uint32_t;
+
+class IRemoteDriver;
+class IReplayDriver;
+
+class StreamReader;
+class RDCFile;
+struct SDFile;
+enum class VulkanLayerFlags : uint32_t;
+
+namespace Callstack
+{
+class StackResolver;
+}
 
 struct ICrashHandler
 {
@@ -59,11 +67,14 @@ struct ICrashHandler
 
 struct IFrameCapturer
 {
+  virtual RDCDriver GetFrameCaptureDriver() = 0;
   virtual void StartFrameCapture(void *dev, void *wnd) = 0;
   virtual bool EndFrameCapture(void *dev, void *wnd) = 0;
   virtual bool DiscardFrameCapture(void *dev, void *wnd) = 0;
 };
 
+struct IDeviceProtocolHandler;
+
 // In most cases you don't need to check these individually, use the utility functions below
 // to determine if you're in a capture or replay state. There are utility functions for each
 // state as well.
@@ -161,7 +172,7 @@ enum class SystemChunk : uint32_t
 
 DECLARE_REFLECTION_ENUM(SystemChunk);
 
-enum class RDCDriver
+enum class RDCDriver : uint32_t
 {
   Unknown = 0,
   D3D11 = 1,
@@ -190,15 +201,6 @@ enum class RDCDriver
 
 DECLARE_REFLECTION_ENUM(RDCDriver);
 
-namespace DXBC
-{
-class DXBCFile;
-}
-namespace Callstack
-{
-class StackResolver;
-}
-
 enum ReplayLogType
 {
   eReplay_Full,
@@ -222,11 +224,12 @@ ITERABLE_OPERATORS(VendorExtensions);
 
 struct CaptureData
 {
-  CaptureData(string p, uint64_t t, RDCDriver d, uint32_t f)
+  CaptureData() : timestamp(0), driver(RDCDriver::Unknown), frameNumber(0), retrieved(false) {}
+  CaptureData(rdcstr p, uint64_t t, RDCDriver d, uint32_t f)
       : path(p), timestamp(t), driver(d), frameNumber(f), retrieved(false)
   {
   }
-  string path;
+  rdcstr path;
   uint64_t timestamp;
   RDCDriver driver;
   uint32_t frameNumber;
@@ -318,14 +321,10 @@ inline constexpr float FakeProgress(uint32_t x, uint32_t maxX)
   return 1.0f - (1.0f / (x * (4.0f / float(maxX)) + 1));
 }
 
-class IRemoteDriver;
-class IReplayDriver;
-
-class StreamReader;
-class RDCFile;
-
-typedef ReplayStatus (*RemoteDriverProvider)(RDCFile *rdc, IRemoteDriver **driver);
-typedef ReplayStatus (*ReplayDriverProvider)(RDCFile *rdc, IReplayDriver **driver);
+typedef ReplayStatus (*RemoteDriverProvider)(RDCFile *rdc, const ReplayOptions &opts,
+                                             IRemoteDriver **driver);
+typedef ReplayStatus (*ReplayDriverProvider)(RDCFile *rdc, const ReplayOptions &opts,
+                                             IReplayDriver **driver);
 
 typedef void (*StructuredProcessor)(RDCFile *rdc, SDFile &structData);
 
@@ -334,9 +333,10 @@ typedef ReplayStatus (*CaptureImporter)(const char *filename, StreamReader &read
 typedef ReplayStatus (*CaptureExporter)(const char *filename, const RDCFile &rdc,
                                         const SDFile &structData,
                                         RENDERDOC_ProgressCallback progress);
+typedef IDeviceProtocolHandler *(*ProtocolHandler)();
 
-typedef bool (*VulkanLayerCheck)(VulkanLayerFlags &flags, std::vector<std::string> &myJSONs,
-                                 std::vector<std::string> &otherJSONs);
+typedef bool (*VulkanLayerCheck)(VulkanLayerFlags &flags, rdcarray<rdcstr> &myJSONs,
+                                 rdcarray<rdcstr> &otherJSONs);
 typedef void (*VulkanLayerInstall)(bool systemLevel);
 
 typedef void (*ShutdownFunction)();
@@ -409,14 +409,15 @@ public:
   void Shutdown();
 
   uint64_t GetMicrosecondTimestamp() { return uint64_t(m_Timer.GetMicroseconds()); }
-  const GlobalEnvironment GetGlobalEnvironment() { return m_GlobalEnv; }
-  void ProcessGlobalEnvironment(GlobalEnvironment env, const std::vector<std::string> &args);
+  const GlobalEnvironment &GetGlobalEnvironment() { return m_GlobalEnv; }
+  void ProcessGlobalEnvironment(GlobalEnvironment env, const rdcarray<rdcstr> &args);
 
-  void RegisterShutdownFunction(ShutdownFunction func) { m_ShutdownFunctions.insert(func); }
+  int32_t GetForwardedPortSlot() { return Atomic::Inc32(&m_PortSlot); }
+  void RegisterShutdownFunction(ShutdownFunction func);
   void SetReplayApp(bool replay) { m_Replay = replay; }
   bool IsReplayApp() const { return m_Replay; }
-  const string &GetConfigSetting(string name) { return m_ConfigSettings[name]; }
-  void SetConfigSetting(string name, string value) { m_ConfigSettings[name] = value; }
+  const rdcstr &GetConfigSetting(rdcstr name) { return m_ConfigSettings[name]; }
+  void SetConfigSetting(rdcstr name, rdcstr value) { m_ConfigSettings[name] = value; }
   void BecomeRemoteServer(const char *listenhost, uint16_t port, RENDERDOC_KillCallback killReplay,
                           RENDERDOC_PreviewWindowCallback previewWindow);
 
@@ -437,31 +438,12 @@ public:
   RDCFile *CreateRDC(RDCDriver driver, uint32_t frameNum, const FramePixels &fp);
   void FinishCaptureWriting(RDCFile *rdc, uint32_t frameNumber);
 
-  void AddChildProcess(uint32_t pid, uint32_t ident)
-  {
-    SCOPED_LOCK(m_ChildLock);
-    m_Children.push_back(std::make_pair(pid, ident));
-  }
-  vector<pair<uint32_t, uint32_t> > GetChildProcesses()
-  {
-    SCOPED_LOCK(m_ChildLock);
-    return m_Children;
-  }
+  void AddChildProcess(uint32_t pid, uint32_t ident);
+  rdcarray<rdcpair<uint32_t, uint32_t> > GetChildProcesses();
 
-  vector<CaptureData> GetCaptures()
-  {
-    SCOPED_LOCK(m_CaptureLock);
-    return m_Captures;
-  }
+  rdcarray<CaptureData> GetCaptures();
 
-  void MarkCaptureRetrieved(uint32_t idx)
-  {
-    SCOPED_LOCK(m_CaptureLock);
-    if(idx < m_Captures.size())
-    {
-      m_Captures[idx].retrieved = true;
-    }
-  }
+  void MarkCaptureRetrieved(uint32_t idx);
 
   void RegisterReplayProvider(RDCDriver driver, ReplayDriverProvider provider);
   void RegisterRemoteProvider(RDCDriver driver, RemoteDriverProvider provider);
@@ -471,18 +453,23 @@ public:
   void RegisterCaptureExporter(CaptureExporter exporter, CaptureFileFormat description);
   void RegisterCaptureImportExporter(CaptureImporter importer, CaptureExporter exporter,
                                      CaptureFileFormat description);
+  void RegisterDeviceProtocol(const rdcstr &protocol, ProtocolHandler handler);
 
   StructuredProcessor GetStructuredProcessor(RDCDriver driver);
 
   CaptureExporter GetCaptureExporter(const char *filetype);
   CaptureImporter GetCaptureImporter(const char *filetype);
 
-  std::vector<CaptureFileFormat> GetCaptureFileFormats();
+  rdcarray<rdcstr> GetSupportedDeviceProtocols();
+  IDeviceProtocolHandler *GetDeviceProtocol(const rdcstr &protocol);
+
+  rdcarray<CaptureFileFormat> GetCaptureFileFormats();
+  rdcarray<GPUDevice> GetAvailableGPUs();
 
   void SetVulkanLayerCheck(VulkanLayerCheck callback) { m_VulkanCheck = callback; }
   void SetVulkanLayerInstall(VulkanLayerInstall callback) { m_VulkanInstall = callback; }
-  bool NeedVulkanLayerRegistration(VulkanLayerFlags &flags, std::vector<std::string> &myJSONs,
-                                   std::vector<std::string> &otherJSONs)
+  bool NeedVulkanLayerRegistration(VulkanLayerFlags &flags, rdcarray<rdcstr> &myJSONs,
+                                   rdcarray<rdcstr> &otherJSONs)
   {
     if(m_VulkanCheck)
       return m_VulkanCheck(flags, myJSONs, otherJSONs);
@@ -496,20 +483,20 @@ public:
       m_VulkanInstall(systemLevel);
   }
 
-  Vec4f LightCheckerboardColor() { return m_LightChecker; }
-  Vec4f DarkCheckerboardColor() { return m_DarkChecker; }
-  void SetLightCheckerboardColor(const Vec4f &col) { m_LightChecker = col; }
-  void SetDarkCheckerboardColor(const Vec4f &col) { m_DarkChecker = col; }
+  FloatVector LightCheckerboardColor() { return m_LightChecker; }
+  FloatVector DarkCheckerboardColor() { return m_DarkChecker; }
+  void SetLightCheckerboardColor(const FloatVector &col) { m_LightChecker = col; }
+  void SetDarkCheckerboardColor(const FloatVector &col) { m_DarkChecker = col; }
   bool IsDarkTheme() { return m_DarkTheme; }
   void SetDarkTheme(bool dark) { m_DarkTheme = dark; }
   ReplayStatus CreateProxyReplayDriver(RDCDriver proxyDriver, IReplayDriver **driver);
-  ReplayStatus CreateReplayDriver(RDCFile *rdc, IReplayDriver **driver);
-  ReplayStatus CreateRemoteDriver(RDCFile *rdc, IRemoteDriver **driver);
+  ReplayStatus CreateReplayDriver(RDCFile *rdc, const ReplayOptions &opts, IReplayDriver **driver);
+  ReplayStatus CreateRemoteDriver(RDCFile *rdc, const ReplayOptions &opts, IRemoteDriver **driver);
 
   bool HasReplaySupport(RDCDriver driverType);
 
-  map<RDCDriver, string> GetReplayDrivers();
-  map<RDCDriver, string> GetRemoteDrivers();
+  std::map<RDCDriver, rdcstr> GetReplayDrivers();
+  std::map<RDCDriver, rdcstr> GetRemoteDrivers();
 
   bool HasReplayDriver(RDCDriver driver) const;
   bool HasRemoteDriver(RDCDriver driver) const;
@@ -519,7 +506,7 @@ public:
 
   uint32_t GetTargetControlIdent() const { return m_RemoteIdent; }
   bool IsTargetControlConnected();
-  string GetTargetControlUsername();
+  rdcstr GetTargetControlUsername();
 
   void Tick();
 
@@ -553,7 +540,7 @@ public:
   void TriggerCapture(uint32_t numFrames) { m_Cap = numFrames; }
   uint32_t GetOverlayBits() { return m_Overlay; }
   void MaskOverlayBits(uint32_t And, uint32_t Or) { m_Overlay = (m_Overlay & And) | Or; }
-  void QueueCapture(uint32_t frameNumber) { m_QueuedFrameCaptures.insert(frameNumber); }
+  void QueueCapture(uint32_t frameNumber);
   void SetFocusKeys(RENDERDOC_InputButton *keys, int num)
   {
     m_FocusKeys.resize(num);
@@ -567,8 +554,8 @@ public:
       m_CaptureKeys[i] = keys[i];
   }
 
-  const vector<RENDERDOC_InputButton> &GetFocusKeys() { return m_FocusKeys; }
-  const vector<RENDERDOC_InputButton> &GetCaptureKeys() { return m_CaptureKeys; }
+  const rdcarray<RENDERDOC_InputButton> &GetFocusKeys() { return m_FocusKeys; }
+  const rdcarray<RENDERDOC_InputButton> &GetCaptureKeys() { return m_CaptureKeys; }
   bool ShouldTriggerCapture(uint32_t frameNumber);
 
   enum
@@ -577,7 +564,7 @@ public:
     eOverlay_CaptureDisabled = 0x2,
   };
 
-  string GetOverlayText(RDCDriver driver, uint32_t frameNumber, int flags);
+  rdcstr GetOverlayText(RDCDriver driver, uint32_t frameNumber, int flags);
 
   void CycleActiveWindow();
   uint32_t GetCapturableWindowCount() { return (uint32_t)m_WindowFrameCapturers.size(); }
@@ -585,28 +572,32 @@ private:
   RenderDoc();
   ~RenderDoc();
 
+  void SyncAvailableGPUThread();
+
   static RenderDoc *m_Inst;
 
   bool m_Replay;
 
   uint32_t m_Cap;
 
-  vector<RENDERDOC_InputButton> m_FocusKeys;
-  vector<RENDERDOC_InputButton> m_CaptureKeys;
+  rdcarray<RENDERDOC_InputButton> m_FocusKeys;
+  rdcarray<RENDERDOC_InputButton> m_CaptureKeys;
 
   GlobalEnvironment m_GlobalEnv;
 
+  int32_t m_PortSlot = 0;
+
   FrameTimer m_FrameTimer;
 
-  string m_LoggingFilename;
+  rdcstr m_LoggingFilename;
 
-  string m_Target;
-  string m_CaptureFileTemplate;
-  string m_CurrentLogFile;
+  rdcstr m_Target;
+  rdcstr m_CaptureFileTemplate;
+  rdcstr m_CurrentLogFile;
   CaptureOptions m_Options;
   uint32_t m_Overlay;
 
-  set<uint32_t> m_QueuedFrameCaptures;
+  rdcarray<uint32_t> m_QueuedFrameCaptures;
 
   uint32_t m_RemoteIdent;
   Threading::ThreadHandle m_RemoteThread;
@@ -615,29 +606,34 @@ private:
   Threading::CriticalSection m_DriverLock;
   std::map<RDCDriver, uint64_t> m_ActiveDrivers;
 
-  std::map<std::string, RENDERDOC_ProgressCallback> m_ProgressCallbacks;
+  Threading::ThreadHandle m_AvailableGPUThread = 0;
+  rdcarray<GPUDevice> m_AvailableGPUs;
+
+  std::map<rdcstr, RENDERDOC_ProgressCallback> m_ProgressCallbacks;
 
   Threading::CriticalSection m_CaptureLock;
-  vector<CaptureData> m_Captures;
+  rdcarray<CaptureData> m_Captures;
 
   Threading::CriticalSection m_ChildLock;
-  vector<pair<uint32_t, uint32_t> > m_Children;
+  rdcarray<rdcpair<uint32_t, uint32_t> > m_Children;
 
-  map<string, string> m_ConfigSettings;
+  std::map<rdcstr, rdcstr> m_ConfigSettings;
 
-  map<RDCDriver, ReplayDriverProvider> m_ReplayDriverProviders;
-  map<RDCDriver, RemoteDriverProvider> m_RemoteDriverProviders;
+  std::map<RDCDriver, ReplayDriverProvider> m_ReplayDriverProviders;
+  std::map<RDCDriver, RemoteDriverProvider> m_RemoteDriverProviders;
 
   std::map<RDCDriver, StructuredProcessor> m_StructProcesssors;
 
-  std::vector<CaptureFileFormat> m_ImportExportFormats;
-  std::map<std::string, CaptureImporter> m_Importers;
-  std::map<std::string, CaptureExporter> m_Exporters;
+  rdcarray<CaptureFileFormat> m_ImportExportFormats;
+  std::map<rdcstr, CaptureImporter> m_Importers;
+  std::map<rdcstr, CaptureExporter> m_Exporters;
+
+  std::map<rdcstr, ProtocolHandler> m_Protocols;
 
   VulkanLayerCheck m_VulkanCheck;
   VulkanLayerInstall m_VulkanInstall;
 
-  set<ShutdownFunction> m_ShutdownFunctions;
+  rdcarray<ShutdownFunction> m_ShutdownFunctions;
 
   struct FrameCap
   {
@@ -673,24 +669,24 @@ private:
     }
   };
 
-  Vec4f m_LightChecker = Vec4f(0.81f, 0.81f, 0.81f, 1.0f);
-  Vec4f m_DarkChecker = Vec4f(0.57f, 0.57f, 0.57f, 1.0f);
+  FloatVector m_LightChecker = FloatVector(0.81f, 0.81f, 0.81f, 1.0f);
+  FloatVector m_DarkChecker = FloatVector(0.57f, 0.57f, 0.57f, 1.0f);
   bool m_DarkTheme = false;
 
   int m_CapturesActive;
 
-  map<DeviceWnd, FrameCap> m_WindowFrameCapturers;
+  std::map<DeviceWnd, FrameCap> m_WindowFrameCapturers;
   DeviceWnd m_ActiveWindow;
-  map<void *, IFrameCapturer *> m_DeviceFrameCapturers;
+  std::map<void *, IFrameCapturer *> m_DeviceFrameCapturers;
 
   IFrameCapturer *MatchFrameCapturer(void *dev, void *wnd);
 
-  bool m_VendorExts[ENUM_ARRAY_SIZE(VendorExtensions)] = {};
+  bool m_VendorExts[arraydim<VendorExtensions>()] = {};
 
   volatile bool m_TargetControlThreadShutdown;
   volatile bool m_ControlClientThreadShutdown;
   Threading::CriticalSection m_SingleClientLock;
-  string m_SingleClientName;
+  rdcstr m_SingleClientName;
 
   PerformanceTimer m_Timer;
 
@@ -731,4 +727,12 @@ struct ConversionRegistration
   {
     RenderDoc::Inst().RegisterCaptureExporter(exporter, description);
   }
-};
\ No newline at end of file
+};
+
+struct DeviceProtocolRegistration
+{
+  DeviceProtocolRegistration(const rdcstr &protocol, ProtocolHandler handler)
+  {
+    RenderDoc::Inst().RegisterDeviceProtocol(protocol, handler);
+  }
+};
