@@ -1,36 +1,37 @@
 /******************************************************************************
-* The MIT License (MIT)
-*
-* Copyright (c) 2019 Baldur Karlsson
-*
-* Permission is hereby granted, free of charge, to any person obtaining a copy
-* of this software and associated documentation files (the "Software"), to deal
-* in the Software without restriction, including without limitation the rights
-* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-* copies of the Software, and to permit persons to whom the Software is
-* furnished to do so, subject to the following conditions:
-*
-* The above copyright notice and this permission notice shall be included in
-* all copies or substantial portions of the Software.
-*
-* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-* THE SOFTWARE.
-******************************************************************************/
+ * The MIT License (MIT)
+ *
+ * Copyright (c) 2019-2020 Baldur Karlsson
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
+ ******************************************************************************/
 
-#include "intervals.h"
 #include "common/globalconfig.h"
 
 #if ENABLED(ENABLE_UNIT_TESTS)
 
+#include "api/replay/rdcarray.h"
+#include "intervals.h"
+
 #include "3rdparty/catch/catch.hpp"
 
 #include <stdint.h>
-#include <vector>
 
 struct Interval
 {
@@ -39,7 +40,7 @@ struct Interval
   uint64_t end;
 };
 
-void check_intervals(Intervals<uint64_t> &value, const std::vector<Interval> &expected)
+void check_intervals(Intervals<uint64_t> &value, const rdcarray<Interval> &expected)
 {
   auto i = value.begin();
   auto j = expected.begin();
@@ -53,7 +54,7 @@ void check_intervals(Intervals<uint64_t> &value, const std::vector<Interval> &ex
   CHECK((j == expected.end()));
 }
 
-Intervals<uint64_t> make_intervals(const std::vector<Interval> &intervals)
+Intervals<uint64_t> make_intervals(const rdcarray<Interval> &intervals)
 {
   Intervals<uint64_t> res;
   for(auto i = intervals.begin(); i != intervals.end(); i++)
@@ -440,4 +441,4 @@ TEST_CASE("Test Intervals type", "[intervals]")
   };
 };
 
-#endif    // ENABLED(ENABLE_UNIT_TESTS)
\ No newline at end of file
+#endif    // ENABLED(ENABLE_UNIT_TESTS)
