@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,6 +22,6 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include <string>
+#include "api/replay/rdcstr.h"
 
-std::string LocatePluginFile(const std::string &path, const std::string &fileName);
\ No newline at end of file
+rdcstr LocatePluginFile(const rdcstr &path, const rdcstr &fileName);
