@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -54,7 +54,7 @@ void SetReplayResourceIDs()
 INSTANTIATE_SERIALISE_TYPE(ResourceManagerInternal::WrittenRecord);
 
 template <>
-std::string DoStringise(const FrameRefType &el)
+rdcstr DoStringise(const FrameRefType &el)
 {
   BEGIN_ENUM_STRINGISE(FrameRefType)
   {
@@ -69,9 +69,6 @@ std::string DoStringise(const FrameRefType &el)
 
 FrameRefType ComposeFrameRefs(FrameRefType first, FrameRefType second)
 {
-  RDCASSERT(eFrameRef_Minimum <= first && first <= eFrameRef_Maximum);
-  RDCASSERT(eFrameRef_Minimum <= second && second <= eFrameRef_Maximum);
-
   switch(first)
   {
     case eFrameRef_None:
@@ -86,21 +83,12 @@ FrameRefType ComposeFrameRefs(FrameRefType first, FrameRefType second)
         return second;
 
     case eFrameRef_Read:
-      switch(second)
-      {
-        case eFrameRef_None:
-        case eFrameRef_Read:
-          // Only referenced as `Read` (and possibly `None`)
-          return eFrameRef_Read;
-
-        case eFrameRef_PartialWrite:
-        case eFrameRef_CompleteWrite:
-        case eFrameRef_ReadBeforeWrite:
-          // First read, and then written
-          return eFrameRef_ReadBeforeWrite;
-
-        default: RDCERR("Unknown FrameRefType: %d", second); return eFrameRef_Maximum;
-      }
+    case eFrameRef_WriteBeforeRead:
+      if(IncludesWrite(second))
+        // `first` reads before `second` writes
+        return eFrameRef_ReadBeforeWrite;
+      else
+        return first;
 
     case eFrameRef_CompleteWrite:
     case eFrameRef_ReadBeforeWrite:
@@ -114,31 +102,50 @@ FrameRefType ComposeFrameRefs(FrameRefType first, FrameRefType second)
 
 FrameRefType ComposeFrameRefsUnordered(FrameRefType first, FrameRefType second)
 {
-  RDCASSERT(eFrameRef_Minimum <= first && first <= eFrameRef_Maximum);
-  RDCASSERT(eFrameRef_Minimum <= second && second <= eFrameRef_Maximum);
-
-  // The order of the reference types is irrelevant, so put them in a
-  // consistent order (`first >= second`) to reduce the number of cases to
-  // consider.
-  if(first < second)
-    std::swap(first, second);
-
-  if(first == eFrameRef_Read &&
-     (second == eFrameRef_PartialWrite || second == eFrameRef_CompleteWrite))
-    // The resource is referenced both read and write/clear;
-    // We don't know whether the read or write/clear occurs first;
-    // if the write happens first, the final state would be Read or Clear;
-    // if the read happens first, the final state would be ReadBeforeWrite.
-    // We conservatively return ReadBeforeWrite, because this will force the
-    // resource to be reset before each frame when replaying.
+  if((IncludesRead(first) && IncludesWrite(second)) || (IncludesRead(second) && IncludesWrite(first)))
+    // There is a way to order these references so that the resource is read
+    // and then written. Since read-before-write is the worst case in terms of
+    // reset requirements, we conservatively assume this is the case.
     return eFrameRef_ReadBeforeWrite;
+  else
+    // Otherwise either:
+    // - first and second are each Read or None, or
+    // - first and second are each CompleteWrite, PartialWrite or None.
+    // In either case, Compose(first,second) = Compose(second,first) = max(first,second).
+    return RDCMAX(first, second);
+}
 
-  // In all other cases, we just return the more conservative reference type--
-  // i.e. the reference type with the strongest (re)initialization
-  // requirements for replay. Because larger values in the `FrameRefType` have
-  // stronger (re)initialization requirements, this is simply the maximum
-  // reference type; note that `first >= second` by the earlier swap.
-  return first;
+FrameRefType ComposeFrameRefsDisjoint(FrameRefType x, FrameRefType y)
+{
+  if(x == eFrameRef_ReadBeforeWrite || y == eFrameRef_ReadBeforeWrite)
+    // If any subresource is `ReadBeforeWrite`, then the whole resource is.
+    return eFrameRef_ReadBeforeWrite;
+  else
+    // For all other cases, just return the larger value.
+    return RDCMAX(x, y);
+}
+
+bool IncludesRead(FrameRefType refType)
+{
+  switch(refType)
+  {
+    case eFrameRef_Read:
+    case eFrameRef_WriteBeforeRead:
+    case eFrameRef_ReadBeforeWrite: return true;
+    default: return false;
+  }
+}
+
+bool IncludesWrite(FrameRefType refType)
+{
+  switch(refType)
+  {
+    case eFrameRef_PartialWrite:
+    case eFrameRef_CompleteWrite:
+    case eFrameRef_WriteBeforeRead:
+    case eFrameRef_ReadBeforeWrite: return true;
+    default: return false;
+  }
 }
 
 bool IsDirtyFrameRef(FrameRefType refType)
@@ -170,10 +177,7 @@ void ResourceRecord::Delete(ResourceRecordHandler *mgr)
     DeleteChunks();
 
     if(ResID != ResourceId())
-    {
-      mgr->MarkCleanResource(ResID);
       mgr->RemoveResourceRecord(ResID);
-    }
 
     mgr->DestroyResourceRecord(this);
   }
