@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,15 +27,12 @@
 
 #include <map>
 #include <set>
-#include "api/replay/renderdoc_replay.h"
+#include "api/replay/resourceid.h"
 #include "common/threading.h"
 #include "core/core.h"
 #include "os/os_specific.h"
 #include "serialise/serialiser.h"
 
-using std::set;
-using std::map;
-
 // In what way (read, write, etc) was a resource referenced in a frame -
 // used to determine if initial contents are needed and to what degree.
 // These values are used both as states (representing the cumulative previous
@@ -43,29 +40,33 @@ using std::map;
 // command, modifying the state). This state machine is illustrated below,
 // with states represented in caps, and transitions in lower case.
 //
-//        +------------ NONE -------------+
-//        |              |                |
-//       read          write            clear
-//        |              |                |
-//        V              V                V
-//      READ <--read-- WRITE --clear--> CLEAR
-//        |
-//   write/clear
-//        |
-//        V
-//  READBEFOREWRITE
+//         +------------------ NONE -----------------------------+
+//         |                    |                                |
+//        read            partialWrite                     completeWrite
+//         |                    |                                |
+//         V                    V                                V
+//       READ             PARTIAL_WRITE --completeWrite--> COMPLETE_WRITE
+//         |                    |
+//         |                  read
+//       write                  |
+//         |                    V
+//         |            WRITE_BEFORE_READ
+//         V                    |
+//  READ_BEFORE_WRITE <--write--+
 //
 // Note:
 //  * All resources begin implicitly in the None state.
+//  * The transitions labeled "write" correspond to either PartialWrite or
+//    CompleteWrite (e.g. in the READ state, either a PartialWrite or a
+//    CompleteWrite moves to the READ_BEFORE_WRITE state).
 //  * The state transitions for ReadBeforeWrite are simply the composition of
 //    the transition for read, followed by the transition for write (e.g.
-//    ReadBeforeWrite moves from NONE state to READBEFOREWRITE state).
-//  * All other transitions (excluding ReadBeforeWrite) that are not explicitly
-//    shown leave the state unchanged (e.g. a read in the CLEAR state remains
-//    in the CLEAR state).
-//
-// The enum values are ordered so that larger values correspond to greater
-// requirements for (re)initialization of the resource during replay.
+//    ReadBeforeWrite moves from NONE state to READBEFOREWRITE state);
+//    similarly, the state transitions for WriteBeforeRead are the composition
+//    of the transition for write, followed by the transition for read.
+//  * All other transitions (excluding ReadBeforeWrite and WriteBeforeRead)
+//    that are not explicitly shown leave the state unchanged (e.g. a read in
+//    the COMPLETE_WRITE state remains in the COMPLETE_WRITE state).
 enum FrameRefType
 {
   // Initial state, no reads or writes
@@ -95,10 +96,25 @@ enum FrameRefType
   // initial contents; therefore, the initial contents will need to be reset
   // before each time we replay the frame.
   eFrameRef_ReadBeforeWrite = 4,
+
+  // Partial write followed by read;
+  // For the purpose of correct replay, this is equivalent to `Read`. However,
+  // if this resource is inspected by the user before the write, the future
+  // read could, incorrectly, be observed. This is because read-only resources
+  // are not reset, so the write from the previous replay may still be present.
+  eFrameRef_WriteBeforeRead = 5,
 };
 
+bool IncludesRead(FrameRefType refType);
+
+bool IncludesWrite(FrameRefType refType);
+
 const FrameRefType eFrameRef_Minimum = eFrameRef_None;
-const FrameRefType eFrameRef_Maximum = eFrameRef_ReadBeforeWrite;
+const FrameRefType eFrameRef_Maximum = eFrameRef_WriteBeforeRead;
+
+// Threshold value for resource "age", i.e. how long it wasn't
+// referred with the any write reference.
+const double PERSISTENT_RESOURCE_AGE = 3000;
 
 DECLARE_REFLECTION_ENUM(FrameRefType);
 
@@ -114,6 +130,11 @@ FrameRefType ComposeFrameRefs(FrameRefType first, FrameRefType second);
 // reset for replay.
 FrameRefType ComposeFrameRefsUnordered(FrameRefType first, FrameRefType second);
 
+// Compose frame refs for disjoint subresources.
+// This is used to compute the overall frame ref for images/memory from the
+// frame refs of their subresources.
+FrameRefType ComposeFrameRefsDisjoint(FrameRefType x, FrameRefType y);
+
 bool IsDirtyFrameRef(FrameRefType refType);
 
 // Captures the possible initialization/reset requirements for resources.
@@ -122,41 +143,97 @@ bool IsDirtyFrameRef(FrameRefType refType);
 // init/reset requirements.
 enum InitReqType
 {
-  // Initial contents of the resource are not used, and also not modified.
-  // No initialization/reset is required for correct replay, but may be helpful
-  // to avoid user confusion when analyzing the resource.
-  // Corresponds to `None` ref type.
+  // No initialization required.
   eInitReq_None,
 
-  // Initial contents of the resource are not used, but the contents are
-  // potentially modified during the frame.
-  // No initialization/reset is required for correct replay, but may be helpful
-  // to avoid user confusion when analyzing the resource.
-  // Corresponds to `PartialWrite` and `CompleteWrite` ref types.
+  // Initialize the resource by clearing.
   eInitReq_Clear,
 
-  // Initial contents of the resource are read, but not overwritten;
-  // the resource needs to be initialized before the first replay, but need not
-  // be reset before subsequent replays.
-  // Corresponds to `Read` ref type.
-  eInitReq_InitOnce,
+  // Initialize the resource by copying initial data.
+  eInitReq_Copy,
+};
 
-  // Initial contents of the resource are read, and later overwritten;
-  // the resource needs to be reset before each replay.
-  // Corresponds to `ReadBeforeWrite` ref type.
-  eInitReq_Reset,
+enum InitPolicy
+{
+  // Completely disable optimizations--copy initial data into every resource
+  // before every replay.
+  eInitPolicy_NoOpt,
+
+  // CopyAll--conservative policy which ensures each subresource begins each
+  // replay with the correct initial data.
+  //
+  // Initialization policy:
+  //   Copy initial data into each subresource
+  //
+  // Reset policy:
+  //   Copy initial data into each subresource which is written
+  eInitPolicy_CopyAll,
+
+  // ClearUnread--avoid copying initial data which is never read by the replay
+  // commands. A user inspecting a resource before it is written may observe
+  // cleared data, rather than the actual initial data.
+  //
+  // Initialization policy:
+  //   Copy initial data into each subresource that is read.
+  //   Clear each subresource that is not read.
+  //
+  // Reset policy:
+  //   Copy initial data into each subresource where the initial data is read
+  //   and then overwritten.
+  //   Clear each subresource which is written, but whose initial data is not read.
+  eInitPolicy_ClearUnread,
+
+  // Fastest--Initialize/reset as little as possible for correct replay.
+  // A user inspecting a resource before it is written may observe the data
+  // from a future write (from the previous replay).
+  //
+  // Initialization policy:
+  //   Copy initial data into each subresource that is read.
+  //   Clear each subresource that is not read.
+  //
+  // Reset policy:
+  //   Copy initial data into each subresource where the initial data is read
+  //   and then overwritten.
+  eInitPolicy_Fastest,
 };
 
 // Return the initialization/reset requirements for a FrameRefType
-inline InitReqType InitReq(FrameRefType refType)
+inline InitReqType InitReq(FrameRefType refType, InitPolicy policy, bool initialized)
 {
-  switch(refType)
+#define COPY_ONCE (initialized ? eInitReq_None : eInitReq_Copy)
+#define CLEAR_ONCE (initialized ? eInitReq_None : eInitReq_Clear)
+  switch(policy)
   {
-    case eFrameRef_None: return eInitReq_None;
-    case eFrameRef_Read: return eInitReq_InitOnce;
-    case eFrameRef_ReadBeforeWrite: return eInitReq_Reset;
-    default: return eInitReq_Clear;
+    case eInitPolicy_NoOpt: return eInitReq_Copy;
+    case eInitPolicy_CopyAll:
+      switch(refType)
+      {
+        case eFrameRef_None: return COPY_ONCE;
+        case eFrameRef_Read: return COPY_ONCE;
+        default: return eInitReq_Copy;
+      }
+    case eInitPolicy_ClearUnread:
+      switch(refType)
+      {
+        case eFrameRef_None: return CLEAR_ONCE;
+        case eFrameRef_Read: return COPY_ONCE;
+        case eFrameRef_ReadBeforeWrite: return eInitReq_Copy;
+        case eFrameRef_WriteBeforeRead: return eInitReq_Copy;
+        default: return eInitReq_Clear;
+      }
+    case eInitPolicy_Fastest:
+      switch(refType)
+      {
+        case eFrameRef_None: return CLEAR_ONCE;
+        case eFrameRef_Read: return COPY_ONCE;
+        case eFrameRef_ReadBeforeWrite: return eInitReq_Copy;
+        case eFrameRef_WriteBeforeRead: return COPY_ONCE;
+        default: return CLEAR_ONCE;
+      }
+    default: RDCERR("Unknown initialization policy (%d).", policy); return eInitReq_Copy;
   }
+#undef COPY_ONCE
+#undef CLEAR_ONCE
 }
 
 // handle marking a resource referenced for read or write and storing RAW access etc.
@@ -199,8 +276,6 @@ class ResourceRecordHandler
 {
 public:
   virtual void MarkDirtyResource(ResourceId id) = 0;
-  virtual void MarkCleanResource(ResourceId id) = 0;
-  virtual void MarkPendingDirty(ResourceId id) = 0;
   virtual void RemoveResourceRecord(ResourceId id) = 0;
   virtual void MarkResourceFrameReferenced(ResourceId id, FrameRefType refType) = 0;
   virtual void DestroyResourceRecord(ResourceRecord *record) = 0;
@@ -235,6 +310,9 @@ struct ResourceRecord
   ~ResourceRecord() { SAFE_DELETE(m_ChunkLock); }
   void AddParent(ResourceRecord *r)
   {
+    if(r == this)
+      return;
+
     if(Parents.find(r) == Parents.end())
     {
       r->AddRef();
@@ -263,7 +341,7 @@ struct ResourceRecord
   }
 
   void MarkDataUnwritten() { DataWritten = false; }
-  void Insert(map<int32_t, Chunk *> &recordlist)
+  void Insert(std::map<int32_t, Chunk *> &recordlist)
   {
     bool dataWritten = DataWritten;
 
@@ -292,14 +370,7 @@ struct ResourceRecord
   void RemoveChunk(Chunk *chunk)
   {
     LockChunks();
-    for(auto it = m_Chunks.begin(); it != m_Chunks.end(); ++it)
-    {
-      if(it->second == chunk)
-      {
-        m_Chunks.erase(it);
-        break;
-      }
-    }
+    m_Chunks.removeOneIf([chunk](const rdcpair<int32_t, Chunk *> &c) { return c.second == chunk; });
     UnlockChunks();
   }
 
@@ -308,7 +379,7 @@ struct ResourceRecord
     if(ID == 0)
       ID = GetID();
     LockChunks();
-    m_Chunks.push_back(std::make_pair(ID, chunk));
+    m_Chunks.push_back({ID, chunk});
     UnlockChunks();
   }
 
@@ -422,10 +493,10 @@ protected:
     return Atomic::Inc32(&globalIDCounter);
   }
 
-  std::vector<std::pair<int32_t, Chunk *>> m_Chunks;
+  rdcarray<rdcpair<int32_t, Chunk *>> m_Chunks;
   Threading::CriticalSection *m_ChunkLock;
 
-  map<ResourceId, FrameRefType> m_FrameRefs;
+  std::map<ResourceId, FrameRefType> m_FrameRefs;
 };
 
 template <typename Compose>
@@ -456,7 +527,7 @@ public:
   typedef typename Configuration::RecordType RecordType;
   typedef typename Configuration::InitialContentData InitialContentData;
 
-  ResourceManager();
+  ResourceManager(CaptureState &state);
   virtual ~ResourceManager();
 
   void Shutdown();
@@ -492,14 +563,6 @@ public:
   // of the frame.
   inline void MarkDirtyResource(ResourceId res);
 
-  // for use when we might be mid-capture, this will get flushed to dirty state before the
-  // next frame but is safe to use mid-capture
-  void MarkPendingDirty(ResourceId res);
-  void FlushPendingDirty();
-
-  // this can be used when the resource is cleared or similar and it's in a known state
-  void MarkCleanResource(ResourceId res);
-
   // returns if the resource has been marked as dirty
   bool IsResourceDirty(ResourceId res);
 
@@ -562,6 +625,18 @@ public:
   WrappedResourceType GetWrapper(RealResourceType real);
   void RemoveWrapper(RealResourceType real);
 
+  void Prepare_ResourceInitialStateIfNeeded(ResourceId id);
+  void Prepare_ResourceIfActivePostponed(ResourceId id);
+
+  void UpdateLastWriteTime(ResourceId id);
+  void ResetLastWriteTimes();
+  void ResetCaptureStartTime();
+
+  bool HasPersistentAge(ResourceId id);
+  bool IsResourcePostponed(ResourceId id);
+  bool IsResourcePersistent(ResourceId id);
+
+  virtual bool IsResourceTrackedForPersistency(const WrappedResourceType &res) { return false; }
 protected:
   friend InitialContentData;
   // 'interface' to implement by derived classes
@@ -569,16 +644,17 @@ protected:
 
   virtual bool ResourceTypeRelease(WrappedResourceType res) = 0;
 
-  virtual bool Force_InitialState(WrappedResourceType res, bool prepare) = 0;
-  virtual bool AllowDeletedResource_InitialState() { return false; }
-  virtual bool Need_InitialStateChunk(WrappedResourceType res) = 0;
+  virtual bool Need_InitialStateChunk(ResourceId id, const InitialContentData &initial)
+  {
+    return true;
+  }
   virtual bool Prepare_InitialState(WrappedResourceType res) = 0;
-  virtual uint32_t GetSize_InitialState(ResourceId id, WrappedResourceType res) = 0;
-  virtual bool Serialise_InitialState(WriteSerialiser &ser, ResourceId id,
-                                      WrappedResourceType res) = 0;
+  virtual uint64_t GetSize_InitialState(ResourceId id, const InitialContentData &initial) = 0;
+  virtual bool Serialise_InitialState(WriteSerialiser &ser, ResourceId id, RecordType *record,
+                                      const InitialContentData *initialData) = 0;
   virtual void Create_InitialState(ResourceId id, WrappedResourceType live, bool hasData) = 0;
-  virtual void Apply_InitialState(WrappedResourceType live, InitialContentData initial) = 0;
-  virtual std::vector<ResourceId> InitialContentResources();
+  virtual void Apply_InitialState(WrappedResourceType live, const InitialContentData &initial) = 0;
+  virtual rdcarray<ResourceId> InitialContentResources();
 
   // very coarse lock, protects EVERYTHING. This could certainly be improved and it may be a
   // bottleneck
@@ -595,40 +671,71 @@ protected:
 
   // used during capture - map from real resource to its wrapper (other way can be done just with an
   // Unwrap)
-  map<RealResourceType, WrappedResourceType> m_WrapperMap;
+  std::map<RealResourceType, WrappedResourceType> m_WrapperMap;
 
   // used during capture - holds resources referenced in current frame (and how they're referenced)
-  map<ResourceId, FrameRefType> m_FrameReferencedResources;
+  std::map<ResourceId, FrameRefType> m_FrameReferencedResources;
 
   // used during capture - holds resources marked as dirty, needing initial contents
-  set<ResourceId> m_DirtyResources;
-  set<ResourceId> m_PendingDirtyResources;
+  std::set<ResourceId> m_DirtyResources;
+
+  struct InitialContentDataOrChunk
+  {
+    Chunk *chunk = NULL;
+    InitialContentData data;
+
+    void Free(ResourceManager *mgr)
+    {
+      if(chunk)
+      {
+        delete chunk;
+        chunk = NULL;
+      }
+
+      data.Free(mgr);
+    }
+  };
 
   // used during capture or replay - holds initial contents
-  map<ResourceId, InitialContentData> m_InitialContents;
-  // on capture, if a chunk was prepared in Prepare_InitialContents and added, don't re-serialise.
-  // Some initial contents may not need the delayed readback.
-  map<ResourceId, Chunk *> m_InitialChunks;
+  std::map<ResourceId, InitialContentDataOrChunk> m_InitialContents;
 
   // used during capture or replay - map of resources currently alive with their real IDs, used in
   // capture and replay.
-  map<ResourceId, WrappedResourceType> m_CurrentResourceMap;
+  std::map<ResourceId, WrappedResourceType> m_CurrentResourceMap;
 
   // used during replay - maps back and forth from original id to live id and vice-versa
-  map<ResourceId, ResourceId> m_OriginalIDs, m_LiveIDs;
+  std::map<ResourceId, ResourceId> m_OriginalIDs, m_LiveIDs;
 
   // used during replay - holds resources allocated and the original id that they represent
-  map<ResourceId, WrappedResourceType> m_LiveResourceMap;
+  std::map<ResourceId, WrappedResourceType> m_LiveResourceMap;
 
   // used during capture - holds resource records by id.
-  map<ResourceId, RecordType *> m_ResourceRecords;
+  std::map<ResourceId, RecordType *> m_ResourceRecords;
 
   // used during replay - holds current resource replacements
-  map<ResourceId, ResourceId> m_Replacements;
+  std::map<ResourceId, ResourceId> m_Replacements;
+
+  // During initial resources preparation, persistent resources are
+  // postponed until serializing to RDC file.
+  std::set<ResourceId> m_PostponedResourceIDs;
+
+  // On marking resource write-referenced in frame, its last write
+  // time is reset. The time is used to determine persistent resources,
+  // and is checked against the `PERSISTENT_RESOURCE_AGE`.
+  std::map<ResourceId, double> m_LastWriteTime;
+
+  // Timestamp at the beginning of the frame capture. Used to determine which
+  // resources to refresh for their last write time (see `m_LastWriteTime`).
+  double m_captureStartTime;
+
+  PerformanceTimer m_ResourcesUpdateTimer;
+
+  // The capture state is propagated by a specific driver.
+  CaptureState &m_State;
 };
 
 template <typename Configuration>
-ResourceManager<Configuration>::ResourceManager()
+ResourceManager<Configuration>::ResourceManager(CaptureState &state) : m_State(state)
 {
   if(RenderDoc::Inst().GetCrashHandler())
     RenderDoc::Inst().GetCrashHandler()->RegisterMemoryRegion(this, sizeof(ResourceManager));
@@ -674,6 +781,15 @@ void ResourceManager<Configuration>::MarkResourceFrameReferenced(ResourceId id,
   if(id == ResourceId())
     return;
 
+  if(IsDirtyFrameRef(refType))
+  {
+    Prepare_ResourceIfActivePostponed(id);
+    UpdateLastWriteTime(id);
+  }
+
+  if(IsBackgroundCapturing(m_State))
+    return;
+
   bool newRef = MarkReferenced(m_FrameReferencedResources, id, refType, comp);
 
   if(newRef)
@@ -702,26 +818,6 @@ void ResourceManager<Configuration>::MarkDirtyResource(ResourceId res)
   m_DirtyResources.insert(res);
 }
 
-template <typename Configuration>
-void ResourceManager<Configuration>::MarkPendingDirty(ResourceId res)
-{
-  SCOPED_LOCK(m_Lock);
-
-  if(res == ResourceId())
-    return;
-
-  m_PendingDirtyResources.insert(res);
-}
-
-template <typename Configuration>
-void ResourceManager<Configuration>::FlushPendingDirty()
-{
-  SCOPED_LOCK(m_Lock);
-
-  m_DirtyResources.insert(m_PendingDirtyResources.begin(), m_PendingDirtyResources.end());
-  m_PendingDirtyResources.clear();
-}
-
 template <typename Configuration>
 bool ResourceManager<Configuration>::IsResourceDirty(ResourceId res)
 {
@@ -733,20 +829,6 @@ bool ResourceManager<Configuration>::IsResourceDirty(ResourceId res)
   return m_DirtyResources.find(res) != m_DirtyResources.end();
 }
 
-template <typename Configuration>
-void ResourceManager<Configuration>::MarkCleanResource(ResourceId res)
-{
-  SCOPED_LOCK(m_Lock);
-
-  if(res == ResourceId())
-    return;
-
-  if(IsResourceDirty(res))
-  {
-    m_DirtyResources.erase(res);
-  }
-}
-
 template <typename Configuration>
 void ResourceManager<Configuration>::SetInitialContents(ResourceId id, InitialContentData contents)
 {
@@ -757,13 +839,9 @@ void ResourceManager<Configuration>::SetInitialContents(ResourceId id, InitialCo
   auto it = m_InitialContents.find(id);
 
   if(it != m_InitialContents.end())
-  {
-    // free previous contents
     it->second.Free(this);
-    m_InitialContents.erase(it);
-  }
 
-  m_InitialContents[id] = contents;
+  m_InitialContents[id].data = contents;
 }
 
 template <typename Configuration>
@@ -772,19 +850,14 @@ void ResourceManager<Configuration>::SetInitialChunk(ResourceId id, Chunk *chunk
   SCOPED_LOCK(m_Lock);
 
   RDCASSERT(id != ResourceId());
-
-  auto it = m_InitialChunks.find(id);
-
   RDCASSERT(chunk->GetChunkType<SystemChunk>() == SystemChunk::InitialContents);
 
-  if(it != m_InitialChunks.end())
-  {
-    RDCERR("Initial chunk set for ID %llu twice", id);
-    delete chunk;
-    return;
-  }
+  InitialContentDataOrChunk &data = m_InitialContents[id];
 
-  m_InitialChunks[id] = chunk;
+  if(data.chunk)
+    delete data.chunk;
+
+  data.chunk = chunk;
 }
 
 template <typename Configuration>
@@ -797,7 +870,7 @@ typename Configuration::InitialContentData ResourceManager<Configuration>::GetIn
     return InitialContentData();
 
   if(m_InitialContents.find(id) != m_InitialContents.end())
-    return m_InitialContents[id];
+    return m_InitialContents[id].data;
 
   return InitialContentData();
 }
@@ -828,11 +901,13 @@ void ResourceManager<Configuration>::Serialise_InitialContentsNeeded(WriteSerial
 
   SCOPED_LOCK(m_Lock);
 
-  std::vector<WrittenRecord> WrittenRecords;
+  rdcarray<WrittenRecord> WrittenRecords;
 
   // reasonable estimate, and these records are small
   WrittenRecords.reserve(m_FrameReferencedResources.size());
 
+  // all resources that were recorded as being modified should be included in the list of those
+  // needing initial contents
   for(auto it = m_FrameReferencedResources.begin(); it != m_FrameReferencedResources.end(); ++it)
   {
     RecordType *record = GetResourceRecord(it->first);
@@ -844,9 +919,10 @@ void ResourceManager<Configuration>::Serialise_InitialContentsNeeded(WriteSerial
     }
   }
 
-  for(auto it = m_DirtyResources.begin(); it != m_DirtyResources.end(); ++it)
+  // any resources that had initial contents generated should also be included
+  for(auto it = m_InitialContents.begin(); it != m_InitialContents.end(); ++it)
   {
-    ResourceId id = *it;
+    ResourceId id = it->first;
     auto ref = m_FrameReferencedResources.find(id);
     if(ref == m_FrameReferencedResources.end() || !IsDirtyFrameRef(ref->second))
     {
@@ -856,7 +932,7 @@ void ResourceManager<Configuration>::Serialise_InitialContentsNeeded(WriteSerial
     }
   }
 
-  uint32_t chunkSize = uint32_t(WrittenRecords.size() * sizeof(WrittenRecord) + 16);
+  uint64_t chunkSize = uint64_t(WrittenRecords.size() * sizeof(WrittenRecord) + 16);
 
   SCOPED_SERIALISE_CHUNK(SystemChunk::InitialContentsList, chunkSize);
   SERIALISE_ELEMENT(WrittenRecords);
@@ -872,11 +948,97 @@ void ResourceManager<Configuration>::FreeInitialContents()
     if(!m_InitialContents.empty())
       m_InitialContents.erase(m_InitialContents.begin());
   }
+  m_PostponedResourceIDs.clear();
+}
 
-  for(auto it = m_InitialChunks.begin(); it != m_InitialChunks.end(); ++it)
-    delete it->second;
+template <typename Configuration>
+void ResourceManager<Configuration>::Prepare_ResourceInitialStateIfNeeded(ResourceId id)
+{
+  SCOPED_LOCK(m_Lock);
 
-  m_InitialChunks.clear();
+  if(!IsResourcePostponed(id))
+    return;
+
+  WrappedResourceType res = GetCurrentResource(id);
+  Prepare_InitialState(res);
+
+  m_PostponedResourceIDs.erase(id);
+}
+
+template <typename Configuration>
+void ResourceManager<Configuration>::Prepare_ResourceIfActivePostponed(ResourceId id)
+{
+  SCOPED_LOCK(m_Lock);
+
+  // If the resource was postponed during Active Capture, we need to prepare it
+  // right away, since next Read might be invalid.
+  if(!IsActiveCapturing(m_State) || !IsResourcePostponed(id))
+    return;
+
+  RDCDEBUG("Preparing resource %s after it has been postponed.", ToStr(id).c_str());
+  Prepare_ResourceInitialStateIfNeeded(id);
+}
+
+template <typename Configuration>
+inline void ResourceManager<Configuration>::UpdateLastWriteTime(ResourceId id)
+{
+  SCOPED_LOCK(m_Lock);
+  m_LastWriteTime[id] = m_ResourcesUpdateTimer.GetMilliseconds();
+}
+
+template <typename Configuration>
+inline void ResourceManager<Configuration>::ResetCaptureStartTime()
+{
+  SCOPED_LOCK(m_Lock);
+  // This time is used to analyze which resources to refresh
+  // for their last write time.
+  m_captureStartTime = m_ResourcesUpdateTimer.GetMilliseconds();
+}
+
+template <typename Configuration>
+inline void ResourceManager<Configuration>::ResetLastWriteTimes()
+{
+  SCOPED_LOCK(m_Lock);
+  for(auto it = m_LastWriteTime.begin(); it != m_LastWriteTime.end(); ++it)
+  {
+    // Reset only those resources which were below the threshold on
+    // capture start. Other resource are already above the threshold.
+    if(m_captureStartTime - it->second <= PERSISTENT_RESOURCE_AGE)
+      it->second = m_ResourcesUpdateTimer.GetMilliseconds();
+  }
+}
+
+template <typename Configuration>
+inline bool ResourceManager<Configuration>::HasPersistentAge(ResourceId id)
+{
+  SCOPED_LOCK(m_Lock);
+
+  auto it = m_LastWriteTime.find(id);
+
+  if(it == m_LastWriteTime.end())
+    return true;
+
+  return m_ResourcesUpdateTimer.GetMilliseconds() - it->second >= PERSISTENT_RESOURCE_AGE;
+}
+
+template <typename Configuration>
+inline bool ResourceManager<Configuration>::IsResourcePostponed(ResourceId id)
+{
+  SCOPED_LOCK(m_Lock);
+  return m_PostponedResourceIDs.find(id) != m_PostponedResourceIDs.end();
+}
+
+template <typename Configuration>
+inline bool ResourceManager<Configuration>::IsResourcePersistent(ResourceId id)
+{
+  SCOPED_LOCK(m_Lock);
+
+  WrappedResourceType res = GetCurrentResource(id);
+
+  if(!IsResourceTrackedForPersistency(res))
+    return false;
+
+  return HasPersistentAge(id);
 }
 
 template <typename Configuration>
@@ -884,9 +1046,9 @@ void ResourceManager<Configuration>::CreateInitialContents(ReadSerialiser &ser)
 {
   using namespace ResourceManagerInternal;
 
-  set<ResourceId> neededInitials;
+  std::set<ResourceId> neededInitials;
 
-  std::vector<WrittenRecord> WrittenRecords;
+  rdcarray<WrittenRecord> WrittenRecords;
   SERIALISE_ELEMENT(WrittenRecords);
 
   for(const WrittenRecord &wr : WrittenRecords)
@@ -920,21 +1082,21 @@ template <typename Configuration>
 void ResourceManager<Configuration>::ApplyInitialContents()
 {
   RDCDEBUG("Applying initial contents");
-  std::vector<ResourceId> resources = InitialContentResources();
+  rdcarray<ResourceId> resources = InitialContentResources();
   for(auto it = resources.begin(); it != resources.end(); ++it)
   {
     ResourceId id = *it;
-    InitialContentData data = m_InitialContents[id];
+    const InitialContentDataOrChunk &data = m_InitialContents[id];
     WrappedResourceType live = GetLiveResource(id);
-    Apply_InitialState(live, data);
+    Apply_InitialState(live, data.data);
   }
   RDCDEBUG("Applied %d", (uint32_t)resources.size());
 }
 
 template <typename Configuration>
-std::vector<ResourceId> ResourceManager<Configuration>::InitialContentResources()
+rdcarray<ResourceId> ResourceManager<Configuration>::InitialContentResources()
 {
-  std::vector<ResourceId> resources;
+  rdcarray<ResourceId> resources;
   for(auto it = m_InitialContents.begin(); it != m_InitialContents.end(); ++it)
   {
     ResourceId id = it->first;
@@ -961,7 +1123,7 @@ void ResourceManager<Configuration>::MarkUnwrittenResources()
 template <typename Configuration>
 void ResourceManager<Configuration>::InsertReferencedChunks(WriteSerialiser &ser)
 {
-  map<int32_t, Chunk *> sortedChunks;
+  std::map<int32_t, Chunk *> sortedChunks;
 
   SCOPED_LOCK(m_Lock);
 
@@ -1026,41 +1188,37 @@ void ResourceManager<Configuration>::PrepareInitialContents()
     RenderDoc::Inst().SetProgress(CaptureProgress::PrepareInitialStates, idx / num);
     idx += 1.0f;
 
+    // if somehow this resource has been deleted but is still dirty, we can't prepare it. Resources
+    // deleted prior to beginning the frame capture cannot linger and be needed - we only need to
+    // care about resources deleted after this point (mid-capture)
     if(!HasCurrentResource(id))
       continue;
 
     RecordType *record = GetResourceRecord(id);
     WrappedResourceType res = GetCurrentResource(id);
 
+    // don't prepare internal resources, or those without a record
     if(record == NULL || record->InternalResource)
       continue;
 
+    if(IsResourcePersistent(id))
+    {
+      m_PostponedResourceIDs.insert(id);
+      // Set empty contents here, it'll be prepared on serialization.
+      SetInitialContents(id, InitialContentData());
+      continue;
+    }
+
     prepared++;
 
 #if ENABLED(VERBOSE_DIRTY_RESOURCES)
-    RDCDEBUG("Prepare Resource %llu", id);
+    RDCDEBUG("Prepare Resource %s", ToStr(id).c_str());
 #endif
 
     Prepare_InitialState(res);
   }
 
   RDCDEBUG("Prepared %u dirty resources", prepared);
-
-  prepared = 0;
-
-  for(auto it = m_CurrentResourceMap.begin(); it != m_CurrentResourceMap.end(); ++it)
-  {
-    if(it->second == (WrappedResourceType)RecordType::NullResource)
-      continue;
-
-    if(Force_InitialState(it->second, true))
-    {
-      prepared++;
-      Prepare_InitialState(it->second);
-    }
-  }
-
-  RDCDEBUG("Force-prepared %u dirty resources", prepared);
 }
 
 template <typename Configuration>
@@ -1071,14 +1229,14 @@ void ResourceManager<Configuration>::InsertInitialContentsChunks(WriteSerialiser
   uint32_t dirty = 0;
   uint32_t skipped = 0;
 
-  RDCDEBUG("Checking %u possibly dirty resources", (uint32_t)m_DirtyResources.size());
+  RDCDEBUG("Checking %u resources with initial contents", (uint32_t)m_InitialContents.size());
 
-  float num = float(m_DirtyResources.size());
+  float num = float(m_InitialContents.size());
   float idx = 0.0f;
 
-  for(auto it = m_DirtyResources.begin(); it != m_DirtyResources.end(); ++it)
+  for(auto it = m_InitialContents.begin(); it != m_InitialContents.end(); ++it)
   {
-    ResourceId id = *it;
+    ResourceId id = it->first;
 
     RenderDoc::Inst().SetProgress(CaptureProgress::SerialiseInitialStates, idx / num);
     idx += 1.0f;
@@ -1087,32 +1245,18 @@ void ResourceManager<Configuration>::InsertInitialContentsChunks(WriteSerialiser
        !RenderDoc::Inst().GetCaptureOptions().refAllResources)
     {
 #if ENABLED(VERBOSE_DIRTY_RESOURCES)
-      RDCDEBUG("Dirty tesource %llu is GPU dirty but not referenced - skipping", id);
+      RDCDEBUG("Dirty tesource %s is GPU dirty but not referenced - skipping", ToStr(id).c_str());
 #endif
       skipped++;
       continue;
     }
 
-    WrappedResourceType res = (WrappedResourceType)RecordType::NullResource;
-    bool isAlive = HasCurrentResource(id);
-
-    if(!AllowDeletedResource_InitialState() && !isAlive)
-    {
-#if ENABLED(VERBOSE_DIRTY_RESOURCES)
-      RDCDEBUG("Resource %llu no longer exists - skipping", id);
-#endif
-      continue;
-    }
-
-    if(isAlive)
-      res = GetCurrentResource(id);
-
     RecordType *record = GetResourceRecord(id);
 
     if(record == NULL)
     {
 #if ENABLED(VERBOSE_DIRTY_RESOURCES)
-      RDCDEBUG("Resource %llu has no resource record - skipping", id);
+      RDCDEBUG("Resource %s has no resource record - skipping", ToStr(id).c_str());
 #endif
       continue;
     }
@@ -1120,78 +1264,45 @@ void ResourceManager<Configuration>::InsertInitialContentsChunks(WriteSerialiser
     if(record->InternalResource)
     {
 #if ENABLED(VERBOSE_DIRTY_RESOURCES)
-      RDCDEBUG("Resource %llu is special - skipping", id);
+      RDCDEBUG("Resource %s is special - skipping", ToStr(id).c_str());
 #endif
       continue;
     }
 
 #if ENABLED(VERBOSE_DIRTY_RESOURCES)
-    RDCDEBUG("Serialising dirty Resource %llu", id);
+    RDCDEBUG("Serialising dirty Resource %s", ToStr(id).c_str());
 #endif
 
+    // Load postponed resource if needed.
+    Prepare_ResourceInitialStateIfNeeded(id);
+
     dirty++;
 
-    if(!Need_InitialStateChunk(res))
+    if(!Need_InitialStateChunk(id, it->second.data))
     {
-      // just need to grab data, don't create chunk
-      Serialise_InitialState(ser, id, res);
+      // this was handled in ApplyInitialContentsNonChunks(), do nothing as there's no point copying
+      // the data again (it's already been serialised).
       continue;
     }
 
-    auto preparedChunk = m_InitialChunks.find(id);
-    if(preparedChunk != m_InitialChunks.end())
+    if(it->second.chunk)
     {
-      preparedChunk->second->Write(ser);
-      m_InitialChunks.erase(preparedChunk);
+      it->second.chunk->Write(ser);
     }
     else
     {
-      uint32_t size = GetSize_InitialState(id, res);
+      uint64_t size = GetSize_InitialState(id, it->second.data);
 
       SCOPED_SERIALISE_CHUNK(SystemChunk::InitialContents, size);
 
-      Serialise_InitialState(ser, id, res);
+      Serialise_InitialState(ser, id, record, &it->second.data);
     }
+
+    // Reset back to empty contents, unloading the actual resource.
+    SetInitialContents(id, InitialContentData());
   }
 
-  RDCDEBUG("Serialised %u dirty resources, skipped %u unreferenced", dirty, skipped);
-
-  dirty = 0;
-
-  for(auto it = m_CurrentResourceMap.begin(); it != m_CurrentResourceMap.end(); ++it)
-  {
-    if(it->second == (WrappedResourceType)RecordType::NullResource)
-      continue;
-
-    if(Force_InitialState(it->second, false))
-    {
-      dirty++;
-
-      auto preparedChunk = m_InitialChunks.find(it->first);
-      if(preparedChunk != m_InitialChunks.end())
-      {
-        preparedChunk->second->Write(ser);
-        m_InitialChunks.erase(preparedChunk);
-      }
-      else
-      {
-        uint32_t size = GetSize_InitialState(it->first, it->second);
-
-        SCOPED_SERIALISE_CHUNK(SystemChunk::InitialContents, size);
-
-        Serialise_InitialState(ser, it->first, it->second);
-      }
-    }
-  }
-
-  RDCDEBUG("Force-serialised %u dirty resources", dirty);
-
-  // delete/cleanup any chunks that weren't used (maybe the resource was not
-  // referenced).
-  for(auto it = m_InitialChunks.begin(); it != m_InitialChunks.end(); ++it)
-    delete it->second;
-
-  m_InitialChunks.clear();
+  RDCDEBUG("Serialised %u resources, skipped %u unreferenced", dirty, skipped);
 }
 
 template <typename Configuration>
@@ -1199,9 +1310,9 @@ void ResourceManager<Configuration>::ApplyInitialContentsNonChunks(WriteSerialis
 {
   SCOPED_LOCK(m_Lock);
 
-  for(auto it = m_DirtyResources.begin(); it != m_DirtyResources.end(); ++it)
+  for(auto it = m_InitialContents.begin(); it != m_InitialContents.end(); ++it)
   {
-    ResourceId id = *it;
+    ResourceId id = it->first;
 
     if(m_FrameReferencedResources.find(id) == m_FrameReferencedResources.end() &&
        !RenderDoc::Inst().GetCaptureOptions().refAllResources)
@@ -1209,22 +1320,13 @@ void ResourceManager<Configuration>::ApplyInitialContentsNonChunks(WriteSerialis
       continue;
     }
 
-    WrappedResourceType res = (WrappedResourceType)RecordType::NullResource;
-    bool isAlive = HasCurrentResource(id);
-
-    if(!AllowDeletedResource_InitialState() && !isAlive)
-      continue;
-
-    if(isAlive)
-      res = GetCurrentResource(id);
-
     RecordType *record = GetResourceRecord(id);
 
     if(!record || record->InternalResource)
       continue;
 
-    if(!Need_InitialStateChunk(res))
-      Serialise_InitialState(ser, id, res);
+    if(!Need_InitialStateChunk(id, it->second.data))
+      Serialise_InitialState(ser, id, record, &it->second.data);
   }
 }
 
@@ -1238,7 +1340,11 @@ void ResourceManager<Configuration>::ClearReferencedResources()
     RecordType *record = GetResourceRecord(it->first);
 
     if(record)
+    {
+      if(IncludesWrite(it->second))
+        MarkDirtyResource(it->first);
       record->Delete(this);
+    }
   }
 
   m_FrameReferencedResources.clear();
@@ -1411,7 +1517,7 @@ void ResourceManager<Configuration>::AddLiveResource(ResourceId origid, WrappedR
 
   if(m_LiveResourceMap.find(origid) != m_LiveResourceMap.end())
   {
-    RDCERR("Releasing live resource for duplicate creation: %llu", origid);
+    RDCERR("Releasing live resource for duplicate creation: %s", ToStr(origid).c_str());
     ResourceTypeRelease(m_LiveResourceMap[origid]);
     m_LiveResourceMap.erase(origid);
   }
@@ -1500,7 +1606,14 @@ void ResourceManager<Configuration>::ReleaseCurrentResource(ResourceId id)
   SCOPED_LOCK(m_Lock);
 
   RDCASSERT(m_CurrentResourceMap.find(id) != m_CurrentResourceMap.end(), id);
+
+  // We potentially need to prepare this resource on Active Capture,
+  // if it was postponed, but is about to go away.
+  Prepare_ResourceIfActivePostponed(id);
+
   m_CurrentResourceMap.erase(id);
+  m_DirtyResources.erase(id);
+  m_LastWriteTime.erase(id);
 }
 
 template <typename Configuration>
