@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,9 +25,11 @@
 
 #include "android/android.h"
 #include "api/replay/renderdoc_replay.h"
+#include "common/threading.h"
 #include "core/core.h"
 #include "jpeg-compressor/jpgd.h"
 #include "os/os_specific.h"
+#include "replay/replay_driver.h"
 #include "serialise/serialiser.h"
 
 static const uint32_t TargetControlProtocolVersion = 5;
@@ -72,7 +74,7 @@ enum PacketType : uint32_t
 DECLARE_REFLECTION_ENUM(PacketType);
 
 template <>
-std::string DoStringise(const PacketType &el)
+rdcstr DoStringise(const PacketType &el)
 {
   BEGIN_ENUM_STRINGISE(PacketType);
   {
@@ -98,6 +100,8 @@ std::string DoStringise(const PacketType &el)
 
 void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *client)
 {
+  Threading::SetCurrentThreadName("TargetControlClientThread");
+
   Threading::KeepModuleAlive();
 
   WriteSerialiser writer(new StreamWriter(client, Ownership::Nothing), Ownership::Stream);
@@ -106,7 +110,7 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
   writer.SetStreamingMode(true);
   reader.SetStreamingMode(true);
 
-  std::string target = RenderDoc::Inst().GetCurrentTarget();
+  rdcstr target = RenderDoc::Inst().GetCurrentTarget();
   uint32_t mypid = Process::GetCurrentPID();
 
   {
@@ -139,8 +143,8 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
   const int progresstime = 100;    // update capture progress every 100ms
   int curtime = 0;
 
-  std::vector<CaptureData> captures;
-  std::vector<pair<uint32_t, uint32_t> > children;
+  rdcarray<CaptureData> captures;
+  rdcarray<rdcpair<uint32_t, uint32_t> > children;
   std::map<RDCDriver, bool> drivers;
   float prevCaptureProgress = captureProgress;
   uint32_t prevWindows = 0;
@@ -158,8 +162,8 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
 
     std::map<RDCDriver, bool> curdrivers = RenderDoc::Inst().GetActiveDrivers();
 
-    std::vector<CaptureData> caps = RenderDoc::Inst().GetCaptures();
-    std::vector<pair<uint32_t, uint32_t> > childprocs = RenderDoc::Inst().GetChildProcesses();
+    rdcarray<CaptureData> caps = RenderDoc::Inst().GetCaptures();
+    rdcarray<rdcpair<uint32_t, uint32_t> > childprocs = RenderDoc::Inst().GetChildProcesses();
 
     uint32_t curWindows = RenderDoc::Inst().GetCapturableWindowCount();
 
@@ -202,7 +206,7 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
 
       captures.push_back(caps[idx]);
 
-      std::string path = FileIO::GetFullPathname(captures.back().path);
+      rdcstr path = FileIO::GetFullPathname(captures.back().path);
 
       bytebuf buf;
 
@@ -221,9 +225,13 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
         SERIALISE_ELEMENT(path);
         SERIALISE_ELEMENT(buf);
         if(version >= 3)
+        {
           SERIALISE_ELEMENT(captures.back().driver);
+        }
         if(version >= 5)
+        {
           SERIALISE_ELEMENT(captures.back().frameNumber);
+        }
       }
     }
     else if(childprocs.size() != children.size())
@@ -337,7 +345,7 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
           SCOPED_SERIALISE_CHUNK(ePacket_CopyCapture);
           SERIALISE_ELEMENT(id);
 
-          std::string filename = caps[id].path;
+          rdcstr filename = caps[id].path;
 
           StreamReader fileStream(FileIO::fopen(filename.c_str(), "rb"));
           ser.SerialiseStream(filename, fileStream);
@@ -373,6 +381,8 @@ void RenderDoc::TargetControlClientThread(uint32_t version, Network::Socket *cli
 
 void RenderDoc::TargetControlServerThread(Network::Socket *sock)
 {
+  Threading::SetCurrentThreadName("TargetControlServerThread");
+
   Threading::KeepModuleAlive();
 
   RenderDoc::Inst().m_SingleClientName = "";
@@ -401,8 +411,8 @@ void RenderDoc::TargetControlServerThread(Network::Socket *sock)
       continue;
     }
 
-    std::string existingClient;
-    std::string newClient;
+    rdcstr existingClient;
+    rdcstr newClient;
     uint32_t version;
     bool kick = false;
 
@@ -470,7 +480,7 @@ void RenderDoc::TargetControlServerThread(Network::Socket *sock)
 
       ser.SetStreamingMode(true);
 
-      std::string target = RenderDoc::Inst().GetCurrentTarget();
+      rdcstr target = RenderDoc::Inst().GetCurrentTarget();
       {
         SCOPED_SERIALISE_CHUNK(ePacket_Busy);
         SERIALISE_ELEMENT(TargetControlProtocolVersion);
@@ -496,12 +506,12 @@ void RenderDoc::TargetControlServerThread(Network::Socket *sock)
 struct TargetControl : public ITargetControl
 {
 public:
-  TargetControl(Network::Socket *sock, std::string clientName, bool forceConnection)
+  TargetControl(Network::Socket *sock, rdcstr clientName, bool forceConnection)
       : m_Socket(sock),
         reader(new StreamReader(sock, Ownership::Nothing), Ownership::Stream),
         writer(new StreamWriter(sock, Ownership::Nothing), Ownership::Stream)
   {
-    std::vector<byte> payload;
+    bytebuf payload;
 
     writer.SetStreamingMode(true);
     reader.SetStreamingMode(true);
@@ -688,7 +698,7 @@ public:
     else if(type == ePacket_Busy)
     {
       READ_DATA_SCOPE();
-      SERIALISE_ELEMENT(msg.busy.clientName).Named("Client Name");
+      SERIALISE_ELEMENT(msg.busy.clientName).Named("Client Name"_lit);
 
       SAFE_DELETE(m_Socket);
 
@@ -701,8 +711,8 @@ public:
       msg.type = TargetControlMessageType::NewChild;
 
       READ_DATA_SCOPE();
-      SERIALISE_ELEMENT(msg.newChild.processId).Named("PID");
-      SERIALISE_ELEMENT(msg.newChild.ident).Named("Child ident");
+      SERIALISE_ELEMENT(msg.newChild.processId).Named("PID"_lit);
+      SERIALISE_ELEMENT(msg.newChild.ident).Named("Child ident"_lit);
 
       RDCLOG("Got a new child process: %u %u", msg.newChild.processId, msg.newChild.ident);
 
@@ -714,7 +724,7 @@ public:
       msg.type = TargetControlMessageType::CaptureProgress;
 
       READ_DATA_SCOPE();
-      SERIALISE_ELEMENT(msg.capProgress).Named("Capture Progress");
+      SERIALISE_ELEMENT(msg.capProgress).Named("Capture Progress"_lit);
 
       reader.EndChunk();
       return msg;
@@ -729,12 +739,14 @@ public:
 
       {
         READ_DATA_SCOPE();
-        SERIALISE_ELEMENT(msg.newCapture.captureId).Named("Capture ID");
-        SERIALISE_ELEMENT(msg.newCapture.timestamp).Named("timestamp");
-        SERIALISE_ELEMENT(msg.newCapture.path).Named("path");
+        SERIALISE_ELEMENT(msg.newCapture.captureId).Named("Capture ID"_lit);
+        SERIALISE_ELEMENT(msg.newCapture.timestamp).Named("timestamp"_lit);
+        SERIALISE_ELEMENT(msg.newCapture.path).Named("path"_lit);
         SERIALISE_ELEMENT(thumbnail);
         if(m_Version >= 3)
+        {
           SERIALISE_ELEMENT(driver);
+        }
         if(m_Version >= 5)
         {
           SERIALISE_ELEMENT(msg.newCapture.frameNumber);
@@ -809,7 +821,7 @@ public:
       msg.type = TargetControlMessageType::CaptureCopied;
 
       READ_DATA_SCOPE();
-      SERIALISE_ELEMENT(msg.newCapture.captureId).Named("Capture ID");
+      SERIALISE_ELEMENT(msg.newCapture.captureId).Named("Capture ID"_lit);
 
       msg.newCapture.path = m_CaptureCopies[msg.newCapture.captureId];
 
@@ -854,31 +866,35 @@ private:
   Network::Socket *m_Socket;
   WriteSerialiser writer;
   ReadSerialiser reader;
-  std::string m_Target, m_API, m_BusyClient;
+  rdcstr m_Target, m_API, m_BusyClient;
   uint32_t m_Version, m_PID;
 
-  std::map<uint32_t, std::string> m_CaptureCopies;
+  std::map<uint32_t, rdcstr> m_CaptureCopies;
 };
 
 extern "C" RENDERDOC_API ITargetControl *RENDERDOC_CC RENDERDOC_CreateTargetControl(
-    const char *host, uint32_t ident, const char *clientName, bool forceConnection)
+    const char *URL, uint32_t ident, const char *clientName, bool forceConnection)
 {
-  std::string s = "localhost";
-  if(host != NULL && host[0] != '\0')
-    s = host;
+  rdcstr host = "localhost";
+  if(URL != NULL && URL[0] != '\0')
+    host = URL;
 
-  bool android = false;
+  rdcstr deviceID = host;
+  uint16_t port = ident & 0xffff;
 
-  if(host != NULL && Android::IsHostADB(host))
+  IDeviceProtocolHandler *protocol = RenderDoc::Inst().GetDeviceProtocol(deviceID);
+
+  if(protocol)
   {
-    android = true;
-    s = "127.0.0.1";
+    deviceID = protocol->GetDeviceID(deviceID);
+    host = protocol->RemapHostname(deviceID);
+    if(host.empty())
+      return NULL;
 
-    // we don't need the index or device ID here, because the port is already the right one
-    // forwarded to the right device.
+    port = protocol->RemapPort(deviceID, port);
   }
 
-  Network::Socket *sock = Network::CreateClientSocket(s.c_str(), ident & 0xffff, 750);
+  Network::Socket *sock = Network::CreateClientSocket(host.c_str(), port, 750);
 
   if(sock == NULL)
     return NULL;
