@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,6 +25,11 @@
 
 #pragma once
 
+#if !defined(CONCAT)
+#define CONCAT2(a, b) a##b
+#define CONCAT(a, b) CONCAT2(a, b)
+#endif
+
 #define DECLARE_EMBED(filename)                   \
   extern unsigned char CONCAT(data_, filename)[]; \
   extern int CONCAT(CONCAT(data_, filename), _len);
@@ -58,5 +63,6 @@ DECLARE_EMBED(glsl_ms2array_comp);
 DECLARE_EMBED(glsl_deptharr2ms_frag);
 DECLARE_EMBED(glsl_depthms2arr_frag);
 DECLARE_EMBED(glsl_gles_texsample_h);
+DECLARE_EMBED(glsl_texremap_frag);
 
 #undef DECLARE_EMBED
