@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,14 +25,14 @@
 
 #include "d3d11_common.h"
 #include "core/core.h"
-#include "driver/d3d11/d3d11_renderstate.h"
-#include "driver/d3d11/d3d11_resources.h"
 #include "serialise/serialiser.h"
 #include "strings/string_utils.h"
+#include "d3d11_renderstate.h"
+#include "d3d11_resources.h"
 
 WrappedID3D11Device *D3D11MarkerRegion::device;
 
-D3D11MarkerRegion::D3D11MarkerRegion(const std::string &marker)
+D3D11MarkerRegion::D3D11MarkerRegion(const rdcstr &marker)
 {
   D3D11MarkerRegion::Begin(marker);
 }
@@ -42,7 +42,7 @@ D3D11MarkerRegion::~D3D11MarkerRegion()
   D3D11MarkerRegion::End();
 }
 
-void D3D11MarkerRegion::Set(const std::string &marker)
+void D3D11MarkerRegion::Set(const rdcstr &marker)
 {
   if(device == NULL)
     return;
@@ -53,7 +53,7 @@ void D3D11MarkerRegion::Set(const std::string &marker)
     annot->SetMarker(StringFormat::UTF82Wide(marker).c_str());
 }
 
-void D3D11MarkerRegion::Begin(const std::string &marker)
+void D3D11MarkerRegion::Begin(const rdcstr &marker)
 {
   if(device == NULL)
     return;
@@ -453,15 +453,6 @@ ResourceRange::ResourceRange(ID3D11Resource *res, UINT mip, UINT slice)
   stencilReadOnly = false;
 }
 
-D3D11InitParams::D3D11InitParams()
-{
-  DriverType = D3D_DRIVER_TYPE_UNKNOWN;
-  Flags = 0;
-  SDKVersion = D3D11_SDK_VERSION;
-  NumFeatureLevels = 0;
-  RDCEraseEl(FeatureLevels);
-}
-
 bool D3D11InitParams::IsSupportedVersion(uint64_t ver)
 {
   if(ver == CurrentVersion)
@@ -472,6 +463,10 @@ bool D3D11InitParams::IsSupportedVersion(uint64_t ver)
   if(ver == 0x0F)
     return true;
 
+  // 0x10 -> 0x11 - added serialisation of adapter descriptor in D3D11InitParams
+  if(ver == 0x10)
+    return true;
+
   return false;
 }
 
@@ -483,6 +478,14 @@ void DoSerialise(SerialiserType &ser, D3D11InitParams &el)
   SERIALISE_MEMBER(SDKVersion);
   SERIALISE_MEMBER(NumFeatureLevels);
   SERIALISE_MEMBER(FeatureLevels);
+  if(ser.VersionAtLeast(0x11))
+  {
+    SERIALISE_MEMBER(AdapterDesc);
+  }
+  else
+  {
+    RDCEraseEl(el.AdapterDesc);
+  }
 }
 
 INSTANTIATE_SERIALISE_TYPE(D3D11InitParams);
@@ -619,47 +622,44 @@ TextureFilter MakeFilter(D3D11_FILTER filter)
     filter = D3D11_FILTER(filter & 0x7f);
   }
 
-  if(filter == D3D11_FILTER_ANISOTROPIC)
+  switch(filter)
   {
-    ret.minify = ret.magnify = ret.mip = FilterMode::Anisotropic;
-  }
-  else
-  {
-    switch(filter)
-    {
-      case D3D11_FILTER_MIN_MAG_MIP_POINT:
-        ret.minify = ret.magnify = ret.mip = FilterMode::Point;
-        break;
-      case D3D11_FILTER_MIN_MAG_POINT_MIP_LINEAR:
-        ret.minify = ret.magnify = FilterMode::Point;
-        ret.mip = FilterMode::Linear;
-        break;
-      case D3D11_FILTER_MIN_POINT_MAG_LINEAR_MIP_POINT:
-        ret.minify = FilterMode::Point;
-        ret.magnify = FilterMode::Linear;
-        ret.mip = FilterMode::Point;
-        break;
-      case D3D11_FILTER_MIN_POINT_MAG_MIP_LINEAR:
-        ret.minify = FilterMode::Point;
-        ret.magnify = ret.mip = FilterMode::Linear;
-        break;
-      case D3D11_FILTER_MIN_LINEAR_MAG_MIP_POINT:
-        ret.minify = FilterMode::Linear;
-        ret.magnify = ret.mip = FilterMode::Point;
-        break;
-      case D3D11_FILTER_MIN_LINEAR_MAG_POINT_MIP_LINEAR:
-        ret.minify = FilterMode::Linear;
-        ret.magnify = FilterMode::Point;
-        ret.mip = FilterMode::Linear;
-        break;
-      case D3D11_FILTER_MIN_MAG_LINEAR_MIP_POINT:
-        ret.minify = ret.magnify = FilterMode::Linear;
-        ret.mip = FilterMode::Point;
-        break;
-      case D3D11_FILTER_MIN_MAG_MIP_LINEAR:
-        ret.minify = ret.magnify = ret.mip = FilterMode::Linear;
-        break;
-    }
+    case D3D11_FILTER_ANISOTROPIC:
+      ret.minify = ret.magnify = ret.mip = FilterMode::Anisotropic;
+      break;
+    case D3D11_FILTER_MIN_MAG_MIP_POINT:
+      ret.minify = ret.magnify = ret.mip = FilterMode::Point;
+      break;
+    case D3D11_FILTER_MIN_MAG_POINT_MIP_LINEAR:
+      ret.minify = ret.magnify = FilterMode::Point;
+      ret.mip = FilterMode::Linear;
+      break;
+    case D3D11_FILTER_MIN_POINT_MAG_LINEAR_MIP_POINT:
+      ret.minify = FilterMode::Point;
+      ret.magnify = FilterMode::Linear;
+      ret.mip = FilterMode::Point;
+      break;
+    case D3D11_FILTER_MIN_POINT_MAG_MIP_LINEAR:
+      ret.minify = FilterMode::Point;
+      ret.magnify = ret.mip = FilterMode::Linear;
+      break;
+    case D3D11_FILTER_MIN_LINEAR_MAG_MIP_POINT:
+      ret.minify = FilterMode::Linear;
+      ret.magnify = ret.mip = FilterMode::Point;
+      break;
+    case D3D11_FILTER_MIN_LINEAR_MAG_POINT_MIP_LINEAR:
+      ret.minify = FilterMode::Linear;
+      ret.magnify = FilterMode::Point;
+      ret.mip = FilterMode::Linear;
+      break;
+    case D3D11_FILTER_MIN_MAG_LINEAR_MIP_POINT:
+      ret.minify = ret.magnify = FilterMode::Linear;
+      ret.mip = FilterMode::Point;
+      break;
+    case D3D11_FILTER_MIN_MAG_MIP_LINEAR:
+      ret.minify = ret.magnify = ret.mip = FilterMode::Linear;
+      break;
+    default: break;
   }
 
   return ret;
