@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,7 +27,6 @@
 
 #define INITGUID
 
-#include "api/replay/renderdoc_replay.h"
 #include "core/core.h"
 #include "driver/dx/official/d3d11_4.h"
 #include "driver/dx/official/dxgi1_3.h"
@@ -40,10 +39,10 @@ struct D3D11RenderState;
 // replay only class for handling marker regions
 struct D3D11MarkerRegion
 {
-  D3D11MarkerRegion(const std::string &marker);
+  D3D11MarkerRegion(const rdcstr &marker);
   ~D3D11MarkerRegion();
-  static void Set(const std::string &marker);
-  static void Begin(const std::string &marker);
+  static void Set(const rdcstr &marker);
+  static void Begin(const rdcstr &marker);
   static void End();
 
   static WrappedID3D11Device *device;
@@ -187,20 +186,32 @@ inline void SetDebugName(T *pObj, const char *name)
 }
 
 template <class T>
-inline const char *GetDebugName(T *pObj)
+inline rdcstr GetDebugName(T *pObj)
 {
-  static char tmpBuf[1024] = {0};
-  UINT size = 1023;
+  static const UINT DEBUG_NAME_SIZE = 1024;
+  static wchar_t tmpBuf[DEBUG_NAME_SIZE] = {0};    // Used for both char and wchar_t
   if(pObj)
   {
-    HRESULT hr = pObj->GetPrivateData(WKPDID_D3DDebugObjectName, &size, tmpBuf);
-    if(FAILED(hr))
-      return "";
+    // Try char first
+    UINT size = DEBUG_NAME_SIZE - 1;
+    HRESULT hr = pObj->GetPrivateData(WKPDID_D3DDebugObjectName, &size, (char *)tmpBuf);
+    if(SUCCEEDED(hr))
+    {
+      return rdcstr((char *)tmpBuf, size);
+    }
 
-    tmpBuf[size] = 0;
-    return tmpBuf;
+    // Try wchar_t
+    size = (DEBUG_NAME_SIZE - 1) * sizeof(wchar_t);
+    hr = pObj->GetPrivateData(WKPDID_D3DDebugObjectNameW, &size, tmpBuf);
+    if(SUCCEEDED(hr))
+    {
+      size /= 2;    // Convert from bytes read to wide char count
+      rdcwstr sName(tmpBuf, size);
+      return StringFormat::Wide2UTF8(sName);
+    }
   }
-  return "";
+
+  return rdcstr();
 }
 
 class RefCounter
@@ -409,6 +420,9 @@ enum class D3D11Chunk : uint32_t
   PostExecuteCommandList,
   PostFinishCommandListSet,
   SwapDeviceContextState,
+  ExternalDXGIResource,
+  OpenSharedResource1,
+  OpenSharedResourceByName,
   Max,
 };
 
