@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,12 +23,14 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "driver/d3d11/d3d11_context.h"
-#include "driver/d3d11/d3d11_device.h"
-#include "driver/d3d11/d3d11_manager.h"
-#include "driver/d3d11/d3d11_renderstate.h"
-#include "driver/d3d11/d3d11_resources.h"
+#include "d3d11_context.h"
+#include <algorithm>
 #include "strings/string_utils.h"
+#include "d3d11_device.h"
+#include "d3d11_manager.h"
+#include "d3d11_renderstate.h"
+#include "d3d11_replay.h"
+#include "d3d11_resources.h"
 
 WRAPPED_POOL_INST(WrappedID3D11DeviceContext);
 WRAPPED_POOL_INST(WrappedID3D11CommandList);
@@ -147,6 +149,7 @@ WrappedID3D11DeviceContext::WrappedID3D11DeviceContext(WrappedID3D11Device *real
   if(!RenderDoc::Inst().IsReplayApp())
   {
     m_ContextRecord = m_pDevice->GetResourceManager()->AddResourceRecord(m_ResourceID);
+    m_ContextRecord->ResType = Resource_DeviceContext;
     m_ContextRecord->DataInSerialiser = false;
     m_ContextRecord->InternalResource = true;
     m_ContextRecord->Length = 0;
@@ -161,8 +164,6 @@ WrappedID3D11DeviceContext::WrappedID3D11DeviceContext(WrappedID3D11Device *real
   m_FailureReason = CaptureSucceeded;
   m_EmptyCommandList = true;
 
-  m_PresentChunk = false;
-
   m_DrawcallStack.push_back(&m_ParentDrawcall);
 
   m_CurEventID = 0;
@@ -253,7 +254,7 @@ D3D11ResourceManager *WrappedID3D11DeviceContext::GetResourceManager()
   return m_pDevice->GetResourceManager();
 }
 
-std::string WrappedID3D11DeviceContext::GetChunkName(uint32_t idx)
+rdcstr WrappedID3D11DeviceContext::GetChunkName(uint32_t idx)
 {
   if((SystemChunk)idx < SystemChunk::FirstDriverChunk)
     return ToStr((SystemChunk)idx);
@@ -296,7 +297,7 @@ bool WrappedID3D11DeviceContext::Serialise_BeginCaptureFrame(SerialiserType &ser
   // used for a DrawAuto(). If the stream-out happens in frame we don't need to worry,
   // but if it references a buffer from before we need to have that counter available
   // on replay to 'fake' the DrawAuto() just as a Draw() with known values
-  std::vector<HiddenCounter> HiddenStreamOutCounters;
+  rdcarray<HiddenCounter> HiddenStreamOutCounters;
 
   if(ser.IsWriting())
   {
@@ -537,49 +538,63 @@ void WrappedID3D11DeviceContext::EndCaptureFrame()
   ser.SetDrawChunk();
   SCOPED_SERIALISE_CHUNK(SystemChunk::CaptureEnd);
 
-  SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+  SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
 
   m_ContextRecord->AddChunk(scope.Get());
 }
 
+template <typename SerialiserType>
+bool WrappedID3D11DeviceContext::Serialise_Present(SerialiserType &ser, UINT SyncInterval, UINT Flags)
+{
+  // we don't do anything with these parameters, they're just here to store
+  // them for user benefits
+  SERIALISE_ELEMENT(SyncInterval);
+  SERIALISE_ELEMENT(Flags);
+
+  SERIALISE_CHECK_READ_ERRORS();
+
+  if(IsReplayingAndReading() && IsLoading(m_State))
+  {
+    AddEvent();
+
+    DrawcallDescription draw;
+
+    draw.copyDestination = m_pDevice->GetBackbufferResourceID();
+
+    draw.name = StringFormat::Fmt("Present(%s)", ToStr(draw.copyDestination).c_str());
+    draw.flags |= DrawFlags::Present;
+
+    AddDrawcall(draw, true);
+  }
+
+  return true;
+}
+
 void WrappedID3D11DeviceContext::Present(UINT SyncInterval, UINT Flags)
 {
-  WriteSerialiser &ser = m_ScratchSerialiser;
-  SCOPED_SERIALISE_CHUNK(D3D11Chunk::SwapchainPresent);
-  SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
-  SERIALISE_ELEMENT(SyncInterval);
-  SERIALISE_ELEMENT(Flags);
+  SERIALISE_TIME_CALL();
 
-  m_ContextRecord->AddChunk(scope.Get());
+  if(IsActiveCapturing(m_State))
+  {
+    USE_SCRATCH_SERIALISER();
+    GET_SERIALISER.SetDrawChunk();
+    SCOPED_SERIALISE_CHUNK(D3D11Chunk::SwapchainPresent);
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
+    Serialise_Present(ser, SyncInterval, Flags);
+
+    m_ContextRecord->AddChunk(scope.Get());
+  }
 }
 
-void WrappedID3D11DeviceContext::FreeCaptureData()
+bool WrappedID3D11DeviceContext::ShadowStorageInUse(D3D11ResourceRecord *record)
 {
-  SCOPED_LOCK(m_pDevice->D3DLock());
+  ResourceId id = record->GetResourceID();
 
-  for(auto it = WrappedID3D11Buffer::m_BufferList.begin();
-      it != WrappedID3D11Buffer::m_BufferList.end(); ++it)
-  {
-    D3D11ResourceRecord *record = m_pDevice->GetResourceManager()->GetResourceRecord(it->first);
+  for(auto mapit = m_OpenMaps.begin(); mapit != m_OpenMaps.end(); ++mapit)
+    if(mapit->first.resource == id)
+      return true;
 
-    if(record == NULL)
-      continue;
-
-    bool inuse = false;
-    for(auto mapit = m_OpenMaps.begin(); mapit != m_OpenMaps.end(); ++mapit)
-    {
-      if(mapit->first.resource == it->first)
-      {
-        inuse = true;
-        break;
-      }
-    }
-
-    if(inuse)
-      continue;
-
-    record->FreeShadowStorage();
-  }
+  return false;
 }
 
 void WrappedID3D11DeviceContext::CleanupCapture()
@@ -620,14 +635,6 @@ void WrappedID3D11DeviceContext::CleanupCapture()
   m_ContextRecord->UnlockChunks();
 
   m_ContextRecord->FreeParents(m_pDevice->GetResourceManager());
-
-  for(auto it = m_MissingTracks.begin(); it != m_MissingTracks.end(); ++it)
-  {
-    if(m_pDevice->GetResourceManager()->HasResourceRecord(*it))
-      MarkDirtyResource(*it);
-  }
-
-  m_MissingTracks.clear();
 }
 
 void WrappedID3D11DeviceContext::BeginFrame()
@@ -641,9 +648,6 @@ void WrappedID3D11DeviceContext::BeginFrame()
 void WrappedID3D11DeviceContext::EndFrame()
 {
   DrainAnnotationQueue();
-
-  if(IsBackgroundCapturing(m_State))
-    m_pDevice->GetResourceManager()->FlushPendingDirty();
 }
 
 bool WrappedID3D11DeviceContext::IsFL11_1()
@@ -653,7 +657,7 @@ bool WrappedID3D11DeviceContext::IsFL11_1()
 
 bool WrappedID3D11DeviceContext::ProcessChunk(ReadSerialiser &ser, D3D11Chunk chunk)
 {
-  SERIALISE_ELEMENT(m_CurContextId).Named("Context").TypedAs("ID3D11DeviceContext *");
+  SERIALISE_ELEMENT(m_CurContextId).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -856,47 +860,86 @@ bool WrappedID3D11DeviceContext::ProcessChunk(ReadSerialiser &ser, D3D11Chunk ch
       break;
 
     case D3D11Chunk::SwapchainPresent:
-    {
-      // we don't do anything with these parameters, they're just here to store
-      // them for user benefits
-      UINT SyncInterval = 0, Flags = 0;
-      SERIALISE_ELEMENT(SyncInterval);
-      SERIALISE_ELEMENT(Flags);
+      ret = Serialise_Present(ser, 0, 0);
+      break;
 
-      SERIALISE_CHECK_READ_ERRORS();
+    // in order to get a warning if we miss a case, we explicitly handle the device creation chunks
+    // here. If we actually encounter one it's an error (we shouldn't see these inside the captured
+    // frame itself)
+    case D3D11Chunk::DeviceInitialisation:
+    case D3D11Chunk::SetResourceName:
+    case D3D11Chunk::CreateSwapBuffer:
+    case D3D11Chunk::CreateTexture1D:
+    case D3D11Chunk::CreateTexture2D:
+    case D3D11Chunk::CreateTexture2D1:
+    case D3D11Chunk::CreateTexture3D:
+    case D3D11Chunk::CreateTexture3D1:
+    case D3D11Chunk::CreateBuffer:
+    case D3D11Chunk::CreateVertexShader:
+    case D3D11Chunk::CreateHullShader:
+    case D3D11Chunk::CreateDomainShader:
+    case D3D11Chunk::CreateGeometryShader:
+    case D3D11Chunk::CreateGeometryShaderWithStreamOutput:
+    case D3D11Chunk::CreatePixelShader:
+    case D3D11Chunk::CreateComputeShader:
+    case D3D11Chunk::GetClassInstance:
+    case D3D11Chunk::CreateClassInstance:
+    case D3D11Chunk::CreateClassLinkage:
+    case D3D11Chunk::CreateShaderResourceView:
+    case D3D11Chunk::CreateShaderResourceView1:
+    case D3D11Chunk::CreateRenderTargetView:
+    case D3D11Chunk::CreateRenderTargetView1:
+    case D3D11Chunk::CreateDepthStencilView:
+    case D3D11Chunk::CreateUnorderedAccessView:
+    case D3D11Chunk::CreateUnorderedAccessView1:
+    case D3D11Chunk::CreateInputLayout:
+    case D3D11Chunk::CreateBlendState:
+    case D3D11Chunk::CreateBlendState1:
+    case D3D11Chunk::CreateDepthStencilState:
+    case D3D11Chunk::CreateRasterizerState:
+    case D3D11Chunk::CreateRasterizerState1:
+    case D3D11Chunk::CreateRasterizerState2:
+    case D3D11Chunk::CreateSamplerState:
+    case D3D11Chunk::CreateQuery:
+    case D3D11Chunk::CreateQuery1:
+    case D3D11Chunk::CreatePredicate:
+    case D3D11Chunk::CreateCounter:
+    case D3D11Chunk::CreateDeferredContext:
+    case D3D11Chunk::SetExceptionMode:
+    case D3D11Chunk::ExternalDXGIResource:
+    case D3D11Chunk::OpenSharedResource:
+    case D3D11Chunk::OpenSharedResource1:
+    case D3D11Chunk::OpenSharedResourceByName:
+    case D3D11Chunk::SetShaderDebugPath:
+      RDCERR("Unexpected chunk while processing frame: %s", ToStr(chunk).c_str());
+      return false;
+
+    // no explicit default so that we have compiler warnings if a chunk isn't explicitly handled.
+    case D3D11Chunk::Max: break;
+  }
+
+  {
+    SystemChunk system = (SystemChunk)chunk;
+
+    if(system == SystemChunk::CaptureEnd)
+    {
+      if(IsLoading(m_State) && m_LastChunk != D3D11Chunk::SwapchainPresent)
+      {
+        DrawcallDescription draw;
+        draw.name = "End of Capture";
+        draw.flags |= DrawFlags::Present;
+
+        draw.copyDestination = m_pDevice->GetBackbufferResourceID();
+
+        AddDrawcall(draw, true);
+      }
 
       ret = true;
-
-      m_PresentChunk = true;
-      break;
     }
-    default:
+    else if(!ret)
     {
-      SystemChunk system = (SystemChunk)chunk;
-
-      if(system == SystemChunk::CaptureEnd)
-      {
-        if(IsLoading(m_State))
-        {
-          if(!m_PresentChunk)
-            AddEvent();
-
-          DrawcallDescription draw;
-          draw.name = "End of Frame";
-          draw.flags |= DrawFlags::Present;
-
-          draw.copyDestination = m_pDevice->GetBackbufferResourceID();
-
-          AddDrawcall(draw, true);
-        }
-
-        ret = true;
-      }
-      else
-      {
-        RDCERR("Unrecognised Chunk type %d", chunk);
-      }
-      break;
+      RDCERR("Unrecognised Chunk type %d", chunk);
+      return false;
     }
   }
 
@@ -1101,18 +1144,22 @@ void WrappedID3D11DeviceContext::AddEvent()
   m_CurEvents.push_back(apievent);
 
   if(IsLoading(m_State))
-    m_Events.push_back(apievent);
+  {
+    m_Events.resize(apievent.eventId + 1);
+    m_Events[apievent.eventId] = apievent;
+  }
 }
 
-const APIEvent &WrappedID3D11DeviceContext::GetEvent(uint32_t eventId)
+const APIEvent &WrappedID3D11DeviceContext::GetEvent(uint32_t eventId) const
 {
-  for(const APIEvent &e : m_Events)
-  {
-    if(e.eventId >= eventId)
-      return e;
-  }
+  // start at where the requested eventId would be
+  size_t idx = eventId;
 
-  return m_Events.back();
+  // find the next valid event (some may be skipped)
+  while(idx < m_Events.size() - 1 && m_Events[idx].eventId == 0)
+    idx++;
+
+  return m_Events[RDCMIN(idx, m_Events.size() - 1)];
 }
 
 void WrappedID3D11DeviceContext::ReplayFakeContext(ResourceId id)
@@ -1224,13 +1271,15 @@ ReplayStatus WrappedID3D11DeviceContext::ReplayLog(CaptureState readType, uint32
     if((SystemChunk)chunktype == SystemChunk::CaptureEnd)
       break;
 
+    m_LastChunk = chunktype;
     m_CurEventID++;
   }
 
   if(IsLoading(m_State))
   {
-    m_pDevice->GetFrameRecord().drawcallList = m_ParentDrawcall.children;
-    m_pDevice->GetFrameRecord().frameInfo.debugMessages = m_pDevice->GetDebugMessages();
+    m_pDevice->GetReplay()->WriteFrameRecord().drawcallList = m_ParentDrawcall.children;
+    m_pDevice->GetReplay()->WriteFrameRecord().frameInfo.debugMessages =
+        m_pDevice->GetDebugMessages();
 
     for(auto it = WrappedID3D11Buffer::m_BufferList.begin();
         it != WrappedID3D11Buffer::m_BufferList.end(); ++it)
@@ -1257,9 +1306,9 @@ ReplayStatus WrappedID3D11DeviceContext::ReplayLog(CaptureState readType, uint32
     // we don't have duplicate uses
     for(auto it = m_ResourceUses.begin(); it != m_ResourceUses.end(); ++it)
     {
-      vector<EventUsage> &v = it->second;
+      rdcarray<EventUsage> &v = it->second;
       std::sort(v.begin(), v.end());
-      v.erase(std::unique(v.begin(), v.end()), v.end());
+      v.erase(std::unique(v.begin(), v.end()) - v.begin(), ~0U);
 
 #if CHECK_UNUSED_INITIAL_STATES
       ResourceId resid = m_pDevice->GetResourceManager()->GetOriginalID(it->first);
