@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,17 +25,12 @@
 
 #pragma once
 
-#include <list>
 #include <map>
-#include "api/replay/renderdoc_replay.h"
 #include "core/core.h"
 #include "d3d11_common.h"
 #include "d3d11_manager.h"
 #include "d3d11_video.h"
 
-using std::map;
-using std::list;
-
 struct MapIntercept
 {
   MapIntercept()
@@ -121,11 +116,11 @@ private:
     }
   };
 
-  set<ResourceId> m_DeferredDirty;
-  set<ResourceId> m_DeferredReferences;
+  std::set<ResourceId> m_DeferredDirty;
+  std::set<ResourceId> m_DeferredReferences;
 
-  set<ResourceId> m_HighTrafficResources;
-  map<MappedResource, MapIntercept> m_OpenMaps;
+  std::set<ResourceId> m_HighTrafficResources;
+  std::map<MappedResource, MapIntercept> m_OpenMaps;
 
   struct StreamOutData
   {
@@ -135,9 +130,9 @@ private:
     uint64_t numPrims;
   };
 
-  map<ResourceId, StreamOutData> m_StreamOutCounters;
+  std::map<ResourceId, StreamOutData> m_StreamOutCounters;
 
-  map<ResourceId, vector<EventUsage> > m_ResourceUses;
+  std::map<ResourceId, rdcarray<EventUsage> > m_ResourceUses;
 
   WrappedID3D11Device *m_pDevice;
   ID3D11DeviceContext *m_pRealContext;
@@ -153,15 +148,13 @@ private:
   bool m_NeedUpdateSubWorkaround;
 
   WriteSerialiser m_ScratchSerialiser;
-  std::set<std::string> m_StringDB;
+  std::set<rdcstr> m_StringDB;
 
   ResourceId m_CurContextId;
 
   StreamReader *m_FrameReader = NULL;
 
-  map<ResourceId, int> m_MapResourceRecordAllocs;
-
-  set<ResourceId> m_MissingTracks;
+  std::map<ResourceId, size_t> m_MapResourceRecordAllocs;
 
   ResourceId m_ResourceID;
   D3D11ResourceRecord *m_ContextRecord;
@@ -181,8 +174,6 @@ private:
     RenderDoc::Inst().AddActiveDriver(RDCDriver::D3D11, false);
   }
 
-  bool m_PresentChunk;
-
   ResourceId m_FakeContext;
 
   bool m_DoStateVerify;
@@ -190,7 +181,7 @@ private:
 
   D3D11RenderState *m_DeferredSavedState;
 
-  vector<APIEvent> m_CurEvents, m_Events;
+  rdcarray<APIEvent> m_CurEvents, m_Events;
   bool m_AddedDrawcall;
 
   bool HasNonMarkerEvents();
@@ -207,9 +198,9 @@ private:
       ANNOT_ENDEVENT
     } m_Type;
     uint32_t m_Col;
-    std::wstring m_Name;
+    rdcstr m_Name;
   };
-  vector<Annotation> m_AnnotationQueue;
+  rdcarray<Annotation> m_AnnotationQueue;
   Threading::CriticalSection m_AnnotLock;
 
   SDFile *m_StructuredFile = NULL;
@@ -217,16 +208,17 @@ private:
   uint64_t m_CurChunkOffset;
   SDChunkMetaData m_ChunkMetadata;
   uint32_t m_CurEventID, m_CurDrawcallID;
+  D3D11Chunk m_LastChunk;
 
   ReplayStatus m_FailedReplayStatus = ReplayStatus::APIReplayFailed;
 
   DrawcallDescription m_ParentDrawcall;
-  map<ResourceId, DrawcallDescription> m_CmdLists;
+  std::map<ResourceId, DrawcallDescription> m_CmdLists;
 
-  list<DrawcallDescription *> m_DrawcallStack;
+  rdcarray<DrawcallDescription *> m_DrawcallStack;
 
   D3D11ResourceManager *GetResourceManager();
-  static std::string GetChunkName(uint32_t idx);
+  static rdcstr GetChunkName(uint32_t idx);
 
   template <typename SerialiserType>
   void Serialise_DebugMessages(SerialiserType &ser);
@@ -292,10 +284,10 @@ public:
   void MarkDirtyResource(ResourceId id);
 
   // insert a fake chunk just to store these parameters
-  void Present(UINT SyncInterval, UINT Flags);
+  IMPLEMENT_FUNCTION_SERIALISED(void, Present, UINT SyncInterval, UINT Flags);
 
   void CleanupCapture();
-  void FreeCaptureData();
+  bool ShadowStorageInUse(D3D11ResourceRecord *record);
 
   bool HasSuccessfulCapture(CaptureFailReason &reason)
   {
@@ -321,11 +313,11 @@ public:
   void SetFrameReader(StreamReader *reader) { m_FrameReader = reader; }
   void MarkResourceReferenced(ResourceId id, FrameRefType refType);
 
-  vector<EventUsage> GetUsage(ResourceId id) { return m_ResourceUses[id]; }
+  rdcarray<EventUsage> GetUsage(ResourceId id) { return m_ResourceUses[id]; }
   void ClearMaps();
 
   uint32_t GetEventID() { return m_CurEventID; }
-  const APIEvent &GetEvent(uint32_t eventId);
+  const APIEvent &GetEvent(uint32_t eventId) const;
 
   const DrawcallDescription &GetRootDraw() { return m_ParentDrawcall; }
   void ThreadSafe_SetMarker(uint32_t col, const wchar_t *name);
