@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -373,13 +373,13 @@ void WrappedID3D11DeviceContext::UpdateSubresource1(ID3D11Resource *pDstResource
   {
     USE_SCRATCH_SERIALISER();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::UpdateSubresource1);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_UpdateSubresource1(ser, pDstResource, DstSubresource, pDstBox, pSrcData, SrcRowPitch,
                                  SrcDepthPitch, CopyFlags);
 
     MarkResourceReferenced(GetIDForResource(pDstResource), eFrameRef_PartialWrite);
 
-    m_MissingTracks.insert(GetIDForResource(pDstResource));
+    MarkDirtyResource(GetIDForResource(pDstResource));
 
     m_ContextRecord->AddChunk(scope.Get());
   }
@@ -459,11 +459,11 @@ void WrappedID3D11DeviceContext::CopySubresourceRegion1(ID3D11Resource *pDstReso
   {
     USE_SCRATCH_SERIALISER();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::CopySubresourceRegion1);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_CopySubresourceRegion1(ser, pDstResource, DstSubresource, DstX, DstY, DstZ,
                                      pSrcResource, SrcSubresource, pSrcBox, CopyFlags);
 
-    m_MissingTracks.insert(GetIDForResource(pDstResource));
+    MarkDirtyResource(GetIDForResource(pDstResource));
 
     m_ContextRecord->AddChunk(scope.Get());
   }
@@ -615,13 +615,13 @@ void WrappedID3D11DeviceContext::ClearView(ID3D11View *pView, const FLOAT Color[
       USE_SCRATCH_SERIALISER();
       ser.SetDrawChunk();
       SCOPED_SERIALISE_CHUNK(D3D11Chunk::ClearView);
-      SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+      SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
       Serialise_ClearView(ser, pView, Color, pRect, NumRects);
 
       ID3D11Resource *viewRes = NULL;
       pView->GetResource(&viewRes);
 
-      m_MissingTracks.insert(GetIDForResource(viewRes));
+      MarkDirtyResource(GetIDForResource(viewRes));
       MarkResourceReferenced(GetIDForResource(viewRes), eFrameRef_PartialWrite);
 
       SAFE_RELEASE(viewRes);
@@ -732,7 +732,7 @@ void WrappedID3D11DeviceContext::VSSetConstantBuffers1(UINT StartSlot, UINT NumB
   {
     USE_SCRATCH_SERIALISER();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::VSSetConstantBuffers1);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_VSSetConstantBuffers1(ser, StartSlot, NumBuffers, ppConstantBuffers, pFirstConstant,
                                     pNumConstants);
 
@@ -863,7 +863,7 @@ void WrappedID3D11DeviceContext::HSSetConstantBuffers1(UINT StartSlot, UINT NumB
   {
     USE_SCRATCH_SERIALISER();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::HSSetConstantBuffers1);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_HSSetConstantBuffers1(ser, StartSlot, NumBuffers, ppConstantBuffers, pFirstConstant,
                                     pNumConstants);
 
@@ -994,7 +994,7 @@ void WrappedID3D11DeviceContext::DSSetConstantBuffers1(UINT StartSlot, UINT NumB
   {
     USE_SCRATCH_SERIALISER();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::DSSetConstantBuffers1);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_DSSetConstantBuffers1(ser, StartSlot, NumBuffers, ppConstantBuffers, pFirstConstant,
                                     pNumConstants);
 
@@ -1125,7 +1125,7 @@ void WrappedID3D11DeviceContext::GSSetConstantBuffers1(UINT StartSlot, UINT NumB
   {
     USE_SCRATCH_SERIALISER();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::GSSetConstantBuffers1);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_GSSetConstantBuffers1(ser, StartSlot, NumBuffers, ppConstantBuffers, pFirstConstant,
                                     pNumConstants);
 
@@ -1256,7 +1256,7 @@ void WrappedID3D11DeviceContext::PSSetConstantBuffers1(UINT StartSlot, UINT NumB
   {
     USE_SCRATCH_SERIALISER();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::PSSetConstantBuffers1);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_PSSetConstantBuffers1(ser, StartSlot, NumBuffers, ppConstantBuffers, pFirstConstant,
                                     pNumConstants);
 
@@ -1387,7 +1387,7 @@ void WrappedID3D11DeviceContext::CSSetConstantBuffers1(UINT StartSlot, UINT NumB
   {
     USE_SCRATCH_SERIALISER();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::CSSetConstantBuffers1);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_CSSetConstantBuffers1(ser, StartSlot, NumBuffers, ppConstantBuffers, pFirstConstant,
                                     pNumConstants);
 
@@ -1765,10 +1765,10 @@ void WrappedID3D11DeviceContext::DiscardResource(ID3D11Resource *pResource)
     USE_SCRATCH_SERIALISER();
     ser.SetDrawChunk();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::DiscardResource);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_DiscardResource(ser, pResource);
 
-    m_MissingTracks.insert(GetIDForResource(pResource));
+    MarkDirtyResource(GetIDForResource(pResource));
     MarkResourceReferenced(GetIDForResource(pResource), eFrameRef_PartialWrite);
 
     m_ContextRecord->AddChunk(scope.Get());
@@ -1918,13 +1918,13 @@ void WrappedID3D11DeviceContext::DiscardView(ID3D11View *pResourceView)
       USE_SCRATCH_SERIALISER();
       ser.SetDrawChunk();
       SCOPED_SERIALISE_CHUNK(D3D11Chunk::DiscardView);
-      SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+      SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
       Serialise_DiscardView(ser, pResourceView);
 
       ID3D11Resource *viewRes = NULL;
       pResourceView->GetResource(&viewRes);
 
-      m_MissingTracks.insert(GetIDForResource(viewRes));
+      MarkDirtyResource(GetIDForResource(viewRes));
       MarkResourceReferenced(GetIDForResource(viewRes), eFrameRef_PartialWrite);
 
       SAFE_RELEASE(viewRes);
@@ -2086,13 +2086,13 @@ void WrappedID3D11DeviceContext::DiscardView1(ID3D11View *pResourceView, const D
       USE_SCRATCH_SERIALISER();
       ser.SetDrawChunk();
       SCOPED_SERIALISE_CHUNK(D3D11Chunk::DiscardView1);
-      SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+      SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
       Serialise_DiscardView1(ser, pResourceView, pRects, NumRects);
 
       ID3D11Resource *viewRes = NULL;
       pResourceView->GetResource(&viewRes);
 
-      m_MissingTracks.insert(GetIDForResource(viewRes));
+      MarkDirtyResource(GetIDForResource(viewRes));
       MarkResourceReferenced(GetIDForResource(viewRes), eFrameRef_PartialWrite);
 
       SAFE_RELEASE(viewRes);
@@ -2125,7 +2125,7 @@ bool WrappedID3D11DeviceContext::Serialise_SwapDeviceContextState(
     state.MarkReferenced(this, true);
   }
 
-  SERIALISE_ELEMENT(state).Named("pState");
+  SERIALISE_ELEMENT(state).Named("pState"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -2192,7 +2192,7 @@ void WrappedID3D11DeviceContext::SwapDeviceContextState(ID3DDeviceContextState *
   {
     USE_SCRATCH_SERIALISER();
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::SwapDeviceContextState);
-    SERIALISE_ELEMENT(m_ResourceID).Named("Context").TypedAs("ID3D11DeviceContext *");
+    SERIALISE_ELEMENT(m_ResourceID).Named("Context"_lit).TypedAs("ID3D11DeviceContext *"_lit);
     Serialise_SwapDeviceContextState(ser, pState, NULL);
 
     m_ContextRecord->AddChunk(scope.Get());
@@ -2353,4 +2353,4 @@ TEST_CASE("Check UpdateSubresource data length calculations", "[d3d]")
   }
 }
 
-#endif
\ No newline at end of file
+#endif
