@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -34,6 +34,7 @@
 #include "d3d11_debug.h"
 #include "d3d11_renderstate.h"
 #include "d3d11_rendertext.h"
+#include "d3d11_replay.h"
 #include "d3d11_resources.h"
 #include "d3d11_shader_cache.h"
 
@@ -101,22 +102,17 @@ WrappedID3D11Device::WrappedID3D11Device(ID3D11Device *realDevice, D3D11InitPara
   m_WrappedMultithread.m_pDevice = this;
   m_WrappedVideo.m_pDevice = this;
 
-  m_FrameCounter = 0;
-  m_FailedFrame = 0;
   m_FailedReason = CaptureSucceeded;
-  m_Failures = 0;
 
   m_ChunkAtomic = 0;
 
-  m_AppControlledCapture = false;
-
   if(RenderDoc::Inst().IsReplayApp())
   {
     m_State = CaptureState::LoadingReplaying;
 
     D3D11MarkerRegion::device = this;
 
-    string shaderSearchPathString = RenderDoc::Inst().GetConfigSetting("shader.debug.searchPaths");
+    rdcstr shaderSearchPathString = RenderDoc::Inst().GetConfigSetting("shader.debug.searchPaths");
     split(shaderSearchPathString, m_ShaderSearchPaths, ';');
 
     ResourceIDGen::SetReplayResourceIDs();
@@ -126,7 +122,7 @@ WrappedID3D11Device::WrappedID3D11Device(ID3D11Device *realDevice, D3D11InitPara
     m_State = CaptureState::BackgroundCapturing;
   }
 
-  m_ResourceManager = new D3D11ResourceManager(this);
+  m_ResourceManager = new D3D11ResourceManager(m_State, this);
 
   m_ShaderCache = new D3D11ShaderCache(this);
 
@@ -137,9 +133,12 @@ WrappedID3D11Device::WrappedID3D11Device(ID3D11Device *realDevice, D3D11InitPara
 
   m_DeviceRecord = NULL;
 
+  m_InitParams = params;
+
   if(!RenderDoc::Inst().IsReplayApp())
   {
     m_DeviceRecord = GetResourceManager()->AddResourceRecord(m_ResourceID);
+    m_DeviceRecord->ResType = Resource_Unknown;
     m_DeviceRecord->DataInSerialiser = false;
     m_DeviceRecord->InternalResource = true;
     m_DeviceRecord->Length = 0;
@@ -166,8 +165,10 @@ WrappedID3D11Device::WrappedID3D11Device(ID3D11Device *realDevice, D3D11InitPara
           DXGI_ADAPTER_DESC desc = {};
           pDXGIAdapter->GetDesc(&desc);
 
+          m_InitParams.AdapterDesc = desc;
+
           GPUVendor vendor = GPUVendorFromPCIVendor(desc.VendorId);
-          std::string descString = GetDriverVersion(desc);
+          rdcstr descString = GetDriverVersion(desc);
 
           RDCLOG("New D3D11 device created: %s / %s", ToStr(vendor).c_str(), descString.c_str());
 
@@ -188,6 +189,8 @@ WrappedID3D11Device::WrappedID3D11Device(ID3D11Device *realDevice, D3D11InitPara
   m_pImmediateContext->GetScratchSerialiser().SetChunkMetadataRecording(
       m_ScratchSerialiser.GetChunkMetadataRecording());
 
+  m_Replay = new D3D11Replay(this);
+
   m_pInfoQueue = NULL;
   if(realDevice)
   {
@@ -235,10 +238,6 @@ WrappedID3D11Device::WrappedID3D11Device(ID3D11Device *realDevice, D3D11InitPara
     RDCDEBUG("Couldn't get ID3D11InfoQueue.");
   }
 
-  m_Replay.SetDevice(this);
-
-  m_InitParams = params;
-
   if(realDevice)
     m_DebugManager = new D3D11DebugManager(this);
 
@@ -294,7 +293,7 @@ WrappedID3D11Device::~WrappedID3D11Device()
   for(auto it = m_SwapChains.begin(); it != m_SwapChains.end(); ++it)
     SAFE_RELEASE(it->second);
 
-  m_Replay.DestroyResources();
+  m_Replay->DestroyResources();
 
   SAFE_DELETE(m_DebugManager);
   SAFE_DELETE(m_TextRenderer);
@@ -331,6 +330,8 @@ WrappedID3D11Device::~WrappedID3D11Device()
     RDCASSERT(WrappedID3D11Texture3D1::m_TextureList.empty());
   }
 
+  delete m_Replay;
+
   if(RenderDoc::Inst().GetCrashHandler())
     RenderDoc::Inst().GetCrashHandler()->UnregisterMemoryRegion(this);
 }
@@ -484,6 +485,10 @@ HRESULT WrappedID3D11Device::QueryInterface(REFIID riid, void **ppvObject)
   static const GUID ID3D10Device_uuid = {
       0x9b7e4c0f, 0x342c, 0x4106, {0xa1, 0x9f, 0x4f, 0x27, 0x04, 0xf6, 0x89, 0xf0}};
 
+  // ID3D10DeviceChild UUID {9B7E4C00-342C-4106-A19F-4F2704F689F0}
+  static const GUID ID3D10DeviceChild_uuid = {
+      0x9b7e4c00, 0x342c, 0x4106, {0xa1, 0x9f, 0x4f, 0x27, 0x04, 0xf6, 0x89, 0xf0}};
+
   // ID3D12Device UUID {189819f1-1db6-4b57-be54-1821339b85f7}
   static const GUID ID3D12Device_uuid = {
       0x189819f1, 0x1db6, 0x4b57, {0xbe, 0x54, 0x18, 0x21, 0x33, 0x9b, 0x85, 0xf7}};
@@ -492,6 +497,10 @@ HRESULT WrappedID3D11Device::QueryInterface(REFIID riid, void **ppvObject)
   static const GUID ID3D11ShaderTraceFactory_uuid = {
       0x1fbad429, 0x66ab, 0x41cc, {0x96, 0x17, 0x66, 0x7a, 0xc1, 0x0e, 0x44, 0x59}};
 
+  // ID3D11On12Device UUID {85611e73-70a9-490e-9614-a9e302777904}
+  static const GUID ID3D11On12Device_uuid = {
+      0x85611e73, 0x70a9, 0x490e, {0x96, 0x14, 0xa9, 0xe3, 0x02, 0x77, 0x79, 0x04}};
+
   // RenderDoc UUID {A7AA6116-9C8D-4BBA-9083-B4D816B71B78}
   static const GUID IRenderDoc_uuid = {
       0xa7aa6116, 0x9c8d, 0x4bba, {0x90, 0x83, 0xb4, 0xd8, 0x16, 0xb7, 0x1b, 0x78}};
@@ -572,6 +581,22 @@ HRESULT WrappedID3D11Device::QueryInterface(REFIID riid, void **ppvObject)
       return hr;
     }
   }
+  else if(riid == __uuidof(IDXGIDevice4))
+  {
+    hr = m_pDevice->QueryInterface(riid, ppvObject);
+
+    if(SUCCEEDED(hr))
+    {
+      IDXGIDevice4 *real = (IDXGIDevice4 *)(*ppvObject);
+      *ppvObject = (IDXGIDevice4 *)(new WrappedIDXGIDevice4(real, this));
+      return S_OK;
+    }
+    else
+    {
+      *ppvObject = NULL;
+      return hr;
+    }
+  }
   else if(riid == __uuidof(ID3D11Device))
   {
     AddRef();
@@ -584,6 +609,12 @@ HRESULT WrappedID3D11Device::QueryInterface(REFIID riid, void **ppvObject)
     *ppvObject = NULL;
     return E_NOINTERFACE;
   }
+  else if(riid == ID3D10DeviceChild_uuid)
+  {
+    RDCWARN("Trying to get ID3D10DeviceChild - not supported.");
+    *ppvObject = NULL;
+    return E_NOINTERFACE;
+  }
   else if(riid == ID3D12Device_uuid)
   {
     RDCWARN("Trying to get ID3D12Device - not supported.");
@@ -675,6 +706,12 @@ HRESULT WrappedID3D11Device::QueryInterface(REFIID riid, void **ppvObject)
     *ppvObject = NULL;
     return E_NOINTERFACE;
   }
+  else if(riid == ID3D11On12Device_uuid)
+  {
+    RDCWARN("Trying to get ID3D11On12Device. Not supported at this time.");
+    *ppvObject = NULL;
+    return E_NOINTERFACE;
+  }
   else if(riid == __uuidof(ID3D11InfoQueue))
   {
     RDCWARN(
@@ -744,7 +781,7 @@ HRESULT WrappedID3D11Device::QueryInterface(REFIID riid, void **ppvObject)
   return m_RefCounter.QueryInterface(riid, ppvObject);
 }
 
-std::string WrappedID3D11Device::GetChunkName(uint32_t idx)
+rdcstr WrappedID3D11Device::GetChunkName(uint32_t idx)
 {
   if((SystemChunk)idx < SystemChunk::FirstDriverChunk)
     return ToStr((SystemChunk)idx);
@@ -753,7 +790,7 @@ std::string WrappedID3D11Device::GetChunkName(uint32_t idx)
 }
 
 void WrappedID3D11Device::AddDebugMessage(MessageCategory c, MessageSeverity sv, MessageSource src,
-                                          std::string d)
+                                          rdcstr d)
 {
   // Only add runtime warnings while executing.
   // While reading, add the messages from the log, and while writing add messages
@@ -778,17 +815,30 @@ void WrappedID3D11Device::AddDebugMessage(DebugMessage msg)
     m_DebugMessages.push_back(msg);
 }
 
-std::vector<DebugMessage> WrappedID3D11Device::GetDebugMessages()
+rdcarray<DebugMessage> WrappedID3D11Device::GetDebugMessages()
 {
-  std::vector<DebugMessage> ret;
+  rdcarray<DebugMessage> ret;
 
-  // if reading, m_DebugMessages will contain all the messages (we
-  // don't try and fetch anything from the API). If writing,
-  // m_DebugMessages will contain any manually-added messages.
-  ret.swap(m_DebugMessages);
+  if(IsActiveReplaying(m_State))
+  {
+    // once we're active replaying, m_DebugMessages will contain all the messages from loading
+    // (either from the captured serialised messages, or from ourselves during replay).
+    ret.swap(m_DebugMessages);
 
-  if(IsReplayMode(m_State))
     return ret;
+  }
+
+  if(IsCaptureMode(m_State))
+  {
+    // add any manually-added messages before fetching those from the API
+    ret.swap(m_DebugMessages);
+  }
+
+  // during loading only try and fetch messages if we're doing that deliberately during replay
+  if(IsLoading(m_State) && !m_ReplayOptions.apiValidation)
+    return ret;
+
+  // during capture, and during loading if the option is enabled, we fetch messages from the API
 
   if(!m_pInfoQueue)
     return ret;
@@ -844,15 +894,15 @@ std::vector<DebugMessage> WrappedID3D11Device::GetDebugMessages()
     switch(message->Severity)
     {
       case D3D11_MESSAGE_SEVERITY_CORRUPTION: msg.severity = MessageSeverity::High; break;
-      case D3D11_MESSAGE_SEVERITY_ERROR: msg.severity = MessageSeverity::Medium; break;
-      case D3D11_MESSAGE_SEVERITY_WARNING: msg.severity = MessageSeverity::Low; break;
-      case D3D11_MESSAGE_SEVERITY_INFO: msg.severity = MessageSeverity::Info; break;
+      case D3D11_MESSAGE_SEVERITY_ERROR: msg.severity = MessageSeverity::High; break;
+      case D3D11_MESSAGE_SEVERITY_WARNING: msg.severity = MessageSeverity::Medium; break;
+      case D3D11_MESSAGE_SEVERITY_INFO: msg.severity = MessageSeverity::Low; break;
       case D3D11_MESSAGE_SEVERITY_MESSAGE: msg.severity = MessageSeverity::Info; break;
       default: RDCWARN("Unexpected message severity: %d", message->Severity); break;
     }
 
     msg.messageID = (uint32_t)message->ID;
-    msg.description = string(message->pDescription);
+    msg.description = rdcstr(message->pDescription);
 
     ret.push_back(msg);
 
@@ -879,7 +929,7 @@ bool WrappedID3D11Device::ProcessChunk(ReadSerialiser &ser, D3D11Chunk context)
   {
     case D3D11Chunk::DeviceInitialisation:
     {
-      SERIALISE_ELEMENT_LOCAL(ImmediateContext, ResourceId()).TypedAs("ID3D11DeviceContext *");
+      SERIALISE_ELEMENT_LOCAL(ImmediateContext, ResourceId()).TypedAs("ID3D11DeviceContext *"_lit);
 
       SERIALISE_CHECK_READ_ERRORS();
 
@@ -901,7 +951,8 @@ bool WrappedID3D11Device::ProcessChunk(ReadSerialiser &ser, D3D11Chunk context)
       return true;
     }
     case D3D11Chunk::SetResourceName: return Serialise_SetResourceName(ser, 0x0, "");
-    case D3D11Chunk::CreateSwapBuffer: return Serialise_WrapSwapchainBuffer(ser, 0x0, 0x0, 0, 0x0);
+    case D3D11Chunk::CreateSwapBuffer:
+      return Serialise_WrapSwapchainBuffer(ser, 0x0, DXGI_FORMAT_UNKNOWN, 0, 0x0);
 
     case D3D11Chunk::CreateTexture1D: return Serialise_CreateTexture1D(ser, 0x0, 0x0, 0x0);
     case D3D11Chunk::CreateTexture2D: return Serialise_CreateTexture2D(ser, 0x0, 0x0, 0x0);
@@ -975,49 +1026,145 @@ bool WrappedID3D11Device::ProcessChunk(ReadSerialiser &ser, D3D11Chunk context)
     case D3D11Chunk::CreateDeferredContext: return Serialise_CreateDeferredContext(ser, 0, 0x0);
 
     case D3D11Chunk::SetExceptionMode: return Serialise_SetExceptionMode(ser, 0);
+    case D3D11Chunk::ExternalDXGIResource:
     case D3D11Chunk::OpenSharedResource:
+    case D3D11Chunk::OpenSharedResource1:
+    case D3D11Chunk::OpenSharedResourceByName:
     {
       IID nul;
       return Serialise_OpenSharedResource(ser, 0, nul, NULL);
     }
-    case D3D11Chunk::SetShaderDebugPath: return Serialise_SetShaderDebugPath(ser, NULL, NULL);
-    default:
+    case D3D11Chunk::SetShaderDebugPath:
+      return Serialise_SetShaderDebugPath(ser, NULL, NULL);
+
+    // In order to get a warning if we miss a case, we explicitly handle the context chunks here.
+    // for legacy reasons we forward to the immediate context's chunk processing here, since some
+    // chunks like CopyResource can be serialised in the initialisation phase.
+    case D3D11Chunk::IASetInputLayout:
+    case D3D11Chunk::IASetVertexBuffers:
+    case D3D11Chunk::IASetIndexBuffer:
+    case D3D11Chunk::IASetPrimitiveTopology:
+    case D3D11Chunk::VSSetConstantBuffers:
+    case D3D11Chunk::VSSetShaderResources:
+    case D3D11Chunk::VSSetSamplers:
+    case D3D11Chunk::VSSetShader:
+    case D3D11Chunk::HSSetConstantBuffers:
+    case D3D11Chunk::HSSetShaderResources:
+    case D3D11Chunk::HSSetSamplers:
+    case D3D11Chunk::HSSetShader:
+    case D3D11Chunk::DSSetConstantBuffers:
+    case D3D11Chunk::DSSetShaderResources:
+    case D3D11Chunk::DSSetSamplers:
+    case D3D11Chunk::DSSetShader:
+    case D3D11Chunk::GSSetConstantBuffers:
+    case D3D11Chunk::GSSetShaderResources:
+    case D3D11Chunk::GSSetSamplers:
+    case D3D11Chunk::GSSetShader:
+    case D3D11Chunk::SOSetTargets:
+    case D3D11Chunk::PSSetConstantBuffers:
+    case D3D11Chunk::PSSetShaderResources:
+    case D3D11Chunk::PSSetSamplers:
+    case D3D11Chunk::PSSetShader:
+    case D3D11Chunk::CSSetConstantBuffers:
+    case D3D11Chunk::CSSetShaderResources:
+    case D3D11Chunk::CSSetUnorderedAccessViews:
+    case D3D11Chunk::CSSetSamplers:
+    case D3D11Chunk::CSSetShader:
+    case D3D11Chunk::RSSetViewports:
+    case D3D11Chunk::RSSetScissorRects:
+    case D3D11Chunk::RSSetState:
+    case D3D11Chunk::OMSetRenderTargets:
+    case D3D11Chunk::OMSetRenderTargetsAndUnorderedAccessViews:
+    case D3D11Chunk::OMSetBlendState:
+    case D3D11Chunk::OMSetDepthStencilState:
+    case D3D11Chunk::DrawIndexedInstanced:
+    case D3D11Chunk::DrawInstanced:
+    case D3D11Chunk::DrawIndexed:
+    case D3D11Chunk::Draw:
+    case D3D11Chunk::DrawAuto:
+    case D3D11Chunk::DrawIndexedInstancedIndirect:
+    case D3D11Chunk::DrawInstancedIndirect:
+    case D3D11Chunk::Map:
+    case D3D11Chunk::Unmap:
+    case D3D11Chunk::CopySubresourceRegion:
+    case D3D11Chunk::CopyResource:
+    case D3D11Chunk::UpdateSubresource:
+    case D3D11Chunk::CopyStructureCount:
+    case D3D11Chunk::ResolveSubresource:
+    case D3D11Chunk::GenerateMips:
+    case D3D11Chunk::ClearDepthStencilView:
+    case D3D11Chunk::ClearRenderTargetView:
+    case D3D11Chunk::ClearUnorderedAccessViewUint:
+    case D3D11Chunk::ClearUnorderedAccessViewFloat:
+    case D3D11Chunk::ClearState:
+    case D3D11Chunk::ExecuteCommandList:
+    case D3D11Chunk::Dispatch:
+    case D3D11Chunk::DispatchIndirect:
+    case D3D11Chunk::FinishCommandList:
+    case D3D11Chunk::Flush:
+    case D3D11Chunk::SetPredication:
+    case D3D11Chunk::SetResourceMinLOD:
+    case D3D11Chunk::Begin:
+    case D3D11Chunk::End:
+    case D3D11Chunk::CopySubresourceRegion1:
+    case D3D11Chunk::UpdateSubresource1:
+    case D3D11Chunk::ClearView:
+    case D3D11Chunk::VSSetConstantBuffers1:
+    case D3D11Chunk::HSSetConstantBuffers1:
+    case D3D11Chunk::DSSetConstantBuffers1:
+    case D3D11Chunk::GSSetConstantBuffers1:
+    case D3D11Chunk::PSSetConstantBuffers1:
+    case D3D11Chunk::CSSetConstantBuffers1:
+    case D3D11Chunk::PushMarker:
+    case D3D11Chunk::SetMarker:
+    case D3D11Chunk::PopMarker:
+    case D3D11Chunk::DiscardResource:
+    case D3D11Chunk::DiscardView:
+    case D3D11Chunk::DiscardView1:
+    case D3D11Chunk::PostExecuteCommandList:
+    case D3D11Chunk::PostFinishCommandListSet:
+    case D3D11Chunk::SwapDeviceContextState:
+    case D3D11Chunk::SwapchainPresent:
+      return m_pImmediateContext->ProcessChunk(ser, context);
+
+    // no explicit default so that we have compiler warnings if a chunk isn't explicitly handled.
+    case D3D11Chunk::Max: break;
+  }
+
+  {
+    SystemChunk system = (SystemChunk)context;
+    if(system == SystemChunk::DriverInit)
     {
-      SystemChunk system = (SystemChunk)context;
-      if(system == SystemChunk::DriverInit)
-      {
-        D3D11InitParams InitParams;
-        SERIALISE_ELEMENT(InitParams);
+      D3D11InitParams InitParams;
+      SERIALISE_ELEMENT(InitParams);
 
-        SERIALISE_CHECK_READ_ERRORS();
-      }
-      else if(system == SystemChunk::InitialContentsList)
-      {
-        GetResourceManager()->CreateInitialContents(ser);
+      SERIALISE_CHECK_READ_ERRORS();
+    }
+    else if(system == SystemChunk::InitialContentsList)
+    {
+      GetResourceManager()->CreateInitialContents(ser);
 
-        SERIALISE_CHECK_READ_ERRORS();
-      }
-      else if(system == SystemChunk::InitialContents)
-      {
-        return Serialise_InitialState(ser, ResourceId(), NULL);
-      }
-      else if(system == SystemChunk::CaptureScope)
-      {
-        return Serialise_CaptureScope(ser);
-      }
-      else if(system < SystemChunk::FirstDriverChunk)
-      {
-        RDCERR("Unexpected system chunk in capture data: %u", system);
-        ser.SkipCurrentChunk();
+      SERIALISE_CHECK_READ_ERRORS();
+    }
+    else if(system == SystemChunk::InitialContents)
+    {
+      return Serialise_InitialState(ser, ResourceId(), NULL, NULL);
+    }
+    else if(system == SystemChunk::CaptureScope)
+    {
+      return Serialise_CaptureScope(ser);
+    }
+    else if(system < SystemChunk::FirstDriverChunk)
+    {
+      RDCERR("Unexpected system chunk in capture data: %u", system);
+      ser.SkipCurrentChunk();
 
-        SERIALISE_CHECK_READ_ERRORS();
-      }
-      else
-      {
-        return m_pImmediateContext->ProcessChunk(ser, context);
-      }
-
-      break;
+      SERIALISE_CHECK_READ_ERRORS();
+    }
+    else
+    {
+      RDCERR("Unrecognised Chunk type %d", context);
+      return false;
     }
   }
 
@@ -1027,15 +1174,15 @@ bool WrappedID3D11Device::ProcessChunk(ReadSerialiser &ser, D3D11Chunk context)
 template <typename SerialiserType>
 bool WrappedID3D11Device::Serialise_CaptureScope(SerialiserType &ser)
 {
-  SERIALISE_ELEMENT(m_FrameCounter);
+  SERIALISE_ELEMENT_LOCAL(frameNumber, m_CapturedFrames.back().frameNumber);
 
   SERIALISE_CHECK_READ_ERRORS();
 
   if(IsReplayMode(m_State))
   {
-    m_FrameRecord.frameInfo.frameNumber = m_FrameCounter;
+    GetReplay()->WriteFrameRecord().frameInfo.frameNumber = frameNumber;
 
-    FrameStatistics &stats = m_FrameRecord.frameInfo.stats;
+    FrameStatistics &stats = GetReplay()->WriteFrameRecord().frameInfo.stats;
     RDCEraseEl(stats);
 
     // #mivance GL/Vulkan don't set this so don't get stats in window
@@ -1145,7 +1292,7 @@ ReplayStatus WrappedID3D11Device::ReadLogInitialisation(RDCFile *rdc, bool store
 
     if((SystemChunk)context == SystemChunk::CaptureScope)
     {
-      m_FrameRecord.frameInfo.fileOffset = offsetStart;
+      GetReplay()->WriteFrameRecord().frameInfo.fileOffset = offsetStart;
 
       // read the remaining data into memory and pass to immediate context
       frameDataSize = reader->GetSize() - reader->GetOffset();
@@ -1153,8 +1300,19 @@ ReplayStatus WrappedID3D11Device::ReadLogInitialisation(RDCFile *rdc, bool store
       m_pImmediateContext->SetFrameReader(new StreamReader(reader, frameDataSize));
 
       if(!IsStructuredExporting(m_State))
+      {
+        rdcarray<DebugMessage> savedDebugMessages;
+
+        // save any debug messages we built up
+        savedDebugMessages.swap(m_DebugMessages);
+
         GetResourceManager()->ApplyInitialContents();
 
+        // restore saved messages - which implicitly discards any generated while applying initial
+        // contents
+        savedDebugMessages.swap(m_DebugMessages);
+      }
+
       ReplayStatus status = m_pImmediateContext->ReplayLog(m_State, 0, 0, false);
 
       if(status != ReplayStatus::Succeeded)
@@ -1177,7 +1335,31 @@ ReplayStatus WrappedID3D11Device::ReadLogInitialisation(RDCFile *rdc, bool store
 
   if(!IsStructuredExporting(m_State))
   {
-    SetupDrawcallPointers(m_Drawcalls, GetFrameRecord().drawcallList);
+    SetupDrawcallPointers(m_Drawcalls, GetReplay()->WriteFrameRecord().drawcallList);
+
+    // propagate any UAV names onto counter buffers
+    rdcarray<ResourceId> counterBuffers;
+    GetDebugManager()->GetCounterBuffers(counterBuffers);
+
+    for(ResourceId buffId : counterBuffers)
+    {
+      ID3D11UnorderedAccessView *uav = GetDebugManager()->GetCounterBufferUAV(buffId);
+      ResourceId uavId = GetResourceManager()->GetOriginalID(GetIDForResource(uav));
+
+      ResourceDescription &uavDesc = GetReplay()->GetResourceDesc(uavId);
+      ResourceDescription &bufDesc = GetReplay()->GetResourceDesc(buffId);
+
+      if(uavDesc.autogeneratedName)
+      {
+        uint64_t num;
+        memcpy(&num, &uavId, sizeof(uint64_t));
+        bufDesc.SetCustomName("UAV " + ToStr(num) + " counter");
+      }
+      else
+      {
+        bufDesc.SetCustomName(uavDesc.name + " counter");
+      }
+    }
   }
 
 #if ENABLED(RDOC_DEVEL)
@@ -1194,15 +1376,16 @@ ReplayStatus WrappedID3D11Device::ReadLogInitialisation(RDCFile *rdc, bool store
   }
 #endif
 
-  m_FrameRecord.frameInfo.uncompressedFileSize =
+  GetReplay()->WriteFrameRecord().frameInfo.uncompressedFileSize =
       rdc->GetSectionProperties(sectionIdx).uncompressedSize;
-  m_FrameRecord.frameInfo.compressedFileSize = rdc->GetSectionProperties(sectionIdx).compressedSize;
-  m_FrameRecord.frameInfo.persistentSize = frameDataSize;
-  m_FrameRecord.frameInfo.initDataSize =
+  GetReplay()->WriteFrameRecord().frameInfo.compressedFileSize =
+      rdc->GetSectionProperties(sectionIdx).compressedSize;
+  GetReplay()->WriteFrameRecord().frameInfo.persistentSize = frameDataSize;
+  GetReplay()->WriteFrameRecord().frameInfo.initDataSize =
       chunkInfos[(D3D11Chunk)SystemChunk::InitialContents].totalsize;
 
   RDCDEBUG("Allocating %llu persistant bytes of memory for the log.",
-           m_FrameRecord.frameInfo.persistentSize);
+           GetReplay()->WriteFrameRecord().frameInfo.persistentSize);
 
   return ReplayStatus::Succeeded;
 }
@@ -1252,7 +1435,7 @@ void WrappedID3D11Device::ReplayLog(uint32_t startEventID, uint32_t endEventID,
   D3D11MarkerRegion::Set("!!!!RenderDoc Internal: Done replay");
 }
 
-void WrappedID3D11Device::ReleaseSwapchainResources(WrappedIDXGISwapChain4 *swap, UINT QueueCount,
+void WrappedID3D11Device::ReleaseSwapchainResources(IDXGISwapper *swapper, UINT QueueCount,
                                                     IUnknown *const *ppPresentQueue,
                                                     IUnknown **unwrappedQueues)
 {
@@ -1265,9 +1448,9 @@ void WrappedID3D11Device::ReleaseSwapchainResources(WrappedIDXGISwapChain4 *swap
       unwrappedQueues[i] = ppPresentQueue[i];
   }
 
-  for(int i = 0; i < swap->GetNumBackbuffers(); i++)
+  for(int i = 0; i < swapper->GetNumBackbuffers(); i++)
   {
-    WrappedID3D11Texture2D1 *wrapped11 = (WrappedID3D11Texture2D1 *)swap->GetBackbuffers()[i];
+    WrappedID3D11Texture2D1 *wrapped11 = (WrappedID3D11Texture2D1 *)swapper->GetBackbuffers()[i];
     if(wrapped11)
     {
       ResourceRange range(wrapped11);
@@ -1290,16 +1473,16 @@ void WrappedID3D11Device::ReleaseSwapchainResources(WrappedIDXGISwapChain4 *swap
     wrapped11 = NULL;
   }
 
-  if(swap)
+  HWND wnd = swapper->GetHWND();
+
+  if(wnd)
   {
-    DXGI_SWAP_CHAIN_DESC desc = swap->GetDescWithHWND();
+    Keyboard::RemoveInputWindow(WindowingSystem::Win32, wnd);
 
-    Keyboard::RemoveInputWindow(desc.OutputWindow);
-
-    RenderDoc::Inst().RemoveFrameCapturer((ID3D11Device *)this, desc.OutputWindow);
+    RenderDoc::Inst().RemoveFrameCapturer((ID3D11Device *)this, wnd);
   }
 
-  auto it = m_SwapChains.find(swap);
+  auto it = m_SwapChains.find(swapper);
   if(it != m_SwapChains.end())
   {
     SAFE_RELEASE(it->second);
@@ -1308,15 +1491,14 @@ void WrappedID3D11Device::ReleaseSwapchainResources(WrappedIDXGISwapChain4 *swap
 }
 
 template <typename SerialiserType>
-bool WrappedID3D11Device::Serialise_WrapSwapchainBuffer(SerialiserType &ser,
-                                                        WrappedIDXGISwapChain4 *swap,
-                                                        DXGI_SWAP_CHAIN_DESC *swapDesc, UINT Buffer,
+bool WrappedID3D11Device::Serialise_WrapSwapchainBuffer(SerialiserType &ser, IDXGISwapper *swapper,
+                                                        DXGI_FORMAT bufferFormat, UINT Buffer,
                                                         IUnknown *realSurface)
 {
   WrappedID3D11Texture2D1 *pTex = (WrappedID3D11Texture2D1 *)realSurface;
 
   SERIALISE_ELEMENT(Buffer);
-  SERIALISE_ELEMENT_LOCAL(SwapbufferID, pTex->GetResourceID()).TypedAs("IDXGISwapChain *");
+  SERIALISE_ELEMENT_LOCAL(SwapbufferID, pTex->GetResourceID()).TypedAs("IDXGISwapChain *"_lit);
 
   m_BBID = SwapbufferID;
 
@@ -1365,9 +1547,8 @@ bool WrappedID3D11Device::Serialise_WrapSwapchainBuffer(SerialiserType &ser,
   return true;
 }
 
-IUnknown *WrappedID3D11Device::WrapSwapchainBuffer(WrappedIDXGISwapChain4 *swap,
-                                                   DXGI_SWAP_CHAIN_DESC *swapDesc, UINT buffer,
-                                                   IUnknown *realSurface)
+IUnknown *WrappedID3D11Device::WrapSwapchainBuffer(IDXGISwapper *swapper, DXGI_FORMAT bufferFormat,
+                                                   UINT buffer, IUnknown *realSurface)
 {
   if(GetResourceManager()->HasWrapper((ID3D11DeviceChild *)realSurface))
   {
@@ -1401,6 +1582,7 @@ IUnknown *WrappedID3D11Device::WrapSwapchainBuffer(WrappedIDXGISwapChain4 *swap,
   if(IsCaptureMode(m_State))
   {
     D3D11ResourceRecord *record = GetResourceManager()->AddResourceRecord(id);
+    record->ResType = Resource_Texture2D;
     record->DataInSerialiser = false;
     record->Length = 0;
     record->NumSubResources = 0;
@@ -1412,7 +1594,7 @@ IUnknown *WrappedID3D11Device::WrapSwapchainBuffer(WrappedIDXGISwapChain4 *swap,
 
     SCOPED_SERIALISE_CHUNK(D3D11Chunk::CreateSwapBuffer);
 
-    Serialise_WrapSwapchainBuffer(ser, swap, swapDesc, buffer, pTex);
+    Serialise_WrapSwapchainBuffer(ser, swapper, bufferFormat, buffer, pTex);
 
     record->AddChunk(scope.Get());
   }
@@ -1425,21 +1607,41 @@ IUnknown *WrappedID3D11Device::WrapSwapchainBuffer(WrappedIDXGISwapChain4 *swap,
     if(FAILED(hr))
       RDCERR("Couldn't create RTV for swapchain tex HRESULT: %s", ToStr(hr).c_str());
 
-    m_SwapChains[swap] = rtv;
+    m_SwapChains[swapper] = rtv;
   }
 
-  if(swap)
+  HWND wnd = swapper->GetHWND();
+
+  if(wnd)
   {
-    DXGI_SWAP_CHAIN_DESC sdesc = swap->GetDescWithHWND();
+    Keyboard::AddInputWindow(WindowingSystem::Win32, wnd);
 
-    Keyboard::AddInputWindow(sdesc.OutputWindow);
-
-    RenderDoc::Inst().AddFrameCapturer((ID3D11Device *)this, sdesc.OutputWindow, this);
+    RenderDoc::Inst().AddFrameCapturer((ID3D11Device *)this, wnd, this);
   }
 
   return pTex;
 }
 
+IDXGIResource *WrappedID3D11Device::WrapExternalDXGIResource(IDXGIResource *res)
+{
+  ID3D11Resource *d3d11res;
+  res->QueryInterface(__uuidof(ID3D11Resource), (void **)&d3d11res);
+  if(GetResourceManager()->HasWrapper(d3d11res))
+  {
+    ID3D11DeviceChild *wrapper = GetResourceManager()->GetWrapper(d3d11res);
+    IDXGIResource *ret = NULL;
+    wrapper->QueryInterface(__uuidof(IDXGIResource), (void **)&ret);
+    res->Release();
+    return ret;
+  }
+
+  void *voidRes;
+  // duration will be meaningless but at least we can get timestamp.
+  SERIALISE_TIME_CALL(voidRes = (void *)res);
+  OpenSharedResourceInternal(D3D11Chunk::ExternalDXGIResource, __uuidof(IDXGIResource), &voidRes);
+  return (IDXGIResource *)voidRes;
+}
+
 void WrappedID3D11Device::SetMarker(uint32_t col, const wchar_t *name)
 {
   if(m_pCurrentWrappedDevice == NULL)
@@ -1479,10 +1681,10 @@ void WrappedID3D11Device::StartFrameCapture(void *dev, void *wnd)
   m_FailedFrame = 0;
   m_FailedReason = CaptureSucceeded;
 
-  m_FrameCounter = RDCMAX((uint32_t)m_CapturedFrames.size(), m_FrameCounter);
-
   FrameDescription frame;
-  frame.frameNumber = m_FrameCounter;
+  // for app-controlled captures the frame number is meaningless, so set it here here first. We'll
+  // update it with the actual frame counter when we un-set m_AppControlledCapture.
+  frame.frameNumber = ~0U;
   frame.captureTime = Timing::GetUnixTimestamp();
   m_CapturedFrames.push_back(frame);
 
@@ -1492,7 +1694,7 @@ void WrappedID3D11Device::StartFrameCapture(void *dev, void *wnd)
 
   GetResourceManager()->MarkResourceFrameReferenced(m_ResourceID, eFrameRef_PartialWrite);
 
-  m_pImmediateContext->FreeCaptureData();
+  GetResourceManager()->FreeCaptureData();
 
   m_pImmediateContext->AttemptCapture();
   m_pImmediateContext->BeginCaptureFrame();
@@ -1516,7 +1718,7 @@ void WrappedID3D11Device::StartFrameCapture(void *dev, void *wnd)
   if(m_pInfoQueue)
     m_pInfoQueue->ClearStoredMessages();
 
-  RDCLOG("Starting capture, frame %u", m_FrameCounter);
+  RDCLOG("Starting capture, frame %u", m_CapturedFrames.back().frameNumber);
 }
 
 bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
@@ -1528,22 +1730,20 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
 
   CaptureFailReason reason;
 
-  WrappedIDXGISwapChain4 *swap = NULL;
+  IDXGISwapper *swapper = NULL;
 
   if(wnd)
   {
     for(auto it = m_SwapChains.begin(); it != m_SwapChains.end(); ++it)
     {
-      DXGI_SWAP_CHAIN_DESC swapDesc = it->first->GetDescWithHWND();
-
-      if(swapDesc.OutputWindow == wnd)
+      if(it->first->GetHWND() == wnd)
       {
-        swap = it->first;
+        swapper = it->first;
         break;
       }
     }
 
-    if(swap == NULL)
+    if(swapper == NULL)
     {
       RDCERR("Output window %p provided for frame capture corresponds with no known swap chain", wnd);
       return false;
@@ -1552,7 +1752,10 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
 
   if(m_pImmediateContext->HasSuccessfulCapture(reason))
   {
-    RDCLOG("Finished capture, Frame %u", m_FrameCounter);
+    RDCLOG("Finished capture, Frame %u", m_CapturedFrames.back().frameNumber);
+
+    if(swapper == NULL)
+      swapper = m_LastSwap;
 
     m_Failures = 0;
     m_FailedFrame = 0;
@@ -1578,9 +1781,9 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
     const uint32_t maxSize = 2048;
     RenderDoc::FramePixels fp;
 
-    if(swap != NULL)
+    if(swapper != NULL)
     {
-      ID3D11RenderTargetView *rtv = m_SwapChains[swap];
+      ID3D11RenderTargetView *rtv = m_SwapChains[swapper];
 
       ID3D11Resource *res = NULL;
 
@@ -1733,7 +1936,7 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
         SCOPED_SERIALISE_CHUNK(D3D11Chunk::DeviceInitialisation, 16);
 
         SERIALISE_ELEMENT_LOCAL(ImmediateContext, m_pImmediateContext->GetResourceID())
-            .TypedAs("ID3D11DeviceContext *");
+            .TypedAs("ID3D11DeviceContext *"_lit);
       }
 
       RDCDEBUG("Inserting Resource Serialisers");
@@ -1791,7 +1994,7 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
 
     m_pImmediateContext->CleanupCapture();
 
-    m_pImmediateContext->FreeCaptureData();
+    GetResourceManager()->FreeCaptureData();
 
     for(auto it = m_DeferredContexts.begin(); it != m_DeferredContexts.end(); ++it)
     {
@@ -1821,32 +2024,34 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
       default: break;
     }
 
-    RDCLOG("Failed to capture, frame %u: %s", m_FrameCounter, reasonString);
+    RDCLOG("Failed to capture, frame %u: %s", m_CapturedFrames.back().frameNumber, reasonString);
 
     m_Failures++;
 
-    if((RenderDoc::Inst().GetOverlayBits() & eRENDERDOC_Overlay_Enabled) && swap != NULL)
+    if((RenderDoc::Inst().GetOverlayBits() & eRENDERDOC_Overlay_Enabled) && swapper)
     {
       D3D11RenderState old = *m_pImmediateContext->GetCurrentPipelineState();
 
-      ID3D11RenderTargetView *rtv = m_SwapChains[swap];
+      ID3D11RenderTargetView *rtv = m_SwapChains[swapper];
 
       if(rtv)
       {
         m_pImmediateContext->GetReal()->OMSetRenderTargets(1, &rtv, NULL);
 
-        DXGI_SWAP_CHAIN_DESC swapDesc = swap->GetDescWithHWND();
-        m_TextRenderer->SetOutputDimensions(swapDesc.BufferDesc.Width, swapDesc.BufferDesc.Height);
-        m_TextRenderer->SetOutputWindow(swapDesc.OutputWindow);
+        m_TextRenderer->SetOutputDimensions(swapper->GetWidth(), swapper->GetHeight());
+        m_TextRenderer->SetOutputWindow(swapper->GetHWND());
 
-        m_TextRenderer->RenderText(0.0f, 0.0f, "Failed to capture frame %u: %s", m_FrameCounter,
-                                   reasonString);
+        m_TextRenderer->RenderText(
+            0.0f, 0.0f, StringFormat::Fmt("Failed to capture frame %u: %s",
+                                          m_CapturedFrames.back().frameNumber, reasonString));
       }
 
       old.ApplyState(m_pImmediateContext);
     }
 
-    m_CapturedFrames.back().frameNumber = m_FrameCounter;
+    uint32_t failedFrame = m_CapturedFrames.back().frameNumber;
+
+    m_CapturedFrames.back().frameNumber = m_AppControlledCapture ? ~0U : m_FrameCounter;
 
     m_pImmediateContext->CleanupCapture();
 
@@ -1887,9 +2092,9 @@ bool WrappedID3D11Device::EndFrameCapture(void *dev, void *wnd)
         }
       }
 
-      m_pImmediateContext->FreeCaptureData();
+      GetResourceManager()->FreeCaptureData();
 
-      m_FailedFrame = m_FrameCounter;
+      m_FailedFrame = failedFrame;
       m_FailedReason = reason;
 
       m_State = CaptureState::BackgroundCapturing;
@@ -1979,7 +2184,7 @@ bool WrappedID3D11Device::DiscardFrameCapture(void *dev, void *wnd)
     }
   }
 
-  m_pImmediateContext->FreeCaptureData();
+  GetResourceManager()->FreeCaptureData();
 
   m_State = CaptureState::BackgroundCapturing;
 
@@ -2088,20 +2293,19 @@ void WrappedID3D11Device::UnlockForChunkRemoval()
   }
 }
 
-void WrappedID3D11Device::FirstFrame(WrappedIDXGISwapChain4 *swapChain)
+void WrappedID3D11Device::FirstFrame(IDXGISwapper *swapper)
 {
-  DXGI_SWAP_CHAIN_DESC swapdesc = swapChain->GetDescWithHWND();
-
   // if we have to capture the first frame, begin capturing immediately
   if(IsBackgroundCapturing(m_State) && RenderDoc::Inst().ShouldTriggerCapture(0))
   {
-    RenderDoc::Inst().StartFrameCapture((ID3D11Device *)this, swapdesc.OutputWindow);
+    RenderDoc::Inst().StartFrameCapture((ID3D11Device *)this, swapper->GetHWND());
 
     m_AppControlledCapture = false;
+    m_CapturedFrames.back().frameNumber = 0;
   }
 }
 
-HRESULT WrappedID3D11Device::Present(WrappedIDXGISwapChain4 *swap, UINT SyncInterval, UINT Flags)
+HRESULT WrappedID3D11Device::Present(IDXGISwapper *swapper, UINT SyncInterval, UINT Flags)
 {
   if((Flags & DXGI_PRESENT_TEST) != 0)
     return S_OK;
@@ -2117,8 +2321,9 @@ HRESULT WrappedID3D11Device::Present(WrappedIDXGISwapChain4 *swap, UINT SyncInte
 
   m_pImmediateContext->BeginFrame();
 
-  DXGI_SWAP_CHAIN_DESC swapdesc = swap->GetDescWithHWND();
-  bool activeWindow = RenderDoc::Inst().IsActiveWindow((ID3D11Device *)this, swapdesc.OutputWindow);
+  bool activeWindow = RenderDoc::Inst().IsActiveWindow((ID3D11Device *)this, swapper->GetHWND());
+
+  m_LastSwap = swapper;
 
   if(IsBackgroundCapturing(m_State))
   {
@@ -2128,17 +2333,15 @@ HRESULT WrappedID3D11Device::Present(WrappedIDXGISwapChain4 *swap, UINT SyncInte
 
     if(overlay & eRENDERDOC_Overlay_Enabled)
     {
-      ID3D11RenderTargetView *rtv = m_SwapChains[swap];
+      ID3D11RenderTargetView *rtv = m_SwapChains[swapper];
 
       m_pImmediateContext->GetReal()->OMSetRenderTargets(1, &rtv, NULL);
 
-      DXGI_SWAP_CHAIN_DESC swapDesc = {0};
-      swap->GetDesc(&swapDesc);
-      m_TextRenderer->SetOutputDimensions(swapDesc.BufferDesc.Width, swapDesc.BufferDesc.Height);
-      m_TextRenderer->SetOutputWindow(swapDesc.OutputWindow);
+      m_TextRenderer->SetOutputDimensions(swapper->GetWidth(), swapper->GetHeight());
+      m_TextRenderer->SetOutputWindow(swapper->GetHWND());
 
       int flags = activeWindow ? RenderDoc::eOverlay_ActiveWindow : 0;
-      string overlayText = RenderDoc::Inst().GetOverlayText(RDCDriver::D3D11, m_FrameCounter, flags);
+      rdcstr overlayText = RenderDoc::Inst().GetOverlayText(RDCDriver::D3D11, m_FrameCounter, flags);
 
       if(activeWindow && m_FailedFrame > 0)
       {
@@ -2154,8 +2357,7 @@ HRESULT WrappedID3D11Device::Present(WrappedIDXGISwapChain4 *swap, UINT SyncInte
         overlayText += StringFormat::Fmt("    %s\n", reasonString);
       }
 
-      if(!overlayText.empty())
-        m_TextRenderer->RenderText(0.0f, 0.0f, overlayText.c_str());
+      m_TextRenderer->RenderText(0.0f, 0.0f, overlayText);
 
       old.ApplyState(m_pImmediateContext);
     }
@@ -2163,22 +2365,25 @@ HRESULT WrappedID3D11Device::Present(WrappedIDXGISwapChain4 *swap, UINT SyncInte
 
   RenderDoc::Inst().AddActiveDriver(RDCDriver::D3D11, true);
 
+  // serialise the present call, even for inactive windows
+  if(IsActiveCapturing(m_State))
+    m_pImmediateContext->Present(SyncInterval, Flags);
+
   if(!activeWindow)
     return S_OK;
 
   // kill any current capture that isn't application defined
   if(IsActiveCapturing(m_State) && !m_AppControlledCapture)
   {
-    m_pImmediateContext->Present(SyncInterval, Flags);
-
-    RenderDoc::Inst().EndFrameCapture((ID3D11Device *)this, swapdesc.OutputWindow);
+    RenderDoc::Inst().EndFrameCapture((ID3D11Device *)this, swapper->GetHWND());
   }
 
   if(IsBackgroundCapturing(m_State) && RenderDoc::Inst().ShouldTriggerCapture(m_FrameCounter))
   {
-    RenderDoc::Inst().StartFrameCapture((ID3D11Device *)this, swapdesc.OutputWindow);
+    RenderDoc::Inst().StartFrameCapture((ID3D11Device *)this, swapper->GetHWND());
 
     m_AppControlledCapture = false;
+    m_CapturedFrames.back().frameNumber = m_FrameCounter;
   }
 
   return S_OK;
@@ -2400,8 +2605,6 @@ void WrappedID3D11Device::ReleaseResource(ID3D11DeviceChild *res)
 
   SCOPED_LOCK(m_D3DLock);
 
-  GetResourceManager()->MarkCleanResource(idx);
-
   if(WrappedID3D11DeviceContext::IsAlloc(res))
     RemoveDeferredContext((WrappedID3D11DeviceContext *)res);
 
@@ -2431,3 +2634,13 @@ const DrawcallDescription *WrappedID3D11Device::GetDrawcall(uint32_t eventId)
 
   return m_Drawcalls[eventId];
 }
+
+ResourceDescription &WrappedID3D11Device::GetResourceDesc(ResourceId id)
+{
+  return GetReplay()->GetResourceDesc(id);
+}
+
+FrameStatistics &WrappedID3D11Device::GetFrameStats()
+{
+  return GetReplay()->WriteFrameRecord().frameInfo.stats;
+}
