@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,25 +27,23 @@
 
 #include <stdint.h>
 #include <map>
-#include "api/replay/renderdoc_replay.h"
 #include "common/threading.h"
 #include "common/timing.h"
 #include "core/core.h"
 #include "driver/dxgi/dxgi_wrapped.h"
 #include "d3d11_common.h"
 #include "d3d11_manager.h"
-#include "d3d11_replay.h"
 #include "d3d11_video.h"
 
+class D3D11DebugManager;
 class D3D11TextRenderer;
 class D3D11ShaderCache;
+class D3D11Replay;
 
 #ifndef D3D11_1_UAV_SLOT_COUNT
 #define D3D11_1_UAV_SLOT_COUNT 64
 #endif
 
-using std::map;
-
 enum TextureDisplayType
 {
   TEXDISPLAY_UNKNOWN = 0,
@@ -56,16 +54,16 @@ enum TextureDisplayType
 
 struct D3D11InitParams
 {
-  D3D11InitParams();
+  D3D_DRIVER_TYPE DriverType = D3D_DRIVER_TYPE_UNKNOWN;
+  UINT Flags = 0;
+  UINT SDKVersion = D3D11_SDK_VERSION;
+  UINT NumFeatureLevels = 0;
+  D3D_FEATURE_LEVEL FeatureLevels[16] = {};
 
-  D3D_DRIVER_TYPE DriverType;
-  UINT Flags;
-  UINT SDKVersion;
-  UINT NumFeatureLevels;
-  D3D_FEATURE_LEVEL FeatureLevels[16];
+  DXGI_ADAPTER_DESC AdapterDesc = {};
 
   // check if a frame capture section version is supported
-  static const uint64_t CurrentVersion = 0x10;
+  static const uint64_t CurrentVersion = 0x11;
   static bool IsSupportedVersion(uint64_t ver);
 };
 
@@ -293,7 +291,7 @@ private:
     eInitialContents_ClearDSV = 2,
   };
 
-  D3D11Replay m_Replay;
+  D3D11Replay *m_Replay;
 
   WrappedD3D11Multithread m_WrappedMultithread;
   DummyID3D11InfoQueue m_DummyInfoQueue;
@@ -316,10 +314,11 @@ private:
   D3D11ShaderCache *m_ShaderCache = NULL;
   D3D11ResourceManager *m_ResourceManager = NULL;
 
-  vector<string> m_ShaderSearchPaths;
+  rdcarray<rdcstr> m_ShaderSearchPaths;
 
   D3D11InitParams m_InitParams;
   uint64_t m_SectionVersion;
+  ReplayOptions m_ReplayOptions;
 
   ResourceId m_BBID;
 
@@ -340,17 +339,17 @@ private:
   bool m_D3DThreadSafe = false;
 
   WriteSerialiser m_ScratchSerialiser;
-  std::set<std::string> m_StringDB;
+  std::set<rdcstr> m_StringDB;
 
   ResourceId m_ResourceID;
   D3D11ResourceRecord *m_DeviceRecord;
 
   CaptureState m_State;
-  bool m_AppControlledCapture;
+  bool m_AppControlledCapture = false;
 
   ReplayStatus m_FailedReplayStatus = ReplayStatus::APIReplayFailed;
 
-  set<ID3D11DeviceChild *> m_CachedStateObjects;
+  std::set<ID3D11DeviceChild *> m_CachedStateObjects;
 
   // This function will check if m_CachedStateObjects is growing too large, and if so
   // go through m_CachedStateObjects and release any state objects that are purely
@@ -367,38 +366,41 @@ private:
   // Must be called while m_D3DLock is held.
   void CachedObjectsGarbageCollect();
 
-  set<WrappedID3D11DeviceContext *> m_DeferredContexts;
-  map<ID3D11InputLayout *, vector<D3D11_INPUT_ELEMENT_DESC> > m_LayoutDescs;
-  map<ID3D11InputLayout *, WrappedShader *> m_LayoutShaders;
+  std::set<WrappedID3D11DeviceContext *> m_DeferredContexts;
+  std::map<ID3D11InputLayout *, rdcarray<D3D11_INPUT_ELEMENT_DESC> > m_LayoutDescs;
+  std::map<ID3D11InputLayout *, WrappedShader *> m_LayoutShaders;
 
   static WrappedID3D11Device *m_pCurrentWrappedDevice;
 
-  map<WrappedIDXGISwapChain4 *, ID3D11RenderTargetView *> m_SwapChains;
+  std::map<IDXGISwapper *, ID3D11RenderTargetView *> m_SwapChains;
 
-  uint32_t m_FrameCounter;
-  uint32_t m_FailedFrame;
+  IDXGISwapper *m_LastSwap = NULL;
+
+  uint32_t m_FrameCounter = 0;
+  uint32_t m_FailedFrame = 0;
   CaptureFailReason m_FailedReason;
-  uint32_t m_Failures;
+  uint32_t m_Failures = 0;
 
   SDFile *m_StructuredFile = NULL;
   SDFile m_StoredStructuredData;
 
-  vector<DebugMessage> m_DebugMessages;
+  rdcarray<DebugMessage> m_DebugMessages;
 
-  vector<FrameDescription> m_CapturedFrames;
-  FrameRecord m_FrameRecord;
-  vector<DrawcallDescription *> m_Drawcalls;
+  rdcarray<FrameDescription> m_CapturedFrames;
+  rdcarray<DrawcallDescription *> m_Drawcalls;
 
 public:
   static const int AllocPoolCount = 4;
   ALLOCATE_WITH_WRAPPED_POOL(WrappedID3D11Device, AllocPoolCount);
 
   WrappedID3D11Device(ID3D11Device *realDevice, D3D11InitParams params);
-  void SetInitParams(const D3D11InitParams &params, uint64_t sectionVersion)
+  void SetInitParams(const D3D11InitParams &params, uint64_t sectionVersion, const ReplayOptions &opts)
   {
     m_InitParams = params;
     m_SectionVersion = sectionVersion;
+    m_ReplayOptions = opts;
   }
+  const ReplayOptions &GetReplayOptions() { return m_ReplayOptions; }
   uint64_t GetLogVersion() { return m_SectionVersion; }
   virtual ~WrappedID3D11Device();
 
@@ -413,11 +415,11 @@ public:
   void AddResourceCurChunk(ResourceId id);
 
   ID3D11Device *GetReal() { return m_pDevice; }
-  static std::string GetChunkName(uint32_t idx);
+  static rdcstr GetChunkName(uint32_t idx);
   D3D11ShaderCache *GetShaderCache() { return m_ShaderCache; }
   D3D11ResourceManager *GetResourceManager() { return m_ResourceManager; }
   D3D11DebugManager *GetDebugManager() { return m_DebugManager; }
-  D3D11Replay *GetReplay() { return &m_Replay; }
+  D3D11Replay *GetReplay() { return m_Replay; }
   Threading::CriticalSection &D3DLock() { return m_D3DLock; }
   bool D3DThreadSafe() const { return m_D3DThreadSafe; }
   void SetD3DThreadSafe(bool safe) { m_D3DThreadSafe = safe; }
@@ -430,9 +432,9 @@ public:
   void ReleaseResource(ID3D11DeviceChild *pResource);
 
   ResourceId GetResourceID() { return m_ResourceID; }
-  FrameRecord &GetFrameRecord() { return m_FrameRecord; }
-  FrameStatistics &GetFrameStats() { return m_FrameRecord.frameInfo.stats; }
   const DrawcallDescription *GetDrawcall(uint32_t eventId);
+  ResourceDescription &GetResourceDesc(ResourceId id);
+  FrameStatistics &GetFrameStats();
 
   void ReplayPushEvent() { m_ReplayEventCount++; }
   void ReplayPopEvent() { m_ReplayEventCount = RDCMAX(0, m_ReplayEventCount - 1); }
@@ -442,20 +444,21 @@ public:
   void UnlockForChunkRemoval();
 
   SDFile &GetStructuredFile() { return *m_StructuredFile; }
-  void FirstFrame(WrappedIDXGISwapChain4 *swapChain);
+  void FirstFrame(IDXGISwapper *swapper);
 
-  std::vector<DebugMessage> GetDebugMessages();
+  rdcarray<DebugMessage> GetDebugMessages();
   void AddDebugMessage(DebugMessage msg);
-  void AddDebugMessage(MessageCategory c, MessageSeverity sv, MessageSource src, std::string d);
-  const vector<D3D11_INPUT_ELEMENT_DESC> &GetLayoutDesc(ID3D11InputLayout *layout)
+  void AddDebugMessage(MessageCategory c, MessageSeverity sv, MessageSource src, rdcstr d);
+  const rdcarray<D3D11_INPUT_ELEMENT_DESC> &GetLayoutDesc(ID3D11InputLayout *layout)
   {
     return m_LayoutDescs[layout];
   }
 
-  vector<string> *GetShaderDebugInfoSearchPaths() { return &m_ShaderSearchPaths; }
+  rdcarray<rdcstr> *GetShaderDebugInfoSearchPaths() { return &m_ShaderSearchPaths; }
   template <typename SerialiserType>
   bool Serialise_CaptureScope(SerialiserType &ser);
 
+  RDCDriver GetFrameCaptureDriver() { return RDCDriver::D3D11; }
   void StartFrameCapture(void *dev, void *wnd);
   bool EndFrameCapture(void *dev, void *wnd);
   bool DiscardFrameCapture(void *dev, void *wnd);
@@ -468,7 +471,7 @@ public:
   {
     if(iid == __uuidof(ID3D11Device) || iid == __uuidof(ID3D11Device1) ||
        iid == __uuidof(ID3D11Device2) || iid == __uuidof(ID3D11Device3) ||
-       iid == __uuidof(ID3D11Device4))
+       iid == __uuidof(ID3D11Device4) || iid == __uuidof(ID3D11Device5))
       return true;
 
     return false;
@@ -485,6 +488,8 @@ public:
       return (ID3D11Device3 *)this;
     else if(iid == __uuidof(ID3D11Device4))
       return (ID3D11Device4 *)this;
+    else if(iid == __uuidof(ID3D11Device5))
+      return (ID3D11Device5 *)this;
 
     RDCERR("Requested unknown device interface %s", ToStr(iid).c_str());
 
@@ -494,12 +499,13 @@ public:
   // log replaying
 
   bool Prepare_InitialState(ID3D11DeviceChild *res);
-  uint32_t GetSize_InitialState(ResourceId id, ID3D11DeviceChild *res);
+  uint64_t GetSize_InitialState(ResourceId id, const D3D11InitialContents &initial);
   template <typename SerialiserType>
-  bool Serialise_InitialState(SerialiserType &ser, ResourceId resid, ID3D11DeviceChild *res);
+  bool Serialise_InitialState(SerialiserType &ser, ResourceId resid, D3D11ResourceRecord *record,
+                              const D3D11InitialContents *initial);
 
   void Create_InitialState(ResourceId id, ID3D11DeviceChild *live, bool hasData);
-  void Apply_InitialState(ID3D11DeviceChild *live, D3D11InitialContents initial);
+  void Apply_InitialState(ID3D11DeviceChild *live, const D3D11InitialContents &initial);
 
   void SetStructuredExport(uint64_t sectionVersion)
   {
@@ -520,8 +526,8 @@ public:
                                 const char *Path);
 
   // Swap Chain
-  IMPLEMENT_FUNCTION_SERIALISED(IUnknown *, WrapSwapchainBuffer, WrappedIDXGISwapChain4 *swap,
-                                DXGI_SWAP_CHAIN_DESC *desc, UINT buffer, IUnknown *realSurface);
+  IMPLEMENT_FUNCTION_SERIALISED(IUnknown *, WrapSwapchainBuffer, IDXGISwapper *swapper,
+                                DXGI_FORMAT bufferFormat, UINT buffer, IUnknown *realSurface);
 
 // this is defined as a macro so that we can re-use it to explicitly instantiate these functions as
 // templates in the wrapper definition file.
@@ -537,13 +543,15 @@ public:
 
   SERIALISED_ID3D11DEVICE_FAKE_FUNCTIONS();
 
-  HRESULT Present(WrappedIDXGISwapChain4 *swap, UINT SyncInterval, UINT Flags);
+  HRESULT Present(IDXGISwapper *swapper, UINT SyncInterval, UINT Flags);
 
   void NewSwapchainBuffer(IUnknown *backbuffer);
 
-  void ReleaseSwapchainResources(WrappedIDXGISwapChain4 *swap, UINT QueueCount,
+  void ReleaseSwapchainResources(IDXGISwapper *swapper, UINT QueueCount,
                                  IUnknown *const *ppPresentQueue, IUnknown **unwrappedQueues);
 
+  virtual IDXGIResource *WrapExternalDXGIResource(IDXGIResource *res);
+
   ResourceId GetBackbufferResourceID() { return m_BBID; }
   void InternalRef() { InterlockedIncrement(&m_InternalRefcount); }
   void InternalRelease() { InterlockedDecrement(&m_InternalRefcount); }
@@ -599,10 +607,13 @@ public:
   }
 
   template <typename SerialiserType>
-  std::vector<D3D11_SUBRESOURCE_DATA> Serialise_CreateTextureData(
+  rdcarray<D3D11_SUBRESOURCE_DATA> Serialise_CreateTextureData(
       SerialiserType &ser, ID3D11Resource *tex, ResourceId id, const D3D11_SUBRESOURCE_DATA *data,
       UINT w, UINT h, UINT d, DXGI_FORMAT fmt, UINT mips, UINT arr, bool HasData);
 
+  HRESULT STDMETHODCALLTYPE OpenSharedResourceInternal(D3D11Chunk chunkType,
+                                                       REFIID ReturnedInterface, void **ppResource);
+
 // this is defined as a macro so that we can re-use it to explicitly instantiate these functions as
 // templates in the wrapper definition file.
 #define SERIALISED_ID3D11DEVICE_FUNCTIONS()                                                         \
@@ -871,4 +882,4 @@ public:
                                                 REFIID riid, void **ppFence);
 
   virtual HRESULT STDMETHODCALLTYPE OpenSharedFence(HANDLE hFence, REFIID riid, void **ppFence);
-};
\ No newline at end of file
+};
