@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -73,7 +73,8 @@ bool WrappedID3D11Device::Serialise_CreateBlendState1(SerialiserType &ser,
                                                       ID3D11BlendState1 **ppBlendState)
 {
   SERIALISE_ELEMENT_LOCAL(Descriptor, *pBlendStateDesc);
-  SERIALISE_ELEMENT_LOCAL(pState, GetIDForResource(*ppBlendState)).TypedAs("ID3D11BlendState1 *");
+  SERIALISE_ELEMENT_LOCAL(pState, GetIDForResource(*ppBlendState))
+      .TypedAs("ID3D11BlendState1 *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -163,6 +164,7 @@ HRESULT WrappedID3D11Device::CreateBlendState1(const D3D11_BLEND_DESC1 *pBlendSt
       RDCASSERT(GetResourceManager()->GetResourceRecord(id) == NULL);
 
       D3D11ResourceRecord *record = GetResourceManager()->AddResourceRecord(id);
+      record->ResType = IdentifyTypeByPtr(wrapped);
       record->Length = 0;
 
       record->AddChunk(scope.Get());
@@ -181,7 +183,7 @@ bool WrappedID3D11Device::Serialise_CreateRasterizerState1(
 {
   SERIALISE_ELEMENT_LOCAL(Descriptor, *pRasterizerDesc);
   SERIALISE_ELEMENT_LOCAL(pState, GetIDForResource(*ppRasterizerState))
-      .TypedAs("ID3D11RasterizerState1 *");
+      .TypedAs("ID3D11RasterizerState1 *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -271,6 +273,7 @@ HRESULT WrappedID3D11Device::CreateRasterizerState1(const D3D11_RASTERIZER_DESC1
       RDCASSERT(GetResourceManager()->GetResourceRecord(id) == NULL);
 
       D3D11ResourceRecord *record = GetResourceManager()->AddResourceRecord(id);
+      record->ResType = IdentifyTypeByPtr(wrapped);
       record->Length = 0;
 
       record->AddChunk(scope.Get());
@@ -319,8 +322,22 @@ HRESULT WrappedID3D11Device::OpenSharedResource1(HANDLE hResource, REFIID return
 {
   if(m_pDevice1 == NULL)
     return E_NOINTERFACE;
-  RDCUNIMPLEMENTED("Not wrapping OpenSharedResource1");
-  return m_pDevice1->OpenSharedResource1(hResource, returnedInterface, ppResource);
+
+  if(ppResource == NULL)
+    return E_INVALIDARG;
+
+  HRESULT hr;
+
+  SERIALISE_TIME_CALL(hr = m_pDevice1->OpenSharedResource1(hResource, returnedInterface, ppResource));
+
+  if(FAILED(hr))
+  {
+    IUnknown *unk = (IUnknown *)*ppResource;
+    SAFE_RELEASE(unk);
+    return hr;
+  }
+
+  return OpenSharedResourceInternal(D3D11Chunk::OpenSharedResource1, returnedInterface, ppResource);
 }
 
 HRESULT WrappedID3D11Device::OpenSharedResourceByName(LPCWSTR lpName, DWORD dwDesiredAccess,
@@ -328,8 +345,24 @@ HRESULT WrappedID3D11Device::OpenSharedResourceByName(LPCWSTR lpName, DWORD dwDe
 {
   if(m_pDevice1 == NULL)
     return E_NOINTERFACE;
-  RDCUNIMPLEMENTED("Not wrapping OpenSharedResourceByName");
-  return m_pDevice1->OpenSharedResourceByName(lpName, dwDesiredAccess, returnedInterface, ppResource);
+
+  if(ppResource == NULL)
+    return E_INVALIDARG;
+
+  HRESULT hr;
+
+  SERIALISE_TIME_CALL(hr = m_pDevice1->OpenSharedResourceByName(lpName, dwDesiredAccess,
+                                                                returnedInterface, ppResource));
+
+  if(FAILED(hr))
+  {
+    IUnknown *unk = (IUnknown *)*ppResource;
+    SAFE_RELEASE(unk);
+    return hr;
+  }
+
+  return OpenSharedResourceInternal(D3D11Chunk::OpenSharedResourceByName, returnedInterface,
+                                    ppResource);
 }
 
 #undef IMPLEMENT_FUNCTION_SERIALISED
@@ -337,4 +370,4 @@ HRESULT WrappedID3D11Device::OpenSharedResourceByName(LPCWSTR lpName, DWORD dwDe
   template bool WrappedID3D11Device::CONCAT(Serialise_, func(ReadSerialiser &ser, __VA_ARGS__)); \
   template bool WrappedID3D11Device::CONCAT(Serialise_, func(WriteSerialiser &ser, __VA_ARGS__));
 
-SERIALISED_ID3D11DEVICE1_FUNCTIONS();
\ No newline at end of file
+SERIALISED_ID3D11DEVICE1_FUNCTIONS();
