@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +24,7 @@
 
 #include "d3d11_device.h"
 #include "d3d11_context.h"
+#include "d3d11_debug.h"
 #include "d3d11_resources.h"
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -47,9 +48,10 @@ bool WrappedID3D11Device::Serialise_CreateTexture2D1(SerialiserType &ser,
     SERIALISE_ELEMENT_ARRAY(pInitialData, pInitialData ? numSubresources : 0);
   }
 
-  SERIALISE_ELEMENT_LOCAL(pTexture, GetIDForResource(*ppTexture2D)).TypedAs("ID3D11Texture2D *");
+  SERIALISE_ELEMENT_LOCAL(pTexture, GetIDForResource(*ppTexture2D))
+      .TypedAs("ID3D11Texture2D *"_lit);
 
-  std::vector<D3D11_SUBRESOURCE_DATA> descs =
+  rdcarray<D3D11_SUBRESOURCE_DATA> descs =
       Serialise_CreateTextureData(ser, ppTexture2D ? *ppTexture2D : NULL, pTexture, pInitialData,
                                   Descriptor.Width, Descriptor.Height, 1, Descriptor.Format,
                                   Descriptor.MipLevels, Descriptor.ArraySize, pInitialData != NULL);
@@ -186,9 +188,10 @@ bool WrappedID3D11Device::Serialise_CreateTexture3D1(SerialiserType &ser,
     SERIALISE_ELEMENT_ARRAY(pInitialData, pInitialData ? numSubresources : 0);
   }
 
-  SERIALISE_ELEMENT_LOCAL(pTexture, GetIDForResource(*ppTexture3D)).TypedAs("ID3D11Texture3D *");
+  SERIALISE_ELEMENT_LOCAL(pTexture, GetIDForResource(*ppTexture3D))
+      .TypedAs("ID3D11Texture3D *"_lit);
 
-  std::vector<D3D11_SUBRESOURCE_DATA> descs =
+  rdcarray<D3D11_SUBRESOURCE_DATA> descs =
       Serialise_CreateTextureData(ser, ppTexture3D ? *ppTexture3D : NULL, pTexture, pInitialData,
                                   Descriptor.Width, Descriptor.Height, Descriptor.Depth,
                                   Descriptor.Format, Descriptor.MipLevels, 1, pInitialData != NULL);
@@ -316,7 +319,7 @@ bool WrappedID3D11Device::Serialise_CreateShaderResourceView1(
   SERIALISE_ELEMENT(pResource);
   SERIALISE_ELEMENT_OPT(pDesc);
   SERIALISE_ELEMENT_LOCAL(pView, GetIDForResource(*ppSRView))
-      .TypedAs("ID3D11ShaderResourceView1 *");
+      .TypedAs("ID3D11ShaderResourceView1 *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -438,6 +441,7 @@ HRESULT WrappedID3D11Device::CreateShaderResourceView1(ID3D11Resource *pResource
         RDCASSERT(GetResourceManager()->GetResourceRecord(id) == NULL);
 
         D3D11ResourceRecord *record = GetResourceManager()->AddResourceRecord(id);
+        record->ResType = IdentifyTypeByPtr(wrapped);
         record->Length = 0;
 
         record->AddParent(parent);
@@ -465,7 +469,8 @@ bool WrappedID3D11Device::Serialise_CreateRenderTargetView1(SerialiserType &ser,
 {
   SERIALISE_ELEMENT(pResource);
   SERIALISE_ELEMENT_OPT(pDesc);
-  SERIALISE_ELEMENT_LOCAL(pView, GetIDForResource(*ppRTView)).TypedAs("ID3D11RenderTargetView1 *");
+  SERIALISE_ELEMENT_LOCAL(pView, GetIDForResource(*ppRTView))
+      .TypedAs("ID3D11RenderTargetView1 *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -583,6 +588,7 @@ HRESULT WrappedID3D11Device::CreateRenderTargetView1(ID3D11Resource *pResource,
         RDCASSERT(GetResourceManager()->GetResourceRecord(id) == NULL);
 
         D3D11ResourceRecord *record = GetResourceManager()->AddResourceRecord(id);
+        record->ResType = IdentifyTypeByPtr(wrapped);
         record->Length = 0;
 
         record->AddParent(parent);
@@ -611,7 +617,7 @@ bool WrappedID3D11Device::Serialise_CreateUnorderedAccessView1(
   SERIALISE_ELEMENT(pResource);
   SERIALISE_ELEMENT_OPT(pDesc);
   SERIALISE_ELEMENT_LOCAL(pView, GetIDForResource(*ppUAView))
-      .TypedAs("ID3D11UnorderedAccessView1 *");
+      .TypedAs("ID3D11UnorderedAccessView1 *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -645,6 +651,19 @@ bool WrappedID3D11Device::Serialise_CreateUnorderedAccessView1(
 
     AddResource(pView, ResourceType::View, "Unordered Access View");
     DerivedResource(pResource, pView);
+
+    {
+      D3D11_UNORDERED_ACCESS_VIEW_DESC desc = {};
+      ret->GetDesc(&desc);
+
+      if(desc.ViewDimension == D3D11_UAV_DIMENSION_BUFFER &&
+         (desc.Buffer.Flags & (D3D11_BUFFER_UAV_FLAG_APPEND | D3D11_BUFFER_UAV_FLAG_COUNTER)))
+      {
+        ResourceId counterBuffer = GetDebugManager()->AddCounterUAVBuffer(ret);
+        AddResource(counterBuffer, ResourceType::Buffer, "UAV Counter");
+        DerivedResource(ret, counterBuffer);
+      }
+    }
   }
 
   return true;
@@ -698,6 +717,7 @@ HRESULT WrappedID3D11Device::CreateUnorderedAccessView1(ID3D11Resource *pResourc
         RDCASSERT(GetResourceManager()->GetResourceRecord(id) == NULL);
 
         D3D11ResourceRecord *record = GetResourceManager()->AddResourceRecord(id);
+        record->ResType = IdentifyTypeByPtr(wrapped);
         record->Length = 0;
 
         record->AddParent(parent);
@@ -724,7 +744,7 @@ bool WrappedID3D11Device::Serialise_CreateRasterizerState2(
 {
   SERIALISE_ELEMENT_LOCAL(Descriptor, *pRasterizerDesc);
   SERIALISE_ELEMENT_LOCAL(pState, GetIDForResource(*ppRasterizerState))
-      .TypedAs("ID3D11RasterizerState2 *");
+      .TypedAs("ID3D11RasterizerState2 *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -816,6 +836,7 @@ HRESULT WrappedID3D11Device::CreateRasterizerState2(const D3D11_RASTERIZER_DESC2
       RDCASSERT(GetResourceManager()->GetResourceRecord(id) == NULL);
 
       D3D11ResourceRecord *record = GetResourceManager()->AddResourceRecord(id);
+      record->ResType = IdentifyTypeByPtr(wrapped);
       record->Length = 0;
 
       record->AddChunk(scope.Get());
@@ -833,7 +854,7 @@ bool WrappedID3D11Device::Serialise_CreateQuery1(SerialiserType &ser,
                                                  ID3D11Query1 **ppQuery)
 {
   SERIALISE_ELEMENT_LOCAL(Descriptor, *pQueryDesc);
-  SERIALISE_ELEMENT_LOCAL(pQuery, GetIDForResource(*ppQuery)).TypedAs("ID3D11Query1 *");
+  SERIALISE_ELEMENT_LOCAL(pQuery, GetIDForResource(*ppQuery)).TypedAs("ID3D11Query1 *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -897,6 +918,7 @@ HRESULT WrappedID3D11Device::CreateQuery1(const D3D11_QUERY_DESC1 *pQueryDesc, I
       RDCASSERT(GetResourceManager()->GetResourceRecord(id) == NULL);
 
       D3D11ResourceRecord *record = GetResourceManager()->AddResourceRecord(id);
+      record->ResType = IdentifyTypeByPtr(wrapped);
       record->Length = 0;
 
       record->AddChunk(scope.Get());
@@ -1061,4 +1083,4 @@ HRESULT WrappedID3D11Device::OpenSharedFence(HANDLE hFence, REFIID riid, void **
   template bool WrappedID3D11Device::CONCAT(Serialise_, func(ReadSerialiser &ser, __VA_ARGS__)); \
   template bool WrappedID3D11Device::CONCAT(Serialise_, func(WriteSerialiser &ser, __VA_ARGS__));
 
-SERIALISED_ID3D11DEVICE3_FUNCTIONS();
\ No newline at end of file
+SERIALISED_ID3D11DEVICE3_FUNCTIONS();
