@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -24,10 +24,6 @@
  ******************************************************************************/
 
 #include "data/resource.h"
-#include "driver/d3d11/d3d11_renderstate.h"
-#include "driver/d3d11/d3d11_resources.h"
-#include "driver/shaders/dxbc/dxbc_debug.h"
-#include "maths/camera.h"
 #include "maths/formatpacking.h"
 #include "maths/matrix.h"
 #include "maths/vec.h"
@@ -36,13 +32,17 @@
 #include "d3d11_context.h"
 #include "d3d11_debug.h"
 #include "d3d11_manager.h"
+#include "d3d11_renderstate.h"
+#include "d3d11_replay.h"
+#include "d3d11_resources.h"
 
 #include "data/hlsl/hlsl_cbuffers.h"
 
-ResourceId D3D11Replay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOverlay overlay,
-                                      uint32_t eventId, const vector<uint32_t> &passEvents)
+ResourceId D3D11Replay::RenderOverlay(ResourceId texid, CompType typeCast, FloatVector clearCol,
+                                      DebugOverlay overlay, uint32_t eventId,
+                                      const rdcarray<uint32_t> &passEvents)
 {
-  TextureShaderDetails details = GetDebugManager()->GetShaderDetails(texid, typeHint, false);
+  TextureShaderDetails details = GetDebugManager()->GetShaderDetails(texid, typeCast, false);
 
   D3D11MarkerRegion renderoverlay(StringFormat::Fmt("RenderOverlay %d", overlay));
 
@@ -572,7 +572,7 @@ ResourceId D3D11Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
   }
   else if(overlay == DebugOverlay::ClearBeforePass || overlay == DebugOverlay::ClearBeforeDraw)
   {
-    vector<uint32_t> events = passEvents;
+    rdcarray<uint32_t> events = passEvents;
 
     if(overlay == DebugOverlay::ClearBeforeDraw)
       events.clear();
@@ -607,7 +607,39 @@ ResourceId D3D11Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
 
       for(size_t i = 0; i < ARRAY_COUNT(state.OM.RenderTargets); i++)
         if(state.OM.RenderTargets[i])
-          m_pImmediateContext->ClearRenderTargetView(state.OM.RenderTargets[i], black);
+          m_pImmediateContext->ClearRenderTargetView(state.OM.RenderTargets[i], &clearCol.x);
+
+      // Try to clear depth as well, to help debug shadow rendering
+      if(state.OM.DepthView && IsDepthFormat(details.srvFormat))
+      {
+        if(state.OM.DepthStencilState)
+        {
+          D3D11_DEPTH_STENCIL_DESC desc;
+          state.OM.DepthStencilState->GetDesc(&desc);
+
+          // If the depth func is equal or not equal, don't clear at all since the output would be
+          // altered in an way that would cause replay to produce mostly incorrect results.
+          // Similarly, skip if the depth func is always, as we'd have a 50% chance of guessing the
+          // wrong clear value.
+          if(desc.DepthFunc != D3D11_COMPARISON_EQUAL &&
+             desc.DepthFunc != D3D11_COMPARISON_NOT_EQUAL &&
+             desc.DepthFunc != D3D11_COMPARISON_ALWAYS)
+          {
+            // If the depth func is less or less equal, clear to 1 instead of 0
+            bool depthFuncLess = desc.DepthFunc == D3D11_COMPARISON_LESS ||
+                                 desc.DepthFunc == D3D11_COMPARISON_LESS_EQUAL;
+            float depthClear = depthFuncLess ? 1.0f : 0.0f;
+
+            m_pImmediateContext->ClearDepthStencilView(state.OM.DepthView, D3D11_CLEAR_DEPTH,
+                                                       depthClear, 0);
+          }
+        }
+        else
+        {
+          // Without a depth stencil state set, the comparison func is D3D11_COMPARISON_LESS
+          m_pImmediateContext->ClearDepthStencilView(state.OM.DepthView, D3D11_CLEAR_DEPTH, 1.0f, 0);
+        }
+      }
 
       for(size_t i = 0; i < events.size(); i++)
       {
@@ -659,7 +691,7 @@ ResourceId D3D11Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
     float overlayConsts[] = {0.0f, 0.0f, 0.0f, 0.0f};
     m_pImmediateContext->ClearRenderTargetView(rtv, overlayConsts);
 
-    vector<uint32_t> events = passEvents;
+    rdcarray<uint32_t> events = passEvents;
 
     if(overlay == DebugOverlay::TriangleSizeDraw)
       events.clear();
@@ -785,7 +817,7 @@ ResourceId D3D11Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
   {
     SCOPED_TIMER("Quad Overdraw");
 
-    vector<uint32_t> events = passEvents;
+    rdcarray<uint32_t> events = passEvents;
 
     if(overlay == DebugOverlay::QuadOverdrawDraw)
       events.clear();
