@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,14 +22,15 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
+#include <algorithm>
 #include "data/resource.h"
-#include "driver/d3d11/d3d11_resources.h"
-#include "driver/shaders/dxbc/dxbc_debug.h"
 #include "strings/string_utils.h"
 #include "d3d11_context.h"
 #include "d3d11_debug.h"
 #include "d3d11_manager.h"
 #include "d3d11_renderstate.h"
+#include "d3d11_replay.h"
+#include "d3d11_resources.h"
 
 void D3D11Replay::InitStreamOut()
 {
@@ -204,11 +205,11 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
      ((drawcall->flags & DrawFlags::Instanced) && drawcall->numInstances == 0))
     return;
 
-  DXBC::DXBCFile *dxbcVS = wrappedVS->GetDXBC();
+  DXBC::DXBCContainer *dxbcVS = wrappedVS->GetDXBC();
 
   RDCASSERT(dxbcVS);
 
-  DXBC::DXBCFile *dxbcGS = NULL;
+  DXBC::DXBCContainer *dxbcGS = NULL;
 
   if(gs)
   {
@@ -226,7 +227,7 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
     RDCASSERT(dxbcGS);
   }
 
-  DXBC::DXBCFile *dxbcDS = NULL;
+  DXBC::DXBCContainer *dxbcDS = NULL;
 
   if(ds)
   {
@@ -244,7 +245,7 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
     RDCASSERT(dxbcDS);
   }
 
-  vector<D3D11_SO_DECLARATION_ENTRY> sodecls;
+  rdcarray<D3D11_SO_DECLARATION_ENTRY> sodecls;
 
   UINT stride = 0;
   int posidx = -1;
@@ -252,11 +253,11 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
 
   ID3D11GeometryShader *streamoutGS = NULL;
 
-  if(!dxbcVS->m_OutputSig.empty())
+  if(!dxbcVS->GetReflection()->OutputSig.empty())
   {
-    for(size_t i = 0; i < dxbcVS->m_OutputSig.size(); i++)
+    for(size_t i = 0; i < dxbcVS->GetReflection()->OutputSig.size(); i++)
     {
-      SigParameter &sign = dxbcVS->m_OutputSig[i];
+      const SigParameter &sign = dxbcVS->GetReflection()->OutputSig[i];
 
       D3D11_SO_DECLARATION_ENTRY decl;
 
@@ -283,8 +284,8 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
     if(posidx > 0)
     {
       D3D11_SO_DECLARATION_ENTRY pos = sodecls[posidx];
-      sodecls.erase(sodecls.begin() + posidx);
-      sodecls.insert(sodecls.begin(), pos);
+      sodecls.erase(posidx);
+      sodecls.insert(0, pos);
     }
 
     HRESULT hr = m_pDevice->CreateGeometryShaderWithStreamOutput(
@@ -356,7 +357,7 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
 
       SAFE_RELEASE(idxBuf);
 
-      vector<uint32_t> indices;
+      rdcarray<uint32_t> indices;
 
       uint16_t *idx16 = (uint16_t *)&idxdata[0];
       uint32_t *idx32 = (uint32_t *)&idxdata[0];
@@ -387,13 +388,13 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
         if(it != indices.end() && *it == i32)
           continue;
 
-        indices.insert(it, i32);
+        indices.insert(it - indices.begin(), i32);
       }
 
       // if we read out of bounds, we'll also have a 0 index being referenced
       // (as 0 is read). Don't insert 0 if we already have 0 though
       if(numIndices < drawcall->numIndices && (indices.empty() || indices[0] != 0))
-        indices.insert(indices.begin(), 0);
+        indices.insert(0, 0);
 
       // An index buffer could be something like: 500, 501, 502, 501, 503, 502
       // in which case we can't use the existing index buffer without filling 499 slots of vertex
@@ -408,7 +409,7 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
 
       // we use a map here since the indices may be sparse. Especially considering if an index
       // is 'invalid' like 0xcccccccc then we don't want an array of 3.4 billion entries.
-      map<uint32_t, size_t> indexRemap;
+      std::map<uint32_t, size_t> indexRemap;
       for(size_t i = 0; i < indices.size(); i++)
       {
         // by definition, this index will only appear once in indices[]
@@ -661,14 +662,14 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
     posidx = -1;
     numPosComponents = 0;
 
-    DXBC::DXBCFile *lastShader = dxbcGS;
+    DXBC::DXBCContainer *lastShader = dxbcGS;
     if(dxbcDS)
       lastShader = dxbcDS;
 
     sodecls.clear();
-    for(size_t i = 0; i < lastShader->m_OutputSig.size(); i++)
+    for(size_t i = 0; i < lastShader->GetReflection()->OutputSig.size(); i++)
     {
-      SigParameter &sign = lastShader->m_OutputSig[i];
+      const SigParameter &sign = lastShader->GetReflection()->OutputSig[i];
 
       D3D11_SO_DECLARATION_ENTRY decl;
 
@@ -699,8 +700,8 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
     if(posidx > 0)
     {
       D3D11_SO_DECLARATION_ENTRY pos = sodecls[posidx];
-      sodecls.erase(sodecls.begin() + posidx);
-      sodecls.insert(sodecls.begin(), pos);
+      sodecls.erase(posidx);
+      sodecls.insert(0, pos);
     }
 
     streamoutGS = NULL;
@@ -833,6 +834,9 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
                                              drawcall->instanceOffset);
           m_pImmediateContext->End(m_SOStatsQueries[inst - 1]);
         }
+
+        if((inst % 2000) == 0)
+          SerializeImmediateContext();
       }
     }
 
@@ -841,7 +845,7 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
 
     m_pImmediateContext->CopyResource(m_SOStagingBuffer, m_SOBuffer);
 
-    std::vector<D3D11PostVSData::InstData> instData;
+    rdcarray<D3D11PostVSData::InstData> instData;
 
     if((drawcall->flags & DrawFlags::Instanced) && drawcall->numInstances > 1)
     {
@@ -1001,37 +1005,7 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
     m_PostVSData[eventId].gsout.hasPosOut = posidx >= 0;
     m_PostVSData[eventId].gsout.idxBuf = NULL;
 
-    topo = D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST;
-
-    if(lastShader == dxbcGS)
-    {
-      for(size_t i = 0; i < dxbcGS->GetNumDeclarations(); i++)
-      {
-        const DXBC::ASMDecl &decl = dxbcGS->GetDeclaration(i);
-
-        if(decl.declaration == DXBC::OPCODE_DCL_GS_OUTPUT_PRIMITIVE_TOPOLOGY)
-        {
-          topo = decl.outTopology;
-          break;
-        }
-      }
-    }
-    else if(lastShader == dxbcDS)
-    {
-      for(size_t i = 0; i < dxbcDS->GetNumDeclarations(); i++)
-      {
-        const DXBC::ASMDecl &decl = dxbcDS->GetDeclaration(i);
-
-        if(decl.declaration == DXBC::OPCODE_DCL_TESS_DOMAIN)
-        {
-          if(decl.domain == DXBC::DOMAIN_ISOLINE)
-            topo = D3D11_PRIMITIVE_TOPOLOGY_LINELIST;
-          else
-            topo = D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST;
-          break;
-        }
-      }
-    }
+    topo = lastShader->GetOutputTopology();
 
     m_PostVSData[eventId].gsout.topo = topo;
 
@@ -1068,7 +1042,7 @@ void D3D11Replay::InitPostVSBuffers(uint32_t eventId)
   }
 }
 
-void D3D11Replay::InitPostVSBuffers(const vector<uint32_t> &passEvents)
+void D3D11Replay::InitPostVSBuffers(const rdcarray<uint32_t> &passEvents)
 {
   uint32_t prev = 0;
 
@@ -1088,4 +1062,4 @@ void D3D11Replay::InitPostVSBuffers(const vector<uint32_t> &passEvents)
     if(d)
       InitPostVSBuffers(passEvents[i]);
   }
-}
\ No newline at end of file
+}
