@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,10 +23,10 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "driver/d3d11/d3d11_renderstate.h"
-#include "driver/d3d11/d3d11_context.h"
-#include "driver/d3d11/d3d11_device.h"
-#include "driver/d3d11/d3d11_resources.h"
+#include "d3d11_renderstate.h"
+#include "d3d11_context.h"
+#include "d3d11_device.h"
+#include "d3d11_resources.h"
 
 D3D11RenderState::D3D11RenderState(D3D11RenderState::EmptyInit)
 {
@@ -466,7 +466,7 @@ bool D3D11RenderState::PredicationWouldPass()
   return PredicateValue != data;
 }
 
-void D3D11RenderState::ApplyState(WrappedID3D11DeviceContext *context)
+void D3D11RenderState::ApplyState(WrappedID3D11DeviceContext *context) const
 {
   context->ClearState();
 
@@ -539,15 +539,24 @@ void D3D11RenderState::ApplyState(WrappedID3D11DeviceContext *context)
   context->OMSetDepthStencilState(OM.DepthStencilState, OM.StencRef);
 
   if(context->IsFL11_1())
+  {
     context->OMSetRenderTargetsAndUnorderedAccessViews(
         RDCMIN(OM.UAVStartSlot, (UINT)D3D11_SIMULTANEOUS_RENDER_TARGET_COUNT), OM.RenderTargets,
         OM.DepthView, OM.UAVStartSlot, D3D11_1_UAV_SLOT_COUNT - OM.UAVStartSlot, OM.UAVs,
         UAV_keepcounts);
+  }
   else
-    context->OMSetRenderTargetsAndUnorderedAccessViews(
-        RDCMIN(OM.UAVStartSlot, (UINT)D3D11_SIMULTANEOUS_RENDER_TARGET_COUNT), OM.RenderTargets,
-        OM.DepthView, OM.UAVStartSlot, D3D11_PS_CS_UAV_REGISTER_COUNT - OM.UAVStartSlot, OM.UAVs,
-        UAV_keepcounts);
+  {
+    if(OM.UAVStartSlot == D3D11_PS_CS_UAV_REGISTER_COUNT)
+      context->OMSetRenderTargets(
+          RDCMIN(OM.UAVStartSlot, (UINT)D3D11_SIMULTANEOUS_RENDER_TARGET_COUNT), OM.RenderTargets,
+          OM.DepthView);
+    else
+      context->OMSetRenderTargetsAndUnorderedAccessViews(
+          RDCMIN(OM.UAVStartSlot, (UINT)D3D11_SIMULTANEOUS_RENDER_TARGET_COUNT), OM.RenderTargets,
+          OM.DepthView, OM.UAVStartSlot, D3D11_PS_CS_UAV_REGISTER_COUNT - OM.UAVStartSlot, OM.UAVs,
+          UAV_keepcounts);
+  }
 
   context->SetPredication(Predicate, PredicateValue);
 }
@@ -1154,7 +1163,7 @@ bool D3D11RenderState::InputAssembler::Used_VB(WrappedID3D11Device *device, uint
   if(Layout == NULL)
     return false;
 
-  const vector<D3D11_INPUT_ELEMENT_DESC> &vec = device->GetLayoutDesc(Layout);
+  const rdcarray<D3D11_INPUT_ELEMENT_DESC> &vec = device->GetLayoutDesc(Layout);
 
   for(size_t i = 0; i < vec.size(); i++)
     if(vec[i].InputSlot == slot)
@@ -1173,14 +1182,14 @@ bool D3D11RenderState::Shader::Used_CB(uint32_t slot) const
   if(shad == NULL)
     return false;
 
-  DXBC::DXBCFile *dxbc = shad->GetDXBC();
+  DXBC::DXBCContainer *dxbc = shad->GetDXBC();
 
   // have to assume it's used if there's no DXBC
   if(dxbc == NULL)
     return true;
 
-  for(size_t i = 0; i < dxbc->m_CBuffers.size(); i++)
-    if(dxbc->m_CBuffers[i].reg == slot)
+  for(size_t i = 0; i < dxbc->GetReflection()->CBuffers.size(); i++)
+    if(dxbc->GetReflection()->CBuffers[i].reg == slot)
       return true;
 
   return false;
@@ -1196,13 +1205,13 @@ bool D3D11RenderState::Shader::Used_SRV(uint32_t slot) const
   if(shad == NULL)
     return false;
 
-  DXBC::DXBCFile *dxbc = shad->GetDXBC();
+  DXBC::DXBCContainer *dxbc = shad->GetDXBC();
 
   // have to assume it's used if there's no DXBC
   if(dxbc == NULL)
     return true;
 
-  for(const DXBC::ShaderInputBind &bind : dxbc->m_SRVs)
+  for(const DXBC::ShaderInputBind &bind : dxbc->GetReflection()->SRVs)
     if(bind.reg == slot)
       return true;
 
@@ -1216,13 +1225,13 @@ bool D3D11RenderState::Shader::Used_UAV(uint32_t slot) const
   if(shad == NULL)
     return false;
 
-  DXBC::DXBCFile *dxbc = shad->GetDXBC();
+  DXBC::DXBCContainer *dxbc = shad->GetDXBC();
 
   // have to assume it's used if there's no DXBC
   if(dxbc == NULL)
     return true;
 
-  for(const DXBC::ShaderInputBind &bind : dxbc->m_UAVs)
+  for(const DXBC::ShaderInputBind &bind : dxbc->GetReflection()->UAVs)
     if(bind.reg == slot)
       return true;
 
