@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,22 +22,22 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "maths/camera.h"
 #include "maths/formatpacking.h"
 #include "maths/matrix.h"
 #include "d3d11_context.h"
 #include "d3d11_debug.h"
 #include "d3d11_device.h"
+#include "d3d11_replay.h"
 #include "d3d11_resources.h"
 
 #include "data/hlsl/hlsl_cbuffers.h"
 
-D3D11DebugManager::CacheElem &D3D11DebugManager::GetCachedElem(ResourceId id, CompType typeHint,
+D3D11DebugManager::CacheElem &D3D11DebugManager::GetCachedElem(ResourceId id, CompType typeCast,
                                                                bool raw)
 {
   for(auto it = m_ShaderItemCache.begin(); it != m_ShaderItemCache.end(); ++it)
   {
-    if(it->id == id && it->typeHint == typeHint && it->raw == raw)
+    if(it->id == id && it->typeCast == typeCast && it->raw == raw)
       return *it;
   }
 
@@ -48,11 +48,11 @@ D3D11DebugManager::CacheElem &D3D11DebugManager::GetCachedElem(ResourceId id, Co
     m_ShaderItemCache.pop_back();
   }
 
-  m_ShaderItemCache.push_front(CacheElem(id, typeHint, raw));
+  m_ShaderItemCache.insert(0, CacheElem(id, typeCast, raw));
   return m_ShaderItemCache.front();
 }
 
-TextureShaderDetails D3D11DebugManager::GetShaderDetails(ResourceId id, CompType typeHint,
+TextureShaderDetails D3D11DebugManager::GetShaderDetails(ResourceId id, CompType typeCast,
                                                          bool rawOutput)
 {
   TextureShaderDetails details;
@@ -60,7 +60,7 @@ TextureShaderDetails D3D11DebugManager::GetShaderDetails(ResourceId id, CompType
 
   bool foundResource = false;
 
-  CacheElem &cache = GetCachedElem(id, typeHint, rawOutput);
+  CacheElem &cache = GetCachedElem(id, typeCast, rawOutput);
 
   bool msaaDepth = false;
 
@@ -89,7 +89,7 @@ TextureShaderDetails D3D11DebugManager::GetShaderDetails(ResourceId id, CompType
     details.texArraySize = desc1d.ArraySize;
     details.texMips = desc1d.MipLevels;
 
-    srvFormat = GetTypedFormat(details.texFmt, typeHint);
+    srvFormat = GetTypedFormat(details.texFmt, typeCast);
 
     details.srvResource = wrapTex1D;
 
@@ -151,10 +151,11 @@ TextureShaderDetails D3D11DebugManager::GetShaderDetails(ResourceId id, CompType
       details.texType = eTexType_2DMS;
     }
 
-    if(mode == TEXDISPLAY_DEPTH_TARGET || IsDepthFormat(details.texFmt))
+    if(mode == TEXDISPLAY_DEPTH_TARGET || IsDepthFormat(details.texFmt) ||
+       IsDepthFormat(GetTypedFormat(details.texFmt, typeCast)))
     {
       details.texType = eTexType_Depth;
-      details.texFmt = GetTypedFormat(details.texFmt, typeHint);
+      details.texFmt = GetTypedFormat(details.texFmt, typeCast);
     }
 
     // backbuffer is always interpreted as SRGB data regardless of format specified:
@@ -178,7 +179,7 @@ TextureShaderDetails D3D11DebugManager::GetShaderDetails(ResourceId id, CompType
         details.texFmt = GetSRGBFormat(wrapTex2D->m_RealDescriptor->Format);
     }
 
-    srvFormat = GetTypedFormat(details.texFmt, typeHint);
+    srvFormat = GetTypedFormat(details.texFmt, typeCast);
 
     details.srvResource = wrapTex2D;
 
@@ -245,7 +246,7 @@ TextureShaderDetails D3D11DebugManager::GetShaderDetails(ResourceId id, CompType
     details.texArraySize = 1;
     details.texMips = desc3d.MipLevels;
 
-    srvFormat = GetTypedFormat(details.texFmt, typeHint);
+    srvFormat = GetTypedFormat(details.texFmt, typeCast);
 
     details.srvResource = wrapTex3D;
 
@@ -390,6 +391,7 @@ TextureShaderDetails D3D11DebugManager::GetShaderDetails(ResourceId id, CompType
       RDCERR("Failed to create cache SRV 0, type %d HRESULT: %s", details.texType, ToStr(hr).c_str());
   }
 
+  details.srvFormat = srvFormat;
   details.srv[details.texType] = cache.srv[0];
 
   if(IsYUVFormat(srvFormat))
@@ -447,12 +449,14 @@ TextureShaderDetails D3D11DebugManager::GetShaderDetails(ResourceId id, CompType
   return details;
 }
 
-bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
+bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, TexDisplayFlags flags)
 {
   TexDisplayVSCBuffer vertexData = {};
   TexDisplayPSCBuffer pixelData = {};
   HeatmapData heatmapData = {};
 
+  bool blendAlpha = (flags & eTexDisplay_BlendAlpha) != 0;
+
   {
     if(cfg.overlay == DebugOverlay::QuadOverdrawDraw || cfg.overlay == DebugOverlay::QuadOverdrawPass)
     {
@@ -479,13 +483,6 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
   vertexData.Position.x = x * (2.0f / m_OutputWidth);
   vertexData.Position.y = -y * (2.0f / m_OutputHeight);
 
-  vertexData.ScreenAspect.x =
-      (m_OutputHeight / m_OutputWidth);    // 0.5 = character width / character height
-  vertexData.ScreenAspect.y = 1.0f;
-
-  vertexData.TextureResolution.x = 1.0f / vertexData.ScreenAspect.x;
-  vertexData.TextureResolution.y = 1.0f;
-
   if(cfg.rangeMax <= cfg.rangeMin)
     cfg.rangeMax += 0.00001f;
 
@@ -509,13 +506,25 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
 
   pixelData.FlipY = cfg.flipY ? 1 : 0;
 
-  TextureShaderDetails details = GetDebugManager()->GetShaderDetails(cfg.resourceId, cfg.typeHint,
-                                                                     cfg.rawOutput ? true : false);
+  CompType typeCast = cfg.typeCast;
 
-  int sampleIdx = (int)RDCCLAMP(cfg.sampleIdx, 0U, details.sampleCount - 1);
+  // we create all proxy textures as typeless to allow us to cast, but that means if the remote API
+  // gave us a typed texture and then wants to view it 'typeless' (i.e. as it was created) we need
+  // to restore that type here.
+  if(typeCast == CompType::Typeless)
+  {
+    auto it = m_ProxyTypeCastDefault.find(cfg.resourceId);
+    if(it != m_ProxyTypeCastDefault.end())
+      typeCast = it->second;
+  }
+
+  TextureShaderDetails details =
+      GetDebugManager()->GetShaderDetails(cfg.resourceId, typeCast, cfg.rawOutput ? true : false);
+
+  int sampleIdx = (int)RDCCLAMP(cfg.subresource.sample, 0U, details.sampleCount - 1);
 
   // hacky resolve
-  if(cfg.sampleIdx == ~0U)
+  if(cfg.subresource.sample == ~0U)
     sampleIdx = -int(details.sampleCount);
 
   pixelData.SampleIdx = sampleIdx;
@@ -534,17 +543,13 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
   float tex_x = float(details.texWidth);
   float tex_y = float(details.texType == eTexType_1D ? 100 : details.texHeight);
 
-  vertexData.TextureResolution.x *= tex_x / m_OutputWidth;
-  vertexData.TextureResolution.y *= tex_y / m_OutputHeight;
-
-  pixelData.TextureResolutionPS.x = float(RDCMAX(1U, details.texWidth >> cfg.mip));
-  pixelData.TextureResolutionPS.y = float(RDCMAX(1U, details.texHeight >> cfg.mip));
-  pixelData.TextureResolutionPS.z = float(RDCMAX(1U, details.texDepth >> cfg.mip));
+  pixelData.TextureResolutionPS.x = float(RDCMAX(1U, details.texWidth >> cfg.subresource.mip));
+  pixelData.TextureResolutionPS.y = float(RDCMAX(1U, details.texHeight >> cfg.subresource.mip));
+  pixelData.TextureResolutionPS.z = float(RDCMAX(1U, details.texDepth >> cfg.subresource.mip));
 
   if(details.texArraySize > 1 && details.texType != eTexType_3D)
     pixelData.TextureResolutionPS.z = float(details.texArraySize);
 
-  vertexData.Scale = cfg.scale;
   pixelData.ScalePS = cfg.scale;
 
   pixelData.YUVDownsampleRate = details.YUVDownsampleRate;
@@ -555,20 +560,24 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
     float xscale = m_OutputWidth / tex_x;
     float yscale = m_OutputHeight / tex_y;
 
-    vertexData.Scale = RDCMIN(xscale, yscale);
+    cfg.scale = RDCMIN(xscale, yscale);
 
     if(yscale > xscale)
     {
       vertexData.Position.x = 0;
-      vertexData.Position.y = tex_y * vertexData.Scale / m_OutputHeight - 1.0f;
+      vertexData.Position.y = tex_y * cfg.scale / m_OutputHeight - 1.0f;
     }
     else
     {
       vertexData.Position.y = 0;
-      vertexData.Position.x = 1.0f - tex_x * vertexData.Scale / m_OutputWidth;
+      vertexData.Position.x = 1.0f - tex_x * cfg.scale / m_OutputWidth;
     }
   }
 
+  // normalisation factor for output * selected scale * viewport scale
+  vertexData.VertexScale.x = (tex_x / m_OutputWidth) * cfg.scale * 2.0f;
+  vertexData.VertexScale.y = (tex_y / m_OutputHeight) * cfg.scale * 2.0f;
+
   ID3D11PixelShader *customPS = NULL;
   ID3D11Buffer *customBuff = NULL;
 
@@ -581,7 +590,7 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
       auto dxbc = it->second->GetDXBC();
 
       RDCASSERT(dxbc);
-      RDCASSERT(dxbc->m_Type == D3D11_ShaderType_Pixel);
+      RDCASSERT(dxbc->m_Type == DXBC::ShaderType::Pixel);
 
       if(m_pDevice->GetResourceManager()->HasLiveResource(cfg.customShaderId))
       {
@@ -591,9 +600,9 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
 
         customPS = wrapped;
 
-        for(size_t i = 0; i < dxbc->m_CBuffers.size(); i++)
+        for(size_t i = 0; i < dxbc->GetReflection()->CBuffers.size(); i++)
         {
-          const DXBC::CBuffer &cbuf = dxbc->m_CBuffers[i];
+          const DXBC::CBuffer &cbuf = dxbc->GetReflection()->CBuffers[i];
           if(cbuf.name == "$Globals")
           {
             float *cbufData = new float[cbuf.descriptor.byteSize / sizeof(float) + 1];
@@ -640,7 +649,7 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
                 {
                   uint32_t *d = (uint32_t *)(byteData + var.descriptor.offset);
 
-                  d[0] = cfg.mip;
+                  d[0] = cfg.subresource.mip;
                 }
                 else
                 {
@@ -655,7 +664,7 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
                 {
                   uint32_t *d = (uint32_t *)(byteData + var.descriptor.offset);
 
-                  d[0] = cfg.sliceFace;
+                  d[0] = cfg.subresource.slice;
                 }
                 else
                 {
@@ -670,7 +679,7 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
                 {
                   int32_t *d = (int32_t *)(byteData + var.descriptor.offset);
 
-                  d[0] = cfg.sampleIdx;
+                  d[0] = cfg.subresource.sample;
                 }
                 else
                 {
@@ -708,16 +717,15 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
     }
   }
 
-  vertexData.Scale *= 2.0f;    // viewport is -1 -> 1
-
-  pixelData.MipLevel = (float)cfg.mip;
+  pixelData.MipLevel = (float)cfg.subresource.mip;
   pixelData.OutputDisplayFormat = RESTYPE_TEX2D;
-  pixelData.Slice = float(RDCCLAMP(cfg.sliceFace, 0U, details.texArraySize - 1));
+  pixelData.Slice = float(RDCCLAMP(cfg.subresource.slice, 0U, details.texArraySize - 1));
 
   if(details.texType == eTexType_3D)
   {
     pixelData.OutputDisplayFormat = RESTYPE_TEX3D;
-    pixelData.Slice = float(cfg.sliceFace >> cfg.mip);
+    pixelData.Slice = float(
+        RDCCLAMP(cfg.subresource.slice, 0U, (details.texDepth >> cfg.subresource.mip) - 1) + 0.001f);
   }
   else if(details.texType == eTexType_1D)
   {
@@ -756,19 +764,17 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
 
   int srvOffset = 0;
 
-  if(IsUIntFormat(details.texFmt) ||
-     (IsTypelessFormat(details.texFmt) && cfg.typeHint == CompType::UInt))
+  if(IsUIntFormat(details.texFmt) || (IsTypelessFormat(details.texFmt) && typeCast == CompType::UInt))
   {
     pixelData.OutputDisplayFormat |= TEXDISPLAY_UINT_TEX;
     srvOffset = 10;
   }
-  if(IsIntFormat(details.texFmt) ||
-     (IsTypelessFormat(details.texFmt) && cfg.typeHint == CompType::SInt))
+  if(IsIntFormat(details.texFmt) || (IsTypelessFormat(details.texFmt) && typeCast == CompType::SInt))
   {
     pixelData.OutputDisplayFormat |= TEXDISPLAY_SINT_TEX;
     srvOffset = 20;
   }
-  if(!IsSRGBFormat(details.texFmt) && cfg.linearDisplayAsGamma)
+  if(!IsSRGBFormat(details.texFmt) && !IsSRGBFormat(details.srvFormat) && cfg.linearDisplayAsGamma)
   {
     pixelData.OutputDisplayFormat |= TEXDISPLAY_GAMMA_CURVE;
   }
@@ -791,17 +797,25 @@ bool D3D11Replay::RenderTextureInternal(TextureDisplay cfg, bool blendAlpha)
 
     m_pImmediateContext->RSSetState(m_General.RasterState);
 
-    if(customPS == NULL)
-    {
-      m_pImmediateContext->PSSetShader(m_TexRender.TexDisplayPS, NULL, 0);
-      m_pImmediateContext->PSSetConstantBuffers(0, 1, &psCBuffer);
-      m_pImmediateContext->PSSetConstantBuffers(1, 1, &psHeatCBuffer);
-    }
-    else
+    if(customPS)
     {
       m_pImmediateContext->PSSetShader(customPS, NULL, 0);
       m_pImmediateContext->PSSetConstantBuffers(0, 1, &customBuff);
     }
+    else
+    {
+      m_pImmediateContext->PSSetConstantBuffers(0, 1, &psCBuffer);
+      m_pImmediateContext->PSSetConstantBuffers(1, 1, &psHeatCBuffer);
+
+      if(flags & eTexDisplay_RemapFloat)
+        m_pImmediateContext->PSSetShader(m_TexRender.TexRemapPS[0], NULL, 0);
+      else if(flags & eTexDisplay_RemapUInt)
+        m_pImmediateContext->PSSetShader(m_TexRender.TexRemapPS[1], NULL, 0);
+      else if(flags & eTexDisplay_RemapSInt)
+        m_pImmediateContext->PSSetShader(m_TexRender.TexRemapPS[2], NULL, 0);
+      else
+        m_pImmediateContext->PSSetShader(m_TexRender.TexDisplayPS, NULL, 0);
+    }
 
     ID3D11UnorderedAccessView *NullUAVs[D3D11_1_UAV_SLOT_COUNT] = {0};
     UINT UAV_keepcounts[D3D11_1_UAV_SLOT_COUNT];
