@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,13 +23,13 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "driver/d3d11/d3d11_resources.h"
+#include "d3d11_resources.h"
 #include "3rdparty/lz4/lz4.h"
 #include "api/app/renderdoc_app.h"
-#include "driver/d3d11/d3d11_context.h"
-#include "driver/d3d11/d3d11_renderstate.h"
 #include "driver/dxgi/dxgi_wrapped.h"
 #include "driver/shaders/dxbc/dxbc_reflect.h"
+#include "d3d11_context.h"
+#include "d3d11_renderstate.h"
 
 WRAPPED_POOL_INST(WrappedID3D11Buffer);
 WRAPPED_POOL_INST(WrappedID3D11Texture1D);
@@ -58,29 +58,29 @@ WRAPPED_POOL_INST(WrappedID3D11ClassLinkage);
 WRAPPED_POOL_INST(WrappedID3DDeviceContextState);
 WRAPPED_POOL_INST(WrappedID3D11Fence);
 
-map<ResourceId, WrappedID3D11Texture1D::TextureEntry>
+std::map<ResourceId, WrappedID3D11Texture1D::TextureEntry>
     WrappedTexture<ID3D11Texture1D, D3D11_TEXTURE1D_DESC, ID3D11Texture1D>::m_TextureList;
-map<ResourceId, WrappedID3D11Texture2D1::TextureEntry>
+std::map<ResourceId, WrappedID3D11Texture2D1::TextureEntry>
     WrappedTexture<ID3D11Texture2D, D3D11_TEXTURE2D_DESC, ID3D11Texture2D1>::m_TextureList;
-map<ResourceId, WrappedID3D11Texture3D1::TextureEntry>
+std::map<ResourceId, WrappedID3D11Texture3D1::TextureEntry>
     WrappedTexture<ID3D11Texture3D, D3D11_TEXTURE3D_DESC, ID3D11Texture3D1>::m_TextureList;
-map<ResourceId, WrappedID3D11Buffer::BufferEntry> WrappedID3D11Buffer::m_BufferList;
-map<ResourceId, WrappedShader::ShaderEntry *> WrappedShader::m_ShaderList;
+std::map<ResourceId, WrappedID3D11Buffer::BufferEntry> WrappedID3D11Buffer::m_BufferList;
+std::map<ResourceId, WrappedShader::ShaderEntry *> WrappedShader::m_ShaderList;
 Threading::CriticalSection WrappedShader::m_ShaderListLock;
-std::vector<WrappedID3DDeviceContextState *> WrappedID3DDeviceContextState::m_List;
+rdcarray<WrappedID3DDeviceContextState *> WrappedID3DDeviceContextState::m_List;
 Threading::CriticalSection WrappedID3DDeviceContextState::m_Lock;
 
 const GUID RENDERDOC_ID3D11ShaderGUID_ShaderDebugMagicValue = RENDERDOC_ShaderDebugMagicValue_struct;
 
 void WrappedShader::ShaderEntry::TryReplaceOriginalByteCode()
 {
-  if(!DXBC::DXBCFile::CheckForDebugInfo((const void *)&m_Bytecode[0], m_Bytecode.size()))
+  if(!DXBC::DXBCContainer::CheckForDebugInfo((const void *)&m_Bytecode[0], m_Bytecode.size()))
   {
-    string originalPath = m_DebugInfoPath;
+    rdcstr originalPath = m_DebugInfoPath;
 
     if(originalPath.empty())
       originalPath =
-          DXBC::DXBCFile::GetDebugBinaryPath((const void *)&m_Bytecode[0], m_Bytecode.size());
+          DXBC::DXBCContainer::GetDebugBinaryPath((const void *)&m_Bytecode[0], m_Bytecode.size());
 
     if(!originalPath.empty())
     {
@@ -97,7 +97,7 @@ void WrappedShader::ShaderEntry::TryReplaceOriginalByteCode()
 
       size_t numSearchPaths = m_DebugInfoSearchPaths ? m_DebugInfoSearchPaths->size() : 0;
 
-      string foundPath;
+      rdcstr foundPath;
 
       // while we haven't found a file, keep trying through the search paths. For i==0
       // check the path on its own, in case it's an absolute path.
@@ -111,7 +111,7 @@ void WrappedShader::ShaderEntry::TryReplaceOriginalByteCode()
         }
         else
         {
-          const std::string &searchPath = (*m_DebugInfoSearchPaths)[i - 1];
+          const rdcstr &searchPath = (*m_DebugInfoSearchPaths)[i - 1];
           foundPath = searchPath + "/" + originalPath;
           originalShaderFile = FileIO::fopen(foundPath.c_str(), "rb");
         }
@@ -126,7 +126,7 @@ void WrappedShader::ShaderEntry::TryReplaceOriginalByteCode()
 
       if(lz4 || originalShaderSize >= m_Bytecode.size())
       {
-        vector<byte> originalBytecode;
+        rdcarray<byte> originalBytecode;
 
         originalBytecode.resize((size_t)originalShaderSize);
         FileIO::fread(&originalBytecode[0], sizeof(byte), (size_t)originalShaderSize,
@@ -134,7 +134,7 @@ void WrappedShader::ShaderEntry::TryReplaceOriginalByteCode()
 
         if(lz4)
         {
-          vector<byte> decompressed;
+          rdcarray<byte> decompressed;
 
           // first try decompressing to 1MB flat
           decompressed.resize(100 * 1024);
@@ -166,8 +166,8 @@ void WrappedShader::ShaderEntry::TryReplaceOriginalByteCode()
           memcpy(&originalBytecode[0], &decompressed[0], originalBytecode.size());
         }
 
-        if(DXBC::DXBCFile::CheckForDebugInfo((const void *)&originalBytecode[0],
-                                             originalBytecode.size()))
+        if(DXBC::DXBCContainer::CheckForDebugInfo((const void *)&originalBytecode[0],
+                                                  originalBytecode.size()))
         {
           m_Bytecode.swap(originalBytecode);
         }
@@ -433,6 +433,9 @@ D3D11ResourceType IdentifyTypeByPtr(IUnknown *ptr)
   if(WrappedID3DDeviceContextState::IsAlloc(ptr))
     return Resource_DeviceState;
 
+  if(WrappedID3D11Fence::IsAlloc(ptr))
+    return Resource_Fence;
+
   RDCERR("Unknown type for ptr 0x%p", ptr);
 
   return Resource_Unknown;
@@ -645,8 +648,6 @@ WrappedID3DDeviceContextState::~WrappedID3DDeviceContextState()
 
   {
     SCOPED_LOCK(WrappedID3DDeviceContextState::m_Lock);
-    auto it = std::find(WrappedID3DDeviceContextState::m_List.begin(),
-                        WrappedID3DDeviceContextState::m_List.end(), this);
-    WrappedID3DDeviceContextState::m_List.erase(it);
+    WrappedID3DDeviceContextState::m_List.removeOne(this);
   }
 }
