@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,10 +25,9 @@
 
 #pragma once
 
-#include <algorithm>
-#include "driver/d3d11/d3d11_device.h"
-#include "driver/d3d11/d3d11_manager.h"
-#include "driver/shaders/dxbc/dxbc_inspect.h"
+#include "driver/shaders/dxbc/dxbc_container.h"
+#include "d3d11_device.h"
+#include "d3d11_manager.h"
 
 D3D11ResourceType IdentifyTypeByPtr(IUnknown *ptr);
 ResourceId GetIDForDeviceChild(ID3D11DeviceChild *ptr);
@@ -193,7 +192,7 @@ public:
        riid == __uuidof(IDXGIResource1) || riid == __uuidof(IDXGISurface2))
     {
       // ensure the real object has this interface
-      void *outObj;
+      void *outObj = NULL;
       HRESULT hr = m_pReal->QueryInterface(riid, &outObj);
 
       IUnknown *unk = (IUnknown *)outObj;
@@ -294,7 +293,7 @@ public:
       const char *pStrData = (const char *)pData;
       if(DataSize != 0 && pStrData[DataSize - 1] != '\0')
       {
-        string sName(pStrData, DataSize);
+        rdcstr sName(pStrData, DataSize);
         m_pDevice->SetResourceName(this, sName.c_str());
       }
       else
@@ -302,6 +301,13 @@ public:
         m_pDevice->SetResourceName(this, pStrData);
       }
     }
+    else if(guid == WKPDID_D3DDebugObjectNameW)
+    {
+      const wchar_t *pStrData = (const wchar_t *)pData;
+      rdcwstr wName(pStrData, DataSize / 2);
+      rdcstr sName = StringFormat::Wide2UTF8(wName);
+      m_pDevice->SetResourceName(this, sName.c_str());
+    }
 
     return m_pReal->SetPrivateData(guid, DataSize, pData);
   }
@@ -432,7 +438,7 @@ public:
     uint32_t length;
   };
 
-  static map<ResourceId, BufferEntry> m_BufferList;
+  static std::map<ResourceId, BufferEntry> m_BufferList;
 
   static const int AllocPoolCount = 128 * 1024;
   static const int AllocPoolMaxByteSize = 13 * 1024 * 1024;
@@ -441,18 +447,24 @@ public:
   WrappedID3D11Buffer(ID3D11Buffer *real, uint32_t byteLength, WrappedID3D11Device *device)
       : WrappedResource11(real, device)
   {
-    SCOPED_LOCK(m_pDevice->D3DLock());
+    if(RenderDoc::Inst().IsReplayApp())
+    {
+      SCOPED_LOCK(m_pDevice->D3DLock());
 
-    RDCASSERT(m_BufferList.find(GetResourceID()) == m_BufferList.end());
-    m_BufferList[GetResourceID()] = BufferEntry(this, byteLength);
+      RDCASSERT(m_BufferList.find(GetResourceID()) == m_BufferList.end());
+      m_BufferList[GetResourceID()] = BufferEntry(this, byteLength);
+    }
   }
 
   virtual ~WrappedID3D11Buffer()
   {
     SCOPED_LOCK(m_pDevice->D3DLock());
 
-    if(m_BufferList.find(GetResourceID()) != m_BufferList.end())
-      m_BufferList.erase(GetResourceID());
+    if(RenderDoc::Inst().IsReplayApp())
+    {
+      if(m_BufferList.find(GetResourceID()) != m_BufferList.end())
+        m_BufferList.erase(GetResourceID());
+    }
 
     Shutdown();
   }
@@ -478,17 +490,20 @@ public:
     TextureDisplayType m_Type;
   };
 
-  static map<ResourceId, TextureEntry> m_TextureList;
+  static std::map<ResourceId, TextureEntry> m_TextureList;
 
   WrappedTexture(NestedType *real, WrappedID3D11Device *device, TextureDisplayType type)
       : WrappedResource11(real, device)
   {
     if(type != TEXDISPLAY_UNKNOWN)
     {
-      SCOPED_LOCK(m_pDevice->D3DLock());
+      if(RenderDoc::Inst().IsReplayApp())
+      {
+        SCOPED_LOCK(m_pDevice->D3DLock());
 
-      RDCASSERT(m_TextureList.find(GetResourceID()) == m_TextureList.end());
-      m_TextureList[GetResourceID()] = TextureEntry(this, type);
+        RDCASSERT(m_TextureList.find(GetResourceID()) == m_TextureList.end());
+        m_TextureList[GetResourceID()] = TextureEntry(this, type);
+      }
     }
   }
 
@@ -496,8 +511,11 @@ public:
   {
     SCOPED_LOCK(m_pDevice->D3DLock());
 
-    if(m_TextureList.find(GetResourceID()) != m_TextureList.end())
-      m_TextureList.erase(GetResourceID());
+    if(RenderDoc::Inst().IsReplayApp())
+    {
+      if(m_TextureList.find(GetResourceID()) != m_TextureList.end())
+        m_TextureList.erase(GetResourceID());
+    }
 
     Shutdown();
   }
@@ -891,7 +909,7 @@ public:
     ShaderEntry(WrappedID3D11Device *device, ResourceId id, const byte *code, size_t codeLen)
     {
       m_ID = id;
-      m_Bytecode.assign(code, code + codeLen);
+      m_Bytecode.assign(code, codeLen);
       m_DebugInfoSearchPaths = device->GetShaderDebugInfoSearchPaths();
       m_DXBCFile = NULL;
     }
@@ -901,13 +919,13 @@ public:
       SAFE_DELETE(m_DXBCFile);
     }
 
-    void SetDebugInfoPath(const std::string &path) { m_DebugInfoPath = path; }
-    DXBC::DXBCFile *GetDXBC()
+    void SetDebugInfoPath(const rdcstr &path) { m_DebugInfoPath = path; }
+    DXBC::DXBCContainer *GetDXBC()
     {
       if(m_DXBCFile == NULL && !m_Bytecode.empty())
       {
         TryReplaceOriginalByteCode();
-        m_DXBCFile = new DXBC::DXBCFile((const void *)&m_Bytecode[0], m_Bytecode.size());
+        m_DXBCFile = new DXBC::DXBCContainer((const void *)&m_Bytecode[0], m_Bytecode.size());
       }
       return m_DXBCFile;
     }
@@ -937,18 +955,18 @@ public:
 
     ResourceId m_ID;
 
-    std::string m_DebugInfoPath;
-    vector<std::string> *m_DebugInfoSearchPaths;
+    rdcstr m_DebugInfoPath;
+    rdcarray<rdcstr> *m_DebugInfoSearchPaths;
 
-    vector<byte> m_Bytecode;
+    rdcarray<byte> m_Bytecode;
 
     bool m_Built = false;
-    DXBC::DXBCFile *m_DXBCFile;
+    DXBC::DXBCContainer *m_DXBCFile;
     ShaderReflection m_Details;
     ShaderBindpointMapping m_Mapping;
   };
 
-  static map<ResourceId, ShaderEntry *> m_ShaderList;
+  static std::map<ResourceId, ShaderEntry *> m_ShaderList;
   static Threading::CriticalSection m_ShaderListLock;
 
   WrappedShader(WrappedID3D11Device *device, ResourceId origId, ResourceId liveId, const byte *code,
@@ -973,7 +991,7 @@ public:
     }
   }
 
-  DXBC::DXBCFile *GetDXBC()
+  DXBC::DXBCContainer *GetDXBC()
   {
     SCOPED_LOCK(m_ShaderListLock);
     return m_ShaderList[m_ID]->GetDXBC();
@@ -1265,8 +1283,8 @@ class WrappedID3D11CommandList : public WrappedDeviceChild11<ID3D11CommandList>
   bool m_Successful;    // indicates whether we have all of the commands serialised for this command
                         // list
 
-  set<ResourceId> m_Dirty;
-  set<ResourceId> m_References;
+  std::set<ResourceId> m_Dirty;
+  std::set<ResourceId> m_References;
 
 public:
   ALLOCATE_WITH_WRAPPED_POOL(WrappedID3D11CommandList);
@@ -1294,14 +1312,14 @@ public:
 
   WrappedID3D11DeviceContext *GetContext() { return m_pContext; }
   bool IsCaptured() { return m_Successful; }
-  void SwapReferences(set<ResourceId> &refs) { m_References.swap(refs); }
-  void SwapDirtyResources(set<ResourceId> &dirty) { m_Dirty.swap(dirty); }
+  void SwapReferences(std::set<ResourceId> &refs) { m_References.swap(refs); }
+  void SwapDirtyResources(std::set<ResourceId> &dirty) { m_Dirty.swap(dirty); }
   void MarkDirtyResources(D3D11ResourceManager *manager)
   {
     for(auto it = m_Dirty.begin(); it != m_Dirty.end(); ++it)
       manager->MarkDirtyResource(*it);
   }
-  void MarkDirtyResources(set<ResourceId> &missingTracks)
+  void MarkDirtyResources(std::set<ResourceId> &missingTracks)
   {
     for(auto it = m_Dirty.begin(); it != m_Dirty.end(); ++it)
       missingTracks.insert(*it);
@@ -1318,7 +1336,7 @@ class WrappedID3DDeviceContextState : public WrappedDeviceChild11<ID3DDeviceCont
 public:
   ALLOCATE_WITH_WRAPPED_POOL(WrappedID3DDeviceContextState);
 
-  static std::vector<WrappedID3DDeviceContextState *> m_List;
+  static rdcarray<WrappedID3DDeviceContextState *> m_List;
   static Threading::CriticalSection m_Lock;
   D3D11RenderState *state;
 
@@ -1476,4 +1494,4 @@ template <>
 inline ResourceId GetIDForResource(ID3D11View *v)
 {
   return GetIDForDeviceChild(v);
-}
\ No newline at end of file
+}
