@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,7 @@
 #include "d3d11_shader_cache.h"
 #include "common/shader_cache.h"
 #include "driver/dx/official/d3dcompiler.h"
-#include "driver/shaders/dxbc/dxbc_inspect.h"
+#include "driver/shaders/dxbc/dxbc_container.h"
 #include "strings/string_utils.h"
 #include "d3d11_device.h"
 #include "d3d11_resources.h"
@@ -34,25 +34,34 @@ typedef HRESULT(WINAPI *pD3DCreateBlob)(SIZE_T Size, ID3DBlob **ppBlob);
 
 struct D3DBlobShaderCallbacks
 {
-  D3DBlobShaderCallbacks()
+  pD3DCreateBlob GetCreateBlob() const
   {
-    HMODULE d3dcompiler = GetD3DCompiler();
+    static pD3DCreateBlob blobCreate = NULL;
 
-    if(d3dcompiler == NULL)
-      RDCFATAL("Can't get handle to d3dcompiler_??.dll");
+    if(!blobCreate)
+    {
+      HMODULE d3dcompiler = GetD3DCompiler();
 
-    m_BlobCreate = (pD3DCreateBlob)GetProcAddress(d3dcompiler, "D3DCreateBlob");
+      if(d3dcompiler == NULL)
+        RDCFATAL("Can't get handle to d3dcompiler_??.dll");
 
-    if(m_BlobCreate == NULL)
-      RDCFATAL("d3dcompiler.dll doesn't contain D3DCreateBlob");
+      blobCreate = (pD3DCreateBlob)GetProcAddress(d3dcompiler, "D3DCreateBlob");
+
+      if(blobCreate == NULL)
+        RDCFATAL("d3dcompiler.dll doesn't contain D3DCreateBlob");
+    }
+
+    return blobCreate;
   }
 
   bool Create(uint32_t size, byte *data, ID3DBlob **ret) const
   {
     RDCASSERT(ret);
 
+    pD3DCreateBlob blobCreate = GetCreateBlob();
+
     *ret = NULL;
-    HRESULT hr = m_BlobCreate((SIZE_T)size, ret);
+    HRESULT hr = blobCreate((SIZE_T)size, ret);
 
     if(FAILED(hr))
     {
@@ -68,18 +77,18 @@ struct D3DBlobShaderCallbacks
   void Destroy(ID3DBlob *blob) const { blob->Release(); }
   uint32_t GetSize(ID3DBlob *blob) const { return (uint32_t)blob->GetBufferSize(); }
   const byte *GetData(ID3DBlob *blob) const { return (const byte *)blob->GetBufferPointer(); }
-  pD3DCreateBlob m_BlobCreate;
+  pD3DCreateBlob m_BlobCreate = NULL;
 } D3D11ShaderCacheCallbacks;
 
 struct EmbeddedD3D11Includer : public ID3DInclude
 {
-  std::string texsample = GetEmbeddedResource(hlsl_texsample_h);
-  std::string cbuffers = GetEmbeddedResource(hlsl_cbuffers_h);
+  rdcstr texsample = GetEmbeddedResource(hlsl_texsample_h);
+  rdcstr cbuffers = GetEmbeddedResource(hlsl_cbuffers_h);
 
   virtual HRESULT STDMETHODCALLTYPE Open(D3D_INCLUDE_TYPE IncludeType, LPCSTR pFileName,
                                          LPCVOID pParentData, LPCVOID *ppData, UINT *pBytes) override
   {
-    std::string *str;
+    rdcstr *str;
 
     if(!strcmp(pFileName, "hlsl_texsample.h"))
       str = &texsample;
@@ -123,9 +132,9 @@ D3D11ShaderCache::~D3D11ShaderCache()
   }
 }
 
-std::string D3D11ShaderCache::GetShaderBlob(const char *source, const char *entry,
-                                            const uint32_t compileFlags, const char *profile,
-                                            ID3DBlob **srcblob)
+rdcstr D3D11ShaderCache::GetShaderBlob(const char *source, const char *entry,
+                                       const uint32_t compileFlags, const char *profile,
+                                       ID3DBlob **srcblob)
 {
   EmbeddedD3D11Includer includer;
 
@@ -167,13 +176,13 @@ std::string D3D11ShaderCache::GetShaderBlob(const char *source, const char *entr
   hr = compileFunc(source, strlen(source), entry, NULL, &includer, entry, profile, flags, 0,
                    &byteBlob, &errBlob);
 
-  std::string errors = "";
+  rdcstr errors = "";
 
   if(errBlob)
   {
     errors = (char *)errBlob->GetBufferPointer();
 
-    std::string logerror = errors;
+    rdcstr logerror = errors;
     if(logerror.length() > 1024)
       logerror = logerror.substr(0, 1024) + "...";
 
@@ -204,7 +213,7 @@ std::string D3D11ShaderCache::GetShaderBlob(const char *source, const char *entr
 ID3D11VertexShader *D3D11ShaderCache::MakeVShader(const char *source, const char *entry,
                                                   const char *profile, int numInputDescs,
                                                   D3D11_INPUT_ELEMENT_DESC *inputs,
-                                                  ID3D11InputLayout **ret, std::vector<byte> *blob)
+                                                  ID3D11InputLayout **ret, rdcarray<byte> *blob)
 {
   ID3DBlob *byteBlob = NULL;
 
