@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,7 @@
 #include "d3d11_resources.h"
 
 template <>
-std::string DoStringise(const D3D11ResourceType &el)
+rdcstr DoStringise(const D3D11ResourceType &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11ResourceType);
   {
@@ -57,13 +57,13 @@ std::string DoStringise(const D3D11ResourceType &el)
 }
 
 template <>
-std::string DoStringise(const D3D11Chunk &el)
+rdcstr DoStringise(const D3D11Chunk &el)
 {
-  RDCCOMPILE_ASSERT((uint32_t)D3D11Chunk::Max == 1127, "Chunks changed without updating names");
+  RDCCOMPILE_ASSERT((uint32_t)D3D11Chunk::Max == 1130, "Chunks changed without updating names");
 
   BEGIN_ENUM_STRINGISE(D3D11Chunk)
   {
-    STRINGISE_ENUM_CLASS_NAMED(DeviceInitialisation, "Device Initialisation");
+    STRINGISE_ENUM_CLASS_NAMED(DeviceInitialisation, "Internal: Device Initialisation");
     STRINGISE_ENUM_CLASS_NAMED(SetResourceName, "ID3D11Resource::SetDebugName");
     STRINGISE_ENUM_CLASS_NAMED(CreateSwapBuffer, "IDXGISwapChain::GetBuffer");
     STRINGISE_ENUM_CLASS_NAMED(CreateTexture1D, "ID3D11Device::CreateTexture1D");
@@ -191,7 +191,7 @@ std::string DoStringise(const D3D11Chunk &el)
     STRINGISE_ENUM_CLASS_NAMED(PushMarker, "Push Debug Region");
     STRINGISE_ENUM_CLASS_NAMED(SetMarker, "Set Marker");
     STRINGISE_ENUM_CLASS_NAMED(PopMarker, "Pop Debug Region");
-    STRINGISE_ENUM_CLASS_NAMED(SetShaderDebugPath, "SetShaderDebugPath");
+    STRINGISE_ENUM_CLASS_NAMED(SetShaderDebugPath, "Internal: SetShaderDebugPath");
     STRINGISE_ENUM_CLASS_NAMED(DiscardResource, "ID3D11DeviceContext1::DiscardResource");
     STRINGISE_ENUM_CLASS_NAMED(DiscardView, "ID3D11DeviceContext1::DiscardView");
     STRINGISE_ENUM_CLASS_NAMED(DiscardView1, "ID3D11DeviceContext1::DiscardView1");
@@ -209,13 +209,16 @@ std::string DoStringise(const D3D11Chunk &el)
     STRINGISE_ENUM_CLASS_NAMED(PostFinishCommandListSet, "ID3D11DeviceContext::FinishCommandList");
     STRINGISE_ENUM_CLASS_NAMED(SwapDeviceContextState,
                                "ID3D11DeviceContext1::SwapDeviceContextState");
+    STRINGISE_ENUM_CLASS_NAMED(ExternalDXGIResource, "External DXGI Resource import");
+    STRINGISE_ENUM_CLASS_NAMED(OpenSharedResource1, "ID3D11Device1::OpenSharedResource1");
+    STRINGISE_ENUM_CLASS_NAMED(OpenSharedResourceByName, "ID3D11Device1::OpenSharedResourceByName");
     STRINGISE_ENUM_CLASS_NAMED(Max, "Max Chunk");
   }
   END_ENUM_STRINGISE()
 }
 
 template <>
-std::string DoStringise(const D3D11_BIND_FLAG &el)
+rdcstr DoStringise(const D3D11_BIND_FLAG &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_BIND_FLAG);
   {
@@ -236,7 +239,7 @@ std::string DoStringise(const D3D11_BIND_FLAG &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_CPU_ACCESS_FLAG &el)
+rdcstr DoStringise(const D3D11_CPU_ACCESS_FLAG &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_CPU_ACCESS_FLAG);
   {
@@ -249,7 +252,7 @@ std::string DoStringise(const D3D11_CPU_ACCESS_FLAG &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_RESOURCE_MISC_FLAG &el)
+rdcstr DoStringise(const D3D11_RESOURCE_MISC_FLAG &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_RESOURCE_MISC_FLAG);
   {
@@ -277,7 +280,7 @@ std::string DoStringise(const D3D11_RESOURCE_MISC_FLAG &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_COLOR_WRITE_ENABLE &el)
+rdcstr DoStringise(const D3D11_COLOR_WRITE_ENABLE &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_COLOR_WRITE_ENABLE);
   {
@@ -293,7 +296,7 @@ std::string DoStringise(const D3D11_COLOR_WRITE_ENABLE &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_BUFFER_UAV_FLAG &el)
+rdcstr DoStringise(const D3D11_BUFFER_UAV_FLAG &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_BUFFER_UAV_FLAG);
   {
@@ -307,7 +310,7 @@ std::string DoStringise(const D3D11_BUFFER_UAV_FLAG &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_DSV_FLAG &el)
+rdcstr DoStringise(const D3D11_DSV_FLAG &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_DSV_FLAG);
   {
@@ -320,7 +323,7 @@ std::string DoStringise(const D3D11_DSV_FLAG &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_COPY_FLAGS &el)
+rdcstr DoStringise(const D3D11_COPY_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_COPY_FLAGS);
   {
@@ -333,7 +336,7 @@ std::string DoStringise(const D3D11_COPY_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_MAP_FLAG &el)
+rdcstr DoStringise(const D3D11_MAP_FLAG &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_MAP_FLAG);
   {
@@ -345,7 +348,7 @@ std::string DoStringise(const D3D11_MAP_FLAG &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_CLEAR_FLAG &el)
+rdcstr DoStringise(const D3D11_CLEAR_FLAG &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_CLEAR_FLAG);
   {
@@ -358,7 +361,7 @@ std::string DoStringise(const D3D11_CLEAR_FLAG &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_BUFFEREX_SRV_FLAG &el)
+rdcstr DoStringise(const D3D11_BUFFEREX_SRV_FLAG &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D11_BUFFEREX_SRV_FLAG);
   {
@@ -370,7 +373,7 @@ std::string DoStringise(const D3D11_BUFFEREX_SRV_FLAG &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_TEXTURE_LAYOUT &el)
+rdcstr DoStringise(const D3D11_TEXTURE_LAYOUT &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_TEXTURE_LAYOUT);
   {
@@ -382,7 +385,7 @@ std::string DoStringise(const D3D11_TEXTURE_LAYOUT &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_DEPTH_WRITE_MASK &el)
+rdcstr DoStringise(const D3D11_DEPTH_WRITE_MASK &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_DEPTH_WRITE_MASK);
   {
@@ -393,7 +396,7 @@ std::string DoStringise(const D3D11_DEPTH_WRITE_MASK &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_COMPARISON_FUNC &el)
+rdcstr DoStringise(const D3D11_COMPARISON_FUNC &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_COMPARISON_FUNC);
   {
@@ -410,7 +413,7 @@ std::string DoStringise(const D3D11_COMPARISON_FUNC &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_STENCIL_OP &el)
+rdcstr DoStringise(const D3D11_STENCIL_OP &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_STENCIL_OP);
   {
@@ -427,7 +430,7 @@ std::string DoStringise(const D3D11_STENCIL_OP &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_BLEND &el)
+rdcstr DoStringise(const D3D11_BLEND &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_BLEND);
   {
@@ -453,7 +456,7 @@ std::string DoStringise(const D3D11_BLEND &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_BLEND_OP &el)
+rdcstr DoStringise(const D3D11_BLEND_OP &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_BLEND_OP);
   {
@@ -467,7 +470,7 @@ std::string DoStringise(const D3D11_BLEND_OP &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_CULL_MODE &el)
+rdcstr DoStringise(const D3D11_CULL_MODE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_CULL_MODE);
   {
@@ -479,7 +482,7 @@ std::string DoStringise(const D3D11_CULL_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_FILL_MODE &el)
+rdcstr DoStringise(const D3D11_FILL_MODE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_FILL_MODE);
   {
@@ -490,7 +493,7 @@ std::string DoStringise(const D3D11_FILL_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_CONSERVATIVE_RASTERIZATION_MODE &el)
+rdcstr DoStringise(const D3D11_CONSERVATIVE_RASTERIZATION_MODE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_CONSERVATIVE_RASTERIZATION_MODE);
   {
@@ -501,7 +504,7 @@ std::string DoStringise(const D3D11_CONSERVATIVE_RASTERIZATION_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_TEXTURE_ADDRESS_MODE &el)
+rdcstr DoStringise(const D3D11_TEXTURE_ADDRESS_MODE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_TEXTURE_ADDRESS_MODE);
   {
@@ -515,7 +518,7 @@ std::string DoStringise(const D3D11_TEXTURE_ADDRESS_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_FILTER &el)
+rdcstr DoStringise(const D3D11_FILTER &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_FILTER);
   {
@@ -542,7 +545,7 @@ std::string DoStringise(const D3D11_FILTER &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_SRV_DIMENSION &el)
+rdcstr DoStringise(const D3D11_SRV_DIMENSION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_SRV_DIMENSION);
   {
@@ -562,7 +565,7 @@ std::string DoStringise(const D3D11_SRV_DIMENSION &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_RTV_DIMENSION &el)
+rdcstr DoStringise(const D3D11_RTV_DIMENSION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_RTV_DIMENSION);
   {
@@ -579,7 +582,7 @@ std::string DoStringise(const D3D11_RTV_DIMENSION &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_UAV_DIMENSION &el)
+rdcstr DoStringise(const D3D11_UAV_DIMENSION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_UAV_DIMENSION);
   {
@@ -594,7 +597,7 @@ std::string DoStringise(const D3D11_UAV_DIMENSION &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_DSV_DIMENSION &el)
+rdcstr DoStringise(const D3D11_DSV_DIMENSION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_DSV_DIMENSION);
   {
@@ -609,7 +612,7 @@ std::string DoStringise(const D3D11_DSV_DIMENSION &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_CONTEXT_TYPE &el)
+rdcstr DoStringise(const D3D11_CONTEXT_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_CONTEXT_TYPE);
   {
@@ -623,7 +626,7 @@ std::string DoStringise(const D3D11_CONTEXT_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_QUERY &el)
+rdcstr DoStringise(const D3D11_QUERY &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_QUERY);
   {
@@ -648,7 +651,7 @@ std::string DoStringise(const D3D11_QUERY &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_COUNTER &el)
+rdcstr DoStringise(const D3D11_COUNTER &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_COUNTER);
   {
@@ -658,7 +661,7 @@ std::string DoStringise(const D3D11_COUNTER &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_MAP &el)
+rdcstr DoStringise(const D3D11_MAP &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_MAP);
   {
@@ -672,7 +675,7 @@ std::string DoStringise(const D3D11_MAP &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_PRIMITIVE_TOPOLOGY &el)
+rdcstr DoStringise(const D3D11_PRIMITIVE_TOPOLOGY &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_PRIMITIVE_TOPOLOGY);
   {
@@ -723,7 +726,7 @@ std::string DoStringise(const D3D11_PRIMITIVE_TOPOLOGY &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_USAGE &el)
+rdcstr DoStringise(const D3D11_USAGE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_USAGE);
   {
@@ -736,7 +739,7 @@ std::string DoStringise(const D3D11_USAGE &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_INPUT_CLASSIFICATION &el)
+rdcstr DoStringise(const D3D11_INPUT_CLASSIFICATION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_INPUT_CLASSIFICATION);
   {
@@ -747,7 +750,7 @@ std::string DoStringise(const D3D11_INPUT_CLASSIFICATION &el)
 }
 
 template <>
-std::string DoStringise(const D3D11_LOGIC_OP &el)
+rdcstr DoStringise(const D3D11_LOGIC_OP &el)
 {
   BEGIN_ENUM_STRINGISE(D3D11_LOGIC_OP);
   {
