@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -111,7 +111,7 @@ struct WrappedID3D12DebugCommandList : public ID3D12DebugCommandList2, public ID
   }
 };
 
-class WrappedID3D12GraphicsCommandList : public ID3D12GraphicsCommandList4
+class WrappedID3D12GraphicsCommandList : public ID3D12GraphicsCommandListX
 {
 private:
   ID3D12GraphicsCommandList *m_pList = NULL;
@@ -119,6 +119,7 @@ private:
   ID3D12GraphicsCommandList2 *m_pList2 = NULL;
   ID3D12GraphicsCommandList3 *m_pList3 = NULL;
   ID3D12GraphicsCommandList4 *m_pList4 = NULL;
+  ID3D12GraphicsCommandList5 *m_pList5 = NULL;
 
   RefCounter12<ID3D12GraphicsCommandList> m_RefCounter;
 
@@ -148,7 +149,7 @@ private:
     D3D12_COMMAND_LIST_TYPE type;
   } m_Init;
 
-  static std::string GetChunkName(uint32_t idx);
+  static rdcstr GetChunkName(uint32_t idx);
   D3D12ResourceManager *GetResourceManager() { return m_pDevice->GetResourceManager(); }
 public:
   static const int AllocPoolCount = 8192;
@@ -165,6 +166,7 @@ public:
   ID3D12GraphicsCommandList2 *GetReal2() { return m_pList2; }
   ID3D12GraphicsCommandList3 *GetReal3() { return m_pList3; }
   ID3D12GraphicsCommandList4 *GetReal4() { return m_pList4; }
+  ID3D12GraphicsCommandList5 *GetReal5() { return m_pList5; }
   WrappedID3D12Device *GetWrappedDevice() { return m_pDevice; }
   D3D12ResourceRecord *GetResourceRecord() { return m_ListRecord; }
   D3D12ResourceRecord *GetCreationRecord() { return m_CreationRecord; }
@@ -173,7 +175,8 @@ public:
   ID3D12GraphicsCommandList2 *GetCrackedList2();
   ID3D12GraphicsCommandList3 *GetCrackedList3();
   ID3D12GraphicsCommandList4 *GetCrackedList4();
-  ID3D12GraphicsCommandList4 *GetWrappedCrackedList();
+  ID3D12GraphicsCommandList5 *GetCrackedList5();
+  ID3D12GraphicsCommandListX *GetWrappedCrackedList();
 
   void SetAMDMarkerInterface(IAmdExtD3DCommandListMarker *marker) { m_AMDMarkers = marker; }
   void SetCommandData(D3D12CommandData *cmd) { m_Cmd = cmd; }
@@ -209,7 +212,15 @@ public:
   HRESULT STDMETHODCALLTYPE SetPrivateData(REFGUID guid, UINT DataSize, const void *pData)
   {
     if(guid == WKPDID_D3DDebugObjectName)
+    {
       m_pDevice->SetName(this, (const char *)pData);
+    }
+    else if(guid == WKPDID_D3DDebugObjectNameW)
+    {
+      rdcwstr wName((const wchar_t *)pData, DataSize / 2);
+      rdcstr sName = StringFormat::Wide2UTF8(wName);
+      m_pDevice->SetName(this, sName.c_str());
+    }
 
     return m_pList->SetPrivateData(guid, DataSize, pData);
   }
@@ -221,7 +232,7 @@ public:
 
   HRESULT STDMETHODCALLTYPE SetName(LPCWSTR Name)
   {
-    string utf8 = StringFormat::Wide2UTF8(Name);
+    rdcstr utf8 = StringFormat::Wide2UTF8(Name);
     m_pDevice->SetName(this, utf8.c_str());
 
     return m_pList->SetName(Name);
@@ -518,6 +529,15 @@ public:
 
   IMPLEMENT_FUNCTION_SERIALISED(virtual void STDMETHODCALLTYPE, DispatchRays,
                                 _In_ const D3D12_DISPATCH_RAYS_DESC *pDesc);
+
+  //////////////////////////////
+  // implement ID3D12GraphicsCommandList5
+  IMPLEMENT_FUNCTION_SERIALISED(virtual void STDMETHODCALLTYPE, RSSetShadingRate,
+                                D3D12_SHADING_RATE baseShadingRate,
+                                const D3D12_SHADING_RATE_COMBINER *combiners);
+
+  IMPLEMENT_FUNCTION_SERIALISED(virtual void STDMETHODCALLTYPE, RSSetShadingRateImage,
+                                ID3D12Resource *shadingRateImage);
 };
 
 template <>
@@ -543,18 +563,23 @@ template <>
 ResourceId GetResID(ID3D12GraphicsCommandList3 *obj);
 template <>
 ResourceId GetResID(ID3D12GraphicsCommandList4 *obj);
+template <>
+ResourceId GetResID(ID3D12GraphicsCommandList5 *obj);
 
 ID3D12GraphicsCommandList *Unwrap(ID3D12GraphicsCommandList1 *obj);
 ID3D12GraphicsCommandList *Unwrap(ID3D12GraphicsCommandList2 *obj);
 ID3D12GraphicsCommandList *Unwrap(ID3D12GraphicsCommandList3 *obj);
 ID3D12GraphicsCommandList *Unwrap(ID3D12GraphicsCommandList4 *obj);
+ID3D12GraphicsCommandList *Unwrap(ID3D12GraphicsCommandList5 *obj);
 
 ID3D12GraphicsCommandList1 *Unwrap1(ID3D12GraphicsCommandList1 *obj);
 ID3D12GraphicsCommandList2 *Unwrap2(ID3D12GraphicsCommandList2 *obj);
 ID3D12GraphicsCommandList3 *Unwrap3(ID3D12GraphicsCommandList3 *obj);
 ID3D12GraphicsCommandList4 *Unwrap4(ID3D12GraphicsCommandList4 *obj);
+ID3D12GraphicsCommandList5 *Unwrap5(ID3D12GraphicsCommandList5 *obj);
 
 WrappedID3D12GraphicsCommandList *GetWrapped(ID3D12GraphicsCommandList1 *obj);
 WrappedID3D12GraphicsCommandList *GetWrapped(ID3D12GraphicsCommandList2 *obj);
 WrappedID3D12GraphicsCommandList *GetWrapped(ID3D12GraphicsCommandList3 *obj);
-WrappedID3D12GraphicsCommandList *GetWrapped(ID3D12GraphicsCommandList4 *obj);
\ No newline at end of file
+WrappedID3D12GraphicsCommandList *GetWrapped(ID3D12GraphicsCommandList4 *obj);
+WrappedID3D12GraphicsCommandList *GetWrapped(ID3D12GraphicsCommandList5 *obj);
