@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -57,7 +57,7 @@ bool WrappedID3D12GraphicsCommandList::Serialise_AtomicCopyBufferUINT(
     {
       if(m_Cmd->InRerecordRange(m_Cmd->m_LastCmdListID))
       {
-        std::vector<ID3D12Resource *> deps;
+        rdcarray<ID3D12Resource *> deps;
         deps.resize(Dependencies);
         for(size_t i = 0; i < deps.size(); i++)
           deps[i] = Unwrap(ppDependentResources[i]);
@@ -70,7 +70,7 @@ bool WrappedID3D12GraphicsCommandList::Serialise_AtomicCopyBufferUINT(
     }
     else
     {
-      std::vector<ID3D12Resource *> deps;
+      rdcarray<ID3D12Resource *> deps;
       deps.resize(Dependencies);
       for(size_t i = 0; i < deps.size(); i++)
         deps[i] = Unwrap(ppDependentResources[i]);
@@ -100,14 +100,14 @@ bool WrappedID3D12GraphicsCommandList::Serialise_AtomicCopyBufferUINT(
 
         if(pSrcBuffer == pDstBuffer)
         {
-          drawNode.resourceUsage.push_back(std::make_pair(
+          drawNode.resourceUsage.push_back(make_rdcpair(
               GetResID(pSrcBuffer), EventUsage(drawNode.draw.eventId, ResourceUsage::Copy)));
         }
         else
         {
-          drawNode.resourceUsage.push_back(std::make_pair(
+          drawNode.resourceUsage.push_back(make_rdcpair(
               GetResID(pSrcBuffer), EventUsage(drawNode.draw.eventId, ResourceUsage::CopySrc)));
-          drawNode.resourceUsage.push_back(std::make_pair(
+          drawNode.resourceUsage.push_back(make_rdcpair(
               GetResID(pDstBuffer), EventUsage(drawNode.draw.eventId, ResourceUsage::CopyDst)));
         }
       }
@@ -175,7 +175,7 @@ bool WrappedID3D12GraphicsCommandList::Serialise_AtomicCopyBufferUINT64(
     {
       if(m_Cmd->InRerecordRange(m_Cmd->m_LastCmdListID))
       {
-        std::vector<ID3D12Resource *> deps;
+        rdcarray<ID3D12Resource *> deps;
         deps.resize(Dependencies);
         for(size_t i = 0; i < deps.size(); i++)
           deps[i] = Unwrap(ppDependentResources[i]);
@@ -188,7 +188,7 @@ bool WrappedID3D12GraphicsCommandList::Serialise_AtomicCopyBufferUINT64(
     }
     else
     {
-      std::vector<ID3D12Resource *> deps;
+      rdcarray<ID3D12Resource *> deps;
       deps.resize(Dependencies);
       for(size_t i = 0; i < deps.size(); i++)
         deps[i] = Unwrap(ppDependentResources[i]);
@@ -218,14 +218,14 @@ bool WrappedID3D12GraphicsCommandList::Serialise_AtomicCopyBufferUINT64(
 
         if(pSrcBuffer == pDstBuffer)
         {
-          drawNode.resourceUsage.push_back(std::make_pair(
+          drawNode.resourceUsage.push_back(make_rdcpair(
               GetResID(pSrcBuffer), EventUsage(drawNode.draw.eventId, ResourceUsage::Copy)));
         }
         else
         {
-          drawNode.resourceUsage.push_back(std::make_pair(
+          drawNode.resourceUsage.push_back(make_rdcpair(
               GetResID(pSrcBuffer), EventUsage(drawNode.draw.eventId, ResourceUsage::CopySrc)));
-          drawNode.resourceUsage.push_back(std::make_pair(
+          drawNode.resourceUsage.push_back(make_rdcpair(
               GetResID(pDstBuffer), EventUsage(drawNode.draw.eventId, ResourceUsage::CopyDst)));
         }
       }
@@ -362,12 +362,9 @@ bool WrappedID3D12GraphicsCommandList::Serialise_SetSamplePositions(
         {
           D3D12RenderState &state = m_Cmd->m_RenderState;
 
-          std::vector<D3D12_SAMPLE_POSITION> pos(
-              pSamplePositions, pSamplePositions + (NumSamplesPerPixel * NumPixels));
-
           state.samplePos.NumSamplesPerPixel = NumSamplesPerPixel;
           state.samplePos.NumPixels = NumPixels;
-          state.samplePos.Positions.swap(pos);
+          state.samplePos.Positions.assign(pSamplePositions, NumSamplesPerPixel * NumPixels);
         }
       }
     }
@@ -379,12 +376,9 @@ bool WrappedID3D12GraphicsCommandList::Serialise_SetSamplePositions(
       {
         D3D12RenderState &state = m_Cmd->m_BakedCmdListInfo[m_Cmd->m_LastCmdListID].state;
 
-        std::vector<D3D12_SAMPLE_POSITION> pos(pSamplePositions,
-                                               pSamplePositions + (NumSamplesPerPixel * NumPixels));
-
         state.samplePos.NumSamplesPerPixel = NumSamplesPerPixel;
         state.samplePos.NumPixels = NumPixels;
-        state.samplePos.Positions.swap(pos);
+        state.samplePos.Positions.assign(pSamplePositions, NumSamplesPerPixel * NumPixels);
       }
     }
   }
@@ -475,14 +469,14 @@ bool WrappedID3D12GraphicsCommandList::Serialise_ResolveSubresourceRegion(
 
         if(pSrcResource == pDstResource)
         {
-          drawNode.resourceUsage.push_back(std::make_pair(
+          drawNode.resourceUsage.push_back(make_rdcpair(
               GetResID(pSrcResource), EventUsage(drawNode.draw.eventId, ResourceUsage::Resolve)));
         }
         else
         {
-          drawNode.resourceUsage.push_back(std::make_pair(
+          drawNode.resourceUsage.push_back(make_rdcpair(
               GetResID(pSrcResource), EventUsage(drawNode.draw.eventId, ResourceUsage::ResolveSrc)));
-          drawNode.resourceUsage.push_back(std::make_pair(
+          drawNode.resourceUsage.push_back(make_rdcpair(
               GetResID(pDstResource), EventUsage(drawNode.draw.eventId, ResourceUsage::ResolveDst)));
         }
       }
