@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,6 @@
 
 #pragma once
 
-#include <list>
 #include "common/wrapped_pool.h"
 #include "d3d12_commands.h"
 #include "d3d12_common.h"
@@ -68,18 +67,43 @@ struct WrappedID3D12DebugCommandQueue : public ID3D12DebugCommandQueue
   }
 };
 
+struct WrappedDownlevelQueue : public ID3D12CommandQueueDownlevel
+{
+  WrappedID3D12CommandQueue &m_pQueue;
+
+  WrappedDownlevelQueue(WrappedID3D12CommandQueue &q) : m_pQueue(q) {}
+  //////////////////////////////
+  // implement IUnknown
+  HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);
+  ULONG STDMETHODCALLTYPE AddRef();
+  ULONG STDMETHODCALLTYPE Release();
+  //////////////////////////////
+  // implement ID3D12CommandQueueDownlevel
+  virtual HRESULT STDMETHODCALLTYPE Present(ID3D12GraphicsCommandList *pOpenCommandList,
+                                            ID3D12Resource *pSourceTex2D, HWND hWindow,
+                                            D3D12_DOWNLEVEL_PRESENT_FLAGS Flags);
+};
+
 class WrappedID3D12GraphicsCommandList;
 
 class WrappedID3D12CommandQueue : public ID3D12CommandQueue,
                                   public RefCounter12<ID3D12CommandQueue>,
-                                  public ID3DDevice
+                                  public ID3DDevice,
+                                  public IDXGISwapper
 {
   friend class WrappedID3D12GraphicsCommandList;
 
+  ID3D12CommandQueueDownlevel *m_pDownlevel;
+
+  WrappedDownlevelQueue m_WrappedDownlevel;
+
   WrappedID3D12Device *m_pDevice;
 
   WrappedID3D12GraphicsCommandList *m_ReplayList;
 
+  ID3D12Resource *m_pPresentSource = NULL;
+  HWND m_pPresentHWND = NULL;
+
   ResourceId m_ResourceID;
   D3D12ResourceRecord *m_QueueRecord;
 
@@ -91,12 +115,12 @@ class WrappedID3D12CommandQueue : public ID3D12CommandQueue,
 
   WrappedID3D12DebugCommandQueue m_WrappedDebug;
 
-  vector<D3D12ResourceRecord *> m_CmdListRecords;
+  rdcarray<D3D12ResourceRecord *> m_CmdListRecords;
 
   // D3D12 guarantees that queues are thread-safe
   Threading::CriticalSection m_Lock;
 
-  std::set<std::string> m_StringDB;
+  std::set<rdcstr> m_StringDB;
 
   WriteSerialiser &GetThreadSerialiser();
 
@@ -108,11 +132,10 @@ class WrappedID3D12CommandQueue : public ID3D12CommandQueue,
   D3D12CommandData m_Cmd;
 
   ResourceId m_PrevQueueId;
-  ResourceId m_BackbufferID;
 
   bool ProcessChunk(ReadSerialiser &ser, D3D12Chunk context);
 
-  static std::string GetChunkName(uint32_t idx);
+  static rdcstr GetChunkName(uint32_t idx);
   D3D12ResourceManager *GetResourceManager() { return m_pDevice->GetResourceManager(); }
 public:
   static const int AllocPoolCount = 16;
@@ -126,18 +149,17 @@ public:
   ID3D12CommandQueue *GetReal() { return m_pReal; }
   D3D12ResourceRecord *GetResourceRecord() { return m_QueueRecord; }
   WrappedID3D12Device *GetWrappedDevice() { return m_pDevice; }
-  const vector<D3D12ResourceRecord *> &GetCmdLists() { return m_CmdListRecords; }
+  const rdcarray<D3D12ResourceRecord *> &GetCmdLists() { return m_CmdListRecords; }
   D3D12DrawcallTreeNode &GetParentDrawcall() { return m_Cmd.m_ParentDrawcall; }
   const APIEvent &GetEvent(uint32_t eventId);
   uint32_t GetMaxEID() { return m_Cmd.m_Events.back().eventId; }
-  ResourceId GetBackbufferResourceID() { return m_BackbufferID; }
   void ClearAfterCapture();
 
   ReplayStatus ReplayLog(CaptureState readType, uint32_t startEventID, uint32_t endEventID,
                          bool partial);
   void SetFrameReader(StreamReader *reader) { m_FrameReader = reader; }
   D3D12CommandData *GetCommandData() { return &m_Cmd; }
-  const vector<EventUsage> &GetUsage(ResourceId id) { return m_Cmd.m_ResourceUses[id]; }
+  const rdcarray<EventUsage> &GetUsage(ResourceId id) { return m_Cmd.m_ResourceUses[id]; }
   // interface for DXGI
   virtual IUnknown *GetRealIUnknown() { return GetReal(); }
   virtual IID GetBackbufferUUID() { return __uuidof(ID3D12Resource); }
@@ -155,27 +177,52 @@ public:
     return NULL;
   }
   // the rest forward to the device
-  virtual void FirstFrame(WrappedIDXGISwapChain4 *swapChain) { m_pDevice->FirstFrame(swapChain); }
+  virtual void FirstFrame(IDXGISwapper *swapper) { m_pDevice->FirstFrame(swapper); }
   virtual void NewSwapchainBuffer(IUnknown *backbuffer)
   {
     m_pDevice->NewSwapchainBuffer(backbuffer);
   }
-  virtual void ReleaseSwapchainResources(WrappedIDXGISwapChain4 *swapChain, UINT QueueCount,
+  virtual void ReleaseSwapchainResources(IDXGISwapper *swapper, UINT QueueCount,
                                          IUnknown *const *ppPresentQueue, IUnknown **unwrappedQueues)
   {
-    m_pDevice->ReleaseSwapchainResources(swapChain, QueueCount, ppPresentQueue, unwrappedQueues);
+    m_pDevice->ReleaseSwapchainResources(swapper, QueueCount, ppPresentQueue, unwrappedQueues);
   }
-  virtual IUnknown *WrapSwapchainBuffer(WrappedIDXGISwapChain4 *swap, DXGI_SWAP_CHAIN_DESC *swapDesc,
+  virtual IUnknown *WrapSwapchainBuffer(IDXGISwapper *swapper, DXGI_FORMAT bufferFormat,
                                         UINT buffer, IUnknown *realSurface)
   {
-    return m_pDevice->WrapSwapchainBuffer(swap, swapDesc, buffer, realSurface);
+    return m_pDevice->WrapSwapchainBuffer(swapper, bufferFormat, buffer, realSurface);
   }
-
-  virtual HRESULT Present(WrappedIDXGISwapChain4 *swapChain, UINT SyncInterval, UINT Flags)
+  virtual IDXGIResource *WrapExternalDXGIResource(IDXGIResource *res)
   {
-    return m_pDevice->Present(swapChain, SyncInterval, Flags);
+    return m_pDevice->WrapExternalDXGIResource(res);
   }
 
+  virtual HRESULT Present(IDXGISwapper *swapper, UINT SyncInterval, UINT Flags)
+  {
+    return m_pDevice->Present(swapper, SyncInterval, Flags);
+  }
+
+  // fake pretending to be a swapchain for when we're doing downlevel mega-hacky presents
+  virtual ID3DDevice *GetD3DDevice() { return this; }
+  virtual int GetNumBackbuffers() { return 1; }
+  virtual IUnknown **GetBackbuffers() { return (IUnknown **)&m_pPresentSource; }
+  virtual int GetLastPresentedBuffer() { return 0; }
+  virtual UINT GetWidth()
+  {
+    D3D12_RESOURCE_DESC desc = m_pPresentSource->GetDesc();
+    return (UINT)desc.Width;
+  }
+  virtual UINT GetHeight()
+  {
+    D3D12_RESOURCE_DESC desc = m_pPresentSource->GetDesc();
+    return desc.Height;
+  }
+  virtual DXGI_FORMAT GetFormat()
+  {
+    D3D12_RESOURCE_DESC desc = m_pPresentSource->GetDesc();
+    return desc.Format;
+  }
+  virtual HWND GetHWND() { return m_pPresentHWND; }
   //////////////////////////////
   // implement IUnknown
 
@@ -194,7 +241,15 @@ public:
   HRESULT STDMETHODCALLTYPE SetPrivateData(REFGUID guid, UINT DataSize, const void *pData)
   {
     if(guid == WKPDID_D3DDebugObjectName)
+    {
       m_pDevice->SetName(this, (const char *)pData);
+    }
+    else if(guid == WKPDID_D3DDebugObjectNameW)
+    {
+      rdcwstr wName((const wchar_t *)pData, DataSize / 2);
+      rdcstr sName = StringFormat::Wide2UTF8(wName);
+      m_pDevice->SetName(this, sName.c_str());
+    }
 
     return m_pReal->SetPrivateData(guid, DataSize, pData);
   }
@@ -206,7 +261,7 @@ public:
 
   HRESULT STDMETHODCALLTYPE SetName(LPCWSTR Name)
   {
-    string utf8 = StringFormat::Wide2UTF8(Name);
+    rdcstr utf8 = StringFormat::Wide2UTF8(Name);
     m_pDevice->SetName(this, utf8.c_str());
 
     return m_pReal->SetName(Name);
@@ -244,7 +299,7 @@ public:
 
   virtual void ExecuteCommandListsInternal(UINT NumCommandLists,
                                            ID3D12CommandList *const *ppCommandLists,
-                                           bool InFrameCaptureBoundary);
+                                           bool InFrameCaptureBoundary, bool SkipRealExecute);
 
   IMPLEMENT_FUNCTION_SERIALISED(virtual void STDMETHODCALLTYPE, SetMarker, UINT Metadata,
                                 const void *pData, UINT Size);
@@ -267,6 +322,11 @@ public:
                                 UINT64 *pGpuTimestamp, UINT64 *pCpuTimestamp);
 
   virtual D3D12_COMMAND_QUEUE_DESC STDMETHODCALLTYPE GetDesc() { return m_pReal->GetDesc(); }
+  //////////////////////////////
+  // implement ID3D12CommandQueueDownlevel
+  virtual HRESULT STDMETHODCALLTYPE Present(ID3D12GraphicsCommandList *pOpenCommandList,
+                                            ID3D12Resource *pSourceTex2D, HWND hWindow,
+                                            D3D12_DOWNLEVEL_PRESENT_FLAGS Flags);
 };
 
 template <>
@@ -276,4 +336,4 @@ template <>
 ResourceId GetResID(ID3D12CommandQueue *obj);
 
 template <>
-D3D12ResourceRecord *GetRecord(ID3D12CommandQueue *obj);
\ No newline at end of file
+D3D12ResourceRecord *GetRecord(ID3D12CommandQueue *obj);
