@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -83,7 +83,7 @@ bool WrappedID3D12CommandQueue::Serialise_ExecuteCommandLists(SerialiserType &se
   SERIALISE_ELEMENT_ARRAY(ppCommandLists, NumCommandLists);
 
   {
-    std::vector<DebugMessage> DebugMessages;
+    rdcarray<DebugMessage> DebugMessages;
 
     if(ser.IsWriting())
       DebugMessages = m_pDevice->GetDebugMessages();
@@ -92,6 +92,10 @@ bool WrappedID3D12CommandQueue::Serialise_ExecuteCommandLists(SerialiserType &se
 
     if(ser.IsReading() && IsLoading(m_State))
     {
+      // if we're using replay-time API validation, ignore messages from capture time
+      if(m_pDevice->GetReplayOptions().apiValidation)
+        DebugMessages.clear();
+
       for(const DebugMessage &msg : DebugMessages)
         m_Cmd.m_EventMessages.push_back(msg);
     }
@@ -166,7 +170,7 @@ bool WrappedID3D12CommandQueue::Serialise_ExecuteCommandLists(SerialiserType &se
         m_pDevice->ApplyBarriers(m_Cmd.m_BakedCmdListInfo[cmd].barriers);
       }
 
-      std::string basename = StringFormat::Fmt("ExecuteCommandLists(%u)", NumCommandLists);
+      rdcstr basename = StringFormat::Fmt("ExecuteCommandLists(%u)", NumCommandLists);
 
       for(uint32_t c = 0; c < NumCommandLists; c++)
       {
@@ -193,8 +197,8 @@ bool WrappedID3D12CommandQueue::Serialise_ExecuteCommandLists(SerialiserType &se
 
         for(size_t e = 0; e < cmdListInfo.draw->executedCmds.size(); e++)
         {
-          vector<uint32_t> &submits = m_Cmd.m_Partial[D3D12CommandData::Secondary]
-                                          .cmdListExecs[cmdListInfo.draw->executedCmds[e]];
+          rdcarray<uint32_t> &submits = m_Cmd.m_Partial[D3D12CommandData::Secondary]
+                                            .cmdListExecs[cmdListInfo.draw->executedCmds[e]];
 
           for(size_t s = 0; s < submits.size(); s++)
             submits[s] += m_Cmd.m_RootEventID;
@@ -267,7 +271,7 @@ bool WrappedID3D12CommandQueue::Serialise_ExecuteCommandLists(SerialiserType &se
 
         uint32_t eid = startEID;
 
-        std::vector<ID3D12CommandList *> rerecordedCmds;
+        rdcarray<ID3D12CommandList *> rerecordedCmds;
 
         for(uint32_t c = 0; c < NumCommandLists; c++)
         {
@@ -324,12 +328,13 @@ bool WrappedID3D12CommandQueue::Serialise_ExecuteCommandLists(SerialiserType &se
 void WrappedID3D12CommandQueue::ExecuteCommandLists(UINT NumCommandLists,
                                                     ID3D12CommandList *const *ppCommandLists)
 {
-  ExecuteCommandListsInternal(NumCommandLists, ppCommandLists, false);
+  ExecuteCommandListsInternal(NumCommandLists, ppCommandLists, false, false);
 }
 
 void WrappedID3D12CommandQueue::ExecuteCommandListsInternal(UINT NumCommandLists,
                                                             ID3D12CommandList *const *ppCommandLists,
-                                                            bool InFrameCaptureBoundary)
+                                                            bool InFrameCaptureBoundary,
+                                                            bool SkipRealExecute)
 {
   ID3D12CommandList **unwrapped = m_pDevice->GetTempArray<ID3D12CommandList *>(NumCommandLists);
   for(UINT i = 0; i < NumCommandLists; i++)
@@ -344,7 +349,10 @@ void WrappedID3D12CommandQueue::ExecuteCommandListsInternal(UINT NumCommandLists
   if(IsActiveCapturing(m_State))
     m_pDevice->AddCaptureSubmission();
 
-  SERIALISE_TIME_CALL(m_pReal->ExecuteCommandLists(NumCommandLists, unwrapped));
+  if(!SkipRealExecute)
+  {
+    SERIALISE_TIME_CALL(m_pReal->ExecuteCommandLists(NumCommandLists, unwrapped));
+  }
 
   if(IsCaptureMode(m_State))
   {
@@ -375,25 +383,17 @@ void WrappedID3D12CommandQueue::ExecuteCommandListsInternal(UINT NumCommandLists
       // the submit chunk to the frame record don't have to be protected.
       // Only the decision of whether we're inframe or not, and marking
       // dirty.
-      if(capframe)
-      {
-        for(auto it = record->bakedCommands->cmdInfo->dirtied.begin();
-            it != record->bakedCommands->cmdInfo->dirtied.end(); ++it)
-          GetResourceManager()->MarkPendingDirty(*it);
-      }
-      else
-      {
-        for(auto it = record->bakedCommands->cmdInfo->dirtied.begin();
-            it != record->bakedCommands->cmdInfo->dirtied.end(); ++it)
-          GetResourceManager()->MarkDirtyResource(*it);
-      }
+
+      for(auto it = record->bakedCommands->cmdInfo->dirtied.begin();
+          it != record->bakedCommands->cmdInfo->dirtied.end(); ++it)
+        GetResourceManager()->MarkDirtyResource(*it);
 
       if(capframe)
       {
         // any descriptor copies or writes could reference new resources not in the
         // bound descs list yet. So we take all of those referenced descriptors and
         // include them to see if we need to flush
-        std::vector<D3D12Descriptor> dynDescRefs;
+        rdcarray<D3D12Descriptor> dynDescRefs;
         m_pDevice->GetDynamicDescriptorReferences(dynDescRefs);
 
         for(size_t d = 0; d < dynDescRefs.size(); d++)
@@ -475,7 +475,7 @@ void WrappedID3D12CommandQueue::ExecuteCommandListsInternal(UINT NumCommandLists
 
     if(capframe)
     {
-      std::vector<MapState> maps = m_pDevice->GetMaps();
+      rdcarray<MapState> maps = m_pDevice->GetMaps();
 
       for(auto it = maps.begin(); it != maps.end(); ++it)
       {
@@ -521,7 +521,7 @@ void WrappedID3D12CommandQueue::ExecuteCommandListsInternal(UINT NumCommandLists
           // update comparison shadow for next time
           memcpy(ref, res->GetMap(subres), size);
 
-          GetResourceManager()->MarkPendingDirty(res->GetResourceID());
+          GetResourceManager()->MarkDirtyResource(res->GetResourceID());
         }
         else
         {
@@ -548,7 +548,7 @@ template <typename SerialiserType>
 bool WrappedID3D12CommandQueue::Serialise_SetMarker(SerialiserType &ser, UINT Metadata,
                                                     const void *pData, UINT Size)
 {
-  std::string MarkerText = "";
+  rdcstr MarkerText = "";
 
   if(ser.IsWriting() && pData && Size)
     MarkerText = DecodeMarkerString(Metadata, pData, Size);
@@ -597,7 +597,7 @@ template <typename SerialiserType>
 bool WrappedID3D12CommandQueue::Serialise_BeginEvent(SerialiserType &ser, UINT Metadata,
                                                      const void *pData, UINT Size)
 {
-  std::string MarkerText = "";
+  rdcstr MarkerText = "";
 
   if(ser.IsWriting() && pData && Size)
     MarkerText = DecodeMarkerString(Metadata, pData, Size);
@@ -699,6 +699,7 @@ bool WrappedID3D12CommandQueue::Serialise_Signal(SerialiserType &ser, ID3D12Fenc
   if(IsReplayingAndReading() && pFence)
   {
     m_pReal->Signal(Unwrap(pFence), Value);
+    m_pDevice->GPUSync();
   }
 
   return true;
@@ -773,6 +774,64 @@ HRESULT STDMETHODCALLTYPE WrappedID3D12CommandQueue::GetClockCalibration(UINT64
   return m_pReal->GetClockCalibration(pGpuTimestamp, pCpuTimestamp);
 }
 
+HRESULT STDMETHODCALLTYPE WrappedID3D12CommandQueue::Present(
+    _In_ ID3D12GraphicsCommandList *pOpenCommandList, _In_ ID3D12Resource *pSourceTex2D,
+    _In_ HWND hWindow, D3D12_DOWNLEVEL_PRESENT_FLAGS Flags)
+{
+  // D3D12 on windows 7
+  if(!RenderDoc::Inst().GetCaptureOptions().allowVSync)
+  {
+    Flags = D3D12_DOWNLEVEL_PRESENT_FLAG_NONE;
+  }
+
+  // store the timestamp, thread ID etc. Don't store the duration
+  SERIALISE_TIME_CALL();
+
+  if(IsCaptureMode(m_State))
+  {
+    WrappedID3D12GraphicsCommandList *list = (WrappedID3D12GraphicsCommandList *)pOpenCommandList;
+
+    // add a marker
+    D3D12MarkerRegion::Set(list, "ID3D12CommandQueueDownlevel::Present()");
+
+    // the list is implicitly closed, serialise that
+    D3D12ResourceRecord *listRecord = GetRecord(list);
+
+    {
+      CACHE_THREAD_SERIALISER();
+      ser.SetDrawChunk();
+      SCOPED_SERIALISE_CHUNK(D3D12Chunk::List_Close);
+      list->Serialise_Close(ser);
+
+      listRecord->AddChunk(scope.Get());
+    }
+
+    listRecord->Bake();
+
+    // this queue implicitly submits the list, serialise that
+    ID3D12CommandList *submitlist = list;
+    ExecuteCommandListsInternal(1, &submitlist, false, true);
+  }
+
+  if(m_pPresentHWND != NULL)
+  {
+    // don't let the device actually release any refs on the resource, just make it release internal
+    // resources
+    m_pPresentSource->AddRef();
+    m_pDevice->ReleaseSwapchainResources(this, 0, NULL, NULL);
+  }
+
+  m_pPresentSource = pSourceTex2D;
+  m_pPresentHWND = hWindow;
+
+  m_pDevice->WrapSwapchainBuffer(this, GetFormat(), 0, m_pPresentSource);
+
+  m_pDevice->Present(pOpenCommandList, this,
+                     Flags == D3D12_DOWNLEVEL_PRESENT_FLAG_WAIT_FOR_VBLANK ? 1 : 0, 0);
+
+  return m_pDownlevel->Present(Unwrap(pOpenCommandList), Unwrap(pSourceTex2D), hWindow, Flags);
+}
+
 INSTANTIATE_FUNCTION_SERIALISED(
     void, WrappedID3D12CommandQueue, UpdateTileMappings, ID3D12Resource *pResource,
     UINT NumResourceRegions, const D3D12_TILED_RESOURCE_COORDINATE *pResourceRegionStartCoordinates,
@@ -798,4 +857,4 @@ INSTANTIATE_FUNCTION_SERIALISED(void, WrappedID3D12CommandQueue, Wait, ID3D12Fen
 INSTANTIATE_FUNCTION_SERIALISED(void, WrappedID3D12CommandQueue, Signal, ID3D12Fence *pFence,
                                 UINT64 Value);
 INSTANTIATE_FUNCTION_SERIALISED(void, WrappedID3D12CommandQueue, Wait, ID3D12Fence *pFence,
-                                UINT64 Value);
\ No newline at end of file
+                                UINT64 Value);
