@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -72,6 +72,14 @@ ID3D12GraphicsCommandList *Unwrap(ID3D12GraphicsCommandList4 *obj)
   return ((WrappedID3D12GraphicsCommandList *)obj)->GetReal();
 }
 
+ID3D12GraphicsCommandList *Unwrap(ID3D12GraphicsCommandList5 *obj)
+{
+  if(obj == NULL)
+    return NULL;
+
+  return ((WrappedID3D12GraphicsCommandList *)obj)->GetReal();
+}
+
 ID3D12GraphicsCommandList1 *Unwrap1(ID3D12GraphicsCommandList1 *obj)
 {
   if(obj == NULL)
@@ -104,6 +112,14 @@ ID3D12GraphicsCommandList4 *Unwrap4(ID3D12GraphicsCommandList4 *obj)
   return ((WrappedID3D12GraphicsCommandList *)obj)->GetReal4();
 }
 
+ID3D12GraphicsCommandList5 *Unwrap5(ID3D12GraphicsCommandList5 *obj)
+{
+  if(obj == NULL)
+    return NULL;
+
+  return ((WrappedID3D12GraphicsCommandList *)obj)->GetReal5();
+}
+
 template <>
 ID3D12CommandList *Unwrap(ID3D12CommandList *obj)
 {
@@ -158,6 +174,15 @@ ResourceId GetResID(ID3D12GraphicsCommandList4 *obj)
   return ((WrappedID3D12GraphicsCommandList *)obj)->GetResourceID();
 }
 
+template <>
+ResourceId GetResID(ID3D12GraphicsCommandList5 *obj)
+{
+  if(obj == NULL)
+    return ResourceId();
+
+  return ((WrappedID3D12GraphicsCommandList *)obj)->GetResourceID();
+}
+
 template <>
 ResourceId GetResID(ID3D12CommandList *obj)
 {
@@ -232,6 +257,11 @@ WrappedID3D12GraphicsCommandList *GetWrapped(ID3D12GraphicsCommandList4 *obj)
   return ((WrappedID3D12GraphicsCommandList *)obj);
 }
 
+WrappedID3D12GraphicsCommandList *GetWrapped(ID3D12GraphicsCommandList5 *obj)
+{
+  return ((WrappedID3D12GraphicsCommandList *)obj);
+}
+
 ULONG STDMETHODCALLTYPE WrappedID3D12DebugCommandQueue::AddRef()
 {
   m_pQueue->AddRef();
@@ -256,17 +286,43 @@ ULONG STDMETHODCALLTYPE WrappedID3D12DebugCommandList::Release()
   return 1;
 }
 
+HRESULT STDMETHODCALLTYPE WrappedDownlevelQueue::QueryInterface(REFIID riid, void **ppvObject)
+{
+  return m_pQueue.QueryInterface(riid, ppvObject);
+}
+
+ULONG STDMETHODCALLTYPE WrappedDownlevelQueue::AddRef()
+{
+  return m_pQueue.AddRef();
+}
+
+ULONG STDMETHODCALLTYPE WrappedDownlevelQueue::Release()
+{
+  return m_pQueue.Release();
+}
+
+HRESULT STDMETHODCALLTYPE WrappedDownlevelQueue::Present(ID3D12GraphicsCommandList *pOpenCommandList,
+                                                         ID3D12Resource *pSourceTex2D, HWND hWindow,
+                                                         D3D12_DOWNLEVEL_PRESENT_FLAGS Flags)
+{
+  return m_pQueue.Present(pOpenCommandList, pSourceTex2D, hWindow, Flags);
+}
+
 WrappedID3D12CommandQueue::WrappedID3D12CommandQueue(ID3D12CommandQueue *real,
                                                      WrappedID3D12Device *device, CaptureState &state)
-    : RefCounter12(real), m_pDevice(device), m_State(state)
+    : RefCounter12(real), m_pDevice(device), m_State(state), m_WrappedDownlevel(*this)
 {
   if(RenderDoc::Inst().GetCrashHandler())
     RenderDoc::Inst().GetCrashHandler()->RegisterMemoryRegion(this,
                                                               sizeof(WrappedID3D12CommandQueue));
 
   m_WrappedDebug.m_pReal = NULL;
+  m_pDownlevel = NULL;
   if(m_pReal)
+  {
     m_pReal->QueryInterface(__uuidof(ID3D12DebugCommandQueue), (void **)&m_WrappedDebug.m_pReal);
+    m_pReal->QueryInterface(__uuidof(ID3D12CommandQueueDownlevel), (void **)&m_pDownlevel);
+  }
 
   if(RenderDoc::Inst().IsReplayApp())
   {
@@ -305,6 +361,8 @@ WrappedID3D12CommandQueue::~WrappedID3D12CommandQueue()
   m_pDevice->GetResourceManager()->ReleaseCurrentResource(GetResourceID());
   m_pDevice->RemoveQueue(this);
 
+  SAFE_RELEASE(m_pDownlevel);
+
   for(size_t i = 0; i < m_Cmd.m_IndirectBuffers.size(); i++)
     SAFE_RELEASE(m_Cmd.m_IndirectBuffers[i]);
 
@@ -357,6 +415,19 @@ HRESULT STDMETHODCALLTYPE WrappedID3D12CommandQueue::QueryInterface(REFIID riid,
     AddRef();
     return S_OK;
   }
+  else if(riid == __uuidof(ID3D12CommandQueueDownlevel))
+  {
+    if(m_pDownlevel)
+    {
+      *ppvObject = &m_WrappedDownlevel;
+      AddRef();
+      return S_OK;
+    }
+    else
+    {
+      return E_NOINTERFACE;
+    }
+  }
   else
   {
     WarnUnknownGUID("ID3D12CommandQueue", riid);
@@ -381,7 +452,7 @@ WriteSerialiser &WrappedID3D12CommandQueue::GetThreadSerialiser()
   return m_pDevice->GetThreadSerialiser();
 }
 
-std::string WrappedID3D12CommandQueue::GetChunkName(uint32_t idx)
+rdcstr WrappedID3D12CommandQueue::GetChunkName(uint32_t idx)
 {
   if((SystemChunk)idx < SystemChunk::FirstDriverChunk)
     return ToStr((SystemChunk)idx);
@@ -391,13 +462,14 @@ std::string WrappedID3D12CommandQueue::GetChunkName(uint32_t idx)
 
 const APIEvent &WrappedID3D12CommandQueue::GetEvent(uint32_t eventId)
 {
-  for(const APIEvent &e : m_Cmd.m_Events)
-  {
-    if(e.eventId >= eventId)
-      return e;
-  }
+  // start at where the requested eventId would be
+  size_t idx = eventId;
 
-  return m_Cmd.m_Events.back();
+  // find the next valid event (some may be skipped)
+  while(idx < m_Cmd.m_Events.size() - 1 && m_Cmd.m_Events[idx].eventId == 0)
+    idx++;
+
+  return m_Cmd.m_Events[RDCMIN(idx, m_Cmd.m_Events.size() - 1)];
 }
 
 bool WrappedID3D12CommandQueue::ProcessChunk(ReadSerialiser &ser, D3D12Chunk chunk)
@@ -418,7 +490,7 @@ bool WrappedID3D12CommandQueue::ProcessChunk(ReadSerialiser &ser, D3D12Chunk chu
       break;
     case D3D12Chunk::Device_CopyDescriptors:
     case D3D12Chunk::Device_CopyDescriptorsSimple:
-      ret = m_pDevice->Serialise_DynamicDescriptorCopies(ser, std::vector<DynamicDescriptorCopy>());
+      ret = m_pDevice->Serialise_DynamicDescriptorCopies(ser, rdcarray<DynamicDescriptorCopy>());
       break;
 
     case D3D12Chunk::Queue_ExecuteCommandLists:
@@ -612,6 +684,12 @@ bool WrappedID3D12CommandQueue::ProcessChunk(ReadSerialiser &ser, D3D12Chunk chu
       ret = m_ReplayList->Serialise_BeginRenderPass(ser, 0, NULL, NULL, D3D12_RENDER_PASS_FLAG_NONE);
       break;
     case D3D12Chunk::List_EndRenderPass: ret = m_ReplayList->Serialise_EndRenderPass(ser); break;
+    case D3D12Chunk::List_RSSetShadingRate:
+      ret = m_ReplayList->Serialise_RSSetShadingRate(ser, D3D12_SHADING_RATE_1X1, NULL);
+      break;
+    case D3D12Chunk::List_RSSetShadingRateImage:
+      ret = m_ReplayList->Serialise_RSSetShadingRateImage(ser, NULL);
+      break;
 
     case D3D12Chunk::PushMarker: ret = m_ReplayList->Serialise_BeginEvent(ser, 0, NULL, 0); break;
     case D3D12Chunk::PopMarker: ret = m_ReplayList->Serialise_EndEvent(ser); break;
@@ -628,38 +706,78 @@ bool WrappedID3D12CommandQueue::ProcessChunk(ReadSerialiser &ser, D3D12Chunk chu
       // Just in case it gets exported and imported, completely ignore it.
       return true;
 
-    default:
-    {
-      SystemChunk system = (SystemChunk)chunk;
+    case D3D12Chunk::Swapchain_Present: ret = m_pDevice->Serialise_Present(ser, NULL, 0, 0); break;
 
-      if(system == SystemChunk::CaptureEnd)
-      {
-        SERIALISE_ELEMENT_LOCAL(PresentedImage, ResourceId()).TypedAs("ID3D12Resource *");
-
-        SERIALISE_CHECK_READ_ERRORS();
-
-        m_BackbufferID = PresentedImage;
-
-        if(IsLoading(m_State))
-        {
-          m_Cmd.AddEvent();
-
-          DrawcallDescription draw;
-          draw.name = "Present()";
-          draw.flags |= DrawFlags::Present;
-
-          draw.copyDestination = m_BackbufferID;
-
-          m_Cmd.AddDrawcall(draw, true);
-        }
-
-        ret = true;
-      }
-      else
-      {
-        RDCERR("Unrecognised Chunk type %s", ToStr(chunk).c_str());
-      }
+    case D3D12Chunk::List_ClearState:
+      ret = m_ReplayList->Serialise_ClearState(ser, NULL);
       break;
+
+    // in order to get a warning if we miss a case, we explicitly handle the device creation chunks
+    // here. If we actually encounter one it's an error (we shouldn't see these inside the captured
+    // frame itself)
+    case D3D12Chunk::Device_CreateCommandQueue:
+    case D3D12Chunk::Device_CreateCommandAllocator:
+    case D3D12Chunk::Device_CreateCommandList:
+    case D3D12Chunk::Device_CreateGraphicsPipeline:
+    case D3D12Chunk::Device_CreateComputePipeline:
+    case D3D12Chunk::Device_CreateDescriptorHeap:
+    case D3D12Chunk::Device_CreateRootSignature:
+    case D3D12Chunk::Device_CreateCommandSignature:
+    case D3D12Chunk::Device_CreateHeap:
+    case D3D12Chunk::Device_CreateCommittedResource:
+    case D3D12Chunk::Device_CreatePlacedResource:
+    case D3D12Chunk::Device_CreateReservedResource:
+    case D3D12Chunk::Device_CreateQueryHeap:
+    case D3D12Chunk::Device_CreateFence:
+    case D3D12Chunk::SetName:
+    case D3D12Chunk::SetShaderDebugPath:
+    case D3D12Chunk::CreateSwapBuffer:
+    case D3D12Chunk::Device_CreatePipelineState:
+    case D3D12Chunk::Device_CreateHeapFromAddress:
+    case D3D12Chunk::Device_CreateHeapFromFileMapping:
+    case D3D12Chunk::Device_OpenSharedHandle:
+    case D3D12Chunk::Device_CreateCommandList1:
+    case D3D12Chunk::Device_CreateCommittedResource1:
+    case D3D12Chunk::Device_CreateHeap1:
+    case D3D12Chunk::Device_ExternalDXGIResource:
+      RDCERR("Unexpected chunk while processing frame: %s", ToStr(chunk).c_str());
+      return false;
+
+    // no explicit default so that we have compiler warnings if a chunk isn't explicitly handled.
+    case D3D12Chunk::Max: break;
+  }
+
+  {
+    SystemChunk system = (SystemChunk)chunk;
+
+    if(system == SystemChunk::CaptureEnd)
+    {
+      SERIALISE_ELEMENT_LOCAL(PresentedImage, ResourceId()).TypedAs("ID3D12Resource *"_lit);
+
+      SERIALISE_CHECK_READ_ERRORS();
+
+      if(PresentedImage != ResourceId())
+        m_Cmd.m_LastPresentedImage = PresentedImage;
+
+      if(IsLoading(m_State) && m_Cmd.m_LastChunk != D3D12Chunk::Swapchain_Present)
+      {
+        m_Cmd.AddEvent();
+
+        DrawcallDescription draw;
+        draw.name = "End of Capture";
+        draw.flags |= DrawFlags::Present;
+
+        draw.copyDestination = m_Cmd.m_LastPresentedImage;
+
+        m_Cmd.AddDrawcall(draw, true);
+      }
+
+      ret = true;
+    }
+    else if(!ret)
+    {
+      RDCERR("Unrecognised Chunk type %s", ToStr(chunk).c_str());
+      return false;
     }
   }
 
@@ -817,6 +935,8 @@ ReplayStatus WrappedID3D12CommandQueue::ReplayLog(CaptureState readType, uint32_
     if(IsActiveReplaying(m_State) && startEventID == endEventID)
       break;
 
+    m_Cmd.m_LastChunk = context;
+
     // increment root event ID either if we didn't just replay a cmd
     // buffer event, OR if we are doing a frame sub-section replay,
     // in which case it's up to the calling code to make sure we only
@@ -841,11 +961,6 @@ ReplayStatus WrappedID3D12CommandQueue::ReplayLog(CaptureState readType, uint32_
 
   m_StructuredFile = NULL;
 
-  if(IsLoading(m_State))
-  {
-    std::sort(m_Cmd.m_Events.begin(), m_Cmd.m_Events.end());
-  }
-
   for(size_t i = 0; i < m_Cmd.m_RerecordCmdList.size(); i++)
     SAFE_RELEASE(m_Cmd.m_RerecordCmdList[i]);
 
@@ -868,6 +983,7 @@ WrappedID3D12GraphicsCommandList::WrappedID3D12GraphicsCommandList(ID3D12Graphic
   m_pList2 = NULL;
   m_pList3 = NULL;
   m_pList4 = NULL;
+  m_pList5 = NULL;
 
   m_WrappedDebug.m_pReal = NULL;
   m_WrappedDebug.m_pReal1 = NULL;
@@ -883,6 +999,7 @@ WrappedID3D12GraphicsCommandList::WrappedID3D12GraphicsCommandList(ID3D12Graphic
     m_pList->QueryInterface(__uuidof(ID3D12GraphicsCommandList2), (void **)&m_pList2);
     m_pList->QueryInterface(__uuidof(ID3D12GraphicsCommandList3), (void **)&m_pList3);
     m_pList->QueryInterface(__uuidof(ID3D12GraphicsCommandList4), (void **)&m_pList4);
+    m_pList->QueryInterface(__uuidof(ID3D12GraphicsCommandList5), (void **)&m_pList5);
   }
 
   // create a temporary and grab its resource ID
@@ -959,6 +1076,7 @@ WrappedID3D12GraphicsCommandList::~WrappedID3D12GraphicsCommandList()
   SAFE_RELEASE(m_WrappedDebug.m_pReal);
   SAFE_RELEASE(m_WrappedDebug.m_pReal1);
   SAFE_RELEASE(m_WrappedDebug.m_pReal2);
+  SAFE_RELEASE(m_pList5);
   SAFE_RELEASE(m_pList4);
   SAFE_RELEASE(m_pList3);
   SAFE_RELEASE(m_pList2);
@@ -990,7 +1108,7 @@ WriteSerialiser &WrappedID3D12GraphicsCommandList::GetThreadSerialiser()
   return m_pDevice->GetThreadSerialiser();
 }
 
-std::string WrappedID3D12GraphicsCommandList::GetChunkName(uint32_t idx)
+rdcstr WrappedID3D12GraphicsCommandList::GetChunkName(uint32_t idx)
 {
   if((SystemChunk)idx < SystemChunk::FirstDriverChunk)
     return ToStr((SystemChunk)idx);
@@ -1104,6 +1222,19 @@ HRESULT STDMETHODCALLTYPE WrappedID3D12GraphicsCommandList::QueryInterface(REFII
       return E_NOINTERFACE;
     }
   }
+  else if(riid == __uuidof(ID3D12GraphicsCommandList5))
+  {
+    if(m_pList5)
+    {
+      *ppvObject = (ID3D12GraphicsCommandList5 *)this;
+      AddRef();
+      return S_OK;
+    }
+    else
+    {
+      return E_NOINTERFACE;
+    }
+  }
   else if(riid == __uuidof(ID3D12CommandList))
   {
     *ppvObject = (ID3D12CommandList *)this;
@@ -1138,7 +1269,7 @@ HRESULT STDMETHODCALLTYPE WrappedID3D12GraphicsCommandList::QueryInterface(REFII
 
 void BakedCmdListInfo::ShiftForRemoved(uint32_t shiftDrawID, uint32_t shiftEID, size_t idx)
 {
-  std::vector<D3D12DrawcallTreeNode> &draws = draw->children;
+  rdcarray<D3D12DrawcallTreeNode> &draws = draw->children;
 
   drawCount -= shiftDrawID;
   eventCount -= shiftEID;
@@ -1251,7 +1382,7 @@ void D3D12CommandData::GetIndirectBuffer(size_t size, ID3D12Resource **buf, uint
   m_IndirectOffset = AlignUp16(m_IndirectOffset + size);
 }
 
-uint32_t D3D12CommandData::HandlePreCallback(ID3D12GraphicsCommandList4 *list, bool dispatch,
+uint32_t D3D12CommandData::HandlePreCallback(ID3D12GraphicsCommandListX *list, bool dispatch,
                                              uint32_t multiDrawOffset)
 {
   if(!m_DrawcallCallback)
@@ -1337,7 +1468,7 @@ bool D3D12CommandData::IsPartialCmdList(ResourceId cmdid)
   return false;
 }
 
-ID3D12GraphicsCommandList4 *D3D12CommandData::RerecordCmdList(ResourceId cmdid,
+ID3D12GraphicsCommandListX *D3D12CommandData::RerecordCmdList(ResourceId cmdid,
                                                               PartialReplayIndex partialType)
 {
   if(m_OutsideCmdList != NULL)
@@ -1378,6 +1509,11 @@ void D3D12CommandData::AddEvent()
 
   apievent.callstack = m_ChunkMetadata.callstack;
 
+  // if we're using replay-time debug messages, fetch them now since we can do better to correlate
+  // to events on replay
+  if(m_pDevice->GetReplayOptions().apiValidation)
+    m_EventMessages = m_pDevice->GetDebugMessages();
+
   for(size_t i = 0; i < m_EventMessages.size(); i++)
     m_EventMessages[i].eventId = apievent.eventId;
 
@@ -1385,14 +1521,15 @@ void D3D12CommandData::AddEvent()
   {
     m_BakedCmdListInfo[m_LastCmdListID].curEvents.push_back(apievent);
 
-    std::vector<DebugMessage> &msgs = m_BakedCmdListInfo[m_LastCmdListID].debugMessages;
+    rdcarray<DebugMessage> &msgs = m_BakedCmdListInfo[m_LastCmdListID].debugMessages;
 
-    msgs.insert(msgs.end(), m_EventMessages.begin(), m_EventMessages.end());
+    msgs.append(m_EventMessages);
   }
   else
   {
     m_RootEvents.push_back(apievent);
-    m_Events.push_back(apievent);
+    m_Events.resize(apievent.eventId + 1);
+    m_Events[apievent.eventId] = apievent;
 
     for(auto it = m_EventMessages.begin(); it != m_EventMessages.end(); ++it)
       m_pDevice->AddDebugMessage(*it);
@@ -1407,7 +1544,7 @@ void D3D12CommandData::AddUsage(D3D12DrawcallTreeNode &drawNode, ResourceId id,
   if(id == ResourceId())
     return;
 
-  drawNode.resourceUsage.push_back(std::make_pair(id, EventUsage(EID, usage)));
+  drawNode.resourceUsage.push_back(make_rdcpair(id, EventUsage(EID, usage)));
 }
 
 void D3D12CommandData::AddUsage(const D3D12RenderState &state, D3D12DrawcallTreeNode &drawNode)
@@ -1434,38 +1571,38 @@ void D3D12CommandData::AddUsage(const D3D12RenderState &state, D3D12DrawcallTree
 
     if(d.flags & DrawFlags::Indexed && state.ibuffer.buf != ResourceId())
       drawNode.resourceUsage.push_back(
-          std::make_pair(state.ibuffer.buf, EventUsage(e, ResourceUsage::IndexBuffer)));
+          make_rdcpair(state.ibuffer.buf, EventUsage(e, ResourceUsage::IndexBuffer)));
 
     for(size_t i = 0; i < state.vbuffers.size(); i++)
     {
       if(state.vbuffers[i].buf != ResourceId())
         drawNode.resourceUsage.push_back(
-            std::make_pair(state.vbuffers[i].buf, EventUsage(e, ResourceUsage::VertexBuffer)));
+            make_rdcpair(state.vbuffers[i].buf, EventUsage(e, ResourceUsage::VertexBuffer)));
     }
 
     for(size_t i = 0; i < state.streamouts.size(); i++)
     {
       if(state.streamouts[i].buf != ResourceId())
         drawNode.resourceUsage.push_back(
-            std::make_pair(state.streamouts[i].buf, EventUsage(e, ResourceUsage::StreamOut)));
+            make_rdcpair(state.streamouts[i].buf, EventUsage(e, ResourceUsage::StreamOut)));
       if(state.streamouts[i].countbuf != ResourceId())
         drawNode.resourceUsage.push_back(
-            std::make_pair(state.streamouts[i].countbuf, EventUsage(e, ResourceUsage::StreamOut)));
+            make_rdcpair(state.streamouts[i].countbuf, EventUsage(e, ResourceUsage::StreamOut)));
     }
 
-    vector<ResourceId> rts = state.GetRTVIDs();
+    rdcarray<ResourceId> rts = state.GetRTVIDs();
 
     for(size_t i = 0; i < rts.size(); i++)
     {
       if(rts[i] != ResourceId())
         drawNode.resourceUsage.push_back(
-            std::make_pair(rts[i], EventUsage(e, ResourceUsage::ColorTarget)));
+            make_rdcpair(rts[i], EventUsage(e, ResourceUsage::ColorTarget)));
     }
 
     ResourceId id = state.GetDSVID();
     if(id != ResourceId())
       drawNode.resourceUsage.push_back(
-          std::make_pair(id, EventUsage(e, ResourceUsage::DepthStencilTarget)));
+          make_rdcpair(id, EventUsage(e, ResourceUsage::DepthStencilTarget)));
   }
 
   if(rootdata)
@@ -1473,12 +1610,12 @@ void D3D12CommandData::AddUsage(const D3D12RenderState &state, D3D12DrawcallTree
     WrappedID3D12RootSignature *sig =
         m_pDevice->GetResourceManager()->GetCurrentAs<WrappedID3D12RootSignature>(rootdata->rootsig);
 
-    for(size_t rootEl = 0; rootEl < sig->sig.params.size(); rootEl++)
+    for(size_t rootEl = 0; rootEl < sig->sig.Parameters.size(); rootEl++)
     {
       if(rootEl >= rootdata->sigelems.size())
         break;
 
-      const D3D12RootSignatureParameter &p = sig->sig.params[rootEl];
+      const D3D12RootSignatureParameter &p = sig->sig.Parameters[rootEl];
       const D3D12RenderState::SignatureElement &el = rootdata->sigelems[rootEl];
 
       ResourceUsage cb = ResourceUsage::CS_Constants;
@@ -1619,7 +1756,7 @@ void D3D12CommandData::AddDrawcall(const DrawcallDescription &d, bool hasEvents,
     draw.topology = MakePrimitiveTopology(m_BakedCmdListInfo[m_LastCmdListID].state.topo);
     draw.indexByteWidth = m_BakedCmdListInfo[m_LastCmdListID].state.ibuffer.bytewidth;
 
-    vector<ResourceId> rts = m_BakedCmdListInfo[m_LastCmdListID].state.GetRTVIDs();
+    rdcarray<ResourceId> rts = m_BakedCmdListInfo[m_LastCmdListID].state.GetRTVIDs();
 
     for(size_t i = 0; i < ARRAY_COUNT(draw.outputs); i++)
     {
@@ -1645,9 +1782,9 @@ void D3D12CommandData::AddDrawcall(const DrawcallDescription &d, bool hasEvents,
 
   if(hasEvents)
   {
-    vector<APIEvent> &srcEvents = m_LastCmdListID != ResourceId()
-                                      ? m_BakedCmdListInfo[m_LastCmdListID].curEvents
-                                      : m_RootEvents;
+    rdcarray<APIEvent> &srcEvents = m_LastCmdListID != ResourceId()
+                                        ? m_BakedCmdListInfo[m_LastCmdListID].curEvents
+                                        : m_RootEvents;
 
     draw.events = srcEvents;
     srcEvents.clear();
@@ -1664,7 +1801,8 @@ void D3D12CommandData::AddDrawcall(const DrawcallDescription &d, bool hasEvents,
     if(m_LastCmdListID != ResourceId() && addUsage)
       AddUsage(m_BakedCmdListInfo[m_LastCmdListID].state, node);
 
-    node.children.insert(node.children.begin(), draw.children.begin(), draw.children.end());
+    for(const DrawcallDescription &child : draw.children)
+      node.children.push_back(D3D12DrawcallTreeNode(child));
     GetDrawcallStack().back()->children.push_back(node);
   }
   else
@@ -1672,7 +1810,7 @@ void D3D12CommandData::AddDrawcall(const DrawcallDescription &d, bool hasEvents,
 }
 
 void D3D12CommandData::InsertDrawsAndRefreshIDs(ResourceId cmd,
-                                                vector<D3D12DrawcallTreeNode> &cmdBufNodes)
+                                                rdcarray<D3D12DrawcallTreeNode> &cmdBufNodes)
 {
   // assign new drawcall IDs
   for(size_t i = 0; i < cmdBufNodes.size(); i++)
@@ -1694,14 +1832,15 @@ void D3D12CommandData::InsertDrawsAndRefreshIDs(ResourceId cmd,
     for(APIEvent &ev : n.draw.events)
     {
       ev.eventId += m_RootEventID;
-      m_Events.push_back(ev);
+      m_Events.resize(ev.eventId + 1);
+      m_Events[ev.eventId] = ev;
     }
 
     DrawcallUse use(m_Events.back().fileOffset, n.draw.eventId, cmd, cmdBufNodes[i].draw.eventId);
 
     // insert in sorted location
     auto drawit = std::lower_bound(m_DrawcallUses.begin(), m_DrawcallUses.end(), use);
-    m_DrawcallUses.insert(drawit, use);
+    m_DrawcallUses.insert(drawit - m_DrawcallUses.begin(), use);
 
     RDCASSERT(n.children.empty());
 
