@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,6 @@
 
 #pragma once
 
-#include "api/replay/renderdoc_replay.h"
 #include "common/common.h"
 #include "d3d12_common.h"
 #include "d3d12_state.h"
@@ -36,13 +35,13 @@ struct D3D12DrawcallTreeNode
   D3D12DrawcallTreeNode(const D3D12DrawcallTreeNode &other) { *this = other; }
   ~D3D12DrawcallTreeNode() { SAFE_DELETE(state); }
   DrawcallDescription draw;
-  vector<D3D12DrawcallTreeNode> children;
+  rdcarray<D3D12DrawcallTreeNode> children;
 
   D3D12RenderState *state = NULL;
 
-  vector<pair<ResourceId, EventUsage> > resourceUsage;
+  rdcarray<rdcpair<ResourceId, EventUsage> > resourceUsage;
 
-  vector<ResourceId> executedCmds;
+  rdcarray<ResourceId> executedCmds;
 
   D3D12DrawcallTreeNode &operator=(const DrawcallDescription &d)
   {
@@ -86,9 +85,9 @@ struct D3D12DrawcallTreeNode
     }
   }
 
-  vector<DrawcallDescription> Bake()
+  rdcarray<DrawcallDescription> Bake()
   {
-    vector<DrawcallDescription> ret;
+    rdcarray<DrawcallDescription> ret;
     if(children.empty())
       return ret;
 
@@ -120,17 +119,17 @@ struct D3D12DrawcallCallback
   // and do the real draw by returning true. OR they can do nothing in PreDraw,
   // do the real draw, then in PostDraw return true to apply the modifications
   // which are then undone in PostRedraw.
-  virtual void PreDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) = 0;
-  virtual bool PostDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) = 0;
-  virtual void PostRedraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) = 0;
+  virtual void PreDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) = 0;
+  virtual bool PostDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) = 0;
+  virtual void PostRedraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) = 0;
 
   // same principle as above, but for dispatch calls
-  virtual void PreDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) = 0;
-  virtual bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) = 0;
-  virtual void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) = 0;
+  virtual void PreDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) = 0;
+  virtual bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) = 0;
+  virtual void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) = 0;
 
   // called immediately before a command list is closed
-  virtual void PreCloseCommandList(ID3D12GraphicsCommandList4 *cmd) = 0;
+  virtual void PreCloseCommandList(ID3D12GraphicsCommandListX *cmd) = 0;
   // if a command list is recorded once and submitted N > 1 times, then the same
   // drawcall will have several EIDs that refer to it. We'll only do the full
   // callbacks above for the first EID, then call this function for the others
@@ -140,31 +139,31 @@ struct D3D12DrawcallCallback
   // helper functions to downcast command list because we know it's wrapped
   void PreDraw(uint32_t eid, ID3D12GraphicsCommandList *cmd)
   {
-    return PreDraw(eid, (ID3D12GraphicsCommandList4 *)cmd);
+    return PreDraw(eid, (ID3D12GraphicsCommandListX *)cmd);
   }
   bool PostDraw(uint32_t eid, ID3D12GraphicsCommandList *cmd)
   {
-    return PostDraw(eid, (ID3D12GraphicsCommandList4 *)cmd);
+    return PostDraw(eid, (ID3D12GraphicsCommandListX *)cmd);
   }
   void PostRedraw(uint32_t eid, ID3D12GraphicsCommandList *cmd)
   {
-    return PostRedraw(eid, (ID3D12GraphicsCommandList4 *)cmd);
+    return PostRedraw(eid, (ID3D12GraphicsCommandListX *)cmd);
   }
   void PreDispatch(uint32_t eid, ID3D12GraphicsCommandList *cmd)
   {
-    return PreDispatch(eid, (ID3D12GraphicsCommandList4 *)cmd);
+    return PreDispatch(eid, (ID3D12GraphicsCommandListX *)cmd);
   }
   bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandList *cmd)
   {
-    return PostDispatch(eid, (ID3D12GraphicsCommandList4 *)cmd);
+    return PostDispatch(eid, (ID3D12GraphicsCommandListX *)cmd);
   }
   void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandList *cmd)
   {
-    return PostRedispatch(eid, (ID3D12GraphicsCommandList4 *)cmd);
+    return PostRedispatch(eid, (ID3D12GraphicsCommandListX *)cmd);
   }
   void PreCloseCommandList(ID3D12GraphicsCommandList *cmd)
   {
-    return PreCloseCommandList((ID3D12GraphicsCommandList4 *)cmd);
+    return PreCloseCommandList((ID3D12GraphicsCommandListX *)cmd);
   }
 };
 
@@ -189,27 +188,24 @@ struct BakedCmdListInfo
     UINT realCount = 0;
   };
 
-  vector<ID3D12GraphicsCommandList4 *> crackedLists;
-  vector<ExecuteData> executeEvents;
+  rdcarray<ID3D12GraphicsCommandListX *> crackedLists;
+  rdcarray<ExecuteData> executeEvents;
 
-  vector<APIEvent> curEvents;
-  vector<DebugMessage> debugMessages;
-  std::list<D3D12DrawcallTreeNode *> drawStack;
+  rdcarray<APIEvent> curEvents;
+  rdcarray<DebugMessage> debugMessages;
+  rdcarray<D3D12DrawcallTreeNode *> drawStack;
 
-  vector<pair<ResourceId, EventUsage> > resourceUsage;
+  rdcarray<rdcpair<ResourceId, EventUsage> > resourceUsage;
 
   ResourceId allocator;
   D3D12_COMMAND_LIST_TYPE type;
   UINT nodeMask;
   D3D12RenderState state;
 
-  vector<D3D12_RESOURCE_BARRIER> barriers;
+  rdcarray<D3D12_RESOURCE_BARRIER> barriers;
 
   ResourceId parentList;
 
-  // if a render pass is active when we early-close a list, we need to end it
-  bool renderPassActive = false;
-
   // modified during recording to ensure we end any markers that should be ended but weren't due to
   // a partial replay
   int markerCount;
@@ -235,13 +231,13 @@ struct D3D12CommandData
 
   ResourceId m_LastCmdListID;
 
-  map<ResourceId, ID3D12CommandAllocator *> m_CrackedAllocators;
+  std::map<ResourceId, ID3D12CommandAllocator *> m_CrackedAllocators;
 
-  vector<ID3D12Resource *> m_IndirectBuffers;
+  rdcarray<ID3D12Resource *> m_IndirectBuffers;
   static const uint64_t m_IndirectSize = 4 * 1024 * 1024;
   uint64_t m_IndirectOffset;
 
-  map<ResourceId, BakedCmdListInfo> m_BakedCmdListInfo;
+  std::map<ResourceId, BakedCmdListInfo> m_BakedCmdListInfo;
 
   D3D12RenderState m_RenderState;
 
@@ -263,6 +259,7 @@ struct D3D12CommandData
     {
       partialParent = ResourceId();
       baseEvent = 0;
+      renderPassActive = false;
     }
 
     // this records where in the frame a command list was executed, so that we know if our replay
@@ -275,7 +272,7 @@ struct D3D12CommandData
     // However, a single baked command list can be executed multiple times - so we have to have a
     // list of base events
     // Map from bakeID -> vector<baseEventID>
-    std::map<ResourceId, std::vector<uint32_t> > cmdListExecs;
+    std::map<ResourceId, rdcarray<uint32_t> > cmdListExecs;
 
     // This is just the baked ID of the parent command list that's partially replayed
     // If we are in the middle of a partial replay - allows fast checking in all CmdList chunks,
@@ -290,15 +287,18 @@ struct D3D12CommandData
     // executecmdlists, but also allows the recording to 'rebase' the
     // last event ID by subtracting this, to know how far to record
     uint32_t baseEvent;
+
+    // if a render pass is active when we early-close a list, we need to end it
+    bool renderPassActive;
   } m_Partial[ePartialNum];
 
   // if we're replaying just a single draw or a particular command
   // list subsection of command events, we don't go through the
   // whole original command lists to set up the partial replay,
   // so we just set this command list
-  ID3D12GraphicsCommandList4 *m_OutsideCmdList = NULL;
+  ID3D12GraphicsCommandListX *m_OutsideCmdList = NULL;
 
-  void InsertDrawsAndRefreshIDs(ResourceId cmd, vector<D3D12DrawcallTreeNode> &cmdBufNodes);
+  void InsertDrawsAndRefreshIDs(ResourceId cmd, rdcarray<D3D12DrawcallTreeNode> &cmdBufNodes);
 
   // this is a list of uint64_t file offset -> uint32_t EIDs of where each
   // drawcall is used. E.g. the drawcall at offset 873954 is EID 50. If a
@@ -322,31 +322,34 @@ struct D3D12CommandData
       return eventId < o.eventId;
     }
   };
-  vector<DrawcallUse> m_DrawcallUses;
+  rdcarray<DrawcallUse> m_DrawcallUses;
 
-  vector<DebugMessage> m_EventMessages;
+  rdcarray<DebugMessage> m_EventMessages;
 
-  std::map<ResourceId, ID3D12GraphicsCommandList4 *> m_RerecordCmds;
-  std::vector<ID3D12GraphicsCommandList4 *> m_RerecordCmdList;
+  std::map<ResourceId, ID3D12GraphicsCommandListX *> m_RerecordCmds;
+  rdcarray<ID3D12GraphicsCommandListX *> m_RerecordCmdList;
 
   bool m_AddedDrawcall;
 
-  vector<APIEvent> m_RootEvents, m_Events;
+  rdcarray<APIEvent> m_RootEvents, m_Events;
 
   uint64_t m_CurChunkOffset;
   SDChunkMetaData m_ChunkMetadata;
   uint32_t m_RootEventID, m_RootDrawcallID;
   uint32_t m_FirstEventID, m_LastEventID;
+  D3D12Chunk m_LastChunk;
+
+  ResourceId m_LastPresentedImage;
 
   SDFile *m_StructuredFile;
 
-  map<ResourceId, vector<EventUsage> > m_ResourceUses;
+  std::map<ResourceId, rdcarray<EventUsage> > m_ResourceUses;
 
   D3D12DrawcallTreeNode m_ParentDrawcall;
 
-  std::list<D3D12DrawcallTreeNode *> m_RootDrawcallStack;
+  rdcarray<D3D12DrawcallTreeNode *> m_RootDrawcallStack;
 
-  std::list<D3D12DrawcallTreeNode *> &GetDrawcallStack()
+  rdcarray<D3D12DrawcallTreeNode *> &GetDrawcallStack()
   {
     if(m_LastCmdListID != ResourceId())
       return m_BakedCmdListInfo[m_LastCmdListID].drawStack;
@@ -360,13 +363,13 @@ struct D3D12CommandData
 
   // util function to handle fetching the right eventId, calling any
   // aliases then calling PreDraw/PreDispatch.
-  uint32_t HandlePreCallback(ID3D12GraphicsCommandList4 *list, bool dispatch = false,
+  uint32_t HandlePreCallback(ID3D12GraphicsCommandListX *list, bool dispatch = false,
                              uint32_t multiDrawOffset = 0);
 
   bool InRerecordRange(ResourceId cmdid);
   bool HasRerecordCmdList(ResourceId cmdid);
   bool IsPartialCmdList(ResourceId cmdid);
-  ID3D12GraphicsCommandList4 *RerecordCmdList(ResourceId cmdid,
+  ID3D12GraphicsCommandListX *RerecordCmdList(ResourceId cmdid,
                                               PartialReplayIndex partialType = ePartialNum);
 
   void AddDrawcall(const DrawcallDescription &d, bool hasEvents, bool addUsage = true);
