@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,7 +28,7 @@
 #include "d3d12_manager.h"
 #include "d3d12_resources.h"
 
-D3D12MarkerRegion::D3D12MarkerRegion(ID3D12GraphicsCommandList *l, const std::string &marker)
+D3D12MarkerRegion::D3D12MarkerRegion(ID3D12GraphicsCommandList *l, const rdcstr &marker)
 {
   list = l;
   queue = NULL;
@@ -36,7 +36,7 @@ D3D12MarkerRegion::D3D12MarkerRegion(ID3D12GraphicsCommandList *l, const std::st
   D3D12MarkerRegion::Begin(list, marker);
 }
 
-D3D12MarkerRegion::D3D12MarkerRegion(ID3D12CommandQueue *q, const std::string &marker)
+D3D12MarkerRegion::D3D12MarkerRegion(ID3D12CommandQueue *q, const rdcstr &marker)
 {
   list = NULL;
   queue = q;
@@ -52,39 +52,46 @@ D3D12MarkerRegion::~D3D12MarkerRegion()
     D3D12MarkerRegion::End(queue);
 }
 
-void D3D12MarkerRegion::Begin(ID3D12GraphicsCommandList *list, const std::string &marker)
+void D3D12MarkerRegion::Begin(ID3D12GraphicsCommandList *list, const rdcstr &marker)
 {
   if(list)
   {
-    std::wstring text = StringFormat::UTF82Wide(marker);
-    list->BeginEvent(0, text.c_str(), (UINT)text.size());
+    // Some debuggers (but not all) will assume the event string is null-terminated, and
+    // display one less character than specified by the size. Append a space to pad the
+    // output without visibly changing the event marker for other debuggers.
+    rdcwstr text = StringFormat::UTF82Wide(marker + " ");
+    UINT size = (UINT)text.length() * sizeof(wchar_t);
+    list->BeginEvent(0, text.c_str(), size);
   }
 }
 
-void D3D12MarkerRegion::Begin(ID3D12CommandQueue *queue, const std::string &marker)
+void D3D12MarkerRegion::Begin(ID3D12CommandQueue *queue, const rdcstr &marker)
 {
   if(queue)
   {
-    std::wstring text = StringFormat::UTF82Wide(marker);
-    queue->BeginEvent(0, text.c_str(), (UINT)text.size());
+    rdcwstr text = StringFormat::UTF82Wide(marker + " ");
+    UINT size = (UINT)text.length() * sizeof(wchar_t);
+    queue->BeginEvent(0, text.c_str(), size);
   }
 }
 
-void D3D12MarkerRegion::Set(ID3D12GraphicsCommandList *list, const std::string &marker)
+void D3D12MarkerRegion::Set(ID3D12GraphicsCommandList *list, const rdcstr &marker)
 {
   if(list)
   {
-    std::wstring text = StringFormat::UTF82Wide(marker);
-    list->SetMarker(0, text.c_str(), (UINT)text.size());
+    rdcwstr text = StringFormat::UTF82Wide(marker + " ");
+    UINT size = (UINT)text.length() * sizeof(wchar_t);
+    list->SetMarker(0, text.c_str(), size);
   }
 }
 
-void D3D12MarkerRegion::Set(ID3D12CommandQueue *queue, const std::string &marker)
+void D3D12MarkerRegion::Set(ID3D12CommandQueue *queue, const rdcstr &marker)
 {
   if(queue)
   {
-    std::wstring text = StringFormat::UTF82Wide(marker);
-    queue->SetMarker(0, text.c_str(), (UINT)text.size());
+    rdcwstr text = StringFormat::UTF82Wide(marker + " ");
+    UINT size = (UINT)text.length() * sizeof(wchar_t);
+    queue->SetMarker(0, text.c_str(), size);
   }
 }
 
@@ -140,6 +147,10 @@ bool EnableD3D12DebugLayer(PFN_D3D12_GET_DEBUG_INTERFACE getDebugInterface)
 
     return true;
   }
+  else if(hr == DXGI_ERROR_SDK_COMPONENT_MISSING)
+  {
+    RDCWARN("Debug layer not available: DXGI_ERROR_SDK_COMPONENT_MISSING");
+  }
   else
   {
     RDCERR("Couldn't enable debug layer: %x", hr);
@@ -148,9 +159,36 @@ bool EnableD3D12DebugLayer(PFN_D3D12_GET_DEBUG_INTERFACE getDebugInterface)
   return false;
 }
 
-D3D12InitParams::D3D12InitParams()
+HRESULT EnumAdapterByLuid(IDXGIFactory1 *factory, LUID luid, IDXGIAdapter **pAdapter)
 {
-  MinimumFeatureLevel = D3D_FEATURE_LEVEL_11_0;
+  HRESULT hr = S_OK;
+
+  *pAdapter = NULL;
+
+  for(UINT i = 0; i < 10; i++)
+  {
+    IDXGIAdapter *adapter = NULL;
+    hr = factory->EnumAdapters(i, &adapter);
+    if(hr == S_OK && adapter)
+    {
+      DXGI_ADAPTER_DESC desc;
+      adapter->GetDesc(&desc);
+
+      if(desc.AdapterLuid.LowPart == luid.LowPart && desc.AdapterLuid.HighPart == luid.HighPart)
+      {
+        *pAdapter = adapter;
+        return S_OK;
+      }
+
+      adapter->Release();
+    }
+    else
+    {
+      break;
+    }
+  }
+
+  return E_FAIL;
 }
 
 bool D3D12InitParams::IsSupportedVersion(uint64_t ver)
@@ -158,14 +196,23 @@ bool D3D12InitParams::IsSupportedVersion(uint64_t ver)
   if(ver == CurrentVersion)
     return true;
 
+  // 0x4 -> 0x5 - CPU_DESCRIPTOR_HANDLE serialised inline as D3D12Descriptor in appropriate
+  //              list-recording functions
+  if(ver == 0x4)
+    return true;
+
   // 0x5 -> 0x6 - Multiply by number of planes in format when serialising initial states -
   //              i.e. stencil is saved with depth in initial states.
   if(ver == 0x5)
     return true;
 
-  // 0x4 -> 0x5 - CPU_DESCRIPTOR_HANDLE serialised inline as D3D12Descriptor in appropriate
-  //              list-recording functions
-  if(ver == 0x4)
+  // 0x6 -> 0x7 - Fixed serialisation of D3D12_WRITEBUFFERIMMEDIATE_PARAMETER to properly replay the
+  //              GPU address
+  if(ver == 0x6)
+    return true;
+
+  // 0x7 -> 0x8 - Added serialisation of adapter descriptor in D3D12InitParams
+  if(ver == 0x7)
     return true;
 
   return false;
@@ -175,6 +222,15 @@ template <typename SerialiserType>
 void DoSerialise(SerialiserType &ser, D3D12InitParams &el)
 {
   SERIALISE_MEMBER(MinimumFeatureLevel);
+
+  if(ser.VersionAtLeast(0x8))
+  {
+    SERIALISE_MEMBER(AdapterDesc);
+  }
+  else
+  {
+    RDCEraseEl(el.AdapterDesc);
+  }
 }
 
 INSTANTIATE_SERIALISE_TYPE(D3D12InitParams);
@@ -194,6 +250,7 @@ TextureType MakeTextureDim(D3D12_SRV_DIMENSION dim)
     case D3D12_SRV_DIMENSION_TEXTURE3D: return TextureType::Texture3D;
     case D3D12_SRV_DIMENSION_TEXTURECUBE: return TextureType::TextureCube;
     case D3D12_SRV_DIMENSION_TEXTURECUBEARRAY: return TextureType::TextureCubeArray;
+    default: break;
   }
 
   return TextureType::Unknown;
@@ -212,6 +269,7 @@ TextureType MakeTextureDim(D3D12_RTV_DIMENSION dim)
     case D3D12_RTV_DIMENSION_TEXTURE2DMS: return TextureType::Texture2DMS;
     case D3D12_RTV_DIMENSION_TEXTURE2DMSARRAY: return TextureType::Texture2DMSArray;
     case D3D12_RTV_DIMENSION_TEXTURE3D: return TextureType::Texture3D;
+    default: break;
   }
 
   return TextureType::Unknown;
@@ -228,6 +286,7 @@ TextureType MakeTextureDim(D3D12_DSV_DIMENSION dim)
     case D3D12_DSV_DIMENSION_TEXTURE2DARRAY: return TextureType::Texture2DArray;
     case D3D12_DSV_DIMENSION_TEXTURE2DMS: return TextureType::Texture2DMS;
     case D3D12_DSV_DIMENSION_TEXTURE2DMSARRAY: return TextureType::Texture2DMSArray;
+    default: break;
   }
 
   return TextureType::Unknown;
@@ -244,6 +303,7 @@ TextureType MakeTextureDim(D3D12_UAV_DIMENSION dim)
     case D3D12_UAV_DIMENSION_TEXTURE2D: return TextureType::Texture2D;
     case D3D12_UAV_DIMENSION_TEXTURE2DARRAY: return TextureType::Texture2DArray;
     case D3D12_UAV_DIMENSION_TEXTURE3D: return TextureType::Texture3D;
+    default: break;
   }
 
   return TextureType::Unknown;
@@ -310,47 +370,44 @@ TextureFilter MakeFilter(D3D12_FILTER filter)
     filter = D3D12_FILTER(filter & 0x7f);
   }
 
-  if(filter == D3D12_FILTER_ANISOTROPIC)
+  switch(filter)
   {
-    ret.minify = ret.magnify = ret.mip = FilterMode::Anisotropic;
-  }
-  else
-  {
-    switch(filter)
-    {
-      case D3D12_FILTER_MIN_MAG_MIP_POINT:
-        ret.minify = ret.magnify = ret.mip = FilterMode::Point;
-        break;
-      case D3D12_FILTER_MIN_MAG_POINT_MIP_LINEAR:
-        ret.minify = ret.magnify = FilterMode::Point;
-        ret.mip = FilterMode::Linear;
-        break;
-      case D3D12_FILTER_MIN_POINT_MAG_LINEAR_MIP_POINT:
-        ret.minify = FilterMode::Point;
-        ret.magnify = FilterMode::Linear;
-        ret.mip = FilterMode::Point;
-        break;
-      case D3D12_FILTER_MIN_POINT_MAG_MIP_LINEAR:
-        ret.minify = FilterMode::Point;
-        ret.magnify = ret.mip = FilterMode::Linear;
-        break;
-      case D3D12_FILTER_MIN_LINEAR_MAG_MIP_POINT:
-        ret.minify = FilterMode::Linear;
-        ret.magnify = ret.mip = FilterMode::Point;
-        break;
-      case D3D12_FILTER_MIN_LINEAR_MAG_POINT_MIP_LINEAR:
-        ret.minify = FilterMode::Linear;
-        ret.magnify = FilterMode::Point;
-        ret.mip = FilterMode::Linear;
-        break;
-      case D3D12_FILTER_MIN_MAG_LINEAR_MIP_POINT:
-        ret.minify = ret.magnify = FilterMode::Linear;
-        ret.mip = FilterMode::Point;
-        break;
-      case D3D12_FILTER_MIN_MAG_MIP_LINEAR:
-        ret.minify = ret.magnify = ret.mip = FilterMode::Linear;
-        break;
-    }
+    case D3D12_FILTER_ANISOTROPIC:
+      ret.minify = ret.magnify = ret.mip = FilterMode::Anisotropic;
+      break;
+    case D3D12_FILTER_MIN_MAG_MIP_POINT:
+      ret.minify = ret.magnify = ret.mip = FilterMode::Point;
+      break;
+    case D3D12_FILTER_MIN_MAG_POINT_MIP_LINEAR:
+      ret.minify = ret.magnify = FilterMode::Point;
+      ret.mip = FilterMode::Linear;
+      break;
+    case D3D12_FILTER_MIN_POINT_MAG_LINEAR_MIP_POINT:
+      ret.minify = FilterMode::Point;
+      ret.magnify = FilterMode::Linear;
+      ret.mip = FilterMode::Point;
+      break;
+    case D3D12_FILTER_MIN_POINT_MAG_MIP_LINEAR:
+      ret.minify = FilterMode::Point;
+      ret.magnify = ret.mip = FilterMode::Linear;
+      break;
+    case D3D12_FILTER_MIN_LINEAR_MAG_MIP_POINT:
+      ret.minify = FilterMode::Linear;
+      ret.magnify = ret.mip = FilterMode::Point;
+      break;
+    case D3D12_FILTER_MIN_LINEAR_MAG_POINT_MIP_LINEAR:
+      ret.minify = FilterMode::Linear;
+      ret.magnify = FilterMode::Point;
+      ret.mip = FilterMode::Linear;
+      break;
+    case D3D12_FILTER_MIN_MAG_LINEAR_MIP_POINT:
+      ret.minify = ret.magnify = FilterMode::Linear;
+      ret.mip = FilterMode::Point;
+      break;
+    case D3D12_FILTER_MIN_MAG_MIP_LINEAR:
+      ret.minify = ret.magnify = ret.mip = FilterMode::Linear;
+      break;
+    default: break;
   }
 
   return ret;
@@ -620,7 +677,7 @@ inline void PIX3DecodeStringInfo(const UINT64 BlobData, UINT64 &Alignment, UINT6
   IsShortcut = (BlobData >> PIXEventsStringIsShortcutBitShift) & PIXEventsStringIsShortcutWriteMask;
 }
 
-const UINT64 *PIX3DecodeStringParam(const UINT64 *pData, std::string &DecodedString)
+const UINT64 *PIX3DecodeStringParam(const UINT64 *pData, rdcstr &DecodedString)
 {
   UINT64 alignment;
   UINT64 copyChunkSize;
@@ -633,16 +690,16 @@ const UINT64 *PIX3DecodeStringParam(const UINT64 *pData, std::string &DecodedStr
   if(isANSI)
   {
     const char *c = (const char *)pData;
-    UINT formatStringByteCount = UINT(strlen((const char *)pData));
-    DecodedString = std::string(c, c + formatStringByteCount);
-    totalStringBytes = formatStringByteCount + 1;
+    UINT formatStringCharCount = UINT(strlen((const char *)pData));
+    DecodedString = rdcstr(c, formatStringCharCount);
+    totalStringBytes = formatStringCharCount + 1;
   }
   else
   {
     const wchar_t *w = (const wchar_t *)pData;
-    UINT formatStringByteCount = UINT(wcslen((const wchar_t *)pData));
-    DecodedString = StringFormat::Wide2UTF8(std::wstring(w, w + formatStringByteCount));
-    totalStringBytes = (formatStringByteCount + 1) * sizeof(wchar_t);
+    UINT formatStringCharCount = UINT(wcslen((const wchar_t *)pData));
+    DecodedString = StringFormat::Wide2UTF8(rdcwstr(w, formatStringCharCount));
+    totalStringBytes = (formatStringCharCount + 1) * sizeof(wchar_t);
   }
 
   UINT64 byteChunks = ((totalStringBytes + copyChunkSize - 1) / copyChunkSize) * copyChunkSize;
@@ -652,18 +709,18 @@ const UINT64 *PIX3DecodeStringParam(const UINT64 *pData, std::string &DecodedStr
   return pData;
 }
 
-string PIX3SprintfParams(const std::string &Format, const UINT64 *pData)
+rdcstr PIX3SprintfParams(const rdcstr &Format, const UINT64 *pData)
 {
-  std::string finalString;
-  std::string formatPart;
-  size_t lastFind = 0;
+  rdcstr finalString;
+  rdcstr formatPart;
+  int32_t lastFind = 0;
 
-  for(size_t found = Format.find_first_of("%"); found != std::string::npos;)
+  for(int32_t found = Format.indexOf('%'); found >= 0;)
   {
     finalString += Format.substr(lastFind, found - lastFind);
 
-    size_t endOfFormat = Format.find_first_of("%diufFeEgGxXoscpaAn", found + 1);
-    if(endOfFormat == std::string::npos)
+    int32_t endOfFormat = Format.find_first_of("%diufFeEgGxXoscpaAn", found + 1);
+    if(endOfFormat < 0)
     {
       finalString += "<FORMAT_ERROR>";
       break;
@@ -674,22 +731,19 @@ string PIX3SprintfParams(const std::string &Format, const UINT64 *pData)
     // strings
     if(formatPart.back() == 's')
     {
-      std::string stringParam;
+      rdcstr stringParam;
       pData = PIX3DecodeStringParam(pData, stringParam);
       finalString += stringParam;
     }
     // numerical values
     else
     {
-      static const UINT MAX_CHARACTERS_FOR_VALUE = 32;
-      char formattedValue[MAX_CHARACTERS_FOR_VALUE];
-      StringFormat::snprintf(formattedValue, MAX_CHARACTERS_FOR_VALUE, formatPart.c_str(), *pData);
-      finalString += formattedValue;
+      finalString += StringFormat::Fmt(formatPart.c_str(), *pData);
       ++pData;
     }
 
     lastFind = endOfFormat + 1;
-    found = Format.find_first_of("%", lastFind);
+    found = Format.indexOf('%', lastFind);
   }
 
   finalString += Format.substr(lastFind);
@@ -697,7 +751,7 @@ string PIX3SprintfParams(const std::string &Format, const UINT64 *pData)
   return finalString;
 }
 
-std::string PIX3DecodeEventString(const UINT64 *pData)
+rdcstr PIX3DecodeEventString(const UINT64 *pData)
 {
   // event header
   UINT64 timestamp;
@@ -724,7 +778,7 @@ std::string PIX3DecodeEventString(const UINT64 *pData)
   ++pData;
 
   // format string
-  std::string formatString;
+  rdcstr formatString;
   pData = PIX3DecodeStringParam(pData, formatString);
 
   if(eventType == ePIXEvent_BeginEvent_NoArgs)
@@ -1137,4 +1191,4 @@ D3D12_PACKED_PIPELINE_STATE_STREAM_DESC &D3D12_PACKED_PIPELINE_STATE_STREAM_DESC
   }
 
   return *this;
-}
\ No newline at end of file
+}
