@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,10 +29,11 @@
 #include "d3d12_command_queue.h"
 #include "d3d12_common.h"
 #include "d3d12_device.h"
+#include "d3d12_replay.h"
 
-vector<GPUCounter> D3D12Replay::EnumerateCounters()
+rdcarray<GPUCounter> D3D12Replay::EnumerateCounters()
 {
-  vector<GPUCounter> ret;
+  rdcarray<GPUCounter> ret;
 
   ret.push_back(GPUCounter::EventGPUDuration);
   ret.push_back(GPUCounter::InputVerticesRead);
@@ -50,8 +51,7 @@ vector<GPUCounter> D3D12Replay::EnumerateCounters()
 
   if(m_pAMDCounters)
   {
-    vector<GPUCounter> amdCounters = m_pAMDCounters->GetPublicCounterIds();
-    ret.insert(ret.end(), amdCounters.begin(), amdCounters.end());
+    ret.append(m_pAMDCounters->GetPublicCounterIds());
   }
 
   return ret;
@@ -191,7 +191,7 @@ CounterDescription D3D12Replay::DescribeCounter(GPUCounter counterID)
 struct D3D12AMDDrawCallback : public D3D12DrawcallCallback
 {
   D3D12AMDDrawCallback(WrappedID3D12Device *dev, D3D12Replay *rp, uint32_t &sampleIndex,
-                       vector<uint32_t> &eventIDs)
+                       rdcarray<uint32_t> &eventIDs)
       : m_pDevice(dev), m_pReplay(rp), m_pSampleId(&sampleIndex), m_pEventIds(&eventIDs)
   {
     m_pDevice->GetQueue()->GetCommandData()->m_DrawcallCallback = this;
@@ -202,7 +202,7 @@ struct D3D12AMDDrawCallback : public D3D12DrawcallCallback
     m_pDevice->GetQueue()->GetCommandData()->m_DrawcallCallback = NULL;
   }
 
-  void PreDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override
+  void PreDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
   {
     m_pEventIds->push_back(eid);
 
@@ -220,7 +220,7 @@ struct D3D12AMDDrawCallback : public D3D12DrawcallCallback
     ++*m_pSampleId;
   }
 
-  bool PostDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override
+  bool PostDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
   {
     WrappedID3D12GraphicsCommandList *pWrappedCmdList = (WrappedID3D12GraphicsCommandList *)cmd;
 
@@ -228,7 +228,7 @@ struct D3D12AMDDrawCallback : public D3D12DrawcallCallback
     return false;
   }
 
-  void PreCloseCommandList(ID3D12GraphicsCommandList4 *cmd) override
+  void PreCloseCommandList(ID3D12GraphicsCommandListX *cmd) override
   {
     WrappedID3D12GraphicsCommandList *pWrappedCmdList = (WrappedID3D12GraphicsCommandList *)cmd;
 
@@ -241,38 +241,38 @@ struct D3D12AMDDrawCallback : public D3D12DrawcallCallback
     }
   }
 
-  void PostRedraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override {}
+  void PostRedraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override {}
   // we don't need to distinguish, call the Draw functions
-  void PreDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override { PreDraw(eid, cmd); }
-  bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override
+  void PreDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override { PreDraw(eid, cmd); }
+  bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
   {
     return PostDraw(eid, cmd);
   }
-  void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override
+  void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
   {
     PostRedraw(eid, cmd);
   }
 
   void AliasEvent(uint32_t primary, uint32_t alias) override
   {
-    m_AliasEvents.push_back(std::make_pair(primary, alias));
+    m_AliasEvents.push_back(make_rdcpair(primary, alias));
   }
 
   uint32_t *m_pSampleId;
   WrappedID3D12Device *m_pDevice;
   D3D12Replay *m_pReplay;
-  vector<uint32_t> *m_pEventIds;
-  set<ID3D12GraphicsCommandList *> m_begunCommandLists;
+  rdcarray<uint32_t> *m_pEventIds;
+  std::set<ID3D12GraphicsCommandList *> m_begunCommandLists;
 
   // events which are the 'same' from being the same command buffer resubmitted
   // multiple times in the frame. We will only get the full callback when we're
   // recording the command buffer, and will be given the first EID. After that
   // we'll just be told which other EIDs alias this event.
-  vector<pair<uint32_t, uint32_t> > m_AliasEvents;
+  rdcarray<rdcpair<uint32_t, uint32_t> > m_AliasEvents;
 };
 
 void D3D12Replay::FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex,
-                                vector<uint32_t> *eventIDs)
+                                rdcarray<uint32_t> *eventIDs)
 {
   uint32_t maxEID = m_pDevice->GetQueue()->GetMaxEID();
 
@@ -282,13 +282,13 @@ void D3D12Replay::FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex,
   m_pDevice->ReplayLog(*eventStartID, maxEID, eReplay_Full);
 }
 
-vector<CounterResult> D3D12Replay::FetchCountersAMD(const vector<GPUCounter> &counters)
+rdcarray<CounterResult> D3D12Replay::FetchCountersAMD(const rdcarray<GPUCounter> &counters)
 {
   ID3D12Device *d3dDevice = m_pDevice->GetReal();
 
   if(!m_pAMDCounters->BeginMeasurementMode(AMDCounters::ApiType::Dx12, (void *)d3dDevice))
   {
-    return vector<CounterResult>();
+    return rdcarray<CounterResult>();
   }
 
   uint32_t sessionID = m_pAMDCounters->CreateSession();
@@ -309,7 +309,7 @@ vector<CounterResult> D3D12Replay::FetchCountersAMD(const vector<GPUCounter> &co
 
   uint32_t sampleIndex = 0;
 
-  vector<uint32_t> eventIDs;
+  rdcarray<uint32_t> eventIDs;
 
   for(uint32_t i = 0; i < passCount; i++)
   {
@@ -328,7 +328,7 @@ vector<CounterResult> D3D12Replay::FetchCountersAMD(const vector<GPUCounter> &co
 
   m_pAMDCounters->EndSesssion(sessionID);
 
-  vector<CounterResult> ret =
+  rdcarray<CounterResult> ret =
       m_pAMDCounters->GetCounterData(sessionID, sampleIndex, eventIDs, counters);
 
   for(size_t i = 0; i < m_pAMDDrawCallback->m_AliasEvents.size(); i++)
@@ -340,11 +340,11 @@ vector<CounterResult> D3D12Replay::FetchCountersAMD(const vector<GPUCounter> &co
       search.eventId = m_pAMDDrawCallback->m_AliasEvents[i].first;
 
       // find the result we're aliasing
-      auto it = std::find(ret.begin(), ret.end(), search);
-      if(it != ret.end())
+      int32_t idx = ret.indexOf(search);
+      if(idx >= 0)
       {
         // duplicate the result and append
-        CounterResult aliased = *it;
+        CounterResult aliased = ret[idx];
         aliased.eventId = m_pAMDDrawCallback->m_AliasEvents[i].second;
         ret.push_back(aliased);
       }
@@ -378,7 +378,7 @@ struct D3D12GPUTimerCallback : public D3D12DrawcallCallback
     m_pDevice->GetQueue()->GetCommandData()->m_DrawcallCallback = this;
   }
   ~D3D12GPUTimerCallback() { m_pDevice->GetQueue()->GetCommandData()->m_DrawcallCallback = NULL; }
-  void PreDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override
+  void PreDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
   {
     if(cmd->GetType() == D3D12_COMMAND_LIST_TYPE_DIRECT)
     {
@@ -388,7 +388,7 @@ struct D3D12GPUTimerCallback : public D3D12DrawcallCallback
     cmd->EndQuery(m_TimerQueryHeap, D3D12_QUERY_TYPE_TIMESTAMP, m_NumTimestampQueries * 2 + 0);
   }
 
-  bool PostDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override
+  bool PostDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
   {
     cmd->EndQuery(m_TimerQueryHeap, D3D12_QUERY_TYPE_TIMESTAMP, m_NumTimestampQueries * 2 + 1);
     m_NumTimestampQueries++;
@@ -401,25 +401,25 @@ struct D3D12GPUTimerCallback : public D3D12DrawcallCallback
 
       m_NumStatsQueries++;
     }
-    m_Results.push_back(std::make_pair(eid, direct));
+    m_Results.push_back(make_rdcpair(eid, direct));
     return false;
   }
 
-  void PostRedraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override {}
+  void PostRedraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override {}
   // we don't need to distinguish, call the Draw functions
-  void PreDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override { PreDraw(eid, cmd); }
-  bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override
+  void PreDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override { PreDraw(eid, cmd); }
+  bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
   {
     return PostDraw(eid, cmd);
   }
-  void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override
+  void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
   {
     PostRedraw(eid, cmd);
   }
-  void PreCloseCommandList(ID3D12GraphicsCommandList4 *cmd) override{};
+  void PreCloseCommandList(ID3D12GraphicsCommandListX *cmd) override{};
   void AliasEvent(uint32_t primary, uint32_t alias) override
   {
-    m_AliasEvents.push_back(std::make_pair(primary, alias));
+    m_AliasEvents.push_back(make_rdcpair(primary, alias));
   }
 
   WrappedID3D12Device *m_pDevice;
@@ -427,7 +427,7 @@ struct D3D12GPUTimerCallback : public D3D12DrawcallCallback
   ID3D12QueryHeap *m_TimerQueryHeap;
   ID3D12QueryHeap *m_PipeStatsQueryHeap;
   ID3D12QueryHeap *m_OcclusionQueryHeap;
-  vector<pair<uint32_t, bool> > m_Results;
+  rdcarray<rdcpair<uint32_t, bool> > m_Results;
 
   uint32_t m_NumStatsQueries;
   uint32_t m_NumTimestampQueries;
@@ -436,14 +436,14 @@ struct D3D12GPUTimerCallback : public D3D12DrawcallCallback
   // multiple times in the frame. We will only get the full callback when we're
   // recording the command buffer, and will be given the first EID. After that
   // we'll just be told which other EIDs alias this event.
-  vector<pair<uint32_t, uint32_t> > m_AliasEvents;
+  rdcarray<rdcpair<uint32_t, uint32_t> > m_AliasEvents;
 };
 
-vector<CounterResult> D3D12Replay::FetchCounters(const vector<GPUCounter> &counters)
+rdcarray<CounterResult> D3D12Replay::FetchCounters(const rdcarray<GPUCounter> &counters)
 {
   uint32_t maxEID = m_pDevice->GetQueue()->GetMaxEID();
 
-  vector<CounterResult> ret;
+  rdcarray<CounterResult> ret;
   if(counters.empty())
   {
     RDCERR("No counters specified to FetchCounters");
@@ -452,14 +452,14 @@ vector<CounterResult> D3D12Replay::FetchCounters(const vector<GPUCounter> &count
 
   SCOPED_TIMER("Fetch Counters, counters to fetch %u", counters.size());
 
-  vector<GPUCounter> d3dCounters;
+  rdcarray<GPUCounter> d3dCounters;
   std::copy_if(counters.begin(), counters.end(), std::back_inserter(d3dCounters),
                [](const GPUCounter &c) { return !IsAMDCounter(c); });
 
   if(m_pAMDCounters)
   {
     // Filter out the AMD counters
-    vector<GPUCounter> amdCounters;
+    rdcarray<GPUCounter> amdCounters;
     std::copy_if(counters.begin(), counters.end(), std::back_inserter(amdCounters),
                  [](const GPUCounter &c) { return IsAMDCounter(c); });
 
@@ -628,14 +628,14 @@ vector<CounterResult> D3D12Replay::FetchCounters(const vector<GPUCounter> &count
       occlusion++;
     }
 
-    for(size_t c = 0; c < counters.size(); c++)
+    for(size_t c = 0; c < d3dCounters.size(); c++)
     {
       CounterResult result;
 
       result.eventId = cb.m_Results[i].first;
-      result.counter = counters[c];
+      result.counter = d3dCounters[c];
 
-      switch(counters[c])
+      switch(d3dCounters[c])
       {
         case GPUCounter::EventGPUDuration:
         {
@@ -655,6 +655,8 @@ vector<CounterResult> D3D12Replay::FetchCounters(const vector<GPUCounter> &count
         case GPUCounter::GSInvocations: result.value.u64 = pipeStats.GSInvocations; break;
         case GPUCounter::PSInvocations: result.value.u64 = pipeStats.PSInvocations; break;
         case GPUCounter::CSInvocations: result.value.u64 = pipeStats.CSInvocations; break;
+
+        default: break;
       }
       ret.push_back(result);
     }
@@ -669,11 +671,11 @@ vector<CounterResult> D3D12Replay::FetchCounters(const vector<GPUCounter> &count
       search.eventId = cb.m_AliasEvents[i].first;
 
       // find the result we're aliasing
-      auto it = std::find(ret.begin(), ret.end(), search);
-      if(it != ret.end())
+      int32_t idx = ret.indexOf(search);
+      if(idx >= 0)
       {
         // duplicate the result and append
-        CounterResult aliased = *it;
+        CounterResult aliased = ret[idx];
         aliased.eventId = cb.m_AliasEvents[i].second;
         ret.push_back(aliased);
       }
