@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,7 @@
 #include "data/resource.h"
 #include "driver/dx/official/d3dcompiler.h"
 #include "driver/dxgi/dxgi_common.h"
-#include "maths/camera.h"
+#include "driver/shaders/dxbc/dxbc_bytecode.h"
 #include "maths/formatpacking.h"
 #include "maths/matrix.h"
 #include "maths/vec.h"
@@ -35,6 +35,7 @@
 #include "d3d12_command_list.h"
 #include "d3d12_command_queue.h"
 #include "d3d12_device.h"
+#include "d3d12_replay.h"
 #include "d3d12_shader_cache.h"
 
 #include "data/hlsl/hlsl_cbuffers.h"
@@ -255,8 +256,16 @@ D3D12DebugManager::D3D12DebugManager(WrappedID3D12Device *wrapper)
     rm->SetInternalResource(m_MeshRootSig);
   }
 
+  if(!CreateMathIntrinsicsResources())
   {
-    std::string meshhlsl = GetEmbeddedResource(mesh_hlsl);
+    RDCERR("Failed to create resources for shader debugging math intrinsics");
+    SAFE_RELEASE(m_MathIntrinsicsRootSig);
+    SAFE_RELEASE(m_MathIntrinsicsPso);
+    SAFE_RELEASE(m_MathIntrinsicsResultBuffer);
+  }
+
+  {
+    rdcstr meshhlsl = GetEmbeddedResource(mesh_hlsl);
 
     shaderCache->GetShaderBlob(meshhlsl.c_str(), "RENDERDOC_MeshVS", D3DCOMPILE_WARNINGS_ARE_ERRORS,
                                "vs_5_0", &m_MeshVS);
@@ -267,14 +276,14 @@ D3D12DebugManager::D3D12DebugManager(WrappedID3D12Device *wrapper)
   }
 
   {
-    std::string hlsl = GetEmbeddedResource(misc_hlsl);
+    rdcstr hlsl = GetEmbeddedResource(misc_hlsl);
 
     shaderCache->GetShaderBlob(hlsl.c_str(), "RENDERDOC_FullscreenVS",
                                D3DCOMPILE_WARNINGS_ARE_ERRORS, "vs_5_0", &m_FullscreenVS);
   }
 
   {
-    std::string multisamplehlsl = GetEmbeddedResource(multisample_hlsl);
+    rdcstr multisamplehlsl = GetEmbeddedResource(multisample_hlsl);
 
     shaderCache->GetShaderBlob(multisamplehlsl.c_str(), "RENDERDOC_CopyMSToArray",
                                D3DCOMPILE_WARNINGS_ARE_ERRORS, "ps_5_0", &m_IntMS2Array);
@@ -341,7 +350,7 @@ D3D12DebugManager::D3D12DebugManager(WrappedID3D12Device *wrapper)
   m_pDevice->InternalRef();
 
   // safe to upcast - this is a wrapped object
-  m_DebugList = (ID3D12GraphicsCommandList4 *)list;
+  m_DebugList = (ID3D12GraphicsCommandListX *)list;
 
   if(FAILED(hr))
   {
@@ -372,6 +381,10 @@ D3D12DebugManager::~D3D12DebugManager()
   SAFE_RELEASE(m_MeshPS);
   SAFE_RELEASE(m_MeshRootSig);
 
+  SAFE_RELEASE(m_MathIntrinsicsRootSig);
+  SAFE_RELEASE(m_MathIntrinsicsPso);
+  SAFE_RELEASE(m_MathIntrinsicsResultBuffer);
+
   SAFE_RELEASE(m_ArrayMSAARootSig);
   SAFE_RELEASE(m_FullscreenVS);
 
@@ -396,6 +409,145 @@ D3D12DebugManager::~D3D12DebugManager()
     RenderDoc::Inst().GetCrashHandler()->UnregisterMemoryRegion(this);
 }
 
+bool D3D12DebugManager::CreateMathIntrinsicsResources()
+{
+  rdcstr csProgram =
+      "RWStructuredBuffer<float4> outval : register(u0);\n"
+      "cbuffer srcOper : register(b0) { float4 inval; };\n"
+      "cbuffer srcInstr : register(b1) { uint operation; };\n";
+
+  // Assign constants to each supported instruction
+  csProgram += StringFormat::Fmt("static const uint OPCODE_RCP = %u;\n", DXBCBytecode::OPCODE_RCP);
+  csProgram += StringFormat::Fmt("static const uint OPCODE_RSQ = %u;\n", DXBCBytecode::OPCODE_RSQ);
+  csProgram += StringFormat::Fmt("static const uint OPCODE_EXP = %u;\n", DXBCBytecode::OPCODE_EXP);
+  csProgram += StringFormat::Fmt("static const uint OPCODE_LOG = %u;\n", DXBCBytecode::OPCODE_LOG);
+  csProgram +=
+      StringFormat::Fmt("static const uint OPCODE_SINCOS = %u;\n", DXBCBytecode::OPCODE_SINCOS);
+
+  csProgram +=
+      "[numthreads(1, 1, 1)]\n"
+      "void main(){\n"
+      "  switch(operation){\n"
+      "    case OPCODE_RCP: outval[0] = rcp(inval); break;\n"
+      "    case OPCODE_RSQ: outval[0] = rsqrt(inval); break;\n"
+      "    case OPCODE_EXP: outval[0] = exp2(inval); break;\n"
+      "    case OPCODE_LOG: outval[0] = log2(inval); break;\n"
+      "    case OPCODE_SINCOS: sincos(inval, outval[0], outval[1]); break;\n"
+      "  }\n}\n";
+
+  ID3DBlob *csBlob = NULL;
+  UINT flags = D3DCOMPILE_DEBUG | D3DCOMPILE_WARNINGS_ARE_ERRORS;
+  if(m_pDevice->GetShaderCache()->GetShaderBlob(csProgram.c_str(), "main", flags, "cs_5_0",
+                                                &csBlob) != "")
+  {
+    RDCERR("Failed to create shader to calculate math intrinsic");
+    return false;
+  }
+
+  D3D12RootSignature rootSig;
+
+  // Constants param for the operation inputs
+  D3D12RootSignatureParameter constantParam;
+  constantParam.ParameterType = D3D12_ROOT_PARAMETER_TYPE_32BIT_CONSTANTS;
+  constantParam.ShaderVisibility = D3D12_SHADER_VISIBILITY_ALL;
+  constantParam.Constants.Num32BitValues = 4;    // Input is 4 floats
+  constantParam.Constants.ShaderRegister = 0;
+  constantParam.Constants.RegisterSpace = 0;
+  rootSig.Parameters.push_back(constantParam);
+
+  // Constants param for the opcode
+  constantParam.Constants.Num32BitValues = 1;
+  constantParam.Constants.ShaderRegister = 1;
+  constantParam.Constants.RegisterSpace = 0;
+  rootSig.Parameters.push_back(constantParam);
+
+  // UAV param for the output
+  D3D12RootSignatureParameter uavParam;
+  uavParam.ParameterType = D3D12_ROOT_PARAMETER_TYPE_UAV;
+  uavParam.ShaderVisibility = D3D12_SHADER_VISIBILITY_ALL;
+  uavParam.Descriptor.ShaderRegister = 0;
+  uavParam.Descriptor.RegisterSpace = 0;
+  uavParam.Descriptor.Flags = D3D12_ROOT_DESCRIPTOR_FLAG_NONE;
+  rootSig.Parameters.push_back(uavParam);
+
+  ID3DBlob *root = m_pDevice->GetShaderCache()->MakeRootSig(rootSig);
+  if(root == NULL)
+  {
+    RDCERR("Failed to create root signature for shader debugging");
+    SAFE_RELEASE(csBlob);
+    return false;
+  }
+  HRESULT hr = m_pDevice->CreateRootSignature(0, root->GetBufferPointer(), root->GetBufferSize(),
+                                              __uuidof(ID3D12RootSignature),
+                                              (void **)&m_MathIntrinsicsRootSig);
+  m_pDevice->InternalRef();
+  SAFE_RELEASE(root);
+  if(FAILED(hr))
+  {
+    RDCERR("Failed to create root signature for shader debugging HRESULT: %s", ToStr(hr).c_str());
+    SAFE_RELEASE(csBlob);
+    return false;
+  }
+  m_pDevice->GetResourceManager()->SetInternalResource(m_MathIntrinsicsRootSig);
+
+  D3D12_COMPUTE_PIPELINE_STATE_DESC psoDesc;
+  psoDesc.pRootSignature = m_MathIntrinsicsRootSig;
+  psoDesc.CS.BytecodeLength = csBlob->GetBufferSize();
+  psoDesc.CS.pShaderBytecode = csBlob->GetBufferPointer();
+  psoDesc.NodeMask = 0;
+  psoDesc.CachedPSO.pCachedBlob = NULL;
+  psoDesc.CachedPSO.CachedBlobSizeInBytes = 0;
+  psoDesc.Flags = D3D12_PIPELINE_STATE_FLAG_NONE;
+
+  hr = m_pDevice->CreateComputePipelineState(&psoDesc, __uuidof(ID3D12PipelineState),
+                                             (void **)&m_MathIntrinsicsPso);
+  m_pDevice->InternalRef();
+  SAFE_RELEASE(csBlob);
+  if(FAILED(hr))
+  {
+    RDCERR("Failed to create PSO for shader debugging HRESULT: %s", ToStr(hr).c_str());
+    SAFE_RELEASE(m_MathIntrinsicsRootSig);
+    return false;
+  }
+  m_pDevice->GetResourceManager()->SetInternalResource(m_MathIntrinsicsPso);
+
+  // Create buffer to store computed result
+  D3D12_RESOURCE_DESC rdesc;
+  ZeroMemory(&rdesc, sizeof(D3D12_RESOURCE_DESC));
+  rdesc.Dimension = D3D12_RESOURCE_DIMENSION_BUFFER;
+  rdesc.Width = sizeof(float) * 8;    // Output buffer is 2x float4
+  rdesc.Height = 1;
+  rdesc.DepthOrArraySize = 1;
+  rdesc.MipLevels = 1;
+  rdesc.Format = DXGI_FORMAT_UNKNOWN;
+  rdesc.Flags = D3D12_RESOURCE_FLAG_ALLOW_UNORDERED_ACCESS;
+  rdesc.Layout = D3D12_TEXTURE_LAYOUT_ROW_MAJOR;
+  rdesc.SampleDesc.Count = 1;
+  rdesc.SampleDesc.Quality = 0;
+
+  D3D12_HEAP_PROPERTIES heapProps;
+  heapProps.Type = D3D12_HEAP_TYPE_DEFAULT;
+  heapProps.CPUPageProperty = D3D12_CPU_PAGE_PROPERTY_UNKNOWN;
+  heapProps.MemoryPoolPreference = D3D12_MEMORY_POOL_UNKNOWN;
+  heapProps.CreationNodeMask = 1;
+  heapProps.VisibleNodeMask = 1;
+
+  hr = m_pDevice->CreateCommittedResource(
+      &heapProps, D3D12_HEAP_FLAG_NONE, &rdesc, D3D12_RESOURCE_STATE_UNORDERED_ACCESS, NULL,
+      __uuidof(ID3D12Resource), (void **)&m_MathIntrinsicsResultBuffer);
+  m_pDevice->InternalRef();
+  if(FAILED(hr))
+  {
+    RDCERR("Failed to create buffer for pixel shader debugging HRESULT: %s", ToStr(hr).c_str());
+    SAFE_RELEASE(m_MathIntrinsicsRootSig);
+    SAFE_RELEASE(m_MathIntrinsicsPso);
+    return false;
+  }
+  m_pDevice->GetResourceManager()->SetInternalResource(m_MathIntrinsicsResultBuffer);
+
+  return true;
+}
+
 ID3D12Resource *D3D12DebugManager::MakeCBuffer(UINT64 size)
 {
   ID3D12Resource *ret;
@@ -471,7 +623,7 @@ D3D12_GPU_VIRTUAL_ADDRESS D3D12DebugManager::UploadConstants(const void *data, s
   return ret;
 }
 
-ID3D12GraphicsCommandList4 *D3D12DebugManager::ResetDebugList()
+ID3D12GraphicsCommandListX *D3D12DebugManager::ResetDebugList()
 {
   m_DebugList->Reset(m_DebugAlloc, NULL);
 
@@ -828,7 +980,7 @@ void D3D12Replay::GeneralMisc::Init(WrappedID3D12Device *device, D3D12DebugManag
   }
 
   {
-    std::string hlsl = GetEmbeddedResource(misc_hlsl);
+    rdcstr hlsl = GetEmbeddedResource(misc_hlsl);
 
     ID3DBlob *FullscreenVS = NULL;
     ID3DBlob *CheckerboardPS = NULL;
@@ -961,7 +1113,7 @@ void D3D12Replay::TextureRendering::Init(WrappedID3D12Device *device, D3D12Debug
   }
 
   {
-    std::string hlsl = GetEmbeddedResource(texdisplay_hlsl);
+    rdcstr hlsl = GetEmbeddedResource(texdisplay_hlsl);
 
     ID3DBlob *TexDisplayPS = NULL;
 
@@ -1046,6 +1198,52 @@ void D3D12Replay::TextureRendering::Init(WrappedID3D12Device *device, D3D12Debug
     }
 
     SAFE_RELEASE(TexDisplayPS);
+
+    hlsl = GetEmbeddedResource(texremap_hlsl);
+
+    ID3DBlob *TexRemap[3] = {};
+    DXGI_FORMAT formats[3] = {
+        DXGI_FORMAT_R8G8B8A8_TYPELESS, DXGI_FORMAT_R16G16B16A16_TYPELESS,
+        DXGI_FORMAT_R32G32B32A32_TYPELESS,
+    };
+
+    shaderCache->GetShaderBlob(hlsl.c_str(), "RENDERDOC_TexRemapFloat",
+                               D3DCOMPILE_WARNINGS_ARE_ERRORS, "ps_5_0", &TexRemap[0]);
+    RDCASSERT(TexRemap[0]);
+    shaderCache->GetShaderBlob(hlsl.c_str(), "RENDERDOC_TexRemapUInt",
+                               D3DCOMPILE_WARNINGS_ARE_ERRORS, "ps_5_0", &TexRemap[1]);
+    RDCASSERT(TexRemap[1]);
+    shaderCache->GetShaderBlob(hlsl.c_str(), "RENDERDOC_TexRemapSInt",
+                               D3DCOMPILE_WARNINGS_ARE_ERRORS, "ps_5_0", &TexRemap[2]);
+    RDCASSERT(TexRemap[2]);
+
+    for(int f = 0; f < 3; f++)
+    {
+      for(int i = 0; i < 3; i++)
+      {
+        pipeDesc.PS.BytecodeLength = TexRemap[i]->GetBufferSize();
+        pipeDesc.PS.pShaderBytecode = TexRemap[i]->GetBufferPointer();
+
+        if(i == 0)
+          pipeDesc.RTVFormats[0] = GetFloatTypedFormat(formats[f]);
+        else if(i == 1)
+          pipeDesc.RTVFormats[0] = GetUIntTypedFormat(formats[f]);
+        else
+          pipeDesc.RTVFormats[0] = GetSIntTypedFormat(formats[f]);
+
+        hr = device->CreateGraphicsPipelineState(&pipeDesc, __uuidof(ID3D12PipelineState),
+                                                 (void **)&m_TexRemapPipe[f][i]);
+
+        if(FAILED(hr))
+        {
+          RDCERR("Couldn't create m_TexRemapPipe for %s! HRESULT: %s",
+                 ToStr(pipeDesc.RTVFormats[0]).c_str(), ToStr(hr).c_str());
+        }
+      }
+    }
+
+    for(int i = 0; i < 3; i++)
+      SAFE_RELEASE(TexRemap[i]);
   }
 
   shaderCache->SetCaching(false);
@@ -1060,6 +1258,13 @@ void D3D12Replay::TextureRendering::Release()
   SAFE_RELEASE(F32Pipe);
   SAFE_RELEASE(RootSig);
   SAFE_RELEASE(VS);
+  for(int f = 0; f < 3; f++)
+  {
+    for(int i = 0; i < 3; i++)
+    {
+      SAFE_RELEASE(m_TexRemapPipe[f][i]);
+    }
+  }
 }
 
 void D3D12Replay::OverlayRendering::Init(WrappedID3D12Device *device, D3D12DebugManager *debug)
@@ -1071,7 +1276,7 @@ void D3D12Replay::OverlayRendering::Init(WrappedID3D12Device *device, D3D12Debug
   shaderCache->SetCaching(true);
 
   {
-    std::string meshhlsl = GetEmbeddedResource(mesh_hlsl);
+    rdcstr meshhlsl = GetEmbeddedResource(mesh_hlsl);
 
     shaderCache->GetShaderBlob(meshhlsl.c_str(), "RENDERDOC_TriangleSizeGS",
                                D3DCOMPILE_WARNINGS_ARE_ERRORS, "gs_5_0", &TriangleSizeGS);
@@ -1081,7 +1286,7 @@ void D3D12Replay::OverlayRendering::Init(WrappedID3D12Device *device, D3D12Debug
     shaderCache->GetShaderBlob(meshhlsl.c_str(), "RENDERDOC_MeshVS", D3DCOMPILE_WARNINGS_ARE_ERRORS,
                                "vs_5_0", &MeshVS);
 
-    std::string hlsl = GetEmbeddedResource(quadoverdraw_hlsl);
+    rdcstr hlsl = GetEmbeddedResource(quadoverdraw_hlsl);
 
     shaderCache->GetShaderBlob(hlsl.c_str(), "RENDERDOC_QuadOverdrawPS",
                                D3DCOMPILE_WARNINGS_ARE_ERRORS, "ps_5_0", &QuadOverdrawWritePS);
@@ -1102,7 +1307,7 @@ void D3D12Replay::OverlayRendering::Init(WrappedID3D12Device *device, D3D12Debug
   }
 
   {
-    std::string hlsl = GetEmbeddedResource(misc_hlsl);
+    rdcstr hlsl = GetEmbeddedResource(misc_hlsl);
 
     ID3DBlob *FullscreenVS = NULL;
     shaderCache->GetShaderBlob(hlsl.c_str(), "RENDERDOC_FullscreenVS",
@@ -1194,7 +1399,7 @@ void D3D12Replay::VertexPicking::Init(WrappedID3D12Device *device, D3D12DebugMan
   }
 
   {
-    std::string meshhlsl = GetEmbeddedResource(mesh_hlsl);
+    rdcstr meshhlsl = GetEmbeddedResource(mesh_hlsl);
 
     ID3DBlob *meshPickCS;
 
@@ -1367,7 +1572,7 @@ void D3D12Replay::HistogramMinMax::Init(WrappedID3D12Device *device, D3D12DebugM
   }
 
   {
-    std::string histogramhlsl = GetEmbeddedResource(histogram_hlsl);
+    rdcstr histogramhlsl = GetEmbeddedResource(histogram_hlsl);
 
     D3D12_COMPUTE_PIPELINE_STATE_DESC compPipeDesc = {};
 
@@ -1386,9 +1591,9 @@ void D3D12Replay::HistogramMinMax::Init(WrappedID3D12Device *device, D3D12DebugM
         ID3DBlob *result = NULL;
         ID3DBlob *histogram = NULL;
 
-        string hlsl = string("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
-        hlsl += string("#define UINT_TEX ") + (i == 1 ? "1" : "0") + "\n";
-        hlsl += string("#define SINT_TEX ") + (i == 2 ? "1" : "0") + "\n";
+        rdcstr hlsl = rdcstr("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
+        hlsl += rdcstr("#define UINT_TEX ") + (i == 1 ? "1" : "0") + "\n";
+        hlsl += rdcstr("#define SINT_TEX ") + (i == 2 ? "1" : "0") + "\n";
         hlsl += histogramhlsl;
 
         shaderCache->GetShaderBlob(hlsl.c_str(), "RENDERDOC_TileMinMaxCS",
@@ -1579,3 +1784,110 @@ void D3D12Replay::HistogramMinMax::Release()
   SAFE_RELEASE(MinMaxResultBuffer);
   SAFE_RELEASE(MinMaxTileBuffer);
 }
+
+void MoveRootSignatureElementsToRegisterSpace(D3D12RootSignature &sig, uint32_t registerSpace,
+                                              D3D12DescriptorType type,
+                                              D3D12_SHADER_VISIBILITY visibility)
+{
+  // This function is used when root signature elements need to be added to a specific register
+  // space, such as for debug overlays. We can't remove elements from the root signature entirely
+  // because then then the root signature indices wouldn't match up as expected. Instead move them
+  // into the specified register space so that another register space (commonly space0) can be used
+  // for other purposes.
+  size_t numParams = sig.Parameters.size();
+  for(size_t i = 0; i < numParams; i++)
+  {
+    if(sig.Parameters[i].ShaderVisibility == visibility ||
+       sig.Parameters[i].ShaderVisibility == D3D12_SHADER_VISIBILITY_ALL)
+    {
+      D3D12_ROOT_PARAMETER_TYPE rootType = sig.Parameters[i].ParameterType;
+      if(rootType == D3D12_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE)
+      {
+        size_t numRanges = sig.Parameters[i].ranges.size();
+        for(size_t r = 0; r < numRanges; r++)
+        {
+          D3D12_DESCRIPTOR_RANGE_TYPE rangeType = sig.Parameters[i].ranges[r].RangeType;
+          if(rangeType == D3D12_DESCRIPTOR_RANGE_TYPE_CBV && type == D3D12DescriptorType::CBV)
+          {
+            sig.Parameters[i].ranges[r].RegisterSpace = registerSpace;
+          }
+          else if(rangeType == D3D12_DESCRIPTOR_RANGE_TYPE_SRV && type == D3D12DescriptorType::SRV)
+          {
+            sig.Parameters[i].ranges[r].RegisterSpace = registerSpace;
+          }
+          else if(rangeType == D3D12_DESCRIPTOR_RANGE_TYPE_UAV && type == D3D12DescriptorType::UAV)
+          {
+            sig.Parameters[i].ranges[r].RegisterSpace = registerSpace;
+          }
+        }
+      }
+      else if(rootType == D3D12_ROOT_PARAMETER_TYPE_CBV && type == D3D12DescriptorType::CBV)
+      {
+        sig.Parameters[i].Descriptor.RegisterSpace = registerSpace;
+      }
+      else if(rootType == D3D12_ROOT_PARAMETER_TYPE_SRV && type == D3D12DescriptorType::SRV)
+      {
+        sig.Parameters[i].Descriptor.RegisterSpace = registerSpace;
+      }
+      else if(rootType == D3D12_ROOT_PARAMETER_TYPE_UAV && type == D3D12DescriptorType::UAV)
+      {
+        sig.Parameters[i].Descriptor.RegisterSpace = registerSpace;
+      }
+    }
+  }
+}
+
+void AddDebugDescriptorToRenderState(WrappedID3D12Device *pDevice, D3D12RenderState &rs,
+                                     const PortableHandle &handle,
+                                     D3D12_DESCRIPTOR_HEAP_TYPE heapType, uint32_t sigElem,
+                                     std::set<ResourceId> &copiedHeaps)
+{
+  if(rs.graphics.sigelems.size() <= sigElem)
+    rs.graphics.sigelems.resize(sigElem + 1);
+
+  PortableHandle newHandle = handle;
+
+  // If a CBV_SRV_UAV heap is already set and hasn't had a debug descriptor copied in,
+  // copy the desired descriptor in and add the heap to the set of heaps that have had
+  // a debug descriptor set. If there's no available heapOtherwise we can set our own heap.
+
+  // It is the responsibility of the caller to keep track of the set of copied heaps to
+  // avoid overwriting another debug descriptor that may be needed.
+
+  for(size_t i = 0; i < rs.heaps.size(); i++)
+  {
+    WrappedID3D12DescriptorHeap *h =
+        pDevice->GetResourceManager()->GetCurrentAs<WrappedID3D12DescriptorHeap>(rs.heaps[i]);
+    if(h->GetDesc().Type == heapType)
+    {
+      // use the last descriptor
+      D3D12_CPU_DESCRIPTOR_HANDLE dst = h->GetCPUDescriptorHandleForHeapStart();
+      dst.ptr += (h->GetDesc().NumDescriptors - 1) * sizeof(D3D12Descriptor);
+
+      if(copiedHeaps.find(rs.heaps[i]) == copiedHeaps.end())
+      {
+        WrappedID3D12DescriptorHeap *h2 =
+            pDevice->GetResourceManager()->GetCurrentAs<WrappedID3D12DescriptorHeap>(handle.heap);
+        D3D12_CPU_DESCRIPTOR_HANDLE src = h2->GetCPUDescriptorHandleForHeapStart();
+        src.ptr += handle.index * sizeof(D3D12Descriptor);
+
+        // can't do a copy because the src heap is CPU write-only (shader visible). So instead,
+        // create directly
+        D3D12Descriptor *srcDesc = (D3D12Descriptor *)src.ptr;
+        srcDesc->Create(heapType, pDevice, dst);
+
+        copiedHeaps.insert(rs.heaps[i]);
+      }
+
+      newHandle = ToPortableHandle(dst);
+
+      break;
+    }
+  }
+
+  if(newHandle.heap == handle.heap)
+    rs.heaps.push_back(handle.heap);
+
+  rs.graphics.sigelems[sigElem] =
+      D3D12RenderState::SignatureElement(eRootTable, newHandle.heap, newHandle.index);
+}
