@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,6 @@
 
 #include <stdint.h>
 #include <map>
-#include "api/replay/renderdoc_replay.h"
 #include "common/threading.h"
 #include "common/timing.h"
 #include "common/wrapped_pool.h"
@@ -35,18 +34,17 @@
 #include "replay/replay_driver.h"
 #include "d3d12_common.h"
 #include "d3d12_manager.h"
-#include "d3d12_replay.h"
 
 struct IAmdExtD3DFactory;
 
 struct D3D12InitParams
 {
-  D3D12InitParams();
+  D3D_FEATURE_LEVEL MinimumFeatureLevel = D3D_FEATURE_LEVEL_11_0;
 
-  D3D_FEATURE_LEVEL MinimumFeatureLevel;
+  DXGI_ADAPTER_DESC AdapterDesc = {};
 
   // check if a frame capture section version is supported
-  static const uint64_t CurrentVersion = 0x6;
+  static const uint64_t CurrentVersion = 0x8;
 
   static bool IsSupportedVersion(uint64_t ver);
 };
@@ -55,7 +53,9 @@ DECLARE_REFLECTION_STRUCT(D3D12InitParams);
 
 class WrappedID3D12Device;
 class WrappedID3D12Resource1;
+class WrappedID3D12PipelineState;
 
+class D3D12Replay;
 class D3D12TextRenderer;
 class D3D12ShaderCache;
 class D3D12DebugManager;
@@ -268,6 +268,76 @@ struct DummyID3D12DebugDevice : public ID3D12DebugDevice2, public ID3D12DebugDev
   }
 };
 
+struct WrappedDownlevelDevice : public ID3D12DeviceDownlevel
+{
+  WrappedID3D12Device &m_pDevice;
+
+  WrappedDownlevelDevice(WrappedID3D12Device &dev) : m_pDevice(dev) {}
+  //////////////////////////////
+  // implement IUnknown
+  HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);
+  ULONG STDMETHODCALLTYPE AddRef();
+  ULONG STDMETHODCALLTYPE Release();
+  //////////////////////////////
+  // implement ID3D12DeviceDownlevel
+  virtual HRESULT STDMETHODCALLTYPE
+  QueryVideoMemoryInfo(UINT NodeIndex, DXGI_MEMORY_SEGMENT_GROUP MemorySegmentGroup,
+                       _Out_ DXGI_QUERY_VIDEO_MEMORY_INFO *pVideoMemoryInfo);
+};
+
+struct WrappedDRED : public ID3D12DeviceRemovedExtendedData
+{
+  WrappedID3D12Device &m_pDevice;
+  ID3D12DeviceRemovedExtendedData *m_pReal = NULL;
+
+  WrappedDRED(WrappedID3D12Device &dev) : m_pDevice(dev) {}
+  //////////////////////////////
+  // implement IUnknown
+  HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);
+  ULONG STDMETHODCALLTYPE AddRef();
+  ULONG STDMETHODCALLTYPE Release();
+
+  //////////////////////////////
+  // implement ID3D12DeviceRemovedExtendedDataSettings
+  virtual HRESULT STDMETHODCALLTYPE GetAutoBreadcrumbsOutput(D3D12_DRED_AUTO_BREADCRUMBS_OUTPUT *pOutput)
+  {
+    return m_pReal->GetAutoBreadcrumbsOutput(pOutput);
+  }
+
+  virtual HRESULT STDMETHODCALLTYPE GetPageFaultAllocationOutput(D3D12_DRED_PAGE_FAULT_OUTPUT *pOutput)
+  {
+    return m_pReal->GetPageFaultAllocationOutput(pOutput);
+  }
+};
+
+struct WrappedDREDSettings : public ID3D12DeviceRemovedExtendedDataSettings
+{
+  WrappedID3D12Device &m_pDevice;
+  ID3D12DeviceRemovedExtendedDataSettings *m_pReal = NULL;
+
+  WrappedDREDSettings(WrappedID3D12Device &dev) : m_pDevice(dev) {}
+  //////////////////////////////
+  // implement IUnknown
+  HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);
+  ULONG STDMETHODCALLTYPE AddRef();
+  ULONG STDMETHODCALLTYPE Release();
+
+  //////////////////////////////
+  // implement ID3D12DeviceRemovedExtendedDataSettings
+  virtual void STDMETHODCALLTYPE SetAutoBreadcrumbsEnablement(D3D12_DRED_ENABLEMENT setting)
+  {
+    m_pReal->SetAutoBreadcrumbsEnablement(setting);
+  }
+  virtual void STDMETHODCALLTYPE SetPageFaultEnablement(D3D12_DRED_ENABLEMENT setting)
+  {
+    m_pReal->SetPageFaultEnablement(setting);
+  }
+  virtual void STDMETHODCALLTYPE SetWatsonDumpEnablement(D3D12_DRED_ENABLEMENT setting)
+  {
+    m_pReal->SetWatsonDumpEnablement(setting);
+  }
+};
+
 class WrappedID3D12CommandQueue;
 
 #define IMPLEMENT_FUNCTION_THREAD_SERIALISED(ret, func, ...) \
@@ -275,7 +345,7 @@ class WrappedID3D12CommandQueue;
   template <typename SerialiserType>                         \
   bool CONCAT(Serialise_, func(SerialiserType &ser, __VA_ARGS__));
 
-class WrappedID3D12Device : public IFrameCapturer, public ID3DDevice, public ID3D12Device5
+class WrappedID3D12Device : public IFrameCapturer, public ID3DDevice, public ID3D12Device6
 {
 private:
   ID3D12Device *m_pDevice;
@@ -284,10 +354,12 @@ private:
   ID3D12Device3 *m_pDevice3;
   ID3D12Device4 *m_pDevice4;
   ID3D12Device5 *m_pDevice5;
+  ID3D12Device6 *m_pDevice6;
+  ID3D12DeviceDownlevel *m_pDownlevel;
 
   // list of all queues being captured
-  std::vector<WrappedID3D12CommandQueue *> m_Queues;
-  std::vector<ID3D12Fence *> m_QueueFences;
+  rdcarray<WrappedID3D12CommandQueue *> m_Queues;
+  rdcarray<ID3D12Fence *> m_QueueFences;
 
   // the queue we use for all internal work, the first DIRECT queue
   WrappedID3D12CommandQueue *m_Queue;
@@ -299,13 +371,17 @@ private:
   HANDLE m_GPUSyncHandle;
   UINT64 m_GPUSyncCounter;
 
-  std::vector<ID3D12CommandAllocator *> m_CommandAllocators;
+  WrappedDownlevelDevice m_WrappedDownlevel;
+  WrappedDRED m_DRED;
+  WrappedDREDSettings m_DREDSettings;
+
+  rdcarray<ID3D12CommandAllocator *> m_CommandAllocators;
 
   D3D12_CPU_DESCRIPTOR_HANDLE AllocRTV();
   void FreeRTV(D3D12_CPU_DESCRIPTOR_HANDLE handle);
 
-  std::vector<D3D12_CPU_DESCRIPTOR_HANDLE> m_FreeRTVs;
-  std::vector<D3D12_CPU_DESCRIPTOR_HANDLE> m_UsedRTVs;
+  rdcarray<D3D12_CPU_DESCRIPTOR_HANDLE> m_FreeRTVs;
+  rdcarray<D3D12_CPU_DESCRIPTOR_HANDLE> m_UsedRTVs;
 
   void CreateInternalResources();
   void DestroyInternalResources();
@@ -317,15 +393,15 @@ private:
   DummyID3D12DebugDevice m_DummyDebug;
   WrappedID3D12DebugDevice m_WrappedDebug;
 
-  D3D12Replay m_Replay;
+  D3D12Replay *m_Replay;
   D3D12ShaderCache *m_ShaderCache = NULL;
   D3D12TextRenderer *m_TextRenderer = NULL;
 
-  set<ResourceId> m_UploadResourceIds;
-  map<uint64_t, ID3D12Resource *> m_UploadBuffers;
+  std::set<ResourceId> m_UploadResourceIds;
+  std::map<uint64_t, ID3D12Resource *> m_UploadBuffers;
 
   Threading::CriticalSection m_MapsLock;
-  vector<MapState> m_Maps;
+  rdcarray<MapState> m_Maps;
 
   bool ProcessChunk(ReadSerialiser &ser, D3D12Chunk context);
 
@@ -337,7 +413,7 @@ private:
   uint64_t threadSerialiserTLSSlot;
 
   Threading::CriticalSection m_ThreadSerialisersLock;
-  std::vector<WriteSerialiser *> m_ThreadSerialisers;
+  rdcarray<WriteSerialiser *> m_ThreadSerialisers;
 
   uint64_t tempMemoryTLSSlot;
   struct TempMem
@@ -347,21 +423,22 @@ private:
     size_t size;
   };
   Threading::CriticalSection m_ThreadTempMemLock;
-  vector<TempMem *> m_ThreadTempMem;
+  rdcarray<TempMem *> m_ThreadTempMem;
 
-  vector<DebugMessage> m_DebugMessages;
+  rdcarray<DebugMessage> m_DebugMessages;
 
   SDFile *m_StructuredFile = NULL;
   SDFile m_StoredStructuredData;
 
-  uint32_t m_FrameCounter;
-  vector<FrameDescription> m_CapturedFrames;
-  FrameRecord m_FrameRecord;
-  vector<DrawcallDescription *> m_Drawcalls;
+  uint32_t m_FrameCounter = 0;
+  rdcarray<FrameDescription> m_CapturedFrames;
+  rdcarray<DrawcallDescription *> m_Drawcalls;
 
   ReplayStatus m_FailedReplayStatus = ReplayStatus::APIReplayFailed;
 
-  bool m_AppControlledCapture;
+  bool m_AppControlledCapture = false;
+
+  bool m_InvalidPSO = false;
 
   Threading::RWLock m_CapTransitionLock;
   CaptureState m_State;
@@ -370,20 +447,21 @@ private:
 
   D3D12InitParams m_InitParams;
   uint64_t m_SectionVersion;
+  ReplayOptions m_ReplayOptions;
   ID3D12InfoQueue *m_pInfoQueue;
 
   D3D12ResourceRecord *m_FrameCaptureRecord;
   Chunk *m_HeaderChunk;
 
-  std::set<std::string> m_StringDB;
+  std::set<rdcstr> m_StringDB;
 
   ResourceId m_ResourceID;
   D3D12ResourceRecord *m_DeviceRecord;
 
   Threading::CriticalSection m_DynDescLock;
-  std::vector<DynamicDescriptorCopy> m_DynamicDescriptorCopies;
-  std::vector<DynamicDescriptorWrite> m_DynamicDescriptorWrites;
-  std::vector<D3D12Descriptor> m_DynamicDescriptorRefs;
+  rdcarray<DynamicDescriptorCopy> m_DynamicDescriptorCopies;
+  rdcarray<DynamicDescriptorWrite> m_DynamicDescriptorWrites;
+  rdcarray<D3D12Descriptor> m_DynamicDescriptorRefs;
 
   GPUAddressRangeTracker m_GPUAddresses;
 
@@ -391,26 +469,28 @@ private:
 
   // used both on capture and replay side to track resource states. Only locked
   // in capture
-  map<ResourceId, SubresourceStateVector> m_ResourceStates;
+  std::map<ResourceId, SubresourceStateVector> m_ResourceStates;
   Threading::CriticalSection m_ResourceStatesLock;
 
-  set<ResourceId> m_Cubemaps;
+  std::set<ResourceId> m_Cubemaps;
 
-  map<ResourceId, string> m_ResourceNames;
+  std::map<ResourceId, rdcstr> m_ResourceNames;
+
+  // only valid on replay
+  std::map<ResourceId, WrappedID3D12Resource1 *> *m_ResourceList = NULL;
+  rdcarray<WrappedID3D12PipelineState *> *m_PipelineList = NULL;
 
   struct SwapPresentInfo
   {
-    D3D12_CPU_DESCRIPTOR_HANDLE rtvs[8];
+    D3D12_CPU_DESCRIPTOR_HANDLE rtvs[8] = {};
 
     WrappedID3D12CommandQueue *queue;
-
-    int32_t lastPresentedBuffer;
   };
 
-  map<WrappedIDXGISwapChain4 *, SwapPresentInfo> m_SwapChains;
-  map<ResourceId, DXGI_FORMAT> m_BackbufferFormat;
+  std::map<IDXGISwapper *, SwapPresentInfo> m_SwapChains;
+  std::map<ResourceId, DXGI_FORMAT> m_BackbufferFormat;
 
-  WrappedIDXGISwapChain4 *m_LastSwap;
+  IDXGISwapper *m_LastSwap = NULL;
 
   D3D12_FEATURE_DATA_D3D12_OPTIONS m_D3D12Opts;
   D3D12_FEATURE_DATA_D3D12_OPTIONS1 m_D3D12Opts1;
@@ -420,10 +500,13 @@ private:
 
   template <typename SerialiserType>
   bool Serialise_CaptureScope(SerialiserType &ser);
-  void EndCaptureFrame(ID3D12Resource *presentImage);
+  void EndCaptureFrame();
 
   bool m_debugLayerEnabled;
 
+  static Threading::CriticalSection m_DeviceWrappersLock;
+  static std::map<ID3D12Device *, WrappedID3D12Device *> m_DeviceWrappers;
+
 public:
   static const int AllocPoolCount = 4;
   ALLOCATE_WITH_WRAPPED_POOL(WrappedID3D12Device, AllocPoolCount);
@@ -432,6 +515,9 @@ public:
   bool IsDebugLayerEnabled() const { return m_debugLayerEnabled; }
   virtual ~WrappedID3D12Device();
 
+  static WrappedID3D12Device *Create(ID3D12Device *realDevice, D3D12InitParams params,
+                                     bool enabledDebugLayer);
+
   UINT GetUnwrappedDescriptorIncrement(D3D12_DESCRIPTOR_HEAP_TYPE type)
   {
     return m_DescriptorIncrements[type];
@@ -443,6 +529,9 @@ public:
   const D3D12_FEATURE_DATA_D3D12_OPTIONS3 &GetOpts3() { return m_D3D12Opts3; }
   void RemoveQueue(WrappedID3D12CommandQueue *queue);
 
+  // only valid on replay
+  std::map<ResourceId, WrappedID3D12Resource1 *> &GetResourceList() { return *m_ResourceList; }
+  rdcarray<WrappedID3D12PipelineState *> &GetPipelineList() { return *m_PipelineList; }
   ////////////////////////////////////////////////////////////////
   // non wrapping interface
 
@@ -451,49 +540,54 @@ public:
   WriteSerialiser &GetThreadSerialiser();
 
   ID3D12Device *GetReal() { return m_pDevice; }
-  static std::string GetChunkName(uint32_t idx);
+  static rdcstr GetChunkName(uint32_t idx);
   D3D12ResourceManager *GetResourceManager() { return m_ResourceManager; }
   D3D12ShaderCache *GetShaderCache() { return m_ShaderCache; }
-  D3D12DebugManager *GetDebugManager() { return m_Replay.GetDebugManager(); }
+  D3D12DebugManager *GetDebugManager();
   ResourceId GetResourceID() { return m_ResourceID; }
   Threading::RWLock &GetCapTransitionLock() { return m_CapTransitionLock; }
   void ReleaseSwapchainResources(IDXGISwapChain *swap, IUnknown **backbuffers, int numBackbuffers);
-  void FirstFrame(WrappedIDXGISwapChain4 *swap);
-  FrameRecord &GetFrameRecord() { return m_FrameRecord; }
+  void FirstFrame(IDXGISwapper *swapper);
   const DrawcallDescription *GetDrawcall(uint32_t eventId);
 
   ResourceId GetFrameCaptureResourceId() { return m_FrameCaptureRecord->GetResourceID(); }
-  void AddDebugMessage(MessageCategory c, MessageSeverity sv, MessageSource src, std::string d);
+  void AddDebugMessage(MessageCategory c, MessageSeverity sv, MessageSource src, rdcstr d);
   void AddDebugMessage(const DebugMessage &msg);
-  vector<DebugMessage> GetDebugMessages();
+  rdcarray<DebugMessage> GetDebugMessages();
 
+  ResourceDescription &GetResourceDesc(ResourceId id);
   void AddResource(ResourceId id, ResourceType type, const char *defaultNamePrefix);
   void DerivedResource(ResourceId parent, ResourceId child);
   void DerivedResource(ID3D12DeviceChild *parent, ResourceId child);
   void AddResourceCurChunk(ResourceDescription &descr);
   void AddResourceCurChunk(ResourceId id);
 
-  const string &GetResourceName(ResourceId id) { return m_ResourceNames[id]; }
-  vector<D3D12_RESOURCE_STATES> &GetSubresourceStates(ResourceId id)
+  const rdcstr &GetResourceName(ResourceId id) { return m_ResourceNames[id]; }
+  rdcarray<D3D12_RESOURCE_STATES> &GetSubresourceStates(ResourceId id)
   {
     return m_ResourceStates[id];
   }
-  const map<ResourceId, SubresourceStateVector> &GetSubresourceStates() { return m_ResourceStates; }
-  const map<ResourceId, DXGI_FORMAT> &GetBackbufferFormats() { return m_BackbufferFormat; }
+  const std::map<ResourceId, SubresourceStateVector> &GetSubresourceStates()
+  {
+    return m_ResourceStates;
+  }
+  const std::map<ResourceId, DXGI_FORMAT> &GetBackbufferFormats() { return m_BackbufferFormat; }
   void SetLogFile(const char *logfile);
-  void SetInitParams(const D3D12InitParams &params, uint64_t sectionVersion)
+  void SetInitParams(const D3D12InitParams &params, uint64_t sectionVersion, const ReplayOptions &opts)
   {
     m_InitParams = params;
     m_SectionVersion = sectionVersion;
+    m_ReplayOptions = opts;
   }
+  const ReplayOptions &GetReplayOptions() { return m_ReplayOptions; }
   uint64_t GetLogVersion() { return m_SectionVersion; }
   CaptureState GetState() { return m_State; }
-  D3D12Replay *GetReplay() { return &m_Replay; }
+  D3D12Replay *GetReplay() { return m_Replay; }
   WrappedID3D12CommandQueue *GetQueue() { return m_Queue; }
   ID3D12CommandAllocator *GetAlloc() { return m_Alloc; }
-  void ApplyBarriers(vector<D3D12_RESOURCE_BARRIER> &barriers);
+  void ApplyBarriers(rdcarray<D3D12_RESOURCE_BARRIER> &barriers);
 
-  void GetDynamicDescriptorReferences(std::vector<D3D12Descriptor> &refs)
+  void GetDynamicDescriptorReferences(rdcarray<D3D12Descriptor> &refs)
   {
     SCOPED_LOCK(m_DynDescLock);
     m_DynamicDescriptorRefs.swap(refs);
@@ -522,11 +616,11 @@ public:
       submittedcmds.clear();
     }
 
-    vector<ID3D12GraphicsCommandList4 *> freecmds;
+    rdcarray<ID3D12GraphicsCommandListX *> freecmds;
     // -> GetNextCmd() ->
-    vector<ID3D12GraphicsCommandList4 *> pendingcmds;
+    rdcarray<ID3D12GraphicsCommandListX *> pendingcmds;
     // -> ExecuteLists() ->
-    vector<ID3D12GraphicsCommandList4 *> submittedcmds;
+    rdcarray<ID3D12GraphicsCommandListX *> submittedcmds;
     // -> FlushLists()--------back to freecmds--------^
   } m_InternalCmds;
 
@@ -534,28 +628,34 @@ public:
   // creating fewer temporary lists and making too bloated lists
   static const int initialStateMaxBatch = 100;
   int initStateCurBatch;
-  ID3D12GraphicsCommandList4 *initStateCurList;
+  ID3D12GraphicsCommandListX *initStateCurList;
 
-  ID3D12GraphicsCommandList4 *GetNewList();
-  ID3D12GraphicsCommandList4 *GetInitialStateList();
+  ID3D12GraphicsCommandListX *GetNewList();
+  ID3D12GraphicsCommandListX *GetInitialStateList();
   void CloseInitialStateList();
   ID3D12Resource *GetUploadBuffer(uint64_t chunkOffset, uint64_t byteSize);
   void ApplyInitialContents();
 
   void AddCaptureSubmission();
 
-  void ExecuteList(ID3D12GraphicsCommandList4 *list, WrappedID3D12CommandQueue *queue = NULL,
+  void ExecuteList(ID3D12GraphicsCommandListX *list, WrappedID3D12CommandQueue *queue = NULL,
                    bool InFrameCaptureBoundary = false);
+  void MarkListExecuted(ID3D12GraphicsCommandListX *list);
   void ExecuteLists(WrappedID3D12CommandQueue *queue = NULL, bool InFrameCaptureBoundary = false);
   void FlushLists(bool forceSync = false, ID3D12CommandQueue *queue = NULL);
 
   void GPUSync(ID3D12CommandQueue *queue = NULL, ID3D12Fence *fence = NULL);
   void GPUSyncAllQueues();
 
+  RDCDriver GetFrameCaptureDriver() { return RDCDriver::D3D12; }
   void StartFrameCapture(void *dev, void *wnd);
   bool EndFrameCapture(void *dev, void *wnd);
   bool DiscardFrameCapture(void *dev, void *wnd);
 
+  template <typename SerialiserType>
+  bool Serialise_Present(SerialiserType &ser, ID3D12Resource *PresentedImage, UINT SyncInterval,
+                         UINT Flags);
+
   template <typename SerialiserType>
   bool Serialise_BeginCaptureFrame(SerialiserType &ser);
 
@@ -563,7 +663,7 @@ public:
   bool Serialise_DynamicDescriptorWrite(SerialiserType &ser, const DynamicDescriptorWrite *write);
   template <typename SerialiserType>
   bool Serialise_DynamicDescriptorCopies(SerialiserType &ser,
-                                         const std::vector<DynamicDescriptorCopy> &DescriptorCopies);
+                                         const rdcarray<DynamicDescriptorCopy> &DescriptorCopies);
 
   ReplayStatus ReadLogInitialisation(RDCFile *rdc, bool storeStructuredBuffers);
   void ReplayLog(uint32_t startEventID, uint32_t endEventID, ReplayLogType replayType);
@@ -581,7 +681,8 @@ public:
   {
     if(iid == __uuidof(ID3D12Device) || iid == __uuidof(ID3D12Device1) ||
        iid == __uuidof(ID3D12Device2) || iid == __uuidof(ID3D12Device3) ||
-       iid == __uuidof(ID3D12Device4) || iid == __uuidof(ID3D12Device5))
+       iid == __uuidof(ID3D12Device4) || iid == __uuidof(ID3D12Device5) ||
+       iid == __uuidof(ID3D12Device5))
       return true;
 
     return false;
@@ -600,21 +701,29 @@ public:
       return (ID3D12Device4 *)this;
     else if(iid == __uuidof(ID3D12Device5))
       return (ID3D12Device5 *)this;
+    else if(iid == __uuidof(ID3D12Device6))
+      return (ID3D12Device6 *)this;
 
     RDCERR("Requested unknown device interface %s", ToStr(iid).c_str());
 
     return NULL;
   }
   // Swap Chain
-  IMPLEMENT_FUNCTION_THREAD_SERIALISED(IUnknown *, WrapSwapchainBuffer,
-                                       WrappedIDXGISwapChain4 *swap, DXGI_SWAP_CHAIN_DESC *desc,
-                                       UINT buffer, IUnknown *realSurface);
-  HRESULT Present(WrappedIDXGISwapChain4 *swap, UINT SyncInterval, UINT Flags);
+  IMPLEMENT_FUNCTION_THREAD_SERIALISED(IUnknown *, WrapSwapchainBuffer, IDXGISwapper *swapper,
+                                       DXGI_FORMAT bufferFormat, UINT buffer, IUnknown *realSurface);
+  HRESULT Present(ID3D12GraphicsCommandList *pOverlayCommandList, IDXGISwapper *swapper,
+                  UINT SyncInterval, UINT Flags);
+  HRESULT Present(IDXGISwapper *swapper, UINT SyncInterval, UINT Flags)
+  {
+    return Present(NULL, swapper, SyncInterval, Flags);
+  }
 
   void NewSwapchainBuffer(IUnknown *backbuffer);
-  void ReleaseSwapchainResources(WrappedIDXGISwapChain4 *swap, UINT QueueCount,
+  void ReleaseSwapchainResources(IDXGISwapper *swapper, UINT QueueCount,
                                  IUnknown *const *ppPresentQueue, IUnknown **unwrappedQueues);
 
+  IDXGIResource *WrapExternalDXGIResource(IDXGIResource *res);
+
   void Map(ID3D12Resource *Resource, UINT Subresource);
   void Unmap(ID3D12Resource *Resource, UINT Subresource, byte *mapPtr,
              const D3D12_RANGE *pWrittenRange);
@@ -661,6 +770,12 @@ public:
       this->AddRef();
       return S_OK;
     }
+    else if(riid == __uuidof(ID3D12Device6))
+    {
+      *ppvDevice = (ID3D12Device6 *)this;
+      this->AddRef();
+      return S_OK;
+    }
 
     return E_NOINTERFACE;
   }
@@ -671,9 +786,9 @@ public:
                                        UINT Subresource, const D3D12_BOX *pDstBox,
                                        const void *pSrcData, UINT SrcRowPitch, UINT SrcDepthPitch);
 
-  vector<MapState> GetMaps()
+  rdcarray<MapState> GetMaps()
   {
-    vector<MapState> ret;
+    rdcarray<MapState> ret;
     {
       SCOPED_LOCK(m_MapsLock);
       ret = m_Maps;
@@ -850,6 +965,9 @@ public:
                                        const SECURITY_ATTRIBUTES *pAttributes, DWORD Access,
                                        LPCWSTR Name, HANDLE *pHandle);
 
+  HRESULT STDMETHODCALLTYPE OpenSharedHandleInternal(D3D12Chunk chunkType, REFIID riid,
+                                                     void **ppvObj);
+
   IMPLEMENT_FUNCTION_THREAD_SERIALISED(virtual HRESULT STDMETHODCALLTYPE, OpenSharedHandle,
                                        HANDLE NTHandle, REFIID riid, void **ppvObj);
 
@@ -1010,4 +1128,16 @@ public:
   virtual D3D12_DRIVER_MATCHING_IDENTIFIER_STATUS STDMETHODCALLTYPE CheckDriverMatchingIdentifier(
       _In_ D3D12_SERIALIZED_DATA_TYPE SerializedDataType,
       _In_ const D3D12_SERIALIZED_DATA_DRIVER_MATCHING_IDENTIFIER *pIdentifierToCheck);
+
+  //////////////////////////////
+  // implement ID3D12DeviceDownlevel
+  virtual HRESULT STDMETHODCALLTYPE
+  QueryVideoMemoryInfo(UINT NodeIndex, DXGI_MEMORY_SEGMENT_GROUP MemorySegmentGroup,
+                       _Out_ DXGI_QUERY_VIDEO_MEMORY_INFO *pVideoMemoryInfo);
+
+  //////////////////////////////
+  // implement ID3D12Device6
+  virtual HRESULT STDMETHODCALLTYPE SetBackgroundProcessingMode(
+      D3D12_BACKGROUND_PROCESSING_MODE Mode, D3D12_MEASUREMENTS_ACTION MeasurementsAction,
+      _In_opt_ HANDLE hEventToSignalUponCompletion, _Out_opt_ BOOL *pbFurtherMeasurementsDesired);
 };
