@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -36,11 +36,11 @@ bool WrappedID3D12Device::Serialise_CreateCommandQueue(SerialiserType &ser,
                                                        const D3D12_COMMAND_QUEUE_DESC *pDesc,
                                                        REFIID riid, void **ppCommandQueue)
 {
-  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc");
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc"_lit);
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pCommandQueue,
                           ((WrappedID3D12CommandQueue *)*ppCommandQueue)->GetResourceID())
-      .TypedAs("ID3D12CommandQueue *");
+      .TypedAs("ID3D12CommandQueue *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -132,6 +132,17 @@ HRESULT WrappedID3D12Device::CreateCommandQueue(const D3D12_COMMAND_QUEUE_DESC *
 
     m_Queues.push_back(wrapped);
 
+    bool capframe = false;
+    {
+      SCOPED_READLOCK(m_CapTransitionLock);
+      capframe = IsActiveCapturing(m_State);
+    }
+
+    // while capturing don't allow any queues to be freed, by adding another refcount, since we
+    // gather any commands submitted to them at the end of the capture.
+    if(capframe)
+      wrapped->AddRef();
+
     *ppCommandQueue = (ID3D12CommandQueue *)wrapped;
   }
 
@@ -144,10 +155,10 @@ bool WrappedID3D12Device::Serialise_CreateCommandAllocator(SerialiserType &ser,
                                                            REFIID riid, void **ppCommandAllocator)
 {
   SERIALISE_ELEMENT(type);
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pCommandAllocator,
                           ((WrappedID3D12CommandAllocator *)*ppCommandAllocator)->GetResourceID())
-      .TypedAs("ID3D12CommandAllocator *");
+      .TypedAs("ID3D12CommandAllocator *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -229,10 +240,10 @@ bool WrappedID3D12Device::Serialise_CreateCommandList(SerialiserType &ser, UINT
   SERIALISE_ELEMENT(type);
   SERIALISE_ELEMENT(pCommandAllocator);
   SERIALISE_ELEMENT(pInitialState);
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pCommandList,
                           ((WrappedID3D12GraphicsCommandList *)*ppCommandList)->GetResourceID())
-      .TypedAs("ID3D12GraphicsCommandList *");
+      .TypedAs("ID3D12GraphicsCommandList *"_lit);
 
   // this chunk is purely for user information and consistency, the command buffer we allocate is
   // a dummy and is not used for anything.
@@ -278,7 +289,8 @@ HRESULT WrappedID3D12Device::CreateCommandList(UINT nodeMask, D3D12_COMMAND_LIST
 
   if(riid != __uuidof(ID3D12GraphicsCommandList) && riid != __uuidof(ID3D12CommandList) &&
      riid != __uuidof(ID3D12GraphicsCommandList1) && riid != __uuidof(ID3D12GraphicsCommandList2) &&
-     riid != __uuidof(ID3D12GraphicsCommandList3) && riid != __uuidof(ID3D12GraphicsCommandList4))
+     riid != __uuidof(ID3D12GraphicsCommandList3) && riid != __uuidof(ID3D12GraphicsCommandList4) &&
+     riid != __uuidof(ID3D12GraphicsCommandList5))
     return E_NOINTERFACE;
 
   void *realptr = NULL;
@@ -301,6 +313,8 @@ HRESULT WrappedID3D12Device::CreateCommandList(UINT nodeMask, D3D12_COMMAND_LIST
     real = (ID3D12GraphicsCommandList3 *)realptr;
   else if(riid == __uuidof(ID3D12GraphicsCommandList4))
     real = (ID3D12GraphicsCommandList4 *)realptr;
+  else if(riid == __uuidof(ID3D12GraphicsCommandList5))
+    real = (ID3D12GraphicsCommandList5 *)realptr;
 
   if(SUCCEEDED(ret))
   {
@@ -351,6 +365,8 @@ HRESULT WrappedID3D12Device::CreateCommandList(UINT nodeMask, D3D12_COMMAND_LIST
       *ppCommandList = (ID3D12GraphicsCommandList3 *)wrapped;
     else if(riid == __uuidof(ID3D12GraphicsCommandList4))
       *ppCommandList = (ID3D12GraphicsCommandList4 *)wrapped;
+    else if(riid == __uuidof(ID3D12GraphicsCommandList5))
+      *ppCommandList = (ID3D12GraphicsCommandList5 *)wrapped;
     else if(riid == __uuidof(ID3D12CommandList))
       *ppCommandList = (ID3D12CommandList *)wrapped;
     else
@@ -365,11 +381,11 @@ bool WrappedID3D12Device::Serialise_CreateGraphicsPipelineState(
     SerialiserType &ser, const D3D12_GRAPHICS_PIPELINE_STATE_DESC *pDesc, REFIID riid,
     void **ppPipelineState)
 {
-  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc");
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc"_lit);
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pPipelineState,
                           ((WrappedID3D12PipelineState *)*ppPipelineState)->GetResourceID())
-      .TypedAs("ID3D12PipelineState *");
+      .TypedAs("ID3D12PipelineState *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -378,6 +394,33 @@ bool WrappedID3D12Device::Serialise_CreateGraphicsPipelineState(
     D3D12_GRAPHICS_PIPELINE_STATE_DESC unwrappedDesc = Descriptor;
     unwrappedDesc.pRootSignature = Unwrap(unwrappedDesc.pRootSignature);
 
+    // check for bytecode - if the user is wrongly using DXIL we will fail to load the capture
+    {
+      D3D12_SHADER_BYTECODE *shaders[] = {
+          &unwrappedDesc.VS, &unwrappedDesc.HS, &unwrappedDesc.DS,
+          &unwrappedDesc.GS, &unwrappedDesc.PS,
+      };
+
+      const char *name[] = {"VS", "HS", "DS", "GS", "PS"};
+
+      for(size_t i = 0; i < ARRAY_COUNT(shaders); i++)
+      {
+        if(shaders[i]->BytecodeLength > 0 && shaders[i]->pShaderBytecode)
+        {
+          if(!DXBC::DXBCContainer::CheckForShaderCode(shaders[i]->pShaderBytecode,
+                                                      shaders[i]->BytecodeLength))
+          {
+            RDCERR(
+                "No shader code found in %s bytecode in pipeline state. "
+                "DXIL is unsupported and must be checked for using CheckFeatureSupport.",
+                name[i]);
+            m_FailedReplayStatus = ReplayStatus::APIReplayFailed;
+            return false;
+          }
+        }
+      }
+    }
+
     ID3D12PipelineState *ret = NULL;
     HRESULT hr = m_pDevice->CreateGraphicsPipelineState(&unwrappedDesc, guid, (void **)&ret);
 
@@ -487,6 +530,32 @@ HRESULT WrappedID3D12Device::CreateGraphicsPipelineState(const D3D12_GRAPHICS_PI
 
   if(SUCCEEDED(ret))
   {
+    // check for bytecode - if the user is wrongly using DXIL we will prevent capturing
+    {
+      D3D12_SHADER_BYTECODE *shaders[] = {
+          &unwrappedDesc.VS, &unwrappedDesc.HS, &unwrappedDesc.DS,
+          &unwrappedDesc.GS, &unwrappedDesc.PS,
+      };
+
+      const char *name[] = {"VS", "HS", "DS", "GS", "PS"};
+
+      for(size_t i = 0; i < ARRAY_COUNT(shaders); i++)
+      {
+        if(shaders[i]->BytecodeLength > 0 && shaders[i]->pShaderBytecode)
+        {
+          if(!DXBC::DXBCContainer::CheckForShaderCode(shaders[i]->pShaderBytecode,
+                                                      shaders[i]->BytecodeLength))
+          {
+            RDCERR(
+                "No shader code found in %s bytecode in pipeline state. "
+                "DXIL is unsupported and must be checked for using CheckFeatureSupport.",
+                name[i]);
+            m_InvalidPSO = true;
+          }
+        }
+      }
+    }
+
     WrappedID3D12PipelineState *wrapped = new WrappedID3D12PipelineState(real, this);
 
     if(IsCaptureMode(m_State))
@@ -581,11 +650,11 @@ bool WrappedID3D12Device::Serialise_CreateComputePipelineState(
     SerialiserType &ser, const D3D12_COMPUTE_PIPELINE_STATE_DESC *pDesc, REFIID riid,
     void **ppPipelineState)
 {
-  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc");
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc"_lit);
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pPipelineState,
                           ((WrappedID3D12PipelineState *)*ppPipelineState)->GetResourceID())
-      .TypedAs("ID3D12PipelineState *");
+      .TypedAs("ID3D12PipelineState *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -594,6 +663,18 @@ bool WrappedID3D12Device::Serialise_CreateComputePipelineState(
     D3D12_COMPUTE_PIPELINE_STATE_DESC unwrappedDesc = Descriptor;
     unwrappedDesc.pRootSignature = Unwrap(unwrappedDesc.pRootSignature);
 
+    // check for bytecode - if the user is wrongly using DXIL we will hard-fail instead of producing
+    // a corrupted capture.
+    if(!DXBC::DXBCContainer::CheckForShaderCode(unwrappedDesc.CS.pShaderBytecode,
+                                                unwrappedDesc.CS.BytecodeLength))
+    {
+      RDCERR(
+          "No shader code found in CS bytecode in pipeline state. "
+          "DXIL is unsupported and must be checked for using CheckFeatureSupport.");
+      m_FailedReplayStatus = ReplayStatus::APIReplayFailed;
+      return false;
+    }
+
     ID3D12PipelineState *ret = NULL;
     HRESULT hr = m_pDevice->CreateComputePipelineState(&unwrappedDesc, guid, (void **)&ret);
 
@@ -647,6 +728,17 @@ HRESULT WrappedID3D12Device::CreateComputePipelineState(const D3D12_COMPUTE_PIPE
 
   if(SUCCEEDED(ret))
   {
+    // check for bytecode - if the user is wrongly using DXIL we will hard-fail instead of producing
+    // a corrupted capture.
+    if(!DXBC::DXBCContainer::CheckForShaderCode(unwrappedDesc.CS.pShaderBytecode,
+                                                unwrappedDesc.CS.BytecodeLength))
+    {
+      RDCERR(
+          "No shader code found in CS bytecode in pipeline state. "
+          "DXIL is unsupported and must be checked for using CheckFeatureSupport.");
+      m_InvalidPSO = true;
+    }
+
     WrappedID3D12PipelineState *wrapped = new WrappedID3D12PipelineState(real, this);
 
     if(IsCaptureMode(m_State))
@@ -687,10 +779,10 @@ bool WrappedID3D12Device::Serialise_CreateDescriptorHeap(
     SerialiserType &ser, const D3D12_DESCRIPTOR_HEAP_DESC *pDescriptorHeapDesc, REFIID riid,
     void **ppvHeap)
 {
-  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDescriptorHeapDesc).Named("pDescriptorHeapDesc");
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDescriptorHeapDesc).Named("pDescriptorHeapDesc"_lit);
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pHeap, ((WrappedID3D12DescriptorHeap *)*ppvHeap)->GetResourceID())
-      .TypedAs("ID3D12DescriptorHeap *");
+      .TypedAs("ID3D12DescriptorHeap *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -764,13 +856,7 @@ HRESULT WrappedID3D12Device::CreateDescriptorHeap(const D3D12_DESCRIPTOR_HEAP_DE
 
       record->AddChunk(scope.Get());
 
-      {
-        SCOPED_READLOCK(m_CapTransitionLock);
-        if(IsBackgroundCapturing(m_State))
-          GetResourceManager()->MarkDirtyResource(wrapped->GetResourceID());
-        else
-          GetResourceManager()->MarkPendingDirty(wrapped->GetResourceID());
-      }
+      GetResourceManager()->MarkDirtyResource(wrapped->GetResourceID());
     }
     else
     {
@@ -792,10 +878,10 @@ bool WrappedID3D12Device::Serialise_CreateRootSignature(SerialiserType &ser, UIN
   SERIALISE_ELEMENT(nodeMask);
   SERIALISE_ELEMENT_ARRAY(pBlobWithRootSignature, blobLengthInBytes_);
   SERIALISE_ELEMENT_LOCAL(blobLengthInBytes, uint64_t(blobLengthInBytes_));
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pRootSignature,
                           ((WrappedID3D12RootSignature *)*ppvRootSignature)->GetResourceID())
-      .TypedAs("ID3D12RootSignature *");
+      .TypedAs("ID3D12RootSignature *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -828,6 +914,13 @@ bool WrappedID3D12Device::Serialise_CreateRootSignature(SerialiserType &ser, UIN
         wrapped->sig =
             GetShaderCache()->GetRootSig(pBlobWithRootSignature, (size_t)blobLengthInBytes);
 
+        {
+          StructuredSerialiser structuriser(ser.GetStructuredFile().chunks.back(), &GetChunkName);
+          structuriser.SetUserData(GetResourceManager());
+
+          structuriser.Serialise("UnpackedSignature"_lit, wrapped->sig);
+        }
+
         GetResourceManager()->AddLiveResource(pRootSignature, ret);
       }
 
@@ -1188,14 +1281,14 @@ bool WrappedID3D12Device::Serialise_CreateCommittedResource(
     const D3D12_RESOURCE_DESC *pDesc, D3D12_RESOURCE_STATES InitialResourceState,
     const D3D12_CLEAR_VALUE *pOptimizedClearValue, REFIID riidResource, void **ppvResource)
 {
-  SERIALISE_ELEMENT_LOCAL(props, *pHeapProperties).Named("pHeapProperties");
+  SERIALISE_ELEMENT_LOCAL(props, *pHeapProperties).Named("pHeapProperties"_lit);
   SERIALISE_ELEMENT(HeapFlags);
-  SERIALISE_ELEMENT_LOCAL(desc, *pDesc).Named("pDesc");
+  SERIALISE_ELEMENT_LOCAL(desc, *pDesc).Named("pDesc"_lit);
   SERIALISE_ELEMENT(InitialResourceState);
   SERIALISE_ELEMENT_OPT(pOptimizedClearValue);
-  SERIALISE_ELEMENT_LOCAL(guid, riidResource).Named("riidResource");
+  SERIALISE_ELEMENT_LOCAL(guid, riidResource).Named("riidResource"_lit);
   SERIALISE_ELEMENT_LOCAL(pResource, ((WrappedID3D12Resource1 *)*ppvResource)->GetResourceID())
-      .TypedAs("ID3D12Resource *");
+      .TypedAs("ID3D12Resource *"_lit);
 
   SERIALISE_ELEMENT_LOCAL(gpuAddress,
                           ((WrappedID3D12Resource1 *)*ppvResource)->GetGPUVirtualAddressIfBuffer())
@@ -1253,7 +1346,7 @@ bool WrappedID3D12Device::Serialise_CreateCommittedResource(
       GetResourceManager()->AddLiveResource(pResource, ret);
 
       SubresourceStateVector &states = m_ResourceStates[GetResID(ret)];
-      states.resize(GetNumSubresources(m_pDevice, &desc), InitialResourceState);
+      states.fill(GetNumSubresources(m_pDevice, &desc), InitialResourceState);
 
       ResourceType type = ResourceType::Texture;
       const char *prefix = "Texture";
@@ -1312,10 +1405,21 @@ HRESULT WrappedID3D12Device::CreateCommittedResource(const D3D12_HEAP_PROPERTIES
   if(riidResource != __uuidof(ID3D12Resource) && riidResource != __uuidof(ID3D12Resource1))
     return E_NOINTERFACE;
 
+  const D3D12_RESOURCE_DESC *pCreateDesc = pDesc;
+  D3D12_RESOURCE_DESC localDesc;
+
+  if(pDesc && pDesc->Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE2D && pDesc->SampleDesc.Count > 1)
+  {
+    localDesc = *pDesc;
+    // need to be able to create SRVs of MSAA textures to copy out their contents
+    localDesc.Flags &= ~D3D12_RESOURCE_FLAG_DENY_SHADER_RESOURCE;
+    pCreateDesc = &localDesc;
+  }
+
   void *realptr = NULL;
   HRESULT ret;
   SERIALISE_TIME_CALL(ret = m_pDevice->CreateCommittedResource(
-                          pHeapProperties, HeapFlags, pDesc, InitialResourceState,
+                          pHeapProperties, HeapFlags, pCreateDesc, InitialResourceState,
                           pOptimizedClearValue, riidResource, &realptr));
 
   ID3D12Resource *real = NULL;
@@ -1346,13 +1450,7 @@ HRESULT WrappedID3D12Device::CreateCommittedResource(const D3D12_HEAP_PROPERTIES
 
       record->AddChunk(scope.Get());
 
-      {
-        SCOPED_READLOCK(m_CapTransitionLock);
-        if(IsBackgroundCapturing(m_State))
-          GetResourceManager()->MarkDirtyResource(wrapped->GetResourceID());
-        else
-          GetResourceManager()->MarkPendingDirty(wrapped->GetResourceID());
-      }
+      GetResourceManager()->MarkDirtyResource(wrapped->GetResourceID());
     }
     else
     {
@@ -1363,7 +1461,7 @@ HRESULT WrappedID3D12Device::CreateCommittedResource(const D3D12_HEAP_PROPERTIES
       SCOPED_LOCK(m_ResourceStatesLock);
       SubresourceStateVector &states = m_ResourceStates[wrapped->GetResourceID()];
 
-      states.resize(GetNumSubresources(m_pDevice, pDesc), InitialResourceState);
+      states.fill(GetNumSubresources(m_pDevice, pDesc), InitialResourceState);
     }
 
     *ppvResource = (ID3D12Resource *)wrapped;
@@ -1376,10 +1474,10 @@ template <typename SerialiserType>
 bool WrappedID3D12Device::Serialise_CreateHeap(SerialiserType &ser, const D3D12_HEAP_DESC *pDesc,
                                                REFIID riid, void **ppvHeap)
 {
-  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc");
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc"_lit);
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pHeap, ((WrappedID3D12Heap1 *)*ppvHeap)->GetResourceID())
-      .TypedAs("ID3D12Heap *");
+      .TypedAs("ID3D12Heap *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1468,12 +1566,12 @@ bool WrappedID3D12Device::Serialise_CreatePlacedResource(
 {
   SERIALISE_ELEMENT(pHeap);
   SERIALISE_ELEMENT(HeapOffset);
-  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc");
+  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc"_lit);
   SERIALISE_ELEMENT(InitialState);
   SERIALISE_ELEMENT_OPT(pOptimizedClearValue);
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pResource, ((WrappedID3D12Resource1 *)*ppvResource)->GetResourceID())
-      .TypedAs("ID3D12Resource *");
+      .TypedAs("ID3D12Resource *"_lit);
 
   SERIALISE_ELEMENT_LOCAL(gpuAddress,
                           ((WrappedID3D12Resource1 *)*ppvResource)->GetGPUVirtualAddressIfBuffer())
@@ -1526,7 +1624,7 @@ bool WrappedID3D12Device::Serialise_CreatePlacedResource(
       GetResourceManager()->AddLiveResource(pResource, ret);
 
       SubresourceStateVector &states = m_ResourceStates[GetResID(ret)];
-      states.resize(GetNumSubresources(m_pDevice, &Descriptor), InitialState);
+      states.fill(GetNumSubresources(m_pDevice, &Descriptor), InitialState);
     }
 
     ResourceType type = ResourceType::Texture;
@@ -1585,9 +1683,20 @@ HRESULT WrappedID3D12Device::CreatePlacedResource(ID3D12Heap *pHeap, UINT64 Heap
   if(riid != __uuidof(ID3D12Resource))
     return E_NOINTERFACE;
 
+  const D3D12_RESOURCE_DESC *pCreateDesc = pDesc;
+  D3D12_RESOURCE_DESC localDesc;
+
+  if(pDesc && pDesc->Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE2D && pDesc->SampleDesc.Count > 1)
+  {
+    localDesc = *pDesc;
+    // need to be able to create SRVs of MSAA textures to copy out their contents
+    localDesc.Flags &= ~D3D12_RESOURCE_FLAG_DENY_SHADER_RESOURCE;
+    pCreateDesc = &localDesc;
+  }
+
   ID3D12Resource *real = NULL;
   HRESULT ret;
-  SERIALISE_TIME_CALL(ret = m_pDevice->CreatePlacedResource(Unwrap(pHeap), HeapOffset, pDesc,
+  SERIALISE_TIME_CALL(ret = m_pDevice->CreatePlacedResource(Unwrap(pHeap), HeapOffset, pCreateDesc,
                                                             InitialState, pOptimizedClearValue,
                                                             riid, (void **)&real));
 
@@ -1616,13 +1725,7 @@ HRESULT WrappedID3D12Device::CreatePlacedResource(ID3D12Heap *pHeap, UINT64 Heap
       record->AddParent(GetRecord(pHeap));
       record->AddChunk(scope.Get());
 
-      {
-        SCOPED_READLOCK(m_CapTransitionLock);
-        if(IsBackgroundCapturing(m_State))
-          GetResourceManager()->MarkDirtyResource(wrapped->GetResourceID());
-        else
-          GetResourceManager()->MarkPendingDirty(wrapped->GetResourceID());
-      }
+      GetResourceManager()->MarkDirtyResource(wrapped->GetResourceID());
     }
     else
     {
@@ -1633,7 +1736,7 @@ HRESULT WrappedID3D12Device::CreatePlacedResource(ID3D12Heap *pHeap, UINT64 Heap
       SCOPED_LOCK(m_ResourceStatesLock);
       SubresourceStateVector &states = m_ResourceStates[wrapped->GetResourceID()];
 
-      states.resize(GetNumSubresources(m_pDevice, pDesc), InitialState);
+      states.fill(GetNumSubresources(m_pDevice, pDesc), InitialState);
     }
 
     *ppvResource = (ID3D12Resource *)wrapped;
@@ -1667,9 +1770,9 @@ bool WrappedID3D12Device::Serialise_CreateFence(SerialiserType &ser, UINT64 Init
 {
   SERIALISE_ELEMENT(InitialValue);
   SERIALISE_ELEMENT(Flags);
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pFence, ((WrappedID3D12Fence1 *)*ppFence)->GetResourceID())
-      .TypedAs("ID3D12Fence *");
+      .TypedAs("ID3D12Fence *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1734,7 +1837,7 @@ HRESULT WrappedID3D12Device::CreateFence(UINT64 InitialValue, D3D12_FENCE_FLAGS
       Serialise_CreateFence(ser, InitialValue, Flags, riid, (void **)&wrapped);
 
       D3D12ResourceRecord *record = GetResourceManager()->AddResourceRecord(wrapped->GetResourceID());
-      record->type = Resource_Resource;
+      record->type = Resource_Fence;
       record->Length = 0;
       wrapped->SetResourceRecord(record);
 
@@ -1756,10 +1859,10 @@ bool WrappedID3D12Device::Serialise_CreateQueryHeap(SerialiserType &ser,
                                                     const D3D12_QUERY_HEAP_DESC *pDesc, REFIID riid,
                                                     void **ppvHeap)
 {
-  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc");
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc"_lit);
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pQueryHeap, ((WrappedID3D12QueryHeap *)*ppvHeap)->GetResourceID())
-      .TypedAs("ID3D12QueryHeap *");
+      .TypedAs("ID3D12QueryHeap *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1834,12 +1937,12 @@ bool WrappedID3D12Device::Serialise_CreateCommandSignature(SerialiserType &ser,
                                                            ID3D12RootSignature *pRootSignature,
                                                            REFIID riid, void **ppvCommandSignature)
 {
-  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc");
+  SERIALISE_ELEMENT_LOCAL(Descriptor, *pDesc).Named("pDesc"_lit);
   SERIALISE_ELEMENT(pRootSignature);
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pCommandSignature,
                           ((WrappedID3D12CommandSignature *)*ppvCommandSignature)->GetResourceID())
-      .TypedAs("ID3D12CommandSignature *");
+      .TypedAs("ID3D12CommandSignature *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1859,8 +1962,7 @@ bool WrappedID3D12Device::Serialise_CreateCommandSignature(SerialiserType &ser,
       WrappedID3D12CommandSignature *wrapped = new WrappedID3D12CommandSignature(ret, this);
 
       wrapped->sig.ByteStride = Descriptor.ByteStride;
-      wrapped->sig.arguments.insert(wrapped->sig.arguments.begin(), Descriptor.pArgumentDescs,
-                                    Descriptor.pArgumentDescs + Descriptor.NumArgumentDescs);
+      wrapped->sig.arguments.assign(Descriptor.pArgumentDescs, Descriptor.NumArgumentDescs);
 
       wrapped->sig.graphics = true;
       wrapped->sig.numDraws = 0;
@@ -1960,7 +2062,7 @@ HRESULT WrappedID3D12Device::CreateSharedHandle(ID3D12DeviceChild *pObject,
 
 template <typename SerialiserType>
 bool WrappedID3D12Device::Serialise_DynamicDescriptorCopies(
-    SerialiserType &ser, const std::vector<DynamicDescriptorCopy> &DescriptorCopies)
+    SerialiserType &ser, const rdcarray<DynamicDescriptorCopy> &DescriptorCopies)
 {
   SERIALISE_ELEMENT(DescriptorCopies);
 
@@ -2004,7 +2106,7 @@ void WrappedID3D12Device::CopyDescriptors(
   D3D12Descriptor *src = GetWrapped(pSrcDescriptorRangeStarts[0]);
   D3D12Descriptor *dst = GetWrapped(pDestDescriptorRangeStarts[0]);
 
-  std::vector<DynamicDescriptorCopy> copies;
+  rdcarray<DynamicDescriptorCopy> copies;
 
   bool capframe = false;
 
@@ -2085,7 +2187,7 @@ void WrappedID3D12Device::CopyDescriptors(
 
     {
       SCOPED_LOCK(m_DynDescLock);
-      m_DynamicDescriptorCopies.insert(m_DynamicDescriptorCopies.end(), copies.begin(), copies.end());
+      m_DynamicDescriptorCopies.append(copies);
       for(size_t i = 0; i < copies.size(); i++)
       {
         copies[i].src->GetHeap()->AddRef();
@@ -2152,14 +2254,14 @@ void WrappedID3D12Device::CopyDescriptorsSimple(UINT NumDescriptors,
       GetResourceManager()->MarkResourceFrameReferenced(desc->GetHeapResourceId(), eFrameRef_Read);
     }
 
-    std::vector<DynamicDescriptorCopy> copies;
+    rdcarray<DynamicDescriptorCopy> copies;
     copies.reserve(NumDescriptors);
     for(UINT i = 0; i < NumDescriptors; i++)
       copies.push_back(DynamicDescriptorCopy(&dst[i], &src[i], DescriptorHeapsType));
 
     {
       SCOPED_LOCK(m_DynDescLock);
-      m_DynamicDescriptorCopies.insert(m_DynamicDescriptorCopies.end(), copies.begin(), copies.end());
+      m_DynamicDescriptorCopies.append(copies);
       for(size_t i = 0; i < copies.size(); i++)
       {
         copies[i].src->GetHeap()->AddRef();
@@ -2185,24 +2287,66 @@ void WrappedID3D12Device::CopyDescriptorsSimple(UINT NumDescriptors,
 }
 
 template <typename SerialiserType>
-bool WrappedID3D12Device::Serialise_OpenSharedHandle(SerialiserType &ser, HANDLE NTHandle,
-                                                     REFIID riid, void **ppvObj)
+bool WrappedID3D12Device::Serialise_OpenSharedHandle(SerialiserType &ser, HANDLE, REFIID riid,
+                                                     void **ppvObj)
 {
   SERIALISE_ELEMENT_LOCAL(ResourceRIID, riid);
 
   SERIALISE_CHECK_READ_ERRORS();
 
-  bool isRes = (ResourceRIID == __uuidof(ID3D12Resource) ? true : false);
-  if(isRes)
+  bool isRes = ResourceRIID == __uuidof(ID3D12Resource) || ResourceRIID == __uuidof(ID3D12Resource1);
+  bool isFence = ResourceRIID == __uuidof(ID3D12Fence) || ResourceRIID == __uuidof(ID3D12Fence1);
+  if(isFence)
+  {
+    ID3D12Fence *fence = NULL;
+    if(ser.IsWriting())
+    {
+      fence = (ID3D12Fence *)(ID3D12DeviceChild *)*ppvObj;
+      if(ResourceRIID == __uuidof(ID3D12Fence1))
+        fence = (ID3D12Fence1 *)(ID3D12DeviceChild *)*ppvObj;
+    }
+
+    SERIALISE_ELEMENT_LOCAL(resourceId, GetResID(fence));
+
+    UINT64 fakeInitialValue = 0;
+    D3D12_FENCE_FLAGS fakeFlags = D3D12_FENCE_FLAG_NONE;
+
+    // maybe in future this can be determined?
+    SERIALISE_ELEMENT_LOCAL(initialValue, fakeInitialValue);
+    SERIALISE_ELEMENT_LOCAL(flags, fakeFlags);
+
+    if(IsReplayingAndReading())
+    {
+      HRESULT hr;
+      ID3D12Fence *ret;
+      hr = m_pDevice->CreateFence(initialValue, flags, __uuidof(ID3D12Fence), (void **)&ret);
+      if(FAILED(hr))
+      {
+        RDCERR("Failed on resource serialise-creation, HRESULT: %s", ToStr(hr).c_str());
+        return false;
+      }
+      else
+      {
+        ret = new WrappedID3D12Fence1(ret, this);
+
+        GetResourceManager()->AddLiveResource(resourceId, ret);
+      }
+
+      AddResource(resourceId, ResourceType::Sync, "Fence");
+    }
+  }
+  else if(isRes)
   {
     D3D12_RESOURCE_DESC desc;
     D3D12_HEAP_PROPERTIES heapProperties;
     D3D12_HEAP_FLAGS heapFlags;
 
-    ID3D12Resource *res{};
+    ID3D12Resource *res = NULL;
     if(ser.IsWriting())
     {
-      res = (ID3D12Resource *)*ppvObj;
+      res = (ID3D12Resource *)(ID3D12DeviceChild *)*ppvObj;
+      if(ResourceRIID == __uuidof(ID3D12Resource1))
+        res = (ID3D12Resource1 *)(ID3D12DeviceChild *)*ppvObj;
       desc = res->GetDesc();
       res->GetHeapProperties(&heapProperties, &heapFlags);
     }
@@ -2222,8 +2366,9 @@ bool WrappedID3D12Device::Serialise_OpenSharedHandle(SerialiserType &ser, HANDLE
 
       HRESULT hr;
       ID3D12Resource *ret;
-      hr = m_pDevice->CreateCommittedResource(&heapProperties, heapFlags, &desc, InitialResourceState,
-                                              pOptimizedClearValue, ResourceRIID, (void **)&ret);
+      hr = m_pDevice->CreateCommittedResource(&heapProperties, heapFlags, &desc,
+                                              InitialResourceState, pOptimizedClearValue,
+                                              __uuidof(ID3D12Resource), (void **)&ret);
       if(FAILED(hr))
       {
         RDCERR("Failed on resource serialise-creation, HRESULT: %s", ToStr(hr).c_str());
@@ -2239,7 +2384,7 @@ bool WrappedID3D12Device::Serialise_OpenSharedHandle(SerialiserType &ser, HANDLE
         GetResourceManager()->AddLiveResource(resourceId, ret);
 
         SubresourceStateVector &states = m_ResourceStates[GetResID(ret)];
-        states.resize(GetNumSubresources(m_pDevice, &desc), InitialResourceState);
+        states.fill(GetNumSubresources(m_pDevice, &desc), InitialResourceState);
 
         ResourceType type = ResourceType::Texture;
         const char *prefix = "Texture";
@@ -2290,6 +2435,25 @@ bool WrappedID3D12Device::Serialise_OpenSharedHandle(SerialiserType &ser, HANDLE
 }
 
 HRESULT WrappedID3D12Device::OpenSharedHandle(HANDLE NTHandle, REFIID riid, void **ppvObj)
+{
+  if(ppvObj == NULL)
+    return E_INVALIDARG;
+
+  HRESULT hr;
+
+  SERIALISE_TIME_CALL(hr = m_pDevice->OpenSharedHandle(NTHandle, riid, ppvObj));
+
+  if(FAILED(hr))
+  {
+    IUnknown *unk = (IUnknown *)*ppvObj;
+    SAFE_RELEASE(unk);
+    return hr;
+  }
+
+  return OpenSharedHandleInternal(D3D12Chunk::Device_OpenSharedHandle, riid, ppvObj);
+}
+
+HRESULT WrappedID3D12Device::OpenSharedHandleInternal(D3D12Chunk chunkType, REFIID riid, void **ppvObj)
 {
   if(IsReplayMode(m_State))
   {
@@ -2297,38 +2461,88 @@ HRESULT WrappedID3D12Device::OpenSharedHandle(HANDLE NTHandle, REFIID riid, void
     return E_NOTIMPL;
   }
 
-  if(ppvObj == NULL)
-    return E_INVALIDARG;
-
-  bool isRes = (riid == __uuidof(ID3D12Resource) ? true : false);
-  if(isRes)
+  bool isDXGIRes = riid == __uuidof(IDXGIResource) || riid == __uuidof(IDXGIResource1);
+  bool isRes = riid == __uuidof(ID3D12Resource) || riid == __uuidof(ID3D12Resource1);
+  bool isFence = riid == __uuidof(ID3D12Fence) || riid == __uuidof(ID3D12Fence1);
+  if(isDXGIRes || isRes || isFence)
   {
-    void *ret = NULL;
-    HRESULT hr;
-    SERIALISE_TIME_CALL(hr = m_pDevice->OpenSharedHandle(NTHandle, riid, &ret));
+    void *ret = *ppvObj;
+    HRESULT hr = S_OK;
 
-    if(FAILED(hr))
+    if(isDXGIRes)
     {
-      IUnknown *unk = (IUnknown *)ret;
-      SAFE_RELEASE(unk);
-      return hr;
+      IDXGIResource *dxgiRes = (IDXGIResource *)ret;
+      if(riid == __uuidof(IDXGIResource1))
+        dxgiRes = (IDXGIResource1 *)ret;
+
+      ID3D12Resource *d3d12Res = NULL;
+      hr = dxgiRes->QueryInterface(__uuidof(ID3D12Resource), (void **)&d3d12Res);
+
+      // if we can't get a d3d11Res then we can't properly wrap this resource,
+      // whatever it is.
+      if(FAILED(hr) || d3d12Res == NULL)
+      {
+        SAFE_RELEASE(d3d12Res);
+        SAFE_RELEASE(dxgiRes);
+        return E_NOINTERFACE;
+      }
+
+      // release this interface
+      SAFE_RELEASE(dxgiRes);
+
+      // and use this one, so it'll be casted back below
+      ret = (void *)d3d12Res;
+      isRes = true;
     }
-    else
+
+    ID3D12DeviceChild *wrappedDeviceChild = NULL;
+    D3D12ResourceRecord *record = NULL;
+
+    if(isFence)
+    {
+      ID3D12Fence *real = (ID3D12Fence *)ret;
+      if(riid == __uuidof(ID3D12Fence1))
+        real = (ID3D12Fence1 *)ret;
+
+      WrappedID3D12Fence1 *wrapped = new WrappedID3D12Fence1(real, this);
+
+      wrappedDeviceChild = wrapped;
+
+      *ppvObj = (ID3D12Fence *)wrapped;
+      if(riid == __uuidof(ID3D12Fence1))
+        *ppvObj = (ID3D12Fence1 *)wrapped;
+
+      record = GetResourceManager()->AddResourceRecord(wrapped->GetResourceID());
+      record->type = Resource_Fence;
+      record->Length = 0;
+      wrapped->SetResourceRecord(record);
+    }
+    else if(isRes)
     {
       ID3D12Resource *real = (ID3D12Resource *)ret;
+      if(riid == __uuidof(ID3D12Resource1))
+        real = (ID3D12Resource1 *)ret;
 
       WrappedID3D12Resource1 *wrapped = new WrappedID3D12Resource1(real, this);
-      *ppvObj = (ID3D12Resource *)wrapped;
 
-      CACHE_THREAD_SERIALISER();
+      wrappedDeviceChild = wrapped;
 
-      SCOPED_SERIALISE_CHUNK(D3D12Chunk::Device_OpenSharedHandle);
-      Serialise_OpenSharedHandle(ser, NTHandle, riid, ppvObj);
+      if(isDXGIRes)
+      {
+        wrapped->QueryInterface(riid, ppvObj);
+        wrapped->Release();
+      }
+      else
+      {
+        *ppvObj = (ID3D12Resource *)wrapped;
+        if(riid == __uuidof(ID3D12Resource1))
+          *ppvObj = (ID3D12Resource1 *)wrapped;
+      }
 
       D3D12_RESOURCE_DESC desc = wrapped->GetDesc();
       D3D12_RESOURCE_STATES InitialResourceState = D3D12_RESOURCE_STATE_COMMON;
 
-      D3D12ResourceRecord *record = GetResourceManager()->AddResourceRecord(wrapped->GetResourceID());
+      record = GetResourceManager()->AddResourceRecord(wrapped->GetResourceID());
       record->type = Resource_Resource;
       record->Length = 0;
       wrapped->SetResourceRecord(record);
@@ -2336,27 +2550,31 @@ HRESULT WrappedID3D12Device::OpenSharedHandle(HANDLE NTHandle, REFIID riid, void
       record->m_MapsCount = GetNumSubresources(this, &desc);
       record->m_Maps = new D3D12ResourceRecord::MapData[record->m_MapsCount];
 
-      record->AddChunk(scope.Get());
-
-      {
-        SCOPED_READLOCK(m_CapTransitionLock);
-        if(IsBackgroundCapturing(m_State))
-          GetResourceManager()->MarkDirtyResource(wrapped->GetResourceID());
-        else
-          GetResourceManager()->MarkPendingDirty(wrapped->GetResourceID());
-      }
+      GetResourceManager()->MarkDirtyResource(wrapped->GetResourceID());
 
       {
         SCOPED_LOCK(m_ResourceStatesLock);
         SubresourceStateVector &states = m_ResourceStates[wrapped->GetResourceID()];
 
-        states.resize(GetNumSubresources(m_pDevice, &desc), InitialResourceState);
+        states.fill(GetNumSubresources(m_pDevice, &desc), InitialResourceState);
       }
-
-      return S_OK;
     }
+
+    CACHE_THREAD_SERIALISER();
+
+    SCOPED_SERIALISE_CHUNK(chunkType);
+    Serialise_OpenSharedHandle(ser, 0, riid, (void **)&wrappedDeviceChild);
+
+    record->AddChunk(scope.Get());
+
+    return S_OK;
   }
 
+  RDCERR("Unknown OpenSharedResourceInternal GUID: %s", ToStr(riid).c_str());
+
+  IUnknown *unk = (IUnknown *)*ppvObj;
+  SAFE_RELEASE(unk);
+
   return E_NOINTERFACE;
 }
 
@@ -2497,15 +2715,22 @@ HRESULT WrappedID3D12Device::CheckFeatureSupport(D3D12_FEATURE Feature, void *pF
   }
   if(Feature == D3D12_FEATURE_D3D12_OPTIONS5)
   {
-    D3D12_FEATURE_DATA_D3D12_OPTIONS5 *opts =
-        (D3D12_FEATURE_DATA_D3D12_OPTIONS5 *)pFeatureSupportData;
-    if(FeatureSupportDataSize != sizeof(D3D12_FEATURE_DATA_D3D12_OPTIONS5))
-      return E_INVALIDARG;
+    HRESULT hr = m_pDevice->CheckFeatureSupport(Feature, pFeatureSupportData, FeatureSupportDataSize);
 
-    // don't support raytracing
-    opts->RaytracingTier = D3D12_RAYTRACING_TIER_NOT_SUPPORTED;
+    if(SUCCEEDED(hr))
+    {
+      D3D12_FEATURE_DATA_D3D12_OPTIONS5 *opts =
+          (D3D12_FEATURE_DATA_D3D12_OPTIONS5 *)pFeatureSupportData;
+      if(FeatureSupportDataSize != sizeof(D3D12_FEATURE_DATA_D3D12_OPTIONS5))
+        return E_INVALIDARG;
 
-    return S_OK;
+      // don't support raytracing
+      opts->RaytracingTier = D3D12_RAYTRACING_TIER_NOT_SUPPORTED;
+
+      return S_OK;
+    }
+
+    return hr;
   }
   return m_pDevice->CheckFeatureSupport(Feature, pFeatureSupportData, FeatureSupportDataSize);
 }
@@ -2595,6 +2820,6 @@ INSTANTIATE_FUNCTION_SERIALISED(void, WrappedID3D12Device, CreateCommandSignatur
                                 ID3D12RootSignature *pRootSignature, REFIID riid,
                                 void **ppvCommandSignature);
 INSTANTIATE_FUNCTION_SERIALISED(void, WrappedID3D12Device, DynamicDescriptorCopies,
-                                const std::vector<DynamicDescriptorCopy> &DescriptorCopies);
+                                const rdcarray<DynamicDescriptorCopy> &DescriptorCopies);
 INSTANTIATE_FUNCTION_SERIALISED(void, WrappedID3D12Device, OpenSharedHandle, HANDLE NTHandle,
                                 REFIID riid, void **ppvObj);
