@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,11 +32,11 @@ bool WrappedID3D12Device::Serialise_CreatePipelineState(SerialiserType &ser,
                                                         REFIID riid, void **ppPipelineState)
 {
   SERIALISE_ELEMENT_LOCAL(Descriptor, D3D12_EXPANDED_PIPELINE_STATE_STREAM_DESC(*pDesc))
-      .Named("pDesc");
-  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid");
+      .Named("pDesc"_lit);
+  SERIALISE_ELEMENT_LOCAL(guid, riid).Named("riid"_lit);
   SERIALISE_ELEMENT_LOCAL(pPipelineState,
                           ((WrappedID3D12PipelineState *)*ppPipelineState)->GetResourceID())
-      .TypedAs("ID3D12PipelineState *");
+      .TypedAs("ID3D12PipelineState *"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
