@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,7 @@
  ******************************************************************************/
 
 #include "d3d12_manager.h"
+#include <algorithm>
 #include "driver/dxgi/dxgi_common.h"
 #include "d3d12_command_list.h"
 #include "d3d12_command_queue.h"
@@ -165,7 +166,7 @@ void D3D12Descriptor::Create(D3D12_DESCRIPTOR_HEAP_TYPE heapType, WrappedID3D12D
       {
         desc = res ? NULL : defaultSRV();
 
-        const map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
+        const std::map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
 
         auto it = bbs.find(GetResID(res));
 
@@ -189,7 +190,7 @@ void D3D12Descriptor::Create(D3D12_DESCRIPTOR_HEAP_TYPE heapType, WrappedID3D12D
       }
       else if(desc->Format == DXGI_FORMAT_UNKNOWN)
       {
-        const map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
+        const std::map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
 
         auto it = bbs.find(GetResID(res));
 
@@ -257,7 +258,7 @@ void D3D12Descriptor::Create(D3D12_DESCRIPTOR_HEAP_TYPE heapType, WrappedID3D12D
       {
         desc = res ? NULL : defaultRTV();
 
-        const map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
+        const std::map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
 
         auto it = bbs.find(GetResID(res));
 
@@ -279,7 +280,7 @@ void D3D12Descriptor::Create(D3D12_DESCRIPTOR_HEAP_TYPE heapType, WrappedID3D12D
       }
       else if(desc->Format == DXGI_FORMAT_UNKNOWN)
       {
-        const map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
+        const std::map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
 
         auto it = bbs.find(GetResID(res));
 
@@ -366,7 +367,7 @@ void D3D12Descriptor::Create(D3D12_DESCRIPTOR_HEAP_TYPE heapType, WrappedID3D12D
       {
         desc = res ? NULL : defaultUAV();
 
-        const map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
+        const std::map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
 
         auto it = bbs.find(GetResID(res));
 
@@ -388,7 +389,7 @@ void D3D12Descriptor::Create(D3D12_DESCRIPTOR_HEAP_TYPE heapType, WrappedID3D12D
       }
       else if(desc->Format == DXGI_FORMAT_UNKNOWN)
       {
-        const map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
+        const std::map<ResourceId, DXGI_FORMAT> &bbs = dev->GetBackbufferFormats();
 
         auto it = bbs.find(GetResID(res));
 
@@ -498,9 +499,7 @@ void D3D12Descriptor::GetRefIDs(ResourceId &id, ResourceId &id2, FrameRefType &r
       id = WrappedID3D12Resource1::GetResIDFromAddr(data.nonsamp.cbv.BufferLocation);
       break;
     case D3D12DescriptorType::SRV: id = data.nonsamp.resource; break;
-    case D3D12DescriptorType::UAV:
-      id2 = data.nonsamp.counterResource;
-    // deliberate fall-through
+    case D3D12DescriptorType::UAV: id2 = data.nonsamp.counterResource; DELIBERATE_FALLTHROUGH();
     case D3D12DescriptorType::RTV:
     case D3D12DescriptorType::DSV:
       ref = eFrameRef_PartialWrite;
@@ -650,8 +649,8 @@ D3D12Descriptor *DescriptorFromPortableHandle(D3D12ResourceManager *manager, Por
 #define BARRIER_ASSERT(...)
 #endif
 
-void D3D12ResourceManager::ApplyBarriers(vector<D3D12_RESOURCE_BARRIER> &barriers,
-                                         map<ResourceId, SubresourceStateVector> &states)
+void D3D12ResourceManager::ApplyBarriers(rdcarray<D3D12_RESOURCE_BARRIER> &barriers,
+                                         std::map<ResourceId, SubresourceStateVector> &states)
 {
   for(size_t b = 0; b < barriers.size(); b++)
   {
@@ -684,7 +683,7 @@ void D3D12ResourceManager::ApplyBarriers(vector<D3D12_RESOURCE_BARRIER> &barrier
 
 template <typename SerialiserType>
 void D3D12ResourceManager::SerialiseResourceStates(SerialiserType &ser,
-                                                   std::vector<D3D12_RESOURCE_BARRIER> &barriers,
+                                                   rdcarray<D3D12_RESOURCE_BARRIER> &barriers,
                                                    std::map<ResourceId, SubresourceStateVector> &states)
 {
   SERIALISE_ELEMENT_LOCAL(NumMems, (uint32_t)states.size());
@@ -693,7 +692,7 @@ void D3D12ResourceManager::SerialiseResourceStates(SerialiserType &ser,
 
   for(uint32_t i = 0; i < NumMems; i++)
   {
-    SERIALISE_ELEMENT_LOCAL(Resource, srcit->first).TypedAs("ID3D12Resource *");
+    SERIALISE_ELEMENT_LOCAL(Resource, srcit->first).TypedAs("ID3D12Resource *"_lit);
     SERIALISE_ELEMENT_LOCAL(States, srcit->second);
 
     ResourceId liveid;
@@ -721,22 +720,18 @@ void D3D12ResourceManager::SerialiseResourceStates(SerialiserType &ser,
   }
 
   // erase any do-nothing barriers
-  for(auto it = barriers.begin(); it != barriers.end();)
-  {
-    if(it->Transition.StateBefore == it->Transition.StateAfter)
-      it = barriers.erase(it);
-    else
-      ++it;
-  }
+  barriers.removeIf([](const D3D12_RESOURCE_BARRIER &barrier) {
+    return barrier.Transition.StateBefore == barrier.Transition.StateAfter;
+  });
 
   ApplyBarriers(barriers, states);
 }
 
 template void D3D12ResourceManager::SerialiseResourceStates(
-    ReadSerialiser &ser, std::vector<D3D12_RESOURCE_BARRIER> &barriers,
+    ReadSerialiser &ser, rdcarray<D3D12_RESOURCE_BARRIER> &barriers,
     std::map<ResourceId, SubresourceStateVector> &states);
 template void D3D12ResourceManager::SerialiseResourceStates(
-    WriteSerialiser &ser, std::vector<D3D12_RESOURCE_BARRIER> &barriers,
+    WriteSerialiser &ser, rdcarray<D3D12_RESOURCE_BARRIER> &barriers,
     std::map<ResourceId, SubresourceStateVector> &states);
 
 void D3D12ResourceManager::SetInternalResource(ID3D12DeviceChild *res)
@@ -761,3 +756,63 @@ bool D3D12ResourceManager::ResourceTypeRelease(ID3D12DeviceChild *res)
 
   return true;
 }
+
+void GPUAddressRangeTracker::AddTo(const GPUAddressRange &range)
+{
+  SCOPED_WRITELOCK(addressLock);
+  auto it = std::lower_bound(addresses.begin(), addresses.end(), range.start);
+
+  addresses.insert(it - addresses.begin(), range);
+}
+
+void GPUAddressRangeTracker::RemoveFrom(const GPUAddressRange &range)
+{
+  {
+    SCOPED_WRITELOCK(addressLock);
+    size_t i = std::lower_bound(addresses.begin(), addresses.end(), range.start) - addresses.begin();
+
+    // there might be multiple buffers with the same range start, find the exact range for this
+    // buffer
+    while(i < addresses.size() && addresses[i].start == range.start)
+    {
+      if(addresses[i].id == range.id)
+      {
+        addresses.erase(i);
+        return;
+      }
+
+      ++i;
+    }
+  }
+
+  RDCERR("Couldn't find matching range to remove for %s", ToStr(range.id).c_str());
+}
+
+void GPUAddressRangeTracker::GetResIDFromAddr(D3D12_GPU_VIRTUAL_ADDRESS addr, ResourceId &id,
+                                              UINT64 &offs)
+{
+  id = ResourceId();
+  offs = 0;
+
+  if(addr == 0)
+    return;
+
+  GPUAddressRange range;
+
+  // this should really be a read-write lock
+  {
+    SCOPED_READLOCK(addressLock);
+
+    auto it = std::lower_bound(addresses.begin(), addresses.end(), addr);
+    if(it == addresses.end())
+      return;
+
+    range = *it;
+  }
+
+  if(addr < range.start || addr >= range.end)
+    return;
+
+  id = range.id;
+  offs = addr - range.start;
+}
