@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,17 +22,20 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "driver/dx/official/d3dcompiler.h"
 #include "driver/dxgi/dxgi_common.h"
 #include "d3d12_command_list.h"
 #include "d3d12_command_queue.h"
 #include "d3d12_debug.h"
 #include "d3d12_device.h"
 
+#include "driver/dx/official/d3dcompiler.h"
+
 void D3D12DebugManager::CopyTex2DMSToArray(ID3D12Resource *destArray, ID3D12Resource *srcMS)
 {
   // this function operates during capture so we work on unwrapped objects
 
+  D3D12MarkerRegion renderoverlay(m_pDevice->GetQueue(), "CopyTex2DMSToArray");
+
   D3D12_RESOURCE_DESC descMS = srcMS->GetDesc();
   D3D12_RESOURCE_DESC descArr = destArray->GetDesc();
 
@@ -100,8 +103,10 @@ void D3D12DebugManager::CopyTex2DMSToArray(ID3D12Resource *destArray, ID3D12Reso
       case DXGI_FORMAT_D16_UNORM:
       case DXGI_FORMAT_R16_TYPELESS:
         dsvDesc.Format = DXGI_FORMAT_D16_UNORM;
-        srvDesc.Format = DXGI_FORMAT_R16_FLOAT;
+        srvDesc.Format = DXGI_FORMAT_R16_UNORM;
         break;
+
+      default: break;
     }
   }
 
@@ -294,6 +299,8 @@ void D3D12DebugManager::CopyArrayToTex2DMS(ID3D12Resource *destMS, ID3D12Resourc
 {
   bool singleSliceMode = (selectedSlice != ~0U);
 
+  D3D12MarkerRegion renderoverlay(m_pDevice->GetQueue(), "CopyArrayToTex2DMS");
+
   D3D12_RESOURCE_DESC descArr = srcArray->GetDesc();
   D3D12_RESOURCE_DESC descMS = destMS->GetDesc();
 
@@ -364,8 +371,10 @@ void D3D12DebugManager::CopyArrayToTex2DMS(ID3D12Resource *destMS, ID3D12Resourc
       case DXGI_FORMAT_D16_UNORM:
       case DXGI_FORMAT_R16_TYPELESS:
         dsvDesc.Format = DXGI_FORMAT_D16_UNORM;
-        srvDesc.Format = DXGI_FORMAT_R16_FLOAT;
+        srvDesc.Format = DXGI_FORMAT_R16_UNORM;
         break;
+
+      default: break;
     }
   }
 
@@ -520,8 +529,6 @@ void D3D12DebugManager::CopyArrayToTex2DMS(ID3D12Resource *destMS, ID3D12Resourc
 
     dsvDesc.Flags = D3D12_DSV_FLAG_READ_ONLY_DEPTH;
     dsvDesc.Texture2DMSArray.ArraySize = 1;
-    m_pDevice->CreateDepthStencilView(destMS, &dsvDesc, dsv);
-    list->OMSetRenderTargets(0, NULL, FALSE, &dsv);
     list->SetPipelineState(psoStencil);
 
     // loop over every array slice in MS texture
@@ -530,7 +537,10 @@ void D3D12DebugManager::CopyArrayToTex2DMS(ID3D12Resource *destMS, ID3D12Resourc
       if(singleSliceMode)
         slice = selectedSlice;
 
-      dsvDesc.Texture2DArray.FirstArraySlice = slice;
+      dsvDesc.Texture2DMSArray.FirstArraySlice = slice;
+
+      m_pDevice->CreateDepthStencilView(destMS, &dsvDesc, dsv);
+      list->OMSetRenderTargets(0, NULL, FALSE, &dsv);
 
       // loop over every stencil value (in theory we could use SV_StencilRef, but it's optional
       // and would mean a different shader)
