@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,6 @@
 #include "data/resource.h"
 #include "driver/dx/official/d3dcompiler.h"
 #include "driver/dxgi/dxgi_common.h"
-#include "maths/camera.h"
 #include "maths/formatpacking.h"
 #include "maths/matrix.h"
 #include "maths/vec.h"
@@ -36,6 +35,7 @@
 #include "d3d12_command_queue.h"
 #include "d3d12_debug.h"
 #include "d3d12_device.h"
+#include "d3d12_replay.h"
 #include "d3d12_shader_cache.h"
 
 #include "data/hlsl/hlsl_cbuffers.h"
@@ -43,7 +43,7 @@
 struct D3D12QuadOverdrawCallback : public D3D12DrawcallCallback
 {
   D3D12QuadOverdrawCallback(WrappedID3D12Device *dev, D3D12_SHADER_BYTECODE quadWrite,
-                            const vector<uint32_t> &events, PortableHandle uav)
+                            const rdcarray<uint32_t> &events, PortableHandle uav)
       : m_pDevice(dev), m_QuadWritePS(quadWrite), m_Events(events), m_UAV(uav)
   {
     m_pDevice->GetQueue()->GetCommandData()->m_DrawcallCallback = this;
@@ -52,9 +52,9 @@ struct D3D12QuadOverdrawCallback : public D3D12DrawcallCallback
   {
     m_pDevice->GetQueue()->GetCommandData()->m_DrawcallCallback = NULL;
   }
-  void PreDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd)
+  void PreDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd)
   {
-    if(std::find(m_Events.begin(), m_Events.end(), eid) == m_Events.end())
+    if(!m_Events.contains(eid))
       return;
 
     // we customise the pipeline to disable framebuffer writes, but perform normal testing
@@ -81,30 +81,9 @@ struct D3D12QuadOverdrawCallback : public D3D12DrawcallCallback
 
       D3D12RootSignature modsig = sig->sig;
 
-      // make sure no other UAV tables overlap. We can't remove elements entirely because then the
-      // root signature indices wouldn't match up as expected.
-      // Instead move them into an unused space.
-      for(size_t i = 0; i < modsig.params.size(); i++)
-      {
-        if(modsig.params[i].ShaderVisibility == D3D12_SHADER_VISIBILITY_PIXEL ||
-           modsig.params[i].ShaderVisibility == D3D12_SHADER_VISIBILITY_ALL)
-        {
-          // use different register spaces for each just in case
-          UINT regSpace = modsig.maxSpaceIndex + modsig.params[i].ShaderVisibility;
-
-          if(modsig.params[i].ParameterType == D3D12_ROOT_PARAMETER_TYPE_UAV)
-          {
-            modsig.params[i].Descriptor.RegisterSpace = regSpace;
-          }
-          else if(modsig.params[i].ParameterType == D3D12_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE)
-          {
-            for(size_t r = 0; r < modsig.params[i].ranges.size(); r++)
-            {
-              modsig.params[i].ranges[r].RegisterSpace = regSpace;
-            }
-          }
-        }
-      }
+      UINT regSpace = modsig.maxSpaceIndex + 1;
+      MoveRootSignatureElementsToRegisterSpace(modsig, regSpace, D3D12DescriptorType::UAV,
+                                               D3D12_SHADER_VISIBILITY_PIXEL);
 
       D3D12_DESCRIPTOR_RANGE1 range;
       range.RangeType = D3D12_DESCRIPTOR_RANGE_TYPE_UAV;
@@ -114,19 +93,14 @@ struct D3D12QuadOverdrawCallback : public D3D12DrawcallCallback
       range.Flags = D3D12_DESCRIPTOR_RANGE_FLAG_NONE;
       range.OffsetInDescriptorsFromTableStart = 0;
 
-      modsig.params.push_back(D3D12RootSignatureParameter());
-      D3D12RootSignatureParameter &param = modsig.params.back();
+      modsig.Parameters.push_back(D3D12RootSignatureParameter());
+      D3D12RootSignatureParameter &param = modsig.Parameters.back();
       param.ParameterType = D3D12_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE;
       param.ShaderVisibility = D3D12_SHADER_VISIBILITY_PIXEL;
       param.DescriptorTable.NumDescriptorRanges = 1;
       param.DescriptorTable.pDescriptorRanges = &range;
 
-      cache.sigElem = uint32_t(modsig.params.size() - 1);
-
-      std::vector<D3D12_ROOT_PARAMETER1> params;
-      params.resize(modsig.params.size());
-      for(size_t i = 0; i < params.size(); i++)
-        params[i] = modsig.params[i];
+      cache.sigElem = uint32_t(modsig.Parameters.size() - 1);
 
       ID3DBlob *root = m_pDevice->GetShaderCache()->MakeRootSig(modsig);
 
@@ -171,49 +145,8 @@ struct D3D12QuadOverdrawCallback : public D3D12DrawcallCallback
     rs.pipe = GetResID(cache.pipe);
     rs.graphics.rootsig = GetResID(cache.sig);
 
-    if(rs.graphics.sigelems.size() <= cache.sigElem)
-      rs.graphics.sigelems.resize(cache.sigElem + 1);
-
-    PortableHandle uav = m_UAV;
-
-    // if a CBV_SRV_UAV heap is already set, we need to copy our descriptor in
-    // if we haven't already. Otherwise we can set our own heap.
-    for(size_t i = 0; i < rs.heaps.size(); i++)
-    {
-      WrappedID3D12DescriptorHeap *h =
-          m_pDevice->GetResourceManager()->GetCurrentAs<WrappedID3D12DescriptorHeap>(rs.heaps[i]);
-      if(h->GetDesc().Type == D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV)
-      {
-        // use the last descriptor
-        D3D12_CPU_DESCRIPTOR_HANDLE dst = h->GetCPUDescriptorHandleForHeapStart();
-        dst.ptr += (h->GetDesc().NumDescriptors - 1) * sizeof(D3D12Descriptor);
-
-        if(m_CopiedHeaps.find(rs.heaps[i]) == m_CopiedHeaps.end())
-        {
-          WrappedID3D12DescriptorHeap *h2 =
-              m_pDevice->GetResourceManager()->GetCurrentAs<WrappedID3D12DescriptorHeap>(m_UAV.heap);
-          D3D12_CPU_DESCRIPTOR_HANDLE src = h2->GetCPUDescriptorHandleForHeapStart();
-          src.ptr += m_UAV.index * sizeof(D3D12Descriptor);
-
-          // can't do a copy because the src heap is CPU write-only (shader visible). So instead,
-          // create directly
-          D3D12Descriptor *srcDesc = (D3D12Descriptor *)src.ptr;
-          srcDesc->Create(D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV, m_pDevice, dst);
-
-          m_CopiedHeaps.insert(rs.heaps[i]);
-        }
-
-        uav = ToPortableHandle(dst);
-
-        break;
-      }
-    }
-
-    if(uav.heap == m_UAV.heap)
-      rs.heaps.push_back(m_UAV.heap);
-
-    rs.graphics.sigelems[cache.sigElem] =
-        D3D12RenderState::SignatureElement(eRootTable, uav.heap, uav.index);
+    AddDebugDescriptorToRenderState(m_pDevice, rs, m_UAV, D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV,
+                                    cache.sigElem, m_CopiedHeaps);
 
     // as we're changing the root signature, we need to reapply all elements,
     // so just apply all state
@@ -221,9 +154,9 @@ struct D3D12QuadOverdrawCallback : public D3D12DrawcallCallback
       rs.ApplyState(m_pDevice, cmd);
   }
 
-  bool PostDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd)
+  bool PostDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd)
   {
-    if(std::find(m_Events.begin(), m_Events.end(), eid) == m_Events.end())
+    if(!m_Events.contains(eid))
       return false;
 
     // restore the render state and go ahead with the real draw
@@ -235,16 +168,16 @@ struct D3D12QuadOverdrawCallback : public D3D12DrawcallCallback
     return true;
   }
 
-  void PostRedraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd)
+  void PostRedraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd)
   {
     // nothing to do
   }
 
   // Dispatches don't rasterize, so do nothing
-  void PreDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) {}
-  bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) { return false; }
-  void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) {}
-  void PreCloseCommandList(ID3D12GraphicsCommandList4 *cmd) {}
+  void PreDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) {}
+  bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) { return false; }
+  void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) {}
+  void PreCloseCommandList(ID3D12GraphicsCommandListX *cmd) {}
   void AliasEvent(uint32_t primary, uint32_t alias)
   {
     // don't care
@@ -252,7 +185,7 @@ struct D3D12QuadOverdrawCallback : public D3D12DrawcallCallback
 
   WrappedID3D12Device *m_pDevice;
   D3D12_SHADER_BYTECODE m_QuadWritePS;
-  const vector<uint32_t> &m_Events;
+  const rdcarray<uint32_t> &m_Events;
   PortableHandle m_UAV;
 
   // cache modified pipelines
@@ -262,15 +195,16 @@ struct D3D12QuadOverdrawCallback : public D3D12DrawcallCallback
     uint32_t sigElem;
     ID3D12PipelineState *pipe;
   };
-  map<ResourceId, CachedPipeline> m_PipelineCache;
+  std::map<ResourceId, CachedPipeline> m_PipelineCache;
   std::set<ResourceId> m_CopiedHeaps;
   D3D12RenderState m_PrevState;
 };
 
-ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOverlay overlay,
-                                      uint32_t eventId, const vector<uint32_t> &passEvents)
+ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeCast, FloatVector clearCol,
+                                      DebugOverlay overlay, uint32_t eventId,
+                                      const rdcarray<uint32_t> &passEvents)
 {
-  ID3D12Resource *resource = WrappedID3D12Resource1::GetList()[texid];
+  ID3D12Resource *resource = m_pDevice->GetResourceList()[texid];
 
   if(resource == NULL)
     return ResourceId();
@@ -280,9 +214,9 @@ ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
 
   D3D12_RESOURCE_DESC resourceDesc = resource->GetDesc();
 
-  std::vector<D3D12_RESOURCE_BARRIER> barriers;
+  rdcarray<D3D12_RESOURCE_BARRIER> barriers;
   int resType = 0;
-  GetDebugManager()->PrepareTextureSampling(resource, typeHint, resType, barriers);
+  GetDebugManager()->PrepareTextureSampling(resource, typeCast, resType, barriers);
 
   D3D12_RESOURCE_DESC overlayTexDesc;
   overlayTexDesc.Alignment = 0;
@@ -371,10 +305,10 @@ ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
 
     ID3D12GraphicsCommandList *list = m_pDevice->GetNewList();
 
-    const vector<D3D12_RESOURCE_STATES> &states =
+    const rdcarray<D3D12_RESOURCE_STATES> &states =
         m_pDevice->GetSubresourceStates(GetResID(realDepth));
 
-    vector<D3D12_RESOURCE_BARRIER> depthBarriers;
+    rdcarray<D3D12_RESOURCE_BARRIER> depthBarriers;
     depthBarriers.reserve(states.size());
     for(size_t i = 0; i < states.size(); i++)
     {
@@ -713,7 +647,7 @@ ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
   }
   else if(overlay == DebugOverlay::ClearBeforePass || overlay == DebugOverlay::ClearBeforeDraw)
   {
-    vector<uint32_t> events = passEvents;
+    rdcarray<uint32_t> events = passEvents;
 
     if(overlay == DebugOverlay::ClearBeforeDraw)
       events.clear();
@@ -725,7 +659,7 @@ ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
       list->Close();
       list = NULL;
 
-      std::vector<D3D12Descriptor> rts = rs.rts;
+      rdcarray<D3D12Descriptor> rts = rs.rts;
 
       if(overlay == DebugOverlay::ClearBeforePass)
         m_pDevice->ReplayLog(0, events[0], eReplay_WithoutDraw);
@@ -738,7 +672,35 @@ ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
 
         if(desc.GetResResourceId() != ResourceId())
           Unwrap(list)->ClearRenderTargetView(Unwrap(GetDebugManager()->GetTempDescriptor(desc)),
-                                              black, 0, NULL);
+                                              &clearCol.x, 0, NULL);
+      }
+
+      // Try to clear depth as well, to help debug shadow rendering
+      if(rs.dsv.GetResResourceId() != ResourceId() && IsDepthFormat(resourceDesc.Format))
+      {
+        WrappedID3D12PipelineState *origPSO =
+            m_pDevice->GetResourceManager()->GetCurrentAs<WrappedID3D12PipelineState>(rs.pipe);
+        if(origPSO && origPSO->IsGraphics())
+        {
+          D3D12_COMPARISON_FUNC depthFunc = origPSO->graphics->DepthStencilState.DepthFunc;
+
+          // If the depth func is equal or not equal, don't clear at all since the output would be
+          // altered in an way that would cause replay to produce mostly incorrect results.
+          // Similarly, skip if the depth func is always, as we'd have a 50% chance of guessing the
+          // wrong clear value.
+          if(depthFunc != D3D12_COMPARISON_FUNC_EQUAL &&
+             depthFunc != D3D12_COMPARISON_FUNC_NOT_EQUAL &&
+             depthFunc != D3D12_COMPARISON_FUNC_ALWAYS)
+          {
+            // If the depth func is less or less equal, clear to 1 instead of 0
+            bool depthFuncLess = depthFunc == D3D12_COMPARISON_FUNC_LESS ||
+                                 depthFunc == D3D12_COMPARISON_FUNC_LESS_EQUAL;
+            float depthClear = depthFuncLess ? 1.0f : 0.0f;
+
+            Unwrap(list)->ClearDepthStencilView(Unwrap(GetDebugManager()->GetTempDescriptor(rs.dsv)),
+                                                D3D12_CLEAR_FLAG_DEPTH, depthClear, 0, 0, NULL);
+          }
+        }
       }
 
       list->Close();
@@ -825,7 +787,7 @@ ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
     {
       SCOPED_TIMER("Triangle size");
 
-      vector<uint32_t> events = passEvents;
+      rdcarray<uint32_t> events = passEvents;
 
       if(overlay == DebugOverlay::TriangleSizeDraw)
         events.clear();
@@ -836,7 +798,7 @@ ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
 
         // remove any non-drawcalls, like the pass boundary.
         if(!(draw->flags & DrawFlags::Drawcall))
-          events.erase(events.begin());
+          events.erase(0);
         else
           break;
       }
@@ -1019,7 +981,7 @@ ResourceId D3D12Replay::RenderOverlay(ResourceId texid, CompType typeHint, Debug
   {
     SCOPED_TIMER("Quad Overdraw");
 
-    vector<uint32_t> events = passEvents;
+    rdcarray<uint32_t> events = passEvents;
 
     if(overlay == DebugOverlay::QuadOverdrawDraw)
       events.clear();
