@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,15 +22,17 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
+#include <algorithm>
 #include "driver/dxgi/dxgi_common.h"
 #include "strings/string_utils.h"
 #include "d3d12_command_list.h"
 #include "d3d12_command_queue.h"
 #include "d3d12_debug.h"
 #include "d3d12_device.h"
+#include "d3d12_replay.h"
 #include "d3d12_shader_cache.h"
 
-void D3D12Replay::CreateSOBuffers()
+bool D3D12Replay::CreateSOBuffers()
 {
   HRESULT hr = S_OK;
 
@@ -39,6 +41,15 @@ void D3D12Replay::CreateSOBuffers()
   SAFE_RELEASE(m_SOPatchedIndexBuffer);
   SAFE_RELEASE(m_SOQueryHeap);
 
+  if(m_SOBufferSize >= 0xFFFF0000ULL)
+  {
+    RDCERR(
+        "Stream-out buffer size %llu is close to or over 4GB, out of memory very likely so "
+        "skipping",
+        m_SOBufferSize);
+    return false;
+  }
+
   D3D12_RESOURCE_DESC soBufDesc;
   soBufDesc.Alignment = 0;
   soBufDesc.DepthOrArraySize = 1;
@@ -65,14 +76,14 @@ void D3D12Replay::CreateSOBuffers()
                                           D3D12_RESOURCE_STATE_STREAM_OUT, NULL,
                                           __uuidof(ID3D12Resource), (void **)&m_SOBuffer);
 
-  m_SOBuffer->SetName(L"m_SOBuffer");
-
   if(FAILED(hr))
   {
     RDCERR("Failed to create SO output buffer, HRESULT: %s", ToStr(hr).c_str());
-    return;
+    return false;
   }
 
+  m_SOBuffer->SetName(L"m_SOBuffer");
+
   soBufDesc.Flags = D3D12_RESOURCE_FLAG_NONE;
   heapProps.Type = D3D12_HEAP_TYPE_READBACK;
 
@@ -80,14 +91,14 @@ void D3D12Replay::CreateSOBuffers()
                                           D3D12_RESOURCE_STATE_COPY_DEST, NULL,
                                           __uuidof(ID3D12Resource), (void **)&m_SOStagingBuffer);
 
-  m_SOStagingBuffer->SetName(L"m_SOStagingBuffer");
-
   if(FAILED(hr))
   {
     RDCERR("Failed to create readback buffer, HRESULT: %s", ToStr(hr).c_str());
-    return;
+    return false;
   }
 
+  m_SOStagingBuffer->SetName(L"m_SOStagingBuffer");
+
   // this is a buffer of unique indices, so it allows for
   // the worst case - float4 per vertex, all unique indices.
   soBufDesc.Width = m_SOBufferSize / sizeof(Vec4f);
@@ -97,14 +108,14 @@ void D3D12Replay::CreateSOBuffers()
       &heapProps, D3D12_HEAP_FLAG_NONE, &soBufDesc, D3D12_RESOURCE_STATE_GENERIC_READ, NULL,
       __uuidof(ID3D12Resource), (void **)&m_SOPatchedIndexBuffer);
 
-  m_SOPatchedIndexBuffer->SetName(L"m_SOPatchedIndexBuffer");
-
   if(FAILED(hr))
   {
     RDCERR("Failed to create SO index buffer, HRESULT: %s", ToStr(hr).c_str());
-    return;
+    return false;
   }
 
+  m_SOPatchedIndexBuffer->SetName(L"m_SOPatchedIndexBuffer");
+
   D3D12_QUERY_HEAP_DESC queryDesc;
   queryDesc.Count = 16;
   queryDesc.NodeMask = 1;
@@ -114,7 +125,7 @@ void D3D12Replay::CreateSOBuffers()
   if(FAILED(hr))
   {
     RDCERR("Failed to create SO query heap, HRESULT: %s", ToStr(hr).c_str());
-    return;
+    return false;
   }
 
   D3D12_UNORDERED_ACCESS_VIEW_DESC counterDesc = {};
@@ -128,6 +139,8 @@ void D3D12Replay::CreateSOBuffers()
 
   m_pDevice->CreateUnorderedAccessView(m_SOBuffer, NULL, &counterDesc,
                                        GetDebugManager()->GetUAVClearHandle(STREAM_OUT_UAV));
+
+  return true;
 }
 
 void D3D12Replay::ClearPostVSCache()
@@ -181,11 +194,11 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
   if(drawcall->numIndices == 0 || drawcall->numInstances == 0)
     return;
 
-  DXBC::DXBCFile *dxbcVS = vs->GetDXBC();
+  DXBC::DXBCContainer *dxbcVS = vs->GetDXBC();
 
   RDCASSERT(dxbcVS);
 
-  DXBC::DXBCFile *dxbcGS = NULL;
+  DXBC::DXBCContainer *dxbcGS = NULL;
 
   WrappedID3D12Shader *gs = origPSO->GS();
 
@@ -196,7 +209,7 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
     RDCASSERT(dxbcGS);
   }
 
-  DXBC::DXBCFile *dxbcDS = NULL;
+  DXBC::DXBCContainer *dxbcDS = NULL;
 
   WrappedID3D12Shader *ds = origPSO->DS();
 
@@ -236,15 +249,15 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
     }
   }
 
-  vector<D3D12_SO_DECLARATION_ENTRY> sodecls;
+  rdcarray<D3D12_SO_DECLARATION_ENTRY> sodecls;
 
   UINT stride = 0;
   int posidx = -1;
   int numPosComponents = 0;
 
-  if(!dxbcVS->m_OutputSig.empty())
+  if(!dxbcVS->GetReflection()->OutputSig.empty())
   {
-    for(const SigParameter &sign : dxbcVS->m_OutputSig)
+    for(const SigParameter &sign : dxbcVS->GetReflection()->OutputSig)
     {
       D3D12_SO_DECLARATION_ENTRY decl;
 
@@ -277,8 +290,8 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
     if(posidx > 0)
     {
       D3D12_SO_DECLARATION_ENTRY pos = sodecls[posidx];
-      sodecls.erase(sodecls.begin() + posidx);
-      sodecls.insert(sodecls.begin(), pos);
+      sodecls.erase(posidx);
+      sodecls.insert(0, pos);
     }
 
     // set up stream output entries and buffers
@@ -341,7 +354,7 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
       recreate = true;
     }
 
-    ID3D12GraphicsCommandList4 *list = NULL;
+    ID3D12GraphicsCommandListX *list = NULL;
 
     if(!(drawcall->flags & DrawFlags::Indexed))
     {
@@ -349,7 +362,12 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
       {
         m_pDevice->GPUSync();
 
-        CreateSOBuffers();
+        if(!CreateSOBuffers())
+        {
+          m_PostVSData[eventId].vsin.topo = topo;
+          m_PostVSData[eventId].vsout.topo = topo;
+          return;
+        }
       }
 
       list = GetDebugManager()->ResetDebugList();
@@ -381,7 +399,7 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
         GetBufferData(rs.ibuffer.buf, rs.ibuffer.offs + drawcall->indexOffset * rs.ibuffer.bytewidth,
                       RDCMIN(drawcall->numIndices * rs.ibuffer.bytewidth, rs.ibuffer.size), idxdata);
 
-      vector<uint32_t> indices;
+      rdcarray<uint32_t> indices;
 
       uint16_t *idx16 = (uint16_t *)&idxdata[0];
       uint32_t *idx32 = (uint32_t *)&idxdata[0];
@@ -412,13 +430,13 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
         if(it != indices.end() && *it == i32)
           continue;
 
-        indices.insert(it, i32);
+        indices.insert(it - indices.begin(), i32);
       }
 
       // if we read out of bounds, we'll also have a 0 index being referenced
       // (as 0 is read). Don't insert 0 if we already have 0 though
       if(numIndices < drawcall->numIndices && (indices.empty() || indices[0] != 0))
-        indices.insert(indices.begin(), 0);
+        indices.insert(0, 0);
 
       // An index buffer could be something like: 500, 501, 502, 501, 503, 502
       // in which case we can't use the existing index buffer without filling 499 slots of vertex
@@ -433,7 +451,7 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
 
       // we use a map here since the indices may be sparse. Especially considering if an index
       // is 'invalid' like 0xcccccccc then we don't want an array of 3.4 billion entries.
-      map<uint32_t, size_t> indexRemap;
+      std::map<uint32_t, size_t> indexRemap;
       for(size_t i = 0; i < indices.size(); i++)
       {
         // by definition, this index will only appear once in indices[]
@@ -454,7 +472,12 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
       {
         m_pDevice->GPUSync();
 
-        CreateSOBuffers();
+        if(!CreateSOBuffers())
+        {
+          m_PostVSData[eventId].vsin.topo = topo;
+          m_PostVSData[eventId].vsout.topo = topo;
+          return;
+        }
       }
 
       GetDebugManager()->FillBuffer(m_SOPatchedIndexBuffer, 0, &indices[0],
@@ -758,12 +781,12 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
     posidx = -1;
     numPosComponents = 0;
 
-    DXBC::DXBCFile *lastShader = dxbcGS;
+    DXBC::DXBCContainer *lastShader = dxbcGS;
     if(dxbcDS)
       lastShader = dxbcDS;
 
     sodecls.clear();
-    for(const SigParameter &sign : lastShader->m_OutputSig)
+    for(const SigParameter &sign : lastShader->GetReflection()->OutputSig)
     {
       D3D12_SO_DECLARATION_ENTRY decl;
 
@@ -794,8 +817,8 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
     if(posidx > 0)
     {
       D3D12_SO_DECLARATION_ENTRY pos = sodecls[posidx];
-      sodecls.erase(sodecls.begin() + posidx);
-      sodecls.insert(sodecls.begin(), pos);
+      sodecls.erase(posidx);
+      sodecls.insert(0, pos);
     }
 
     // enable the other shader stages again
@@ -826,7 +849,7 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
 
     D3D12_STREAM_OUTPUT_BUFFER_VIEW view;
 
-    ID3D12GraphicsCommandList4 *list = NULL;
+    ID3D12GraphicsCommandListX *list = NULL;
 
     view.BufferFilledSizeLocation = m_SOBuffer->GetGPUVirtualAddress();
     view.BufferLocation = m_SOBuffer->GetGPUVirtualAddress() + 64;
@@ -900,7 +923,13 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
         uint64_t oldSize = m_SOBufferSize;
         m_SOBufferSize = CalcMeshOutputSize(m_SOBufferSize, outputSize);
         RDCWARN("Resizing stream-out buffer from %llu to %llu for output", oldSize, m_SOBufferSize);
-        CreateSOBuffers();
+
+        if(!CreateSOBuffers())
+        {
+          m_PostVSData[eventId].vsin.topo = topo;
+          m_PostVSData[eventId].vsout.topo = topo;
+          return;
+        }
       }
 
       GetDebugManager()->ResetDebugAlloc();
@@ -1040,7 +1069,13 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
           uint64_t oldSize = m_SOBufferSize;
           m_SOBufferSize = CalcMeshOutputSize(m_SOBufferSize, outputSize);
           RDCWARN("Resizing stream-out buffer from %llu to %llu for output", oldSize, m_SOBufferSize);
-          CreateSOBuffers();
+
+          if(!CreateSOBuffers())
+          {
+            m_PostVSData[eventId].vsin.topo = topo;
+            m_PostVSData[eventId].vsout.topo = topo;
+            return;
+          }
 
           continue;
         }
@@ -1102,7 +1137,7 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
     uint64_t *counters = (uint64_t *)byteData;
 
     uint64_t numBytesWritten = 0;
-    std::vector<D3D12PostVSData::InstData> instData;
+    rdcarray<D3D12PostVSData::InstData> instData;
     if(drawcall->numInstances > 1)
     {
       uint64_t prevByteCount = 0;
@@ -1244,37 +1279,7 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
     m_PostVSData[eventId].gsout.hasPosOut = posidx >= 0;
     m_PostVSData[eventId].gsout.idxBuf = NULL;
 
-    topo = D3D_PRIMITIVE_TOPOLOGY_TRIANGLELIST;
-
-    if(lastShader == dxbcGS)
-    {
-      for(size_t i = 0; i < dxbcGS->GetNumDeclarations(); i++)
-      {
-        const DXBC::ASMDecl &decl = dxbcGS->GetDeclaration(i);
-
-        if(decl.declaration == DXBC::OPCODE_DCL_GS_OUTPUT_PRIMITIVE_TOPOLOGY)
-        {
-          topo = decl.outTopology;
-          break;
-        }
-      }
-    }
-    else if(lastShader == dxbcDS)
-    {
-      for(size_t i = 0; i < dxbcDS->GetNumDeclarations(); i++)
-      {
-        const DXBC::ASMDecl &decl = dxbcDS->GetDeclaration(i);
-
-        if(decl.declaration == DXBC::OPCODE_DCL_TESS_DOMAIN)
-        {
-          if(decl.domain == DXBC::DOMAIN_ISOLINE)
-            topo = D3D_PRIMITIVE_TOPOLOGY_LINELIST;
-          else
-            topo = D3D_PRIMITIVE_TOPOLOGY_TRIANGLELIST;
-          break;
-        }
-      }
-    }
+    topo = lastShader->GetOutputTopology();
 
     m_PostVSData[eventId].gsout.topo = topo;
 
@@ -1302,37 +1307,37 @@ void D3D12Replay::InitPostVSBuffers(uint32_t eventId)
 struct D3D12InitPostVSCallback : public D3D12DrawcallCallback
 {
   D3D12InitPostVSCallback(WrappedID3D12Device *dev, D3D12Replay *replay,
-                          const vector<uint32_t> &events)
+                          const rdcarray<uint32_t> &events)
       : m_pDevice(dev), m_Replay(replay), m_Events(events)
   {
     m_pDevice->GetQueue()->GetCommandData()->m_DrawcallCallback = this;
   }
   ~D3D12InitPostVSCallback() { m_pDevice->GetQueue()->GetCommandData()->m_DrawcallCallback = NULL; }
-  void PreDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override
+  void PreDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override
   {
-    if(std::find(m_Events.begin(), m_Events.end(), eid) != m_Events.end())
+    if(m_Events.contains(eid))
       m_Replay->InitPostVSBuffers(eid);
   }
 
-  bool PostDraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override { return false; }
-  void PostRedraw(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override {}
+  bool PostDraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override { return false; }
+  void PostRedraw(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override {}
   // Dispatches don't rasterize, so do nothing
-  void PreDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override {}
-  bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override { return false; }
-  void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandList4 *cmd) override {}
-  void PreCloseCommandList(ID3D12GraphicsCommandList4 *cmd) override {}
+  void PreDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override {}
+  bool PostDispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override { return false; }
+  void PostRedispatch(uint32_t eid, ID3D12GraphicsCommandListX *cmd) override {}
+  void PreCloseCommandList(ID3D12GraphicsCommandListX *cmd) override {}
   void AliasEvent(uint32_t primary, uint32_t alias) override
   {
-    if(std::find(m_Events.begin(), m_Events.end(), primary) != m_Events.end())
+    if(m_Events.contains(primary))
       m_Replay->AliasPostVSBuffers(primary, alias);
   }
 
   WrappedID3D12Device *m_pDevice;
   D3D12Replay *m_Replay;
-  const vector<uint32_t> &m_Events;
+  const rdcarray<uint32_t> &m_Events;
 };
 
-void D3D12Replay::InitPostVSBuffers(const vector<uint32_t> &events)
+void D3D12Replay::InitPostVSBuffers(const rdcarray<uint32_t> &events)
 {
   // first we must replay up to the first event without replaying it. This ensures any
   // non-command buffer calls like memory unmaps etc all happen correctly before this
