@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,6 +31,7 @@
 #include "d3d12_command_queue.h"
 #include "d3d12_debug.h"
 #include "d3d12_device.h"
+#include "d3d12_replay.h"
 
 #include "data/hlsl/hlsl_cbuffers.h"
 
@@ -80,6 +81,10 @@ MeshDisplayPipelines D3D12DebugManager::CacheMeshDisplayPipelines(const MeshForm
     key |= 1ULL << bit;
   bit++;
 
+  if(primary.allowRestart)
+    key |= 1ULL << bit;
+  bit++;
+
   // only 64 bits, make sure they all fit
   RDCASSERT(bit < 64);
 
@@ -100,6 +105,15 @@ MeshDisplayPipelines D3D12DebugManager::CacheMeshDisplayPipelines(const MeshForm
   pipeDesc.SampleMask = 0xFFFFFFFF;
   pipeDesc.SampleDesc.Count = D3D12_MSAA_SAMPLECOUNT;
   pipeDesc.IBStripCutValue = D3D12_INDEX_BUFFER_STRIP_CUT_VALUE_DISABLED;
+
+  if(primary.allowRestart)
+  {
+    if(primary.indexByteStride == 2)
+      pipeDesc.IBStripCutValue = D3D12_INDEX_BUFFER_STRIP_CUT_VALUE_0xFFFF;
+    else
+      pipeDesc.IBStripCutValue = D3D12_INDEX_BUFFER_STRIP_CUT_VALUE_0xFFFFFFFF;
+  }
+
   D3D_PRIMITIVE_TOPOLOGY topo = MakeD3DPrimitiveTopology(primary.topology);
 
   if(topo == D3D_PRIMITIVE_TOPOLOGY_POINTLIST ||
@@ -213,7 +227,7 @@ MeshDisplayPipelines D3D12DebugManager::CacheMeshDisplayPipelines(const MeshForm
   return cache;
 }
 
-void D3D12Replay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &secondaryDraws,
+void D3D12Replay::RenderMesh(uint32_t eventId, const rdcarray<MeshFormat> &secondaryDraws,
                              const MeshDisplay &cfg)
 {
   if(cfg.position.vertexResourceId == ResourceId() || cfg.position.numIndices == 0)
@@ -359,7 +373,7 @@ void D3D12Replay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &seconda
 
     // we source all data from the first instanced value in the instanced case, so make sure we
     // offset correctly here.
-    if(cfg.position.instanced)
+    if(cfg.position.instanced && cfg.position.instStepRate)
       offs += cfg.position.vertexByteStride * (cfg.curInstance / cfg.position.instStepRate);
 
     D3D12_VERTEX_BUFFER_VIEW view;
@@ -392,7 +406,7 @@ void D3D12Replay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &seconda
 
     // we source all data from the first instanced value in the instanced case, so make sure we
     // offset correctly here.
-    if(cfg.second.instanced)
+    if(cfg.second.instanced && cfg.second.instStepRate)
       offs += cfg.second.vertexByteStride * (cfg.curInstance / cfg.second.instStepRate);
 
     D3D12_VERTEX_BUFFER_VIEW view;
@@ -411,7 +425,12 @@ void D3D12Replay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &seconda
     {
       default:
       case SolidShade::Solid: pipe = cache.pipes[MeshDisplayPipelines::ePipe_SolidDepth]; break;
-      case SolidShade::Lit: pipe = cache.pipes[MeshDisplayPipelines::ePipe_Lit]; break;
+      case SolidShade::Lit:
+        pipe = cache.pipes[MeshDisplayPipelines::ePipe_Lit];
+        // point list topologies don't have lighting obvious, just render them as solid
+        if(!pipe)
+          pipe = cache.pipes[MeshDisplayPipelines::ePipe_SolidDepth];
+        break;
       case SolidShade::Secondary: pipe = cache.pipes[MeshDisplayPipelines::ePipe_Secondary]; break;
     }
 
@@ -645,16 +664,16 @@ void D3D12Replay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &seconda
     FloatVector activeVertex;
 
     // primitive this vert is a part of (red prim, optional)
-    vector<FloatVector> activePrim;
+    rdcarray<FloatVector> activePrim;
 
     // for patch lists, to show other verts in patch (green dots, optional)
     // for non-patch lists, we use the activePrim and adjacentPrimVertices
     // to show what other verts are related
-    vector<FloatVector> inactiveVertices;
+    rdcarray<FloatVector> inactiveVertices;
 
     // adjacency (line or tri, strips or lists) (green prims, optional)
     // will be N*M long, N adjacent prims of M verts each. M = primSize below
-    vector<FloatVector> adjacentPrimVertices;
+    rdcarray<FloatVector> adjacentPrimVertices;
 
     helper.topology = Topology::TriangleList;
     uint32_t primSize = 3;    // number of verts per primitive
@@ -777,7 +796,7 @@ void D3D12Replay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &seconda
 
       if(!inactiveVertices.empty())
       {
-        std::vector<FloatVector> inactiveVB;
+        rdcarray<FloatVector> inactiveVB;
         inactiveVB.reserve(inactiveVertices.size() * 4);
 
         for(size_t i = 0; i < inactiveVertices.size(); i++)
