@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,19 +29,20 @@
 #include "d3d12_command_queue.h"
 #include "d3d12_debug.h"
 #include "d3d12_device.h"
+#include "d3d12_replay.h"
 
 #include "data/hlsl/hlsl_cbuffers.h"
 
-void D3D12DebugManager::PrepareTextureSampling(ID3D12Resource *resource, CompType typeHint,
+void D3D12DebugManager::PrepareTextureSampling(ID3D12Resource *resource, CompType typeCast,
                                                int &resType,
-                                               std::vector<D3D12_RESOURCE_BARRIER> &barriers)
+                                               rdcarray<D3D12_RESOURCE_BARRIER> &barriers)
 {
   int srvOffset = 0;
 
   D3D12_RESOURCE_DESC resourceDesc = resource->GetDesc();
 
   D3D12_SHADER_RESOURCE_VIEW_DESC srvDesc = {};
-  srvDesc.Format = GetTypedFormat(resourceDesc.Format, typeHint);
+  srvDesc.Format = GetTypedFormat(resourceDesc.Format, typeCast);
   srvDesc.Shader4ComponentMapping = D3D12_DEFAULT_SHADER_4_COMPONENT_MAPPING;
 
   if(resourceDesc.Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE3D)
@@ -154,6 +155,7 @@ void D3D12DebugManager::PrepareTextureSampling(ID3D12Resource *resource, CompTyp
         altSRVDesc = srvDesc;
         altSRVDesc.Format = DXGI_FORMAT_X24_TYPELESS_G8_UINT;
         break;
+      default: break;
     }
   }
 
@@ -168,7 +170,7 @@ void D3D12DebugManager::PrepareTextureSampling(ID3D12Resource *resource, CompTyp
   }
 
   // transition resource to D3D12_RESOURCE_STATE_PIXEL_SHADER_RESOURCE
-  const vector<D3D12_RESOURCE_STATES> &states = m_pDevice->GetSubresourceStates(GetResID(resource));
+  const rdcarray<D3D12_RESOURCE_STATES> &states = m_pDevice->GetSubresourceStates(GetResID(resource));
 
   barriers.reserve(states.size());
   for(size_t i = 0; i < states.size(); i++)
@@ -331,7 +333,7 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
 {
   const bool blendAlpha = (flags & eTexDisplay_BlendAlpha) != 0;
 
-  ID3D12Resource *resource = WrappedID3D12Resource1::GetList()[cfg.resourceId];
+  ID3D12Resource *resource = m_pDevice->GetResourceList()[cfg.resourceId];
 
   if(resource == NULL)
     return false;
@@ -366,12 +368,6 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
   vertexData.Position.x = x * (2.0f / m_OutputWidth);
   vertexData.Position.y = -y * (2.0f / m_OutputHeight);
 
-  vertexData.ScreenAspect.x = m_OutputHeight / m_OutputWidth;
-  vertexData.ScreenAspect.y = 1.0f;
-
-  vertexData.TextureResolution.x = 1.0f / vertexData.ScreenAspect.x;
-  vertexData.TextureResolution.y = 1.0f;
-
   if(cfg.rangeMax <= cfg.rangeMin)
     cfg.rangeMax += 0.00001f;
 
@@ -397,10 +393,10 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
 
   D3D12_RESOURCE_DESC resourceDesc = resource->GetDesc();
 
-  pixelData.SampleIdx = (int)RDCCLAMP(cfg.sampleIdx, 0U, resourceDesc.SampleDesc.Count - 1);
+  pixelData.SampleIdx = (int)RDCCLAMP(cfg.subresource.sample, 0U, resourceDesc.SampleDesc.Count - 1);
 
   // hacky resolve
-  if(cfg.sampleIdx == ~0U)
+  if(cfg.subresource.sample == ~0U)
     pixelData.SampleIdx = -int(resourceDesc.SampleDesc.Count);
 
   if(resourceDesc.Format == DXGI_FORMAT_UNKNOWN)
@@ -416,18 +412,16 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
   float tex_y =
       float(resourceDesc.Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE1D ? 100 : resourceDesc.Height);
 
-  vertexData.TextureResolution.x *= tex_x / m_OutputWidth;
-  vertexData.TextureResolution.y *= tex_y / m_OutputHeight;
-
-  pixelData.TextureResolutionPS.x = float(RDCMAX(1U, uint32_t(resourceDesc.Width >> cfg.mip)));
-  pixelData.TextureResolutionPS.y = float(RDCMAX(1U, uint32_t(resourceDesc.Height >> cfg.mip)));
+  pixelData.TextureResolutionPS.x =
+      float(RDCMAX(1U, uint32_t(resourceDesc.Width >> cfg.subresource.mip)));
+  pixelData.TextureResolutionPS.y =
+      float(RDCMAX(1U, uint32_t(resourceDesc.Height >> cfg.subresource.mip)));
   pixelData.TextureResolutionPS.z =
-      float(RDCMAX(1U, uint32_t(resourceDesc.DepthOrArraySize >> cfg.mip)));
+      float(RDCMAX(1U, uint32_t(resourceDesc.DepthOrArraySize >> cfg.subresource.mip)));
 
   if(resourceDesc.DepthOrArraySize > 1 && resourceDesc.Dimension != D3D12_RESOURCE_DIMENSION_TEXTURE3D)
     pixelData.TextureResolutionPS.z = float(resourceDesc.DepthOrArraySize);
 
-  vertexData.Scale = cfg.scale;
   pixelData.ScalePS = cfg.scale;
 
   if(cfg.scale <= 0.0f)
@@ -435,32 +429,39 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
     float xscale = m_OutputWidth / tex_x;
     float yscale = m_OutputHeight / tex_y;
 
-    vertexData.Scale = RDCMIN(xscale, yscale);
+    cfg.scale = RDCMIN(xscale, yscale);
 
     if(yscale > xscale)
     {
       vertexData.Position.x = 0;
-      vertexData.Position.y = tex_y * vertexData.Scale / m_OutputHeight - 1.0f;
+      vertexData.Position.y = tex_y * cfg.scale / m_OutputHeight - 1.0f;
     }
     else
     {
       vertexData.Position.y = 0;
-      vertexData.Position.x = 1.0f - tex_x * vertexData.Scale / m_OutputWidth;
+      vertexData.Position.x = 1.0f - tex_x * cfg.scale / m_OutputWidth;
     }
   }
 
-  vertexData.Scale *= 2.0f;    // viewport is -1 -> 1
+  // normalisation factor for output * selected scale * viewport scale
+  vertexData.VertexScale.x = (tex_x / m_OutputWidth) * cfg.scale * 2.0f;
+  vertexData.VertexScale.y = (tex_y / m_OutputHeight) * cfg.scale * 2.0f;
 
-  pixelData.MipLevel = (float)cfg.mip;
+  pixelData.MipLevel = (float)cfg.subresource.mip;
   pixelData.OutputDisplayFormat = RESTYPE_TEX2D;
-  pixelData.Slice = float(RDCCLAMP(cfg.sliceFace, 0U, uint32_t(resourceDesc.DepthOrArraySize - 1)));
 
   if(resourceDesc.Dimension == D3D12_RESOURCE_DIMENSION_TEXTURE3D)
-    pixelData.Slice = float(cfg.sliceFace >> cfg.mip);
+    pixelData.Slice =
+        float(RDCCLAMP(cfg.subresource.slice, 0U,
+                       uint32_t((resourceDesc.DepthOrArraySize >> cfg.subresource.mip) - 1)) +
+              0.001f);
+  else
+    pixelData.Slice = float(
+        RDCCLAMP(cfg.subresource.slice, 0U, uint32_t(resourceDesc.DepthOrArraySize - 1)) + 0.001f);
 
-  std::vector<D3D12_RESOURCE_BARRIER> barriers;
+  rdcarray<D3D12_RESOURCE_BARRIER> barriers;
   int resType = 0;
-  GetDebugManager()->PrepareTextureSampling(resource, cfg.typeHint, resType, barriers);
+  GetDebugManager()->PrepareTextureSampling(resource, cfg.typeCast, resType, barriers);
 
   pixelData.OutputDisplayFormat = resType;
 
@@ -475,7 +476,9 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
   else if(IsIntFormat(resourceDesc.Format))
     pixelData.OutputDisplayFormat |= TEXDISPLAY_SINT_TEX;
 
-  if(!IsSRGBFormat(resourceDesc.Format) && cfg.linearDisplayAsGamma)
+  // Check both the resource format and view format for sRGB
+  if(!IsSRGBFormat(resourceDesc.Format) && cfg.typeCast != CompType::UNormSRGB &&
+     cfg.linearDisplayAsGamma)
     pixelData.OutputDisplayFormat |= TEXDISPLAY_GAMMA_CURVE;
 
   Vec4u YUVDownsampleRate = {}, YUVAChannels = {};
@@ -525,14 +528,14 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
     if(FAILED(hr))
       return false;
 
-    DXBC::DXBCFile *dxbc = shader->GetDXBC();
+    DXBC::DXBCContainer *dxbc = shader->GetDXBC();
 
     RDCASSERT(dxbc);
-    RDCASSERT(dxbc->m_Type == D3D11_ShaderType_Pixel);
+    RDCASSERT(dxbc->m_Type == DXBC::ShaderType::Pixel);
 
-    for(size_t i = 0; i < dxbc->m_CBuffers.size(); i++)
+    for(size_t i = 0; i < dxbc->GetReflection()->CBuffers.size(); i++)
     {
-      const DXBC::CBuffer &cbuf = dxbc->m_CBuffers[i];
+      const DXBC::CBuffer &cbuf = dxbc->GetReflection()->CBuffers[i];
       if(cbuf.name == "$Globals")
       {
         float *cbufData = new float[cbuf.descriptor.byteSize / sizeof(float) + 1];
@@ -583,7 +586,7 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
             {
               uint32_t *d = (uint32_t *)(byteData + var.descriptor.offset);
 
-              d[0] = cfg.mip;
+              d[0] = cfg.subresource.mip;
             }
             else
             {
@@ -598,7 +601,7 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
             {
               uint32_t *d = (uint32_t *)(byteData + var.descriptor.offset);
 
-              d[0] = cfg.sliceFace;
+              d[0] = cfg.subresource.slice;
             }
             else
             {
@@ -613,7 +616,7 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
             {
               int32_t *d = (int32_t *)(byteData + var.descriptor.offset);
 
-              d[0] = cfg.sampleIdx;
+              d[0] = cfg.subresource.sample;
             }
             else
             {
@@ -673,11 +676,31 @@ bool D3D12Replay::RenderTextureInternal(D3D12_CPU_DESCRIPTOR_HANDLE rtv, Texture
     {
       list->SetPipelineState(customPSO);
     }
+    else if(flags & (eTexDisplay_RemapFloat | eTexDisplay_RemapUInt | eTexDisplay_RemapSInt))
+    {
+      int i = 0;
+      if(flags & eTexDisplay_RemapFloat)
+        i = 0;
+      else if(flags & eTexDisplay_RemapUInt)
+        i = 1;
+      else if(flags & eTexDisplay_RemapSInt)
+        i = 2;
+
+      int f = 0;
+      if(flags & eTexDisplay_32Render)
+        f = 2;
+      else if(flags & eTexDisplay_16Render)
+        f = 1;
+      else
+        f = 0;
+
+      list->SetPipelineState(m_TexRender.m_TexRemapPipe[f][i]);
+    }
     else if(cfg.rawOutput || !blendAlpha || cfg.customShaderId != ResourceId())
     {
-      if(flags & eTexDisplay_F32Render)
+      if(flags & eTexDisplay_32Render)
         list->SetPipelineState(m_TexRender.F32Pipe);
-      else if(flags & eTexDisplay_F16Render)
+      else if(flags & eTexDisplay_16Render)
         list->SetPipelineState(m_TexRender.F16Pipe);
       else if(flags & eTexDisplay_LinearRender)
         list->SetPipelineState(m_TexRender.LinearPipe);
