@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,6 @@
 #include "d3d12_command_queue.h"
 
 GPUAddressRangeTracker WrappedID3D12Resource1::m_Addresses;
-std::map<ResourceId, WrappedID3D12Resource1 *> *WrappedID3D12Resource1::m_List = NULL;
 std::map<WrappedID3D12PipelineState::DXBCKey, WrappedID3D12Shader *> WrappedID3D12Shader::m_Shaders;
 bool WrappedID3D12Shader::m_InternalResources = false;
 
@@ -37,13 +36,13 @@ const GUID RENDERDOC_ID3D12ShaderGUID_ShaderDebugMagicValue = RENDERDOC_ShaderDe
 
 void WrappedID3D12Shader::TryReplaceOriginalByteCode()
 {
-  if(!DXBC::DXBCFile::CheckForDebugInfo((const void *)&m_Bytecode[0], m_Bytecode.size()))
+  if(!DXBC::DXBCContainer::CheckForDebugInfo((const void *)&m_Bytecode[0], m_Bytecode.size()))
   {
-    string originalPath = m_DebugInfoPath;
+    rdcstr originalPath = m_DebugInfoPath;
 
     if(originalPath.empty())
       originalPath =
-          DXBC::DXBCFile::GetDebugBinaryPath((const void *)&m_Bytecode[0], m_Bytecode.size());
+          DXBC::DXBCContainer::GetDebugBinaryPath((const void *)&m_Bytecode[0], m_Bytecode.size());
 
     if(!originalPath.empty())
     {
@@ -60,7 +59,7 @@ void WrappedID3D12Shader::TryReplaceOriginalByteCode()
 
       size_t numSearchPaths = m_DebugInfoSearchPaths ? m_DebugInfoSearchPaths->size() : 0;
 
-      string foundPath;
+      rdcstr foundPath;
 
       // while we haven't found a file, keep trying through the search paths. For i==0
       // check the path on its own, in case it's an absolute path.
@@ -74,7 +73,7 @@ void WrappedID3D12Shader::TryReplaceOriginalByteCode()
         }
         else
         {
-          const std::string &searchPath = (*m_DebugInfoSearchPaths)[i - 1];
+          const rdcstr &searchPath = (*m_DebugInfoSearchPaths)[i - 1];
           foundPath = searchPath + "/" + originalPath;
           originalShaderFile = FileIO::fopen(foundPath.c_str(), "rb");
         }
@@ -89,7 +88,7 @@ void WrappedID3D12Shader::TryReplaceOriginalByteCode()
 
       if(lz4 || originalShaderSize >= m_Bytecode.size())
       {
-        vector<byte> originalBytecode;
+        rdcarray<byte> originalBytecode;
 
         originalBytecode.resize((size_t)originalShaderSize);
         FileIO::fread(&originalBytecode[0], sizeof(byte), (size_t)originalShaderSize,
@@ -97,7 +96,7 @@ void WrappedID3D12Shader::TryReplaceOriginalByteCode()
 
         if(lz4)
         {
-          vector<byte> decompressed;
+          rdcarray<byte> decompressed;
 
           // first try decompressing to 1MB flat
           decompressed.resize(100 * 1024);
@@ -129,8 +128,8 @@ void WrappedID3D12Shader::TryReplaceOriginalByteCode()
           memcpy(&originalBytecode[0], &decompressed[0], originalBytecode.size());
         }
 
-        if(DXBC::DXBCFile::CheckForDebugInfo((const void *)&originalBytecode[0],
-                                             originalBytecode.size()))
+        if(DXBC::DXBCContainer::CheckForDebugInfo((const void *)&originalBytecode[0],
+                                                  originalBytecode.size()))
         {
           m_Bytecode.swap(originalBytecode);
         }
@@ -304,8 +303,8 @@ WrappedID3D12Resource1::~WrappedID3D12Resource1()
     }
   }
 
-  if(m_List)
-    (*m_List).erase(GetResourceID());
+  if(IsReplayMode(m_pDevice->GetState()))
+    m_pDevice->GetResourceList().erase(GetResourceID());
 
   // assuming only valid for buffers
   if(m_pReal->GetDesc().Dimension == D3D12_RESOURCE_DIMENSION_BUFFER)
@@ -325,6 +324,8 @@ WrappedID3D12Resource1::~WrappedID3D12Resource1()
 
 byte *WrappedID3D12Resource1::GetMap(UINT Subresource)
 {
+  SCOPED_LOCK(GetResourceRecord()->m_MapLock);
+
   D3D12ResourceRecord::MapData *map = GetResourceRecord()->m_Maps;
   size_t mapcount = GetResourceRecord()->m_MapsCount;
 
@@ -336,6 +337,8 @@ byte *WrappedID3D12Resource1::GetMap(UINT Subresource)
 
 byte *WrappedID3D12Resource1::GetShadow(UINT Subresource)
 {
+  SCOPED_LOCK(GetResourceRecord()->m_MapLock);
+
   D3D12ResourceRecord::MapData *map = GetResourceRecord()->m_Maps;
 
   return map[Subresource].shadowPtr;
@@ -343,6 +346,8 @@ byte *WrappedID3D12Resource1::GetShadow(UINT Subresource)
 
 void WrappedID3D12Resource1::AllocShadow(UINT Subresource, size_t size)
 {
+  SCOPED_LOCK(GetResourceRecord()->m_MapLock);
+
   D3D12ResourceRecord::MapData *map = GetResourceRecord()->m_Maps;
 
   if(map[Subresource].shadowPtr == NULL)
@@ -351,6 +356,8 @@ void WrappedID3D12Resource1::AllocShadow(UINT Subresource, size_t size)
 
 void WrappedID3D12Resource1::FreeShadow()
 {
+  SCOPED_LOCK(GetResourceRecord()->m_MapLock);
+
   D3D12ResourceRecord::MapData *map = GetResourceRecord()->m_Maps;
   size_t mapcount = GetResourceRecord()->m_MapsCount;
 
@@ -383,14 +390,15 @@ HRESULT STDMETHODCALLTYPE WrappedID3D12Resource1::Map(UINT Subresource,
 
   if(SUCCEEDED(hr) && GetResourceRecord())
   {
+    SCOPED_LOCK(GetResourceRecord()->m_MapLock);
+
     D3D12ResourceRecord::MapData *map = GetResourceRecord()->m_Maps;
 
     map[Subresource].realPtr = (byte *)mapPtr;
-
-    int32_t refcount = Atomic::Inc32(&map[Subresource].refcount);
+    map[Subresource].refcount++;
 
     // on the first map, register this so we can flush any updates in case it's left persistant
-    if(refcount == 1)
+    if(map[Subresource].refcount == 1)
       m_pDevice->Map(this, Subresource);
   }
 
@@ -403,20 +411,23 @@ void STDMETHODCALLTYPE WrappedID3D12Resource1::Unmap(UINT Subresource,
   if(GetResourceRecord())
   {
     D3D12ResourceRecord::MapData *map = GetResourceRecord()->m_Maps;
-    size_t mapcount = GetResourceRecord()->m_MapsCount;
 
-    // may not have a map if e.g. no pointer was requested
-    if(Subresource < mapcount)
     {
-      int32_t refcount = Atomic::Dec32(&map[Subresource].refcount);
+      SCOPED_LOCK(GetResourceRecord()->m_MapLock);
 
-      if(refcount == 0)
+      // may not have a ref at all if e.g. no pointer was requested
+      if(map[Subresource].refcount >= 1)
       {
-        m_pDevice->Unmap(this, Subresource, map[Subresource].realPtr, pWrittenRange);
+        map[Subresource].refcount--;
 
-        FreeAlignedBuffer(map[Subresource].shadowPtr);
-        map[Subresource].realPtr = NULL;
-        map[Subresource].shadowPtr = NULL;
+        if(map[Subresource].refcount == 0)
+        {
+          m_pDevice->Unmap(this, Subresource, map[Subresource].realPtr, pWrittenRange);
+
+          FreeAlignedBuffer(map[Subresource].shadowPtr);
+          map[Subresource].realPtr = NULL;
+          map[Subresource].shadowPtr = NULL;
+        }
       }
     }
   }
@@ -437,17 +448,8 @@ HRESULT STDMETHODCALLTYPE WrappedID3D12Resource1::WriteToSubresource(UINT DstSub
 
   if(GetResourceRecord())
   {
-    size_t mapcount = GetResourceRecord()->m_MapsCount;
-
-    if(DstSubresource < mapcount)
-    {
-      m_pDevice->WriteToSubresource(this, DstSubresource, pDstBox, pSrcData, SrcDepthPitch,
-                                    SrcDepthPitch);
-    }
-    else
-    {
-      RDCERR("WriteToSubresource without matching map!");
-    }
+    m_pDevice->WriteToSubresource(this, DstSubresource, pDstBox, pSrcData, SrcRowPitch,
+                                  SrcDepthPitch);
   }
 
   return ret;
