@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #pragma once
 
-#include "driver/shaders/dxbc/dxbc_inspect.h"
+#include "driver/shaders/dxbc/dxbc_container.h"
 #include "d3d12_device.h"
 #include "d3d12_manager.h"
 
@@ -129,14 +129,17 @@ public:
       AddRef();
       return S_OK;
     }
-    if(riid == __uuidof(NestedType))
+    else if(riid == __uuidof(NestedType))
     {
       *ppvObject = (NestedType *)this;
       AddRef();
       return S_OK;
     }
-    if(riid == __uuidof(NestedType1) && m_pReal)
+    else if(riid == __uuidof(NestedType1))
     {
+      if(!m_pReal)
+        return E_NOINTERFACE;
+
       // check that the real interface supports this
       NestedType1 *dummy = NULL;
       HRESULT check = m_pReal->QueryInterface(riid, (void **)&dummy);
@@ -150,8 +153,11 @@ public:
       AddRef();
       return S_OK;
     }
-    if(riid == __uuidof(NestedType2) && m_pReal)
+    else if(riid == __uuidof(NestedType2))
     {
+      if(!m_pReal)
+        return E_NOINTERFACE;
+
       // check that the real interface supports this
       NestedType2 *dummy = NULL;
       HRESULT check = m_pReal->QueryInterface(riid, (void **)&dummy);
@@ -165,13 +171,32 @@ public:
       AddRef();
       return S_OK;
     }
-    if(riid == __uuidof(ID3D12Object))
+    else if(riid == __uuidof(ID3D12Pageable))
+    {
+      // not all child classes support this, so check it on the real interface
+      if(!m_pReal)
+        return E_NOINTERFACE;
+
+      // check that the real interface supports this
+      ID3D12Pageable *dummy = NULL;
+      HRESULT check = m_pReal->QueryInterface(riid, (void **)&dummy);
+
+      SAFE_RELEASE(dummy);
+
+      if(FAILED(check))
+        return check;
+
+      *ppvObject = (ID3D12Pageable *)this;
+      AddRef();
+      return S_OK;
+    }
+    else if(riid == __uuidof(ID3D12Object))
     {
       *ppvObject = (ID3D12DeviceChild *)this;
       AddRef();
       return S_OK;
     }
-    if(riid == __uuidof(ID3D12DeviceChild))
+    else if(riid == __uuidof(ID3D12DeviceChild))
     {
       *ppvObject = (ID3D12DeviceChild *)this;
       AddRef();
@@ -270,7 +295,15 @@ public:
       return m_pDevice->SetShaderDebugPath(this, (const char *)pData);
 
     if(guid == WKPDID_D3DDebugObjectName)
+    {
       m_pDevice->SetName(this, (const char *)pData);
+    }
+    else if(guid == WKPDID_D3DDebugObjectNameW)
+    {
+      rdcwstr wName((const wchar_t *)pData, DataSize / 2);
+      rdcstr sName = StringFormat::Wide2UTF8(wName);
+      m_pDevice->SetName(this, sName.c_str());
+    }
 
     if(!m_pReal)
       return S_OK;
@@ -288,7 +321,7 @@ public:
 
   HRESULT STDMETHODCALLTYPE SetName(LPCWSTR Name)
   {
-    string utf8 = Name ? StringFormat::Wide2UTF8(Name) : "";
+    rdcstr utf8 = Name ? StringFormat::Wide2UTF8(Name) : "";
     m_pDevice->SetName(this, utf8.c_str());
 
     if(!m_pReal)
@@ -606,7 +639,7 @@ public:
     DXBCKey(const D3D12_SHADER_BYTECODE &byteCode)
     {
       byteLen = (uint32_t)byteCode.BytecodeLength;
-      DXBC::DXBCFile::GetHash(hash, byteCode.pShaderBytecode, byteCode.BytecodeLength);
+      DXBC::DXBCContainer::GetHash(hash, byteCode.pShaderBytecode, byteCode.BytecodeLength);
     }
 
     // assume that byte length + hash is enough to uniquely identify a shader bytecode
@@ -649,8 +682,7 @@ public:
     ShaderEntry(const D3D12_SHADER_BYTECODE &byteCode, WrappedID3D12Device *device)
         : WrappedDeviceChild12(NULL, device), m_Key(byteCode)
     {
-      const byte *code = (const byte *)byteCode.pShaderBytecode;
-      m_Bytecode.assign(code, code + byteCode.BytecodeLength);
+      m_Bytecode.assign((const byte *)byteCode.pShaderBytecode, byteCode.BytecodeLength);
       m_DebugInfoSearchPaths = NULL;
       m_DXBCFile = NULL;
 
@@ -660,7 +692,7 @@ public:
       {
         device->AddResource(GetResourceID(), ResourceType::Shader, "Shader");
 
-        ResourceDescription &desc = device->GetReplay()->GetResourceDesc(GetResourceID());
+        ResourceDescription &desc = device->GetResourceDesc(GetResourceID());
         // this will be appended to in the function above.
         desc.initialisationChunks.clear();
 
@@ -691,10 +723,6 @@ public:
       else
         shader->AddRef();
 
-      if(pipeline &&
-         std::find(shader->m_Pipes.begin(), shader->m_Pipes.end(), pipeline) == shader->m_Pipes.end())
-        shader->m_Pipes.push_back(pipeline);
-
       return shader;
     }
 
@@ -707,7 +735,7 @@ public:
     }
 
     DXBCKey GetKey() { return m_Key; }
-    void SetDebugInfoPath(vector<std::string> *searchPaths, const std::string &path)
+    void SetDebugInfoPath(rdcarray<rdcstr> *searchPaths, const rdcstr &path)
     {
       m_DebugInfoSearchPaths = searchPaths;
       m_DebugInfoPath = path;
@@ -721,12 +749,12 @@ public:
       return ret;
     }
 
-    DXBC::DXBCFile *GetDXBC()
+    DXBC::DXBCContainer *GetDXBC()
     {
       if(m_DXBCFile == NULL && !m_Bytecode.empty())
       {
         TryReplaceOriginalByteCode();
-        m_DXBCFile = new DXBC::DXBCFile((const void *)&m_Bytecode[0], m_Bytecode.size());
+        m_DXBCFile = new DXBC::DXBCContainer((const void *)&m_Bytecode[0], m_Bytecode.size());
       }
       return m_DXBCFile;
     }
@@ -746,8 +774,6 @@ public:
       return m_Mapping;
     }
 
-    vector<WrappedID3D12PipelineState *> m_Pipes;
-
   private:
     ShaderEntry(const ShaderEntry &e);
     void TryReplaceOriginalByteCode();
@@ -757,17 +783,17 @@ public:
 
     DXBCKey m_Key;
 
-    std::string m_DebugInfoPath;
-    vector<std::string> *m_DebugInfoSearchPaths;
+    rdcstr m_DebugInfoPath;
+    rdcarray<rdcstr> *m_DebugInfoSearchPaths;
 
-    vector<byte> m_Bytecode;
+    rdcarray<byte> m_Bytecode;
 
     bool m_Built;
-    DXBC::DXBCFile *m_DXBCFile;
+    DXBC::DXBCContainer *m_DXBCFile;
     ShaderReflection m_Details;
     ShaderBindpointMapping m_Mapping;
 
-    static map<DXBCKey, ShaderEntry *> m_Shaders;
+    static std::map<DXBCKey, ShaderEntry *> m_Shaders;
   };
 
   enum
@@ -784,9 +810,14 @@ public:
   WrappedID3D12PipelineState(ID3D12PipelineState *real, WrappedID3D12Device *device)
       : WrappedDeviceChild12(real, device)
   {
+    if(IsReplayMode(m_pDevice->GetState()))
+      m_pDevice->GetPipelineList().push_back(this);
   }
   virtual ~WrappedID3D12PipelineState()
   {
+    if(IsReplayMode(m_pDevice->GetState()))
+      m_pDevice->GetPipelineList().removeOne(this);
+
     Shutdown();
 
     if(graphics)
@@ -855,9 +886,6 @@ public:
   static const int AllocMaxByteSize = 1536 * 1024;
   ALLOCATE_WITH_WRAPPED_POOL(WrappedID3D12Resource1, AllocPoolCount, AllocMaxByteSize, false);
 
-  static std::map<ResourceId, WrappedID3D12Resource1 *> *m_List;
-
-  static std::map<ResourceId, WrappedID3D12Resource1 *> &GetList() { return *m_List; }
   static void RefBuffers(D3D12ResourceManager *rm);
 
   static void GetResIDFromAddr(D3D12_GPU_VIRTUAL_ADDRESS addr, ResourceId &id, UINT64 &offs)
@@ -884,8 +912,8 @@ public:
   WrappedID3D12Resource1(ID3D12Resource *real, WrappedID3D12Device *device)
       : WrappedDeviceChild12(real, device)
   {
-    if(m_List)
-      (*m_List)[GetResourceID()] = this;
+    if(IsReplayMode(device->GetState()))
+      device->GetResourceList()[GetResourceID()] = this;
 
     real->QueryInterface(__uuidof(ID3D12Resource1), (void **)&m_pReal1);
 
