@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -36,7 +36,7 @@ enum D3D12ResourceBarrierSubresource
 DECLARE_REFLECTION_ENUM(D3D12ResourceBarrierSubresource);
 
 template <>
-std::string DoStringise(const D3D12ResourceBarrierSubresource &el)
+rdcstr DoStringise(const D3D12ResourceBarrierSubresource &el)
 {
   RDCCOMPILE_ASSERT(sizeof(D3D12ResourceBarrierSubresource) == sizeof(uint32_t),
                     "Enum isn't uint sized");
@@ -54,11 +54,11 @@ enum D3D12ComponentMapping
 DECLARE_REFLECTION_ENUM(D3D12ComponentMapping);
 
 template <>
-std::string DoStringise(const D3D12ComponentMapping &el)
+rdcstr DoStringise(const D3D12ComponentMapping &el)
 {
   RDCCOMPILE_ASSERT(sizeof(D3D12ComponentMapping) == sizeof(uint32_t), "Enum isn't uint sized");
 
-  std::string ret;
+  rdcstr ret;
 
   // value should always be <= 5, see D3D12_SHADER_COMPONENT_MAPPING
   const char mapping[] = {'R', 'G', 'B', 'A', '0', '1', '?', '!'};
@@ -104,6 +104,97 @@ void DoSerialiseViaResourceId(SerialiserType &ser, Interface *&el)
 
 SERIALISE_D3D_INTERFACES();
 
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12_DESCRIPTOR_RANGE1 &el)
+{
+  SERIALISE_MEMBER(RangeType);
+  SERIALISE_MEMBER(NumDescriptors);
+  SERIALISE_MEMBER(BaseShaderRegister);
+  SERIALISE_MEMBER(RegisterSpace);
+  SERIALISE_MEMBER(Flags);
+  SERIALISE_MEMBER(OffsetInDescriptorsFromTableStart);
+}
+
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12_ROOT_DESCRIPTOR_TABLE1 &el)
+{
+  SERIALISE_MEMBER(NumDescriptorRanges);
+  SERIALISE_MEMBER_ARRAY(pDescriptorRanges, NumDescriptorRanges);
+}
+
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12_ROOT_CONSTANTS &el)
+{
+  SERIALISE_MEMBER(ShaderRegister);
+  SERIALISE_MEMBER(RegisterSpace);
+  SERIALISE_MEMBER(Num32BitValues);
+}
+
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12_ROOT_DESCRIPTOR1 &el)
+{
+  SERIALISE_MEMBER(ShaderRegister);
+  SERIALISE_MEMBER(RegisterSpace);
+  SERIALISE_MEMBER(Flags);
+}
+
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12_STATIC_SAMPLER_DESC &el)
+{
+  SERIALISE_MEMBER(Filter);
+  SERIALISE_MEMBER(AddressU);
+  SERIALISE_MEMBER(AddressV);
+  SERIALISE_MEMBER(AddressW);
+  SERIALISE_MEMBER(MipLODBias);
+  SERIALISE_MEMBER(MaxAnisotropy);
+  SERIALISE_MEMBER(ComparisonFunc);
+  SERIALISE_MEMBER(BorderColor);
+  SERIALISE_MEMBER(MinLOD);
+  SERIALISE_MEMBER(MaxLOD);
+  SERIALISE_MEMBER(ShaderRegister);
+  SERIALISE_MEMBER(RegisterSpace);
+  SERIALISE_MEMBER(ShaderVisibility);
+}
+
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12RootSignatureParameter &el)
+{
+  RDCASSERTMSG(
+      "root signature parameter serialisation is only supported for structured serialisers",
+      ser.IsDummy());
+
+  SERIALISE_MEMBER(ParameterType);
+  switch(el.ParameterType)
+  {
+    case D3D12_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE:
+    {
+      SERIALISE_MEMBER(DescriptorTable);
+      break;
+    }
+    case D3D12_ROOT_PARAMETER_TYPE_32BIT_CONSTANTS:
+    {
+      SERIALISE_MEMBER(Constants);
+      break;
+    }
+    case D3D12_ROOT_PARAMETER_TYPE_CBV:
+    case D3D12_ROOT_PARAMETER_TYPE_SRV:
+    case D3D12_ROOT_PARAMETER_TYPE_UAV:
+    {
+      SERIALISE_MEMBER(Descriptor);
+      break;
+    }
+  }
+  SERIALISE_MEMBER(ShaderVisibility);
+}
+
+template <class SerialiserType>
+void DoSerialise(SerialiserType &ser, D3D12RootSignature &el)
+{
+  SERIALISE_MEMBER(Flags);
+  SERIALISE_MEMBER(Parameters);
+  SERIALISE_MEMBER(StaticSamplers);
+}
+
 // similarly we serialise handles, buffer locations, through Id + indices
 
 template <class SerialiserType>
@@ -130,7 +221,7 @@ void DoSerialise(SerialiserType &ser, D3D12_CPU_DESCRIPTOR_HANDLE &el)
     if(rm)
       el.ptr = (SIZE_T)DescriptorFromPortableHandle(rm, ph);
     else
-      el.ptr = 0;
+      ser.ClearObj(el.ptr);
   }
 }
 
@@ -151,7 +242,7 @@ void DoSerialise(SerialiserType &ser, D3D12_GPU_DESCRIPTOR_HANDLE &el)
     if(rm)
       el.ptr = (SIZE_T)DescriptorFromPortableHandle(rm, ph);
     else
-      el.ptr = 0;
+      ser.ClearObj(el.ptr);
   }
 }
 
@@ -170,8 +261,8 @@ void DoSerialise(SerialiserType &ser, DynamicDescriptorCopy &el)
     src = ToPortableHandle(el.src);
   }
 
-  ser.Serialise("dst", dst);
-  ser.Serialise("src", src);
+  ser.Serialise("dst"_lit, dst);
+  ser.Serialise("src"_lit, src);
 
   if(ser.IsReading())
   {
@@ -199,15 +290,15 @@ void DoSerialise(SerialiserType &ser, D3D12BufferLocation &el)
   if(ser.IsWriting())
     WrappedID3D12Resource1::GetResIDFromAddr(el.Location, buffer, offs);
 
-  ser.Serialise("Buffer", buffer);
-  ser.Serialise("Offset", offs);
+  ser.Serialise("Buffer"_lit, buffer);
+  ser.Serialise("Offset"_lit, offs);
 
   if(ser.IsReading())
   {
     if(rm && buffer != ResourceId() && rm->HasLiveResource(buffer))
       el.Location = rm->GetLiveAs<ID3D12Resource>(buffer)->GetGPUVirtualAddress() + offs;
     else
-      el.Location = 0;
+      ser.ClearObj(el.Location);
   }
 }
 
@@ -215,12 +306,12 @@ template <class SerialiserType>
 void DoSerialise(SerialiserType &ser, D3D12Descriptor &el)
 {
   D3D12DescriptorType type = el.GetType();
-  ser.Serialise("type", type);
+  ser.Serialise("type"_lit, type);
 
   ID3D12DescriptorHeap *heap = (ID3D12DescriptorHeap *)el.data.samp.heap;
 
-  ser.Serialise("heap", heap);
-  ser.Serialise("index", el.data.samp.idx);
+  ser.Serialise("heap"_lit, heap);
+  ser.Serialise("index"_lit, el.data.samp.idx);
 
   if(ser.IsReading())
   {
@@ -235,91 +326,86 @@ void DoSerialise(SerialiserType &ser, D3D12Descriptor &el)
   // also invisibly backwards compatible
   D3D12ResourceManager *rm = (D3D12ResourceManager *)ser.GetUserData();
 
-  ID3D12Resource *resource = NULL;
-  ID3D12Resource *counterResource = NULL;
-
   switch(type)
   {
     case D3D12DescriptorType::Sampler:
     {
-      ser.Serialise("Descriptor", el.data.samp.desc);
+      ser.Serialise("Descriptor"_lit, el.data.samp.desc);
       RDCASSERTEQUAL(el.GetType(), D3D12DescriptorType::Sampler);
       break;
     }
     case D3D12DescriptorType::CBV:
     {
-      ser.Serialise("Descriptor", el.data.nonsamp.cbv);
+      ser.Serialise("Descriptor"_lit, el.data.nonsamp.cbv);
       break;
     }
     case D3D12DescriptorType::SRV:
     {
-      if(ser.IsWriting() && rm && rm->HasCurrentResource(el.data.nonsamp.resource))
-        resource = rm->GetCurrentAs<ID3D12Resource>(el.data.nonsamp.resource);
-
-      ser.Serialise("Resource", resource);
+      ser.Serialise("Resource"_lit, el.data.nonsamp.resource).TypedAs("ID3D12Resource *"_lit);
 
+      // convert to Live ID on replay
       if(ser.IsReading())
-        el.data.nonsamp.resource = GetResID(resource);
+        el.data.nonsamp.resource = rm->HasLiveResource(el.data.nonsamp.resource)
+                                       ? rm->GetLiveID(el.data.nonsamp.resource)
+                                       : ResourceId();
 
       // special case because of squeezed descriptor
       D3D12_SHADER_RESOURCE_VIEW_DESC desc;
       if(ser.IsWriting())
         desc = el.data.nonsamp.srv.AsDesc();
-      ser.Serialise("Descriptor", desc);
+      ser.Serialise("Descriptor"_lit, desc);
       if(ser.IsReading())
         el.data.nonsamp.srv.Init(desc);
       break;
     }
     case D3D12DescriptorType::RTV:
     {
-      if(ser.IsWriting() && rm && rm->HasCurrentResource(el.data.nonsamp.resource))
-        resource = rm->GetCurrentAs<ID3D12Resource>(el.data.nonsamp.resource);
-
-      ser.Serialise("Resource", resource);
+      ser.Serialise("Resource"_lit, el.data.nonsamp.resource).TypedAs("ID3D12Resource *"_lit);
 
+      // convert to Live ID on replay
       if(ser.IsReading())
-        el.data.nonsamp.resource = GetResID(resource);
+        el.data.nonsamp.resource = rm->HasLiveResource(el.data.nonsamp.resource)
+                                       ? rm->GetLiveID(el.data.nonsamp.resource)
+                                       : ResourceId();
 
-      ser.Serialise("Descriptor", el.data.nonsamp.rtv);
+      ser.Serialise("Descriptor"_lit, el.data.nonsamp.rtv);
       break;
     }
     case D3D12DescriptorType::DSV:
     {
-      if(ser.IsWriting() && rm && rm->HasCurrentResource(el.data.nonsamp.resource))
-        resource = rm->GetCurrentAs<ID3D12Resource>(el.data.nonsamp.resource);
-
-      ser.Serialise("Resource", resource);
+      ser.Serialise("Resource"_lit, el.data.nonsamp.resource).TypedAs("ID3D12Resource *"_lit);
 
+      // convert to Live ID on replay
       if(ser.IsReading())
-        el.data.nonsamp.resource = GetResID(resource);
+        el.data.nonsamp.resource = rm->HasLiveResource(el.data.nonsamp.resource)
+                                       ? rm->GetLiveID(el.data.nonsamp.resource)
+                                       : ResourceId();
 
-      ser.Serialise("Descriptor", el.data.nonsamp.dsv);
+      ser.Serialise("Descriptor"_lit, el.data.nonsamp.dsv);
       break;
     }
     case D3D12DescriptorType::UAV:
     {
-      if(ser.IsWriting())
-      {
-        if(rm && rm->HasCurrentResource(el.data.nonsamp.resource))
-          resource = rm->GetCurrentAs<ID3D12Resource>(el.data.nonsamp.resource);
-        if(rm && rm->HasCurrentResource(el.data.nonsamp.counterResource))
-          counterResource = rm->GetCurrentAs<ID3D12Resource>(el.data.nonsamp.counterResource);
-      }
-
-      ser.Serialise("Resource", resource);
-      ser.Serialise("CounterResource", counterResource);
+      ser.Serialise("Resource"_lit, el.data.nonsamp.resource).TypedAs("ID3D12Resource *"_lit);
+      ser.Serialise("CounterResource"_lit, el.data.nonsamp.counterResource)
+          .TypedAs("ID3D12Resource *"_lit);
 
+      // convert to Live ID on replay
       if(ser.IsReading())
       {
-        el.data.nonsamp.resource = GetResID(resource);
-        el.data.nonsamp.counterResource = GetResID(counterResource);
+        el.data.nonsamp.resource = rm->HasLiveResource(el.data.nonsamp.resource)
+                                       ? rm->GetLiveID(el.data.nonsamp.resource)
+                                       : ResourceId();
+        el.data.nonsamp.counterResource = rm->HasLiveResource(el.data.nonsamp.counterResource)
+                                              ? rm->GetLiveID(el.data.nonsamp.counterResource)
+                                              : ResourceId();
       }
 
       // special case because of squeezed descriptor
       D3D12_UNORDERED_ACCESS_VIEW_DESC desc;
       if(ser.IsWriting())
         desc = el.data.nonsamp.uav.AsDesc();
-      ser.Serialise("Descriptor", desc);
+      ser.Serialise("Descriptor"_lit, desc);
       if(ser.IsReading())
         el.data.nonsamp.uav.Init(desc);
       break;
@@ -406,7 +492,7 @@ void DoSerialise(SerialiserType &ser, D3D12_SHADER_BYTECODE &el)
   // don't serialise size_t, otherwise capture/replay between different bit-ness won't work
   {
     uint64_t BytecodeLength = el.BytecodeLength;
-    ser.Serialise("BytecodeLength", BytecodeLength);
+    ser.Serialise("BytecodeLength"_lit, BytecodeLength);
     if(ser.IsReading())
       el.BytecodeLength = (size_t)BytecodeLength;
   }
@@ -569,7 +655,7 @@ void DoSerialise(SerialiserType &ser, D3D12_CACHED_PIPELINE_STATE &el)
   // don't serialise these, just set to NULL/0. See the definition of SERIALISE_MEMBER_DUMMY
   SERIALISE_MEMBER_ARRAY_EMPTY(pCachedBlob);
   uint64_t CachedBlobSizeInBytes = 0;
-  ser.Serialise("CachedBlobSizeInBytes", CachedBlobSizeInBytes);
+  ser.Serialise("CachedBlobSizeInBytes"_lit, CachedBlobSizeInBytes);
 
   if(ser.IsReading())
     el.CachedBlobSizeInBytes = (SIZE_T)CachedBlobSizeInBytes;
@@ -1148,8 +1234,8 @@ void DoSerialise(SerialiserType &ser, D3D12_RANGE &el)
   uint64_t Begin = el.Begin;
   uint64_t End = el.End;
 
-  ser.Serialise("Begin", Begin);
-  ser.Serialise("End", End);
+  ser.Serialise("Begin"_lit, Begin);
+  ser.Serialise("End"_lit, End);
 
   if(ser.IsReading())
   {
@@ -1255,7 +1341,17 @@ void DoSerialise(SerialiserType &ser, D3D12_SUBRESOURCE_RANGE_UINT64 &el)
 template <class SerialiserType>
 void DoSerialise(SerialiserType &ser, D3D12_WRITEBUFFERIMMEDIATE_PARAMETER &el)
 {
-  SERIALISE_MEMBER(Dest);
+  if(ser.VersionAtLeast(0x7))
+  {
+    SERIALISE_MEMBER_TYPED(D3D12BufferLocation, Dest);
+  }
+  else
+  {
+    RDCERR(
+        "Replay will crash - old capture with corrupted D3D12_WRITEBUFFERIMMEDIATE_PARAMETER. "
+        "Re-capture to fix this.");
+    SERIALISE_MEMBER(Dest);
+  }
   SERIALISE_MEMBER(Value);
 }
 
@@ -1379,6 +1475,7 @@ void DoSerialise(SerialiserType &ser, D3D12_DISPATCH_ARGUMENTS &el)
   SERIALISE_MEMBER(ThreadGroupCountZ);
 }
 
+INSTANTIATE_SERIALISE_TYPE(D3D12RootSignature);
 INSTANTIATE_SERIALISE_TYPE(PortableHandle);
 INSTANTIATE_SERIALISE_TYPE(D3D12_CPU_DESCRIPTOR_HANDLE);
 INSTANTIATE_SERIALISE_TYPE(D3D12_GPU_DESCRIPTOR_HANDLE);
