@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,8 +27,8 @@
 #include <map>
 #include <string>
 #include <vector>
-#include "api/replay/renderdoc_replay.h"
 #include "driver/dx/official/d3d11_4.h"
+#include "d3d12_common.h"
 
 class WrappedID3D11Device;
 
@@ -38,11 +38,11 @@ public:
   D3D12ShaderCache();
   ~D3D12ShaderCache();
 
-  std::string GetShaderBlob(const char *source, const char *entry, const uint32_t compileFlags,
-                            const char *profile, ID3DBlob **srcblob);
+  rdcstr GetShaderBlob(const char *source, const char *entry, const uint32_t compileFlags,
+                       const char *profile, ID3DBlob **srcblob);
 
   D3D12RootSignature GetRootSig(const void *data, size_t dataSize);
-  ID3DBlob *MakeRootSig(const std::vector<D3D12_ROOT_PARAMETER1> &params,
+  ID3DBlob *MakeRootSig(const rdcarray<D3D12_ROOT_PARAMETER1> &params,
                         D3D12_ROOT_SIGNATURE_FLAGS Flags = D3D12_ROOT_SIGNATURE_FLAG_NONE,
                         UINT NumStaticSamplers = 0,
                         const D3D12_STATIC_SAMPLER_DESC *StaticSamplers = NULL);
@@ -56,4 +56,4 @@ private:
 
   bool m_ShaderCacheDirty = false, m_CacheShaders = false;
   std::map<uint32_t, ID3DBlob *> m_ShaderCache;
-};
\ No newline at end of file
+};
