@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -55,9 +55,9 @@ D3D12RenderState &D3D12RenderState::operator=(const D3D12RenderState &o)
   return *this;
 }
 
-vector<ResourceId> D3D12RenderState::GetRTVIDs() const
+rdcarray<ResourceId> D3D12RenderState::GetRTVIDs() const
 {
-  vector<ResourceId> ret;
+  rdcarray<ResourceId> ret;
 
   for(UINT i = 0; i < rts.size(); i++)
   {
@@ -73,7 +73,7 @@ ResourceId D3D12RenderState::GetDSVID() const
   return dsv.GetResResourceId();
 }
 
-void D3D12RenderState::ApplyState(WrappedID3D12Device *dev, ID3D12GraphicsCommandList4 *cmd) const
+void D3D12RenderState::ApplyState(WrappedID3D12Device *dev, ID3D12GraphicsCommandListX *cmd) const
 {
   D3D12_COMMAND_LIST_TYPE type = cmd->GetType();
 
@@ -157,14 +157,7 @@ void D3D12RenderState::ApplyState(WrappedID3D12Device *dev, ID3D12GraphicsComman
     }
   }
 
-  std::vector<ID3D12DescriptorHeap *> descHeaps;
-  descHeaps.resize(heaps.size());
-
-  for(size_t i = 0; i < heaps.size(); i++)
-    descHeaps[i] = GetResourceManager()->GetCurrentAs<ID3D12DescriptorHeap>(heaps[i]);
-
-  if(!descHeaps.empty())
-    cmd->SetDescriptorHeaps((UINT)descHeaps.size(), &descHeaps[0]);
+  ApplyDescriptorHeaps(cmd);
 
   if(graphics.rootsig != ResourceId())
   {
@@ -183,14 +176,25 @@ void D3D12RenderState::ApplyState(WrappedID3D12Device *dev, ID3D12GraphicsComman
   }
 }
 
-void D3D12RenderState::ApplyComputeRootElements(ID3D12GraphicsCommandList4 *cmd) const
+void D3D12RenderState::ApplyDescriptorHeaps(ID3D12GraphicsCommandListX *cmd) const
+{
+  rdcarray<ID3D12DescriptorHeap *> descHeaps;
+  descHeaps.resize(heaps.size());
+
+  for(size_t i = 0; i < heaps.size(); i++)
+    descHeaps[i] = GetResourceManager()->GetCurrentAs<ID3D12DescriptorHeap>(heaps[i]);
+
+  if(!descHeaps.empty())
+    cmd->SetDescriptorHeaps((UINT)descHeaps.size(), &descHeaps[0]);
+}
+
+void D3D12RenderState::ApplyComputeRootElements(ID3D12GraphicsCommandListX *cmd) const
 {
   for(size_t i = 0; i < compute.sigelems.size(); i++)
   {
     // just don't set tables that aren't in the descriptor heaps, since it's invalid and can crash
     // and is probably just from stale bindings that aren't going to be used
-    if(compute.sigelems[i].type != eRootTable ||
-       std::find(heaps.begin(), heaps.end(), compute.sigelems[i].id) != heaps.end())
+    if(compute.sigelems[i].type != eRootTable || heaps.contains(compute.sigelems[i].id))
     {
       compute.sigelems[i].SetToCompute(GetResourceManager(), cmd, (UINT)i);
     }
@@ -202,14 +206,13 @@ void D3D12RenderState::ApplyComputeRootElements(ID3D12GraphicsCommandList4 *cmd)
   }
 }
 
-void D3D12RenderState::ApplyGraphicsRootElements(ID3D12GraphicsCommandList4 *cmd) const
+void D3D12RenderState::ApplyGraphicsRootElements(ID3D12GraphicsCommandListX *cmd) const
 {
   for(size_t i = 0; i < graphics.sigelems.size(); i++)
   {
     // just don't set tables that aren't in the descriptor heaps, since it's invalid and can crash
     // and is probably just from stale bindings that aren't going to be used
-    if(graphics.sigelems[i].type != eRootTable ||
-       std::find(heaps.begin(), heaps.end(), graphics.sigelems[i].id) != heaps.end())
+    if(graphics.sigelems[i].type != eRootTable || heaps.contains(graphics.sigelems[i].id))
     {
       graphics.sigelems[i].SetToGraphics(GetResourceManager(), cmd, (UINT)i);
     }
