@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -46,21 +46,31 @@ struct D3D12RenderState
   D3D12RenderState() = default;
   D3D12RenderState &operator=(const D3D12RenderState &o);
 
-  void ApplyState(WrappedID3D12Device *dev, ID3D12GraphicsCommandList4 *list) const;
-  void ApplyComputeRootElements(ID3D12GraphicsCommandList4 *cmd) const;
-  void ApplyGraphicsRootElements(ID3D12GraphicsCommandList4 *cmd) const;
+  void ApplyState(WrappedID3D12Device *dev, ID3D12GraphicsCommandListX *list) const;
+  void ApplyDescriptorHeaps(ID3D12GraphicsCommandListX *list) const;
+  void ApplyComputeRootElements(ID3D12GraphicsCommandListX *cmd) const;
+  void ApplyGraphicsRootElements(ID3D12GraphicsCommandListX *cmd) const;
 
-  vector<D3D12_VIEWPORT> views;
-  vector<D3D12_RECT> scissors;
+  rdcarray<D3D12_VIEWPORT> views;
+  rdcarray<D3D12_RECT> scissors;
 
   // these are D3D12Descriptor copies since the values of the descriptors are read during
   // OMSetRenderTargets and may not exist anywhere after that if they are immediately overwritten.
-  vector<D3D12Descriptor> rts;
+  rdcarray<D3D12Descriptor> rts;
   D3D12Descriptor dsv;
 
-  vector<ResourceId> GetRTVIDs() const;
+  bool renderpass = false;
+  rdcarray<D3D12_RENDER_PASS_RENDER_TARGET_DESC> rpRTs;
+  D3D12_RENDER_PASS_DEPTH_STENCIL_DESC rpDSV;
+  D3D12_RENDER_PASS_FLAGS rpFlags;
+
+  rdcarray<ResourceId> GetRTVIDs() const;
   ResourceId GetDSVID() const;
 
+  ResourceId shadingRateImage;
+  D3D12_SHADING_RATE shadingRate;
+  D3D12_SHADING_RATE_COMBINER shadingRateCombiners[2];
+
   struct SignatureElement
   {
     SignatureElement() : type(eRootUnknown), offset(0) {}
@@ -140,10 +150,10 @@ struct D3D12RenderState
 
     ResourceId id;
     UINT64 offset;
-    vector<UINT> constants;
+    rdcarray<UINT> constants;
   };
 
-  vector<ResourceId> heaps;
+  rdcarray<ResourceId> heaps;
 
   struct StreamOut
   {
@@ -154,13 +164,13 @@ struct D3D12RenderState
     ResourceId countbuf;
     UINT64 countoffs;
   };
-  vector<StreamOut> streamouts;
+  rdcarray<StreamOut> streamouts;
 
   struct RootSignature
   {
     ResourceId rootsig;
 
-    vector<SignatureElement> sigelems;
+    rdcarray<SignatureElement> sigelems;
   } compute, graphics;
 
   ResourceId pipe;
@@ -170,7 +180,7 @@ struct D3D12RenderState
   struct SamplePositions
   {
     UINT NumSamplesPerPixel, NumPixels;
-    std::vector<D3D12_SAMPLE_POSITION> Positions;
+    rdcarray<D3D12_SAMPLE_POSITION> Positions;
   } samplePos;
 
   D3D12_PRIMITIVE_TOPOLOGY topo = D3D_PRIMITIVE_TOPOLOGY_UNDEFINED;
@@ -194,7 +204,7 @@ struct D3D12RenderState
     UINT stride;
     UINT size;
   };
-  vector<VertBuffer> vbuffers;
+  rdcarray<VertBuffer> vbuffers;
 
   D3D12ResourceManager *GetResourceManager() const { return m_ResourceManager; }
   D3D12ResourceManager *m_ResourceManager = NULL;
