@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,11 +26,9 @@
 #include "d3d12_resources.h"
 
 template <>
-std::string DoStringise(const D3D12Chunk &el)
+rdcstr DoStringise(const D3D12Chunk &el)
 {
-  RDCCOMPILE_ASSERT((uint32_t)D3D12Chunk::Max == 1102, "Chunks changed without updating names");
-  RDCCOMPILE_ASSERT((uint32_t)D3D12Chunk::List_EndRenderPass == 1101,
-                    "New chunks must be appended otherwise it breaks old captures");
+  RDCCOMPILE_ASSERT((uint32_t)D3D12Chunk::Max == 1107, "Chunks changed without updating names");
 
   BEGIN_ENUM_STRINGISE(D3D12Chunk)
   {
@@ -38,7 +36,7 @@ std::string DoStringise(const D3D12Chunk &el)
     STRINGISE_ENUM_CLASS_NAMED(PushMarker, "Push Debug Region");
     STRINGISE_ENUM_CLASS_NAMED(SetMarker, "Set Marker");
     STRINGISE_ENUM_CLASS_NAMED(PopMarker, "Pop Debug Region");
-    STRINGISE_ENUM_CLASS_NAMED(SetShaderDebugPath, "SetShaderDebugPath");
+    STRINGISE_ENUM_CLASS_NAMED(SetShaderDebugPath, "Internal: SetShaderDebugPath");
     STRINGISE_ENUM_CLASS_NAMED(CreateSwapBuffer, "IDXGISwapChain::GetBuffer");
     STRINGISE_ENUM_CLASS_NAMED(Device_CreateCommandQueue, "ID3D12Device::CreateCommandQueue");
     STRINGISE_ENUM_CLASS_NAMED(Device_CreateCommandAllocator,
@@ -188,13 +186,20 @@ std::string DoStringise(const D3D12Chunk &el)
                                "ID3D12GraphicsCommandList2::WriteBufferImmediate");
     STRINGISE_ENUM_CLASS_NAMED(List_BeginRenderPass, "ID3D12GraphicsCommandList4::BeginRenderPass");
     STRINGISE_ENUM_CLASS_NAMED(List_EndRenderPass, "ID3D12GraphicsCommandList4::EndRenderPass");
+    STRINGISE_ENUM_CLASS_NAMED(List_RSSetShadingRate,
+                               "ID3D12GraphicsCommandList5::RSSetShadingRate");
+    STRINGISE_ENUM_CLASS_NAMED(List_RSSetShadingRateImage,
+                               "ID3D12GraphicsCommandList5::RSSetShadingRateImage");
+    STRINGISE_ENUM_CLASS_NAMED(Device_ExternalDXGIResource, "External DXGI Resource import");
+    STRINGISE_ENUM_CLASS_NAMED(Swapchain_Present, "IDXGISwapChain::Present");
+    STRINGISE_ENUM_CLASS_NAMED(List_ClearState, "ID3D12GraphicsCommandList::ClearState");
     STRINGISE_ENUM_CLASS_NAMED(Max, "Max Chunk");
   }
   END_ENUM_STRINGISE()
 }
 
 template <>
-std::string DoStringise(const D3D12DescriptorType &el)
+rdcstr DoStringise(const D3D12DescriptorType &el)
 {
   if((uint32_t)el < (uint32_t)D3D12DescriptorType::CBV)
     return "Sampler";
@@ -212,7 +217,7 @@ std::string DoStringise(const D3D12DescriptorType &el)
 }
 
 template <>
-std::string DoStringise(const D3D12ResourceType &el)
+rdcstr DoStringise(const D3D12ResourceType &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12ResourceType);
   {
@@ -236,7 +241,7 @@ std::string DoStringise(const D3D12ResourceType &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_HEAP_TYPE &el)
+rdcstr DoStringise(const D3D12_HEAP_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_HEAP_TYPE);
   {
@@ -249,7 +254,7 @@ std::string DoStringise(const D3D12_HEAP_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_CPU_PAGE_PROPERTY &el)
+rdcstr DoStringise(const D3D12_CPU_PAGE_PROPERTY &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_CPU_PAGE_PROPERTY);
   {
@@ -262,7 +267,7 @@ std::string DoStringise(const D3D12_CPU_PAGE_PROPERTY &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_MEMORY_POOL &el)
+rdcstr DoStringise(const D3D12_MEMORY_POOL &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_MEMORY_POOL);
   {
@@ -274,7 +279,7 @@ std::string DoStringise(const D3D12_MEMORY_POOL &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_QUERY_HEAP_TYPE &el)
+rdcstr DoStringise(const D3D12_QUERY_HEAP_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_QUERY_HEAP_TYPE);
   {
@@ -287,7 +292,7 @@ std::string DoStringise(const D3D12_QUERY_HEAP_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_QUERY_TYPE &el)
+rdcstr DoStringise(const D3D12_QUERY_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_QUERY_TYPE);
   {
@@ -304,7 +309,7 @@ std::string DoStringise(const D3D12_QUERY_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_PREDICATION_OP &el)
+rdcstr DoStringise(const D3D12_PREDICATION_OP &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_PREDICATION_OP);
   {
@@ -315,7 +320,7 @@ std::string DoStringise(const D3D12_PREDICATION_OP &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_DESCRIPTOR_HEAP_TYPE &el)
+rdcstr DoStringise(const D3D12_DESCRIPTOR_HEAP_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_DESCRIPTOR_HEAP_TYPE);
   {
@@ -328,7 +333,7 @@ std::string DoStringise(const D3D12_DESCRIPTOR_HEAP_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RESOURCE_BARRIER_TYPE &el)
+rdcstr DoStringise(const D3D12_RESOURCE_BARRIER_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_RESOURCE_BARRIER_TYPE);
   {
@@ -340,7 +345,7 @@ std::string DoStringise(const D3D12_RESOURCE_BARRIER_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_SRV_DIMENSION &el)
+rdcstr DoStringise(const D3D12_SRV_DIMENSION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_SRV_DIMENSION);
   {
@@ -360,7 +365,7 @@ std::string DoStringise(const D3D12_SRV_DIMENSION &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RTV_DIMENSION &el)
+rdcstr DoStringise(const D3D12_RTV_DIMENSION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_RTV_DIMENSION);
   {
@@ -378,7 +383,7 @@ std::string DoStringise(const D3D12_RTV_DIMENSION &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_UAV_DIMENSION &el)
+rdcstr DoStringise(const D3D12_UAV_DIMENSION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_UAV_DIMENSION);
   {
@@ -393,7 +398,7 @@ std::string DoStringise(const D3D12_UAV_DIMENSION &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_DSV_DIMENSION &el)
+rdcstr DoStringise(const D3D12_DSV_DIMENSION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_DSV_DIMENSION);
   {
@@ -409,7 +414,7 @@ std::string DoStringise(const D3D12_DSV_DIMENSION &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_FILTER &el)
+rdcstr DoStringise(const D3D12_FILTER &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_FILTER);
   {
@@ -454,7 +459,7 @@ std::string DoStringise(const D3D12_FILTER &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_TEXTURE_ADDRESS_MODE &el)
+rdcstr DoStringise(const D3D12_TEXTURE_ADDRESS_MODE &el)
 {
   // possible for unused fields via 0-initialisation
   if((int)el == 0)
@@ -472,7 +477,7 @@ std::string DoStringise(const D3D12_TEXTURE_ADDRESS_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_BLEND &el)
+rdcstr DoStringise(const D3D12_BLEND &el)
 {
   // possible for unused fields via 0-initialisation
   if((int)el == 0)
@@ -502,7 +507,7 @@ std::string DoStringise(const D3D12_BLEND &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_BLEND_OP &el)
+rdcstr DoStringise(const D3D12_BLEND_OP &el)
 {
   // possible for unused fields via 0-initialisation
   if((int)el == 0)
@@ -520,7 +525,7 @@ std::string DoStringise(const D3D12_BLEND_OP &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_LOGIC_OP &el)
+rdcstr DoStringise(const D3D12_LOGIC_OP &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_LOGIC_OP);
   {
@@ -545,7 +550,7 @@ std::string DoStringise(const D3D12_LOGIC_OP &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_FILL_MODE &el)
+rdcstr DoStringise(const D3D12_FILL_MODE &el)
 {
   // possible for unused fields via 0-initialisation
   if((int)el == 0)
@@ -560,7 +565,7 @@ std::string DoStringise(const D3D12_FILL_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_CULL_MODE &el)
+rdcstr DoStringise(const D3D12_CULL_MODE &el)
 {
   // possible for unused fields via 0-initialisation
   if((int)el == 0)
@@ -576,7 +581,7 @@ std::string DoStringise(const D3D12_CULL_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_CONSERVATIVE_RASTERIZATION_MODE &el)
+rdcstr DoStringise(const D3D12_CONSERVATIVE_RASTERIZATION_MODE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_CONSERVATIVE_RASTERIZATION_MODE);
   {
@@ -587,7 +592,7 @@ std::string DoStringise(const D3D12_CONSERVATIVE_RASTERIZATION_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_COMPARISON_FUNC &el)
+rdcstr DoStringise(const D3D12_COMPARISON_FUNC &el)
 {
   // possible for unused fields via 0-initialisation
   if((int)el == 0)
@@ -608,7 +613,7 @@ std::string DoStringise(const D3D12_COMPARISON_FUNC &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_DEPTH_WRITE_MASK &el)
+rdcstr DoStringise(const D3D12_DEPTH_WRITE_MASK &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_DEPTH_WRITE_MASK);
   {
@@ -619,7 +624,7 @@ std::string DoStringise(const D3D12_DEPTH_WRITE_MASK &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_STENCIL_OP &el)
+rdcstr DoStringise(const D3D12_STENCIL_OP &el)
 {
   // possible for unused fields via 0-initialisation
   if((int)el == 0)
@@ -640,7 +645,7 @@ std::string DoStringise(const D3D12_STENCIL_OP &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_INPUT_CLASSIFICATION &el)
+rdcstr DoStringise(const D3D12_INPUT_CLASSIFICATION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_INPUT_CLASSIFICATION);
   {
@@ -651,7 +656,7 @@ std::string DoStringise(const D3D12_INPUT_CLASSIFICATION &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_INDEX_BUFFER_STRIP_CUT_VALUE &el)
+rdcstr DoStringise(const D3D12_INDEX_BUFFER_STRIP_CUT_VALUE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_INDEX_BUFFER_STRIP_CUT_VALUE);
   {
@@ -663,7 +668,7 @@ std::string DoStringise(const D3D12_INDEX_BUFFER_STRIP_CUT_VALUE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_PRIMITIVE_TOPOLOGY_TYPE &el)
+rdcstr DoStringise(const D3D12_PRIMITIVE_TOPOLOGY_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_PRIMITIVE_TOPOLOGY_TYPE);
   {
@@ -677,7 +682,7 @@ std::string DoStringise(const D3D12_PRIMITIVE_TOPOLOGY_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_INDIRECT_ARGUMENT_TYPE &el)
+rdcstr DoStringise(const D3D12_INDIRECT_ARGUMENT_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_INDIRECT_ARGUMENT_TYPE);
   {
@@ -695,7 +700,7 @@ std::string DoStringise(const D3D12_INDIRECT_ARGUMENT_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_COMMAND_LIST_TYPE &el)
+rdcstr DoStringise(const D3D12_COMMAND_LIST_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_COMMAND_LIST_TYPE);
   {
@@ -708,7 +713,7 @@ std::string DoStringise(const D3D12_COMMAND_LIST_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_TEXTURE_COPY_TYPE &el)
+rdcstr DoStringise(const D3D12_TEXTURE_COPY_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_TEXTURE_COPY_TYPE);
   {
@@ -719,7 +724,7 @@ std::string DoStringise(const D3D12_TEXTURE_COPY_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RESOURCE_DIMENSION &el)
+rdcstr DoStringise(const D3D12_RESOURCE_DIMENSION &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_RESOURCE_DIMENSION);
   {
@@ -733,7 +738,7 @@ std::string DoStringise(const D3D12_RESOURCE_DIMENSION &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_TEXTURE_LAYOUT &el)
+rdcstr DoStringise(const D3D12_TEXTURE_LAYOUT &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_TEXTURE_LAYOUT);
   {
@@ -746,7 +751,7 @@ std::string DoStringise(const D3D12_TEXTURE_LAYOUT &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_TILED_RESOURCES_TIER &el)
+rdcstr DoStringise(const D3D12_TILED_RESOURCES_TIER &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_TILED_RESOURCES_TIER);
   {
@@ -759,7 +764,7 @@ std::string DoStringise(const D3D12_TILED_RESOURCES_TIER &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RESOLVE_MODE &el)
+rdcstr DoStringise(const D3D12_RESOLVE_MODE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_RESOLVE_MODE);
   {
@@ -772,7 +777,7 @@ std::string DoStringise(const D3D12_RESOLVE_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_WRITEBUFFERIMMEDIATE_MODE &el)
+rdcstr DoStringise(const D3D12_WRITEBUFFERIMMEDIATE_MODE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_WRITEBUFFERIMMEDIATE_MODE);
   {
@@ -784,7 +789,7 @@ std::string DoStringise(const D3D12_WRITEBUFFERIMMEDIATE_MODE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE &el)
+rdcstr DoStringise(const D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE);
   {
@@ -797,7 +802,7 @@ std::string DoStringise(const D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RENDER_PASS_ENDING_ACCESS_TYPE &el)
+rdcstr DoStringise(const D3D12_RENDER_PASS_ENDING_ACCESS_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D12_RENDER_PASS_ENDING_ACCESS_TYPE);
   {
@@ -810,7 +815,91 @@ std::string DoStringise(const D3D12_RENDER_PASS_ENDING_ACCESS_TYPE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_CLEAR_FLAGS &el)
+rdcstr DoStringise(const D3D12_SHADING_RATE &el)
+{
+  BEGIN_ENUM_STRINGISE(D3D12_SHADING_RATE);
+  {
+    STRINGISE_ENUM(D3D12_SHADING_RATE_1X1)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_1X2)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_2X1)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_2X2)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_2X4)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_4X2)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_4X4)
+  }
+  END_ENUM_STRINGISE();
+}
+
+template <>
+rdcstr DoStringise(const D3D12_SHADING_RATE_COMBINER &el)
+{
+  BEGIN_ENUM_STRINGISE(D3D12_SHADING_RATE_COMBINER);
+  {
+    STRINGISE_ENUM(D3D12_SHADING_RATE_COMBINER_PASSTHROUGH)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_COMBINER_OVERRIDE)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_COMBINER_MIN)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_COMBINER_MAX)
+    STRINGISE_ENUM(D3D12_SHADING_RATE_COMBINER_SUM)
+  }
+  END_ENUM_STRINGISE();
+}
+
+template <>
+rdcstr DoStringise(const D3D12_ROOT_PARAMETER_TYPE &el)
+{
+  BEGIN_ENUM_STRINGISE(D3D12_ROOT_PARAMETER_TYPE);
+  {
+    STRINGISE_ENUM(D3D12_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE)
+    STRINGISE_ENUM(D3D12_ROOT_PARAMETER_TYPE_32BIT_CONSTANTS)
+    STRINGISE_ENUM(D3D12_ROOT_PARAMETER_TYPE_CBV)
+    STRINGISE_ENUM(D3D12_ROOT_PARAMETER_TYPE_SRV)
+    STRINGISE_ENUM(D3D12_ROOT_PARAMETER_TYPE_UAV)
+  }
+  END_ENUM_STRINGISE();
+}
+
+template <>
+rdcstr DoStringise(const D3D12_SHADER_VISIBILITY &el)
+{
+  BEGIN_ENUM_STRINGISE(D3D12_SHADER_VISIBILITY);
+  {
+    STRINGISE_ENUM(D3D12_SHADER_VISIBILITY_ALL)
+    STRINGISE_ENUM(D3D12_SHADER_VISIBILITY_VERTEX)
+    STRINGISE_ENUM(D3D12_SHADER_VISIBILITY_HULL)
+    STRINGISE_ENUM(D3D12_SHADER_VISIBILITY_DOMAIN)
+    STRINGISE_ENUM(D3D12_SHADER_VISIBILITY_GEOMETRY)
+    STRINGISE_ENUM(D3D12_SHADER_VISIBILITY_PIXEL)
+  }
+  END_ENUM_STRINGISE();
+}
+
+template <>
+rdcstr DoStringise(const D3D12_STATIC_BORDER_COLOR &el)
+{
+  BEGIN_ENUM_STRINGISE(D3D12_STATIC_BORDER_COLOR);
+  {
+    STRINGISE_ENUM(D3D12_STATIC_BORDER_COLOR_TRANSPARENT_BLACK)
+    STRINGISE_ENUM(D3D12_STATIC_BORDER_COLOR_OPAQUE_BLACK)
+    STRINGISE_ENUM(D3D12_STATIC_BORDER_COLOR_OPAQUE_WHITE)
+  }
+  END_ENUM_STRINGISE();
+}
+
+template <>
+rdcstr DoStringise(const D3D12_DESCRIPTOR_RANGE_TYPE &el)
+{
+  BEGIN_ENUM_STRINGISE(D3D12_DESCRIPTOR_RANGE_TYPE);
+  {
+    STRINGISE_ENUM(D3D12_DESCRIPTOR_RANGE_TYPE_SRV)
+    STRINGISE_ENUM(D3D12_DESCRIPTOR_RANGE_TYPE_UAV)
+    STRINGISE_ENUM(D3D12_DESCRIPTOR_RANGE_TYPE_CBV)
+    STRINGISE_ENUM(D3D12_DESCRIPTOR_RANGE_TYPE_SAMPLER)
+  }
+  END_ENUM_STRINGISE();
+}
+
+template <>
+rdcstr DoStringise(const D3D12_CLEAR_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_CLEAR_FLAGS);
   {
@@ -821,7 +910,7 @@ std::string DoStringise(const D3D12_CLEAR_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_BUFFER_SRV_FLAGS &el)
+rdcstr DoStringise(const D3D12_BUFFER_SRV_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_BUFFER_SRV_FLAGS);
   {
@@ -833,7 +922,7 @@ std::string DoStringise(const D3D12_BUFFER_SRV_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_DSV_FLAGS &el)
+rdcstr DoStringise(const D3D12_DSV_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_DSV_FLAGS);
   {
@@ -846,7 +935,7 @@ std::string DoStringise(const D3D12_DSV_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_BUFFER_UAV_FLAGS &el)
+rdcstr DoStringise(const D3D12_BUFFER_UAV_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_BUFFER_UAV_FLAGS);
   {
@@ -858,7 +947,7 @@ std::string DoStringise(const D3D12_BUFFER_UAV_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_HEAP_FLAGS &el)
+rdcstr DoStringise(const D3D12_HEAP_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_HEAP_FLAGS);
   {
@@ -879,7 +968,7 @@ std::string DoStringise(const D3D12_HEAP_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_FENCE_FLAGS &el)
+rdcstr DoStringise(const D3D12_FENCE_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_FENCE_FLAGS);
   {
@@ -892,7 +981,7 @@ std::string DoStringise(const D3D12_FENCE_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_DESCRIPTOR_HEAP_FLAGS &el)
+rdcstr DoStringise(const D3D12_DESCRIPTOR_HEAP_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_DESCRIPTOR_HEAP_FLAGS);
   {
@@ -904,7 +993,7 @@ std::string DoStringise(const D3D12_DESCRIPTOR_HEAP_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RESOURCE_BARRIER_FLAGS &el)
+rdcstr DoStringise(const D3D12_RESOURCE_BARRIER_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_RESOURCE_BARRIER_FLAGS);
   {
@@ -917,7 +1006,7 @@ std::string DoStringise(const D3D12_RESOURCE_BARRIER_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RESOURCE_STATES &el)
+rdcstr DoStringise(const D3D12_RESOURCE_STATES &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_RESOURCE_STATES);
   {
@@ -944,7 +1033,7 @@ std::string DoStringise(const D3D12_RESOURCE_STATES &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_PIPELINE_STATE_FLAGS &el)
+rdcstr DoStringise(const D3D12_PIPELINE_STATE_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_PIPELINE_STATE_FLAGS);
   {
@@ -956,7 +1045,7 @@ std::string DoStringise(const D3D12_PIPELINE_STATE_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RESOURCE_FLAGS &el)
+rdcstr DoStringise(const D3D12_RESOURCE_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_RESOURCE_FLAGS);
   {
@@ -973,7 +1062,7 @@ std::string DoStringise(const D3D12_RESOURCE_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_COMMAND_QUEUE_FLAGS &el)
+rdcstr DoStringise(const D3D12_COMMAND_QUEUE_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_COMMAND_QUEUE_FLAGS);
   {
@@ -985,7 +1074,7 @@ std::string DoStringise(const D3D12_COMMAND_QUEUE_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_TILE_COPY_FLAGS &el)
+rdcstr DoStringise(const D3D12_TILE_COPY_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_TILE_COPY_FLAGS);
   {
@@ -999,7 +1088,7 @@ std::string DoStringise(const D3D12_TILE_COPY_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_TILE_MAPPING_FLAGS &el)
+rdcstr DoStringise(const D3D12_TILE_MAPPING_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_TILE_MAPPING_FLAGS);
   {
@@ -1011,7 +1100,7 @@ std::string DoStringise(const D3D12_TILE_MAPPING_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_TILE_RANGE_FLAGS &el)
+rdcstr DoStringise(const D3D12_TILE_RANGE_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_TILE_RANGE_FLAGS);
   {
@@ -1025,7 +1114,7 @@ std::string DoStringise(const D3D12_TILE_RANGE_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_COLOR_WRITE_ENABLE &el)
+rdcstr DoStringise(const D3D12_COLOR_WRITE_ENABLE &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_COLOR_WRITE_ENABLE);
   {
@@ -1040,7 +1129,7 @@ std::string DoStringise(const D3D12_COLOR_WRITE_ENABLE &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_VIEW_INSTANCING_FLAGS &el)
+rdcstr DoStringise(const D3D12_VIEW_INSTANCING_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_VIEW_INSTANCING_FLAGS);
   {
@@ -1052,7 +1141,7 @@ std::string DoStringise(const D3D12_VIEW_INSTANCING_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_COMMAND_LIST_FLAGS &el)
+rdcstr DoStringise(const D3D12_COMMAND_LIST_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_COMMAND_LIST_FLAGS);
   {
@@ -1062,7 +1151,7 @@ std::string DoStringise(const D3D12_COMMAND_LIST_FLAGS &el)
 }
 
 template <>
-std::string DoStringise(const D3D12_RENDER_PASS_FLAGS &el)
+rdcstr DoStringise(const D3D12_RENDER_PASS_FLAGS &el)
 {
   BEGIN_BITFIELD_STRINGISE(D3D12_RENDER_PASS_FLAGS);
   {
@@ -1074,3 +1163,53 @@ std::string DoStringise(const D3D12_RENDER_PASS_FLAGS &el)
   }
   END_BITFIELD_STRINGISE();
 }
+
+template <>
+rdcstr DoStringise(const D3D12_ROOT_SIGNATURE_FLAGS &el)
+{
+  BEGIN_BITFIELD_STRINGISE(D3D12_ROOT_SIGNATURE_FLAGS);
+  {
+    STRINGISE_BITFIELD_VALUE(D3D12_ROOT_SIGNATURE_FLAG_NONE);
+
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_SIGNATURE_FLAG_ALLOW_INPUT_ASSEMBLER_INPUT_LAYOUT);
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_SIGNATURE_FLAG_DENY_VERTEX_SHADER_ROOT_ACCESS);
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_SIGNATURE_FLAG_DENY_HULL_SHADER_ROOT_ACCESS);
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_SIGNATURE_FLAG_DENY_DOMAIN_SHADER_ROOT_ACCESS);
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_SIGNATURE_FLAG_DENY_GEOMETRY_SHADER_ROOT_ACCESS);
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_SIGNATURE_FLAG_DENY_PIXEL_SHADER_ROOT_ACCESS);
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_SIGNATURE_FLAG_ALLOW_STREAM_OUTPUT);
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_SIGNATURE_FLAG_LOCAL_ROOT_SIGNATURE);
+  }
+  END_BITFIELD_STRINGISE();
+}
+
+template <>
+rdcstr DoStringise(const D3D12_ROOT_DESCRIPTOR_FLAGS &el)
+{
+  BEGIN_BITFIELD_STRINGISE(D3D12_ROOT_DESCRIPTOR_FLAGS);
+  {
+    STRINGISE_BITFIELD_VALUE(D3D12_ROOT_DESCRIPTOR_FLAG_NONE);
+
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_DESCRIPTOR_FLAG_DATA_VOLATILE);
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_DESCRIPTOR_FLAG_DATA_STATIC_WHILE_SET_AT_EXECUTE);
+    STRINGISE_BITFIELD_BIT(D3D12_ROOT_DESCRIPTOR_FLAG_DATA_STATIC);
+  }
+  END_BITFIELD_STRINGISE();
+}
+
+template <>
+rdcstr DoStringise(const D3D12_DESCRIPTOR_RANGE_FLAGS &el)
+{
+  BEGIN_BITFIELD_STRINGISE(D3D12_DESCRIPTOR_RANGE_FLAGS);
+  {
+    STRINGISE_BITFIELD_VALUE(D3D12_DESCRIPTOR_RANGE_FLAG_NONE);
+
+    STRINGISE_BITFIELD_BIT(D3D12_DESCRIPTOR_RANGE_FLAG_DESCRIPTORS_VOLATILE);
+    STRINGISE_BITFIELD_BIT(D3D12_DESCRIPTOR_RANGE_FLAG_DATA_VOLATILE);
+    STRINGISE_BITFIELD_BIT(D3D12_DESCRIPTOR_RANGE_FLAG_DATA_STATIC_WHILE_SET_AT_EXECUTE);
+    STRINGISE_BITFIELD_BIT(D3D12_DESCRIPTOR_RANGE_FLAG_DATA_STATIC);
+    STRINGISE_BITFIELD_BIT(
+        D3D12_DESCRIPTOR_RANGE_FLAG_DESCRIPTORS_STATIC_KEEPING_BUFFER_BOUNDS_CHECKS);
+  }
+  END_BITFIELD_STRINGISE();
+}
