@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,7 @@
 #include "d3d8_debug.h"
 #include "os/os_specific.h"
 #include "stb/stb_truetype.h"
+#include "strings/string_utils.h"
 
 D3D8DebugManager::D3D8DebugManager(WrappedD3DDevice8 *wrapper)
     : m_WrappedDevice(wrapper), m_fvf(D3DFVF_XYZ | D3DFVF_TEX1)
@@ -44,7 +45,7 @@ bool D3D8DebugManager::InitFontRendering()
   int width = FONT_TEX_WIDTH;
   int height = FONT_TEX_HEIGHT;
 
-  string font = GetEmbeddedResource(sourcecodepro_ttf);
+  rdcstr font = GetEmbeddedResource(sourcecodepro_ttf);
   byte *ttfdata = (byte *)font.c_str();
 
   const int firstChar = int(' ') + 1;
@@ -127,34 +128,24 @@ void D3D8DebugManager::SetOutputWindow(HWND w)
   // m_supersamplingY = float(m_height) / float(rect.bottom - rect.top);
 }
 
-void D3D8DebugManager::RenderText(float x, float y, const char *textfmt, ...)
+void D3D8DebugManager::RenderText(float x, float y, const rdcstr &text)
 {
-  static char tmpBuf[4096];
+  rdcarray<rdcstr> lines;
+  split(text, lines, '\n');
 
-  va_list args;
-  va_start(args, textfmt);
-  StringFormat::vsnprintf(tmpBuf, 4095, textfmt, args);
-  tmpBuf[4095] = '\0';
-  va_end(args);
-
-  RenderTextInternal(x, y, tmpBuf);
+  for(const rdcstr &line : lines)
+  {
+    RenderTextInternal(x, y, line);
+    y += 1.0f;
+  }
 }
 
-void D3D8DebugManager::RenderTextInternal(float x, float y, const char *text)
+void D3D8DebugManager::RenderTextInternal(float x, float y, const rdcstr &text)
 {
-  if(char *t = strchr((char *)text, '\n'))
-  {
-    *t = 0;
-    RenderTextInternal(x, y, text);
-    RenderTextInternal(x, y + 1.0f, t + 1);
-    *t = '\n';
-    return;
-  }
-
-  if(strlen(text) == 0)
+  if(text.empty())
     return;
 
-  RDCASSERT(strlen(text) < FONT_MAX_CHARS);
+  RDCASSERT(text.size() < FONT_MAX_CHARS);
 
   // transforms
   float width = (float)m_width;
@@ -265,7 +256,7 @@ void D3D8DebugManager::RenderTextInternal(float x, float y, const char *text)
   Quad background;
   UINT triangleCount = 0;
   {
-    UINT quadCount = (UINT)strlen(text);    // calculate string length
+    UINT quadCount = (UINT)text.size();    // calculate string length
 
     triangleCount = quadCount * 2;
     // create text VB
@@ -315,4 +306,4 @@ void D3D8DebugManager::RenderTextInternal(float x, float y, const char *text)
                                             sizeof(Vertex));
     delete[] quads;
   }
-}
\ No newline at end of file
+}
