@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,9 +24,7 @@
 
 #pragma once
 
-#include <list>
 #include <map>
-#include <utility>
 #include "driver/dx/official/d3d8.h"
 #include "stb/stb_truetype.h"
 #include "d3d8_common.h"
@@ -38,7 +36,7 @@ public:
   D3D8DebugManager(WrappedD3DDevice8 *wrapper);
   ~D3D8DebugManager();
 
-  void RenderText(float x, float y, const char *textfmt, ...);
+  void RenderText(float x, float y, const rdcstr &text);
 
   void SetOutputDimensions(int w, int h)
   {
@@ -51,7 +49,7 @@ public:
   bool InitFontRendering();
   void ShutdownFontRendering();
 
-  void RenderTextInternal(float x, float y, const char *text);
+  void RenderTextInternal(float x, float y, const rdcstr &text);
 
   static const int FONT_TEX_WIDTH = 256;
   static const int FONT_TEX_HEIGHT = 128;
