@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,6 +38,7 @@ public:
 
   void LazyInit();
 
+  RDCDriver GetFrameCaptureDriver() { return RDCDriver::D3D8; }
   void StartFrameCapture(void *dev, void *wnd);
   bool EndFrameCapture(void *dev, void *wnd);
   bool DiscardFrameCapture(void *dev, void *wnd);
@@ -228,7 +229,7 @@ private:
   RefCounter8 m_SoftRefCounter;
   bool m_Alive;
 
-  uint32_t m_FrameCounter;
+  uint32_t m_FrameCounter = 0;
 };
 
 ////////////////////////////////////////////////////////////////////////////
@@ -274,4 +275,4 @@ public:
 
 private:
   IDirect3D8 *m_direct3D;
-};
\ No newline at end of file
+};
