@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -171,7 +171,7 @@ public:
     uint32_t length;
   };
 
-  static map<ResourceId, BufferEntry> m_BufferList;
+  static std::map<ResourceId, BufferEntry> m_BufferList;
 
   WrappedD3DBuffer8(NestedType *real, uint32_t byteLength, WrappedD3DDevice8 *device)
       : WrappedIDirect3DResource8(real, device)
@@ -333,4 +333,4 @@ ResourceId GetResID(IUnknown *ptr);
 template <>
 IUnknown *Unwrap(IUnknown *ptr);
 template <>
-D3D8ResourceRecord *GetRecord(IUnknown *ptr);
\ No newline at end of file
+D3D8ResourceRecord *GetRecord(IUnknown *ptr);
