@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,7 @@
 #include "dxgi_common.h"
 
 template <>
-std::string DoStringise(const DXGI_FORMAT &el)
+rdcstr DoStringise(const DXGI_FORMAT &el)
 {
   BEGIN_ENUM_STRINGISE(DXGI_FORMAT);
   {
@@ -158,7 +158,7 @@ std::string DoStringise(const DXGI_FORMAT &el)
 
 // not technically DXGI, but makes more sense to have it here common between D3D versions
 template <>
-std::string DoStringise(const IID &el)
+rdcstr DoStringise(const IID &el)
 {
   return StringFormat::Fmt("GUID {%08x-%04hx-%04hx-%02x%02x-%02x%02x%02x%02x%02x%02x}", el.Data1,
                            el.Data2, el.Data3, el.Data4[0], el.Data4[1], el.Data4[2], el.Data4[3],
@@ -166,7 +166,7 @@ std::string DoStringise(const IID &el)
 }
 
 template <>
-std::string DoStringise(const D3D_FEATURE_LEVEL &el)
+rdcstr DoStringise(const D3D_FEATURE_LEVEL &el)
 {
   BEGIN_ENUM_STRINGISE(D3D_FEATURE_LEVEL);
   {
@@ -184,10 +184,11 @@ std::string DoStringise(const D3D_FEATURE_LEVEL &el)
 }
 
 template <>
-std::string DoStringise(const D3D_DRIVER_TYPE &el)
+rdcstr DoStringise(const D3D_DRIVER_TYPE &el)
 {
   BEGIN_ENUM_STRINGISE(D3D_DRIVER_TYPE);
   {
+    STRINGISE_ENUM(D3D_DRIVER_TYPE_UNKNOWN)
     STRINGISE_ENUM(D3D_DRIVER_TYPE_HARDWARE)
     STRINGISE_ENUM(D3D_DRIVER_TYPE_REFERENCE)
     STRINGISE_ENUM(D3D_DRIVER_TYPE_NULL)
@@ -199,13 +200,14 @@ std::string DoStringise(const D3D_DRIVER_TYPE &el)
 
 // for HRESULT
 template <>
-std::string DoStringise(const long &el)
+rdcstr DoStringise(const long &el)
 {
   HRESULT hr = (HRESULT)el;
 
   switch(hr)
   {
     STRINGISE_ENUM(S_OK)
+    STRINGISE_ENUM(S_FALSE)
     STRINGISE_ENUM(DXGI_ERROR_INVALID_CALL)
     STRINGISE_ENUM(DXGI_ERROR_NOT_FOUND)
     STRINGISE_ENUM(DXGI_ERROR_DEVICE_REMOVED)
@@ -213,6 +215,7 @@ std::string DoStringise(const long &el)
     STRINGISE_ENUM(DXGI_ERROR_DEVICE_RESET)
     STRINGISE_ENUM(DXGI_ERROR_WAS_STILL_DRAWING)
     STRINGISE_ENUM(DXGI_ERROR_UNSUPPORTED)
+    STRINGISE_ENUM(DXGI_ERROR_ACCESS_DENIED)
     STRINGISE_ENUM(E_OUTOFMEMORY)
     STRINGISE_ENUM(E_INVALIDARG)
     STRINGISE_ENUM(E_NOINTERFACE)
@@ -239,4 +242,4 @@ TEST_CASE("DXGI ToStr", "[tostr][d3d]")
   CHECK(ToStr(guid) == "GUID {2dc365d4-c562-411e-9d34-1659a4f31ddd}");
 }
 
-#endif
\ No newline at end of file
+#endif
