@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,18 +23,17 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "driver/dxgi/dxgi_wrapped.h"
-#include <stddef.h>
-#include <stdio.h>
+#include "dxgi_wrapped.h"
 #include "core/core.h"
 #include "serialise/serialiser.h"
+#include "dxgi_common.h"
 
 ID3D11Resource *UnwrapDXResource(void *dxObject);
 IDXGIResource *UnwrapDXGIResource(void *dxgiObject);
 
 WRAPPED_POOL_INST(WrappedIDXGIDevice4);
 
-std::vector<D3DDeviceCallback> WrappedIDXGISwapChain4::m_D3DCallbacks;
+rdcarray<D3DDeviceCallback> WrappedIDXGISwapChain4::m_D3DCallbacks;
 
 ID3DDevice *GetD3DDevice(IUnknown *pDevice)
 {
@@ -79,6 +78,30 @@ bool RefCountDXGIObject::HandleWrap(REFIID riid, void **ppvObject)
     *ppvObject = (IDXGIAdapter *)(new WrappedIDXGIAdapter4(real));
     return true;
   }
+  else if(riid == __uuidof(IDXGIAdapter1))
+  {
+    IDXGIAdapter1 *real = (IDXGIAdapter1 *)(*ppvObject);
+    *ppvObject = (IDXGIAdapter1 *)(new WrappedIDXGIAdapter4(real));
+    return true;
+  }
+  else if(riid == __uuidof(IDXGIAdapter2))
+  {
+    IDXGIAdapter2 *real = (IDXGIAdapter2 *)(*ppvObject);
+    *ppvObject = (IDXGIAdapter2 *)(new WrappedIDXGIAdapter4(real));
+    return true;
+  }
+  else if(riid == __uuidof(IDXGIAdapter3))
+  {
+    IDXGIAdapter3 *real = (IDXGIAdapter3 *)(*ppvObject);
+    *ppvObject = (IDXGIAdapter3 *)(new WrappedIDXGIAdapter4(real));
+    return true;
+  }
+  else if(riid == __uuidof(IDXGIAdapter4))
+  {
+    IDXGIAdapter4 *real = (IDXGIAdapter4 *)(*ppvObject);
+    *ppvObject = (IDXGIAdapter4 *)(new WrappedIDXGIAdapter4(real));
+    return true;
+  }
   else if(riid == __uuidof(IDXGIFactory))
   {
     // yes I know PRECISELY how fucked up this is. Speak to microsoft - after KB2670838 the internal
@@ -97,30 +120,12 @@ bool RefCountDXGIObject::HandleWrap(REFIID riid, void **ppvObject)
     RDCERR("Unexpected uuid in RefCountDXGIObject::HandleWrap");
     return false;
   }
-  else if(riid == __uuidof(IDXGIAdapter1))
-  {
-    IDXGIAdapter1 *real = (IDXGIAdapter1 *)(*ppvObject);
-    *ppvObject = (IDXGIAdapter1 *)(new WrappedIDXGIAdapter4(real));
-    return true;
-  }
   else if(riid == __uuidof(IDXGIFactory1))
   {
     IDXGIFactory1 *real = (IDXGIFactory1 *)(*ppvObject);
     *ppvObject = (IDXGIFactory1 *)(new WrappedIDXGIFactory(real));
     return true;
   }
-  else if(riid == __uuidof(IDXGIAdapter2))
-  {
-    IDXGIAdapter2 *real = (IDXGIAdapter2 *)(*ppvObject);
-    *ppvObject = (IDXGIAdapter2 *)(new WrappedIDXGIAdapter4(real));
-    return true;
-  }
-  else if(riid == __uuidof(IDXGIAdapter3))
-  {
-    IDXGIAdapter3 *real = (IDXGIAdapter3 *)(*ppvObject);
-    *ppvObject = (IDXGIAdapter3 *)(new WrappedIDXGIAdapter4(real));
-    return true;
-  }
   else if(riid == __uuidof(IDXGIFactory2))
   {
     IDXGIFactory2 *real = (IDXGIFactory2 *)(*ppvObject);
@@ -354,6 +359,8 @@ HRESULT WrappedIDXGISwapChain4::ResizeBuffers(
 
   WrapBuffersAfterResize();
 
+  m_LastPresentedBuffer = -1;
+
   return ret;
 }
 
@@ -388,6 +395,8 @@ HRESULT WrappedIDXGISwapChain4::ResizeBuffers1(_In_ UINT BufferCount, _In_ UINT
 
   WrapBuffersAfterResize();
 
+  m_LastPresentedBuffer = -1;
+
   return ret;
 }
 
@@ -475,7 +484,7 @@ HRESULT WrappedIDXGISwapChain4::GetBuffer(
     {
       DXGI_SWAP_CHAIN_DESC desc;
       GetDesc(&desc);
-      tex = m_pDevice->WrapSwapchainBuffer(this, &desc, Buffer, realSurface);
+      tex = m_pDevice->WrapSwapchainBuffer(this, desc.BufferDesc.Format, Buffer, realSurface);
     }
 
     // if the original UUID was IDXGISurface, fixup for the expected interface being returned
@@ -536,7 +545,11 @@ HRESULT WrappedIDXGISwapChain4::Present(
     SyncInterval = 0;
   }
 
-  m_pDevice->Present(this, SyncInterval, Flags);
+  if((Flags & DXGI_PRESENT_TEST) == 0)
+  {
+    TickLastPresentedBuffer();
+    m_pDevice->Present(this, SyncInterval, Flags);
+  }
 
   return m_pReal->Present(SyncInterval, Flags);
 }
@@ -549,7 +562,11 @@ HRESULT WrappedIDXGISwapChain4::Present1(UINT SyncInterval, UINT Flags,
     SyncInterval = 0;
   }
 
-  m_pDevice->Present(this, SyncInterval, Flags);
+  if((Flags & DXGI_PRESENT_TEST) == 0)
+  {
+    TickLastPresentedBuffer();
+    m_pDevice->Present(this, SyncInterval, Flags);
+  }
 
   return m_pReal1->Present1(SyncInterval, Flags, pPresentParameters);
 }
@@ -595,6 +612,125 @@ WrappedIDXGIOutput6::~WrappedIDXGIOutput6()
   SAFE_RELEASE(m_Owner);
 }
 
+HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::FindClosestMatchingMode(
+    const DXGI_MODE_DESC *pModeToMatch, DXGI_MODE_DESC *pClosestMatch, IUnknown *pConcernedDevice)
+{
+  ID3DDevice *wrapDevice = GetD3DDevice(pConcernedDevice);
+
+  if(wrapDevice)
+    return m_pReal->FindClosestMatchingMode(pModeToMatch, pClosestMatch,
+                                            wrapDevice->GetRealIUnknown());
+
+  if(pConcernedDevice)
+    RDCERR("Unrecognised device in FindClosestMatchingMode()");
+
+  return E_INVALIDARG;
+}
+
+HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::TakeOwnership(IUnknown *pDevice, BOOL Exclusive)
+{
+  ID3DDevice *wrapDevice = GetD3DDevice(pDevice);
+
+  if(wrapDevice)
+    return m_pReal->TakeOwnership(wrapDevice->GetRealIUnknown(), Exclusive);
+
+  // since this is supposed to be an internal function, allow passing through the pointer directly
+  // just in case
+  return m_pReal->TakeOwnership(pDevice, Exclusive);
+}
+
+HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::FindClosestMatchingMode1(
+    const DXGI_MODE_DESC1 *pModeToMatch, DXGI_MODE_DESC1 *pClosestMatch, IUnknown *pConcernedDevice)
+{
+  ID3DDevice *wrapDevice = GetD3DDevice(pConcernedDevice);
+
+  if(wrapDevice)
+    return m_pReal1->FindClosestMatchingMode1(pModeToMatch, pClosestMatch,
+                                              wrapDevice->GetRealIUnknown());
+
+  if(pConcernedDevice)
+    RDCERR("Unrecognised device in FindClosestMatchingMode1()");
+
+  return E_INVALIDARG;
+}
+
+HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::GetDisplaySurfaceData1(IDXGIResource *pDestination)
+{
+  return m_pReal1->GetDisplaySurfaceData1(UnwrapDXGIResource(pDestination));
+}
+
+HRESULT STDMETHODCALLTYPE
+WrappedIDXGIOutput6::DuplicateOutput(IUnknown *pDevice, IDXGIOutputDuplication **ppOutputDuplication)
+{
+  if(!ppOutputDuplication)
+    return E_INVALIDARG;
+
+  ID3DDevice *wrapDevice = GetD3DDevice(pDevice);
+
+  if(wrapDevice)
+  {
+    IDXGIOutputDuplication *dup = NULL;
+    HRESULT ret = m_pReal1->DuplicateOutput(wrapDevice->GetRealIUnknown(), &dup);
+
+    if(SUCCEEDED(ret) && dup)
+      dup = new WrappedIDXGIOutputDuplication(wrapDevice, dup);
+
+    *ppOutputDuplication = dup;
+
+    return ret;
+  }
+
+  if(pDevice)
+    RDCERR("Unrecognised device in DuplicateOutput()");
+
+  return E_INVALIDARG;
+}
+
+HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::CheckOverlaySupport(DXGI_FORMAT EnumFormat,
+                                                                   IUnknown *pConcernedDevice,
+                                                                   UINT *pFlags)
+{
+  ID3DDevice *wrapDevice = GetD3DDevice(pConcernedDevice);
+
+  if(wrapDevice)
+    return m_pReal3->CheckOverlaySupport(EnumFormat, wrapDevice->GetRealIUnknown(), pFlags);
+
+  if(pConcernedDevice)
+    RDCERR("Unrecognised device in CheckOverlaySupport()");
+
+  return E_INVALIDARG;
+}
+
+HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::DuplicateOutput1(
+    IUnknown *pDevice, UINT Flags, UINT SupportedFormatsCount, const DXGI_FORMAT *pSupportedFormats,
+    IDXGIOutputDuplication **ppOutputDuplication)
+{
+  if(!ppOutputDuplication)
+    return E_INVALIDARG;
+
+  ID3DDevice *wrapDevice = GetD3DDevice(pDevice);
+
+  if(wrapDevice)
+  {
+    IDXGIOutputDuplication *dup = NULL;
+    HRESULT ret =
+        m_pReal5->DuplicateOutput1(wrapDevice->GetRealIUnknown(), Flags, SupportedFormatsCount,
+                                   pSupportedFormats, ppOutputDuplication);
+
+    if(SUCCEEDED(ret) && dup)
+      dup = new WrappedIDXGIOutputDuplication(wrapDevice, dup);
+
+    *ppOutputDuplication = dup;
+
+    return ret;
+  }
+
+  if(pDevice)
+    RDCERR("Unrecognised device in DuplicateOutput()");
+
+  return E_INVALIDARG;
+}
+
 HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::QueryInterface(REFIID riid, void **ppvObject)
 {
   if(riid == __uuidof(IDXGIOutput))
@@ -644,7 +780,7 @@ HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::QueryInterface(REFIID riid, void
   }
   else if(riid == __uuidof(IDXGIOutput4))
   {
-    if(m_pReal3)
+    if(m_pReal4)
     {
       AddRef();
       *ppvObject = (IDXGIOutput4 *)this;
@@ -657,7 +793,7 @@ HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::QueryInterface(REFIID riid, void
   }
   else if(riid == __uuidof(IDXGIOutput5))
   {
-    if(m_pReal3)
+    if(m_pReal5)
     {
       AddRef();
       *ppvObject = (IDXGIOutput5 *)this;
@@ -668,6 +804,19 @@ HRESULT STDMETHODCALLTYPE WrappedIDXGIOutput6::QueryInterface(REFIID riid, void
       return E_NOINTERFACE;
     }
   }
+  else if(riid == __uuidof(IDXGIOutput6))
+  {
+    if(m_pReal6)
+    {
+      AddRef();
+      *ppvObject = (IDXGIOutput6 *)this;
+      return S_OK;
+    }
+    else
+    {
+      return E_NOINTERFACE;
+    }
+  }
   else
   {
     WarnUnknownGUID("IDXGIOutput", riid);
@@ -745,6 +894,19 @@ HRESULT STDMETHODCALLTYPE WrappedIDXGIAdapter4::QueryInterface(REFIID riid, void
       return E_NOINTERFACE;
     }
   }
+  else if(riid == __uuidof(IDXGIAdapter4))
+  {
+    if(m_pReal3)
+    {
+      AddRef();
+      *ppvObject = (IDXGIAdapter4 *)this;
+      return S_OK;
+    }
+    else
+    {
+      return E_NOINTERFACE;
+    }
+  }
   else
   {
     WarnUnknownGUID("IDXGIAdapter", riid);
@@ -857,9 +1019,9 @@ HRESULT STDMETHODCALLTYPE WrappedIDXGIDevice4::QueryInterface(REFIID riid, void
   return RefCountDXGIObject::QueryInterface(riid, ppvObject);
 }
 
-std::vector<IDXGIResource *> UnwrapResourceSet(UINT NumResources, IDXGIResource *const *ppResources)
+rdcarray<IDXGIResource *> UnwrapResourceSet(UINT NumResources, IDXGIResource *const *ppResources)
 {
-  std::vector<IDXGIResource *> resources;
+  rdcarray<IDXGIResource *> resources;
   resources.resize(NumResources);
   for(UINT i = 0; i < NumResources; i++)
   {
@@ -879,7 +1041,7 @@ HRESULT STDMETHODCALLTYPE WrappedIDXGIDevice4::OfferResources(UINT NumResources,
                                                               IDXGIResource *const *ppResources,
                                                               DXGI_OFFER_RESOURCE_PRIORITY Priority)
 {
-  std::vector<IDXGIResource *> resources = UnwrapResourceSet(NumResources, ppResources);
+  rdcarray<IDXGIResource *> resources = UnwrapResourceSet(NumResources, ppResources);
   return m_pReal2->OfferResources(NumResources, resources.data(), Priority);
 }
 
@@ -887,7 +1049,7 @@ HRESULT STDMETHODCALLTYPE WrappedIDXGIDevice4::ReclaimResources(UINT NumResource
                                                                 IDXGIResource *const *ppResources,
                                                                 BOOL *pDiscarded)
 {
-  std::vector<IDXGIResource *> resources = UnwrapResourceSet(NumResources, ppResources);
+  rdcarray<IDXGIResource *> resources = UnwrapResourceSet(NumResources, ppResources);
   return m_pReal2->ReclaimResources(NumResources, resources.data(), pDiscarded);
 }
 
@@ -896,7 +1058,7 @@ HRESULT STDMETHODCALLTYPE WrappedIDXGIDevice4::OfferResources1(UINT NumResources
                                                                DXGI_OFFER_RESOURCE_PRIORITY Priority,
                                                                UINT Flags)
 {
-  std::vector<IDXGIResource *> resources = UnwrapResourceSet(NumResources, ppResources);
+  rdcarray<IDXGIResource *> resources = UnwrapResourceSet(NumResources, ppResources);
   return m_pReal4->OfferResources1(NumResources, resources.data(), Priority, Flags);
 }
 
@@ -905,7 +1067,7 @@ HRESULT STDMETHODCALLTYPE WrappedIDXGIDevice4::ReclaimResources1(UINT NumResourc
 
                                                                  DXGI_RECLAIM_RESOURCE_RESULTS *pResults)
 {
-  std::vector<IDXGIResource *> resources = UnwrapResourceSet(NumResources, ppResources);
+  rdcarray<IDXGIResource *> resources = UnwrapResourceSet(NumResources, ppResources);
   return m_pReal4->ReclaimResources1(NumResources, resources.data(), pResults);
 }
 
@@ -1249,3 +1411,47 @@ HRESULT WrappedIDXGIFactory::CreateSwapChainForComposition(IUnknown *pDevice,
 
   return m_pReal2->CreateSwapChainForComposition(pDevice, pDesc, unwrappedOutput, ppSwapChain);
 }
+
+WrappedIDXGIOutputDuplication::WrappedIDXGIOutputDuplication(ID3DDevice *device,
+                                                             IDXGIOutputDuplication *real)
+    : RefCountDXGIObject(real), m_Device(device), m_pReal(real)
+{
+}
+
+WrappedIDXGIOutputDuplication::~WrappedIDXGIOutputDuplication()
+{
+  SAFE_RELEASE(m_pReal);
+}
+
+HRESULT STDMETHODCALLTYPE WrappedIDXGIOutputDuplication::AcquireNextFrame(
+    UINT TimeoutInMilliseconds, DXGI_OUTDUPL_FRAME_INFO *pFrameInfo, IDXGIResource **ppDesktopResource)
+{
+  if(!ppDesktopResource)
+    return E_INVALIDARG;
+
+  IDXGIResource *desktop = NULL;
+  HRESULT ret = m_pReal->AcquireNextFrame(TimeoutInMilliseconds, pFrameInfo, &desktop);
+
+  if(SUCCEEDED(ret) && desktop)
+    desktop = m_Device->WrapExternalDXGIResource(desktop);
+
+  *ppDesktopResource = desktop;
+
+  return ret;
+}
+
+HRESULT STDMETHODCALLTYPE WrappedIDXGIOutputDuplication::QueryInterface(REFIID riid, void **ppvObject)
+{
+  if(riid == __uuidof(IDXGIOutputDuplication))
+  {
+    AddRef();
+    *ppvObject = (IDXGIOutputDuplication *)this;
+    return S_OK;
+  }
+  else
+  {
+    WarnUnknownGUID("IDXGIOutputDuplication", riid);
+  }
+
+  return RefCountDXGIObject::QueryInterface(riid, ppvObject);
+}
