@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -77,6 +77,9 @@ CGLError GL_EXPORT_NAME(CGLCreateContext)(CGLPixelFormatObj pix, CGLContextObj s
   CGL.CGLDescribePixelFormat(pix, 0, kCGLPFASamples, &value);
   init.multiSamples = RDCMAX(1, value);
 
+  CGL.CGLDescribePixelFormat(pix, 0, kCGLPFAOpenGLProfile, &value);
+  bool isCore = (value >= kCGLOGLPVersion_3_2_Core);
+
   GLWindowingData data;
   data.wnd = NULL;
   data.ctx = *ctx;
@@ -84,7 +87,7 @@ CGLError GL_EXPORT_NAME(CGLCreateContext)(CGLPixelFormatObj pix, CGLContextObj s
 
   {
     SCOPED_LOCK(glLock);
-    cglhook.driver.CreateContext(data, share, init, true, true);
+    cglhook.driver.CreateContext(data, share, init, isCore, isCore);
   }
 
   return ret;
@@ -175,7 +178,9 @@ CGLError GL_EXPORT_NAME(CGLFlushDrawable)(CGLContextObj ctx)
 
     CGL.CGLGetSurface(ctx, &conn, &window, &surface);
 
-    cglhook.driver.SwapBuffers((void *)(uintptr_t)window);
+    gl_CurChunk = GLChunk::CGLFlushDrawable;
+
+    cglhook.driver.SwapBuffers(WindowingSystem::MacOS, (void *)(uintptr_t)window);
   }
 
   CGLError ret;
