@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,9 +22,9 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "driver/gl/egl_dispatch_table.h"
-#include "driver/gl/gl_driver.h"
 #include "hooks/hooks.h"
+#include "egl_dispatch_table.h"
+#include "gl_driver.h"
 
 #if ENABLED(RDOC_POSIX)
 #include <dlfcn.h>
@@ -38,10 +38,23 @@
 #if ENABLED(RDOC_LINUX)
 namespace Keyboard
 {
-void CloneDisplay(Display *dpy);
+void UseXlibDisplay(Display *dpy);
+WindowingSystem UseUnknownDisplay(void *disp);
+void UseWaylandDisplay(wl_display *disp);
 }
 #endif
 
+struct SurfaceConfig
+{
+  WindowingSystem system;
+  void *wnd;
+};
+
+struct DisplayConfig
+{
+  WindowingSystem system;
+};
+
 class EGLHook : LibraryHook
 {
 public:
@@ -54,7 +67,8 @@ public:
   WrappedOpenGL driver;
   std::set<EGLContext> contexts;
   std::map<EGLContext, EGLConfig> configs;
-  std::map<EGLSurface, EGLNativeWindowType> windows;
+  std::map<EGLSurface, SurfaceConfig> windows;
+  std::map<EGLDisplay, DisplayConfig> displays;
 
   // indicates we're in a swap function, so don't process the swap any further if we recurse - could
   // happen due to driver implementation of one function calling another
@@ -108,6 +122,33 @@ public:
 
 } eglhook;
 
+// On linux if a user doesn't link to libEGL or try to dlopen it, but just calls dlsym with
+// RTLD_NEXT it might successfully one of our functions without anything ever loading libEGL. Then
+// our attempts to call onwards will fail. When any of our functions are called we check to see if
+// DEFAULT_HANDLE is RTLD_NEXT and if so we manually load the library. This will trigger our hook
+// callback and we'll get a specific library handle.
+//
+// On other platforms this is not needed because we know the real library will be loaded before any
+// of our hooks can be called
+static void EnsureRealLibraryLoaded()
+{
+#if ENABLED(RDOC_LINUX)
+  if(eglhook.handle == DEFAULT_HANDLE)
+  {
+    if(!RenderDoc::Inst().IsReplayApp())
+      RDCLOG("Loading libEGL at the last second");
+
+    void *handle = Process::LoadModule("libEGL.so");
+
+    if(!handle)
+      handle = Process::LoadModule("libEGL.so.1");
+
+    if(RenderDoc::Inst().IsReplayApp())
+      eglhook.handle = handle;
+  }
+#endif
+}
+
 HOOK_EXPORT EGLDisplay EGLAPIENTRY eglGetDisplay_renderdoc_hooked(EGLNativeDisplayType display)
 {
   if(RenderDoc::Inst().IsReplayApp())
@@ -118,13 +159,41 @@ HOOK_EXPORT EGLDisplay EGLAPIENTRY eglGetDisplay_renderdoc_hooked(EGLNativeDispl
     return EGL.GetDisplay(display);
   }
 
+  EnsureRealLibraryLoaded();
+
 #if ENABLED(RDOC_LINUX)
-  Keyboard::CloneDisplay(display);
+  Keyboard::UseUnknownDisplay((void *)display);
 #endif
 
   return EGL.GetDisplay(display);
 }
 
+HOOK_EXPORT EGLDisplay EGLAPIENTRY eglGetPlatformDisplay_renderdoc_hooked(EGLenum platform,
+                                                                          void *native_display,
+                                                                          const EGLAttrib *attrib_list)
+{
+  if(RenderDoc::Inst().IsReplayApp())
+  {
+    if(!EGL.GetDisplay)
+      EGL.PopulateForReplay();
+
+    return EGL.GetPlatformDisplay(platform, native_display, attrib_list);
+  }
+
+  EnsureRealLibraryLoaded();
+
+#if ENABLED(RDOC_LINUX)
+  if(platform == EGL_PLATFORM_X11_KHR)
+    Keyboard::UseXlibDisplay((Display *)native_display);
+  else if(platform == EGL_PLATFORM_WAYLAND_KHR)
+    Keyboard::UseWaylandDisplay((wl_display *)native_display);
+  else
+    RDCWARN("Unknown platform %x in eglGetPlatformDisplay", platform);
+#endif
+
+  return EGL.GetPlatformDisplay(platform, native_display, attrib_list);
+}
+
 HOOK_EXPORT EGLBoolean EGLAPIENTRY eglBindAPI_renderdoc_hooked(EGLenum api)
 {
   if(RenderDoc::Inst().IsReplayApp())
@@ -135,6 +204,8 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglBindAPI_renderdoc_hooked(EGLenum api)
     return EGL.BindAPI(api);
   }
 
+  EnsureRealLibraryLoaded();
+
   EGLBoolean ret = EGL.BindAPI(api);
 
   if(ret)
@@ -156,9 +227,11 @@ HOOK_EXPORT EGLContext EGLAPIENTRY eglCreateContext_renderdoc_hooked(EGLDisplay
     return EGL.CreateContext(display, config, shareContext, attribList);
   }
 
+  EnsureRealLibraryLoaded();
+
   LibraryHooks::Refresh();
 
-  vector<EGLint> attribs;
+  rdcarray<EGLint> attribs;
 
   // modify attribList to our liking
   {
@@ -186,9 +259,18 @@ HOOK_EXPORT EGLContext EGLAPIENTRY eglCreateContext_renderdoc_hooked(EGLDisplay
           else
             value &= ~EGL_CONTEXT_OPENGL_DEBUG_BIT_KHR;
 
+          // remove NO_ERROR bit
+          value &= ~GL_CONTEXT_FLAG_NO_ERROR_BIT_KHR;
+
           flagsFound = true;
         }
 
+        if(name == EGL_CONTEXT_OPENGL_NO_ERROR_KHR)
+        {
+          // remove this attribute so that we can be more stable
+          continue;
+        }
+
         attribs.push_back(name);
         attribs.push_back(value);
       }
@@ -265,6 +347,8 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglDestroyContext_renderdoc_hooked(EGLDisplay
     return EGL.DestroyContext(dpy, ctx);
   }
 
+  EnsureRealLibraryLoaded();
+
   eglhook.driver.SetDriverType(eglhook.activeAPI);
   {
     SCOPED_LOCK(glLock);
@@ -288,13 +372,43 @@ HOOK_EXPORT EGLSurface EGLAPIENTRY eglCreateWindowSurface_renderdoc_hooked(EGLDi
     return EGL.CreateWindowSurface(dpy, config, win, attrib_list);
   }
 
+  EnsureRealLibraryLoaded();
+
   EGLSurface ret = EGL.CreateWindowSurface(dpy, config, win, attrib_list);
 
   if(ret)
   {
     SCOPED_LOCK(glLock);
 
-    eglhook.windows[ret] = win;
+    // spec says it's implementation dependent what happens, so we assume that we're using the same
+    // window system as the display
+    eglhook.windows[ret] = {eglhook.displays[dpy].system, (void *)win};
+  }
+
+  return ret;
+}
+
+HOOK_EXPORT EGLSurface EGLAPIENTRY eglCreatePlatformWindowSurface_renderdoc_hooked(
+    EGLDisplay dpy, EGLConfig config, void *native_window, const EGLAttrib *attrib_list)
+{
+  if(RenderDoc::Inst().IsReplayApp())
+  {
+    if(!EGL.CreatePlatformWindowSurface)
+      EGL.PopulateForReplay();
+
+    return EGL.CreatePlatformWindowSurface(dpy, config, native_window, attrib_list);
+  }
+
+  EnsureRealLibraryLoaded();
+
+  EGLSurface ret = EGL.CreatePlatformWindowSurface(dpy, config, native_window, attrib_list);
+
+  if(ret)
+  {
+    SCOPED_LOCK(glLock);
+
+    // spec guarantees that we're using the same window system as the display
+    eglhook.windows[ret] = {eglhook.displays[dpy].system, native_window};
   }
 
   return ret;
@@ -312,6 +426,8 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglMakeCurrent_renderdoc_hooked(EGLDisplay di
     return EGL.MakeCurrent(display, draw, read, ctx);
   }
 
+  EnsureRealLibraryLoaded();
+
   EGLBoolean ret = EGL.MakeCurrent(display, draw, read, ctx);
 
   if(ret)
@@ -332,17 +448,20 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglMakeCurrent_renderdoc_hooked(EGLDisplay di
       GL.DriverForEmulation(&eglhook.driver);
     }
 
+    SurfaceConfig cfg = eglhook.windows[draw];
+
     GLWindowingData data;
     data.egl_dpy = display;
     data.egl_wnd = draw;
     data.egl_ctx = ctx;
-    data.wnd = (decltype(data.wnd))eglhook.windows[draw];
+    data.wnd = (decltype(data.wnd))cfg.wnd;
 
     if(!data.wnd)
     {
       // could be a pbuffer surface or other offscreen rendering. We want a valid wnd, so set it to
       // a dummy value
       data.wnd = (decltype(data.wnd))(void *)(uintptr_t(0xdeadbeef) + uintptr_t(draw));
+      cfg.system = WindowingSystem::Headless;
     }
 
     // we could query this out technically but it's easier to keep a map
@@ -368,6 +487,8 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglSwapBuffers_renderdoc_hooked(EGLDisplay dp
     return EGL.SwapBuffers(dpy, surface);
   }
 
+  EnsureRealLibraryLoaded();
+
   SCOPED_LOCK(glLock);
 
   eglhook.driver.SetDriverType(eglhook.activeAPI);
@@ -380,7 +501,11 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglSwapBuffers_renderdoc_hooked(EGLDisplay dp
 
     eglhook.RefreshWindowParameters(data);
 
-    eglhook.driver.SwapBuffers(surface);
+    SurfaceConfig cfg = eglhook.windows[surface];
+
+    gl_CurChunk = GLChunk::eglSwapBuffers;
+
+    eglhook.driver.SwapBuffers(cfg.system, cfg.wnd);
   }
 
   {
@@ -404,11 +529,19 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglPostSubBufferNV_renderdoc_hooked(EGLDispla
     return EGL.PostSubBufferNV(dpy, surface, x, y, width, height);
   }
 
+  EnsureRealLibraryLoaded();
+
   SCOPED_LOCK(glLock);
 
   eglhook.driver.SetDriverType(eglhook.activeAPI);
   if(!eglhook.driver.UsesVRFrameMarkers() && !eglhook.swapping)
-    eglhook.driver.SwapBuffers((void *)eglhook.windows[surface]);
+  {
+    SurfaceConfig cfg = eglhook.windows[surface];
+
+    gl_CurChunk = GLChunk::eglPostSubBufferNV;
+
+    eglhook.driver.SwapBuffers(cfg.system, cfg.wnd);
+  }
 
   {
     eglhook.swapping = true;
@@ -431,11 +564,19 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglSwapBuffersWithDamageEXT_renderdoc_hooked(
     return EGL.SwapBuffersWithDamageEXT(dpy, surface, rects, n_rects);
   }
 
+  EnsureRealLibraryLoaded();
+
   SCOPED_LOCK(glLock);
 
   eglhook.driver.SetDriverType(eglhook.activeAPI);
   if(!eglhook.driver.UsesVRFrameMarkers() && !eglhook.swapping)
-    eglhook.driver.SwapBuffers((void *)eglhook.windows[surface]);
+  {
+    SurfaceConfig cfg = eglhook.windows[surface];
+
+    gl_CurChunk = GLChunk::eglSwapBuffersWithDamageEXT;
+
+    eglhook.driver.SwapBuffers(cfg.system, cfg.wnd);
+  }
 
   {
     eglhook.swapping = true;
@@ -458,11 +599,19 @@ HOOK_EXPORT EGLBoolean EGLAPIENTRY eglSwapBuffersWithDamageKHR_renderdoc_hooked(
     return EGL.SwapBuffersWithDamageKHR(dpy, surface, rects, n_rects);
   }
 
+  EnsureRealLibraryLoaded();
+
   SCOPED_LOCK(glLock);
 
   eglhook.driver.SetDriverType(eglhook.activeAPI);
   if(!eglhook.driver.UsesVRFrameMarkers() && !eglhook.swapping)
-    eglhook.driver.SwapBuffers((void *)eglhook.windows[surface]);
+  {
+    SurfaceConfig cfg = eglhook.windows[surface];
+
+    gl_CurChunk = GLChunk::eglSwapBuffersWithDamageKHR;
+
+    eglhook.driver.SwapBuffers(cfg.system, cfg.wnd);
+  }
 
   {
     eglhook.swapping = true;
@@ -483,6 +632,8 @@ eglGetProcAddress_renderdoc_hooked(const char *func)
     return EGL.GetProcAddress(func);
   }
 
+  EnsureRealLibraryLoaded();
+
   __eglMustCastToProperFunctionPointerType realFunc = NULL;
   {
     ScopedSuppressHooking suppress;
@@ -495,9 +646,9 @@ eglGetProcAddress_renderdoc_hooked(const char *func)
     return realFunc;
 
 // return our egl hooks
-#define GPA_FUNCTION(name)                                                                          \
-  if(!strcmp(func, "egl" STRINGIZE(name)))                                                          \
-    return (__eglMustCastToProperFunctionPointerType)&CONCAT(egl, CONCAT(name, _renderdoc_hooked)); \
+#define GPA_FUNCTION(name, isext, replayrequired) \
+  if(!strcmp(func, "egl" STRINGIZE(name)))        \
+    return (__eglMustCastToProperFunctionPointerType)&CONCAT(egl, CONCAT(name, _renderdoc_hooked));
   EGL_HOOKED_SYMBOLS(GPA_FUNCTION)
 #undef GPA_FUNCTION
 
@@ -524,6 +675,12 @@ HOOK_EXPORT EGLDisplay EGLAPIENTRY eglGetDisplay(EGLNativeDisplayType display)
   return eglGetDisplay_renderdoc_hooked(display);
 }
 
+HOOK_EXPORT EGLDisplay EGLAPIENTRY eglGetPlatformDisplay(EGLenum platform, void *native_display,
+                                                         const EGLAttrib *attrib_list)
+{
+  return eglGetPlatformDisplay_renderdoc_hooked(platform, native_display, attrib_list);
+}
+
 HOOK_EXPORT EGLContext EGLAPIENTRY eglCreateContext(EGLDisplay display, EGLConfig config,
                                                     EGLContext shareContext, EGLint const *attribList)
 {
@@ -542,6 +699,13 @@ HOOK_EXPORT EGLSurface EGLAPIENTRY eglCreateWindowSurface(EGLDisplay dpy, EGLCon
   return eglCreateWindowSurface_renderdoc_hooked(dpy, config, win, attrib_list);
 }
 
+HOOK_EXPORT EGLSurface EGLAPIENTRY eglCreatePlatformWindowSurface(EGLDisplay dpy, EGLConfig config,
+                                                                  void *native_window,
+                                                                  const EGLAttrib *attrib_list)
+{
+  return eglCreatePlatformWindowSurface_renderdoc_hooked(dpy, config, native_window, attrib_list);
+}
+
 HOOK_EXPORT EGLBoolean EGLAPIENTRY eglMakeCurrent(EGLDisplay display, EGLSurface draw,
                                                   EGLSurface read, EGLContext ctx)
 {
@@ -584,6 +748,7 @@ HOOK_EXPORT __eglMustCastToProperFunctionPointerType EGLAPIENTRY eglGetProcAddre
   typedef ret (*CONCAT(function, _hooktype))();                                           \
   HOOK_EXPORT ret EGLAPIENTRY function()                                                  \
   {                                                                                       \
+    EnsureRealLibraryLoaded();                                                            \
     CONCAT(function, _hooktype)                                                           \
     real = (CONCAT(function, _hooktype))Process::GetFunctionAddress(eglhook.handle,       \
                                                                     STRINGIZE(function)); \
@@ -594,6 +759,7 @@ HOOK_EXPORT __eglMustCastToProperFunctionPointerType EGLAPIENTRY eglGetProcAddre
   typedef ret (*CONCAT(function, _hooktype))(t1);                                         \
   HOOK_EXPORT ret EGLAPIENTRY function(t1 p1)                                             \
   {                                                                                       \
+    EnsureRealLibraryLoaded();                                                            \
     CONCAT(function, _hooktype)                                                           \
     real = (CONCAT(function, _hooktype))Process::GetFunctionAddress(eglhook.handle,       \
                                                                     STRINGIZE(function)); \
@@ -604,6 +770,7 @@ HOOK_EXPORT __eglMustCastToProperFunctionPointerType EGLAPIENTRY eglGetProcAddre
   typedef ret (*CONCAT(function, _hooktype))(t1, t2);                                     \
   HOOK_EXPORT ret EGLAPIENTRY function(t1 p1, t2 p2)                                      \
   {                                                                                       \
+    EnsureRealLibraryLoaded();                                                            \
     CONCAT(function, _hooktype)                                                           \
     real = (CONCAT(function, _hooktype))Process::GetFunctionAddress(eglhook.handle,       \
                                                                     STRINGIZE(function)); \
@@ -614,6 +781,7 @@ HOOK_EXPORT __eglMustCastToProperFunctionPointerType EGLAPIENTRY eglGetProcAddre
   typedef ret (*CONCAT(function, _hooktype))(t1, t2, t3);                                 \
   HOOK_EXPORT ret EGLAPIENTRY function(t1 p1, t2 p2, t3 p3)                               \
   {                                                                                       \
+    EnsureRealLibraryLoaded();                                                            \
     CONCAT(function, _hooktype)                                                           \
     real = (CONCAT(function, _hooktype))Process::GetFunctionAddress(eglhook.handle,       \
                                                                     STRINGIZE(function)); \
@@ -624,6 +792,7 @@ HOOK_EXPORT __eglMustCastToProperFunctionPointerType EGLAPIENTRY eglGetProcAddre
   typedef ret (*CONCAT(function, _hooktype))(t1, t2, t3, t4);                             \
   HOOK_EXPORT ret EGLAPIENTRY function(t1 p1, t2 p2, t3 p3, t4 p4)                        \
   {                                                                                       \
+    EnsureRealLibraryLoaded();                                                            \
     CONCAT(function, _hooktype)                                                           \
     real = (CONCAT(function, _hooktype))Process::GetFunctionAddress(eglhook.handle,       \
                                                                     STRINGIZE(function)); \
@@ -634,6 +803,7 @@ HOOK_EXPORT __eglMustCastToProperFunctionPointerType EGLAPIENTRY eglGetProcAddre
   typedef ret (*CONCAT(function, _hooktype))(t1, t2, t3, t4, t5);                         \
   HOOK_EXPORT ret EGLAPIENTRY function(t1 p1, t2 p2, t3 p3, t4 p4, t5 p5)                 \
   {                                                                                       \
+    EnsureRealLibraryLoaded();                                                            \
     CONCAT(function, _hooktype)                                                           \
     real = (CONCAT(function, _hooktype))Process::GetFunctionAddress(eglhook.handle,       \
                                                                     STRINGIZE(function)); \
@@ -687,6 +857,20 @@ EGL_PASSTHRU_0(EGLBoolean, eglWaitClient)
 /* EGL 1.4 */
 EGL_PASSTHRU_0(EGLContext, eglGetCurrentContext)
 
+/* EGL 1.5 */
+EGL_PASSTHRU_3(EGLSync, eglCreateSync, EGLDisplay, dpy, EGLenum, type, const EGLAttrib *, attrib_list)
+EGL_PASSTHRU_2(EGLBoolean, eglDestroySync, EGLDisplay, dpy, EGLSync, sync)
+EGL_PASSTHRU_4(EGLint, eglClientWaitSync, EGLDisplay, dpy, EGLSync, sync, EGLint, flags, EGLTime,
+               timeout)
+EGL_PASSTHRU_4(EGLBoolean, eglGetSyncAttrib, EGLDisplay, dpy, EGLSync, sync, EGLint, attribute,
+               EGLAttrib *, value)
+EGL_PASSTHRU_5(EGLImage, eglCreateImage, EGLDisplay, dpy, EGLContext, ctx, EGLenum, target,
+               EGLClientBuffer, buffer, const EGLAttrib *, attrib_list)
+EGL_PASSTHRU_2(EGLBoolean, eglDestroyImage, EGLDisplay, dpy, EGLImage, image)
+EGL_PASSTHRU_4(EGLSurface, eglCreatePlatformPixmapSurface, EGLDisplay, dpy, EGLConfig, config,
+               void *, native_pixmap, const EGLAttrib *, attrib_list)
+EGL_PASSTHRU_3(EGLBoolean, eglWaitSync, EGLDisplay, dpy, EGLSync, sync, EGLint, flags)
+
 static void EGLHooked(void *handle)
 {
   RDCDEBUG("EGL library hooked");
@@ -701,7 +885,7 @@ static void EGLHooked(void *handle)
   RDCASSERT(!RenderDoc::Inst().IsReplayApp());
 
 // fetch non-hooked functions into our dispatch table
-#define EGL_FETCH(func, isext)                                                                  \
+#define EGL_FETCH(func, isext, replayrequired)                                                  \
   EGL.func = (CONCAT(PFN_egl, func))Process::GetFunctionAddress(handle, "egl" STRINGIZE(func)); \
   if(!EGL.func && CheckConstParam(isext))                                                       \
     EGL.func = (CONCAT(PFN_egl, func))EGL.GetProcAddress("egl" STRINGIZE(func));
@@ -726,27 +910,59 @@ static void EGLHooked(void *handle)
 }
 
 #if ENABLED(RDOC_WIN32)
+
 bool ShouldHookEGL()
 {
   const char *toggle = Process::GetEnvVariable("RENDERDOC_HOOK_EGL");
 
   // if the var is set to 0, then don't hook EGL
   if(toggle && toggle[0] == '0')
+  {
+    RDCLOG(
+        "EGL hooks disabled by RENDERDOC_HOOK_EGL environment variable - "
+        "if GLES emulator is in use, underlying API will be captured");
     return false;
+  }
 
   return true;
 }
+
+#elif ENABLED(RDOC_ANDROID)
+
+bool ShouldHookEGL()
+{
+  void *egl_handle = dlopen("libEGL.so", RTLD_LAZY);
+  PFN_eglQueryString query_string = (PFN_eglQueryString)dlsym(egl_handle, "eglQueryString");
+  if(!query_string)
+  {
+    RDCERR("Unable to find eglQueryString entry point, enabling EGL hooking");
+    return true;
+  }
+
+  const char *eglExts = query_string(EGL_NO_DISPLAY, EGL_EXTENSIONS);
+
+  if(eglExts && strstr(eglExts, "EGL_ANDROID_GLES_layers"))
+  {
+    RDCLOG("EGL_ANDROID_GLES_layers detected, disabling EGL hooks - GLES layering in effect");
+    return false;
+  }
+
+  return true;
+}
+
+#else
+
+bool ShouldHookEGL()
+{
+  return true;
+}
+
 #endif
 
 void EGLHook::RegisterHooks()
 {
-#if ENABLED(RDOC_WIN32)
   if(!ShouldHookEGL())
-  {
-    RDCLOG("EGL hooks disabled - if GLES emulator is in use, underlying API will be captured");
     return;
-  }
-#endif
 
   RDCLOG("Registering EGL hooks");
 
@@ -778,10 +994,56 @@ void EGLHook::RegisterHooks()
 #endif
 
 // register EGL hooks
-#define EGL_REGISTER(func, isext)                                                 \
+#define EGL_REGISTER(func, isext, replayrequired)                                 \
   LibraryHooks::RegisterFunctionHook(                                             \
       "libEGL" LIBSUFFIX, FunctionHook("egl" STRINGIZE(func), (void **)&EGL.func, \
                                        (void *)&CONCAT(egl, CONCAT(func, _renderdoc_hooked))));
   EGL_HOOKED_SYMBOLS(EGL_REGISTER)
 #undef EGL_REGISTER
 }
+
+// Android GLES layering support
+#if ENABLED(RDOC_ANDROID)
+
+typedef __eglMustCastToProperFunctionPointerType(EGLAPIENTRY *PFNEGLGETNEXTLAYERPROCADDRESSPROC)(
+    void *, const char *funcName);
+
+HOOK_EXPORT void AndroidGLESLayer_Initialize(void *layer_id,
+                                             PFNEGLGETNEXTLAYERPROCADDRESSPROC next_gpa)
+{
+  RDCLOG("Initialising Android GLES layer with ID %p", layer_id);
+
+  // as a hook callback this is only called while capturing
+  RDCASSERT(!RenderDoc::Inst().IsReplayApp());
+
+// populate EGL dispatch table with the next layer's function pointers. Fetch all 'hooked' and
+// non-hooked functions
+#define EGL_FETCH(func, isext, replayrequired)                                 \
+  EGL.func = (CONCAT(PFN_egl, func))next_gpa(layer_id, "egl" STRINGIZE(func)); \
+  if(!EGL.func)                                                                \
+    RDCWARN("Couldn't fetch function pointer for egl" STRINGIZE(func));
+  EGL_HOOKED_SYMBOLS(EGL_FETCH)
+  EGL_NONHOOKED_SYMBOLS(EGL_FETCH)
+#undef EGL_FETCH
+
+  // populate GL dispatch table with the next layer's function pointers
+  GL.PopulateWithCallback(
+      [layer_id, next_gpa](const char *f) { return (void *)next_gpa(layer_id, f); });
+}
+
+HOOK_EXPORT void *AndroidGLESLayer_GetProcAddress(const char *funcName,
+                                                  __eglMustCastToProperFunctionPointerType next)
+{
+// return our egl hooks
+#define GPA_FUNCTION(name, isext, replayrequired) \
+  if(!strcmp(funcName, "egl" STRINGIZE(name)))    \
+    return (void *)&CONCAT(egl, CONCAT(name, _renderdoc_hooked));
+  EGL_HOOKED_SYMBOLS(GPA_FUNCTION)
+#undef GPA_FUNCTION
+
+  // otherwise, consult our database of hooks
+  // Android GLES layer spec expects us to return next unmodified for functions we don't support
+  return HookedGetProcAddress(funcName, (void *)next);
+}
+
+#endif    // ENABLED(RDOC_ANDROID)
