@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -24,6 +24,7 @@
  ******************************************************************************/
 
 #include "gl_common.h"
+#include "common/formatting.h"
 #include "core/core.h"
 #include "strings/string_utils.h"
 #include "gl_dispatch_table.h"
@@ -61,21 +62,10 @@ bool CheckReplayContext()
   REQUIRE_FUNC(glGetStringi);
   REQUIRE_FUNC(glGetIntegerv);
 
-// we can't do without these extensions, but they should be present on any reasonable driver
-// as they should have minimal or no hardware requirement. They were present on mesa 10.6
-// for all drivers which dates to mid 2015.
-#undef EXT_TO_CHECK
-#define EXT_TO_CHECK(ver, glesver, ext) ext,
-  enum
-  {
-    EXTENSION_CHECKS() ext_count,
-  };
-  bool exts[ext_count] = {};
-
   RDCLOG("Running GL replay on: %s / %s / %s", GL.glGetString(eGL_VENDOR),
          GL.glGetString(eGL_RENDERER), GL.glGetString(eGL_VERSION));
 
-  string extensionString = "";
+  rdcstr extensionString = "";
 
   GLint numExts = 0;
   GL.glGetIntegerv(eGL_NUM_EXTENSIONS, &numExts);
@@ -93,14 +83,6 @@ bool CheckReplayContext()
 
     // skip the "GL_"
     ext += 3;
-
-#undef EXT_TO_CHECK
-#define EXT_TO_CHECK(ver, glesver, extname)                                       \
-  if((!IsGLES && GLCoreVersion >= ver) || (IsGLES && GLCoreVersion >= glesver) || \
-     !strcmp(ext, STRINGIZE(extname)))                                            \
-    exts[extname] = true;
-
-    EXTENSION_CHECKS()
   }
 
   if(!extensionString.empty())
@@ -378,14 +360,8 @@ static void CheckExtFromString(const char *ext)
   }
 }
 
-void FetchEnabledExtensions()
+void GetContextVersion(bool &ctxGLES, int &ctxVersion)
 {
-  GLint numExts = 0;
-  if(GL.glGetIntegerv)
-    GL.glGetIntegerv(eGL_NUM_EXTENSIONS, &numExts);
-
-  RDCEraseEl(HasExt);
-
   if(GL.glGetString)
   {
     const char *version = (const char *)GL.glGetString(eGL_VERSION);
@@ -393,17 +369,30 @@ void FetchEnabledExtensions()
     // check whether we are using OpenGL ES
     // GL_VERSION for OpenGL ES:
     //   "OpenGL ES N.M vendor-specific information"
-    if(strncmp(version, "OpenGL ES", 9) == 0)
+    // for 1.x:
+    //   "OpenGL ES-XX N.M vendor-specific information"
+    if(!strncmp(version, "OpenGL ES-", 10))
     {
-      IsGLES = true;
+      ctxGLES = true;
+
+      // assume 1.0, doesn't matter if it's 1.1
+      ctxVersion = 10;
+    }
+    else if(!strncmp(version, "OpenGL ES", 9))
+    {
+      ctxGLES = true;
 
       int mj = int(version[10] - '0');
       int mn = int(version[12] - '0');
-      GLCoreVersion = RDCMAX(GLCoreVersion, mj * 10 + mn);
+      ctxVersion = mj * 10 + mn;
     }
     else
     {
-      IsGLES = false;
+      ctxGLES = false;
+
+      int mj = int(version[0] - '0');
+      int mn = int(version[1] - '0');
+      ctxVersion = mj * 10 + mn;
     }
   }
 
@@ -413,14 +402,33 @@ void FetchEnabledExtensions()
     GL.glGetIntegerv(eGL_MAJOR_VERSION, &mj);
     GL.glGetIntegerv(eGL_MINOR_VERSION, &mn);
 
-    GLCoreVersion = RDCMAX(GLCoreVersion, mj * 10 + mn);
+    if(mj > 0)
+      ctxVersion = mj * 10 + mn;
   }
+}
+
+void FetchEnabledExtensions()
+{
+  RDCEraseEl(HasExt);
+
+  int ctxVersion = 0;
+  bool ctxGLES = false;
+  GetContextVersion(ctxGLES, ctxVersion);
+
+  GLCoreVersion = RDCMAX(GLCoreVersion, ctxVersion);
+  IsGLES = ctxGLES;
 
   RDCLOG("Checking enabled extensions, running as %s %d.%d", IsGLES ? "OpenGL ES" : "OpenGL",
-         (GLCoreVersion / 10), (GLCoreVersion % 10));
+         (ctxVersion / 10), (ctxVersion % 10));
 
-  if(GL.glGetStringi)
+  // only use glGetStringi on 3.0 contexts and above (ES and GL), even if we have the function
+  // pointer
+  if(GL.glGetStringi && ctxVersion >= 30)
   {
+    GLint numExts = 0;
+    if(GL.glGetIntegerv)
+      GL.glGetIntegerv(eGL_NUM_EXTENSIONS, &numExts);
+
     for(int i = 0; i < numExts; i++)
     {
       const char *ext = (const char *)GL.glGetStringi(eGL_EXTENSIONS, (GLuint)i);
@@ -428,14 +436,14 @@ void FetchEnabledExtensions()
       CheckExtFromString(ext);
     }
   }
-  else if(GL.glGetString && IsGLES && GLCoreVersion < 30)
+  else if(GL.glGetString)
   {
-    std::string extstr = (const char *)GL.glGetString(eGL_EXTENSIONS);
+    rdcstr extstr = (const char *)GL.glGetString(eGL_EXTENSIONS);
 
-    std::vector<std::string> extlist;
+    rdcarray<rdcstr> extlist;
     split(extstr, extlist, ' ');
 
-    for(const std::string &e : extlist)
+    for(const rdcstr &e : extlist)
       CheckExtFromString(e.c_str());
   }
 
@@ -448,17 +456,25 @@ void FetchEnabledExtensions()
 
     HasExt[ARB_program_interface_query] = false;
   }
+
+  if(!IsGLES && GLCoreVersion < 42 && HasExt[ARB_compute_shader])
+  {
+    RDCERR("GL implementation has ARB_compute_shader but is not at least 4.2. Disabling compute.");
+    HasExt[ARB_compute_shader] = false;
+  }
 }
 
 void DoVendorChecks(GLPlatform &platform, GLWindowingData context)
 {
   const char *vendor = "";
+  const char *renderer = "";
+  const char *version = "";
 
   if(GL.glGetString)
   {
     vendor = (const char *)GL.glGetString(eGL_VENDOR);
-    const char *renderer = (const char *)GL.glGetString(eGL_RENDERER);
-    const char *version = (const char *)GL.glGetString(eGL_VERSION);
+    renderer = (const char *)GL.glGetString(eGL_RENDERER);
+    version = (const char *)GL.glGetString(eGL_VERSION);
 
     RDCLOG("Vendor checks for %u (%s / %s / %s)", GLCoreVersion, vendor, renderer, version);
   }
@@ -721,7 +737,7 @@ void DoVendorChecks(GLPlatform &platform, GLWindowingData context)
   }
 
   // don't have a test for this, just have to enable it all the time, for now.
-  VendorCheck[VendorCheck_NV_avoid_D32S8_copy] = true;
+  VendorCheck[VendorCheck_NV_avoid_D32S8_copy] = false;
 
   // On 32-bit calling this function could actually lead to crashes (issues with
   // esp being saved across the call), so since the work-around is low-cost of just
@@ -743,6 +759,40 @@ void DoVendorChecks(GLPlatform &platform, GLWindowingData context)
   // avoiding use of the DSA function and always doing our emulated version.
   VendorCheck[VendorCheck_AMD_vertex_array_elem_buffer_query] = true;
 
+  // Qualcomm's implementation of glCopyImageSubData is buggy on some drivers and can cause GPU
+  // crashes or corrupted data. We force the initial state copies to happen via our emulation which
+  // uses framebuffer blits.
+  if(strstr(vendor, "Qualcomm") || strstr(vendor, "Adreno") || strstr(renderer, "Qualcomm") ||
+     strstr(renderer, "Adreno"))
+  {
+    bool broken = true;
+
+    // the bug should be fixed in version 325 and above
+    const char *qualcommver = strstr(version, "V@");
+    uint32_t ver = 0;
+
+    if(qualcommver)
+    {
+      qualcommver += 2;
+
+      while(*qualcommver >= '0' && *qualcommver <= '9')
+      {
+        ver *= 10;
+        ver += int((*qualcommver) - '0');
+        qualcommver++;
+      }
+
+      if(ver >= 325)
+        broken = false;
+    }
+
+    if(broken)
+    {
+      RDCWARN("Detected Qualcomm driver version %u, Using hack to avoid glCopyImageSubData", ver);
+      VendorCheck[VendorCheck_Qualcomm_avoid_glCopyImageSubData] = true;
+    }
+  }
+
   if(IsGLES)
   {
     // Check whether reading from the depth, stencil and depth-stencil buffers using glReadPixels is
@@ -761,7 +811,7 @@ void DoVendorChecks(GLPlatform &platform, GLWindowingData context)
   }
 }
 
-GLMarkerRegion::GLMarkerRegion(const std::string &marker, GLenum source, GLuint id)
+GLMarkerRegion::GLMarkerRegion(const rdcstr &marker, GLenum source, GLuint id)
 {
   Begin(marker, source, id);
 }
@@ -771,7 +821,7 @@ GLMarkerRegion::~GLMarkerRegion()
   End();
 }
 
-void GLMarkerRegion::Begin(const std::string &marker, GLenum source, GLuint id)
+void GLMarkerRegion::Begin(const rdcstr &marker, GLenum source, GLuint id)
 {
   if(!HasExt[KHR_debug] || !GL.glPushDebugGroup)
     return;
@@ -779,7 +829,7 @@ void GLMarkerRegion::Begin(const std::string &marker, GLenum source, GLuint id)
   GL.glPushDebugGroup(source, id, -1, marker.c_str());
 }
 
-void GLMarkerRegion::Set(const std::string &marker, GLenum source, GLuint id, GLenum severity)
+void GLMarkerRegion::Set(const rdcstr &marker, GLenum source, GLuint id, GLenum severity)
 {
   if(!HasExt[KHR_debug] || !GL.glDebugMessageInsert)
     return;
@@ -895,7 +945,7 @@ void GLPushPopState::Push(bool modern)
   GL.glGetIntegerv(eGL_ARRAY_BUFFER_BINDING, (GLint *)&arraybuf);
 
   // we get the current program but only try to restore it if it's non-0
-  prog = 0;
+  prog = 0xDEADBEEF;
   if(modern)
     GL.glGetIntegerv(eGL_CURRENT_PROGRAM, (GLint *)&prog);
 
@@ -904,7 +954,7 @@ void GLPushPopState::Push(bool modern)
 
   // since we will use the fixed function pipeline, also need to check for program pipeline
   // bindings (if we weren't, our program would override)
-  pipe = 0;
+  pipe = 0xDEADBEEF;
   if(modern && HasExt[ARB_separate_shader_objects])
     GL.glGetIntegerv(eGL_PROGRAM_PIPELINE_BINDING, (GLint *)&pipe);
 
@@ -1056,21 +1106,27 @@ void GLPushPopState::Pop(bool modern)
   if(modern)
   {
     for(size_t i = 0; i < ARRAY_COUNT(idxubo); i++)
-      GL.glBindBufferRange(eGL_UNIFORM_BUFFER, (GLuint)i, idxubo[i].buf, (GLintptr)idxubo[i].offs,
-                           (GLsizeiptr)idxubo[i].size);
+    {
+      if(idxubo[i].offs == 0 && idxubo[i].size == 0)
+        GL.glBindBufferBase(eGL_UNIFORM_BUFFER, (GLuint)i, idxubo[i].buf);
+      else
+        GL.glBindBufferRange(eGL_UNIFORM_BUFFER, (GLuint)i, idxubo[i].buf, (GLintptr)idxubo[i].offs,
+                             (GLsizeiptr)idxubo[i].size);
+    }
 
     GL.glBindBuffer(eGL_UNIFORM_BUFFER, ubo);
 
     GL.glUseProgram(prog);
+    GL.glBindProgramPipeline(pipe);
 
     GL.glBindVertexArray(VAO);
   }
   else
   {
     // only restore these if there was a setting and the function pointer exists
-    if(GL.glUseProgram && prog != 0)
+    if(GL.glUseProgram && prog != 0xDEADBEEF)
       GL.glUseProgram(prog);
-    if(GL.glBindProgramPipeline && pipe != 0)
+    if(GL.glBindProgramPipeline && pipe != 0xDEADBEEF)
       GL.glBindProgramPipeline(pipe);
   }
 }
@@ -1114,6 +1170,14 @@ bool GLInitParams::IsSupportedVersion(uint64_t ver)
   if(ver == 0x1E)
     return true;
 
+  // 0x1F -> 0x20 - added renderer and version in GLInitParams
+  if(ver == 0x1F)
+    return true;
+
+  // 0x20 -> 0x21 - added multisampled texture initial data
+  if(ver == 0x20)
+    return true;
+
   return false;
 }
 
@@ -1129,6 +1193,11 @@ void DoSerialise(SerialiserType &ser, GLInitParams &el)
   SERIALISE_MEMBER(height);
   if(ser.VersionAtLeast(0x1D))
     SERIALISE_MEMBER(isYFlipped);
+  if(ser.VersionAtLeast(0x20))
+  {
+    SERIALISE_MEMBER(renderer);
+    SERIALISE_MEMBER(version);
+  }
 }
 
 INSTANTIATE_SERIALISE_TYPE(GLInitParams);
@@ -1207,6 +1276,55 @@ GLenum BufferEnum(size_t idx)
   return eGL_NONE;
 }
 
+size_t TextureIdx(GLenum buf)
+{
+  switch(buf)
+  {
+    case eGL_TEXTURE_1D: return 0;
+    case eGL_TEXTURE_1D_ARRAY: return 1;
+    case eGL_TEXTURE_2D: return 2;
+    case eGL_TEXTURE_2D_ARRAY: return 3;
+    case eGL_TEXTURE_2D_MULTISAMPLE: return 4;
+    case eGL_TEXTURE_2D_MULTISAMPLE_ARRAY: return 5;
+    case eGL_TEXTURE_RECTANGLE: return 6;
+    case eGL_TEXTURE_3D: return 7;
+    case eGL_TEXTURE_CUBE_MAP:
+    case eGL_TEXTURE_CUBE_MAP_POSITIVE_X:
+    case eGL_TEXTURE_CUBE_MAP_NEGATIVE_X:
+    case eGL_TEXTURE_CUBE_MAP_POSITIVE_Y:
+    case eGL_TEXTURE_CUBE_MAP_NEGATIVE_Y:
+    case eGL_TEXTURE_CUBE_MAP_POSITIVE_Z:
+    case eGL_TEXTURE_CUBE_MAP_NEGATIVE_Z: return 8;
+    case eGL_TEXTURE_CUBE_MAP_ARRAY: return 9;
+    case eGL_TEXTURE_BUFFER: return 10;
+    default: RDCERR("Unexpected enum as texture target: %s", ToStr(buf).c_str());
+  }
+
+  return 0;
+}
+
+GLenum TextureEnum(size_t idx)
+{
+  GLenum enums[] = {
+      eGL_TEXTURE_1D,
+      eGL_TEXTURE_1D_ARRAY,
+      eGL_TEXTURE_2D,
+      eGL_TEXTURE_2D_ARRAY,
+      eGL_TEXTURE_2D_MULTISAMPLE,
+      eGL_TEXTURE_2D_MULTISAMPLE_ARRAY,
+      eGL_TEXTURE_RECTANGLE,
+      eGL_TEXTURE_3D,
+      eGL_TEXTURE_CUBE_MAP,
+      eGL_TEXTURE_CUBE_MAP_ARRAY,
+      eGL_TEXTURE_BUFFER,
+  };
+
+  if(idx < ARRAY_COUNT(enums))
+    return enums[idx];
+
+  return eGL_NONE;
+}
+
 size_t QueryIdx(GLenum query)
 {
   size_t idx = 0;
@@ -1279,7 +1397,8 @@ size_t GLTypeSize(GLenum type)
     case eGL_HALF_FLOAT: return 2;
     case eGL_UNSIGNED_INT:
     case eGL_INT:
-    case eGL_FLOAT: return 4;
+    case eGL_FLOAT:
+    case eGL_UNSIGNED_INT_8_8_8_8_REV: return 4;
     case eGL_DOUBLE: return 8;
     default: RDCWARN("Unhandled element type %s", ToStr(type).c_str());
   }
@@ -1448,13 +1567,13 @@ BufferCategory MakeBufferCategory(GLenum bufferTarget)
 {
   switch(bufferTarget)
   {
-    case eGL_ARRAY_BUFFER: return BufferCategory::Vertex; break;
-    case eGL_ELEMENT_ARRAY_BUFFER: return BufferCategory::Index; break;
-    case eGL_UNIFORM_BUFFER: return BufferCategory::Constants; break;
-    case eGL_SHADER_STORAGE_BUFFER: return BufferCategory::ReadWrite; break;
+    case eGL_ARRAY_BUFFER: return BufferCategory::Vertex;
+    case eGL_ELEMENT_ARRAY_BUFFER: return BufferCategory::Index;
+    case eGL_UNIFORM_BUFFER: return BufferCategory::Constants;
+    case eGL_SHADER_STORAGE_BUFFER: return BufferCategory::ReadWrite;
     case eGL_DRAW_INDIRECT_BUFFER:
     case eGL_DISPATCH_INDIRECT_BUFFER:
-    case eGL_PARAMETER_BUFFER_ARB: return BufferCategory::Indirect; break;
+    case eGL_PARAMETER_BUFFER_ARB: return BufferCategory::Indirect;
     default: break;
   }
   return BufferCategory::NoFlags;
@@ -1630,70 +1749,6 @@ BlendOperation MakeBlendOp(GLenum op)
   return BlendOperation::Add;
 }
 
-const char *BlendString(GLenum blendenum)
-{
-  switch(blendenum)
-  {
-    case eGL_FUNC_ADD: return "ADD";
-    case eGL_FUNC_SUBTRACT: return "SUBTRACT";
-    case eGL_FUNC_REVERSE_SUBTRACT: return "INV_SUBTRACT";
-    case eGL_MIN: return "MIN";
-    case eGL_MAX: return "MAX";
-    case GL_ZERO: return "ZERO";
-    case GL_ONE: return "ONE";
-    case eGL_SRC_COLOR: return "SRC_COLOR";
-    case eGL_ONE_MINUS_SRC_COLOR: return "INV_SRC_COLOR";
-    case eGL_DST_COLOR: return "DST_COLOR";
-    case eGL_ONE_MINUS_DST_COLOR: return "INV_DST_COLOR";
-    case eGL_SRC_ALPHA: return "SRC_ALPHA";
-    case eGL_ONE_MINUS_SRC_ALPHA: return "INV_SRC_ALPHA";
-    case eGL_DST_ALPHA: return "DST_ALPHA";
-    case eGL_ONE_MINUS_DST_ALPHA: return "INV_DST_ALPHA";
-    case eGL_CONSTANT_COLOR: return "CONST_COLOR";
-    case eGL_ONE_MINUS_CONSTANT_COLOR: return "INV_CONST_COLOR";
-    case eGL_CONSTANT_ALPHA: return "CONST_ALPHA";
-    case eGL_ONE_MINUS_CONSTANT_ALPHA: return "INV_CONST_ALPHA";
-    case eGL_SRC_ALPHA_SATURATE: return "SRC_ALPHA_SAT";
-    case eGL_SRC1_COLOR: return "SRC1_COL";
-    case eGL_ONE_MINUS_SRC1_COLOR: return "INV_SRC1_COL";
-    case eGL_SRC1_ALPHA: return "SRC1_ALPHA";
-    case eGL_ONE_MINUS_SRC1_ALPHA: return "INV_SRC1_ALPHA";
-    default: break;
-  }
-
-  static string unknown = ToStr(blendenum).substr(3);    // 3 = strlen("GL_");
-
-  RDCERR("Unknown blend enum: %s", unknown.c_str());
-
-  return unknown.c_str();
-}
-
-const char *SamplerString(GLenum smpenum)
-{
-  switch(smpenum)
-  {
-    case eGL_NONE: return "NONE";
-    case eGL_NEAREST: return "NEAREST";
-    case eGL_LINEAR: return "LINEAR";
-    case eGL_NEAREST_MIPMAP_NEAREST: return "NEAREST_MIP_NEAREST";
-    case eGL_LINEAR_MIPMAP_NEAREST: return "LINEAR_MIP_NEAREST";
-    case eGL_NEAREST_MIPMAP_LINEAR: return "NEAREST_MIP_LINEAR";
-    case eGL_LINEAR_MIPMAP_LINEAR: return "LINEAR_MIP_LINEAR";
-    case eGL_CLAMP_TO_EDGE: return "CLAMP_EDGE";
-    case eGL_MIRRORED_REPEAT: return "MIRR_REPEAT";
-    case eGL_REPEAT: return "REPEAT";
-    case eGL_MIRROR_CLAMP_TO_EDGE: return "MIRR_CLAMP_EDGE";
-    case eGL_CLAMP_TO_BORDER: return "CLAMP_BORDER";
-    default: break;
-  }
-
-  static string unknown = ToStr(smpenum).substr(3);    // 3 = strlen("GL_");
-
-  RDCERR("Unknown blend enum: %s", unknown.c_str());
-
-  return unknown.c_str();
-}
-
 ResourceFormat MakeResourceFormat(GLenum target, GLenum fmt)
 {
   ResourceFormat ret;
@@ -1707,22 +1762,78 @@ ResourceFormat MakeResourceFormat(GLenum target, GLenum fmt)
   }
 
   // special handling for formats that don't query neatly
-  if(fmt == eGL_LUMINANCE8_EXT || fmt == eGL_INTENSITY8_EXT || fmt == eGL_ALPHA8_EXT)
+  if(fmt == eGL_LUMINANCE8_EXT || fmt == eGL_INTENSITY8_EXT || fmt == eGL_LUMINANCE)
   {
     ret.compByteWidth = 1;
     ret.compCount = 1;
     ret.compType = CompType::UNorm;
     return ret;
   }
+  else if(fmt == eGL_ALPHA || fmt == eGL_ALPHA8_EXT)
+  {
+    ret.compByteWidth = 1;
+    ret.compCount = 1;
+    ret.compType = CompType::UNorm;
+    ret.type = ResourceFormatType::A8;
+    return ret;
+  }
+  else if(fmt == eGL_LUMINANCE8_ALPHA8_EXT || fmt == eGL_LUMINANCE_ALPHA)
+  {
+    ret.compByteWidth = 1;
+    ret.compCount = 2;
+    ret.compType = CompType::UNorm;
+    return ret;
+  }
+  else if(fmt == eGL_RGBA2)
+  {
+    // pretend it's RGBA8 and hope for the best
+    ret.compByteWidth = 1;
+    ret.compCount = 4;
+    ret.compType = CompType::UNorm;
+    return ret;
+  }
 
   if(IsCompressedFormat(fmt))
   {
     switch(fmt)
     {
+      // BC1
       case eGL_COMPRESSED_RGB_S3TC_DXT1_EXT:
       case eGL_COMPRESSED_SRGB_S3TC_DXT1_EXT: ret.compCount = 3; break;
       case eGL_COMPRESSED_RGBA_S3TC_DXT1_EXT:
-      case eGL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT: ret.compCount = 4; break;
+      case eGL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT:
+        ret.compCount = 4;
+        break;
+
+      // BC2
+      case eGL_COMPRESSED_RGBA_S3TC_DXT3_EXT:
+      case eGL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT:
+        ret.compCount = 4;
+        break;
+
+      // BC3
+      case eGL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
+      case eGL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT:
+        ret.compCount = 4;
+        break;
+      // BC4
+      case eGL_COMPRESSED_RED_RGTC1:
+      case eGL_COMPRESSED_SIGNED_RED_RGTC1:
+        ret.compCount = 1;
+        break;
+      // BC5
+      case eGL_COMPRESSED_RG_RGTC2:
+      case eGL_COMPRESSED_SIGNED_RG_RGTC2:
+        ret.compCount = 2;
+        break;
+      // BC6
+      case eGL_COMPRESSED_RGB_BPTC_SIGNED_FLOAT_ARB:
+      case eGL_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT_ARB:
+        ret.compCount = 3;
+        break;
+      // BC7
+      case eGL_COMPRESSED_RGBA_BPTC_UNORM_ARB:
+      case eGL_COMPRESSED_SRGB_ALPHA_BPTC_UNORM_ARB: ret.compCount = 4; break;
 
       case eGL_COMPRESSED_RGBA8_ETC2_EAC:
       case eGL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC: ret.compCount = 4; break;
@@ -1742,6 +1853,35 @@ ResourceFormat MakeResourceFormat(GLenum target, GLenum fmt)
       case eGL_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2:
       case eGL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2: ret.compCount = 4; break;
 
+      case eGL_COMPRESSED_RGBA_ASTC_4x4_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_5x4_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_5x5_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_6x5_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_6x6_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_8x5_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_8x6_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_8x8_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_10x5_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_10x6_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_10x8_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_10x10_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_12x10_KHR:
+      case eGL_COMPRESSED_RGBA_ASTC_12x12_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_5x4_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_5x5_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_6x5_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_6x6_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x5_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x6_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_8x8_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x5_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x6_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x8_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_10x10_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x10_KHR:
+      case eGL_COMPRESSED_SRGB8_ALPHA8_ASTC_12x12_KHR: ret.compCount = 4; break;
+
       default: break;
     }
 
@@ -1874,15 +2014,36 @@ ResourceFormat MakeResourceFormat(GLenum target, GLenum fmt)
   // handle certain non compressed but special formats
   switch(fmt)
   {
-    case eGL_R11F_G11F_B10F: ret.type = ResourceFormatType::R11G11B10; break;
-    case eGL_RGB565: ret.type = ResourceFormatType::R5G6B5; break;
-    case eGL_RGB5_A1: ret.type = ResourceFormatType::R5G5B5A1; break;
-    case eGL_RGB9_E5: ret.type = ResourceFormatType::R9G9B9E5; break;
-    case eGL_RGBA4: ret.type = ResourceFormatType::R4G4B4A4; break;
+    case eGL_R11F_G11F_B10F:
+      ret.type = ResourceFormatType::R11G11B10;
+      ret.compType = CompType::Float;
+      ret.compCount = 3;
+      break;
+    case eGL_RGB565:
+      ret.type = ResourceFormatType::R5G6B5;
+      ret.compType = CompType::UNorm;
+      ret.compCount = 3;
+      break;
+    case eGL_RGB5_A1:
+      ret.type = ResourceFormatType::R5G5B5A1;
+      ret.compType = CompType::UNorm;
+      ret.compCount = 4;
+      break;
+    case eGL_RGB9_E5:
+      ret.type = ResourceFormatType::R9G9B9E5;
+      ret.compType = CompType::Float;
+      ret.compCount = 3;
+      break;
+    case eGL_RGBA4:
+      ret.type = ResourceFormatType::R4G4B4A4;
+      ret.compType = CompType::UNorm;
+      ret.compCount = 4;
+      break;
     case eGL_RGB10_A2:
     case eGL_RGB10_A2UI:
       ret.type = ResourceFormatType::R10G10B10A2;
       ret.compType = fmt == eGL_RGB10_A2 ? CompType::UNorm : CompType::UInt;
+      ret.compCount = 4;
       break;
     default: break;
   }
@@ -1967,7 +2128,7 @@ ResourceFormat MakeResourceFormat(GLenum target, GLenum fmt)
     }
 
     GL.glGetInternalformativ(target, fmt, eGL_COLOR_ENCODING, sizeof(GLint), &data[0]);
-    if(edata[0] == eGL_SRGB)
+    if(edata[0] == eGL_SRGB || fmt == eGL_SR8_EXT || fmt == eGL_SRG8_EXT)
       ret.compType = CompType::UNormSRGB;
   }
   else if(isdepth == GL_TRUE || isstencil == GL_TRUE)
@@ -2097,9 +2258,15 @@ GLenum MakeGLFormat(ResourceFormat fmt)
       case ResourceFormatType::R4G4B4A4: ret = eGL_RGBA4; break;
       case ResourceFormatType::D24S8: ret = eGL_DEPTH24_STENCIL8; break;
       case ResourceFormatType::D32S8: ret = eGL_DEPTH32F_STENCIL8; break;
-      case ResourceFormatType::ASTC: RDCERR("ASTC can't be decoded unambiguously"); break;
-      case ResourceFormatType::PVRTC: RDCERR("PVRTC can't be decoded unambiguously"); break;
+      case ResourceFormatType::D16S8: return eGL_NONE;
+      case ResourceFormatType::ASTC:
+        RDCWARN("ASTC can't be decoded unambiguously");
+        return eGL_NONE;
+      case ResourceFormatType::PVRTC:
+        RDCWARN("PVRTC can't be decoded unambiguously");
+        return eGL_NONE;
       case ResourceFormatType::S8: ret = eGL_STENCIL_INDEX8; break;
+      case ResourceFormatType::A8: ret = eGL_ALPHA8_EXT; break;
       case ResourceFormatType::Undefined: return eGL_NONE;
       default: RDCERR("Unsupported resource format type %u", fmt.type); break;
     }
@@ -2120,7 +2287,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
         ret = eGL_RGBA32F;
       else if(fmt.compType == CompType::SInt)
         ret = eGL_RGBA32I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_RGBA32UI;
       else
         RDCERR("Unrecognised component type");
@@ -2131,7 +2298,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
         ret = eGL_RGBA16F;
       else if(fmt.compType == CompType::SInt)
         ret = eGL_RGBA16I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_RGBA16UI;
       else if(fmt.compType == CompType::SNorm)
         ret = eGL_RGBA16_SNORM;
@@ -2144,7 +2311,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
     {
       if(fmt.compType == CompType::SInt)
         ret = eGL_RGBA8I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_RGBA8UI;
       else if(fmt.compType == CompType::SNorm)
         ret = eGL_RGBA8_SNORM;
@@ -2170,7 +2337,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
         ret = eGL_RGB32F;
       else if(fmt.compType == CompType::SInt)
         ret = eGL_RGB32I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_RGB32UI;
       else
         RDCERR("Unrecognised component type");
@@ -2181,7 +2348,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
         ret = eGL_RGB16F;
       else if(fmt.compType == CompType::SInt)
         ret = eGL_RGB16I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_RGB16UI;
       else if(fmt.compType == CompType::SNorm)
         ret = eGL_RGB16_SNORM;
@@ -2194,7 +2361,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
     {
       if(fmt.compType == CompType::SInt)
         ret = eGL_RGB8I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_RGB8UI;
       else if(fmt.compType == CompType::SNorm)
         ret = eGL_RGB8_SNORM;
@@ -2210,13 +2377,17 @@ GLenum MakeGLFormat(ResourceFormat fmt)
   }
   else if(fmt.compCount == 2)
   {
-    if(fmt.compByteWidth == 4)
+    if(fmt.SRGBCorrected())
+    {
+      ret = eGL_SRG8_EXT;
+    }
+    else if(fmt.compByteWidth == 4)
     {
       if(fmt.compType == CompType::Float)
         ret = eGL_RG32F;
       else if(fmt.compType == CompType::SInt)
         ret = eGL_RG32I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_RG32UI;
       else
         RDCERR("Unrecognised component type");
@@ -2227,7 +2398,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
         ret = eGL_RG16F;
       else if(fmt.compType == CompType::SInt)
         ret = eGL_RG16I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_RG16UI;
       else if(fmt.compType == CompType::SNorm)
         ret = eGL_RG16_SNORM;
@@ -2240,7 +2411,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
     {
       if(fmt.compType == CompType::SInt)
         ret = eGL_RG8I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_RG8UI;
       else if(fmt.compType == CompType::SNorm)
         ret = eGL_RG8_SNORM;
@@ -2256,13 +2427,17 @@ GLenum MakeGLFormat(ResourceFormat fmt)
   }
   else if(fmt.compCount == 1)
   {
-    if(fmt.compByteWidth == 4)
+    if(fmt.SRGBCorrected())
+    {
+      ret = eGL_SR8_EXT;
+    }
+    else if(fmt.compByteWidth == 4)
     {
       if(fmt.compType == CompType::Float)
         ret = eGL_R32F;
       else if(fmt.compType == CompType::SInt)
         ret = eGL_R32I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_R32UI;
       else if(fmt.compType == CompType::Depth)
         ret = eGL_DEPTH_COMPONENT32F;
@@ -2279,7 +2454,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
         ret = eGL_R16F;
       else if(fmt.compType == CompType::SInt)
         ret = eGL_R16I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_R16UI;
       else if(fmt.compType == CompType::SNorm)
         ret = eGL_R16_SNORM;
@@ -2294,7 +2469,7 @@ GLenum MakeGLFormat(ResourceFormat fmt)
     {
       if(fmt.compType == CompType::SInt)
         ret = eGL_R8I;
-      else if(fmt.compType == CompType::UInt)
+      else if(fmt.compType == CompType::UInt || fmt.compType == CompType::Typeless)
         ret = eGL_R8UI;
       else if(fmt.compType == CompType::SNorm)
         ret = eGL_R8_SNORM;
@@ -2410,10 +2585,12 @@ TEST_CASE("GL formats", "[format][gl]")
       eGL_R8_SNORM,
       eGL_R8UI,
       eGL_R8I,
+      eGL_SR8_EXT,
       eGL_RG8,
       eGL_RG8_SNORM,
       eGL_RG8UI,
       eGL_RG8I,
+      eGL_SRG8_EXT,
       eGL_RGB8,
       eGL_RGB8_SNORM,
       eGL_RGB8UI,
@@ -2586,7 +2763,7 @@ TEST_CASE("GL formats", "[format][gl]")
       if(fmt.type != ResourceFormatType::Regular)
         continue;
 
-      INFO("Format is " << f);
+      INFO("Format is " << ToStr(f));
 
       uint32_t size = fmt.compCount * fmt.compByteWidth * 123 * 456;
 
