@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -48,21 +48,19 @@ DECLARE_REFLECTION_ENUM(RDCGLenum);
 // appropriate libEGL.dll into plugins/gles/ in your RenderDoc folder.
 #define RENDERDOC_SUPPORT_GL
 #define RENDERDOC_SUPPORT_GLES
-
-// checks a runtime opt-out option to disallow hooking EGL on windows. This means that the
-// underlying GL or D3D calls will be captured instead.
-bool ShouldHookEGL();
+#define RENDERDOC_SUPPORT_EGL
 
 #else
 
 // other cases are defined by build-time configuration
 
-// for consistency, we always enable this on other platforms - if GLES support is disabled at
-// compile time then it does nothing.
-#define RENDERDOC_HOOK_EGL OPTION_ON
-
 #endif
 
+// checks a runtime opt-out option to disallow hooking EGL. On Windows with no EGL support then the
+// EGL library is an emulator, so this means we'll capture the underlying calls it makes to the
+// native API. On Android this is useful if we detect that GLES layering is supported.
+bool ShouldHookEGL();
+
 #define GL_APICALL
 
 // official headers
@@ -129,7 +127,7 @@ struct GLWindowingData
 
 #endif
 
-#if defined(RENDERDOC_SUPPORT_GLES)
+#if defined(RENDERDOC_SUPPORT_EGL)
 
 // force include the elgplatform.h, as we want to use
 // our own because the system one could be a bit older and
@@ -147,7 +145,7 @@ struct GLWindowingData
     dpy = NULL;
     ctx = NULL;
     wnd = (GLWindowPtr)NULL;
-    egl_wnd = (GLESWindowPtr)NULL;
+    egl_wnd = (EGLSurface)NULL;
     cfg = NULL;
   }
 
@@ -163,35 +161,30 @@ struct GLWindowingData
   typedef void *GLConfigPtr;
 #endif
 
-#if defined(RENDERDOC_SUPPORT_GLES)
-  typedef EGLDisplay GLESDisplayPtr;
-  typedef EGLContext GLESContextPtr;
-  typedef EGLSurface GLESWindowPtr;
-  typedef EGLConfig GLESConfigPtr;
-#else
-  typedef void *GLESDisplayPtr;
-  typedef void *GLESContextPtr;
-  typedef void *GLESWindowPtr;
-  typedef void *GLESConfigPtr;
+#if !defined(RENDERDOC_SUPPORT_EGL)
+  typedef void *EGLDisplay;
+  typedef void *EGLContext;
+  typedef void *EGLSurface;
+  typedef void *EGLConfig;
 #endif
 
   union
   {
     GLDisplayPtr dpy;
-    GLESDisplayPtr egl_dpy;
+    EGLDisplay egl_dpy;
   };
   union
   {
     GLContextPtr ctx;
-    GLESContextPtr egl_ctx;
+    EGLContext egl_ctx;
   };
   union
   {
     GLConfigPtr cfg;
-    GLESConfigPtr egl_cfg;
+    EGLConfig egl_cfg;
   };
   GLWindowPtr wnd;
-  GLESWindowPtr egl_wnd;
+  EGLSurface egl_wnd;
 };
 
 #elif ENABLED(RDOC_APPLE)
@@ -267,8 +260,6 @@ struct GLWindowingData
 #error "Unknown platform"
 #endif
 
-#include "api/replay/renderdoc_replay.h"
-
 struct GLPlatform
 {
   // simple wrapper for OS functions to make/delete a context
@@ -284,12 +275,13 @@ struct GLPlatform
                                            GLWindowingData share_context) = 0;
 
   // for 'backwards compatible' overlay rendering
-  virtual void DrawQuads(float width, float height, const std::vector<Vec4f> &vertices) = 0;
+  virtual void DrawQuads(float width, float height, const rdcarray<Vec4f> &vertices) = 0;
 
   // for initialisation at replay time
+  virtual bool CanCreateGLContext() = 0;
   virtual bool CanCreateGLESContext() = 0;
   virtual bool PopulateForReplay() = 0;
-  virtual ReplayStatus InitialiseAPI(GLWindowingData &replayContext, RDCDriver api) = 0;
+  virtual ReplayStatus InitialiseAPI(GLWindowingData &replayContext, RDCDriver api, bool debug) = 0;
   virtual void *GetReplayFunction(const char *funcname) = 0;
 };
 
@@ -308,12 +300,13 @@ class GLDummyPlatform : public GLPlatform
   {
     return GLWindowingData();
   }
-  virtual void DrawQuads(float width, float height, const std::vector<Vec4f> &vertices) {}
+  virtual void DrawQuads(float width, float height, const rdcarray<Vec4f> &vertices) {}
   virtual void *GetReplayFunction(const char *funcname) { return NULL; }
   // for initialisation at replay time
+  virtual bool CanCreateGLContext() { return true; }
   virtual bool CanCreateGLESContext() { return true; }
   virtual bool PopulateForReplay() { return true; }
-  virtual ReplayStatus InitialiseAPI(GLWindowingData &replayContext, RDCDriver api)
+  virtual ReplayStatus InitialiseAPI(GLWindowingData &replayContext, RDCDriver api, bool debug)
   {
     return ReplayStatus::Succeeded;
   }
@@ -325,7 +318,7 @@ struct GLVersion
   int minor;
 };
 
-std::vector<GLVersion> GetReplayVersions(RDCDriver api);
+rdcarray<GLVersion> GetReplayVersions(RDCDriver api);
 
 #if defined(RENDERDOC_SUPPORT_GL)
 
@@ -334,7 +327,7 @@ GLPlatform &GetGLPlatform();
 
 #endif
 
-#if defined(RENDERDOC_SUPPORT_GLES)
+#if defined(RENDERDOC_SUPPORT_EGL)
 
 // using EGL. Different name since it both platform GL and EGL libraries can be available at once
 GLPlatform &GetEGLPlatform();
@@ -388,8 +381,6 @@ typedef BOOL(APIENTRY *PFNWGLDXLOCKOBJECTSNVPROC)(HANDLE hDevice, GLint count, H
 typedef BOOL(APIENTRY *PFNWGLDXUNLOCKOBJECTSNVPROC)(HANDLE hDevice, GLint count, HANDLE *hObjects);
 #endif
 
-#include "api/replay/renderdoc_replay.h"
-
 // bit of a hack, to work around C4127: conditional expression is constant
 // on template parameters
 template <typename T>
@@ -453,14 +444,13 @@ extern Threading::CriticalSection glLock;
 // replay only class for handling marker regions
 struct GLMarkerRegion
 {
-  GLMarkerRegion(const std::string &marker, GLenum source = eGL_DEBUG_SOURCE_APPLICATION,
-                 GLuint id = 0);
+  GLMarkerRegion(const rdcstr &marker, GLenum source = eGL_DEBUG_SOURCE_APPLICATION, GLuint id = 0);
   ~GLMarkerRegion();
 
-  static void Begin(const std::string &marker, GLenum source = eGL_DEBUG_SOURCE_APPLICATION,
+  static void Begin(const rdcstr &marker, GLenum source = eGL_DEBUG_SOURCE_APPLICATION,
                     GLuint id = 0);
-  static void Set(const std::string &marker, GLenum source = eGL_DEBUG_SOURCE_APPLICATION,
-                  GLuint id = 0, GLenum severity = eGL_DEBUG_SEVERITY_NOTIFICATION);
+  static void Set(const rdcstr &marker, GLenum source = eGL_DEBUG_SOURCE_APPLICATION, GLuint id = 0,
+                  GLenum severity = eGL_DEBUG_SEVERITY_NOTIFICATION);
   static void End();
 };
 
@@ -507,7 +497,7 @@ struct GLPushPopState
 };
 
 template <class DispatchTable>
-void DrawQuads(DispatchTable &GL, float width, float height, const std::vector<Vec4f> &vertices)
+void DrawQuads(DispatchTable &GL, float width, float height, const rdcarray<Vec4f> &vertices)
 {
   const GLenum GL_MATRIX_MODE = (GLenum)0x0BA0;
   const GLenum GL_MODELVIEW = (GLenum)0x1700;
@@ -548,6 +538,9 @@ size_t GLTypeSize(GLenum type);
 size_t BufferIdx(GLenum buf);
 GLenum BufferEnum(size_t idx);
 
+size_t TextureIdx(GLenum buf);
+GLenum TextureEnum(size_t idx);
+
 size_t QueryIdx(GLenum query);
 GLenum QueryEnum(size_t idx);
 
@@ -568,14 +561,15 @@ StencilOperation MakeStencilOp(GLenum op);
 LogicOperation MakeLogicOp(GLenum op);
 BlendMultiplier MakeBlendMultiplier(GLenum blend);
 BlendOperation MakeBlendOp(GLenum op);
-const char *BlendString(GLenum blendenum);
-const char *SamplerString(GLenum smpenum);
 
 void ClearGLErrors();
 
 GLuint GetBoundVertexBuffer(GLuint idx);
 GLint GetNumVertexBuffers();
 
+struct ShaderReflection;
+struct ShaderBindpointMapping;
+
 void EvaluateSPIRVBindpointMapping(GLuint curProg, int shadIdx, const ShaderReflection *refl,
                                    ShaderBindpointMapping &mapping);
 
@@ -686,7 +680,9 @@ extern bool IsGLES;
   EXT_TO_CHECK(33, 30, ARB_explicit_attrib_location)             \
   EXT_TO_CHECK(33, 30, ARB_sampler_objects)                      \
   EXT_TO_CHECK(33, 30, ARB_texture_swizzle)                      \
-  EXT_TO_CHECK(33, 30, ARB_occlusion_query2)                     \
+  EXT_TO_CHECK(33, 99, ARB_occlusion_query2)                     \
+  EXT_TO_CHECK(33, 99, ARB_timer_query)                          \
+  EXT_TO_CHECK(33, 30, ARB_shader_bit_encoding)                  \
   EXT_TO_CHECK(40, 32, ARB_draw_buffers_blend)                   \
   EXT_TO_CHECK(40, 31, ARB_draw_indirect)                        \
   EXT_TO_CHECK(40, 32, ARB_gpu_shader5)                          \
@@ -711,6 +707,7 @@ extern bool IsGLES;
   EXT_TO_CHECK(43, 31, ARB_compute_shader)                       \
   EXT_TO_CHECK(43, 32, ARB_copy_image)                           \
   EXT_TO_CHECK(43, 30, ARB_ES3_compatibility)                    \
+  EXT_TO_CHECK(43, 30, ARB_invalidate_subdata)                   \
   EXT_TO_CHECK(43, 99, ARB_internalformat_query2)                \
   EXT_TO_CHECK(43, 31, ARB_program_interface_query)              \
   EXT_TO_CHECK(43, 31, ARB_shader_storage_buffer_object)         \
@@ -729,6 +726,7 @@ extern bool IsGLES;
   EXT_TO_CHECK(46, 99, ARB_polygon_offset_clamp)                 \
   EXT_TO_CHECK(46, 99, ARB_texture_filter_anisotropic)           \
   EXT_TO_CHECK(46, 99, ARB_pipeline_statistics_query)            \
+  EXT_TO_CHECK(46, 99, ARB_gl_spirv)                             \
   EXT_TO_CHECK(99, 99, ARB_indirect_parameters)                  \
   EXT_TO_CHECK(99, 99, ARB_seamless_cubemap_per_texture)         \
   EXT_TO_CHECK(99, 99, EXT_depth_bounds_test)                    \
@@ -757,7 +755,8 @@ extern bool IsGLES;
   EXT_TO_CHECK(99, 99, NV_read_depth_stencil)                    \
   EXT_TO_CHECK(99, 99, EXT_disjoint_timer_query)                 \
   EXT_TO_CHECK(99, 99, EXT_multisampled_render_to_texture)       \
-  EXT_TO_CHECK(99, 99, OVR_multiview)
+  EXT_TO_CHECK(99, 99, OVR_multiview)                            \
+  EXT_TO_CHECK(99, 99, EXT_discard_framebuffer)
 
 // GL extensions equivalents
 // Either promoted extensions from EXT to ARB, or
@@ -785,6 +784,8 @@ extern bool IsGLES;
   EXT_COMP_CHECK(ARB_texture_storage_multisample, OES_texture_storage_multisample_2d_array) \
   EXT_COMP_CHECK(ARB_texture_view, EXT_texture_view)                                        \
   EXT_COMP_CHECK(ARB_texture_view, OES_texture_view)                                        \
+  EXT_COMP_CHECK(ARB_timer_query, EXT_timer_query)                                          \
+  EXT_COMP_CHECK(ARB_timer_query, EXT_disjoint_timer_query)                                 \
   EXT_COMP_CHECK(ARB_viewport_array, NV_viewport_array)                                     \
   EXT_COMP_CHECK(ARB_viewport_array, OES_viewport_array)                                    \
   EXT_COMP_CHECK(ARB_texture_buffer_object, EXT_texture_buffer)                             \
@@ -818,13 +819,24 @@ enum VendorCheckEnum
   VendorCheck_NV_ClearNamedFramebufferfiBugs,
   VendorCheck_AMD_copy_compressed_cubemaps,
   VendorCheck_AMD_vertex_array_elem_buffer_query,
+  VendorCheck_Qualcomm_avoid_glCopyImageSubData,
   VendorCheck_Count,
 };
 extern bool VendorCheck[VendorCheck_Count];
 
+enum class ShaderType;
+
 // fills out the extension supported array and the version-specific checks above
 void DoVendorChecks(GLPlatform &platform, GLWindowingData context);
+void GetContextVersion(bool &ctxGLES, int &ctxVersion);
 void FetchEnabledExtensions();
+void GetGLSLVersions(ShaderType &shaderType, int &glslVersion, int &glslBaseVer, int &glslCSVer);
+
+GLuint CreateShader(GLenum shaderType, const rdcstr &src);
+GLuint CreateSPIRVShader(GLenum shaderType, const rdcstr &src);
+GLuint CreateShaderProgram(const rdcstr &vs, const rdcstr &fs, const rdcstr &gs = "");
+GLuint CreateShaderProgram(GLuint vs, GLuint fs, GLuint gs = 0);
+GLuint CreateCShaderProgram(const rdcstr &cs);
 
 // verify that we got a replay context that we can work with
 bool CheckReplayContext();
@@ -835,14 +847,24 @@ bool ValidateFunctionPointers();
 
 struct ShaderReflection;
 
-void CopyProgramUniforms(GLuint progSrc, GLuint progDst);
+struct PerStageReflections
+{
+  const ShaderReflection *refls[6] = {};
+  const ShaderBindpointMapping *mappings[6] = {};
+};
+
+void CopyProgramUniforms(const PerStageReflections &srcStages, GLuint progSrc,
+                         const PerStageReflections &dstStages, GLuint progDst,
+                         std::map<GLint, GLint> *locTranslate = NULL);
 template <typename SerialiserType>
-void SerialiseProgramUniforms(SerialiserType &ser, CaptureState state, GLuint prog,
-                              map<GLint, GLint> *locTranslate);
-void CopyProgramAttribBindings(GLuint progsrc, GLuint progdst, ShaderReflection *refl);
-void CopyProgramFragDataBindings(GLuint progsrc, GLuint progdst, ShaderReflection *refl);
+void SerialiseProgramUniforms(SerialiserType &ser, CaptureState state,
+                              const PerStageReflections &stages, GLuint prog,
+                              std::map<GLint, GLint> *locTranslate);
+bool CopyProgramAttribBindings(GLuint progsrc, GLuint progdst, ShaderReflection *refl);
+bool CopyProgramFragDataBindings(GLuint progsrc, GLuint progdst, ShaderReflection *refl);
 template <typename SerialiserType>
-void SerialiseProgramBindings(SerialiserType &ser, CaptureState state, GLuint prog);
+bool SerialiseProgramBindings(SerialiserType &ser, CaptureState state,
+                              const PerStageReflections &stages, GLuint prog);
 
 struct DrawElementsIndirectCommand
 {
@@ -2148,6 +2170,16 @@ enum class GLChunk : uint32_t
   glGetPerfQueryInfoINTEL,
 
   glBlendEquationARB,
+  glPrimitiveBoundingBoxARB,
+
+  SwapBuffers,
+  wglSwapBuffers,
+  glXSwapBuffers,
+  CGLFlushDrawable,
+  eglSwapBuffers,
+  eglPostSubBufferNV,
+  eglSwapBuffersWithDamageEXT,
+  eglSwapBuffersWithDamageKHR,
 
   Max,
 };
