@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,11 +30,12 @@
 #include "gl_replay.h"
 #include "gl_resources.h"
 
-vector<GPUCounter> GLReplay::EnumerateCounters()
+rdcarray<GPUCounter> GLReplay::EnumerateCounters()
 {
-  vector<GPUCounter> ret;
+  rdcarray<GPUCounter> ret;
 
-  ret.push_back(GPUCounter::EventGPUDuration);
+  if(HasExt[ARB_timer_query])
+    ret.push_back(GPUCounter::EventGPUDuration);
 
   if(HasExt[ARB_occlusion_query2])
     ret.push_back(GPUCounter::SamplesPassed);
@@ -56,14 +57,12 @@ vector<GPUCounter> GLReplay::EnumerateCounters()
 
   if(m_pAMDCounters)
   {
-    vector<GPUCounter> amdCounters = m_pAMDCounters->GetPublicCounterIds();
-    ret.insert(ret.end(), amdCounters.begin(), amdCounters.end());
+    ret.append(m_pAMDCounters->GetPublicCounterIds());
   }
 
   if(m_pIntelCounters)
   {
-    vector<GPUCounter> intelCounters = m_pIntelCounters->GetPublicCounterIds();
-    ret.insert(ret.end(), intelCounters.begin(), intelCounters.end());
+    ret.append(m_pIntelCounters->GetPublicCounterIds());
   }
 
   return ret;
@@ -213,14 +212,14 @@ CounterDescription GLReplay::DescribeCounter(GPUCounter counterID)
 
 struct GPUQueries
 {
-  GLuint obj[ENUM_ARRAY_SIZE(GPUCounter)];
+  GLuint obj[arraydim<GPUCounter>()];
   uint32_t eventId;
 };
 
 struct GLCounterContext
 {
   uint32_t eventStart;
-  vector<GPUQueries> queries;
+  rdcarray<GPUQueries> queries;
 };
 
 GLenum glCounters[] = {
@@ -241,7 +240,7 @@ GLenum glCounters[] = {
 };
 
 void GLReplay::FillTimers(GLCounterContext &ctx, const DrawcallDescription &drawnode,
-                          const vector<GPUCounter> &counters)
+                          const rdcarray<GPUCounter> &counters)
 {
   if(drawnode.children.empty())
     return;
@@ -301,7 +300,7 @@ void GLReplay::FillTimers(GLCounterContext &ctx, const DrawcallDescription &draw
 }
 
 void GLReplay::FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex,
-                             vector<uint32_t> *eventIDs, const DrawcallDescription &drawnode)
+                             rdcarray<uint32_t> *eventIDs, const DrawcallDescription &drawnode)
 {
   if(drawnode.children.empty())
     return;
@@ -330,11 +329,11 @@ void GLReplay::FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex,
   }
 }
 
-vector<CounterResult> GLReplay::FetchCountersAMD(const vector<GPUCounter> &counters)
+rdcarray<CounterResult> GLReplay::FetchCountersAMD(const rdcarray<GPUCounter> &counters)
 {
   if(!m_pAMDCounters->BeginMeasurementMode(AMDCounters::ApiType::Ogl, m_ReplayCtx.ctx))
   {
-    return vector<CounterResult>();
+    return rdcarray<CounterResult>();
   }
 
   uint32_t sessionID = m_pAMDCounters->CreateSession();
@@ -355,7 +354,7 @@ vector<CounterResult> GLReplay::FetchCountersAMD(const vector<GPUCounter> &count
 
   uint32_t sampleIndex = 0;
 
-  vector<uint32_t> eventIDs;
+  rdcarray<uint32_t> eventIDs;
 
   m_pDriver->ReplayMarkers(false);
 
@@ -378,7 +377,7 @@ vector<CounterResult> GLReplay::FetchCountersAMD(const vector<GPUCounter> &count
 
   m_pAMDCounters->EndSesssion(sessionID);
 
-  std::vector<CounterResult> ret =
+  rdcarray<CounterResult> ret =
       m_pAMDCounters->GetCounterData(sessionID, sampleIndex, eventIDs, counters);
 
   m_pAMDCounters->EndMeasurementMode();
@@ -387,7 +386,7 @@ vector<CounterResult> GLReplay::FetchCountersAMD(const vector<GPUCounter> &count
 }
 
 void GLReplay::FillTimersIntel(uint32_t *eventStartID, uint32_t *sampleIndex,
-                               vector<uint32_t> *eventIDs, const DrawcallDescription &drawnode)
+                               rdcarray<uint32_t> *eventIDs, const DrawcallDescription &drawnode)
 {
   if(drawnode.children.empty())
     return;
@@ -416,7 +415,7 @@ void GLReplay::FillTimersIntel(uint32_t *eventStartID, uint32_t *sampleIndex,
   }
 }
 
-vector<CounterResult> GLReplay::FetchCountersIntel(const vector<GPUCounter> &counters)
+rdcarray<CounterResult> GLReplay::FetchCountersIntel(const rdcarray<GPUCounter> &counters)
 {
   m_pIntelCounters->DisableAllCounters();
 
@@ -435,7 +434,7 @@ vector<CounterResult> GLReplay::FetchCountersIntel(const vector<GPUCounter> &cou
 
   uint32_t sampleIndex = 0;
 
-  vector<uint32_t> eventIDs;
+  rdcarray<uint32_t> eventIDs;
 
   m_pDriver->ReplayMarkers(false);
 
@@ -455,16 +454,16 @@ vector<CounterResult> GLReplay::FetchCountersIntel(const vector<GPUCounter> &cou
 
   m_pDriver->ReplayMarkers(true);
 
-  std::vector<CounterResult> ret = m_pIntelCounters->GetCounterData(sampleIndex, eventIDs, counters);
+  rdcarray<CounterResult> ret = m_pIntelCounters->GetCounterData(sampleIndex, eventIDs, counters);
 
   m_pIntelCounters->EndSession();
 
   return ret;
 }
 
-vector<CounterResult> GLReplay::FetchCounters(const vector<GPUCounter> &allCounters)
+rdcarray<CounterResult> GLReplay::FetchCounters(const rdcarray<GPUCounter> &allCounters)
 {
-  vector<CounterResult> ret;
+  rdcarray<CounterResult> ret;
 
   if(allCounters.empty())
   {
@@ -474,14 +473,14 @@ vector<CounterResult> GLReplay::FetchCounters(const vector<GPUCounter> &allCount
 
   MakeCurrentReplayContext(&m_ReplayCtx);
 
-  vector<GPUCounter> counters;
+  rdcarray<GPUCounter> counters;
   std::copy_if(allCounters.begin(), allCounters.end(), std::back_inserter(counters),
                [](const GPUCounter &c) { return IsGenericCounter(c); });
 
   if(m_pAMDCounters)
   {
     // Filter out the AMD counters
-    vector<GPUCounter> amdCounters;
+    rdcarray<GPUCounter> amdCounters;
     std::copy_if(allCounters.begin(), allCounters.end(), std::back_inserter(amdCounters),
                  [](const GPUCounter &c) { return IsAMDCounter(c); });
 
@@ -494,7 +493,7 @@ vector<CounterResult> GLReplay::FetchCounters(const vector<GPUCounter> &allCount
   if(m_pIntelCounters)
   {
     // Filter out the Intel counters
-    vector<GPUCounter> intelCounters;
+    rdcarray<GPUCounter> intelCounters;
     std::copy_if(allCounters.begin(), allCounters.end(), std::back_inserter(intelCounters),
                  [](const GPUCounter &c) { return IsIntelCounter(c); });
 
@@ -523,8 +522,11 @@ vector<CounterResult> GLReplay::FetchCounters(const vector<GPUCounter> &allCount
   double nanosToSecs = 1.0 / 1000000000.0;
 
   GLuint prevbind = 0;
-  m_pDriver->glGetIntegerv(eGL_QUERY_BUFFER_BINDING, (GLint *)&prevbind);
-  m_pDriver->glBindBuffer(eGL_QUERY_BUFFER, 0);
+  if(HasExt[ARB_query_buffer_object])
+  {
+    m_pDriver->glGetIntegerv(eGL_QUERY_BUFFER_BINDING, (GLint *)&prevbind);
+    m_pDriver->glBindBuffer(eGL_QUERY_BUFFER, 0);
+  }
 
   for(size_t i = 0; i < ctx.queries.size(); i++)
   {
@@ -556,7 +558,8 @@ vector<CounterResult> GLReplay::FetchCounters(const vector<GPUCounter> &allCount
     }
   }
 
-  m_pDriver->glBindBuffer(eGL_QUERY_BUFFER, prevbind);
+  if(HasExt[ARB_query_buffer_object])
+    m_pDriver->glBindBuffer(eGL_QUERY_BUFFER, prevbind);
 
   for(size_t i = 0; i < ctx.queries.size(); i++)
     for(uint32_t c = 0; c < counters.size(); c++)
