@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -42,7 +42,7 @@ struct GLResourceManagerConfiguration
 class GLResourceManager : public ResourceManager<GLResourceManagerConfiguration>
 {
 public:
-  GLResourceManager(WrappedOpenGL *driver);
+  GLResourceManager(CaptureState &state, WrappedOpenGL *driver);
   ~GLResourceManager() {}
   void Shutdown()
   {
@@ -110,7 +110,6 @@ public:
       {
         ++count;
         ResourceId res = it->second;
-        MarkCleanResource(res);
         if(HasResourceRecord(res))
           GetResourceRecord(res)->Delete(this);
         ReleaseCurrentResource(it->second);
@@ -163,6 +162,8 @@ public:
     auto it = m_CurrentResourceIds.find(res);
     if(it != m_CurrentResourceIds.end())
     {
+      m_Names.erase(it->second);
+
       ReleaseCurrentResource(it->second);
       m_CurrentResourceIds.erase(res);
     }
@@ -203,20 +204,43 @@ public:
 
   using ResourceManager::MarkResourceFrameReferenced;
 
+  void MarkResourceFrameReferenced(ResourceId id, FrameRefType refType)
+  {
+    GLResourceRecord *record = GetResourceRecord(id);
+    if(record && record->viewSource != ResourceId())
+      ResourceManager::MarkResourceFrameReferenced(record->viewSource, refType);
+
+    ResourceManager::MarkResourceFrameReferenced(id, refType);
+  }
+
   void MarkResourceFrameReferenced(GLResource res, FrameRefType refType)
   {
     // we allow VAO 0 as a special case
     if(res.name == 0 && res.Namespace != eResVertexArray)
       return;
-    ResourceManager::MarkResourceFrameReferenced(GetID(res), refType);
+    GLResourceManager::MarkResourceFrameReferenced(GetID(res), refType);
   }
 
-  using ResourceManager::MarkDirtyResource;
+  void MarkDirtyResource(ResourceId id)
+  {
+    GLResourceRecord *record = GetResourceRecord(id);
+    if(record && record->viewSource != ResourceId())
+      ResourceManager::MarkDirtyResource(record->viewSource);
 
-  void MarkDirtyResource(GLResource res) { return ResourceManager::MarkDirtyResource(GetID(res)); }
-  using ResourceManager::MarkCleanResource;
+    return ResourceManager::MarkDirtyResource(id);
+  }
+  void MarkDirtyResource(GLResource res)
+  {
+    return GLResourceManager::MarkDirtyResource(GetID(res));
+  }
+  // Mark resource as dirty and write-referenced.
+  // Write-referenced resources are used to track resource "age".
+  void MarkDirtyWithWriteReference(GLResource res)
+  {
+    MarkResourceFrameReferenced(res, eFrameRef_ReadBeforeWrite);
+    MarkDirtyResource(res);
+  }
 
-  void MarkCleanResource(GLResource res) { return ResourceManager::MarkCleanResource(GetID(res)); }
   void RegisterSync(ContextPair &ctx, GLsync sync, GLuint &name, ResourceId &id)
   {
     name = (GLuint)Atomic::Inc64(&m_SyncName);
@@ -228,9 +252,11 @@ public:
 
   GLsync GetSync(GLuint name) { return m_CurrentSyncs[name]; }
   ResourceId GetSyncID(GLsync sync) { return m_SyncIDs[sync]; }
-  // KHR_debug storage on replay
-  const std::string &GetName(ResourceId id) { return m_Names[id]; }
-  void SetName(ResourceId id, const std::string &name) { m_Names[id] = name; }
+  // KHR_debug storage
+  const rdcstr &GetName(ResourceId id) { return m_Names[id]; }
+  void SetName(ResourceId id, const rdcstr &name) { m_Names[id] = name; }
+  void SetName(GLResource res, const rdcstr &name) { SetName(GetID(res), name); }
+  rdcstr GetName(GLResource res) { return GetName(GetID(res)); }
   // we need to find all the children bound to VAOs/FBOs and mark them referenced. The reason for
   // this is that say a VAO became high traffic and we stopped serialising buffer binds, but then it
   // is never modified in a frame and none of the buffers are ever referenced. They would be
@@ -239,43 +265,41 @@ public:
   void MarkVAOReferenced(GLResource res, FrameRefType ref, bool allowFake0 = false);
   void MarkFBOReferenced(GLResource res, FrameRefType ref);
 
+  bool IsResourceTrackedForPersistency(const GLResource &res);
+
   template <typename SerialiserType>
-  bool Serialise_InitialState(SerialiserType &ser, ResourceId resid, GLResource res);
+  bool Serialise_InitialState(SerialiserType &ser, ResourceId id, GLResourceRecord *record,
+                              const GLInitialContents *initial);
 
   void ContextPrepare_InitialState(GLResource res);
-  bool Serialise_InitialState(WriteSerialiser &ser, ResourceId resid, GLResource res)
+  bool Serialise_InitialState(WriteSerialiser &ser, ResourceId id, GLResourceRecord *record,
+                              const GLInitialContents *initial)
   {
-    return Serialise_InitialState<WriteSerialiser>(ser, resid, res);
+    return Serialise_InitialState<WriteSerialiser>(ser, id, record, initial);
   }
 
   void SetInternalResource(GLResource res);
 
 private:
   bool ResourceTypeRelease(GLResource res);
-  bool Force_InitialState(GLResource res, bool prepare);
-  bool Need_InitialStateChunk(GLResource res);
   bool Prepare_InitialState(GLResource res);
-  uint32_t GetSize_InitialState(ResourceId resid, GLResource res);
+  uint64_t GetSize_InitialState(ResourceId resid, const GLInitialContents &initial);
 
-  void CreateTextureImage(GLuint tex, GLenum internalFormat, GLenum internalFormatHint,
-                          GLenum textype, GLint dim, GLint width, GLint height, GLint depth,
-                          GLint samples, int mips);
   void PrepareTextureInitialContents(ResourceId liveid, ResourceId origid, GLResource res);
 
   void Create_InitialState(ResourceId id, GLResource live, bool hasData);
-  void Apply_InitialState(GLResource live, GLInitialContents initial);
+  void Apply_InitialState(GLResource live, const GLInitialContents &initial);
 
-  map<GLResource, GLResourceRecord *> m_GLResourceRecords;
+  std::map<GLResource, GLResourceRecord *> m_GLResourceRecords;
 
-  map<GLResource, ResourceId> m_CurrentResourceIds;
+  std::map<GLResource, ResourceId> m_CurrentResourceIds;
 
   // sync objects must be treated differently as they're not GLuint names, but pointer sized.
   // We manually give them GLuint names so they're otherwise namespaced as (eResSync, GLuint)
-  map<GLsync, ResourceId> m_SyncIDs;
-  map<GLuint, GLsync> m_CurrentSyncs;
-  map<ResourceId, std::string> m_Names;
+  std::map<GLsync, ResourceId> m_SyncIDs;
+  std::map<GLuint, GLsync> m_CurrentSyncs;
+  std::map<ResourceId, rdcstr> m_Names;
   volatile int64_t m_SyncName;
 
-  CaptureState m_State;
   WrappedOpenGL *m_Driver;
 };
