@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,6 @@
 #include <algorithm>
 #include "common/common.h"
 #include "data/glsl_shaders.h"
-#include "maths/camera.h"
 #include "maths/formatpacking.h"
 #include "maths/matrix.h"
 #include "strings/string_utils.h"
@@ -37,7 +36,8 @@
 #define OPENGL 1
 #include "data/glsl/glsl_ubos_cpp.h"
 
-void GLReplay::CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint fragShader)
+bool GLReplay::CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint fragShader,
+                                    GLuint fragShaderSPIRV)
 {
   WrappedOpenGL &drv = *m_pDriver;
 
@@ -60,11 +60,14 @@ void GLReplay::CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint frag
   // the reflection for the vertex shader, used to copy vertex bindings
   ShaderReflection *vsRefl = NULL;
 
+  bool HasSPIRVShaders = false;
+  bool HasGLSLShaders = false;
+
   if(Program == 0)
   {
     if(Pipeline == 0)
     {
-      return;
+      return false;
     }
     else
     {
@@ -76,6 +79,14 @@ void GLReplay::CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint frag
       {
         if(pipeDetails.stageShaders[i] != ResourceId())
         {
+          const WrappedOpenGL::ShaderData &shadDetails =
+              m_pDriver->m_Shaders[pipeDetails.stageShaders[i]];
+
+          if(shadDetails.reflection.encoding == ShaderEncoding::SPIRV)
+            HasSPIRVShaders = true;
+          else
+            HasGLSLShaders = true;
+
           programs[i] =
               m_pDriver->GetResourceManager()->GetCurrentResource(pipeDetails.stagePrograms[i]).name;
           shaders[i] =
@@ -88,13 +99,10 @@ void GLReplay::CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint frag
 
             if(progDetails.shaderProgramUnlinkable)
             {
-              const WrappedOpenGL::ShaderData &shadDetails =
-                  m_pDriver->m_Shaders[pipeDetails.stageShaders[i]];
-
-              std::vector<const char *> sources;
+              rdcarray<const char *> sources;
               sources.reserve(shadDetails.sources.size());
 
-              for(const std::string &s : shadDetails.sources)
+              for(const rdcstr &s : shadDetails.sources)
                 sources.push_back(s.c_str());
 
               shaders[i] = tmpShaders[i] = drv.glCreateShader(ShaderEnum(i));
@@ -114,7 +122,7 @@ void GLReplay::CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint frag
           }
 
           if(i == 0)
-            vsRefl = GetShader(pipeDetails.stageShaders[i], ShaderEntryPoint());
+            vsRefl = GetShader(ResourceId(), pipeDetails.stageShaders[i], ShaderEntryPoint());
         }
       }
     }
@@ -133,21 +141,40 @@ void GLReplay::CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint frag
         shaders[i] =
             m_pDriver->GetResourceManager()->GetCurrentResource(progDetails.stageShaders[i]).name;
 
+        const WrappedOpenGL::ShaderData &shadDetails =
+            m_pDriver->m_Shaders[progDetails.stageShaders[i]];
+
+        if(shadDetails.reflection.encoding == ShaderEncoding::SPIRV)
+          HasSPIRVShaders = true;
+        else
+          HasGLSLShaders = true;
+
         if(i == 0)
-          vsRefl = GetShader(progDetails.stageShaders[0], ShaderEntryPoint());
+          vsRefl = GetShader(ResourceId(), progDetails.stageShaders[0], ShaderEntryPoint());
       }
     }
   }
 
+  if(HasGLSLShaders && HasSPIRVShaders)
+    RDCERR("Unsupported - mixed GLSL and SPIR-V shaders in pipeline");
+
   // attach the shaders
   for(size_t i = 0; i < 4; i++)
     if(shaders[i])
       drv.glAttachShader(DebugData.overlayProg, shaders[i]);
 
-  drv.glAttachShader(DebugData.overlayProg, fragShader);
+  if(HasSPIRVShaders)
+  {
+    RDCASSERT(fragShaderSPIRV);
+    drv.glAttachShader(DebugData.overlayProg, fragShaderSPIRV);
+  }
+  else
+  {
+    drv.glAttachShader(DebugData.overlayProg, fragShader);
+  }
 
   // copy the vertex attribs over from the source program
-  if(vsRefl && programs[0])
+  if(vsRefl && programs[0] && !HasSPIRVShaders)
     CopyProgramAttribBindings(programs[0], DebugData.overlayProg, vsRefl);
 
   // link the overlay program
@@ -158,7 +185,10 @@ void GLReplay::CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint frag
     if(shaders[i])
       drv.glDetachShader(DebugData.overlayProg, shaders[i]);
 
-  drv.glDetachShader(DebugData.overlayProg, fragShader);
+  if(HasSPIRVShaders)
+    drv.glDetachShader(DebugData.overlayProg, fragShaderSPIRV);
+  else
+    drv.glDetachShader(DebugData.overlayProg, fragShader);
 
   // delete any temporaries
   for(size_t i = 0; i < 4; i++)
@@ -173,18 +203,33 @@ void GLReplay::CreateOverlayProgram(GLuint Program, GLuint Pipeline, GLuint frag
   {
     drv.glGetProgramInfoLog(DebugData.overlayProg, 1024, NULL, buffer);
     RDCERR("Error linking overlay program: %s", buffer);
-    return;
+    return false;
   }
 
   // copy the uniform values over from the source program. This is redundant but harmless if the
   // same program is bound to multiple stages. It's just inefficient
-  for(size_t i = 0; i < 4; i++)
-    if(programs[i])
-      CopyProgramUniforms(programs[i], DebugData.overlayProg);
+  {
+    PerStageReflections dstStages;
+    m_pDriver->FillReflectionArray(ProgramRes(ctx, DebugData.overlayProg), dstStages);
+
+    for(size_t i = 0; i < 4; i++)
+    {
+      if(programs[i])
+      {
+        PerStageReflections stages;
+        m_pDriver->FillReflectionArray(ProgramRes(ctx, programs[i]), stages);
+
+        CopyProgramUniforms(stages, programs[i], dstStages, DebugData.overlayProg);
+      }
+    }
+  }
+
+  return HasSPIRVShaders;
 }
 
-ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOverlay overlay,
-                                   uint32_t eventId, const vector<uint32_t> &passEvents)
+ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeCast, FloatVector clearCol,
+                                   DebugOverlay overlay, uint32_t eventId,
+                                   const rdcarray<uint32_t> &passEvents)
 {
   WrappedOpenGL &drv = *m_pDriver;
 
@@ -216,7 +261,7 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
 
   if(IsGLES)
   {
-    shaderType = eShaderGLSLES;
+    shaderType = ShaderType::GLSLES;
 
     // default to 100 just in case something is broken
     glslVer = 100;
@@ -243,20 +288,19 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
   {
     // Desktop GL is vastly better, it can link any program so we just default to 150
     glslVer = 150;
-    shaderType = eShaderGLSL;
+    shaderType = ShaderType::GLSL;
   }
 
   // this is always compatible.
   {
-    std::string source =
-        GenerateGLSLShader(GetEmbeddedResource(glsl_fixedcol_frag), shaderType, glslVer);
+    rdcstr source = GenerateGLSLShader(GetEmbeddedResource(glsl_fixedcol_frag), shaderType, glslVer);
     DebugData.fixedcolFragShader = CreateShader(eGL_FRAGMENT_SHADER, source);
   }
 
   // this is not supported on GLES
-  if(shaderType == eShaderGLSL)
+  if(shaderType == ShaderType::GLSL)
   {
-    std::string defines = "";
+    rdcstr defines = "";
 
     if(!HasExt[ARB_derivative_control])
     {
@@ -273,9 +317,15 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
     // needs these extensions
     if(HasExt[ARB_gpu_shader5] && HasExt[ARB_shader_image_load_store])
     {
-      std::string source =
+      rdcstr source =
           GenerateGLSLShader(GetEmbeddedResource(glsl_quadwrite_frag), shaderType, glslVer, defines);
       DebugData.quadoverdrawFragShader = CreateShader(eGL_FRAGMENT_SHADER, source);
+
+      // we expect if the SPIR-V extension is present then we've compiled this variant.
+      if(HasExt[ARB_gl_spirv])
+      {
+        RDCASSERT(DebugData.quadoverdrawFragShaderSPIRV);
+      }
     }
   }
   else
@@ -287,9 +337,20 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
   // we bind the separable program created for each shader, and copy
   // uniforms and attrib bindings from the 'real' programs, wherever
   // they are.
-  CreateOverlayProgram(rs.Program.name, rs.Pipeline.name, DebugData.fixedcolFragShader);
+  bool spirvOverlay =
+      CreateOverlayProgram(rs.Program.name, rs.Pipeline.name, DebugData.fixedcolFragShader,
+                           DebugData.fixedcolFragShaderSPIRV);
   drv.glUseProgram(DebugData.overlayProg);
 
+  GLint overlayFixedColLocation = 0;
+
+  // on SPIR-V overlays we don't query the location, it's baked into the shader
+  if(spirvOverlay)
+    overlayFixedColLocation = 99;
+  else
+    overlayFixedColLocation =
+        drv.glGetUniformLocation(DebugData.overlayProg, "RENDERDOC_Fixed_Color");
+
   auto &texDetails = m_pDriver->m_Textures[texid];
 
   GLenum texBindingEnum = eGL_TEXTURE_2D;
@@ -386,9 +447,8 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
     float black[] = {0.0f, 0.0f, 0.0f, 0.5f};
     drv.glClearBufferfv(eGL_COLOR, 0, black);
 
-    GLint colLoc = drv.glGetUniformLocation(DebugData.overlayProg, "RENDERDOC_Fixed_Color");
     float colVal[] = {0.8f, 0.1f, 0.8f, 1.0f};
-    drv.glProgramUniform4fv(DebugData.overlayProg, colLoc, 1, colVal);
+    drv.glProgramUniform4fv(DebugData.overlayProg, overlayFixedColLocation, 1, colVal);
 
     ReplayLog(eventId, eReplay_OnlyDraw);
   }
@@ -397,9 +457,8 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
     float wireCol[] = {200.0f / 255.0f, 255.0f / 255.0f, 0.0f / 255.0f, 0.0f};
     drv.glClearBufferfv(eGL_COLOR, 0, wireCol);
 
-    GLint colLoc = drv.glGetUniformLocation(DebugData.overlayProg, "RENDERDOC_Fixed_Color");
     wireCol[3] = 1.0f;
-    drv.glProgramUniform4fv(DebugData.overlayProg, colLoc, 1, wireCol);
+    drv.glProgramUniform4fv(DebugData.overlayProg, overlayFixedColLocation, 1, wireCol);
 
     if(!IsGLES)
     {
@@ -417,12 +476,12 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
 
       const DrawcallDescription *draw = m_pDriver->GetDrawcall(eventId);
 
-      std::vector<uint32_t> patchedIndices;
+      rdcarray<uint32_t> patchedIndices;
 
       // readback the index buffer data
       if(idxbuf)
       {
-        std::vector<byte> idxs;
+        rdcarray<byte> idxs;
         uint32_t offset = draw->indexOffset * draw->indexByteWidth;
         uint32_t length = 1;
         drv.glGetNamedBufferParameterivEXT(idxbuf, eGL_BUFFER_SIZE, (GLint *)&length);
@@ -453,7 +512,7 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
       else
       {
         // generate 'index' list
-        std::vector<uint32_t> idxs;
+        rdcarray<uint32_t> idxs;
         idxs.resize(draw->numIndices);
         for(uint32_t i = 0; i < draw->numIndices; i++)
           idxs[i] = i;
@@ -591,9 +650,8 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
     float black[] = {0.0f, 0.0f, 0.0f, 0.0f};
     drv.glClearBufferfv(eGL_COLOR, 0, black);
 
-    GLint colLoc = drv.glGetUniformLocation(DebugData.overlayProg, "RENDERDOC_Fixed_Color");
     float red[] = {1.0f, 0.0f, 0.0f, 1.0f};
-    drv.glProgramUniform4fv(DebugData.overlayProg, colLoc, 1, red);
+    drv.glProgramUniform4fv(DebugData.overlayProg, overlayFixedColLocation, 1, red);
 
     ReplayLog(eventId, eReplay_OnlyDraw);
 
@@ -776,7 +834,7 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
     drv.glBindFramebuffer(eGL_READ_FRAMEBUFFER, rs.DrawFBO.name);
 
     float green[] = {0.0f, 1.0f, 0.0f, 1.0f};
-    drv.glProgramUniform4fv(DebugData.overlayProg, colLoc, 1, green);
+    drv.glProgramUniform4fv(DebugData.overlayProg, overlayFixedColLocation, 1, green);
 
     if(overlay == DebugOverlay::Depth)
     {
@@ -834,8 +892,7 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
     col[0] = 1.0f;
     col[3] = 1.0f;
 
-    GLint colLoc = drv.glGetUniformLocation(DebugData.overlayProg, "RENDERDOC_Fixed_Color");
-    drv.glProgramUniform4fv(DebugData.overlayProg, colLoc, 1, col);
+    drv.glProgramUniform4fv(DebugData.overlayProg, overlayFixedColLocation, 1, col);
 
     ReplayLog(eventId, eReplay_OnlyDraw);
 
@@ -847,7 +904,7 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
     col[0] = 0.0f;
     col[1] = 1.0f;
 
-    drv.glProgramUniform4fv(DebugData.overlayProg, colLoc, 1, col);
+    drv.glProgramUniform4fv(DebugData.overlayProg, overlayFixedColLocation, 1, col);
 
     ReplayLog(eventId, eReplay_OnlyDraw);
   }
@@ -856,7 +913,7 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
     float col[] = {0.0f, 0.0f, 0.0f, 0.0f};
     drv.glClearBufferfv(eGL_COLOR, 0, col);
 
-    vector<uint32_t> events = passEvents;
+    rdcarray<uint32_t> events = passEvents;
 
     if(overlay == DebugOverlay::ClearBeforeDraw)
       events.clear();
@@ -875,9 +932,26 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
         rs.ApplyState(&drv);
       }
 
-      float black[] = {0.0f, 0.0f, 0.0f, 0.0f};
       for(int i = 0; i < 8; i++)
-        drv.glClearBufferfv(eGL_COLOR, i, black);
+        drv.glClearBufferfv(eGL_COLOR, i, &clearCol.x);
+
+      // Try to clear depth as well, to help debug shadow rendering
+      if(IsDepthStencilFormat(texDetails.internalFormat))
+      {
+        // If the depth func is equal or not equal, don't clear at all since the output would be
+        // altered in an way that would cause replay to produce mostly incorrect results.
+        // Similarly, skip if the depth func is always, as we'd have a 50% chance of guessing the
+        // wrong clear value.
+        if(rs.DepthFunc != eGL_EQUAL && rs.DepthFunc != eGL_NOTEQUAL && rs.DepthFunc != eGL_ALWAYS)
+        {
+          // Don't use the render state's depth clear value, as this overlay should show as much
+          // information as possible. Instead, clear to the value that would cause the most depth
+          // writes to happen.
+          bool depthFuncLess = rs.DepthFunc == eGL_LESS || rs.DepthFunc == eGL_LEQUAL;
+          float depthClear = depthFuncLess ? 1.0f : 0.0f;
+          drv.glClearBufferfv(eGL_DEPTH, 0, &depthClear);
+        }
+      }
 
       for(size_t i = 0; i < events.size(); i++)
       {
@@ -914,7 +988,7 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
     *v = Vec4f(rs.Viewports[0].width, rs.Viewports[0].height);
     drv.glUnmapBuffer(eGL_COPY_WRITE_BUFFER);
 
-    vector<uint32_t> events = passEvents;
+    rdcarray<uint32_t> events = passEvents;
 
     if(overlay == DebugOverlay::TriangleSizeDraw)
       events.clear();
@@ -1283,7 +1357,7 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
       float black[] = {0.0f, 0.0f, 0.0f, 0.0f};
       drv.glClearBufferfv(eGL_COLOR, 0, black);
 
-      vector<uint32_t> events = passEvents;
+      rdcarray<uint32_t> events = passEvents;
 
       if(overlay == DebugOverlay::QuadOverdrawDraw)
         events.clear();
@@ -1429,15 +1503,19 @@ ResourceId GLReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOve
           // replace fragment shader. This is exactly what we did
           // at the start of this function for the single-event case, but now we have
           // to do it for every event
-          CreateOverlayProgram(prog, pipe, DebugData.quadoverdrawFragShader);
+          spirvOverlay = CreateOverlayProgram(prog, pipe, DebugData.quadoverdrawFragShader,
+                                              DebugData.quadoverdrawFragShaderSPIRV);
           drv.glUseProgram(DebugData.overlayProg);
           drv.glBindProgramPipeline(0);
 
-          GLint loc = drv.glGetUniformLocation(DebugData.overlayProg, "overdrawImage");
-          if(loc != -1)
-            drv.glUniform1i(loc, 0);
-          else
-            RDCERR("Couldn't get location of overdrawImage");
+          if(!spirvOverlay)
+          {
+            GLint loc = drv.glGetUniformLocation(DebugData.overlayProg, "overdrawImage");
+            if(loc != -1)
+              drv.glUniform1i(loc, 0);
+            else
+              RDCERR("Couldn't get location of overdrawImage");
+          }
 
           drv.glBindFramebuffer(eGL_READ_FRAMEBUFFER, curdrawfbo);
           SafeBlitFramebuffer(0, 0, texDetails.width, texDetails.height, 0, 0, texDetails.width,
