@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,7 +33,7 @@
 #define OPENGL 1
 #include "data/glsl/glsl_ubos_cpp.h"
 
-void GLReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &secondaryDraws,
+void GLReplay::RenderMesh(uint32_t eventId, const rdcarray<MeshFormat> &secondaryDraws,
                           const MeshDisplay &cfg)
 {
   WrappedOpenGL &drv = *m_pDriver;
@@ -68,6 +68,20 @@ void GLReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &secondaryD
   if(HasExt[EXT_framebuffer_sRGB])
     drv.glEnable(eGL_FRAMEBUFFER_SRGB);
 
+  if(cfg.position.allowRestart)
+  {
+    if(IsGLES)
+    {
+      drv.glEnable(eGL_PRIMITIVE_RESTART_FIXED_INDEX);
+    }
+    else
+    {
+      drv.glEnable(eGL_PRIMITIVE_RESTART);
+      drv.glPrimitiveRestartIndex(cfg.position.restartIndex &
+                                  (0xFFFFFFFFU >> ((4 - cfg.position.indexByteStride) * 8)));
+    }
+  }
+
   drv.glDisable(eGL_CULL_FACE);
 
   if(cfg.position.unproject)
@@ -124,6 +138,15 @@ void GLReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &secondaryD
         GLuint vb = m_pDriver->GetResourceManager()->GetCurrentResource(fmt.vertexResourceId).name;
         drv.glBindVertexBuffer(0, vb, (GLintptr)fmt.vertexByteOffset, fmt.vertexByteStride);
 
+        {
+          GLint bytesize = 0;
+          drv.glGetNamedBufferParameterivEXT(vb, eGL_BUFFER_SIZE, &bytesize);
+
+          // skip empty source buffers
+          if(bytesize == 0)
+            continue;
+        }
+
         GLenum secondarytopo = MakeGLPrimitiveTopology(fmt.topology);
 
         if(fmt.indexByteStride)
@@ -253,6 +276,16 @@ void GLReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &secondaryD
 
     GLuint vb =
         m_pDriver->GetResourceManager()->GetCurrentResource(meshData[i]->vertexResourceId).name;
+
+    {
+      GLint bytesize = 0;
+      drv.glGetNamedBufferParameterivEXT(vb, eGL_BUFFER_SIZE, &bytesize);
+
+      // skip empty source buffers
+      if(bytesize == 0)
+        continue;
+    }
+
     drv.glBindVertexBuffer(i, vb, offs, meshData[i]->vertexByteStride);
 
     if(meshData[i]->instanced)
@@ -518,16 +551,16 @@ void GLReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &secondaryD
     FloatVector activeVertex;
 
     // primitive this vert is a part of (red prim, optional)
-    vector<FloatVector> activePrim;
+    rdcarray<FloatVector> activePrim;
 
     // for patch lists, to show other verts in patch (green dots, optional)
     // for non-patch lists, we use the activePrim and adjacentPrimVertices
     // to show what other verts are related
-    vector<FloatVector> inactiveVertices;
+    rdcarray<FloatVector> inactiveVertices;
 
     // adjacency (line or tri, strips or lists) (green prims, optional)
     // will be N*M long, N adjacent prims of M verts each. M = primSize below
-    vector<FloatVector> adjacentPrimVertices;
+    rdcarray<FloatVector> adjacentPrimVertices;
 
     GLenum primTopo = eGL_TRIANGLES;
     uint32_t primSize = 3;    // number of verts per primitive
