@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -78,7 +78,7 @@ struct GLRenderState
   void Clear();
 
   void MarkReferenced(WrappedOpenGL *driver, bool initial) const;
-  void MarkDirty(WrappedOpenGL *driver);
+  void MarkDirty(WrappedOpenGL *driver) const;
 
   enum
   {
