@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2014-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,15 +26,17 @@
 #include <algorithm>
 #include <functional>
 #include "3rdparty/glslang/glslang/Public/ShaderLang.h"
+#include "driver/shaders/spirv/glslang_compile.h"
 #include "gl_driver.h"
 
 template <>
-std::string DoStringise(const FFVertexOutput &el)
+rdcstr DoStringise(const FFVertexOutput &el)
 {
   BEGIN_ENUM_STRINGISE(FFVertexOutput);
   {
     STRINGISE_ENUM_CLASS_NAMED(PointSize, "gl_PointSize");
     STRINGISE_ENUM_CLASS_NAMED(ClipDistance, "gl_ClipDistance");
+    STRINGISE_ENUM_CLASS_NAMED(CullDistance, "gl_CullDistance");
     STRINGISE_ENUM_CLASS_NAMED(ClipVertex, "gl_ClipVertex");
     STRINGISE_ENUM_CLASS_NAMED(FrontColor, "gl_FrontColor");
     STRINGISE_ENUM_CLASS_NAMED(BackColor, "gl_BackColor");
@@ -76,30 +78,29 @@ void sort(rdcarray<ShaderConstant> &vars)
     sort(vars[i].type.members);
 }
 
-void CheckVertexOutputUses(const std::vector<std::string> &sources,
-                           FixedFunctionVertexOutputs &outputUsage)
+void CheckVertexOutputUses(const rdcarray<rdcstr> &sources, FixedFunctionVertexOutputs &outputUsage)
 {
   outputUsage = FixedFunctionVertexOutputs();
 
   for(FFVertexOutput output : values<FFVertexOutput>())
   {
     // we consider an output used if we encounter a '=' before either a ';' or the end of the string
-    std::string name = ToStr(output);
+    rdcstr name = ToStr(output);
 
     for(size_t i = 0; i < sources.size(); i++)
     {
-      const std::string &s = sources[i];
+      const rdcstr &s = sources[i];
 
-      size_t offs = 0;
+      int32_t offs = 0;
 
       for(;;)
       {
         offs = s.find(name, offs);
 
-        if(offs == string::npos)
+        if(offs < 0)
           break;
 
-        while(offs < s.length())
+        while(offs < s.count())
         {
           if(s[offs] == '=')
           {
@@ -179,12 +180,12 @@ static bool iswhitespace(char c)
   return isspacetab(c) || isnewline(c);
 }
 
-GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string> sources,
-                                  vector<string> *includepaths)
+GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, rdcarray<rdcstr> sources,
+                                  rdcarray<rdcstr> *includepaths)
 {
   // in and out blocks are added separately, in case one is there already
   const char *blockIdentifiers[2] = {"in gl_PerVertex", "out gl_PerVertex"};
-  string blocks[2] = {"", ""};
+  rdcstr blocks[2] = {"", ""};
 
   if(type == eGL_VERTEX_SHADER)
   {
@@ -242,14 +243,14 @@ GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string
     // (this is probably most likely for clipdistance if it's redeclared with a size)
 
     // we start by concatenating the source strings to make parsing easier.
-    std::string combined;
+    rdcstr combined;
 
     for(size_t i = 0; i < sources.size(); i++)
       combined += sources[i];
 
     for(int attempt = 0; attempt < 2; attempt++)
     {
-      std::string src = combined;
+      rdcstr src = combined;
 
       if(attempt == 1)
       {
@@ -269,8 +270,14 @@ GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string
 
         glslang::TShader::ForbidIncluder incl;
 
-        bool success = sh.preprocess(&DefaultResources, 100, ENoProfile, false, false,
-                                     EShMsgOnlyPreprocessor, &src, incl);
+        bool success;
+
+        {
+          std::string outstr;
+          success = sh.preprocess(GetDefaultResources(), 100, ENoProfile, false, false,
+                                  EShMsgOnlyPreprocessor, &outstr, incl);
+          src.assign(outstr.c_str(), outstr.size());
+        }
 
         if(!success)
         {
@@ -285,28 +292,28 @@ GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string
         if(type == eGL_VERTEX_SHADER && blocktype == 0)
           continue;
 
-        string block = blocks[blocktype];
+        rdcstr block = blocks[blocktype];
         const char *identifier = blockIdentifiers[blocktype];
 
         // if we find the 'identifier' (ie. the block name),
         // assume this block is already present and stop.
         // only try and insert this block if the shader doesn't already have it
-        if(src.find(identifier) != string::npos)
+        if(src.contains(identifier))
         {
           continue;
         }
 
         {
-          size_t len = src.length();
+          int32_t len = src.count();
 
           // find if this source contains a #version, accounting for whitespace
-          size_t it = 0;
+          int32_t it = 0;
 
-          while(it != string::npos)
+          while(it >= 0)
           {
             it = src.find("#", it);
 
-            if(it == string::npos)
+            if(it < 0)
               break;
 
             // advance past the #
@@ -324,10 +331,12 @@ GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string
           }
 
           // no #version found
-          if(it == string::npos)
+          if(it < 0)
           {
-            // insert at the start
-            it = 0;
+            // ensure we're using at least #version 130
+            src.insert(0, "#version 130\n");
+            // insert after that
+            it = 13;
           }
           else
           {
@@ -337,6 +346,11 @@ GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string
             while(it < len && isspacetab(src[it]))
               ++it;
 
+            // if the version is less than 130 we need to upgrade it to even be able to use out
+            // blocks
+            if(src[it] == '1' && src[it + 1] < '3')
+              src[it + 1] = '3';
+
             // skip number
             while(it < len && src[it] >= '0' && src[it] <= '9')
               ++it;
@@ -355,7 +369,7 @@ GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string
             it++;
 
             // next line after #version, insert the extension declaration
-            if(it < src.length())
+            if(it < src.count())
               src.insert(it, "#extension GL_ARB_separate_shader_objects : enable\n");
 
             // how deep are we in an #if. We want to place our definition
@@ -440,11 +454,12 @@ GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string
 
               // see if we have a precision statement, if so skip that
               const char precision[] = "precision";
-              if(it + sizeof(precision) < len && !strncmp(&src[it], precision, sizeof(precision) - 1))
+              if(int32_t(it + sizeof(precision)) < len &&
+                 !strncmp(&src[it], precision, sizeof(precision) - 1))
               {
                 // since we're speculating here (although what else could it be?) we don't modify
                 // it until we're sure.
-                size_t pit = it + sizeof(precision);
+                int32_t pit = it + sizeof(precision);
 
                 // skip whitespace
                 while(pit < len && isspacetab(src[pit]))
@@ -455,15 +470,15 @@ GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string
                 const char mediump[] = "mediump";
                 const char highp[] = "highp";
 
-                bool precisionMatch =
-                    (pit + sizeof(lowp) < len && !strncmp(&src[pit], lowp, sizeof(lowp) - 1) &&
-                     isspacetab(src[pit + sizeof(lowp) - 1]));
-                precisionMatch |= (pit + sizeof(mediump) < len &&
+                bool precisionMatch = (int32_t(pit + sizeof(lowp)) < len &&
+                                       !strncmp(&src[pit], lowp, sizeof(lowp) - 1) &&
+                                       isspacetab(src[pit + sizeof(lowp) - 1]));
+                precisionMatch |= (int32_t(pit + sizeof(mediump)) < len &&
                                    !strncmp(&src[pit], mediump, sizeof(mediump) - 1) &&
                                    isspacetab(src[pit + sizeof(mediump) - 1]));
-                precisionMatch |=
-                    (pit + sizeof(highp) < len && !strncmp(&src[pit], highp, sizeof(highp) - 1) &&
-                     isspacetab(src[pit + sizeof(highp) - 1]));
+                precisionMatch |= (int32_t(pit + sizeof(highp)) < len &&
+                                   !strncmp(&src[pit], highp, sizeof(highp) - 1) &&
+                                   isspacetab(src[pit + sizeof(highp) - 1]));
 
                 if(precisionMatch)
                 {
@@ -491,7 +506,7 @@ GLuint MakeSeparableShaderProgram(WrappedOpenGL &drv, GLenum type, vector<string
             }
           }
 
-          if(it < src.length())
+          if(it < src.count())
             src.insert(it, block);
         }
       }
@@ -728,9 +743,11 @@ void ReconstructVarTree(GLenum query, GLuint sepProg, GLuint varIdx, GLint numPa
   }
 
   var.type.descriptor.rowMajorStorage = (values[6] > 0);
-  var.type.descriptor.arrayByteStride = values[7];
   var.type.descriptor.matrixByteStride = (uint8_t)values[8];
 
+  RDCASSERTMSG("Stride is too large for uint16_t", values[7] <= 0xffff);
+  var.type.descriptor.arrayByteStride = RDCMIN((uint32_t)values[7], 0xffffu) & 0xffff;
+
   bool bareUniform = false;
 
   // for plain uniforms we won't get an array/matrix byte stride. Calculate tightly packed strides
@@ -777,7 +794,7 @@ void ReconstructVarTree(GLenum query, GLuint sepProg, GLuint varIdx, GLint numPa
   var.name.resize(values[1] - 1);
   GL.glGetProgramResourceName(sepProg, query, varIdx, values[1], NULL, &var.name[0]);
 
-  std::string fullname = var.name;
+  rdcstr fullname = var.name;
 
   int32_t c = values[1] - 1;
 
@@ -868,9 +885,15 @@ void ReconstructVarTree(GLenum query, GLuint sepProg, GLuint varIdx, GLint numPa
     parentVar.type.descriptor.type = var.type.descriptor.type;
     parentVar.type.descriptor.elements =
         isarray && !multiDimArray ? RDCMAX(1U, uint32_t(arrayIdx + 1)) : 0;
-    parentVar.type.descriptor.arrayByteStride = topLevelStride;
     parentVar.type.descriptor.matrixByteStride = 0;
 
+    RDCASSERTMSG("Stride is too large for uint16_t", topLevelStride <= 0xffff);
+    parentVar.type.descriptor.arrayByteStride = RDCMIN((uint32_t)topLevelStride, 0xffffu) & 0xffff;
+
+    // consider all block-level SSBO structs to have infinite elements if they are an array at all
+    if(blockLevel && topLevelStride && isarray)
+      parentVar.type.descriptor.elements = ~0U;
+
     if(!blockLevel)
       topLevelStride = 0;
 
@@ -985,7 +1008,7 @@ void ReconstructVarTree(GLenum query, GLuint sepProg, GLuint varIdx, GLint numPa
     bool duplicate = false;
 
     // nm points into var.name's storage, so copy out to a temporary
-    std::string n = nm;
+    rdcstr n = nm;
     var.name = n;
 
     if(bareUniform && !multiDimArray)
@@ -1050,8 +1073,8 @@ int ParseVersionStatement(const char *version)
   return ret;
 }
 
-static void AddSigParameter(vector<SigParameter> &sigs, uint32_t &regIndex, const SigParameter &sig,
-                            const char *nm, int rows, int arrayIdx)
+static void AddSigParameter(rdcarray<SigParameter> &sigs, uint32_t &regIndex,
+                            const SigParameter &sig, const char *nm, int rows, int arrayIdx)
 {
   if(rows == 1)
   {
@@ -1059,6 +1082,8 @@ static void AddSigParameter(vector<SigParameter> &sigs, uint32_t &regIndex, cons
 
     if(s.regIndex == ~0U)
       s.regIndex = regIndex++;
+    else if(arrayIdx >= 0)
+      s.regIndex += arrayIdx;
 
     if(arrayIdx >= 0)
     {
@@ -1076,6 +1101,10 @@ static void AddSigParameter(vector<SigParameter> &sigs, uint32_t &regIndex, cons
 
       if(s.regIndex == ~0U)
         s.regIndex = regIndex++;
+      else if(arrayIdx >= 0)
+        s.regIndex += rows * arrayIdx + r;
+      else
+        s.regIndex += r;
 
       if(arrayIdx >= 0)
       {
@@ -1095,6 +1124,11 @@ static void AddSigParameter(vector<SigParameter> &sigs, uint32_t &regIndex, cons
 void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &refl,
                           const FixedFunctionVertexOutputs &outputUsage)
 {
+  refl.stage = MakeShaderStage(shadType);
+  refl.entryPoint = "main";
+  refl.encoding = ShaderEncoding::GLSL;
+  refl.debugInfo.encoding = ShaderEncoding::GLSL;
+
   if(shadType == eGL_COMPUTE_SHADER)
   {
     GL.glGetProgramiv(sepProg, eGL_COMPUTE_WORK_GROUP_SIZE, (GLint *)refl.dispatchThreadsDimension);
@@ -1623,7 +1657,7 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
     GL.glGetProgramResourceName(sepProg, eGL_UNIFORM, u, values[1], NULL, namebuf);
     namebuf[values[1]] = 0;
 
-    string name = namebuf;
+    rdcstr name = namebuf;
 
     delete[] namebuf;
 
@@ -1640,7 +1674,7 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
       name = name.substr(0, name.length() - 3);    // trim off [0] on the end
       for(int i = 1; i < values[4]; i++)
       {
-        string arrname = StringFormat::Fmt("%s[%d]", name.c_str(), i);
+        rdcstr arrname = StringFormat::Fmt("%s[%d]", name.c_str(), i);
 
         res.bindPoint = (int32_t)reslist.size();
         res.name = arrname;
@@ -1650,7 +1684,7 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
     }
   }
 
-  vector<int32_t> ssbos;
+  rdcarray<int32_t> ssbos;
   uint32_t ssboMembers = 0;
 
   GLint numSSBOs = 0;
@@ -1688,9 +1722,27 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
       GL.glGetProgramResourceiv(sepProg, eGL_SHADER_STORAGE_BLOCK, u, 1, &propName, 1, NULL,
                                 (GLint *)&numMembers);
 
+      char *arr = strchr(nm, '[');
+      const bool isArray = (arr != NULL);
+
+      uint32_t arrayIdx = 0;
+      if(isArray)
+      {
+        arr++;
+        while(*arr >= '0' && *arr <= '9')
+        {
+          arrayIdx *= 10;
+          arrayIdx += int(*arr) - int('0');
+          arr++;
+        }
+      }
+
       rwresources.push_back(res);
       ssbos.push_back(res.bindPoint);
-      ssboMembers += numMembers;
+
+      // only count members from the first array index
+      if(!isArray || arrayIdx == 0)
+        ssboMembers += numMembers;
 
       delete[] nm;
     }
@@ -1704,11 +1756,49 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
       ReconstructVarTree(eGL_BUFFER_VARIABLE, sepProg, i, (GLint)ssbos.size(), members, NULL);
     }
 
+    // if we have members in an array ssbo, broadcast this to any ssbo with the same basename
+    // without members, since the variables will only be reported as belonging to one block
+    for(size_t ssbo = 0; ssbo < ssbos.size(); ssbo++)
+    {
+      rdcstr basename = rwresources[ssbos[ssbo]].name;
+      int arrOffs = basename.indexOf('[');
+      if(arrOffs > 0)
+      {
+        basename.erase(arrOffs, basename.size());
+
+        if(!members[ssbo].empty())
+        {
+          for(size_t ssbo2 = 0; ssbo2 < ssbos.size(); ssbo2++)
+          {
+            if(!members[ssbo2].empty())
+              continue;
+
+            rdcstr basename2 = rwresources[ssbos[ssbo2]].name;
+            arrOffs = basename2.indexOf('[');
+            if(arrOffs > 0)
+            {
+              basename2.erase(arrOffs, basename2.size());
+
+              if(basename == basename2)
+              {
+                members[ssbo2] = members[ssbo];
+              }
+            }
+          }
+        }
+      }
+    }
+
     for(size_t ssbo = 0; ssbo < ssbos.size(); ssbo++)
     {
       sort(members[ssbo]);
 
-      if(rwresources[ssbos[ssbo]].name == members[ssbo][0].name)
+      rdcstr basename = rwresources[ssbos[ssbo]].name;
+      int arrOffs = basename.indexOf('[');
+      if(arrOffs > 0)
+        basename.erase(arrOffs, basename.size());
+
+      if(!members[ssbo].empty() && basename == members[ssbo][0].name)
         std::swap(rwresources[ssbos[ssbo]].variableType.members, members[ssbo][0].type.members);
       else
         std::swap(rwresources[ssbos[ssbo]].variableType.members, members[ssbo]);
@@ -1718,7 +1808,7 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
     // number of elements, and make all child byteOffset values relative to their parent
     for(size_t ssbo = 0; ssbo < ssbos.size(); ssbo++)
     {
-      rdcarray<ShaderConstant> &ssboVars = rwresources[ssbo].variableType.members;
+      rdcarray<ShaderConstant> &ssboVars = rwresources[ssbos[ssbo]].variableType.members;
       for(size_t rootMember = 0; rootMember + 1 < ssboVars.size(); rootMember++)
       {
         ShaderConstant &member = ssboVars[rootMember];
@@ -1742,7 +1832,7 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
   rdcarray<ShaderConstant> globalUniforms;
 
   GLint numUBOs = 0;
-  vector<string> uboNames;
+  rdcarray<rdcstr> uboNames;
   rdcarray<ShaderConstant> *ubos = NULL;
 
   {
@@ -1787,6 +1877,10 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
                                   (GLint *)&cblock.byteSize);
 
         sort(ubos[i]);
+
+        for(ShaderConstant &member : ubos[i])
+          MakeChildByteOffsetsRelative(member);
+
         std::swap(cblock.variables, ubos[i]);
 
         refl.constantBlocks.push_back(cblock);
@@ -1821,7 +1915,7 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
 
     if(numInputs > 0)
     {
-      vector<SigParameter> sigs;
+      rdcarray<SigParameter> sigs;
       sigs.reserve(numInputs);
 
       uint32_t regIndex = 0;
@@ -2010,7 +2104,7 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
         {
           // we consider an output used if we encounter a '=' before either a ';' or the end of the
           // string
-          std::string outName = ToStr(ffoutput);
+          rdcstr outName = ToStr(ffoutput);
 
           // we do a substring search so that gl_ClipDistance matches gl_ClipDistance[0]
           if(strstr(varname, outName.c_str()))
@@ -2134,11 +2228,11 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
         }
         else
         {
-          std::string basename = nm;
+          rdcstr basename = nm;
           if(basename[basename.size() - 3] == '[' && basename[basename.size() - 2] == '0' &&
              basename[basename.size() - 1] == ']')
           {
-            basename.erase(basename.size() - 3);
+            basename.resize(basename.size() - 3);
             for(int a = 0; a < values[3]; a++)
               AddSigParameter(sigs, regIndex, sig, basename.c_str(), rows, a);
           }
@@ -2156,7 +2250,18 @@ void MakeShaderReflection(GLenum shadType, GLuint sepProg, ShaderReflection &ref
         bool operator()(const SigParameter &a, const SigParameter &b)
         {
           if(a.systemValue == b.systemValue)
-            return a.regIndex < b.regIndex;
+          {
+            if(a.regIndex != b.regIndex)
+              return a.regIndex < b.regIndex;
+
+            return a.varName < b.varName;
+          }
+
+          if(a.systemValue == ShaderBuiltin::Undefined)
+            return false;
+          if(b.systemValue == ShaderBuiltin::Undefined)
+            return true;
+
           return a.systemValue < b.systemValue;
         }
       };
@@ -2210,7 +2315,7 @@ void GetBindpointMapping(GLuint curProg, int shadIdx, const ShaderReflection *re
       }
 
       // handle sampler arrays, use the base name
-      std::string name = refl->readOnlyResources[i].name.c_str();
+      rdcstr name = refl->readOnlyResources[i].name.c_str();
       if(name.back() == ']')
       {
         do
@@ -2260,7 +2365,7 @@ void GetBindpointMapping(GLuint curProg, int shadIdx, const ShaderReflection *re
       }
 
       // handle sampler arrays, use the base name
-      std::string name = refl->readWriteResources[i].name.c_str();
+      rdcstr name = refl->readWriteResources[i].name.c_str();
       if(name.back() == ']')
       {
         do
@@ -2448,13 +2553,13 @@ void GetBindpointMapping(GLuint curProg, int shadIdx, const ShaderReflection *re
         continue;
 
       int32_t matrixRow = 0;
-      std::string varName = refl->inputSignature[i].varName;
+      rdcstr varName = refl->inputSignature[i].varName;
 
-      size_t offs = varName.find(":row");
-      if(offs != std::string::npos)
+      int32_t offs = varName.find(":row");
+      if(offs >= 0)
       {
         matrixRow = varName[offs + 4] - '0';
-        varName.erase(offs);
+        varName.resize(offs);
       }
 
       GLint loc = GL.glGetAttribLocation(curProg, varName.c_str());
@@ -2571,6 +2676,23 @@ void EvaluateSPIRVBindpointMapping(GLuint curProg, int shadIdx, const ShaderRefl
     }
   }
 
+  for(size_t i = 0; i < mapping.inputAttributes.size(); i++)
+    if(mapping.inputAttributes[i] < 0)
+      mapping.inputAttributes[i] = -1;
+
+  // ensure the mapping contains a value for all valid vertex attribs, even if it doesn't use them.
+  GLint numVAttribBindings = 16;
+  GL.glGetIntegerv(eGL_MAX_VERTEX_ATTRIBS, &numVAttribBindings);
+
+  // if more vertex attribs exist, resize and set them all to -1
+  if(numVAttribBindings > mapping.inputAttributes.count())
+  {
+    size_t prevSize = mapping.inputAttributes.size();
+    mapping.inputAttributes.resize(numVAttribBindings);
+    for(size_t i = prevSize; i < mapping.inputAttributes.size(); i++)
+      mapping.inputAttributes[i] = -1;
+  }
+
 #if ENABLED(RDOC_DEVEL)
   for(size_t i = 1; i < ARRAY_COUNT(dummyReadback); i++)
     if(dummyReadback[i] != 0x6c7b8a9d)
@@ -2579,7 +2701,7 @@ void EvaluateSPIRVBindpointMapping(GLuint curProg, int shadIdx, const ShaderRefl
 }
 
 // first int - the mapping index, second int - the binding
-typedef std::vector<std::pair<size_t, int> > Permutation;
+typedef rdcarray<rdcpair<size_t, int> > Permutation;
 
 // copy permutation by value since we mutate it to track the algorithm
 static void ApplyPermutation(Permutation permutation, std::function<void(size_t, size_t)> DoSwap)
@@ -2679,7 +2801,7 @@ void ResortBindings(ShaderReflection *refl, ShaderBindpointMapping *mapping)
   // sort by the binding
   struct permutation_sort
   {
-    bool operator()(const std::pair<size_t, int> &a, const std::pair<size_t, int> &b) const
+    bool operator()(const rdcpair<size_t, int> &a, const rdcpair<size_t, int> &b) const
     {
       return a.second < b.second;
     }
@@ -2687,7 +2809,7 @@ void ResortBindings(ShaderReflection *refl, ShaderBindpointMapping *mapping)
 
   permutation.resize(mapping->readOnlyResources.size());
   for(size_t i = 0; i < mapping->readOnlyResources.size(); i++)
-    permutation[i] = std::make_pair(i, mapping->readOnlyResources[i].bind);
+    permutation[i] = make_rdcpair(i, mapping->readOnlyResources[i].bind);
 
   std::sort(permutation.begin(), permutation.end(), permutation_sort());
 
@@ -2702,7 +2824,7 @@ void ResortBindings(ShaderReflection *refl, ShaderBindpointMapping *mapping)
 
   permutation.resize(mapping->readWriteResources.size());
   for(size_t i = 0; i < mapping->readWriteResources.size(); i++)
-    permutation[i] = std::make_pair(i, mapping->readWriteResources[i].bind);
+    permutation[i] = make_rdcpair(i, mapping->readWriteResources[i].bind);
 
   std::sort(permutation.begin(), permutation.end(), permutation_sort());
 
@@ -2715,7 +2837,7 @@ void ResortBindings(ShaderReflection *refl, ShaderBindpointMapping *mapping)
 
   permutation.resize(mapping->constantBlocks.size());
   for(size_t i = 0; i < mapping->constantBlocks.size(); i++)
-    permutation[i] = std::make_pair(i, mapping->constantBlocks[i].bind);
+    permutation[i] = make_rdcpair(i, mapping->constantBlocks[i].bind);
 
   std::sort(permutation.begin(), permutation.end(), permutation_sort());
 
@@ -2725,4 +2847,4 @@ void ResortBindings(ShaderReflection *refl, ShaderBindpointMapping *mapping)
 
   for(size_t i = 0; i < permutation.size(); i++)
     refl->constantBlocks[permutation[i].first].bindPoint = (int)i;
-}
\ No newline at end of file
+}
