@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,13 +23,13 @@
  ******************************************************************************/
 
 #include <dlfcn.h>
-#include "driver/gl/gl_driver.h"
-#include "driver/gl/glx_dispatch_table.h"
 #include "hooks/hooks.h"
+#include "gl_driver.h"
+#include "glx_dispatch_table.h"
 
 namespace Keyboard
 {
-void CloneDisplay(Display *dpy);
+void UseXlibDisplay(Display *dpy);
 }
 
 class GLXHook : LibraryHook
@@ -38,47 +38,14 @@ public:
   GLXHook() : driver(GetGLPlatform()) {}
   void RegisterHooks();
 
-  XID UnwrapGLXWindow(XID id)
-  {
-    // if it's a GLXWindow
-    auto it = m_GLXWindowMap.find(id);
-
-    if(it != m_GLXWindowMap.end())
-    {
-      // return the drawable used at creation time
-      return it->second;
-    }
-
-    // otherwise just use the id as-is
-    return id;
-  }
-
-  void AddGLXWindow(GLXWindow glx, Window win) { m_GLXWindowMap[glx] = win; }
-  void RemoveGLXWindow(GLXWindow glx)
-  {
-    auto it = m_GLXWindowMap.find(glx);
-
-    if(it != m_GLXWindowMap.end())
-      m_GLXWindowMap.erase(it);
-  }
-
   void UpdateWindowSize(GLWindowingData data, Display *dpy, GLXDrawable drawable)
   {
     if(!data.ctx || !drawable)
       return;
 
-    // if we use the GLXDrawable in XGetGeometry and it's a GLXWindow, then we get
-    // a BadDrawable error and things go south. Instead we track GLXWindows created
-    // in glXCreateWindow/glXDestroyWindow and look up the source window it was
-    // created from to use that.
-    // If the drawable didn't come through there, it just passes through unscathed
-    // through this function
-    Drawable d = UnwrapGLXWindow(drawable);
-
-    Window root;
-    int x, y;
-    unsigned int width, height, border_width, depth;
-    XGetGeometry(dpy, d, &root, &x, &y, &width, &height, &border_width, &depth);
+    unsigned int width = 1, height = 1;
+    GLX.glXQueryDrawable(dpy, drawable, GLX_WIDTH, &width);
+    GLX.glXQueryDrawable(dpy, drawable, GLX_HEIGHT, &height);
 
     GLInitParams &params = driver.GetInitParams(data);
     params.width = width;
@@ -89,10 +56,31 @@ public:
   void *handle = RTLD_NEXT;
   WrappedOpenGL driver;
   std::set<GLXContext> contexts;
-
-  std::map<XID, XID> m_GLXWindowMap;
 } glxhook;
 
+// On linux if a user doesn't link to libGL/libGLX or try to dlopen it, but just calls dlsym with
+// RTLD_NEXT it might successfully one of our functions without anything ever loading libEGL. Then
+// our attempts to call onwards will fail. When any of our functions are called we check to see if
+// DEFAULT_HANDLE is RTLD_NEXT and if so we manually load the library. This will trigger our hook
+// callback and we'll get a specific library handle.
+static void EnsureRealLibraryLoaded()
+{
+  if(glxhook.handle == RTLD_NEXT)
+  {
+    if(!RenderDoc::Inst().IsReplayApp())
+      RDCLOG("Loading libGL at the last second");
+
+    void *handle = Process::LoadModule("libGL.so.1");
+    if(!handle)
+      handle = Process::LoadModule("libGL.so");
+    if(!handle)
+      handle = Process::LoadModule("libGLX.so.0");
+
+    if(RenderDoc::Inst().IsReplayApp())
+      glxhook.handle = handle;
+  }
+}
+
 HOOK_EXPORT GLXContext glXCreateContext_renderdoc_hooked(Display *dpy, XVisualInfo *vis,
                                                          GLXContext shareList, Bool direct)
 {
@@ -104,6 +92,8 @@ HOOK_EXPORT GLXContext glXCreateContext_renderdoc_hooked(Display *dpy, XVisualIn
     return GLX.glXCreateContext(dpy, vis, shareList, direct);
   }
 
+  EnsureRealLibraryLoaded();
+
   GLXContext ret = GLX.glXCreateContext(dpy, vis, shareList, direct);
 
   // don't continue if context creation failed
@@ -117,7 +107,7 @@ HOOK_EXPORT GLXContext glXCreateContext_renderdoc_hooked(Display *dpy, XVisualIn
 
   int value = 0;
 
-  Keyboard::CloneDisplay(dpy);
+  Keyboard::UseXlibDisplay(dpy);
 
   GLX.glXGetConfig(dpy, vis, GLX_BUFFER_SIZE, &value);
   init.colorBits = value;
@@ -156,6 +146,8 @@ HOOK_EXPORT void glXDestroyContext_renderdoc_hooked(Display *dpy, GLXContext ctx
     return GLX.glXDestroyContext(dpy, ctx);
   }
 
+  EnsureRealLibraryLoaded();
+
   {
     SCOPED_LOCK(glLock);
     glxhook.driver.DeleteContext(ctx);
@@ -177,10 +169,12 @@ HOOK_EXPORT GLXContext glXCreateContextAttribsARB_renderdoc_hooked(Display *dpy,
     return GLX.glXCreateContextAttribsARB(dpy, config, shareList, direct, attribList);
   }
 
+  EnsureRealLibraryLoaded();
+
   int defaultAttribList[] = {0};
 
   const int *attribs = attribList ? attribList : defaultAttribList;
-  vector<int> attribVec;
+  rdcarray<int> attribVec;
 
   // modify attribs to our liking
   {
@@ -258,7 +252,7 @@ HOOK_EXPORT GLXContext glXCreateContextAttribsARB_renderdoc_hooked(Display *dpy,
 
   int value = 0;
 
-  Keyboard::CloneDisplay(dpy);
+  Keyboard::UseXlibDisplay(dpy);
 
   GLX.glXGetConfig(dpy, vis, GLX_BUFFER_SIZE, &value);
   init.colorBits = value;
@@ -299,6 +293,8 @@ HOOK_EXPORT Bool glXMakeCurrent_renderdoc_hooked(Display *dpy, GLXDrawable drawa
     return GLX.glXMakeCurrent(dpy, drawable, ctx);
   }
 
+  EnsureRealLibraryLoaded();
+
   Bool ret = GLX.glXMakeCurrent(dpy, drawable, ctx);
 
   if(ret)
@@ -367,6 +363,8 @@ HOOK_EXPORT Bool glXMakeContextCurrent_renderdoc_hooked(Display *dpy, GLXDrawabl
     return GLX.glXMakeContextCurrent(dpy, draw, read, ctx);
   }
 
+  EnsureRealLibraryLoaded();
+
   Bool ret = GLX.glXMakeContextCurrent(dpy, draw, read, ctx);
 
   if(ret)
@@ -433,6 +431,8 @@ HOOK_EXPORT void glXSwapBuffers_renderdoc_hooked(Display *dpy, GLXDrawable drawa
     return GLX.glXSwapBuffers(dpy, drawable);
   }
 
+  EnsureRealLibraryLoaded();
+
   SCOPED_LOCK(glLock);
 
   {
@@ -445,50 +445,13 @@ HOOK_EXPORT void glXSwapBuffers_renderdoc_hooked(Display *dpy, GLXDrawable drawa
     glxhook.UpdateWindowSize(data, dpy, drawable);
   }
 
-  glxhook.driver.SwapBuffers((void *)drawable);
+  gl_CurChunk = GLChunk::glXSwapBuffers;
+
+  glxhook.driver.SwapBuffers(WindowingSystem::Xlib, (void *)drawable);
 
   GLX.glXSwapBuffers(dpy, drawable);
 }
 
-HOOK_EXPORT GLXWindow glXCreateWindow_renderdoc_hooked(Display *dpy, GLXFBConfig config, Window win,
-                                                       const int *attribList)
-{
-  if(RenderDoc::Inst().IsReplayApp())
-  {
-    if(!GLX.glXCreateWindow)
-      GLX.PopulateForReplay();
-
-    return GLX.glXCreateWindow(dpy, config, win, attribList);
-  }
-
-  GLXWindow ret = GLX.glXCreateWindow(dpy, config, win, attribList);
-
-  {
-    SCOPED_LOCK(glLock);
-    glxhook.AddGLXWindow(ret, win);
-  }
-
-  return ret;
-}
-
-HOOK_EXPORT void glXDestroyWindow_renderdoc_hooked(Display *dpy, GLXWindow window)
-{
-  if(RenderDoc::Inst().IsReplayApp())
-  {
-    if(!GLX.glXDestroyWindow)
-      GLX.PopulateForReplay();
-
-    return GLX.glXDestroyWindow(dpy, window);
-  }
-
-  {
-    SCOPED_LOCK(glLock);
-    glxhook.RemoveGLXWindow(window);
-  }
-
-  return GLX.glXDestroyWindow(dpy, window);
-}
-
 HOOK_EXPORT __GLXextFuncPtr glXGetProcAddress_renderdoc_hooked(const GLubyte *f)
 {
   if(RenderDoc::Inst().IsReplayApp())
@@ -499,6 +462,8 @@ HOOK_EXPORT __GLXextFuncPtr glXGetProcAddress_renderdoc_hooked(const GLubyte *f)
     return GLX.glXGetProcAddress(f);
   }
 
+  EnsureRealLibraryLoaded();
+
   const char *func = (const char *)f;
 
   __GLXextFuncPtr realFunc = NULL;
@@ -525,10 +490,6 @@ HOOK_EXPORT __GLXextFuncPtr glXGetProcAddress_renderdoc_hooked(const GLubyte *f)
     return (__GLXextFuncPtr)&glXMakeContextCurrent_renderdoc_hooked;
   if(!strcmp(func, "glXSwapBuffers"))
     return (__GLXextFuncPtr)&glXSwapBuffers_renderdoc_hooked;
-  if(!strcmp(func, "glXCreateWindow"))
-    return (__GLXextFuncPtr)&glXCreateWindow_renderdoc_hooked;
-  if(!strcmp(func, "glXDestroyWindow"))
-    return (__GLXextFuncPtr)&glXDestroyWindow_renderdoc_hooked;
   if(!strcmp(func, "glXGetProcAddress"))
     return (__GLXextFuncPtr)&glXGetProcAddress_renderdoc_hooked;
   if(!strcmp(func, "glXGetProcAddressARB"))
@@ -587,17 +548,6 @@ HOOK_EXPORT void glXSwapBuffers(Display *dpy, GLXDrawable drawable)
   return glXSwapBuffers_renderdoc_hooked(dpy, drawable);
 }
 
-HOOK_EXPORT GLXWindow glXCreateWindow(Display *dpy, GLXFBConfig config, Window win,
-                                      const int *attribList)
-{
-  return glXCreateWindow_renderdoc_hooked(dpy, config, win, attribList);
-}
-
-HOOK_EXPORT void glXDestroyWindow(Display *dpy, GLXWindow window)
-{
-  return glXDestroyWindow_renderdoc_hooked(dpy, window);
-}
-
 HOOK_EXPORT __GLXextFuncPtr glXGetProcAddress(const GLubyte *f)
 {
   return glXGetProcAddress_renderdoc_hooked(f);
@@ -616,6 +566,7 @@ HOOK_EXPORT __GLXextFuncPtr glXGetProcAddressARB(const GLubyte *f)
   typedef ret (*CONCAT(function, _hooktype))();                                     \
   extern "C" __attribute__((visibility("default"))) ret function()                  \
   {                                                                                 \
+    EnsureRealLibraryLoaded();                                                      \
     CONCAT(function, _hooktype)                                                     \
     real = (CONCAT(function, _hooktype))dlsym(glxhook.handle, STRINGIZE(function)); \
     return real();                                                                  \
@@ -625,6 +576,7 @@ HOOK_EXPORT __GLXextFuncPtr glXGetProcAddressARB(const GLubyte *f)
   typedef ret (*CONCAT(function, _hooktype))(t1);                                   \
   extern "C" __attribute__((visibility("default"))) ret function(t1 p1)             \
   {                                                                                 \
+    EnsureRealLibraryLoaded();                                                      \
     CONCAT(function, _hooktype)                                                     \
     real = (CONCAT(function, _hooktype))dlsym(glxhook.handle, STRINGIZE(function)); \
     return real(p1);                                                                \
@@ -634,6 +586,7 @@ HOOK_EXPORT __GLXextFuncPtr glXGetProcAddressARB(const GLubyte *f)
   typedef ret (*CONCAT(function, _hooktype))(t1, t2);                               \
   extern "C" __attribute__((visibility("default"))) ret function(t1 p1, t2 p2)      \
   {                                                                                 \
+    EnsureRealLibraryLoaded();                                                      \
     CONCAT(function, _hooktype)                                                     \
     real = (CONCAT(function, _hooktype))dlsym(glxhook.handle, STRINGIZE(function)); \
     return real(p1, p2);                                                            \
@@ -643,6 +596,7 @@ HOOK_EXPORT __GLXextFuncPtr glXGetProcAddressARB(const GLubyte *f)
   typedef ret (*CONCAT(function, _hooktype))(t1, t2, t3);                             \
   extern "C" __attribute__((visibility("default"))) ret function(t1 p1, t2 p2, t3 p3) \
   {                                                                                   \
+    EnsureRealLibraryLoaded();                                                        \
     CONCAT(function, _hooktype)                                                       \
     real = (CONCAT(function, _hooktype))dlsym(glxhook.handle, STRINGIZE(function));   \
     return real(p1, p2, p3);                                                          \
@@ -652,6 +606,7 @@ HOOK_EXPORT __GLXextFuncPtr glXGetProcAddressARB(const GLubyte *f)
   typedef ret (*CONCAT(function, _hooktype))(t1, t2, t3, t4);                                \
   extern "C" __attribute__((visibility("default"))) ret function(t1 p1, t2 p2, t3 p3, t4 p4) \
   {                                                                                          \
+    EnsureRealLibraryLoaded();                                                               \
     CONCAT(function, _hooktype)                                                              \
     real = (CONCAT(function, _hooktype))dlsym(glxhook.handle, STRINGIZE(function));          \
     return real(p1, p2, p3, p4);                                                             \
@@ -661,6 +616,7 @@ HOOK_EXPORT __GLXextFuncPtr glXGetProcAddressARB(const GLubyte *f)
   typedef ret (*CONCAT(function, _hooktype))(t1, t2, t3, t4, t5);                                   \
   extern "C" __attribute__((visibility("default"))) ret function(t1 p1, t2 p2, t3 p3, t4 p4, t5 p5) \
   {                                                                                                 \
+    EnsureRealLibraryLoaded();                                                                      \
     CONCAT(function, _hooktype)                                                                     \
     real = (CONCAT(function, _hooktype))dlsym(glxhook.handle, STRINGIZE(function));                 \
     return real(p1, p2, p3, p4, p5);                                                                \
@@ -706,6 +662,9 @@ GLX_PASSTHRU_2(void, glXDestroyPixmap, Display *, dpy, GLXPixmap, pixmap);
 GLX_PASSTHRU_3(GLXPbuffer, glXCreatePbuffer, Display *, dpy, GLXFBConfig, config, const int *,
                attrib_list);
 GLX_PASSTHRU_2(void, glXDestroyPbuffer, Display *, dpy, GLXPbuffer, pbuf);
+GLX_PASSTHRU_4(GLXWindow, glXCreateWindow, Display *, dpy, GLXFBConfig, config, Window, window,
+               const int *, attrib_list);
+GLX_PASSTHRU_2(void, glXDestroyWindow, Display *, dpy, GLXWindow, window);
 
 static void GLXHooked(void *handle)
 {
