@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,8 +22,8 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "driver/gl/gl_common.h"
-#include "driver/gl/wgl_dispatch_table.h"
+#include "gl_common.h"
+#include "wgl_dispatch_table.h"
 
 #define WINDOW_CLASS_NAME L"renderdocGLclass"
 
@@ -107,12 +107,15 @@ class WGLPlatform : public GLPlatform
     if(!WGL.wglGetPixelFormatAttribivARB || !WGL.wglCreateContextAttribsARB)
       return ret;
 
-    RDCASSERT(window.system == WindowingSystem::Win32 || window.system == WindowingSystem::Unknown,
+    RDCASSERT(window.system == WindowingSystem::Win32 || window.system == WindowingSystem::Unknown ||
+                  window.system == WindowingSystem::Headless,
               window.system);
 
-    HWND w = window.win32.window;
+    HWND w = NULL;
 
-    if(w == NULL)
+    if(window.system == WindowingSystem::Win32)
+      w = window.win32.window;
+    else
       w = CreateWindowEx(WS_EX_CLIENTEDGE, WINDOW_CLASS_NAME, L"", WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL,
                          GetModuleHandle(NULL), NULL);
@@ -242,6 +245,7 @@ class WGLPlatform : public GLPlatform
     return Process::GetFunctionAddress(Process::LoadModule("opengl32.dll"), funcname);
   }
 
+  bool CanCreateGLContext() { return true; }
   bool CanCreateGLESContext()
   {
     bool success = WGL.PopulateForReplay();
@@ -288,8 +292,13 @@ class WGLPlatform : public GLPlatform
   }
 
   bool PopulateForReplay() { return WGL.PopulateForReplay(); }
-  ReplayStatus InitialiseAPI(GLWindowingData &replayContext, RDCDriver api)
+  ReplayStatus InitialiseAPI(GLWindowingData &replayContext, RDCDriver api, bool debug)
   {
+// force debug in development builds
+#if ENABLED(RDOC_DEVEL)
+    debug = true;
+#endif
+
     RDCASSERT(api == RDCDriver::OpenGL || api == RDCDriver::OpenGLES);
 
     bool success = RegisterClass();
@@ -367,18 +376,14 @@ class WGLPlatform : public GLPlatform
     int &minor = attribs[i];
     attribs[i++] = 0;
     attribs[i++] = WGL_CONTEXT_FLAGS_ARB;
-#if ENABLED(RDOC_DEVEL)
-    attribs[i++] = WGL_CONTEXT_DEBUG_BIT_ARB;
-#else
-    attribs[i++] = 0;
-#endif
+    attribs[i++] = debug ? WGL_CONTEXT_DEBUG_BIT_ARB : 0;
     attribs[i++] = WGL_CONTEXT_PROFILE_MASK_ARB;
     attribs[i++] = api == RDCDriver::OpenGLES ? WGL_CONTEXT_ES2_PROFILE_BIT_EXT
                                               : WGL_CONTEXT_CORE_PROFILE_BIT_ARB;
 
     rc = NULL;
 
-    std::vector<GLVersion> versions = GetReplayVersions(api);
+    rdcarray<GLVersion> versions = GetReplayVersions(api);
 
     for(GLVersion v : versions)
     {
@@ -516,7 +521,7 @@ class WGLPlatform : public GLPlatform
     return true;
   }
 
-  void DrawQuads(float width, float height, const std::vector<Vec4f> &vertices)
+  void DrawQuads(float width, float height, const rdcarray<Vec4f> &vertices)
   {
     ::DrawQuads(WGL, width, height, vertices);
   }
