@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -493,7 +493,7 @@ bool WrappedOpenGL::Serialise_wglDXLockObjectsNV(SerialiserType &ser, GLResource
         }
 
         // serialise without allocating memory as we already have our scratch buf sized.
-        ser.Serialise("SubresourceContents", scratchBuf, size, SerialiserFlags::NoFlags);
+        ser.Serialise("SubresourceContents"_lit, scratchBuf, size, SerialiserFlags::NoFlags);
 
         if(IsReplayingAndReading() && !ser.IsErrored())
         {
@@ -535,7 +535,7 @@ bool WrappedOpenGL::Serialise_glCreateMemoryObjectsEXT(SerialiserType &ser, GLsi
 {
   SERIALISE_ELEMENT(n);
   SERIALISE_ELEMENT_LOCAL(memory, GetResourceManager()->GetID(ExtMemRes(GetCtx(), *memoryObjects)))
-      .TypedAs("GLResource");
+      .TypedAs("GLResource"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -810,7 +810,7 @@ bool WrappedOpenGL::Serialise_glGenSemaphoresEXT(SerialiserType &ser, GLsizei n,
 {
   SERIALISE_ELEMENT(n);
   SERIALISE_ELEMENT_LOCAL(semaphore, GetResourceManager()->GetID(ExtSemRes(GetCtx(), *semaphores)))
-      .TypedAs("GLResource");
+      .TypedAs("GLResource"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1084,7 +1084,7 @@ bool WrappedOpenGL::Serialise_glWaitSemaphoreEXT(SerialiserType &ser, GLuint sem
                                                  const GLenum *srcLayouts)
 {
   // can't serialise arrays of GL handles since they're not wrapped or typed :(.
-  std::vector<GLResource> buffers, textures;
+  rdcarray<GLResource> buffers, textures;
 
   if(ser.IsWriting())
   {
@@ -1153,7 +1153,7 @@ bool WrappedOpenGL::Serialise_glSignalSemaphoreEXT(SerialiserType &ser, GLuint s
                                                    const GLenum *dstLayouts)
 {
   // can't serialise arrays of GL handles since they're not wrapped or typed :(.
-  std::vector<GLResource> buffers, textures;
+  rdcarray<GLResource> buffers, textures;
 
   if(ser.IsWriting())
   {
@@ -1478,9 +1478,9 @@ void WrappedOpenGL::glTexStorageMem1DEXT(GLenum target, GLsizei levels, GLenum i
 {
   SERIALISE_TIME_CALL(GL.glTexStorageMem1DEXT(target, levels, internalFormat, width, memory, offset));
 
-  if(IsCaptureMode(m_State))
+  if(IsCaptureMode(m_State) && !IsProxyTarget(target))
   {
-    GLResourceRecord *record = GetCtxData().GetActiveTexRecord();
+    GLResourceRecord *record = GetCtxData().GetActiveTexRecord(target);
 
     if(!record)
     {
@@ -1592,9 +1592,9 @@ void WrappedOpenGL::glTexStorageMem2DEXT(GLenum target, GLsizei levels, GLenum i
   SERIALISE_TIME_CALL(
       GL.glTexStorageMem2DEXT(target, levels, internalFormat, width, height, memory, offset));
 
-  if(IsCaptureMode(m_State))
+  if(IsCaptureMode(m_State) && !IsProxyTarget(target))
   {
-    GLResourceRecord *record = GetCtxData().GetActiveTexRecord();
+    GLResourceRecord *record = GetCtxData().GetActiveTexRecord(target);
 
     if(!record)
     {
@@ -1715,9 +1715,9 @@ void WrappedOpenGL::glTexStorageMem2DMultisampleEXT(GLenum target, GLsizei sampl
   SERIALISE_TIME_CALL(GL.glTexStorageMem2DMultisampleEXT(
       target, samples, internalFormat, width, height, fixedSampleLocations, memory, offset));
 
-  if(IsCaptureMode(m_State))
+  if(IsCaptureMode(m_State) && !IsProxyTarget(target))
   {
-    GLResourceRecord *record = GetCtxData().GetActiveTexRecord();
+    GLResourceRecord *record = GetCtxData().GetActiveTexRecord(target);
 
     if(!record)
     {
@@ -1830,9 +1830,9 @@ void WrappedOpenGL::glTexStorageMem3DEXT(GLenum target, GLsizei levels, GLenum i
   SERIALISE_TIME_CALL(GL.glTexStorageMem3DEXT(target, levels, internalFormat, width, height, depth,
                                               memory, offset));
 
-  if(IsCaptureMode(m_State))
+  if(IsCaptureMode(m_State) && !IsProxyTarget(target))
   {
-    GLResourceRecord *record = GetCtxData().GetActiveTexRecord();
+    GLResourceRecord *record = GetCtxData().GetActiveTexRecord(target);
 
     if(!record)
     {
@@ -1951,9 +1951,9 @@ void WrappedOpenGL::glTexStorageMem3DMultisampleEXT(GLenum target, GLsizei sampl
   SERIALISE_TIME_CALL(GL.glTexStorageMem3DMultisampleEXT(
       target, samples, internalFormat, width, height, depth, fixedSampleLocations, memory, offset));
 
-  if(IsCaptureMode(m_State))
+  if(IsCaptureMode(m_State) && !IsProxyTarget(target))
   {
-    GLResourceRecord *record = GetCtxData().GetActiveTexRecord();
+    GLResourceRecord *record = GetCtxData().GetActiveTexRecord(target);
 
     if(!record)
     {
@@ -2032,4 +2032,4 @@ INSTANTIATE_FUNCTION_SERIALISED(void, glImportSemaphoreWin32NameEXT, GLuint sema
                                 GLenum handleType, const void *name);
 INSTANTIATE_FUNCTION_SERIALISED(GLboolean, glAcquireKeyedMutexWin32EXT, GLuint memory, GLuint64 key,
                                 GLuint timeout);
-INSTANTIATE_FUNCTION_SERIALISED(GLboolean, glReleaseKeyedMutexWin32EXT, GLuint memory, GLuint64 key);
\ No newline at end of file
+INSTANTIATE_FUNCTION_SERIALISED(GLboolean, glReleaseKeyedMutexWin32EXT, GLuint memory, GLuint64 key);
