@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -37,7 +37,7 @@ bool WrappedOpenGL::Serialise_glGenSamplers(SerialiserType &ser, GLsizei n, GLui
 {
   SERIALISE_ELEMENT(n);
   SERIALISE_ELEMENT_LOCAL(sampler, GetResourceManager()->GetID(SamplerRes(GetCtx(), *samplers)))
-      .TypedAs("GLResource");
+      .TypedAs("GLResource"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -97,7 +97,7 @@ bool WrappedOpenGL::Serialise_glCreateSamplers(SerialiserType &ser, GLsizei n, G
 {
   SERIALISE_ELEMENT(n);
   SERIALISE_ELEMENT_LOCAL(sampler, GetResourceManager()->GetID(SamplerRes(GetCtx(), *samplers)))
-      .TypedAs("GLResource");
+      .TypedAs("GLResource"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -184,7 +184,7 @@ bool WrappedOpenGL::Serialise_glBindSamplers(SerialiserType &ser, GLuint first,
                                              const GLuint *samplerHandles)
 {
   // can't serialise arrays of GL handles since they're not wrapped or typed :(.
-  std::vector<GLResource> samplers;
+  rdcarray<GLResource> samplers;
 
   if(ser.IsWriting())
   {
@@ -201,7 +201,7 @@ bool WrappedOpenGL::Serialise_glBindSamplers(SerialiserType &ser, GLuint first,
 
   if(IsReplayingAndReading())
   {
-    std::vector<GLuint> samps;
+    rdcarray<GLuint> samps;
     samps.reserve(count);
     for(int32_t i = 0; i < count; i++)
       samps.push_back(samplers[i].name);
@@ -288,6 +288,9 @@ void WrappedOpenGL::glSamplerParameteri(GLuint sampler, GLenum pname, GLint para
       record->AddChunk(scope.Get());
       record->UpdateCount++;
 
+      GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
+                                                        eFrameRef_ReadBeforeWrite);
+
       if(record->UpdateCount > 20)
       {
         m_HighTrafficResources.insert(record->GetResourceID());
@@ -298,7 +301,7 @@ void WrappedOpenGL::glSamplerParameteri(GLuint sampler, GLenum pname, GLint para
     {
       GetContextRecord()->AddChunk(scope.Get());
       GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
-                                                        eFrameRef_Read);
+                                                        eFrameRef_ReadBeforeWrite);
     }
   }
 }
@@ -348,6 +351,9 @@ void WrappedOpenGL::glSamplerParameterf(GLuint sampler, GLenum pname, GLfloat pa
       record->AddChunk(scope.Get());
       record->UpdateCount++;
 
+      GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
+                                                        eFrameRef_ReadBeforeWrite);
+
       if(record->UpdateCount > 20)
       {
         m_HighTrafficResources.insert(record->GetResourceID());
@@ -358,7 +364,7 @@ void WrappedOpenGL::glSamplerParameterf(GLuint sampler, GLenum pname, GLfloat pa
     {
       GetContextRecord()->AddChunk(scope.Get());
       GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
-                                                        eFrameRef_Read);
+                                                        eFrameRef_ReadBeforeWrite);
     }
   }
 }
@@ -409,6 +415,9 @@ void WrappedOpenGL::glSamplerParameteriv(GLuint sampler, GLenum pname, const GLi
       record->AddChunk(scope.Get());
       record->UpdateCount++;
 
+      GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
+                                                        eFrameRef_ReadBeforeWrite);
+
       if(record->UpdateCount > 20)
       {
         m_HighTrafficResources.insert(record->GetResourceID());
@@ -419,7 +428,7 @@ void WrappedOpenGL::glSamplerParameteriv(GLuint sampler, GLenum pname, const GLi
     {
       GetContextRecord()->AddChunk(scope.Get());
       GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
-                                                        eFrameRef_Read);
+                                                        eFrameRef_ReadBeforeWrite);
     }
   }
 }
@@ -470,6 +479,9 @@ void WrappedOpenGL::glSamplerParameterfv(GLuint sampler, GLenum pname, const GLf
       record->AddChunk(scope.Get());
       record->UpdateCount++;
 
+      GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
+                                                        eFrameRef_ReadBeforeWrite);
+
       if(record->UpdateCount > 20)
       {
         m_HighTrafficResources.insert(record->GetResourceID());
@@ -480,7 +492,7 @@ void WrappedOpenGL::glSamplerParameterfv(GLuint sampler, GLenum pname, const GLf
     {
       GetContextRecord()->AddChunk(scope.Get());
       GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
-                                                        eFrameRef_Read);
+                                                        eFrameRef_ReadBeforeWrite);
     }
   }
 }
@@ -531,6 +543,9 @@ void WrappedOpenGL::glSamplerParameterIiv(GLuint sampler, GLenum pname, const GL
       record->AddChunk(scope.Get());
       record->UpdateCount++;
 
+      GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
+                                                        eFrameRef_ReadBeforeWrite);
+
       if(record->UpdateCount > 20)
       {
         m_HighTrafficResources.insert(record->GetResourceID());
@@ -541,7 +556,7 @@ void WrappedOpenGL::glSamplerParameterIiv(GLuint sampler, GLenum pname, const GL
     {
       GetContextRecord()->AddChunk(scope.Get());
       GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
-                                                        eFrameRef_Read);
+                                                        eFrameRef_ReadBeforeWrite);
     }
   }
 }
@@ -592,6 +607,9 @@ void WrappedOpenGL::glSamplerParameterIuiv(GLuint sampler, GLenum pname, const G
       record->AddChunk(scope.Get());
       record->UpdateCount++;
 
+      GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
+                                                        eFrameRef_ReadBeforeWrite);
+
       if(record->UpdateCount > 20)
       {
         m_HighTrafficResources.insert(record->GetResourceID());
@@ -602,7 +620,7 @@ void WrappedOpenGL::glSamplerParameterIuiv(GLuint sampler, GLenum pname, const G
     {
       GetContextRecord()->AddChunk(scope.Get());
       GetResourceManager()->MarkResourceFrameReferenced(SamplerRes(GetCtx(), sampler),
-                                                        eFrameRef_Read);
+                                                        eFrameRef_ReadBeforeWrite);
     }
   }
 }
