@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,10 +23,12 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
+#include <ctype.h>
 #include "../gl_driver.h"
 #include "../gl_shader_refl.h"
 #include "common/common.h"
-#include "driver/shaders/spirv/spirv_common.h"
+#include "driver/shaders/spirv/glslang_compile.h"
+#include "driver/shaders/spirv/spirv_compile.h"
 #include "strings/string_utils.h"
 
 enum GLshaderbitfield
@@ -36,7 +38,7 @@ enum GLshaderbitfield
 DECLARE_REFLECTION_ENUM(GLshaderbitfield);
 
 template <>
-std::string DoStringise(const GLshaderbitfield &el)
+rdcstr DoStringise(const GLshaderbitfield &el)
 {
   RDCCOMPILE_ASSERT(sizeof(GLshaderbitfield) == sizeof(GLbitfield) &&
                         sizeof(GLshaderbitfield) == sizeof(uint32_t),
@@ -61,24 +63,23 @@ void WrappedOpenGL::ShaderData::ProcessSPIRVCompilation(WrappedOpenGL &drv, Reso
                                                         const GLuint *pConstantValue)
 {
   reflection.resourceId = id;
-  reflection.entryPoint = pEntryPoint;
-  reflection.stage = MakeShaderStage(type);
-  reflection.encoding = ShaderEncoding::SPIRV;
-  reflection.rawBytes.assign((byte *)spirv.spirv.data(), spirv.spirv.size() * sizeof(uint32_t));
 
-  // we discard this too, because we don't need it - we don't do any SPIR-V patching in GL
-  SPIRVPatchData patchData;
+  rdcarray<SpecConstant> specInfo;
+  for(size_t i = 0; i < specInfo.size(); i++)
+  {
+    specInfo.push_back(SpecConstant(pConstantIndex[i], pConstantValue[i], 4));
+  }
 
-  spirv.MakeReflection(GraphicsAPI::OpenGL, ShaderStage(ShaderIdx(type)), pEntryPoint, reflection,
-                       mapping, patchData);
+  spirv.MakeReflection(GraphicsAPI::OpenGL, ShaderStage(ShaderIdx(type)), pEntryPoint, specInfo,
+                       reflection, mapping, patchData);
 
   version = 460;
 
   entryPoint = pEntryPoint;
   if(numSpecializationConstants > 0)
   {
-    specIDs.assign(pConstantIndex, pConstantIndex + numSpecializationConstants);
-    specValues.assign(pConstantValue, pConstantValue + numSpecializationConstants);
+    specIDs.assign(pConstantIndex, numSpecializationConstants);
+    specValues.assign(pConstantValue, numSpecializationConstants);
   }
 }
 
@@ -91,7 +92,7 @@ void WrappedOpenGL::ShaderData::ProcessCompilation(WrappedOpenGL &drv, ResourceI
 
   entryPoint = "main";
 
-  string concatenated;
+  rdcstr concatenated;
 
   for(size_t i = 0; i < sources.size(); i++)
   {
@@ -108,9 +109,9 @@ void WrappedOpenGL::ShaderData::ProcessCompilation(WrappedOpenGL &drv, ResourceI
     concatenated += sources[i];
   }
 
-  size_t offs = concatenated.find("#version");
+  int32_t offs = concatenated.find("#version");
 
-  if(offs == std::string::npos)
+  if(offs < 0)
   {
     // if there's no #version it's assumed to be 100 which we set below
     version = 0;
@@ -118,9 +119,9 @@ void WrappedOpenGL::ShaderData::ProcessCompilation(WrappedOpenGL &drv, ResourceI
   else
   {
     // see if we find a second result after the first
-    size_t offs2 = concatenated.find("#version", offs + 1);
+    int32_t offs2 = concatenated.find("#version", offs + 1);
 
-    if(offs2 == std::string::npos)
+    if(offs2 < 0)
     {
       version = ParseVersionStatement(concatenated.c_str() + offs);
     }
@@ -172,7 +173,7 @@ void WrappedOpenGL::ShaderData::ProcessCompilation(WrappedOpenGL &drv, ResourceI
           break;
         }
 
-        std::string versionText(search, search + sizeof("#version") - 1);
+        rdcstr versionText(search, sizeof("#version") - 1);
 
         // if we found the version, parse it
         if(versionText == "#version")
@@ -198,7 +199,7 @@ void WrappedOpenGL::ShaderData::ProcessCompilation(WrappedOpenGL &drv, ResourceI
   // if we don't have program_interface_query, need to compile the shader with glslang to be able
   // to reflect with. This is needed on capture or replay
   if(!HasExt[ARB_program_interface_query] && status == 1)
-    glslangShader = CompileShaderForReflection(SPIRVShaderStage(ShaderIdx(type)), sources);
+    glslangShader = CompileShaderForReflection(rdcspv::ShaderStage(ShaderIdx(type)), sources);
 
   if(IsReplayMode(drv.GetState()) && !drv.IsInternalShader())
   {
@@ -262,27 +263,21 @@ void WrappedOpenGL::ShaderData::ProcessCompilation(WrappedOpenGL &drv, ResourceI
 
       if(reflected)
       {
-        vector<uint32_t> spirvwords;
+        rdcarray<uint32_t> spirvwords;
 
-        SPIRVCompilationSettings settings(SPIRVSourceLanguage::OpenGLGLSL,
-                                          SPIRVShaderStage(ShaderIdx(type)));
+        rdcspv::CompilationSettings settings(rdcspv::InputLanguage::OpenGLGLSL,
+                                             rdcspv::ShaderStage(ShaderIdx(type)));
 
-        string s = CompileSPIRV(settings, sources, spirvwords);
+        rdcstr s = rdcspv::Compile(settings, sources, spirvwords);
         if(!spirvwords.empty())
-          ParseSPIRV(&spirvwords.front(), spirvwords.size(), spirv);
+          spirv.Parse(spirvwords);
         else
-          disassembly = s;
+          disassembly = "Disassembly to SPIR-V failed:\n\n" + s;
 
         reflection.resourceId = id;
-        reflection.entryPoint = "main";
 
-        reflection.stage = MakeShaderStage(type);
-
-        reflection.encoding = ShaderEncoding::GLSL;
         reflection.rawBytes.assign((byte *)concatenated.c_str(), concatenated.size());
 
-        reflection.debugInfo.encoding = ShaderEncoding::GLSL;
-
         reflection.debugInfo.files.resize(1);
         reflection.debugInfo.files[0].filename = "main.glsl";
         reflection.debugInfo.files[0].contents = concatenated;
@@ -300,7 +295,7 @@ bool WrappedOpenGL::Serialise_glCreateShader(SerialiserType &ser, GLenum type, G
 {
   SERIALISE_ELEMENT(type);
   SERIALISE_ELEMENT_LOCAL(Shader, GetResourceManager()->GetID(ShaderRes(GetCtx(), shader)))
-      .TypedAs("GLResource");
+      .TypedAs("GLResource"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -365,15 +360,15 @@ bool WrappedOpenGL::Serialise_glShaderSource(SerialiserType &ser, GLuint shaderH
 
   // serialisation can't handle the length parameter neatly, so we compromise by serialising via a
   // vector
-  std::vector<std::string> sources;
+  rdcarray<rdcstr> sources;
 
   if(ser.IsWriting())
   {
     sources.reserve(count);
     for(GLsizei c = 0; c < count; c++)
     {
-      sources.push_back((length && length[c] > 0) ? std::string(source[c], source[c] + length[c])
-                                                  : std::string(source[c]));
+      sources.push_back((length && length[c] >= 0) ? rdcstr(source[c], length[c])
+                                                   : rdcstr(source[c]));
     }
   }
 
@@ -385,7 +380,7 @@ bool WrappedOpenGL::Serialise_glShaderSource(SerialiserType &ser, GLuint shaderH
 
   if(IsReplayingAndReading())
   {
-    std::vector<const char *> strs;
+    rdcarray<const char *> strs;
     for(size_t i = 0; i < sources.size(); i++)
       strs.push_back(sources[i].c_str());
 
@@ -404,7 +399,7 @@ bool WrappedOpenGL::Serialise_glShaderSource(SerialiserType &ser, GLuint shaderH
     // so people who do that should be moderately ashamed.
     if(m_Shaders[liveId].reflection.resourceId != ResourceId())
     {
-      m_Shaders[liveId].spirv = SPVModule();
+      m_Shaders[liveId].spirv = rdcspv::Reflector();
       m_Shaders[liveId].reflection = ShaderReflection();
     }
 
@@ -443,7 +438,8 @@ void WrappedOpenGL::glShaderSource(GLuint shader, GLsizei count, const GLchar *c
     m_Shaders[id].sources.reserve(count);
 
     for(GLsizei i = 0; i < count; i++)
-      m_Shaders[id].sources.push_back(string[i]);
+      m_Shaders[id].sources.push_back((length && length[i] >= 0) ? rdcstr(string[i], length[i])
+                                                                 : rdcstr(string[i]));
   }
 }
 
@@ -565,9 +561,6 @@ void WrappedOpenGL::glAttachShader(GLuint program, GLuint shader)
       }
     }
 
-    // if we're capturing and don't have ARB_program_interface_query we're going to have to emulate
-    // it using glslang for compilation and reflection
-    if(IsReplayMode(m_State) || !HasExt[ARB_program_interface_query])
     {
       ResourceId progid = GetResourceManager()->GetID(ProgramRes(GetCtx(), program));
       ResourceId shadid = GetResourceManager()->GetID(ShaderRes(GetCtx(), shader));
@@ -637,23 +630,13 @@ void WrappedOpenGL::glDetachShader(GLuint program, GLuint shader)
       }
     }
 
-    // if we're capturing and don't have ARB_program_interface_query we're going to have to emulate
-    // it using glslang for compilation and reflection
-    if(IsReplayMode(m_State) || !HasExt[ARB_program_interface_query])
     {
       ResourceId progid = GetResourceManager()->GetID(ProgramRes(GetCtx(), program));
       ResourceId shadid = GetResourceManager()->GetID(ShaderRes(GetCtx(), shader));
 
       if(!m_Programs[progid].linked)
       {
-        for(auto it = m_Programs[progid].shaders.begin(); it != m_Programs[progid].shaders.end(); ++it)
-        {
-          if(*it == shadid)
-          {
-            m_Programs[progid].shaders.erase(it);
-            break;
-          }
-        }
+        m_Programs[progid].shaders.removeOne(shadid);
       }
     }
   }
@@ -671,13 +654,13 @@ bool WrappedOpenGL::Serialise_glCreateShaderProgramv(SerialiserType &ser, GLenum
   SERIALISE_ELEMENT(count);
   SERIALISE_ELEMENT_ARRAY(strings, count);
   SERIALISE_ELEMENT_LOCAL(Program, GetResourceManager()->GetID(ProgramRes(GetCtx(), program)))
-      .TypedAs("GLResource");
+      .TypedAs("GLResource"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
   if(IsReplayingAndReading())
   {
-    std::vector<std::string> src;
+    rdcarray<rdcstr> src;
     for(GLsizei i = 0; i < count; i++)
       src.push_back(strings[i]);
 
@@ -740,6 +723,15 @@ GLuint WrappedOpenGL::glCreateShaderProgramv(GLenum type, GLsizei count, const G
     GetResourceManager()->MarkDirtyResource(id);
 
     record->AddChunk(chunk);
+
+    // we need initial contents for programs to know any initial bindings potentially if they change
+    // over the frame, and for uniform location remapping.
+    // We just inject a call to prepare the initial contents now, any other post-create data setting
+    // will be replayed as expected.
+    if(IsActiveCapturing(m_State))
+    {
+      GetResourceManager()->ContextPrepare_InitialState(res);
+    }
   }
   else
   {
@@ -753,7 +745,7 @@ template <typename SerialiserType>
 bool WrappedOpenGL::Serialise_glCreateProgram(SerialiserType &ser, GLuint program)
 {
   SERIALISE_ELEMENT_LOCAL(Program, GetResourceManager()->GetID(ProgramRes(GetCtx(), program)))
-      .TypedAs("GLResource");
+      .TypedAs("GLResource"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -840,7 +832,7 @@ bool WrappedOpenGL::Serialise_glLinkProgram(SerialiserType &ser, GLuint programH
 
     if(!HasExt[ARB_program_interface_query])
     {
-      std::vector<glslang::TShader *> glslangShaders;
+      rdcarray<glslang::TShader *> glslangShaders;
 
       for(ResourceId id : progDetails.stageShaders)
       {
@@ -885,6 +877,15 @@ void WrappedOpenGL::glLinkProgram(GLuint program)
 
       record->AddChunk(scope.Get());
     }
+
+    // we need initial contents for programs to know any initial bindings potentially if they change
+    // over the frame, and for uniform location remapping.
+    // We just inject a call to prepare the initial contents now, any other post-link data setting
+    // will be replayed as expected.
+    if(IsActiveCapturing(m_State))
+    {
+      GetResourceManager()->ContextPrepare_InitialState(ProgramRes(GetCtx(), program));
+    }
   }
 
   {
@@ -905,7 +906,7 @@ void WrappedOpenGL::glLinkProgram(GLuint program)
 
     if(!HasExt[ARB_program_interface_query])
     {
-      std::vector<glslang::TShader *> glslangShaders;
+      rdcarray<glslang::TShader *> glslangShaders;
 
       for(ResourceId id : progDetails.stageShaders)
       {
@@ -940,7 +941,8 @@ bool WrappedOpenGL::Serialise_glUniformBlockBinding(SerialiserType &ser, GLuint
 
   if(IsReplayingAndReading())
   {
-    GL.glUniformBlockBinding(program.name, uniformBlockIndex, uniformBlockBinding);
+    if(program.name)
+      GL.glUniformBlockBinding(program.name, uniformBlockIndex, uniformBlockBinding);
 
     AddResourceInitChunk(program);
   }
@@ -953,19 +955,15 @@ void WrappedOpenGL::glUniformBlockBinding(GLuint program, GLuint uniformBlockInd
 {
   SERIALISE_TIME_CALL(GL.glUniformBlockBinding(program, uniformBlockIndex, uniformBlockBinding));
 
-  if(IsCaptureMode(m_State))
+  // we should only capture this while active, since the initial states will grab everything at the
+  // start of the frame and we only want to pick up dynamic changes after that.
+  if(IsActiveCapturing(m_State))
   {
-    GLResourceRecord *record = GetResourceManager()->GetResourceRecord(ProgramRes(GetCtx(), program));
-    RDCASSERTMSG("Couldn't identify object passed to function. Mismatched or bad GLuint?", record,
-                 program);
-    if(record)
-    {
-      USE_SCRATCH_SERIALISER();
-      SCOPED_SERIALISE_CHUNK(gl_CurChunk);
-      Serialise_glUniformBlockBinding(ser, program, uniformBlockIndex, uniformBlockBinding);
+    USE_SCRATCH_SERIALISER();
+    SCOPED_SERIALISE_CHUNK(gl_CurChunk);
+    Serialise_glUniformBlockBinding(ser, program, uniformBlockIndex, uniformBlockBinding);
 
-      record->AddChunk(scope.Get());
-    }
+    GetContextRecord()->AddChunk(scope.Get());
   }
 }
 
@@ -1259,7 +1257,8 @@ void WrappedOpenGL::glDeleteProgram(GLuint program)
   GLResource res = ProgramRes(GetCtx(), program);
   if(GetResourceManager()->HasCurrentResource(res))
   {
-    GetResourceManager()->MarkCleanResource(res);
+    m_Programs.erase(GetResourceManager()->GetID(res));
+
     if(GetResourceManager()->HasResourceRecord(res))
       GetResourceManager()->GetResourceRecord(res)->Delete(GetResourceManager());
     GetResourceManager()->UnregisterResource(res);
@@ -1327,7 +1326,7 @@ bool WrappedOpenGL::Serialise_glShaderBinary(SerialiserType &ser, GLsizei count,
 
     GL.glShaderBinary(1, &shader.name, binaryformat, binary, length);
 
-    m_Shaders[liveId].spirvWords.assign((uint32_t *)binary, (uint32_t *)((byte *)binary + length));
+    m_Shaders[liveId].spirvWords.assign((uint32_t *)binary, length / sizeof(uint32_t));
 
     AddResourceInitChunk(shader);
   }
@@ -1343,6 +1342,15 @@ void WrappedOpenGL::glShaderBinary(GLsizei count, const GLuint *shaders, GLenum
   if(IsReplayMode(m_State))
   {
     GL.glShaderBinary(count, shaders, binaryformat, binary, length);
+
+    if(binaryformat == eGL_SHADER_BINARY_FORMAT_SPIR_V)
+    {
+      for(GLsizei i = 0; i < count; i++)
+      {
+        ResourceId liveId = GetResourceManager()->GetID(ShaderRes(GetCtx(), shaders[i]));
+        m_Shaders[liveId].spirvWords.assign((uint32_t *)binary, length / sizeof(uint32_t));
+      }
+    }
   }
   else if(IsCaptureMode(m_State) && binaryformat == eGL_SHADER_BINARY_FORMAT_SPIR_V)
   {
@@ -1361,6 +1369,9 @@ void WrappedOpenGL::glShaderBinary(GLsizei count, const GLuint *shaders, GLenum
         Serialise_glShaderBinary(ser, 1, shaders + i, binaryformat, binary, length);
 
         record->AddChunk(scope.Get());
+
+        m_Shaders[record->GetResourceID()].spirvWords.assign((uint32_t *)binary,
+                                                             length / sizeof(uint32_t));
       }
     }
   }
@@ -1457,14 +1468,11 @@ void WrappedOpenGL::glUseProgramStages(GLuint pipeline, GLbitfield stages, GLuin
     if(record == NULL)
       return;
 
-    if(program)
+    if(IsActiveCapturing(m_State))
     {
-      GLResourceRecord *progrecord =
-          GetResourceManager()->GetResourceRecord(ProgramRes(GetCtx(), program));
-      RDCASSERT(progrecord);
-
-      if(progrecord)
-        record->AddParent(progrecord);
+      GetResourceManager()->MarkResourceFrameReferenced(record->Resource, eFrameRef_ReadBeforeWrite);
+      GetResourceManager()->MarkResourceFrameReferenced(ProgramRes(GetCtx(), program),
+                                                        eFrameRef_Read);
     }
 
     if(m_HighTrafficResources.find(record->GetResourceID()) != m_HighTrafficResources.end() &&
@@ -1541,7 +1549,7 @@ bool WrappedOpenGL::Serialise_glGenProgramPipelines(SerialiserType &ser, GLsizei
 {
   SERIALISE_ELEMENT(n);
   SERIALISE_ELEMENT_LOCAL(pipeline, GetResourceManager()->GetID(ProgramPipeRes(GetCtx(), *pipelines)))
-      .TypedAs("GLResource");
+      .TypedAs("GLResource"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1602,7 +1610,7 @@ bool WrappedOpenGL::Serialise_glCreateProgramPipelines(SerialiserType &ser, GLsi
 {
   SERIALISE_ELEMENT(n);
   SERIALISE_ELEMENT_LOCAL(pipeline, GetResourceManager()->GetID(ProgramPipeRes(GetCtx(), *pipelines)))
-      .TypedAs("GLResource");
+      .TypedAs("GLResource"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1685,6 +1693,19 @@ void WrappedOpenGL::glBindProgramPipeline(GLuint pipeline)
     GetContextRecord()->AddChunk(scope.Get());
     GetResourceManager()->MarkResourceFrameReferenced(ProgramPipeRes(GetCtx(), pipeline),
                                                       eFrameRef_Read);
+    // mark all the sub programs referenced
+    GLenum programBinds[] = {
+        eGL_VERTEX_SHADER,       eGL_FRAGMENT_SHADER,        eGL_GEOMETRY_SHADER,
+        eGL_TESS_CONTROL_SHADER, eGL_TESS_EVALUATION_SHADER, eGL_COMPUTE_SHADER,
+    };
+
+    for(GLenum progbind : programBinds)
+    {
+      GLuint prog = 0;
+      GL.glGetProgramPipelineiv(pipeline, progbind, (GLint *)&prog);
+      if(prog)
+        GetResourceManager()->MarkResourceFrameReferenced(ProgramRes(GetCtx(), prog), eFrameRef_Read);
+    }
   }
 }
 
@@ -1727,6 +1748,8 @@ void WrappedOpenGL::glDeleteProgramPipelines(GLsizei n, const GLuint *pipelines)
     GLResource res = ProgramPipeRes(GetCtx(), pipelines[i]);
     if(GetResourceManager()->HasCurrentResource(res))
     {
+      m_Pipelines.erase(GetResourceManager()->GetID(res));
+
       if(GetResourceManager()->HasResourceRecord(res))
         GetResourceManager()->GetResourceRecord(res)->Delete(GetResourceManager());
       GetResourceManager()->UnregisterResource(res);
@@ -1819,12 +1842,11 @@ bool WrappedOpenGL::Serialise_glNamedStringARB(SerialiserType &ser, GLenum type,
 {
   SERIALISE_ELEMENT(type);
   SERIALISE_ELEMENT(namelen);
-  SERIALISE_ELEMENT_LOCAL(
-      name, nameStr ? std::string(nameStr, nameStr + (namelen > 0 ? namelen : strlen(nameStr))) : "");
+  SERIALISE_ELEMENT_LOCAL(name,
+                          nameStr ? rdcstr(nameStr, namelen >= 0 ? namelen : strlen(nameStr)) : "");
   SERIALISE_ELEMENT(stringlen);
   SERIALISE_ELEMENT_LOCAL(
-      value,
-      valStr ? std::string(valStr, valStr + (stringlen > 0 ? stringlen : strlen(valStr))) : "");
+      value, valStr ? rdcstr(valStr, stringlen >= 0 ? stringlen : strlen(valStr)) : "");
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1862,8 +1884,8 @@ bool WrappedOpenGL::Serialise_glDeleteNamedStringARB(SerialiserType &ser, GLint
                                                      const GLchar *nameStr)
 {
   SERIALISE_ELEMENT(namelen);
-  SERIALISE_ELEMENT_LOCAL(
-      name, nameStr ? std::string(nameStr, nameStr + (namelen > 0 ? namelen : strlen(nameStr))) : "");
+  SERIALISE_ELEMENT_LOCAL(name,
+                          nameStr ? rdcstr(nameStr, namelen >= 0 ? namelen : strlen(nameStr)) : "");
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1926,8 +1948,7 @@ bool WrappedOpenGL::Serialise_glSpecializeShader(SerialiserType &ser, GLuint sha
     GL.glSpecializeShader(shader.name, pEntryPoint, numSpecializationConstants, pConstantIndex,
                           pConstantValue);
 
-    ParseSPIRV(m_Shaders[liveId].spirvWords.data(), m_Shaders[liveId].spirvWords.size(),
-               m_Shaders[liveId].spirv);
+    m_Shaders[liveId].spirv.Parse(m_Shaders[liveId].spirvWords);
 
     m_Shaders[liveId].ProcessSPIRVCompilation(*this, GetResourceManager()->GetOriginalID(liveId),
                                               shader.name, pEntryPoint, numSpecializationConstants,
@@ -1959,8 +1980,25 @@ void WrappedOpenGL::glSpecializeShader(GLuint shader, const GLchar *pEntryPoint,
                                    pConstantIndex, pConstantValue);
 
       record->AddChunk(scope.Get());
+
+      ResourceId id = record->GetResourceID();
+
+      m_Shaders[id].spirv.Parse(m_Shaders[id].spirvWords);
+
+      m_Shaders[id].ProcessSPIRVCompilation(
+          *this, id, shader, pEntryPoint, numSpecializationConstants, pConstantIndex, pConstantValue);
     }
   }
+  else
+  {
+    ResourceId liveId = GetResourceManager()->GetID(ShaderRes(GetCtx(), shader));
+
+    m_Shaders[liveId].spirv.Parse(m_Shaders[liveId].spirvWords);
+
+    m_Shaders[liveId].ProcessSPIRVCompilation(*this, liveId, shader, pEntryPoint,
+                                              numSpecializationConstants, pConstantIndex,
+                                              pConstantValue);
+  }
 }
 
 INSTANTIATE_FUNCTION_SERIALISED(void, glCreateShader, GLenum type, GLuint shader);
