@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -102,13 +102,13 @@ bool WrappedOpenGL::Serialise_glProgramUniformVector(SerialiserType &ser, GLuint
 
   // we don't want to allocate since we've already handled that
   if(elemBaseType == SDBasic::Float && elemSize == sizeof(float))
-    ser.Serialise("values", v.f, arrayLength, SerialiserFlags::NoFlags);
+    ser.Serialise("values"_lit, v.f, arrayLength, SerialiserFlags::NoFlags);
   else if(elemBaseType == SDBasic::Float)
-    ser.Serialise("values", v.d, arrayLength, SerialiserFlags::NoFlags);
+    ser.Serialise("values"_lit, v.d, arrayLength, SerialiserFlags::NoFlags);
   else if(elemBaseType == SDBasic::SignedInteger)
-    ser.Serialise("values", v.i, arrayLength, SerialiserFlags::NoFlags);
+    ser.Serialise("values"_lit, v.i, arrayLength, SerialiserFlags::NoFlags);
   else if(elemBaseType == SDBasic::UnsignedInteger)
-    ser.Serialise("values", v.u, arrayLength, SerialiserFlags::NoFlags);
+    ser.Serialise("values"_lit, v.u, arrayLength, SerialiserFlags::NoFlags);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -227,9 +227,9 @@ bool WrappedOpenGL::Serialise_glProgramUniformMatrix(SerialiserType &ser, GLuint
 
   // we don't want to allocate since we've already handled that
   if(elemSize == sizeof(float))
-    ser.Serialise("values", v.f, arrayLength, SerialiserFlags::NoFlags);
+    ser.Serialise("values"_lit, v.f, arrayLength, SerialiserFlags::NoFlags);
   else
-    ser.Serialise("values", v.d, arrayLength, SerialiserFlags::NoFlags);
+    ser.Serialise("values"_lit, v.d, arrayLength, SerialiserFlags::NoFlags);
 
   SERIALISE_CHECK_READ_ERRORS();
 
