@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,31 +24,32 @@
 
 #include "amd_isa.h"
 #include "common/common.h"
+#include "common/formatting.h"
 #include "core/plugins.h"
-#include "driver/shaders/spirv/spirv_common.h"
+#include "os/os_specific.h"
 #include "strings/string_utils.h"
 #include "amd_isa_devices.h"
 
 namespace GCNISA
 {
 #if ENABLED(RDOC_WIN32)
-static const std::string amdspv_name = "amdspv.exe";
-static const std::string virtualcontext_name = "VirtualContext.exe";
+static const rdcstr amdspv_name = "amdspv.exe";
+static const rdcstr virtualcontext_name = "VirtualContext.exe";
 #else
-static const std::string amdspv_name = "amdspv.sh";
-static const std::string virtualcontext_name = "VirtualContext";
+static const rdcstr amdspv_name = "amdspv.sh";
+static const rdcstr virtualcontext_name = "VirtualContext";
 #endif
 
-std::string pluginPath = "amd/isa";
+rdcstr pluginPath = "amd/isa";
 
 // in amd_isa_<plat>.cpp
-std::string DisassembleDXBC(const bytebuf &shaderBytes, const std::string &target);
+rdcstr DisassembleDXBC(const bytebuf &shaderBytes, const rdcstr &target);
 
 static bool IsSupported(ShaderEncoding encoding)
 {
   if(encoding == ShaderEncoding::GLSL)
   {
-    std::string vc = LocatePluginFile(pluginPath, virtualcontext_name);
+    rdcstr vc = LocatePluginFile(pluginPath, virtualcontext_name);
 
     Process::ProcessResult result = {};
     Process::LaunchProcess(vc.c_str(), get_dirname(vc).c_str(), "", true, &result);
@@ -63,7 +64,7 @@ static bool IsSupported(ShaderEncoding encoding)
   if(encoding == ShaderEncoding::SPIRV)
   {
     // TODO need to check if an AMD context is running
-    std::string amdspv = LocatePluginFile(pluginPath, amdspv_name);
+    rdcstr amdspv = LocatePluginFile(pluginPath, amdspv_name);
 
     Process::ProcessResult result = {};
     Process::LaunchProcess(amdspv.c_str(), get_dirname(amdspv).c_str(), "", true, &result);
@@ -78,7 +79,7 @@ static bool IsSupported(ShaderEncoding encoding)
   // we only need to check if we can get atidxx64.dll
   if(encoding == ShaderEncoding::DXBC)
   {
-    std::string test = DisassembleDXBC(bytebuf(), "");
+    rdcstr test = DisassembleDXBC(bytebuf(), "");
 
     return test.empty();
   }
@@ -86,7 +87,7 @@ static bool IsSupported(ShaderEncoding encoding)
   return false;
 }
 
-void GetTargets(GraphicsAPI api, std::vector<std::string> &targets)
+void GetTargets(GraphicsAPI api, rdcarray<rdcstr> &targets)
 {
   targets.reserve(asicCount + 1);
 
@@ -110,17 +111,10 @@ void GetTargets(GraphicsAPI api, std::vector<std::string> &targets)
 
   if(IsSupported(primary) || IsSupported(secondary))
   {
-    // OpenGL doesn't support AMDIL
-    if(api != GraphicsAPI::OpenGL)
-      targets.push_back("AMDIL");
-
-    int apiBitmask = 1 << (int)api;
+    targets.push_back("AMDIL");
 
     for(int i = 0; i < asicCount; i++)
-    {
-      if(asicInfo[i].apiBitmask & apiBitmask)
-        targets.push_back(asicInfo[i].name);
-    }
+      targets.push_back(asicInfo[i].name);
   }
   else
   {
@@ -130,7 +124,7 @@ void GetTargets(GraphicsAPI api, std::vector<std::string> &targets)
   }
 }
 
-std::string DisassembleSPIRV(ShaderStage stage, const bytebuf &shaderBytes, const std::string &target)
+rdcstr DisassembleSPIRV(ShaderStage stage, const bytebuf &shaderBytes, const rdcstr &target)
 {
   if(!IsSupported(ShaderEncoding::SPIRV))
   {
@@ -142,7 +136,7 @@ std::string DisassembleSPIRV(ShaderStage stage, const bytebuf &shaderBytes, cons
 ; https://github.com/baldurk/renderdoc/wiki/GCN-ISA)";
   }
 
-  std::string cmdLine = "-Dall -l";
+  rdcstr cmdLine = "-Dall -l";
 
   bool found = false;
 
@@ -184,8 +178,8 @@ std::string DisassembleSPIRV(ShaderStage stage, const bytebuf &shaderBytes, cons
     case ShaderStage::Count: return "; Cannot identify shader type";
   }
 
-  std::string tempPath = FileIO::GetTempFolderFilename() + "rdoc_isa__";
-  std::string inPath = StringFormat::Fmt("%sin.spv", tempPath.c_str());
+  rdcstr tempPath = FileIO::GetTempFolderFilename() + "rdoc_isa__";
+  rdcstr inPath = StringFormat::Fmt("%sin.spv", tempPath.c_str());
 
   cmdLine += StringFormat::Fmt(
       " -set in.spv=\"%sin.spv\" out.%s.palIlText=\"%sout.il\" out.%s.isa=\"%sout.bin\" "
@@ -194,15 +188,15 @@ std::string DisassembleSPIRV(ShaderStage stage, const bytebuf &shaderBytes, cons
       tempPath.c_str(), stageName, tempPath.c_str(), stageName, tempPath.c_str(), stageName,
       tempPath.c_str(), stageName, tempPath.c_str(), tempPath.c_str());
 
-  FileIO::dump(inPath.c_str(), shaderBytes.data(), shaderBytes.size());
+  FileIO::WriteAll(inPath.c_str(), shaderBytes);
 
   // try to locate the amdspv relative to our running program
-  std::string amdspv = LocatePluginFile(pluginPath, amdspv_name);
+  rdcstr amdspv = LocatePluginFile(pluginPath, amdspv_name);
 
   Process::ProcessResult result = {};
   Process::LaunchProcess(amdspv.c_str(), get_dirname(amdspv).c_str(), cmdLine.c_str(), true, &result);
 
-  if(result.strStdout.find("SUCCESS") == std::string::npos)
+  if(result.strStdout.find("SUCCESS") < 0)
   {
     return "; Failed to Disassemble - " + result.strStdout;
   }
@@ -212,29 +206,24 @@ std::string DisassembleSPIRV(ShaderStage stage, const bytebuf &shaderBytes, cons
   FileIO::Delete(StringFormat::Fmt("%sout.log", tempPath.c_str()).c_str());
   FileIO::Delete(StringFormat::Fmt("%sout.bin", tempPath.c_str()).c_str());
 
-  std::string ret;
+  rdcstr ret;
 
   if(amdil)
   {
-    vector<byte> data;
-    FileIO::slurp(StringFormat::Fmt("%sout.il", tempPath.c_str()).c_str(), data);
-
-    ret = std::string(data.data(), data.data() + data.size());
+    FileIO::ReadAll(StringFormat::Fmt("%sout.il", tempPath.c_str()).c_str(), ret);
   }
   else
   {
-    vector<byte> data;
-    FileIO::slurp(StringFormat::Fmt("%sout.txt", tempPath.c_str()).c_str(), data);
+    FileIO::ReadAll(StringFormat::Fmt("%sout.txt", tempPath.c_str()).c_str(), ret);
 
-    ret = std::string(data.data(), data.data() + data.size());
-
-    std::string statsfile = StringFormat::Fmt("%sstats.txt", tempPath.c_str());
+    rdcstr statsfile = StringFormat::Fmt("%sstats.txt", tempPath.c_str());
 
     if(FileIO::exists(statsfile.c_str()))
     {
-      FileIO::slurp(statsfile.c_str(), data);
+      rdcstr stats;
+      FileIO::ReadAll(statsfile.c_str(), stats);
 
-      ret += std::string(data.data(), data.data() + data.size());
+      ret += "\n\n" + stats;
     }
   }
 
@@ -242,14 +231,14 @@ std::string DisassembleSPIRV(ShaderStage stage, const bytebuf &shaderBytes, cons
   FileIO::Delete(StringFormat::Fmt("%sout.txt", tempPath.c_str()).c_str());
   FileIO::Delete(StringFormat::Fmt("%sstats.txt", tempPath.c_str()).c_str());
 
-  std::string header = StringFormat::Fmt("; Disassembly for %s\n\n", target.c_str());
+  rdcstr header = StringFormat::Fmt("; Disassembly for %s\n\n", target.c_str());
 
-  ret.insert(ret.begin(), header.begin(), header.end());
+  ret.insert(0, header.begin(), header.size());
 
   return ret;
 }
 
-std::string DisassembleGLSL(ShaderStage stage, const bytebuf &shaderBytes, const std::string &target)
+rdcstr DisassembleGLSL(ShaderStage stage, const bytebuf &shaderBytes, const rdcstr &target)
 {
   if(!IsSupported(ShaderEncoding::GLSL))
   {
@@ -292,13 +281,14 @@ std::string DisassembleGLSL(ShaderStage stage, const bytebuf &shaderBytes, const
     case ShaderStage::Count: return "; Cannot identify shader type";
   }
 
-  std::string tempPath = FileIO::GetTempFolderFilename() + "rdoc_isa__";
-  std::string inPath = StringFormat::Fmt("%sin.%s", tempPath.c_str(), stageName);
-  std::string outPath = StringFormat::Fmt("%sout.txt", tempPath.c_str());
-  std::string binPath = StringFormat::Fmt("%sout.bin", tempPath.c_str());
-  std::string statsPath = StringFormat::Fmt("%sstats.txt", tempPath.c_str());
+  rdcstr tempPath = FileIO::GetTempFolderFilename() + "rdoc_isa__";
+  rdcstr inPath = StringFormat::Fmt("%sin.%s", tempPath.c_str(), stageName);
+  rdcstr outPath = StringFormat::Fmt("%sout.txt", tempPath.c_str());
+  rdcstr binPath = StringFormat::Fmt("%sout.bin", tempPath.c_str());
+  rdcstr statsPath = StringFormat::Fmt("%sstats.txt", tempPath.c_str());
+  rdcstr ilPath = StringFormat::Fmt("%sil.txt", tempPath.c_str());
 
-  std::string cmdLine = "\"";
+  rdcstr cmdLine = "\"";
 
   // ISA disassembly
   for(int i = 0; i < 6; i++)
@@ -333,6 +323,13 @@ std::string DisassembleGLSL(ShaderStage stage, const bytebuf &shaderBytes, const
     }
   }
 
+  // dummy values
+  if(!found)
+  {
+    const asic &a = asicInfo[0];
+    cmdLine += StringFormat::Fmt("%d;%d;", a.chipFamily, a.chipRevision);
+  }
+
   // input files
   for(int i = 0; i < 6; i++)
   {
@@ -342,21 +339,36 @@ std::string DisassembleGLSL(ShaderStage stage, const bytebuf &shaderBytes, const
     cmdLine += ';';
   }
 
+  cmdLine += ";\"";
+
+  // amdil files
+  for(int i = 0; i < 6; i++)
+  {
+    if(i == stageIndex)
+      cmdLine += ilPath;
+
+    cmdLine += ';';
+  }
+
+  if(!found && target == "AMDIL")
+  {
+    outPath = ilPath;
+    found = true;
+  }
+
   if(!found)
     return "; Invalid ISA Target specified";
 
-  cmdLine += ";\"";
-
-  FileIO::dump(inPath.c_str(), shaderBytes.data(), shaderBytes.size());
+  FileIO::WriteAll(inPath.c_str(), shaderBytes);
 
   // try to locate the amdspv relative to our running program
-  std::string vc = LocatePluginFile(pluginPath, virtualcontext_name);
+  rdcstr vc = LocatePluginFile(pluginPath, virtualcontext_name);
 
   Process::ProcessResult result = {};
   Process::LaunchProcess(vc.c_str(), get_dirname(vc).c_str(), cmdLine.c_str(), true, &result);
 
-  if(result.retCode != 0 || result.strStdout.find("Error") != string::npos ||
-     result.strStdout.empty() || !FileIO::exists(outPath.c_str()))
+  if(result.retCode != 0 || result.strStdout.find("Error") >= 0 || result.strStdout.empty() ||
+     !FileIO::exists(outPath.c_str()))
   {
     return "; Failed to Disassemble - check AMD driver is currently running\n\n; " + result.strStdout;
   }
@@ -365,33 +377,34 @@ std::string DisassembleGLSL(ShaderStage stage, const bytebuf &shaderBytes, const
   FileIO::Delete(inPath.c_str());
   FileIO::Delete(binPath.c_str());
 
-  std::string ret;
+  rdcstr ret;
 
   {
-    vector<byte> data;
-    FileIO::slurp(outPath.c_str(), data);
-    ret = std::string(data.data(), data.data() + data.size());
+    FileIO::ReadAll(outPath.c_str(), ret);
+
+    while(ret.back() == '\0')
+      ret.pop_back();
 
     if(FileIO::exists(statsPath.c_str()))
     {
-      FileIO::slurp(statsPath.c_str(), data);
-      ret += "\n\n";
-      ret += std::string(data.data(), data.data() + data.size());
+      rdcstr stats;
+      FileIO::ReadAll(statsPath.c_str(), stats);
+      ret += "\n\n" + stats;
     }
   }
 
   FileIO::Delete(outPath.c_str());
   FileIO::Delete(statsPath.c_str());
 
-  std::string header = StringFormat::Fmt("; Disassembly for %s\n\n", target.c_str());
+  rdcstr header = StringFormat::Fmt("; Disassembly for %s\n\n", target.c_str());
 
-  ret.insert(ret.begin(), header.begin(), header.end());
+  ret.insert(0, header.begin(), header.size());
 
   return ret;
 }
 
-std::string Disassemble(ShaderEncoding encoding, ShaderStage stage, const bytebuf &shaderBytes,
-                        const std::string &target)
+rdcstr Disassemble(ShaderEncoding encoding, ShaderStage stage, const bytebuf &shaderBytes,
+                   const rdcstr &target)
 {
   if(encoding == ShaderEncoding::DXBC)
     return DisassembleDXBC(shaderBytes, target);
