@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +24,7 @@
 
 #include "amd_isa.h"
 #include "common/common.h"
+#include "common/formatting.h"
 #include "core/plugins.h"
 #include "official/RGA/Common/AmdDxGsaCompile.h"
 #include "official/RGA/elf/elf32.h"
@@ -45,7 +46,7 @@ https://github.com/baldurk/renderdoc/wiki/GCN-ISA)";
 
 namespace GCNISA
 {
-extern std::string pluginPath;
+extern rdcstr pluginPath;
 
 static HMODULE GetAMDModule()
 {
@@ -59,7 +60,21 @@ static HMODULE GetAMDModule()
   return module;
 }
 
-std::string DisassembleDXBC(const bytebuf &shaderBytes, const std::string &target)
+void SafelyCompile(PfnAmdDxGsaCompileShader compileShader, AmdDxGsaCompileShaderInput &in,
+                   AmdDxGsaCompileShaderOutput &out)
+{
+  __try
+  {
+    compileShader(&in, &out);
+  }
+  __except(EXCEPTION_EXECUTE_HANDLER)
+  {
+    out.pShaderBinary = NULL;
+    out.shaderBinarySize = 0;
+  }
+}
+
+rdcstr DisassembleDXBC(const bytebuf &shaderBytes, const rdcstr &target)
 {
   HMODULE mod = GetAMDModule();
 
@@ -136,7 +151,7 @@ std::string DisassembleDXBC(const bytebuf &shaderBytes, const std::string &targe
   const uint32_t numChunks = *dxbc;
   dxbc++;
 
-  std::vector<uint32_t> chunkOffsets;
+  rdcarray<uint32_t> chunkOffsets;
   for(uint32_t i = 0; i < numChunks; i++)
   {
     if(dxbc >= end)
@@ -175,7 +190,7 @@ std::string DisassembleDXBC(const bytebuf &shaderBytes, const std::string &targe
 
   out.size = sizeof(out);
 
-  compileShader(&in, &out);
+  SafelyCompile(compileShader, in, out);
 
   if(out.pShaderBinary == NULL || out.shaderBinarySize < 16)
     return "; Failed to disassemble shader";
@@ -184,7 +199,7 @@ std::string DisassembleDXBC(const bytebuf &shaderBytes, const std::string &targe
 
   const Elf32_Ehdr *elfHeader = (const Elf32_Ehdr *)elf;
 
-  std::string ret;
+  rdcstr ret;
 
   // minimal code to extract data from ELF. We assume the ELF we got back is well-formed.
   if(IS_ELF(*elfHeader) && elfHeader->e_ident[EI_CLASS] == ELFCLASS32)
@@ -228,8 +243,8 @@ std::string DisassembleDXBC(const bytebuf &shaderBytes, const std::string &targe
 
     if(stats && !amdil)
     {
-      std::string statStr = StringFormat::Fmt(
-          R"(; -------- Statistics ---------------------
+      ret.insert(0, StringFormat::Fmt(
+                        R"(; -------- Statistics ---------------------
 ; SGPRs: %u out of %u used
 ; VGPRs: %u out of %u used
 ; LDS: %u out of %u bytes used
@@ -237,16 +252,13 @@ std::string DisassembleDXBC(const bytebuf &shaderBytes, const std::string &targe
 ; Instructions: %u ALU, %u Control Flow, %u TFETCH
 
 )",
-          stats->numSgprsUsed, stats->availableSgprs, stats->numVgprsUsed, stats->availableVgprs,
-          stats->usedLdsBytes, stats->availableLdsBytes, stats->usedScratchBytes, stats->numAluInst,
-          stats->numControlFlowInst, stats->numTfetchInst);
-
-      ret.insert(ret.begin(), statStr.begin(), statStr.end());
+                        stats->numSgprsUsed, stats->availableSgprs, stats->numVgprsUsed,
+                        stats->availableVgprs, stats->usedLdsBytes, stats->availableLdsBytes,
+                        stats->usedScratchBytes, stats->numAluInst, stats->numControlFlowInst,
+                        stats->numTfetchInst));
     }
 
-    std::string header = StringFormat::Fmt("; Disassembly for %s\n\n", target.c_str());
-
-    ret.insert(ret.begin(), header.begin(), header.end());
+    ret.insert(0, StringFormat::Fmt("; Disassembly for %s\n\n", target.c_str()));
   }
   else
   {
