@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,14 +24,20 @@
 
 #include "common/common.h"
 
+// some headers that DriverStorePath.h needs but doesn't include
+#include <windows.h>
+#include <new>
+#include <string>
+
 #include "official/DriverStorePath.h"
 #include "intel_counters.h"
 
 #include <set>
+#include "driver/dx/official/d3d11.h"
 
-const static std::vector<std::string> metricSetBlacklist = {"TestOa"};
+const static rdcarray<rdcstr> metricSetBlacklist = {"TestOa"};
 
-HMODULE IntelCounters::m_MDLibraryHandle = 0;
+void *IntelCounters::m_MDLibraryHandle = 0;
 IMetricsDevice_1_5 *IntelCounters::m_metricsDevice = NULL;
 OpenMetricsDevice_fn IntelCounters::OpenMetricsDevice = NULL;
 CloseMetricsDevice_fn IntelCounters::CloseMetricsDevice = NULL;
@@ -51,9 +57,9 @@ IntelCounters::~IntelCounters()
   m_metricsDevice = NULL;
 
   if(m_MDLibraryHandle)
-    FreeLibrary(m_MDLibraryHandle);
+    FreeLibrary((HMODULE)m_MDLibraryHandle);
 
-  m_MDLibraryHandle = (HMODULE)0;
+  m_MDLibraryHandle = NULL;
   OpenMetricsDevice = NULL;
   CloseMetricsDevice = NULL;
 }
@@ -63,16 +69,17 @@ void IntelCounters::Load()
   if(m_metricsDevice)
     return;
 
-  m_MDLibraryHandle = LoadDynamicLibrary(L"igdmd64.dll", NULL, 0);
+  m_MDLibraryHandle = (void *)LoadDynamicLibrary(L"igdmd64.dll", NULL, 0);
   if(!m_MDLibraryHandle)
     return;
 
-  OpenMetricsDevice = (OpenMetricsDevice_fn)GetProcAddress(m_MDLibraryHandle, "OpenMetricsDevice");
+  OpenMetricsDevice =
+      (OpenMetricsDevice_fn)GetProcAddress((HMODULE)m_MDLibraryHandle, "OpenMetricsDevice");
   if(!OpenMetricsDevice)
     return;
 
   CloseMetricsDevice =
-      (CloseMetricsDevice_fn)GetProcAddress(m_MDLibraryHandle, "CloseMetricsDevice");
+      (CloseMetricsDevice_fn)GetProcAddress((HMODULE)m_MDLibraryHandle, "CloseMetricsDevice");
   if(!CloseMetricsDevice)
     return;
 
@@ -97,10 +104,10 @@ bool IntelCounters::Init(void *pContext)
      (deviceParams->Version.MajorNumber == 1 && deviceParams->Version.MinorNumber < 1))
   {
     CloseMetricsDevice(m_metricsDevice);
-    FreeLibrary(m_MDLibraryHandle);
+    FreeLibrary((HMODULE)m_MDLibraryHandle);
 
     m_metricsDevice = NULL;
-    m_MDLibraryHandle = (HMODULE)0;
+    m_MDLibraryHandle = NULL;
     OpenMetricsDevice = NULL;
     CloseMetricsDevice = NULL;
     return false;
@@ -116,11 +123,11 @@ CounterDescription IntelCounters::GetCounterDescription(GPUCounter counter)
   return m_Counters[GPUCounterToCounterIndex(counter)];
 }
 
-std::vector<CounterDescription> IntelCounters::EnumerateCounters()
+rdcarray<CounterDescription> IntelCounters::EnumerateCounters()
 {
   m_Counters.clear();
   m_allMetricSets.clear();
-  std::set<std::string> addedMetrics;
+  std::set<rdcstr> addedMetrics;
   TMetricsDeviceParams_1_2 *deviceParams = m_metricsDevice->GetParams();
   for(unsigned int i = 0; i < deviceParams->ConcurrentGroupsCount; ++i)
   {
@@ -138,8 +145,7 @@ std::vector<CounterDescription> IntelCounters::EnumerateCounters()
       m_allMetricSets.push_back(metricSet);
       TMetricSetParams_1_0 *setParams = metricSet->GetParams();
 
-      if(std::find(metricSetBlacklist.begin(), metricSetBlacklist.end(), setParams->SymbolName) !=
-         metricSetBlacklist.end())
+      if(metricSetBlacklist.contains(setParams->SymbolName))
         continue;
 
       for(unsigned int k = 0; k < setParams->MetricsCount; ++k)
@@ -197,7 +203,7 @@ std::vector<CounterDescription> IntelCounters::EnumerateCounters()
         counterDesc.counter = (GPUCounter)counterId;
         m_Counters.push_back(counterDesc);
         addedMetrics.insert(counterDesc.name);
-        m_metricLocation[(GPUCounter)counterId] = std::pair<uint32_t, uint32_t>(j, k);
+        m_metricLocation[(GPUCounter)counterId] = rdcpair<uint32_t, uint32_t>(j, k);
         m_counterIds.push_back((GPUCounter)counterId);
       }
     }
@@ -220,16 +226,26 @@ uint32_t IntelCounters::BeginSession()
   TMetricsDeviceParams_1_2 *deviceParams = m_metricsDevice->GetParams();
   if(deviceParams->Version.MajorNumber < 1 ||
      (deviceParams->Version.MajorNumber == 1 && deviceParams->Version.MinorNumber < 2))
+  {
+    RDCERR("Bad metrics device API version %u.%u", deviceParams->Version.MajorNumber,
+           deviceParams->Version.MinorNumber);
     return 0;
+  }
 
   IOverride_1_2 *frequencyOverride = m_metricsDevice->GetOverrideByName("FrequencyOverride");
   if(!frequencyOverride)
+  {
+    RDCERR("Couldn't fetch Frequency Override");
     return 0;
+  }
 
   TTypedValue_1_0 *maxFreqSymbol =
       m_metricsDevice->GetGlobalSymbolValueByName("GpuMaxFrequencyMHz");
   if(!maxFreqSymbol)
+  {
+    RDCERR("Couldn't get global symbol GpuMaxFrequencyMHz");
     return 0;
+  }
 
   TSetFrequencyOverrideParams_1_2 params;
   params.Enable = true;
@@ -278,15 +294,15 @@ void IntelCounters::EnableCounter(GPUCounter index)
 {
   uint32_t metricSetIdx = m_metricLocation[index].first;
   IMetricSet_1_1 *metricSet = m_allMetricSets[metricSetIdx];
-  auto iter = std::find(m_subscribedMetricSets.begin(), m_subscribedMetricSets.end(), metricSet);
-  if(iter == m_subscribedMetricSets.end())
+  int idx = m_subscribedMetricSets.indexOf(metricSet);
+  if(idx < 0)
   {
     m_subscribedMetricSets.push_back(metricSet);
     metricSetIdx = (uint32_t)m_subscribedMetricSets.size() - 1;
   }
   else
   {
-    metricSetIdx = (uint32_t)(iter - m_subscribedMetricSets.begin());
+    metricSetIdx = (uint32_t)idx;
   }
   m_subscribedMetricsByCounterSet[metricSetIdx].push_back(index);
 }
@@ -309,22 +325,32 @@ void IntelCounters::BeginSample()
       (D3D11_COUNTER)m_subscribedMetricSets[m_passIndex]->GetParams()->ApiSpecificId.D3D1XDevDependentId;
 
   if(counter_desc.Counter == 0)
+  {
+    RDCERR("Invalid D3D11 counter for sample in pass %u", m_passIndex);
     return;
+  }
 
   TCompletionCode res = m_subscribedMetricSets[m_passIndex]->Activate();
   if(res != TCompletionCode::CC_OK)
+  {
+    RDCERR("Couldn't activate subscribed metric set in pass %u", m_passIndex);
     return;
+  }
 
   HRESULT hr = m_device->CreateCounter(&counter_desc, &m_counter);
   if(FAILED(hr))
   {
+    RDCERR("Couldn't create D3D11 counter %u in pass %u", counter_desc.Counter, m_passIndex);
     m_subscribedMetricSets[m_passIndex]->Deactivate();
     return;
   }
 
   res = m_subscribedMetricSets[m_passIndex]->Deactivate();
   if(res != TCompletionCode::CC_OK)
+  {
+    RDCERR("Couldn't deactivate subscribed metric set in pass %u", m_passIndex);
     return;
+  }
 
   m_deviceContext->Begin(m_counter);
 }
@@ -334,6 +360,9 @@ void IntelCounters::EndSample()
   if(!m_metricsDevice)
     return;
 
+  if(!m_counter)
+    return;
+
   m_deviceContext->End(m_counter);
   HRESULT hr = S_OK;
   uint32_t iteration = 0;
@@ -349,7 +378,10 @@ void IntelCounters::EndSample()
   m_counter->Release();
 
   if(hr != S_OK)
+  {
+    RDCERR("Couldn't get counter data after %u attempts: %s", max_attempts, ToStr(hr).c_str());
     return;
+  }
 
   IMetricSet_1_1 *metricSet = m_subscribedMetricSets[m_passIndex];
   TMetricSetParams_1_0 *metricSetParams = metricSet->GetParams();
@@ -360,23 +392,25 @@ void IntelCounters::EndSample()
       (uint32_t)m_queryResult.size() * sizeof(TTypedValue_1_0), &calculatedReportCount, false);
 
   if(res != TCompletionCode::CC_OK)
+  {
+    RDCERR("Couldn't calculate metric set for sample in pass %u", m_passIndex);
     return;
+  }
 
   for(size_t i = 0; i < m_subscribedMetricsByCounterSet[m_passIndex].size(); i++)
   {
     GPUCounter counterId = m_subscribedMetricsByCounterSet[m_passIndex][i];
     uint32_t counterIndex = m_metricLocation[counterId].second;
 
-    m_results[std::pair<GPUCounter, uint32_t>(counterId, m_sampleIndex)] =
-        m_queryResult[counterIndex];
+    m_results[rdcpair<GPUCounter, uint32_t>(counterId, m_sampleIndex)] = m_queryResult[counterIndex];
   }
   m_sampleIndex++;
 }
 
-std::vector<CounterResult> IntelCounters::GetCounterData(const std::vector<uint32_t> &eventIDs,
-                                                         const std::vector<GPUCounter> &counters)
+rdcarray<CounterResult> IntelCounters::GetCounterData(const rdcarray<uint32_t> &eventIDs,
+                                                      const rdcarray<GPUCounter> &counters)
 {
-  std::vector<CounterResult> ret;
+  rdcarray<CounterResult> ret;
   for(uint32_t sample = 0; sample < (uint32_t)eventIDs.size(); sample++)
   {
     for(size_t counter = 0; counter < counters.size(); counter++)
@@ -390,7 +424,7 @@ std::vector<CounterResult> IntelCounters::GetCounterData(const std::vector<uint3
           if(desc.resultByteWidth == sizeof(uint32_t))
           {
             uint32_t value =
-                m_results[std::pair<GPUCounter, uint32_t>(desc.counter, sample)].ValueUInt32;
+                m_results[rdcpair<GPUCounter, uint32_t>(desc.counter, sample)].ValueUInt32;
             if(desc.unit == CounterUnit::Percentage)
             {
               value = RDCCLAMP(value, 0U, 100U);
@@ -401,7 +435,7 @@ std::vector<CounterResult> IntelCounters::GetCounterData(const std::vector<uint3
           else if(desc.resultByteWidth == sizeof(uint64_t))
           {
             uint64_t value =
-                m_results[std::pair<GPUCounter, uint32_t>(desc.counter, sample)].ValueUInt64;
+                m_results[rdcpair<GPUCounter, uint32_t>(desc.counter, sample)].ValueUInt64;
 
             if(desc.unit == CounterUnit::Percentage)
             {
@@ -418,11 +452,11 @@ std::vector<CounterResult> IntelCounters::GetCounterData(const std::vector<uint3
         break;
         case CompType::Float:
         {
-          float value = m_results[std::pair<GPUCounter, uint32_t>(desc.counter, sample)].ValueFloat;
+          float value = m_results[rdcpair<GPUCounter, uint32_t>(desc.counter, sample)].ValueFloat;
           if(desc.unit == CounterUnit::Seconds)
           {
             float nanoseconds =
-                (float)m_results[std::pair<GPUCounter, uint32_t>(desc.counter, sample)].ValueUInt64;
+                (float)m_results[rdcpair<GPUCounter, uint32_t>(desc.counter, sample)].ValueUInt64;
             value = nanoseconds / 1e9f;
           }
 
