@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +24,7 @@
 
 #include "dxbc_compile.h"
 #include "common/common.h"
+#include "os/os_specific.h"
 #include "strings/string_utils.h"
 
 // gives us an address to identify this dll with
@@ -60,12 +61,10 @@ HMODULE GetD3DCompiler()
   // we'll have to loadlibrary the version that ships with
   // RenderDoc.
 
-  std::string dllFile;
+  rdcstr dllFile;
   FileIO::GetLibraryFilename(dllFile);
 
-  std::string dll = get_dirname(dllFile) + "/d3dcompiler_47.dll";
+  rdcstr dll = get_dirname(dllFile) + "/d3dcompiler_47.dll";
 
-  ret = LoadLibraryW(StringFormat::UTF82Wide(dll.c_str()).c_str());
-
-  return ret;
-}
\ No newline at end of file
+  return LoadLibraryW(StringFormat::UTF82Wide(dll.c_str()).data());
+}
