@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,12 +23,11 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "dxbc_inspect.h"
+#include "dxbc_container.h"
 
+#include "os/os_specific.h"
 #include "dxbc_sdbg.h"
 
-using std::make_pair;
-
 namespace DXBC
 {
 static const uint32_t FOURCC_SDBG = MAKE_FOURCC('S', 'D', 'B', 'G');
@@ -68,16 +67,15 @@ SDBGChunk::SDBGChunk(void *data)
   SDBGType *Types = (SDBGType *)(dbgPostHeader + m_Header.types.offset);
   int32_t *Int32DB = (int32_t *)(dbgPostHeader + m_Header.int32DBOffset);
 
-  m_FileHeaders = vector<SDBGFileHeader>(FileHeaders, FileHeaders + m_Header.files.count);
-  m_Instructions =
-      vector<SDBGAsmInstruction>(Instructions, Instructions + m_Header.instructions.count);
-  m_Variables = vector<SDBGVariable>(Variables, Variables + m_Header.variables.count);
-  m_Inputs = vector<SDBGInputRegister>(Inputs, Inputs + m_Header.inputRegisters.count);
-  m_SymbolTable = vector<SDBGSymbol>(SymbolTable, SymbolTable + m_Header.symbolTable.count);
-  m_Scopes = vector<SDBGScope>(Scopes, Scopes + m_Header.scopes.count);
-  m_Types = vector<SDBGType>(Types, Types + m_Header.types.count);
-  m_Int32Database = vector<int32_t>(
-      Int32DB, Int32DB + (m_Header.asciiDBOffset - m_Header.int32DBOffset) / sizeof(int32_t));
+  m_FileHeaders = rdcarray<SDBGFileHeader>(FileHeaders, m_Header.files.count);
+  m_Instructions = rdcarray<SDBGAsmInstruction>(Instructions, m_Header.instructions.count);
+  m_Variables = rdcarray<SDBGVariable>(Variables, m_Header.variables.count);
+  m_Inputs = rdcarray<SDBGInputRegister>(Inputs, m_Header.inputRegisters.count);
+  m_SymbolTable = rdcarray<SDBGSymbol>(SymbolTable, m_Header.symbolTable.count);
+  m_Scopes = rdcarray<SDBGScope>(Scopes, m_Header.scopes.count);
+  m_Types = rdcarray<SDBGType>(Types, m_Header.types.count);
+  m_Int32Database = rdcarray<int32_t>(
+      Int32DB, (m_Header.asciiDBOffset - m_Header.int32DBOffset) / sizeof(int32_t));
 
   char *asciiDatabase = dbgPostHeader + m_Header.asciiDBOffset;
 
@@ -87,11 +85,11 @@ SDBGChunk::SDBGChunk(void *data)
 
   for(size_t i = 0; i < m_FileHeaders.size(); i++)
   {
-    string filename =
-        string(asciiDatabase + m_FileHeaders[i].filenameOffset, m_FileHeaders[i].filenameLen);
-    string source = string(asciiDatabase + m_FileHeaders[i].sourceOffset, m_FileHeaders[i].sourceLen);
+    rdcstr filename =
+        rdcstr(asciiDatabase + m_FileHeaders[i].filenameOffset, m_FileHeaders[i].filenameLen);
+    rdcstr source = rdcstr(asciiDatabase + m_FileHeaders[i].sourceOffset, m_FileHeaders[i].sourceLen);
 
-    this->Files.push_back(make_pair(filename, source));
+    this->Files.push_back(make_rdcpair(filename, source));
   }
 
   // successful grab of info
@@ -127,7 +125,7 @@ void SDBGChunk::GetLocals(size_t instruction, uintptr_t offset,
 {
 }
 
-string SDBGChunk::GetSymbolName(int symbolID)
+rdcstr SDBGChunk::GetSymbolName(int symbolID)
 {
   RDCASSERT(symbolID >= 0 && symbolID < (int)m_SymbolTable.size());
 
@@ -136,14 +134,19 @@ string SDBGChunk::GetSymbolName(int symbolID)
   return GetSymbolName(sym.symbol.offset, sym.symbol.count);
 }
 
-string SDBGChunk::GetSymbolName(int32_t symbolOffset, int32_t symbolLength)
+rdcstr SDBGChunk::GetSymbolName(int32_t symbolOffset, int32_t symbolLength)
 {
   RDCASSERT(symbolOffset < m_Header.compilerSigOffset);
   RDCASSERT(symbolOffset + symbolLength <= m_Header.compilerSigOffset);
 
   int32_t offset = sizeof(m_Header) + m_Header.asciiDBOffset + symbolOffset;
 
-  return string(&m_RawData[offset], symbolLength);
+  return rdcstr(&m_RawData[offset], symbolLength);
 }
 
-};    // namespace DXBC
\ No newline at end of file
+IDebugInfo *MakeSDBGChunk(void *data)
+{
+  return new SDBGChunk(data);
+}
+
+};    // namespace DXBC
