@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -24,9 +24,11 @@
  ******************************************************************************/
 
 #include <algorithm>
-#include "dxbc_inspect.h"
+#include "common/formatting.h"
+#include "dxbc_container.h"
 
 #include "official/cvinfo.h"
+#include "os/os_specific.h"
 #include "dxbc_spdb.h"
 
 // uncomment the following to print (very verbose) debugging prints for SPDB processing
@@ -36,13 +38,11 @@
 #define SPDBLOG(...) (void)(__VA_ARGS__)
 #endif
 
-using std::make_pair;
-
 namespace DXBC
 {
 static const uint32_t FOURCC_SPDB = MAKE_FOURCC('S', 'P', 'D', 'B');
 
-SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
+SPDBChunk::SPDBChunk(Reflection *reflection, void *chunk)
 {
   m_HasDebugInfo = false;
 
@@ -86,7 +86,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
   PageMapping directoryMapping(pages, header->PageSize, (uint32_t *)rootdirIndices, rootdirCount);
   const uint32_t *dirContents = (const uint32_t *)directoryMapping.Data();
 
-  vector<PDBStream> streams;
+  rdcarray<PDBStream> streams;
 
   streams.resize(*dirContents);
   dirContents++;
@@ -132,7 +132,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
   RDCASSERT(hashtable[0] == 0);
   hashtable++;
 
-  map<string, uint32_t> StreamNames;
+  std::map<rdcstr, uint32_t> StreamNames;
 
   uint32_t numset = 0;
   for(uint32_t i = 0; i < maxBit; i++)
@@ -171,26 +171,26 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
       if(filename[0] == 0)
         filename = "shader";
 
-      Files.push_back(make_pair(filename, (char *)fileContents.Data()));
+      Files.push_back({filename, (char *)fileContents.Data()});
     }
   }
 
   struct TypeMember
   {
-    std::string name;
+    rdcstr name;
     uint16_t byteOffset;
     uint32_t typeIndex;
   };
 
   struct TypeDesc
   {
-    std::string name;
+    rdcstr name;
     VarType baseType;
     uint32_t byteSize;
     uint16_t vecSize;
     uint16_t matArrayStride;
     LEAF_ENUM_e leafType;
-    std::vector<TypeMember> members;
+    rdcarray<TypeMember> members;
   };
 
   std::map<uint32_t, TypeDesc> typeInfo;
@@ -418,7 +418,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
 
           uint32_t idx = 0;
 
-          std::vector<TypeMember> &members = typeInfo[id].members;
+          rdcarray<TypeMember> &members = typeInfo[id].members;
 
           byte *iter = (byte *)fieldList->data;
           while(iter < bytes)
@@ -675,7 +675,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
     m_Functions[0] = mainFunc;
   }
 
-  map<uint32_t, string> Names;
+  std::map<uint32_t, rdcstr> Names;
 
   if(StreamNames.find("/names") != StreamNames.end())
   {
@@ -713,7 +713,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
     }
   }
 
-  vector<DBIModule> modules;
+  rdcarray<DBIModule> modules;
 
   {
     PageMapping dbiMapping(pages, header->PageSize, &streams[3].pageIndices[0],
@@ -728,7 +728,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
     while(cur < end)
     {
       DBIModule *mod = (DBIModule *)cur;
-      cur += sizeof(DBIModule) - sizeof(string) * 2;
+      cur += sizeof(DBIModule) - sizeof(rdcstr) * 2;
 
       char *moduleName = (char *)cur;
       cur += strlen(moduleName) + 1;
@@ -741,7 +741,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
         cur++;
 
       DBIModule m;
-      memcpy(&m, mod, sizeof(DBIModule) - sizeof(string) * 2);
+      memcpy(&m, mod, sizeof(DBIModule) - sizeof(rdcstr) * 2);
       m.moduleName = moduleName;
       m.objectName = objectName;
 
@@ -752,11 +752,11 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
     RDCASSERT(cur == end);
   }
 
-  std::vector<Inlinee> inlines;
+  rdcarray<Inlinee> inlines;
 
   PROCSYM32 main = {};
 
-  map<uint32_t, int32_t> FileMapping;    // mapping from hash chunk to index in Files[], or -1
+  std::map<uint32_t, int32_t> FileMapping;    // mapping from hash chunk to index in Files[], or -1
 
   for(size_t m = 0; m < modules.size(); m++)
   {
@@ -772,7 +772,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
 
     RDCASSERT(moduledata[0] == CV_SIGNATURE_C13);
 
-    std::string localName;
+    rdcstr localName;
     CV_typ_t localType = 0;
 
     byte *basePtr = (byte *)&moduledata[1];
@@ -865,7 +865,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
           }
           else if(!strcmp(key, "hlslDefines"))
           {
-            string cmdlineDefines = "// Command line defines:\n\n";
+            rdcstr cmdlineDefines = "// Command line defines:\n\n";
 
             char *c = value;
 
@@ -912,13 +912,14 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
                   char *valend = c;
 
                   cmdlineDefines += "#define ";
-                  cmdlineDefines += string(defstart, defend) + " " + string(valstart, valend);
+                  cmdlineDefines += rdcstr(defstart, defend - defstart) + " " +
+                                    rdcstr(valstart, valend - valstart);
                   cmdlineDefines += "\n";
                 }
                 else
                 {
                   cmdlineDefines += "#define ";
-                  cmdlineDefines += string(defstart, defend);
+                  cmdlineDefines += rdcstr(defstart, defend - defstart);
                   cmdlineDefines += "\n";
                 }
               }
@@ -928,7 +929,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
               }
             }
 
-            Files.push_back(make_pair("@cmdline", cmdlineDefines));
+            Files.push_back({"@cmdline", cmdlineDefines});
           }
 
           key = value + strlen(value) + 1;
@@ -1122,41 +1123,41 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
 
         // CV_HLSLREG_e == OperandType
 
-        switch((OperandType)defrange->regType)
+        switch((DXBCBytecode::OperandType)defrange->regType)
         {
-          case TYPE_TEMP:
+          case DXBCBytecode::TYPE_TEMP:
             range.type = RegisterType::Temporary;
             regtype = "temp";
             regprefix = "r";
             break;
-          case TYPE_INPUT:
-          case TYPE_INPUT_PRIMITIVEID:
-          case TYPE_INPUT_FORK_INSTANCE_ID:
-          case TYPE_INPUT_JOIN_INSTANCE_ID:
-          case TYPE_INPUT_CONTROL_POINT:
-          case TYPE_INPUT_PATCH_CONSTANT:
-          case TYPE_INPUT_DOMAIN_POINT:
-          case TYPE_INPUT_THREAD_ID:
-          case TYPE_INPUT_THREAD_GROUP_ID:
-          case TYPE_INPUT_THREAD_ID_IN_GROUP:
-          case TYPE_INPUT_COVERAGE_MASK:
-          case TYPE_INPUT_THREAD_ID_IN_GROUP_FLATTENED:
-          case TYPE_INPUT_GS_INSTANCE_ID:
+          case DXBCBytecode::TYPE_INPUT:
+          case DXBCBytecode::TYPE_INPUT_PRIMITIVEID:
+          case DXBCBytecode::TYPE_INPUT_FORK_INSTANCE_ID:
+          case DXBCBytecode::TYPE_INPUT_JOIN_INSTANCE_ID:
+          case DXBCBytecode::TYPE_INPUT_CONTROL_POINT:
+          case DXBCBytecode::TYPE_INPUT_PATCH_CONSTANT:
+          case DXBCBytecode::TYPE_INPUT_DOMAIN_POINT:
+          case DXBCBytecode::TYPE_INPUT_THREAD_ID:
+          case DXBCBytecode::TYPE_INPUT_THREAD_GROUP_ID:
+          case DXBCBytecode::TYPE_INPUT_THREAD_ID_IN_GROUP:
+          case DXBCBytecode::TYPE_INPUT_COVERAGE_MASK:
+          case DXBCBytecode::TYPE_INPUT_THREAD_ID_IN_GROUP_FLATTENED:
+          case DXBCBytecode::TYPE_INPUT_GS_INSTANCE_ID:
             range.type = RegisterType::Input;
             regtype = "input";
             regprefix = "v";
             break;
-          case TYPE_OUTPUT:
-          case TYPE_OUTPUT_DEPTH:
-          case TYPE_OUTPUT_DEPTH_LESS_EQUAL:
-          case TYPE_OUTPUT_DEPTH_GREATER_EQUAL:
-          case TYPE_OUTPUT_STENCIL_REF:
-          case TYPE_OUTPUT_COVERAGE_MASK:
+          case DXBCBytecode::TYPE_OUTPUT:
+          case DXBCBytecode::TYPE_OUTPUT_DEPTH:
+          case DXBCBytecode::TYPE_OUTPUT_DEPTH_LESS_EQUAL:
+          case DXBCBytecode::TYPE_OUTPUT_DEPTH_GREATER_EQUAL:
+          case DXBCBytecode::TYPE_OUTPUT_STENCIL_REF:
+          case DXBCBytecode::TYPE_OUTPUT_COVERAGE_MASK:
             range.type = RegisterType::Output;
             regtype = "output";
             regprefix = "o";
             break;
-          case TYPE_INDEXABLE_TEMP:
+          case DXBCBytecode::TYPE_INDEXABLE_TEMP:
             range.type = RegisterType::IndexedTemporary;
             regtype = "indexable";
             regprefix = "x";
@@ -1166,7 +1167,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
         }
 
         // this is a virtual register, not stored
-        if((OperandType)defrange->regType == TYPE_STREAM)
+        if((DXBCBytecode::OperandType)defrange->regType == DXBCBytecode::TYPE_STREAM)
           continue;
 
         const char *space = "";
@@ -1199,41 +1200,47 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
 
         bool builtinoutput = false;
         mapping.var.builtin = ShaderBuiltin::Undefined;
-        switch((OperandType)defrange->regType)
+        switch((DXBCBytecode::OperandType)defrange->regType)
         {
-          case TYPE_OUTPUT_DEPTH:
+          case DXBCBytecode::TYPE_OUTPUT_DEPTH:
             builtinoutput = true;
             mapping.var.builtin = ShaderBuiltin::DepthOutput;
             break;
-          case TYPE_OUTPUT_DEPTH_LESS_EQUAL:
+          case DXBCBytecode::TYPE_OUTPUT_DEPTH_LESS_EQUAL:
             builtinoutput = true;
             mapping.var.builtin = ShaderBuiltin::DepthOutputLessEqual;
             break;
-          case TYPE_OUTPUT_DEPTH_GREATER_EQUAL:
+          case DXBCBytecode::TYPE_OUTPUT_DEPTH_GREATER_EQUAL:
             builtinoutput = true;
             mapping.var.builtin = ShaderBuiltin::DepthOutputGreaterEqual;
             break;
-          case TYPE_OUTPUT_STENCIL_REF:
+          case DXBCBytecode::TYPE_OUTPUT_STENCIL_REF:
             builtinoutput = true;
             mapping.var.builtin = ShaderBuiltin::StencilReference;
             break;
-          case TYPE_OUTPUT_COVERAGE_MASK:
+          case DXBCBytecode::TYPE_OUTPUT_COVERAGE_MASK:
             builtinoutput = true;
             mapping.var.builtin = ShaderBuiltin::MSAACoverage;
             break;
-          case TYPE_INPUT_PRIMITIVEID: mapping.var.builtin = ShaderBuiltin::PrimitiveIndex; break;
-          case TYPE_INPUT_COVERAGE_MASK: mapping.var.builtin = ShaderBuiltin::MSAACoverage; break;
-          case TYPE_INPUT_THREAD_ID:
+          case DXBCBytecode::TYPE_INPUT_PRIMITIVEID:
+            mapping.var.builtin = ShaderBuiltin::PrimitiveIndex;
+            break;
+          case DXBCBytecode::TYPE_INPUT_COVERAGE_MASK:
+            mapping.var.builtin = ShaderBuiltin::MSAACoverage;
+            break;
+          case DXBCBytecode::TYPE_INPUT_THREAD_ID:
             mapping.var.builtin = ShaderBuiltin::DispatchThreadIndex;
             break;
-          case TYPE_INPUT_THREAD_GROUP_ID: mapping.var.builtin = ShaderBuiltin::GroupIndex; break;
-          case TYPE_INPUT_THREAD_ID_IN_GROUP:
+          case DXBCBytecode::TYPE_INPUT_THREAD_GROUP_ID:
+            mapping.var.builtin = ShaderBuiltin::GroupIndex;
+            break;
+          case DXBCBytecode::TYPE_INPUT_THREAD_ID_IN_GROUP:
             mapping.var.builtin = ShaderBuiltin::GroupThreadIndex;
             break;
-          case TYPE_INPUT_THREAD_ID_IN_GROUP_FLATTENED:
+          case DXBCBytecode::TYPE_INPUT_THREAD_ID_IN_GROUP_FLATTENED:
             mapping.var.builtin = ShaderBuiltin::GroupFlatIndex;
             break;
-          case TYPE_INPUT_GS_INSTANCE_ID:
+          case DXBCBytecode::TYPE_INPUT_GS_INSTANCE_ID:
             mapping.var.builtin = ShaderBuiltin::GSInstanceIndex;
             break;
           default: break;
@@ -1245,12 +1252,11 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
 
           if(builtinoutput)
           {
-            for(size_t i = 0; i < dxbc->m_OutputSig.size(); i++)
+            for(size_t i = 0; i < reflection->OutputSig.size(); i++)
             {
-              if(dxbc->m_OutputSig[i].systemValue == mapping.var.builtin)
+              if(reflection->OutputSig[i].systemValue == mapping.var.builtin)
               {
                 regindex = (uint32_t)i;
-                regfirstcomp = 0;
                 found = true;
                 break;
               }
@@ -1258,12 +1264,11 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
           }
           else
           {
-            for(size_t i = 0; i < dxbc->m_InputSig.size(); i++)
+            for(size_t i = 0; i < reflection->InputSig.size(); i++)
             {
-              if(dxbc->m_InputSig[i].systemValue == mapping.var.builtin)
+              if(reflection->InputSig[i].systemValue == mapping.var.builtin)
               {
                 regindex = (uint32_t)i;
-                regfirstcomp = 0;
                 found = true;
                 break;
               }
@@ -1321,7 +1326,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
                      memberOffset + memberLen, varOffset, varOffset + varLen);
             }
 
-            mapping.var.localName = std::string(mapping.var.localName) + "." + mem.name;
+            mapping.var.localName = rdcstr(mapping.var.localName) + "." + mem.name;
 
             // subtract off the offset of this member so we're now relative to it - since it might
             // be a struct itself and we need to recurse.
@@ -1338,7 +1343,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
           {
             RDCERR("No member of %s corresponds to variable range [%u,%u]", vartype->name.c_str(),
                    varOffset, varOffset + varLen);
-            mapping.var.localName = std::string(mapping.var.localName) + ".__unknown__";
+            mapping.var.localName = rdcstr(mapping.var.localName) + ".__unknown__";
             break;
           }
         }
@@ -1433,7 +1438,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
 
           iter += offsetof(FileChecksum, hashData);
 
-          std::string name;
+          rdcstr name;
 
           if(Names.find(checksum->nameIndex) != Names.end())
           {
@@ -1476,6 +1481,28 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
               }
             }
 
+            if(fileIdx == -1)
+            {
+              // if file index is still -1, try again but with normalised names
+              for(char &c : name)
+                if(c == '\\')
+                  c = '/';
+
+              for(size_t i = 0; i < Files.size(); i++)
+              {
+                rdcstr normalised = Files[i].first;
+                for(char &c : normalised)
+                  if(c == '\\')
+                    c = '/';
+
+                if(!_stricmp(normalised.c_str(), name.c_str()))
+                {
+                  fileIdx = (int32_t)i;
+                  break;
+                }
+              }
+            }
+
             FileMapping[chunkOffs] = fileIdx;
           }
           else
@@ -1487,7 +1514,7 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
             // source according to #line
             if(!name.empty())
             {
-              Files.push_back(make_pair(name, ""));
+              Files.push_back({name, ""});
 
               FileMapping[chunkOffs] = (int32_t)Files.size() - 1;
             }
@@ -1563,14 +1590,18 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
           size_t count = (subend - iter) / sizeof(CodeViewInfo::InlineeSourceLine);
           for(size_t i = 0; i < count; i++, inlinee++)
           {
-            inlines[i].id = inlinee->inlinee;
-            inlines[i].fileOffs = inlinee->fileId;
-            inlines[i].baseLineNum = inlinee->sourceLineNum;
+            for(size_t in = 0; in < inlines.size(); in++)
+            {
+              if(inlinee->inlinee == inlines[in].id)
+              {
+                inlines[in].fileOffs = inlinee->fileId;
+                inlines[in].baseLineNum = inlinee->sourceLineNum;
+              }
+            }
           }
         }
         else if(sourceLineType == CV_INLINEE_SOURCE_LINE_SIGNATURE_EX)
         {
-          size_t idx = 0;
           while(iter < subend)
           {
             CodeViewInfo::InlineeSourceLineEx *inlinee = (CodeViewInfo::InlineeSourceLineEx *)iter;
@@ -1578,10 +1609,14 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
             iter += sizeof(CodeViewInfo::InlineeSourceLineEx) +
                     sizeof(CV_off32_t) * inlinee->countOfExtraFiles;
 
-            inlines[idx].id = inlinee->inlinee;
-            inlines[idx].fileOffs = inlinee->fileId;
-            inlines[idx].baseLineNum = inlinee->sourceLineNum;
-            idx++;
+            for(size_t in = 0; in < inlines.size(); in++)
+            {
+              if(inlinee->inlinee == inlines[in].id)
+              {
+                inlines[in].fileOffs = inlinee->fileId;
+                inlines[in].baseLineNum = inlinee->sourceLineNum;
+              }
+            }
           }
         }
       }
@@ -1658,39 +1693,40 @@ SPDBChunk::SPDBChunk(DXBCFile *dxbc, void *chunk)
   delete[] pages;
 
   // save the filenames in their original order
-  std::vector<std::string> filenames;
+  rdcarray<rdcstr> filenames;
   filenames.reserve(Files.size());
   for(size_t i = 0; i < Files.size(); i++)
     filenames.push_back(Files[i].first);
 
   // Sort files according to the order they come in the Names array, this seems to be more reliable
   // about placing the main file first.
-  std::sort(Files.begin(), Files.end(), [&Names](const std::pair<std::string, std::string> &a,
-                                                 const std::pair<std::string, std::string> &b) {
-    // any entries that aren't found in Names at all (like @cmdline that we add) will be sorted to
-    // the end.
-    size_t aIdx = ~0U, bIdx = ~0U;
+  std::sort(Files.begin(), Files.end(),
+            [&Names](const rdcpair<rdcstr, rdcstr> &a, const rdcpair<rdcstr, rdcstr> &b) {
+              // any entries that aren't found in Names at all (like @cmdline that we add) will be
+              // sorted to
+              // the end.
+              size_t aIdx = ~0U, bIdx = ~0U;
 
-    size_t i = 0;
-    for(auto it = Names.begin(); it != Names.end(); ++it)
-    {
-      if(it->second == a.first)
-        aIdx = i;
-      if(it->second == b.first)
-        bIdx = i;
+              size_t i = 0;
+              for(auto it = Names.begin(); it != Names.end(); ++it)
+              {
+                if(it->second == a.first)
+                  aIdx = i;
+                if(it->second == b.first)
+                  bIdx = i;
 
-      i++;
-    }
+                i++;
+              }
 
-    // if neither were found, sort by filename
-    if(aIdx == bIdx)
-      return a.first < b.first;
+              // if neither were found, sort by filename
+              if(aIdx == bIdx)
+                return a.first < b.first;
 
-    return aIdx < bIdx;
-  });
+              return aIdx < bIdx;
+            });
 
   // create a map from filename -> index
-  std::map<std::string, int32_t> remapping;
+  std::map<rdcstr, int32_t> remapping;
   for(size_t i = 0; i < Files.size(); i++)
     remapping[Files[i].first] = (int32_t)i;
 
@@ -1801,4 +1837,9 @@ void SPDBChunk::GetLocals(size_t instruction, uintptr_t offset,
   }
 }
 
-};    // namespace DXBC
\ No newline at end of file
+IDebugInfo *MakeSPDBChunk(Reflection *reflection, void *data)
+{
+  return new SPDBChunk(reflection, data);
+}
+
+};    // namespace DXBC
