@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,15 +27,9 @@
 
 #include <guiddef.h>
 #include <map>
-#include <string>
-#include <utility>
-#include <vector>
-#include "dxbc_disassemble.h"
-
-using std::vector;
-using std::pair;
-using std::string;
-using std::map;
+#include "api/replay/rdcarray.h"
+#include "api/replay/rdcstr.h"
+#include "dxbc_bytecode.h"
 
 namespace DXBC
 {
@@ -69,7 +63,7 @@ public:
 
 private:
   const byte *direct;
-  vector<byte> contiguous;
+  bytebuf contiguous;
 };
 
 struct FileHeaderPage
@@ -177,8 +171,8 @@ struct DBIModule
   int32_t niCompiler;
 
   // invalid when this is read in-place!
-  string moduleName;
-  string objectName;
+  rdcstr moduleName;
+  rdcstr objectName;
 };
 
 struct CompilandDetails
@@ -194,7 +188,7 @@ struct CompilandDetails
   } FrontendVersion, BackendVersion;
 
   // invalid when this is read in-place!
-  string CompilerSig;
+  rdcstr CompilerSig;
 };
 
 struct FileChecksum
@@ -223,19 +217,19 @@ struct Inlinee
   uint64_t parentPtr;
   uint32_t fileOffs;
   uint32_t baseLineNum;
-  std::vector<InstructionLocation> locations;
+  rdcarray<InstructionLocation> locations;
 };
 
 struct Function
 {
   uint32_t type = 0;
-  std::string name;
+  rdcstr name;
 };
 
 struct PDBStream
 {
   uint32_t byteLength;
-  vector<uint32_t> pageIndices;
+  rdcarray<uint32_t> pageIndices;
 };
 
 struct LocalRange
@@ -256,21 +250,21 @@ struct LocalMapping
   uint32_t regFirstComp;
   uint32_t varFirstComp;
   uint32_t numComps;
-  std::vector<LocalRange> gaps;
+  rdcarray<LocalRange> gaps;
 
   LocalVariableMapping var;
 };
 
-class SPDBChunk : public DXBCDebugChunk
+class SPDBChunk : public IDebugInfo
 {
 public:
-  SPDBChunk(DXBCFile *dxbc, void *data);
+  SPDBChunk(Reflection *dxbc, void *data);
   SPDBChunk(const SPDBChunk &) = delete;
   SPDBChunk &operator=(const SPDBChunk &o) = delete;
 
-  std::string GetCompilerSig() const { return m_CompilerSig; }
-  std::string GetEntryFunction() const { return m_Entry; }
-  std::string GetShaderProfile() const { return m_Profile; }
+  rdcstr GetCompilerSig() const { return m_CompilerSig; }
+  rdcstr GetEntryFunction() const { return m_Entry; }
+  rdcstr GetShaderProfile() const { return m_Profile; }
   uint32_t GetShaderCompileFlags() const { return m_ShaderFlags; }
   void GetLineInfo(size_t instruction, uintptr_t offset, LineColumnInfo &lineInfo) const;
 
@@ -280,14 +274,14 @@ public:
 private:
   bool m_HasDebugInfo;
 
-  std::string m_CompilerSig;
+  rdcstr m_CompilerSig;
 
-  std::string m_Entry;
-  std::string m_Profile;
+  rdcstr m_Entry;
+  rdcstr m_Profile;
 
   uint32_t m_ShaderFlags;
 
-  std::vector<LocalMapping> m_Locals;
+  rdcarray<LocalMapping> m_Locals;
 
   std::map<uint32_t, Function> m_Functions;
   std::map<uint32_t, LineColumnInfo> m_Lines;
