@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,8 @@
 extern "C" int getMetalLayerWidth(void *handle);
 extern "C" int getMetalLayerHeight(void *handle);
 
+#if defined(VK_USE_PLATFORM_MACOS_MVK)
+
 VkResult WrappedVulkan::vkCreateMacOSSurfaceMVK(VkInstance instance,
                                                 const VkMacOSSurfaceCreateInfoMVK *pCreateInfo,
                                                 const VkAllocationCallbacks *pAllocator,
@@ -49,31 +51,88 @@ VkResult WrappedVulkan::vkCreateMacOSSurfaceMVK(VkInstance instance,
 
     WrappedVkSurfaceKHR *wrapped = GetWrapped(*pSurface);
 
-    // since there's no point in allocating a full resource record and storing the window
-    // handle under there somewhere, we just cast. We won't use the resource record for anything
-    wrapped->record = (VkResourceRecord *)(uintptr_t)pCreateInfo->pView;
+    wrapped->record =
+        PackWindowHandleInRecord(WindowingSystem::MacOS, (void *)(uintptr_t)pCreateInfo->pView);
   }
 
   return ret;
 }
 
+#endif
+
+#if defined(VK_USE_PLATFORM_METAL_EXT)
+
+VkResult WrappedVulkan::vkCreateMetalSurfaceEXT(VkInstance instance,
+                                                const VkMetalSurfaceCreateInfoEXT *pCreateInfo,
+                                                const VkAllocationCallbacks *pAllocator,
+                                                VkSurfaceKHR *pSurface)
+{
+  // should not come in here at all on replay
+  RDCASSERT(IsCaptureMode(m_State));
+
+  VkResult ret =
+      ObjDisp(instance)->CreateMetalSurfaceEXT(Unwrap(instance), pCreateInfo, pAllocator, pSurface);
+
+  if(ret == VK_SUCCESS)
+  {
+    GetResourceManager()->WrapResource(Unwrap(instance), *pSurface);
+
+    WrappedVkSurfaceKHR *wrapped = GetWrapped(*pSurface);
+
+    wrapped->record =
+        PackWindowHandleInRecord(WindowingSystem::MacOS, (void *)(uintptr_t)pCreateInfo->pLayer);
+  }
+
+  return ret;
+}
+
+#endif
+
 void VulkanReplay::OutputWindow::SetWindowHandle(WindowingData window)
 {
   RDCASSERT(window.system == WindowingSystem::MacOS, window.system);
   wnd = window.macOS.layer;
 }
 
-void VulkanReplay::OutputWindow::CreateSurface(VkInstance inst)
+void VulkanReplay::OutputWindow::CreateSurface(WrappedVulkan *driver, VkInstance inst)
 {
-  VkMacOSSurfaceCreateInfoMVK createInfo;
+#if defined(VK_USE_PLATFORM_METAL_EXT)
+  if(driver->GetExtensions(GetRecord(inst)).ext_EXT_metal_surface)
+  {
+    VkMetalSurfaceCreateInfoEXT createInfo;
 
-  createInfo.sType = VK_STRUCTURE_TYPE_MACOS_SURFACE_CREATE_INFO_MVK;
-  createInfo.pNext = NULL;
-  createInfo.flags = 0;
-  createInfo.pView = wnd;
+    createInfo.sType = VK_STRUCTURE_TYPE_METAL_SURFACE_CREATE_INFO_EXT;
+    createInfo.pNext = NULL;
+    createInfo.flags = 0;
+    createInfo.pLayer = wnd;
 
-  VkResult vkr = ObjDisp(inst)->CreateMacOSSurfaceMVK(Unwrap(inst), &createInfo, NULL, &surface);
-  RDCASSERTEQUAL(vkr, VK_SUCCESS);
+    RDCDEBUG("Creating macOS surface with EXT_metal_surface");
+
+    VkResult vkr = ObjDisp(inst)->CreateMetalSurfaceEXT(Unwrap(inst), &createInfo, NULL, &surface);
+    RDCASSERTEQUAL(vkr, VK_SUCCESS);
+    return;
+  }
+#endif
+
+#if defined(VK_USE_PLATFORM_MACOS_MVK)
+  if(driver->GetExtensions(GetRecord(inst)).ext_MVK_macos_surface)
+  {
+    VkMacOSSurfaceCreateInfoMVK createInfo;
+
+    createInfo.sType = VK_STRUCTURE_TYPE_MACOS_SURFACE_CREATE_INFO_MVK;
+    createInfo.pNext = NULL;
+    createInfo.flags = 0;
+    createInfo.pView = wnd;
+
+    RDCDEBUG("Creating macOS surface with MVK_macos_surface");
+
+    VkResult vkr = ObjDisp(inst)->CreateMacOSSurfaceMVK(Unwrap(inst), &createInfo, NULL, &surface);
+    RDCASSERTEQUAL(vkr, VK_SUCCESS);
+    return;
+  }
+#endif
+
+  RDCERR("No enabled macOS surface extension");
 }
 
 void VulkanReplay::GetOutputWindowDimensions(uint64_t id, int32_t &w, int32_t &h)
@@ -83,17 +142,24 @@ void VulkanReplay::GetOutputWindowDimensions(uint64_t id, int32_t &w, int32_t &h
 
   OutputWindow &outw = m_OutputWindows[id];
 
+  if(outw.m_WindowSystem == WindowingSystem::Headless)
+  {
+    w = outw.width;
+    h = outw.height;
+    return;
+  }
+
   w = getMetalLayerWidth(outw.wnd);
   h = getMetalLayerHeight(outw.wnd);
 }
 
-static const char *VulkanLibraryName = "libvulkan.1.dylib";
+static const rdcstr VulkanLibraryName = "libvulkan.1.dylib"_lit;
 
 void *LoadVulkanLibrary()
 {
   // first try to load the module globally. If so we assume the user has a global (or at least
   // user-wide) configuration that we should use.
-  void *ret = Process::LoadModule(VulkanLibraryName);
+  void *ret = Process::LoadModule(VulkanLibraryName.c_str());
 
   if(ret)
   {
@@ -101,8 +167,17 @@ void *LoadVulkanLibrary()
     return ret;
   }
 
+  // then try the standard SDK install path under /usr/local/lib
+  ret = Process::LoadModule(("/usr/local/lib/" + VulkanLibraryName).c_str());
+
+  if(ret)
+  {
+    RDCLOG("Loaded /usr/local/lib/libvulkan.1.dylib, using installed MoltenVK environment");
+    return ret;
+  }
+
   // if not, we fall back to our embedded libvulkan and also force use of our embedded ICD.
-  std::string libpath;
+  rdcstr libpath;
   FileIO::GetLibraryFilename(libpath);
   libpath = get_dirname(libpath) + "/../plugins/MoltenVK/";
 
@@ -115,4 +190,4 @@ void *LoadVulkanLibrary()
   Process::ApplyEnvironmentModification();
 
   return Process::LoadModule((libpath + VulkanLibraryName).c_str());
-}
\ No newline at end of file
+}
