@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,12 +23,11 @@
  ******************************************************************************/
 
 #include <float.h>
-#include "3rdparty/glslang/SPIRV/GLSL.std.450.h"
-#include "3rdparty/glslang/SPIRV/spirv.hpp"
-#include "driver/shaders/spirv/spirv_common.h"
 #include "driver/shaders/spirv/spirv_editor.h"
+#include "driver/shaders/spirv/spirv_op_helpers.h"
 #include "vk_core.h"
 #include "vk_debug.h"
+#include "vk_replay.h"
 #include "vk_shader_cache.h"
 
 struct feedbackData
@@ -38,40 +37,40 @@ struct feedbackData
 };
 
 void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
-                    const std::map<SPIRVBinding, feedbackData> &offsetMap, VkDeviceAddress addr,
-                    std::vector<uint32_t> &modSpirv)
+                    const std::map<rdcspv::Binding, feedbackData> &offsetMap, VkDeviceAddress addr,
+                    bool bufferAddressKHR, rdcarray<uint32_t> &modSpirv)
 {
-  SPIRVEditor editor(modSpirv);
+  rdcspv::Editor editor(modSpirv);
+
+  editor.Prepare();
 
   const bool useBufferAddress = (addr != 0);
 
-  SPIRVId uint32ID = editor.DeclareType(scalar<uint32_t>());
-  SPIRVId int32ID = editor.DeclareType(scalar<int32_t>());
-  SPIRVId uint64ID, int64ID;
-  SPIRVId uint32StructID;
-  SPIRVId funcParamType;
+  rdcspv::Id uint32ID = editor.DeclareType(rdcspv::scalar<uint32_t>());
+  rdcspv::Id int32ID = editor.DeclareType(rdcspv::scalar<int32_t>());
+  rdcspv::Id uint64ID, int64ID;
+  rdcspv::Id uint32StructID;
+  rdcspv::Id funcParamType;
 
   if(useBufferAddress)
   {
     // declare the int64 types we'll need
-    uint64ID = editor.DeclareType(scalar<uint64_t>());
-    int64ID = editor.DeclareType(scalar<int64_t>());
+    uint64ID = editor.DeclareType(rdcspv::scalar<uint64_t>());
+    int64ID = editor.DeclareType(rdcspv::scalar<int64_t>());
 
-    uint32StructID = editor.AddType(SPIRVOperation(spv::OpTypeStruct, {editor.MakeId(), uint32ID}));
+    uint32StructID = editor.AddType(rdcspv::OpTypeStruct(editor.MakeId(), {uint32ID}));
 
     // any function parameters we add are uint64 byte offsets
     funcParamType = uint64ID;
   }
   else
   {
-    SPIRVId runtimeArrayID =
-        editor.AddType(SPIRVOperation(spv::OpTypeRuntimeArray, {editor.MakeId(), uint32ID}));
+    rdcspv::Id runtimeArrayID = editor.AddType(rdcspv::OpTypeRuntimeArray(editor.MakeId(), uint32ID));
 
-    editor.AddDecoration(SPIRVOperation(
-        spv::OpDecorate, {runtimeArrayID, spv::DecorationArrayStride, sizeof(uint32_t)}));
+    editor.AddDecoration(rdcspv::OpDecorate(
+        runtimeArrayID, rdcspv::DecorationParam<rdcspv::Decoration::ArrayStride>(sizeof(uint32_t))));
 
-    uint32StructID =
-        editor.AddType(SPIRVOperation(spv::OpTypeStruct, {editor.MakeId(), runtimeArrayID}));
+    uint32StructID = editor.AddType(rdcspv::OpTypeStruct(editor.MakeId(), {runtimeArrayID}));
 
     // any function parameters we add are uint32 indices
     funcParamType = uint32ID;
@@ -79,25 +78,25 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
 
   editor.SetName(uint32StructID, "__rd_feedbackStruct");
 
-  editor.AddDecoration(
-      SPIRVOperation(spv::OpMemberDecorate, {uint32StructID, 0, spv::DecorationOffset, 0}));
+  editor.AddDecoration(rdcspv::OpMemberDecorate(
+      uint32StructID, 0, rdcspv::DecorationParam<rdcspv::Decoration::Offset>(0)));
 
   // map from variable ID to watch, to variable ID to get offset from (as a SPIR-V constant,
   // or as either uint64 byte offset for buffer addressing or uint32 ssbo index otherwise)
-  std::map<SPIRVId, SPIRVId> varLookup;
+  std::map<rdcspv::Id, rdcspv::Id> varLookup;
 
   // iterate over all variables. We do this here because in the absence of the buffer address
   // extension we might declare our own below and patch bindings - so we need to look these up now
-  for(const SPIRVVariable &var : editor.GetVariables())
+  for(const rdcspv::Variable &var : editor.GetGlobals())
   {
     // skip variables without one of these storage classes, as they are not descriptors
-    if(var.storageClass != spv::StorageClassUniformConstant &&
-       var.storageClass != spv::StorageClassUniform &&
-       var.storageClass != spv::StorageClassStorageBuffer)
+    if(var.storage != rdcspv::StorageClass::UniformConstant &&
+       var.storage != rdcspv::StorageClass::Uniform &&
+       var.storage != rdcspv::StorageClass::StorageBuffer)
       continue;
 
     // get this variable's binding info
-    SPIRVBinding bind = editor.GetBinding(var.id);
+    rdcspv::Binding bind = editor.GetBinding(var.id);
 
     // if this is one of the bindings we care about
     auto it = offsetMap.find(bind);
@@ -106,7 +105,7 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
       // store the offset for this variable so we watch for access chains and know where to store to
       if(useBufferAddress)
       {
-        SPIRVId id = varLookup[var.id] = editor.AddConstantImmediate<uint64_t>(it->second.offset);
+        rdcspv::Id id = varLookup[var.id] = editor.AddConstantImmediate<uint64_t>(it->second.offset);
 
         editor.SetName(
             id, StringFormat::Fmt("__feedbackOffset_set%u_bind%u", it->first.set, it->first.binding)
@@ -117,7 +116,7 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
         // check that the offset fits in 32-bit word, convert byte offset to uint32 index
         uint64_t index = it->second.offset / 4;
         RDCASSERT(index < 0xFFFFFFFFULL, bind.set, bind.binding, it->second.offset);
-        SPIRVId id = varLookup[var.id] = editor.AddConstantImmediate<uint32_t>(uint32_t(index));
+        rdcspv::Id id = varLookup[var.id] = editor.AddConstantImmediate<uint32_t>(uint32_t(index));
 
         editor.SetName(
             id, StringFormat::Fmt("__feedbackIndex_set%u_bind%u", it->first.set, it->first.binding)
@@ -126,40 +125,44 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
     }
   }
 
-  SPIRVId bufferAddressConst, ssboVar, uint32ptrtype;
+  rdcspv::Id bufferAddressConst, ssboVar, uint32ptrtype;
 
   if(useBufferAddress)
   {
     // add the extension
-    editor.AddExtension("SPV_EXT_physical_storage_buffer");
+    editor.AddExtension(bufferAddressKHR ? "SPV_KHR_physical_storage_buffer"
+                                         : "SPV_EXT_physical_storage_buffer");
 
     // change the memory model to physical storage buffer 64
-    SPIRVOperation op(editor.Begin(SPIRVSection::MemoryModel));
-    op[1] = spv::AddressingModelPhysicalStorageBuffer64EXT;
+    rdcspv::Iter it = editor.Begin(rdcspv::Section::MemoryModel);
+    rdcspv::OpMemoryModel model(it);
+    model.addressingModel = rdcspv::AddressingModel::PhysicalStorageBuffer64;
+    it = model;
 
     // add capabilities
-    editor.AddCapability(spv::CapabilityPhysicalStorageBufferAddressesEXT);
-    editor.AddCapability(spv::CapabilityInt64);
+    editor.AddCapability(rdcspv::Capability::PhysicalStorageBufferAddresses);
+    editor.AddCapability(rdcspv::Capability::Int64);
 
     // declare the address constants and make our pointers physical storage buffer pointers
     bufferAddressConst = editor.AddConstantImmediate<uint64_t>(addr);
     uint32ptrtype =
-        editor.DeclareType(SPIRVPointer(uint32ID, spv::StorageClassPhysicalStorageBufferEXT));
+        editor.DeclareType(rdcspv::Pointer(uint32ID, rdcspv::StorageClass::PhysicalStorageBuffer));
 
     editor.SetName(bufferAddressConst, "__rd_feedbackAddress");
 
     // struct is block decorated
-    editor.AddDecoration(SPIRVOperation(spv::OpDecorate, {uint32StructID, spv::DecorationBlock}));
+    editor.AddDecoration(rdcspv::OpDecorate(uint32StructID, rdcspv::Decoration::Block));
   }
   else
   {
     // the pointers are uniform pointers
-    SPIRVId bufptrtype = editor.DeclareType(SPIRVPointer(uint32StructID, spv::StorageClassUniform));
-    uint32ptrtype = editor.DeclareType(SPIRVPointer(uint32ID, spv::StorageClassUniform));
+    rdcspv::Id bufptrtype =
+        editor.DeclareType(rdcspv::Pointer(uint32StructID, rdcspv::StorageClass::Uniform));
+    uint32ptrtype = editor.DeclareType(rdcspv::Pointer(uint32ID, rdcspv::StorageClass::Uniform));
 
     // patch all bindings up by 1
-    for(SPIRVIterator it = editor.Begin(SPIRVSection::Annotations),
-                      end = editor.End(SPIRVSection::Annotations);
+    for(rdcspv::Iter it = editor.Begin(rdcspv::Section::Annotations),
+                     end = editor.End(rdcspv::Section::Annotations);
         it < end; ++it)
     {
       // we will use descriptor set 0 for our own purposes if we don't have a buffer address.
@@ -169,43 +172,46 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
       // where all descriptor sets are already used. In theory we only have to do this with set 0,
       // but that requires knowing which variables are in set 0 and it's simpler to increase all
       // bindings.
-      if(it.opcode() == spv::OpDecorate && it.word(2) == spv::DecorationBinding)
+      if(it.opcode() == rdcspv::Op::Decorate)
       {
-        RDCASSERT(it.word(3) != 0xffffffff);
-        it.word(3) += 1;
+        rdcspv::OpDecorate dec(it);
+        if(dec.decoration == rdcspv::Decoration::Binding)
+        {
+          RDCASSERT(dec.decoration.binding != 0xffffffff);
+          dec.decoration.binding += 1;
+          it = dec;
+        }
       }
     }
 
     // add our SSBO variable, at set 0 binding 0
     ssboVar = editor.MakeId();
-    editor.AddVariable(
-        SPIRVOperation(spv::OpVariable, {bufptrtype, ssboVar, spv::StorageClassUniform}));
+    editor.AddVariable(rdcspv::OpVariable(bufptrtype, ssboVar, rdcspv::StorageClass::Uniform));
     editor.AddDecoration(
-        SPIRVOperation(spv::OpDecorate, {ssboVar, (uint32_t)spv::DecorationDescriptorSet, 0}));
+        rdcspv::OpDecorate(ssboVar, rdcspv::DecorationParam<rdcspv::Decoration::DescriptorSet>(0)));
     editor.AddDecoration(
-        SPIRVOperation(spv::OpDecorate, {ssboVar, (uint32_t)spv::DecorationBinding, 0}));
+        rdcspv::OpDecorate(ssboVar, rdcspv::DecorationParam<rdcspv::Decoration::Binding>(0)));
 
     editor.SetName(ssboVar, "__rd_feedbackBuffer");
 
     // struct is bufferblock decorated
-    editor.AddDecoration(
-        SPIRVOperation(spv::OpDecorate, {uint32StructID, (uint32_t)spv::DecorationBufferBlock}));
+    editor.AddDecoration(rdcspv::OpDecorate(uint32StructID, rdcspv::Decoration::BufferBlock));
   }
 
-  SPIRVId rtarrayOffset = editor.AddConstantImmediate<uint32_t>(0U);
-  SPIRVId usedValue = editor.AddConstantImmediate<uint32_t>(0xFFFFFFFFU);
-  SPIRVId scope = editor.AddConstantImmediate<uint32_t>(spv::ScopeInvocation);
-  SPIRVId semantics = editor.AddConstantImmediate<uint32_t>(0U);
-  SPIRVId uint32shift = editor.AddConstantImmediate<uint32_t>(2U);
+  rdcspv::Id rtarrayOffset = editor.AddConstantImmediate<uint32_t>(0U);
+  rdcspv::Id usedValue = editor.AddConstantImmediate<uint32_t>(0xFFFFFFFFU);
+  rdcspv::Id scope = editor.AddConstantImmediate<uint32_t>((uint32_t)rdcspv::Scope::Invocation);
+  rdcspv::Id semantics = editor.AddConstantImmediate<uint32_t>(0U);
+  rdcspv::Id uint32shift = editor.AddConstantImmediate<uint32_t>(2U);
 
-  std::map<SPIRVId, SPIRVScalar> intTypeLookup;
+  std::map<rdcspv::Id, rdcspv::Scalar> intTypeLookup;
 
-  for(auto scalarType : editor.GetTypeInfo<SPIRVScalar>())
-    if(scalarType.first.type == spv::OpTypeInt)
+  for(auto scalarType : editor.GetTypeInfo<rdcspv::Scalar>())
+    if(scalarType.first.type == rdcspv::Op::TypeInt)
       intTypeLookup[scalarType.second] = scalarType.first;
 
-  SPIRVId entryID;
-  for(const SPIRVEntry &entry : editor.GetEntries())
+  rdcspv::Id entryID;
+  for(const rdcspv::EntryPoint &entry : editor.GetEntries())
   {
     if(entry.name == entryName)
     {
@@ -214,14 +220,14 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
     }
   }
 
-  SPIRVTypeIds<SPIRVFunction> funcTypes = editor.GetTypes<SPIRVFunction>();
+  rdcspv::TypeToIds<rdcspv::FunctionType> funcTypes = editor.GetTypes<rdcspv::FunctionType>();
 
   // functions that have been patched with annotation & extra function parameters if needed
-  std::set<SPIRVId> patchedFunctions;
+  std::set<rdcspv::Id> patchedFunctions;
 
   // functions we need to patch, with the indices of which parameters have bindings coming along
   // with
-  std::map<SPIRVId, std::vector<size_t>> functionPatchQueue;
+  std::map<rdcspv::Id, rdcarray<size_t>> functionPatchQueue;
 
   // start with the entry point, with no parameters to patch
   functionPatchQueue[entryID] = {};
@@ -229,8 +235,8 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
   // now keep patching functions until we have no more to patch
   while(!functionPatchQueue.empty())
   {
-    SPIRVId funcId;
-    std::vector<size_t> patchArgIndices;
+    rdcspv::Id funcId;
+    rdcarray<size_t> patchArgIndices;
 
     {
       auto it = functionPatchQueue.begin();
@@ -241,28 +247,36 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
       patchedFunctions.insert(funcId);
     }
 
-    SPIRVIterator it = editor.GetID(funcId);
+    rdcspv::Iter it = editor.GetID(funcId);
 
-    RDCASSERT(it.opcode() == spv::OpFunction);
+    RDCASSERT(it.opcode() == rdcspv::Op::Function);
 
     if(!patchArgIndices.empty())
     {
+      rdcspv::OpFunction func(it);
+
       // find the function's type declaration, add the necessary arguments, redeclare and patch it
-      for(const SPIRVTypeId<SPIRVFunction> &funcType : funcTypes)
+      for(const rdcspv::TypeToId<rdcspv::FunctionType> &funcType : funcTypes)
       {
-        if(funcType.second == it.word(4))
+        if(funcType.second == func.functionType)
         {
-          SPIRVFunction patchedFuncType = funcType.first;
+          rdcspv::FunctionType patchedFuncType = funcType.first;
           for(size_t i = 0; i < patchArgIndices.size(); i++)
             patchedFuncType.argumentIds.push_back(funcParamType);
 
-          SPIRVId newFuncTypeID = editor.DeclareType(patchedFuncType);
+          rdcspv::Id newFuncTypeID = editor.DeclareType(patchedFuncType);
 
           // re-fetch the iterator as it might have moved with the type declaration
           it = editor.GetID(funcId);
 
           // change the declared function type
-          it.word(4) = newFuncTypeID;
+          func.functionType = newFuncTypeID;
+
+          editor.PreModify(it);
+
+          it = func;
+
+          editor.PostModify(it);
 
           break;
         }
@@ -272,19 +286,21 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
     ++it;
 
     // onto the OpFunctionParameters. First allocate IDs for all our new function parameters
-    std::vector<SPIRVId> patchedParamIDs;
+    rdcarray<rdcspv::Id> patchedParamIDs;
     for(size_t i = 0; i < patchArgIndices.size(); i++)
       patchedParamIDs.push_back(editor.MakeId());
 
     size_t argIndex = 0;
     size_t watchIndex = 0;
-    while(it.opcode() == spv::OpFunctionParameter)
+    while(it.opcode() == rdcspv::Op::FunctionParameter)
     {
+      rdcspv::OpFunctionParameter param(it);
+
       // if this is a parameter we're patching, add it into varLookup
       if(watchIndex < patchArgIndices.size() && patchArgIndices[watchIndex] == argIndex)
       {
         // when we see use of this parameter, patch it using the added parameter
-        varLookup[it.word(2)] = patchedParamIDs[watchIndex];
+        varLookup[param.result] = patchedParamIDs[watchIndex];
         // watch for the next argument
         watchIndex++;
       }
@@ -296,8 +312,7 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
     // we're past the existing function parameters, now declare our new ones
     for(size_t i = 0; i < patchedParamIDs.size(); i++)
     {
-      editor.AddOperation(
-          it, SPIRVOperation(spv::OpFunctionParameter, {funcParamType, patchedParamIDs[i]}));
+      editor.AddOperation(it, rdcspv::OpFunctionParameter(funcParamType, patchedParamIDs[i]));
       ++it;
     }
 
@@ -305,81 +320,85 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
     for(; it; ++it)
     {
       // finish when we hit the end of the function
-      if(it.opcode() == spv::OpFunctionEnd)
+      if(it.opcode() == rdcspv::Op::FunctionEnd)
         break;
 
       // if we see an OpCopyObject, just add it to the map pointing to the same value
-      if(it.opcode() == spv::OpCopyObject)
+      if(it.opcode() == rdcspv::Op::CopyObject)
       {
-        SPIRVId sourcevar = it.word(3);
+        rdcspv::OpCopyObject copy(it);
 
         // is this a var we want to snoop?
-        auto varIt = varLookup.find(sourcevar);
+        auto varIt = varLookup.find(copy.operand);
         if(varIt != varLookup.end())
         {
-          varLookup[it.word(2)] = varIt->second;
+          varLookup[copy.result] = varIt->second;
         }
       }
 
-      if(it.opcode() == spv::OpFunctionCall)
+      if(it.opcode() == rdcspv::Op::FunctionCall)
       {
+        rdcspv::OpFunctionCall call(it);
+
         // check if any of the variables being passed are ones we care about. Accumulate the added
         // parameters
-        std::vector<uint32_t> funccall;
-        std::vector<size_t> patchArgs;
+        rdcarray<uint32_t> funccall;
+        rdcarray<size_t> patchArgs;
 
         // examine each argument to see if it's one we care about
-        for(size_t i = 4; i < it.size(); i++)
+        for(size_t i = 0; i < call.arguments.size(); i++)
         {
           // if this param we're snooping then pass our offset - whether it's a constant or a
           // function
           // argument itself - into the function call
-          auto varIt = varLookup.find(it.word(i));
+          auto varIt = varLookup.find(call.arguments[i]);
           if(varIt != varLookup.end())
           {
-            funccall.push_back(varIt->second);
-            patchArgs.push_back(i - 4);
+            funccall.push_back(varIt->second.value());
+            patchArgs.push_back(i);
           }
         }
 
+        // if we have parameters to patch, replace the function call
         if(!funccall.empty())
         {
           // prepend all the existing words
           for(size_t i = 1; i < it.size(); i++)
-            funccall.insert(funccall.begin() + i - 1, it.word(i));
+            funccall.insert(i - 1, it.word(i));
 
-          SPIRVIterator oldCall = it;
+          rdcspv::Iter oldCall = it;
 
           // add our patched call afterwards
           it++;
-          editor.AddOperation(it, SPIRVOperation(spv::OpFunctionCall, funccall));
+          editor.AddOperation(it, rdcspv::Operation(rdcspv::Op::FunctionCall, funccall));
 
           // remove the old call
           editor.Remove(oldCall);
-
-          // if this function isn't marked for patching yet, and isn't patched, queue it
-          if(functionPatchQueue[it.word(3)].empty() &&
-             patchedFunctions.find(it.word(3)) == patchedFunctions.end())
-            functionPatchQueue[it.word(3)] = patchArgs;
         }
+
+        // if this function isn't marked for patching yet, and isn't patched, queue it
+        if(functionPatchQueue[call.function].empty() &&
+           patchedFunctions.find(call.function) == patchedFunctions.end())
+          functionPatchQueue[call.function] = patchArgs;
       }
 
       // if we see an access chain of a variable we're snooping, save out the result
-      if(it.opcode() == spv::OpAccessChain || it.opcode() == spv::OpInBoundsAccessChain)
+      if(it.opcode() == rdcspv::Op::AccessChain || it.opcode() == rdcspv::Op::InBoundsAccessChain)
       {
-        SPIRVId sourcevar = it.word(3);
+        rdcspv::OpAccessChain chain(it);
+        chain.op = it.opcode();
 
         // is this a var we want to snoop?
-        auto varIt = varLookup.find(sourcevar);
+        auto varIt = varLookup.find(chain.base);
         if(varIt != varLookup.end())
         {
           // multi-dimensional arrays of descriptors is not allowed - however an access chain could
           // be longer than 5 words (1 index). Think of the case of a uniform buffer where the first
           // index goes into the descriptor array, and further indices go inside the uniform buffer
           // members.
-          RDCASSERT(it.size() >= 5, it.size());
+          RDCASSERT(chain.indexes.size() >= 1, chain.indexes.size());
 
-          SPIRVId index = it.word(4);
+          rdcspv::Id index = chain.indexes[0];
 
           // patch after the access chain
           it++;
@@ -387,15 +406,15 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
           // upcast the index to uint32 or uint64 depending on which path we're taking
           uint32_t targetIndexWidth = useBufferAddress ? 64 : 32;
           {
-            SPIRVId indexType = editor.GetIDType(index);
+            rdcspv::Id indexType = editor.GetIDType(index);
 
-            if(indexType == SPIRVId())
+            if(indexType == rdcspv::Id())
             {
-              RDCERR("Unknown type for ID %u, defaulting to uint32_t", index);
+              RDCERR("Unknown type for ID %u, defaulting to uint32_t", index.value());
               indexType = uint32ID;
             }
 
-            SPIRVScalar indexTypeData = scalar<uint32_t>();
+            rdcspv::Scalar indexTypeData = rdcspv::scalar<uint32_t>();
             auto indexTypeIt = intTypeLookup.find(indexType);
 
             if(indexTypeIt != intTypeLookup.end())
@@ -404,7 +423,7 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
             }
             else
             {
-              RDCERR("Unknown index type ID %u, defaulting to uint32_t", indexType);
+              RDCERR("Unknown index type ID %u, defaulting to uint32_t", indexType.value());
             }
 
             // if it's signed, bitcast it to unsigned
@@ -412,10 +431,9 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
             {
               indexTypeData.signedness = false;
 
-              SPIRVId unsignedIndex = editor.MakeId();
+              rdcspv::Id unsignedIndex = editor.MakeId();
               editor.AddOperation(
-                  it, SPIRVOperation(spv::OpBitcast,
-                                     {editor.DeclareType(indexTypeData), unsignedIndex, index}));
+                  it, rdcspv::OpBitcast(editor.DeclareType(indexTypeData), unsignedIndex, index));
               it++;
 
               index = unsignedIndex;
@@ -424,18 +442,17 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
             // if it's not wide enough, uconvert expand it
             if(indexTypeData.width != targetIndexWidth)
             {
-              SPIRVId extendedtype =
-                  editor.DeclareType(SPIRVScalar(spv::OpTypeInt, targetIndexWidth, false));
-              SPIRVId extendedindex = editor.MakeId();
-              editor.AddOperation(
-                  it, SPIRVOperation(spv::OpUConvert, {extendedtype, extendedindex, index}));
+              rdcspv::Id extendedtype =
+                  editor.DeclareType(rdcspv::Scalar(rdcspv::Op::TypeInt, targetIndexWidth, false));
+              rdcspv::Id extendedindex = editor.MakeId();
+              editor.AddOperation(it, rdcspv::OpUConvert(extendedtype, extendedindex, index));
               it++;
 
               index = extendedindex;
             }
           }
 
-          SPIRVId bufptr;
+          rdcspv::Id bufptr;
 
           if(useBufferAddress)
           {
@@ -443,30 +460,28 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
 
             // get our output slot address by adding an offset to the base pointer
             // baseaddr = bufferAddressConst + bindingOffset
-            SPIRVId baseaddr = editor.MakeId();
-            editor.AddOperation(it, SPIRVOperation(spv::OpIAdd, {uint64ID, baseaddr,
-                                                                 bufferAddressConst, varIt->second}));
+            rdcspv::Id baseaddr = editor.MakeId();
+            editor.AddOperation(
+                it, rdcspv::OpIAdd(uint64ID, baseaddr, bufferAddressConst, varIt->second));
             it++;
 
             // shift the index since this is a byte offset
             // shiftedindex = index << uint32shift
-            SPIRVId shiftedindex = editor.MakeId();
-            editor.AddOperation(it, SPIRVOperation(spv::OpShiftLeftLogical,
-                                                   {uint64ID, shiftedindex, index, uint32shift}));
+            rdcspv::Id shiftedindex = editor.MakeId();
+            editor.AddOperation(
+                it, rdcspv::OpShiftLeftLogical(uint64ID, shiftedindex, index, uint32shift));
             it++;
 
             // add the index on top of that
             // offsetaddr = baseaddr + shiftedindex
-            SPIRVId offsetaddr = editor.MakeId();
-            editor.AddOperation(
-                it, SPIRVOperation(spv::OpIAdd, {uint64ID, offsetaddr, baseaddr, shiftedindex}));
+            rdcspv::Id offsetaddr = editor.MakeId();
+            editor.AddOperation(it, rdcspv::OpIAdd(uint64ID, offsetaddr, baseaddr, shiftedindex));
             it++;
 
             // make a pointer out of it
             // uint32_t *bufptr = (uint32_t *)offsetaddr
             bufptr = editor.MakeId();
-            editor.AddOperation(
-                it, SPIRVOperation(spv::OpConvertUToPtr, {uint32ptrtype, bufptr, offsetaddr}));
+            editor.AddOperation(it, rdcspv::OpConvertUToPtr(uint32ptrtype, bufptr, offsetaddr));
             it++;
           }
           else
@@ -475,25 +490,22 @@ void AnnotateShader(const SPIRVPatchData &patchData, const char *entryName,
 
             // add the index to this binding's base index
             // ssboindex = bindingOffset + index
-            SPIRVId ssboindex = editor.MakeId();
-            editor.AddOperation(
-                it, SPIRVOperation(spv::OpIAdd, {uint32ID, ssboindex, index, varIt->second}));
+            rdcspv::Id ssboindex = editor.MakeId();
+            editor.AddOperation(it, rdcspv::OpIAdd(uint32ID, ssboindex, index, varIt->second));
             it++;
 
             // accesschain to get the pointer we'll atomic into.
             // accesschain is 0 to access rtarray (first member) then ssboindex for array index
             // uint32_t *bufptr = (uint32_t *)&buf.rtarray[ssboindex];
             bufptr = editor.MakeId();
-            editor.AddOperation(
-                it, SPIRVOperation(spv::OpAccessChain,
-                                   {uint32ptrtype, bufptr, ssboVar, rtarrayOffset, ssboindex}));
+            editor.AddOperation(it, rdcspv::OpAccessChain(uint32ptrtype, bufptr, ssboVar,
+                                                          {rtarrayOffset, ssboindex}));
             it++;
           }
 
           // atomically set the uint32 that's pointed to
-          editor.AddOperation(
-              it, SPIRVOperation(spv::OpAtomicUMax,
-                                 {uint32ID, editor.MakeId(), bufptr, scope, semantics, usedValue}));
+          editor.AddOperation(it, rdcspv::OpAtomicUMax(uint32ID, editor.MakeId(), bufptr, scope,
+                                                       semantics, usedValue));
 
           // no it++ here, it will happen implicitly on loop continue
         }
@@ -516,8 +528,11 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
   // if it actually has any data in it later.
   DynamicUsedBinds &result = m_BindlessFeedback.Usage[eventId];
 
-  bool useBufferAddress =
-      ObjDisp(m_Device)->GetBufferDeviceAddressEXT && m_pDriver->GetDeviceFeatures().shaderInt64;
+  bool useBufferAddress = (m_pDriver->GetExtensions(NULL).ext_KHR_buffer_device_address ||
+                           m_pDriver->GetExtensions(NULL).ext_EXT_buffer_device_address) &&
+                          m_pDriver->GetDeviceFeatures().shaderInt64;
+
+  bool useBufferAddressKHR = m_pDriver->GetExtensions(NULL).ext_KHR_buffer_device_address;
 
   const VulkanRenderState &state = m_pDriver->m_RenderState;
   VulkanCreationInfo &creationInfo = m_pDriver->m_CreationInfo;
@@ -538,13 +553,13 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
 
   VkDeviceSize feedbackStorageSize = 0;
 
-  std::map<SPIRVBinding, feedbackData> offsetMap;
+  std::map<rdcspv::Binding, feedbackData> offsetMap;
 
   {
-    const std::vector<ResourceId> &descSetLayoutIds =
+    const rdcarray<ResourceId> &descSetLayoutIds =
         creationInfo.m_PipelineLayout[pipeInfo.layout].descSetLayouts;
 
-    SPIRVBinding key;
+    rdcspv::Binding key;
 
     for(size_t set = 0; set < descSetLayoutIds.size(); set++)
     {
@@ -605,17 +620,23 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
   VkDeviceAddress bufferAddress = 0;
 
   VkDescriptorPool descpool = VK_NULL_HANDLE;
-  std::vector<VkDescriptorSetLayout> setLayouts;
-  std::vector<VkDescriptorSet> descSets;
+  rdcarray<VkDescriptorSetLayout> setLayouts;
+  rdcarray<VkDescriptorSet> descSets;
 
   VkPipelineLayout pipeLayout = VK_NULL_HANDLE;
 
   if(useBufferAddress)
   {
-    VkBufferDeviceAddressInfoEXT getAddressInfo = {VK_STRUCTURE_TYPE_BUFFER_DEVICE_ADDRESS_INFO_EXT};
+    RDCCOMPILE_ASSERT(VK_STRUCTURE_TYPE_BUFFER_DEVICE_ADDRESS_INFO ==
+                          VK_STRUCTURE_TYPE_BUFFER_DEVICE_ADDRESS_INFO_EXT,
+                      "KHR and EXT buffer_device_address should be interchangeable here.");
+    VkBufferDeviceAddressInfo getAddressInfo = {VK_STRUCTURE_TYPE_BUFFER_DEVICE_ADDRESS_INFO};
     getAddressInfo.buffer = m_BindlessFeedback.FeedbackBuffer.buf;
 
-    bufferAddress = m_pDriver->vkGetBufferDeviceAddressEXT(dev, &getAddressInfo);
+    if(useBufferAddressKHR)
+      bufferAddress = m_pDriver->vkGetBufferDeviceAddress(dev, &getAddressInfo);
+    else
+      bufferAddress = m_pDriver->vkGetBufferDeviceAddressEXT(dev, &getAddressInfo);
   }
   else
   {
@@ -638,7 +659,7 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
 
     // create pipeline layout with new descriptor set layouts
     {
-      const std::vector<VkPushConstantRange> &push =
+      const rdcarray<VkPushConstantRange> &push =
           creationInfo.m_PipelineLayout[pipeInfo.layout].pushRanges;
 
       VkPipelineLayoutCreateInfo pipeLayoutInfo = {
@@ -692,9 +713,10 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
     const VulkanCreationInfo::ShaderModule &moduleInfo =
         creationInfo.m_ShaderModule[pipeInfo.shaders[5].module];
 
-    std::vector<uint32_t> modSpirv = moduleInfo.spirv.spirv;
+    rdcarray<uint32_t> modSpirv = moduleInfo.spirv.GetSPIRV();
 
-    AnnotateShader(*pipeInfo.shaders[5].patchData, stage.pName, offsetMap, bufferAddress, modSpirv);
+    AnnotateShader(*pipeInfo.shaders[5].patchData, stage.pName, offsetMap, bufferAddress,
+                   useBufferAddressKHR, modSpirv);
 
     moduleCreateInfo.pCode = modSpirv.data();
     moduleCreateInfo.codeSize = modSpirv.size() * sizeof(uint32_t);
@@ -716,10 +738,10 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
       const VulkanCreationInfo::ShaderModule &moduleInfo =
           creationInfo.m_ShaderModule[pipeInfo.shaders[idx].module];
 
-      std::vector<uint32_t> modSpirv = moduleInfo.spirv.spirv;
+      rdcarray<uint32_t> modSpirv = moduleInfo.spirv.GetSPIRV();
 
       AnnotateShader(*pipeInfo.shaders[idx].patchData, stage.pName, offsetMap, bufferAddress,
-                     modSpirv);
+                     useBufferAddressKHR, modSpirv);
 
       moduleCreateInfo.pCode = modSpirv.data();
       moduleCreateInfo.codeSize = modSpirv.size() * sizeof(uint32_t);
@@ -762,7 +784,10 @@ void VulkanReplay::FetchShaderFeedback(uint32_t eventId)
       modifiedpipe.descSets.resize(1);
 
     for(size_t i = 0; i < descSets.size(); i++)
+    {
+      modifiedpipe.descSets[i].pipeLayout = GetResID(pipeLayout);
       modifiedpipe.descSets[i].descSet = GetResID(descSets[i]);
+    }
   }
 
   {
