@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,102 +27,136 @@
 #include "vk_manager.h"
 #include "vk_resources.h"
 
-// utility struct for firing one-shot command buffers to begin/end markers
-struct ScopedCommandBuffer
-{
-  ScopedCommandBuffer(VkCommandBuffer cmdbuf, WrappedVulkan *vk)
-  {
-    core = vk;
-    cmd = cmdbuf;
-    local = (cmd == VK_NULL_HANDLE);
-
-    if(local)
-    {
-      VkCommandBufferBeginInfo beginInfo = {VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO, NULL,
-                                            VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT};
-
-      cmd = vk->GetNextCmd();
-
-      VkResult vkr = ObjDisp(cmd)->BeginCommandBuffer(Unwrap(cmd), &beginInfo);
-      RDCASSERTEQUAL(vkr, VK_SUCCESS);
-    }
-  }
-  ~ScopedCommandBuffer()
-  {
-    if(local)
-    {
-      VkResult vkr = ObjDisp(cmd)->EndCommandBuffer(Unwrap(cmd));
-      RDCASSERTEQUAL(vkr, VK_SUCCESS);
-
-      core->SubmitCmds();
-    }
-  }
-
-  WrappedVulkan *core;
-  VkCommandBuffer cmd;
-  bool local;
-};
-
 WrappedVulkan *VkMarkerRegion::vk = NULL;
 
-VkMarkerRegion::VkMarkerRegion(const std::string &marker, VkCommandBuffer cmd)
+VkMarkerRegion::VkMarkerRegion(VkCommandBuffer cmd, const rdcstr &marker)
 {
   if(cmd == VK_NULL_HANDLE)
-  {
-    RDCERR("Cannot auto-allocate a command buffer for a scoped VkMarkerRegion");
     return;
-  }
 
   cmdbuf = cmd;
   Begin(marker, cmd);
 }
 
+VkMarkerRegion::VkMarkerRegion(VkQueue q, const rdcstr &marker)
+{
+  if(q == VK_NULL_HANDLE)
+  {
+    if(vk)
+      q = vk->GetQ();
+    else
+      return;
+  }
+
+  queue = q;
+  Begin(marker, q);
+}
+
 VkMarkerRegion::~VkMarkerRegion()
 {
-  if(cmdbuf)
+  if(queue)
+    End(queue);
+  else if(cmdbuf)
     End(cmdbuf);
 }
 
-void VkMarkerRegion::Begin(const std::string &marker, VkCommandBuffer cmd)
+void VkMarkerRegion::Begin(const rdcstr &marker, VkCommandBuffer cmd)
 {
-  if(!vk)
+  if(cmd == VK_NULL_HANDLE)
     return;
 
   // check for presence of the marker extension
-  if(!ObjDisp(vk->GetDev())->CmdDebugMarkerBeginEXT)
+  if(!ObjDisp(cmd)->CmdBeginDebugUtilsLabelEXT)
     return;
 
-  ScopedCommandBuffer scope(cmd, vk);
-
-  VkDebugMarkerMarkerInfoEXT markerInfo = {};
-  markerInfo.sType = VK_STRUCTURE_TYPE_DEBUG_MARKER_MARKER_INFO_EXT;
-  markerInfo.pMarkerName = marker.c_str();
-  ObjDisp(scope.cmd)->CmdDebugMarkerBeginEXT(Unwrap(scope.cmd), &markerInfo);
+  VkDebugUtilsLabelEXT label = {};
+  label.sType = VK_STRUCTURE_TYPE_DEBUG_UTILS_LABEL_EXT;
+  label.pLabelName = marker.c_str();
+  ObjDisp(cmd)->CmdBeginDebugUtilsLabelEXT(Unwrap(cmd), &label);
 }
 
-void VkMarkerRegion::Set(const std::string &marker, VkCommandBuffer cmd)
+void VkMarkerRegion::Set(const rdcstr &marker, VkCommandBuffer cmd)
 {
-  // check for presence of the marker extension
-  if(!ObjDisp(vk->GetDev())->CmdDebugMarkerBeginEXT)
+  if(cmd == VK_NULL_HANDLE)
     return;
 
-  ScopedCommandBuffer scope(cmd, vk);
+  // check for presence of the marker extension
+  if(!ObjDisp(cmd)->CmdInsertDebugUtilsLabelEXT)
+    return;
 
-  VkDebugMarkerMarkerInfoEXT markerInfo = {};
-  markerInfo.sType = VK_STRUCTURE_TYPE_DEBUG_MARKER_MARKER_INFO_EXT;
-  markerInfo.pMarkerName = marker.c_str();
-  ObjDisp(scope.cmd)->CmdDebugMarkerInsertEXT(Unwrap(scope.cmd), &markerInfo);
+  VkDebugUtilsLabelEXT label = {};
+  label.sType = VK_STRUCTURE_TYPE_DEBUG_UTILS_LABEL_EXT;
+  label.pLabelName = marker.c_str();
+  ObjDisp(cmd)->CmdInsertDebugUtilsLabelEXT(Unwrap(cmd), &label);
 }
 
 void VkMarkerRegion::End(VkCommandBuffer cmd)
 {
-  // check for presence of the marker extension
-  if(!ObjDisp(vk->GetDev())->CmdDebugMarkerBeginEXT)
+  if(cmd == VK_NULL_HANDLE)
     return;
 
-  ScopedCommandBuffer scope(cmd, vk);
+  // check for presence of the marker extension
+  if(!ObjDisp(cmd)->CmdEndDebugUtilsLabelEXT)
+    return;
 
-  ObjDisp(scope.cmd)->CmdDebugMarkerEndEXT(Unwrap(scope.cmd));
+  ObjDisp(cmd)->CmdEndDebugUtilsLabelEXT(Unwrap(cmd));
+}
+
+void VkMarkerRegion::Begin(const rdcstr &marker, VkQueue q)
+{
+  if(q == VK_NULL_HANDLE)
+  {
+    if(vk)
+      q = vk->GetQ();
+    else
+      return;
+  }
+
+  // check for presence of the marker extension
+  if(!ObjDisp(q)->QueueBeginDebugUtilsLabelEXT)
+    return;
+
+  VkDebugUtilsLabelEXT label = {};
+  label.sType = VK_STRUCTURE_TYPE_DEBUG_UTILS_LABEL_EXT;
+  label.pLabelName = marker.c_str();
+  ObjDisp(q)->QueueBeginDebugUtilsLabelEXT(Unwrap(q), &label);
+}
+
+void VkMarkerRegion::Set(const rdcstr &marker, VkQueue q)
+{
+  if(q == VK_NULL_HANDLE)
+  {
+    if(vk)
+      q = vk->GetQ();
+    else
+      return;
+  }
+
+  // check for presence of the marker extension
+  if(!ObjDisp(q)->QueueInsertDebugUtilsLabelEXT)
+    return;
+
+  VkDebugUtilsLabelEXT label = {};
+  label.sType = VK_STRUCTURE_TYPE_DEBUG_UTILS_LABEL_EXT;
+  label.pLabelName = marker.c_str();
+  ObjDisp(q)->QueueInsertDebugUtilsLabelEXT(Unwrap(q), &label);
+}
+
+void VkMarkerRegion::End(VkQueue q)
+{
+  if(q == VK_NULL_HANDLE)
+  {
+    if(vk)
+      q = vk->GetQ();
+    else
+      return;
+  }
+
+  // check for presence of the marker extension
+  if(!ObjDisp(q)->QueueEndDebugUtilsLabelEXT)
+    return;
+
+  ObjDisp(q)->QueueEndDebugUtilsLabelEXT(Unwrap(q));
 }
 
 void GPUBuffer::Create(WrappedVulkan *driver, VkDevice dev, VkDeviceSize size, uint32_t ringSize,
@@ -134,10 +168,13 @@ void GPUBuffer::Create(WrappedVulkan *driver, VkDevice dev, VkDeviceSize size, u
 
   align = (VkDeviceSize)driver->GetDeviceProps().limits.minUniformBufferOffsetAlignment;
 
+  // for simplicity, consider the non-coherent atom size also an alignment requirement
+  align = AlignUp(align, driver->GetDeviceProps().limits.nonCoherentAtomSize);
+
   sz = size;
   // offset must be aligned, so ensure we have at least ringSize
   // copies accounting for that
-  totalsize = ringSize == 1 ? size : AlignUp(size, align) * ringSize;
+  totalsize = AlignUp(size, align) * RDCMAX(1U, ringSize);
   curoffset = 0;
 
   ringCount = ringSize;
@@ -163,7 +200,7 @@ void GPUBuffer::Create(WrappedVulkan *driver, VkDevice dev, VkDeviceSize size, u
     bufInfo.usage |= VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT;
 
   if(flags & eGPUBufferAddressable)
-    bufInfo.usage |= VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT_EXT;
+    bufInfo.usage |= VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT;
 
   VkResult vkr = driver->vkCreateBuffer(dev, &bufInfo, NULL, &buf);
   RDCASSERTEQUAL(vkr, VK_SUCCESS);
@@ -208,11 +245,16 @@ void *GPUBuffer::Map(uint32_t *bindoffset, VkDeviceSize usedsize)
   VkDeviceSize offset = bindoffset ? curoffset : 0;
   VkDeviceSize size = usedsize > 0 ? usedsize : sz;
 
-  // wrap around the ring, assuming the ring is large enough
-  // that this memory is now free
+  // align the size so we always consume coherent atoms
+  size = AlignUp(size, align);
+
+  // wrap around the ring as soon as the 'sz' would overflow. This is because if we're using dynamic
+  // offsets in the descriptor the range is still set to that fixed size and the validation
+  // complains if we go off the end (even if it's unused). Rather than constantly update the
+  // descriptor, we just conservatively wrap and waste the last bit of space.
   if(offset + sz > totalsize)
     offset = 0;
-  RDCASSERT(offset + sz <= totalsize);
+  RDCASSERT(offset + size <= totalsize);
 
   // offset must be aligned
   curoffset = AlignUp(offset + size, align);
@@ -270,6 +312,11 @@ bool VkInitParams::IsSupportedVersion(uint64_t ver)
   if(ver == CurrentVersion)
     return true;
 
+  // 0xF -> 0x10 - added serialisation of VkPhysicalDeviceDriverPropertiesKHR into enumerated
+  // physical devices
+  if(ver == 0xF)
+    return true;
+
   // 0xE -> 0xF - serialisation of VkPhysicalDeviceVulkanMemoryModelFeaturesKHR changed in vulkan
   // 1.1.99, adding a new field
   if(ver == 0xE)
@@ -315,10 +362,83 @@ VkAccessFlags MakeAccessMask(VkImageLayout layout)
   return VkAccessFlags(0);
 }
 
-void ReplacePresentableImageLayout(VkImageLayout &layout)
+void SanitiseOldImageLayout(VkImageLayout &layout)
 {
+  // we don't replay with present layouts since we don't create actual swapchains. So change any
+  // present layouts to general layouts
   if(layout == VK_IMAGE_LAYOUT_PRESENT_SRC_KHR || layout == VK_IMAGE_LAYOUT_SHARED_PRESENT_KHR)
     layout = VK_IMAGE_LAYOUT_GENERAL;
+
+  // we can't transition to PREINITIALIZED, so instead use GENERAL. This allows host access so we
+  // can still replay maps of the image's memory. In theory we can still transition from
+  // PREINITIALIZED on replay, but consider that we need to be able to reset layouts and suddenly we
+  // have a problem transitioning from PREINITIALIZED more than once - so for that reason we
+  // instantly promote any images that are PREINITIALIZED to GENERAL at the start of the frame
+  // capture, and from then on treat it as the same
+  if(layout == VK_IMAGE_LAYOUT_PREINITIALIZED)
+    layout = VK_IMAGE_LAYOUT_GENERAL;
+}
+
+void SanitiseNewImageLayout(VkImageLayout &layout)
+{
+  // apply any general image layout sanitisation
+  SanitiseOldImageLayout(layout);
+
+  // we also can't transition to UNDEFINED, so go to GENERAL instead. This is safe since if the
+  // layout was supposed to be undefined before then the only valid transition *from* the state is
+  // UNDEFINED, which will work silently.
+  if(layout == VK_IMAGE_LAYOUT_UNDEFINED)
+    layout = VK_IMAGE_LAYOUT_GENERAL;
+}
+
+void CombineDepthStencilLayouts(rdcarray<VkImageMemoryBarrier> &barriers)
+{
+  for(size_t i = 0; i < barriers.size(); i++)
+  {
+    // only consider barriers on depth
+    // barriers not on D/S at all can be ignored
+    // barriers on both D/S already can be ignored
+    // barriers on stencil only can be ignored, because we expect to always find depth before
+    // stencil
+    if(barriers[i].subresourceRange.aspectMask != VK_IMAGE_ASPECT_DEPTH_BIT)
+      continue;
+
+    // search forward to see if we have an identical barrier on stencil for the same image. We
+    // expect a loose sort so all barriers for the same image are together.
+    // This means when we don't have separate depth-stencil layout support, the aspects should
+    // always be in the same layout so can be combined.
+    for(size_t j = i + 1; j < barriers.size(); j++)
+    {
+      // stop when we reach another image, no more possible matches expected after this
+      if(barriers[i].image != barriers[j].image)
+        break;
+
+      // only consider stencil aspect barriers
+      if(barriers[j].subresourceRange.aspectMask != VK_IMAGE_ASPECT_STENCIL_BIT)
+        continue;
+
+      // if the barriers are equal apart from the aspect mask, we can promote [i] to depth and
+      // stencil, and erase j
+      if(barriers[i].oldLayout == barriers[j].oldLayout &&
+         barriers[i].newLayout == barriers[j].newLayout &&
+         barriers[i].srcAccessMask == barriers[j].srcAccessMask &&
+         barriers[i].dstAccessMask == barriers[j].dstAccessMask &&
+         barriers[i].srcQueueFamilyIndex == barriers[j].srcQueueFamilyIndex &&
+         barriers[i].dstQueueFamilyIndex == barriers[j].dstQueueFamilyIndex &&
+         barriers[i].subresourceRange.baseArrayLayer == barriers[j].subresourceRange.baseArrayLayer &&
+         barriers[i].subresourceRange.baseMipLevel == barriers[j].subresourceRange.baseMipLevel &&
+         barriers[i].subresourceRange.layerCount == barriers[j].subresourceRange.layerCount &&
+         barriers[i].subresourceRange.levelCount == barriers[j].subresourceRange.levelCount)
+      {
+        barriers[i].subresourceRange.aspectMask |= VK_IMAGE_ASPECT_STENCIL_BIT;
+        barriers.erase(j);
+        break;
+      }
+    }
+
+    // either we merged the i'th element and we can skip it, or it's not mergeable and we can skip
+    // it. Either way we can continue the loop at i+1
+  }
 }
 
 int SampleCount(VkSampleCountFlagBits countFlag)
@@ -371,31 +491,31 @@ int StageIndex(VkShaderStageFlagBits stageFlag)
   return 0;
 }
 
-void DoPipelineBarrier(VkCommandBuffer cmd, uint32_t count, VkImageMemoryBarrier *barriers)
+void DoPipelineBarrier(VkCommandBuffer cmd, size_t count, const VkImageMemoryBarrier *barriers)
 {
   RDCASSERT(cmd != VK_NULL_HANDLE);
   ObjDisp(cmd)->CmdPipelineBarrier(Unwrap(cmd), VK_PIPELINE_STAGE_ALL_COMMANDS_BIT,
                                    VK_PIPELINE_STAGE_ALL_COMMANDS_BIT, 0, 0,
-                                   NULL,                // global memory barriers
-                                   0, NULL,             // buffer memory barriers
-                                   count, barriers);    // image memory barriers
+                                   NULL,                          // global memory barriers
+                                   0, NULL,                       // buffer memory barriers
+                                   (uint32_t)count, barriers);    // image memory barriers
 }
 
-void DoPipelineBarrier(VkCommandBuffer cmd, uint32_t count, VkBufferMemoryBarrier *barriers)
+void DoPipelineBarrier(VkCommandBuffer cmd, size_t count, const VkBufferMemoryBarrier *barriers)
 {
   RDCASSERT(cmd != VK_NULL_HANDLE);
   ObjDisp(cmd)->CmdPipelineBarrier(Unwrap(cmd), VK_PIPELINE_STAGE_ALL_COMMANDS_BIT,
                                    VK_PIPELINE_STAGE_ALL_COMMANDS_BIT, 0, 0,
-                                   NULL,               // global memory barriers
-                                   count, barriers,    // buffer memory barriers
-                                   0, NULL);           // image memory barriers
+                                   NULL,                         // global memory barriers
+                                   (uint32_t)count, barriers,    // buffer memory barriers
+                                   0, NULL);                     // image memory barriers
 }
 
-void DoPipelineBarrier(VkCommandBuffer cmd, uint32_t count, VkMemoryBarrier *barriers)
+void DoPipelineBarrier(VkCommandBuffer cmd, size_t count, const VkMemoryBarrier *barriers)
 {
   RDCASSERT(cmd != VK_NULL_HANDLE);
   ObjDisp(cmd)->CmdPipelineBarrier(Unwrap(cmd), VK_PIPELINE_STAGE_ALL_COMMANDS_BIT,
-                                   VK_PIPELINE_STAGE_ALL_COMMANDS_BIT, 0, count,
+                                   VK_PIPELINE_STAGE_ALL_COMMANDS_BIT, 0, (uint32_t)count,
                                    barriers,    // global memory barriers
                                    0, NULL,     // buffer memory barriers
                                    0, NULL);    // image memory barriers
@@ -406,21 +526,17 @@ Topology MakePrimitiveTopology(VkPrimitiveTopology Topo, uint32_t patchControlPo
   switch(Topo)
   {
     default: break;
-    case VK_PRIMITIVE_TOPOLOGY_POINT_LIST: return Topology::PointList; break;
-    case VK_PRIMITIVE_TOPOLOGY_LINE_LIST: return Topology::LineList; break;
-    case VK_PRIMITIVE_TOPOLOGY_LINE_STRIP: return Topology::LineStrip; break;
-    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST: return Topology::TriangleList; break;
-    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP: return Topology::TriangleStrip; break;
-    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_FAN: return Topology::TriangleFan; break;
-    case VK_PRIMITIVE_TOPOLOGY_LINE_LIST_WITH_ADJACENCY: return Topology::LineList_Adj; break;
-    case VK_PRIMITIVE_TOPOLOGY_LINE_STRIP_WITH_ADJACENCY: return Topology::LineStrip_Adj; break;
-    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST_WITH_ADJACENCY:
-      return Topology::TriangleList_Adj;
-      break;
-    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP_WITH_ADJACENCY:
-      return Topology::TriangleStrip_Adj;
-      break;
-    case VK_PRIMITIVE_TOPOLOGY_PATCH_LIST: return PatchList_Topology(patchControlPoints); break;
+    case VK_PRIMITIVE_TOPOLOGY_POINT_LIST: return Topology::PointList;
+    case VK_PRIMITIVE_TOPOLOGY_LINE_LIST: return Topology::LineList;
+    case VK_PRIMITIVE_TOPOLOGY_LINE_STRIP: return Topology::LineStrip;
+    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST: return Topology::TriangleList;
+    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP: return Topology::TriangleStrip;
+    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_FAN: return Topology::TriangleFan;
+    case VK_PRIMITIVE_TOPOLOGY_LINE_LIST_WITH_ADJACENCY: return Topology::LineList_Adj;
+    case VK_PRIMITIVE_TOPOLOGY_LINE_STRIP_WITH_ADJACENCY: return Topology::LineStrip_Adj;
+    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST_WITH_ADJACENCY: return Topology::TriangleList_Adj;
+    case VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP_WITH_ADJACENCY: return Topology::TriangleStrip_Adj;
+    case VK_PRIMITIVE_TOPOLOGY_PATCH_LIST: return PatchList_Topology(patchControlPoints);
   }
 
   return Topology::Unknown;
@@ -539,7 +655,7 @@ FilterMode MakeFilterMode(VkFilter f)
   {
     case VK_FILTER_NEAREST: return FilterMode::Point;
     case VK_FILTER_LINEAR: return FilterMode::Linear;
-    case VK_FILTER_CUBIC_IMG: return FilterMode::Cubic;
+    case VK_FILTER_CUBIC_EXT: return FilterMode::Cubic;
     default: break;
   }
 
@@ -559,7 +675,7 @@ static FilterMode MakeFilterMode(VkSamplerMipmapMode f)
 }
 
 TextureFilter MakeFilter(VkFilter minFilter, VkFilter magFilter, VkSamplerMipmapMode mipmapMode,
-                         bool anisoEnable, bool compareEnable, VkSamplerReductionModeEXT reduction)
+                         bool anisoEnable, bool compareEnable, VkSamplerReductionMode reduction)
 {
   TextureFilter ret;
 
@@ -584,11 +700,9 @@ TextureFilter MakeFilter(VkFilter minFilter, VkFilter magFilter, VkSamplerMipmap
     switch(reduction)
     {
       default:
-      case VK_SAMPLER_REDUCTION_MODE_WEIGHTED_AVERAGE_EXT:
-        ret.filter = FilterFunction::Normal;
-        break;
-      case VK_SAMPLER_REDUCTION_MODE_MIN_EXT: ret.filter = FilterFunction::Minimum; break;
-      case VK_SAMPLER_REDUCTION_MODE_MAX_EXT: ret.filter = FilterFunction::Maximum; break;
+      case VK_SAMPLER_REDUCTION_MODE_WEIGHTED_AVERAGE: ret.filter = FilterFunction::Normal; break;
+      case VK_SAMPLER_REDUCTION_MODE_MIN: ret.filter = FilterFunction::Minimum; break;
+      case VK_SAMPLER_REDUCTION_MODE_MAX: ret.filter = FilterFunction::Maximum; break;
     }
   }
 
@@ -684,6 +798,25 @@ StencilOperation MakeStencilOp(VkStencilOp op)
   return StencilOperation::Keep;
 }
 
+rdcstr HumanDriverName(VkDriverId driverId)
+{
+  switch(driverId)
+  {
+    case VK_DRIVER_ID_AMD_PROPRIETARY: return "AMD Propriertary";
+    case VK_DRIVER_ID_AMD_OPEN_SOURCE: return "AMD Open-source";
+    case VK_DRIVER_ID_MESA_RADV: return "AMD RADV";
+    case VK_DRIVER_ID_INTEL_PROPRIETARY_WINDOWS: return "Intel Propriertary";
+    case VK_DRIVER_ID_INTEL_OPEN_SOURCE_MESA: return "Intel Open-source";
+    default: break;
+  }
+
+  return "";
+}
+
+BASIC_TYPE_SERIALISE_STRINGIFY(VkPackedVersion, (uint32_t &)el, SDBasic::UnsignedInteger, 4);
+
+INSTANTIATE_SERIALISE_TYPE(VkPackedVersion);
+
 template <typename SerialiserType>
 void DoSerialise(SerialiserType &ser, VkInitParams &el)
 {
@@ -691,10 +824,10 @@ void DoSerialise(SerialiserType &ser, VkInitParams &el)
   SERIALISE_MEMBER(EngineName);
   SERIALISE_MEMBER(AppVersion);
   SERIALISE_MEMBER(EngineVersion);
-  SERIALISE_MEMBER(APIVersion);
+  SERIALISE_MEMBER(APIVersion).TypedAs("uint32_t"_lit);
   SERIALISE_MEMBER(Layers);
   SERIALISE_MEMBER(Extensions);
-  SERIALISE_MEMBER(InstanceID).TypedAs("VkInstance");
+  SERIALISE_MEMBER(InstanceID).TypedAs("VkInstance"_lit);
 }
 
 INSTANTIATE_SERIALISE_TYPE(VkInitParams);
@@ -707,6 +840,10 @@ VkDriverInfo::VkDriverInfo(const VkPhysicalDeviceProperties &physProps)
   if(physProps.vendorID == VK_VENDOR_ID_VSI)
     m_Vendor = GPUVendor::Verisilicon;
 
+  // Swiftshader
+  if(physProps.vendorID == 0x1AE0 && physProps.deviceID == 0xC0DE)
+    m_Vendor = GPUVendor::Software;
+
   m_Major = VK_VERSION_MAJOR(physProps.driverVersion);
   m_Minor = VK_VERSION_MINOR(physProps.driverVersion);
   m_Patch = VK_VERSION_PATCH(physProps.driverVersion);
@@ -786,11 +923,11 @@ FrameRefType GetRefType(VkDescriptorType descType)
     case VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER:
     case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER:
     case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC:
-    case VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT: return eFrameRef_Read; break;
+    case VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT: return eFrameRef_Read;
     case VK_DESCRIPTOR_TYPE_STORAGE_IMAGE:
     case VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
     case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER:
-    case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC: return eFrameRef_ReadBeforeWrite; break;
+    case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC: return eFrameRef_ReadBeforeWrite;
     default: RDCERR("Unexpected descriptor type");
   }
 
@@ -832,23 +969,40 @@ bool IsValid(const VkWriteDescriptorSet &write, uint32_t arrayElement)
   return false;
 }
 
-void DescriptorSetSlot::RemoveBindRefs(VkResourceRecord *record)
+void DescriptorSetSlotBufferInfo::SetFrom(const VkDescriptorBufferInfo &bufInfo)
+{
+  buffer = GetResID(bufInfo.buffer);
+  offset = bufInfo.offset;
+  range = bufInfo.range;
+}
+
+void DescriptorSetSlotImageInfo::SetFrom(const VkDescriptorImageInfo &imInfo, bool setSampler,
+                                         bool setImageView)
+{
+  if(setSampler)
+    sampler = GetResID(imInfo.sampler);
+  if(setImageView)
+    imageView = GetResID(imInfo.imageView);
+  imageLayout = imInfo.imageLayout;
+}
+
+void DescriptorSetSlot::RemoveBindRefs(VulkanResourceManager *rm, VkResourceRecord *record)
 {
   SCOPED_LOCK(record->descInfo->refLock);
 
-  if(texelBufferView != VK_NULL_HANDLE)
+  if(texelBufferView != ResourceId())
   {
-    record->RemoveBindFrameRef(GetResID(texelBufferView));
+    record->RemoveBindFrameRef(texelBufferView);
 
-    VkResourceRecord *viewRecord = GetRecord(texelBufferView);
+    VkResourceRecord *viewRecord = rm->GetResourceRecord(texelBufferView);
     if(viewRecord && viewRecord->baseResource != ResourceId())
       record->RemoveBindFrameRef(viewRecord->baseResource);
   }
-  if(imageInfo.imageView != VK_NULL_HANDLE)
+  if(imageInfo.imageView != ResourceId())
   {
-    record->RemoveBindFrameRef(GetResID(imageInfo.imageView));
+    record->RemoveBindFrameRef(imageInfo.imageView);
 
-    VkResourceRecord *viewRecord = GetRecord(imageInfo.imageView);
+    VkResourceRecord *viewRecord = rm->GetResourceRecord(imageInfo.imageView);
     if(viewRecord)
     {
       record->RemoveBindFrameRef(viewRecord->baseResource);
@@ -856,57 +1010,305 @@ void DescriptorSetSlot::RemoveBindRefs(VkResourceRecord *record)
         record->RemoveBindFrameRef(viewRecord->baseResourceMem);
     }
   }
-  if(imageInfo.sampler != VK_NULL_HANDLE)
+  if(imageInfo.sampler != ResourceId())
   {
-    record->RemoveBindFrameRef(GetResID(imageInfo.sampler));
+    record->RemoveBindFrameRef(imageInfo.sampler);
   }
-  if(bufferInfo.buffer != VK_NULL_HANDLE)
+  if(bufferInfo.buffer != ResourceId())
   {
-    record->RemoveBindFrameRef(GetResID(bufferInfo.buffer));
+    record->RemoveBindFrameRef(bufferInfo.buffer);
 
-    VkResourceRecord *bufRecord = GetRecord(bufferInfo.buffer);
+    VkResourceRecord *bufRecord = rm->GetResourceRecord(bufferInfo.buffer);
     if(bufRecord && bufRecord->baseResource != ResourceId())
       record->RemoveBindFrameRef(bufRecord->baseResource);
   }
 
   // NULL everything out now so that we don't accidentally reference an object
   // that was removed already
-  texelBufferView = VK_NULL_HANDLE;
-  bufferInfo.buffer = VK_NULL_HANDLE;
-  imageInfo.imageView = VK_NULL_HANDLE;
-  imageInfo.sampler = VK_NULL_HANDLE;
+  texelBufferView = ResourceId();
+  bufferInfo.buffer = ResourceId();
+  imageInfo.imageView = ResourceId();
+  imageInfo.sampler = ResourceId();
 }
 
-void DescriptorSetSlot::AddBindRefs(VkResourceRecord *record, FrameRefType ref)
+void DescriptorSetSlot::AddBindRefs(VulkanResourceManager *rm, VkResourceRecord *record,
+                                    FrameRefType ref)
 {
   SCOPED_LOCK(record->descInfo->refLock);
 
-  if(texelBufferView != VK_NULL_HANDLE)
+  if(texelBufferView != ResourceId())
   {
-    VkResourceRecord *bufView = GetRecord(texelBufferView);
-    record->AddBindFrameRef(bufView->GetResourceID(), eFrameRef_Read, bufView->resInfo != NULL);
+    VkResourceRecord *bufView = rm->GetResourceRecord(texelBufferView);
+    record->AddBindFrameRef(bufView->GetResourceID(), eFrameRef_Read,
+                            bufView->resInfo && bufView->resInfo->IsSparse());
     if(bufView->baseResource != ResourceId())
       record->AddBindFrameRef(bufView->baseResource, eFrameRef_Read);
     if(bufView->baseResourceMem != ResourceId())
       record->AddMemFrameRef(bufView->baseResourceMem, bufView->memOffset, bufView->memSize, ref);
   }
-  if(imageInfo.imageView != VK_NULL_HANDLE)
+  if(imageInfo.imageView != ResourceId())
   {
-    record->AddBindFrameRef(GetResID(imageInfo.imageView), eFrameRef_Read,
-                            GetRecord(imageInfo.imageView)->resInfo != NULL);
-    record->AddBindFrameRef(GetRecord(imageInfo.imageView)->baseResource, ref);
-    if(GetRecord(imageInfo.imageView)->baseResourceMem != ResourceId())
-      record->AddBindFrameRef(GetRecord(imageInfo.imageView)->baseResourceMem, eFrameRef_Read);
+    VkResourceRecord *view = rm->GetResourceRecord(imageInfo.imageView);
+    record->AddImgFrameRef(view, ref);
   }
-  if(imageInfo.sampler != VK_NULL_HANDLE)
+  if(imageInfo.sampler != ResourceId())
   {
-    record->AddBindFrameRef(GetResID(imageInfo.sampler), eFrameRef_Read);
+    record->AddBindFrameRef(imageInfo.sampler, eFrameRef_Read);
   }
-  if(bufferInfo.buffer != VK_NULL_HANDLE)
+  if(bufferInfo.buffer != ResourceId())
   {
-    VkResourceRecord *buf = GetRecord(bufferInfo.buffer);
-    record->AddBindFrameRef(GetResID(bufferInfo.buffer), eFrameRef_Read, buf->resInfo != NULL);
+    VkResourceRecord *buf = rm->GetResourceRecord(bufferInfo.buffer);
+    record->AddBindFrameRef(bufferInfo.buffer, eFrameRef_Read,
+                            buf->resInfo && buf->resInfo->IsSparse());
     if(buf->baseResource != ResourceId())
       record->AddMemFrameRef(buf->baseResource, buf->memOffset, buf->memSize, ref);
   }
 }
+
+#if ENABLED(ENABLE_UNIT_TESTS)
+
+#undef None
+
+#include "3rdparty/catch/catch.hpp"
+
+bool operator==(const VkImageMemoryBarrier &a, const VkImageMemoryBarrier &b)
+{
+  return memcmp(&a, &b, sizeof(VkImageMemoryBarrier)) == 0;
+}
+
+TEST_CASE("Validate CombineDepthStencilLayouts works", "[vulkan]")
+{
+  VkImage imga, imgb;
+
+  // give the fake handles values
+  {
+    uint64_t a = 1;
+    uint64_t b = 2;
+    memcpy(&imga, &a, sizeof(a));
+    memcpy(&imgb, &b, sizeof(b));
+  }
+
+  rdcarray<VkImageMemoryBarrier> barriers, ref;
+
+  VkImageMemoryBarrier b = {VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER};
+  b.oldLayout = VK_IMAGE_LAYOUT_GENERAL;
+  b.newLayout = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
+
+  SECTION("Ignored cases")
+  {
+    VkImageAspectFlags aspects[] = {VK_IMAGE_ASPECT_COLOR_BIT, VK_IMAGE_ASPECT_DEPTH_BIT,
+                                    VK_IMAGE_ASPECT_STENCIL_BIT,
+                                    VK_IMAGE_ASPECT_DEPTH_BIT | VK_IMAGE_ASPECT_STENCIL_BIT};
+    // only care about split depth and stencil for the same image
+    for(VkImageAspectFlags aspect : aspects)
+    {
+      barriers.clear();
+
+      // whole image
+      b.image = imga;
+      b.subresourceRange = {aspect, 0, 4, 0, 6};
+      barriers.push_back(b);
+
+      b.image = imgb;
+      b.subresourceRange = {aspect, 0, 4, 0, 6};
+      barriers.push_back(b);
+
+      ref = barriers;
+      CombineDepthStencilLayouts(barriers);
+
+      CHECK((ref == barriers));
+
+      barriers.clear();
+
+      // images split into different mips/arrays
+      b.image = imga;
+      b.subresourceRange = {aspect, 0, 1, 0, 3};
+      barriers.push_back(b);
+      b.subresourceRange = {aspect, 1, 2, 0, 3};
+      barriers.push_back(b);
+      b.subresourceRange = {aspect, 3, 1, 0, 3};
+      barriers.push_back(b);
+      b.subresourceRange = {aspect, 0, 4, 3, 3};
+      barriers.push_back(b);
+
+      b.image = imgb;
+      b.subresourceRange = {aspect, 0, 1, 0, 3};
+      barriers.push_back(b);
+      b.subresourceRange = {aspect, 1, 3, 0, 3};
+      barriers.push_back(b);
+      b.subresourceRange = {aspect, 0, 4, 3, 3};
+      barriers.push_back(b);
+
+      ref = barriers;
+      CombineDepthStencilLayouts(barriers);
+
+      CHECK((ref == barriers));
+    }
+  };
+
+  SECTION("Possible but unmergeable cases")
+  {
+    barriers.clear();
+
+    // could merge, but different images
+    b.image = imga;
+    b.subresourceRange = {VK_IMAGE_ASPECT_DEPTH_BIT, 0, 1, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imgb;
+    b.subresourceRange = {VK_IMAGE_ASPECT_STENCIL_BIT, 0, 1, 0, 1};
+    barriers.push_back(b);
+
+    ref = barriers;
+    CombineDepthStencilLayouts(barriers);
+
+    CHECK((ref == barriers));
+
+    barriers.clear();
+
+    // could merge, but different subresource ranges
+    b.image = imga;
+    b.subresourceRange = {VK_IMAGE_ASPECT_DEPTH_BIT, 0, 1, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imgb;
+    b.subresourceRange = {VK_IMAGE_ASPECT_STENCIL_BIT, 0, 2, 0, 1};
+    barriers.push_back(b);
+
+    ref = barriers;
+    CombineDepthStencilLayouts(barriers);
+
+    CHECK((ref == barriers));
+
+    barriers.clear();
+
+    b.image = imga;
+    b.subresourceRange = {VK_IMAGE_ASPECT_DEPTH_BIT, 0, 1, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imgb;
+    b.subresourceRange = {VK_IMAGE_ASPECT_STENCIL_BIT, 0, 1, 1, 1};
+    barriers.push_back(b);
+
+    ref = barriers;
+    CombineDepthStencilLayouts(barriers);
+
+    CHECK((ref == barriers));
+
+    barriers.clear();
+
+    // could merge, but different layouts
+    b.image = imga;
+    b.oldLayout = VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_OPTIMAL_KHR;
+    b.subresourceRange = {VK_IMAGE_ASPECT_DEPTH_BIT, 0, 2, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imgb;
+    b.oldLayout = VK_IMAGE_LAYOUT_STENCIL_READ_ONLY_OPTIMAL_KHR;
+    b.subresourceRange = {VK_IMAGE_ASPECT_STENCIL_BIT, 0, 2, 0, 1};
+    barriers.push_back(b);
+
+    ref = barriers;
+    CombineDepthStencilLayouts(barriers);
+
+    CHECK((ref == barriers));
+
+    barriers.clear();
+
+    b.image = imga;
+    b.newLayout = VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_OPTIMAL_KHR;
+    b.subresourceRange = {VK_IMAGE_ASPECT_DEPTH_BIT, 0, 2, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imgb;
+    b.newLayout = VK_IMAGE_LAYOUT_STENCIL_READ_ONLY_OPTIMAL_KHR;
+    b.subresourceRange = {VK_IMAGE_ASPECT_STENCIL_BIT, 0, 2, 0, 1};
+    barriers.push_back(b);
+
+    ref = barriers;
+    CombineDepthStencilLayouts(barriers);
+
+    CHECK((ref == barriers));
+  };
+
+  SECTION("Whole-image depth and separate stencil barriers are merged when possible")
+  {
+    barriers.clear();
+
+    CHECK((ref == barriers));
+    b.image = imga;
+    b.subresourceRange = {VK_IMAGE_ASPECT_DEPTH_BIT, 0, 1, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imga;
+    b.subresourceRange = {VK_IMAGE_ASPECT_STENCIL_BIT, 0, 1, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imgb;
+    b.subresourceRange = {VK_IMAGE_ASPECT_DEPTH_BIT, 0, 1, 0, 1};
+    barriers.push_back(b);
+
+    ref = barriers;
+    CombineDepthStencilLayouts(barriers);
+
+    REQUIRE(barriers.size() == 2);
+    // aspect mask is combined now
+    CHECK(barriers[0].subresourceRange.aspectMask ==
+          (VK_IMAGE_ASPECT_DEPTH_BIT | VK_IMAGE_ASPECT_STENCIL_BIT));
+    // otherwise the barrier should be the same
+    CHECK(barriers[0].subresourceRange.baseMipLevel == 0);
+    CHECK(barriers[0].subresourceRange.levelCount == 1);
+    CHECK(barriers[0].subresourceRange.baseArrayLayer == 0);
+    CHECK(barriers[0].subresourceRange.layerCount == 1);
+    CHECK(barriers[0].oldLayout == ref[0].oldLayout);
+    CHECK(barriers[0].newLayout == ref[0].newLayout);
+    // the last barrier should be the same
+    CHECK((barriers[1] == ref[2]));
+  };
+
+  SECTION("Split depth and separate stencil barriers are merged when possible")
+  {
+    barriers.clear();
+
+    CHECK((ref == barriers));
+    b.image = imga;
+    b.subresourceRange = {VK_IMAGE_ASPECT_DEPTH_BIT, 0, 1, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imga;
+    b.subresourceRange = {VK_IMAGE_ASPECT_DEPTH_BIT, 1, 1, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imga;
+    b.subresourceRange = {VK_IMAGE_ASPECT_STENCIL_BIT, 0, 1, 0, 1};
+    barriers.push_back(b);
+
+    b.image = imga;
+    b.subresourceRange = {VK_IMAGE_ASPECT_STENCIL_BIT, 1, 1, 0, 1};
+    barriers.push_back(b);
+
+    CombineDepthStencilLayouts(barriers);
+
+    REQUIRE(barriers.size() == 2);
+
+    // aspect mask is combined now
+    CHECK(barriers[0].subresourceRange.aspectMask ==
+          (VK_IMAGE_ASPECT_DEPTH_BIT | VK_IMAGE_ASPECT_STENCIL_BIT));
+    // otherwise the barrier should be the same
+    CHECK(barriers[0].subresourceRange.baseMipLevel == 0);
+    CHECK(barriers[0].subresourceRange.levelCount == 1);
+    CHECK(barriers[0].subresourceRange.baseArrayLayer == 0);
+    CHECK(barriers[0].subresourceRange.layerCount == 1);
+
+    // aspect mask is combined now
+    CHECK(barriers[1].subresourceRange.aspectMask ==
+          (VK_IMAGE_ASPECT_DEPTH_BIT | VK_IMAGE_ASPECT_STENCIL_BIT));
+    // otherwise the barrier should be the same
+    CHECK(barriers[1].subresourceRange.baseMipLevel == 1);
+    CHECK(barriers[1].subresourceRange.levelCount == 1);
+    CHECK(barriers[1].subresourceRange.baseArrayLayer == 0);
+    CHECK(barriers[1].subresourceRange.layerCount == 1);
+  };
+}
+
+#endif
