@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,14 +24,15 @@
 
 #include "vk_debug.h"
 #include <float.h>
-#include "3rdparty/glslang/SPIRV/spirv.hpp"
 #include "data/glsl_shaders.h"
 #include "driver/ihv/amd/amd_counters.h"
 #include "driver/ihv/amd/official/GPUPerfAPI/Include/GPUPerfAPI-VK.h"
+#include "driver/shaders/spirv/spirv_compile.h"
 #include "maths/camera.h"
 #include "maths/formatpacking.h"
 #include "maths/matrix.h"
 #include "vk_core.h"
+#include "vk_replay.h"
 #include "vk_shader_cache.h"
 
 #define VULKAN 1
@@ -695,6 +696,12 @@ VulkanDebugManager::VulkanDebugManager(WrappedVulkan *driver)
 
     m_pDriver->vkDestroyRenderPass(dev, depthMS2ArrayRP, NULL);
 
+    if(!m_pDriver->GetDeviceFeatures().sampleRateShading)
+    {
+      RDCDEBUG("No depth Array -> MSAA copies can be supported without sample rate shading");
+      continue;
+    }
+
     for(size_t s = 0; s < ARRAY_COUNT(sampleCounts); s++)
     {
       // if this sample count isn't supported, don't create it
@@ -968,7 +975,7 @@ uint32_t VulkanReplay::PickVertex(uint32_t eventId, int32_t w, int32_t h, const
                                   uint32_t x, uint32_t y)
 {
   VkDevice dev = m_pDriver->GetDev();
-  const VkLayerDispatchTable *vt = ObjDisp(dev);
+  const VkDevDispatchTable *vt = ObjDisp(dev);
 
   VkMarkerRegion::Begin(StringFormat::Fmt("VulkanReplay::PickVertex(%u, %u)", x, y));
 
@@ -1043,55 +1050,10 @@ uint32_t VulkanReplay::PickVertex(uint32_t eventId, int32_t w, int32_t h, const
     }
   }
 
-  MeshPickUBOData *ubo = (MeshPickUBOData *)m_VertexPick.UBO.Map();
+  const bool fandecode =
+      (cfg.position.topology == Topology::TriangleFan && cfg.position.allowRestart);
 
-  ubo->rayPos = rayPos;
-  ubo->rayDir = rayDir;
-  ubo->use_indices = cfg.position.indexByteStride ? 1U : 0U;
-  ubo->numVerts = cfg.position.numIndices;
-  bool isTriangleMesh = true;
-
-  switch(cfg.position.topology)
-  {
-    case Topology::TriangleList:
-    {
-      ubo->meshMode = MESH_TRIANGLE_LIST;
-      break;
-    };
-    case Topology::TriangleStrip:
-    {
-      ubo->meshMode = MESH_TRIANGLE_STRIP;
-      break;
-    };
-    case Topology::TriangleFan:
-    {
-      ubo->meshMode = MESH_TRIANGLE_FAN;
-      break;
-    };
-    case Topology::TriangleList_Adj:
-    {
-      ubo->meshMode = MESH_TRIANGLE_LIST_ADJ;
-      break;
-    };
-    case Topology::TriangleStrip_Adj:
-    {
-      ubo->meshMode = MESH_TRIANGLE_STRIP_ADJ;
-      break;
-    };
-    default:    // points, lines, patchlists, unknown
-    {
-      ubo->meshMode = MESH_OTHER;
-      isTriangleMesh = false;
-    };
-  }
-
-  // line/point data
-  ubo->unproject = cfg.position.unproject;
-  ubo->mvp = cfg.position.unproject ? pickMVPProj : pickMVP;
-  ubo->coords = Vec2f((float)x, (float)y);
-  ubo->viewport = Vec2f((float)w, (float)h);
-
-  m_VertexPick.UBO.Unmap();
+  uint32_t numIndices = cfg.position.numIndices;
 
   bytebuf idxs;
 
@@ -1110,8 +1072,20 @@ uint32_t VulkanReplay::PickVertex(uint32_t eventId, int32_t w, int32_t h, const
 
   if(!idxs.empty())
   {
+    rdcarray<uint32_t> idxtmp;
+
+    // if it's a triangle fan that allows restart, we'll have to unpack it.
+    // Allocate enough space for the list on the GPU, and enough temporary space to upcast into
+    // first
+    if(fandecode)
+    {
+      idxtmp.resize(numIndices);
+
+      numIndices *= 3;
+    }
+
     // resize up on demand
-    if(m_VertexPick.IBSize < cfg.position.numIndices * sizeof(uint32_t))
+    if(m_VertexPick.IBSize < numIndices * sizeof(uint32_t))
     {
       if(m_VertexPick.IBSize > 0)
       {
@@ -1119,7 +1093,7 @@ uint32_t VulkanReplay::PickVertex(uint32_t eventId, int32_t w, int32_t h, const
         m_VertexPick.IBUpload.Destroy();
       }
 
-      m_VertexPick.IBSize = cfg.position.numIndices * sizeof(uint32_t);
+      m_VertexPick.IBSize = numIndices * sizeof(uint32_t);
 
       m_VertexPick.IB.Create(m_pDriver, dev, m_VertexPick.IBSize, 1,
                              GPUBuffer::eGPUBufferGPULocal | GPUBuffer::eGPUBufferSSBO);
@@ -1127,12 +1101,19 @@ uint32_t VulkanReplay::PickVertex(uint32_t eventId, int32_t w, int32_t h, const
     }
 
     uint32_t *outidxs = (uint32_t *)m_VertexPick.IBUpload.Map();
+    uint32_t *mappedPtr = outidxs;
 
     memset(outidxs, 0, m_VertexPick.IBSize);
 
+    // if we're decoding a fan, we write into our temporary vector first
+    if(fandecode)
+      outidxs = idxtmp.data();
+
     uint16_t *idxs16 = (uint16_t *)&idxs[0];
     uint32_t *idxs32 = (uint32_t *)&idxs[0];
 
+    size_t idxcount = 0;
+
     if(cfg.position.indexByteStride == 2)
     {
       size_t bufsize = idxs.size() / 2;
@@ -1159,6 +1140,7 @@ uint32_t VulkanReplay::PickVertex(uint32_t eventId, int32_t w, int32_t h, const
         }
 
         outidxs[i] = idx;
+        idxcount++;
       }
     }
     else
@@ -1182,9 +1164,31 @@ uint32_t VulkanReplay::PickVertex(uint32_t eventId, int32_t w, int32_t h, const
         maxIndex = RDCMAX(idx, maxIndex);
 
         outidxs[i] = idx;
+        idxcount++;
       }
     }
 
+    // if it's a triangle fan that allows restart, unpack it
+    if(cfg.position.topology == Topology::TriangleFan && cfg.position.allowRestart)
+    {
+      // resize to how many indices were actually read
+      idxtmp.resize(idxcount);
+
+      // patch the index buffer
+      PatchTriangleFanRestartIndexBufer(idxtmp, cfg.position.restartIndex);
+
+      for(uint32_t &idx : idxtmp)
+      {
+        if(idx == cfg.position.restartIndex)
+          idx = 0;
+      }
+
+      numIndices = (uint32_t)idxtmp.size();
+
+      // now copy the decoded list to the GPU
+      memcpy(mappedPtr, idxtmp.data(), idxtmp.size() * sizeof(uint32_t));
+    }
+
     m_VertexPick.IBUpload.Unmap();
   }
 
@@ -1229,6 +1233,59 @@ uint32_t VulkanReplay::PickVertex(uint32_t eventId, int32_t w, int32_t h, const
     m_VertexPick.VBUpload.Unmap();
   }
 
+  MeshPickUBOData *ubo = (MeshPickUBOData *)m_VertexPick.UBO.Map();
+
+  ubo->rayPos = rayPos;
+  ubo->rayDir = rayDir;
+  ubo->use_indices = cfg.position.indexByteStride ? 1U : 0U;
+  ubo->numVerts = numIndices;
+  bool isTriangleMesh = true;
+
+  switch(cfg.position.topology)
+  {
+    case Topology::TriangleList:
+    {
+      ubo->meshMode = MESH_TRIANGLE_LIST;
+      break;
+    };
+    case Topology::TriangleStrip:
+    {
+      ubo->meshMode = MESH_TRIANGLE_STRIP;
+      break;
+    };
+    case Topology::TriangleFan:
+    {
+      if(fandecode)
+        ubo->meshMode = MESH_TRIANGLE_LIST;
+      else
+        ubo->meshMode = MESH_TRIANGLE_FAN;
+      break;
+    };
+    case Topology::TriangleList_Adj:
+    {
+      ubo->meshMode = MESH_TRIANGLE_LIST_ADJ;
+      break;
+    };
+    case Topology::TriangleStrip_Adj:
+    {
+      ubo->meshMode = MESH_TRIANGLE_STRIP_ADJ;
+      break;
+    };
+    default:    // points, lines, patchlists, unknown
+    {
+      ubo->meshMode = MESH_OTHER;
+      isTriangleMesh = false;
+    };
+  }
+
+  // line/point data
+  ubo->unproject = cfg.position.unproject;
+  ubo->mvp = cfg.position.unproject ? pickMVPProj : pickMVP;
+  ubo->coords = Vec2f((float)x, (float)y);
+  ubo->viewport = Vec2f((float)w, (float)h);
+
+  m_VertexPick.UBO.Unmap();
+
   VkDescriptorBufferInfo ibInfo = {};
   VkDescriptorBufferInfo vbInfo = {};
 
@@ -1419,19 +1476,33 @@ uint32_t VulkanReplay::PickVertex(uint32_t eventId, int32_t w, int32_t h, const
 
   VkMarkerRegion::End();
 
+  if(fandecode)
+  {
+    // undo the triangle list expansion
+    if(ret > 2)
+      ret = (ret + 3) / 3 + 1;
+  }
+
   return ret;
 }
 
+const VulkanCreationInfo::Image &VulkanDebugManager::GetImageInfo(ResourceId img)
+{
+  auto it = m_pDriver->m_CreationInfo.m_Image.find(img);
+  RDCASSERT(it != m_pDriver->m_CreationInfo.m_Image.end());
+  return it->second;
+}
+
 void VulkanDebugManager::GetBufferData(ResourceId buff, uint64_t offset, uint64_t len, bytebuf &ret)
 {
   VkDevice dev = m_pDriver->GetDev();
-  const VkLayerDispatchTable *vt = ObjDisp(dev);
+  const VkDevDispatchTable *vt = ObjDisp(dev);
 
   WrappedVkRes *res = m_pDriver->GetResourceManager()->GetCurrentResource(buff);
 
   if(res == VK_NULL_HANDLE)
   {
-    RDCERR("Getting buffer data for unknown buffer/memory %llu!", buff);
+    RDCERR("Getting buffer data for unknown buffer/memory %s!", ToStr(buff).c_str());
     return;
   }
 
@@ -1605,28 +1676,31 @@ void VulkanReplay::CreateResources()
   GPA_vkContextOpenInfo context = {Unwrap(m_pDriver->GetInstance()),
                                    Unwrap(m_pDriver->GetPhysDev()), Unwrap(m_pDriver->GetDev())};
 
-  AMDCounters *counters = NULL;
+  if(!m_pDriver->GetReplay()->IsRemoteProxy())
+  {
+    AMDCounters *counters = NULL;
 
-  GPUVendor vendor = m_pDriver->GetDriverInfo().Vendor();
+    GPUVendor vendor = m_pDriver->GetDriverInfo().Vendor();
 
-  if(vendor == GPUVendor::AMD)
-  {
-    RDCLOG("AMD GPU detected - trying to initialise AMD counters");
-    counters = new AMDCounters();
-  }
-  else
-  {
-    RDCLOG("%s GPU detected - no counters available", ToStr(vendor).c_str());
-  }
+    if(vendor == GPUVendor::AMD)
+    {
+      RDCLOG("AMD GPU detected - trying to initialise AMD counters");
+      counters = new AMDCounters();
+    }
+    else
+    {
+      RDCLOG("%s GPU detected - no counters available", ToStr(vendor).c_str());
+    }
 
-  if(counters && counters->Init(AMDCounters::ApiType::Vk, (void *)&context))
-  {
-    m_pAMDCounters = counters;
-  }
-  else
-  {
-    delete counters;
-    m_pAMDCounters = NULL;
+    if(counters && counters->Init(AMDCounters::ApiType::Vk, (void *)&context))
+    {
+      m_pAMDCounters = counters;
+    }
+    else
+    {
+      delete counters;
+      m_pAMDCounters = NULL;
+    }
   }
 }
 
@@ -1656,6 +1730,7 @@ void VulkanReplay::GeneralMisc::Init(WrappedVulkan *driver, VkDescriptorPool des
       {VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 128},
       {VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC, 128},
       {VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER, 64},
+      {VK_DESCRIPTOR_TYPE_SAMPLER, 32},
       {VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 32},
   };
 
@@ -1690,6 +1765,7 @@ void VulkanReplay::TextureRendering::Init(WrappedVulkan *driver, VkDescriptorPoo
 
   VulkanShaderCache *shaderCache = driver->GetShaderCache();
 
+  CREATE_OBJECT(PointSampler, VK_FILTER_NEAREST);
   CREATE_OBJECT(LinearSampler, VK_FILTER_LINEAR);
 
   CREATE_OBJECT(DescSetLayout,
@@ -1711,6 +1787,8 @@ void VulkanReplay::TextureRendering::Init(WrappedVulkan *driver, VkDescriptorPoo
                     {18, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, 1, VK_SHADER_STAGE_ALL, NULL},
                     {19, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, 1, VK_SHADER_STAGE_ALL, NULL},
                     {20, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, 1, VK_SHADER_STAGE_ALL, NULL},
+                    {50, VK_DESCRIPTOR_TYPE_SAMPLER, 1, VK_SHADER_STAGE_ALL, &PointSampler},
+                    {51, VK_DESCRIPTOR_TYPE_SAMPLER, 1, VK_SHADER_STAGE_ALL, &LinearSampler},
                 });
 
   CREATE_OBJECT(PipeLayout, DescSetLayout, 0);
@@ -1753,6 +1831,8 @@ void VulkanReplay::TextureRendering::Init(WrappedVulkan *driver, VkDescriptorPoo
         0xf,    // writeMask
     };
 
+    ConciseGraphicsPipeline texRemapInfo = texDisplayInfo;
+
     CREATE_OBJECT(Pipeline, texDisplayInfo);
 
     texDisplayInfo.renderPass = RGBA32RP;
@@ -1767,8 +1847,42 @@ void VulkanReplay::TextureRendering::Init(WrappedVulkan *driver, VkDescriptorPoo
     texDisplayInfo.dstBlend = VK_BLEND_FACTOR_ONE_MINUS_SRC_ALPHA;
     CREATE_OBJECT(BlendPipeline, texDisplayInfo);
 
+    VkFormat formats[3] = {VK_FORMAT_R8G8B8A8_UINT, VK_FORMAT_R16G16B16A16_UINT,
+                           VK_FORMAT_R32G32B32A32_UINT};
+    CompType cast[3] = {CompType::Float, CompType::UInt, CompType::SInt};
+    BuiltinShader shaders[3] = {BuiltinShader::TexRemapFloat, BuiltinShader::TexRemapUInt,
+                                BuiltinShader::TexRemapSInt};
+
+    for(int f = 0; f < 3; f++)
+    {
+      for(int i = 0; i < 3; i++)
+      {
+        texRemapInfo.fragment = shaderCache->GetBuiltinModule(shaders[i]);
+
+        CREATE_OBJECT(texRemapInfo.renderPass, GetViewCastedFormat(formats[f], cast[i]));
+
+        CREATE_OBJECT(RemapPipeline[f][i][0], texRemapInfo);
+
+        driver->vkDestroyRenderPass(driver->GetDev(), texRemapInfo.renderPass, NULL);
+      }
+    }
+
     // make versions that only write to green, for doing two-pass stencil writes
-    texDisplayInfo.writeMask = 0x2;
+    texRemapInfo.writeMask = texDisplayInfo.writeMask = 0x2;
+
+    for(int f = 0; f < 3; f++)
+    {
+      for(int i = 0; i < 3; i++)
+      {
+        texRemapInfo.fragment = shaderCache->GetBuiltinModule(shaders[i]);
+
+        CREATE_OBJECT(texRemapInfo.renderPass, GetViewCastedFormat(formats[f], cast[i]));
+
+        CREATE_OBJECT(RemapPipeline[f][i][1], texRemapInfo);
+
+        driver->vkDestroyRenderPass(driver->GetDev(), texRemapInfo.renderPass, NULL);
+      }
+    }
 
     texDisplayInfo.renderPass = SRGBA8RP;
     CREATE_OBJECT(PipelineGreenOnly, texDisplayInfo);
@@ -1842,7 +1956,7 @@ void VulkanReplay::TextureRendering::Init(WrappedVulkan *driver, VkDescriptorPoo
             1,
             sampleCounts[type],
             VK_IMAGE_TILING_OPTIMAL,
-            VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT | VK_IMAGE_USAGE_SAMPLED_BIT,
+            VK_IMAGE_USAGE_SAMPLED_BIT,
             VK_SHARING_MODE_EXCLUSIVE,
             0,
             NULL,
@@ -1996,6 +2110,10 @@ void VulkanReplay::TextureRendering::Destroy(WrappedVulkan *driver)
   driver->vkDestroyPipeline(driver->GetDev(), BlendPipeline, NULL);
   driver->vkDestroyPipeline(driver->GetDev(), F16Pipeline, NULL);
   driver->vkDestroyPipeline(driver->GetDev(), F32Pipeline, NULL);
+  for(size_t f = 0; f < 3; f++)
+    for(size_t i = 0; i < 3; i++)
+      for(size_t g = 0; g < 2; g++)
+        driver->vkDestroyPipeline(driver->GetDev(), RemapPipeline[f][i][g], NULL);
 
   driver->vkDestroyPipeline(driver->GetDev(), PipelineGreenOnly, NULL);
   driver->vkDestroyPipeline(driver->GetDev(), F16PipelineGreenOnly, NULL);
@@ -2003,6 +2121,7 @@ void VulkanReplay::TextureRendering::Destroy(WrappedVulkan *driver)
   UBO.Destroy();
   HeatmapUBO.Destroy();
 
+  driver->vkDestroySampler(driver->GetDev(), PointSampler, NULL);
   driver->vkDestroySampler(driver->GetDev(), LinearSampler, NULL);
 
   for(size_t i = 0; i < ARRAY_COUNT(DummyImages); i++)
@@ -2116,7 +2235,11 @@ void VulkanReplay::OverlayRendering::Init(WrappedVulkan *driver, VkDescriptorPoo
     pipeInfo.fragment = shaderCache->GetBuiltinModule(BuiltinShader::QuadResolveFS);
     pipeInfo.pipeLayout = m_QuadResolvePipeLayout;
 
-    CREATE_OBJECT(m_QuadResolvePipeline[i], pipeInfo);
+    if(pipeInfo.fragment != VK_NULL_HANDLE &&
+       shaderCache->GetBuiltinModule(BuiltinShader::QuadWriteFS) != VK_NULL_HANDLE)
+    {
+      CREATE_OBJECT(m_QuadResolvePipeline[i], pipeInfo);
+    }
 
     driver->vkDestroyRenderPass(driver->GetDev(), RGBA16MSRP, NULL);
   }
@@ -2426,7 +2549,7 @@ void VulkanReplay::HistogramMinMax::Init(WrappedVulkan *driver, VkDescriptorPool
 
   shaderCache->SetCaching(true);
 
-  std::string glsl;
+  rdcstr glsl;
 
   CREATE_OBJECT(m_HistogramDescSetLayout,
                 {
@@ -2453,9 +2576,9 @@ void VulkanReplay::HistogramMinMax::Init(WrappedVulkan *driver, VkDescriptorPool
   for(size_t i = 0; i < ARRAY_COUNT(m_HistogramDescSet); i++)
     CREATE_OBJECT(m_HistogramDescSet[i], descriptorPool, m_HistogramDescSetLayout);
 
-  SPIRVCompilationSettings compileSettings;
-  compileSettings.lang = SPIRVSourceLanguage::VulkanGLSL;
-  compileSettings.stage = SPIRVShaderStage::Compute;
+  rdcspv::CompilationSettings compileSettings;
+  compileSettings.lang = rdcspv::InputLanguage::VulkanGLSL;
+  compileSettings.stage = rdcspv::ShaderStage::Compute;
 
   // type max is one higher than the last RESTYPE, and RESTYPES are 1-indexed
   RDCCOMPILE_ASSERT(RESTYPE_TEXTYPEMAX == ARRAY_COUNT(m_MinMaxTilePipe),
@@ -2468,18 +2591,16 @@ void VulkanReplay::HistogramMinMax::Init(WrappedVulkan *driver, VkDescriptorPool
       SPIRVBlob minmaxtile = NULL;
       SPIRVBlob minmaxresult = NULL;
       SPIRVBlob histogram = NULL;
-      std::string err;
+      rdcstr err;
 
-      std::string defines = "";
+      rdcstr defines = shaderCache->GetGlobalDefines();
 
-      if(driver->GetDriverInfo().TexelFetchBrokenDriver())
-        defines += "#define NO_TEXEL_FETCH\n";
-      defines += string("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
-      defines += string("#define UINT_TEX ") + (f == 1 ? "1" : "0") + "\n";
-      defines += string("#define SINT_TEX ") + (f == 2 ? "1" : "0") + "\n";
+      defines += rdcstr("#define SHADER_RESTYPE ") + ToStr(t) + "\n";
+      defines += rdcstr("#define UINT_TEX ") + (f == 1 ? "1" : "0") + "\n";
+      defines += rdcstr("#define SINT_TEX ") + (f == 2 ? "1" : "0") + "\n";
 
-      glsl =
-          GenerateGLSLShader(GetEmbeddedResource(glsl_histogram_comp), eShaderVulkan, 430, defines);
+      glsl = GenerateGLSLShader(GetEmbeddedResource(glsl_histogram_comp), ShaderType::Vulkan, 430,
+                                defines);
 
       err = shaderCache->GetSPIRVBlob(compileSettings, glsl, histogram);
       if(!err.empty())
@@ -2489,8 +2610,8 @@ void VulkanReplay::HistogramMinMax::Init(WrappedVulkan *driver, VkDescriptorPool
         histogram = NULL;
       }
 
-      glsl =
-          GenerateGLSLShader(GetEmbeddedResource(glsl_minmaxtile_comp), eShaderVulkan, 430, defines);
+      glsl = GenerateGLSLShader(GetEmbeddedResource(glsl_minmaxtile_comp), ShaderType::Vulkan, 430,
+                                defines);
 
       err = shaderCache->GetSPIRVBlob(compileSettings, glsl, minmaxtile);
       if(!err.empty())
@@ -2505,8 +2626,8 @@ void VulkanReplay::HistogramMinMax::Init(WrappedVulkan *driver, VkDescriptorPool
 
       if(t == 1)
       {
-        glsl = GenerateGLSLShader(GetEmbeddedResource(glsl_minmaxresult_comp), eShaderVulkan, 430,
-                                  defines);
+        glsl = GenerateGLSLShader(GetEmbeddedResource(glsl_minmaxresult_comp), ShaderType::Vulkan,
+                                  430, defines);
 
         err = shaderCache->GetSPIRVBlob(compileSettings, glsl, minmaxresult);
         if(!err.empty())
