@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,9 +24,7 @@
 
 #pragma once
 
-#include "api/replay/renderdoc_replay.h"
 #include "core/core.h"
-#include "replay/replay_driver.h"
 #include "vk_common.h"
 #include "vk_core.h"
 
@@ -43,10 +41,12 @@ struct MeshDisplayPipelines
     ePipe_Count,
   };
 
-  VkPipeline pipes[ePipe_Count];
+  VkPipeline pipes[ePipe_Count] = {};
 };
 
-struct SPIRVCompilationSettings;
+struct CopyPixelParams;
+
+struct PixelHistoryResources;
 
 class VulkanResourceManager;
 
@@ -59,6 +59,7 @@ public:
   void GetBufferData(ResourceId buff, uint64_t offset, uint64_t len, bytebuf &ret);
 
   bool IsMS2ArraySupported() { return m_MS2ArrayPipe != VK_NULL_HANDLE; }
+  bool IsArray2MSSupported() { return m_Array2MSPipe != VK_NULL_HANDLE; }
   void CopyTex2DMSToArray(VkImage destArray, VkImage srcMS, VkExtent3D extent, uint32_t layers,
                           uint32_t samples, VkFormat fmt);
   void CopyArrayToTex2DMS(VkImage destMS, VkImage srcArray, VkExtent3D extent, uint32_t layers,
@@ -79,6 +80,16 @@ public:
   void PatchLineStripIndexBuffer(const DrawcallDescription *draw, GPUBuffer &indexBuffer,
                                  uint32_t &indexCount);
 
+  bool PixelHistorySetupResources(PixelHistoryResources &resources, VkExtent3D extent,
+                                  VkFormat format, uint32_t numEvents);
+  bool PixelHistoryDestroyResources(const PixelHistoryResources &resources);
+
+  void PixelHistoryCopyPixel(VkCommandBuffer cmd, CopyPixelParams &p, size_t offset);
+
+  VkImageLayout GetImageLayout(ResourceId image, VkImageAspectFlags aspect, uint32_t mip);
+
+  const VulkanCreationInfo::Image &GetImageInfo(ResourceId img);
+
 private:
   // GetBufferData
   GPUBuffer m_ReadbackWindow;
