@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,7 @@
 #undef CreateSemaphore
 #endif
 
+#include "vk_dispatch_defs.h"
 #include "vk_hookset_defs.h"
 
 void InitReplayTables(void *vulkanModule);
@@ -51,8 +52,8 @@ struct InstanceDeviceInfo
 void InitInstanceExtensionTables(VkInstance instance, InstanceDeviceInfo *info);
 void InitDeviceExtensionTables(VkDevice device, InstanceDeviceInfo *info);
 
-VkLayerDispatchTableExtended *GetDeviceDispatchTable(void *device);
-VkLayerInstanceDispatchTableExtended *GetInstanceDispatchTable(void *instance);
+VkDevDispatchTable *GetDeviceDispatchTable(void *device);
+VkInstDispatchTable *GetInstanceDispatchTable(void *instance);
 
 class WrappedVulkan;
 
