@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,8 +22,6 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "api/replay/renderdoc_replay.h"
-
 #include "vk_core.h"
 #include "vk_replay.h"
 
@@ -43,7 +41,8 @@ VkResult WrappedVulkan::vkCreateStreamDescriptorSurfaceGGP(
 
     WrappedVkSurfaceKHR *wrapped = GetWrapped(*pSurface);
 
-    wrapped->record = (VkResourceRecord *)(uintptr_t)pCreateInfo->streamDescriptor;
+    wrapped->record = PackWindowHandleInRecord(WindowingSystem::GGP,
+                                               (void *)(uintptr_t)pCreateInfo->streamDescriptor);
   }
   return ret;
 }
@@ -54,7 +53,7 @@ void VulkanReplay::OutputWindow::SetWindowHandle(WindowingData window)
   return;    // there are no OS specific handles to save.
 }
 
-void VulkanReplay::OutputWindow::CreateSurface(VkInstance inst)
+void VulkanReplay::OutputWindow::CreateSurface(WrappedVulkan *driver, VkInstance inst)
 {
   VkStreamDescriptorSurfaceCreateInfoGGP createInfo;
 
@@ -76,6 +75,13 @@ void VulkanReplay::GetOutputWindowDimensions(uint64_t id, int32_t &w, int32_t &h
 
   OutputWindow &outw = m_OutputWindows[id];
 
+  if(outw.m_WindowSystem == WindowingSystem::Headless)
+  {
+    w = outw.width;
+    h = outw.height;
+    return;
+  }
+
   RDCLOG("Window system is GGP (%d), size is %d, %d", outw.m_WindowSystem, outw.width, outw.height);
   // No window, specify default resolution.
   w = outw.width != 0 ? outw.width : 1920;
