@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,6 @@
  ******************************************************************************/
 
 #include "vk_info.h"
-#include "3rdparty/glslang/SPIRV/spirv.hpp"
 
 VkDynamicState ConvertDynamicState(VulkanDynamicStateIndex idx)
 {
@@ -46,6 +45,7 @@ VkDynamicState ConvertDynamicState(VulkanDynamicStateIndex idx)
     case VkDynamicViewportCoarseSampleOrderNV:
       return VK_DYNAMIC_STATE_VIEWPORT_COARSE_SAMPLE_ORDER_NV;
     case VkDynamicExclusiveScissorNV: return VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_NV;
+    case VkDynamicLineStippleEXT: return VK_DYNAMIC_STATE_LINE_STIPPLE_EXT;
     case VkDynamicCount: break;
   }
 
@@ -75,6 +75,7 @@ VulkanDynamicStateIndex ConvertDynamicState(VkDynamicState state)
     case VK_DYNAMIC_STATE_VIEWPORT_COARSE_SAMPLE_ORDER_NV:
       return VkDynamicViewportCoarseSampleOrderNV;
     case VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_NV: return VkDynamicExclusiveScissorNV;
+    case VK_DYNAMIC_STATE_LINE_STIPPLE_EXT: return VkDynamicLineStippleEXT;
     case VK_DYNAMIC_STATE_RANGE_SIZE:
     case VK_DYNAMIC_STATE_MAX_ENUM: break;
   }
@@ -114,7 +115,9 @@ void DescSetLayout::Init(VulkanResourceManager *resourceMan, VulkanCreationInfo
        bindings[b].descriptorType == VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC)
       dynamicCount++;
 
-    if(pCreateInfo->pBindings[i].pImmutableSamplers)
+    if((bindings[b].descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER ||
+        bindings[b].descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER) &&
+       pCreateInfo->pBindings[i].pImmutableSamplers)
     {
       bindings[b].immutableSampler = new ResourceId[bindings[b].descriptorCount];
 
@@ -124,7 +127,7 @@ void DescSetLayout::Init(VulkanResourceManager *resourceMan, VulkanCreationInfo
   }
 }
 
-void DescSetLayout::CreateBindingsArray(std::vector<DescriptorSetSlot *> &descBindings) const
+void DescSetLayout::CreateBindingsArray(rdcarray<DescriptorSetSlot *> &descBindings) const
 {
   descBindings.resize(bindings.size());
   for(size_t i = 0; i < bindings.size(); i++)
@@ -135,7 +138,7 @@ void DescSetLayout::CreateBindingsArray(std::vector<DescriptorSetSlot *> &descBi
 }
 
 void DescSetLayout::UpdateBindingsArray(const DescSetLayout &prevLayout,
-                                        std::vector<DescriptorSetSlot *> &descBindings) const
+                                        rdcarray<DescriptorSetSlot *> &descBindings) const
 {
   // if we have fewer bindings now, delete the orphaned bindings arrays
   for(size_t i = bindings.size(); i < prevLayout.bindings.size(); i++)
@@ -202,7 +205,8 @@ bool DescSetLayout::operator==(const DescSetLayout &other) const
   return true;
 }
 
-void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, VulkanCreationInfo &info,
+void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan,
+                                        VulkanCreationInfo &info, ResourceId id,
                                         const VkGraphicsPipelineCreateInfo *pCreateInfo)
 {
   flags = pCreateInfo->flags;
@@ -221,23 +225,22 @@ void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, Vulk
   // VkPipelineShaderStageCreateInfo
   for(uint32_t i = 0; i < pCreateInfo->stageCount; i++)
   {
-    ResourceId id = GetResID(pCreateInfo->pStages[i].module);
+    ResourceId shadid = GetResID(pCreateInfo->pStages[i].module);
 
     // convert shader bit to shader index
     int stageIndex = StageIndex(pCreateInfo->pStages[i].stage);
 
     Shader &shad = shaders[stageIndex];
 
-    shad.module = id;
+    shad.module = shadid;
     shad.entryPoint = pCreateInfo->pStages[i].pName;
 
-    ShaderModule::Reflection &reflData = info.m_ShaderModule[id].m_Reflections[shad.entryPoint];
-
-    reflData.Init(resourceMan, id, info.m_ShaderModule[id].spirv, shad.entryPoint,
-                  pCreateInfo->pStages[i].stage);
+    ShaderModuleReflectionKey key(shad.entryPoint, ResourceId());
 
     if(pCreateInfo->pStages[i].pSpecializationInfo)
     {
+      key = ShaderModuleReflectionKey(shad.entryPoint, id);
+
       const byte *data = (const byte *)pCreateInfo->pStages[i].pSpecializationInfo->pData;
 
       const VkSpecializationMapEntry *maps = pCreateInfo->pStages[i].pSpecializationInfo->pMapEntries;
@@ -245,12 +248,17 @@ void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, Vulk
       {
         SpecConstant spec;
         spec.specID = maps[s].constantID;
-        spec.data.assign(data + maps[s].offset, data + maps[s].offset + maps[s].size);
-        // ignore maps[s].size, assume it's enough for the type
+        memcpy(&spec.value, data + maps[s].offset, maps[s].size);
+        spec.dataSize = maps[s].size;
         shad.specialization.push_back(spec);
       }
     }
 
+    ShaderModuleReflection &reflData = info.m_ShaderModule[shadid].m_Reflections[key];
+
+    reflData.Init(resourceMan, shadid, info.m_ShaderModule[shadid].spirv, shad.entryPoint,
+                  pCreateInfo->pStages[i].stage, shad.specialization);
+
     shad.refl = &reflData.refl;
     shad.mapping = &reflData.mapping;
     shad.patchData = &reflData.patchData;
@@ -375,7 +383,7 @@ void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, Vulk
   if(rastStream)
     rasterizationStream = rastStream->rasterizationStream;
 
-  // VkPipelineRasterizationStateStreamCreateInfoEXT
+  // VkPipelineRasterizationDepthClipStateCreateInfoEXT
 
   // default to the opposite of depthClampEnable
   depthClipEnable = !depthClampEnable;
@@ -401,6 +409,26 @@ void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, Vulk
     extraPrimitiveOverestimationSize = conservRast->extraPrimitiveOverestimationSize;
   }
 
+  // VkPipelineRasterizationLineStateCreateInfoEXT
+  lineRasterMode = VK_LINE_RASTERIZATION_MODE_DEFAULT_EXT;
+  stippleEnabled = false;
+  stippleFactor = stipplePattern = 0;
+
+  const VkPipelineRasterizationLineStateCreateInfoEXT *lineRasterState =
+      (const VkPipelineRasterizationLineStateCreateInfoEXT *)FindNextStruct(
+          pCreateInfo->pRasterizationState,
+          VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_LINE_STATE_CREATE_INFO_EXT);
+  if(lineRasterState)
+  {
+    lineRasterMode = lineRasterState->lineRasterizationMode;
+    stippleEnabled = lineRasterState->stippledLineEnable ? true : false;
+    if(lineRasterState->stippledLineEnable)
+    {
+      stippleFactor = lineRasterState->lineStippleFactor;
+      stipplePattern = lineRasterState->lineStipplePattern;
+    }
+  }
+
   // VkPipelineMultisampleStateCreateInfo
   if(pCreateInfo->pMultisampleState)
   {
@@ -429,10 +457,8 @@ void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, Vulk
       if(!dynamicStates[VkDynamicSampleLocationsEXT])
       {
         sampleLocations.gridSize = sampleLoc->sampleLocationsInfo.sampleLocationGridSize;
-        sampleLocations.locations.insert(sampleLocations.locations.begin(),
-                                         sampleLoc->sampleLocationsInfo.pSampleLocations,
-                                         sampleLoc->sampleLocationsInfo.pSampleLocations +
-                                             sampleLoc->sampleLocationsInfo.sampleLocationsCount);
+        sampleLocations.locations.assign(sampleLoc->sampleLocationsInfo.pSampleLocations,
+                                         sampleLoc->sampleLocationsInfo.sampleLocationsCount);
 
         RDCASSERTEQUAL(sampleLoc->sampleLocationsInfo.sampleLocationsPerPixel, rasterizationSamples);
       }
@@ -522,7 +548,7 @@ void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, Vulk
 }
 
 void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, VulkanCreationInfo &info,
-                                        const VkComputePipelineCreateInfo *pCreateInfo)
+                                        ResourceId id, const VkComputePipelineCreateInfo *pCreateInfo)
 {
   flags = pCreateInfo->flags;
 
@@ -532,19 +558,18 @@ void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, Vulk
 
   // VkPipelineShaderStageCreateInfo
   {
-    ResourceId id = GetResID(pCreateInfo->stage.module);
+    ResourceId shadid = GetResID(pCreateInfo->stage.module);
     Shader &shad = shaders[5];    // 5 is the compute shader's index (VS, TCS, TES, GS, FS, CS)
 
-    shad.module = id;
+    shad.module = shadid;
     shad.entryPoint = pCreateInfo->stage.pName;
 
-    ShaderModule::Reflection &reflData = info.m_ShaderModule[id].m_Reflections[shad.entryPoint];
-
-    reflData.Init(resourceMan, id, info.m_ShaderModule[id].spirv, shad.entryPoint,
-                  pCreateInfo->stage.stage);
+    ShaderModuleReflectionKey key(shad.entryPoint, ResourceId());
 
     if(pCreateInfo->stage.pSpecializationInfo)
     {
+      key = ShaderModuleReflectionKey(shad.entryPoint, id);
+
       const byte *data = (const byte *)pCreateInfo->stage.pSpecializationInfo->pData;
 
       const VkSpecializationMapEntry *maps = pCreateInfo->stage.pSpecializationInfo->pMapEntries;
@@ -552,12 +577,17 @@ void VulkanCreationInfo::Pipeline::Init(VulkanResourceManager *resourceMan, Vulk
       {
         SpecConstant spec;
         spec.specID = maps[s].constantID;
-        spec.data.assign(data + maps[s].offset, data + maps[s].offset + maps[s].size);
-        // ignore maps[s].size, assume it's enough for the type
+        memcpy(&spec.value, data + maps[s].offset, maps[s].size);
+        spec.dataSize = maps[s].size;
         shad.specialization.push_back(spec);
       }
     }
 
+    ShaderModuleReflection &reflData = info.m_ShaderModule[shadid].m_Reflections[key];
+
+    reflData.Init(resourceMan, shadid, info.m_ShaderModule[shadid].spirv, shad.entryPoint,
+                  pCreateInfo->stage.stage, shad.specialization);
+
     shad.refl = &reflData.refl;
     shad.mapping = &reflData.mapping;
     shad.patchData = &reflData.patchData;
@@ -638,8 +668,8 @@ void VulkanCreationInfo::RenderPass::Init(VulkanResourceManager *resourceMan,
     dst.storeOp = pCreateInfo->pAttachments[i].storeOp;
     dst.stencilLoadOp = pCreateInfo->pAttachments[i].stencilLoadOp;
     dst.stencilStoreOp = pCreateInfo->pAttachments[i].stencilStoreOp;
-    dst.initialLayout = pCreateInfo->pAttachments[i].initialLayout;
-    dst.finalLayout = pCreateInfo->pAttachments[i].finalLayout;
+    dst.stencilInitialLayout = dst.initialLayout = pCreateInfo->pAttachments[i].initialLayout;
+    dst.stencilFinalLayout = dst.finalLayout = pCreateInfo->pAttachments[i].finalLayout;
   }
 
   // VK_KHR_multiview
@@ -660,10 +690,11 @@ void VulkanCreationInfo::RenderPass::Init(VulkanResourceManager *resourceMan,
 
     dst.inputAttachments.resize(src.inputAttachmentCount);
     dst.inputLayouts.resize(src.inputAttachmentCount);
+    dst.inputStencilLayouts.resize(src.inputAttachmentCount);
     for(uint32_t i = 0; i < src.inputAttachmentCount; i++)
     {
       dst.inputAttachments[i] = src.pInputAttachments[i].attachment;
-      dst.inputLayouts[i] = src.pInputAttachments[i].layout;
+      dst.inputStencilLayouts[i] = dst.inputLayouts[i] = src.pInputAttachments[i].layout;
     }
 
     dst.colorAttachments.resize(src.colorAttachmentCount);
@@ -682,10 +713,11 @@ void VulkanCreationInfo::RenderPass::Init(VulkanResourceManager *resourceMan,
                  src.pDepthStencilAttachment->attachment != VK_ATTACHMENT_UNUSED
              ? (int32_t)src.pDepthStencilAttachment->attachment
              : -1);
-    dst.depthstencilLayout = (src.pDepthStencilAttachment != NULL &&
-                                      src.pDepthStencilAttachment->attachment != VK_ATTACHMENT_UNUSED
-                                  ? src.pDepthStencilAttachment->layout
-                                  : VK_IMAGE_LAYOUT_UNDEFINED);
+    dst.stencilLayout = dst.depthLayout =
+        (src.pDepthStencilAttachment != NULL &&
+                 src.pDepthStencilAttachment->attachment != VK_ATTACHMENT_UNUSED
+             ? src.pDepthStencilAttachment->layout
+             : VK_IMAGE_LAYOUT_UNDEFINED);
 
     dst.fragmentDensityAttachment =
         (fragmentDensity &&
@@ -713,7 +745,7 @@ void VulkanCreationInfo::RenderPass::Init(VulkanResourceManager *resourceMan,
 
 void VulkanCreationInfo::RenderPass::Init(VulkanResourceManager *resourceMan,
                                           VulkanCreationInfo &info,
-                                          const VkRenderPassCreateInfo2KHR *pCreateInfo)
+                                          const VkRenderPassCreateInfo2 *pCreateInfo)
 {
   attachments.resize(pCreateInfo->attachmentCount);
   for(uint32_t i = 0; i < pCreateInfo->attachmentCount; i++)
@@ -726,8 +758,20 @@ void VulkanCreationInfo::RenderPass::Init(VulkanResourceManager *resourceMan,
     dst.storeOp = pCreateInfo->pAttachments[i].storeOp;
     dst.stencilLoadOp = pCreateInfo->pAttachments[i].stencilLoadOp;
     dst.stencilStoreOp = pCreateInfo->pAttachments[i].stencilStoreOp;
-    dst.initialLayout = pCreateInfo->pAttachments[i].initialLayout;
-    dst.finalLayout = pCreateInfo->pAttachments[i].finalLayout;
+    dst.stencilInitialLayout = dst.initialLayout = pCreateInfo->pAttachments[i].initialLayout;
+    dst.stencilFinalLayout = dst.finalLayout = pCreateInfo->pAttachments[i].finalLayout;
+
+    // VK_KHR_separate_depth_stencil_layouts
+    const VkAttachmentDescriptionStencilLayoutKHR *separateStencil =
+        (const VkAttachmentDescriptionStencilLayoutKHR *)FindNextStruct(
+            &pCreateInfo->pAttachments[i],
+            VK_STRUCTURE_TYPE_ATTACHMENT_DESCRIPTION_STENCIL_LAYOUT_KHR);
+
+    if(separateStencil)
+    {
+      dst.stencilInitialLayout = separateStencil->stencilInitialLayout;
+      dst.stencilFinalLayout = separateStencil->stencilFinalLayout;
+    }
   }
 
   // VK_EXT_fragment_density_map
@@ -738,15 +782,21 @@ void VulkanCreationInfo::RenderPass::Init(VulkanResourceManager *resourceMan,
   subpasses.resize(pCreateInfo->subpassCount);
   for(uint32_t subp = 0; subp < pCreateInfo->subpassCount; subp++)
   {
-    const VkSubpassDescription2KHR &src = pCreateInfo->pSubpasses[subp];
+    const VkSubpassDescription2 &src = pCreateInfo->pSubpasses[subp];
     Subpass &dst = subpasses[subp];
 
     dst.inputAttachments.resize(src.inputAttachmentCount);
     dst.inputLayouts.resize(src.inputAttachmentCount);
+    dst.inputStencilLayouts.resize(src.inputAttachmentCount);
     for(uint32_t i = 0; i < src.inputAttachmentCount; i++)
     {
       dst.inputAttachments[i] = src.pInputAttachments[i].attachment;
-      dst.inputLayouts[i] = src.pInputAttachments[i].layout;
+      dst.inputStencilLayouts[i] = dst.inputLayouts[i] = src.pInputAttachments[i].layout;
+      const VkAttachmentReferenceStencilLayoutKHR *stencilLayout =
+          (const VkAttachmentReferenceStencilLayoutKHR *)FindNextStruct(
+              &src.pInputAttachments[i], VK_STRUCTURE_TYPE_ATTACHMENT_REFERENCE_STENCIL_LAYOUT_KHR);
+      if(stencilLayout != NULL)
+        dst.inputStencilLayouts[i] = stencilLayout->stencilLayout;
     }
 
     dst.colorAttachments.resize(src.colorAttachmentCount);
@@ -765,10 +815,19 @@ void VulkanCreationInfo::RenderPass::Init(VulkanResourceManager *resourceMan,
                  src.pDepthStencilAttachment->attachment != VK_ATTACHMENT_UNUSED
              ? (int32_t)src.pDepthStencilAttachment->attachment
              : -1);
-    dst.depthstencilLayout = (src.pDepthStencilAttachment != NULL &&
-                                      src.pDepthStencilAttachment->attachment != VK_ATTACHMENT_UNUSED
-                                  ? src.pDepthStencilAttachment->layout
-                                  : VK_IMAGE_LAYOUT_UNDEFINED);
+    dst.stencilLayout = dst.depthLayout =
+        (src.pDepthStencilAttachment != NULL &&
+                 src.pDepthStencilAttachment->attachment != VK_ATTACHMENT_UNUSED
+             ? src.pDepthStencilAttachment->layout
+             : VK_IMAGE_LAYOUT_UNDEFINED);
+
+    // VK_KHR_separate_depth_stencil_layouts
+    const VkAttachmentReferenceStencilLayoutKHR *separateStencil =
+        (const VkAttachmentReferenceStencilLayoutKHR *)FindNextStruct(
+            src.pDepthStencilAttachment, VK_STRUCTURE_TYPE_ATTACHMENT_REFERENCE_STENCIL_LAYOUT_KHR);
+
+    if(separateStencil)
+      dst.stencilLayout = separateStencil->stencilLayout;
 
     dst.fragmentDensityAttachment =
         (fragmentDensity &&
@@ -798,11 +857,38 @@ void VulkanCreationInfo::Framebuffer::Init(VulkanResourceManager *resourceMan,
   height = pCreateInfo->height;
   layers = pCreateInfo->layers;
 
+  imageless = false;
+
   attachments.resize(pCreateInfo->attachmentCount);
-  for(uint32_t i = 0; i < pCreateInfo->attachmentCount; i++)
+  if(pCreateInfo->flags & VK_FRAMEBUFFER_CREATE_IMAGELESS_BIT)
   {
-    attachments[i].view = GetResID(pCreateInfo->pAttachments[i]);
-    attachments[i].format = info.m_ImageView[attachments[i].view].format;
+    imageless = true;
+
+    // VK_KHR_imageless_framebuffer
+    const VkFramebufferAttachmentsCreateInfo *attachmentsInfo =
+        (const VkFramebufferAttachmentsCreateInfo *)FindNextStruct(
+            pCreateInfo, VK_STRUCTURE_TYPE_FRAMEBUFFER_ATTACHMENTS_CREATE_INFO);
+
+    RDCASSERTEQUAL(pCreateInfo->attachmentCount, attachmentsInfo->attachmentImageInfoCount);
+
+    for(uint32_t i = 0; i < pCreateInfo->attachmentCount; i++)
+    {
+      attachments[i].createdView = ResourceId();
+      // there must be at least one format in the list so we can safely look at [0].
+      // also all entries must be compatible, so if [0] doesn't have stencil then none of them do,
+      // and vice-versa.
+      attachments[i].hasStencil =
+          IsStencilFormat(attachmentsInfo->pAttachmentImageInfos[i].pViewFormats[0]);
+    }
+  }
+  else
+  {
+    for(uint32_t i = 0; i < pCreateInfo->attachmentCount; i++)
+    {
+      attachments[i].createdView = GetResID(pCreateInfo->pAttachments[i]);
+      attachments[i].hasStencil =
+          IsStencilFormat(info.m_ImageView[attachments[i].createdView].format);
+    }
   }
 }
 
@@ -818,6 +904,7 @@ void VulkanCreationInfo::Buffer::Init(VulkanResourceManager *resourceMan, Vulkan
 {
   usage = pCreateInfo->usage;
   size = pCreateInfo->size;
+  gpuAddress = 0;
 }
 
 void VulkanCreationInfo::BufferView::Init(VulkanResourceManager *resourceMan,
@@ -873,11 +960,11 @@ void VulkanCreationInfo::Sampler::Init(VulkanResourceManager *resourceMan, Vulka
   borderColor = pCreateInfo->borderColor;
   unnormalizedCoordinates = pCreateInfo->unnormalizedCoordinates != 0;
 
-  reductionMode = VK_SAMPLER_REDUCTION_MODE_WEIGHTED_AVERAGE_EXT;
+  reductionMode = VK_SAMPLER_REDUCTION_MODE_WEIGHTED_AVERAGE;
 
-  const VkSamplerReductionModeCreateInfoEXT *reduction =
-      (const VkSamplerReductionModeCreateInfoEXT *)FindNextStruct(
-          pCreateInfo, VK_STRUCTURE_TYPE_SAMPLER_REDUCTION_MODE_CREATE_INFO_EXT);
+  const VkSamplerReductionModeCreateInfo *reduction =
+      (const VkSamplerReductionModeCreateInfo *)FindNextStruct(
+          pCreateInfo, VK_STRUCTURE_TYPE_SAMPLER_REDUCTION_MODE_CREATE_INFO);
   if(reduction)
   {
     reductionMode = reduction->reductionMode;
@@ -896,7 +983,7 @@ static TextureSwizzle Convert(VkComponentSwizzle s, int i)
 {
   switch(s)
   {
-    default: RDCWARN("Unexpected component swizzle value %d", (int)s);
+    default: RDCWARN("Unexpected component swizzle value %d", (int)s); DELIBERATE_FALLTHROUGH();
     case VK_COMPONENT_SWIZZLE_IDENTITY: break;
     case VK_COMPONENT_SWIZZLE_ZERO: return TextureSwizzle::Zero;
     case VK_COMPONENT_SWIZZLE_ONE: return TextureSwizzle::One;
@@ -989,31 +1076,26 @@ void VulkanCreationInfo::ShaderModule::Init(VulkanResourceManager *resourceMan,
   else
   {
     RDCASSERT(pCreateInfo->codeSize % sizeof(uint32_t) == 0);
-    ParseSPIRV((uint32_t *)pCreateInfo->pCode, pCreateInfo->codeSize / sizeof(uint32_t), spirv);
+    spirv.Parse(rdcarray<uint32_t>((uint32_t *)(pCreateInfo->pCode),
+                                   pCreateInfo->codeSize / sizeof(uint32_t)));
   }
 }
 
-void VulkanCreationInfo::ShaderModule::Reflection::Init(VulkanResourceManager *resourceMan,
-                                                        ResourceId id, const SPVModule &spv,
-                                                        const std::string &entry,
-                                                        VkShaderStageFlagBits stage)
+void VulkanCreationInfo::ShaderModuleReflection::Init(VulkanResourceManager *resourceMan,
+                                                      ResourceId id, const rdcspv::Reflector &spv,
+                                                      const rdcstr &entry,
+                                                      VkShaderStageFlagBits stage,
+                                                      const rdcarray<SpecConstant> &specInfo)
 {
   if(entryPoint.empty())
   {
     entryPoint = entry;
     stageIndex = StageIndex(stage);
 
-    spv.MakeReflection(GraphicsAPI::Vulkan, ShaderStage(stageIndex), entryPoint, refl, mapping,
-                       patchData);
+    spv.MakeReflection(GraphicsAPI::Vulkan, ShaderStage(stageIndex), entryPoint, specInfo, refl,
+                       mapping, patchData);
 
     refl.resourceId = resourceMan->GetOriginalID(id);
-    refl.entryPoint = entryPoint;
-
-    if(!spv.spirv.empty())
-    {
-      refl.encoding = ShaderEncoding::SPIRV;
-      refl.rawBytes.assign((byte *)spv.spirv.data(), spv.spirv.size() * sizeof(uint32_t));
-    }
   }
 }
 
@@ -1022,7 +1104,7 @@ void VulkanCreationInfo::DescSetPool::Init(VulkanResourceManager *resourceMan,
                                            const VkDescriptorPoolCreateInfo *pCreateInfo)
 {
   maxSets = pCreateInfo->maxSets;
-  poolSizes.assign(pCreateInfo->pPoolSizes, pCreateInfo->pPoolSizes + pCreateInfo->poolSizeCount);
+  poolSizes.assign(pCreateInfo->pPoolSizes, pCreateInfo->poolSizeCount);
 }
 
 void VulkanCreationInfo::DescSetPool::CreateOverflow(VkDevice device,
@@ -1053,8 +1135,7 @@ void VulkanCreationInfo::DescSetPool::CreateOverflow(VkDevice device,
 void DescUpdateTemplate::Init(VulkanResourceManager *resourceMan, VulkanCreationInfo &info,
                               const VkDescriptorUpdateTemplateCreateInfo *pCreateInfo)
 {
-  updates.insert(updates.begin(), pCreateInfo->pDescriptorUpdateEntries,
-                 pCreateInfo->pDescriptorUpdateEntries + pCreateInfo->descriptorUpdateEntryCount);
+  updates.assign(pCreateInfo->pDescriptorUpdateEntries, pCreateInfo->descriptorUpdateEntryCount);
 
   bindPoint = pCreateInfo->pipelineBindPoint;
 
@@ -1115,7 +1196,7 @@ void DescUpdateTemplate::Init(VulkanResourceManager *resourceMan, VulkanCreation
     }
     else
     {
-      const std::vector<ResourceId> &descSetLayouts =
+      const rdcarray<ResourceId> &descSetLayouts =
           info.m_PipelineLayout[GetResID(pCreateInfo->pipelineLayout)].descSetLayouts;
 
       layout = info.m_DescSetLayout[descSetLayouts[pCreateInfo->set]];
@@ -1190,6 +1271,7 @@ void DescUpdateTemplate::Apply(const void *pData, DescUpdateTemplateApplication
       write.pBufferInfo = &application.bufInfo[idx];
     }
 
-    application.writes.push_back(write);
+    if(write.descriptorCount != 0)
+      application.writes.push_back(write);
   }
 }
