@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -83,112 +83,25 @@ uint32_t WrappedVulkan::PhysicalDeviceData::GetMemoryIndex(uint32_t resourceRequ
   return best;
 }
 
-#define CREATE_NON_COHERENT_ATTRACTIVE_MEMORY 0
-
-void WrappedVulkan::RemapMemoryIndices(VkPhysicalDeviceMemoryProperties *memProps,
-                                       uint32_t **memIdxMap)
-{
-  uint32_t *memmap = new uint32_t[VK_MAX_MEMORY_TYPES];
-  *memIdxMap = memmap;
-  m_MemIdxMaps.push_back(memmap);
-
-  for(size_t i = 0; i < VK_MAX_MEMORY_TYPES; i++)
-    memmap[i] = ~0U;
-
-// basic idea here:
-// We want to discourage coherent memory maps as much as possible while capturing,
-// as they're painful to track. Unfortunately the spec guarantees that at least
-// one such memory type will be available, and we must follow that.
-//
-// So, rather than removing the coherent memory type we make it as unappealing as
-// possible and try and ensure that only someone looking specifically for a coherent
-// memory type will find it. That way hopefully memory selection algorithms will
-// pick non-coherent memory and do proper flushing as necessary.
-
-// we want to add a new heap, hopefully there is room
-#if CREATE_NON_COHERENT_ATTRACTIVE_MEMORY
-  RDCASSERT(memProps->memoryHeapCount < VK_MAX_MEMORY_HEAPS - 1);
-
-  uint32_t coherentHeap = memProps->memoryHeapCount;
-  memProps->memoryHeapCount++;
-
-  // make a new heap that's tiny. If any applications look at heap sizes to determine
-  // viability, they'll dislike the look of this one (the real heaps should be much
-  // bigger).
-  memProps->memoryHeaps[coherentHeap].flags = 0;    // not device local
-  memProps->memoryHeaps[coherentHeap].size = 32 * 1024 * 1024;
-#endif
-
-  // for every coherent memory type, add a non-coherent type first, then
-  // mark the coherent type with our crappy heap
-
-  uint32_t origCount = memProps->memoryTypeCount;
-  VkMemoryType origTypes[VK_MAX_MEMORY_TYPES];
-  memcpy(origTypes, memProps->memoryTypes, sizeof(origTypes));
-
-  uint32_t newtypeidx = 0;
-
-  for(uint32_t i = 0; i < origCount; i++)
-  {
-#if CREATE_NON_COHERENT_ATTRACTIVE_MEMORY
-    if((origTypes[i].propertyFlags & VK_MEMORY_PROPERTY_HOST_COHERENT_BIT) != 0)
-    {
-      // coherent type found.
-
-      // can we still add a new type without exceeding the max?
-      if(memProps->memoryTypeCount + 1 <= VK_MAX_MEMORY_TYPES)
-      {
-        // copy both types from the original type
-        memProps->memoryTypes[newtypeidx] = origTypes[i];
-        memProps->memoryTypes[newtypeidx + 1] = origTypes[i];
-
-        // mark first as non-coherent, cached
-        memProps->memoryTypes[newtypeidx].propertyFlags &= ~VK_MEMORY_PROPERTY_HOST_COHERENT_BIT;
-        memProps->memoryTypes[newtypeidx].propertyFlags |= VK_MEMORY_PROPERTY_HOST_CACHED_BIT;
-
-        // point second at bad heap
-        memProps->memoryTypes[newtypeidx + 1].heapIndex = coherentHeap;
-
-        // point both new types at this original type
-        memmap[newtypeidx++] = i;
-        memmap[newtypeidx++] = i;
-
-        // we added a type
-        memProps->memoryTypeCount++;
-      }
-      else
-      {
-        // can't add a new type, but we can at least repoint this coherent
-        // type at the bad heap to discourage use
-        memProps->memoryTypes[newtypeidx] = origTypes[i];
-        memProps->memoryTypes[newtypeidx].heapIndex = coherentHeap;
-        memmap[newtypeidx++] = i;
-      }
-    }
-    else
-#endif
-    {
-      // non-coherent already or non-hostvisible, just copy through
-      memProps->memoryTypes[newtypeidx] = origTypes[i];
-      memmap[newtypeidx++] = i;
-    }
-  }
-}
-
 MemoryAllocation WrappedVulkan::AllocateMemoryForResource(bool buffer, VkMemoryRequirements mrq,
                                                           MemoryScope scope, MemoryType type)
 {
+  const VkDeviceSize nonCoherentAtomSize = GetDeviceProps().limits.nonCoherentAtomSize;
+
   MemoryAllocation ret;
   ret.scope = scope;
   ret.type = type;
   ret.buffer = buffer;
   ret.size = AlignUp(mrq.size, mrq.alignment);
+  // for ease, ensure all allocations are multiples of the non-coherent atom size, so we can
+  // invalidate/flush safely. This is at most 256 bytes which is likely already satisfied.
+  ret.size = AlignUp(ret.size, nonCoherentAtomSize);
 
-  RDCDEBUG("Allocating 0x%llx with alignment 0x%llx in 0x%x for a %s (%s in %s)", ret.size,
-           mrq.alignment, mrq.memoryTypeBits, buffer ? "buffer" : "image", ToStr(type).c_str(),
-           ToStr(scope).c_str());
+  RDCDEBUG("Allocating 0x%llx (0x%llx requested) with alignment 0x%llx in 0x%x for a %s (%s in %s)",
+           ret.size, mrq.size, mrq.alignment, mrq.memoryTypeBits, buffer ? "buffer" : "image",
+           ToStr(type).c_str(), ToStr(scope).c_str());
 
-  std::vector<MemoryAllocation> &blockList = m_MemoryBlocks[(size_t)scope];
+  rdcarray<MemoryAllocation> &blockList = m_MemoryBlocks[(size_t)scope];
 
   // first try to find a match
   int i = 0;
@@ -328,6 +241,9 @@ MemoryAllocation WrappedVulkan::AllocateMemoryForResource(bool buffer, VkMemoryR
     ret.mem = chunk.mem;
   }
 
+  // ensure the returned size is accurate to what was requested, not what we padded
+  ret.size = mrq.size;
+
   return ret;
 }
 
@@ -355,7 +271,7 @@ MemoryAllocation WrappedVulkan::AllocateMemoryForResource(VkBuffer buf, MemorySc
 
 void WrappedVulkan::FreeAllMemory(MemoryScope scope)
 {
-  std::vector<MemoryAllocation> &allocList = m_MemoryBlocks[(size_t)scope];
+  rdcarray<MemoryAllocation> &allocList = m_MemoryBlocks[(size_t)scope];
 
   if(allocList.empty())
     return;
