@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,8 +32,7 @@
 void VulkanDebugManager::CopyTex2DMSToArray(VkImage destArray, VkImage srcMS, VkExtent3D extent,
                                             uint32_t layers, uint32_t samples, VkFormat fmt)
 {
-  if(!m_pDriver->GetDeviceFeatures().shaderStorageImageMultisample ||
-     !m_pDriver->GetDeviceFeatures().shaderStorageImageWriteWithoutFormat)
+  if(!m_pDriver->GetDeviceFeatures().shaderStorageImageWriteWithoutFormat)
     return;
 
   if(m_MS2ArrayPipe == VK_NULL_HANDLE)
@@ -84,6 +83,8 @@ void VulkanDebugManager::CopyTex2DMSToArray(VkImage destArray, VkImage srcMS, Vk
     return;
   }
 
+  VkMarkerRegion region("CopyTex2DMSToArray");
+
   if(IsStencilOnlyFormat(fmt))
     viewInfo.subresourceRange.aspectMask = VK_IMAGE_ASPECT_STENCIL_BIT;
   else if(IsDepthOrStencilFormat(fmt))
@@ -176,6 +177,8 @@ void VulkanDebugManager::CopyDepthTex2DMSToArray(VkImage destArray, VkImage srcM
   if(pipe == VK_NULL_HANDLE)
     return;
 
+  VkMarkerRegion region("CopyDepthTex2DMSToArray");
+
   VkDevice dev = m_Device;
 
   VkResult vkr = VK_SUCCESS;
@@ -354,7 +357,7 @@ void VulkanDebugManager::CopyDepthTex2DMSToArray(VkImage destArray, VkImage srcM
     {
       params.w = numStencil == 1 ? 1000 : s;    // currentStencil;
 
-      ObjDisp(cmd)->CmdSetStencilReference(Unwrap(cmd), VK_STENCIL_FRONT_AND_BACK, s);
+      ObjDisp(cmd)->CmdSetStencilReference(Unwrap(cmd), VK_STENCIL_FACE_FRONT_AND_BACK, s);
       ObjDisp(cmd)->CmdPushConstants(Unwrap(cmd), Unwrap(m_ArrayMSPipeLayout), VK_SHADER_STAGE_ALL,
                                      0, sizeof(Vec4u), &params);
       ObjDisp(cmd)->CmdDraw(Unwrap(cmd), 4, 1, 0, 0);
@@ -438,6 +441,8 @@ void VulkanDebugManager::CopyArrayToTex2DMS(VkImage destMS, VkImage srcArray, Vk
     return;
   }
 
+  VkMarkerRegion region("CopyArrayToTex2DMS");
+
   if(IsStencilOnlyFormat(fmt))
     viewInfo.subresourceRange.aspectMask = VK_IMAGE_ASPECT_STENCIL_BIT;
   else if(IsDepthOrStencilFormat(fmt))
@@ -539,6 +544,8 @@ void VulkanDebugManager::CopyDepthArrayToTex2DMS(VkImage destMS, VkImage srcArra
   if(pipe == VK_NULL_HANDLE)
     return;
 
+  VkMarkerRegion region("CopyDepthArrayToTex2DMS");
+
   VkDevice dev = m_Device;
 
   VkResult vkr = VK_SUCCESS;
@@ -717,7 +724,7 @@ void VulkanDebugManager::CopyDepthArrayToTex2DMS(VkImage destMS, VkImage srcArra
     {
       params.w = numStencil == 1 ? 1000 : s;    // currentStencil;
 
-      ObjDisp(cmd)->CmdSetStencilReference(Unwrap(cmd), VK_STENCIL_FRONT_AND_BACK, s);
+      ObjDisp(cmd)->CmdSetStencilReference(Unwrap(cmd), VK_STENCIL_FACE_FRONT_AND_BACK, s);
       ObjDisp(cmd)->CmdPushConstants(Unwrap(cmd), Unwrap(m_ArrayMSPipeLayout), VK_SHADER_STAGE_ALL,
                                      0, sizeof(Vec4u), &params);
       ObjDisp(cmd)->CmdDraw(Unwrap(cmd), 4, 1, 0, 0);
