@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,15 +23,17 @@
  ******************************************************************************/
 
 #include <float.h>
-#include "3rdparty/glslang/SPIRV/spirv.hpp"
+#include <math.h>
+#include <algorithm>
 #include "data/glsl_shaders.h"
 #include "driver/shaders/spirv/spirv_common.h"
-#include "maths/camera.h"
+#include "driver/shaders/spirv/spirv_gen.h"
 #include "maths/formatpacking.h"
 #include "maths/matrix.h"
 #include "strings/string_utils.h"
 #include "vk_core.h"
 #include "vk_debug.h"
+#include "vk_replay.h"
 #include "vk_shader_cache.h"
 
 #define VULKAN 1
@@ -40,7 +42,7 @@
 struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
 {
   VulkanQuadOverdrawCallback(WrappedVulkan *vk, VkDescriptorSetLayout descSetLayout,
-                             VkDescriptorSet descSet, const vector<uint32_t> &events)
+                             VkDescriptorSet descSet, const rdcarray<uint32_t> &events)
       : m_pDriver(vk),
         m_DescSetLayout(descSetLayout),
         m_DescSet(descSet),
@@ -52,7 +54,7 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
   ~VulkanQuadOverdrawCallback() { m_pDriver->SetDrawcallCB(NULL); }
   void PreDraw(uint32_t eid, VkCommandBuffer cmd)
   {
-    if(std::find(m_Events.begin(), m_Events.end(), eid) == m_Events.end())
+    if(!m_Events.contains(eid))
       return;
 
     // we customise the pipeline to disable framebuffer writes, but perform normal testing
@@ -65,10 +67,10 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
     VulkanRenderState &pipestate = m_pDriver->GetRenderState();
 
     // check cache first
-    pair<uint32_t, VkPipeline> pipe = m_PipelineCache[pipestate.graphics.pipeline];
+    CachedPipeline pipe = m_PipelineCache[pipestate.graphics.pipeline];
 
     // if we don't get a hit, create a modified pipeline
-    if(pipe.second == VK_NULL_HANDLE)
+    if(pipe.pipe == VK_NULL_HANDLE)
     {
       VulkanCreationInfo &c = *pipestate.m_CreationInfo;
 
@@ -88,7 +90,7 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
       // this layout has storage image and
       descSetLayouts[descSet] = m_DescSetLayout;
 
-      const vector<VkPushConstantRange> &push = c.m_PipelineLayout[p.layout].pushRanges;
+      const rdcarray<VkPushConstantRange> &push = c.m_PipelineLayout[p.layout].pushRanges;
 
       VkPipelineLayoutCreateInfo pipeLayoutInfo = {
           VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO,
@@ -101,9 +103,8 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
       };
 
       // create pipeline layout with same descriptor set layouts, plus our mesh output set
-      VkPipelineLayout pipeLayout;
-      vkr =
-          m_pDriver->vkCreatePipelineLayout(m_pDriver->GetDev(), &pipeLayoutInfo, NULL, &pipeLayout);
+      vkr = m_pDriver->vkCreatePipelineLayout(m_pDriver->GetDev(), &pipeLayoutInfo, NULL,
+                                              &pipe.pipeLayout);
       RDCASSERTEQUAL(vkr, VK_SUCCESS);
 
       SAFE_DELETE_ARRAY(descSetLayouts);
@@ -113,7 +114,7 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
                                                             pipestate.graphics.pipeline);
 
       // repoint pipeline layout
-      pipeCreateInfo.layout = pipeLayout;
+      pipeCreateInfo.layout = pipe.pipeLayout;
 
       // disable colour writes/blends
       VkPipelineColorBlendStateCreateInfo *cb =
@@ -138,17 +139,18 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
           (VkPipelineRasterizationStateCreateInfo *)pipeCreateInfo.pRasterizationState;
       rs->rasterizerDiscardEnable = false;
 
-      std::vector<uint32_t> spirv =
+      rdcarray<uint32_t> spirv =
           *m_pDriver->GetShaderCache()->GetBuiltinBlob(BuiltinShader::QuadWriteFS);
 
       // patch spirv, change descriptor set to descSet value
       size_t it = 5;
       while(it < spirv.size())
       {
-        uint16_t WordCount = spirv[it] >> spv::WordCountShift;
-        spv::Op opcode = spv::Op(spirv[it] & spv::OpCodeMask);
+        uint16_t WordCount = spirv[it] >> rdcspv::WordCountShift;
+        rdcspv::Op opcode = rdcspv::Op(spirv[it] & rdcspv::OpCodeMask);
 
-        if(opcode == spv::OpDecorate && spirv[it + 2] == spv::DecorationDescriptorSet)
+        if(opcode == rdcspv::Op::Decorate &&
+           spirv[it + 2] == (uint32_t)rdcspv::Decoration::DescriptorSet)
         {
           spirv[it + 3] = descSet;
           break;
@@ -169,13 +171,9 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
 
       VkDevice dev = m_pDriver->GetDev();
 
-      vkr = ObjDisp(dev)->CreateShaderModule(Unwrap(dev), &modinfo, NULL, &module);
+      vkr = m_pDriver->vkCreateShaderModule(dev, &modinfo, NULL, &module);
       RDCASSERTEQUAL(vkr, VK_SUCCESS);
 
-      m_pDriver->GetResourceManager()->WrapResource(Unwrap(dev), module);
-
-      m_pDriver->GetResourceManager()->AddLiveResource(GetResID(module), module);
-
       bool found = false;
       for(uint32_t i = 0; i < pipeCreateInfo.stageCount; i++)
       {
@@ -206,22 +204,22 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
       }
 
       vkr = m_pDriver->vkCreateGraphicsPipelines(dev, VK_NULL_HANDLE, 1, &pipeCreateInfo, NULL,
-                                                 &pipe.second);
+                                                 &pipe.pipe);
       RDCASSERTEQUAL(vkr, VK_SUCCESS);
 
-      ObjDisp(dev)->DestroyShaderModule(Unwrap(dev), Unwrap(module), NULL);
-      m_pDriver->GetResourceManager()->ReleaseWrappedResource(module);
+      m_pDriver->vkDestroyShaderModule(dev, module, NULL);
 
-      pipe.first = descSet;
+      pipe.descSet = descSet;
 
       m_PipelineCache[pipestate.graphics.pipeline] = pipe;
     }
 
     // modify state for first draw call
-    pipestate.graphics.pipeline = GetResID(pipe.second);
-    RDCASSERT(pipestate.graphics.descSets.size() >= pipe.first);
-    pipestate.graphics.descSets.resize(pipe.first + 1);
-    pipestate.graphics.descSets[pipe.first].descSet = GetResID(m_DescSet);
+    pipestate.graphics.pipeline = GetResID(pipe.pipe);
+    RDCASSERT(pipestate.graphics.descSets.size() >= pipe.descSet);
+    pipestate.graphics.descSets.resize(pipe.descSet + 1);
+    pipestate.graphics.descSets[pipe.descSet].pipeLayout = GetResID(pipe.pipeLayout);
+    pipestate.graphics.descSets[pipe.descSet].descSet = GetResID(m_DescSet);
 
     if(cmd)
       pipestate.BindPipeline(cmd, VulkanRenderState::BindGraphics, false);
@@ -229,7 +227,7 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
 
   bool PostDraw(uint32_t eid, VkCommandBuffer cmd)
   {
-    if(std::find(m_Events.begin(), m_Events.end(), eid) == m_Events.end())
+    if(!m_Events.contains(eid))
       return false;
 
     // restore the render state and go ahead with the real draw
@@ -263,10 +261,16 @@ struct VulkanQuadOverdrawCallback : public VulkanDrawcallCallback
   WrappedVulkan *m_pDriver;
   VkDescriptorSetLayout m_DescSetLayout;
   VkDescriptorSet m_DescSet;
-  const vector<uint32_t> &m_Events;
+  const rdcarray<uint32_t> &m_Events;
 
   // cache modified pipelines
-  map<ResourceId, pair<uint32_t, VkPipeline> > m_PipelineCache;
+  struct CachedPipeline
+  {
+    uint32_t descSet;
+    VkPipelineLayout pipeLayout;
+    VkPipeline pipe;
+  };
+  std::map<ResourceId, CachedPipeline> m_PipelineCache;
   VulkanRenderState m_PrevState;
 };
 
@@ -278,7 +282,7 @@ void VulkanDebugManager::PatchFixedColShader(VkShaderModule &mod, float col[4])
     float *data;
   } alias;
 
-  std::vector<uint32_t> spv = *m_pDriver->GetShaderCache()->GetBuiltinBlob(BuiltinShader::FixedColFS);
+  rdcarray<uint32_t> spv = *m_pDriver->GetShaderCache()->GetBuiltinBlob(BuiltinShader::FixedColFS);
 
   alias.spirv = &spv[0];
   size_t spirvLength = spv.size();
@@ -288,10 +292,10 @@ void VulkanDebugManager::PatchFixedColShader(VkShaderModule &mod, float col[4])
   size_t it = 5;
   while(it < spirvLength)
   {
-    uint16_t WordCount = alias.spirv[it] >> spv::WordCountShift;
-    spv::Op opcode = spv::Op(alias.spirv[it] & spv::OpCodeMask);
+    uint16_t WordCount = alias.spirv[it] >> rdcspv::WordCountShift;
+    rdcspv::Op opcode = rdcspv::Op(alias.spirv[it] & rdcspv::OpCodeMask);
 
-    if(opcode == spv::OpConstant)
+    if(opcode == rdcspv::Op::Constant)
     {
       if(alias.data[it + 3] >= 1.0f && alias.data[it + 3] <= 1.5f)
         alias.data[it + 3] = col[0];
@@ -332,6 +336,7 @@ void VulkanDebugManager::PatchLineStripIndexBuffer(const DrawcallDescription *dr
 
   bytebuf indices;
 
+  uint8_t *idx8 = NULL;
   uint16_t *idx16 = NULL;
   uint32_t *idx32 = NULL;
 
@@ -341,16 +346,18 @@ void VulkanDebugManager::PatchLineStripIndexBuffer(const DrawcallDescription *dr
                   rs.ibuffer.offs + uint64_t(draw->indexOffset) * draw->indexByteWidth,
                   uint64_t(draw->numIndices) * draw->indexByteWidth, indices);
 
-    if(rs.ibuffer.bytewidth == 2)
-      idx16 = (uint16_t *)indices.data();
-    else
+    if(rs.ibuffer.bytewidth == 4)
       idx32 = (uint32_t *)indices.data();
+    else if(rs.ibuffer.bytewidth == 1)
+      idx8 = (uint8_t *)indices.data();
+    else
+      idx16 = (uint16_t *)indices.data();
   }
 
   // we just patch up to 32-bit since we'll be adding more indices and we might overflow 16-bit.
-  std::vector<uint32_t> patchedIndices;
+  rdcarray<uint32_t> patchedIndices;
 
-  ::PatchLineStripIndexBuffer(draw, NULL, idx16, idx32, patchedIndices);
+  ::PatchLineStripIndexBuffer(draw, idx8, idx16, idx32, patchedIndices);
 
   indexBuffer.Create(m_pDriver, m_Device, patchedIndices.size() * sizeof(uint32_t), 1,
                      GPUBuffer::eGPUBufferIBuffer);
@@ -391,10 +398,11 @@ void VulkanDebugManager::PatchLineStripIndexBuffer(const DrawcallDescription *dr
   indexCount = (uint32_t)patchedIndices.size();
 }
 
-ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, DebugOverlay overlay,
-                                       uint32_t eventId, const std::vector<uint32_t> &passEvents)
+ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeCast, FloatVector clearCol,
+                                       DebugOverlay overlay, uint32_t eventId,
+                                       const rdcarray<uint32_t> &passEvents)
 {
-  const VkLayerDispatchTable *vt = ObjDisp(m_Device);
+  const VkDevDispatchTable *vt = ObjDisp(m_Device);
 
   VulkanShaderCache *shaderCache = m_pDriver->GetShaderCache();
 
@@ -405,7 +413,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
   // refreshed before the UI layer can update the current texture.
   {
     const VulkanCreationInfo::Framebuffer &fb =
-        m_pDriver->m_CreationInfo.m_Framebuffer[m_pDriver->m_RenderState.framebuffer];
+        m_pDriver->m_CreationInfo.m_Framebuffer[m_pDriver->m_RenderState.GetFramebuffer()];
 
     if(fb.width != iminfo.extent.width || fb.height != iminfo.extent.height)
       return GetResID(m_Overlay.Image);
@@ -648,7 +656,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
   else if(overlay == DebugOverlay::Drawcall || overlay == DebugOverlay::Wireframe)
   {
     float highlightCol[] = {0.8f, 0.1f, 0.8f, 1.0f};
-    float clearCol[] = {0.0f, 0.0f, 0.0f, 0.5f};
+    float bgclearCol[] = {0.0f, 0.0f, 0.0f, 0.5f};
 
     if(overlay == DebugOverlay::Wireframe)
     {
@@ -656,10 +664,10 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
       highlightCol[1] = 1.0f;
       highlightCol[2] = 0.0f;
 
-      clearCol[0] = 200 / 255.0f;
-      clearCol[1] = 1.0f;
-      clearCol[2] = 0.0f;
-      clearCol[3] = 0.0f;
+      bgclearCol[0] = 200 / 255.0f;
+      bgclearCol[1] = 1.0f;
+      bgclearCol[2] = 0.0f;
+      bgclearCol[3] = 0.0f;
     }
 
     VkImageMemoryBarrier barrier = {VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER,
@@ -676,7 +684,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
     DoPipelineBarrier(cmd, 1, &barrier);
 
     vt->CmdClearColorImage(Unwrap(cmd), Unwrap(m_Overlay.Image), VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL,
-                           (VkClearColorValue *)clearCol, 1, &subresourceRange);
+                           (VkClearColorValue *)bgclearCol, 1, &subresourceRange);
 
     std::swap(barrier.oldLayout, barrier.newLayout);
     std::swap(barrier.srcAccessMask, barrier.dstAccessMask);
@@ -714,11 +722,25 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
     rs->cullMode = VK_CULL_MODE_NONE;
     rs->rasterizerDiscardEnable = false;
 
+    // disable all discard rectangles
+    RemoveNextStruct(&pipeCreateInfo,
+                     VK_STRUCTURE_TYPE_PIPELINE_DISCARD_RECTANGLE_STATE_CREATE_INFO_EXT);
+
     if(m_pDriver->GetDeviceFeatures().depthClamp)
     {
       rs->depthClampEnable = true;
     }
 
+    // disable line stipple
+    VkPipelineRasterizationLineStateCreateInfoEXT *lineRasterState =
+        (VkPipelineRasterizationLineStateCreateInfoEXT *)FindNextStruct(
+            rs, VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_LINE_STATE_CREATE_INFO_EXT);
+
+    if(lineRasterState)
+    {
+      lineRasterState->stippledLineEnable = VK_FALSE;
+    }
+
     uint32_t patchedIndexCount = 0;
     GPUBuffer patchedIB;
 
@@ -829,7 +851,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
     // modify state
     m_pDriver->m_RenderState.renderPass = GetResID(m_Overlay.NoDepthRP);
     m_pDriver->m_RenderState.subpass = 0;
-    m_pDriver->m_RenderState.framebuffer = GetResID(m_Overlay.NoDepthFB);
+    m_pDriver->m_RenderState.SetFramebuffer(GetResID(m_Overlay.NoDepthFB));
 
     m_pDriver->m_RenderState.graphics.pipeline = GetResID(pipe);
 
@@ -966,9 +988,16 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
       ubo->RectPosition = Vec2f(viewport.x, viewport.y);
       ubo->RectSize = Vec2f(viewport.width, viewport.height);
 
-      if(m_pDriver->m_ExtensionsEnabled[VkCheckExt_AMD_neg_viewport] ||
-         m_pDriver->m_ExtensionsEnabled[VkCheckExt_KHR_maintenance1])
-        ubo->RectSize.y = fabs(viewport.height);
+      if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_AMD_negative_viewport_height ||
+         m_pDriver->GetExtensions(GetRecord(m_Device)).ext_KHR_maintenance1)
+      {
+        ubo->RectSize.y = fabsf(viewport.height);
+
+        // VK_KHR_maintenance1 requires the position to be adjusted as well
+        if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_KHR_maintenance1 &&
+           viewport.height < 0.0f)
+          ubo->RectPosition.y += viewport.height;
+      }
 
       m_Overlay.m_CheckerUBO.Unmap();
 
@@ -1163,7 +1192,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
     // modify state
     m_pDriver->m_RenderState.renderPass = GetResID(m_Overlay.NoDepthRP);
     m_pDriver->m_RenderState.subpass = 0;
-    m_pDriver->m_RenderState.framebuffer = GetResID(m_Overlay.NoDepthFB);
+    m_pDriver->m_RenderState.SetFramebuffer(GetResID(m_Overlay.NoDepthFB));
 
     m_pDriver->m_RenderState.graphics.pipeline = GetResID(pipe[0]);
 
@@ -1238,7 +1267,8 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
     // make a renderpass and framebuffer for rendering to overlay color and using
     // depth buffer from the orignial render
-    if(dsIdx >= 0 && dsIdx < (int32_t)createinfo.m_Framebuffer[state.framebuffer].attachments.size())
+    if(dsIdx >= 0 &&
+       dsIdx < (int32_t)createinfo.m_Framebuffer[state.GetFramebuffer()].attachments.size())
     {
       VkAttachmentDescription attDescs[] = {
           {0, VK_FORMAT_R16G16B16A16_SFLOAT, VK_SAMPLE_COUNT_1_BIT, VK_ATTACHMENT_LOAD_OP_LOAD,
@@ -1251,7 +1281,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
            VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL},
       };
 
-      ResourceId depthView = createinfo.m_Framebuffer[state.framebuffer].attachments[dsIdx].view;
+      ResourceId depthView = state.GetFramebufferAttachments()[dsIdx];
       VulkanCreationInfo::ImageView &depthViewInfo = createinfo.m_ImageView[depthView];
 
       ResourceId depthIm = depthViewInfo.image;
@@ -1260,8 +1290,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
       attDescs[1].format = depthImageInfo.format;
       attDescs[0].samples = attDescs[1].samples = iminfo.samples;
 
-      std::vector<ImageRegionState> &depthStates =
-          m_pDriver->m_ImageLayouts[depthIm].subresourceStates;
+      rdcarray<ImageRegionState> &depthStates = m_pDriver->m_ImageLayouts[depthIm].subresourceStates;
 
       for(ImageRegionState &ds : depthStates)
       {
@@ -1461,7 +1490,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
     // modify state
     m_pDriver->m_RenderState.renderPass = GetResID(m_Overlay.NoDepthRP);
     m_pDriver->m_RenderState.subpass = 0;
-    m_pDriver->m_RenderState.framebuffer = GetResID(m_Overlay.NoDepthFB);
+    m_pDriver->m_RenderState.SetFramebuffer(GetResID(m_Overlay.NoDepthFB));
 
     m_pDriver->m_RenderState.graphics.pipeline = GetResID(failpipe);
 
@@ -1483,7 +1512,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
     if(depthRP != VK_NULL_HANDLE)
     {
       m_pDriver->m_RenderState.renderPass = GetResID(depthRP);
-      m_pDriver->m_RenderState.framebuffer = GetResID(depthFB);
+      m_pDriver->m_RenderState.SetFramebuffer(GetResID(depthFB));
     }
 
     m_pDriver->ReplayLog(0, eventId, eReplay_OnlyDraw);
@@ -1538,7 +1567,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
     DoPipelineBarrier(cmd, 1, &barrier);
 
-    vector<uint32_t> events = passEvents;
+    rdcarray<uint32_t> events = passEvents;
 
     if(overlay == DebugOverlay::ClearBeforeDraw)
       events.clear();
@@ -1589,19 +1618,47 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
       m_pDriver->m_RenderState.BeginRenderPassAndApplyState(cmd, VulkanRenderState::BindGraphics);
 
-      VkClearAttachment blackclear = {VK_IMAGE_ASPECT_COLOR_BIT, 0, {}};
-      vector<VkClearAttachment> atts;
+      VkClearAttachment clearatt = {VK_IMAGE_ASPECT_COLOR_BIT, 0, {}};
+      memcpy(clearatt.clearValue.color.float32, &clearCol.x,
+             sizeof(clearatt.clearValue.color.float32));
+      rdcarray<VkClearAttachment> atts;
 
       VulkanCreationInfo::Framebuffer &fb =
-          m_pDriver->m_CreationInfo.m_Framebuffer[m_pDriver->m_RenderState.framebuffer];
+          m_pDriver->m_CreationInfo.m_Framebuffer[m_pDriver->m_RenderState.GetFramebuffer()];
       VulkanCreationInfo::RenderPass &rp =
           m_pDriver->m_CreationInfo.m_RenderPass[m_pDriver->m_RenderState.renderPass];
 
       for(size_t i = 0; i < rp.subpasses[m_pDriver->m_RenderState.subpass].colorAttachments.size();
           i++)
       {
-        blackclear.colorAttachment = (uint32_t)i;
-        atts.push_back(blackclear);
+        clearatt.colorAttachment = (uint32_t)i;
+        atts.push_back(clearatt);
+      }
+
+      // Try to clear depth as well, to help debug shadow rendering
+      if(m_pDriver->m_RenderState.graphics.pipeline != ResourceId() &&
+         IsDepthOrStencilFormat(iminfo.format))
+      {
+        const VulkanCreationInfo::Pipeline &p =
+            m_pDriver->m_CreationInfo.m_Pipeline[m_pDriver->m_RenderState.graphics.pipeline];
+
+        // If the depth func is equal or not equal, don't clear at all since the output would be
+        // altered in an way that would cause replay to produce mostly incorrect results.
+        // Similarly, skip if the depth func is always, as we'd have a 50% chance of guessing the
+        // wrong clear value.
+        if(p.depthCompareOp != VK_COMPARE_OP_EQUAL && p.depthCompareOp != VK_COMPARE_OP_NOT_EQUAL &&
+           p.depthCompareOp != VK_COMPARE_OP_ALWAYS)
+        {
+          // If the depth func is less or less equal, clear to 1 instead of 0
+          bool depthFuncLess = p.depthCompareOp == VK_COMPARE_OP_LESS ||
+                               p.depthCompareOp == VK_COMPARE_OP_LESS_OR_EQUAL;
+          float depthClear = depthFuncLess ? 1.0f : 0.0f;
+
+          VkClearAttachment clearDepthAtt = {VK_IMAGE_ASPECT_DEPTH_BIT, 0, {}};
+          clearDepthAtt.clearValue.depthStencil.depth = depthClear;
+
+          atts.push_back(clearDepthAtt);
+        }
       }
 
       VkClearRect rect = {
@@ -1662,7 +1719,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
       DoPipelineBarrier(cmd, 1, &barrier);
 
-      vector<uint32_t> events = passEvents;
+      rdcarray<uint32_t> events = passEvents;
 
       if(overlay == DebugOverlay::QuadOverdrawDraw)
         events.clear();
@@ -1676,7 +1733,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
       {
         const DrawcallDescription *draw = m_pDriver->GetDrawcall(events[0]);
         if(draw->flags & DrawFlags::BeginPass)
-          events.erase(events.begin());
+          events.erase(0);
       }
 
       VkImage quadImg;
@@ -1854,7 +1911,8 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
       for(auto it = cb.m_PipelineCache.begin(); it != cb.m_PipelineCache.end(); ++it)
       {
-        m_pDriver->vkDestroyPipeline(m_Device, it->second.second, NULL);
+        m_pDriver->vkDestroyPipeline(m_Device, it->second.pipe, NULL);
+        m_pDriver->vkDestroyPipelineLayout(m_Device, it->second.pipeLayout, NULL);
       }
     }
 
@@ -1870,6 +1928,17 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
   {
     VulkanRenderState prevstate = m_pDriver->m_RenderState;
 
+    VkPipelineShaderStageCreateInfo stages[3] = {
+        {VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO, NULL, 0, VK_SHADER_STAGE_VERTEX_BIT,
+         shaderCache->GetBuiltinModule(BuiltinShader::MeshVS), "main", NULL},
+        {VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO, NULL, 0, VK_SHADER_STAGE_FRAGMENT_BIT,
+         shaderCache->GetBuiltinModule(BuiltinShader::TrisizeFS), "main", NULL},
+        {VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO, NULL, 0, VK_SHADER_STAGE_GEOMETRY_BIT,
+         shaderCache->GetBuiltinModule(BuiltinShader::TrisizeGS), "main", NULL},
+    };
+
+    if(stages[0].module != VK_NULL_HANDLE && stages[1].module != VK_NULL_HANDLE &&
+       stages[2].module != VK_NULL_HANDLE)
     {
       SCOPED_TIMER("Triangle Size");
 
@@ -1906,7 +1975,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
       m_pDriver->SubmitCmds();
 #endif
 
-      vector<uint32_t> events = passEvents;
+      rdcarray<uint32_t> events = passEvents;
 
       if(overlay == DebugOverlay::TriangleSizeDraw)
         events.clear();
@@ -1917,7 +1986,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
         // remove any non-drawcalls, like the pass boundary.
         if(!draw || !(draw->flags & DrawFlags::Drawcall))
-          events.erase(events.begin());
+          events.erase(0);
         else
           break;
       }
@@ -1980,7 +2049,8 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
       // make a renderpass and framebuffer for rendering to overlay color and using
       // depth buffer from the orignial render
-      if(dsIdx >= 0 && dsIdx < (int32_t)createinfo.m_Framebuffer[state.framebuffer].attachments.size())
+      if(dsIdx >= 0 &&
+         dsIdx < (int32_t)createinfo.m_Framebuffer[state.GetFramebuffer()].attachments.size())
       {
         depthUsed = true;
 
@@ -1995,7 +2065,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
              VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL},
         };
 
-        ResourceId depthView = createinfo.m_Framebuffer[state.framebuffer].attachments[dsIdx].view;
+        ResourceId depthView = state.GetFramebufferAttachments()[dsIdx];
         VulkanCreationInfo::ImageView &depthViewInfo = createinfo.m_ImageView[depthView];
 
         ResourceId depthIm = depthViewInfo.image;
@@ -2004,7 +2074,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
         attDescs[1].format = depthImageInfo.format;
         attDescs[0].samples = attDescs[1].samples = iminfo.samples;
 
-        std::vector<ImageRegionState> &depthStates =
+        rdcarray<ImageRegionState> &depthStates =
             m_pDriver->m_ImageLayouts[depthIm].subresourceStates;
 
         for(ImageRegionState &ds : depthStates)
@@ -2076,19 +2146,24 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
       m_pDriver->GetShaderCache()->MakeGraphicsPipelineInfo(pipeCreateInfo, state.graphics.pipeline);
 
-      VkPipelineShaderStageCreateInfo stages[3] = {
-          {VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO, NULL, 0, VK_SHADER_STAGE_VERTEX_BIT,
-           shaderCache->GetBuiltinModule(BuiltinShader::MeshVS), "main", NULL},
-          {VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO, NULL, 0, VK_SHADER_STAGE_FRAGMENT_BIT,
-           shaderCache->GetBuiltinModule(BuiltinShader::TrisizeFS), "main", NULL},
-          {VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO, NULL, 0, VK_SHADER_STAGE_GEOMETRY_BIT,
-           shaderCache->GetBuiltinModule(BuiltinShader::TrisizeGS), "main", NULL},
-      };
-
       VkPipelineInputAssemblyStateCreateInfo ia = {
           VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO};
+
+      // most topologies are decomposed into triangle lists on output
       ia.topology = VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST;
 
+      if(pipeCreateInfo.pInputAssemblyState->topology == VK_PRIMITIVE_TOPOLOGY_POINT_LIST)
+        ia.topology = VK_PRIMITIVE_TOPOLOGY_POINT_LIST;
+      else if(pipeCreateInfo.pInputAssemblyState->topology == VK_PRIMITIVE_TOPOLOGY_LINE_LIST ||
+              pipeCreateInfo.pInputAssemblyState->topology ==
+                  VK_PRIMITIVE_TOPOLOGY_LINE_LIST_WITH_ADJACENCY ||
+              pipeCreateInfo.pInputAssemblyState->topology == VK_PRIMITIVE_TOPOLOGY_LINE_STRIP ||
+              pipeCreateInfo.pInputAssemblyState->topology ==
+                  VK_PRIMITIVE_TOPOLOGY_LINE_STRIP_WITH_ADJACENCY)
+        ia.topology = VK_PRIMITIVE_TOPOLOGY_LINE_LIST;
+      else if(pipeCreateInfo.pInputAssemblyState->topology == VK_PRIMITIVE_TOPOLOGY_TRIANGLE_FAN)
+        ia.topology = VK_PRIMITIVE_TOPOLOGY_TRIANGLE_FAN;
+
       VkVertexInputBindingDescription binds[] = {
           // primary
           {0, 0, VK_VERTEX_INPUT_RATE_VERTEX},
@@ -2144,9 +2219,9 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
       pipeCreateInfo.pVertexInputState = &vi;
       pipeCreateInfo.pColorBlendState = &cb;
 
-      typedef std::pair<uint32_t, Topology> PipeKey;
+      typedef rdcpair<uint32_t, Topology> PipeKey;
 
-      map<PipeKey, VkPipeline> pipes;
+      std::map<PipeKey, VkPipeline> pipes;
 
       cmd = m_pDriver->GetNextCmd();
 
@@ -2186,7 +2261,7 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
 
             binds[0].stride = binds[1].stride = fmt.vertexByteStride;
 
-            PipeKey key = std::make_pair(fmt.vertexByteStride, fmt.topology);
+            PipeKey key = make_rdcpair(fmt.vertexByteStride, fmt.topology);
             VkPipeline pipe = pipes[key];
 
             if(pipe == VK_NULL_HANDLE)
@@ -2266,6 +2341,8 @@ ResourceId VulkanReplay::RenderOverlay(ResourceId texid, CompType typeHint, Debu
               VkIndexType idxtype = VK_INDEX_TYPE_UINT16;
               if(fmt.indexByteStride == 4)
                 idxtype = VK_INDEX_TYPE_UINT32;
+              else if(fmt.indexByteStride == 1)
+                idxtype = VK_INDEX_TYPE_UINT8_EXT;
 
               if(fmt.indexResourceId != ResourceId())
               {
