@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,6 +22,7 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
+#include <algorithm>
 #include "api/replay/version.h"
 #include "strings/string_utils.h"
 #include "vk_core.h"
@@ -36,21 +37,25 @@ bool VulkanReplay::IsOutputWindowVisible(uint64_t id)
   if(id == 0 || m_OutputWindows.find(id) == m_OutputWindows.end())
     return false;
 
+  if(m_OutputWindows[id].m_WindowSystem == WindowingSystem::Headless)
+    return true;
+
   VULKANNOTIMP("Optimisation missing - output window always returning true");
 
   return true;
 }
 
-void WrappedVulkan::AddRequiredExtensions(bool instance, vector<string> &extensionList,
-                                          const std::set<string> &supportedExtensions)
+void WrappedVulkan::AddRequiredExtensions(bool instance, rdcarray<rdcstr> &extensionList,
+                                          const std::set<rdcstr> &supportedExtensions)
 {
   bool device = !instance;
 
 // check if our compile-time options expect any WSI to be available, or if it's all disabled
 #define EXPECT_WSI 0
 
-#if(defined(VK_USE_PLATFORM_ANDROID_KHR) || defined(VK_USE_PLATFORM_XCB_KHR) || \
-    defined(VK_USE_PLATFORM_XLIB_KHR) || defined(VK_USE_PLATFORM_MACOS_MVK) ||  \
+#if(defined(VK_USE_PLATFORM_ANDROID_KHR) || defined(VK_USE_PLATFORM_XCB_KHR) ||  \
+    defined(VK_USE_PLATFORM_WAYLAND_KHR) || defined(VK_USE_PLATFORM_XLIB_KHR) || \
+    defined(VK_USE_PLATFORM_MACOS_MVK) || defined(VK_USE_PLATFORM_METAL_EXT) ||  \
     defined(VK_USE_PLATFORM_GGP))
 
 #undef EXPECT_WSI
@@ -61,10 +66,21 @@ void WrappedVulkan::AddRequiredExtensions(bool instance, vector<string> &extensi
   if(instance)
   {
     // don't add duplicates
-    if(std::find(extensionList.begin(), extensionList.end(), VK_KHR_SURFACE_EXTENSION_NAME) ==
-       extensionList.end())
+    if(!extensionList.contains(VK_KHR_SURFACE_EXTENSION_NAME))
       extensionList.push_back(VK_KHR_SURFACE_EXTENSION_NAME);
 
+#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
+    // check if supported
+    if(supportedExtensions.find(VK_KHR_WAYLAND_SURFACE_EXTENSION_NAME) != supportedExtensions.end())
+    {
+      m_SupportedWindowSystems.push_back(WindowingSystem::Wayland);
+
+      // don't add duplicates
+      if(!extensionList.contains(VK_KHR_WAYLAND_SURFACE_EXTENSION_NAME))
+        extensionList.push_back(VK_KHR_WAYLAND_SURFACE_EXTENSION_NAME);
+    }
+#endif
+
 #if defined(VK_USE_PLATFORM_XCB_KHR)
     // check if supported
     if(supportedExtensions.find(VK_KHR_XCB_SURFACE_EXTENSION_NAME) != supportedExtensions.end())
@@ -72,11 +88,8 @@ void WrappedVulkan::AddRequiredExtensions(bool instance, vector<string> &extensi
       m_SupportedWindowSystems.push_back(WindowingSystem::XCB);
 
       // don't add duplicates
-      if(std::find(extensionList.begin(), extensionList.end(), VK_KHR_XCB_SURFACE_EXTENSION_NAME) ==
-         extensionList.end())
-      {
+      if(!extensionList.contains(VK_KHR_XCB_SURFACE_EXTENSION_NAME))
         extensionList.push_back(VK_KHR_XCB_SURFACE_EXTENSION_NAME);
-      }
     }
 #endif
 
@@ -87,26 +100,36 @@ void WrappedVulkan::AddRequiredExtensions(bool instance, vector<string> &extensi
       m_SupportedWindowSystems.push_back(WindowingSystem::Xlib);
 
       // don't add duplicates
-      if(std::find(extensionList.begin(), extensionList.end(), VK_KHR_XLIB_SURFACE_EXTENSION_NAME) ==
-         extensionList.end())
-      {
+      if(!extensionList.contains(VK_KHR_XLIB_SURFACE_EXTENSION_NAME))
         extensionList.push_back(VK_KHR_XLIB_SURFACE_EXTENSION_NAME);
-      }
+    }
+#endif
+
+#if defined(VK_USE_PLATFORM_METAL_EXT)
+    // check if supported
+    if(supportedExtensions.find(VK_EXT_METAL_SURFACE_EXTENSION_NAME) != supportedExtensions.end())
+    {
+      m_SupportedWindowSystems.push_back(WindowingSystem::MacOS);
+
+      RDCLOG("Will create surfaces using " VK_EXT_METAL_SURFACE_EXTENSION_NAME);
+
+      // don't add duplicates, application will have added this but just be sure
+      if(!extensionList.contains(VK_EXT_METAL_SURFACE_EXTENSION_NAME))
+        extensionList.push_back(VK_EXT_METAL_SURFACE_EXTENSION_NAME);
     }
 #endif
 
 #if defined(VK_USE_PLATFORM_MACOS_MVK)
-    // must be supported
-    RDCASSERT(supportedExtensions.find(VK_MVK_MACOS_SURFACE_EXTENSION_NAME) !=
-              supportedExtensions.end());
-
-    m_SupportedWindowSystems.push_back(WindowingSystem::MacOS);
-
-    // don't add duplicates, application will have added this but just be sure
-    if(std::find(extensionList.begin(), extensionList.end(), VK_MVK_MACOS_SURFACE_EXTENSION_NAME) ==
-       extensionList.end())
+    // check if supported
+    if(supportedExtensions.find(VK_MVK_MACOS_SURFACE_EXTENSION_NAME) != supportedExtensions.end())
     {
-      extensionList.push_back(VK_MVK_MACOS_SURFACE_EXTENSION_NAME);
+      m_SupportedWindowSystems.push_back(WindowingSystem::MacOS);
+
+      RDCLOG("Will create surfaces using " VK_MVK_MACOS_SURFACE_EXTENSION_NAME);
+
+      // don't add duplicates, application will have added this but just be sure
+      if(!extensionList.contains(VK_MVK_MACOS_SURFACE_EXTENSION_NAME))
+        extensionList.push_back(VK_MVK_MACOS_SURFACE_EXTENSION_NAME);
     }
 #endif
 
@@ -118,11 +141,8 @@ void WrappedVulkan::AddRequiredExtensions(bool instance, vector<string> &extensi
     m_SupportedWindowSystems.push_back(WindowingSystem::Android);
 
     // don't add duplicates, application will have added this but just be sure
-    if(std::find(extensionList.begin(), extensionList.end(),
-                 VK_KHR_ANDROID_SURFACE_EXTENSION_NAME) == extensionList.end())
-    {
+    if(!extensionList.contains(VK_KHR_ANDROID_SURFACE_EXTENSION_NAME))
       extensionList.push_back(VK_KHR_ANDROID_SURFACE_EXTENSION_NAME);
-    }
 #endif
 
 #if defined(VK_USE_PLATFORM_GGP)
@@ -133,11 +153,8 @@ void WrappedVulkan::AddRequiredExtensions(bool instance, vector<string> &extensi
     m_SupportedWindowSystems.push_back(WindowingSystem::GGP);
 
     // don't add duplicates, application will have added this but just be sure
-    if(std::find(extensionList.begin(), extensionList.end(),
-                 VK_GGP_STREAM_DESCRIPTOR_SURFACE_EXTENSION_NAME) == extensionList.end())
-    {
+    if(!extensionList.contains(VK_GGP_STREAM_DESCRIPTOR_SURFACE_EXTENSION_NAME))
       extensionList.push_back(VK_GGP_STREAM_DESCRIPTOR_SURFACE_EXTENSION_NAME);
-    }
 #endif
 
 #if EXPECT_WSI
@@ -158,9 +175,17 @@ void WrappedVulkan::AddRequiredExtensions(bool instance, vector<string> &extensi
     {
       RDCWARN("No WSI support - only headless replay allowed.");
 
-#if defined(VK_USE_PLATFORM_MACOS_MVK)
+#if defined(VK_USE_PLATFORM_MACOS_MVK) && defined(VK_USE_PLATFORM_METAL_EXT)
+      RDCWARN("macOS Output requires the '%s' or '%s' extensions to be present",
+              VK_MVK_MACOS_SURFACE_EXTENSION_NAME, VK_EXT_METAL_SURFACE_EXTENSION_NAME);
+#elif defined(VK_USE_PLATFORM_MACOS_MVK) || defined(VK_USE_PLATFORM_METAL_EXT)
       RDCWARN("macOS Output requires the '%s' extension to be present",
-              VK_MVK_MACOS_SURFACE_EXTENSION_NAME);
+#if defined(VK_USE_PLATFORM_MACOS_MVK)
+              VK_MVK_MACOS_SURFACE_EXTENSION_NAME
+#elif defined(VK_USE_PLATFORM_METAL_EXT)
+              VK_EXT_METAL_SURFACE_EXTENSION_NAME
+#endif
+              );
 #endif
 
 #if defined(VK_USE_PLATFORM_ANDROID_KHR)
@@ -168,6 +193,11 @@ void WrappedVulkan::AddRequiredExtensions(bool instance, vector<string> &extensi
               VK_KHR_ANDROID_SURFACE_EXTENSION_NAME);
 #endif
 
+#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
+      RDCWARN("Wayland Output requires the '%s' extension to be present",
+              VK_KHR_WAYLAND_SURFACE_EXTENSION_NAME);
+#endif
+
 #if defined(VK_USE_PLATFORM_XCB_KHR)
       RDCWARN("XCB Output requires the '%s' extension to be present",
               VK_KHR_XCB_SURFACE_EXTENSION_NAME);
@@ -208,8 +238,8 @@ extern unsigned char driver_vulkan_renderdoc_json[];
 extern int driver_vulkan_renderdoc_json_len;
 
 #if ENABLED(RDOC_ANDROID)
-bool VulkanReplay::CheckVulkanLayer(VulkanLayerFlags &flags, std::vector<std::string> &myJSONs,
-                                    std::vector<std::string> &otherJSONs)
+bool VulkanReplay::CheckVulkanLayer(VulkanLayerFlags &flags, rdcarray<rdcstr> &myJSONs,
+                                    rdcarray<rdcstr> &otherJSONs)
 {
   return false;
 }
@@ -218,23 +248,23 @@ void VulkanReplay::InstallVulkanLayer(bool systemLevel)
 {
 }
 #else
-static std::string GenerateJSON(const std::string &sopath)
+static rdcstr GenerateJSON(const rdcstr &sopath)
 {
   char *txt = (char *)driver_vulkan_renderdoc_json;
   int len = driver_vulkan_renderdoc_json_len;
 
-  string json = string(txt, txt + len);
+  rdcstr json = rdcstr(txt, len);
 
-  const char dllPathString[] = ".\\\\renderdoc.dll";
+  const char modulePathString[] = "@VULKAN_LAYER_MODULE_PATH@";
 
-  size_t idx = json.find(dllPathString);
+  int32_t idx = json.find(modulePathString);
 
-  json = json.substr(0, idx) + sopath + json.substr(idx + sizeof(dllPathString) - 1);
+  json = json.substr(0, idx) + sopath + json.substr(idx + sizeof(modulePathString) - 1);
 
-  const char majorString[] = "[MAJOR]";
+  const char majorString[] = "@RENDERDOC_VERSION_MAJOR@";
 
   idx = json.find(majorString);
-  while(idx != string::npos)
+  while(idx >= 0)
   {
     json = json.substr(0, idx) + STRINGIZE(RENDERDOC_VERSION_MAJOR) +
            json.substr(idx + sizeof(majorString) - 1);
@@ -242,10 +272,10 @@ static std::string GenerateJSON(const std::string &sopath)
     idx = json.find(majorString);
   }
 
-  const char minorString[] = "[MINOR]";
+  const char minorString[] = "@RENDERDOC_VERSION_MINOR@";
 
   idx = json.find(minorString);
-  while(idx != string::npos)
+  while(idx >= 0)
   {
     json = json.substr(0, idx) + STRINGIZE(RENDERDOC_VERSION_MINOR) +
            json.substr(idx + sizeof(minorString) - 1);
@@ -253,15 +283,26 @@ static std::string GenerateJSON(const std::string &sopath)
     idx = json.find(minorString);
   }
 
+  const char enableVarString[] = "@VULKAN_ENABLE_VAR@";
+
+  idx = json.find(enableVarString);
+  while(idx >= 0)
+  {
+    json = json.substr(0, idx) + STRINGIZE(ENABLE_VULKAN_RENDERDOC_CAPTURE) +
+           json.substr(idx + sizeof(enableVarString) - 1);
+
+    idx = json.find(enableVarString);
+  }
+
   return json;
 }
 
-static bool FileExists(const std::string &path)
+static bool FileExists(const rdcstr &path)
 {
   return access(path.c_str(), F_OK) == 0;
 }
 
-static std::string GetSOFromJSON(const std::string &json)
+static rdcstr GetSOFromJSON(const rdcstr &json)
 {
   char *json_string = new char[1024];
   memset(json_string, 0, 1024);
@@ -275,7 +316,7 @@ static std::string GetSOFromJSON(const std::string &json)
     fclose(f);
   }
 
-  string ret = "";
+  rdcstr ret = "";
 
   // The line is:
   // "library_path": "/foo/bar/librenderdoc.so",
@@ -310,7 +351,7 @@ enum class LayerPath : int
 
 ITERABLE_OPERATORS(LayerPath);
 
-string LayerRegistrationPath(LayerPath path)
+rdcstr LayerRegistrationPath(LayerPath path)
 {
   switch(path)
   {
@@ -320,10 +361,10 @@ string LayerRegistrationPath(LayerPath path)
     {
       const char *xdg = getenv("XDG_DATA_HOME");
       if(xdg && FileIO::exists(xdg))
-        return string(xdg) + "/vulkan/implicit_layer.d/renderdoc_capture.json";
+        return rdcstr(xdg) + "/vulkan/implicit_layer.d/renderdoc_capture.json";
 
       const char *home_path = getenv("HOME");
-      return string(home_path != NULL ? home_path : "") +
+      return rdcstr(home_path != NULL ? home_path : "") +
              "/.local/share/vulkan/implicit_layer.d/renderdoc_capture.json";
     }
     default: break;
@@ -332,9 +373,9 @@ string LayerRegistrationPath(LayerPath path)
   return "";
 }
 
-void MakeParentDirs(std::string file)
+void MakeParentDirs(rdcstr file)
 {
-  std::string dir = get_dirname(file);
+  rdcstr dir = get_dirname(file);
 
   if(dir == "/" || dir.empty())
     return;
@@ -347,15 +388,15 @@ void MakeParentDirs(std::string file)
   mkdir(dir.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
 }
 
-bool VulkanReplay::CheckVulkanLayer(VulkanLayerFlags &flags, std::vector<std::string> &myJSONs,
-                                    std::vector<std::string> &otherJSONs)
+bool VulkanReplay::CheckVulkanLayer(VulkanLayerFlags &flags, rdcarray<rdcstr> &myJSONs,
+                                    rdcarray<rdcstr> &otherJSONs)
 {
   // see if the user has suppressed all this checking as a "I know what I'm doing" measure
 
   const char *home_path = getenv("HOME");
   if(home_path == NULL)
     home_path = "";
-  if(FileExists(string(home_path) + "/.renderdoc/ignore_vulkan_layer_issues"))
+  if(FileExists(rdcstr(home_path) + "/.renderdoc/ignore_vulkan_layer_issues"))
   {
     flags = VulkanLayerFlags::ThisInstallRegistered;
     return false;
@@ -365,7 +406,7 @@ bool VulkanReplay::CheckVulkanLayer(VulkanLayerFlags &flags, std::vector<std::st
   // check that there's only one layer registered, and it points to the same .so file that
   // we are running with in this instance of renderdoccmd
 
-  string librenderdoc_path;
+  rdcstr librenderdoc_path;
   FileIO::GetLibraryFilename(librenderdoc_path);
 
   if(librenderdoc_path.empty() || !FileExists(librenderdoc_path))
@@ -406,7 +447,7 @@ bool VulkanReplay::CheckVulkanLayer(VulkanLayerFlags &flags, std::vector<std::st
       numMatch++;
   }
 
-  flags = VulkanLayerFlags::CouldElevate | VulkanLayerFlags::UpdateAllowed;
+  flags = VulkanLayerFlags::UserRegisterable | VulkanLayerFlags::UpdateAllowed;
 
   if(numMatch >= 1)
     flags |= VulkanLayerFlags::ThisInstallRegistered;
@@ -415,11 +456,23 @@ bool VulkanReplay::CheckVulkanLayer(VulkanLayerFlags &flags, std::vector<std::st
   if(numExist == 1 && numMatch == 1)
     return false;
 
+  if(numMatch == 1 && exist[(int)LayerPath::etc] && match[(int)LayerPath::etc])
+  {
+    // if only /etc is registered matching us, keep things simple and don't allow unregistering it
+    // and registering the /home. Just unregister the /home that doesn't match
+    flags &= ~(VulkanLayerFlags::UserRegisterable | VulkanLayerFlags::UpdateAllowed);
+  }
+
   if(exist[(int)LayerPath::usr] && !match[(int)LayerPath::usr])
     otherJSONs.push_back(LayerRegistrationPath(LayerPath::usr));
 
   if(exist[(int)LayerPath::etc] && !match[(int)LayerPath::etc])
+  {
+    // if the /etc manifest doesn't match we need to elevate to fix it regardless of whether we
+    // delete it in favour of a /home manifest, or if we update it.
+    flags |= VulkanLayerFlags::NeedElevation;
     otherJSONs.push_back(LayerRegistrationPath(LayerPath::etc));
+  }
 
   if(exist[(int)LayerPath::home] && !match[(int)LayerPath::home])
     otherJSONs.push_back(LayerRegistrationPath(LayerPath::home));
@@ -429,12 +482,40 @@ bool VulkanReplay::CheckVulkanLayer(VulkanLayerFlags &flags, std::vector<std::st
 
   if(exist[(int)LayerPath::usr] && match[(int)LayerPath::usr])
   {
-    // just need to unregister others
+    // just need to unregister others, but we can't user-local register anymore (as that would
+    // require removing the one in /usr which we can't do)
+    flags &= ~VulkanLayerFlags::UserRegisterable;
+
+    // any other manifests that exist, even if they match, are considered others.
+    if(exist[(int)LayerPath::home])
+    {
+      otherJSONs.push_back(LayerRegistrationPath(LayerPath::home));
+      flags |= VulkanLayerFlags::OtherInstallsRegistered;
+    }
+
+    // any other manifests that exist, even if they match, are considered others.
+    if(exist[(int)LayerPath::etc])
+    {
+      otherJSONs.push_back(LayerRegistrationPath(LayerPath::etc));
+      flags |= VulkanLayerFlags::OtherInstallsRegistered | VulkanLayerFlags::NeedElevation;
+    }
   }
   else
   {
-    myJSONs.push_back(LayerRegistrationPath(LayerPath::etc));
-    myJSONs.push_back(LayerRegistrationPath(LayerPath::home));
+    // if we have multiple matches but they are all correct, and there are no other JSONs we just
+    // report that home needs to be unregistered.
+    if(otherJSONs.empty() && exist[(int)LayerPath::etc] && match[(int)LayerPath::etc])
+    {
+      flags &= ~(VulkanLayerFlags::UserRegisterable | VulkanLayerFlags::UpdateAllowed);
+      flags |= VulkanLayerFlags::OtherInstallsRegistered;
+      myJSONs.push_back(LayerRegistrationPath(LayerPath::etc));
+      otherJSONs.push_back(LayerRegistrationPath(LayerPath::home));
+    }
+    else
+    {
+      myJSONs.push_back(LayerRegistrationPath(LayerPath::etc));
+      myJSONs.push_back(LayerRegistrationPath(LayerPath::home));
+    }
   }
 
   if(exist[(int)LayerPath::usr] && !match[(int)LayerPath::usr])
@@ -449,7 +530,39 @@ bool VulkanReplay::CheckVulkanLayer(VulkanLayerFlags &flags, std::vector<std::st
 
 void VulkanReplay::InstallVulkanLayer(bool systemLevel)
 {
-  std::string homePath = LayerRegistrationPath(LayerPath::home);
+  rdcstr usrPath = LayerRegistrationPath(LayerPath::usr);
+  rdcstr homePath = LayerRegistrationPath(LayerPath::home);
+  rdcstr etcPath = LayerRegistrationPath(LayerPath::etc);
+
+  if(FileExists(usrPath))
+  {
+    // if the usr path exists, all we can do is try to remove etc & home. This assumes a
+    // system-level install
+    if(!systemLevel)
+    {
+      RDCERR("Can't register user-local with manifest under /usr");
+      return;
+    }
+
+    if(FileExists(homePath))
+    {
+      if(unlink(homePath.c_str()) < 0)
+      {
+        const char *const errtext = strerror(errno);
+        RDCERR("Error removing %s: %s", homePath.c_str(), errtext);
+      }
+    }
+    if(FileExists(etcPath))
+    {
+      if(unlink(etcPath.c_str()) < 0)
+      {
+        const char *const errtext = strerror(errno);
+        RDCERR("Error removing %s: %s", etcPath.c_str(), errtext);
+      }
+    }
+
+    return;
+  }
 
   // if we want to install to the system and there's a registration in $HOME, delete it
   if(systemLevel && FileExists(homePath))
@@ -461,8 +574,6 @@ void VulkanReplay::InstallVulkanLayer(bool systemLevel)
     }
   }
 
-  std::string etcPath = LayerRegistrationPath(LayerPath::etc);
-
   // and vice-versa
   if(!systemLevel && FileExists(etcPath))
   {
@@ -475,9 +586,9 @@ void VulkanReplay::InstallVulkanLayer(bool systemLevel)
 
   LayerPath idx = systemLevel ? LayerPath::etc : LayerPath::home;
 
-  string jsonPath = LayerRegistrationPath(idx);
-  string path = GetSOFromJSON(jsonPath);
-  string libPath;
+  rdcstr jsonPath = LayerRegistrationPath(idx);
+  rdcstr path = GetSOFromJSON(jsonPath);
+  rdcstr libPath;
   FileIO::GetLibraryFilename(libPath);
 
   if(path != libPath)
@@ -499,4 +610,4 @@ void VulkanReplay::InstallVulkanLayer(bool systemLevel)
     }
   }
 }
-#endif
\ No newline at end of file
+#endif
