@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,6 +28,7 @@
 #include "maths/matrix.h"
 #include "vk_core.h"
 #include "vk_debug.h"
+#include "vk_replay.h"
 #include "vk_shader_cache.h"
 
 #define VULKAN 1
@@ -83,6 +84,10 @@ MeshDisplayPipelines VulkanDebugManager::CacheMeshDisplayPipelines(VkPipelineLay
     key |= 1ULL << bit;
   bit++;
 
+  if(primary.allowRestart)
+    key |= 1ULL << bit;
+  bit++;
+
   // only 64 bits, make sure they all fit
   RDCASSERT(bit < 64);
 
@@ -91,7 +96,7 @@ MeshDisplayPipelines VulkanDebugManager::CacheMeshDisplayPipelines(VkPipelineLay
   if(cache.pipes[(uint32_t)SolidShade::NoSolid] != VK_NULL_HANDLE)
     return cache;
 
-  const VkLayerDispatchTable *vt = ObjDisp(m_Device);
+  const VkDevDispatchTable *vt = ObjDisp(m_Device);
   VkResult vkr = VK_SUCCESS;
 
   // should we try and evict old pipelines from the cache here?
@@ -138,10 +143,7 @@ MeshDisplayPipelines VulkanDebugManager::CacheMeshDisplayPipelines(VkPipelineLay
       false,
   };
 
-  if(IsStrip(primary.topology))
-  {
-    ia.primitiveRestartEnable = true;
-  }
+  ia.primitiveRestartEnable = primary.allowRestart && SupportsRestart(primary.topology);
 
   VkRect2D scissor = {{0, 0}, {16384, 16384}};
 
@@ -294,6 +296,20 @@ MeshDisplayPipelines VulkanDebugManager::CacheMeshDisplayPipelines(VkPipelineLay
   rs.lineWidth = 1.0f;
   ds.depthTestEnable = false;
 
+  // to be friendlier to implementations that don't support LINE polygon mode, when the topology is
+  // already lines, we can just use fill. This is most commonly used for the helpers
+  if(primary.topology == Topology::LineList)
+    rs.polygonMode = VK_POLYGON_MODE_FILL;
+
+  // if the device doesn't support non-solid fill mode, fall back to fill. We don't try to patch
+  // index buffers for mesh render since it's not worth the trouble - mesh rendering happens locally
+  // and typically the local machine is capable enough to do line raster.
+  if(!m_pDriver->GetDeviceFeatures().fillModeNonSolid)
+  {
+    RDCWARN("Can't render mesh wireframes without non-solid fill mode support");
+    rs.polygonMode = VK_POLYGON_MODE_FILL;
+  }
+
   vkr = vt->CreateGraphicsPipelines(Unwrap(m_Device), VK_NULL_HANDLE, 1, &pipeInfo, NULL,
                                     &cache.pipes[MeshDisplayPipelines::ePipe_Wire]);
   RDCASSERTEQUAL(vkr, VK_SUCCESS);
@@ -340,7 +356,7 @@ MeshDisplayPipelines VulkanDebugManager::CacheMeshDisplayPipelines(VkPipelineLay
   stages[2].stage = VK_SHADER_STAGE_GEOMETRY_BIT;
   pipeInfo.stageCount = 3;
 
-  if(stages[2].module != VK_NULL_HANDLE)
+  if(stages[2].module != VK_NULL_HANDLE && ia.topology != VK_PRIMITIVE_TOPOLOGY_POINT_LIST)
   {
     vkr = vt->CreateGraphicsPipelines(Unwrap(m_Device), VK_NULL_HANDLE, 1, &pipeInfo, NULL,
                                       &cache.pipes[MeshDisplayPipelines::ePipe_Lit]);
@@ -356,7 +372,7 @@ MeshDisplayPipelines VulkanDebugManager::CacheMeshDisplayPipelines(VkPipelineLay
   return cache;
 }
 
-void VulkanReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &secondaryDraws,
+void VulkanReplay::RenderMesh(uint32_t eventId, const rdcarray<MeshFormat> &secondaryDraws,
                               const MeshDisplay &cfg)
 {
   if(cfg.position.vertexResourceId == ResourceId() || cfg.position.numIndices == 0)
@@ -370,12 +386,12 @@ void VulkanReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &second
 
   // if the swapchain failed to create, do nothing. We will try to recreate it
   // again in CheckResizeOutputWindow (once per render 'frame')
-  if(outw.swap == VK_NULL_HANDLE)
+  if(outw.m_WindowSystem != WindowingSystem::Headless && outw.swap == VK_NULL_HANDLE)
     return;
 
   VkDevice dev = m_pDriver->GetDev();
   VkCommandBuffer cmd = m_pDriver->GetNextCmd();
-  const VkLayerDispatchTable *vt = ObjDisp(dev);
+  const VkDevDispatchTable *vt = ObjDisp(dev);
 
   VkResult vkr = VK_SUCCESS;
 
@@ -501,6 +517,8 @@ void VulkanReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &second
           VkIndexType idxtype = VK_INDEX_TYPE_UINT16;
           if(fmt.indexByteStride == 4)
             idxtype = VK_INDEX_TYPE_UINT32;
+          else if(fmt.indexByteStride == 1)
+            idxtype = VK_INDEX_TYPE_UINT8_EXT;
 
           if(fmt.indexResourceId != ResourceId())
           {
@@ -550,7 +568,7 @@ void VulkanReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &second
 
     // we source all data from the first instanced value in the instanced case, so make sure we
     // offset correctly here.
-    if(cfg.position.instanced)
+    if(cfg.position.instanced && cfg.position.instStepRate)
       offs += cfg.position.vertexByteStride * (cfg.curInstance / cfg.position.instStepRate);
 
     vt->CmdBindVertexBuffers(Unwrap(cmd), 0, 1, UnwrapPtr(vb), &offs);
@@ -571,7 +589,7 @@ void VulkanReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &second
 
     // we source all data from the first instanced value in the instanced case, so make sure we
     // offset correctly here.
-    if(cfg.second.instanced)
+    if(cfg.second.instanced && cfg.second.instStepRate)
       offs += cfg.second.vertexByteStride * (cfg.curInstance / cfg.second.instStepRate);
 
     vt->CmdBindVertexBuffers(Unwrap(cmd), 1, 1, UnwrapPtr(vb), &offs);
@@ -585,7 +603,12 @@ void VulkanReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &second
     {
       default:
       case SolidShade::Solid: pipe = cache.pipes[MeshDisplayPipelines::ePipe_SolidDepth]; break;
-      case SolidShade::Lit: pipe = cache.pipes[MeshDisplayPipelines::ePipe_Lit]; break;
+      case SolidShade::Lit:
+        pipe = cache.pipes[MeshDisplayPipelines::ePipe_Lit];
+        // point list topologies don't have lighting obvious, just render them as solid
+        if(pipe == VK_NULL_HANDLE)
+          pipe = cache.pipes[MeshDisplayPipelines::ePipe_SolidDepth];
+        break;
       case SolidShade::Secondary: pipe = cache.pipes[MeshDisplayPipelines::ePipe_Secondary]; break;
     }
 
@@ -622,6 +645,8 @@ void VulkanReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &second
       VkIndexType idxtype = VK_INDEX_TYPE_UINT16;
       if(cfg.position.indexByteStride == 4)
         idxtype = VK_INDEX_TYPE_UINT32;
+      else if(cfg.position.indexByteStride == 1)
+        idxtype = VK_INDEX_TYPE_UINT8_EXT;
 
       if(cfg.position.indexResourceId != ResourceId())
       {
@@ -669,6 +694,8 @@ void VulkanReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &second
       VkIndexType idxtype = VK_INDEX_TYPE_UINT16;
       if(cfg.position.indexByteStride == 4)
         idxtype = VK_INDEX_TYPE_UINT32;
+      else if(cfg.position.indexByteStride == 1)
+        idxtype = VK_INDEX_TYPE_UINT8_EXT;
 
       if(cfg.position.indexResourceId != ResourceId())
       {
@@ -881,16 +908,16 @@ void VulkanReplay::RenderMesh(uint32_t eventId, const vector<MeshFormat> &second
     FloatVector activeVertex;
 
     // primitive this vert is a part of (red prim, optional)
-    vector<FloatVector> activePrim;
+    rdcarray<FloatVector> activePrim;
 
     // for patch lists, to show other verts in patch (green dots, optional)
     // for non-patch lists, we use the activePrim and adjacentPrimVertices
     // to show what other verts are related
-    vector<FloatVector> inactiveVertices;
+    rdcarray<FloatVector> inactiveVertices;
 
     // adjacency (line or tri, strips or lists) (green prims, optional)
     // will be N*M long, N adjacent prims of M verts each. M = primSize below
-    vector<FloatVector> adjacentPrimVertices;
+    rdcarray<FloatVector> adjacentPrimVertices;
 
     helper.topology = Topology::TriangleList;
     uint32_t primSize = 3;    // number of verts per primitive
