@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,7 @@
 #include "vk_rendertext.h"
 #include "3rdparty/stb/stb_truetype.h"
 #include "maths/matrix.h"
+#include "strings/string_utils.h"
 #include "vk_shader_cache.h"
 
 #define VULKAN 1
@@ -222,10 +223,10 @@ VulkanTextRenderer::VulkanTextRenderer(WrappedVulkan *driver)
                                         VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT};
 
   VkDescriptorSetLayoutBinding layoutBinding[] = {
-      {0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_ALL, NULL},
-      {1, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 1, VK_SHADER_STAGE_ALL, NULL},
-      {2, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_ALL, NULL},
-      {3, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, 1, VK_SHADER_STAGE_ALL, NULL},
+      {0, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_VERTEX_BIT, NULL},
+      {1, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER, 1, VK_SHADER_STAGE_VERTEX_BIT, NULL},
+      {2, VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC, 1, VK_SHADER_STAGE_VERTEX_BIT, NULL},
+      {3, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, 1, VK_SHADER_STAGE_FRAGMENT_BIT, NULL},
   };
 
   VkDescriptorSetLayoutCreateInfo descsetLayoutInfo = {
@@ -263,7 +264,7 @@ VulkanTextRenderer::VulkanTextRenderer(WrappedVulkan *driver)
 
   // we only use a subset of the [MAX_SINGLE_LINE_LENGTH] array needed for each line, so this ring
   // can be smaller
-  m_TextStringUBO.Create(driver, dev, 4096, 10, 0);
+  m_TextStringUBO.Create(driver, dev, 4096, 20, 0);
   RDCCOMPILE_ASSERT(sizeof(StringUBOData) <= 4096, "font uniforms size");
 
   rm->SetInternalResource(GetResID(m_TextStringUBO.buf));
@@ -317,7 +318,7 @@ VulkanTextRenderer::VulkanTextRenderer(WrappedVulkan *driver)
         VK_IMAGE_USAGE_SAMPLED_BIT | VK_IMAGE_USAGE_TRANSFER_DST_BIT,
     };
 
-    std::string font = GetEmbeddedResource(sourcecodepro_ttf);
+    rdcstr font = GetEmbeddedResource(sourcecodepro_ttf);
     byte *ttfdata = (byte *)font.c_str();
 
     const int firstChar = FONT_FIRST_CHAR;
@@ -597,32 +598,22 @@ void VulkanTextRenderer::BeginText(const TextPrintState &textstate)
 }
 
 void VulkanTextRenderer::RenderText(const TextPrintState &textstate, float x, float y,
-                                    const char *textfmt, ...)
+                                    const rdcstr &text)
 {
-  static char tmpBuf[4096];
+  rdcarray<rdcstr> lines;
+  split(text, lines, '\n');
 
-  va_list args;
-  va_start(args, textfmt);
-  StringFormat::vsnprintf(tmpBuf, 4095, textfmt, args);
-  tmpBuf[4095] = '\0';
-  va_end(args);
-
-  RenderTextInternal(textstate, x, y, tmpBuf);
+  for(const rdcstr &line : lines)
+  {
+    RenderTextInternal(textstate, x, y, line);
+    y += 1.0f;
+  }
 }
 
 void VulkanTextRenderer::RenderTextInternal(const TextPrintState &textstate, float x, float y,
-                                            const char *text)
+                                            const rdcstr &text)
 {
-  if(char *t = strchr((char *)text, '\n'))
-  {
-    *t = 0;
-    RenderTextInternal(textstate, x, y, text);
-    RenderTextInternal(textstate, x, y + 1.0f, t + 1);
-    *t = '\n';
-    return;
-  }
-
-  if(strlen(text) == 0)
+  if(text.empty())
     return;
 
   uint32_t offsets[2] = {0};
@@ -643,14 +634,14 @@ void VulkanTextRenderer::RenderTextInternal(const TextPrintState &textstate, flo
 
   m_TextGeneralUBO.Unmap();
 
-  size_t len = strlen(text);
+  size_t len = text.size();
 
   RDCASSERT(len <= MAX_SINGLE_LINE_LENGTH);
 
   // only map enough for our string
   StringUBOData *stringData = (StringUBOData *)m_TextStringUBO.Map(&offsets[1], len * sizeof(Vec4u));
 
-  for(size_t i = 0; i < strlen(text); i++)
+  for(size_t i = 0; i < len; i++)
     stringData->chars[i].x = uint32_t(text[i] - ' ');
 
   m_TextStringUBO.Unmap();
@@ -659,10 +650,10 @@ void VulkanTextRenderer::RenderTextInternal(const TextPrintState &textstate, flo
       ->CmdBindDescriptorSets(Unwrap(textstate.cmd), VK_PIPELINE_BIND_POINT_GRAPHICS,
                               Unwrap(m_TextPipeLayout), 0, 1, UnwrapPtr(m_TextDescSet), 2, offsets);
 
-  ObjDisp(textstate.cmd)->CmdDraw(Unwrap(textstate.cmd), 6 * (uint32_t)strlen(text), 1, 0, 0);
+  ObjDisp(textstate.cmd)->CmdDraw(Unwrap(textstate.cmd), 6 * (uint32_t)len, 1, 0, 0);
 }
 
 void VulkanTextRenderer::EndText(const TextPrintState &textstate)
 {
   ObjDisp(textstate.cmd)->CmdEndRenderPass(Unwrap(textstate.cmd));
-}
\ No newline at end of file
+}
