@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,21 +22,21 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "maths/camera.h"
 #include "maths/formatpacking.h"
 #include "maths/matrix.h"
 #include "vk_core.h"
 #include "vk_debug.h"
+#include "vk_replay.h"
 
 #define VULKAN 1
 #include "data/glsl/glsl_ubos_cpp.h"
 
 void VulkanReplay::CreateTexImageView(VkImage liveIm, const VulkanCreationInfo::Image &iminfo,
-                                      CompType typeHint, TextureDisplayViews &views)
+                                      CompType typeCast, TextureDisplayViews &views)
 {
   VkDevice dev = m_pDriver->GetDev();
 
-  if(views.typeHint != typeHint)
+  if(views.typeCast != typeCast)
   {
     // if the type hint has changed, recreate the image views
     for(size_t i = 0; i < ARRAY_COUNT(views.views); i++)
@@ -46,9 +46,9 @@ void VulkanReplay::CreateTexImageView(VkImage liveIm, const VulkanCreationInfo::
     }
   }
 
-  views.typeHint = typeHint;
+  views.typeCast = typeCast;
 
-  VkFormat fmt = views.castedFormat = GetViewCastedFormat(iminfo.format, typeHint);
+  VkFormat fmt = views.castedFormat = GetViewCastedFormat(iminfo.format, typeCast);
 
   // all types have at least views[0] populated, so if it's still there, we can just return
   if(views.views[0] != VK_NULL_HANDLE)
@@ -129,7 +129,7 @@ bool VulkanReplay::RenderTexture(TextureDisplay cfg)
 
   // if the swapchain failed to create, do nothing. We will try to recreate it
   // again in CheckResizeOutputWindow (once per render 'frame')
-  if(outw.swap == VK_NULL_HANDLE)
+  if(outw.m_WindowSystem != WindowingSystem::Headless && outw.swap == VK_NULL_HANDLE)
     return false;
 
   VkRenderPassBeginInfo rpbegin = {
@@ -152,20 +152,23 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
 {
   const bool blendAlpha = (flags & eTexDisplay_BlendAlpha) != 0;
   const bool mipShift = (flags & eTexDisplay_MipShift) != 0;
-  const bool f16render = (flags & eTexDisplay_F16Render) != 0;
+  const bool f16render = (flags & eTexDisplay_16Render) != 0;
   const bool greenonly = (flags & eTexDisplay_GreenOnly) != 0;
-  const bool f32render = (flags & eTexDisplay_F32Render) != 0;
+  const bool f32render = (flags & eTexDisplay_32Render) != 0;
 
   VkDevice dev = m_pDriver->GetDev();
-  VkCommandBuffer cmd = m_pDriver->GetNextCmd();
-  const VkLayerDispatchTable *vt = ObjDisp(dev);
+  const VkDevDispatchTable *vt = ObjDisp(dev);
 
   ImageLayouts &layouts = m_pDriver->m_ImageLayouts[cfg.resourceId];
   VulkanCreationInfo::Image &iminfo = m_pDriver->m_CreationInfo.m_Image[cfg.resourceId];
   TextureDisplayViews &texviews = m_TexRender.TextureViews[cfg.resourceId];
   VkImage liveIm = m_pDriver->GetResourceManager()->GetCurrentHandle<VkImage>(cfg.resourceId);
+  const ImageInfo &imageInfo = layouts.imageInfo;
 
-  CreateTexImageView(liveIm, iminfo, cfg.typeHint, texviews);
+  if(!layouts.isMemoryBound)
+    return false;
+
+  CreateTexImageView(liveIm, iminfo, cfg.typeCast, texviews);
 
   int displayformat = 0;
   uint32_t descSetBinding = 0;
@@ -189,14 +192,14 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
   int viewIndex = 0;
 
   // if we're displaying the stencil, set up for stencil display
-  if(layouts.format == VK_FORMAT_S8_UINT ||
-     (IsStencilFormat(layouts.format) && !cfg.red && cfg.green))
+  if(imageInfo.format == VK_FORMAT_S8_UINT ||
+     (IsStencilFormat(imageInfo.format) && !cfg.red && cfg.green))
   {
     descSetBinding = 10;
     displayformat |= TEXDISPLAY_UINT_TEX;
 
     // for stencil we use view 1 as long as it's a depth-stencil texture
-    if(IsDepthAndStencilFormat(layouts.format))
+    if(IsDepthAndStencilFormat(imageInfo.format))
       viewIndex = 1;
 
     // rescale the range so that stencil seems to fit to 0-1
@@ -271,34 +274,37 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
 
   data->FlipY = cfg.flipY ? 1 : 0;
 
-  data->MipLevel = (int)cfg.mip;
+  data->MipLevel = (int)cfg.subresource.mip;
   data->Slice = 0;
   if(iminfo.type != VK_IMAGE_TYPE_3D)
   {
-    data->Slice = (float)cfg.sliceFace + 0.001f;
+    uint32_t numSlices = RDCMAX((uint32_t)iminfo.arrayLayers, 1U);
+
+    uint32_t sliceFace = RDCCLAMP(cfg.subresource.slice, 0U, numSlices - 1);
+    data->Slice = (float)sliceFace + 0.001f;
   }
   else
   {
-    uint32_t sliceFace = RDCCLAMP(cfg.sliceFace, 0U, uint32_t(iminfo.samples - 1));
-    data->Slice = (float)(sliceFace >> cfg.mip);
+    uint32_t sliceFace = RDCCLAMP(cfg.subresource.slice, 0U, iminfo.extent.depth - 1);
+    data->Slice = (float)sliceFace + 0.001f;
   }
 
-  data->TextureResolutionPS.x = float(RDCMAX(1, tex_x >> cfg.mip));
-  data->TextureResolutionPS.y = float(RDCMAX(1, tex_y >> cfg.mip));
-  data->TextureResolutionPS.z = float(RDCMAX(1, tex_z >> cfg.mip));
+  data->TextureResolutionPS.x = float(RDCMAX(1, tex_x >> cfg.subresource.mip));
+  data->TextureResolutionPS.y = float(RDCMAX(1, tex_y >> cfg.subresource.mip));
+  data->TextureResolutionPS.z = float(RDCMAX(1, tex_z >> cfg.subresource.mip));
 
   if(mipShift)
-    data->MipShift = float(1 << cfg.mip);
+    data->MipShift = float(1 << cfg.subresource.mip);
   else
     data->MipShift = 1.0f;
 
   data->Scale = cfg.scale;
 
-  int sampleIdx = (int)RDCCLAMP(cfg.sampleIdx, 0U, (uint32_t)SampleCount(iminfo.samples));
+  int sampleIdx = (int)RDCCLAMP(cfg.subresource.sample, 0U, (uint32_t)SampleCount(iminfo.samples));
 
-  sampleIdx = cfg.sampleIdx;
+  sampleIdx = cfg.subresource.sample;
 
-  if(cfg.sampleIdx == ~0U)
+  if(cfg.subresource.sample == ~0U)
     sampleIdx = -SampleCount(iminfo.samples);
 
   data->SampleIdx = sampleIdx;
@@ -361,8 +367,8 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
     customData->texDim.y = iminfo.extent.height;
     customData->texDim.z = iminfo.extent.depth;
     customData->texDim.w = iminfo.mipLevels;
-    customData->selectedMip = cfg.mip;
-    customData->selectedSliceFace = cfg.sliceFace;
+    customData->selectedMip = cfg.subresource.mip;
+    customData->selectedSliceFace = cfg.subresource.slice;
     customData->selectedSample = sampleIdx;
     customData->texType = (uint32_t)textype;
     customData->YUVDownsampleRate = YUVDownsampleRate;
@@ -405,7 +411,7 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
   imdesc.imageLayout = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
   imdesc.imageView = Unwrap(liveImView);
   imdesc.sampler = Unwrap(m_General.PointSampler);
-  if(cfg.mip == 0 && cfg.scale < 1.0f)
+  if(cfg.subresource.mip == 0 && cfg.scale < 1.0f)
     imdesc.sampler = Unwrap(m_TexRender.LinearSampler);
 
   VkDescriptorImageInfo altimdesc[2] = {};
@@ -415,7 +421,7 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
     altimdesc[i - 1].imageLayout = VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
     altimdesc[i - 1].imageView = Unwrap(texviews.views[i]);
     altimdesc[i - 1].sampler = Unwrap(m_General.PointSampler);
-    if(cfg.mip == 0 && cfg.scale < 1.0f)
+    if(cfg.subresource.mip == 0 && cfg.scale < 1.0f)
       altimdesc[i - 1].sampler = Unwrap(m_TexRender.LinearSampler);
   }
 
@@ -440,7 +446,7 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
        VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC, NULL, &heatubodesc, NULL},
   };
 
-  vector<VkWriteDescriptorSet> writeSets;
+  rdcarray<VkWriteDescriptorSet> writeSets;
   for(size_t i = 0; i < ARRAY_COUNT(writeSet); i++)
   {
     if(writeSet[i].descriptorCount > 0)
@@ -470,61 +476,18 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
 
   vt->UpdateDescriptorSets(Unwrap(dev), (uint32_t)writeSets.size(), &writeSets[0], 0, NULL);
 
-  VkImageMemoryBarrier srcimBarrier = {
-      VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER,
-      NULL,
-      0,
-      0,
-      VK_IMAGE_LAYOUT_UNDEFINED,
-      VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL,
-      layouts.queueFamilyIndex,
-      m_pDriver->GetQueueFamilyIndex(),
-      Unwrap(liveIm),
-      {0, 0, 1, 0, 1}    // will be overwritten by subresourceRange
-  };
-
-  // ensure all previous writes have completed
-  srcimBarrier.srcAccessMask = VK_ACCESS_ALL_WRITE_BITS;
-  // before we go reading
-  srcimBarrier.dstAccessMask = VK_ACCESS_SHADER_READ_BIT;
+  VkCommandBuffer cmd = m_pDriver->GetNextCmd();
 
   VkCommandBufferBeginInfo beginInfo = {VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO, NULL,
                                         VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT};
 
   vt->BeginCommandBuffer(Unwrap(cmd), &beginInfo);
 
-  VkCommandBuffer extQCmd = VK_NULL_HANDLE;
-  VkResult vkr = VK_SUCCESS;
-
-  if(srcimBarrier.srcQueueFamilyIndex != srcimBarrier.dstQueueFamilyIndex)
-  {
-    extQCmd = m_pDriver->GetExtQueueCmd(srcimBarrier.srcQueueFamilyIndex);
-
-    vkr = ObjDisp(extQCmd)->BeginCommandBuffer(Unwrap(extQCmd), &beginInfo);
-    RDCASSERTEQUAL(vkr, VK_SUCCESS);
-  }
-
-  for(size_t si = 0; si < layouts.subresourceStates.size(); si++)
-  {
-    srcimBarrier.subresourceRange = layouts.subresourceStates[si].subresourceRange;
-    srcimBarrier.oldLayout = layouts.subresourceStates[si].newLayout;
-    srcimBarrier.srcAccessMask = VK_ACCESS_ALL_WRITE_BITS | MakeAccessMask(srcimBarrier.oldLayout);
-    DoPipelineBarrier(cmd, 1, &srcimBarrier);
-
-    if(extQCmd != VK_NULL_HANDLE)
-      DoPipelineBarrier(extQCmd, 1, &srcimBarrier);
-  }
-
-  if(extQCmd != VK_NULL_HANDLE)
-  {
-    vkr = ObjDisp(extQCmd)->EndCommandBuffer(Unwrap(extQCmd));
-    RDCASSERTEQUAL(vkr, VK_SUCCESS);
-
-    m_pDriver->SubmitAndFlushExtQueue(layouts.queueFamilyIndex);
-  }
-
-  srcimBarrier.oldLayout = srcimBarrier.newLayout;
-  srcimBarrier.srcAccessMask = srcimBarrier.dstAccessMask;
+  rdcarray<VkImageMemoryBarrier> setupBarriers, cleanupBarriers;
+  bool extQCleanup = false;
+  m_pDriver->TempTransition(liveIm, VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL,
+                            VK_ACCESS_SHADER_READ_BIT, setupBarriers, cleanupBarriers, extQCleanup);
+  DoPipelineBarrier(cmd, setupBarriers.size(), setupBarriers.data());
 
   {
     vt->CmdBeginRenderPass(Unwrap(cmd), &rpbegin, VK_SUBPASS_CONTENTS_INLINE);
@@ -536,6 +499,26 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
       GetDebugManager()->CreateCustomShaderPipeline(cfg.customShaderId, m_TexRender.PipeLayout);
       pipe = GetDebugManager()->GetCustomPipeline();
     }
+    else if(flags & (eTexDisplay_RemapFloat | eTexDisplay_RemapUInt | eTexDisplay_RemapSInt))
+    {
+      int i = 0;
+      if(flags & eTexDisplay_RemapFloat)
+        i = 0;
+      else if(flags & eTexDisplay_RemapUInt)
+        i = 1;
+      else if(flags & eTexDisplay_RemapSInt)
+        i = 2;
+
+      int f = 0;
+      if(flags & eTexDisplay_32Render)
+        f = 2;
+      else if(flags & eTexDisplay_16Render)
+        f = 1;
+      else
+        f = 0;
+
+      pipe = m_TexRender.RemapPipeline[f][i][greenonly ? 1 : 0];
+    }
     else if(f16render)
     {
       pipe = greenonly ? m_TexRender.F16PipelineGreenOnly : m_TexRender.F16Pipeline;
@@ -575,37 +558,17 @@ bool VulkanReplay::RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginIn
     vt->CmdEndRenderPass(Unwrap(cmd));
   }
 
-  std::swap(srcimBarrier.srcQueueFamilyIndex, srcimBarrier.dstQueueFamilyIndex);
-
-  if(extQCmd != VK_NULL_HANDLE)
-  {
-    vkr = ObjDisp(extQCmd)->BeginCommandBuffer(Unwrap(extQCmd), &beginInfo);
-    RDCASSERTEQUAL(vkr, VK_SUCCESS);
-  }
-
-  for(size_t si = 0; si < layouts.subresourceStates.size(); si++)
-  {
-    srcimBarrier.subresourceRange = layouts.subresourceStates[si].subresourceRange;
-    srcimBarrier.newLayout = layouts.subresourceStates[si].newLayout;
-    srcimBarrier.dstAccessMask = MakeAccessMask(srcimBarrier.newLayout);
-    DoPipelineBarrier(cmd, 1, &srcimBarrier);
-
-    if(extQCmd != VK_NULL_HANDLE)
-      DoPipelineBarrier(extQCmd, 1, &srcimBarrier);
-  }
+  DoPipelineBarrier(cmd, cleanupBarriers.size(), cleanupBarriers.data());
 
   vt->EndCommandBuffer(Unwrap(cmd));
 
-  if(extQCmd != VK_NULL_HANDLE)
+  if(extQCleanup)
   {
     // ensure work is completed before we pass ownership back to original queue
     m_pDriver->SubmitCmds();
     m_pDriver->FlushQ();
 
-    vkr = ObjDisp(extQCmd)->EndCommandBuffer(Unwrap(extQCmd));
-    RDCASSERTEQUAL(vkr, VK_SUCCESS);
-
-    m_pDriver->SubmitAndFlushExtQueue(layouts.queueFamilyIndex);
+    m_pDriver->SubmitExtQBarriers(~0U, cleanupBarriers);
   }
 
 #if ENABLED(SINGLE_FLUSH_VALIDATE)
