@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -79,13 +79,33 @@
 
 #endif
 
+#if ENABLED(RDOC_WAYLAND)
+
+#define WINDOW_HANDLE_WAYLAND \
+  struct                      \
+  {                           \
+    wl_display *display;      \
+    wl_surface *window;       \
+  } wayland;
+
+#else
+
+#define WINDOW_HANDLE_WAYLAND \
+  struct                      \
+  {                           \
+  } wayland;
+
+#endif
+
 #define WINDOW_HANDLE_DECL \
   WINDOW_HANDLE_XLIB       \
-  WINDOW_HANDLE_XCB
+  WINDOW_HANDLE_XCB        \
+  WINDOW_HANDLE_WAYLAND
 
 #define WINDOW_HANDLE_INIT \
   RDCEraseEl(xlib);        \
-  RDCEraseEl(xcb);
+  RDCEraseEl(xcb);         \
+  RDCEraseEl(wayland);
 
 #elif ENABLED(RDOC_APPLE) || ENABLED(RDOC_GGP)
 
@@ -99,7 +119,6 @@
 #endif
 
 #include <map>
-using std::map;
 
 // similar to RDCUNIMPLEMENTED but for things that are hit often so we don't want to fire the
 // debugbreak.
@@ -112,13 +131,7 @@ using std::map;
     msgprinted = true;                                   \
   } while((void)0, 0)
 
-#define MSAA_MESH_VIEW OPTION_ON
-
-#if ENABLED(MSAA_MESH_VIEW)
 #define VULKAN_MESH_VIEW_SAMPLES VK_SAMPLE_COUNT_4_BIT
-#else
-#define VULKAN_MESH_VIEW_SAMPLES VK_SAMPLE_COUNT_1_BIT
-#endif
 
 class AMDCounters;
 class WrappedVulkan;
@@ -148,7 +161,7 @@ struct VulkanPostVSData
     uint32_t instStride;
 
     // complex case - expansion per instance
-    std::vector<InstData> instData;
+    rdcarray<InstData> instData;
 
     uint32_t numViews;
 
@@ -214,14 +227,26 @@ struct BindIdx
 struct DynamicUsedBinds
 {
   bool compute = false, valid = false;
-  std::vector<BindIdx> used;
+  rdcarray<BindIdx> used;
+};
+
+enum TexDisplayFlags
+{
+  eTexDisplay_16Render = 0x1,
+  eTexDisplay_32Render = 0x2,
+  eTexDisplay_BlendAlpha = 0x4,
+  eTexDisplay_MipShift = 0x8,
+  eTexDisplay_GreenOnly = 0x10,
+  eTexDisplay_RemapFloat = 0x20,
+  eTexDisplay_RemapUInt = 0x40,
+  eTexDisplay_RemapSInt = 0x80,
 };
 
 class VulkanReplay : public IReplayDriver
 {
 public:
-  VulkanReplay();
-
+  VulkanReplay(WrappedVulkan *d);
+  virtual ~VulkanReplay() {}
   void SetRGP(AMDRGPControl *rgp) { m_RGP = rgp; }
   void SetProxy(bool p) { m_Proxy = p; }
   bool IsRemoteProxy() { return m_Proxy; }
@@ -230,29 +255,30 @@ public:
   void CreateResources();
   void DestroyResources();
 
-  void SetDriver(WrappedVulkan *d) { m_pDriver = d; }
   DriverInformation GetDriverInfo() { return m_DriverInfo; }
+  rdcarray<GPUDevice> GetAvailableGPUs();
   APIProperties GetAPIProperties();
 
   ResourceDescription &GetResourceDesc(ResourceId id);
-  const std::vector<ResourceDescription> &GetResources();
+  const rdcarray<ResourceDescription> &GetResources();
 
-  std::vector<ResourceId> GetBuffers();
+  rdcarray<ResourceId> GetBuffers();
   BufferDescription GetBuffer(ResourceId id);
 
-  std::vector<ResourceId> GetTextures();
+  rdcarray<ResourceId> GetTextures();
   TextureDescription GetTexture(ResourceId id);
 
   rdcarray<ShaderEntryPoint> GetShaderEntryPoints(ResourceId shader);
-  ShaderReflection *GetShader(ResourceId shader, ShaderEntryPoint entry);
+  ShaderReflection *GetShader(ResourceId pipeline, ResourceId shader, ShaderEntryPoint entry);
 
-  vector<string> GetDisassemblyTargets();
-  string DisassembleShader(ResourceId pipeline, const ShaderReflection *refl, const string &target);
+  rdcarray<rdcstr> GetDisassemblyTargets();
+  rdcstr DisassembleShader(ResourceId pipeline, const ShaderReflection *refl, const rdcstr &target);
 
-  vector<EventUsage> GetUsage(ResourceId id);
+  rdcarray<EventUsage> GetUsage(ResourceId id);
 
-  FrameRecord GetFrameRecord();
-  vector<DebugMessage> GetDebugMessages();
+  FrameRecord &WriteFrameRecord() { return m_FrameRecord; }
+  FrameRecord GetFrameRecord() { return m_FrameRecord; }
+  rdcarray<DebugMessage> GetDebugMessages();
 
   void SavePipelineState(uint32_t eventId);
   const D3D11Pipe::State *GetD3D11PipelineState() { return NULL; }
@@ -265,15 +291,17 @@ public:
   void ReplayLog(uint32_t endEventID, ReplayLogType replayType);
   const SDFile &GetStructuredFile();
 
-  vector<uint32_t> GetPassEvents(uint32_t eventId);
+  rdcarray<uint32_t> GetPassEvents(uint32_t eventId);
 
-  vector<WindowingSystem> GetSupportedWindowSystems();
+  rdcarray<WindowingSystem> GetSupportedWindowSystems();
 
   AMDRGPControl *GetRGPControl() { return m_RGP; }
   uint64_t MakeOutputWindow(WindowingData window, bool depth);
   void DestroyOutputWindow(uint64_t id);
   bool CheckResizeOutputWindow(uint64_t id);
   void GetOutputWindowDimensions(uint64_t id, int32_t &w, int32_t &h);
+  void SetOutputWindowDimensions(uint64_t id, int32_t w, int32_t h);
+  void GetOutputWindowData(uint64_t id, bytebuf &retData);
   void ClearOutputWindowColor(uint64_t id, FloatVector col);
   void ClearOutputWindowDepth(uint64_t id, float depth, uint8_t stencil);
   void BindOutputWindow(uint64_t id, bool depth);
@@ -282,18 +310,19 @@ public:
 
   ResourceId GetLiveID(ResourceId id);
 
-  vector<GPUCounter> EnumerateCounters();
+  rdcarray<GPUCounter> EnumerateCounters();
   CounterDescription DescribeCounter(GPUCounter counterID);
-  vector<CounterResult> FetchCounters(const vector<GPUCounter> &counters);
+  rdcarray<CounterResult> FetchCounters(const rdcarray<GPUCounter> &counters);
 
-  bool GetMinMax(ResourceId texid, uint32_t sliceFace, uint32_t mip, uint32_t sample,
-                 CompType typeHint, float *minval, float *maxval);
-  bool GetHistogram(ResourceId texid, uint32_t sliceFace, uint32_t mip, uint32_t sample,
-                    CompType typeHint, float minval, float maxval, bool channels[4],
-                    vector<uint32_t> &histogram);
+  void PickPixel(ResourceId texture, uint32_t x, uint32_t y, const Subresource &sub,
+                 CompType typeCast, float pixel[4]);
+  bool GetMinMax(ResourceId texid, const Subresource &sub, CompType typeCast, float *minval,
+                 float *maxval);
+  bool GetHistogram(ResourceId texid, const Subresource &sub, CompType typeCast, float minval,
+                    float maxval, bool channels[4], rdcarray<uint32_t> &histogram);
 
   void InitPostVSBuffers(uint32_t eventId);
-  void InitPostVSBuffers(const std::vector<uint32_t> &passEvents);
+  void InitPostVSBuffers(const rdcarray<uint32_t> &passEvents);
 
   // indicates that EID alias is the same as eventId
   void AliasPostVSBuffers(uint32_t eventId, uint32_t alias) { m_PostVS.Alias[alias] = eventId; }
@@ -301,23 +330,29 @@ public:
                               MeshDataStage stage);
 
   void GetBufferData(ResourceId buff, uint64_t offset, uint64_t len, bytebuf &retData);
-  void GetTextureData(ResourceId tex, uint32_t arrayIdx, uint32_t mip,
-                      const GetTextureDataParams &params, bytebuf &data);
+  void GetTextureData(ResourceId tex, const Subresource &sub, const GetTextureDataParams &params,
+                      bytebuf &data);
 
   void ReplaceResource(ResourceId from, ResourceId to);
   void RemoveReplacement(ResourceId id);
 
-  void RenderMesh(uint32_t eventId, const vector<MeshFormat> &secondaryDraws, const MeshDisplay &cfg);
+  void RenderMesh(uint32_t eventId, const rdcarray<MeshFormat> &secondaryDraws,
+                  const MeshDisplay &cfg);
 
+  rdcarray<ShaderEncoding> GetCustomShaderEncodings()
+  {
+    return {ShaderEncoding::SPIRV, ShaderEncoding::GLSL};
+  }
   rdcarray<ShaderEncoding> GetTargetShaderEncodings()
   {
     return {ShaderEncoding::SPIRV, ShaderEncoding::GLSL};
   }
-  void BuildTargetShader(ShaderEncoding sourceEncoding, bytebuf source, string entry,
-                         const ShaderCompileFlags &compileFlags, ShaderStage type, ResourceId *id,
-                         string *errors);
-  void BuildCustomShader(string source, string entry, const ShaderCompileFlags &compileFlags,
-                         ShaderStage type, ResourceId *id, string *errors);
+  void BuildTargetShader(ShaderEncoding sourceEncoding, const bytebuf &source, const rdcstr &entry,
+                         const ShaderCompileFlags &compileFlags, ShaderStage type, ResourceId &id,
+                         rdcstr &errors);
+  void BuildCustomShader(ShaderEncoding sourceEncoding, const bytebuf &source, const rdcstr &entry,
+                         const ShaderCompileFlags &compileFlags, ShaderStage type, ResourceId &id,
+                         rdcstr &errors);
   void FreeCustomShader(ResourceId id);
 
   bool RenderTexture(TextureDisplay cfg);
@@ -326,32 +361,30 @@ public:
 
   void RenderHighlightBox(float w, float h, float scale);
 
-  void FillCBufferVariables(ResourceId shader, string entryPoint, uint32_t cbufSlot,
-                            rdcarray<ShaderVariable> &outvars, const bytebuf &data);
+  void FillCBufferVariables(ResourceId pipeline, ResourceId shader, rdcstr entryPoint,
+                            uint32_t cbufSlot, rdcarray<ShaderVariable> &outvars,
+                            const bytebuf &data);
 
-  vector<PixelModification> PixelHistory(vector<EventUsage> events, ResourceId target, uint32_t x,
-                                         uint32_t y, uint32_t slice, uint32_t mip,
-                                         uint32_t sampleIdx, CompType typeHint);
+  rdcarray<PixelModification> PixelHistory(rdcarray<EventUsage> events, ResourceId target, uint32_t x,
+                                           uint32_t y, const Subresource &sub, CompType typeCast);
   ShaderDebugTrace DebugVertex(uint32_t eventId, uint32_t vertid, uint32_t instid, uint32_t idx,
                                uint32_t instOffset, uint32_t vertOffset);
   ShaderDebugTrace DebugPixel(uint32_t eventId, uint32_t x, uint32_t y, uint32_t sample,
                               uint32_t primitive);
   ShaderDebugTrace DebugThread(uint32_t eventId, const uint32_t groupid[3],
                                const uint32_t threadid[3]);
-  void PickPixel(ResourceId texture, uint32_t x, uint32_t y, uint32_t sliceFace, uint32_t mip,
-                 uint32_t sample, CompType typeHint, float pixel[4]);
   uint32_t PickVertex(uint32_t eventId, int32_t width, int32_t height, const MeshDisplay &cfg,
                       uint32_t x, uint32_t y);
 
-  ResourceId RenderOverlay(ResourceId cfg, CompType typeHint, DebugOverlay overlay,
-                           uint32_t eventId, const vector<uint32_t> &passEvents);
-  ResourceId ApplyCustomShader(ResourceId shader, ResourceId texid, uint32_t mip, uint32_t arrayIdx,
-                               uint32_t sampleIdx, CompType typeHint);
+  ResourceId RenderOverlay(ResourceId cfg, CompType typeCast, FloatVector clearCol,
+                           DebugOverlay overlay, uint32_t eventId,
+                           const rdcarray<uint32_t> &passEvents);
+  ResourceId ApplyCustomShader(ResourceId shader, ResourceId texid, const Subresource &sub,
+                               CompType typeCast);
 
   ResourceId CreateProxyTexture(const TextureDescription &templateTex);
-  void SetProxyTextureData(ResourceId texid, uint32_t arrayIdx, uint32_t mip, byte *data,
-                           size_t dataSize);
-  bool IsTextureSupported(const ResourceFormat &format);
+  void SetProxyTextureData(ResourceId texid, const Subresource &sub, byte *data, size_t dataSize);
+  bool IsTextureSupported(const TextureDescription &tex);
   bool NeedRemapForFetch(const ResourceFormat &format);
 
   ResourceId CreateProxyBuffer(const BufferDescription &templateBuf);
@@ -369,8 +402,8 @@ public:
   // but for developers running builds locally or just in case, we need to be able to update the
   // layer registration ourselves.
   // These functions are defined in vk_<platform>.cpp
-  static bool CheckVulkanLayer(VulkanLayerFlags &flags, std::vector<std::string> &myJSONs,
-                               std::vector<std::string> &otherJSONs);
+  static bool CheckVulkanLayer(VulkanLayerFlags &flags, rdcarray<rdcstr> &myJSONs,
+                               rdcarray<rdcstr> &otherJSONs);
   static void InstallVulkanLayer(bool systemLevel);
   void GetInitialDriverVersion();
   void SetDriverInformation(const VkPhysicalDeviceProperties &props);
@@ -381,8 +414,8 @@ private:
   void ClearFeedbackCache();
 
   void PatchReservedDescriptors(const VulkanStatePipeline &pipe, VkDescriptorPool &descpool,
-                                std::vector<VkDescriptorSetLayout> &setLayouts,
-                                std::vector<VkDescriptorSet> &descSets,
+                                rdcarray<VkDescriptorSetLayout> &setLayouts,
+                                rdcarray<VkDescriptorSet> &descSets,
                                 VkShaderStageFlagBits patchedBindingStage,
                                 const VkDescriptorSetLayoutBinding *newBindings,
                                 size_t newBindingsCount);
@@ -391,10 +424,12 @@ private:
   void FetchTessGSOut(uint32_t eventId);
   void ClearPostVSCache();
 
+  void RefreshDerivedReplacements();
+
   bool RenderTextureInternal(TextureDisplay cfg, VkRenderPassBeginInfo rpbegin, int flags);
 
-  bool GetMinMax(ResourceId texid, uint32_t sliceFace, uint32_t mip, uint32_t sample,
-                 CompType typeHint, bool stencil, float *minval, float *maxval);
+  bool GetMinMax(ResourceId texid, const Subresource &sub, CompType typeCast, bool stencil,
+                 float *minval, float *maxval);
 
   VulkanDebugManager *GetDebugManager();
   VulkanResourceManager *GetResourceManager();
@@ -403,20 +438,19 @@ private:
   {
     OutputWindow();
 
-    void SetCol(VkDeviceMemory mem, VkImage img);
-    void SetDS(VkDeviceMemory mem, VkImage img);
     void Create(WrappedVulkan *driver, VkDevice device, bool depth);
     void Destroy(WrappedVulkan *driver, VkDevice device);
 
     // implemented in vk_replay_platform.cpp
-    void CreateSurface(VkInstance inst);
+    void CreateSurface(WrappedVulkan *driver, VkInstance inst);
     void SetWindowHandle(WindowingData window);
 
     WindowingSystem m_WindowSystem;
 
     WINDOW_HANDLE_DECL;
 
-    bool fresh;
+    bool fresh = true;
+    bool outofdate = false;
 
     uint32_t width, height;
 
@@ -451,7 +485,7 @@ private:
     VulkanResourceManager *m_ResourceManager;
   };
 
-  map<uint64_t, OutputWindow> m_OutputWindows;
+  std::map<uint64_t, OutputWindow> m_OutputWindows;
   uint64_t m_OutputWinID;
   uint64_t m_ActiveWinID;
   bool m_BindDepth;
@@ -461,18 +495,11 @@ private:
 
   bool m_Proxy;
 
+  FrameRecord m_FrameRecord;
+
   WrappedVulkan *m_pDriver = NULL;
   VkDevice m_Device = VK_NULL_HANDLE;
 
-  enum TexDisplayFlags
-  {
-    eTexDisplay_F16Render = 0x1,
-    eTexDisplay_F32Render = 0x2,
-    eTexDisplay_BlendAlpha = 0x4,
-    eTexDisplay_MipShift = 0x8,
-    eTexDisplay_GreenOnly = 0x10,
-  };
-
   // General use/misc items that are used in many places
   struct GeneralMisc
   {
@@ -485,7 +512,7 @@ private:
 
   struct TextureDisplayViews
   {
-    CompType typeHint;
+    CompType typeCast;
     VkFormat castedFormat;    // the format after applying the above type hint
 
     // for a color/depth-only textures, views[0] is the view and views[1] and views[2] are NULL
@@ -506,6 +533,8 @@ private:
     VkPipeline BlendPipeline = VK_NULL_HANDLE;
     VkPipeline F16Pipeline = VK_NULL_HANDLE;
     VkPipeline F32Pipeline = VK_NULL_HANDLE;
+    // for each of 8-bit, 16-bit, 32-bit and float, uint, sint, and for normal/green-only
+    VkPipeline RemapPipeline[3][3][2] = {};
 
     VkPipeline PipelineGreenOnly = VK_NULL_HANDLE;
     VkPipeline F16PipelineGreenOnly = VK_NULL_HANDLE;
@@ -517,6 +546,7 @@ private:
     VkDescriptorSet DescSet[16] = {VK_NULL_HANDLE};
     uint32_t NextSet = 0;
 
+    VkSampler PointSampler = VK_NULL_HANDLE;
     VkSampler LinearSampler = VK_NULL_HANDLE;
 
     // descriptors must be valid even if they're skipped dynamically in the shader, so we create
@@ -660,22 +690,48 @@ private:
     std::map<uint32_t, DynamicUsedBinds> Usage;
   } m_BindlessFeedback;
 
-  std::vector<ResourceDescription> m_Resources;
+  rdcarray<ResourceDescription> m_Resources;
   std::map<ResourceId, size_t> m_ResourceIdx;
 
   VKPipe::State m_VulkanPipelineState;
 
   DriverInformation m_DriverInfo;
 
+  struct PipelineExecutables
+  {
+    VkShaderStageFlags stages;
+    rdcstr name, description;
+    uint32_t subgroupSize;
+    rdcarray<VkPipelineExecutableStatisticKHR> statistics;
+    rdcarray<VkPipelineExecutableInternalRepresentationKHR> representations;
+
+    // internal data, pointed to from representations above
+    rdcarray<bytebuf> irbytes;
+  };
+
+  std::map<ResourceId, rdcarray<PipelineExecutables>> m_PipelineExecutables;
+
+  void CachePipelineExecutables(ResourceId pipeline);
+
   void CreateTexImageView(VkImage liveIm, const VulkanCreationInfo::Image &iminfo,
-                          CompType typeHint, TextureDisplayViews &views);
+                          CompType typeCast, TextureDisplayViews &views);
 
-  void FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex, vector<uint32_t> *eventIDs);
+  void FillTimersAMD(uint32_t *eventStartID, uint32_t *sampleIndex, rdcarray<uint32_t> *eventIDs);
 
-  vector<CounterResult> FetchCountersAMD(const vector<GPUCounter> &counters);
+  rdcarray<CounterResult> FetchCountersAMD(const rdcarray<GPUCounter> &counters);
 
   AMDCounters *m_pAMDCounters = NULL;
   AMDRGPControl *m_RGP = NULL;
 
   VulkanAMDDrawCallback *m_pAMDDrawCallback = NULL;
+
+  rdcarray<CounterResult> FetchCountersKHR(const rdcarray<GPUCounter> &counters);
+
+  rdcarray<VkPerformanceCounterKHR> m_KHRCounters;
+  rdcarray<VkPerformanceCounterDescriptionKHR> m_KHRCountersDescriptions;
+
+  void convertKhrCounterResult(CounterResult &rdcResult,
+                               const VkPerformanceCounterResultKHR &khrResult,
+                               VkPerformanceCounterUnitKHR khrUnit,
+                               VkPerformanceCounterStorageKHR khrStorage);
 };
