@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,6 @@
 #include "vk_shader_cache.h"
 #include "common/shader_cache.h"
 #include "data/glsl_shaders.h"
-#include "driver/shaders/spirv/spirv_common.h"
 #include "strings/string_utils.h"
 
 enum class FeatureCheck
@@ -34,6 +33,7 @@ enum class FeatureCheck
   ShaderMSAAStorage = 0x1,
   FragmentStores = 0x2,
   NonMetalBackend = 0x4,
+  FormatlessWrite = 0x8,
 };
 
 BITMASK_OPERATORS(FeatureCheck);
@@ -42,48 +42,55 @@ struct BuiltinShaderConfig
 {
   BuiltinShader builtin;
   EmbeddedResourceType resource;
-  SPIRVShaderStage stage;
+  rdcspv::ShaderStage stage;
   FeatureCheck checks;
   bool uniforms;
 };
 
 static const BuiltinShaderConfig builtinShaders[] = {
-    {BuiltinShader::BlitVS, EmbeddedResource(glsl_blit_vert), SPIRVShaderStage::Vertex,
+    {BuiltinShader::BlitVS, EmbeddedResource(glsl_blit_vert), rdcspv::ShaderStage::Vertex,
      FeatureCheck::NoCheck, true},
     {BuiltinShader::CheckerboardFS, EmbeddedResource(glsl_checkerboard_frag),
-     SPIRVShaderStage::Fragment, FeatureCheck::NoCheck, true},
+     rdcspv::ShaderStage::Fragment, FeatureCheck::NoCheck, true},
     {BuiltinShader::TexDisplayFS, EmbeddedResource(glsl_texdisplay_frag),
-     SPIRVShaderStage::Fragment, FeatureCheck::NoCheck, true},
-    {BuiltinShader::FixedColFS, EmbeddedResource(glsl_fixedcol_frag), SPIRVShaderStage::Fragment,
+     rdcspv::ShaderStage::Fragment, FeatureCheck::NoCheck, true},
+    {BuiltinShader::FixedColFS, EmbeddedResource(glsl_fixedcol_frag), rdcspv::ShaderStage::Fragment,
      FeatureCheck::NoCheck, false},
-    {BuiltinShader::TextVS, EmbeddedResource(glsl_vktext_vert), SPIRVShaderStage::Vertex,
+    {BuiltinShader::TextVS, EmbeddedResource(glsl_vktext_vert), rdcspv::ShaderStage::Vertex,
      FeatureCheck::NoCheck, true},
-    {BuiltinShader::TextFS, EmbeddedResource(glsl_vktext_frag), SPIRVShaderStage::Fragment,
+    {BuiltinShader::TextFS, EmbeddedResource(glsl_vktext_frag), rdcspv::ShaderStage::Fragment,
      FeatureCheck::NoCheck, true},
-    {BuiltinShader::MeshVS, EmbeddedResource(glsl_mesh_vert), SPIRVShaderStage::Vertex,
+    {BuiltinShader::MeshVS, EmbeddedResource(glsl_mesh_vert), rdcspv::ShaderStage::Vertex,
      FeatureCheck::NoCheck, true},
-    {BuiltinShader::MeshGS, EmbeddedResource(glsl_mesh_geom), SPIRVShaderStage::Geometry,
+    {BuiltinShader::MeshGS, EmbeddedResource(glsl_mesh_geom), rdcspv::ShaderStage::Geometry,
      FeatureCheck::NoCheck, true},
-    {BuiltinShader::MeshFS, EmbeddedResource(glsl_mesh_frag), SPIRVShaderStage::Fragment,
+    {BuiltinShader::MeshFS, EmbeddedResource(glsl_mesh_frag), rdcspv::ShaderStage::Fragment,
      FeatureCheck::NoCheck, true},
-    {BuiltinShader::MeshCS, EmbeddedResource(glsl_mesh_comp), SPIRVShaderStage::Compute,
+    {BuiltinShader::MeshCS, EmbeddedResource(glsl_mesh_comp), rdcspv::ShaderStage::Compute,
      FeatureCheck::NoCheck, true},
     {BuiltinShader::QuadResolveFS, EmbeddedResource(glsl_quadresolve_frag),
-     SPIRVShaderStage::Fragment, FeatureCheck::FragmentStores, true},
-    {BuiltinShader::QuadWriteFS, EmbeddedResource(glsl_quadwrite_frag), SPIRVShaderStage::Fragment,
-     FeatureCheck::FragmentStores, false},
-    {BuiltinShader::TrisizeGS, EmbeddedResource(glsl_trisize_geom), SPIRVShaderStage::Geometry,
+     rdcspv::ShaderStage::Fragment, FeatureCheck::FragmentStores, true},
+    {BuiltinShader::QuadWriteFS, EmbeddedResource(glsl_quadwrite_frag), rdcspv::ShaderStage::Fragment,
+     FeatureCheck::FragmentStores | FeatureCheck::NonMetalBackend, false},
+    {BuiltinShader::TrisizeGS, EmbeddedResource(glsl_trisize_geom), rdcspv::ShaderStage::Geometry,
      FeatureCheck::NoCheck, true},
-    {BuiltinShader::TrisizeFS, EmbeddedResource(glsl_trisize_frag), SPIRVShaderStage::Fragment,
+    {BuiltinShader::TrisizeFS, EmbeddedResource(glsl_trisize_frag), rdcspv::ShaderStage::Fragment,
      FeatureCheck::NoCheck, true},
-    {BuiltinShader::MS2ArrayCS, EmbeddedResource(glsl_ms2array_comp), SPIRVShaderStage::Compute,
-     FeatureCheck::ShaderMSAAStorage | FeatureCheck::NonMetalBackend, true},
-    {BuiltinShader::Array2MSCS, EmbeddedResource(glsl_array2ms_comp), SPIRVShaderStage::Compute,
-     FeatureCheck::ShaderMSAAStorage | FeatureCheck::NonMetalBackend, true},
+    {BuiltinShader::MS2ArrayCS, EmbeddedResource(glsl_ms2array_comp), rdcspv::ShaderStage::Compute,
+     FeatureCheck::FormatlessWrite | FeatureCheck::NonMetalBackend, true},
+    {BuiltinShader::Array2MSCS, EmbeddedResource(glsl_array2ms_comp), rdcspv::ShaderStage::Compute,
+     FeatureCheck::ShaderMSAAStorage | FeatureCheck::FormatlessWrite | FeatureCheck::NonMetalBackend,
+     true},
     {BuiltinShader::DepthMS2ArrayFS, EmbeddedResource(glsl_depthms2arr_frag),
-     SPIRVShaderStage::Fragment, FeatureCheck::NonMetalBackend, true},
+     rdcspv::ShaderStage::Fragment, FeatureCheck::NonMetalBackend, true},
     {BuiltinShader::DepthArray2MSFS, EmbeddedResource(glsl_deptharr2ms_frag),
-     SPIRVShaderStage::Fragment, FeatureCheck::NonMetalBackend, true},
+     rdcspv::ShaderStage::Fragment, FeatureCheck::NonMetalBackend, true},
+    {BuiltinShader::TexRemapFloat, EmbeddedResource(glsl_texremap_frag),
+     rdcspv::ShaderStage::Fragment, FeatureCheck::NoCheck, true},
+    {BuiltinShader::TexRemapUInt, EmbeddedResource(glsl_texremap_frag),
+     rdcspv::ShaderStage::Fragment, FeatureCheck::NoCheck, true},
+    {BuiltinShader::TexRemapSInt, EmbeddedResource(glsl_texremap_frag),
+     rdcspv::ShaderStage::Fragment, FeatureCheck::NoCheck, true},
 };
 
 RDCCOMPILE_ASSERT(ARRAY_COUNT(builtinShaders) == arraydim<BuiltinShader>(),
@@ -95,7 +102,7 @@ struct VulkanBlobShaderCallbacks
   {
     RDCASSERT(ret);
 
-    SPIRVBlob blob = new std::vector<uint32_t>();
+    SPIRVBlob blob = new rdcarray<uint32_t>();
 
     blob->resize(size / sizeof(uint32_t));
 
@@ -128,9 +135,15 @@ VulkanShaderCache::VulkanShaderCache(WrappedVulkan *driver)
   VkDriverInfo driverVersion = driver->GetDriverInfo();
   const VkPhysicalDeviceFeatures &features = driver->GetDeviceFeatures();
 
-  std::string src;
-  SPIRVCompilationSettings compileSettings;
-  compileSettings.lang = SPIRVSourceLanguage::VulkanGLSL;
+  m_GlobalDefines = "#define HAS_BIT_CONVERSION 1\n";
+  if(driverVersion.TexelFetchBrokenDriver())
+    m_GlobalDefines += "#define NO_TEXEL_FETCH\n";
+  if(driverVersion.RunningOnMetal())
+    m_GlobalDefines += "#define METAL_BACKEND\n";
+
+  rdcstr src;
+  rdcspv::CompilationSettings compileSettings;
+  compileSettings.lang = rdcspv::InputLanguage::VulkanGLSL;
 
   for(auto i : indices<BuiltinShader>())
   {
@@ -141,7 +154,15 @@ VulkanShaderCache::VulkanShaderCache(WrappedVulkan *driver)
     if(config.checks & FeatureCheck::ShaderMSAAStorage)
     {
       if(driverVersion.TexelFetchBrokenDriver() || driverVersion.AMDStorageMSAABrokenDriver() ||
-         !features.shaderStorageImageMultisample || !features.shaderStorageImageWriteWithoutFormat)
+         !features.shaderStorageImageMultisample)
+      {
+        continue;
+      }
+    }
+
+    if(config.checks & FeatureCheck::FormatlessWrite)
+    {
+      if(!features.shaderStorageImageWriteWithoutFormat)
       {
         continue;
       }
@@ -161,18 +182,23 @@ VulkanShaderCache::VulkanShaderCache(WrappedVulkan *driver)
         continue;
     }
 
-    if(config.stage == SPIRVShaderStage::Geometry && !features.geometryShader)
+    if(config.stage == rdcspv::ShaderStage::Geometry && !features.geometryShader)
       continue;
 
-    std::string defines = "";
-    if(driverVersion.TexelFetchBrokenDriver())
-      defines += "#define NO_TEXEL_FETCH\n";
+    rdcstr defines = m_GlobalDefines;
 
-    src =
-        GenerateGLSLShader(GetDynamicEmbeddedResource(config.resource), eShaderVulkan, 430, defines);
+    if(config.builtin == BuiltinShader::TexRemapFloat)
+      defines += rdcstr("#define UINT_TEX 0\n#define SINT_TEX 0\n");
+    else if(config.builtin == BuiltinShader::TexRemapUInt)
+      defines += rdcstr("#define UINT_TEX 1\n#define SINT_TEX 0\n");
+    else if(config.builtin == BuiltinShader::TexRemapSInt)
+      defines += rdcstr("#define UINT_TEX 0\n#define SINT_TEX 1\n");
+
+    src = GenerateGLSLShader(GetDynamicEmbeddedResource(config.resource), ShaderType::Vulkan, 430,
+                             defines);
 
     compileSettings.stage = config.stage;
-    std::string err = GetSPIRVBlob(compileSettings, src, m_BuiltinShaderBlobs[i]);
+    rdcstr err = GetSPIRVBlob(compileSettings, src, m_BuiltinShaderBlobs[i]);
 
     if(!err.empty() || m_BuiltinShaderBlobs[i] == VK_NULL_HANDLE)
     {
@@ -216,8 +242,8 @@ VulkanShaderCache::~VulkanShaderCache()
     m_pDriver->vkDestroyShaderModule(m_Device, m_BuiltinShaderModules[i], NULL);
 }
 
-std::string VulkanShaderCache::GetSPIRVBlob(const SPIRVCompilationSettings &settings,
-                                            const std::string &src, SPIRVBlob &outBlob)
+rdcstr VulkanShaderCache::GetSPIRVBlob(const rdcspv::CompilationSettings &settings,
+                                       const rdcstr &src, SPIRVBlob &outBlob)
 {
   RDCASSERT(!src.empty());
 
@@ -234,12 +260,12 @@ std::string VulkanShaderCache::GetSPIRVBlob(const SPIRVCompilationSettings &sett
     return "";
   }
 
-  SPIRVBlob spirv = new std::vector<uint32_t>();
-  std::string errors = CompileSPIRV(settings, {src}, *spirv);
+  SPIRVBlob spirv = new rdcarray<uint32_t>();
+  rdcstr errors = rdcspv::Compile(settings, {src}, *spirv);
 
   if(!errors.empty())
   {
-    std::string logerror = errors;
+    rdcstr logerror = errors;
     if(logerror.length() > 1024)
       logerror = logerror.substr(0, 1024) + "...";
 
@@ -270,26 +296,24 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
 
   static VkPipelineShaderStageCreateInfo stages[6];
   static VkSpecializationInfo specInfo[6];
-  static vector<VkSpecializationMapEntry> specMapEntries;
-  static std::vector<byte> specdata;
+  static rdcarray<VkSpecializationMapEntry> specMapEntries;
+
+  // the specialization constants can't use more than a uint64_t, so we just over-allocate
+  static rdcarray<uint64_t> specdata;
 
   size_t specEntries = 0;
-  size_t specSize = 0;
 
   for(uint32_t i = 0; i < 6; i++)
-  {
     specEntries += pipeInfo.shaders[i].specialization.size();
-    for(size_t s = 0; s < pipeInfo.shaders[i].specialization.size(); s++)
-      specSize += pipeInfo.shaders[i].specialization[s].data.size();
-  }
 
   specMapEntries.resize(specEntries);
-  specdata.resize(specSize);
+  specdata.resize(specEntries);
 
   VkSpecializationMapEntry *entry = specMapEntries.data();
+  uint64_t *data = specdata.data();
 
   uint32_t stageCount = 0;
-  specSize = 0;
+  uint32_t dataOffset = 0;
 
   // reserve space for spec constants
   for(uint32_t i = 0; i < 6; i++)
@@ -312,16 +336,15 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
         for(size_t s = 0; s < pipeInfo.shaders[i].specialization.size(); s++)
         {
           entry[s].constantID = pipeInfo.shaders[i].specialization[s].specID;
-          entry[s].size = pipeInfo.shaders[i].specialization[s].data.size();
-          entry[s].offset = (uint32_t)specSize;
+          entry[s].size = pipeInfo.shaders[i].specialization[s].dataSize;
+          entry[s].offset = dataOffset * sizeof(uint64_t);
 
-          specSize += entry[s].size;
+          data[dataOffset] = pipeInfo.shaders[i].specialization[s].value;
 
-          memcpy(&specdata[0] + entry[s].offset, pipeInfo.shaders[i].specialization[s].data.data(),
-                 entry[s].size);
+          dataOffset++;
         }
 
-        specInfo[i].dataSize = specdata.size();
+        specInfo[i].dataSize = specdata.size() * sizeof(uint64_t);
         specInfo[i].pData = specdata.data();
 
         entry += specInfo[i].mapEntryCount;
@@ -332,7 +355,10 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   }
 
   static VkPipelineVertexInputStateCreateInfo vi = {
-      VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO};
+      VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO,
+  };
+
+  vi.pNext = NULL;
 
   static VkVertexInputAttributeDescription viattr[128] = {};
   static VkVertexInputBindingDescription vibind[128] = {};
@@ -364,7 +390,7 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   };
   static VkVertexInputBindingDivisorDescriptionEXT vibindDivisors[128] = {};
 
-  if(m_pDriver->m_ExtensionsEnabled[VkCheckExt_EXT_vertex_divisor])
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_vertex_attribute_divisor)
   {
     vertexDivisor.pVertexBindingDivisors = vibindDivisors;
     vertexDivisor.vertexBindingDivisorCount = vi.vertexBindingDescriptionCount;
@@ -375,6 +401,7 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
       vibindDivisors[i].divisor = pipeInfo.vertexBindings[i].instanceDivisor;
     }
 
+    vertexDivisor.pNext = vi.pNext;
     vi.pNext = &vertexDivisor;
   }
 
@@ -390,8 +417,22 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   static VkPipelineTessellationStateCreateInfo tess = {
       VK_STRUCTURE_TYPE_PIPELINE_TESSELLATION_STATE_CREATE_INFO};
 
+  tess.pNext = NULL;
+
   tess.patchControlPoints = pipeInfo.patchControlPoints;
 
+  static VkPipelineTessellationDomainOriginStateCreateInfo tessDomain = {
+      VK_STRUCTURE_TYPE_PIPELINE_TESSELLATION_DOMAIN_ORIGIN_STATE_CREATE_INFO,
+  };
+
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_KHR_maintenance2)
+  {
+    tessDomain.domainOrigin = pipeInfo.tessellationDomainOrigin;
+
+    tessDomain.pNext = tess.pNext;
+    tess.pNext = &tessDomain;
+  }
+
   static VkPipelineViewportStateCreateInfo vp = {
       VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO};
 
@@ -415,8 +456,10 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
       VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO,
   };
 
+  rs.pNext = NULL;
+
   rs.depthClampEnable = pipeInfo.depthClampEnable;
-  rs.rasterizerDiscardEnable = pipeInfo.rasterizerDiscardEnable,
+  rs.rasterizerDiscardEnable = pipeInfo.rasterizerDiscardEnable;
   rs.polygonMode = pipeInfo.polygonMode;
   rs.cullMode = pipeInfo.cullMode;
   rs.frontFace = pipeInfo.frontFace;
@@ -430,15 +473,59 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
       VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_CONSERVATIVE_STATE_CREATE_INFO_EXT,
   };
 
-  if(m_pDriver->m_ExtensionsEnabled[VkCheckExt_EXT_conserv_rast])
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_conservative_rasterization)
   {
     conservRast.conservativeRasterizationMode = pipeInfo.conservativeRasterizationMode;
     conservRast.extraPrimitiveOverestimationSize = pipeInfo.extraPrimitiveOverestimationSize;
+
+    conservRast.pNext = rs.pNext;
     rs.pNext = &conservRast;
   }
 
+  static VkPipelineRasterizationStateStreamCreateInfoEXT rastStream = {
+      VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_STREAM_CREATE_INFO_EXT,
+  };
+
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_transform_feedback)
+  {
+    rastStream.rasterizationStream = pipeInfo.rasterizationStream;
+
+    rastStream.pNext = rs.pNext;
+    rs.pNext = &rastStream;
+  }
+
+  static VkPipelineRasterizationDepthClipStateCreateInfoEXT depthClipState = {
+      VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_DEPTH_CLIP_STATE_CREATE_INFO_EXT,
+  };
+
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_depth_clip_enable)
+  {
+    depthClipState.depthClipEnable = pipeInfo.depthClipEnable;
+
+    depthClipState.pNext = rs.pNext;
+    rs.pNext = &depthClipState;
+  }
+
+  static VkPipelineRasterizationLineStateCreateInfoEXT lineRasterState = {
+      VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_LINE_STATE_CREATE_INFO_EXT,
+  };
+
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_line_rasterization)
+  {
+    lineRasterState.lineRasterizationMode = pipeInfo.lineRasterMode;
+    lineRasterState.stippledLineEnable = pipeInfo.stippleEnabled;
+    lineRasterState.lineStippleFactor = pipeInfo.stippleFactor;
+    lineRasterState.lineStipplePattern = pipeInfo.stipplePattern;
+
+    lineRasterState.pNext = rs.pNext;
+    rs.pNext = &lineRasterState;
+  }
+
   static VkPipelineMultisampleStateCreateInfo msaa = {
-      VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO};
+      VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO,
+  };
+
+  msaa.pNext = NULL;
 
   msaa.rasterizationSamples = pipeInfo.rasterizationSamples;
   msaa.sampleShadingEnable = pipeInfo.sampleShadingEnable;
@@ -447,6 +534,23 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
   msaa.alphaToCoverageEnable = pipeInfo.alphaToCoverageEnable;
   msaa.alphaToOneEnable = pipeInfo.alphaToOneEnable;
 
+  static VkPipelineSampleLocationsStateCreateInfoEXT sampleLoc = {
+      VK_STRUCTURE_TYPE_PIPELINE_SAMPLE_LOCATIONS_STATE_CREATE_INFO_EXT,
+  };
+
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_sample_locations)
+  {
+    sampleLoc.sampleLocationsEnable = pipeInfo.sampleLocations.enabled;
+    sampleLoc.sampleLocationsInfo.sampleLocationGridSize = pipeInfo.sampleLocations.gridSize;
+    sampleLoc.sampleLocationsInfo.sampleLocationsPerPixel = pipeInfo.rasterizationSamples;
+    sampleLoc.sampleLocationsInfo.sampleLocationsCount =
+        (uint32_t)pipeInfo.sampleLocations.locations.size();
+    sampleLoc.sampleLocationsInfo.pSampleLocations = pipeInfo.sampleLocations.locations.data();
+
+    sampleLoc.pNext = msaa.pNext;
+    msaa.pNext = &sampleLoc;
+  }
+
   static VkPipelineDepthStencilStateCreateInfo ds = {
       VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO};
 
@@ -521,6 +625,20 @@ void VulkanShaderCache::MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &p
       0,                 // base pipeline index
   };
 
+  static VkPipelineDiscardRectangleStateCreateInfoEXT discardRects = {
+      VK_STRUCTURE_TYPE_PIPELINE_DISCARD_RECTANGLE_STATE_CREATE_INFO_EXT,
+  };
+
+  if(m_pDriver->GetExtensions(GetRecord(m_Device)).ext_EXT_discard_rectangles)
+  {
+    discardRects.discardRectangleMode = pipeInfo.discardMode;
+    discardRects.discardRectangleCount = (uint32_t)pipeInfo.discardRectangles.size();
+    discardRects.pDiscardRectangles = pipeInfo.discardRectangles.data();
+
+    discardRects.pNext = ret.pNext;
+    ret.pNext = &discardRects;
+  }
+
   // never create derivatives
   ret.flags &= ~VK_PIPELINE_CREATE_DERIVATIVE_BIT;
 
@@ -536,19 +654,17 @@ void VulkanShaderCache::MakeComputePipelineInfo(VkComputePipelineCreateInfo &pip
 
   VkPipelineShaderStageCreateInfo stage;    // Returned by value
   static VkSpecializationInfo specInfo;
-  static vector<VkSpecializationMapEntry> specMapEntries;
-  static std::vector<byte> specdata;
+  static rdcarray<VkSpecializationMapEntry> specMapEntries;
+
+  // the specialization constants can't use more than a uint64_t, so we just over-allocate
+  static rdcarray<uint64_t> specdata;
 
   const uint32_t i = 5;    // Compute stage
   RDCASSERT(pipeInfo.shaders[i].module != ResourceId());
 
   size_t specEntries = pipeInfo.shaders[i].specialization.size();
-  size_t specSize = 0;
-  for(size_t s = 0; s < pipeInfo.shaders[i].specialization.size(); s++)
-    specSize += pipeInfo.shaders[i].specialization[s].data.size();
-
-  specdata.resize(specSize);
 
+  specdata.resize(specEntries);
   specMapEntries.resize(specEntries);
   VkSpecializationMapEntry *entry = &specMapEntries[0];
 
@@ -560,7 +676,7 @@ void VulkanShaderCache::MakeComputePipelineInfo(VkComputePipelineCreateInfo &pip
   stage.pSpecializationInfo = NULL;
   stage.flags = VK_SHADER_STAGE_COMPUTE_BIT;
 
-  specSize = 0;
+  uint32_t dataOffset = 0;
 
   if(!pipeInfo.shaders[i].specialization.empty())
   {
@@ -571,13 +687,12 @@ void VulkanShaderCache::MakeComputePipelineInfo(VkComputePipelineCreateInfo &pip
     for(size_t s = 0; s < pipeInfo.shaders[i].specialization.size(); s++)
     {
       entry[s].constantID = pipeInfo.shaders[i].specialization[s].specID;
-      entry[s].size = pipeInfo.shaders[i].specialization[s].data.size();
-      entry[s].offset = (uint32_t)specSize;
+      entry[s].size = pipeInfo.shaders[i].specialization[s].dataSize;
+      entry[s].offset = dataOffset;
 
-      specSize += entry[s].size;
+      specdata[dataOffset] = pipeInfo.shaders[i].specialization[s].value;
 
-      memcpy(&specdata[0] + entry[s].offset, pipeInfo.shaders[i].specialization[s].data.data(),
-             entry[s].size);
+      dataOffset++;
     }
 
     specInfo.dataSize = specdata.size();
