@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,11 +24,11 @@
 
 #pragma once
 
-#include "api/replay/renderdoc_replay.h"
 #include "core/core.h"
+#include "driver/shaders/spirv/spirv_compile.h"
 #include "vk_core.h"
 
-typedef std::vector<uint32_t> *SPIRVBlob;
+typedef rdcarray<uint32_t> *SPIRVBlob;
 
 enum class BuiltinShader
 {
@@ -51,6 +51,9 @@ enum class BuiltinShader
   Array2MSCS,
   DepthMS2ArrayFS,
   DepthArray2MSFS,
+  TexRemapFloat,
+  TexRemapUInt,
+  TexRemapSInt,
   Count,
 };
 
@@ -62,8 +65,8 @@ public:
   VulkanShaderCache(WrappedVulkan *driver);
   ~VulkanShaderCache();
 
-  std::string GetSPIRVBlob(const SPIRVCompilationSettings &settings, const std::string &src,
-                           SPIRVBlob &outBlob);
+  rdcstr GetSPIRVBlob(const rdcspv::CompilationSettings &settings, const rdcstr &src,
+                      SPIRVBlob &outBlob);
 
   SPIRVBlob GetBuiltinBlob(BuiltinShader builtin) { return m_BuiltinShaderBlobs[(size_t)builtin]; }
   VkShaderModule GetBuiltinModule(BuiltinShader builtin)
@@ -74,6 +77,7 @@ public:
   void MakeGraphicsPipelineInfo(VkGraphicsPipelineCreateInfo &pipeCreateInfo, ResourceId pipeline);
   void MakeComputePipelineInfo(VkComputePipelineCreateInfo &pipeCreateInfo, ResourceId pipeline);
 
+  rdcstr GetGlobalDefines() { return m_GlobalDefines; }
   void SetCaching(bool enabled) { m_CacheShaders = enabled; }
 private:
   static const uint32_t m_ShaderCacheMagic = 0xf00d00d5;
@@ -82,6 +86,8 @@ private:
   WrappedVulkan *m_pDriver = NULL;
   VkDevice m_Device = VK_NULL_HANDLE;
 
+  rdcstr m_GlobalDefines;
+
   bool m_ShaderCacheDirty = false, m_CacheShaders = false;
   std::map<uint32_t, SPIRVBlob> m_ShaderCache;
 
