@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,61 +30,7 @@
 VulkanRenderState::VulkanRenderState(WrappedVulkan *driver, VulkanCreationInfo *createInfo)
     : m_CreationInfo(createInfo), m_pDriver(driver)
 {
-  compute.pipeline = graphics.pipeline = renderPass = framebuffer = ResourceId();
-  compute.descSets.clear();
-  graphics.descSets.clear();
-
-  views.clear();
-  scissors.clear();
-  lineWidth = 1.0f;
-  RDCEraseEl(bias);
-  RDCEraseEl(blendConst);
-  mindepth = 0.0f;
-  maxdepth = 1.0f;
-  RDCEraseEl(front);
-  RDCEraseEl(back);
-  RDCEraseEl(pushconsts);
-
-  renderPass = ResourceId();
-  subpass = 0;
-
-  RDCEraseEl(renderArea);
-
   RDCEraseEl(ibuffer);
-  vbuffers.clear();
-
-  RDCEraseEl(conditionalRendering);
-}
-
-VulkanRenderState &VulkanRenderState::operator=(const VulkanRenderState &o)
-{
-  views = o.views;
-  scissors = o.scissors;
-  lineWidth = o.lineWidth;
-  bias = o.bias;
-  memcpy(blendConst, o.blendConst, sizeof(blendConst));
-  mindepth = o.mindepth;
-  maxdepth = o.maxdepth;
-  front = o.front;
-  back = o.back;
-  memcpy(pushconsts, o.pushconsts, sizeof(pushconsts));
-  renderPass = o.renderPass;
-  subpass = o.subpass;
-  framebuffer = o.framebuffer;
-  renderArea = o.renderArea;
-
-  compute.pipeline = o.compute.pipeline;
-  compute.descSets = o.compute.descSets;
-
-  graphics.pipeline = o.graphics.pipeline;
-  graphics.descSets = o.graphics.descSets;
-
-  ibuffer = o.ibuffer;
-  vbuffers = o.vbuffers;
-
-  conditionalRendering = o.conditionalRendering;
-
-  return *this;
 }
 
 void VulkanRenderState::BeginRenderPassAndApplyState(VkCommandBuffer cmd, PipelineBinding binding)
@@ -99,61 +45,39 @@ void VulkanRenderState::BeginRenderPassAndApplyState(VkCommandBuffer cmd, Pipeli
 
   RDCASSERT(ARRAY_COUNT(empty) >= m_CreationInfo->m_RenderPass[renderPass].attachments.size());
 
+  VulkanCreationInfo::Framebuffer fbinfo = m_CreationInfo->m_Framebuffer[framebuffer];
+
   VkRenderPassBeginInfo rpbegin = {
       VK_STRUCTURE_TYPE_RENDER_PASS_BEGIN_INFO,
       NULL,
       Unwrap(m_CreationInfo->m_RenderPass[renderPass].loadRPs[subpass]),
-      Unwrap(m_CreationInfo->m_Framebuffer[framebuffer].loadFBs[subpass]),
+      Unwrap(fbinfo.loadFBs[subpass]),
       renderArea,
       (uint32_t)m_CreationInfo->m_RenderPass[renderPass].attachments.size(),
       empty,
   };
+
+  VkRenderPassAttachmentBeginInfo imagelessAttachments = {
+      VK_STRUCTURE_TYPE_RENDER_PASS_ATTACHMENT_BEGIN_INFO,
+  };
+  rdcarray<VkImageView> imagelessViews;
+
+  if(fbinfo.imageless)
+  {
+    rpbegin.pNext = &imagelessAttachments;
+    imagelessAttachments.attachmentCount = (uint32_t)fbattachments.size();
+
+    for(size_t i = 0; i < fbattachments.size(); i++)
+      imagelessViews.push_back(
+          Unwrap(GetResourceManager()->GetCurrentHandle<VkImageView>(fbattachments[i])));
+
+    imagelessAttachments.pAttachments = imagelessViews.data();
+  }
+
   ObjDisp(cmd)->CmdBeginRenderPass(Unwrap(cmd), &rpbegin, VK_SUBPASS_CONTENTS_INLINE);
 
   BindPipeline(cmd, binding, true);
 
-  if(ibuffer.buf != ResourceId())
-    ObjDisp(cmd)->CmdBindIndexBuffer(
-        Unwrap(cmd), Unwrap(GetResourceManager()->GetCurrentHandle<VkBuffer>(ibuffer.buf)),
-        ibuffer.offs, ibuffer.bytewidth == 4 ? VK_INDEX_TYPE_UINT32 : VK_INDEX_TYPE_UINT16);
-
-  for(size_t i = 0; i < vbuffers.size(); i++)
-  {
-    if(vbuffers[i].buf == ResourceId())
-      continue;
-
-    ObjDisp(cmd)->CmdBindVertexBuffers(
-        Unwrap(cmd), (uint32_t)i, 1,
-        UnwrapPtr(GetResourceManager()->GetCurrentHandle<VkBuffer>(vbuffers[i].buf)),
-        &vbuffers[i].offs);
-  }
-
-  for(size_t i = 0; i < xfbbuffers.size(); i++)
-  {
-    if(xfbbuffers[i].buf == ResourceId())
-      continue;
-
-    ObjDisp(cmd)->CmdBindTransformFeedbackBuffersEXT(
-        Unwrap(cmd), (uint32_t)i, 1,
-        UnwrapPtr(GetResourceManager()->GetCurrentHandle<VkBuffer>(xfbbuffers[i].buf)),
-        &xfbbuffers[i].offs, &xfbbuffers[i].size);
-  }
-
-  if(!xfbcounters.empty())
-  {
-    std::vector<VkBuffer> buffers;
-    std::vector<VkDeviceSize> offsets;
-
-    for(size_t i = 0; i < xfbcounters.size(); i++)
-    {
-      buffers.push_back(Unwrap(GetResourceManager()->GetCurrentHandle<VkBuffer>(xfbcounters[i].buf)));
-      offsets.push_back(xfbcounters[i].offs);
-    }
-
-    ObjDisp(cmd)->CmdBeginTransformFeedbackEXT(
-        Unwrap(cmd), firstxfbcounter, (uint32_t)xfbcounters.size(), buffers.data(), offsets.data());
-  }
-
   if(IsConditionalRenderingEnabled())
   {
     VkConditionalRenderingBeginInfoEXT beginInfo;
@@ -177,8 +101,8 @@ void VulkanRenderState::EndTransformFeedback(VkCommandBuffer cmd)
 {
   if(!xfbcounters.empty())
   {
-    std::vector<VkBuffer> buffers;
-    std::vector<VkDeviceSize> offsets;
+    rdcarray<VkBuffer> buffers;
+    rdcarray<VkDeviceSize> offsets;
 
     for(size_t i = 0; i < xfbcounters.size(); i++)
     {
@@ -216,7 +140,7 @@ void VulkanRenderState::BindPipeline(VkCommandBuffer cmd, PipelineBinding bindin
     ResourceId pipeLayoutId = m_CreationInfo->m_Pipeline[graphics.pipeline].layout;
     VkPipelineLayout layout = GetResourceManager()->GetCurrentHandle<VkPipelineLayout>(pipeLayoutId);
 
-    const vector<VkPushConstantRange> &pushRanges =
+    const rdcarray<VkPushConstantRange> &pushRanges =
         m_CreationInfo->m_PipelineLayout[pipeLayoutId].pushRanges;
 
     bool dynamicStates[VkDynamicCount] = {0};
@@ -276,82 +200,66 @@ void VulkanRenderState::BindPipeline(VkCommandBuffer cmd, PipelineBinding bindin
       ObjDisp(cmd)->CmdSetDiscardRectangleEXT(Unwrap(cmd), 0, (uint32_t)discardRectangles.size(),
                                               &discardRectangles[0]);
 
+    if(dynamicStates[VkDynamicLineStippleEXT])
+      ObjDisp(cmd)->CmdSetLineStippleEXT(Unwrap(cmd), stippleFactor, stipplePattern);
+
     // only set push constant ranges that the layout uses
     for(size_t i = 0; i < pushRanges.size(); i++)
       ObjDisp(cmd)->CmdPushConstants(Unwrap(cmd), Unwrap(layout), pushRanges[i].stageFlags,
                                      pushRanges[i].offset, pushRanges[i].size,
                                      pushconsts + pushRanges[i].offset);
 
-    const vector<ResourceId> &descSetLayouts =
-        m_CreationInfo->m_PipelineLayout[pipeLayoutId].descSetLayouts;
+    BindDescriptorSets(cmd, graphics, VK_PIPELINE_BIND_POINT_GRAPHICS);
 
-    // only iterate over the desc sets that this layout actually uses, not all that were bound
-    for(size_t i = 0; i < descSetLayouts.size(); i++)
+    if(ibuffer.buf != ResourceId())
     {
-      const DescSetLayout &descLayout = m_CreationInfo->m_DescSetLayout[descSetLayouts[i]];
+      VkIndexType type = VK_INDEX_TYPE_UINT16;
+      if(ibuffer.bytewidth == 4)
+        type = VK_INDEX_TYPE_UINT32;
+      else if(ibuffer.bytewidth == 1)
+        type = VK_INDEX_TYPE_UINT8_EXT;
 
-      if(i < graphics.descSets.size() && graphics.descSets[i].descSet != ResourceId())
+      ObjDisp(cmd)->CmdBindIndexBuffer(
+          Unwrap(cmd), Unwrap(GetResourceManager()->GetCurrentHandle<VkBuffer>(ibuffer.buf)),
+          ibuffer.offs, type);
+    }
+
+    for(size_t i = 0; i < vbuffers.size(); i++)
+    {
+      if(vbuffers[i].buf == ResourceId())
+        continue;
+
+      ObjDisp(cmd)->CmdBindVertexBuffers(
+          Unwrap(cmd), (uint32_t)i, 1,
+          UnwrapPtr(GetResourceManager()->GetCurrentHandle<VkBuffer>(vbuffers[i].buf)),
+          &vbuffers[i].offs);
+    }
+
+    for(size_t i = 0; i < xfbbuffers.size(); i++)
+    {
+      if(xfbbuffers[i].buf == ResourceId())
+        continue;
+
+      ObjDisp(cmd)->CmdBindTransformFeedbackBuffersEXT(
+          Unwrap(cmd), (uint32_t)i, 1,
+          UnwrapPtr(GetResourceManager()->GetCurrentHandle<VkBuffer>(xfbbuffers[i].buf)),
+          &xfbbuffers[i].offs, &xfbbuffers[i].size);
+    }
+
+    if(!xfbcounters.empty())
+    {
+      rdcarray<VkBuffer> buffers;
+      rdcarray<VkDeviceSize> offsets;
+
+      for(size_t i = 0; i < xfbcounters.size(); i++)
       {
-        // if we come to a descriptor set that isn't compatible, stop setting descriptor sets from
-        // here on.
-        // We can get into this situation if for example we have many sets bound at some point, then
-        // there's a pipeline change that causes most or all of them to be invalidated as
-        // incompatible, then the program only re-binds some subset that it knows is statically used
-        // by the next drawcall. The remaining sets are invalid, but also unused and this is
-        // explicitly allowed by the spec. We just have to make sure we don't try to actively bind
-        // an incompatible descriptor set.
-        ResourceId createdDescSetLayoutId =
-            m_pDriver->GetDescLayoutForDescSet(graphics.descSets[i].descSet);
-
-        if(descSetLayouts[i] != createdDescSetLayoutId)
-        {
-          const DescSetLayout &createdDescLayout =
-              m_CreationInfo->m_DescSetLayout[createdDescSetLayoutId];
-
-          if(descLayout != createdDescLayout)
-          {
-            // this set is incompatible, don't rebind it. Assume the application knows the shader
-            // doesn't need this set, and the binding is just stale
-            continue;
-          }
-        }
-
-        // if there are dynamic buffers, pass along the offsets
-
-        uint32_t *dynamicOffsets = NULL;
-
-        if(descLayout.dynamicCount > 0)
-        {
-          dynamicOffsets = &graphics.descSets[i].offsets[0];
-
-          if(graphics.descSets[i].offsets.size() < descLayout.dynamicCount)
-          {
-            dynamicOffsets = new uint32_t[descLayout.dynamicCount];
-            for(uint32_t o = 0; o < descLayout.dynamicCount; o++)
-            {
-              if(o < graphics.descSets[i].offsets.size())
-              {
-                dynamicOffsets[o] = graphics.descSets[i].offsets[o];
-              }
-              else
-              {
-                dynamicOffsets[o] = 0;
-                RDCWARN("Missing dynamic offset for set %u!", (uint32_t)i);
-              }
-            }
-          }
-        }
-
-        BindDescriptorSet(descLayout, cmd, layout, VK_PIPELINE_BIND_POINT_GRAPHICS, (uint32_t)i,
-                          dynamicOffsets);
-
-        if(graphics.descSets[i].offsets.size() < descLayout.dynamicCount)
-          SAFE_DELETE_ARRAY(dynamicOffsets);
-      }
-      else
-      {
-        RDCWARN("Descriptor set is not bound but pipeline layout expects one");
+        buffers.push_back(
+            Unwrap(GetResourceManager()->GetCurrentHandle<VkBuffer>(xfbcounters[i].buf)));
+        offsets.push_back(xfbcounters[i].offs);
       }
+
+      ObjDisp(cmd)->CmdBeginTransformFeedbackEXT(
+          Unwrap(cmd), firstxfbcounter, (uint32_t)xfbcounters.size(), buffers.data(), offsets.data());
     }
   }
 
@@ -364,7 +272,7 @@ void VulkanRenderState::BindPipeline(VkCommandBuffer cmd, PipelineBinding bindin
     ResourceId pipeLayoutId = m_CreationInfo->m_Pipeline[compute.pipeline].layout;
     VkPipelineLayout layout = GetResourceManager()->GetCurrentHandle<VkPipelineLayout>(pipeLayoutId);
 
-    const vector<VkPushConstantRange> &pushRanges =
+    const rdcarray<VkPushConstantRange> &pushRanges =
         m_CreationInfo->m_PipelineLayout[pipeLayoutId].pushRanges;
 
     // only set push constant ranges that the layout uses
@@ -373,58 +281,92 @@ void VulkanRenderState::BindPipeline(VkCommandBuffer cmd, PipelineBinding bindin
                                      pushRanges[i].offset, pushRanges[i].size,
                                      pushconsts + pushRanges[i].offset);
 
-    const vector<ResourceId> &descSetLayouts =
-        m_CreationInfo->m_PipelineLayout[pipeLayoutId].descSetLayouts;
+    BindDescriptorSets(cmd, compute, VK_PIPELINE_BIND_POINT_COMPUTE);
+  }
+}
 
-    for(size_t i = 0; i < descSetLayouts.size(); i++)
+void VulkanRenderState::BindDescriptorSets(VkCommandBuffer cmd, VulkanStatePipeline &pipe,
+                                           VkPipelineBindPoint bindPoint)
+{
+  ResourceId pipeLayoutId = m_CreationInfo->m_Pipeline[pipe.pipeline].layout;
+  const rdcarray<ResourceId> &descSetLayouts =
+      m_CreationInfo->m_PipelineLayout[pipeLayoutId].descSetLayouts;
+
+  for(size_t i = 0; i < descSetLayouts.size(); i++)
+  {
+    const DescSetLayout &descLayout = m_CreationInfo->m_DescSetLayout[descSetLayouts[i]];
+
+    if(i < pipe.descSets.size() && pipe.descSets[i].descSet != ResourceId())
     {
-      const DescSetLayout &descLayout = m_CreationInfo->m_DescSetLayout[descSetLayouts[i]];
+      // if we come to a descriptor set that isn't compatible, stop setting descriptor sets from
+      // here on.
+      // We can get into this situation if for example we have many sets bound at some point, then
+      // there's a pipeline change that causes most or all of them to be invalidated as
+      // incompatible, then the program only re-binds some subset that it knows is statically used
+      // by the next drawcall. The remaining sets are invalid, but also unused and this is
+      // explicitly allowed by the spec. We just have to make sure we don't try to actively bind
+      // an incompatible descriptor set.
+      ResourceId createdDescSetLayoutId =
+          m_pDriver->GetDescLayoutForDescSet(pipe.descSets[i].descSet);
 
-      if(i < compute.descSets.size() && compute.descSets[i].descSet != ResourceId())
+      if(descSetLayouts[i] != createdDescSetLayoutId)
       {
-        // if there are dynamic buffers, pass along the offsets
+        const DescSetLayout &createdDescLayout =
+            m_CreationInfo->m_DescSetLayout[createdDescSetLayoutId];
 
-        uint32_t *dynamicOffsets = NULL;
-
-        if(descLayout.dynamicCount > 0)
+        if(descLayout != createdDescLayout)
         {
-          dynamicOffsets = &compute.descSets[i].offsets[0];
+          // this set is incompatible, don't rebind it. Assume the application knows the shader
+          // doesn't need this set, and the binding is just stale
+          continue;
+        }
+      }
 
-          if(compute.descSets[i].offsets.size() < descLayout.dynamicCount)
+      // if there are dynamic buffers, pass along the offsets
+
+      uint32_t *dynamicOffsets = NULL;
+
+      if(descLayout.dynamicCount > 0)
+      {
+        dynamicOffsets = &pipe.descSets[i].offsets[0];
+
+        if(pipe.descSets[i].offsets.size() < descLayout.dynamicCount)
+        {
+          dynamicOffsets = new uint32_t[descLayout.dynamicCount];
+          for(uint32_t o = 0; o < descLayout.dynamicCount; o++)
           {
-            dynamicOffsets = new uint32_t[descLayout.dynamicCount];
-            for(uint32_t o = 0; o < descLayout.dynamicCount; o++)
+            if(o < pipe.descSets[i].offsets.size())
             {
-              if(o < compute.descSets[i].offsets.size())
-              {
-                dynamicOffsets[o] = compute.descSets[i].offsets[o];
-              }
-              else
-              {
-                dynamicOffsets[o] = 0;
-                RDCWARN("Missing dynamic offset for set %u!", (uint32_t)i);
-              }
+              dynamicOffsets[o] = pipe.descSets[i].offsets[o];
+            }
+            else
+            {
+              dynamicOffsets[o] = 0;
+              RDCWARN("Missing dynamic offset for set %u!", (uint32_t)i);
             }
           }
         }
-
-        BindDescriptorSet(descLayout, cmd, layout, VK_PIPELINE_BIND_POINT_COMPUTE, (uint32_t)i,
-                          dynamicOffsets);
-
-        if(compute.descSets[i].offsets.size() < descLayout.dynamicCount)
-          SAFE_DELETE_ARRAY(dynamicOffsets);
       }
+
+      BindDescriptorSet(descLayout, cmd, bindPoint, (uint32_t)i, dynamicOffsets);
+
+      if(pipe.descSets[i].offsets.size() < descLayout.dynamicCount)
+        SAFE_DELETE_ARRAY(dynamicOffsets);
     }
   }
 }
 
 void VulkanRenderState::BindDescriptorSet(const DescSetLayout &descLayout, VkCommandBuffer cmd,
-                                          VkPipelineLayout layout, VkPipelineBindPoint bindPoint,
-                                          uint32_t setIndex, uint32_t *dynamicOffsets)
+                                          VkPipelineBindPoint bindPoint, uint32_t setIndex,
+                                          uint32_t *dynamicOffsets)
 {
   ResourceId descSet = (bindPoint == VK_PIPELINE_BIND_POINT_GRAPHICS)
                            ? graphics.descSets[setIndex].descSet
                            : compute.descSets[setIndex].descSet;
+  ResourceId pipeLayout = (bindPoint == VK_PIPELINE_BIND_POINT_GRAPHICS)
+                              ? graphics.descSets[setIndex].pipeLayout
+                              : compute.descSets[setIndex].pipeLayout;
+  VkPipelineLayout layout = GetResourceManager()->GetCurrentHandle<VkPipelineLayout>(pipeLayout);
 
   if((descLayout.flags & VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR) == 0)
   {
@@ -437,24 +379,30 @@ void VulkanRenderState::BindDescriptorSet(const DescSetLayout &descLayout, VkCom
   {
     // this isn't a real descriptor set, it's a push descriptor, so we need to push the
     // current state.
-    std::vector<VkWriteDescriptorSet> writes;
+    rdcarray<VkWriteDescriptorSet> writes;
+
+    // any allocated arrays
+    rdcarray<VkDescriptorImageInfo *> allocImgWrites;
+    rdcarray<VkDescriptorBufferInfo *> allocBufWrites;
+    rdcarray<VkBufferView *> allocBufViewWrites;
 
     WrappedVulkan::DescriptorSetInfo &setInfo = m_pDriver->m_DescriptorSetState[descSet];
 
+    VkWriteDescriptorSet push = {VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET};
+
     for(size_t b = 0; b < descLayout.bindings.size(); b++)
     {
-      VkWriteDescriptorSet push = {};
+      const DescSetLayout::Binding &bind = descLayout.bindings[b];
 
       // skip if this binding isn't used
-      if(descLayout.bindings[b].descriptorCount == 0 || descLayout.bindings[b].stageFlags == 0)
+      if(bind.descriptorCount == 0 || bind.stageFlags == 0)
         continue;
 
-      push.sType = VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET;
       // push.dstSet; // unused for push descriptors
       push.dstBinding = (uint32_t)b;
       push.dstArrayElement = 0;
-      push.descriptorType = descLayout.bindings[b].descriptorType;
-      push.descriptorCount = descLayout.bindings[b].descriptorCount;
+      push.descriptorType = bind.descriptorType;
+      push.descriptorCount = bind.descriptorCount;
 
       DescriptorSetSlot *slots = setInfo.currentBindings[b];
 
@@ -464,9 +412,11 @@ void VulkanRenderState::BindDescriptorSet(const DescSetLayout &descLayout, VkCom
         VkBufferView *dst = new VkBufferView[push.descriptorCount];
 
         for(uint32_t a = 0; a < push.descriptorCount; a++)
-          dst[a] = Unwrap(slots[a].texelBufferView);
+          dst[a] =
+              Unwrap(GetResourceManager()->GetCurrentHandle<VkBufferView>(slots[a].texelBufferView));
 
         push.pTexelBufferView = dst;
+        allocBufViewWrites.push_back(dst);
       }
       else if(push.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER ||
               push.descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER ||
@@ -478,12 +428,16 @@ void VulkanRenderState::BindDescriptorSet(const DescSetLayout &descLayout, VkCom
 
         for(uint32_t a = 0; a < push.descriptorCount; a++)
         {
-          dst[a] = slots[a].imageInfo;
-          dst[a].sampler = Unwrap(dst[a].sampler);
-          dst[a].imageView = Unwrap(dst[a].imageView);
+          const DescriptorSetSlotImageInfo &src = slots[a].imageInfo;
+
+          dst[a].imageLayout = src.imageLayout;
+          dst[a].sampler = Unwrap(GetResourceManager()->GetCurrentHandle<VkSampler>(src.sampler));
+          dst[a].imageView =
+              Unwrap(GetResourceManager()->GetCurrentHandle<VkImageView>(src.imageView));
         }
 
         push.pImageInfo = dst;
+        allocImgWrites.push_back(dst);
       }
       else
       {
@@ -491,26 +445,90 @@ void VulkanRenderState::BindDescriptorSet(const DescSetLayout &descLayout, VkCom
 
         for(uint32_t a = 0; a < push.descriptorCount; a++)
         {
-          dst[a] = slots[a].bufferInfo;
-          dst[a].buffer = Unwrap(dst[a].buffer);
+          const DescriptorSetSlotBufferInfo &src = slots[a].bufferInfo;
+
+          dst[a].offset = src.offset;
+          dst[a].range = src.range;
+          dst[a].buffer = Unwrap(GetResourceManager()->GetCurrentHandle<VkBuffer>(src.buffer));
         }
 
         push.pBufferInfo = dst;
+        allocBufWrites.push_back(dst);
       }
 
-      writes.push_back(push);
+      // start with no descriptors
+      push.descriptorCount = 0;
+
+      for(uint32_t w = 0; w < bind.descriptorCount; w++)
+      {
+        // if this push is valid, we increment the descriptor count and continue
+        if(IsValid(push, w - push.dstArrayElement))
+        {
+          push.descriptorCount++;
+        }
+        else
+        {
+          // if this push isn't valid, then we first check to see if we had any previous
+          // pending pushs in the array we were going to batch together, if so we add them.
+          if(push.descriptorCount > 0)
+            writes.push_back(push);
+
+          // skip past any previous descriptors we just wrote, as well as the current invalid
+          // one
+          if(push.pBufferInfo)
+            push.pBufferInfo += push.descriptorCount + 1;
+          if(push.pImageInfo)
+            push.pImageInfo += push.descriptorCount + 1;
+          if(push.pTexelBufferView)
+            push.pTexelBufferView += push.descriptorCount + 1;
+
+          // now start again from 0 descriptors, at the next array element
+          push.dstArrayElement += push.descriptorCount + 1;
+          push.descriptorCount = 0;
+        }
+      }
+
+      // if there are any left, add them here
+      if(push.descriptorCount > 0)
+        writes.push_back(push);
+
+      // don't leak the arrays and cause double deletes, NULL them after each time
+      push.pImageInfo = NULL;
+      push.pBufferInfo = NULL;
+      push.pTexelBufferView = NULL;
     }
 
     ObjDisp(cmd)->CmdPushDescriptorSetKHR(Unwrap(cmd), bindPoint, Unwrap(layout), setIndex,
                                           (uint32_t)writes.size(), writes.data());
 
-    // delete dynamically allocated arrays
-    for(VkWriteDescriptorSet &push : writes)
-    {
-      SAFE_DELETE_ARRAY(push.pImageInfo);
-      SAFE_DELETE_ARRAY(push.pBufferInfo);
-      SAFE_DELETE_ARRAY(push.pTexelBufferView);
-    }
+    // delete allocated arrays for descriptor writes
+    for(VkDescriptorBufferInfo *a : allocBufWrites)
+      delete[] a;
+    for(VkDescriptorImageInfo *a : allocImgWrites)
+      delete[] a;
+    for(VkBufferView *a : allocBufViewWrites)
+      delete[] a;
+  }
+}
+
+void VulkanRenderState::SetFramebuffer(ResourceId fb,
+                                       const VkRenderPassAttachmentBeginInfo *attachmentsInfo)
+{
+  framebuffer = fb;
+
+  VulkanCreationInfo::Framebuffer fbinfo = m_CreationInfo->m_Framebuffer[fb];
+
+  fbattachments.resize(fbinfo.attachments.size());
+
+  if(!fbinfo.imageless)
+  {
+    for(size_t i = 0; i < fbinfo.attachments.size(); i++)
+      fbattachments[i] = fbinfo.attachments[i].createdView;
+  }
+  else
+  {
+    for(size_t i = 0; i < fbinfo.attachments.size(); i++)
+      fbattachments[i] = GetResID(attachmentsInfo->pAttachments[i]);
   }
 }
 
