@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,6 @@
 
 #pragma once
 
-#include <vector>
 #include "vk_common.h"
 
 struct VulkanCreationInfo;
@@ -38,10 +37,11 @@ struct VulkanStatePipeline
 
   struct DescriptorAndOffsets
   {
+    ResourceId pipeLayout;
     ResourceId descSet;
-    vector<uint32_t> offsets;
+    rdcarray<uint32_t> offsets;
   };
-  vector<DescriptorAndOffsets> descSets;
+  rdcarray<DescriptorAndOffsets> descSets;
 };
 
 struct VulkanRenderState
@@ -54,7 +54,6 @@ struct VulkanRenderState
   };
 
   VulkanRenderState(WrappedVulkan *driver, VulkanCreationInfo *createInfo);
-  VulkanRenderState &operator=(const VulkanRenderState &o);
   void BeginRenderPassAndApplyState(VkCommandBuffer cmd, PipelineBinding binding);
   void EndRenderPass(VkCommandBuffer cmd);
 
@@ -63,91 +62,115 @@ struct VulkanRenderState
   void EndConditionalRendering(VkCommandBuffer cmd);
 
   void BindPipeline(VkCommandBuffer cmd, PipelineBinding binding, bool subpass0);
+
+  void BindDescriptorSets(VkCommandBuffer cmd, VulkanStatePipeline &pipe,
+                          VkPipelineBindPoint bindPoint);
+
   void BindDescriptorSet(const DescSetLayout &descLayout, VkCommandBuffer cmd,
-                         VkPipelineLayout layout, VkPipelineBindPoint bindPoint, uint32_t setIndex,
-                         uint32_t *dynamicOffsets);
+                         VkPipelineBindPoint bindPoint, uint32_t setIndex, uint32_t *dynamicOffsets);
 
   bool IsConditionalRenderingEnabled();
 
   // dynamic state
-  vector<VkViewport> views;
-  vector<VkRect2D> scissors;
-  float lineWidth;
+  rdcarray<VkViewport> views;
+  rdcarray<VkRect2D> scissors;
+  float lineWidth = 1.0f;
   struct
   {
-    float depth, biasclamp, slope;
+    float depth = 0.0f;
+    float biasclamp = 0.0f;
+    float slope = 0.0f;
   } bias;
-  float blendConst[4];
-  float mindepth, maxdepth;
+  float blendConst[4] = {};
+  float mindepth = 0.0f;
+  float maxdepth = 1.0f;
   struct
   {
-    uint32_t compare, write, ref;
+    uint32_t compare = 0;
+    uint32_t write = 0;
+    uint32_t ref = 0;
   } front, back;
 
   struct
   {
     VkSampleCountFlagBits sampleCount;
     VkExtent2D gridSize;
-    std::vector<VkSampleLocationEXT> locations;
+    rdcarray<VkSampleLocationEXT> locations;
   } sampleLocations;
 
-  std::vector<VkRect2D> discardRectangles;
+  rdcarray<VkRect2D> discardRectangles;
+
+  uint32_t stippleFactor = 0;
+  uint16_t stipplePattern = 0;
 
   // this should be big enough for any implementation
-  byte pushconsts[1024];
+  byte pushconsts[1024] = {};
   // the actual number of bytes that have been uploaded
   uint32_t pushConstSize = 0;
 
   ResourceId renderPass;
-  uint32_t subpass;
+  uint32_t subpass = 0;
 
-  ResourceId framebuffer;
-  VkRect2D renderArea;
+  // framebuffer accessors - to allow for imageless framebuffers and prevent accidentally changing
+  // only the framebuffer without updating the attachments
+  void SetFramebuffer(ResourceId fb, const VkRenderPassAttachmentBeginInfo *attachmentsInfo = NULL);
+  void SetFramebuffer(ResourceId fb, const rdcarray<ResourceId> &dynamicAttachments)
+  {
+    framebuffer = fb;
+    fbattachments = dynamicAttachments;
+  }
+  ResourceId GetFramebuffer() const { return framebuffer; }
+  const rdcarray<ResourceId> &GetFramebufferAttachments() const { return fbattachments; }
+  //
+
+  VkRect2D renderArea = {};
 
   VulkanStatePipeline compute, graphics;
 
   struct IdxBuffer
   {
     ResourceId buf;
-    VkDeviceSize offs;
-    int bytewidth;
+    VkDeviceSize offs = 0;
+    int bytewidth = 0;
   } ibuffer;
 
   struct VertBuffer
   {
     ResourceId buf;
-    VkDeviceSize offs;
+    VkDeviceSize offs = 0;
   };
-  vector<VertBuffer> vbuffers;
+  rdcarray<VertBuffer> vbuffers;
 
   struct XFBBuffer
   {
     ResourceId buf;
-    VkDeviceSize offs;
-    VkDeviceSize size;
+    VkDeviceSize offs = 0;
+    VkDeviceSize size = 0;
   };
-  vector<XFBBuffer> xfbbuffers;
+  rdcarray<XFBBuffer> xfbbuffers;
 
   struct XFBCounter
   {
     ResourceId buf;
-    VkDeviceSize offs;
+    VkDeviceSize offs = 0;
   };
   uint32_t firstxfbcounter = 0;
-  vector<XFBCounter> xfbcounters;
+  rdcarray<XFBCounter> xfbcounters;
 
   struct ConditionalRendering
   {
     ResourceId buffer;
-    VkDeviceSize offset;
-    VkConditionalRenderingFlagsEXT flags;
+    VkDeviceSize offset = 0;
+    VkConditionalRenderingFlagsEXT flags = 0;
 
-    bool forceDisable;
+    bool forceDisable = false;
   } conditionalRendering;
 
   VulkanResourceManager *GetResourceManager();
   VulkanCreationInfo *m_CreationInfo;
   WrappedVulkan *m_pDriver;
 
-  std::vector<VkImageMemoryBarrier> rpBarriers;
+private:
+  ResourceId framebuffer;
+  rdcarray<ResourceId> fbattachments;
 };
