@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -45,7 +45,9 @@ VkDescriptorSetLayoutCreateInfo WrappedVulkan::UnwrapInfo(const VkDescriptorSetL
   {
     unwrapped[i] = info->pBindings[i];
 
-    if(unwrapped[i].pImmutableSamplers)
+    if((unwrapped[i].descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER ||
+        unwrapped[i].descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER) &&
+       unwrapped[i].pImmutableSamplers)
     {
       VkSampler *unwrappedSamplers = nextSampler;
       nextSampler += unwrapped[i].descriptorCount;
@@ -81,8 +83,10 @@ VkDescriptorUpdateTemplateCreateInfo WrappedVulkan::UnwrapInfo(
 {
   VkDescriptorUpdateTemplateCreateInfo ret = *info;
 
-  ret.pipelineLayout = Unwrap(ret.pipelineLayout);
-  ret.descriptorSetLayout = Unwrap(ret.descriptorSetLayout);
+  if(ret.templateType == VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_PUSH_DESCRIPTORS_KHR)
+    ret.pipelineLayout = Unwrap(ret.pipelineLayout);
+  if(ret.templateType == VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_DESCRIPTOR_SET)
+    ret.descriptorSetLayout = Unwrap(ret.descriptorSetLayout);
 
   return ret;
 }
@@ -176,7 +180,8 @@ bool WrappedVulkan::Serialise_vkCreateDescriptorPool(SerialiserType &ser, VkDevi
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(DescriptorPool, GetResID(*pDescriptorPool)).TypedAs("VkDescriptorPool");
+  SERIALISE_ELEMENT_LOCAL(DescriptorPool, GetResID(*pDescriptorPool))
+      .TypedAs("VkDescriptorPool"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -252,7 +257,7 @@ bool WrappedVulkan::Serialise_vkCreateDescriptorSetLayout(
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(SetLayout, GetResID(*pSetLayout)).TypedAs("VkDescriptorSetLayout");
+  SERIALISE_ELEMENT_LOCAL(SetLayout, GetResID(*pSetLayout)).TypedAs("VkDescriptorSetLayout"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -378,7 +383,7 @@ bool WrappedVulkan::Serialise_vkAllocateDescriptorSets(SerialiserType &ser, VkDe
 {
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(AllocateInfo, *pAllocateInfo);
-  SERIALISE_ELEMENT_LOCAL(DescriptorSet, GetResID(*pDescriptorSets)).TypedAs("VkDescriptorSet");
+  SERIALISE_ELEMENT_LOCAL(DescriptorSet, GetResID(*pDescriptorSets)).TypedAs("VkDescriptorSet"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -392,9 +397,10 @@ bool WrappedVulkan::Serialise_vkAllocateDescriptorSets(SerialiserType &ser, VkDe
     if(ret != VK_SUCCESS)
     {
       RDCWARN(
-          "Failed to allocate descriptor set %llu from pool %llu on replay. Assuming pool was "
+          "Failed to allocate descriptor set %s from pool %s on replay. Assuming pool was "
           "reset and re-used mid-capture, so overflowing.",
-          DescriptorSet, GetResourceManager()->GetOriginalID(GetResID(AllocateInfo.descriptorPool)));
+          ToStr(DescriptorSet).c_str(),
+          ToStr(GetResourceManager()->GetOriginalID(GetResID(AllocateInfo.descriptorPool))).c_str());
 
       VulkanCreationInfo::DescSetPool &poolInfo =
           m_CreationInfo.m_DescSetPool[GetResID(AllocateInfo.descriptorPool)];
@@ -506,21 +512,11 @@ VkResult WrappedVulkan::vkAllocateDescriptorSets(VkDevice device,
       record->AddParent(poolrecord);
       record->AddParent(GetResourceManager()->GetResourceRecord(layoutID));
 
-      bool capframe = false;
-      // just always treat descriptor sets as dirty
-      {
-        SCOPED_LOCK(m_CapTransitionLock);
-        capframe = IsActiveCapturing(m_State);
-      }
-
       // only mark descriptor set as dirty if it's not a push descriptor layout
       if((layoutRecord->descInfo->layout->flags &
           VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR) == 0)
       {
-        if(capframe)
-          GetResourceManager()->MarkPendingDirty(id);
-        else
-          GetResourceManager()->MarkDirtyResource(id);
+        GetResourceManager()->MarkDirtyResource(id);
       }
 
       record->descInfo = new DescriptorSetData();
@@ -587,6 +583,10 @@ void WrappedVulkan::ReplayDescriptorSetWrite(VkDevice device, const VkWriteDescr
   if(!valid)
     return;
 
+  // ignore empty writes, for some reason this is valid with descriptor update templates.
+  if(writeDesc.descriptorCount == 0)
+    return;
+
   const DescSetLayout &layout =
       m_CreationInfo.m_DescSetLayout[m_DescriptorSetState[GetResID(writeDesc.dstSet)].layout];
 
@@ -653,7 +653,7 @@ void WrappedVulkan::ReplayDescriptorSetWrite(VkDevice device, const VkWriteDescr
     ObjDisp(device)->UpdateDescriptorSets(Unwrap(device), 1, &unwrapped, 0, NULL);
 
     // update our local tracking
-    std::vector<DescriptorSetSlot *> &bindings =
+    rdcarray<DescriptorSetSlot *> &bindings =
         m_DescriptorSetState[GetResID(writeDesc.dstSet)].currentBindings;
 
     {
@@ -677,7 +677,7 @@ void WrappedVulkan::ReplayDescriptorSetWrite(VkDevice device, const VkWriteDescr
             curIdx = 0;
           }
 
-          (*bind)[curIdx].texelBufferView = writeDesc.pTexelBufferView[d];
+          (*bind)[curIdx].texelBufferView = GetResID(writeDesc.pTexelBufferView[d]);
         }
       }
       else if(writeDesc.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER ||
@@ -697,7 +697,18 @@ void WrappedVulkan::ReplayDescriptorSetWrite(VkDevice device, const VkWriteDescr
             curIdx = 0;
           }
 
-          (*bind)[curIdx].imageInfo = writeDesc.pImageInfo[d];
+          bool sampler = true;
+          bool imageView = true;
+
+          // ignore descriptors not part of the write, as they might not even point to a valid
+          // object so trying to get their ID could crash
+          if(layoutBinding->immutableSampler ||
+             writeDesc.descriptorType != VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER)
+            sampler = false;
+          if(writeDesc.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER)
+            imageView = false;
+
+          (*bind)[curIdx].imageInfo.SetFrom(writeDesc.pImageInfo[d], sampler, imageView);
         }
       }
       else
@@ -713,7 +724,7 @@ void WrappedVulkan::ReplayDescriptorSetWrite(VkDevice device, const VkWriteDescr
             curIdx = 0;
           }
 
-          (*bind)[curIdx].bufferInfo = writeDesc.pBufferInfo[d];
+          (*bind)[curIdx].bufferInfo.SetFrom(writeDesc.pBufferInfo[d]);
         }
       }
     }
@@ -733,8 +744,8 @@ void WrappedVulkan::ReplayDescriptorSetCopy(VkDevice device, const VkCopyDescrip
   ResourceId srcSetId = GetResID(copyDesc.srcSet);
 
   // update our local tracking
-  std::vector<DescriptorSetSlot *> &dstbindings = m_DescriptorSetState[dstSetId].currentBindings;
-  std::vector<DescriptorSetSlot *> &srcbindings = m_DescriptorSetState[srcSetId].currentBindings;
+  rdcarray<DescriptorSetSlot *> &dstbindings = m_DescriptorSetState[dstSetId].currentBindings;
+  rdcarray<DescriptorSetSlot *> &srcbindings = m_DescriptorSetState[srcSetId].currentBindings;
 
   {
     RDCASSERT(copyDesc.dstBinding < dstbindings.size());
@@ -838,6 +849,31 @@ void WrappedVulkan::vkUpdateDescriptorSets(VkDevice device, uint32_t writeCount,
     for(uint32_t i = 0; i < writeCount; i++)
     {
       unwrappedWrites[i] = pDescriptorWrites[i];
+
+      bool hasImmutable = false;
+
+      if(IsCaptureMode(m_State))
+      {
+        VkResourceRecord *record = GetRecord(unwrappedWrites[i].dstSet);
+        RDCASSERT(record->descInfo && record->descInfo->layout);
+        const DescSetLayout &layout = *record->descInfo->layout;
+
+        RDCASSERT(unwrappedWrites[i].dstBinding < record->descInfo->descBindings.size());
+        const DescSetLayout::Binding *layoutBinding = &layout.bindings[unwrappedWrites[i].dstBinding];
+
+        hasImmutable = layoutBinding->immutableSampler != NULL;
+      }
+      else
+      {
+        const DescSetLayout &layout =
+            m_CreationInfo
+                .m_DescSetLayout[m_DescriptorSetState[GetResID(unwrappedWrites[i].dstSet)].layout];
+
+        const DescSetLayout::Binding *layoutBinding = &layout.bindings[unwrappedWrites[i].dstBinding];
+
+        hasImmutable = layoutBinding->immutableSampler != NULL;
+      }
+
       unwrappedWrites[i].dstSet = Unwrap(unwrappedWrites[i].dstSet);
 
       VkDescriptorBufferInfo *bufInfos = nextDescriptors;
@@ -866,7 +902,8 @@ void WrappedVulkan::vkUpdateDescriptorSets(VkDevice device, uint32_t writeCount,
       {
         bool hasSampler =
             (pDescriptorWrites[i].descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER ||
-             pDescriptorWrites[i].descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER);
+             pDescriptorWrites[i].descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER) &&
+            !hasImmutable;
         bool hasImage =
             (pDescriptorWrites[i].descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER ||
              pDescriptorWrites[i].descriptorType == VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE ||
@@ -906,68 +943,69 @@ void WrappedVulkan::vkUpdateDescriptorSets(VkDevice device, uint32_t writeCount,
         Unwrap(device), writeCount, unwrappedWrites, copyCount, unwrappedCopies));
   }
 
-  bool capframe = false;
   {
-    SCOPED_LOCK(m_CapTransitionLock);
-    capframe = IsActiveCapturing(m_State);
-  }
-
-  if(capframe)
-  {
-    // don't have to mark referenced any of the resources pointed to by the descriptor set - that's
-    // handled on queue submission by marking ref'd all the current bindings of the sets referenced
-    // by the cmd buffer
+    SCOPED_READLOCK(m_CapTransitionLock);
 
+    if(IsActiveCapturing(m_State))
     {
-      CACHE_THREAD_SERIALISER();
+      // don't have to mark referenced any of the resources pointed to by the descriptor set -
+      // that's
+      // handled on queue submission by marking ref'd all the current bindings of the sets
+      // referenced
+      // by the cmd buffer
 
-      SCOPED_SERIALISE_CHUNK(VulkanChunk::vkUpdateDescriptorSets);
-      Serialise_vkUpdateDescriptorSets(ser, device, writeCount, pDescriptorWrites, copyCount,
-                                       pDescriptorCopies);
-
-      m_FrameCaptureRecord->AddChunk(scope.Get());
-    }
-
-    // previously we would not mark descriptor set destinations as ref'd here. This is because all
-    // descriptor sets are implicitly dirty and they're only actually *needed* when bound - we can
-    // safely skip any updates of unused descriptor sets. However for consistency with template
-    // updates below, we pull them in here even if they won't technically be needed.
-
-    for(uint32_t i = 0; i < writeCount; i++)
-    {
-      GetResourceManager()->MarkResourceFrameReferenced(GetResID(pDescriptorWrites[i].dstSet),
-                                                        eFrameRef_PartialWrite);
-    }
-
-    for(uint32_t i = 0; i < copyCount; i++)
-    {
-      // At the same time as ref'ing the source set, we must ref all of its resources (via the
-      // bindFrameRefs). This is because they must be valid even if the source set is not ever bound
-      // (and so its bindings aren't pulled in).
-      //
-      // We just ref all rather than looking at only the copied sets to keep things simple.
-      // This does mean a slightly conservative ref'ing if the dest set doesn't end up getting
-      // bound, but we only do this during frame capture so it's not too bad.
-
-      GetResourceManager()->MarkResourceFrameReferenced(GetResID(pDescriptorCopies[i].dstSet),
-                                                        eFrameRef_PartialWrite);
-      GetResourceManager()->MarkResourceFrameReferenced(GetResID(pDescriptorCopies[i].srcSet),
-                                                        eFrameRef_Read);
-
-      VkResourceRecord *setrecord = GetRecord(pDescriptorCopies[i].srcSet);
-
-      SCOPED_LOCK(setrecord->descInfo->refLock);
-
-      for(auto refit = setrecord->descInfo->bindFrameRefs.begin();
-          refit != setrecord->descInfo->bindFrameRefs.end(); ++refit)
       {
-        GetResourceManager()->MarkResourceFrameReferenced(refit->first, refit->second.second);
+        CACHE_THREAD_SERIALISER();
 
-        if(refit->second.first & DescriptorSetData::SPARSE_REF_BIT)
+        SCOPED_SERIALISE_CHUNK(VulkanChunk::vkUpdateDescriptorSets);
+        Serialise_vkUpdateDescriptorSets(ser, device, writeCount, pDescriptorWrites, copyCount,
+                                         pDescriptorCopies);
+
+        m_FrameCaptureRecord->AddChunk(scope.Get());
+      }
+
+      // previously we would not mark descriptor set destinations as ref'd here. This is because all
+      // descriptor sets are implicitly dirty and they're only actually *needed* when bound - we can
+      // safely skip any updates of unused descriptor sets. However for consistency with template
+      // updates below, we pull them in here even if they won't technically be needed.
+
+      for(uint32_t i = 0; i < writeCount; i++)
+      {
+        GetResourceManager()->MarkResourceFrameReferenced(GetResID(pDescriptorWrites[i].dstSet),
+                                                          eFrameRef_PartialWrite);
+      }
+
+      for(uint32_t i = 0; i < copyCount; i++)
+      {
+        // At the same time as ref'ing the source set, we must ref all of its resources (via the
+        // bindFrameRefs). This is because they must be valid even if the source set is not ever
+        // bound
+        // (and so its bindings aren't pulled in).
+        //
+        // We just ref all rather than looking at only the copied sets to keep things simple.
+        // This does mean a slightly conservative ref'ing if the dest set doesn't end up getting
+        // bound, but we only do this during frame capture so it's not too bad.
+
+        GetResourceManager()->MarkResourceFrameReferenced(GetResID(pDescriptorCopies[i].dstSet),
+                                                          eFrameRef_PartialWrite);
+        GetResourceManager()->MarkResourceFrameReferenced(GetResID(pDescriptorCopies[i].srcSet),
+                                                          eFrameRef_Read);
+
+        VkResourceRecord *setrecord = GetRecord(pDescriptorCopies[i].srcSet);
+
+        SCOPED_LOCK(setrecord->descInfo->refLock);
+
+        for(auto refit = setrecord->descInfo->bindFrameRefs.begin();
+            refit != setrecord->descInfo->bindFrameRefs.end(); ++refit)
         {
-          VkResourceRecord *record = GetResourceManager()->GetResourceRecord(refit->first);
+          GetResourceManager()->MarkResourceFrameReferenced(refit->first, refit->second.second);
 
-          GetResourceManager()->MarkSparseMapReferenced(record->resInfo);
+          if(refit->second.first & DescriptorSetData::SPARSE_REF_BIT)
+          {
+            VkResourceRecord *record = GetResourceManager()->GetResourceRecord(refit->first);
+
+            GetResourceManager()->MarkSparseMapReferenced(record->resInfo);
+          }
         }
       }
     }
@@ -1032,12 +1070,12 @@ void WrappedVulkan::vkUpdateDescriptorSets(VkDevice device, uint32_t writeCount,
 
         DescriptorSetSlot &bind = (*binding)[curIdx];
 
-        bind.RemoveBindRefs(record);
+        bind.RemoveBindRefs(GetResourceManager(), record);
 
         if(descWrite.descriptorType == VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER ||
            descWrite.descriptorType == VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER)
         {
-          bind.texelBufferView = descWrite.pTexelBufferView[d];
+          bind.texelBufferView = GetResID(descWrite.pTexelBufferView[d]);
         }
         else if(descWrite.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER ||
                 descWrite.descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER ||
@@ -1045,21 +1083,25 @@ void WrappedVulkan::vkUpdateDescriptorSets(VkDevice device, uint32_t writeCount,
                 descWrite.descriptorType == VK_DESCRIPTOR_TYPE_STORAGE_IMAGE ||
                 descWrite.descriptorType == VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT)
         {
-          bind.imageInfo = descWrite.pImageInfo[d];
+          bool sampler = true;
+          bool imageView = true;
 
-          // ignore descriptors not part of the write, by NULL'ing out those members
-          // as they might not even point to a valid object
+          // ignore descriptors not part of the write, as they might not even point to a valid
+          // object so trying to get their ID could crash
+          if(layoutBinding->immutableSampler ||
+             descWrite.descriptorType != VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER)
+            sampler = false;
           if(descWrite.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER)
-            bind.imageInfo.imageView = VK_NULL_HANDLE;
-          else if(descWrite.descriptorType != VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER)
-            bind.imageInfo.sampler = VK_NULL_HANDLE;
+            imageView = false;
+
+          bind.imageInfo.SetFrom(descWrite.pImageInfo[d], sampler, imageView);
         }
         else
         {
-          bind.bufferInfo = descWrite.pBufferInfo[d];
+          bind.bufferInfo.SetFrom(descWrite.pBufferInfo[d]);
         }
 
-        bind.AddBindRefs(record, ref);
+        bind.AddBindRefs(GetResourceManager(), record, ref);
       }
     }
 
@@ -1116,9 +1158,9 @@ void WrappedVulkan::vkUpdateDescriptorSets(VkDevice device, uint32_t writeCount,
 
         DescriptorSetSlot &bind = (*dstbinding)[curDstIdx];
 
-        bind.RemoveBindRefs(dstrecord);
+        bind.RemoveBindRefs(GetResourceManager(), dstrecord);
         bind = (*srcbinding)[curSrcIdx];
-        bind.AddBindRefs(dstrecord, ref);
+        bind.AddBindRefs(GetResourceManager(), dstrecord, ref);
       }
     }
   }
@@ -1133,7 +1175,7 @@ bool WrappedVulkan::Serialise_vkCreateDescriptorUpdateTemplate(
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
   SERIALISE_ELEMENT_LOCAL(DescriptorUpdateTemplate, GetResID(*pDescriptorUpdateTemplate))
-      .TypedAs("VkDescriptorUpdateTemplate");
+      .TypedAs("VkDescriptorUpdateTemplate"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1141,8 +1183,9 @@ bool WrappedVulkan::Serialise_vkCreateDescriptorUpdateTemplate(
   {
     VkDescriptorUpdateTemplate templ = VK_NULL_HANDLE;
 
+    VkDescriptorUpdateTemplateCreateInfo unwrapped = UnwrapInfo(&CreateInfo);
     VkResult ret =
-        ObjDisp(device)->CreateDescriptorUpdateTemplate(Unwrap(device), &CreateInfo, NULL, &templ);
+        ObjDisp(device)->CreateDescriptorUpdateTemplate(Unwrap(device), &unwrapped, NULL, &templ);
 
     if(ret != VK_SUCCESS)
     {
@@ -1232,7 +1275,7 @@ bool WrappedVulkan::Serialise_vkUpdateDescriptorSetWithTemplate(
     GetRecord(descriptorUpdateTemplate)->descTemplateInfo->Apply(pData, apply);
   }
 
-  SERIALISE_ELEMENT(apply.writes).Named("Decoded Writes");
+  SERIALISE_ELEMENT(apply.writes).Named("Decoded Writes"_lit);
 
   Serialise_DebugMessages(ser);
 
@@ -1333,27 +1376,25 @@ void WrappedVulkan::vkUpdateDescriptorSetWithTemplate(
         Unwrap(device), Unwrap(descriptorSet), Unwrap(descriptorUpdateTemplate), memory));
   }
 
-  bool capframe = false;
   {
-    SCOPED_LOCK(m_CapTransitionLock);
-    capframe = IsActiveCapturing(m_State);
-  }
+    SCOPED_READLOCK(m_CapTransitionLock);
 
-  if(capframe)
-  {
-    CACHE_THREAD_SERIALISER();
+    if(IsActiveCapturing(m_State))
+    {
+      CACHE_THREAD_SERIALISER();
 
-    SCOPED_SERIALISE_CHUNK(VulkanChunk::vkUpdateDescriptorSetWithTemplate);
-    Serialise_vkUpdateDescriptorSetWithTemplate(ser, device, descriptorSet,
-                                                descriptorUpdateTemplate, pData);
+      SCOPED_SERIALISE_CHUNK(VulkanChunk::vkUpdateDescriptorSetWithTemplate);
+      Serialise_vkUpdateDescriptorSetWithTemplate(ser, device, descriptorSet,
+                                                  descriptorUpdateTemplate, pData);
 
-    m_FrameCaptureRecord->AddChunk(scope.Get());
+      m_FrameCaptureRecord->AddChunk(scope.Get());
 
-    // mark the destination set and template as referenced
-    GetResourceManager()->MarkResourceFrameReferenced(GetResID(descriptorSet),
-                                                      eFrameRef_PartialWrite);
-    GetResourceManager()->MarkResourceFrameReferenced(GetResID(descriptorUpdateTemplate),
-                                                      eFrameRef_Read);
+      // mark the destination set and template as referenced
+      GetResourceManager()->MarkResourceFrameReferenced(GetResID(descriptorSet),
+                                                        eFrameRef_PartialWrite);
+      GetResourceManager()->MarkResourceFrameReferenced(GetResID(descriptorUpdateTemplate),
+                                                        eFrameRef_Read);
+    }
   }
 
   // need to track descriptor set contents whether capframing or idle
@@ -1400,12 +1441,12 @@ void WrappedVulkan::vkUpdateDescriptorSetWithTemplate(
 
         DescriptorSetSlot &bind = (*binding)[curIdx];
 
-        bind.RemoveBindRefs(record);
+        bind.RemoveBindRefs(GetResourceManager(), record);
 
         if(entry.descriptorType == VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER ||
            entry.descriptorType == VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER)
         {
-          bind.texelBufferView = *(VkBufferView *)src;
+          bind.texelBufferView = GetResID(*(const VkBufferView *)src);
         }
         else if(entry.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER ||
                 entry.descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER ||
@@ -1413,21 +1454,27 @@ void WrappedVulkan::vkUpdateDescriptorSetWithTemplate(
                 entry.descriptorType == VK_DESCRIPTOR_TYPE_STORAGE_IMAGE ||
                 entry.descriptorType == VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT)
         {
-          bind.imageInfo = *(VkDescriptorImageInfo *)src;
+          const VkDescriptorImageInfo &srcInfo = *(const VkDescriptorImageInfo *)src;
 
-          // ignore descriptors not part of the write, by NULL'ing out those members
-          // as they might not even point to a valid object
+          bool sampler = true;
+          bool imageView = true;
+
+          // ignore descriptors not part of the write, as they might not even point to a valid
+          // object so trying to get their ID could crash
+          if(layoutBinding->immutableSampler ||
+             entry.descriptorType != VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER)
+            sampler = false;
           if(entry.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER)
-            bind.imageInfo.imageView = VK_NULL_HANDLE;
-          else if(entry.descriptorType != VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER)
-            bind.imageInfo.sampler = VK_NULL_HANDLE;
+            imageView = false;
+
+          bind.imageInfo.SetFrom(srcInfo, sampler, imageView);
         }
         else
         {
-          bind.bufferInfo = *(VkDescriptorBufferInfo *)src;
+          bind.bufferInfo.SetFrom(*(const VkDescriptorBufferInfo *)src);
         }
 
-        bind.AddBindRefs(record, ref);
+        bind.AddBindRefs(GetResourceManager(), record, ref);
       }
     }
   }
