@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -410,7 +410,7 @@ bool WrappedVulkan::Serialise_vkCmdSetStencilCompareMask(SerialiserType &ser,
                                                          uint32_t compareMask)
 {
   SERIALISE_ELEMENT(commandBuffer);
-  SERIALISE_ELEMENT_TYPED(VkStencilFaceFlagBits, faceMask).TypedAs("VkStencilFaceFlags");
+  SERIALISE_ELEMENT_TYPED(VkStencilFaceFlagBits, faceMask).TypedAs("VkStencilFaceFlags"_lit);
   SERIALISE_ELEMENT(compareMask);
 
   Serialise_DebugMessages(ser);
@@ -476,7 +476,7 @@ bool WrappedVulkan::Serialise_vkCmdSetStencilWriteMask(SerialiserType &ser,
                                                        uint32_t writeMask)
 {
   SERIALISE_ELEMENT(commandBuffer);
-  SERIALISE_ELEMENT_TYPED(VkStencilFaceFlagBits, faceMask).TypedAs("VkStencilFaceFlags");
+  SERIALISE_ELEMENT_TYPED(VkStencilFaceFlagBits, faceMask).TypedAs("VkStencilFaceFlags"_lit);
   SERIALISE_ELEMENT(writeMask);
 
   Serialise_DebugMessages(ser);
@@ -542,7 +542,7 @@ bool WrappedVulkan::Serialise_vkCmdSetStencilReference(SerialiserType &ser,
                                                        uint32_t reference)
 {
   SERIALISE_ELEMENT(commandBuffer);
-  SERIALISE_ELEMENT_TYPED(VkStencilFaceFlagBits, faceMask).TypedAs("VkStencilFaceFlags");
+  SERIALISE_ELEMENT_TYPED(VkStencilFaceFlagBits, faceMask).TypedAs("VkStencilFaceFlags"_lit);
   SERIALISE_ELEMENT(reference);
 
   Serialise_DebugMessages(ser);
@@ -607,7 +607,7 @@ bool WrappedVulkan::Serialise_vkCmdSetSampleLocationsEXT(
     const VkSampleLocationsInfoEXT *pSampleLocationsInfo)
 {
   SERIALISE_ELEMENT(commandBuffer);
-  SERIALISE_ELEMENT_LOCAL(sampleInfo, *pSampleLocationsInfo).Named("pSampleLocationsInfo");
+  SERIALISE_ELEMENT_LOCAL(sampleInfo, *pSampleLocationsInfo).Named("pSampleLocationsInfo"_lit);
 
   Serialise_DebugMessages(ser);
 
@@ -625,10 +625,8 @@ bool WrappedVulkan::Serialise_vkCmdSetSampleLocationsEXT(
 
         if(ShouldUpdateRenderState(m_LastCmdBufferID))
         {
-          m_RenderState.sampleLocations.locations.clear();
-          m_RenderState.sampleLocations.locations.insert(
-              m_RenderState.sampleLocations.locations.begin(), sampleInfo.pSampleLocations,
-              sampleInfo.pSampleLocations + sampleInfo.sampleLocationsCount);
+          m_RenderState.sampleLocations.locations.assign(sampleInfo.pSampleLocations,
+                                                         sampleInfo.sampleLocationsCount);
           m_RenderState.sampleLocations.gridSize = sampleInfo.sampleLocationGridSize;
           m_RenderState.sampleLocations.sampleCount = sampleInfo.sampleLocationsPerPixel;
         }
@@ -742,6 +740,72 @@ void WrappedVulkan::vkCmdSetDiscardRectangleEXT(VkCommandBuffer commandBuffer,
   }
 }
 
+template <typename SerialiserType>
+bool WrappedVulkan::Serialise_vkCmdSetLineStippleEXT(SerialiserType &ser,
+                                                     VkCommandBuffer commandBuffer,
+                                                     uint32_t lineStippleFactor,
+                                                     uint16_t lineStipplePattern)
+{
+  SERIALISE_ELEMENT(commandBuffer);
+  SERIALISE_ELEMENT(lineStippleFactor);
+  SERIALISE_ELEMENT(lineStipplePattern);
+
+  Serialise_DebugMessages(ser);
+
+  SERIALISE_CHECK_READ_ERRORS();
+
+  if(IsReplayingAndReading())
+  {
+    m_LastCmdBufferID = GetResourceManager()->GetOriginalID(GetResID(commandBuffer));
+
+    if(IsActiveReplaying(m_State))
+    {
+      if(InRerecordRange(m_LastCmdBufferID))
+      {
+        commandBuffer = RerecordCmdBuf(m_LastCmdBufferID);
+
+        if(ShouldUpdateRenderState(m_LastCmdBufferID))
+        {
+          m_RenderState.stippleFactor = lineStippleFactor;
+          m_RenderState.stipplePattern = lineStipplePattern;
+        }
+      }
+      else
+      {
+        commandBuffer = VK_NULL_HANDLE;
+      }
+    }
+
+    if(commandBuffer != VK_NULL_HANDLE)
+      ObjDisp(commandBuffer)
+          ->CmdSetLineStippleEXT(Unwrap(commandBuffer), lineStippleFactor, lineStipplePattern);
+  }
+
+  return true;
+}
+
+void WrappedVulkan::vkCmdSetLineStippleEXT(VkCommandBuffer commandBuffer,
+                                           uint32_t lineStippleFactor, uint16_t lineStipplePattern)
+{
+  SCOPED_DBG_SINK();
+
+  SERIALISE_TIME_CALL(
+      ObjDisp(commandBuffer)
+          ->CmdSetLineStippleEXT(Unwrap(commandBuffer), lineStippleFactor, lineStipplePattern));
+
+  if(IsCaptureMode(m_State))
+  {
+    VkResourceRecord *record = GetRecord(commandBuffer);
+
+    CACHE_THREAD_SERIALISER();
+
+    SCOPED_SERIALISE_CHUNK(VulkanChunk::vkCmdSetLineStippleEXT);
+    Serialise_vkCmdSetLineStippleEXT(ser, commandBuffer, lineStippleFactor, lineStipplePattern);
+
+    record->AddChunk(scope.Get());
+  }
+}
+
 INSTANTIATE_FUNCTION_SERIALISED(void, vkCmdSetViewport, VkCommandBuffer commandBuffer,
                                 uint32_t firstViewport, uint32_t viewportCount,
                                 const VkViewport *pViewports);
@@ -777,4 +841,7 @@ INSTANTIATE_FUNCTION_SERIALISED(void, vkCmdSetSampleLocationsEXT, VkCommandBuffe
 
 INSTANTIATE_FUNCTION_SERIALISED(void, vkCmdSetDiscardRectangleEXT, VkCommandBuffer commandBuffer,
                                 uint32_t firstDiscardRectangle, uint32_t discardRectangleCount,
-                                const VkRect2D *pDiscardRectangles);
\ No newline at end of file
+                                const VkRect2D *pDiscardRectangles);
+
+INSTANTIATE_FUNCTION_SERIALISED(void, vkCmdSetLineStippleEXT, VkCommandBuffer commandBuffer,
+                                uint32_t lineStippleFactor, uint16_t lineStipplePattern);
