@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,8 +23,9 @@
  ******************************************************************************/
 
 #include "../vk_core.h"
+#include "api/replay/version.h"
 
-static char fakeRenderDocUUID[VK_UUID_SIZE + 1] = {};
+static char fakeRenderDocUUID[VK_UUID_SIZE] = {};
 
 void MakeFakeUUID()
 {
@@ -36,7 +37,9 @@ void MakeFakeUUID()
     // rdocyymmddHHMMSS
     // we pass size+1 so that there's room for a null terminator (the UUID doesn't
     // need a null terminator as it's a fixed size non-string array)
-    StringFormat::sntimef(fakeRenderDocUUID, VK_UUID_SIZE + 1, "rdoc%y%m%d%H%M%S");
+    rdcstr uuid = StringFormat::sntimef(Timing::GetUTCTime(), "rdoc%y%m%d%H%M%S");
+    RDCASSERT(uuid.size() == sizeof(fakeRenderDocUUID));
+    memcpy(fakeRenderDocUUID, uuid.c_str(), RDCMIN(VK_UUID_SIZE, uuid.count()));
   }
 }
 
@@ -44,6 +47,20 @@ void WrappedVulkan::vkGetPhysicalDeviceFeatures(VkPhysicalDevice physicalDevice,
                                                 VkPhysicalDeviceFeatures *pFeatures)
 {
   ObjDisp(physicalDevice)->GetPhysicalDeviceFeatures(Unwrap(physicalDevice), pFeatures);
+
+  if(pFeatures)
+  {
+    // sparse features not supported
+    pFeatures->sparseBinding = VK_FALSE;
+    pFeatures->sparseResidencyBuffer = VK_FALSE;
+    pFeatures->sparseResidencyImage2D = VK_FALSE;
+    pFeatures->sparseResidencyImage3D = VK_FALSE;
+    pFeatures->sparseResidency2Samples = VK_FALSE;
+    pFeatures->sparseResidency4Samples = VK_FALSE;
+    pFeatures->sparseResidency8Samples = VK_FALSE;
+    pFeatures->sparseResidency16Samples = VK_FALSE;
+    pFeatures->sparseResidencyAliased = VK_FALSE;
+  }
 }
 
 void WrappedVulkan::vkGetPhysicalDeviceFormatProperties(VkPhysicalDevice physicalDevice,
@@ -211,12 +228,6 @@ void WrappedVulkan::vkGetPhysicalDeviceQueueFamilyProperties(
 void WrappedVulkan::vkGetPhysicalDeviceMemoryProperties(
     VkPhysicalDevice physicalDevice, VkPhysicalDeviceMemoryProperties *pMemoryProperties)
 {
-  if(pMemoryProperties)
-  {
-    *pMemoryProperties = *GetRecord(physicalDevice)->memProps;
-    return;
-  }
-
   ObjDisp(physicalDevice)->GetPhysicalDeviceMemoryProperties(Unwrap(physicalDevice), pMemoryProperties);
 }
 
@@ -237,21 +248,6 @@ void WrappedVulkan::vkGetBufferMemoryRequirements(VkDevice device, VkBuffer buff
     *pMemoryRequirements = GetRecord(buffer)->resInfo->memreqs;
   else
     ObjDisp(device)->GetBufferMemoryRequirements(Unwrap(device), Unwrap(buffer), pMemoryRequirements);
-
-  // don't do remapping here on replay.
-  if(IsReplayMode(m_State))
-    return;
-
-  uint32_t bits = pMemoryRequirements->memoryTypeBits;
-  uint32_t *memIdxMap = GetRecord(device)->memIdxMap;
-
-  pMemoryRequirements->memoryTypeBits = 0;
-
-  // for each of our fake memory indices, check if the real
-  // memory type it points to is set - if so, set our fake bit
-  for(uint32_t i = 0; i < VK_MAX_MEMORY_TYPES; i++)
-    if(memIdxMap[i] < 32U && (bits & (1U << memIdxMap[i])))
-      pMemoryRequirements->memoryTypeBits |= (1U << i);
 }
 
 void WrappedVulkan::vkGetImageMemoryRequirements(VkDevice device, VkImage image,
@@ -265,21 +261,6 @@ void WrappedVulkan::vkGetImageMemoryRequirements(VkDevice device, VkImage image,
   else
     ObjDisp(device)->GetImageMemoryRequirements(Unwrap(device), Unwrap(image), pMemoryRequirements);
 
-  // don't do remapping here on replay.
-  if(IsReplayMode(m_State))
-    return;
-
-  uint32_t bits = pMemoryRequirements->memoryTypeBits;
-  uint32_t *memIdxMap = GetRecord(device)->memIdxMap;
-
-  pMemoryRequirements->memoryTypeBits = 0;
-
-  // for each of our fake memory indices, check if the real
-  // memory type it points to is set - if so, set our fake bit
-  for(uint32_t i = 0; i < VK_MAX_MEMORY_TYPES; i++)
-    if(memIdxMap[i] < 32U && (bits & (1U << memIdxMap[i])))
-      pMemoryRequirements->memoryTypeBits |= (1U << i);
-
   // AMD can have some variability in the returned size, so we need to pad the reported size to
   // allow for this. The variability isn't quite clear, but for now we assume aligning size to
   // alignment * 4 should be sufficient (adding on a fixed padding won't help the problem as it
@@ -326,21 +307,6 @@ void WrappedVulkan::vkGetBufferMemoryRequirements2(VkDevice device,
   // pessimistic for the case of external memory bound resources. See vkCreateBuffer/vkCreateImage
   if(IsCaptureMode(m_State) && GetRecord(pInfo->buffer)->resInfo)
     pMemoryRequirements->memoryRequirements = GetRecord(pInfo->buffer)->resInfo->memreqs;
-
-  // don't do remapping here on replay.
-  if(IsReplayMode(m_State))
-    return;
-
-  uint32_t bits = pMemoryRequirements->memoryRequirements.memoryTypeBits;
-  uint32_t *memIdxMap = GetRecord(device)->memIdxMap;
-
-  pMemoryRequirements->memoryRequirements.memoryTypeBits = 0;
-
-  // for each of our fake memory indices, check if the real
-  // memory type it points to is set - if so, set our fake bit
-  for(uint32_t i = 0; i < VK_MAX_MEMORY_TYPES; i++)
-    if(memIdxMap[i] < 32U && (bits & (1U << memIdxMap[i])))
-      pMemoryRequirements->memoryRequirements.memoryTypeBits |= (1U << i);
 }
 
 void WrappedVulkan::vkGetImageMemoryRequirements2(VkDevice device,
@@ -361,17 +327,6 @@ void WrappedVulkan::vkGetImageMemoryRequirements2(VkDevice device,
   if(IsReplayMode(m_State))
     return;
 
-  uint32_t bits = pMemoryRequirements->memoryRequirements.memoryTypeBits;
-  uint32_t *memIdxMap = GetRecord(device)->memIdxMap;
-
-  pMemoryRequirements->memoryRequirements.memoryTypeBits = 0;
-
-  // for each of our fake memory indices, check if the real
-  // memory type it points to is set - if so, set our fake bit
-  for(uint32_t i = 0; i < VK_MAX_MEMORY_TYPES; i++)
-    if(memIdxMap[i] < 32U && (bits & (1U << memIdxMap[i])))
-      pMemoryRequirements->memoryRequirements.memoryTypeBits |= (1U << i);
-
   // AMD can have some variability in the returned size, so we need to pad the reported size to
   // allow for this. The variability isn't quite clear, but for now we assume aligning size to
   // alignment * 4 should be sufficient (adding on a fixed padding won't help the problem as it
@@ -502,7 +457,7 @@ VkResult WrappedVulkan::vkGetMemoryWin32HandleKHR(
 }
 
 VkResult WrappedVulkan::vkGetMemoryWin32HandlePropertiesKHR(
-    VkDevice device, VkExternalMemoryHandleTypeFlagBitsKHR handleType, HANDLE handle,
+    VkDevice device, VkExternalMemoryHandleTypeFlagBits handleType, HANDLE handle,
     VkMemoryWin32HandlePropertiesKHR *pMemoryWin32HandleProperties)
 {
   return ObjDisp(device)->GetMemoryWin32HandlePropertiesKHR(Unwrap(device), handleType, handle,
@@ -519,7 +474,7 @@ VkResult WrappedVulkan::vkGetMemoryFdKHR(VkDevice device, const VkMemoryGetFdInf
 }
 
 VkResult WrappedVulkan::vkGetMemoryFdPropertiesKHR(VkDevice device,
-                                                   VkExternalMemoryHandleTypeFlagBitsKHR handleType,
+                                                   VkExternalMemoryHandleTypeFlagBits handleType,
                                                    int fd,
                                                    VkMemoryFdPropertiesKHR *pMemoryFdProperties)
 {
@@ -570,6 +525,36 @@ void WrappedVulkan::vkGetPhysicalDeviceFeatures2(VkPhysicalDevice physicalDevice
     RDCWARN("Forcibly disabling support for protected memory");
     protectedMem->protectedMemory = VK_FALSE;
   }
+
+  // in Vulkan 1.2 buffer_device_address can be used without an extension, so we can't hide the
+  // extension when capture/replay is not supported. Instead we hide the feature bit here.
+  VkPhysicalDeviceVulkan12Features *vulkan12 = (VkPhysicalDeviceVulkan12Features *)FindNextStruct(
+      pFeatures, VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES);
+
+  if(vulkan12)
+  {
+    if(vulkan12->bufferDeviceAddressCaptureReplay == VK_FALSE)
+    {
+      RDCWARN(
+          "VkPhysicalDeviceVulkan12Features::bufferDeviceAddressCaptureReplay is false, "
+          "can't support capture of bufferDeviceAddress");
+      vulkan12->bufferDeviceAddress = vulkan12->bufferDeviceAddressMultiDevice = VK_FALSE;
+    }
+  }
+
+  if(pFeatures)
+  {
+    // sparse features not supported
+    pFeatures->features.sparseBinding = VK_FALSE;
+    pFeatures->features.sparseResidencyBuffer = VK_FALSE;
+    pFeatures->features.sparseResidencyImage2D = VK_FALSE;
+    pFeatures->features.sparseResidencyImage3D = VK_FALSE;
+    pFeatures->features.sparseResidency2Samples = VK_FALSE;
+    pFeatures->features.sparseResidency4Samples = VK_FALSE;
+    pFeatures->features.sparseResidency8Samples = VK_FALSE;
+    pFeatures->features.sparseResidency16Samples = VK_FALSE;
+    pFeatures->features.sparseResidencyAliased = VK_FALSE;
+  }
 }
 
 void WrappedVulkan::vkGetPhysicalDeviceProperties2(VkPhysicalDevice physicalDevice,
@@ -728,4 +713,134 @@ VkDeviceAddress WrappedVulkan::vkGetBufferDeviceAddressEXT(VkDevice device,
   VkBufferDeviceAddressInfoEXT unwrappedInfo = *pInfo;
   unwrappedInfo.buffer = Unwrap(unwrappedInfo.buffer);
   return ObjDisp(device)->GetBufferDeviceAddressEXT(Unwrap(device), &unwrappedInfo);
-}
\ No newline at end of file
+}
+
+VkResult WrappedVulkan::vkGetPipelineExecutablePropertiesKHR(
+    VkDevice device, const VkPipelineInfoKHR *pPipelineInfo, uint32_t *pExecutableCount,
+    VkPipelineExecutablePropertiesKHR *pProperties)
+{
+  VkPipelineInfoKHR unwrappedInfo = *pPipelineInfo;
+  unwrappedInfo.pipeline = Unwrap(unwrappedInfo.pipeline);
+  return ObjDisp(device)->GetPipelineExecutablePropertiesKHR(Unwrap(device), &unwrappedInfo,
+                                                             pExecutableCount, pProperties);
+}
+
+VkResult WrappedVulkan::vkGetPipelineExecutableStatisticsKHR(
+    VkDevice device, const VkPipelineExecutableInfoKHR *pExecutableInfo, uint32_t *pStatisticCount,
+    VkPipelineExecutableStatisticKHR *pStatistics)
+{
+  VkPipelineExecutableInfoKHR unwrappedInfo = *pExecutableInfo;
+  unwrappedInfo.pipeline = Unwrap(unwrappedInfo.pipeline);
+  return ObjDisp(device)->GetPipelineExecutableStatisticsKHR(Unwrap(device), &unwrappedInfo,
+                                                             pStatisticCount, pStatistics);
+}
+
+VkResult WrappedVulkan::vkGetPipelineExecutableInternalRepresentationsKHR(
+    VkDevice device, const VkPipelineExecutableInfoKHR *pExecutableInfo,
+    uint32_t *pInternalRepresentationCount,
+    VkPipelineExecutableInternalRepresentationKHR *pInternalRepresentations)
+{
+  VkPipelineExecutableInfoKHR unwrappedInfo = *pExecutableInfo;
+  unwrappedInfo.pipeline = Unwrap(unwrappedInfo.pipeline);
+  return ObjDisp(device)->GetPipelineExecutableInternalRepresentationsKHR(
+      Unwrap(device), &unwrappedInfo, pInternalRepresentationCount, pInternalRepresentations);
+}
+
+VkDeviceAddress WrappedVulkan::vkGetBufferDeviceAddress(VkDevice device,
+                                                        VkBufferDeviceAddressInfo *pInfo)
+{
+  VkBufferDeviceAddressInfo unwrappedInfo = *pInfo;
+  unwrappedInfo.buffer = Unwrap(unwrappedInfo.buffer);
+  return ObjDisp(device)->GetBufferDeviceAddress(Unwrap(device), &unwrappedInfo);
+}
+
+uint64_t WrappedVulkan::vkGetBufferOpaqueCaptureAddress(VkDevice device,
+                                                        VkBufferDeviceAddressInfo *pInfo)
+{
+  VkBufferDeviceAddressInfo unwrappedInfo = *pInfo;
+  unwrappedInfo.buffer = Unwrap(unwrappedInfo.buffer);
+  return ObjDisp(device)->GetBufferOpaqueCaptureAddress(Unwrap(device), &unwrappedInfo);
+}
+
+uint64_t WrappedVulkan::vkGetDeviceMemoryOpaqueCaptureAddress(
+    VkDevice device, VkDeviceMemoryOpaqueCaptureAddressInfo *pInfo)
+{
+  VkDeviceMemoryOpaqueCaptureAddressInfo unwrappedInfo = *pInfo;
+  unwrappedInfo.memory = Unwrap(unwrappedInfo.memory);
+  return ObjDisp(device)->GetDeviceMemoryOpaqueCaptureAddress(Unwrap(device), &unwrappedInfo);
+}
+
+VkResult WrappedVulkan::vkGetPhysicalDeviceToolPropertiesEXT(
+    VkPhysicalDevice physicalDevice, uint32_t *pToolCount,
+    VkPhysicalDeviceToolPropertiesEXT *pToolProperties)
+{
+  // check how many tools are downstream. The function pointer will be NULL if no-one else supports
+  // this extension except us.
+  uint32_t downstreamCount = 0;
+  if(ObjDisp(physicalDevice)->GetPhysicalDeviceToolPropertiesEXT != NULL)
+    ObjDisp(physicalDevice)
+        ->GetPhysicalDeviceToolPropertiesEXT(Unwrap(physicalDevice), &downstreamCount, NULL);
+
+  // if we're just enumerating, pToolProperties is NULL, so set the tool count and return
+  if(pToolCount && pToolProperties == NULL)
+  {
+    *pToolCount = downstreamCount + 1;
+    return VK_SUCCESS;
+  }
+
+  // otherwise we expect both to be non-NULL
+  if(pToolCount == NULL || pToolProperties == NULL)
+    return VK_INCOMPLETE;
+
+  // this is how much space is in the array, don't forget it
+  uint32_t availableCount = *pToolCount;
+
+  VkResult vkr = VK_SUCCESS;
+
+  if(ObjDisp(physicalDevice)->GetPhysicalDeviceToolPropertiesEXT != NULL)
+  {
+    // call downstream to populate the array (up to what's available)
+    // this writes up to availableCount properties into pToolProperties, and sets the number written
+    // in pToolCount
+    vkr = ObjDisp(physicalDevice)
+              ->GetPhysicalDeviceToolPropertiesEXT(Unwrap(physicalDevice), pToolCount,
+                                                   pToolProperties);
+  }
+  else
+  {
+    // nothing written downstream
+    *pToolCount = 0;
+  }
+
+  // if available isn't enough, return VK_INCOMPLETE now
+  if(vkr == VK_INCOMPLETE || availableCount < downstreamCount + 1)
+    return VK_INCOMPLETE;
+
+  // otherwise we write our own properties in after any downstream properties, then increment
+  // pToolCount
+
+  VkPhysicalDeviceToolPropertiesEXT &props = *(pToolProperties + *pToolCount);
+
+  const rdcstr name = "RenderDoc"_lit;
+  const rdcstr version = StringFormat::Fmt(
+      "%s (%s)", FULL_VERSION_STRING, GitVersionHash[0] == 'N' ? "Unknown revision" : GitVersionHash);
+  const rdcstr description = "Debugging capture layer for RenderDoc"_lit;
+
+  RDCASSERTMSG("Name is too long for VkPhysicalDeviceToolPropertiesEXT",
+               name.length() < sizeof(props.name));
+  RDCASSERTMSG("Version is too long for VkPhysicalDeviceToolPropertiesEXT",
+               version.length() < sizeof(props.version));
+  RDCASSERTMSG("Description is too long for VkPhysicalDeviceToolPropertiesEXT",
+               description.length() < sizeof(props.description));
+
+  memcpy(props.name, name.c_str(), name.length() + 1);
+  memcpy(props.version, version.c_str(), version.length() + 1);
+  props.purposes = VK_TOOL_PURPOSE_TRACING_BIT_EXT | VK_TOOL_PURPOSE_DEBUG_MARKERS_BIT_EXT |
+                   VK_TOOL_PURPOSE_MODIFYING_FEATURES_BIT_EXT;
+  memcpy(props.description, description.c_str(), description.length() + 1);
+  // do not tell people about the layer
+  RDCEraseEl(props.layer);
+
+  (*pToolCount)++;
+  return VK_SUCCESS;
+}
