@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +24,29 @@
 
 #include "../vk_core.h"
 
+static void PatchSeparateStencil(VkAttachmentDescription &att, const VkAttachmentReference *ref)
+{
+}
+
+static void PatchSeparateStencil(VkAttachmentDescription2KHR &att,
+                                 const VkAttachmentReference2KHR *ref)
+{
+  // VK_KHR_separate_depth_stencil_layouts
+  const VkAttachmentReferenceStencilLayoutKHR *separateStencilRef =
+      (const VkAttachmentReferenceStencilLayoutKHR *)FindNextStruct(
+          ref, VK_STRUCTURE_TYPE_ATTACHMENT_REFERENCE_STENCIL_LAYOUT_KHR);
+
+  VkAttachmentDescriptionStencilLayoutKHR *separateStencilAtt =
+      (VkAttachmentDescriptionStencilLayoutKHR *)FindNextStruct(
+          &att, VK_STRUCTURE_TYPE_ATTACHMENT_DESCRIPTION_STENCIL_LAYOUT_KHR);
+
+  if(separateStencilRef && separateStencilAtt)
+  {
+    separateStencilAtt->stencilInitialLayout = separateStencilAtt->stencilFinalLayout =
+        separateStencilRef->stencilLayout;
+  }
+}
+
 template <typename RPCreateInfo>
 static void MakeSubpassLoadRP(RPCreateInfo &info, const RPCreateInfo *origInfo, uint32_t s)
 {
@@ -67,6 +90,10 @@ static void MakeSubpassLoadRP(RPCreateInfo &info, const RPCreateInfo *origInfo,
     att[sub->pDepthStencilAttachment->attachment].initialLayout =
         att[sub->pDepthStencilAttachment->attachment].finalLayout =
             sub->pDepthStencilAttachment->layout;
+
+    // need to do this via an external overloaded function because FindNextStruct will fail to
+    // compile on VkAttachmentReference
+    PatchSeparateStencil(att[sub->pDepthStencilAttachment->attachment], sub->pDepthStencilAttachment);
   }
 }
 
@@ -75,12 +102,15 @@ VkFramebufferCreateInfo WrappedVulkan::UnwrapInfo(const VkFramebufferCreateInfo
 {
   VkFramebufferCreateInfo ret = *info;
 
-  VkImageView *unwrapped = GetTempArray<VkImageView>(info->attachmentCount);
-  for(uint32_t i = 0; i < info->attachmentCount; i++)
-    unwrapped[i] = Unwrap(info->pAttachments[i]);
+  if((ret.flags & VK_FRAMEBUFFER_CREATE_IMAGELESS_BIT) == 0)
+  {
+    VkImageView *unwrapped = GetTempArray<VkImageView>(info->attachmentCount);
+    for(uint32_t i = 0; i < info->attachmentCount; i++)
+      unwrapped[i] = Unwrap(info->pAttachments[i]);
+    ret.pAttachments = unwrapped;
+  }
 
   ret.renderPass = Unwrap(ret.renderPass);
-  ret.pAttachments = unwrapped;
 
   return ret;
 }
@@ -95,7 +125,9 @@ VkFramebufferCreateInfo WrappedVulkan::UnwrapInfo(const VkFramebufferCreateInfo
     if(obj == VK_NULL_HANDLE)                                                                      \
       return;                                                                                      \
     type unwrappedObj = Unwrap(obj);                                                               \
-    m_ForcedReferences.erase(GetResID(obj));                                                       \
+    m_ForcedReferences.removeOne(GetRecord(obj));                                                  \
+    if(IsReplayMode(m_State))                                                                      \
+      m_CreationInfo.erase(GetResID(obj));                                                         \
     GetResourceManager()->ReleaseWrappedResource(obj, true);                                       \
     ObjDisp(device)->func(Unwrap(device), unwrappedObj, pAllocator);                               \
   }
@@ -200,7 +232,7 @@ bool WrappedVulkan::ReleaseResource(WrappedVkRes *res)
 
   // MULTIDEVICE need to get the actual device that created this object
   VkDevice dev = m_Device;
-  const VkLayerDispatchTable *vt = m_Device != VK_NULL_HANDLE ? ObjDisp(dev) : NULL;
+  const VkDevDispatchTable *vt = m_Device != VK_NULL_HANDLE ? ObjDisp(dev) : NULL;
 
   WrappedVkNonDispRes *nondisp = (WrappedVkNonDispRes *)res;
   WrappedVkDispRes *disp = (WrappedVkDispRes *)res;
@@ -420,7 +452,7 @@ bool WrappedVulkan::Serialise_vkCreateSampler(SerialiserType &ser, VkDevice devi
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(Sampler, GetResID(*pSampler)).TypedAs("VkSampler");
+  SERIALISE_ELEMENT_LOCAL(Sampler, GetResID(*pSampler)).TypedAs("VkSampler"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -544,7 +576,7 @@ bool WrappedVulkan::Serialise_vkCreateFramebuffer(SerialiserType &ser, VkDevice
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(Framebuffer, GetResID(*pFramebuffer)).TypedAs("VkFramebuffer");
+  SERIALISE_ELEMENT_LOCAL(Framebuffer, GetResID(*pFramebuffer)).TypedAs("VkFramebuffer"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -628,8 +660,11 @@ bool WrappedVulkan::Serialise_vkCreateFramebuffer(SerialiserType &ser, VkDevice
     DerivedResource(device, Framebuffer);
     DerivedResource(CreateInfo.renderPass, Framebuffer);
 
-    for(uint32_t i = 0; i < CreateInfo.attachmentCount; i++)
-      DerivedResource(CreateInfo.pAttachments[i], Framebuffer);
+    if((CreateInfo.flags & VK_FRAMEBUFFER_CREATE_IMAGELESS_BIT) == 0)
+    {
+      for(uint32_t i = 0; i < CreateInfo.attachmentCount; i++)
+        DerivedResource(CreateInfo.pAttachments[i], Framebuffer);
+    }
   }
 
   return true;
@@ -668,21 +703,60 @@ VkResult WrappedVulkan::vkCreateFramebuffer(VkDevice device,
       VkResourceRecord *rpRecord = GetRecord(pCreateInfo->renderPass);
       record->AddParent(rpRecord);
 
-      uint32_t arrayCount = pCreateInfo->attachmentCount + 1;
+      // *2 in case we need separate barriers for depth and stencil, +1 for the terminating null
+      // attachment info
+      uint32_t arrayCount = pCreateInfo->attachmentCount * 2 + 1;
 
       record->imageAttachments = new AttachmentInfo[arrayCount];
       RDCEraseMem(record->imageAttachments, sizeof(AttachmentInfo) * arrayCount);
 
-      for(uint32_t i = 0; i < pCreateInfo->attachmentCount; i++)
+      for(uint32_t i = 0, a = 0; i < pCreateInfo->attachmentCount; i++, a++)
       {
-        VkResourceRecord *attRecord = GetRecord(pCreateInfo->pAttachments[i]);
-        record->AddParent(attRecord);
+        record->imageAttachments[a].barrier = rpRecord->imageAttachments[a].barrier;
 
-        record->imageAttachments[i].record = attRecord;
-        record->imageAttachments[i].barrier = rpRecord->imageAttachments[i].barrier;
-        record->imageAttachments[i].barrier.image =
-            GetResourceManager()->GetCurrentHandle<VkImage>(attRecord->baseResource);
-        record->imageAttachments[i].barrier.subresourceRange = attRecord->viewRange;
+        if((pCreateInfo->flags & VK_FRAMEBUFFER_CREATE_IMAGELESS_BIT) == 0)
+        {
+          VkResourceRecord *attRecord = GetRecord(pCreateInfo->pAttachments[i]);
+          record->AddParent(attRecord);
+
+          record->imageAttachments[a].record = attRecord;
+          record->imageAttachments[a].barrier.image =
+              GetResourceManager()->GetCurrentHandle<VkImage>(attRecord->baseResource);
+          record->imageAttachments[a].barrier.subresourceRange = attRecord->viewRange;
+
+          ImageLayouts *layout = NULL;
+          {
+            SCOPED_LOCK(m_ImageLayoutsLock);
+            layout = &m_ImageLayouts[attRecord->GetResourceID()];
+          }
+
+          if(layout->imageInfo.extent.depth > 1)
+          {
+            record->imageAttachments[a].barrier.subresourceRange.baseArrayLayer = 0;
+            record->imageAttachments[a].barrier.subresourceRange.layerCount = 1;
+          }
+
+          // if the renderpass specifies an aspect mask that mean split depth/stencil handling, so
+          // respect the aspect mask and the next one will be stencil
+          if(rpRecord->imageAttachments[a].barrier.subresourceRange.aspectMask != 0)
+          {
+            record->imageAttachments[a].barrier.subresourceRange.aspectMask =
+                rpRecord->imageAttachments[a].barrier.subresourceRange.aspectMask;
+
+            a++;
+
+            // copy most properties from previous barrier
+            record->imageAttachments[a].record = record->imageAttachments[a - 1].record;
+            record->imageAttachments[a].barrier = record->imageAttachments[a - 1].barrier;
+            // copy aspect mask and layouts from the next barrier in the RP
+            record->imageAttachments[a].barrier.subresourceRange.aspectMask =
+                rpRecord->imageAttachments[a].barrier.subresourceRange.aspectMask;
+            record->imageAttachments[a].barrier.oldLayout =
+                rpRecord->imageAttachments[a].barrier.oldLayout;
+            record->imageAttachments[a].barrier.newLayout =
+                rpRecord->imageAttachments[a].barrier.newLayout;
+          }
+        }
       }
     }
     else
@@ -728,7 +802,7 @@ bool WrappedVulkan::Serialise_vkCreateRenderPass(SerialiserType &ser, VkDevice d
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(RenderPass, GetResID(*pRenderPass)).TypedAs("VkRenderPass");
+  SERIALISE_ELEMENT_LOCAL(RenderPass, GetResID(*pRenderPass)).TypedAs("VkRenderPass"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -755,9 +829,9 @@ bool WrappedVulkan::Serialise_vkCreateRenderPass(SerialiserType &ser, VkDevice d
       if(att[i].stencilLoadOp == VK_ATTACHMENT_LOAD_OP_DONT_CARE)
         att[i].stencilLoadOp = VK_ATTACHMENT_LOAD_OP_LOAD;
 
-      // renderpass can't start or end in presentable layout on replay
-      ReplacePresentableImageLayout(att[i].initialLayout);
-      ReplacePresentableImageLayout(att[i].finalLayout);
+      // sanitise the actual layouts used to create the renderpass
+      SanitiseOldImageLayout(att[i].initialLayout);
+      SanitiseNewImageLayout(att[i].finalLayout);
     }
 
     VkResult ret = ObjDisp(device)->CreateRenderPass(Unwrap(device), &CreateInfo, NULL, &rp);
@@ -887,8 +961,10 @@ VkResult WrappedVulkan::vkCreateRenderPass(VkDevice device, const VkRenderPassCr
       VkResourceRecord *record = GetResourceManager()->AddResourceRecord(*pRenderPass);
       record->AddChunk(chunk);
 
-      // +1 for the terminal value
-      uint32_t arrayCount = pCreateInfo->attachmentCount + 1;
+      // *2 in case we need separate barriers for depth and stencil, +1 for the terminating null
+      // attachment info (though separate depth/stencil buffers aren't needed here, we keep the
+      // array size the same)
+      uint32_t arrayCount = pCreateInfo->attachmentCount * 2 + 1;
 
       record->imageAttachments = new AttachmentInfo[arrayCount];
 
@@ -950,15 +1026,15 @@ VkResult WrappedVulkan::vkCreateRenderPass(VkDevice device, const VkRenderPassCr
 }
 
 template <typename SerialiserType>
-bool WrappedVulkan::Serialise_vkCreateRenderPass2KHR(SerialiserType &ser, VkDevice device,
-                                                     const VkRenderPassCreateInfo2KHR *pCreateInfo,
-                                                     const VkAllocationCallbacks *pAllocator,
-                                                     VkRenderPass *pRenderPass)
+bool WrappedVulkan::Serialise_vkCreateRenderPass2(SerialiserType &ser, VkDevice device,
+                                                  const VkRenderPassCreateInfo2 *pCreateInfo,
+                                                  const VkAllocationCallbacks *pAllocator,
+                                                  VkRenderPass *pRenderPass)
 {
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(RenderPass, GetResID(*pRenderPass)).TypedAs("VkRenderPass");
+  SERIALISE_ELEMENT_LOCAL(RenderPass, GetResID(*pRenderPass)).TypedAs("VkRenderPass"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -974,7 +1050,7 @@ bool WrappedVulkan::Serialise_vkCreateRenderPass2KHR(SerialiserType &ser, VkDevi
     // Likewise we don't want to throw away data before we're ready, so change
     // any load ops to LOAD instead of DONT_CARE (which is valid!). We of course
     // leave any LOAD_OP_CLEAR alone.
-    VkAttachmentDescription2KHR *att = (VkAttachmentDescription2KHR *)CreateInfo.pAttachments;
+    VkAttachmentDescription2 *att = (VkAttachmentDescription2 *)CreateInfo.pAttachments;
     for(uint32_t i = 0; i < CreateInfo.attachmentCount; i++)
     {
       att[i].storeOp = VK_ATTACHMENT_STORE_OP_STORE;
@@ -986,11 +1062,24 @@ bool WrappedVulkan::Serialise_vkCreateRenderPass2KHR(SerialiserType &ser, VkDevi
         att[i].stencilLoadOp = VK_ATTACHMENT_LOAD_OP_LOAD;
 
       // renderpass can't start or end in presentable layout on replay
-      ReplacePresentableImageLayout(att[i].initialLayout);
-      ReplacePresentableImageLayout(att[i].finalLayout);
+      SanitiseOldImageLayout(att[i].initialLayout);
+      SanitiseNewImageLayout(att[i].finalLayout);
+
+      // VK_KHR_separate_depth_stencil_layouts
+      VkAttachmentDescriptionStencilLayoutKHR *separateStencil =
+          (VkAttachmentDescriptionStencilLayoutKHR *)FindNextStruct(
+              &att[i], VK_STRUCTURE_TYPE_ATTACHMENT_DESCRIPTION_STENCIL_LAYOUT_KHR);
+
+      if(separateStencil)
+      {
+        // there's not much to sanitise here but since we're loading, we sanitise undefined to
+        // general.
+        SanitiseOldImageLayout(separateStencil->stencilInitialLayout);
+        SanitiseNewImageLayout(separateStencil->stencilFinalLayout);
+      }
     }
 
-    VkResult ret = ObjDisp(device)->CreateRenderPass2KHR(Unwrap(device), &CreateInfo, NULL, &rp);
+    VkResult ret = ObjDisp(device)->CreateRenderPass2(Unwrap(device), &CreateInfo, NULL, &rp);
 
     if(ret != VK_SUCCESS)
     {
@@ -1026,7 +1115,7 @@ bool WrappedVulkan::Serialise_vkCreateRenderPass2KHR(SerialiserType &ser, VkDevi
           att[i].stencilLoadOp = VK_ATTACHMENT_LOAD_OP_LOAD;
         }
 
-        VkRenderPassCreateInfo2KHR loadInfo = CreateInfo;
+        VkRenderPassCreateInfo2 loadInfo = CreateInfo;
 
         rpinfo.loadRPs.resize(CreateInfo.subpassCount);
 
@@ -1035,8 +1124,8 @@ bool WrappedVulkan::Serialise_vkCreateRenderPass2KHR(SerialiserType &ser, VkDevi
         {
           MakeSubpassLoadRP(loadInfo, &CreateInfo, s);
 
-          ret = ObjDisp(device)->CreateRenderPass2KHR(Unwrap(device), &loadInfo, NULL,
-                                                      &rpinfo.loadRPs[s]);
+          ret = ObjDisp(device)->CreateRenderPass2(Unwrap(device), &loadInfo, NULL,
+                                                   &rpinfo.loadRPs[s]);
           RDCASSERTEQUAL(ret, VK_SUCCESS);
 
           // handle the loadRP being a duplicate
@@ -1073,14 +1162,14 @@ bool WrappedVulkan::Serialise_vkCreateRenderPass2KHR(SerialiserType &ser, VkDevi
   return true;
 }
 
-VkResult WrappedVulkan::vkCreateRenderPass2KHR(VkDevice device,
-                                               const VkRenderPassCreateInfo2KHR *pCreateInfo,
-                                               const VkAllocationCallbacks *pAllocator,
-                                               VkRenderPass *pRenderPass)
+VkResult WrappedVulkan::vkCreateRenderPass2(VkDevice device,
+                                            const VkRenderPassCreateInfo2 *pCreateInfo,
+                                            const VkAllocationCallbacks *pAllocator,
+                                            VkRenderPass *pRenderPass)
 {
   VkResult ret;
-  SERIALISE_TIME_CALL(ret = ObjDisp(device)->CreateRenderPass2KHR(Unwrap(device), pCreateInfo,
-                                                                  pAllocator, pRenderPass));
+  SERIALISE_TIME_CALL(ret = ObjDisp(device)->CreateRenderPass2(Unwrap(device), pCreateInfo,
+                                                               pAllocator, pRenderPass));
 
   if(ret == VK_SUCCESS)
   {
@@ -1093,8 +1182,8 @@ VkResult WrappedVulkan::vkCreateRenderPass2KHR(VkDevice device,
       {
         CACHE_THREAD_SERIALISER();
 
-        SCOPED_SERIALISE_CHUNK(VulkanChunk::vkCreateRenderPass2KHR);
-        Serialise_vkCreateRenderPass2KHR(ser, device, pCreateInfo, NULL, pRenderPass);
+        SCOPED_SERIALISE_CHUNK(VulkanChunk::vkCreateRenderPass2);
+        Serialise_vkCreateRenderPass2(ser, device, pCreateInfo, NULL, pRenderPass);
 
         chunk = scope.Get();
       }
@@ -1102,19 +1191,41 @@ VkResult WrappedVulkan::vkCreateRenderPass2KHR(VkDevice device,
       VkResourceRecord *record = GetResourceManager()->AddResourceRecord(*pRenderPass);
       record->AddChunk(chunk);
 
-      // +1 for the terminal value
-      uint32_t arrayCount = pCreateInfo->attachmentCount + 1;
+      // *2 in case we need separate barriers for depth and stencil, +1 for the terminating null
+      // attachment info
+      uint32_t arrayCount = pCreateInfo->attachmentCount * 2 + 1;
 
       record->imageAttachments = new AttachmentInfo[arrayCount];
 
       RDCEraseMem(record->imageAttachments, sizeof(AttachmentInfo) * arrayCount);
 
-      for(uint32_t i = 0; i < pCreateInfo->attachmentCount; i++)
+      for(uint32_t i = 0, a = 0; i < pCreateInfo->attachmentCount; i++, a++)
       {
-        record->imageAttachments[i].record = NULL;
-        record->imageAttachments[i].barrier.sType = VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER;
-        record->imageAttachments[i].barrier.oldLayout = pCreateInfo->pAttachments[i].initialLayout;
-        record->imageAttachments[i].barrier.newLayout = pCreateInfo->pAttachments[i].finalLayout;
+        record->imageAttachments[a].record = NULL;
+        record->imageAttachments[a].barrier.sType = VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER;
+        record->imageAttachments[a].barrier.oldLayout = pCreateInfo->pAttachments[i].initialLayout;
+        record->imageAttachments[a].barrier.newLayout = pCreateInfo->pAttachments[i].finalLayout;
+
+        // VK_KHR_separate_depth_stencil_layouts
+        VkAttachmentDescriptionStencilLayoutKHR *separateStencil =
+            (VkAttachmentDescriptionStencilLayoutKHR *)FindNextStruct(
+                &pCreateInfo->pAttachments[i],
+                VK_STRUCTURE_TYPE_ATTACHMENT_DESCRIPTION_STENCIL_LAYOUT_KHR);
+
+        if(separateStencil)
+        {
+          record->imageAttachments[a].barrier.subresourceRange.aspectMask = VK_IMAGE_ASPECT_DEPTH_BIT;
+
+          // add a separate barrier for stencil
+          a++;
+
+          record->imageAttachments[a].record = NULL;
+          record->imageAttachments[a].barrier.sType = VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER;
+          record->imageAttachments[a].barrier.oldLayout = separateStencil->stencilInitialLayout;
+          record->imageAttachments[a].barrier.newLayout = separateStencil->stencilFinalLayout;
+          record->imageAttachments[a].barrier.subresourceRange.aspectMask =
+              VK_IMAGE_ASPECT_STENCIL_BIT;
+        }
       }
     }
     else
@@ -1124,9 +1235,9 @@ VkResult WrappedVulkan::vkCreateRenderPass2KHR(VkDevice device,
       VulkanCreationInfo::RenderPass rpinfo;
       rpinfo.Init(GetResourceManager(), m_CreationInfo, pCreateInfo);
 
-      VkRenderPassCreateInfo2KHR info = *pCreateInfo;
+      VkRenderPassCreateInfo2 info = *pCreateInfo;
 
-      VkAttachmentDescription2KHR atts[16];
+      VkAttachmentDescription2 atts[16];
       RDCASSERT(ARRAY_COUNT(atts) >= (size_t)info.attachmentCount);
 
       // make a version of the render pass that loads from its attachments,
@@ -1148,7 +1259,7 @@ VkResult WrappedVulkan::vkCreateRenderPass2KHR(VkDevice device,
       {
         MakeSubpassLoadRP(info, pCreateInfo, s);
 
-        ret = ObjDisp(device)->CreateRenderPass2KHR(Unwrap(device), &info, NULL, &rpinfo.loadRPs[s]);
+        ret = ObjDisp(device)->CreateRenderPass2(Unwrap(device), &info, NULL, &rpinfo.loadRPs[s]);
         RDCASSERTEQUAL(ret, VK_SUCCESS);
 
         ResourceId loadRPid = GetResourceManager()->WrapResource(Unwrap(device), rpinfo.loadRPs[s]);
@@ -1173,7 +1284,7 @@ bool WrappedVulkan::Serialise_vkCreateQueryPool(SerialiserType &ser, VkDevice de
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(QueryPool, GetResID(*pQueryPool)).TypedAs("VkQueryPool");
+  SERIALISE_ELEMENT_LOCAL(QueryPool, GetResID(*pQueryPool)).TypedAs("VkQueryPool"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1288,9 +1399,9 @@ VkResult WrappedVulkan::vkGetQueryPoolResults(VkDevice device, VkQueryPool query
 }
 
 template <typename SerialiserType>
-bool WrappedVulkan::Serialise_vkResetQueryPoolEXT(SerialiserType &ser, VkDevice device,
-                                                  VkQueryPool queryPool, uint32_t firstQuery,
-                                                  uint32_t queryCount)
+bool WrappedVulkan::Serialise_vkResetQueryPool(SerialiserType &ser, VkDevice device,
+                                               VkQueryPool queryPool, uint32_t firstQuery,
+                                               uint32_t queryCount)
 {
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT(queryPool);
@@ -1303,26 +1414,26 @@ bool WrappedVulkan::Serialise_vkResetQueryPoolEXT(SerialiserType &ser, VkDevice
 
   if(IsReplayingAndReading())
   {
-    ObjDisp(device)->ResetQueryPoolEXT(Unwrap(device), Unwrap(queryPool), firstQuery, queryCount);
+    ObjDisp(device)->ResetQueryPool(Unwrap(device), Unwrap(queryPool), firstQuery, queryCount);
   }
 
   return true;
 }
 
-void WrappedVulkan::vkResetQueryPoolEXT(VkDevice device, VkQueryPool queryPool, uint32_t firstQuery,
-                                        uint32_t queryCount)
+void WrappedVulkan::vkResetQueryPool(VkDevice device, VkQueryPool queryPool, uint32_t firstQuery,
+                                     uint32_t queryCount)
 {
   SCOPED_DBG_SINK();
 
-  SERIALISE_TIME_CALL(ObjDisp(device)->ResetQueryPoolEXT(Unwrap(device), Unwrap(queryPool),
-                                                         firstQuery, queryCount));
+  SERIALISE_TIME_CALL(
+      ObjDisp(device)->ResetQueryPool(Unwrap(device), Unwrap(queryPool), firstQuery, queryCount));
 
   if(IsActiveCapturing(m_State))
   {
     CACHE_THREAD_SERIALISER();
 
-    SCOPED_SERIALISE_CHUNK(VulkanChunk::vkResetQueryPoolEXT);
-    Serialise_vkResetQueryPoolEXT(ser, device, queryPool, firstQuery, queryCount);
+    SCOPED_SERIALISE_CHUNK(VulkanChunk::vkResetQueryPool);
+    Serialise_vkResetQueryPool(ser, device, queryPool, firstQuery, queryCount);
 
     m_FrameCaptureRecord->AddChunk(scope.Get());
     GetResourceManager()->MarkResourceFrameReferenced(GetResID(queryPool), eFrameRef_Read);
@@ -1338,7 +1449,7 @@ bool WrappedVulkan::Serialise_vkCreateSamplerYcbcrConversion(
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
   SERIALISE_ELEMENT_LOCAL(ycbcrConversion, GetResID(*pYcbcrConversion))
-      .TypedAs("VkSamplerYcbcrConversion");
+      .TypedAs("VkSamplerYcbcrConversion"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -1583,6 +1694,20 @@ void WrappedVulkan::vkDebugReportMessageEXT(VkInstance instance, VkDebugReportFl
                                                   location, messageCode, pLayerPrefix, pMessage);
 }
 
+void WrappedVulkan::vkSetHdrMetadataEXT(VkDevice device, uint32_t swapchainCount,
+                                        const VkSwapchainKHR *pSwapchains,
+                                        const VkHdrMetadataEXT *pMetadata)
+{
+  return ObjDisp(device)->SetHdrMetadataEXT(Unwrap(device), swapchainCount,
+                                            UnwrapArray(pSwapchains, swapchainCount), pMetadata);
+}
+
+void WrappedVulkan::vkSetLocalDimmingAMD(VkDevice device, VkSwapchainKHR swapChain,
+                                         VkBool32 localDimmingEnable)
+{
+  return ObjDisp(device)->SetLocalDimmingAMD(Unwrap(device), Unwrap(swapChain), localDimmingEnable);
+}
+
 // we use VkObjectType as the object type since it mostly overlaps with the debug report enum so in
 // most cases we can upcast it. There's an overload to translate the few that might conflict.
 // Likewise to re-use the switch in most cases, we return both the record and the unwrapped
@@ -1670,6 +1795,7 @@ static ObjData GetObjData(VkObjectType objType, uint64_t object)
     case VK_OBJECT_TYPE_OBJECT_TABLE_NVX:
     case VK_OBJECT_TYPE_INDIRECT_COMMANDS_LAYOUT_NVX:
     case VK_OBJECT_TYPE_ACCELERATION_STRUCTURE_NV:
+    case VK_OBJECT_TYPE_PERFORMANCE_CONFIGURATION_INTEL:
     case VK_OBJECT_TYPE_UNKNOWN:
     case VK_OBJECT_TYPE_RANGE_SIZE:
     case VK_OBJECT_TYPE_MAX_ENUM: break;
@@ -1723,13 +1849,13 @@ static ObjData GetObjData(VkDebugReportObjectTypeEXT objType, uint64_t object)
     castType = VK_OBJECT_TYPE_DISPLAY_KHR;
   else if(objType == VK_DEBUG_REPORT_OBJECT_TYPE_DISPLAY_MODE_KHR_EXT)
     castType = VK_OBJECT_TYPE_DISPLAY_MODE_KHR;
-  else if(objType == VK_DEBUG_REPORT_OBJECT_TYPE_DEBUG_REPORT_EXT)
+  else if(objType == VK_DEBUG_REPORT_OBJECT_TYPE_DEBUG_REPORT_CALLBACK_EXT_EXT)
     castType = VK_OBJECT_TYPE_DEBUG_REPORT_CALLBACK_EXT;
   else if(objType == VK_DEBUG_REPORT_OBJECT_TYPE_OBJECT_TABLE_NVX_EXT)
     castType = VK_OBJECT_TYPE_OBJECT_TABLE_NVX;
   else if(objType == VK_DEBUG_REPORT_OBJECT_TYPE_INDIRECT_COMMANDS_LAYOUT_NVX_EXT)
     castType = VK_OBJECT_TYPE_INDIRECT_COMMANDS_LAYOUT_NVX;
-  else if(objType == VK_DEBUG_REPORT_OBJECT_TYPE_VALIDATION_CACHE_EXT)
+  else if(objType == VK_DEBUG_REPORT_OBJECT_TYPE_VALIDATION_CACHE_EXT_EXT)
     castType = VK_OBJECT_TYPE_VALIDATION_CACHE_EXT;
   else if(objType == VK_DEBUG_REPORT_OBJECT_TYPE_SAMPLER_YCBCR_CONVERSION_EXT)
     castType = VK_OBJECT_TYPE_SAMPLER_YCBCR_CONVERSION;
@@ -1747,7 +1873,7 @@ static ObjData GetObjData(VkDebugReportObjectTypeEXT objType, uint64_t object)
 
 template <typename SerialiserType>
 bool WrappedVulkan::Serialise_SetShaderDebugPath(SerialiserType &ser, VkShaderModule ShaderObject,
-                                                 std::string DebugPath)
+                                                 rdcstr DebugPath)
 {
   SERIALISE_ELEMENT(ShaderObject);
   SERIALISE_ELEMENT(DebugPath);
@@ -1776,8 +1902,7 @@ VkResult WrappedVulkan::vkDebugMarkerSetObjectTagEXT(VkDevice device,
     {
       CACHE_THREAD_SERIALISER();
 
-      char *tag = (char *)pTagInfo->pTag;
-      std::string DebugPath = std::string(tag, tag + pTagInfo->tagSize);
+      rdcstr DebugPath = rdcstr((char *)pTagInfo->pTag, pTagInfo->tagSize);
 
       SCOPED_SERIALISE_CHUNK(VulkanChunk::SetShaderDebugPath);
       Serialise_SetShaderDebugPath(ser, (VkShaderModule)(uint64_t)data.record->Resource, DebugPath);
@@ -1816,7 +1941,7 @@ bool WrappedVulkan::Serialise_vkDebugMarkerSetObjectNameEXT(
     else
       m_CreationInfo.m_Names[GetResourceManager()->GetLiveID(Object)] = ObjectName;
 
-    ResourceDescription &descr = GetReplay()->GetResourceDesc(Object);
+    ResourceDescription &descr = GetResourceDesc(Object);
 
     AddResourceCurChunk(descr);
     descr.SetCustomName(ObjectName);
@@ -1925,7 +2050,7 @@ bool WrappedVulkan::Serialise_vkSetDebugUtilsObjectNameEXT(
     else
       m_CreationInfo.m_Names[GetResourceManager()->GetLiveID(Object)] = ObjectName;
 
-    ResourceDescription &descr = GetReplay()->GetResourceDesc(Object);
+    ResourceDescription &descr = GetResourceDesc(Object);
 
     AddResourceCurChunk(descr);
     descr.SetCustomName(ObjectName);
@@ -1975,8 +2100,7 @@ VkResult WrappedVulkan::vkSetDebugUtilsObjectTagEXT(VkDevice device,
     {
       CACHE_THREAD_SERIALISER();
 
-      char *tag = (char *)pTagInfo->pTag;
-      std::string DebugPath = std::string(tag, tag + pTagInfo->tagSize);
+      rdcstr DebugPath = rdcstr((char *)pTagInfo->pTag, pTagInfo->tagSize);
 
       SCOPED_SERIALISE_CHUNK(VulkanChunk::SetShaderDebugPath);
       Serialise_SetShaderDebugPath(ser, (VkShaderModule)(uint64_t)data.record->Resource, DebugPath);
@@ -2007,8 +2131,8 @@ INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkCreateRenderPass, VkDevice device,
                                 const VkRenderPassCreateInfo *pCreateInfo,
                                 const VkAllocationCallbacks *pAllocator, VkRenderPass *pRenderPass);
 
-INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkCreateRenderPass2KHR, VkDevice device,
-                                const VkRenderPassCreateInfo2KHR *pCreateInfo,
+INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkCreateRenderPass2, VkDevice device,
+                                const VkRenderPassCreateInfo2 *pCreateInfo,
                                 const VkAllocationCallbacks *pAllocator, VkRenderPass *pRenderPass);
 
 INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkCreateQueryPool, VkDevice device,
@@ -2016,7 +2140,7 @@ INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkCreateQueryPool, VkDevice device,
                                 const VkAllocationCallbacks *pAllocator, VkQueryPool *pQueryPool);
 
 INSTANTIATE_FUNCTION_SERIALISED(void, SetShaderDebugPath, VkShaderModule ShaderObject,
-                                std::string DebugPath);
+                                rdcstr DebugPath);
 
 INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkDebugMarkerSetObjectNameEXT, VkDevice device,
                                 const VkDebugMarkerObjectNameInfoEXT *pNameInfo);
@@ -2029,5 +2153,49 @@ INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkCreateSamplerYcbcrConversion, VkDevi
                                 const VkAllocationCallbacks *pAllocator,
                                 VkSamplerYcbcrConversion *pYcbcrConversion);
 
-INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkResetQueryPoolEXT, VkDevice device,
-                                VkQueryPool queryPool, uint32_t firstQuery, uint32_t queryCount);
+INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkResetQueryPool, VkDevice device, VkQueryPool queryPool,
+                                uint32_t firstQuery, uint32_t queryCount);
+
+VkResult WrappedVulkan::vkGetRefreshCycleDurationGOOGLE(
+    VkDevice device, VkSwapchainKHR swapchain, VkRefreshCycleDurationGOOGLE *pDisplayTimingProperties)
+{
+  return ObjDisp(device)->GetRefreshCycleDurationGOOGLE(Unwrap(device), Unwrap(swapchain),
+                                                        pDisplayTimingProperties);
+}
+
+VkResult WrappedVulkan::vkGetPastPresentationTimingGOOGLE(
+    VkDevice device, VkSwapchainKHR swapchain, uint32_t *pPresentationTimingCount,
+    VkPastPresentationTimingGOOGLE *pPresentationTimings)
+{
+  return ObjDisp(device)->GetPastPresentationTimingGOOGLE(
+      Unwrap(device), Unwrap(swapchain), pPresentationTimingCount, pPresentationTimings);
+}
+
+VkResult WrappedVulkan::vkEnumeratePhysicalDeviceQueueFamilyPerformanceQueryCountersKHR(
+    VkPhysicalDevice physicalDevice, uint32_t queueFamilyIndex, uint32_t *pCounterCount,
+    VkPerformanceCounterKHR *pCounters, VkPerformanceCounterDescriptionKHR *pCounterDescriptions)
+{
+  return ObjDisp(physicalDevice)
+      ->EnumeratePhysicalDeviceQueueFamilyPerformanceQueryCountersKHR(
+          Unwrap(physicalDevice), queueFamilyIndex, pCounterCount, pCounters, pCounterDescriptions);
+}
+
+void WrappedVulkan::vkGetPhysicalDeviceQueueFamilyPerformanceQueryPassesKHR(
+    VkPhysicalDevice physicalDevice,
+    const VkQueryPoolPerformanceCreateInfoKHR *pPerformanceQueryCreateInfo, uint32_t *pNumPasses)
+{
+  ObjDisp(physicalDevice)
+      ->GetPhysicalDeviceQueueFamilyPerformanceQueryPassesKHR(
+          Unwrap(physicalDevice), pPerformanceQueryCreateInfo, pNumPasses);
+}
+
+VkResult WrappedVulkan::vkAcquireProfilingLockKHR(VkDevice device,
+                                                  const VkAcquireProfilingLockInfoKHR *pInfo)
+{
+  return ObjDisp(device)->AcquireProfilingLockKHR(Unwrap(device), pInfo);
+}
+
+void WrappedVulkan::vkReleaseProfilingLockKHR(VkDevice device)
+{
+  ObjDisp(device)->ReleaseProfilingLockKHR(Unwrap(device));
+}
