@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,6 +22,7 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
+#include <algorithm>
 #include "../vk_core.h"
 #include "../vk_debug.h"
 
@@ -33,7 +34,7 @@ bool WrappedVulkan::Serialise_vkGetDeviceQueue(SerialiserType &ser, VkDevice dev
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT(queueFamilyIndex);
   SERIALISE_ELEMENT(queueIndex);
-  SERIALISE_ELEMENT_LOCAL(Queue, GetResID(*pQueue)).TypedAs("VkQueue");
+  SERIALISE_ELEMENT_LOCAL(Queue, GetResID(*pQueue)).TypedAs("VkQueue"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -50,8 +51,18 @@ bool WrappedVulkan::Serialise_vkGetDeviceQueue(SerialiserType &ser, VkDevice dev
 
     ObjDisp(device)->GetDeviceQueue(Unwrap(device), remapFamily, remapIndex, &queue);
 
-    GetResourceManager()->WrapResource(Unwrap(device), queue);
-    GetResourceManager()->AddLiveResource(Queue, queue);
+    if(GetResourceManager()->HasWrapper(ToTypedHandle(queue)))
+    {
+      ResourceId live = GetResourceManager()->GetDispWrapper(queue)->id;
+
+      // whenever the new ID is requested, return the old ID, via replacements.
+      GetResourceManager()->ReplaceResource(Queue, GetResourceManager()->GetOriginalID(live));
+    }
+    else
+    {
+      GetResourceManager()->WrapResource(Unwrap(device), queue);
+      GetResourceManager()->AddLiveResource(Queue, queue);
+    }
 
     if(remapFamily == m_QueueFamilyIdx && m_Queue == VK_NULL_HANDLE)
     {
@@ -159,6 +170,13 @@ void WrappedVulkan::vkGetDeviceQueue(VkDevice device, uint32_t queueFamilyIndex,
   }
 }
 
+static void appendChain(VkBaseInStructure *chain, void *item)
+{
+  while(chain->pNext != NULL)
+    chain = (VkBaseInStructure *)chain->pNext;
+  chain->pNext = (VkBaseInStructure *)item;
+}
+
 template <typename SerialiserType>
 bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue, uint32_t submitCount,
                                             const VkSubmitInfo *pSubmits, VkFence fence)
@@ -178,8 +196,8 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
     // before executing this, as we don't have the sync information to properly sync.
     if(m_PrevQueue != queue)
     {
-      RDCDEBUG("Previous queue execution was on queue %llu, now executing %llu, syncing GPU",
-               GetResID(m_PrevQueue), GetResID(queue));
+      RDCDEBUG("Previous queue execution was on queue %s, now executing %s, syncing GPU",
+               ToStr(GetResID(m_PrevQueue)).c_str(), ToStr(GetResID(queue)).c_str());
       if(m_PrevQueue != VK_NULL_HANDLE)
         ObjDisp(m_PrevQueue)->QueueWaitIdle(Unwrap(m_PrevQueue));
 
@@ -202,7 +220,7 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
 
       // insert in sorted location
       auto drawit = std::lower_bound(m_DrawcallUses.begin(), m_DrawcallUses.end(), use);
-      m_DrawcallUses.insert(drawit, use);
+      m_DrawcallUses.insert(drawit - m_DrawcallUses.begin(), use);
     }
 
     for(uint32_t sub = 0; sub < submitCount; sub++)
@@ -232,6 +250,7 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
         tempMem += unwrapped.commandBufferCount * sizeof(VkCommandBuffer);
 
         UnwrapNextChain(m_State, "VkSubmitInfo", tempMem, (VkBaseInStructure *)&unwrapped);
+        appendChain((VkBaseInStructure *)&unwrapped, m_SubmitChain);
 
         ObjDisp(queue)->QueueSubmit(Unwrap(queue), 1, &unwrapped, VK_NULL_HANDLE);
 
@@ -240,25 +259,26 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
         // we're adding multiple events, need to increment ourselves
         m_RootEventID++;
 
-        std::string basename = StringFormat::Fmt("vkQueueSubmit(%u)", submitInfo.commandBufferCount);
+        rdcstr basename = StringFormat::Fmt("vkQueueSubmit(%u)", submitInfo.commandBufferCount);
 
         for(uint32_t c = 0; c < submitInfo.commandBufferCount; c++)
         {
-          ResourceId cmd =
-              GetResourceManager()->GetOriginalID(GetResID(submitInfo.pCommandBuffers[c]));
+          ResourceId liveCmd = GetResID(submitInfo.pCommandBuffers[c]);
+          ResourceId cmd = GetResourceManager()->GetOriginalID(liveCmd);
 
           BakedCmdBufferInfo &cmdBufInfo = m_BakedCmdBufferInfo[cmd];
 
           GetResourceManager()->ApplyBarriers(m_CreationInfo.m_Queue[GetResID(queue)],
-                                              m_BakedCmdBufferInfo[cmd].imgbarriers, m_ImageLayouts);
+                                              m_BakedCmdBufferInfo[liveCmd].imgbarriers,
+                                              m_ImageLayouts);
 
-          std::string name = StringFormat::Fmt("=> %s[%u]: vkBeginCommandBuffer(%s)",
-                                               basename.c_str(), c, ToStr(cmd).c_str());
+          rdcstr name = StringFormat::Fmt("=> %s[%u]: vkBeginCommandBuffer(%s)", basename.c_str(),
+                                          c, ToStr(cmd).c_str());
 
           // add a fake marker
           DrawcallDescription draw;
           draw.name = name;
-          draw.flags |= DrawFlags::SetMarker;
+          draw.flags |= DrawFlags::PassBoundary | DrawFlags::BeginPass;
           AddEvent();
 
           m_RootEvents.back().chunkIndex = cmdBufInfo.beginChunk;
@@ -273,7 +293,7 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
 
           for(size_t e = 0; e < cmdBufInfo.draw->executedCmds.size(); e++)
           {
-            std::vector<Submission> &submits =
+            rdcarray<Submission> &submits =
                 m_Partial[Secondary].cmdBufferSubmits[cmdBufInfo.draw->executedCmds[e]];
 
             for(size_t s = 0; s < submits.size(); s++)
@@ -301,6 +321,7 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
           name = StringFormat::Fmt("=> %s[%u]: vkEndCommandBuffer(%s)", basename.c_str(), c,
                                    ToStr(cmd).c_str());
           draw.name = name;
+          draw.flags = DrawFlags::PassBoundary | DrawFlags::EndPass;
           AddEvent();
 
           m_RootEvents.back().chunkIndex = cmdBufInfo.endChunk;
@@ -354,7 +375,7 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
 
           uint32_t eid = startEID;
 
-          std::vector<VkCommandBuffer> rerecordedCmds;
+          rdcarray<VkCommandBuffer> rerecordedCmds;
 
           for(uint32_t c = 0; c < submitInfo.commandBufferCount; c++)
           {
@@ -374,8 +395,8 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
               VkCommandBuffer cmd = RerecordCmdBuf(cmdId);
               ResourceId rerecord = GetResID(cmd);
 #if ENABLED(VERBOSE_PARTIAL_REPLAY)
-              RDCDEBUG("Queue Submit re-recorded replay of %llu, using %llu (%u -> %u <= %u)",
-                       cmdId, rerecord, eid, end, m_LastEventID);
+              RDCDEBUG("Queue Submit re-recorded replay of %s, using %s (%u -> %u <= %u)",
+                       ToStr(cmdId).c_str(), ToStr(rerecord).c_str(), eid, end, m_LastEventID);
 #endif
               rerecordedCmds.push_back(Unwrap(cmd));
 
@@ -386,7 +407,7 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
             else
             {
 #if ENABLED(VERBOSE_PARTIAL_REPLAY)
-              RDCDEBUG("Queue not submitting %llu", cmdId);
+              RDCDEBUG("Queue not submitting %s", ToStr(cmdId).c_str());
 #endif
             }
 
@@ -400,13 +421,14 @@ bool WrappedVulkan::Serialise_vkQueueSubmit(SerialiserType &ser, VkQueue queue,
           byte *tempMem = GetTempMemory(GetNextPatchSize(rerecordedSubmit.pNext));
 
           UnwrapNextChain(m_State, "VkSubmitInfo", tempMem, (VkBaseInStructure *)&rerecordedSubmit);
+          appendChain((VkBaseInStructure *)&rerecordedSubmit, m_SubmitChain);
 
           rerecordedSubmit.commandBufferCount = (uint32_t)rerecordedCmds.size();
           rerecordedSubmit.pCommandBuffers = &rerecordedCmds[0];
 
 #if ENABLED(SINGLE_FLUSH_VALIDATE)
           rerecordedSubmit.commandBufferCount = 1;
-          for(uint32_t i = 0; i < rerecordedSubmit.commandBufferCount; i++)
+          for(size_t i = 0; i < rerecordedCmds.size(); i++)
           {
             ObjDisp(queue)->QueueSubmit(Unwrap(queue), 1, &rerecordedSubmit, VK_NULL_HANDLE);
             rerecordedSubmit.pCommandBuffers++;
@@ -517,7 +539,7 @@ bool WrappedVulkan::PatchIndirectDraw(VkIndirectPatchType type, DrawcallDescript
 
 void WrappedVulkan::InsertDrawsAndRefreshIDs(BakedCmdBufferInfo &cmdBufInfo)
 {
-  vector<VulkanDrawcallTreeNode> &cmdBufNodes = cmdBufInfo.draw->children;
+  rdcarray<VulkanDrawcallTreeNode> &cmdBufNodes = cmdBufInfo.draw->children;
 
   // assign new drawcall IDs
   for(size_t i = 0; i < cmdBufNodes.size(); i++)
@@ -595,10 +617,7 @@ void WrappedVulkan::InsertDrawsAndRefreshIDs(BakedCmdBufferInfo &cmdBufInfo)
         uint32_t shiftCount = n.indirectPatch.count - indirectCount;
 
         // i is the pushmarker, so i + 1 is the first of the sub draws.
-        // i + 1 + n.indirectPatch.count is the last of the draws, we don't want to erase the next
-        // one (the popmarker)
-        cmdBufNodes.erase(cmdBufNodes.begin() + i + 1 + indirectCount,
-                          cmdBufNodes.begin() + i + 1 + n.indirectPatch.count);
+        cmdBufNodes.erase(i + 1 + indirectCount, shiftCount);
         for(size_t j = i + 1 + indirectCount; j < cmdBufNodes.size(); j++)
         {
           cmdBufNodes[j].draw.eventId -= shiftCount;
@@ -616,6 +635,18 @@ void WrappedVulkan::InsertDrawsAndRefreshIDs(BakedCmdBufferInfo &cmdBufInfo)
           if(cmdBufInfo.debugMessages[j].eventId >= cmdBufNodes[i].draw.eventId + indirectCount + 2)
             cmdBufInfo.debugMessages[j].eventId -= shiftCount;
         }
+
+        for(size_t e = 0; e < cmdBufInfo.draw->executedCmds.size(); e++)
+        {
+          rdcarray<Submission> &submits =
+              m_Partial[Secondary].cmdBufferSubmits[cmdBufInfo.draw->executedCmds[e]];
+
+          for(size_t s = 0; s < submits.size(); s++)
+          {
+            if(submits[s].baseEvent >= cmdBufNodes[i].draw.eventId + indirectCount + 2)
+              submits[s].baseEvent -= shiftCount;
+          }
+        }
       }
 
       // indirect count versions always have a multidraw marker regions, but static count of 1 would
@@ -675,7 +706,8 @@ void WrappedVulkan::InsertDrawsAndRefreshIDs(BakedCmdBufferInfo &cmdBufInfo)
     for(APIEvent &ev : n.draw.events)
     {
       ev.eventId += m_RootEventID;
-      m_Events.push_back(ev);
+      m_Events.resize(ev.eventId + 1);
+      m_Events[ev.eventId] = ev;
     }
 
     if(!n.draw.events.empty())
@@ -684,7 +716,7 @@ void WrappedVulkan::InsertDrawsAndRefreshIDs(BakedCmdBufferInfo &cmdBufInfo)
 
       // insert in sorted location
       auto drawit = std::lower_bound(m_DrawcallUses.begin(), m_DrawcallUses.end(), use);
-      m_DrawcallUses.insert(drawit, use);
+      m_DrawcallUses.insert(drawit - m_DrawcallUses.begin(), use);
     }
 
     RDCASSERT(n.children.empty());
@@ -694,6 +726,7 @@ void WrappedVulkan::InsertDrawsAndRefreshIDs(BakedCmdBufferInfo &cmdBufInfo)
       EventUsage u = it->second;
       u.eventId += m_RootEventID;
       m_ResourceUses[it->first].push_back(u);
+      m_EventFlags[u.eventId] |= PipeRWUsageEventFlags(u.usage);
     }
 
     GetDrawcallStack().back()->children.push_back(n);
@@ -772,269 +805,257 @@ VkResult WrappedVulkan::vkQueueSubmit(VkQueue queue, uint32_t submitCount,
       unwrappedSignalSems[o] = Unwrap(pSubmits[i].pSignalSemaphores[o]);
 
     UnwrapNextChain(m_State, "VkSubmitInfo", memory, (VkBaseInStructure *)&unwrappedSubmits[i]);
+    appendChain((VkBaseInStructure *)&unwrappedSubmits[i], m_SubmitChain);
   }
 
   VkResult ret;
   SERIALISE_TIME_CALL(ret = ObjDisp(queue)->QueueSubmit(Unwrap(queue), submitCount,
                                                         unwrappedSubmits, Unwrap(fence)));
 
-  bool capframe = false;
   bool present = false;
 
-  set<ResourceId> refdIDs;
-
-  VkResourceRecord *queueRecord = GetRecord(queue);
-
-  for(uint32_t s = 0; s < submitCount; s++)
   {
-    for(uint32_t i = 0; i < pSubmits[s].commandBufferCount; i++)
+    SCOPED_READLOCK(m_CapTransitionLock);
+
+    bool capframe = IsActiveCapturing(m_State);
+
+    std::set<ResourceId> refdIDs;
+
+    VkResourceRecord *queueRecord = GetRecord(queue);
+
+    for(uint32_t s = 0; s < submitCount; s++)
     {
-      ResourceId cmd = GetResID(pSubmits[s].pCommandBuffers[i]);
-
-      VkResourceRecord *record = GetRecord(pSubmits[s].pCommandBuffers[i]);
-      present |= record->bakedCommands->cmdInfo->present;
-
+      for(uint32_t i = 0; i < pSubmits[s].commandBufferCount; i++)
       {
-        SCOPED_LOCK(m_ImageLayoutsLock);
-        GetResourceManager()->ApplyBarriers(queueRecord->queueFamilyIndex,
-                                            record->bakedCommands->cmdInfo->imgbarriers,
-                                            m_ImageLayouts);
-      }
+        ResourceId cmd = GetResID(pSubmits[s].pCommandBuffers[i]);
+
+        VkResourceRecord *record = GetRecord(pSubmits[s].pCommandBuffers[i]);
+        present |= record->bakedCommands->cmdInfo->present;
 
-      // need to lock the whole section of code, not just the check on
-      // m_State, as we also need to make sure we don't check the state,
-      // start marking dirty resources then while we're doing so the
-      // state becomes capframe.
-      // the next sections where we mark resources referenced and add
-      // the submit chunk to the frame record don't have to be protected.
-      // Only the decision of whether we're inframe or not, and marking
-      // dirty.
-      {
-        SCOPED_LOCK(m_CapTransitionLock);
-        if(IsActiveCapturing(m_State))
         {
-          for(auto it = record->bakedCommands->cmdInfo->dirtied.begin();
-              it != record->bakedCommands->cmdInfo->dirtied.end(); ++it)
-          {
-            if(GetResourceManager()->HasCurrentResource(*it))
-              GetResourceManager()->MarkPendingDirty(*it);
-          }
-
-          // with EXT_descriptor_indexing a binding might have been updated after
-          // vkCmdBindDescriptorSets, so we need to track dirtied here at the last second.
-          for(auto it = record->bakedCommands->cmdInfo->boundDescSets.begin();
-              it != record->bakedCommands->cmdInfo->boundDescSets.end(); ++it)
-          {
-            VkResourceRecord *setrecord = GetRecord(*it);
-
-            SCOPED_LOCK(setrecord->descInfo->refLock);
-
-            const std::map<ResourceId, pair<uint32_t, FrameRefType>> &frameRefs =
-                setrecord->descInfo->bindFrameRefs;
-
-            for(auto refit = frameRefs.begin(); refit != frameRefs.end(); ++refit)
-            {
-              if(refit->second.second == eFrameRef_PartialWrite ||
-                 refit->second.second == eFrameRef_ReadBeforeWrite)
-              {
-                if(GetResourceManager()->HasCurrentResource(refit->first))
-                  GetResourceManager()->MarkPendingDirty(refit->first);
-              }
-            }
-          }
-
-          capframe = true;
+          SCOPED_LOCK(m_ImageLayoutsLock);
+          GetResourceManager()->ApplyBarriers(queueRecord->queueFamilyIndex,
+                                              record->bakedCommands->cmdInfo->imgbarriers,
+                                              m_ImageLayouts);
         }
-        else
+
+        for(auto it = record->bakedCommands->cmdInfo->dirtied.begin();
+            it != record->bakedCommands->cmdInfo->dirtied.end(); ++it)
         {
-          for(auto it = record->bakedCommands->cmdInfo->dirtied.begin();
-              it != record->bakedCommands->cmdInfo->dirtied.end(); ++it)
-          {
-            if(GetResourceManager()->HasCurrentResource(*it))
-              GetResourceManager()->MarkDirtyResource(*it);
-          }
+          if(GetResourceManager()->HasCurrentResource(*it))
+            GetResourceManager()->MarkDirtyResource(*it);
         }
-      }
 
-      if(capframe)
-      {
-        // for each bound descriptor set, mark it referenced as well as all resources currently
-        // bound to it
+        // with EXT_descriptor_indexing a binding might have been updated after
+        // vkCmdBindDescriptorSets, so we need to track dirtied here at the last second.
         for(auto it = record->bakedCommands->cmdInfo->boundDescSets.begin();
             it != record->bakedCommands->cmdInfo->boundDescSets.end(); ++it)
         {
-          GetResourceManager()->MarkResourceFrameReferenced(GetResID(*it), eFrameRef_Read);
-
           VkResourceRecord *setrecord = GetRecord(*it);
 
           SCOPED_LOCK(setrecord->descInfo->refLock);
 
-          for(auto refit = setrecord->descInfo->bindFrameRefs.begin();
-              refit != setrecord->descInfo->bindFrameRefs.end(); ++refit)
+          const std::map<ResourceId, rdcpair<uint32_t, FrameRefType>> &frameRefs =
+              setrecord->descInfo->bindFrameRefs;
+
+          for(auto refit = frameRefs.begin(); refit != frameRefs.end(); ++refit)
           {
-            refdIDs.insert(refit->first);
-            GetResourceManager()->MarkResourceFrameReferenced(refit->first, refit->second.second);
-
-            if(refit->second.first & DescriptorSetData::SPARSE_REF_BIT)
+            if(refit->second.second == eFrameRef_PartialWrite ||
+               refit->second.second == eFrameRef_ReadBeforeWrite)
             {
-              VkResourceRecord *sparserecord = GetResourceManager()->GetResourceRecord(refit->first);
-
-              GetResourceManager()->MarkSparseMapReferenced(sparserecord->resInfo);
+              if(GetResourceManager()->HasCurrentResource(refit->first))
+                GetResourceManager()->MarkDirtyResource(refit->first);
             }
           }
-          GetResourceManager()->MergeReferencedMemory(setrecord->descInfo->bindMemRefs);
         }
 
-        for(auto it = record->bakedCommands->cmdInfo->sparse.begin();
-            it != record->bakedCommands->cmdInfo->sparse.end(); ++it)
-          GetResourceManager()->MarkSparseMapReferenced(*it);
-
-        // pull in frame refs from this baked command buffer
-        record->bakedCommands->AddResourceReferences(GetResourceManager());
-        record->bakedCommands->AddReferencedIDs(refdIDs);
-
-        GetResourceManager()->MergeReferencedMemory(record->bakedCommands->cmdInfo->memFrameRefs);
-
-        // ref the parent command buffer's alloc record, this will pull in the cmd buffer pool
-        GetResourceManager()->MarkResourceFrameReferenced(
-            record->cmdInfo->allocRecord->GetResourceID(), eFrameRef_Read);
-
-        for(size_t sub = 0; sub < record->bakedCommands->cmdInfo->subcmds.size(); sub++)
+        if(capframe)
         {
-          record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands->AddResourceReferences(
-              GetResourceManager());
-          record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands->AddReferencedIDs(refdIDs);
-          GetResourceManager()->MergeReferencedMemory(
-              record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands->cmdInfo->memFrameRefs);
+          // for each bound descriptor set, mark it referenced as well as all resources currently
+          // bound to it
+          for(auto it = record->bakedCommands->cmdInfo->boundDescSets.begin();
+              it != record->bakedCommands->cmdInfo->boundDescSets.end(); ++it)
+          {
+            GetResourceManager()->MarkResourceFrameReferenced(GetResID(*it), eFrameRef_Read);
+
+            VkResourceRecord *setrecord = GetRecord(*it);
+
+            SCOPED_LOCK(setrecord->descInfo->refLock);
+
+            for(auto refit = setrecord->descInfo->bindFrameRefs.begin();
+                refit != setrecord->descInfo->bindFrameRefs.end(); ++refit)
+            {
+              refdIDs.insert(refit->first);
+              GetResourceManager()->MarkResourceFrameReferenced(refit->first, refit->second.second);
+
+              if(refit->second.first & DescriptorSetData::SPARSE_REF_BIT)
+              {
+                VkResourceRecord *sparserecord =
+                    GetResourceManager()->GetResourceRecord(refit->first);
+
+                GetResourceManager()->MarkSparseMapReferenced(sparserecord->resInfo);
+              }
+            }
+            GetResourceManager()->MergeReferencedImages(setrecord->descInfo->bindImgRefs);
+            GetResourceManager()->MergeReferencedMemory(setrecord->descInfo->bindMemRefs);
+          }
+
+          for(auto it = record->bakedCommands->cmdInfo->sparse.begin();
+              it != record->bakedCommands->cmdInfo->sparse.end(); ++it)
+            GetResourceManager()->MarkSparseMapReferenced(*it);
+
+          // pull in frame refs from this baked command buffer
+          record->bakedCommands->AddResourceReferences(GetResourceManager());
+          record->bakedCommands->AddReferencedIDs(refdIDs);
+
+          GetResourceManager()->MergeReferencedImages(record->bakedCommands->cmdInfo->imgFrameRefs);
+          GetResourceManager()->MergeReferencedMemory(record->bakedCommands->cmdInfo->memFrameRefs);
+
+          // ref the parent command buffer's alloc record, this will pull in the cmd buffer pool
           GetResourceManager()->MarkResourceFrameReferenced(
-              record->bakedCommands->cmdInfo->subcmds[sub]->cmdInfo->allocRecord->GetResourceID(),
-              eFrameRef_Read);
+              record->cmdInfo->allocRecord->GetResourceID(), eFrameRef_Read);
 
-          record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands->AddRef();
-        }
-
-        {
-          SCOPED_LOCK(m_CmdBufferRecordsLock);
-          m_CmdBufferRecords.push_back(record->bakedCommands);
           for(size_t sub = 0; sub < record->bakedCommands->cmdInfo->subcmds.size(); sub++)
-            m_CmdBufferRecords.push_back(record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands);
+          {
+            record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands->AddResourceReferences(
+                GetResourceManager());
+            record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands->AddReferencedIDs(refdIDs);
+            GetResourceManager()->MergeReferencedImages(
+                record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands->cmdInfo->imgFrameRefs);
+            GetResourceManager()->MergeReferencedMemory(
+                record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands->cmdInfo->memFrameRefs);
+            GetResourceManager()->MarkResourceFrameReferenced(
+                record->bakedCommands->cmdInfo->subcmds[sub]->cmdInfo->allocRecord->GetResourceID(),
+                eFrameRef_Read);
+
+            record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands->AddRef();
+          }
+
+          {
+            SCOPED_LOCK(m_CmdBufferRecordsLock);
+            m_CmdBufferRecords.push_back(record->bakedCommands);
+            for(size_t sub = 0; sub < record->bakedCommands->cmdInfo->subcmds.size(); sub++)
+              m_CmdBufferRecords.push_back(
+                  record->bakedCommands->cmdInfo->subcmds[sub]->bakedCommands);
+          }
+
+          record->bakedCommands->AddRef();
         }
 
-        record->bakedCommands->AddRef();
+        record->cmdInfo->dirtied.clear();
+      }
+    }
+
+    if(capframe)
+    {
+      GetResourceManager()->MarkResourceFrameReferenced(GetResID(queue), eFrameRef_Read);
+
+      if(fence != VK_NULL_HANDLE)
+        GetResourceManager()->MarkResourceFrameReferenced(GetResID(fence), eFrameRef_Read);
+
+      rdcarray<VkResourceRecord *> maps;
+      {
+        SCOPED_LOCK(m_CoherentMapsLock);
+        maps = m_CoherentMaps;
       }
 
-      record->cmdInfo->dirtied.clear();
-    }
-  }
-
-  if(capframe)
-  {
-    GetResourceManager()->MarkResourceFrameReferenced(GetResID(queue), eFrameRef_Read);
-
-    if(fence != VK_NULL_HANDLE)
-      GetResourceManager()->MarkResourceFrameReferenced(GetResID(fence), eFrameRef_Read);
-
-    std::vector<VkResourceRecord *> maps;
-    {
-      SCOPED_LOCK(m_CoherentMapsLock);
-      maps = m_CoherentMaps;
-    }
-
-    for(auto it = maps.begin(); it != maps.end(); ++it)
-    {
-      VkResourceRecord *record = *it;
-      MemMapState &state = *record->memMapState;
-
-      // potential persistent map
-      if(state.mapCoherent && state.mappedPtr && !state.mapFlushed)
+      for(auto it = maps.begin(); it != maps.end(); ++it)
       {
-        // only need to flush memory that could affect this submitted batch of work
-        if(refdIDs.find(record->GetResourceID()) == refdIDs.end())
-        {
-          RDCDEBUG("Map of memory %llu not referenced in this queue - not flushing",
-                   record->GetResourceID());
-          continue;
-        }
+        VkResourceRecord *record = *it;
+        MemMapState &state = *record->memMapState;
 
-        size_t diffStart = 0, diffEnd = 0;
-        bool found = true;
+        // potential persistent map
+        if(state.mapCoherent && state.mappedPtr && !state.mapFlushed)
+        {
+          // only need to flush memory that could affect this submitted batch of work
+          if(refdIDs.find(record->GetResourceID()) == refdIDs.end())
+          {
+            RDCDEBUG("Map of memory %s not referenced in this queue - not flushing",
+                     ToStr(record->GetResourceID()).c_str());
+            continue;
+          }
+
+          size_t diffStart = 0, diffEnd = 0;
+          bool found = true;
 
 // enabled as this is necessary for programs with very large coherent mappings
 // (> 1GB) as otherwise more than a couple of vkQueueSubmit calls leads to vast
 // memory allocation. There might still be bugs lurking in here though
 #if 1
-        // this causes vkFlushMappedMemoryRanges call to allocate and copy to refData
-        // from serialised buffer. We want to copy *precisely* the serialised data,
-        // otherwise there is a gap in time between serialising out a snapshot of
-        // the buffer and whenever we then copy into the ref data, e.g. below.
-        // during this time, data could be written to the buffer and it won't have
-        // been caught in the serialised snapshot, and if it doesn't change then
-        // it *also* won't be caught in any future FindDiffRange() calls.
-        //
-        // Likewise once refData is allocated, the call below will also update it
-        // with the data serialised out for the same reason.
-        //
-        // Note: it's still possible that data is being written to by the
-        // application while it's being serialised out in the snapshot below. That
-        // is OK, since the application is responsible for ensuring it's not writing
-        // data that would be needed by the GPU in this submit. As long as the
-        // refdata we use for future use is identical to what was serialised, we
-        // shouldn't miss anything
-        state.needRefData = true;
+          // this causes vkFlushMappedMemoryRanges call to allocate and copy to refData
+          // from serialised buffer. We want to copy *precisely* the serialised data,
+          // otherwise there is a gap in time between serialising out a snapshot of
+          // the buffer and whenever we then copy into the ref data, e.g. below.
+          // during this time, data could be written to the buffer and it won't have
+          // been caught in the serialised snapshot, and if it doesn't change then
+          // it *also* won't be caught in any future FindDiffRange() calls.
+          //
+          // Likewise once refData is allocated, the call below will also update it
+          // with the data serialised out for the same reason.
+          //
+          // Note: it's still possible that data is being written to by the
+          // application while it's being serialised out in the snapshot below. That
+          // is OK, since the application is responsible for ensuring it's not writing
+          // data that would be needed by the GPU in this submit. As long as the
+          // refdata we use for future use is identical to what was serialised, we
+          // shouldn't miss anything
+          state.needRefData = true;
 
-        // if we have a previous set of data, compare.
-        // otherwise just serialise it all
-        if(state.refData)
-          found = FindDiffRange((byte *)state.mappedPtr, state.refData, (size_t)state.mapSize,
-                                diffStart, diffEnd);
-        else
+          // if we have a previous set of data, compare.
+          // otherwise just serialise it all
+          if(state.refData)
+            found = FindDiffRange((byte *)state.mappedPtr, state.refData, (size_t)state.mapSize,
+                                  diffStart, diffEnd);
+          else
 #endif
-          diffEnd = (size_t)state.mapSize;
-
-        if(found)
-        {
-          // MULTIDEVICE should find the device for this queue.
-          // MULTIDEVICE only want to flush maps associated with this queue
-          VkDevice dev = GetDev();
+            diffEnd = (size_t)state.mapSize;
 
+          if(found)
           {
-            RDCLOG("Persistent map flush forced for %llu (%llu -> %llu)", record->GetResourceID(),
-                   (uint64_t)diffStart, (uint64_t)diffEnd);
-            VkMappedMemoryRange range = {VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE, NULL,
-                                         (VkDeviceMemory)(uint64_t)record->Resource,
-                                         state.mapOffset + diffStart, diffEnd - diffStart};
-            vkFlushMappedMemoryRanges(dev, 1, &range);
-            state.mapFlushed = false;
-          }
+            // MULTIDEVICE should find the device for this queue.
+            // MULTIDEVICE only want to flush maps associated with this queue
+            VkDevice dev = GetDev();
 
-          GetResourceManager()->MarkPendingDirty(record->GetResourceID());
-        }
-        else
-        {
-          RDCDEBUG("Persistent map flush not needed for %llu", record->GetResourceID());
+            {
+              RDCLOG("Persistent map flush forced for %s (%llu -> %llu)",
+                     ToStr(record->GetResourceID()).c_str(), (uint64_t)diffStart, (uint64_t)diffEnd);
+              VkMappedMemoryRange range = {VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE, NULL,
+                                           (VkDeviceMemory)(uint64_t)record->Resource,
+                                           state.mapOffset + diffStart, diffEnd - diffStart};
+              vkFlushMappedMemoryRanges(dev, 1, &range);
+              state.mapFlushed = false;
+            }
+
+            GetResourceManager()->MarkDirtyResource(record->GetResourceID());
+          }
+          else
+          {
+            RDCDEBUG("Persistent map flush not needed for %s",
+                     ToStr(record->GetResourceID()).c_str());
+          }
         }
       }
-    }
 
-    {
-      CACHE_THREAD_SERIALISER();
+      {
+        CACHE_THREAD_SERIALISER();
 
-      ser.SetDrawChunk();
-      SCOPED_SERIALISE_CHUNK(VulkanChunk::vkQueueSubmit);
-      Serialise_vkQueueSubmit(ser, queue, submitCount, pSubmits, fence);
+        ser.SetDrawChunk();
+        SCOPED_SERIALISE_CHUNK(VulkanChunk::vkQueueSubmit);
+        Serialise_vkQueueSubmit(ser, queue, submitCount, pSubmits, fence);
 
-      m_FrameCaptureRecord->AddChunk(scope.Get());
-    }
+        m_FrameCaptureRecord->AddChunk(scope.Get());
+      }
 
-    for(uint32_t s = 0; s < submitCount; s++)
-    {
-      for(uint32_t sem = 0; sem < pSubmits[s].waitSemaphoreCount; sem++)
-        GetResourceManager()->MarkResourceFrameReferenced(
-            GetResID(pSubmits[s].pWaitSemaphores[sem]), eFrameRef_Read);
+      for(uint32_t s = 0; s < submitCount; s++)
+      {
+        for(uint32_t sem = 0; sem < pSubmits[s].waitSemaphoreCount; sem++)
+          GetResourceManager()->MarkResourceFrameReferenced(
+              GetResID(pSubmits[s].pWaitSemaphores[sem]), eFrameRef_Read);
 
-      for(uint32_t sem = 0; sem < pSubmits[s].signalSemaphoreCount; sem++)
-        GetResourceManager()->MarkResourceFrameReferenced(
-            GetResID(pSubmits[s].pSignalSemaphores[sem]), eFrameRef_Read);
+        for(uint32_t sem = 0; sem < pSubmits[s].signalSemaphoreCount; sem++)
+          GetResourceManager()->MarkResourceFrameReferenced(
+              GetResID(pSubmits[s].pSignalSemaphores[sem]), eFrameRef_Read);
+      }
     }
   }
 
@@ -1515,7 +1536,7 @@ bool WrappedVulkan::Serialise_vkGetDeviceQueue2(SerialiserType &ser, VkDevice de
 {
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(QueueInfo, *pQueueInfo);
-  SERIALISE_ELEMENT_LOCAL(Queue, GetResID(*pQueue)).TypedAs("VkQueue");
+  SERIALISE_ELEMENT_LOCAL(Queue, GetResID(*pQueue)).TypedAs("VkQueue"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
