@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
  ******************************************************************************/
 
 #include "../vk_core.h"
-#include "driver/shaders/spirv/spirv_common.h"
+#include "driver/shaders/spirv/spirv_reflect.h"
 
 template <>
 VkComputePipelineCreateInfo *WrappedVulkan::UnwrapInfos(const VkComputePipelineCreateInfo *info,
@@ -102,7 +102,8 @@ bool WrappedVulkan::Serialise_vkCreatePipelineLayout(SerialiserType &ser, VkDevi
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(PipelineLayout, GetResID(*pPipelineLayout)).TypedAs("VkPipelineLayout");
+  SERIALISE_ELEMENT_LOCAL(PipelineLayout, GetResID(*pPipelineLayout))
+      .TypedAs("VkPipelineLayout"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -212,7 +213,7 @@ bool WrappedVulkan::Serialise_vkCreateShaderModule(SerialiserType &ser, VkDevice
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(ShaderModule, GetResID(*pShaderModule)).TypedAs("VkShaderModule");
+  SERIALISE_ELEMENT_LOCAL(ShaderModule, GetResID(*pShaderModule)).TypedAs("VkShaderModule"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -316,7 +317,7 @@ bool WrappedVulkan::Serialise_vkCreatePipelineCache(SerialiserType &ser, VkDevic
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(PipelineCache, GetResID(*pPipelineCache)).TypedAs("VkPipelineCache");
+  SERIALISE_ELEMENT_LOCAL(PipelineCache, GetResID(*pPipelineCache)).TypedAs("VkPipelineCache"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -406,7 +407,7 @@ bool WrappedVulkan::Serialise_vkCreateGraphicsPipelines(
   SERIALISE_ELEMENT(count);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfos);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(Pipeline, GetResID(*pPipelines)).TypedAs("VkPipeline");
+  SERIALISE_ELEMENT_LOCAL(Pipeline, GetResID(*pPipelines)).TypedAs("VkPipeline"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -420,6 +421,13 @@ bool WrappedVulkan::Serialise_vkCreateGraphicsPipelines(
     // don't use pipeline caches on replay
     pipelineCache = VK_NULL_HANDLE;
 
+    // if we have pipeline executable properties, capture the data
+    if(GetExtensions(NULL).ext_KHR_pipeline_executable_properties)
+    {
+      CreateInfo.flags |= (VK_PIPELINE_CREATE_CAPTURE_STATISTICS_BIT_KHR |
+                           VK_PIPELINE_CREATE_CAPTURE_INTERNAL_REPRESENTATIONS_BIT_KHR);
+    }
+
     VkGraphicsPipelineCreateInfo *unwrapped = UnwrapInfos(&CreateInfo, 1);
     VkResult ret = ObjDisp(device)->CreateGraphicsPipelines(Unwrap(device), Unwrap(pipelineCache),
                                                             1, unwrapped, NULL, &pipe);
@@ -451,7 +459,7 @@ bool WrappedVulkan::Serialise_vkCreateGraphicsPipelines(
 
         VulkanCreationInfo::Pipeline &pipeInfo = m_CreationInfo.m_Pipeline[live];
 
-        pipeInfo.Init(GetResourceManager(), m_CreationInfo, &CreateInfo);
+        pipeInfo.Init(GetResourceManager(), m_CreationInfo, live, &CreateInfo);
 
         ResourceId renderPassID = GetResID(CreateInfo.renderPass);
 
@@ -547,8 +555,8 @@ VkResult WrappedVulkan::vkCreateGraphicsPipelines(VkDevice device, VkPipelineCac
             VkResourceRecord *baserecord = GetRecord(pCreateInfos[i].basePipelineHandle);
             record->AddParent(baserecord);
 
-            RDCDEBUG("Creating pipeline %llu base is %llu", record->GetResourceID(),
-                     baserecord->GetResourceID());
+            RDCDEBUG("Creating pipeline %s base is %s", ToStr(record->GetResourceID()).c_str(),
+                     ToStr(baserecord->GetResourceID()).c_str());
           }
           else if(pCreateInfos[i].basePipelineIndex != -1 &&
                   pCreateInfos[i].basePipelineIndex < (int)i)
@@ -580,7 +588,8 @@ VkResult WrappedVulkan::vkCreateGraphicsPipelines(VkDevice device, VkPipelineCac
       {
         GetResourceManager()->AddLiveResource(id, pPipelines[i]);
 
-        m_CreationInfo.m_Pipeline[id].Init(GetResourceManager(), m_CreationInfo, &pCreateInfos[i]);
+        m_CreationInfo.m_Pipeline[id].Init(GetResourceManager(), m_CreationInfo, id,
+                                           &pCreateInfos[i]);
       }
     }
   }
@@ -600,7 +609,7 @@ bool WrappedVulkan::Serialise_vkCreateComputePipelines(SerialiserType &ser, VkDe
   SERIALISE_ELEMENT(count);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfos);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(Pipeline, GetResID(*pPipelines)).TypedAs("VkPipeline");
+  SERIALISE_ELEMENT_LOCAL(Pipeline, GetResID(*pPipelines)).TypedAs("VkPipeline"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -613,6 +622,13 @@ bool WrappedVulkan::Serialise_vkCreateComputePipelines(SerialiserType &ser, VkDe
     // don't use pipeline caches on replay
     pipelineCache = VK_NULL_HANDLE;
 
+    // if we have pipeline executable properties, capture the data
+    if(GetExtensions(NULL).ext_KHR_pipeline_executable_properties)
+    {
+      CreateInfo.flags |= (VK_PIPELINE_CREATE_CAPTURE_STATISTICS_BIT_KHR |
+                           VK_PIPELINE_CREATE_CAPTURE_INTERNAL_REPRESENTATIONS_BIT_KHR);
+    }
+
     VkComputePipelineCreateInfo *unwrapped = UnwrapInfos(&CreateInfo, 1);
     VkResult ret = ObjDisp(device)->CreateComputePipelines(Unwrap(device), Unwrap(pipelineCache), 1,
                                                            unwrapped, NULL, &pipe);
@@ -642,11 +658,11 @@ bool WrappedVulkan::Serialise_vkCreateComputePipelines(SerialiserType &ser, VkDe
         live = GetResourceManager()->WrapResource(Unwrap(device), pipe);
         GetResourceManager()->AddLiveResource(Pipeline, pipe);
 
-        m_CreationInfo.m_Pipeline[live].Init(GetResourceManager(), m_CreationInfo, &CreateInfo);
+        m_CreationInfo.m_Pipeline[live].Init(GetResourceManager(), m_CreationInfo, live, &CreateInfo);
       }
     }
 
-    AddResource(Pipeline, ResourceType::PipelineState, "Graphics Pipeline");
+    AddResource(Pipeline, ResourceType::PipelineState, "Compute Pipeline");
     DerivedResource(device, Pipeline);
     if(origCache != VK_NULL_HANDLE)
       DerivedResource(origCache, Pipeline);
@@ -743,7 +759,8 @@ VkResult WrappedVulkan::vkCreateComputePipelines(VkDevice device, VkPipelineCach
       {
         GetResourceManager()->AddLiveResource(id, pPipelines[i]);
 
-        m_CreationInfo.m_Pipeline[id].Init(GetResourceManager(), m_CreationInfo, &pCreateInfos[i]);
+        m_CreationInfo.m_Pipeline[id].Init(GetResourceManager(), m_CreationInfo, id,
+                                           &pCreateInfos[i]);
       }
     }
   }
