@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -104,7 +104,7 @@ bool WrappedVulkan::Serialise_vkCreateFence(SerialiserType &ser, VkDevice device
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(Fence, GetResID(*pFence)).TypedAs("VkFence");
+  SERIALISE_ELEMENT_LOCAL(Fence, GetResID(*pFence)).TypedAs("VkFence"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -206,13 +206,26 @@ VkResult WrappedVulkan::vkGetFenceStatus(VkDevice device, VkFence fence)
 
   if(IsActiveCapturing(m_State))
   {
-    CACHE_THREAD_SERIALISER();
+    bool alreadySerialised = false;
 
-    SCOPED_SERIALISE_CHUNK(VulkanChunk::vkGetFenceStatus);
-    Serialise_vkGetFenceStatus(ser, device, fence);
+    {
+      m_FrameCaptureRecord->LockChunks();
+      alreadySerialised = (m_FrameCaptureRecord->NumChunks() > 0 &&
+                           m_FrameCaptureRecord->GetLastChunk()->GetChunkType<VulkanChunk>() ==
+                               VulkanChunk::vkGetFenceStatus);
+      m_FrameCaptureRecord->UnlockChunks();
+    }
 
-    m_FrameCaptureRecord->AddChunk(scope.Get());
-    GetResourceManager()->MarkResourceFrameReferenced(GetResID(fence), eFrameRef_Read);
+    if(!alreadySerialised)
+    {
+      CACHE_THREAD_SERIALISER();
+
+      SCOPED_SERIALISE_CHUNK(VulkanChunk::vkGetFenceStatus);
+      Serialise_vkGetFenceStatus(ser, device, fence);
+
+      m_FrameCaptureRecord->AddChunk(scope.Get());
+      GetResourceManager()->MarkResourceFrameReferenced(GetResID(fence), eFrameRef_Read);
+    }
   }
 
   return ret;
@@ -322,7 +335,7 @@ bool WrappedVulkan::Serialise_vkCreateEvent(SerialiserType &ser, VkDevice device
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(Event, GetResID(*pEvent)).TypedAs("VkEvent");
+  SERIALISE_ELEMENT_LOCAL(Event, GetResID(*pEvent)).TypedAs("VkEvent"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -492,12 +505,25 @@ VkResult WrappedVulkan::vkGetEventStatus(VkDevice device, VkEvent event)
 
   if(IsActiveCapturing(m_State))
   {
-    CACHE_THREAD_SERIALISER();
+    bool alreadySerialised = false;
 
-    SCOPED_SERIALISE_CHUNK(VulkanChunk::vkGetEventStatus);
-    Serialise_vkGetEventStatus(ser, device, event);
+    {
+      m_FrameCaptureRecord->LockChunks();
+      alreadySerialised = (m_FrameCaptureRecord->NumChunks() > 0 &&
+                           m_FrameCaptureRecord->GetLastChunk()->GetChunkType<VulkanChunk>() ==
+                               VulkanChunk::vkGetEventStatus);
+      m_FrameCaptureRecord->UnlockChunks();
+    }
 
-    m_FrameCaptureRecord->AddChunk(scope.Get());
+    if(!alreadySerialised)
+    {
+      CACHE_THREAD_SERIALISER();
+
+      SCOPED_SERIALISE_CHUNK(VulkanChunk::vkGetEventStatus);
+      Serialise_vkGetEventStatus(ser, device, event);
+
+      m_FrameCaptureRecord->AddChunk(scope.Get());
+    }
   }
 
   return ret;
@@ -512,7 +538,7 @@ bool WrappedVulkan::Serialise_vkCreateSemaphore(SerialiserType &ser, VkDevice de
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(Semaphore, GetResID(*pSemaphore)).TypedAs("VkSemaphore");
+  SERIALISE_ELEMENT_LOCAL(Semaphore, GetResID(*pSemaphore)).TypedAs("VkSemaphore"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -615,7 +641,7 @@ bool WrappedVulkan::Serialise_vkCmdSetEvent(SerialiserType &ser, VkCommandBuffer
 {
   SERIALISE_ELEMENT(commandBuffer);
   SERIALISE_ELEMENT(event);
-  SERIALISE_ELEMENT_TYPED(VkPipelineStageFlagBits, stageMask).TypedAs("VkPipelineStageFlags");
+  SERIALISE_ELEMENT_TYPED(VkPipelineStageFlagBits, stageMask).TypedAs("VkPipelineStageFlags"_lit);
 
   Serialise_DebugMessages(ser);
 
@@ -670,7 +696,7 @@ bool WrappedVulkan::Serialise_vkCmdResetEvent(SerialiserType &ser, VkCommandBuff
 {
   SERIALISE_ELEMENT(commandBuffer);
   SERIALISE_ELEMENT(event);
-  SERIALISE_ELEMENT_TYPED(VkPipelineStageFlagBits, stageMask).TypedAs("VkPipelineStageFlags");
+  SERIALISE_ELEMENT_TYPED(VkPipelineStageFlagBits, stageMask).TypedAs("VkPipelineStageFlags"_lit);
 
   Serialise_DebugMessages(ser);
 
@@ -735,8 +761,10 @@ bool WrappedVulkan::Serialise_vkCmdWaitEvents(
   SERIALISE_ELEMENT(eventCount);
   SERIALISE_ELEMENT_ARRAY(pEvents, eventCount);
 
-  SERIALISE_ELEMENT_TYPED(VkPipelineStageFlagBits, srcStageMask).TypedAs("VkPipelineStageFlags");
-  SERIALISE_ELEMENT_TYPED(VkPipelineStageFlagBits, dstStageMask).TypedAs("VkPipelineStageFlags");
+  SERIALISE_ELEMENT_TYPED(VkPipelineStageFlagBits, srcStageMask)
+      .TypedAs("VkPipelineStageFlags"_lit);
+  SERIALISE_ELEMENT_TYPED(VkPipelineStageFlagBits, dstStageMask)
+      .TypedAs("VkPipelineStageFlags"_lit);
 
   SERIALISE_ELEMENT(memoryBarrierCount);
   SERIALISE_ELEMENT_ARRAY(pMemoryBarriers, memoryBarrierCount);
@@ -747,8 +775,8 @@ bool WrappedVulkan::Serialise_vkCmdWaitEvents(
 
   SERIALISE_CHECK_READ_ERRORS();
 
-  std::vector<VkImageMemoryBarrier> imgBarriers;
-  std::vector<VkBufferMemoryBarrier> bufBarriers;
+  rdcarray<VkImageMemoryBarrier> imgBarriers;
+  rdcarray<VkBufferMemoryBarrier> bufBarriers;
 
   // it's possible for buffer or image to be NULL if it refers to a resource that is otherwise
   // not in the log (barriers do not mark resources referenced). If the resource in question does
@@ -777,8 +805,6 @@ bool WrappedVulkan::Serialise_vkCmdWaitEvents(
       {
         imgBarriers.push_back(pImageMemoryBarriers[i]);
         imgBarriers.back().image = Unwrap(imgBarriers.back().image);
-        ReplacePresentableImageLayout(imgBarriers.back().oldLayout);
-        ReplacePresentableImageLayout(imgBarriers.back().newLayout);
 
         RemapQueueFamilyIndices(imgBarriers.back().srcQueueFamilyIndex,
                                 imgBarriers.back().dstQueueFamilyIndex);
@@ -814,18 +840,25 @@ bool WrappedVulkan::Serialise_vkCmdWaitEvents(
       m_PersistentEvents.push_back(ev);
     }
 
+    ResourceId cmd = GetResID(commandBuffer);
+    GetResourceManager()->RecordBarriers(m_BakedCmdBufferInfo[cmd].imgbarriers, m_ImageLayouts,
+                                         (uint32_t)imgBarriers.size(), &imgBarriers[0]);
+
     if(commandBuffer != VK_NULL_HANDLE)
     {
+      // now sanitise layouts before passing to vulkan
+      for(VkImageMemoryBarrier &barrier : imgBarriers)
+      {
+        SanitiseOldImageLayout(barrier.oldLayout);
+        SanitiseNewImageLayout(barrier.newLayout);
+      }
+
       ObjDisp(commandBuffer)->CmdSetEvent(Unwrap(commandBuffer), ev, VK_PIPELINE_STAGE_ALL_COMMANDS_BIT);
       ObjDisp(commandBuffer)
           ->CmdWaitEvents(Unwrap(commandBuffer), 1, &ev, srcStageMask, dstStageMask,
                           memoryBarrierCount, pMemoryBarriers, (uint32_t)bufBarriers.size(),
                           bufBarriers.data(), (uint32_t)imgBarriers.size(), imgBarriers.data());
     }
-
-    ResourceId cmd = GetResID(commandBuffer);
-    GetResourceManager()->RecordBarriers(m_BakedCmdBufferInfo[cmd].imgbarriers, m_ImageLayouts,
-                                         (uint32_t)imgBarriers.size(), &imgBarriers[0]);
   }
 
   return true;
@@ -930,6 +963,157 @@ VkResult WrappedVulkan::vkGetFenceFdKHR(VkDevice device, const VkFenceGetFdInfoK
   return ObjDisp(device)->GetFenceFdKHR(Unwrap(device), &unwrappedInfo, pFd);
 }
 
+template <typename SerialiserType>
+bool WrappedVulkan::Serialise_vkGetSemaphoreCounterValue(SerialiserType &ser, VkDevice device,
+                                                         VkSemaphore semaphore, uint64_t *pValue)
+{
+  SERIALISE_ELEMENT(device);
+  SERIALISE_ELEMENT(semaphore);
+  SERIALISE_ELEMENT_OPT(pValue);
+
+  Serialise_DebugMessages(ser);
+
+  SERIALISE_CHECK_READ_ERRORS();
+
+  if(IsReplayingAndReading())
+  {
+    ObjDisp(device)->DeviceWaitIdle(Unwrap(device));
+  }
+
+  return true;
+}
+
+VkResult WrappedVulkan::vkGetSemaphoreCounterValue(VkDevice device, VkSemaphore semaphore,
+                                                   uint64_t *pValue)
+{
+  SCOPED_DBG_SINK();
+
+  VkResult ret;
+  SERIALISE_TIME_CALL(
+      ret = ObjDisp(device)->GetSemaphoreCounterValue(Unwrap(device), Unwrap(semaphore), pValue));
+
+  if(IsActiveCapturing(m_State))
+  {
+    bool alreadySerialised = false;
+
+    {
+      m_FrameCaptureRecord->LockChunks();
+      alreadySerialised = (m_FrameCaptureRecord->NumChunks() > 0 &&
+                           m_FrameCaptureRecord->GetLastChunk()->GetChunkType<VulkanChunk>() ==
+                               VulkanChunk::vkGetSemaphoreCounterValue);
+      m_FrameCaptureRecord->UnlockChunks();
+    }
+
+    if(!alreadySerialised)
+    {
+      CACHE_THREAD_SERIALISER();
+
+      SCOPED_SERIALISE_CHUNK(VulkanChunk::vkGetSemaphoreCounterValue);
+      Serialise_vkGetSemaphoreCounterValue(ser, device, semaphore, pValue);
+
+      m_FrameCaptureRecord->AddChunk(scope.Get());
+      GetResourceManager()->MarkResourceFrameReferenced(GetResID(semaphore), eFrameRef_Read);
+    }
+  }
+
+  return ret;
+}
+
+template <typename SerialiserType>
+bool WrappedVulkan::Serialise_vkWaitSemaphores(SerialiserType &ser, VkDevice device,
+                                               const VkSemaphoreWaitInfo *pWaitInfo, uint64_t timeout)
+{
+  SERIALISE_ELEMENT(device);
+  SERIALISE_ELEMENT_LOCAL(WaitInfo, *pWaitInfo);
+  SERIALISE_ELEMENT(timeout);
+
+  Serialise_DebugMessages(ser);
+
+  SERIALISE_CHECK_READ_ERRORS();
+
+  if(IsReplayingAndReading())
+  {
+    ObjDisp(device)->DeviceWaitIdle(Unwrap(device));
+  }
+
+  return true;
+}
+
+VkResult WrappedVulkan::vkWaitSemaphores(VkDevice device, const VkSemaphoreWaitInfo *pWaitInfo,
+                                         uint64_t timeout)
+{
+  SCOPED_DBG_SINK();
+
+  VkSemaphore *unwrappedSems = GetTempArray<VkSemaphore>(pWaitInfo->semaphoreCount);
+  for(uint32_t i = 0; i < pWaitInfo->semaphoreCount; i++)
+    unwrappedSems[i] = Unwrap(pWaitInfo->pSemaphores[i]);
+
+  VkSemaphoreWaitInfo unwrapped = *pWaitInfo;
+  unwrapped.pSemaphores = unwrappedSems;
+
+  VkResult ret;
+  SERIALISE_TIME_CALL(ret = ObjDisp(device)->WaitSemaphores(Unwrap(device), &unwrapped, timeout));
+
+  if(IsActiveCapturing(m_State))
+  {
+    CACHE_THREAD_SERIALISER();
+
+    SCOPED_SERIALISE_CHUNK(VulkanChunk::vkWaitSemaphores);
+    Serialise_vkWaitSemaphores(ser, device, pWaitInfo, timeout);
+
+    m_FrameCaptureRecord->AddChunk(scope.Get());
+    for(uint32_t i = 0; i < pWaitInfo->semaphoreCount; i++)
+      GetResourceManager()->MarkResourceFrameReferenced(GetResID(pWaitInfo->pSemaphores[i]),
+                                                        eFrameRef_Read);
+  }
+
+  return ret;
+}
+
+template <typename SerialiserType>
+bool WrappedVulkan::Serialise_vkSignalSemaphore(SerialiserType &ser, VkDevice device,
+                                                const VkSemaphoreSignalInfo *pSignalInfo)
+{
+  SERIALISE_ELEMENT(device);
+  SERIALISE_ELEMENT_LOCAL(SignalInfo, *pSignalInfo);
+
+  Serialise_DebugMessages(ser);
+
+  SERIALISE_CHECK_READ_ERRORS();
+
+  if(IsReplayingAndReading())
+  {
+    ObjDisp(device)->DeviceWaitIdle(Unwrap(device));
+  }
+
+  return true;
+}
+
+VkResult WrappedVulkan::vkSignalSemaphore(VkDevice device, const VkSemaphoreSignalInfo *pSignalInfo)
+{
+  VkSemaphoreSignalInfo unwrapped = *pSignalInfo;
+  unwrapped.semaphore = Unwrap(unwrapped.semaphore);
+
+  SCOPED_DBG_SINK();
+
+  VkResult ret;
+  SERIALISE_TIME_CALL(ret = ObjDisp(device)->SignalSemaphore(Unwrap(device), &unwrapped));
+
+  if(IsActiveCapturing(m_State))
+  {
+    CACHE_THREAD_SERIALISER();
+
+    SCOPED_SERIALISE_CHUNK(VulkanChunk::vkSignalSemaphore);
+    Serialise_vkSignalSemaphore(ser, device, pSignalInfo);
+
+    m_FrameCaptureRecord->AddChunk(scope.Get());
+    GetResourceManager()->MarkResourceFrameReferenced(GetResID(pSignalInfo->semaphore),
+                                                      eFrameRef_Read);
+  }
+
+  return ret;
+}
+
 #if defined(VK_USE_PLATFORM_WIN32_KHR)
 
 VkResult WrappedVulkan::vkImportSemaphoreWin32HandleKHR(
@@ -1008,3 +1192,12 @@ INSTANTIATE_FUNCTION_SERIALISED(void, vkCmdWaitEvents, VkCommandBuffer commandBu
                                 const VkBufferMemoryBarrier *pBufferMemoryBarriers,
                                 uint32_t imageMemoryBarrierCount,
                                 const VkImageMemoryBarrier *pImageMemoryBarriers);
+
+INSTANTIATE_FUNCTION_SERIALISED(void, vkGetSemaphoreCounterValue, VkDevice device,
+                                VkSemaphore semaphore, uint64_t *pValue);
+
+INSTANTIATE_FUNCTION_SERIALISED(void, vkWaitSemaphores, VkDevice device,
+                                const VkSemaphoreWaitInfo *pWaitInfo, uint64_t timeout);
+
+INSTANTIATE_FUNCTION_SERIALISED(void, vkSignalSemaphore, VkDevice device,
+                                const VkSemaphoreSignalInfo *pSignalInfo);
