@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -67,6 +67,41 @@ VkResult WrappedVulkan::vkGetPhysicalDeviceSurfacePresentModesKHR(VkPhysicalDevi
                                                 pPresentModeCount, pPresentModes);
 }
 
+#if defined(VK_USE_PLATFORM_WIN32_KHR)
+
+VkResult WrappedVulkan::vkGetDeviceGroupSurfacePresentModes2EXT(
+    VkDevice device, const VkPhysicalDeviceSurfaceInfo2KHR *pSurfaceInfo,
+    VkDeviceGroupPresentModeFlagsKHR *pModes)
+{
+  VkPhysicalDeviceSurfaceInfo2KHR unwrapped = *pSurfaceInfo;
+  unwrapped.surface = Unwrap(unwrapped.surface);
+
+  return ObjDisp(device)->GetDeviceGroupSurfacePresentModes2EXT(Unwrap(device), &unwrapped, pModes);
+}
+
+VkResult WrappedVulkan::vkAcquireFullScreenExclusiveModeEXT(VkDevice device, VkSwapchainKHR swapchain)
+{
+  return ObjDisp(device)->AcquireFullScreenExclusiveModeEXT(Unwrap(device), Unwrap(swapchain));
+}
+
+VkResult WrappedVulkan::vkReleaseFullScreenExclusiveModeEXT(VkDevice device, VkSwapchainKHR swapchain)
+{
+  return ObjDisp(device)->ReleaseFullScreenExclusiveModeEXT(Unwrap(device), Unwrap(swapchain));
+}
+
+VkResult WrappedVulkan::vkGetPhysicalDeviceSurfacePresentModes2EXT(
+    VkPhysicalDevice physicalDevice, const VkPhysicalDeviceSurfaceInfo2KHR *pSurfaceInfo,
+    uint32_t *pPresentModeCount, VkPresentModeKHR *pPresentModes)
+{
+  VkPhysicalDeviceSurfaceInfo2KHR unwrapped = *pSurfaceInfo;
+  unwrapped.surface = Unwrap(unwrapped.surface);
+
+  return ObjDisp(physicalDevice)
+      ->GetPhysicalDeviceSurfacePresentModes2EXT(Unwrap(physicalDevice), &unwrapped,
+                                                 pPresentModeCount, pPresentModes);
+}
+#endif
+
 VkResult WrappedVulkan::vkGetPhysicalDeviceSurfaceCapabilities2EXT(
     VkPhysicalDevice physicalDevice, VkSurfaceKHR surface,
     VkSurfaceCapabilities2EXT *pSurfaceCapabilities)
@@ -185,9 +220,9 @@ bool WrappedVulkan::Serialise_vkGetSwapchainImagesKHR(SerialiserType &ser, VkDev
                                                       VkImage *pSwapchainImages)
 {
   SERIALISE_ELEMENT(device);
-  SERIALISE_ELEMENT_LOCAL(Swapchain, GetResID(swapchain)).TypedAs("VkSwapchainKHR");
+  SERIALISE_ELEMENT_LOCAL(Swapchain, GetResID(swapchain)).TypedAs("VkSwapchainKHR"_lit);
   SERIALISE_ELEMENT_LOCAL(SwapchainImageIndex, *pCount);
-  SERIALISE_ELEMENT_LOCAL(SwapchainImage, GetResID(*pSwapchainImages)).TypedAs("VkImage");
+  SERIALISE_ELEMENT_LOCAL(SwapchainImage, GetResID(*pSwapchainImages)).TypedAs("VkImage"_lit);
 
   SERIALISE_CHECK_READ_ERRORS();
 
@@ -205,8 +240,8 @@ bool WrappedVulkan::Serialise_vkGetSwapchainImagesKHR(SerialiserType &ser, VkDev
 
     // do this one manually since there's no live version of the swapchain, and DerivedResource()
     // assumes we're passing it a live ID (or live resource)
-    GetReplay()->GetResourceDesc(Swapchain).derivedResources.push_back(SwapchainImage);
-    GetReplay()->GetResourceDesc(SwapchainImage).parentResources.push_back(Swapchain);
+    GetResourceDesc(Swapchain).derivedResources.push_back(SwapchainImage);
+    GetResourceDesc(SwapchainImage).parentResources.push_back(Swapchain);
 
     m_CreationInfo.m_Image[GetResID(swapInfo.images[SwapchainImageIndex].im)] =
         m_CreationInfo.m_Image[Swapchain];
@@ -263,6 +298,9 @@ VkResult WrappedVulkan::vkGetSwapchainImagesKHR(VkDevice device, VkSwapchainKHR
 
         record->AddParent(swaprecord);
 
+        record->resInfo = new ResourceInfo();
+        record->resInfo->imageInfo = ImageInfo(*swaprecord->swapInfo);
+
         // note we add the chunk to the swap record, that way when the swapchain is created it will
         // always create all of its images on replay. The image's record is kept around for
         // reference tracking and any other chunks. Because it has a parent relationship on the
@@ -293,7 +331,7 @@ bool WrappedVulkan::Serialise_vkCreateSwapchainKHR(SerialiserType &ser, VkDevice
   SERIALISE_ELEMENT(device);
   SERIALISE_ELEMENT_LOCAL(CreateInfo, *pCreateInfo);
   SERIALISE_ELEMENT_OPT(pAllocator);
-  SERIALISE_ELEMENT_LOCAL(SwapChain, GetResID(*pSwapChain)).TypedAs("VkSwapchainKHR");
+  SERIALISE_ELEMENT_LOCAL(SwapChain, GetResID(*pSwapChain)).TypedAs("VkSwapchainKHR"_lit);
 
   uint32_t NumImages = 0;
 
@@ -411,11 +449,15 @@ bool WrappedVulkan::Serialise_vkCreateSwapchainKHR(SerialiserType &ser, VkDevice
       range.layerCount = CreateInfo.imageArrayLayers;
       range.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
 
-      m_ImageLayouts[liveId].extent = iminfo.extent;
-      m_ImageLayouts[liveId].format = iminfo.format;
+      ImageLayouts &layouts = m_ImageLayouts[liveId];
 
-      m_ImageLayouts[liveId].subresourceStates.clear();
-      m_ImageLayouts[liveId].subresourceStates.push_back(ImageRegionState(
+      layouts.imageInfo = ImageInfo(swapinfo);
+
+      layouts.isMemoryBound = true;
+      layouts.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
+
+      layouts.subresourceStates.clear();
+      layouts.subresourceStates.push_back(ImageRegionState(
           VK_QUEUE_FAMILY_IGNORED, range, UNKNOWN_PREV_IMG_LAYOUT, VK_IMAGE_LAYOUT_UNDEFINED));
     }
   }
@@ -449,7 +491,7 @@ void WrappedVulkan::WrapAndProcessCreatedSwapchain(VkDevice device,
     SwapchainInfo &swapInfo = *record->swapInfo;
 
     // sneaky casting of window handle into record
-    swapInfo.wndHandle = (RENDERDOC_WindowHandle)GetRecord(pCreateInfo->surface);
+    swapInfo.wndHandle = ((PackedWindowHandle *)GetRecord(pCreateInfo->surface))->handle;
 
     {
       SCOPED_LOCK(m_SwapLookupLock);
@@ -464,7 +506,7 @@ void WrappedVulkan::WrapAndProcessCreatedSwapchain(VkDevice device,
 
     VkResult vkr = VK_SUCCESS;
 
-    const VkLayerDispatchTable *vt = ObjDisp(device);
+    const VkDevDispatchTable *vt = ObjDisp(device);
 
     {
       VkAttachmentDescription attDesc = {
@@ -546,14 +588,18 @@ void WrappedVulkan::WrapAndProcessCreatedSwapchain(VkDevice device,
         range.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
 
         // fill out image info so we track resource state barriers
+        ImageLayouts *layout = NULL;
         {
           SCOPED_LOCK(m_ImageLayoutsLock);
-          m_ImageLayouts[imid].format = pCreateInfo->imageFormat;
-
-          m_ImageLayouts[imid].subresourceStates.clear();
-          m_ImageLayouts[imid].subresourceStates.push_back(ImageRegionState(
-              VK_QUEUE_FAMILY_IGNORED, range, UNKNOWN_PREV_IMG_LAYOUT, VK_IMAGE_LAYOUT_UNDEFINED));
+          layout = &m_ImageLayouts[imid];
         }
+        layout->imageInfo = GetRecord(images[i])->resInfo->imageInfo;
+        layout->isMemoryBound = true;
+        layout->initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
+
+        layout->subresourceStates.clear();
+        layout->subresourceStates.push_back(ImageRegionState(
+            VK_QUEUE_FAMILY_IGNORED, range, UNKNOWN_PREV_IMG_LAYOUT, VK_IMAGE_LAYOUT_UNDEFINED));
 
         {
           VkImageViewCreateInfo info = {
@@ -624,6 +670,49 @@ VkResult WrappedVulkan::vkCreateSwapchainKHR(VkDevice device,
   return ret;
 }
 
+template <typename SerialiserType>
+bool WrappedVulkan::Serialise_vkQueuePresentKHR(SerialiserType &ser, VkQueue queue,
+                                                const VkPresentInfoKHR *pPresentInfo)
+{
+  SERIALISE_ELEMENT(queue);
+  SERIALISE_ELEMENT_LOCAL(PresentInfo, *pPresentInfo);
+
+  ResourceId PresentedImage;
+
+  if(ser.IsWriting())
+  {
+    VkResourceRecord *swaprecord = GetRecord(pPresentInfo->pSwapchains[0]);
+
+    SwapchainInfo &swapInfo = *swaprecord->swapInfo;
+
+    PresentedImage = GetResID(swapInfo.images[pPresentInfo->pImageIndices[0]].im);
+  }
+
+  // we don't have all the information we need about swapchains on replay to get the presented image
+  // just from the PresentInfo struct, so we hide it away here
+  SERIALISE_ELEMENT(PresentedImage).Hidden();
+
+  Serialise_DebugMessages(ser);
+
+  SERIALISE_CHECK_READ_ERRORS();
+
+  if(IsReplayingAndReading() && IsLoading(m_State))
+  {
+    AddEvent();
+
+    DrawcallDescription draw;
+
+    draw.name = StringFormat::Fmt("vkQueuePresentKHR(%s)", ToStr(PresentedImage).c_str());
+    draw.flags |= DrawFlags::Present;
+
+    m_LastPresentedImage = draw.copyDestination = PresentedImage;
+
+    AddDrawcall(draw, true);
+  }
+
+  return true;
+}
+
 VkResult WrappedVulkan::vkQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR *pPresentInfo)
 {
   AdvanceFrame();
@@ -633,8 +722,8 @@ VkResult WrappedVulkan::vkQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR
     RDCWARN("Presenting multiple swapchains at once - only first will be processed");
   }
 
-  vector<VkSwapchainKHR> unwrappedSwaps;
-  vector<VkSemaphore> unwrappedSems;
+  rdcarray<VkSwapchainKHR> unwrappedSwaps;
+  rdcarray<VkSemaphore> unwrappedSems;
 
   VkPresentInfoKHR unwrappedInfo = *pPresentInfo;
 
@@ -654,7 +743,8 @@ VkResult WrappedVulkan::vkQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR
     if(next->sType != VK_STRUCTURE_TYPE_DISPLAY_PRESENT_INFO_KHR &&
        next->sType != VK_STRUCTURE_TYPE_DEVICE_GROUP_PRESENT_INFO_KHR &&
        next->sType != VK_STRUCTURE_TYPE_PRESENT_FRAME_TOKEN_GGP &&
-       next->sType != VK_STRUCTURE_TYPE_PRESENT_REGIONS_KHR)
+       next->sType != VK_STRUCTURE_TYPE_PRESENT_REGIONS_KHR &&
+       next->sType != VK_STRUCTURE_TYPE_PRESENT_TIMES_INFO_GOOGLE)
     {
       RDCWARN("Unsupported pNext structure in pPresentInfo: %s", ToStr(next->sType).c_str());
     }
@@ -690,7 +780,7 @@ VkResult WrappedVulkan::vkQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR
 
       uint32_t swapQueueIndex = m_ImageLayouts[GetResID(im)].queueFamilyIndex;
 
-      VkLayerDispatchTable *vt = ObjDisp(GetDev());
+      VkDevDispatchTable *vt = ObjDisp(GetDev());
 
       TextPrintState textstate = {
           GetNextCmd(),
@@ -742,15 +832,17 @@ VkResult WrappedVulkan::vkQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR
         SubmitAndFlushExtQueue(swapQueueIndex);
       }
 
-      m_TextRenderer->BeginText(textstate);
-
       int flags = activeWindow ? RenderDoc::eOverlay_ActiveWindow : 0;
-      string overlayText = RenderDoc::Inst().GetOverlayText(RDCDriver::Vulkan, m_FrameCounter, flags);
+      rdcstr overlayText = RenderDoc::Inst().GetOverlayText(RDCDriver::Vulkan, m_FrameCounter, flags);
 
       if(!overlayText.empty())
-        m_TextRenderer->RenderText(textstate, 0.0f, 0.0f, overlayText.c_str());
+      {
+        m_TextRenderer->BeginText(textstate);
 
-      m_TextRenderer->EndText(textstate);
+        m_TextRenderer->RenderText(textstate, 0.0f, 0.0f, overlayText);
+
+        m_TextRenderer->EndText(textstate);
+      }
 
       std::swap(bbBarrier.srcQueueFamilyIndex, bbBarrier.dstQueueFamilyIndex);
       std::swap(bbBarrier.oldLayout, bbBarrier.newLayout);
@@ -761,8 +853,8 @@ VkResult WrappedVulkan::vkQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR
 
       ObjDisp(textstate.cmd)->EndCommandBuffer(Unwrap(textstate.cmd));
 
-      std::vector<VkPipelineStageFlags> waitStage(unwrappedSems.size(),
-                                                  VK_PIPELINE_STAGE_ALL_COMMANDS_BIT);
+      rdcarray<VkPipelineStageFlags> waitStage;
+      waitStage.fill(unwrappedSems.size(), VK_PIPELINE_STAGE_ALL_COMMANDS_BIT);
       SubmitCmds(unwrappedSems.data(), waitStage.data(), (uint32_t)unwrappedSems.size());
 
       if(swapQueueIndex != m_QueueFamilyIdx)
@@ -783,7 +875,18 @@ VkResult WrappedVulkan::vkQueuePresentKHR(VkQueue queue, const VkPresentInfoKHR
     }
   }
 
-  VkResult vkr = ObjDisp(queue)->QueuePresentKHR(Unwrap(queue), &unwrappedInfo);
+  VkResult vkr;
+  SERIALISE_TIME_CALL(vkr = ObjDisp(queue)->QueuePresentKHR(Unwrap(queue), &unwrappedInfo));
+
+  if(IsActiveCapturing(m_State))
+  {
+    CACHE_THREAD_SERIALISER();
+
+    SCOPED_SERIALISE_CHUNK(VulkanChunk::vkQueuePresentKHR);
+    Serialise_vkQueuePresentKHR(ser, queue, pPresentInfo);
+
+    m_FrameCaptureRecord->AddChunk(scope.Get());
+  }
 
   Present(LayerDisp(m_Instance), swapInfo.wndHandle);
 
@@ -799,7 +902,11 @@ void WrappedVulkan::vkDestroySurfaceKHR(VkInstance instance, VkSurfaceKHR surfac
 
   // record pointer has window handle packed in
   if(wrapper->record)
-    Keyboard::RemoveInputWindow((void *)wrapper->record);
+  {
+    PackedWindowHandle *wnd = (PackedWindowHandle *)wrapper->record;
+    Keyboard::RemoveInputWindow(wnd->system, wnd->handle);
+    delete wnd;
+  }
 
   // now set record pointer back to NULL so no-one tries to delete it
   wrapper->record = NULL;
@@ -1038,6 +1145,39 @@ VkResult WrappedVulkan::vkGetSwapchainStatusKHR(VkDevice device, VkSwapchainKHR
   return ObjDisp(device)->GetSwapchainStatusKHR(Unwrap(device), Unwrap(swapchain));
 }
 
+VkResult WrappedVulkan::vkCreateHeadlessSurfaceEXT(VkInstance instance,
+                                                   const VkHeadlessSurfaceCreateInfoEXT *pCreateInfo,
+                                                   const VkAllocationCallbacks *pAllocator,
+                                                   VkSurfaceKHR *pSurface)
+{
+  // should not come in here at all on replay
+  RDCASSERT(IsCaptureMode(m_State));
+
+  VkResult ret = ObjDisp(instance)->CreateHeadlessSurfaceEXT(Unwrap(instance), pCreateInfo,
+                                                             pAllocator, pSurface);
+
+  if(ret == VK_SUCCESS)
+  {
+    // we must wrap surfaces to be consistent with the rest of the code and surface handling,
+    // but there's nothing actually to do here - no meaningful data we care about here.
+    GetResourceManager()->WrapResource(Unwrap(instance), *pSurface);
+
+    WrappedVkSurfaceKHR *wrapped = GetWrapped(*pSurface);
+
+    // we don't have an actual OS handle to identify this window. Instead construct something
+    // that should be unique and hopefully not clashing/overlapping with other window handles
+    // in use.
+    uintptr_t fakeWindowHandle = (uintptr_t)wrapped->real.handle;
+
+    // since there's no point in allocating a full resource record and storing the window
+    // handle under there somewhere, we just cast. We won't use the resource record for anything
+
+    wrapped->record = (VkResourceRecord *)fakeWindowHandle;
+  }
+
+  return ret;
+}
+
 INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkCreateSwapchainKHR, VkDevice device,
                                 const VkSwapchainCreateInfoKHR *pCreateInfo,
                                 const VkAllocationCallbacks *pAllocator, VkSwapchainKHR *pSwapchain);
@@ -1045,3 +1185,6 @@ INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkCreateSwapchainKHR, VkDevice device,
 INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkGetSwapchainImagesKHR, VkDevice device,
                                 VkSwapchainKHR swapchain, uint32_t *pSwapchainImageCount,
                                 VkImage *pSwapchainImages);
+
+INSTANTIATE_FUNCTION_SERIALISED(VkResult, vkQueuePresentKHR, VkQueue queue,
+                                const VkPresentInfoKHR *pPresentInfo);
