@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -26,9 +26,9 @@
 #include "hooks.h"
 #include "common/common.h"
 
-static std::vector<LibraryHook *> &LibList()
+static rdcarray<LibraryHook *> &LibList()
 {
-  static std::vector<LibraryHook *> libs;
+  static rdcarray<LibraryHook *> libs;
   return libs;
 }
 
