@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -34,7 +34,7 @@ struct FunctionHook
   FunctionHook() : orig(NULL), hook(NULL) {}
   FunctionHook(const char *f, void **o, void *d) : function(f), orig(o), hook(d) {}
   bool operator<(const FunctionHook &h) const { return function < h.function; }
-  std::string function;
+  rdcstr function;
   void **orig;
   void *hook;
 };
@@ -186,7 +186,7 @@ struct LibraryHook
 private:
   friend class LibraryHooks;
 
-  static std::vector<LibraryHook *> m_Libraries;
+  static rdcarray<LibraryHook *> m_Libraries;
 };
 
 template <typename FuncType>
