@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -26,6 +26,7 @@
 #pragma once
 
 #include <stdint.h>
+#include "half_convert.h"
 #include "vec.h"
 
 inline Vec4f ConvertFromR10G10B10A2(uint32_t data)
@@ -73,71 +74,8 @@ inline uint32_t ConvertToR10G10B10A2(Vec4f data)
          (uint32_t(w * 3) << 30);
 }
 
-inline Vec3f ConvertFromR11G11B10(uint32_t data)
-{
-  uint32_t mantissas[3] = {
-      (data >> 0) & 0x3f, (data >> 11) & 0x3f, (data >> 22) & 0x1f,
-  };
-  int32_t exponents[3] = {
-      int32_t(data >> 6) & 0x1f, int32_t(data >> 17) & 0x1f, int32_t(data >> 27) & 0x1f,
-  };
-
-  Vec3f ret;
-  uint32_t *retu = (uint32_t *)&ret.x;
-
-  // floats have 23 bit mantissa, 8bit exponent
-  // R11G11B10 has 6/6/5 bit mantissas, 5bit exponents
-
-  const int mantissaShift[] = {23 - 6, 23 - 6, 23 - 5};
-
-  for(int i = 0; i < 3; i++)
-  {
-    if(mantissas[i] == 0 && exponents[i] == 0)
-    {
-      retu[i] = 0;
-    }
-    else
-    {
-      if(exponents[i] == 0x1f)
-      {
-        // infinity or nan
-        retu[i] = 0x7f800000 | mantissas[i] << mantissaShift[i];
-      }
-      else if(exponents[i] != 0)
-      {
-        // shift exponent and mantissa to the right range for 32bit floats
-        retu[i] = (exponents[i] + (127 - 15)) << 23 | mantissas[i] << mantissaShift[i];
-      }
-      else if(exponents[i] == 0)
-      {
-        // we know xMantissa isn't 0 also, or it would have been caught above
-
-        exponents[i] = 1;
-
-        // shift until hidden bit is set
-        while((mantissas[i] & 0x40) == 0)
-        {
-          mantissas[i] <<= 1;
-          exponents[i]--;
-        }
-
-        // remove the hidden bit
-        mantissas[i] &= ~0x40;
-
-        retu[i] = (exponents[i] + (127 - 15)) << 23 | mantissas[i] << mantissaShift[i];
-      }
-    }
-  }
-
-  return ret;
-}
-
-/*
-inline uint32_t ConvertToR11G11B10(Vec3f data)
-{
-  return 0;
-}
-*/
+Vec3f ConvertFromR11G11B10(uint32_t data);
+uint32_t ConvertToR11G11B10(Vec3f data);
 
 inline Vec4f ConvertFromB5G5R5A1(uint16_t data)
 {
@@ -157,33 +95,12 @@ inline Vec4f ConvertFromB4G4R4A4(uint16_t data)
                (float)((data >> 8) & 0xf) / 15.0f, (float)((data >> 12) & 0xf) / 15.0f);
 }
 
-extern float SRGB8_lookuptable[256];
+float ConvertFromSRGB8(uint8_t comp);
+float ConvertSRGBToLinear(float srgbF);
+Vec4f ConvertSRGBToLinear(Vec4f srgbF);
+float ConvertLinearToSRGB(float linear);
 
-inline float ConvertFromSRGB8(uint8_t comp)
-{
-  return SRGB8_lookuptable[comp];
-}
-
-inline float ConvertSRGBToLinear(float srgbF)
-{
-  if(srgbF <= 0.04045f)
-    return srgbF / 12.92f;
-
-  if(srgbF < 0.0f)
-    srgbF = 0.0f;
-  else if(srgbF > 1.0f)
-    srgbF = 1.0f;
-
-  return powf((0.055f + srgbF) / 1.055f, 2.4f);
-}
-
-inline Vec4f ConvertSRGBToLinear(Vec4f srgbF)
-{
-  return Vec4f(ConvertSRGBToLinear(srgbF.x), ConvertSRGBToLinear(srgbF.y),
-               ConvertSRGBToLinear(srgbF.z), srgbF.w);
-}
+typedef uint8_t byte;
 
 struct ResourceFormat;
 float ConvertComponent(const ResourceFormat &fmt, const byte *data);
-
-#include "half_convert.h"
