@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -24,38 +24,14 @@
  ******************************************************************************/
 
 #include "os/os_specific.h"
-#include <stdarg.h>
+#include "api/replay/control_types.h"
 #include "strings/string_utils.h"
 
-using std::string;
-
-int utf8printf(char *buf, size_t bufsize, const char *fmt, va_list args);
+int utf8printv(char *buf, size_t bufsize, const char *fmt, va_list args);
 
 namespace StringFormat
 {
-int snprintf(char *str, size_t bufSize, const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-
-  int ret = StringFormat::vsnprintf(str, bufSize, fmt, args);
-
-  va_end(args);
-
-  return ret;
-}
-
-void sntimef(char *str, size_t bufSize, const char *format)
-{
-  StringFormat::sntimef(Timing::GetUTCTime(), str, bufSize, format);
-}
-
-int vsnprintf(char *str, size_t bufSize, const char *format, va_list args)
-{
-  return ::utf8printf(str, bufSize, format, args);
-}
-
-string Fmt(const char *format, ...)
+rdcstr Fmt(const char *format, ...)
 {
   va_list args;
   va_start(args, format);
@@ -63,85 +39,28 @@ string Fmt(const char *format, ...)
   va_list args2;
   va_copy(args2, args);
 
-  int size = StringFormat::vsnprintf(NULL, 0, format, args2);
+  int size = ::utf8printv(NULL, 0, format, args2);
 
-  char *buf = new char[size + 1];
-  StringFormat::vsnprintf(buf, size + 1, format, args);
-  buf[size] = 0;
+  rdcstr ret;
+  ret.resize(size);
+  ::utf8printv(ret.data(), size + 1, format, args);
 
   va_end(args);
   va_end(args2);
 
-  string ret = buf;
-
-  delete[] buf;
-
   return ret;
 }
 
-int Wide2UTF8(wchar_t chr, char mbchr[4])
-{
-  // U+00000 -> U+00007F 1 byte  0xxxxxxx
-  // U+00080 -> U+0007FF 2 bytes 110xxxxx 10xxxxxx
-  // U+00800 -> U+00FFFF 3 bytes 1110xxxx 10xxxxxx 10xxxxxx
-  // U+10000 -> U+1FFFFF 4 bytes 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
-
-  // upcast to uint32_t, so we do the same processing on windows where
-  // sizeof(wchar_t) == 2
-  uint32_t wc = (uint32_t)chr;
-
-  if(wc > 0x10FFFF)
-    wc = 0xFFFD;    // replacement character
-
-  if(wc <= 0x7f)
-  {
-    mbchr[0] = (char)wc;
-    return 1;
-  }
-  else if(wc <= 0x7ff)
-  {
-    mbchr[1] = 0x80 | (char)(wc & 0x3f);
-    wc >>= 6;
-    mbchr[0] = 0xC0 | (char)(wc & 0x1f);
-    return 2;
-  }
-  else if(wc <= 0xffff)
-  {
-    mbchr[2] = 0x80 | (char)(wc & 0x3f);
-    wc >>= 6;
-    mbchr[1] = 0x80 | (char)(wc & 0x3f);
-    wc >>= 6;
-    mbchr[0] = 0xE0 | (char)(wc & 0x0f);
-    wc >>= 4;
-    return 3;
-  }
-  else
-  {
-    // invalid codepoints above 0x10FFFF were replaced above
-    mbchr[3] = 0x80 | (char)(wc & 0x3f);
-    wc >>= 6;
-    mbchr[2] = 0x80 | (char)(wc & 0x3f);
-    wc >>= 6;
-    mbchr[1] = 0x80 | (char)(wc & 0x3f);
-    wc >>= 6;
-    mbchr[0] = 0xF0 | (char)(wc & 0x07);
-    wc >>= 3;
-    return 4;
-  }
-}
-
 };    // namespace StringFormat
 
-string Callstack::AddressDetails::formattedString(const char *commonPath)
+rdcstr Callstack::AddressDetails::formattedString(const char *commonPath)
 {
-  char fmt[512] = {0};
-
   const char *f = filename.c_str();
 
   if(commonPath)
   {
-    string common = strlower(string(commonPath));
-    string fn = strlower(filename.substr(0, common.length()));
+    rdcstr common = strlower(rdcstr(commonPath));
+    rdcstr fn = strlower(filename.substr(0, common.length()));
 
     if(common == fn)
     {
@@ -150,16 +69,14 @@ string Callstack::AddressDetails::formattedString(const char *commonPath)
   }
 
   if(line > 0)
-    StringFormat::snprintf(fmt, 511, "%s line %d", function.c_str(), line);
+    return StringFormat::Fmt("%s line %d", function.c_str(), line);
   else
-    StringFormat::snprintf(fmt, 511, "%s", function.c_str());
-
-  return fmt;
+    return function;
 }
 
-string OSUtility::MakeMachineIdentString(uint64_t ident)
+rdcstr OSUtility::MakeMachineIdentString(uint64_t ident)
 {
-  string ret = "";
+  rdcstr ret = "";
 
   if(ident & MachineIdent_Windows)
     ret += "Windows ";
@@ -206,7 +123,7 @@ TEST_CASE("Test OS-specific functions", "[osspecific]")
 {
   SECTION("GetLibraryFilename")
   {
-    std::string libPath;
+    rdcstr libPath;
     FileIO::GetLibraryFilename(libPath);
     CHECK_FALSE(libPath.empty());
   }
@@ -239,7 +156,7 @@ TEST_CASE("Test OS-specific functions", "[osspecific]")
     var = Process::GetEnvVariable("__renderdoc__unit_test_var");
 
     CHECK(var);
-    CHECK(var == std::string("test_value"));
+    CHECK(var == rdcstr("test_value"));
 
     Process::RegisterEnvironmentModification(mod);
     Process::ApplyEnvironmentModification();
@@ -247,7 +164,7 @@ TEST_CASE("Test OS-specific functions", "[osspecific]")
     var = Process::GetEnvVariable("__renderdoc__unit_test_var");
 
     CHECK(var);
-    CHECK(var == std::string("test_value;test_value"));
+    CHECK(var == rdcstr("test_value;test_value"));
 
     mod.sep = EnvSep::Colon;
 
@@ -257,7 +174,7 @@ TEST_CASE("Test OS-specific functions", "[osspecific]")
     var = Process::GetEnvVariable("__renderdoc__unit_test_var");
 
     CHECK(var);
-    CHECK(var == std::string("test_value;test_value:test_value"));
+    CHECK(var == rdcstr("test_value;test_value:test_value"));
 
     mod.value = "prepend";
     mod.sep = EnvSep::SemiColon;
@@ -269,7 +186,7 @@ TEST_CASE("Test OS-specific functions", "[osspecific]")
     var = Process::GetEnvVariable("__renderdoc__unit_test_var");
 
     CHECK(var);
-    CHECK(var == std::string("prepend;test_value;test_value:test_value"));
+    CHECK(var == rdcstr("prepend;test_value;test_value:test_value"));
 
     mod.value = "reset";
     mod.sep = EnvSep::SemiColon;
@@ -281,7 +198,26 @@ TEST_CASE("Test OS-specific functions", "[osspecific]")
     var = Process::GetEnvVariable("__renderdoc__unit_test_var");
 
     CHECK(var);
-    CHECK(var == std::string("reset"));
+    CHECK(var == rdcstr("reset"));
+  };
+
+  SECTION("UTF-8 to wide conversion")
+  {
+    const rdcstr s = "ελληνικά";
+
+    rdcwstr ws = StringFormat::UTF82Wide(s);
+
+    CHECK(ws.length() == 8);
+    CHECK(ws[0] == L'\x3b5');
+    CHECK(ws[ws.length() - 1] == L'\x3ac');
+    CHECK(ws[ws.length()] == L'\0');
+
+    rdcstr s2 = StringFormat::Wide2UTF8(ws);
+
+    CHECK(s == s2);
+
+    CHECK(StringFormat::UTF82Wide(rdcstr()).length() == 0);
+    CHECK(StringFormat::UTF82Wide(rdcstr()).c_str()[0] == 0);
   };
 
   SECTION("Timing")
