@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,8 @@
  ******************************************************************************/
 
 #include <unistd.h>
+#include "common/common.h"
+#include "common/formatting.h"
 #include "os/os_specific.h"
 #include "strings/string_utils.h"
 
@@ -37,7 +39,7 @@ int GetIdentPort(pid_t childPid)
 {
   int ret = 0;
 
-  string procfile = StringFormat::Fmt("/proc/%d/net/unix", (int)childPid);
+  rdcstr procfile = StringFormat::Fmt("/proc/%d/net/unix", (int)childPid);
 
   // try for a little while for the /proc entry to appear
   for(int retry = 0; retry < 10; retry++)
@@ -148,9 +150,9 @@ const char *Process::GetEnvVariable(const char *name)
                          StringFormat::Fmt("debug.rdoc.%s variable_is_not_set", name).c_str(), true,
                          &result);
 
-  static std::string settingsOutput;
+  static rdcstr settingsOutput;
 
-  settingsOutput = trim(result.strStdout);
+  settingsOutput = result.strStdout.trimmed();
 
   if(settingsOutput == "variable_is_not_set")
     return NULL;
@@ -178,4 +180,4 @@ uint64_t Process::GetMemoryUsage()
     return vmPages * (uint64_t)sysconf(_SC_PAGESIZE);
 
   return 0;
-}
\ No newline at end of file
+}
