@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,8 @@
 #include <android/log.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include "common/common.h"
+#include "common/formatting.h"
 #include "os/os_specific.h"
 
 #define LOGCAT_TAG "renderdoc"
@@ -40,11 +42,11 @@ bool PlatformHasKeyInput()
   return false;
 }
 
-void AddInputWindow(void *wnd)
+void AddInputWindow(WindowingSystem windowSystem, void *wnd)
 {
 }
 
-void RemoveInputWindow(void *wnd)
+void RemoveInputWindow(WindowingSystem windowSystem, void *wnd)
 {
 }
 
@@ -56,33 +58,31 @@ bool GetKeyState(int key)
 
 namespace FileIO
 {
-string GetTempRootPath()
+rdcstr GetTempRootPath()
 {
   // Save captures in the app's private /sdcard directory, which doesnt require
   // WRITE_EXTERNAL_STORAGE permissions. There is no security enforced here,
   // so the replay server can load it as it has READ_EXTERNAL_STORAGE.
   // This is the same as returned by getExternalFilesDir(). It might possibly change in the future.
-  string package;
+  rdcstr package;
   GetExecutableFilename(package);
   return "/sdcard/Android/data/" + package + "/files";
 }
 
-string GetAppFolderFilename(const string &filename)
+rdcstr GetAppFolderFilename(const rdcstr &filename)
 {
-  return GetTempRootPath() + string("/") + filename;
+  return GetTempRootPath() + rdcstr("/") + filename;
 }
 
 // For RenderDoc's apk, this returns our package name
 // For other APKs, we use it to get the writable temp directory.
-void GetExecutableFilename(string &selfName)
+void GetExecutableFilename(rdcstr &selfName)
 {
-  char buf[4096];
-  snprintf(buf, sizeof(buf), "/proc/%u/cmdline", getpid());
-  int fd = open(buf, O_RDONLY);
+  int fd = open(StringFormat::Fmt("/proc/%u/cmdline", getpid()).c_str(), O_RDONLY);
   if(fd < 0)
-  {
     return;
-  }
+
+  char buf[4096];
   ssize_t len = read(fd, buf, sizeof(buf));
   close(fd);
   if(len < 0 || len == sizeof(buf))
@@ -90,10 +90,13 @@ void GetExecutableFilename(string &selfName)
     return;
   }
 
-  selfName = buf;
+  // Strip any process name from cmdline (android:process)
+  rdcstr cmdline = buf;
+  rdcstr filename = cmdline.substr(0, cmdline.find(":"));
+  selfName = filename;
 }
 
-void GetLibraryFilename(string &selfName)
+void GetLibraryFilename(rdcstr &selfName)
 {
   RDCERR("GetLibraryFilename is not defined on Android");
   GetExecutableFilename(selfName);
@@ -102,13 +105,13 @@ void GetLibraryFilename(string &selfName)
 
 namespace StringFormat
 {
-std::string Wide2UTF8(const std::wstring &s)
+rdcstr Wide2UTF8(const rdcwstr &s)
 {
   RDCFATAL("Converting wide strings to UTF-8 is not supported on Android!");
   return "";
 }
 
-std::wstring UTF82Wide(const std::string &s)
+rdcwstr UTF82Wide(const rdcstr &s)
 {
   RDCFATAL("Converting UTF-8 to wide strings is not supported on Android!");
   return L"";
