@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,6 @@
 #include <string.h>
 #include <algorithm>
 #include <map>
-#include <string>
 #include "3rdparty/plthook/plthook.h"
 #include "common/threading.h"
 #include "hooks/hooks.h"
@@ -35,15 +34,17 @@
 
 Threading::CriticalSection libLock;
 
-static std::map<std::string, std::vector<FunctionLoadCallback>> libraryCallbacks;
-static std::vector<std::string> libraryHooks;
-static std::vector<FunctionHook> functionHooks;
+static std::map<rdcstr, rdcarray<FunctionLoadCallback>> libraryCallbacks;
+static rdcarray<rdcstr> libraryHooks;
+static rdcarray<FunctionHook> functionHooks;
 
 void *intercept_dlopen(const char *filename, int flag, void *ret);
 void plthook_lib(void *handle);
 
 typedef void *(*DLOPENPROC)(const char *, int);
+typedef void *(*DLSYMPROC)(void *, const char *);
 DLOPENPROC realdlopen = NULL;
+DLSYMPROC realdlsym = NULL;
 
 static volatile int32_t tlsbusyflag = 0;
 
@@ -75,6 +76,45 @@ __attribute__((visibility("default"))) void *dlopen(const char *filename, int fl
   return ret;
 }
 
+#if defined(RENDERDOC_HOOK_DLSYM)
+
+#pragma message("ALERT: dlsym() hooking enabled! This is unreliable & relies on glibc internals.")
+
+extern "C" {
+
+__attribute__((visibility("default"))) void *dlsym(void *handle, const char *name);
+
+extern void *_dl_sym(void *, const char *, void *);
+
+void bootstrap_dlsym()
+{
+  realdlsym = (DLSYMPROC)_dl_sym(RTLD_NEXT, "dlsym", (void *)&dlsym);
+}
+
+__attribute__((visibility("default"))) void *dlsym(void *handle, const char *name)
+{
+  if(!strcmp(name, "dlsym"))
+    return (void *)&dlsym;
+
+  if(!strcmp(name, "dlopen") && realdlopen)
+    return (void *)&dlopen;
+
+  if(realdlsym == NULL)
+    bootstrap_dlsym();
+
+  if(realdlsym == NULL)
+  {
+    fprintf(stderr, "Couldn't get onwards dlsym in hooked dlsym\n");
+    exit(-1);
+  }
+
+  return realdlsym(handle, name);
+}
+
+};    // extern "C"
+
+#endif
+
 void plthook_lib(void *handle)
 {
   plthook_t *plthook = NULL;
@@ -107,7 +147,7 @@ static void CheckLoadedLibraries()
   // them and call callbacks.
   for(auto it = libraryHooks.begin(); it != libraryHooks.end(); ++it)
   {
-    std::string libName = *it;
+    rdcstr libName = *it;
     void *handle = realdlopen(libName.c_str(), RTLD_NOW | RTLD_NOLOAD | RTLD_GLOBAL);
 
     if(handle)
@@ -118,7 +158,7 @@ static void CheckLoadedLibraries()
           *hook.orig = dlsym(handle, hook.function.c_str());
       }
 
-      std::vector<FunctionLoadCallback> callbacks;
+      rdcarray<FunctionLoadCallback> callbacks;
 
       // don't call callbacks again if the library is dlopen'd again
       libraryCallbacks[libName].swap(callbacks);
@@ -144,11 +184,11 @@ void *intercept_dlopen(const char *filename, int flag, void *ret)
   if(flag & RTLD_DEEPBIND)
     plthook_lib(ret);
 
-  std::string base = get_basename(filename);
+  rdcstr base = get_basename(filename);
 
   for(auto it = libraryHooks.begin(); it != libraryHooks.end(); ++it)
   {
-    const std::string &libName = *it;
+    const rdcstr &libName = *it;
     if(*it == base)
     {
       RDCDEBUG("Redirecting dlopen to ourselves for %s", filename);
@@ -159,7 +199,7 @@ void *intercept_dlopen(const char *filename, int flag, void *ret)
           *hook.orig = dlsym(ret, hook.function.c_str());
       }
 
-      std::vector<FunctionLoadCallback> callbacks;
+      rdcarray<FunctionLoadCallback> callbacks;
 
       // don't call callbacks again if the library is dlopen'd again
       libraryCallbacks[libName].swap(callbacks);
@@ -218,7 +258,7 @@ void LibraryHooks::RegisterLibraryHook(char const *name, FunctionLoadCallback cb
 {
   SCOPED_LOCK(libLock);
 
-  if(std::find(libraryHooks.begin(), libraryHooks.end(), name) == libraryHooks.end())
+  if(!libraryHooks.contains(name))
     libraryHooks.push_back(name);
 
   if(cb)
@@ -236,4 +276,4 @@ ScopedSuppressHooking::ScopedSuppressHooking()
 
 ScopedSuppressHooking::~ScopedSuppressHooking()
 {
-}
\ No newline at end of file
+}
