@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,7 +32,10 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
-#include "api/app/renderdoc_app.h"
+#include <map>
+//#include "api/app/renderdoc_app.h"
+#include "api/replay/capture_options.h"
+#include "api/replay/control_types.h"
 #include "common/threading.h"
 #include "os/os_specific.h"
 #include "strings/string_utils.h"
@@ -224,19 +227,19 @@ static void SetupZombieCollectionHandler()
 namespace FileIO
 {
 void ReleaseFDAfterFork();
-string FindFileInPath(const string &fileName);
+rdcstr FindFileInPath(const rdcstr &fileName);
 };
 
-static const string GetAbsoluteAppPathFromName(const string &appName)
+static const rdcstr GetAbsoluteAppPathFromName(const rdcstr &appName)
 {
-  string appPath;
+  rdcstr appPath;
 
   // If the application name contains a slash character convert it to an absolute path and return it
-  if(appName.find("/") != string::npos)
+  if(appName.contains("/"))
   {
     char realpathBuffer[PATH_MAX];
-    string appDir = get_dirname(appName);
-    string appBasename = get_basename(appName);
+    rdcstr appDir = get_dirname(appName);
+    rdcstr appBasename = get_basename(appName);
     realpath(appDir.c_str(), realpathBuffer);
     appPath = realpathBuffer;
     appPath += "/" + appBasename;
@@ -247,15 +250,15 @@ static const string GetAbsoluteAppPathFromName(const string &appName)
   return FileIO::FindFileInPath(appName);
 }
 
-static vector<EnvironmentModification> &GetEnvModifications()
+static rdcarray<EnvironmentModification> &GetEnvModifications()
 {
-  static vector<EnvironmentModification> envCallbacks;
+  static rdcarray<EnvironmentModification> envCallbacks;
   return envCallbacks;
 }
 
-static map<string, string> EnvStringToEnvMap(const char **envstring)
+static std::map<rdcstr, rdcstr> EnvStringToEnvMap(const char **envstring)
 {
-  map<string, string> ret;
+  std::map<rdcstr, rdcstr> ret;
 
   const char **e = envstring;
 
@@ -269,11 +272,8 @@ static map<string, string> EnvStringToEnvMap(const char **envstring)
       continue;
     }
 
-    string name;
-    string value;
-
-    name.assign(*e, equals);
-    value = equals + 1;
+    rdcstr name = rdcstr(*e, equals - *e);
+    rdcstr value = equals + 1;
 
     ret[name] = value;
 
@@ -283,30 +283,30 @@ static map<string, string> EnvStringToEnvMap(const char **envstring)
   return ret;
 }
 
-static string shellExpand(const string &in)
+static rdcstr shellExpand(const rdcstr &in)
 {
-  string path = trim(in);
+  rdcstr path = in.trimmed();
 
   // if it begins with ./ then replace with working directory
   if(path[0] == '.' && path[1] == '/')
   {
     char cwd[1024] = {};
     getcwd(cwd, 1023);
-    return string(cwd) + path.substr(1);
+    return rdcstr(cwd) + path.substr(1);
   }
 
   // if it's ~/... then replace with $HOME and return
   if(path[0] == '~' && path[1] == '/')
-    return string(getenv("HOME")) + path.substr(1);
+    return rdcstr(getenv("HOME")) + path.substr(1);
 
   // if it's ~user/... then use getpwname
   if(path[0] == '~')
   {
-    size_t slash = path.find('/');
+    int slash = path.find('/');
 
-    string username;
+    rdcstr username;
 
-    if(slash != string::npos)
+    if(slash >= 0)
     {
       RDCASSERT(slash > 1);
       username = path.substr(1, slash - 1);
@@ -320,17 +320,17 @@ static string shellExpand(const string &in)
 
     if(pwdata)
     {
-      if(slash != string::npos)
-        return string(pwdata->pw_dir) + path.substr(slash);
+      if(slash >= 0)
+        return rdcstr(pwdata->pw_dir) + path.substr(slash);
 
-      return string(pwdata->pw_dir);
+      return rdcstr(pwdata->pw_dir);
     }
   }
 
   return path;
 }
 
-void Process::RegisterEnvironmentModification(EnvironmentModification modif)
+void Process::RegisterEnvironmentModification(const EnvironmentModification &modif)
 {
   GetEnvModifications().push_back(modif);
 }
@@ -346,14 +346,14 @@ void Process::ApplyEnvironmentModification()
 {
   // turn environment string to a UTF-8 map
   char **currentEnvironment = GetCurrentEnvironment();
-  map<string, string> currentEnv = EnvStringToEnvMap((const char **)currentEnvironment);
-  vector<EnvironmentModification> &modifications = GetEnvModifications();
+  std::map<rdcstr, rdcstr> currentEnv = EnvStringToEnvMap((const char **)currentEnvironment);
+  rdcarray<EnvironmentModification> &modifications = GetEnvModifications();
 
   for(size_t i = 0; i < modifications.size(); i++)
   {
     EnvironmentModification &m = modifications[i];
 
-    string value = currentEnv[m.name.c_str()];
+    rdcstr value = currentEnv[m.name.c_str()];
 
     switch(m.mod)
     {
@@ -374,7 +374,7 @@ void Process::ApplyEnvironmentModification()
       {
         if(!value.empty())
         {
-          std::string prep = m.value;
+          rdcstr prep = m.value;
           if(m.sep == EnvSep::Platform || m.sep == EnvSep::Colon)
             prep += ":";
           else if(m.sep == EnvSep::SemiColon)
@@ -418,15 +418,15 @@ static pid_t RunProcess(const char *app, const char *workingDir, const char *cmd
   if(!app)
     return (pid_t)0;
 
-  string appName = app;
-  string workDir = (workingDir && workingDir[0]) ? workingDir : get_dirname(appName);
+  rdcstr appName = app;
+  rdcstr workDir = (workingDir && workingDir[0]) ? workingDir : get_dirname(appName);
 
 // handle funky apple .app folders that aren't actually executables
 #if ENABLED(RDOC_APPLE)
-  if(appName.size() > 5 && appName.rfind(".app") == appName.size() - 4)
+  if(appName.size() > 5 && appName.endsWith(".app"))
   {
-    std::string realAppName = appName + "/Contents/MacOS/" + get_basename(appName);
-    realAppName.erase(realAppName.size() - 4);
+    rdcstr realAppName = appName + "/Contents/MacOS/" + get_basename(appName);
+    realAppName.erase(realAppName.size() - 4, ~0U);
 
     if(FileIO::exists(realAppName.c_str()))
     {
@@ -465,7 +465,7 @@ static pid_t RunProcess(const char *app, const char *workingDir, const char *cmd
 
     c = cmdLine;
 
-    string a;
+    rdcstr a;
 
     argc = 0;    // current argument we're fetching
 
@@ -558,7 +558,7 @@ static pid_t RunProcess(const char *app, const char *workingDir, const char *cmd
     }
   }
 
-  const string appPath(GetAbsoluteAppPathFromName(appName));
+  const rdcstr appPath(GetAbsoluteAppPathFromName(appName));
 
   pid_t childPid = 0;
 
@@ -579,9 +579,13 @@ static pid_t RunProcess(const char *app, const char *workingDir, const char *cmd
         dup2(stdoutPipe[1], STDOUT_FILENO);
         dup2(stderrPipe[1], STDERR_FILENO);
 
-        // Close read ends, as the child will write.
+        // now close all pipe handles - we don't need the read ends, and the write ends have been
+        // duplicated into stdout/stderr above - we don't want these handles to be inherited into
+        // child processes.
         close(stdoutPipe[0]);
         close(stderrPipe[0]);
+        close(stdoutPipe[1]);
+        close(stderrPipe[1]);
       }
 
       chdir(workDir.c_str());
@@ -619,9 +623,9 @@ static pid_t RunProcess(const char *app, const char *workingDir, const char *cmd
   return childPid;
 }
 
-ExecuteResult Process::InjectIntoProcess(uint32_t pid, const rdcarray<EnvironmentModification> &env,
-                                         const char *logfile, const CaptureOptions &opts,
-                                         bool waitForExit)
+rdcpair<ReplayStatus, uint32_t> Process::InjectIntoProcess(
+    uint32_t pid, const rdcarray<EnvironmentModification> &env, const char *logfile,
+    const CaptureOptions &opts, bool waitForExit)
 {
   RDCUNIMPLEMENTED("Injecting into already running processes on linux");
   return {ReplayStatus::InjectionFailed, 0};
@@ -649,27 +653,30 @@ uint32_t Process::LaunchProcess(const char *app, const char *workingDir, const c
   uint32_t ret = (uint32_t)RunProcess(app, workingDir, cmdLine, currentEnvironment,
                                       result ? stdoutPipe : NULL, result ? stderrPipe : NULL);
 
-  if(ret && result)
+  if(result)
   {
     result->strStdout = "";
     result->strStderror = "";
 
-    ssize_t stdoutRead, stderrRead;
-    char chBuf[4096];
-    do
+    if(ret)
     {
-      stdoutRead = read(stdoutPipe[0], chBuf, sizeof(chBuf));
-      if(stdoutRead > 0)
-        result->strStdout += string(chBuf, stdoutRead);
-    } while(stdoutRead > 0);
+      ssize_t stdoutRead, stderrRead;
+      char chBuf[4096];
+      do
+      {
+        stdoutRead = read(stdoutPipe[0], chBuf, sizeof(chBuf));
+        if(stdoutRead > 0)
+          result->strStdout += rdcstr(chBuf, stdoutRead);
+      } while(stdoutRead > 0);
 
-    do
-    {
-      stderrRead = read(stderrPipe[0], chBuf, sizeof(chBuf));
-      if(stderrRead > 0)
-        result->strStderror += string(chBuf, stderrRead);
+      do
+      {
+        stderrRead = read(stderrPipe[0], chBuf, sizeof(chBuf));
+        if(stderrRead > 0)
+          result->strStderror += rdcstr(chBuf, stderrRead);
 
-    } while(stderrRead > 0);
+      } while(stderrRead > 0);
+    }
 
     // Close read ends.
     close(stdoutPipe[0]);
@@ -683,16 +690,15 @@ uint32_t Process::LaunchScript(const char *script, const char *workingDir, const
                                bool internal, ProcessResult *result)
 {
   // Change parameters to invoke command interpreter
-  string args = "-lc \"" + string(script) + " " + string(argList) + "\"";
+  rdcstr args = "-lc \"" + rdcstr(script) + " " + rdcstr(argList) + "\"";
 
   return LaunchProcess("bash", workingDir, args.c_str(), internal, result);
 }
 
-ExecuteResult Process::LaunchAndInjectIntoProcess(const char *app, const char *workingDir,
-                                                  const char *cmdLine,
-                                                  const rdcarray<EnvironmentModification> &envList,
-                                                  const char *capturefile,
-                                                  const CaptureOptions &opts, bool waitForExit)
+rdcpair<ReplayStatus, uint32_t> Process::LaunchAndInjectIntoProcess(
+    const char *app, const char *workingDir, const char *cmdLine,
+    const rdcarray<EnvironmentModification> &envList, const char *capturefile,
+    const CaptureOptions &opts, bool waitForExit)
 {
   if(app == NULL || app[0] == 0)
   {
@@ -702,8 +708,8 @@ ExecuteResult Process::LaunchAndInjectIntoProcess(const char *app, const char *w
 
   // turn environment string to a UTF-8 map
   char **currentEnvironment = GetCurrentEnvironment();
-  map<string, string> env = EnvStringToEnvMap((const char **)currentEnvironment);
-  vector<EnvironmentModification> modifications = GetEnvModifications();
+  std::map<rdcstr, rdcstr> env = EnvStringToEnvMap((const char **)currentEnvironment);
+  rdcarray<EnvironmentModification> modifications = GetEnvModifications();
 
   for(const EnvironmentModification &e : envList)
     modifications.push_back(e);
@@ -711,7 +717,7 @@ ExecuteResult Process::LaunchAndInjectIntoProcess(const char *app, const char *w
   if(capturefile == NULL)
     capturefile = "";
 
-  string binpath, libpath, ownlibpath;
+  rdcstr binpath, libpath, ownlibpath;
   {
     FileIO::GetExecutableFilename(binpath);
     binpath = get_dirname(binpath);
@@ -730,14 +736,14 @@ ExecuteResult Process::LaunchAndInjectIntoProcess(const char *app, const char *w
   FileIO::GetLibraryFilename(ownlibpath);
   ownlibpath = get_dirname(ownlibpath);
 
-  std::string libfile = "librenderdoc" LIB_SUFFIX;
+  rdcstr libfile = "librenderdoc" LIB_SUFFIX;
 
 // on macOS, the path must be absolute
 #if ENABLED(RDOC_APPLE)
   libfile = libpath + "/" + libfile;
 #endif
 
-  string optstr = opts.EncodeAsString();
+  rdcstr optstr = opts.EncodeAsString();
 
   modifications.push_back(
       EnvironmentModification(EnvMod::Append, EnvSep::Platform, LIB_PATH_ENV_VAR, binpath.c_str()));
@@ -758,7 +764,7 @@ ExecuteResult Process::LaunchAndInjectIntoProcess(const char *app, const char *w
   {
     EnvironmentModification &m = modifications[i];
 
-    string &value = env[m.name.c_str()];
+    rdcstr &value = env[m.name.c_str()];
 
     switch(m.mod)
     {
@@ -799,7 +805,7 @@ ExecuteResult Process::LaunchAndInjectIntoProcess(const char *app, const char *w
   int i = 0;
   for(auto it = env.begin(); it != env.end(); it++)
   {
-    string envline = it->first + "=" + it->second;
+    rdcstr envline = it->first + "=" + it->second;
     envp[i] = new char[envline.size() + 1];
     memcpy(envp[i], envline.c_str(), envline.size() + 1);
     i++;
@@ -847,6 +853,11 @@ void Process::StopGlobalHook()
 {
 }
 
+bool Process::IsModuleLoaded(const char *module)
+{
+  return dlopen(module, RTLD_NOW | RTLD_NOLOAD) != NULL;
+}
+
 void *Process::LoadModule(const char *module)
 {
   return dlopen(module, RTLD_NOW);
