@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -37,21 +37,21 @@
 #include <time.h>
 #include <unistd.h>
 #include "api/app/renderdoc_app.h"
+#include "api/replay/data_types.h"
+#include "common/formatting.h"
 #include "common/threading.h"
 #include "os/os_specific.h"
 #include "strings/string_utils.h"
 
-using std::string;
-
 // gives us an address to identify this so with
 static int soLocator = 0;
 
 namespace FileIO
 {
 // in posix/.../..._stringio.cpp
-string GetTempRootPath();
+rdcstr GetTempRootPath();
 
-string GetHomeFolderFilename()
+rdcstr GetHomeFolderFilename()
 {
   passwd *pw = getpwuid(getuid());
   const char *homedir = pw->pw_dir;
@@ -59,14 +59,14 @@ string GetHomeFolderFilename()
   return homedir;
 }
 
-string GetTempFolderFilename()
+rdcstr GetTempFolderFilename()
 {
   return GetTempRootPath() + "/";
 }
 
-void CreateParentDirectory(const string &filename)
+void CreateParentDirectory(const rdcstr &filename)
 {
-  string fn = get_dirname(filename);
+  rdcstr fn = get_dirname(filename);
 
   // want trailing slash so that we create all directories
   fn.push_back('/');
@@ -74,9 +74,9 @@ void CreateParentDirectory(const string &filename)
   if(fn[0] != '/')
     return;
 
-  size_t offs = fn.find('/', 1);
+  int offs = fn.find('/', 1);
 
-  while(offs != string::npos)
+  while(offs >= 0)
   {
     // create directory path from 0 to offs by NULLing the
     // / at offs, mkdir, then set it back
@@ -84,11 +84,11 @@ void CreateParentDirectory(const string &filename)
     mkdir(fn.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
     fn[offs] = '/';
 
-    offs = fn.find_first_of('/', offs + 1);
+    offs = fn.find('/', offs + 1);
   }
 }
 
-bool IsRelativePath(const string &path)
+bool IsRelativePath(const rdcstr &path)
 {
   if(path.empty())
     return false;
@@ -96,17 +96,17 @@ bool IsRelativePath(const string &path)
   return path.front() != '/';
 }
 
-string GetFullPathname(const string &filename)
+rdcstr GetFullPathname(const rdcstr &filename)
 {
   char path[PATH_MAX + 1] = {0};
   realpath(filename.c_str(), path);
 
-  return string(path);
+  return rdcstr(path);
 }
 
-string FindFileInPath(const string &fileName)
+rdcstr FindFileInPath(const rdcstr &fileName)
 {
-  string filePath;
+  rdcstr filePath;
 
   // Search the PATH directory list for the application (like shell which) to get the absolute path
   // Return "" if no exectuable found in the PATH list
@@ -122,7 +122,7 @@ string FindFileInPath(const string &fileName)
   const char *path = strtok(localPath, pathSeparator);
   while(path)
   {
-    string testPath(path);
+    rdcstr testPath(path);
     testPath += "/" + fileName;
     if(!access(testPath.c_str(), X_OK))
     {
@@ -136,14 +136,14 @@ string FindFileInPath(const string &fileName)
   return filePath;
 }
 
-string GetReplayAppFilename()
+rdcstr GetReplayAppFilename()
 {
   // look up the shared object's path via dladdr
   Dl_info info;
   dladdr((void *)&soLocator, &info);
-  string path = info.dli_fname ? info.dli_fname : "";
+  rdcstr path = info.dli_fname ? info.dli_fname : "";
   path = get_dirname(path);
-  string replay = path + "/qrenderdoc";
+  rdcstr replay = path + "/qrenderdoc";
 
   FILE *f = FileIO::fopen(replay.c_str(), "r");
   if(f)
@@ -190,10 +190,10 @@ string GetReplayAppFilename()
   return "qrenderdoc";
 }
 
-void GetDefaultFiles(const char *logBaseName, string &capture_filename, string &logging_filename,
-                     string &target)
+void GetDefaultFiles(const char *logBaseName, rdcstr &capture_filename, rdcstr &logging_filename,
+                     rdcstr &target)
 {
-  string path;
+  rdcstr path;
   GetExecutableFilename(path);
 
   const char *mod = strrchr(path.c_str(), '/');
@@ -204,7 +204,7 @@ void GetDefaultFiles(const char *logBaseName, string &capture_filename, string &
   else
     mod = "unknown";
 
-  target = string(mod);
+  target = rdcstr(mod);
 
   time_t t = time(NULL);
   tm now = *localtime(&t);
@@ -222,27 +222,21 @@ void GetDefaultFiles(const char *logBaseName, string &capture_filename, string &
       temp_folder[--len] = 0;
   }
 
-  char temp_filename[2048 + 128] = {0};
-
-  snprintf(temp_filename, sizeof(temp_filename) - 1, "%s/RenderDoc/%s_%04d.%02d.%02d_%02d.%02d.rdc",
-           temp_folder, mod, 1900 + now.tm_year, now.tm_mon + 1, now.tm_mday, now.tm_hour,
-           now.tm_min);
-
-  capture_filename = string(temp_filename);
-
-  snprintf(temp_filename, sizeof(temp_filename) - 1,
-           "%s/RenderDoc/%s_%04d.%02d.%02d_%02d.%02d.%02d.log", temp_folder, logBaseName,
-           1900 + now.tm_year, now.tm_mon + 1, now.tm_mday, now.tm_hour, now.tm_min, now.tm_sec);
+  capture_filename =
+      StringFormat::Fmt("%s/RenderDoc/%s_%04d.%02d.%02d_%02d.%02d.rdc", temp_folder, mod,
+                        1900 + now.tm_year, now.tm_mon + 1, now.tm_mday, now.tm_hour, now.tm_min);
 
   // set by UI when launching programs so all logging goes to the same file
   char *logfile_override = getenv("RENDERDOC_DEBUG_LOG_FILE");
   if(logfile_override)
-    logging_filename = string(logfile_override);
+    logging_filename = rdcstr(logfile_override);
   else
-    logging_filename = string(temp_filename);
+    logging_filename = StringFormat::Fmt(
+        "%s/RenderDoc/%s_%04d.%02d.%02d_%02d.%02d.%02d.log", temp_folder, logBaseName,
+        1900 + now.tm_year, now.tm_mon + 1, now.tm_mday, now.tm_hour, now.tm_min, now.tm_sec);
 }
 
-uint64_t GetModifiedTimestamp(const string &filename)
+uint64_t GetModifiedTimestamp(const rdcstr &filename)
 {
   struct ::stat st;
   int res = stat(filename.c_str(), &st);
@@ -320,9 +314,9 @@ void Delete(const char *path)
   unlink(path);
 }
 
-std::vector<PathEntry> GetFilesInDirectory(const char *path)
+void GetFilesInDirectory(const char *path, rdcarray<PathEntry> &ret)
 {
-  std::vector<PathEntry> ret;
+  ret.clear();
 
   DIR *d = opendir(path);
 
@@ -336,7 +330,7 @@ std::vector<PathEntry> GetFilesInDirectory(const char *path)
       flags = PathProperty::ErrorAccessDenied;
 
     ret.push_back(PathEntry(path, flags));
-    return ret;
+    return;
   }
 
   dirent *ent = NULL;
@@ -352,7 +346,7 @@ std::vector<PathEntry> GetFilesInDirectory(const char *path)
     if(!strcmp(ent->d_name, ".") || !strcmp(ent->d_name, ".."))
       continue;
 
-    string fullpath = path;
+    rdcstr fullpath = path;
     fullpath += '/';
     fullpath += ent->d_name;
 
@@ -385,8 +379,6 @@ std::vector<PathEntry> GetFilesInDirectory(const char *path)
   // don't care if we hit an error or enumerated all files, just finish
 
   closedir(d);
-
-  return ret;
 }
 
 FILE *fopen(const char *filename, const char *mode)
@@ -394,7 +386,7 @@ FILE *fopen(const char *filename, const char *mode)
   return ::fopen(filename, mode);
 }
 
-std::string ErrorString()
+rdcstr ErrorString()
 {
   int err = errno;
 
@@ -405,26 +397,6 @@ std::string ErrorString()
   return buf;
 }
 
-string getline(FILE *f)
-{
-  string ret;
-
-  while(!FileIO::feof(f))
-  {
-    char c = (char)::fgetc(f);
-
-    if(FileIO::feof(f))
-      break;
-
-    if(c != 0 && c != '\n')
-      ret.push_back(c);
-    else
-      break;
-  }
-
-  return ret;
-}
-
 size_t fread(void *buf, size_t elementSize, size_t count, FILE *f)
 {
   return ::fread(buf, elementSize, count, f);
@@ -473,22 +445,22 @@ bool exists(const char *filename)
   return (res == 0);
 }
 
-static int logfileFD = -1;
+rdcarray<int> logfiles;
 
 // this is used in posix_process.cpp, so that we can close the handle any time that we fork()
 void ReleaseFDAfterFork()
 {
   // we do NOT release the shared lock here, since the file descriptor is shared so we'd be
   // releasing the parent process's lock. Just close our file descriptor
-  if(logfileFD >= 0)
-    close(logfileFD);
+  for(int log : logfiles)
+    close(log);
 }
 
-std::string logfile_readall(const char *filename)
+rdcstr logfile_readall(const char *filename)
 {
   FILE *f = FileIO::fopen(filename, "r");
 
-  std::string ret;
+  rdcstr ret;
 
   if(f == NULL)
     return ret;
@@ -506,35 +478,49 @@ std::string logfile_readall(const char *filename)
   return ret;
 }
 
-bool logfile_open(const char *filename)
+LogFileHandle *logfile_open(const char *filename)
 {
-  logfileFD = open(filename, O_APPEND | O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+  int fd = open(filename, O_APPEND | O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+  if(fd < 0)
+  {
+    RDCWARN("Couldn't open logfile '%s': %d", filename, (int)errno);
+    return NULL;
+  }
+
+  logfiles.push_back(fd);
 
   // acquire a shared lock. Every process acquires a shared lock to the common logfile. Each time a
   // process shuts down and wants to close the logfile, it releases its shared lock and tries to
   // acquire an exclusive lock, to see if it can delete the file. See logfile_close.
-  int err = flock(logfileFD, LOCK_SH | LOCK_NB);
+  int err = flock(fd, LOCK_SH | LOCK_NB);
 
   if(err < 0)
-    RDCWARN("Couldn't acquire shared lock to %s: %d", filename, (int)errno);
+    RDCWARN("Couldn't acquire shared lock to '%s': %d", filename, (int)errno);
 
-  return logfileFD >= 0;
+  return (LogFileHandle *)(uintptr_t)fd;
 }
 
-void logfile_append(const char *msg, size_t length)
+void logfile_append(LogFileHandle *logHandle, const char *msg, size_t length)
 {
-  if(logfileFD >= 0)
-    write(logfileFD, msg, (unsigned int)length);
-}
-
-void logfile_close(const char *filename)
-{
-  if(logfileFD >= 0)
+  if(logHandle)
   {
-    // release our shared lock
-    int err = flock(logfileFD, LOCK_UN | LOCK_NB);
+    int fd = int(uintptr_t(logHandle) & 0xffffffff);
 
-    if(err == 0 && filename)
+    write(fd, msg, (unsigned int)length);
+  }
+}
+
+void logfile_close(LogFileHandle *logHandle, const char *deleteFilename)
+{
+  if(logHandle)
+  {
+    int fd = int(uintptr_t(logHandle) & 0xffffffff);
+
+    // release our shared lock
+    int err = flock(fd, LOCK_UN | LOCK_NB);
+
+    if(err == 0 && deleteFilename)
     {
       // now try to acquire an exclusive lock. If this succeeds, no other processes are using the
       // file (since no other shared locks exist), so we can delete it. If it fails, some other
@@ -542,20 +528,20 @@ void logfile_close(const char *filename)
       // NOTE: there is a race here between acquiring the exclusive lock and unlinking, but we
       // aren't interested in this kind of race - we're interested in whether an application is
       // still running when the UI closes, or vice versa, or similar cases.
-      err = flock(logfileFD, LOCK_EX | LOCK_NB);
+      err = flock(fd, LOCK_EX | LOCK_NB);
 
       if(err == 0)
       {
         // we got the exclusive lock. Now release it, close fd, and unlink the file
-        err = flock(logfileFD, LOCK_UN | LOCK_NB);
+        err = flock(fd, LOCK_UN | LOCK_NB);
 
         // can't really error handle here apart from retrying
         if(err != 0)
-          RDCWARN("Couldn't release exclusive lock to %s: %d", filename, (int)errno);
+          RDCWARN("Couldn't release exclusive lock to '%s': %d", deleteFilename, (int)errno);
 
-        close(logfileFD);
+        close(fd);
 
-        unlink(filename);
+        unlink(deleteFilename);
 
         // return immediately so we don't close again below.
         return;
@@ -563,22 +549,53 @@ void logfile_close(const char *filename)
     }
     else
     {
-      RDCWARN("Couldn't release shared lock to %s: %d", filename, (int)errno);
+      RDCWARN("Couldn't release shared lock to '%s': %d", deleteFilename, (int)errno);
       // nothing to do, we won't try again, just exit. The log might lie around, but that's
       // relatively harmless.
     }
 
-    close(logfileFD);
+    logfiles.removeOne(fd);
+
+    close(fd);
   }
 }
 };
 
 namespace StringFormat
 {
-void sntimef(time_t utcTime, char *str, size_t bufSize, const char *format)
+rdcstr sntimef(time_t utcTime, const char *format)
 {
   tm *tmv = localtime(&utcTime);
 
-  strftime(str, bufSize, format, tmv);
+  // conservatively assume that most formatters will replace like-for-like (e.g. %H with 12) and
+  // a few will increase (%Y to 2019) but generally the string will stay the same size.
+  size_t len = strlen(format) + 16;
+
+  size_t ret = 0;
+  char *buf = NULL;
+
+  // loop until we have successfully formatted
+  while(ret == 0)
+  {
+    // delete any previous buffer
+    delete[] buf;
+
+    // alloate new one of the new size
+    buf = new char[len + 1];
+    buf[len] = 0;
+
+    // try formatting
+    ret = strftime(buf, len, format, tmv);
+
+    // double the length for next time, if this failed
+    len *= 2;
+  }
+
+  rdcstr str = buf;
+
+  // delete successful buffer
+  delete[] buf;
+
+  return str;
 }
 };
