@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2016-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +24,7 @@
 
 #include <time.h>
 #include <unistd.h>
+#include "common/common.h"
 #include "os/os_specific.h"
 
 void CacheDebuggerPresent();
@@ -181,11 +182,11 @@ int64_t nextTLSSlot = 0;
 
 struct TLSData
 {
-  vector<void *> data;
+  rdcarray<void *> data;
 };
 
 static CriticalSection *m_TLSListLock = NULL;
-static vector<TLSData *> *m_TLSList = NULL;
+static rdcarray<TLSData *> *m_TLSList = NULL;
 
 void Init()
 {
@@ -194,7 +195,7 @@ void Init()
     RDCFATAL("Can't allocate OS TLS slot");
 
   m_TLSListLock = new CriticalSection();
-  m_TLSList = new vector<TLSData *>();
+  m_TLSList = new rdcarray<TLSData *>();
 
   CacheDebuggerPresent();
 }
