@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -31,7 +31,7 @@
 #include <functional>
 #include <map>
 #include <set>
-#include <vector>
+#include "common/common.h"
 #include "common/threading.h"
 #include "hooks/hooks.h"
 #include "os/os_specific.h"
@@ -39,11 +39,8 @@
 
 #define VERBOSE_DEBUG_HOOK OPTION_OFF
 
-using std::vector;
-using std::map;
-
 // map from address of IAT entry, to original contents
-map<void **, void *> s_InstalledHooks;
+std::map<void **, void *> s_InstalledHooks;
 Threading::CriticalSection installedLock;
 
 bool ApplyHook(FunctionHook &hook, void **IATentry, bool &already)
@@ -93,11 +90,11 @@ struct DllHookset
   bool hooksfetched = false;
   // if we have multiple copies of the dll loaded (unlikely), the other module handles will be
   // stored here
-  vector<HMODULE> altmodules;
-  vector<FunctionHook> FunctionHooks;
+  rdcarray<HMODULE> altmodules;
+  rdcarray<FunctionHook> FunctionHooks;
   DWORD OrdinalBase = 0;
-  vector<string> OrdinalNames;
-  std::vector<FunctionLoadCallback> Callbacks;
+  rdcarray<rdcstr> OrdinalNames;
+  rdcarray<FunctionLoadCallback> Callbacks;
   Threading::CriticalSection ordinallock;
 
   void FetchOrdinalNames()
@@ -160,12 +157,12 @@ struct CachedHookData
     RDCEraseEl(lowername);
   }
 
-  map<string, DllHookset> DllHooks;
+  std::map<rdcstr, DllHookset> DllHooks;
   HMODULE ownmodule;
   Threading::CriticalSection lock;
   char lowername[512];
 
-  std::set<std::string> ignores;
+  std::set<rdcstr> ignores;
 
   bool missedOrdinals;
 
@@ -245,7 +242,7 @@ struct CachedHookData
           DWORD err = GetLastError();
           char *slash = strrchr(filename, L'\\');
 
-          string basename = slash ? strlower(string(slash + 1)) : "";
+          rdcstr basename = slash ? strlower(rdcstr(slash + 1)) : "";
 
           if(err == 0 && basename == it->first)
           {
@@ -599,7 +596,7 @@ static void HookAllModules()
       }
     }
 
-    std::vector<FunctionLoadCallback> callbacks;
+    rdcarray<FunctionLoadCallback> callbacks;
     // don't call callbacks next time
     callbacks.swap(it->second.Callbacks);
 
@@ -630,10 +627,13 @@ static bool IsAPISet(const wchar_t *filename)
 
 static bool IsAPISet(const char *filename)
 {
-  std::wstring wfn;
+  size_t len = strlen(filename);
+  rdcwstr wfn(len);
+
   // assume ASCII not UTF, just upcast plainly to wchar_t
-  while(*filename)
-    wfn.push_back(wchar_t(*filename++));
+  for(size_t i = 0; i < len; i++)
+    wfn[i] = wchar_t(filename[i]);
+
   return IsAPISet(wfn.c_str());
 }
 
@@ -706,10 +706,7 @@ static bool OrdinalAsString(void *func)
 
 FARPROC WINAPI Hooked_GetProcAddress(HMODULE mod, LPCSTR func)
 {
-  if(mod == NULL || func == NULL)
-    return (FARPROC)NULL;
-
-  if(mod == s_HookData->ownmodule)
+  if(mod == NULL || func == NULL || mod == s_HookData->ownmodule)
     return GetProcAddress(mod, func);
 
 #if ENABLED(VERBOSE_DEBUG_HOOK)
@@ -831,17 +828,17 @@ void LibraryHooks::RegisterFunctionHook(const char *libraryName, const FunctionH
       return;
     }
   }
-  s_HookData->DllHooks[strlower(string(libraryName))].FunctionHooks.push_back(hook);
+  s_HookData->DllHooks[strlower(rdcstr(libraryName))].FunctionHooks.push_back(hook);
 }
 
 void LibraryHooks::RegisterLibraryHook(const char *libraryName, FunctionLoadCallback loadedCallback)
 {
-  s_HookData->DllHooks[strlower(string(libraryName))].Callbacks.push_back(loadedCallback);
+  s_HookData->DllHooks[strlower(rdcstr(libraryName))].Callbacks.push_back(loadedCallback);
 }
 
 void LibraryHooks::IgnoreLibrary(const char *libraryName)
 {
-  std::string lowername = libraryName;
+  rdcstr lowername = libraryName;
 
   for(size_t i = 0; i < lowername.size(); i++)
     lowername[i] = (char)tolower(lowername[i]);
@@ -962,4 +959,4 @@ ScopedSuppressHooking::ScopedSuppressHooking()
 
 ScopedSuppressHooking::~ScopedSuppressHooking()
 {
-}
\ No newline at end of file
+}
