@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,6 +25,9 @@
 
 #include <winsock2.h>
 #include <ws2tcpip.h>
+#include "api/replay/stringise.h"
+#include "common/common.h"
+#include "common/formatting.h"
 #include "os/os_specific.h"
 
 #ifndef WSA_FLAG_NO_HANDLE_INHERIT
@@ -35,7 +38,7 @@
 #define WSA_FLAG_OVERLAPPED 0x01
 #endif
 
-static std::string wsaerr_string(int err)
+static rdcstr wsaerr_string(int err)
 {
   switch(err)
   {
@@ -395,9 +398,8 @@ Socket *CreateClientSocket(const char *host, uint16_t port, int timeoutMS)
   wchar_t portwstr[7] = {0};
 
   {
-    char buf[7] = {0};
-    int n = StringFormat::snprintf(buf, 6, "%d", port);
-    for(int i = 0; i < n && i < 6; i++)
+    rdcstr buf = ToStr(port);
+    for(size_t i = 0; i < buf.size(); i++)
       portwstr[i] = (wchar_t)buf[i];
   }
 
@@ -407,7 +409,7 @@ Socket *CreateClientSocket(const char *host, uint16_t port, int timeoutMS)
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_protocol = IPPROTO_TCP;
 
-  std::wstring whost = StringFormat::UTF82Wide(string(host));
+  rdcwstr whost = StringFormat::UTF82Wide(rdcstr(host));
 
   addrinfoW *addrResult = NULL;
   int res = GetAddrInfoW(whost.c_str(), portwstr, &hints, &addrResult);
