@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,8 +24,9 @@
 
 #include <string.h>
 #include "api/app/renderdoc_app.h"
-#include "api/replay/renderdoc_replay.h"    // for RENDERDOC_API to export the RENDERDOC_GetAPI function
+#include "api/replay/apidefs.h"    // for RENDERDOC_API to export the RENDERDOC_GetAPI function
 #include "common/common.h"
+#include "common/formatting.h"
 #include "core/core.h"
 #include "hooks/hooks.h"
 #include "serialise/rdcfile.h"
@@ -79,7 +80,7 @@ static uint32_t GetNumCaptures()
 
 static uint32_t GetCapture(uint32_t idx, char *filename, uint32_t *pathlength, uint64_t *timestamp)
 {
-  vector<CaptureData> caps = RenderDoc::Inst().GetCaptures();
+  rdcarray<CaptureData> caps = RenderDoc::Inst().GetCaptures();
 
   if(idx >= (uint32_t)caps.size())
   {
@@ -106,10 +107,10 @@ static uint32_t GetCapture(uint32_t idx, char *filename, uint32_t *pathlength, u
 
 static void SetCaptureFileComments(const char *filePath, const char *comments)
 {
-  std::string path;
+  rdcstr path;
   if(filePath == NULL || filePath[0] == 0)
   {
-    vector<CaptureData> caps = RenderDoc::Inst().GetCaptures();
+    rdcarray<CaptureData> caps = RenderDoc::Inst().GetCaptures();
     if(caps.empty())
     {
       RDCERR(
@@ -140,7 +141,7 @@ static void SetCaptureFileComments(const char *filePath, const char *comments)
 
   if(comments)
   {
-    std::string commentsjson = "{\"comments\":\"";
+    rdcstr commentsjson = "{\"comments\":\"";
 
     commentsjson.reserve(strlen(comments));
 
@@ -194,12 +195,12 @@ static uint32_t IsTargetControlConnected()
 
 static uint32_t LaunchReplayUI(uint32_t connectTargetControl, const char *cmdline)
 {
-  string replayapp = FileIO::GetReplayAppFilename();
+  rdcstr replayapp = FileIO::GetReplayAppFilename();
 
   if(replayapp.empty())
     return 0;
 
-  string cmd = cmdline ? cmdline : "";
+  rdcstr cmd = cmdline ? cmdline : "";
   if(connectTargetControl)
     cmd += StringFormat::Fmt(" --targetcontrol localhost:%u",
                              RenderDoc::Inst().GetTargetControlIdent());
@@ -312,7 +313,7 @@ extern "C" RENDERDOC_API int RENDERDOC_CC RENDERDOC_GetAPI(RENDERDOC_Version ver
   int ret = 0;
   int major = 0, minor = 0, patch = 0;
 
-  string supportedVersions = "";
+  rdcstr supportedVersions = "";
 
 #define API_VERSION_HANDLE(enumver, actualver)                     \
   supportedVersions += " " STRINGIZE(CONCAT(API_, enumver));       \
