@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,7 +22,6 @@
  * THE SOFTWARE.
  ******************************************************************************/
 
-#include "api/replay/renderdoc_replay.h"
 #include "core/core.h"
 #include "jpeg-compressor/jpgd.h"
 #include "jpeg-compressor/jpge.h"
@@ -35,10 +34,8 @@
 
 static void writeToByteVector(void *context, void *data, int size)
 {
-  std::vector<byte> *vec = (std::vector<byte> *)context;
-  byte *start = (byte *)data;
-  byte *end = start + size;
-  vec->insert(vec->end(), start, end);
+  bytebuf *buf = (bytebuf *)context;
+  buf->append((byte *)data, size);
 }
 
 static RDCDriver driverFromName(const char *driverName)
@@ -120,9 +117,10 @@ public:
   rdcstr ErrorString() { return m_ErrorString; }
   void Shutdown() { delete this; }
   ReplaySupport LocalReplaySupport() { return m_Support; }
-  const char *DriverName() { return m_DriverName.c_str(); }
+  rdcstr DriverName() { return m_DriverName; }
   const char *RecordedMachineIdent() { return m_Ident.c_str(); }
-  rdcpair<ReplayStatus, IReplayController *> OpenCapture(RENDERDOC_ProgressCallback progress);
+  rdcpair<ReplayStatus, IReplayController *> OpenCapture(const ReplayOptions &opts,
+                                                         RENDERDOC_ProgressCallback progress);
 
   void SetMetadata(const char *driverName, uint64_t machineIdent, FileType thumbType,
                    uint32_t thumbWidth, uint32_t thumbHeight, const bytebuf &thumbData);
@@ -135,6 +133,7 @@ public:
     return RenderDoc::Inst().GetCaptureFileFormats();
   }
 
+  rdcarray<GPUDevice> GetAvailableGPUs() { return RenderDoc::Inst().GetAvailableGPUs(); }
   const SDFile &GetStructuredData()
   {
     // decompile to structured data on demand.
@@ -170,7 +169,7 @@ public:
   bool WriteSection(const SectionProperties &props, const bytebuf &contents);
 
   bool HasCallstacks();
-  bool InitResolver(RENDERDOC_ProgressCallback progress);
+  bool InitResolver(bool interactive, RENDERDOC_ProgressCallback progress);
   rdcarray<rdcstr> GetResolve(const rdcarray<uint64_t> &callstack);
 
 private:
@@ -183,7 +182,7 @@ private:
 
   SDFile m_StructuredData;
 
-  std::string m_DriverName, m_Ident, m_ErrorString;
+  rdcstr m_DriverName, m_Ident, m_ErrorString;
   ReplaySupport m_Support = ReplaySupport::Unsupported;
 };
 
@@ -244,14 +243,12 @@ ReplayStatus CaptureFile::OpenBuffer(const bytebuf &buffer, const char *filetype
 {
   CaptureImporter importer = RenderDoc::Inst().GetCaptureImporter(filetype);
 
-  std::vector<byte> vec(buffer.begin(), buffer.end());
-
   if(importer)
   {
     ReplayStatus ret;
 
     {
-      StreamReader reader(vec);
+      StreamReader reader(buffer);
       m_RDC = new RDCFile;
       ret = importer(NULL, reader, m_RDC, m_StructuredData, progress);
     }
@@ -272,7 +269,7 @@ ReplayStatus CaptureFile::OpenBuffer(const bytebuf &buffer, const char *filetype
       progress(0.0f);
 
     m_RDC = new RDCFile;
-    m_RDC->Open(vec);
+    m_RDC->Open(buffer);
 
     if(progress)
       progress(1.0f);
@@ -298,10 +295,10 @@ ReplayStatus CaptureFile::Init()
 
   switch(m_RDC->ErrorCode())
   {
-    case ContainerError::FileNotFound: return ReplayStatus::FileNotFound; break;
-    case ContainerError::FileIO: return ReplayStatus::FileIOFailed; break;
-    case ContainerError::Corrupt: return ReplayStatus::FileCorrupted; break;
-    case ContainerError::UnsupportedVersion: return ReplayStatus::FileIncompatibleVersion; break;
+    case ContainerError::FileNotFound: return ReplayStatus::FileNotFound;
+    case ContainerError::FileIO: return ReplayStatus::FileIOFailed;
+    case ContainerError::Corrupt: return ReplayStatus::FileCorrupted;
+    case ContainerError::UnsupportedVersion: return ReplayStatus::FileIncompatibleVersion;
     case ContainerError::NoError:
     {
       RDCDriver driverType = m_RDC->GetDriver();
@@ -352,24 +349,27 @@ void CaptureFile::InitStructuredData(RENDERDOC_ProgressCallback progress /*= REN
   }
 }
 
-rdcpair<ReplayStatus, IReplayController *> CaptureFile::OpenCapture(RENDERDOC_ProgressCallback progress)
+rdcpair<ReplayStatus, IReplayController *> CaptureFile::OpenCapture(const ReplayOptions &opts,
+                                                                    RENDERDOC_ProgressCallback progress)
 {
   if(!m_RDC || m_RDC->ErrorCode() != ContainerError::NoError)
-    return make_rdcpair<ReplayStatus, IReplayController *>(ReplayStatus::InternalError, NULL);
+    return rdcpair<ReplayStatus, IReplayController *>(ReplayStatus::InternalError, NULL);
 
   ReplayController *render = new ReplayController();
   ReplayStatus ret;
 
+  LogReplayOptions(opts);
+
   RenderDoc::Inst().SetProgressCallback<LoadProgress>(progress);
 
-  ret = render->CreateDevice(m_RDC);
+  ret = render->CreateDevice(m_RDC, opts);
 
   RenderDoc::Inst().SetProgressCallback<LoadProgress>(RENDERDOC_ProgressCallback());
 
   if(ret != ReplayStatus::Succeeded)
     SAFE_DELETE(render);
 
-  return make_rdcpair<ReplayStatus, IReplayController *>(ret, render);
+  return rdcpair<ReplayStatus, IReplayController *>(ret, render);
 }
 
 void CaptureFile::SetMetadata(const char *driverName, uint64_t machineIdent, FileType thumbType,
@@ -452,8 +452,8 @@ ReplayStatus CaptureFile::Convert(const char *filename, const char *filetype, co
   {
     switch(output.ErrorCode())
     {
-      case ContainerError::FileNotFound: return ReplayStatus::FileNotFound; break;
-      case ContainerError::FileIO: return ReplayStatus::FileIOFailed; break;
+      case ContainerError::FileNotFound: return ReplayStatus::FileNotFound;
+      case ContainerError::FileIO: return ReplayStatus::FileIOFailed;
       default: break;
     }
     return ReplayStatus::InternalError;
@@ -624,13 +624,13 @@ Thumbnail CaptureFile::GetThumbnail(FileType type, uint32_t maxsize)
       }
     }
 
-    std::vector<byte> encodedBytes;
+    bytebuf encodedBytes;
 
     switch(type)
     {
       case FileType::Raw:
       {
-        encodedBytes.assign(thumbpixels, thumbpixels + (thumbwidth * thumbheight * 3));
+        encodedBytes.assign(thumbpixels, thumbwidth * thumbheight * 3);
         break;
       }
       case FileType::JPG:
@@ -756,7 +756,7 @@ bool CaptureFile::HasCallstacks()
   return m_RDC && m_RDC->SectionIndex(SectionType::ResolveDatabase) >= 0;
 }
 
-bool CaptureFile::InitResolver(RENDERDOC_ProgressCallback progress)
+bool CaptureFile::InitResolver(bool interactive, RENDERDOC_ProgressCallback progress)
 {
   if(!HasCallstacks())
   {
@@ -774,7 +774,7 @@ bool CaptureFile::InitResolver(RENDERDOC_ProgressCallback progress)
 
   StreamReader *reader = m_RDC->ReadSection(idx);
 
-  std::vector<byte> buf;
+  bytebuf buf;
   buf.resize((size_t)reader->GetSize());
   bool success = reader->Read(buf.data(), reader->GetSize());
 
@@ -789,7 +789,7 @@ bool CaptureFile::InitResolver(RENDERDOC_ProgressCallback progress)
   if(progress)
     progress(0.002f);
 
-  m_Resolver = Callstack::MakeResolver(buf.data(), buf.size(), progress);
+  m_Resolver = Callstack::MakeResolver(interactive, buf.data(), buf.size(), progress);
 
   if(!m_Resolver)
   {
