@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,10 +27,12 @@
 #include "api/replay/renderdoc_replay.h"
 #include "api/replay/version.h"
 #include "common/common.h"
+#include "common/formatting.h"
 #include "core/core.h"
 #include "maths/camera.h"
 #include "maths/formatpacking.h"
 #include "miniz/miniz.h"
+#include "replay/replay_driver.h"
 #include "strings/string_utils.h"
 
 // these entry points are for the replay/analysis side - not for the application.
@@ -143,13 +145,13 @@ extern "C" RENDERDOC_API uint32_t RENDERDOC_CC RENDERDOC_VertexOffset(Topology t
     case Topology::LineLoop:
     case Topology::TriangleStrip:
     case Topology::LineStrip_Adj:
+    case Topology::TriangleFan:
       // for strips, each new vertex creates a new primitive
       return primitive;
     case Topology::TriangleStrip_Adj:
       // triangle strip with adjacency is a special case as every other
       // vert is purely for adjacency so it's doubled
       return primitive * 2;
-    case Topology::TriangleFan: RDCERR("Cannot get VertexOffset for triangle fan!"); break;
   }
 
   return primitive * RENDERDOC_NumVerticesPerPrimitive(topology);
@@ -205,10 +207,8 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_SetColors(FloatVector darkC
                                                                FloatVector lightChecker,
                                                                bool darkTheme)
 {
-  RenderDoc::Inst().SetDarkCheckerboardColor(
-      Vec4f(darkChecker.x, darkChecker.y, darkChecker.z, darkChecker.w));
-  RenderDoc::Inst().SetLightCheckerboardColor(
-      Vec4f(lightChecker.x, lightChecker.y, lightChecker.z, lightChecker.w));
+  RenderDoc::Inst().SetDarkCheckerboardColor(darkChecker);
+  RenderDoc::Inst().SetLightCheckerboardColor(lightChecker);
   RenderDoc::Inst().SetDarkTheme(darkTheme);
 }
 
@@ -226,17 +226,31 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_SetDebugLogFile(const char
 
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_LogText(const char *text)
 {
-  rdclog_direct(Timing::GetUTCTime(), Process::GetCurrentPID(), LogType::Comment, "EXT", "external",
-                0, "%s", text);
+  rdclog_direct(FILL_AUTO_VALUE, FILL_AUTO_VALUE, LogType::Comment, "EXT", "external", 0, "%s", text);
 }
 
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_LogMessage(LogType type, const char *project,
                                                                 const char *file, unsigned int line,
                                                                 const char *text)
 {
-  rdclog_direct(Timing::GetUTCTime(), Process::GetCurrentPID(), type, project ? project : "UNK?",
+  rdclog_direct(FILL_AUTO_VALUE, FILL_AUTO_VALUE, type, project ? project : "UNK?",
                 file ? file : "unknown", line, "%s", text);
 
+  // see comment in common.h
+  RDCCOMPILE_ASSERT((uint32_t)LogType::Debug == (uint32_t)LogType__Internal::Debug,
+                    "External and internal LogType enums must match");
+  RDCCOMPILE_ASSERT((uint32_t)LogType::Comment == (uint32_t)LogType__Internal::Comment,
+                    "External and internal LogType enums must match");
+  RDCCOMPILE_ASSERT((uint32_t)LogType::Warning == (uint32_t)LogType__Internal::Warning,
+                    "External and internal LogType enums must match");
+  RDCCOMPILE_ASSERT((uint32_t)LogType::Error == (uint32_t)LogType__Internal::Error,
+                    "External and internal LogType enums must match");
+  RDCCOMPILE_ASSERT((uint32_t)LogType::Fatal == (uint32_t)LogType__Internal::Fatal,
+                    "External and internal LogType enums must match");
+  RDCCOMPILE_ASSERT((uint32_t)LogType::Count == (uint32_t)LogType__Internal::Count,
+                    "External and internal LogType enums must match");
+  RDCCOMPILE_ASSERT(arraydim<LogType>() == 5, "External and internal LogType enums must match");
+
 #if ENABLED(DEBUGBREAK_ON_ERROR_LOG)
   if(type == LogType::Error)
     RDCBREAK();
@@ -251,10 +265,15 @@ extern "C" RENDERDOC_API const char *RENDERDOC_CC RENDERDOC_GetLogFile()
   return RDCGETLOGFILE();
 }
 
+extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_GetLogFileContents(rdcstr &logfile)
+{
+  logfile = FileIO::logfile_readall(RDCGETLOGFILE());
+}
+
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_InitGlobalEnv(GlobalEnvironment env,
                                                                    const rdcarray<rdcstr> &args)
 {
-  std::vector<std::string> argsVec;
+  rdcarray<rdcstr> argsVec;
   argsVec.reserve(args.size());
   for(const rdcstr &a : args)
     argsVec.push_back(a.c_str());
@@ -283,7 +302,11 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_CreateBugReport(const char
   mz_zip_archive zip;
   RDCEraseEl(zip);
 
-  report = FileIO::GetTempFolderFilename() + "/renderdoc_report.zip";
+  if(report.empty())
+  {
+    report = FileIO::GetTempFolderFilename() +
+             StringFormat::sntimef(Timing::GetUTCTime(), "/renderdoc_report_%H%M%S.zip");
+  }
 
   FileIO::Delete(report.c_str());
 
@@ -294,7 +317,7 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_CreateBugReport(const char
 
   if(logfile && logfile[0])
   {
-    std::string contents = FileIO::logfile_readall(logfile);
+    rdcstr contents = FileIO::logfile_readall(logfile);
     mz_zip_writer_add_mem(&zip, "error.log", contents.data(), contents.length(), MZ_BEST_COMPRESSION);
   }
 
@@ -323,8 +346,13 @@ RENDERDOC_ExecuteAndInject(const char *app, const char *workingDir, const char *
                            const rdcarray<EnvironmentModification> &env, const char *capturefile,
                            const CaptureOptions &opts, bool waitForExit)
 {
-  return Process::LaunchAndInjectIntoProcess(app, workingDir, cmdLine, env, capturefile, opts,
-                                             waitForExit != 0);
+  rdcpair<ReplayStatus, uint32_t> status = Process::LaunchAndInjectIntoProcess(
+      app, workingDir, cmdLine, env, capturefile, opts, waitForExit != 0);
+
+  ExecuteResult ret;
+  ret.status = status.first;
+  ret.ident = status.second;
+  return ret;
 }
 
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_GetDefaultCaptureOptions(CaptureOptions *opts)
@@ -358,25 +386,42 @@ extern "C" RENDERDOC_API ExecuteResult RENDERDOC_CC
 RENDERDOC_InjectIntoProcess(uint32_t pid, const rdcarray<EnvironmentModification> &env,
                             const char *capturefile, const CaptureOptions &opts, bool waitForExit)
 {
-  return Process::InjectIntoProcess(pid, env, capturefile, opts, waitForExit != 0);
+  rdcpair<ReplayStatus, uint32_t> status =
+      Process::InjectIntoProcess(pid, env, capturefile, opts, waitForExit != 0);
+
+  ExecuteResult ret;
+  ret.status = status.first;
+  ret.ident = status.second;
+  return ret;
 }
 
-extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_FreeArrayMem(const void *mem)
+extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_FreeArrayMem(void *mem)
 {
-  free((void *)mem);
+  free(mem);
+}
+
+// not exported, this is needed for calling from the container allocate functions
+void RENDERDOC_OutOfMemory(uint64_t sz)
+{
+  RDCFATAL("Allocation failed for %llu bytes", sz);
 }
 
 extern "C" RENDERDOC_API void *RENDERDOC_CC RENDERDOC_AllocArrayMem(uint64_t sz)
 {
-  return malloc((size_t)sz);
+  void *ret = malloc((size_t)sz);
+  if(ret == NULL)
+    RENDERDOC_OutOfMemory(sz);
+  return ret;
 }
 
-extern "C" RENDERDOC_API uint32_t RENDERDOC_CC RENDERDOC_EnumerateRemoteTargets(const char *host,
+extern "C" RENDERDOC_API uint32_t RENDERDOC_CC RENDERDOC_EnumerateRemoteTargets(const char *URL,
                                                                                 uint32_t nextIdent)
 {
-  string s = "localhost";
-  if(host != NULL && host[0] != '\0')
-    s = host;
+  rdcstr host = "localhost";
+  if(URL != NULL && URL[0] != '\0')
+    host = URL;
+
+  rdcstr deviceID = host;
 
   // initial case is we're called with 0, start with the first port.
   // otherwise we're called with the last successful ident, so increment
@@ -386,32 +431,30 @@ extern "C" RENDERDOC_API uint32_t RENDERDOC_CC RENDERDOC_EnumerateRemoteTargets(
   else
     nextIdent++;
 
-  bool isAndroid = false;
-  uint32_t lastIdent = RenderDoc_LastTargetControlPort;
-  if(host != NULL && Android::IsHostADB(host))
+  IDeviceProtocolHandler *protocol = RenderDoc::Inst().GetDeviceProtocol(deviceID);
+
+  if(protocol)
   {
-    int index = 0;
-    std::string deviceID;
-    Android::ExtractDeviceIDAndIndex(host, index, deviceID);
-
-    isAndroid = true;
-
-    // each subsequent device gets a new range of ports. The deviceID isn't needed since we already
-    // forwarded the ports to the right devices.
-    if(nextIdent == RenderDoc_FirstTargetControlPort)
-      nextIdent += RenderDoc_AndroidPortOffset * (index + 1);
-    lastIdent += RenderDoc_AndroidPortOffset * (index + 1);
-
-    s = "127.0.0.1";
+    deviceID = protocol->GetDeviceID(deviceID);
+    host = protocol->RemapHostname(deviceID);
+    if(host.empty())
+      return 0;
   }
 
-  for(; nextIdent <= lastIdent; nextIdent++)
+  for(; nextIdent <= RenderDoc_LastTargetControlPort; nextIdent++)
   {
-    Network::Socket *sock = Network::CreateClientSocket(s.c_str(), (uint16_t)nextIdent, 250);
+    uint16_t port = (uint16_t)nextIdent;
+    if(protocol)
+      port = protocol->RemapPort(deviceID, port);
+
+    if(port == 0)
+      return 0;
+
+    Network::Socket *sock = Network::CreateClientSocket(host.c_str(), port, 250);
 
     if(sock)
     {
-      if(isAndroid)
+      if(protocol)
       {
         Threading::Sleep(100);
         (void)sock->IsRecvDataWaiting();
@@ -431,14 +474,21 @@ extern "C" RENDERDOC_API uint32_t RENDERDOC_CC RENDERDOC_EnumerateRemoteTargets(
   return 0;
 }
 
-extern "C" RENDERDOC_API uint32_t RENDERDOC_CC RENDERDOC_GetDefaultRemoteServerPort()
+extern "C" RENDERDOC_API void RENDERDOC_CC
+RENDERDOC_GetSupportedDeviceProtocols(rdcarray<rdcstr> *supportedProtocols)
 {
-  return RenderDoc_RemoteServerPort;
+  *supportedProtocols = RenderDoc::Inst().GetSupportedDeviceProtocols();
 }
 
-extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_BecomeRemoteServer(
-    const char *listenhost, uint32_t port, RENDERDOC_KillCallback killReplay,
-    RENDERDOC_PreviewWindowCallback previewWindow)
+extern "C" RENDERDOC_API IDeviceProtocolController *RENDERDOC_CC
+RENDERDOC_GetDeviceProtocolController(const rdcstr &protocol)
+{
+  return RenderDoc::Inst().GetDeviceProtocol(protocol);
+}
+
+extern "C" RENDERDOC_API void RENDERDOC_CC
+RENDERDOC_BecomeRemoteServer(const char *listenhost, RENDERDOC_KillCallback killReplay,
+                             RENDERDOC_PreviewWindowCallback previewWindow)
 {
   if(listenhost == NULL || listenhost[0] == 0)
     listenhost = "0.0.0.0";
@@ -454,14 +504,15 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_BecomeRemoteServer(
       return ret;
     };
 
-  if(port == 0)
-    port = RENDERDOC_GetDefaultRemoteServerPort();
-
-  RenderDoc::Inst().BecomeRemoteServer(listenhost, (uint16_t)port, killReplay, previewWindow);
+  RenderDoc::Inst().BecomeRemoteServer(listenhost, RenderDoc_RemoteServerPort, killReplay,
+                                       previewWindow);
 }
 
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_StartSelfHostCapture(const char *dllname)
 {
+  if(!Process::IsModuleLoaded(dllname))
+    return;
+
   void *module = Process::LoadModule(dllname);
 
   if(module == NULL)
@@ -485,6 +536,9 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_StartSelfHostCapture(const
 
 extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_EndSelfHostCapture(const char *dllname)
 {
+  if(!Process::IsModuleLoaded(dllname))
+    return;
+
   void *module = Process::LoadModule(dllname);
 
   if(module == NULL)
@@ -510,8 +564,8 @@ extern "C" RENDERDOC_API bool RENDERDOC_CC
 RENDERDOC_NeedVulkanLayerRegistration(VulkanLayerRegistrationInfo *info)
 {
   VulkanLayerFlags flags = VulkanLayerFlags::NoFlags;
-  std::vector<std::string> myJSONs;
-  std::vector<std::string> otherJSONs;
+  rdcarray<rdcstr> myJSONs;
+  rdcarray<rdcstr> otherJSONs;
 
   bool ret = RenderDoc::Inst().NeedVulkanLayerRegistration(flags, myJSONs, otherJSONs);
 
@@ -605,7 +659,7 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_UpdateInstalledVersionNumbe
         DWORD Version = (RENDERDOC_VERSION_MAJOR << 24) | (RENDERDOC_VERSION_MINOR << 16);
         DWORD VersionMajor = RENDERDOC_VERSION_MAJOR;
         DWORD VersionMinor = RENDERDOC_VERSION_MINOR;
-        std::string DisplayVersion = MAJOR_MINOR_VERSION_STRING ".0";
+        rdcstr DisplayVersion = MAJOR_MINOR_VERSION_STRING ".0";
 
         RegSetValueExA(subkey, "Version", 0, REG_DWORD, (const BYTE *)&Version, sizeof(Version));
         RegSetValueExA(subkey, "VersionMajor", 0, REG_DWORD, (const BYTE *)&VersionMajor,
@@ -628,9 +682,9 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_UpdateInstalledVersionNumbe
 #endif
 }
 
-static std::string ResourceFormatName(const ResourceFormat &fmt)
+static rdcstr ResourceFormatName(const ResourceFormat &fmt)
 {
-  std::string ret;
+  rdcstr ret;
 
   if(fmt.Special())
   {
@@ -690,12 +744,12 @@ static std::string ResourceFormatName(const ResourceFormat &fmt)
         ret = fmt.BGRAOrder() ? "B10G10R10A2" : "R10G10B10A2";
         break;
       case ResourceFormatType::R11G11B10: return "R11G11B10_FLOAT";
-      case ResourceFormatType::R5G6B5: return fmt.BGRAOrder() ? "R5G6B5_UNORM" : "B5G6R5_UNORM";
+      case ResourceFormatType::R5G6B5: return fmt.BGRAOrder() ? "B5G6R5_UNORM" : "R5G6B5_UNORM";
       case ResourceFormatType::R5G5B5A1:
-        return fmt.BGRAOrder() ? "R5G5B5A1_UNORM" : "B5G5R5A1_UNORM";
+        return fmt.BGRAOrder() ? "B5G5R5A1_UNORM" : "R5G5B5A1_UNORM";
       case ResourceFormatType::R9G9B9E5: return "R9G9B9E5_FLOAT";
       case ResourceFormatType::R4G4B4A4:
-        return fmt.BGRAOrder() ? "R4G4B4A4_UNORM" : "B4G4R4A4_UNORM";
+        return fmt.BGRAOrder() ? "B4G4R4A4_UNORM" : "R4G4B4A4_UNORM";
       case ResourceFormatType::R4G4: return "R4G4_UNORM";
       case ResourceFormatType::D16S8:
         return fmt.compType == CompType::Typeless ? "D16S8_TYPELESS" : "D16S8";
@@ -704,6 +758,7 @@ static std::string ResourceFormatName(const ResourceFormat &fmt)
       case ResourceFormatType::D32S8:
         return fmt.compType == CompType::Typeless ? "D32S8_TYPELESS" : "D32S8";
       case ResourceFormatType::S8: return "S8";
+      case ResourceFormatType::A8: return "A8_UNORM";
       case ResourceFormatType::YUV8:
       case ResourceFormatType::YUV10:
       case ResourceFormatType::YUV12:
@@ -805,7 +860,7 @@ extern "C" RENDERDOC_API void RENDERDOC_CC RENDERDOC_ResourceFormatName(const Re
   name = ResourceFormatName(fmt);
 }
 
-static void TestPrintMsg(const std::string &msg)
+static void TestPrintMsg(const rdcstr &msg)
 {
   OSUtility::WriteOutput(OSUtility::Output_DebugMon, msg.c_str());
   OSUtility::WriteOutput(OSUtility::Output_StdErr, msg.c_str());
@@ -817,30 +872,30 @@ extern "C" RENDERDOC_API int RENDERDOC_CC RENDERDOC_RunFunctionalTests(int pytho
 #if ENABLED(RDOC_WIN32)
   const char *moduledir = "/pymodules";
   const char *modulename = "renderdoc.pyd";
-  std::string pythonlibs[] = {"python3?.dll"};
+  rdcstr pythonlibs[] = {"python3?.dll"};
 #elif ENABLED(RDOC_LINUX)
   const char *moduledir = "";
   const char *modulename = "renderdoc.so";
   // we don't care about pymalloc or not
-  std::string pythonlibs[] = {"libpython3.?m.so.1.0", "libpython3.?.so.1.0", "libpython3.?m.so",
-                              "libpython3.?.so"};
+  rdcstr pythonlibs[] = {"libpython3.?m.so.1.0", "libpython3.?.so.1.0", "libpython3.?m.so",
+                         "libpython3.?.so"};
 #else
   const char *moduledir = "";
   const char *modulename = "";
-  std::string pythonlibs[] = {};
+  rdcstr pythonlibs[] = {};
   TestPrintMsg(
       "Running functional tests not directly supported on this platform.\n"
       "Try running util/test/run_tests.py manually.\n");
   return 1;
 #endif
 
-  std::string libPath;
+  rdcstr libPath;
   FileIO::GetLibraryFilename(libPath);
 
   libPath = get_dirname(libPath);
-  std::string modulePath = libPath + moduledir;
+  rdcstr modulePath = libPath + moduledir;
 
-  std::string moduleFilename = modulePath + "/" + modulename;
+  rdcstr moduleFilename = modulePath + "/" + modulename;
 
   if(!FileIO::exists(moduleFilename.c_str()))
   {
@@ -852,7 +907,7 @@ extern "C" RENDERDOC_API int RENDERDOC_CC RENDERDOC_RunFunctionalTests(int pytho
   // directories from the library will put us at the project root. This is the most common scenario
   // and we don't add handling for locating the script elsewhere as in that case the user can run it
   // directly. This is just intended as a useful shortcut for common cases.
-  std::string scriptPath = libPath + "/../../util/test/run_tests.py";
+  rdcstr scriptPath = libPath + "/../../util/test/run_tests.py";
 
   if(!FileIO::exists(scriptPath.c_str()))
   {
@@ -862,7 +917,7 @@ extern "C" RENDERDOC_API int RENDERDOC_CC RENDERDOC_RunFunctionalTests(int pytho
 
   void *handle = NULL;
 
-  for(std::string py : pythonlibs)
+  for(rdcstr py : pythonlibs)
   {
     // patch up the python minor version
     char *ver = strchr(&py[0], '?');
@@ -892,29 +947,28 @@ extern "C" RENDERDOC_API int RENDERDOC_CC RENDERDOC_RunFunctionalTests(int pytho
     return 1;
   }
 
-  std::vector<std::wstring> wideArgs(args.size());
+  rdcarray<rdcwstr> wideArgs(args.size());
 
   for(size_t i = 0; i < args.size(); i++)
     wideArgs[i] = StringFormat::UTF82Wide(args[i]);
 
   // insert fake arguments to point at the script and our modules
-  wideArgs.insert(wideArgs.begin(),
-                  {
-                      L"python",
-                      // specify script path
-                      StringFormat::UTF82Wide(scriptPath),
-                      // specify native library path
-                      L"--renderdoc", StringFormat::UTF82Wide(libPath),
-                      // specify python module path
-                      L"--pyrenderdoc", StringFormat::UTF82Wide(modulePath),
-                      // force in-process as we can't fork out to python to pass args
-                      L"--in-process",
-                  });
+  wideArgs.insert(0, {
+                         L"python",
+                         // specify script path
+                         StringFormat::UTF82Wide(scriptPath),
+                         // specify native library path
+                         L"--renderdoc", StringFormat::UTF82Wide(libPath),
+                         // specify python module path
+                         L"--pyrenderdoc", StringFormat::UTF82Wide(modulePath),
+                         // force in-process as we can't fork out to python to pass args
+                         L"--in-process",
+                     });
 
-  std::vector<wchar_t *> wideArgStrings(wideArgs.size());
+  rdcarray<wchar_t *> wideArgStrings(wideArgs.size());
 
   for(size_t i = 0; i < wideArgs.size(); i++)
-    wideArgStrings[i] = &wideArgs[i][0];
+    wideArgStrings[i] = wideArgs[i].data();
 
   return mainFunc((int)wideArgStrings.size(), wideArgStrings.data());
 }
