@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -40,130 +40,6 @@
 #include "strings/string_utils.h"
 #include "tinyexr/tinyexr.h"
 
-float ConvertComponent(const ResourceFormat &fmt, const byte *data)
-{
-  if(fmt.compByteWidth == 8)
-  {
-    // we just downcast
-    const uint64_t *u64 = (const uint64_t *)data;
-    const int64_t *i64 = (const int64_t *)data;
-
-    if(fmt.compType == CompType::Double || fmt.compType == CompType::Float)
-    {
-      return float(*(const double *)u64);
-    }
-    else if(fmt.compType == CompType::UInt || fmt.compType == CompType::UScaled)
-    {
-      return float(*u64);
-    }
-    else if(fmt.compType == CompType::SInt || fmt.compType == CompType::SScaled)
-    {
-      return float(*i64);
-    }
-  }
-  else if(fmt.compByteWidth == 4)
-  {
-    const uint32_t *u32 = (const uint32_t *)data;
-    const int32_t *i32 = (const int32_t *)data;
-
-    if(fmt.compType == CompType::Float || fmt.compType == CompType::Depth)
-    {
-      return *(const float *)u32;
-    }
-    else if(fmt.compType == CompType::UInt || fmt.compType == CompType::UScaled)
-    {
-      return float(*u32);
-    }
-    else if(fmt.compType == CompType::SInt || fmt.compType == CompType::SScaled)
-    {
-      return float(*i32);
-    }
-  }
-  else if(fmt.compByteWidth == 3 && fmt.compType == CompType::Depth)
-  {
-    // 24-bit depth is a weird edge case we need to assemble it by hand
-    const uint8_t *u8 = (const uint8_t *)data;
-
-    uint32_t depth = 0;
-    depth |= uint32_t(u8[1]);
-    depth |= uint32_t(u8[2]) << 8;
-    depth |= uint32_t(u8[3]) << 16;
-
-    return float(depth) / float(16777215.0f);
-  }
-  else if(fmt.compByteWidth == 2)
-  {
-    const uint16_t *u16 = (const uint16_t *)data;
-    const int16_t *i16 = (const int16_t *)data;
-
-    if(fmt.compType == CompType::Float)
-    {
-      return ConvertFromHalf(*u16);
-    }
-    else if(fmt.compType == CompType::UInt || fmt.compType == CompType::UScaled)
-    {
-      return float(*u16);
-    }
-    else if(fmt.compType == CompType::SInt || fmt.compType == CompType::SScaled)
-    {
-      return float(*i16);
-    }
-    // 16-bit depth is UNORM
-    else if(fmt.compType == CompType::UNorm || fmt.compType == CompType::Depth)
-    {
-      return float(*u16) / 65535.0f;
-    }
-    else if(fmt.compType == CompType::SNorm)
-    {
-      float f = -1.0f;
-
-      if(*i16 == -32768)
-        f = -1.0f;
-      else
-        f = ((float)*i16) / 32767.0f;
-
-      return f;
-    }
-  }
-  else if(fmt.compByteWidth == 1)
-  {
-    const uint8_t *u8 = (const uint8_t *)data;
-    const int8_t *i8 = (const int8_t *)data;
-
-    if(fmt.compType == CompType::UInt || fmt.compType == CompType::UScaled)
-    {
-      return float(*u8);
-    }
-    else if(fmt.compType == CompType::SInt || fmt.compType == CompType::SScaled)
-    {
-      return float(*i8);
-    }
-    else if(fmt.compType == CompType::UNormSRGB)
-    {
-      return SRGB8_lookuptable[*u8];
-    }
-    else if(fmt.compType == CompType::UNorm)
-    {
-      return float(*u8) / 255.0f;
-    }
-    else if(fmt.compType == CompType::SNorm)
-    {
-      float f = -1.0f;
-
-      if(*i8 == -128)
-        f = -1.0f;
-      else
-        f = ((float)*i8) / 127.0f;
-
-      return f;
-    }
-  }
-
-  RDCERR("Unexpected format to convert from %u %u", fmt.compByteWidth, fmt.compType);
-
-  return 0.0f;
-}
-
 static void fileWriteFunc(void *context, void *data, int size)
 {
   FileIO::fwrite(data, 1, size, (FILE *)context);
@@ -272,13 +148,13 @@ rdcarray<rdcstr> ReplayController::GetDisassemblyTargets()
 
   rdcarray<rdcstr> ret;
 
-  vector<string> targets = m_pDevice->GetDisassemblyTargets();
+  rdcarray<rdcstr> targets = m_pDevice->GetDisassemblyTargets();
 
   ret.reserve(targets.size());
-  for(const std::string &t : targets)
+  for(const rdcstr &t : targets)
     ret.push_back(t);
 
-  for(const std::string &t : m_GCNTargets)
+  for(const rdcstr &t : m_GCNTargets)
     ret.push_back(t);
 
   return ret;
@@ -289,11 +165,11 @@ rdcstr ReplayController::DisassembleShader(ResourceId pipeline, const ShaderRefl
 {
   CHECK_REPLAY_THREAD();
 
-  for(const std::string &t : m_GCNTargets)
+  for(const rdcstr &t : m_GCNTargets)
     if(t == target)
       return GCNISA::Disassemble(refl->encoding, refl->stage, refl->rawBytes, target);
 
-  return m_pDevice->DisassembleShader(pipeline, refl, target);
+  return m_pDevice->DisassembleShader(m_pDevice->GetLiveID(pipeline), refl, target);
 }
 
 FrameDescription ReplayController::GetFrameInfo()
@@ -427,7 +303,7 @@ void ReplayController::AddFakeMarkers()
   if(ContainsMarker(draws))
     return;
 
-  std::vector<DrawcallDescription> ret;
+  rdcarray<DrawcallDescription> ret;
 
   int depthpassID = 1;
   int copypassID = 1;
@@ -539,9 +415,7 @@ rdcarray<CounterResult> ReplayController::FetchCounters(const rdcarray<GPUCounte
 {
   CHECK_REPLAY_THREAD();
 
-  std::vector<GPUCounter> counterArray(counters.begin(), counters.end());
-
-  return m_pDevice->FetchCounters(counterArray);
+  return m_pDevice->FetchCounters(counters);
 }
 
 rdcarray<GPUCounter> ReplayController::EnumerateCounters()
@@ -593,11 +467,12 @@ rdcarray<ShaderEntryPoint> ReplayController::GetShaderEntryPoints(ResourceId sha
   return m_pDevice->GetShaderEntryPoints(m_pDevice->GetLiveID(shader));
 }
 
-ShaderReflection *ReplayController::GetShader(ResourceId shader, ShaderEntryPoint entry)
+ShaderReflection *ReplayController::GetShader(ResourceId pipeline, ResourceId shader,
+                                              ShaderEntryPoint entry)
 {
   CHECK_REPLAY_THREAD();
 
-  return m_pDevice->GetShader(m_pDevice->GetLiveID(shader), entry);
+  return m_pDevice->GetShader(m_pDevice->GetLiveID(pipeline), m_pDevice->GetLiveID(shader), entry);
 }
 
 rdcarray<EventUsage> ReplayController::GetUsage(ResourceId id)
@@ -642,7 +517,7 @@ bytebuf ReplayController::GetBufferData(ResourceId buff, uint64_t offset, uint64
 
   if(liveId == ResourceId())
   {
-    RDCERR("Couldn't get Live ID for %llu getting buffer data", buff);
+    RDCERR("Couldn't get Live ID for %s getting buffer data", ToStr(buff).c_str());
     return retData;
   }
 
@@ -651,7 +526,7 @@ bytebuf ReplayController::GetBufferData(ResourceId buff, uint64_t offset, uint64
   return retData;
 }
 
-bytebuf ReplayController::GetTextureData(ResourceId tex, uint32_t arrayIdx, uint32_t mip)
+bytebuf ReplayController::GetTextureData(ResourceId tex, const Subresource &sub)
 {
   CHECK_REPLAY_THREAD();
 
@@ -661,11 +536,11 @@ bytebuf ReplayController::GetTextureData(ResourceId tex, uint32_t arrayIdx, uint
 
   if(liveId == ResourceId())
   {
-    RDCERR("Couldn't get Live ID for %llu getting texture data", tex);
+    RDCERR("Couldn't get Live ID for %s getting texture data", ToStr(tex).c_str());
     return ret;
   }
 
-  m_pDevice->GetTextureData(liveId, arrayIdx, mip, GetTextureDataParams(), ret);
+  m_pDevice->GetTextureData(liveId, sub, GetTextureDataParams(), ret);
 
   return ret;
 }
@@ -679,7 +554,7 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
 
   if(liveid == ResourceId())
   {
-    RDCERR("Couldn't get Live ID for %llu getting texture data", sd.resourceId);
+    RDCERR("Couldn't get Live ID for %s getting texture data", ToStr(sd.resourceId).c_str());
     return false;
   }
 
@@ -696,7 +571,7 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
   else
   {
     if(sd.sample.sampleIndex != ~0U)
-      sd.sample.sampleIndex = RDCCLAMP(sd.sample.sampleIndex, 0U, td.msSamp);
+      sd.sample.sampleIndex = RDCCLAMP((uint32_t)sd.sample.sampleIndex, 0U, td.msSamp);
   }
 
   // don't support cube cruciform for non cubemaps, or
@@ -727,6 +602,9 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
   uint32_t sampleCount = td.msSamp;
   bool multisampled = td.msSamp > 1;
 
+  if(sd.sample.mapToArray)
+    sd.sample.sampleIndex = 0;
+
   bool resolveSamples = (sd.sample.sampleIndex == ~0U);
 
   if(resolveSamples)
@@ -738,7 +616,7 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
 
   // treat any multisampled texture as if it were an array
   // of <sample count> dimension (on top of potential existing array
-  // dimension). GetTextureData() uses the same convention.
+  // dimension).
   if(td.msSamp > 1)
   {
     td.arraysize *= td.msSamp;
@@ -833,7 +711,7 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
     // otherwise take all mips, as by default
   }
 
-  vector<byte *> subdata;
+  rdcarray<byte *> subdata;
 
   bool downcast = false;
 
@@ -918,7 +796,8 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
   {
     switch(td.format.type)
     {
-      case ResourceFormatType::S8: bytesPerPixel = 1; break;
+      case ResourceFormatType::S8:
+      case ResourceFormatType::A8: bytesPerPixel = 1; break;
       case ResourceFormatType::R10G10B10A2:
       case ResourceFormatType::R9G9B9E5:
       case ResourceFormatType::R11G11B10:
@@ -953,14 +832,16 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
 
       GetTextureDataParams params;
       params.forDiskSave = true;
-      params.typeHint = sd.typeHint;
+      params.typeCast = sd.typeCast;
       params.resolve = resolveSamples;
       params.remap = remap;
       params.blackPoint = sd.comp.blackPoint;
       params.whitePoint = sd.comp.whitePoint;
 
+      Subresource sub = {mip, slice / sampleCount, slice % sampleCount};
+
       bytebuf data;
-      m_pDevice->GetTextureData(liveid, slice, mip, params, data);
+      m_pDevice->GetTextureData(liveid, sub, params, data);
 
       if(data.empty())
       {
@@ -1161,15 +1042,15 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
           case 4:
             memcpy(&subdata[0][(y * td.width + x) * pixelStride + 3 * compWidth], &max,
                    td.format.compByteWidth);
-          // deliberate fallthrough
+            DELIBERATE_FALLTHROUGH();
           case 3:
             memcpy(&subdata[0][(y * td.width + x) * pixelStride + 2 * compWidth], &val,
                    td.format.compByteWidth);
-          // deliberate fallthrough
+            DELIBERATE_FALLTHROUGH();
           case 2:
             memcpy(&subdata[0][(y * td.width + x) * pixelStride + 1 * compWidth], &val,
                    td.format.compByteWidth);
-          // deliberate fallthrough
+            DELIBERATE_FALLTHROUGH();
           case 1:
             memcpy(&subdata[0][(y * td.width + x) * pixelStride + 0 * compWidth], &val,
                    td.format.compByteWidth);
@@ -1203,9 +1084,9 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
                               : RenderDoc::Inst().DarkCheckerboardColor();
           }
 
-          col.x = powf(col.x, 1.0f / 2.2f);
-          col.y = powf(col.y, 1.0f / 2.2f);
-          col.z = powf(col.z, 1.0f / 2.2f);
+          col.x = ConvertLinearToSRGB(col.x);
+          col.y = ConvertLinearToSRGB(col.y);
+          col.z = ConvertLinearToSRGB(col.z);
 
           FloatVector pixel = FloatVector(float(r) / 255.0f, float(g) / 255.0f, float(b) / 255.0f,
                                           float(a) / 255.0f);
@@ -1277,10 +1158,15 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
     {
       dds_data ddsData;
 
+      ResourceFormat saveFmt = td.format;
+      // use typeCast to inform typeless saving, otherwise it will get lost
+      if(saveFmt.compType == CompType::Typeless)
+        saveFmt.compType = sd.typeCast;
+
       ddsData.width = td.width;
       ddsData.height = td.height;
       ddsData.depth = td.depth;
-      ddsData.format = td.format;
+      ddsData.format = saveFmt;
       ddsData.mips = numMips;
       ddsData.slices = numSlices / td.depth;
       ddsData.subdata = &subdata[0];
@@ -1302,10 +1188,6 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
     }
     else if(sd.destType == FileType::PNG)
     {
-      // discard alpha if requested
-      for(uint32_t p = 0; sd.alpha == AlphaMapping::Discard && p < td.width * td.height; p++)
-        subdata[0][p * 4 + 3] = 255;
-
       int ret = stbi_write_png_to_func(fileWriteFunc, (void *)f, td.width, td.height, numComps,
                                        subdata[0], rowPitch);
       success = (ret != 0);
@@ -1315,10 +1197,6 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
     }
     else if(sd.destType == FileType::TGA)
     {
-      // discard alpha if requested
-      for(uint32_t p = 0; sd.alpha == AlphaMapping::Discard && p < td.width * td.height; p++)
-        subdata[0][p * 4 + 3] = 255;
-
       int ret = stbi_write_tga_to_func(fileWriteFunc, (void *)f, td.width, td.height, numComps,
                                        subdata[0]);
       success = (ret != 0);
@@ -1370,7 +1248,7 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
 
       ResourceFormat saveFmt = td.format;
       if(saveFmt.compType == CompType::Typeless)
-        saveFmt.compType = sd.typeHint;
+        saveFmt.compType = sd.typeCast;
       if(saveFmt.compType == CompType::Typeless)
         saveFmt.compType = saveFmt.compByteWidth == 4 ? CompType::Float : CompType::UNorm;
 
@@ -1502,6 +1380,14 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
         int reqTypes[4] = {TINYEXR_PIXELTYPE_HALF, TINYEXR_PIXELTYPE_HALF, TINYEXR_PIXELTYPE_HALF,
                            TINYEXR_PIXELTYPE_HALF};
 
+        if(saveFmt.compByteWidth == 4)
+        {
+          for(size_t channel = 0; channel < 4; channel++)
+          {
+            reqTypes[channel] = TINYEXR_PIXELTYPE_FLOAT;
+          }
+        }
+
         // must be in this order as many viewers don't pay attention to channels and just assume
         // they are in this order
         EXRChannelInfo bgraChannels[4] = {
@@ -1551,30 +1437,39 @@ bool ReplayController::SaveTexture(const TextureSave &saveData, const char *path
   return success;
 }
 
-rdcarray<PixelModification> ReplayController::PixelHistory(ResourceId target, uint32_t x,
-                                                           uint32_t y, uint32_t slice, uint32_t mip,
-                                                           uint32_t sampleIdx, CompType typeHint)
+rdcarray<PixelModification> ReplayController::PixelHistory(ResourceId target, uint32_t x, uint32_t y,
+                                                           const Subresource &sub, CompType typeCast)
 {
   CHECK_REPLAY_THREAD();
 
   rdcarray<PixelModification> ret;
 
+  Subresource subresource = sub;
+
   for(size_t t = 0; t < m_Textures.size(); t++)
   {
     if(m_Textures[t].resourceId == target)
     {
       if(x >= m_Textures[t].width || y >= m_Textures[t].height)
       {
-        RDCDEBUG("PixelHistory out of bounds on %llu (%u,%u) vs (%u,%u)", target, x, y,
+        RDCDEBUG("PixelHistory out of bounds on %s (%u,%u) vs (%u,%u)", ToStr(target).c_str(), x, y,
                  m_Textures[t].width, m_Textures[t].height);
         return ret;
       }
 
       if(m_Textures[t].msSamp == 1)
-        sampleIdx = ~0U;
+        subresource.sample = ~0U;
 
-      slice = RDCCLAMP(slice, 0U, m_Textures[t].arraysize);
-      mip = RDCCLAMP(mip, 0U, m_Textures[t].mips);
+      if(m_Textures[t].dimension == 3)
+      {
+        subresource.slice = RDCCLAMP(subresource.slice, 0U, m_Textures[t].depth >> subresource.mip);
+      }
+      else
+      {
+        subresource.slice = RDCCLAMP(subresource.slice, 0U, m_Textures[t].arraysize);
+      }
+
+      subresource.mip = RDCCLAMP(subresource.mip, 0U, m_Textures[t].mips - 1);
 
       break;
     }
@@ -1585,9 +1480,9 @@ rdcarray<PixelModification> ReplayController::PixelHistory(ResourceId target, ui
   if(id == ResourceId())
     return ret;
 
-  std::vector<EventUsage> usage = m_pDevice->GetUsage(id);
+  rdcarray<EventUsage> usage = m_pDevice->GetUsage(id);
 
-  vector<EventUsage> events;
+  rdcarray<EventUsage> events;
 
   for(size_t i = 0; i < usage.size(); i++)
   {
@@ -1646,7 +1541,7 @@ rdcarray<PixelModification> ReplayController::PixelHistory(ResourceId target, ui
 
   if(events.empty())
   {
-    RDCDEBUG("Target %llu not written to before %u", target, m_EventID);
+    RDCDEBUG("Target %s not written to before %u", ToStr(target).c_str(), m_EventID);
     return ret;
   }
 
@@ -1655,13 +1550,58 @@ rdcarray<PixelModification> ReplayController::PixelHistory(ResourceId target, ui
   if(id == ResourceId())
     return ret;
 
-  ret = m_pDevice->PixelHistory(events, id, x, y, slice, mip, sampleIdx, typeHint);
+  ret = m_pDevice->PixelHistory(events, id, x, y, subresource, typeCast);
 
   SetFrameEvent(m_EventID, true);
 
   return ret;
 }
 
+PixelValue ReplayController::PickPixel(ResourceId tex, uint32_t x, uint32_t y,
+                                       const Subresource &sub, CompType typeCast)
+{
+  CHECK_REPLAY_THREAD();
+
+  PixelValue ret;
+
+  RDCEraseEl(ret.floatValue);
+
+  if(tex == ResourceId())
+    return ret;
+
+  m_pDevice->PickPixel(m_pDevice->GetLiveID(tex), x, y, sub, typeCast, ret.floatValue);
+
+  return ret;
+}
+
+rdcpair<PixelValue, PixelValue> ReplayController::GetMinMax(ResourceId textureId,
+                                                            const Subresource &sub, CompType typeCast)
+{
+  CHECK_REPLAY_THREAD();
+
+  PixelValue minval = {{0.0f, 0.0f, 0.0f, 0.0f}};
+  PixelValue maxval = {{1.0f, 1.0f, 1.0f, 1.0f}};
+
+  m_pDevice->GetMinMax(m_pDevice->GetLiveID(textureId), sub, typeCast, &minval.floatValue[0],
+                       &maxval.floatValue[0]);
+
+  return make_rdcpair(minval, maxval);
+}
+
+rdcarray<uint32_t> ReplayController::GetHistogram(ResourceId textureId, const Subresource &sub,
+                                                  CompType typeCast, float minval, float maxval,
+                                                  bool channels[4])
+{
+  CHECK_REPLAY_THREAD();
+
+  rdcarray<uint32_t> hist;
+
+  m_pDevice->GetHistogram(m_pDevice->GetLiveID(textureId), sub, typeCast, minval, maxval, channels,
+                          hist);
+
+  return hist;
+}
+
 ShaderDebugTrace *ReplayController::DebugVertex(uint32_t vertid, uint32_t instid, uint32_t idx,
                                                 uint32_t instOffset, uint32_t vertOffset)
 {
@@ -1711,7 +1651,8 @@ void ReplayController::FreeTrace(ShaderDebugTrace *trace)
 }
 
 rdcarray<ShaderVariable> ReplayController::GetCBufferVariableContents(
-    ResourceId shader, const char *entryPoint, uint32_t cbufslot, ResourceId buffer, uint64_t offs)
+    ResourceId pipeline, ResourceId shader, const char *entryPoint, uint32_t cbufslot,
+    ResourceId buffer, uint64_t offs)
 {
   CHECK_REPLAY_THREAD();
 
@@ -1725,10 +1666,11 @@ rdcarray<ShaderVariable> ReplayController::GetCBufferVariableContents(
 
   rdcarray<ShaderVariable> v;
 
+  pipeline = m_pDevice->GetLiveID(pipeline);
   shader = m_pDevice->GetLiveID(shader);
 
   if(shader != ResourceId())
-    m_pDevice->FillCBufferVariables(shader, entryPoint, cbufslot, v, data);
+    m_pDevice->FillCBufferVariables(pipeline, shader, entryPoint, cbufslot, v, data);
 
   return v;
 }
@@ -1752,7 +1694,9 @@ rdcstr ReplayController::CreateRGPProfile(WindowingData window)
     return "";
   }
 
-  std::string path = FileIO::GetTempFolderFilename() + "/renderdoc_rgp_capture.rgp";
+  rdcstr path = FileIO::GetTempFolderFilename() + "/renderdoc_rgp_capture.rgp";
+
+  FileIO::Delete(path.c_str());
 
   ReplayOutput *output = CreateOutput(window, ReplayOutputType::Texture);
 
@@ -1819,10 +1763,9 @@ void ReplayController::ReplayLoop(WindowingData window, ResourceId texid)
 
   TextureDisplay d;
   d.resourceId = texid;
-  d.mip = 0;
-  d.sampleIdx = ~0U;
+  d.subresource = {0, 0, ~0U};
   d.overlay = DebugOverlay::NoOverlay;
-  d.typeHint = CompType::Typeless;
+  d.typeCast = CompType::Typeless;
   d.hdrMultiplier = -1.0f;
   d.linearDisplayAsGamma = true;
   d.flipY = false;
@@ -1831,7 +1774,6 @@ void ReplayController::ReplayLoop(WindowingData window, ResourceId texid)
   d.scale = 1.0f;
   d.xOffset = 0.0f;
   d.yOffset = 0.0f;
-  d.sliceFace = 0;
   d.rawOutput = false;
   d.red = d.green = d.blue = true;
   d.alpha = false;
@@ -1888,17 +1830,19 @@ void ReplayController::ShutdownOutput(IReplayOutput *output)
 {
   CHECK_REPLAY_THREAD();
 
-  for(auto it = m_Outputs.begin(); it != m_Outputs.end(); ++it)
-  {
-    if((IReplayOutput *)*it == output)
+  size_t sz = m_Outputs.size();
+  m_Outputs.removeOneIf([output](const ReplayOutput *o) {
+    if((IReplayOutput *)o == output)
     {
-      delete *it;
-      m_Outputs.erase(it);
-      return;
+      delete o;
+      return true;
     }
-  }
 
-  RDCERR("Unrecognised output");
+    return false;
+  });
+
+  if(m_Outputs.size() == sz)
+    RDCERR("Unrecognised output");
 }
 
 void ReplayController::Shutdown()
@@ -1908,6 +1852,13 @@ void ReplayController::Shutdown()
   delete this;
 }
 
+rdcarray<ShaderEncoding> ReplayController::GetCustomShaderEncodings()
+{
+  CHECK_REPLAY_THREAD();
+
+  return m_pDevice->GetCustomShaderEncodings();
+}
+
 rdcarray<ShaderEncoding> ReplayController::GetTargetShaderEncodings()
 {
   CHECK_REPLAY_THREAD();
@@ -1924,12 +1875,12 @@ rdcpair<ResourceId, rdcstr> ReplayController::BuildTargetShader(
   rdcarray<ShaderEncoding> encodings = m_pDevice->GetTargetShaderEncodings();
 
   if(encodings.indexOf(sourceEncoding) == -1)
-    return make_rdcpair<ResourceId, rdcstr>(
+    return rdcpair<ResourceId, rdcstr>(
         ResourceId(),
         StringFormat::Fmt("Shader Encoding '%s' is not supported", ToStr(sourceEncoding).c_str()));
 
   ResourceId id;
-  string errs;
+  rdcstr errs;
 
   switch(type)
   {
@@ -1942,21 +1893,22 @@ rdcpair<ResourceId, rdcstr> ReplayController::BuildTargetShader(
     default: RDCERR("Unexpected type in BuildShader!"); return rdcpair<ResourceId, rdcstr>();
   }
 
-  m_pDevice->BuildTargetShader(sourceEncoding, source, entry, compileFlags, type, &id, &errs);
+  m_pDevice->BuildTargetShader(sourceEncoding, source, entry, compileFlags, type, id, errs);
 
   if(id != ResourceId())
     m_TargetResources.insert(id);
 
-  return make_rdcpair<ResourceId, rdcstr>(id, errs);
+  return rdcpair<ResourceId, rdcstr>(id, errs);
 }
 
 rdcpair<ResourceId, rdcstr> ReplayController::BuildCustomShader(
-    const char *entry, const char *source, const ShaderCompileFlags &compileFlags, ShaderStage type)
+    const char *entry, ShaderEncoding sourceEncoding, bytebuf source,
+    const ShaderCompileFlags &compileFlags, ShaderStage type)
 {
   CHECK_REPLAY_THREAD();
 
   ResourceId id;
-  string errs;
+  rdcstr errs;
 
   switch(type)
   {
@@ -1969,12 +1921,21 @@ rdcpair<ResourceId, rdcstr> ReplayController::BuildCustomShader(
     default: RDCERR("Unexpected type in BuildShader!"); return rdcpair<ResourceId, rdcstr>();
   }
 
-  m_pDevice->BuildCustomShader(source, entry, compileFlags, type, &id, &errs);
+  RDCLOG("Building custom shader");
+
+  m_pDevice->BuildCustomShader(sourceEncoding, source, entry, compileFlags, type, id, errs);
 
   if(id != ResourceId())
+  {
+    RDCLOG("Successfully built custom shader");
     m_CustomShaders.insert(id);
+  }
+  else
+  {
+    RDCLOG("Failed to build custom shader");
+  }
 
-  return make_rdcpair<ResourceId, rdcstr>(id, errs);
+  return rdcpair<ResourceId, rdcstr>(id, errs);
 }
 
 void ReplayController::FreeTargetResource(ResourceId id)
@@ -2019,12 +1980,12 @@ void ReplayController::RemoveReplacement(ResourceId id)
       m_Outputs[i]->Display();
 }
 
-ReplayStatus ReplayController::CreateDevice(RDCFile *rdc)
+ReplayStatus ReplayController::CreateDevice(RDCFile *rdc, const ReplayOptions &opts)
 {
   CHECK_REPLAY_THREAD();
 
   IReplayDriver *driver = NULL;
-  ReplayStatus status = RenderDoc::Inst().CreateReplayDriver(rdc, &driver);
+  ReplayStatus status = RenderDoc::Inst().CreateReplayDriver(rdc, opts, &driver);
 
   if(driver && status == ReplayStatus::Succeeded)
   {
@@ -2067,7 +2028,7 @@ ReplayStatus ReplayController::PostCreateInit(IReplayDriver *device, RDCFile *rd
   GCNISA::GetTargets(m_APIProps.pipelineType, m_GCNTargets);
 
   {
-    std::vector<ResourceId> ids = m_pDevice->GetBuffers();
+    rdcarray<ResourceId> ids = m_pDevice->GetBuffers();
 
     m_Buffers.resize(ids.size());
 
@@ -2076,7 +2037,7 @@ ReplayStatus ReplayController::PostCreateInit(IReplayDriver *device, RDCFile *rd
   }
 
   {
-    std::vector<ResourceId> ids = m_pDevice->GetTextures();
+    rdcarray<ResourceId> ids = m_pDevice->GetTextures();
 
     m_Textures.resize(ids.size());
 
