@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -27,7 +27,7 @@
 #include "serialise/serialiser.h"
 
 template <>
-std::string DoStringise(const RemapTexture &el)
+rdcstr DoStringise(const RemapTexture &el)
 {
   BEGIN_ENUM_STRINGISE(RemapTexture);
   {
@@ -35,7 +35,6 @@ std::string DoStringise(const RemapTexture &el)
     STRINGISE_ENUM_CLASS(RGBA8)
     STRINGISE_ENUM_CLASS(RGBA16)
     STRINGISE_ENUM_CLASS(RGBA32)
-    STRINGISE_ENUM_CLASS(D32S8)
   }
   END_ENUM_STRINGISE();
 }
@@ -44,7 +43,7 @@ template <typename SerialiserType>
 void DoSerialise(SerialiserType &ser, GetTextureDataParams &el)
 {
   SERIALISE_MEMBER(forDiskSave);
-  SERIALISE_MEMBER(typeHint);
+  SERIALISE_MEMBER(typeCast);
   SERIALISE_MEMBER(resolve);
   SERIALISE_MEMBER(remap);
   SERIALISE_MEMBER(blackPoint);
@@ -59,7 +58,7 @@ static bool PreviousNextExcludedMarker(DrawcallDescription *draw)
                              DrawFlags::APICalls));
 }
 
-static DrawcallDescription *SetupDrawcallPointers(vector<DrawcallDescription *> &drawcallTable,
+static DrawcallDescription *SetupDrawcallPointers(rdcarray<DrawcallDescription *> &drawcallTable,
                                                   rdcarray<DrawcallDescription> &draws,
                                                   DrawcallDescription *parent,
                                                   DrawcallDescription *&previous)
@@ -118,7 +117,7 @@ static DrawcallDescription *SetupDrawcallPointers(vector<DrawcallDescription *>
   return ret;
 }
 
-void SetupDrawcallPointers(std::vector<DrawcallDescription *> &drawcallTable,
+void SetupDrawcallPointers(rdcarray<DrawcallDescription *> &drawcallTable,
                            rdcarray<DrawcallDescription> &draws)
 {
   DrawcallDescription *previous = NULL;
@@ -131,7 +130,7 @@ void SetupDrawcallPointers(std::vector<DrawcallDescription *> &drawcallTable,
   // joining the 'real' linked list after one step.
 
   previous = NULL;
-  std::vector<DrawcallDescription *> markers;
+  rdcarray<DrawcallDescription *> markers;
 
   for(DrawcallDescription *draw : drawcallTable)
   {
@@ -166,7 +165,7 @@ void SetupDrawcallPointers(std::vector<DrawcallDescription *> &drawcallTable,
 }
 
 void PatchLineStripIndexBuffer(const DrawcallDescription *draw, uint8_t *idx8, uint16_t *idx16,
-                               uint32_t *idx32, std::vector<uint32_t> &patchedIndices)
+                               uint32_t *idx32, rdcarray<uint32_t> &patchedIndices)
 {
   const uint32_t restart = 0xffffffff;
 
@@ -259,7 +258,70 @@ void PatchLineStripIndexBuffer(const DrawcallDescription *draw, uint8_t *idx8, u
 #undef IDX_VALUE
 }
 
-void StandardFillCBufferVariable(uint32_t dataOffset, const bytebuf &data, ShaderVariable &outvar,
+void PatchTriangleFanRestartIndexBufer(rdcarray<uint32_t> &patchedIndices, uint32_t restartIndex)
+{
+  if(patchedIndices.empty())
+    return;
+
+  rdcarray<uint32_t> newIndices;
+
+  uint32_t firstIndex = patchedIndices[0];
+
+  size_t i = 1;
+  // while we have at least two indices left
+  while(i + 1 < patchedIndices.size())
+  {
+    uint32_t a = patchedIndices[i];
+    uint32_t b = patchedIndices[i + 1];
+
+    if(a != restartIndex && b != restartIndex)
+    {
+      // no restart, add primitive
+      newIndices.push_back(firstIndex);
+      newIndices.push_back(a);
+      newIndices.push_back(b);
+
+      i++;
+      continue;
+    }
+    else if(b == restartIndex)
+    {
+      // we've already added the last triangle before the restart in the previous iteration, just
+      // continue so we hit the a == restartIndex case below
+      i++;
+    }
+    else if(a == restartIndex)
+    {
+      // new first index is b
+      firstIndex = b;
+      // skip both the restartIndex value and the first index, and begin at the next real index (if
+      // it exists)
+      i += 2;
+
+      uint32_t next[2] = {b, b};
+
+      // if this is the last vertex, the triangle will be degenerate
+      if(i < patchedIndices.size())
+        next[0] = patchedIndices[i];
+      if(i + 1 < patchedIndices.size())
+        next[1] = patchedIndices[i + 1];
+
+      // output 3 dummy degenerate triangles so vertex ID mapping is easy
+      // we rotate the triangles so the important vertex is last in each.
+      for(size_t dummy = 0; dummy < 3; dummy++)
+      {
+        newIndices.push_back(restartIndex);
+        newIndices.push_back(restartIndex);
+        newIndices.push_back(restartIndex);
+      }
+    }
+  }
+
+  newIndices.swap(patchedIndices);
+}
+
+void StandardFillCBufferVariable(ResourceId shader, const ShaderVariableDescriptor &desc,
+                                 uint32_t dataOffset, const bytebuf &data, ShaderVariable &outvar,
                                  uint32_t matStride)
 {
   const VarType type = outvar.type;
@@ -354,18 +416,21 @@ void StandardFillCBufferVariable(uint32_t dataOffset, const bytebuf &data, Shade
       }
     }
   }
+
+  if(desc.pointerTypeID != ~0U)
+    outvar.SetTypedPointer(outvar.value.u64v[0], shader, desc.pointerTypeID);
 }
 
-static void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
+static void StandardFillCBufferVariables(ResourceId shader, const rdcarray<ShaderConstant> &invars,
                                          rdcarray<ShaderVariable> &outvars, const bytebuf &data,
                                          uint32_t baseOffset)
 {
   for(size_t v = 0; v < invars.size(); v++)
   {
-    std::string basename = invars[v].name;
+    rdcstr basename = invars[v].name;
 
-    uint32_t rows = invars[v].type.descriptor.rows;
-    uint32_t cols = invars[v].type.descriptor.columns;
+    uint8_t rows = invars[v].type.descriptor.rows;
+    uint8_t cols = invars[v].type.descriptor.columns;
     uint32_t elems = RDCMAX(1U, invars[v].type.descriptor.elements);
     const bool rowMajor = invars[v].type.descriptor.rowMajorStorage != 0;
     const bool isArray = elems > 1;
@@ -382,7 +447,7 @@ static void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
       var.type = VarType::Float;
       var.rowMajor = rowMajor;
 
-      vector<ShaderVariable> varmembers;
+      rdcarray<ShaderVariable> varmembers;
 
       if(isArray)
       {
@@ -395,7 +460,7 @@ static void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
           vr.type = VarType::Float;
           vr.rowMajor = rowMajor;
 
-          StandardFillCBufferVariables(invars[v].type.members, vr.members, data, dataOffset);
+          StandardFillCBufferVariables(shader, invars[v].type.members, vr.members, data, dataOffset);
 
           dataOffset += invars[v].type.descriptor.arrayByteStride;
 
@@ -408,7 +473,7 @@ static void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
       {
         var.isStruct = true;
 
-        StandardFillCBufferVariables(invars[v].type.members, var.members, data, dataOffset);
+        StandardFillCBufferVariables(shader, invars[v].type.members, var.members, data, dataOffset);
       }
 
       outvars.push_back(var);
@@ -417,7 +482,7 @@ static void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
     }
 
     size_t outIdx = outvars.size();
-    outvars.resize(outvars.size() + 1);
+    outvars.push_back({});
 
     {
       const VarType type = invars[v].type.descriptor.type;
@@ -435,7 +500,8 @@ static void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
       {
         outvars[outIdx].rows = rows;
 
-        StandardFillCBufferVariable(dataOffset, data, outvars[outIdx], matStride);
+        StandardFillCBufferVariable(shader, invars[v].type.descriptor, dataOffset, data,
+                                    outvars[outIdx], matStride);
       }
       else
       {
@@ -443,10 +509,10 @@ static void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
         var.rows = 0;
         var.columns = 0;
 
-        vector<ShaderVariable> varmembers;
+        rdcarray<ShaderVariable> varmembers;
         varmembers.resize(elems);
 
-        std::string base = outvars[outIdx].name;
+        rdcstr base = outvars[outIdx].name;
 
         for(uint32_t e = 0; e < elems; e++)
         {
@@ -461,7 +527,8 @@ static void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
 
           dataOffset += invars[v].type.descriptor.arrayByteStride;
 
-          StandardFillCBufferVariable(rowDataOffset, data, varmembers[e], matStride);
+          StandardFillCBufferVariable(shader, invars[v].type.descriptor, rowDataOffset, data,
+                                      varmembers[e], matStride);
         }
 
         {
@@ -473,11 +540,11 @@ static void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
   }
 }
 
-void StandardFillCBufferVariables(const rdcarray<ShaderConstant> &invars,
+void StandardFillCBufferVariables(ResourceId shader, const rdcarray<ShaderConstant> &invars,
                                   rdcarray<ShaderVariable> &outvars, const bytebuf &data)
 {
   // start with offset 0
-  StandardFillCBufferVariables(invars, outvars, data, 0);
+  StandardFillCBufferVariables(shader, invars, outvars, data, 0);
 }
 
 uint64_t CalcMeshOutputSize(uint64_t curSize, uint64_t requiredOutput)
@@ -509,7 +576,8 @@ FloatVector HighlightCache::InterpretVertex(const byte *data, uint32_t vert, con
 
     vert = indices[vert];
 
-    if(IsStrip(cfg.position.topology))
+    if(SupportsRestart(cfg.position.topology) && cfg.position.topology != Topology::TriangleFan &&
+       cfg.position.allowRestart)
     {
       if((cfg.position.indexByteStride == 1 && vert == 0xff) ||
          (cfg.position.indexByteStride == 2 && vert == 0xffff) ||
@@ -610,7 +678,7 @@ uint64_t inthash(ResourceId id, uint64_t seed)
 
 void HighlightCache::CacheHighlightingData(uint32_t eventId, const MeshDisplay &cfg)
 {
-  std::string ident;
+  rdcstr ident;
 
   uint64_t newKey = 5381;
 
@@ -625,6 +693,8 @@ void HighlightCache::CacheHighlightingData(uint32_t eventId, const MeshDisplay &
   newKey = inthash(cfg.position.vertexByteStride, newKey);
   newKey = inthash(cfg.position.indexResourceId, newKey);
   newKey = inthash(cfg.position.vertexResourceId, newKey);
+  newKey = inthash((uint64_t)cfg.position.allowRestart, newKey);
+  newKey = inthash((uint64_t)cfg.position.restartIndex, newKey);
 
   if(cacheKey != newKey)
   {
@@ -687,7 +757,7 @@ void HighlightCache::CacheHighlightingData(uint32_t eventId, const MeshDisplay &
         maxIndex += add;
 
       uint32_t primRestart = 0;
-      if(IsStrip(cfg.position.topology))
+      if(SupportsRestart(cfg.position.topology) && cfg.position.allowRestart)
       {
         if(cfg.position.indexByteStride == 1)
           primRestart = 0xff;
@@ -719,13 +789,20 @@ void HighlightCache::CacheHighlightingData(uint32_t eventId, const MeshDisplay &
 
     driver->GetBufferData(cfg.position.vertexResourceId, cfg.position.vertexByteOffset,
                           (maxIndex + 1) * cfg.position.vertexByteStride, vertexData);
+
+    // if it's a fan AND it uses primitive restart, decompose it into a triangle list because the
+    // restart changes the central vertex
+    if(cfg.position.topology == Topology::TriangleFan && cfg.position.allowRestart)
+    {
+      PatchTriangleFanRestartIndexBufer(indices, cfg.position.restartIndex);
+    }
   }
 }
 
 bool HighlightCache::FetchHighlightPositions(const MeshDisplay &cfg, FloatVector &activeVertex,
-                                             vector<FloatVector> &activePrim,
-                                             vector<FloatVector> &adjacentPrimVertices,
-                                             vector<FloatVector> &inactiveVertices)
+                                             rdcarray<FloatVector> &activePrim,
+                                             rdcarray<FloatVector> &adjacentPrimVertices,
+                                             rdcarray<FloatVector> &inactiveVertices)
 {
   bool valid = true;
 
@@ -735,10 +812,20 @@ bool HighlightCache::FetchHighlightPositions(const MeshDisplay &cfg, FloatVector
   uint32_t idx = cfg.highlightVert;
   Topology meshtopo = cfg.position.topology;
 
+  // if it's a fan AND it uses primitive restart, it was decomposed into a triangle list
+  if(meshtopo == Topology::TriangleFan && cfg.position.allowRestart)
+  {
+    meshtopo = Topology::TriangleList;
+
+    // due to triangle fan expansion the index we need to look up is adjusted
+    if(idx > 2)
+      idx = (idx - 1) * 3 - 1;
+  }
+
   activeVertex = InterpretVertex(data, idx, cfg, dataEnd, true, valid);
 
   uint32_t primRestart = 0;
-  if(IsStrip(meshtopo))
+  if(SupportsRestart(meshtopo) && cfg.position.allowRestart)
   {
     if(cfg.position.indexByteStride == 1)
       primRestart = 0xff;
@@ -834,6 +921,19 @@ bool HighlightCache::FetchHighlightPositions(const MeshDisplay &cfg, FloatVector
     activePrim.push_back(InterpretVertex(data, v + 0, cfg, dataEnd, true, valid));
     activePrim.push_back(InterpretVertex(data, v + 1, cfg, dataEnd, true, valid));
   }
+  else if(meshtopo == Topology::TriangleFan)
+  {
+    // find first vert in primitive. In fans a vert isn't
+    // in only one primitive, so we pick the first primitive
+    // it's in. This means the first N points are in the first
+    // primitive, and thereafter each point is in the next primitive
+    uint32_t v = RDCMAX(idx, 2U) - 1;
+
+    // first vert in the whole fan
+    activePrim.push_back(InterpretVertex(data, 0, cfg, dataEnd, true, valid));
+    activePrim.push_back(InterpretVertex(data, v + 0, cfg, dataEnd, true, valid));
+    activePrim.push_back(InterpretVertex(data, v + 1, cfg, dataEnd, true, valid));
+  }
   else if(meshtopo == Topology::TriangleStrip)
   {
     // find first vert in primitive. In strips a vert isn't
@@ -1029,7 +1129,7 @@ bool HighlightCache::FetchHighlightPositions(const MeshDisplay &cfg, FloatVector
   }
   else if(meshtopo >= Topology::PatchList)
   {
-    uint32_t dim = PatchList_Count(cfg.position.topology);
+    uint32_t dim = PatchList_Count(meshtopo);
 
     uint32_t v0 = uint32_t(idx / dim) * dim;
 
