@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -24,6 +24,7 @@
  ******************************************************************************/
 
 #include "common/common.h"
+#include "maths/formatpacking.h"
 #include "maths/matrix.h"
 #include "strings/string_utils.h"
 #include "replay_controller.h"
@@ -50,7 +51,16 @@ static uint64_t GetHandle(WindowingData window)
 #endif
   }
 
-  RDCERR("Unrecognised window system %d", system);
+  if(window.system == WindowingSystem::Wayland)
+  {
+#if ENABLED(RDOC_WAYLAND)
+    return (uint64_t)window.wayland.window;
+#else
+    RDCERR("Wayland windowing system data passed in, but support is not compiled in");
+#endif
+  }
+
+  RDCERR("Unrecognised window system %s", ToStr(window.system).c_str());
 
   return 0;
 
@@ -82,7 +92,7 @@ ReplayOutput::ReplayOutput(ReplayController *parent, WindowingData window, Repla
 
   m_MainOutput.dirty = true;
 
-  m_OverlayDirty = true;
+  m_OverlayDirty = false;
   m_ForceOverlayRefresh = false;
 
   m_pDevice = parent->GetDevice();
@@ -116,17 +126,6 @@ ReplayOutput::ReplayOutput(ReplayController *parent, WindowingData window, Repla
     RenderDoc::Inst().GetCrashHandler()->RegisterMemoryRegion(this, sizeof(ReplayController));
 }
 
-void ReplayOutput::SetDimensions(int32_t width, int32_t height)
-{
-  CHECK_REPLAY_THREAD();
-
-  if(m_MainOutput.outputID == 0)
-  {
-    m_Width = width;
-    m_Height = height;
-  }
-}
-
 ReplayOutput::~ReplayOutput()
 {
   CHECK_REPLAY_THREAD();
@@ -146,15 +145,43 @@ void ReplayOutput::Shutdown()
   m_pRenderer->ShutdownOutput(this);
 }
 
+void ReplayOutput::SetDimensions(int32_t width, int32_t height)
+{
+  CHECK_REPLAY_THREAD();
+
+  m_pDevice->SetOutputWindowDimensions(m_MainOutput.outputID, width > 0 ? width : 1,
+                                       height > 0 ? height : 1);
+  m_pDevice->GetOutputWindowDimensions(m_MainOutput.outputID, m_Width, m_Height);
+}
+
+bytebuf ReplayOutput::ReadbackOutputTexture()
+{
+  CHECK_REPLAY_THREAD();
+
+  bytebuf data;
+  m_pDevice->GetOutputWindowData(m_MainOutput.outputID, data);
+  return data;
+}
+
+rdcpair<int32_t, int32_t> ReplayOutput::GetDimensions()
+{
+  CHECK_REPLAY_THREAD();
+
+  return make_rdcpair(m_Width, m_Height);
+}
+
 void ReplayOutput::SetTextureDisplay(const TextureDisplay &o)
 {
   CHECK_REPLAY_THREAD();
 
-  if(o.overlay != m_RenderData.texDisplay.overlay || o.typeHint != m_RenderData.texDisplay.typeHint ||
-     o.resourceId != m_RenderData.texDisplay.resourceId)
+  bool wasClearBeforeDraw = (m_RenderData.texDisplay.overlay == DebugOverlay::ClearBeforeDraw ||
+                             m_RenderData.texDisplay.overlay == DebugOverlay::ClearBeforePass);
+
+  if(o.overlay != m_RenderData.texDisplay.overlay ||
+     (o.overlay != DebugOverlay::NoOverlay && (o.typeCast != m_RenderData.texDisplay.typeCast ||
+                                               o.resourceId != m_RenderData.texDisplay.resourceId)))
   {
-    if(m_RenderData.texDisplay.overlay == DebugOverlay::ClearBeforeDraw ||
-       m_RenderData.texDisplay.overlay == DebugOverlay::ClearBeforePass)
+    if(wasClearBeforeDraw)
     {
       // by necessity these overlays modify the actual texture, not an
       // independent overlay texture. So if we disable them, we must
@@ -163,6 +190,8 @@ void ReplayOutput::SetTextureDisplay(const TextureDisplay &o)
     }
     m_OverlayDirty = true;
   }
+  if(wasClearBeforeDraw && o.backgroundColor != m_RenderData.texDisplay.backgroundColor)
+    m_OverlayDirty = true;
   m_RenderData.texDisplay = o;
   m_MainOutput.dirty = true;
 }
@@ -183,7 +212,7 @@ void ReplayOutput::SetFrameEvent(int eventId)
 
   m_EventID = eventId;
 
-  m_OverlayDirty = true;
+  m_OverlayDirty = (m_RenderData.texDisplay.overlay != DebugOverlay::NoOverlay);
   m_MainOutput.dirty = true;
 
   for(size_t i = 0; i < m_Thumbnails.size(); i++)
@@ -238,9 +267,15 @@ void ReplayOutput::RefreshOverlay()
   {
     if(draw && m_pDevice->IsRenderOutput(m_RenderData.texDisplay.resourceId))
     {
+      FloatVector f = m_RenderData.texDisplay.backgroundColor;
+
+      f.x = ConvertLinearToSRGB(f.x);
+      f.y = ConvertLinearToSRGB(f.y);
+      f.z = ConvertLinearToSRGB(f.z);
+
       m_OverlayResourceId = m_pDevice->RenderOverlay(
-          m_pDevice->GetLiveID(m_RenderData.texDisplay.resourceId),
-          m_RenderData.texDisplay.typeHint, m_RenderData.texDisplay.overlay, m_EventID, passEvents);
+          m_pDevice->GetLiveID(m_RenderData.texDisplay.resourceId), m_RenderData.texDisplay.typeCast,
+          f, m_RenderData.texDisplay.overlay, m_EventID, passEvents);
       m_OverlayDirty = false;
     }
     else
@@ -248,6 +283,10 @@ void ReplayOutput::RefreshOverlay()
       m_OverlayResourceId = ResourceId();
     }
   }
+  else
+  {
+    m_OverlayDirty = false;
+  }
 }
 
 ResourceId ReplayOutput::GetCustomShaderTexID()
@@ -294,13 +333,14 @@ bool ReplayOutput::SetPixelContext(WindowingData window)
   return m_PixelContext.outputID != 0;
 }
 
-bool ReplayOutput::AddThumbnail(WindowingData window, ResourceId texID, CompType typeHint)
+bool ReplayOutput::AddThumbnail(WindowingData window, ResourceId texID, const Subresource &sub,
+                                CompType typeCast)
 {
   CHECK_REPLAY_THREAD();
 
   OutputPair p;
 
-  RDCASSERT(window.system != WindowingSystem::Unknown);
+  RDCASSERT(window.system != WindowingSystem::Unknown && window.system != WindowingSystem::Headless);
 
   bool depthMode = false;
 
@@ -319,11 +359,9 @@ bool ReplayOutput::AddThumbnail(WindowingData window, ResourceId texID, CompType
     if(m_Thumbnails[i].wndHandle == GetHandle(window))
     {
       m_Thumbnails[i].texture = texID;
-
       m_Thumbnails[i].depthMode = depthMode;
-
-      m_Thumbnails[i].typeHint = typeHint;
-
+      m_Thumbnails[i].sub = sub;
+      m_Thumbnails[i].typeCast = typeCast;
       m_Thumbnails[i].dirty = true;
 
       return true;
@@ -334,7 +372,8 @@ bool ReplayOutput::AddThumbnail(WindowingData window, ResourceId texID, CompType
   p.outputID = m_pDevice->MakeOutputWindow(window, false);
   p.texture = texID;
   p.depthMode = depthMode;
-  p.typeHint = typeHint;
+  p.sub = sub;
+  p.typeCast = typeCast;
   p.dirty = true;
 
   RDCASSERT(p.outputID > 0);
@@ -344,107 +383,13 @@ bool ReplayOutput::AddThumbnail(WindowingData window, ResourceId texID, CompType
   return true;
 }
 
-rdcpair<PixelValue, PixelValue> ReplayOutput::GetMinMax()
-{
-  CHECK_REPLAY_THREAD();
-
-  PixelValue minval = {{0.0f, 0.0f, 0.0f, 0.0f}};
-  PixelValue maxval = {{1.0f, 1.0f, 1.0f, 1.0f}};
-
-  ResourceId tex = m_pDevice->GetLiveID(m_RenderData.texDisplay.resourceId);
-
-  CompType typeHint = m_RenderData.texDisplay.typeHint;
-  uint32_t slice = m_RenderData.texDisplay.sliceFace;
-  uint32_t mip = m_RenderData.texDisplay.mip;
-  uint32_t sample = m_RenderData.texDisplay.sampleIdx;
-
-  if(m_RenderData.texDisplay.customShaderId != ResourceId() &&
-     m_CustomShaderResourceId != ResourceId())
-  {
-    tex = m_CustomShaderResourceId;
-    typeHint = CompType::Typeless;
-    slice = 0;
-    sample = 0;
-  }
-
-  m_pDevice->GetMinMax(tex, slice, mip, sample, typeHint, &minval.floatValue[0],
-                       &maxval.floatValue[0]);
-
-  return make_rdcpair(minval, maxval);
-}
-
-rdcarray<uint32_t> ReplayOutput::GetHistogram(float minval, float maxval, bool channels[4])
-{
-  CHECK_REPLAY_THREAD();
-
-  vector<uint32_t> hist;
-
-  ResourceId tex = m_pDevice->GetLiveID(m_RenderData.texDisplay.resourceId);
-
-  CompType typeHint = m_RenderData.texDisplay.typeHint;
-  uint32_t slice = m_RenderData.texDisplay.sliceFace;
-  uint32_t mip = m_RenderData.texDisplay.mip;
-  uint32_t sample = m_RenderData.texDisplay.sampleIdx;
-
-  if(m_RenderData.texDisplay.customShaderId != ResourceId() &&
-     m_CustomShaderResourceId != ResourceId())
-  {
-    tex = m_CustomShaderResourceId;
-    typeHint = CompType::Typeless;
-    slice = 0;
-    sample = 0;
-  }
-
-  m_pDevice->GetHistogram(tex, slice, mip, sample, typeHint, minval, maxval, channels, hist);
-
-  return hist;
-}
-
-PixelValue ReplayOutput::PickPixel(ResourceId tex, bool customShader, uint32_t x, uint32_t y,
-                                   uint32_t sliceFace, uint32_t mip, uint32_t sample)
-{
-  CHECK_REPLAY_THREAD();
-
-  PixelValue ret;
-
-  RDCEraseEl(ret.floatValue);
-
-  if(tex == ResourceId())
-    return ret;
-
-  CompType typeHint = m_RenderData.texDisplay.typeHint;
-
-  if(customShader && m_RenderData.texDisplay.customShaderId != ResourceId() &&
-     m_CustomShaderResourceId != ResourceId())
-  {
-    tex = m_CustomShaderResourceId;
-    typeHint = CompType::Typeless;
-  }
-
-  // for 'heatmap' type overlays, pick from the overlay texture
-  if((m_RenderData.texDisplay.overlay == DebugOverlay::QuadOverdrawDraw ||
-      m_RenderData.texDisplay.overlay == DebugOverlay::QuadOverdrawPass ||
-      m_RenderData.texDisplay.overlay == DebugOverlay::TriangleSizeDraw ||
-      m_RenderData.texDisplay.overlay == DebugOverlay::TriangleSizePass) &&
-     m_OverlayResourceId != ResourceId())
-  {
-    tex = m_OverlayResourceId;
-    typeHint = CompType::Typeless;
-  }
-
-  m_pDevice->PickPixel(m_pDevice->GetLiveID(tex), x, y, sliceFace, mip, sample, typeHint,
-                       ret.floatValue);
-
-  return ret;
-}
-
 rdcpair<uint32_t, uint32_t> ReplayOutput::PickVertex(uint32_t eventId, uint32_t x, uint32_t y)
 {
   CHECK_REPLAY_THREAD();
 
   DrawcallDescription *draw = m_pRenderer->GetDrawcallByEID(eventId);
 
-  const rdcpair<uint32_t, uint32_t> errorReturn = make_rdcpair(~0U, ~0U);
+  const rdcpair<uint32_t, uint32_t> errorReturn = {~0U, ~0U};
 
   if(!draw)
     return errorReturn;
@@ -578,7 +523,7 @@ void ReplayOutput::DisplayContext()
 
   const float contextZoom = 8.0f;
 
-  disp.scale = contextZoom / float(1 << disp.mip);
+  disp.scale = contextZoom / float(1 << disp.subresource.mip);
 
   int32_t width = 0, height = 0;
   m_pDevice->GetOutputWindowDimensions(m_PixelContext.outputID, width, height);
@@ -589,11 +534,11 @@ void ReplayOutput::DisplayContext()
   int x = (int)m_ContextX;
   int y = (int)m_ContextY;
 
-  x >>= disp.mip;
-  x <<= disp.mip;
+  x >>= disp.subresource.mip;
+  x <<= disp.subresource.mip;
 
-  y >>= disp.mip;
-  y <<= disp.mip;
+  y >>= disp.subresource.mip;
+  y <<= disp.subresource.mip;
 
   disp.xOffset = -(float)x * disp.scale;
   disp.yOffset = -(float)y * disp.scale;
@@ -691,21 +636,20 @@ void ReplayOutput::Display()
     disp.hdrMultiplier = -1.0f;
     disp.linearDisplayAsGamma = true;
     disp.flipY = false;
-    disp.mip = 0;
-    disp.sampleIdx = ~0U;
+    disp.subresource = m_Thumbnails[i].sub;
+    disp.subresource.sample = ~0U;
     disp.customShaderId = ResourceId();
     disp.resourceId = m_pDevice->GetLiveID(m_Thumbnails[i].texture);
-    disp.typeHint = m_Thumbnails[i].typeHint;
+    disp.typeCast = m_Thumbnails[i].typeCast;
     disp.scale = -1.0f;
     disp.rangeMin = 0.0f;
     disp.rangeMax = 1.0f;
-    disp.sliceFace = 0;
     disp.xOffset = 0.0f;
     disp.yOffset = 0.0f;
     disp.rawOutput = false;
     disp.overlay = DebugOverlay::NoOverlay;
 
-    if(m_Thumbnails[i].typeHint == CompType::SNorm)
+    if(m_Thumbnails[i].typeCast == CompType::SNorm)
       disp.rangeMin = -1.0f;
 
     if(m_Thumbnails[i].depthMode)
@@ -756,14 +700,14 @@ void ReplayOutput::DisplayTex()
 
   if(m_RenderData.texDisplay.customShaderId != ResourceId())
   {
-    m_CustomShaderResourceId = m_pDevice->ApplyCustomShader(
-        m_RenderData.texDisplay.customShaderId, texDisplay.resourceId, texDisplay.mip,
-        texDisplay.sliceFace, texDisplay.sampleIdx, texDisplay.typeHint);
+    m_CustomShaderResourceId =
+        m_pDevice->ApplyCustomShader(m_RenderData.texDisplay.customShaderId, texDisplay.resourceId,
+                                     texDisplay.subresource, texDisplay.typeCast);
 
     texDisplay.resourceId = m_pDevice->GetLiveID(m_CustomShaderResourceId);
-    texDisplay.typeHint = CompType::Typeless;
+    texDisplay.typeCast = CompType::Typeless;
     texDisplay.customShaderId = ResourceId();
-    texDisplay.sliceFace = 0;
+    texDisplay.subresource.slice = 0;
   }
 
   FloatVector color;
@@ -799,7 +743,7 @@ void ReplayOutput::DisplayTex()
     texDisplay.rangeMin = 0.0f;
     texDisplay.rangeMax = 1.0f;
     texDisplay.linearDisplayAsGamma = false;
-    texDisplay.typeHint = CompType::Typeless;
+    texDisplay.typeCast = CompType::Typeless;
 
     m_pDevice->RenderTexture(texDisplay);
   }
@@ -844,7 +788,7 @@ void ReplayOutput::DisplayMesh()
   mesh.second.vertexResourceId = m_pDevice->GetLiveID(mesh.second.vertexResourceId);
   mesh.second.indexResourceId = m_pDevice->GetLiveID(mesh.second.indexResourceId);
 
-  vector<MeshFormat> secondaryDraws;
+  rdcarray<MeshFormat> secondaryDraws;
 
   // we choose a pallette here so that the colours stay consistent (i.e the
   // current draw is always the same colour), but also to indicate somewhat
