@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,8 +23,11 @@
  ******************************************************************************/
 
 #include <utility>
+#include "api/replay/structured_data.h"
 #include "common/common.h"
+#include "common/formatting.h"
 #include "serialise/rdcfile.h"
+#include "strings/string_utils.h"
 
 #include "3rdparty/miniz/miniz.h"
 #include "3rdparty/pugixml/pugixml.hpp"
@@ -41,7 +44,7 @@ static const char *typeNames[] = {
 };
 
 template <typename inttype>
-std::string GetBufferName(inttype i)
+rdcstr GetBufferName(inttype i)
 {
   return StringFormat::Fmt("%06u", (uint32_t)i);
 }
@@ -86,7 +89,7 @@ static constexpr byte FromHex(const char c)
                                      : (c >= 'a' && c <= 'f' ? byte(c - 'a') + 10 : 0));
 }
 
-static void HexEncode(const std::vector<byte> &in, std::string &out)
+static void HexEncode(const bytebuf &in, rdcstr &out)
 {
   const size_t bytesPerLine = 32;
   const size_t bytesPerGroup = 4;
@@ -103,7 +106,7 @@ static void HexEncode(const std::vector<byte> &in, std::string &out)
   out = "\n";
 
   // accumulate ascii representation for each line
-  std::string ascii;
+  rdcstr ascii;
 
   size_t i = 0;
   for(byte c : in)
@@ -149,7 +152,7 @@ static void HexEncode(const std::vector<byte> &in, std::string &out)
   }
 }
 
-static void HexDecode(const char *str, const char *end, std::vector<byte> &out)
+static void HexDecode(const char *str, const char *end, bytebuf &out)
 {
   out.reserve((end - str) / 2);
 
@@ -160,7 +163,7 @@ static void HexDecode(const char *str, const char *end, std::vector<byte> &out)
   {
     if(IsHex(str[0]) && IsHex(str[1]))
     {
-      out.push_back((FromHex(str[0]) << 4) | FromHex(str[1]));
+      out.push_back(byte((FromHex(str[0]) << 4) | FromHex(str[1])));
 
       str += 2;
 
@@ -370,7 +373,7 @@ static ReplayStatus Structured2XML(const char *filename, const RDCFile &file, ui
     pugi::xml_node type = xSection.append_child("type");
     type.text() = (uint32_t)props.type;
 
-    std::vector<byte> contents;
+    bytebuf contents;
     contents.resize((size_t)reader->GetSize());
     reader->Read(contents.data(), reader->GetSize());
 
@@ -384,7 +387,7 @@ static ReplayStatus Structured2XML(const char *filename, const RDCFile &file, ui
     else
     {
       // encode to simple hex. Not efficient, but easy.
-      std::string hexdata;
+      rdcstr hexdata;
       hexdata.reserve(contents.size() * 2);
       HexEncode(contents, hexdata);
       data.text().set(hexdata.c_str());
@@ -454,7 +457,7 @@ static SDObject *XML2Obj(pugi::xml_node &obj)
   SDObject *ret =
       new SDObject(obj.attribute("name").as_string(), obj.attribute("typename").as_string());
 
-  std::string name = obj.name();
+  rdcstr name = obj.name();
 
   for(size_t i = 0; i < ARRAY_COUNT(typeNames); i++)
   {
@@ -578,7 +581,7 @@ static ReplayStatus XML2Structured(const char *xml, const ThumbTypeAndData &thum
     }
 
     RDCDriver driver = (RDCDriver)xDriver.attribute("id").as_uint();
-    std::string driverName = xDriver.text().as_string();
+    rdcstr driverName = xDriver.text().as_string();
 
     pugi::xml_node xIdent = xDriver.next_sibling();
 
@@ -609,36 +612,37 @@ static ReplayStatus XML2Structured(const char *xml, const ThumbTypeAndData &thum
     rdc->SetData(driver, driverName.c_str(), machineIdent, rdcthumb);
   }
 
-  // push in other sections
-  pugi::xml_node xSection = xHeader.next_sibling();
-
-  if(!strcmp(xSection.name(), "extended_thumbnail"))
-  {
-    SectionProperties props = {};
-    props.type = SectionType::ExtendedThumbnail;
-    props.version = 1;
-    StreamWriter *w = rdc->WriteSection(props);
-
-    ExtThumbnailHeader header;
-    header.width = (uint16_t)xSection.attribute("width").as_uint();
-    header.height = (uint16_t)xSection.attribute("height").as_uint();
-    header.len = (uint32_t)extThumb.data.size();
-    header.format = extThumb.format;
-    w->Write(header);
-    w->Write(extThumb.data.data(), extThumb.data.size());
-
-    w->Finish();
-
-    delete w;
-
-    xSection = xSection.next_sibling();
-  }
-
   if(progress)
     progress(StructuredProgress(0.1f));
 
-  while(!strcmp(xSection.name(), "section"))
+  // push in other sections
+  pugi::xml_node xSection = xHeader.next_sibling();
+
+  while(!strcmp(xSection.name(), "section") || !strcmp(xSection.name(), "extended_thumbnail"))
   {
+    if(!strcmp(xSection.name(), "extended_thumbnail"))
+    {
+      SectionProperties props = {};
+      props.type = SectionType::ExtendedThumbnail;
+      props.version = 1;
+      StreamWriter *w = rdc->WriteSection(props);
+
+      ExtThumbnailHeader header;
+      header.width = (uint16_t)xSection.attribute("width").as_uint();
+      header.height = (uint16_t)xSection.attribute("height").as_uint();
+      header.len = (uint32_t)extThumb.data.size();
+      header.format = extThumb.format;
+      w->Write(header);
+      w->Write(extThumb.data.data(), extThumb.data.size());
+
+      w->Finish();
+
+      delete w;
+
+      xSection = xSection.next_sibling();
+      continue;
+    }
+
     SectionProperties props;
 
     if(xSection.attribute("ascii"))
@@ -694,7 +698,7 @@ static ReplayStatus XML2Structured(const char *xml, const ThumbTypeAndData &thum
     }
     else
     {
-      std::vector<byte> decoded;
+      bytebuf decoded;
       HexDecode(str, str + len, decoded);
       writer->Write(decoded.data(), decoded.size());
     }
@@ -762,7 +766,7 @@ static ReplayStatus XML2Structured(const char *xml, const ThumbTypeAndData &thum
 
       chunk->metadata.flags |= SDChunkFlags::OpaqueChunk;
 
-      chunk->data.children.push_back(new SDObject("Opaque chunk", "Byte Buffer"));
+      chunk->data.children.push_back(new SDObject("Opaque chunk"_lit, "Byte Buffer"_lit));
       chunk->data.children[0]->type.basetype = SDBasic::Buffer;
       chunk->data.children[0]->type.byteSize = opaque.attribute("byteLength").as_ullong();
       chunk->data.children[0]->data.basic.u = opaque.text().as_ullong();
@@ -784,12 +788,11 @@ static ReplayStatus XML2Structured(const char *xml, const ThumbTypeAndData &thum
   return ReplayStatus::Succeeded;
 }
 
-static ReplayStatus Buffers2ZIP(const std::string &filename, const RDCFile &file,
+static ReplayStatus Buffers2ZIP(const rdcstr &filename, const RDCFile &file,
                                 const StructuredBufferList &buffers,
                                 RENDERDOC_ProgressCallback progress)
 {
-  std::string zipFile = filename;
-  zipFile.erase(zipFile.size() - 4);    // remove the .xml, leave only the .zip
+  rdcstr zipFile = strip_extension(filename);
 
   mz_zip_archive zip;
   memset(&zip, 0, sizeof(zip));
@@ -866,12 +869,10 @@ static ReplayStatus Buffers2ZIP(const std::string &filename, const RDCFile &file
   return ReplayStatus::Succeeded;
 }
 
-static bool ZIP2Buffers(const std::string &filename, ThumbTypeAndData &thumb,
-                        ThumbTypeAndData &extThumb, StructuredBufferList &buffers,
-                        RENDERDOC_ProgressCallback progress)
+static bool ZIP2Buffers(const rdcstr &filename, ThumbTypeAndData &thumb, ThumbTypeAndData &extThumb,
+                        StructuredBufferList &buffers, RENDERDOC_ProgressCallback progress)
 {
-  std::string zipFile = filename;
-  zipFile.erase(zipFile.size() - 4);    // remove the .xml, leave only the .zip
+  rdcstr zipFile = strip_extension(filename);
 
   if(!FileIO::exists(zipFile.c_str()))
   {
@@ -954,12 +955,11 @@ ReplayStatus importXMLZ(const char *filename, StreamReader &reader, RDCFile *rdc
     }
   }
 
-  uint64_t len = reader.GetSize();
-  char *buf = new char[(size_t)len + 1];
-  reader.Read(buf, (size_t)len);
-  buf[len] = 0;
+  rdcstr buf;
+  buf.resize((size_t)reader.GetSize());
+  reader.Read(buf.data(), buf.size());
 
-  return XML2Structured(buf, thumb, extThumb, structData.buffers, rdc, structData.version,
+  return XML2Structured(buf.c_str(), thumb, extThumb, structData.buffers, rdc, structData.version,
                         structData.chunks, progress);
 }
 
@@ -996,4 +996,4 @@ static ConversionRegistration XMLOnlyConversionRegistration(
         R"(Stores the structured data in an xml tree, with large buffer data omitted - that makes it
 easier to work with but it cannot then be imported.)",
         false,
-    });
\ No newline at end of file
+    });
