@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,11 +29,58 @@
 #include "core/core.h"
 #include "strings/string_utils.h"
 
-#if !defined(RELEASE)
+#if ENABLED(RDOC_DEVEL)
 
 int64_t Chunk::m_LiveChunks = 0;
 int64_t Chunk::m_TotalMem = 0;
 
+void DumpObject(FileIO::LogFileHandle *log, const rdcstr &indent, SDObject *obj)
+{
+  if(obj->NumChildren() > 0)
+  {
+    rdcstr msg =
+        StringFormat::Fmt("%s%s%s %s:\n", indent.c_str(), obj->type.name.c_str(),
+                          obj->type.basetype == SDBasic::Array ? "[]" : "", obj->name.c_str());
+    FileIO::logfile_append(log, msg.c_str(), msg.size());
+    for(size_t i = 0; i < obj->NumChildren(); i++)
+      DumpObject(log, indent + "  ", obj->GetChild(i));
+  }
+  else
+  {
+    rdcstr val;
+    switch(obj->type.basetype)
+    {
+      case SDBasic::Chunk: val = "{Chunk}"; break;
+      case SDBasic::Struct: val = "{Struct}"; break;
+      case SDBasic::Array:
+        // this must be an empty array, or it would have children above
+        val = "{}";
+        break;
+      case SDBasic::Buffer: val = "[buffer]"; break;
+      case SDBasic::Null: val = "NULL"; break;
+      case SDBasic::String: val = obj->data.str; break;
+      case SDBasic::Enum: val = obj->data.str; break;
+      case SDBasic::UnsignedInteger: val = ToStr(obj->data.basic.u); break;
+      case SDBasic::SignedInteger: val = ToStr(obj->data.basic.i); break;
+      case SDBasic::Float: val = ToStr(obj->data.basic.d); break;
+      case SDBasic::Boolean: val = ToStr(obj->data.basic.b); break;
+      case SDBasic::Character: val = ToStr(obj->data.basic.c); break;
+      case SDBasic::Resource: val = ToStr(obj->data.basic.id); break;
+    }
+    rdcstr msg = StringFormat::Fmt("%s%s %s = %s\n", indent.c_str(), obj->type.name.c_str(),
+                                   obj->name.c_str(), val.c_str());
+    FileIO::logfile_append(log, msg.c_str(), msg.size());
+  }
+}
+
+void DumpChunk(bool reading, FileIO::LogFileHandle *log, SDChunk *chunk)
+{
+  rdcstr msg = StringFormat::Fmt("%s %s @ %llu:\n", reading ? "Read" : "Wrote", chunk->name.c_str(),
+                                 chunk->metadata.timestampMicro);
+  FileIO::logfile_append(log, msg.c_str(), msg.size());
+  DumpObject(log, "  ", chunk);
+}
+
 #endif
 
 /////////////////////////////////////////////////////////////
@@ -60,7 +107,7 @@ Serialiser<SerialiserMode::Reading>::~Serialiser()
 }
 
 template <>
-uint32_t Serialiser<SerialiserMode::Reading>::BeginChunk(uint32_t, uint32_t)
+uint32_t Serialiser<SerialiserMode::Reading>::BeginChunk(uint32_t, uint64_t)
 {
   uint32_t chunkID = 0;
 
@@ -85,10 +132,20 @@ uint32_t Serialiser<SerialiserMode::Reading>::BeginChunk(uint32_t, uint32_t)
       uint32_t numFrames = 0;
       m_Read->Read(numFrames);
 
-      m_ChunkMetadata.flags |= SDChunkFlags::HasCallstack;
+      // try to sanity check the number of frames
+      if(numFrames < 4096)
+      {
+        m_ChunkMetadata.flags |= SDChunkFlags::HasCallstack;
 
-      m_ChunkMetadata.callstack.resize((size_t)numFrames);
-      m_Read->Read(m_ChunkMetadata.callstack.data(), m_ChunkMetadata.callstack.byteSize());
+        m_ChunkMetadata.callstack.resize((size_t)numFrames);
+        m_Read->Read(m_ChunkMetadata.callstack.data(), m_ChunkMetadata.callstack.byteSize());
+      }
+      else
+      {
+        RDCERR("Read invalid number of callstack frames: %u", numFrames);
+        // still read the size that we should, even though we expect this to be broken after here
+        m_Read->Read(NULL, numFrames * sizeof(uint64_t));
+      }
     }
 
     if(c & ChunkThreadID)
@@ -100,14 +157,23 @@ uint32_t Serialiser<SerialiserMode::Reading>::BeginChunk(uint32_t, uint32_t)
     if(c & ChunkTimestamp)
       m_Read->Read(m_ChunkMetadata.timestampMicro);
 
-    m_Read->Read(m_ChunkMetadata.length);
+    if(c & Chunk64BitSize)
+    {
+      m_Read->Read(m_ChunkMetadata.length);
+    }
+    else
+    {
+      uint32_t chunkSize = 0;
+      m_Read->Read(chunkSize);
+      m_ChunkMetadata.length = chunkSize;
+    }
 
     m_LastChunkOffset = m_Read->GetOffset();
   }
 
   if(ExportStructure())
   {
-    std::string name = m_ChunkLookup ? m_ChunkLookup(chunkID) : "";
+    rdcstr name = m_ChunkLookup ? m_ChunkLookup(chunkID) : "";
 
     if(name.empty())
       name = "<Unknown Chunk>";
@@ -135,7 +201,7 @@ void Serialiser<SerialiserMode::Reading>::SkipCurrentChunk()
     SDObject &current = *m_StructureStack.back();
 
     current.data.basic.numChildren++;
-    current.data.children.push_back(new SDObject("Opaque chunk", "Byte Buffer"));
+    current.data.children.push_back(new SDObject("Opaque chunk"_lit, "Byte Buffer"_lit));
 
     SDObject &obj = *current.data.children.back();
     obj.type.basetype = SDBasic::Buffer;
@@ -198,6 +264,13 @@ void Serialiser<SerialiserMode::Reading>::EndChunk()
       m_StructureStack.back()->type.byteSize = m_ChunkMetadata.length;
       m_StructureStack.pop_back();
     }
+
+#if ENABLED(RDOC_DEVEL)
+    if(m_DebugDumpLog && !m_StructuredFile->chunks.empty())
+    {
+      DumpChunk(true, m_DebugDumpLog, m_StructuredFile->chunks.back());
+    }
+#endif
   }
 
   // only skip remaining bytes if we have a valid length - if we have a length of 0 we wrote this
@@ -260,7 +333,7 @@ void Serialiser<SerialiserMode::Writing>::SetChunkMetadataRecording(uint32_t fla
 }
 
 template <>
-uint32_t Serialiser<SerialiserMode::Writing>::BeginChunk(uint32_t chunkID, uint32_t byteLength)
+uint32_t Serialiser<SerialiserMode::Writing>::BeginChunk(uint32_t chunkID, uint64_t byteLength)
 {
   {
     // chunk index needs to be valid
@@ -271,6 +344,8 @@ uint32_t Serialiser<SerialiserMode::Writing>::BeginChunk(uint32_t chunkID, uint3
       RDCASSERT(chunkID <= ChunkIndexMask);
 
       c |= m_ChunkFlags;
+      if(byteLength > 0xffffffff)
+        c |= Chunk64BitSize;
 
       m_ChunkMetadata.chunkID = chunkID;
 
@@ -334,13 +409,23 @@ uint32_t Serialiser<SerialiserMode::Writing>::BeginChunk(uint32_t chunkID, uint3
       {
         // write length, assuming it is an upper bound
         m_ChunkFixup = 0;
-        m_Write->Write(byteLength);
+        RDCASSERT(byteLength < 0x100000000 || (c & Chunk64BitSize) != 0);
+        if(c & Chunk64BitSize)
+        {
+          m_Write->Write(byteLength);
+        }
+        else
+        {
+          m_Write->Write(uint32_t(byteLength & 0xffffffff));
+        }
         m_LastChunkOffset = m_Write->GetOffset();
         m_ChunkMetadata.length = byteLength;
       }
       else
       {
         // length will be fixed up in EndChunk
+        // assume that this case will not produce chunks with size larger than can fit in 32 bit
+        // value
         uint32_t chunkSize = 0xbeebfeed;
         m_ChunkFixup = m_Write->GetOffset();
         m_Write->Write(chunkSize);
@@ -348,6 +433,22 @@ uint32_t Serialiser<SerialiserMode::Writing>::BeginChunk(uint32_t chunkID, uint3
     }
   }
 
+  if(ExportStructure())
+  {
+    rdcstr name = m_ChunkLookup ? m_ChunkLookup(chunkID) : "";
+
+    if(name.empty())
+      name = "<Unknown Chunk>";
+
+    SDChunk *chunk = new SDChunk(name.c_str());
+    chunk->metadata = m_ChunkMetadata;
+
+    m_StructuredFile->chunks.push_back(chunk);
+    m_StructureStack.push_back(chunk);
+
+    m_InternalElement = false;
+  }
+
   return chunkID;
 }
 
@@ -371,12 +472,15 @@ void Serialiser<SerialiserMode::Writing>::EndChunk()
     RDCASSERT(curOffset > chunkOffset);
 
     uint64_t chunkLength = (curOffset - chunkOffset) - sizeof(uint32_t);
+    if(chunkLength > 0xffffffff)
+    {
+      RDCERR("!!! CHUNK LENGTH %llu EXCEEDED 32 BIT VALUE. CAPTURE WILL BE CORRUPTED. !!!",
+             chunkLength);
+    }
 
-    RDCASSERT(chunkLength < 0xffffffff);
+    m_Write->WriteAt(chunkOffset, uint32_t(chunkLength & 0xffffffff));
 
-    uint32_t chunklen = (uint32_t)chunkLength;
-
-    m_Write->WriteAt(chunkOffset, chunklen);
+    m_ChunkMetadata.length = chunkLength;
   }
   else
   {
@@ -393,24 +497,43 @@ void Serialiser<SerialiserMode::Writing>::EndChunk()
         m_Write->Write(padByte);
       }
 
-      RDCDEBUG("Chunk estimated at %u bytes, actual length %llu. Added %llu bytes padding.",
+      RDCDEBUG("Chunk estimated at %llu bytes, actual length %llu. Added %llu bytes padding.",
                m_ChunkMetadata.length, writtenLength, numPadBytes);
     }
     else if(writtenLength > m_ChunkMetadata.length)
     {
       RDCERR(
           "!!! "
-          "ESTIMATED UPPER BOUND CHUNK LENGTH %u EXCEEDED: %llu. "
+          "ESTIMATED UPPER BOUND CHUNK LENGTH %llu EXCEEDED: %llu. "
           "CAPTURE WILL BE CORRUPTED. "
           "!!!",
           m_ChunkMetadata.length, writtenLength);
     }
     else
     {
-      RDCDEBUG("Chunk was exactly the estimate of %u bytes.", m_ChunkMetadata.length);
+      RDCDEBUG("Chunk was exactly the estimate of %llu bytes.", m_ChunkMetadata.length);
     }
   }
 
+  if(ExportStructure())
+  {
+    RDCASSERTMSG("Object Stack is imbalanced!", m_StructureStack.size() <= 1,
+                 m_StructureStack.size());
+
+    if(!m_StructureStack.empty())
+    {
+      m_StructureStack.back()->type.byteSize = m_ChunkMetadata.length;
+      m_StructureStack.pop_back();
+    }
+
+#if ENABLED(RDOC_DEVEL)
+    if(m_DebugDumpLog && !m_StructuredFile->chunks.empty())
+    {
+      DumpChunk(false, m_DebugDumpLog, m_StructuredFile->chunks.back());
+    }
+#endif
+  }
+
   // align to the natural chunk alignment
   m_Write->AlignTo<ChunkAlignment>();
 
@@ -475,7 +598,7 @@ void Serialiser<SerialiserMode::Writing>::WriteStructuredFile(const SDFile &file
       for(size_t o = 0; o < chunk.data.children.size(); o++)
       {
         // note, we don't need names because we aren't exporting structured data
-        ser->Serialise("", chunk.data.children[o]);
+        ser->Serialise(""_lit, chunk.data.children[o]);
       }
     }
 
@@ -499,7 +622,7 @@ void Serialiser<SerialiserMode::Writing>::WriteStructuredFile(const SDFile &file
 }
 
 template <>
-std::string DoStringise(const SDBasic &el)
+rdcstr DoStringise(const SDBasic &el)
 {
   BEGIN_ENUM_STRINGISE(SDBasic);
   {
@@ -520,7 +643,7 @@ std::string DoStringise(const SDBasic &el)
 }
 
 template <>
-std::string DoStringise(const SDTypeFlags &el)
+rdcstr DoStringise(const SDTypeFlags &el)
 {
   BEGIN_BITFIELD_STRINGISE(SDTypeFlags);
   {
@@ -530,18 +653,21 @@ std::string DoStringise(const SDTypeFlags &el)
     STRINGISE_BITFIELD_CLASS_BIT(Hidden);
     STRINGISE_BITFIELD_CLASS_BIT(Nullable);
     STRINGISE_BITFIELD_CLASS_BIT(NullString);
+    STRINGISE_BITFIELD_CLASS_BIT(FixedArray);
+    STRINGISE_BITFIELD_CLASS_BIT(Union);
   }
   END_BITFIELD_STRINGISE();
 }
 
 template <>
-std::string DoStringise(const SDChunkFlags &el)
+rdcstr DoStringise(const SDChunkFlags &el)
 {
   BEGIN_BITFIELD_STRINGISE(SDChunkFlags);
   {
     STRINGISE_BITFIELD_CLASS_VALUE(NoFlags);
 
     STRINGISE_BITFIELD_CLASS_BIT(OpaqueChunk);
+    STRINGISE_BITFIELD_CLASS_BIT(HasCallstack);
   }
   END_BITFIELD_STRINGISE();
 }
@@ -580,7 +706,7 @@ void DoSerialise(SerialiserType &ser, StructuredObjectList &el)
   // transfer purposes, we don't make a proper array here and instead just manually serialise count
   // + elements
   uint64_t count = el.size();
-  ser.Serialise("count", count);
+  ser.Serialise("count"_lit, count);
 
   if(ser.IsReading())
     el.resize((size_t)count);
@@ -591,7 +717,7 @@ void DoSerialise(SerialiserType &ser, StructuredObjectList &el)
     if(ser.IsReading())
       el[c] = new SDObject("", "");
 
-    ser.Serialise("$el", *el[c]);
+    ser.Serialise("$el"_lit, *el[c]);
   }
 }
 
@@ -634,7 +760,7 @@ void DoSerialise(SerialiserType &ser, SDObject *el)
   if(el->type.flags & SDTypeFlags::Nullable)
   {
     bool present = el->type.basetype != SDBasic::Null;
-    ser.Serialise("", present);
+    ser.Serialise(""_lit, present);
   }
 
   const SDFile &file = ser.GetStructuredFile();
@@ -644,9 +770,9 @@ void DoSerialise(SerialiserType &ser, SDObject *el)
     case SDBasic::Chunk: RDCERR("Unexpected chunk inside object!"); break;
     case SDBasic::Struct:
       for(size_t o = 0; o < el->data.children.size(); o++)
-        ser.Serialise("", el->data.children[o]);
+        ser.Serialise(""_lit, el->data.children[o]);
       break;
-    case SDBasic::Array: ser.Serialise("", (rdcarray<SDObject *> &)el->data.children); break;
+    case SDBasic::Array: ser.Serialise(""_lit, (rdcarray<SDObject *> &)el->data.children); break;
     case SDBasic::Null:
       // nothing to do, we serialised present flag above
       RDCASSERT(el->type.flags & SDTypeFlags::Nullable);
@@ -656,7 +782,7 @@ void DoSerialise(SerialiserType &ser, SDObject *el)
       size_t bufID = (size_t)el->data.basic.u;
       byte *buf = file.buffers[bufID]->data();
       uint64_t size = file.buffers[bufID]->size();
-      ser.Serialise("", buf, size);
+      ser.Serialise(""_lit, buf, size);
       break;
     }
     case SDBasic::String:
@@ -664,42 +790,42 @@ void DoSerialise(SerialiserType &ser, SDObject *el)
       if(el->type.flags & SDTypeFlags::NullString)
       {
         const char *nullstring = NULL;
-        ser.Serialise("", nullstring);
+        ser.Serialise(""_lit, nullstring);
       }
       else
       {
-        ser.Serialise("", el->data.str);
+        ser.Serialise(""_lit, el->data.str);
       }
       break;
     }
     case SDBasic::Enum:
     {
       uint32_t e = (uint32_t)el->data.basic.u;
-      ser.Serialise("", e);
+      ser.Serialise(""_lit, e);
       break;
     }
-    case SDBasic::Boolean: ser.Serialise("", el->data.basic.b); break;
-    case SDBasic::Character: ser.Serialise("", el->data.basic.c); break;
-    case SDBasic::Resource: ser.Serialise("", el->data.basic.id); break;
+    case SDBasic::Boolean: ser.Serialise(""_lit, el->data.basic.b); break;
+    case SDBasic::Character: ser.Serialise(""_lit, el->data.basic.c); break;
+    case SDBasic::Resource: ser.Serialise(""_lit, el->data.basic.id); break;
     case SDBasic::UnsignedInteger:
       if(el->type.byteSize == 1)
       {
         uint8_t u = uint8_t(el->data.basic.u);
-        ser.Serialise("", u);
+        ser.Serialise(""_lit, u);
       }
       else if(el->type.byteSize == 2)
       {
         uint16_t u = uint16_t(el->data.basic.u);
-        ser.Serialise("", u);
+        ser.Serialise(""_lit, u);
       }
       else if(el->type.byteSize == 4)
       {
         uint32_t u = uint32_t(el->data.basic.u);
-        ser.Serialise("", u);
+        ser.Serialise(""_lit, u);
       }
       else if(el->type.byteSize == 8)
       {
-        ser.Serialise("", el->data.basic.u);
+        ser.Serialise(""_lit, el->data.basic.u);
       }
       else
       {
@@ -710,21 +836,21 @@ void DoSerialise(SerialiserType &ser, SDObject *el)
       if(el->type.byteSize == 1)
       {
         int8_t i = int8_t(el->data.basic.i);
-        ser.Serialise("", i);
+        ser.Serialise(""_lit, i);
       }
       else if(el->type.byteSize == 2)
       {
         int16_t i = int16_t(el->data.basic.i);
-        ser.Serialise("", i);
+        ser.Serialise(""_lit, i);
       }
       else if(el->type.byteSize == 4)
       {
         int32_t i = int32_t(el->data.basic.i);
-        ser.Serialise("", i);
+        ser.Serialise(""_lit, i);
       }
       else if(el->type.byteSize == 8)
       {
-        ser.Serialise("", el->data.basic.i);
+        ser.Serialise(""_lit, el->data.basic.i);
       }
       else
       {
@@ -735,11 +861,11 @@ void DoSerialise(SerialiserType &ser, SDObject *el)
       if(el->type.byteSize == 4)
       {
         float f = float(el->data.basic.d);
-        ser.Serialise("", f);
+        ser.Serialise(""_lit, f);
       }
       else if(el->type.byteSize == 8)
       {
-        ser.Serialise("", el->data.basic.d);
+        ser.Serialise(""_lit, el->data.basic.d);
       }
       else
       {
@@ -753,99 +879,93 @@ void DoSerialise(SerialiserType &ser, SDObject *el)
 // Basic types
 
 template <>
-std::string DoStringise(const std::string &el)
+rdcstr DoStringise(const rdcstr &el)
 {
   return el;
 }
 
 template <>
-std::string DoStringise(const rdcstr &el)
-{
-  return el;
-}
-
-template <>
-std::string DoStringise(void *const &el)
+rdcstr DoStringise(void *const &el)
 {
   return StringFormat::Fmt("%#p", el);
 }
 
 template <>
-std::string DoStringise(const int64_t &el)
+rdcstr DoStringise(const int64_t &el)
 {
   return StringFormat::Fmt("%lld", el);
 }
 
 #if ENABLED(RDOC_SIZET_SEP_TYPE)
 template <>
-std::string DoStringise(const size_t &el)
+rdcstr DoStringise(const size_t &el)
 {
   return StringFormat::Fmt("%llu", (uint64_t)el);
 }
 #endif
 
 template <>
-std::string DoStringise(const uint64_t &el)
+rdcstr DoStringise(const uint64_t &el)
 {
   return StringFormat::Fmt("%llu", el);
 }
 
 template <>
-std::string DoStringise(const uint32_t &el)
+rdcstr DoStringise(const uint32_t &el)
 {
   return StringFormat::Fmt("%u", el);
 }
 
 template <>
-std::string DoStringise(const char &el)
+rdcstr DoStringise(const char &el)
 {
   return StringFormat::Fmt("'%c'", el);
 }
 
 template <>
-std::string DoStringise(const wchar_t &el)
+rdcstr DoStringise(const wchar_t &el)
 {
   return StringFormat::Fmt("'%lc'", el);
 }
 
 template <>
-std::string DoStringise(const byte &el)
+rdcstr DoStringise(const byte &el)
 {
   return StringFormat::Fmt("%hhu", el);
 }
 
 template <>
-std::string DoStringise(const uint16_t &el)
+rdcstr DoStringise(const uint16_t &el)
 {
   return StringFormat::Fmt("%hu", el);
 }
 
 template <>
-std::string DoStringise(const int32_t &el)
+rdcstr DoStringise(const int32_t &el)
 {
   return StringFormat::Fmt("%d", el);
 }
 
 template <>
-std::string DoStringise(const int16_t &el)
+rdcstr DoStringise(const int16_t &el)
 {
   return StringFormat::Fmt("%hd", el);
 }
 
 template <>
-std::string DoStringise(const float &el)
+rdcstr DoStringise(const float &el)
 {
   return StringFormat::Fmt("%0.4f", el);
 }
 
 template <>
-std::string DoStringise(const double &el)
+rdcstr DoStringise(const double &el)
 {
   return StringFormat::Fmt("%0.4lf", el);
 }
 
 template <>
-std::string DoStringise(const bool &el)
+rdcstr DoStringise(const bool &el)
 {
   if(el)
     return "True";
