@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -46,7 +46,7 @@ void WriteAllBasicTypes(WriteSerialiser &ser)
   double k = 11.11011011;
   float l = 12.12012012f;
 
-  std::string m = "mmmm";
+  rdcstr m = "mmmm";
   char n[5] = "nnnn";
   const char *s = "ssss";
 
@@ -113,7 +113,7 @@ TEST_CASE("Read/write basic types", "[serialiser][structured]")
     double k;
     float l;
 
-    std::string m;
+    rdcstr m;
     char n[5];
     const char *s;
 
@@ -159,8 +159,8 @@ TEST_CASE("Read/write basic types", "[serialiser][structured]")
     CHECK(l == 12.12012012f);
 
     CHECK(m == "mmmm");
-    CHECK(std::string(n) == "nnnn");
-    CHECK(std::string(s) == "ssss");
+    CHECK(rdcstr(n) == "nnnn");
+    CHECK(rdcstr(s) == "ssss");
 
     CHECK(t[0] == 20);
     CHECK(t[1] == 20);
@@ -187,7 +187,7 @@ TEST_CASE("Read/write via structured of basic types", "[serialiser]")
   {
     ReadSerialiser ser(new StreamReader(buf->GetData(), buf->GetOffset()), Ownership::Stream);
 
-    ChunkLookup testChunkLoop = [](uint32_t) -> std::string { return "TestChunk"; };
+    ChunkLookup testChunkLoop = [](uint32_t) -> rdcstr { return "TestChunk"; };
 
     ser.ConfigureStructuredExport(testChunkLoop, true);
 
@@ -207,7 +207,7 @@ TEST_CASE("Read/write via structured of basic types", "[serialiser]")
     double k;
     float l;
 
-    std::string m;
+    rdcstr m;
     char n[5];
     const char *s;
 
@@ -499,7 +499,7 @@ TEST_CASE("Read/write chunk metadata", "[serialiser]")
     ser.WriteChunk(1);
 
     uint32_t dummy = 99;
-    ser.Serialise("dummy", dummy);
+    ser.Serialise("dummy"_lit, dummy);
 
     ser.EndChunk();
 
@@ -532,14 +532,14 @@ TEST_CASE("Read/write chunk metadata", "[serialiser]")
   {
     ReadSerialiser ser(new StreamReader(buf->GetData(), buf->GetOffset()), Ownership::Stream);
 
-    ChunkLookup testChunkLoop = [](uint32_t) -> std::string { return "TestChunk"; };
+    ChunkLookup testChunkLoop = [](uint32_t) -> rdcstr { return "TestChunk"; };
 
     ser.ConfigureStructuredExport(testChunkLoop, true);
 
     ser.ReadChunk<uint32_t>();
 
     uint32_t dummy;
-    ser.Serialise("dummy", dummy);
+    ser.Serialise("dummy"_lit, dummy);
 
     ser.EndChunk();
 
@@ -575,7 +575,7 @@ TEST_CASE("Verify multiple chunks can be merged", "[serialiser][chunks]")
   };
 
   // write some chunks individually
-  std::vector<Chunk *> chunks;
+  rdcarray<Chunk *> chunks;
   {
     WriteSerialiser ser(new StreamWriter(StreamWriter::DefaultScratchSize), Ownership::Stream);
 
@@ -595,7 +595,7 @@ TEST_CASE("Verify multiple chunks can be merged", "[serialiser][chunks]")
     {
       SCOPED_SERIALISE_CHUNK(STRING_AND_INT);
 
-      string s = "string in STRING_AND_INT";
+      rdcstr s = "string in STRING_AND_INT";
       int i = 4096;
 
       SERIALISE_ELEMENT(s);
@@ -695,7 +695,7 @@ TEST_CASE("Verify multiple chunks can be merged", "[serialiser][chunks]")
         }
         case STRING_AND_INT:
         {
-          string s;
+          rdcstr s;
           int i = 0;
 
           SERIALISE_ELEMENT(s);
@@ -732,9 +732,8 @@ TEST_CASE("Read/write container types", "[serialiser][structured]")
     {
       SCOPED_SERIALISE_CHUNK(5);
 
-      std::vector<int> v;
-      std::pair<float, string> p;
-      std::list<uint16_t> l;
+      rdcarray<int> v;
+      rdcpair<float, rdcstr> p;
 
       v.push_back(1);
       v.push_back(1);
@@ -743,18 +742,10 @@ TEST_CASE("Read/write container types", "[serialiser][structured]")
       v.push_back(5);
       v.push_back(8);
 
-      p = std::make_pair(3.14159f, "M_PI");
-
-      l.push_back(2U);
-      l.push_back(3U);
-      l.push_back(5U);
-      l.push_back(7U);
-      l.push_back(11U);
-      l.push_back(13U);
+      p = {3.14159f, "M_PI"};
 
       SERIALISE_ELEMENT(v);
       SERIALISE_ELEMENT(p);
-      SERIALISE_ELEMENT(l);
     }
 
     CHECK(buf->GetOffset() <= 128);
@@ -769,13 +760,11 @@ TEST_CASE("Read/write container types", "[serialiser][structured]")
 
     CHECK(chunkID == 5);
 
-    std::vector<int> v;
-    std::pair<float, string> p;
-    std::list<uint16_t> l;
+    rdcarray<int> v;
+    rdcpair<float, rdcstr> p;
 
     SERIALISE_ELEMENT(v);
     SERIALISE_ELEMENT(p);
-    SERIALISE_ELEMENT(l);
 
     ser.EndChunk();
 
@@ -784,7 +773,6 @@ TEST_CASE("Read/write container types", "[serialiser][structured]")
     CHECK(ser.GetReader()->AtEnd());
 
     REQUIRE(v.size() == 6);
-    REQUIRE(l.size() == 6);
 
     CHECK(v[0] == 1);
     CHECK(v[1] == 1);
@@ -795,36 +783,20 @@ TEST_CASE("Read/write container types", "[serialiser][structured]")
 
     CHECK(p.first == 3.14159f);
     CHECK(p.second == "M_PI");
-
-    auto it = l.begin();
-
-    CHECK(*it == 2U);
-    ++it;
-    CHECK(*it == 3U);
-    ++it;
-    CHECK(*it == 5U);
-    ++it;
-    CHECK(*it == 7U);
-    ++it;
-    CHECK(*it == 11U);
-    ++it;
-    CHECK(*it == 13U);
   }
 
   {
     ReadSerialiser ser(new StreamReader(buf->GetData(), buf->GetOffset()), Ownership::Stream);
 
-    ser.ConfigureStructuredExport([](uint32_t) -> std::string { return "TestChunk"; }, true);
+    ser.ConfigureStructuredExport([](uint32_t) -> rdcstr { return "TestChunk"; }, true);
 
     ser.ReadChunk<uint32_t>();
     {
-      std::vector<int32_t> v;
-      std::pair<float, string> p;
-      std::list<uint16_t> l;
+      rdcarray<int32_t> v;
+      rdcpair<float, rdcstr> p;
 
       SERIALISE_ELEMENT(v);
       SERIALISE_ELEMENT(p);
-      SERIALISE_ELEMENT(l);
     }
     ser.EndChunk();
 
@@ -841,7 +813,7 @@ TEST_CASE("Read/write container types", "[serialiser][structured]")
 
     const SDChunk &chunk = *structData.chunks[0];
 
-    CHECK(chunk.data.children.size() == 3);
+    CHECK(chunk.data.children.size() == 2);
 
     for(SDObject *o : chunk.data.children)
       REQUIRE(o);
@@ -906,29 +878,6 @@ TEST_CASE("Read/write container types", "[serialiser][structured]")
       }
     }
 
-    {
-      SDObject &o = *chunk.data.children[childIdx++];
-
-      CHECK(o.name == "l");
-      CHECK(o.type.basetype == SDBasic::Array);
-      CHECK(o.type.byteSize == 6);
-      CHECK(o.type.flags == SDTypeFlags::NoFlags);
-      CHECK(o.data.children.size() == 6);
-
-      for(SDObject *child : o.data.children)
-      {
-        CHECK(child->type.basetype == SDBasic::UnsignedInteger);
-        CHECK(child->type.byteSize == 2);
-      }
-
-      CHECK(o.data.children[0]->data.basic.u == 2U);
-      CHECK(o.data.children[1]->data.basic.u == 3U);
-      CHECK(o.data.children[2]->data.basic.u == 5U);
-      CHECK(o.data.children[3]->data.basic.u == 7U);
-      CHECK(o.data.children[4]->data.basic.u == 11U);
-      CHECK(o.data.children[5]->data.basic.u == 13U);
-    }
-
     StreamWriter *rewriteBuf = new StreamWriter(StreamWriter::DefaultScratchSize);
 
     {
@@ -967,9 +916,9 @@ void DoSerialise(SerialiserType &ser, struct1 &el)
 
 struct struct2
 {
-  string name;
-  std::vector<float> floats;
-  std::vector<struct1> viewports;
+  rdcstr name;
+  rdcarray<float> floats;
+  rdcarray<struct1> viewports;
 };
 
 DECLARE_REFLECTION_STRUCT(struct2);
@@ -993,7 +942,7 @@ enum MySpecialEnum
 DECLARE_REFLECTION_ENUM(MySpecialEnum);
 
 template <>
-std::string DoStringise(const MySpecialEnum &el)
+rdcstr DoStringise(const MySpecialEnum &el)
 {
   BEGIN_ENUM_STRINGISE(MySpecialEnum);
   {
@@ -1018,7 +967,7 @@ TEST_CASE("Read/write complex types", "[serialiser][structured]")
 
     SERIALISE_ELEMENT(enumVal);
 
-    std::vector<struct1> sparseStructArray;
+    rdcarray<struct1> sparseStructArray;
 
     sparseStructArray.resize(10);
 
@@ -1059,7 +1008,7 @@ TEST_CASE("Read/write complex types", "[serialiser][structured]")
 
     SERIALISE_ELEMENT(enumVal);
 
-    std::vector<struct1> sparseStructArray;
+    rdcarray<struct1> sparseStructArray;
 
     SERIALISE_ELEMENT(sparseStructArray);
 
@@ -1119,7 +1068,7 @@ TEST_CASE("Read/write complex types", "[serialiser][structured]")
   {
     ReadSerialiser ser(new StreamReader(buf->GetData(), buf->GetOffset()), Ownership::Stream);
 
-    ser.ConfigureStructuredExport([](uint32_t) -> std::string { return "TestChunk"; }, true);
+    ser.ConfigureStructuredExport([](uint32_t) -> rdcstr { return "TestChunk"; }, true);
 
     ser.ReadChunk<uint32_t>();
     {
@@ -1127,7 +1076,7 @@ TEST_CASE("Read/write complex types", "[serialiser][structured]")
 
       SERIALISE_ELEMENT(enumVal);
 
-      std::vector<struct1> sparseStructArray;
+      rdcarray<struct1> sparseStructArray;
 
       SERIALISE_ELEMENT(sparseStructArray);
 
@@ -1327,7 +1276,7 @@ enum class TestEnumClass
 DECLARE_REFLECTION_ENUM(TestEnumClass);
 
 template <>
-std::string DoStringise(const TestEnumClass &el)
+rdcstr DoStringise(const TestEnumClass &el)
 {
   BEGIN_ENUM_STRINGISE(TestEnumClass)
   {
@@ -1349,7 +1298,7 @@ enum TestEnum
 DECLARE_REFLECTION_ENUM(TestEnum);
 
 template <>
-std::string DoStringise(const TestEnum &el)
+rdcstr DoStringise(const TestEnum &el)
 {
   BEGIN_ENUM_STRINGISE(TestEnum)
   {
@@ -1376,7 +1325,7 @@ DECLARE_REFLECTION_ENUM(TestBitfieldClass);
 BITMASK_OPERATORS(TestBitfieldClass);
 
 template <>
-std::string DoStringise(const TestBitfieldClass &el)
+rdcstr DoStringise(const TestBitfieldClass &el)
 {
   BEGIN_BITFIELD_STRINGISE(TestBitfieldClass)
   {
@@ -1404,7 +1353,7 @@ enum TestBitfield
 DECLARE_REFLECTION_ENUM(TestBitfield);
 
 template <>
-std::string DoStringise(const TestBitfield &el)
+rdcstr DoStringise(const TestBitfield &el)
 {
   BEGIN_BITFIELD_STRINGISE(TestBitfield)
   {
@@ -1419,6 +1368,11 @@ std::string DoStringise(const TestBitfield &el)
   END_BITFIELD_STRINGISE();
 }
 
+void test(const char *aasd)
+{
+  RDCLOG("got a test of %s", aasd);
+}
+
 TEST_CASE("Test stringification works as expected", "[tostr]")
 {
   SECTION("Enum classes")
@@ -1442,6 +1396,17 @@ TEST_CASE("Test stringification works as expected", "[tostr]")
     CHECK(ToStr(foo) == "TestEnumClass(0)");
   };
 
+  SECTION("integers")
+  {
+    uint16_t a = 54;
+    uint32_t b = 22;
+    uint8_t c = 99;
+
+    test(ToStr(a).c_str());
+    test(ToStr(b).c_str());
+    test(ToStr(c).c_str());
+  };
+
   SECTION("plain enums")
   {
     TestEnum foo = TestA;
