@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,8 +26,8 @@
 
 #include <stdio.h>
 #include <functional>
-#include <vector>
 #include "common/common.h"
+#include "os/os_specific.h"
 
 enum class Ownership
 {
@@ -81,7 +81,7 @@ public:
   StreamReader(StreamInvalidType);
   StreamReader(StreamDummyType);
   StreamReader(const byte *buffer, uint64_t bufferSize);
-  StreamReader(const std::vector<byte> &buffer);
+  StreamReader(const bytebuf &buffer);
 
   StreamReader(Network::Socket *sock, Ownership own);
   StreamReader(FILE *file, uint64_t fileSize, Ownership own);
@@ -118,6 +118,12 @@ public:
     return true;
   }
 
+  void Clear(void *data, size_t numBytes)
+  {
+    if(!m_Dummy)
+      memset(data, 0, numBytes);
+  }
+
   bool Read(void *data, uint64_t numBytes)
   {
     if(numBytes == 0 || m_Dummy)
@@ -245,7 +251,7 @@ private:
   Ownership m_Ownership;
 
   // callbacks that will be invoked when this stream is being destroyed
-  std::vector<StreamCloseCallback> m_Callbacks;
+  rdcarray<StreamCloseCallback> m_Callbacks;
 };
 
 class StreamWriter
@@ -496,7 +502,7 @@ private:
   Ownership m_Ownership;
 
   // callbacks that will be invoked when this stream is being destroyed
-  std::vector<StreamCloseCallback> m_Callbacks;
+  rdcarray<StreamCloseCallback> m_Callbacks;
 };
 
-void StreamTransfer(StreamWriter *writer, StreamReader *reader, RENDERDOC_ProgressCallback progress);
\ No newline at end of file
+void StreamTransfer(StreamWriter *writer, StreamReader *reader, RENDERDOC_ProgressCallback progress);
