@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2017-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -170,9 +170,8 @@ TEST_CASE("Test stream I/O operations over the network", "[streamio][network]")
 
     // we have to do the send/receive on threads since it is blocking
 
-    std::vector<uint64_t> receivedValues;
-    std::vector<uint64_t> list = {1,  1,  2,   3,   5,   8,   13,  21,  34,
-                                  55, 89, 144, 233, 377, 610, 987, 1597};
+    rdcarray<uint64_t> receivedValues;
+    rdcarray<uint64_t> list = {1, 1, 2, 3, 5, 8, 13, 21, 34, 55, 89, 144, 233, 377, 610, 987, 1597};
 
     // Tracks the lifetime of each thread.
     volatile int32_t threadA = 0, threadB = 0;
@@ -249,15 +248,15 @@ TEST_CASE("Test stream I/O operations over the network", "[streamio][network]")
     });
 
     recvThread = Threading::CreateThread([&threadB, &reader, &receivedValues]() {
-      uint64_t vals[10];
+      uint64_t tmp[10];
 
-      reader.Read(vals);
+      reader.Read(tmp);
 
       // keep reading indefinitely until we hit an error (i.e. socket disconnected)
       while(!reader.IsErrored())
       {
-        receivedValues.insert(receivedValues.end(), vals, vals + ARRAY_COUNT(vals));
-        reader.Read(vals);
+        receivedValues.append(tmp, ARRAY_COUNT(tmp));
+        reader.Read(tmp);
       }
 
       Atomic::Inc32(&threadB);
@@ -308,4 +307,4 @@ TEST_CASE("Test stream I/O operations over the network", "[streamio][network]")
   delete server;
 };
 
-#endif    // ENABLED(ENABLE_UNIT_TESTS)
\ No newline at end of file
+#endif    // ENABLED(ENABLE_UNIT_TESTS)
