@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2014-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,10 +23,62 @@
  ******************************************************************************/
 
 #include "common/common.h"
+#include "os/os_specific.h"
 
 // grisu2 double-to-string function, returns number of digits written to digits array
 int grisu2(uint64_t mantissa, int exponent, char digits[18], int &kout);
 
+static int wchar2multibyte(wchar_t chr, char mbchr[4])
+{
+  // U+00000 -> U+00007F 1 byte  0xxxxxxx
+  // U+00080 -> U+0007FF 2 bytes 110xxxxx 10xxxxxx
+  // U+00800 -> U+00FFFF 3 bytes 1110xxxx 10xxxxxx 10xxxxxx
+  // U+10000 -> U+1FFFFF 4 bytes 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
+
+  // upcast to uint32_t, so we do the same processing on windows where
+  // sizeof(wchar_t) == 2
+  uint32_t wc = (uint32_t)chr;
+
+  if(wc > 0x10FFFF)
+    wc = 0xFFFD;    // replacement character
+
+  if(wc <= 0x7f)
+  {
+    mbchr[0] = (char)wc;
+    return 1;
+  }
+  else if(wc <= 0x7ff)
+  {
+    mbchr[1] = 0x80 | (char)(wc & 0x3f);
+    wc >>= 6;
+    mbchr[0] = 0xC0 | (char)(wc & 0x1f);
+    return 2;
+  }
+  else if(wc <= 0xffff)
+  {
+    mbchr[2] = 0x80 | (char)(wc & 0x3f);
+    wc >>= 6;
+    mbchr[1] = 0x80 | (char)(wc & 0x3f);
+    wc >>= 6;
+    mbchr[0] = 0xE0 | (char)(wc & 0x0f);
+    wc >>= 4;
+    return 3;
+  }
+  else
+  {
+    // invalid codepoints above 0x10FFFF were replaced above
+    mbchr[3] = 0x80 | (char)(wc & 0x3f);
+    wc >>= 6;
+    mbchr[2] = 0x80 | (char)(wc & 0x3f);
+    wc >>= 6;
+    mbchr[1] = 0x80 | (char)(wc & 0x3f);
+    wc >>= 6;
+    mbchr[0] = 0xF0 | (char)(wc & 0x07);
+    wc >>= 3;
+    return 4;
+  }
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // functions for appending to output (handling running out of buffer space)
 
@@ -922,7 +974,7 @@ void formatargument(char type, void *rawarg, FormatterParams formatter, char *&o
       // convert single wide character to UTF-8 sequence, at most
       // 4 characters
       char mbchr[4];
-      int seqlen = StringFormat::Wide2UTF8(chr, mbchr);
+      int seqlen = wchar2multibyte(chr, mbchr);
       appendstring(output, actualsize, end, mbchr, seqlen);
     }
     else
@@ -954,7 +1006,7 @@ void formatargument(char type, void *rawarg, FormatterParams formatter, char *&o
         len = RDCMIN(len, precision);
 
       // convert the substring to UTF-8
-      string str = StringFormat::Wide2UTF8(std::wstring(ws, ws + len));
+      rdcstr str = StringFormat::Wide2UTF8(rdcwstr(ws, len));
 
       // add left padding, if necessary
       if(formatter.Width != FormatterParams::NoWidth && len < width &&
@@ -1128,7 +1180,7 @@ void formatargument(char type, void *rawarg, FormatterParams formatter, char *&o
   }
 }
 
-int utf8printf(char *buf, size_t bufsize, const char *fmt, va_list args)
+int utf8printv(char *buf, size_t bufsize, const char *fmt, va_list args)
 {
   // format, buffer and string arguments are assumed to be UTF-8 (except wide strings).
   // note that since the format specifiers are entirely ascii, we can byte-copy safely and handle
@@ -1400,22 +1452,23 @@ int utf8printf(char *buf, size_t bufsize, const char *fmt, va_list args)
   return int(actualsize);
 }
 
-#if ENABLED(ENABLE_UNIT_TESTS)
-
-#include "3rdparty/catch/catch.hpp"
-
-int utf8printf_wrapper(char *buf, size_t bufsize, const char *fmt, ...)
+int utf8printf(char *str, size_t bufSize, const char *fmt, ...)
 {
   va_list args;
   va_start(args, fmt);
 
-  int ret = utf8printf(buf, bufsize, fmt, args);
+  int ret = utf8printv(str, bufSize, fmt, args);
 
   va_end(args);
 
   return ret;
 }
 
+#if ENABLED(ENABLE_UNIT_TESTS)
+
+#include "3rdparty/catch/catch.hpp"
+#include "common/formatting.h"
+
 TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
 {
   int fourtytwo = 42;
@@ -1424,7 +1477,7 @@ TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
 
   SECTION("NULL input buffer")
   {
-    int a = utf8printf_wrapper(NULL, 0, "%d %c", fourtytwo, x);
+    int a = utf8printf(NULL, 0, "%d %c", fourtytwo, x);
     int b = snprintf(NULL, 0, "%d %c", fourtytwo, x);
 
     CHECK(a == 4);
@@ -1437,7 +1490,7 @@ TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
     char bufa[] = {0, 0, 0, 0, 0, 0};
     char bufb[] = {0, 0, 0, 0, 0, 0};
 
-    int a = utf8printf_wrapper(bufa, sizeof(bufa), "%d foo", largenum);
+    int a = utf8printf(bufa, sizeof(bufa), "%d foo", largenum);
     int b = snprintf(bufb, sizeof(bufb), "%d foo", largenum);
 
     RDCCOMPILE_ASSERT(sizeof(bufa) == 6, "bufa is mis-sized for test");
@@ -1449,19 +1502,19 @@ TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
     RDCCOMPILE_ASSERT(sizeof(bufa) == sizeof(ref), "ref is mis-sized for test");
 
     {
-      INFO("bufa is '" << std::string(bufa) << "', ref is '" << ref << "'");
+      INFO("bufa is '" << rdcstr(bufa) << "', ref is '" << ref << "'");
       CHECK(memcmp(bufa, ref, sizeof(ref)) == 0);
     }
     {
-      INFO("bufb is '" << std::string(bufb) << "', ref is '" << ref << "'");
+      INFO("bufb is '" << rdcstr(bufb) << "', ref is '" << ref << "'");
       CHECK(memcmp(bufb, ref, sizeof(ref)) == 0);
     }
 
     SECTION("contains NULL terminator")
     {
       memset(bufa, 'a', sizeof(bufa));
-      a = utf8printf_wrapper(bufa, sizeof(bufa), "%d foo", largenum);
-      INFO("bufa is '" << std::string(bufa) << "'");
+      a = utf8printf(bufa, sizeof(bufa), "%d foo", largenum);
+      INFO("bufa is '" << rdcstr(bufa) << "'");
       CHECK(memcmp(bufa, ref, sizeof(ref)) == 0);
     }
   };
@@ -1474,7 +1527,7 @@ TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
     memset(bufa, 'a', sizeof(bufa));
     memset(bufb, 'b', sizeof(bufb));
 
-    int a = utf8printf_wrapper(bufa, sizeof(bufa), "foobar %c %d", x, fourtytwo);
+    int a = utf8printf(bufa, sizeof(bufa), "foobar %c %d", x, fourtytwo);
     int b = snprintf(bufb, sizeof(bufb), "foobar %c %d", x, fourtytwo);
 
     CHECK(a == sizeof(bufa) - 1);
@@ -1486,11 +1539,11 @@ TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
     RDCCOMPILE_ASSERT(sizeof(ref) <= sizeof(bufb), "ref is mis-sized for test");
 
     {
-      INFO("bufa is '" << std::string(bufa) << "', ref is '" << ref << "'");
+      INFO("bufa is '" << rdcstr(bufa) << "', ref is '" << ref << "'");
       CHECK(memcmp(bufa, ref, sizeof(ref)) == 0);
     }
     {
-      INFO("bufb is '" << std::string(bufb) << "', ref is '" << ref << "'");
+      INFO("bufb is '" << rdcstr(bufb) << "', ref is '" << ref << "'");
       CHECK(memcmp(bufb, ref, sizeof(ref)) == 0);
     }
   };
@@ -1503,7 +1556,7 @@ TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
     memset(bufa, 'a', sizeof(bufa));
     memset(bufb, 'b', sizeof(bufb));
 
-    int a = utf8printf_wrapper(bufa, sizeof(bufa), "foobar %c %d", x, fourtytwo);
+    int a = utf8printf(bufa, sizeof(bufa), "foobar %c %d", x, fourtytwo);
     int b = snprintf(bufb, sizeof(bufb), "foobar %c %d", x, fourtytwo);
 
     CHECK(a == sizeof(bufa));
@@ -1515,11 +1568,11 @@ TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
     RDCCOMPILE_ASSERT(sizeof(ref) <= sizeof(bufb), "ref is mis-sized for test");
 
     {
-      INFO("bufa is '" << std::string(bufa) << "', ref is '" << ref << "'");
+      INFO("bufa is '" << rdcstr(bufa) << "', ref is '" << ref << "'");
       CHECK(memcmp(bufa, ref, sizeof(ref)) == 0);
     }
     {
-      INFO("bufb is '" << std::string(bufb) << "', ref is '" << ref << "'");
+      INFO("bufb is '" << rdcstr(bufb) << "', ref is '" << ref << "'");
       CHECK(memcmp(bufb, ref, sizeof(ref)) == 0);
     }
   };
@@ -1532,7 +1585,7 @@ TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
     memset(bufa, 'a', sizeof(bufa));
     memset(bufb, 'b', sizeof(bufb));
 
-    int a = utf8printf_wrapper(bufa, sizeof(bufa), "foobar %c %d", x, fourtytwo);
+    int a = utf8printf(bufa, sizeof(bufa), "foobar %c %d", x, fourtytwo);
     int b = snprintf(bufb, sizeof(bufb), "foobar %c %d", x, fourtytwo);
 
     CHECK(a == 11);
@@ -1545,15 +1598,15 @@ TEST_CASE("utf8printf buffer sizing", "[utf8printf]")
     refb[19] = 'b';
 
     {
-      INFO("bufa is '" << std::string(bufa) << "', refa is '" << refa << "'");
-      INFO("bufa+12 is '" << std::string(bufa + 12, bufa + 20) << "', refa+12 is '"
-                          << std::string(refa + 12, refa + 20) << "'");
+      INFO("bufa is '" << rdcstr(bufa) << "', refa is '" << refa << "'");
+      INFO("bufa+12 is '" << rdcstr(bufa + 12, 8) << "', refa+12 is '" << rdcstr(refa + 12, 8)
+                          << "'");
       CHECK(memcmp(bufa, refa, sizeof(refa)) == 0);
     }
     {
-      INFO("bufb is '" << std::string(bufb) << "', refb is '" << refb << "'");
-      INFO("bufb+12 is '" << std::string(bufb + 12, bufb + 20) << "', refb+12 is '"
-                          << std::string(refb + 12, refb + 20) << "'");
+      INFO("bufb is '" << rdcstr(bufb) << "', refb is '" << refb << "'");
+      INFO("bufb+12 is '" << rdcstr(bufb + 12, 8) << "', refb+12 is '" << rdcstr(refb + 12, 8)
+                          << "'");
       CHECK(memcmp(bufb, refb, sizeof(refb)) == 0);
     }
   };
@@ -1570,8 +1623,8 @@ TEST_CASE("utf8printf standard string formatters", "[utf8printf]")
     CHECK(StringFormat::Fmt("%lc", wc) == "á");
   };
 
-  const std::string s = "ελληνικά";
-  const std::wstring ws = L"\x3b5\x3bb\x3bb\x3b7\x3bd\x3b9\x3ba\x3ac";
+  const rdcstr s = "ελληνικά";
+  const rdcwstr ws = L"\x3b5\x3bb\x3bb\x3b7\x3bd\x3b9\x3ba\x3ac";
 
   SECTION("Basic strings")
   {
