@@ -1,4 +1,5 @@
 package @RENDERDOC_ANDROID_PACKAGE_NAME@;
+import android.os.Build;
 import android.app.Activity;
 import android.view.WindowManager;
 
@@ -13,8 +14,15 @@ public class Loader extends android.app.NativeActivity
     protected void onCreate(android.os.Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
         getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
+
+        // if we're running on something older than Android M (6.0), return now
+        // before requesting permissions as it's not supported
+        if(Build.VERSION.SDK_INT < Build.VERSION_CODES.M)
+            return;
+
         // Popup a dialog if we haven't granted Android storage permissions.
         requestPermissions(new String[]{android.Manifest.permission.WRITE_EXTERNAL_STORAGE}, 1);
+
         // Request is asynchronous, so prevent connection to server until permissions granted.
         while(checkSelfPermission(android.Manifest.permission.WRITE_EXTERNAL_STORAGE)
             != android.content.pm.PackageManager.PERMISSION_GRANTED)
