@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  * Copyright (c) 2014 Crytek
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -42,9 +42,6 @@
 
 #include <replay/renderdoc_replay.h>
 
-using std::string;
-using std::vector;
-
 void Daemonise()
 {
   // don't change dir, but close stdin/stdou
@@ -76,7 +73,7 @@ struct VulkanRegisterCommand : public Command
       std::cout << "Not fixing vulkan layer issues, and suppressing future warnings." << std::endl;
       std::cout << "To undo, remove '$HOME/.renderdoc/ignore_vulkan_layer_issues'." << std::endl;
 
-      string ignorePath = string(getenv("HOME")) + "/.renderdoc/";
+      std::string ignorePath = std::string(getenv("HOME")) + "/.renderdoc/";
 
       mkdir(ignorePath.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
 
@@ -495,7 +492,7 @@ int main(int argc, char *argv[])
 
   // add compiled-in support to version line
   {
-    string support = "APIs supported at compile-time: ";
+    std::string support = "APIs supported at compile-time: ";
     int count = 0;
 
 #if defined(RENDERDOC_SUPPORT_VULKAN)
@@ -540,6 +537,11 @@ int main(int argc, char *argv[])
     count++;
 #endif
 
+#if defined(RENDERDOC_WINDOWING_WAYLAND)
+    support += "Wayland (CAPTURE ONLY), ";
+    count++;
+#endif
+
 #if defined(RENDERDOC_SUPPORT_VULKAN)
     support += "Vulkan KHR_display, ";
     count++;
