@@ -6,6 +6,10 @@ if [ "$PLATFORM" == "Linux" ]; then
 
 	./scripts/compile_linux.sh
 
+elif [ "$PLATFORM" == "macOS" ]; then
+
+	./scripts/compile_macos.sh
+
 else
 
 	./scripts/compile_win32.sh
