@@ -35,7 +35,7 @@ strip --strip-unneeded dist/lib/*
 
 # Copy python modules to where they'd be built natively, for documentation build
 mkdir build
-cp -R /tmp/rdoc_docker/pymodules build/bin
+cp -R /tmp/rdoc_docker/pymodules build/lib
 
 # Step into the docs folder and build
 pushd docs
@@ -64,31 +64,53 @@ if [ ! -d $LLVM_ARM32 ] || [ ! -d $LLVM_ARM64 ] ; then
 	exit 0;
 fi
 
-# Build the arm32 variant
-mkdir build-android-arm32
-pushd build-android-arm32
+AAPT=$(ls $ANDROID_SDK/build-tools/*/aapt 2>/dev/null | tail -n 1)
 
-cmake -DBUILD_ANDROID=1 -DANDROID_ABI=armeabi-v7a -DANDROID_NATIVE_API_LEVEL=23 -DCMAKE_BUILD_TYPE=Release -DSTRIP_ANDROID_LIBRARY=On -DLLVM_DIR=$LLVM_ARM32/lib/cmake/llvm -DUSE_INTERCEPTOR_LIB=On -DCMAKE_MAKE_PROGRAM=make ..
-make -j8
+# Check to see if we already have this built, and don't rebuild
+VERSION32=$($AAPT dump badging build-android-arm32/bin/*apk 2>/dev/null | grep -Eo "versionName='[0-9a-f]*'" | grep -Eo "'.*'" | tr -d "'")
+VERSION64=$($AAPT dump badging build-android-arm64/bin/*apk 2>/dev/null | grep -Eo "versionName='[0-9a-f]*'" | grep -Eo "'.*'" | tr -d "'")
+
+if [ "$VERSION32" == "$GITTAG" ]; then
+
+	echo "Found existing compatible arm32 build at $GITTAG, not rebuilding";
+
+else
+
+	# Build the arm32 variant
+	mkdir build-android-arm32
+	pushd build-android-arm32
+
+	cmake -DBUILD_ANDROID=1 -DANDROID_ABI=armeabi-v7a -DANDROID_NATIVE_API_LEVEL=23 -DCMAKE_BUILD_TYPE=Release -DSTRIP_ANDROID_LIBRARY=On -DLLVM_DIR=$LLVM_ARM32/lib/cmake/llvm -DUSE_INTERCEPTOR_LIB=On -DCMAKE_MAKE_PROGRAM=make ..
+	make -j8
+
+	if ! ls bin/*.apk; then
+		echo "Android build failed"
+		exit 0;
+	fi
+
+	popd # build-android-arm32
 
-if ! ls bin/*.apk; then
-	echo "Android build failed"
-	exit 0;
 fi
 
-popd # build-android-arm32
+if [ "$VERSION64" == "$GITTAG" ]; then
 
-mkdir build-android-arm64
-pushd build-android-arm64
+	echo "Found existing compatible arm64 build at $GITTAG, not rebuilding";
 
-cmake -DBUILD_ANDROID=1 -DANDROID_ABI=arm64-v8a -DANDROID_NATIVE_API_LEVEL=23 -DCMAKE_BUILD_TYPE=Release -DSTRIP_ANDROID_LIBRARY=On -DLLVM_DIR=$LLVM_ARM64/lib/cmake/llvm -DUSE_INTERCEPTOR_LIB=On -DCMAKE_MAKE_PROGRAM=make ..
-make -j8
+else
+
+	mkdir build-android-arm64
+	pushd build-android-arm64
+
+	cmake -DBUILD_ANDROID=1 -DANDROID_ABI=arm64-v8a -DANDROID_NATIVE_API_LEVEL=23 -DCMAKE_BUILD_TYPE=Release -DSTRIP_ANDROID_LIBRARY=On -DLLVM_DIR=$LLVM_ARM64/lib/cmake/llvm -DUSE_INTERCEPTOR_LIB=On -DCMAKE_MAKE_PROGRAM=make ..
+	make -j8
+
+	if ! ls bin/*.apk; then
+		echo "Android build failed"
+		exit 0;
+	fi
+
+	popd # build-android-arm64
 
-if ! ls bin/*.apk; then
-	echo "Android build failed"
-	exit 0;
 fi
 
-popd # build-android-arm64
-
 popd # $REPO_ROOT
