@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "d3d11_test.h"
 
-struct D3D11_Map_Overrun : D3D11GraphicsTest
+RD_TEST(D3D11_Map_Overrun, D3D11GraphicsTest)
 {
   static constexpr const char *Description = "Test overrunning the bounds of a Map() call";
 
@@ -70,10 +70,10 @@ float4 main(v2f IN) : SV_Target0
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create device, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     ID3DBlobPtr vsblob = Compile(common + vertex, "main", "vs_5_0");
@@ -117,4 +117,4 @@ float4 main(v2f IN) : SV_Target0
   }
 };
 
-REGISTER_TEST(D3D11_Map_Overrun);
\ No newline at end of file
+REGISTER_TEST();
