@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,15 +24,15 @@
 
 #include "d3d11_test.h"
 
-struct D3D11_Overlay_Test : D3D11GraphicsTest
+RD_TEST(D3D11_Overlay_Test, D3D11GraphicsTest)
 {
   static constexpr const char *Description =
       "Makes a couple of draws that show off all the overlays in some way";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create device, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     ID3DBlobPtr vsblob = Compile(D3DDefaultVertex, "main", "vs_4_0");
@@ -167,4 +167,4 @@ struct D3D11_Overlay_Test : D3D11GraphicsTest
   }
 };
 
-REGISTER_TEST(D3D11_Overlay_Test);
\ No newline at end of file
+REGISTER_TEST();
