@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,16 +24,16 @@
 
 #include "d3d11_test.h"
 
-struct Refcount_Check : D3D11GraphicsTest
+RD_TEST(D3D11_Refcount_Check, D3D11GraphicsTest)
 {
   static constexpr const char *Description =
       "Ensures that the device etc doesn't delete itself when there are still outstanding "
       "references, and also that it *does* delete itself when any cycle is detected.";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create device, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     ID3D11Debug *dbg = NULL;
@@ -123,4 +123,4 @@ struct Refcount_Check : D3D11GraphicsTest
   }
 };
 
-REGISTER_TEST(Refcount_Check);
\ No newline at end of file
+REGISTER_TEST();
