@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "d3d11_test.h"
 
-struct Saturate : D3D11GraphicsTest
+RD_TEST(D3D11_Saturate, D3D11GraphicsTest)
 {
   static constexpr const char *Description = "Tests using saturate, originally for a bug report";
 
@@ -57,10 +57,10 @@ void main(float4 pos : SV_Position, out float4 a : SV_Target0, out float4 b : SV
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create device, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     ID3DBlobPtr vsblob = Compile(D3DFullscreenQuadVertex, "main", "vs_5_0");
@@ -104,4 +104,4 @@ void main(float4 pos : SV_Position, out float4 a : SV_Target0, out float4 b : SV
   }
 };
 
-REGISTER_TEST(Saturate);
\ No newline at end of file
+REGISTER_TEST();
