@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "d3d11_test.h"
 
-struct Structured_Buffer_MisalignedDirty : D3D11GraphicsTest
+RD_TEST(D3D11_Structured_Buffer_MisalignedDirty, D3D11GraphicsTest)
 {
   static constexpr const char *Description =
       "Test updating a misaligned chunk of a structured buffer";
@@ -51,14 +51,22 @@ float4 main() : SV_Target0
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  void Prepare(int argc, char **argv)
   {
-    // initialise, create window, create device, etc
-    if(!Init(argc, argv))
-      return 3;
+    D3D11GraphicsTest::Prepare(argc, argv);
+
+    if(!Avail.empty())
+      return;
 
     if(!opts.MapNoOverwriteOnDynamicBufferSRV)
-      TEST_ERROR("Can't run Structured_Buffer_MisalignedDirty test without mappable SRVs");
+      Avail = "Requires mappable buffer SRVs";
+  }
+
+  int main()
+  {
+    // initialise, create window, create device, etc
+    if(!Init())
+      return 3;
 
     ID3DBlobPtr vsblob = Compile(D3DDefaultVertex, "main", "vs_5_0");
     ID3DBlobPtr psblob = Compile(pixel, "main", "ps_5_0");
@@ -141,4 +149,4 @@ float4 main() : SV_Target0
   }
 };
 
-REGISTER_TEST(Structured_Buffer_MisalignedDirty);
\ No newline at end of file
+REGISTER_TEST();
