@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2015-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -32,42 +32,110 @@
 
 typedef HRESULT(WINAPI *PFN_CREATE_DXGI_FACTORY)(REFIID, void **);
 
-bool D3D11GraphicsTest::Init(int argc, char **argv)
+namespace
 {
-  // parse parameters here to override parameters
-  GraphicsTest::Init(argc, argv);
+HMODULE d3d11 = NULL;
+HMODULE dxgi = NULL;
+HMODULE d3dcompiler = NULL;
+IDXGIFactoryPtr factory;
+IDXGIAdapterPtr adapter;
+bool warp = false;
 
-  // D3D11 specific can go here
-  // ...
+pD3DCompile dyn_D3DCompile = NULL;
+pD3DStripShader dyn_D3DStripShader = NULL;
+pD3DSetBlobPart dyn_D3DSetBlobPart = NULL;
 
-  HMODULE d3d11 = LoadLibraryA("d3d11.dll");
-  HMODULE dxgi = LoadLibraryA("dxgi.dll");
-  HMODULE d3dcompiler = LoadLibraryA("d3dcompiler_47.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_46.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_45.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_44.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_43.dll");
+PFN_D3D11_CREATE_DEVICE dyn_D3D11CreateDevice = NULL;
+PFN_D3D11_CREATE_DEVICE_AND_SWAP_CHAIN dyn_D3D11CreateDeviceAndSwapChain = NULL;
+};
 
-  if(!d3d11 || !dxgi || !d3dcompiler)
+void D3D11GraphicsTest::Prepare(int argc, char **argv)
+{
+  GraphicsTest::Prepare(argc, argv);
+
+  static bool prepared = false;
+
+  if(!prepared)
   {
-    TEST_ERROR("Couldn't load D3D11");
-    return false;
+    prepared = true;
+
+    d3d11 = LoadLibraryA("d3d11.dll");
+    dxgi = LoadLibraryA("dxgi.dll");
+    d3dcompiler = LoadLibraryA("d3dcompiler_47.dll");
+    if(!d3dcompiler)
+      d3dcompiler = LoadLibraryA("d3dcompiler_46.dll");
+    if(!d3dcompiler)
+      d3dcompiler = LoadLibraryA("d3dcompiler_45.dll");
+    if(!d3dcompiler)
+      d3dcompiler = LoadLibraryA("d3dcompiler_44.dll");
+    if(!d3dcompiler)
+      d3dcompiler = LoadLibraryA("d3dcompiler_43.dll");
+
+    PFN_CREATE_DXGI_FACTORY createFactory = NULL;
+
+    if(dxgi)
+      createFactory = (PFN_CREATE_DXGI_FACTORY)GetProcAddress(dxgi, "CreateDXGIFactory");
+
+    if(d3d11 && d3dcompiler)
+    {
+      dyn_D3D11CreateDevice = (PFN_D3D11_CREATE_DEVICE)GetProcAddress(d3d11, "D3D11CreateDevice");
+      dyn_D3D11CreateDeviceAndSwapChain = (PFN_D3D11_CREATE_DEVICE_AND_SWAP_CHAIN)GetProcAddress(
+          d3d11, "D3D11CreateDeviceAndSwapChain");
+
+      dyn_D3DCompile = (pD3DCompile)GetProcAddress(d3dcompiler, "D3DCompile");
+      dyn_D3DStripShader = (pD3DStripShader)GetProcAddress(d3dcompiler, "D3DStripShader");
+      dyn_D3DSetBlobPart = (pD3DSetBlobPart)GetProcAddress(d3dcompiler, "D3DSetBlobPart");
+    }
+
+    HRESULT hr = S_OK;
+
+    if(createFactory)
+    {
+      hr = createFactory(__uuidof(IDXGIFactory), (void **)&factory);
+
+      if(SUCCEEDED(hr))
+        adapter = ChooseD3DAdapter(factory, argc, argv, warp);
+    }
+
+    if(!d3d11)
+      Avail = "d3d11.dll is not available";
+    else if(!dxgi)
+      Avail = "dxgi.dll is not available";
+    else if(!d3dcompiler)
+      Avail = "d3dcompiler_XX.dll is not available";
+    else if(!factory)
+      Avail = "Couldn't create DXGI factory";
+    else if(!dyn_D3D11CreateDevice || !dyn_D3D11CreateDeviceAndSwapChain || !dyn_D3DCompile ||
+            !dyn_D3DStripShader || !dyn_D3DSetBlobPart)
+      Avail = "Missing required entry point";
+
+    if(dyn_D3D11CreateDevice)
+    {
+      ID3D11DevicePtr tempDev;
+
+      D3D_FEATURE_LEVEL features[] = {D3D_FEATURE_LEVEL_11_0};
+      hr = dyn_D3D11CreateDevice(adapter, D3D_DRIVER_TYPE_HARDWARE, NULL, 0, features, 1,
+                                 D3D11_SDK_VERSION, &tempDev, NULL, NULL);
+
+      if(SUCCEEDED(hr))
+      {
+        tempDev->CheckFeatureSupport(D3D11_FEATURE_D3D11_OPTIONS, &opts, sizeof(opts));
+
+        ID3D11Device1Ptr tempDev1;
+
+        tempDev1 = tempDev;
+        memset(&opts1, 0, sizeof(opts1));
+        if(tempDev1)
+          tempDev1->CheckFeatureSupport(D3D11_FEATURE_D3D11_OPTIONS1, &opts1, sizeof(opts1));
+      }
+    }
   }
+}
 
-  PFN_CREATE_DXGI_FACTORY createFactory =
-      (PFN_CREATE_DXGI_FACTORY)GetProcAddress(dxgi, "CreateDXGIFactory");
-
-  dyn_D3D11CreateDevice = (PFN_D3D11_CREATE_DEVICE)GetProcAddress(d3d11, "D3D11CreateDevice");
-  dyn_D3D11CreateDeviceAndSwapChain = (PFN_D3D11_CREATE_DEVICE_AND_SWAP_CHAIN)GetProcAddress(
-      d3d11, "D3D11CreateDeviceAndSwapChain");
-
-  dyn_D3DCompile = (pD3DCompile)GetProcAddress(d3dcompiler, "D3DCompile");
-  dyn_D3DStripShader = (pD3DStripShader)GetProcAddress(d3dcompiler, "D3DStripShader");
-  dyn_D3DSetBlobPart = (pD3DSetBlobPart)GetProcAddress(d3dcompiler, "D3DSetBlobPart");
+bool D3D11GraphicsTest::Init()
+{
+  if(!GraphicsTest::Init())
+    return false;
 
   D3D_FEATURE_LEVEL features[] = {D3D_FEATURE_LEVEL_11_0};
   D3D_DRIVER_TYPE driver = D3D_DRIVER_TYPE_HARDWARE;
@@ -75,83 +143,14 @@ bool D3D11GraphicsTest::Init(int argc, char **argv)
   if(d3d11_1)
     features[0] = D3D_FEATURE_LEVEL_11_1;
 
+  if(warp)
+    driver = D3D_DRIVER_TYPE_WARP;
+
+  if(adapter)
+    driver = D3D_DRIVER_TYPE_UNKNOWN;
+
   HRESULT hr = S_OK;
 
-  IDXGIFactoryPtr factory = NULL;
-
-  hr = createFactory(__uuidof(IDXGIFactory), (void **)&factory);
-
-  if(factory.GetInterfacePtr() == NULL || FAILED(hr))
-  {
-    TEST_ERROR("CreateDXGIFactory failed: %x", hr);
-    return false;
-  }
-
-  struct AdapterInfo
-  {
-    IDXGIAdapterPtr adapter;
-    DXGI_ADAPTER_DESC desc;
-  };
-
-  std::vector<AdapterInfo> adapters;
-
-  for(UINT i = 0; i < 10; i++)
-  {
-    IDXGIAdapterPtr a;
-    hr = factory->EnumAdapters(i, &a);
-    if(hr == S_OK && a)
-    {
-      DXGI_ADAPTER_DESC desc;
-      a->GetDesc(&desc);
-      adapters.push_back({a, desc});
-    }
-    else
-    {
-      break;
-    }
-  }
-
-  IDXGIAdapterPtr adapter = NULL;
-
-  for(int i = 0; i < argc; i++)
-  {
-    if(!strcmp(argv[i], "--warp"))
-    {
-      driver = D3D_DRIVER_TYPE_WARP;
-      break;
-    }
-    if(!strcmp(argv[i], "--gpu") && i + 1 < argc)
-    {
-      std::string needle = strlower(argv[i + 1]);
-
-      if(needle == "warp")
-      {
-        driver = D3D_DRIVER_TYPE_WARP;
-        break;
-      }
-
-      const bool nv = (needle == "nv" || needle == "nvidia");
-      const bool amd = (needle == "amd");
-      const bool intel = (needle == "intel");
-
-      for(size_t a = 0; a < adapters.size(); a++)
-      {
-        std::string haystack = strlower(Wide2UTF8(adapters[a].desc.Description));
-
-        if(haystack.find(needle) != std::string::npos || (nv && adapters[a].desc.VendorId == 0x10DE) ||
-           (amd && adapters[a].desc.VendorId == 0x1002) ||
-           (intel && adapters[a].desc.VendorId == 0x8086))
-        {
-          driver = D3D_DRIVER_TYPE_UNKNOWN;
-          adapter = adapters[a].adapter;
-          break;
-        }
-      }
-
-      break;
-    }
-  }
-
   UINT flags = createFlags | (debugDevice ? D3D11_CREATE_DEVICE_DEBUG : 0);
 
   if(headless)
@@ -200,7 +199,7 @@ bool D3D11GraphicsTest::Init(int argc, char **argv)
   swapDesc.SampleDesc.Count = backbufferMSAA;
   swapDesc.SampleDesc.Quality = 0;
   swapDesc.OutputWindow = win->wnd;
-  swapDesc.Windowed = fullscreen ? FALSE : TRUE;
+  swapDesc.Windowed = TRUE;
   swapDesc.SwapEffect = DXGI_SWAP_EFFECT_DISCARD;
   swapDesc.Flags = 0;
 
@@ -258,37 +257,6 @@ GraphicsWindow *D3D11GraphicsTest::MakeWindow(int width, int height, const char
   return new Win32Window(width, height, title);
 }
 
-bool D3D11GraphicsTest::IsSupported()
-{
-  static bool checked = false, result = false;
-
-  if(checked)
-    return result;
-
-  checked = true;
-
-  HMODULE d3d11 = LoadLibraryA("d3d11.dll");
-  HMODULE d3dcompiler = LoadLibraryA("d3dcompiler_47.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_46.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_45.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_44.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_43.dll");
-
-  if(d3d11)
-    FreeLibrary(d3d11);
-  if(d3dcompiler)
-    FreeLibrary(d3dcompiler);
-
-  if(d3d11 && d3dcompiler)
-    result = true;
-
-  return result;
-}
-
 void D3D11GraphicsTest::PostDeviceCreate()
 {
   {
@@ -331,12 +299,6 @@ void D3D11GraphicsTest::PostDeviceCreate()
   }
 
   ctx->QueryInterface(__uuidof(ID3DUserDefinedAnnotation), (void **)&annot);
-
-  dev->CheckFeatureSupport(D3D11_FEATURE_D3D11_OPTIONS, &opts, sizeof(opts));
-
-  memset(&opts1, 0, sizeof(opts1));
-  if(dev1)
-    dev1->CheckFeatureSupport(D3D11_FEATURE_D3D11_OPTIONS1, &opts1, sizeof(opts1));
 }
 
 void D3D11GraphicsTest::Shutdown()
@@ -372,6 +334,24 @@ void D3D11GraphicsTest::Present()
   swap->Present(0, 0);
 }
 
+void D3D11GraphicsTest::pushMarker(const std::string &name)
+{
+  if(annot)
+    annot->BeginEvent(UTF82Wide(name).c_str());
+}
+
+void D3D11GraphicsTest::setMarker(const std::string &name)
+{
+  if(annot)
+    annot->SetMarker(UTF82Wide(name).c_str());
+}
+
+void D3D11GraphicsTest::popMarker()
+{
+  if(annot)
+    annot->EndEvent();
+}
+
 std::vector<byte> D3D11GraphicsTest::GetBufferData(ID3D11Buffer *buffer, uint32_t offset, uint32_t len)
 {
   D3D11_MAPPED_SUBRESOURCE mapped;
@@ -432,6 +412,44 @@ void D3D11GraphicsTest::RSSetViewport(D3D11_VIEWPORT view)
   ctx->RSSetViewports(1, &view);
 }
 
+D3D11_RASTERIZER_DESC D3D11GraphicsTest::GetRasterState()
+{
+  ID3D11RasterizerState *state = NULL;
+  ctx->RSGetState(&state);
+
+  D3D11_RASTERIZER_DESC ret;
+
+  if(state)
+  {
+    state->GetDesc(&ret);
+    return ret;
+  }
+
+  ret.FillMode = D3D11_FILL_SOLID;
+  ret.CullMode = D3D11_CULL_BACK;
+  ret.FrontCounterClockwise = FALSE;
+  ret.DepthBias = 0;
+  ret.DepthBiasClamp = 0.0f;
+  ret.SlopeScaledDepthBias = 0.0f;
+  ret.DepthClipEnable = TRUE;
+  ret.ScissorEnable = FALSE;
+  ret.MultisampleEnable = FALSE;
+  ret.AntialiasedLineEnable = FALSE;
+
+  return ret;
+}
+
+void D3D11GraphicsTest::SetRasterState(const D3D11_RASTERIZER_DESC &desc)
+{
+  ID3D11RasterizerState *state = NULL;
+  ctx->RSGetState(&state);
+
+  rastState = NULL;
+  dev->CreateRasterizerState(&desc, &rastState);
+
+  ctx->RSSetState(rastState);
+}
+
 D3D11_DEPTH_STENCIL_DESC D3D11GraphicsTest::GetDepthState()
 {
   ID3D11DepthStencilState *state = NULL;
@@ -613,13 +631,20 @@ ID3D11ComputeShaderPtr D3D11GraphicsTest::CreateCS(ID3DBlobPtr blob)
   return ret;
 }
 
-ID3D11GeometryShaderPtr D3D11GraphicsTest::CreateGS(ID3DBlobPtr blob,
-                                                    const std::vector<D3D11_SO_DECLARATION_ENTRY> &sodecl,
-                                                    const std::vector<UINT> &strides)
+ID3D11GeometryShaderPtr D3D11GraphicsTest::CreateGS(
+    ID3DBlobPtr blob, const std::vector<D3D11_SO_DECLARATION_ENTRY> &sodecl,
+    const std::vector<UINT> &strides, UINT rastStream)
 {
   ID3D11GeometryShaderPtr ret;
   CHECK_HR(dev->CreateGeometryShaderWithStreamOutput(blob->GetBufferPointer(), blob->GetBufferSize(),
                                                      &sodecl[0], (UINT)sodecl.size(), &strides[0],
-                                                     (UINT)strides.size(), 0, NULL, &ret));
+                                                     (UINT)strides.size(), rastStream, NULL, &ret));
+  return ret;
+}
+
+ID3D11GeometryShaderPtr D3D11GraphicsTest::CreateGS(ID3DBlobPtr blob)
+{
+  ID3D11GeometryShaderPtr ret;
+  CHECK_HR(dev->CreateGeometryShader(blob->GetBufferPointer(), blob->GetBufferSize(), NULL, &ret));
   return ret;
 }
