@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2015-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -45,10 +45,10 @@ struct D3D11GraphicsTest : public GraphicsTest
 {
   static const TestAPI API = TestAPI::D3D11;
 
-  bool Init(int argc, char **argv);
+  void Prepare(int argc, char **argv);
+  bool Init();
   void Shutdown();
   GraphicsWindow *MakeWindow(int width, int height, const char *title);
-  bool IsSupported();
 
   void PostDeviceCreate();
 
@@ -75,9 +75,10 @@ struct D3D11GraphicsTest : public GraphicsTest
   ID3D11VertexShaderPtr CreateVS(ID3DBlobPtr blob);
   ID3D11PixelShaderPtr CreatePS(ID3DBlobPtr blob);
   ID3D11ComputeShaderPtr CreateCS(ID3DBlobPtr blob);
+  ID3D11GeometryShaderPtr CreateGS(ID3DBlobPtr blob);
   ID3D11GeometryShaderPtr CreateGS(ID3DBlobPtr blob,
                                    const std::vector<D3D11_SO_DECLARATION_ENTRY> &sodecl,
-                                   const std::vector<UINT> &strides);
+                                   const std::vector<UINT> &strides, UINT rastStream = 0);
 
   ID3DBlobPtr SetBlobPath(std::string name, ID3DBlob *blob);
   void SetBlobPath(std::string name, ID3D11DeviceChild *shader);
@@ -140,6 +141,8 @@ struct D3D11GraphicsTest : public GraphicsTest
 
   void ClearRenderTargetView(ID3D11RenderTargetView *rt, Vec4f col);
 
+  D3D11_RASTERIZER_DESC GetRasterState();
+  void SetRasterState(const D3D11_RASTERIZER_DESC &desc);
   void RSSetViewport(D3D11_VIEWPORT view);
 
   D3D11_DEPTH_STENCIL_DESC GetDepthState();
@@ -149,6 +152,10 @@ struct D3D11GraphicsTest : public GraphicsTest
   bool Running();
   void Present();
 
+  void pushMarker(const std::string &name);
+  void setMarker(const std::string &name);
+  void popMarker();
+
   DXGI_FORMAT backbufferFmt = DXGI_FORMAT_R8G8B8A8_UNORM_SRGB;
   int backbufferCount = 2;
   int backbufferMSAA = 1;
@@ -160,13 +167,7 @@ struct D3D11GraphicsTest : public GraphicsTest
   D3D11_FEATURE_DATA_D3D11_OPTIONS1 opts1;
 
   ID3D11DepthStencilStatePtr depthState;
-
-  pD3DCompile dyn_D3DCompile = NULL;
-  pD3DStripShader dyn_D3DStripShader = NULL;
-  pD3DSetBlobPart dyn_D3DSetBlobPart = NULL;
-
-  PFN_D3D11_CREATE_DEVICE dyn_D3D11CreateDevice = NULL;
-  PFN_D3D11_CREATE_DEVICE_AND_SWAP_CHAIN dyn_D3D11CreateDeviceAndSwapChain = NULL;
+  ID3D11RasterizerStatePtr rastState;
 
   GraphicsWindow *mainWindow = NULL;
 
@@ -185,4 +186,7 @@ struct D3D11GraphicsTest : public GraphicsTest
   ID3D11DeviceContext1Ptr ctx1;
   ID3D11DeviceContext2Ptr ctx2;
   ID3DUserDefinedAnnotationPtr annot;
-};
\ No newline at end of file
+
+private:
+  static bool prepared_d3d11;
+};
