@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -52,7 +52,7 @@ COM_SMARTPTR(IMFByteStream);
 ///////////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////////
 
-struct D3D11_Video_Textures : D3D11GraphicsTest, IMFMediaEngineNotify
+RD_TEST(D3D11_Video_Textures, D3D11GraphicsTest), IMFMediaEngineNotify
 {
   static constexpr const char *Description = "Tests of YUV textures";
 
@@ -214,11 +214,8 @@ float4 main(v2f IN) : SV_Target0
     return S_OK;
   }
 
-  int main(int argc, char **argv)
+  int main()
   {
-    // call this early to parse any data path parameters
-    GraphicsTest::Init(argc, argv);
-
     // check for the existence of the test video
     std::string video_filename = GetDataPath("h264_yu420p_192x108_24fps.mp4");
 
@@ -273,7 +270,7 @@ float4 main(v2f IN) : SV_Target0
     }
 
     // initialise, create window, create device, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     IMFMediaEnginePtr engine = NULL;
@@ -849,4 +846,4 @@ float4 main(v2f IN) : SV_Target0
   }
 };
 
-REGISTER_TEST(D3D11_Video_Textures);
\ No newline at end of file
+REGISTER_TEST();
