@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "d3d12_test.h"
 
-struct D3D12_CBuffer_Zoo : D3D12GraphicsTest
+RD_TEST(D3D12_CBuffer_Zoo, D3D12GraphicsTest)
 {
   static constexpr const char *Description =
       "Tests every kind of constant that can be in a cbuffer to make sure it's decoded "
@@ -36,6 +36,19 @@ struct float3_1 { float3 a; float b; };
 
 struct nested { float3_1 a; float4 b[4]; float3_1 c[4]; };
 
+struct float2_struct { float x; float y; };
+
+struct nested_with_padding
+{
+  float a;                              // 0, <1, 2, 3>
+  float4 b;                             // {4, 5, 6, 7}
+  float c;                              // 8, <9, 10, 11>
+  float3 d[4];                          // [0]: {12, 13, 14}, <15>
+                                        // [1]: {16, 17, 18}, <19>
+                                        // [2]: {20, 21, 22}, <23>
+                                        // [3]: {24, 25, 26}, <27>
+};
+
 cbuffer consts : register(b0)
 {
   // dummy* entries are just to 'reset' packing to avoid pollution between tests
@@ -225,8 +238,22 @@ cbuffer consts : register(b0)
                                           //         <434, 438>
                                           //         <435, 439>
                                           // }
+                                          
+  nested_with_padding ak[2];              // 440 - 467, 468 - 495
 
-  float4 test;                            // {440, 441, 442, 443}
+  float4 dummy13;                         // forces no trailing overlap with ak
+
+  float al;                               // {500}, <501, 502, 503>
+
+  // struct is always float4 aligned, can't be packed with al
+  float2_struct am;                       // {504, 505}, <506, 507>
+
+  // struct allows trailing things into padding
+  float an;                               // {506}
+
+  float4 gldummy4;                        // account for an not overlapping in GL/VK
+
+  float4 test;                            // {512, 513, 514, 515}
 };
 
 // this comes from root signature constants
@@ -238,9 +265,14 @@ cbuffer rootconsts : register(b1)
   float3_1 root_d;
 };
 
+cbuffer hugespace : register(b0, space999999999)
+{
+  float4 huge_val;
+};
+
 float4 main() : SV_Target0
 {
-	return test + root_zero + float4(0.1f, 0.0f, 0.0f, 0.0f);
+	return test + root_zero + huge_val * 1e-30f + float4(0.1f, 0.0f, 0.0f, 0.0f);
 }
 
 )EOSHADER";
@@ -259,14 +291,14 @@ float4 main() : SV_Target0
     float3_1 root_d;
   };
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create device, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     ID3DBlobPtr vsblob = Compile(D3DDefaultVertex, "main", "vs_5_0");
-    ID3DBlobPtr psblob = Compile(pixel, "main", "ps_5_0");
+    ID3DBlobPtr psblob = Compile(pixel, "main", "ps_5_1");
 
     Vec4f cbufferdata[512];
 
@@ -299,6 +331,7 @@ float4 main() : SV_Target0
     ID3D12RootSignaturePtr sig = MakeSig({
         cbvParam(D3D12_SHADER_VISIBILITY_PIXEL, 0, 0),
         constParam(D3D12_SHADER_VISIBILITY_PIXEL, 0, 1, sizeof(rootData) / sizeof(uint32_t)),
+        cbvParam(D3D12_SHADER_VISIBILITY_PIXEL, 999999999, 0),
     });
 
     ID3D12PipelineStatePtr pso = MakePSO().RootSig(sig).InputLayout().VS(vsblob).PS(psblob).RTVs(
@@ -335,6 +368,7 @@ float4 main() : SV_Target0
       cmd->SetGraphicsRootSignature(sig);
       cmd->SetGraphicsRootConstantBufferView(0, cb->GetGPUVirtualAddress());
       cmd->SetGraphicsRoot32BitConstants(1, sizeof(rootData) / sizeof(uint32_t), &rootData, 0);
+      cmd->SetGraphicsRootConstantBufferView(2, cb->GetGPUVirtualAddress() + 256);
 
       RSSetViewport(cmd, {0.0f, 0.0f, (float)screenWidth, (float)screenHeight, 0.0f, 1.0f});
       RSSetScissorRect(cmd, {0, 0, screenWidth, screenHeight});
@@ -356,4 +390,4 @@ float4 main() : SV_Target0
   }
 };
 
-REGISTER_TEST(D3D12_CBuffer_Zoo);
\ No newline at end of file
+REGISTER_TEST();
