@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2018-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -170,7 +170,7 @@ D3D12_ROOT_PARAMETER1 constParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT r
 }
 
 D3D12_ROOT_PARAMETER1 tableParam(D3D12_SHADER_VISIBILITY vis, D3D12_DESCRIPTOR_RANGE_TYPE type,
-                                 UINT space, UINT basereg, UINT numreg)
+                                 UINT space, UINT basereg, UINT numreg, UINT descOffset)
 {
   // this is a super hack but avoids the need to be clumsy with allocation of these structs
   static D3D12_DESCRIPTOR_RANGE1 ranges[32] = {};
@@ -192,7 +192,7 @@ D3D12_ROOT_PARAMETER1 tableParam(D3D12_SHADER_VISIBILITY vis, D3D12_DESCRIPTOR_R
   range.RegisterSpace = space;
   range.BaseShaderRegister = basereg;
   range.NumDescriptors = numreg;
-  range.OffsetInDescriptorsFromTableStart = 0;
+  range.OffsetInDescriptorsFromTableStart = descOffset;
   range.Flags = D3D12_DESCRIPTOR_RANGE_FLAG_NONE;
 
   return ret;
@@ -348,6 +348,12 @@ D3D12TextureCreator &D3D12TextureCreator::Readback()
   return *this;
 }
 
+D3D12TextureCreator &D3D12TextureCreator::CustomHeap(D3D12_HEAP_PROPERTIES heap)
+{
+  m_HeapDesc = heap;
+  return *this;
+}
+
 D3D12TextureCreator &D3D12TextureCreator::InitialState(D3D12_RESOURCE_STATES state)
 {
   m_InitialState = state;
@@ -694,8 +700,6 @@ D3D12ViewCreator &D3D12ViewCreator::FirstMip(UINT mip)
 {
   if(firstMip)
     *firstMip = mip;
-  else
-    TEST_ERROR("This view & resource doesn't support FirstMip");
   return *this;
 }
 
@@ -703,8 +707,6 @@ D3D12ViewCreator &D3D12ViewCreator::NumMips(UINT num)
 {
   if(numMips)
     *numMips = num;
-  else
-    TEST_ERROR("This view & resource doesn't support NumMips");
   return *this;
 }
 
@@ -712,8 +714,6 @@ D3D12ViewCreator &D3D12ViewCreator::FirstSlice(UINT mip)
 {
   if(firstSlice)
     *firstSlice = mip;
-  else
-    TEST_ERROR("This view & resource doesn't support FirstSlice");
   return *this;
 }
 
@@ -721,8 +721,6 @@ D3D12ViewCreator &D3D12ViewCreator::NumSlices(UINT num)
 {
   if(numSlices)
     *numSlices = num;
-  else
-    TEST_ERROR("This view & resource doesn't support NumSlices");
   return *this;
 }
 
@@ -736,8 +734,6 @@ D3D12ViewCreator &D3D12ViewCreator::PlaneSlice(UINT plane)
 {
   if(planeSlice)
     *planeSlice = plane;
-  else
-    TEST_ERROR("This view & resource doesn't support NumSlices");
   return *this;
 }
 
@@ -868,6 +864,16 @@ D3D12PSOCreator::D3D12PSOCreator(D3D12GraphicsTest *test) : m_Test(test)
   GraphicsDesc.BlendState.RenderTarget[0].DestBlendAlpha = D3D12_BLEND_INV_SRC_ALPHA;
   GraphicsDesc.BlendState.RenderTarget[0].BlendOpAlpha = D3D12_BLEND_OP_ADD;
   GraphicsDesc.BlendState.RenderTarget[0].RenderTargetWriteMask = D3D12_COLOR_WRITE_ENABLE_ALL;
+  GraphicsDesc.DepthStencilState.DepthEnable = FALSE;
+  GraphicsDesc.DepthStencilState.DepthWriteMask = D3D12_DEPTH_WRITE_MASK_ALL;
+  GraphicsDesc.DepthStencilState.DepthFunc = D3D12_COMPARISON_FUNC_LESS_EQUAL;
+  GraphicsDesc.DepthStencilState.StencilReadMask = 0xff;
+  GraphicsDesc.DepthStencilState.StencilWriteMask = 0xff;
+  GraphicsDesc.DepthStencilState.FrontFace.StencilFunc = D3D12_COMPARISON_FUNC_EQUAL;
+  GraphicsDesc.DepthStencilState.FrontFace.StencilFailOp = D3D12_STENCIL_OP_KEEP;
+  GraphicsDesc.DepthStencilState.FrontFace.StencilDepthFailOp = D3D12_STENCIL_OP_KEEP;
+  GraphicsDesc.DepthStencilState.FrontFace.StencilPassOp = D3D12_STENCIL_OP_REPLACE;
+  GraphicsDesc.DepthStencilState.BackFace = GraphicsDesc.DepthStencilState.FrontFace;
 }
 
 D3D12PSOCreator &D3D12PSOCreator::VS(ID3DBlobPtr blob)
@@ -966,4 +972,4 @@ D3D12PSOCreator::operator ID3D12PipelineStatePtr() const
                                                       (void **)&pso));
   }
   return pso;
-}
\ No newline at end of file
+}
