@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2018-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -26,17 +26,10 @@
 
 #include <comdef.h>
 #include "dx/d3d_helpers.h"
+#include "dx/official/D3D12Downlevel.h"
 #include "dx/official/d3d12.h"
-#include "dx/official/dxgi1_4.h"
-
-#define COM_SMARTPTR(classname) _COM_SMARTPTR_TYPEDEF(classname, __uuidof(classname))
 
 COM_SMARTPTR(ID3DBlob);
-COM_SMARTPTR(IDXGISwapChain);
-COM_SMARTPTR(IDXGISwapChain1);
-COM_SMARTPTR(IDXGIFactory4);
-COM_SMARTPTR(IDXGIAdapter);
-COM_SMARTPTR(IDXGISurface);
 
 COM_SMARTPTR(ID3D12Debug);
 COM_SMARTPTR(ID3D12Debug1);
@@ -62,6 +55,8 @@ COM_SMARTPTR(ID3D12DescriptorHeap);
 
 COM_SMARTPTR(ID3D12InfoQueue);
 
+COM_SMARTPTR(ID3D12CommandQueueDownlevel);
+
 struct D3D12GraphicsTest;
 
 class D3D12PSOCreator
@@ -146,6 +141,7 @@ public:
 
   D3D12TextureCreator &Upload();
   D3D12TextureCreator &Readback();
+  D3D12TextureCreator &CustomHeap(D3D12_HEAP_PROPERTIES heap);
 
   D3D12TextureCreator &InitialState(D3D12_RESOURCE_STATES state);
 
@@ -257,7 +253,7 @@ D3D12_ROOT_PARAMETER1 cbvParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg
 D3D12_ROOT_PARAMETER1 constParam(D3D12_SHADER_VISIBILITY vis, UINT space, UINT reg, UINT num);
 
 D3D12_ROOT_PARAMETER1 tableParam(D3D12_SHADER_VISIBILITY vis, D3D12_DESCRIPTOR_RANGE_TYPE type,
-                                 UINT space, UINT basereg, UINT numreg);
+                                 UINT space, UINT basereg, UINT numreg, UINT descOffset = 0);
 
 #define GET_REFCOUNT(val, obj) \
   do                           \
