@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,15 +24,15 @@
 
 #include "d3d12_test.h"
 
-struct D3D12_Overlay_Test : D3D12GraphicsTest
+RD_TEST(D3D12_Overlay_Test, D3D12GraphicsTest)
 {
   static constexpr const char *Description =
       "Makes a couple of draws that show off all the overlays in some way";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create device, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     ID3DBlobPtr vsblob = Compile(D3DDefaultVertex, "main", "vs_4_0");
@@ -177,7 +177,7 @@ struct D3D12_Overlay_Test : D3D12GraphicsTest
       cmd->DrawInstanced(3, 1, 6, 0);
 
       // add a marker so we can easily locate this draw
-      cmd->SetMarker(1, "Test Begin", sizeof("Test Begin"));
+      cmd->SetMarker(1, "Test Begin", sizeof("Test Begin") - 1);
 
       cmd->SetPipelineState(pipe);
       cmd->DrawInstanced(24, 1, 9, 0);
@@ -195,4 +195,4 @@ struct D3D12_Overlay_Test : D3D12GraphicsTest
   }
 };
 
-REGISTER_TEST(D3D12_Overlay_Test);
\ No newline at end of file
+REGISTER_TEST();
