@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2018-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -30,36 +30,136 @@
 #include "../renderdoc_app.h"
 #include "../win32/win32_window.h"
 
+typedef HRESULT(WINAPI *PFN_CREATE_DXGI_FACTORY1)(REFIID, void **);
 typedef HRESULT(WINAPI *PFN_CREATE_DXGI_FACTORY2)(UINT, REFIID, void **);
 
-bool D3D12GraphicsTest::Init(int argc, char **argv)
+namespace
 {
-  // parse parameters here to override parameters
-  GraphicsTest::Init(argc, argv);
+HMODULE d3d12 = NULL;
+HMODULE dxgi = NULL;
+HMODULE d3dcompiler = NULL;
+IDXGIFactory1Ptr factory;
+IDXGIAdapterPtr adapter;
+bool d3d12on7 = false;
 
-  // D3D12 specific can go here
-  // ...
+HRESULT EnumAdapterByLuid(LUID luid, IDXGIAdapterPtr &pAdapter)
+{
+  HRESULT hr = S_OK;
 
-  HMODULE d3d12 = LoadLibraryA("d3d12.dll");
-  HMODULE dxgi = LoadLibraryA("dxgi.dll");
-  HMODULE d3dcompiler = LoadLibraryA("d3dcompiler_47.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_46.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_45.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_44.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_43.dll");
+  pAdapter = NULL;
 
-  if(!d3d12 || !dxgi || !d3dcompiler)
+  for(UINT i = 0; i < 10; i++)
   {
-    TEST_ERROR("Couldn't load D3D12");
-    return false;
+    IDXGIAdapterPtr ad;
+    hr = factory->EnumAdapters(i, &ad);
+    if(hr == S_OK && ad)
+    {
+      DXGI_ADAPTER_DESC desc;
+      ad->GetDesc(&desc);
+
+      if(desc.AdapterLuid.LowPart == luid.LowPart && desc.AdapterLuid.HighPart == luid.HighPart)
+      {
+        pAdapter = ad;
+        return S_OK;
+      }
+    }
+    else
+    {
+      break;
+    }
   }
 
-  PFN_CREATE_DXGI_FACTORY2 createFactory2 =
-      (PFN_CREATE_DXGI_FACTORY2)GetProcAddress(dxgi, "CreateDXGIFactory2");
+  return E_FAIL;
+}
+};
+
+void D3D12GraphicsTest::Prepare(int argc, char **argv)
+{
+  GraphicsTest::Prepare(argc, argv);
+
+  static bool prepared = false;
+
+  if(!prepared)
+  {
+    prepared = true;
+
+    d3d12 = LoadLibraryA("d3d12.dll");
+    dxgi = LoadLibraryA("dxgi.dll");
+    d3dcompiler = LoadLibraryA("d3dcompiler_47.dll");
+    if(!d3dcompiler)
+      d3dcompiler = LoadLibraryA("d3dcompiler_46.dll");
+    if(!d3dcompiler)
+      d3dcompiler = LoadLibraryA("d3dcompiler_45.dll");
+    if(!d3dcompiler)
+      d3dcompiler = LoadLibraryA("d3dcompiler_44.dll");
+    if(!d3dcompiler)
+      d3dcompiler = LoadLibraryA("d3dcompiler_43.dll");
+
+    if(!d3d12)
+    {
+      d3d12 = LoadLibraryA("12on7/d3d12.dll");
+      d3d12on7 = (d3d12 != NULL);
+    }
+
+    if(d3d12)
+    {
+      PFN_CREATE_DXGI_FACTORY1 createFactory1 =
+          (PFN_CREATE_DXGI_FACTORY1)GetProcAddress(dxgi, "CreateDXGIFactory1");
+      PFN_CREATE_DXGI_FACTORY2 createFactory2 =
+          (PFN_CREATE_DXGI_FACTORY2)GetProcAddress(dxgi, "CreateDXGIFactory2");
+
+      HRESULT hr = E_FAIL;
+
+      if(createFactory2)
+        hr = createFactory2(debugDevice ? DXGI_CREATE_FACTORY_DEBUG : 0, __uuidof(IDXGIFactory1),
+                            (void **)&factory);
+      else if(createFactory1)
+        hr = createFactory1(__uuidof(IDXGIFactory1), (void **)&factory);
+
+      if(SUCCEEDED(hr))
+      {
+        bool warp = false;
+
+        adapter = ChooseD3DAdapter(factory, argc, argv, warp);
+
+        if(warp && !d3d12on7)
+        {
+          IDXGIFactory4Ptr factory4 = factory;
+          if(factory4)
+            factory4->EnumWarpAdapter(__uuidof(IDXGIAdapter), (void **)&adapter);
+        }
+      }
+    }
+  }
+
+  if(!d3d12)
+    Avail = "d3d12.dll is not available";
+  else if(!dxgi)
+    Avail = "dxgi.dll is not available";
+  else if(!d3dcompiler)
+    Avail = "d3dcompiler_XX.dll is not available";
+  else if(!factory)
+    Avail = "Couldn't create DXGI factory";
+
+  for(int i = 0; i < argc; i++)
+  {
+    if(!strcmp(argv[i], "--gpuva") || !strcmp(argv[i], "--debug-gpu"))
+    {
+      gpuva = true;
+    }
+  }
+
+  m_Factory = factory;
+}
+
+bool D3D12GraphicsTest::Init()
+{
+  // parse parameters here to override parameters
+  if(!GraphicsTest::Init())
+    return false;
+
+  // we can assume d3d12, dxgi and d3dcompiler are valid since we shouldn't be running the test if
+  // Prepare() failed
 
   dyn_D3D12CreateDevice = (PFN_D3D12_CREATE_DEVICE)GetProcAddress(d3d12, "D3D12CreateDevice");
 
@@ -99,101 +199,23 @@ bool D3D12GraphicsTest::Init(int argc, char **argv)
     if(SUCCEEDED(hr) && d3d12Debug)
     {
       d3d12Debug->EnableDebugLayer();
+
+      if(gpuva)
+      {
+        ID3D12Debug1Ptr debug1 = d3d12Debug;
+
+        if(debug1)
+          debug1->SetEnableGPUBasedValidation(true);
+      }
     }
   }
 
   {
     HRESULT hr = S_OK;
 
-    hr = createFactory2(debugDevice ? DXGI_CREATE_FACTORY_DEBUG : 0, __uuidof(IDXGIFactory4),
-                        (void **)&m_Factory);
-
-    if(m_Factory.GetInterfacePtr() == NULL || FAILED(hr))
-    {
-      TEST_ERROR("CreateDXGIFactory2 failed: %x", hr);
-      return false;
-    }
-
-    struct AdapterInfo
-    {
-      IDXGIAdapterPtr adapter;
-      DXGI_ADAPTER_DESC desc;
-    };
-
-    std::vector<AdapterInfo> adapters;
-
-    for(UINT i = 0; i < 10; i++)
-    {
-      IDXGIAdapterPtr a;
-      hr = m_Factory->EnumAdapters(i, &a);
-      if(hr == S_OK && a)
-      {
-        DXGI_ADAPTER_DESC desc;
-        a->GetDesc(&desc);
-        adapters.push_back({a, desc});
-      }
-      else
-      {
-        break;
-      }
-    }
-
-    IDXGIAdapterPtr warp = NULL;
-    m_Factory->EnumWarpAdapter(__uuidof(IDXGIAdapter), (void **)&warp);
-
-    IDXGIAdapterPtr adapter = NULL;
-
-    for(int i = 0; i < argc; i++)
-    {
-      if(!strcmp(argv[i], "--warp"))
-      {
-        adapter = warp;
-        break;
-      }
-      if(!strcmp(argv[i], "--gpu") && i + 1 < argc)
-      {
-        std::string needle = strlower(argv[i + 1]);
-
-        if(needle == "warp")
-        {
-          adapter = warp;
-          break;
-        }
-
-        const bool nv = (needle == "nv" || needle == "nvidia");
-        const bool amd = (needle == "amd");
-        const bool intel = (needle == "intel");
-
-        for(size_t a = 0; a < adapters.size(); a++)
-        {
-          std::string haystack = strlower(Wide2UTF8(adapters[a].desc.Description));
-
-          if(haystack.find(needle) != std::string::npos ||
-             (nv && adapters[a].desc.VendorId == 0x10DE) ||
-             (amd && adapters[a].desc.VendorId == 0x1002) ||
-             (intel && adapters[a].desc.VendorId == 0x8086))
-          {
-            adapter = adapters[a].adapter;
-            break;
-          }
-        }
-
-        break;
-      }
-    }
-
     hr = dyn_D3D12CreateDevice(adapter, D3D_FEATURE_LEVEL_11_0, __uuidof(ID3D12Device),
                                (void **)&dev);
 
-    // if it failed, try again on warp
-    if(FAILED(hr))
-    {
-      adapter = warp;
-      if(adapter)
-        hr = dyn_D3D12CreateDevice(adapter, D3D_FEATURE_LEVEL_11_0, __uuidof(ID3D12Device),
-                                   (void **)&dev);
-    }
-
     if(FAILED(hr))
     {
       TEST_ERROR("D3D12CreateDevice failed: %x", hr);
@@ -205,7 +227,7 @@ bool D3D12GraphicsTest::Init(int argc, char **argv)
     LUID luid = dev->GetAdapterLuid();
 
     IDXGIAdapterPtr pDXGIAdapter;
-    HRESULT hr = m_Factory->EnumAdapterByLuid(luid, __uuidof(IDXGIAdapter), (void **)&pDXGIAdapter);
+    HRESULT hr = EnumAdapterByLuid(dev->GetAdapterLuid(), pDXGIAdapter);
 
     if(FAILED(hr))
     {
@@ -247,10 +269,44 @@ bool D3D12GraphicsTest::Init(int argc, char **argv)
     swapDesc.Stereo = FALSE;
     swapDesc.SwapEffect = DXGI_SWAP_EFFECT_FLIP_DISCARD;
 
-    CHECK_HR(m_Factory->CreateSwapChainForHwnd(queue, win->wnd, &swapDesc, NULL, NULL, &swap));
+    if(!d3d12on7)
+    {
+      IDXGIFactory4Ptr factory4 = m_Factory;
 
-    CHECK_HR(swap->GetBuffer(0, __uuidof(ID3D12Resource), (void **)&bbTex[0]));
-    CHECK_HR(swap->GetBuffer(1, __uuidof(ID3D12Resource), (void **)&bbTex[1]));
+      CHECK_HR(factory4->CreateSwapChainForHwnd(queue, win->wnd, &swapDesc, NULL, NULL, &swap));
+
+      CHECK_HR(swap->GetBuffer(0, __uuidof(ID3D12Resource), (void **)&bbTex[0]));
+      CHECK_HR(swap->GetBuffer(1, __uuidof(ID3D12Resource), (void **)&bbTex[1]));
+    }
+    else
+    {
+      D3D12_RESOURCE_DESC bbDesc;
+      bbDesc.Alignment = 0;
+      bbDesc.DepthOrArraySize = 1;
+      bbDesc.Dimension = D3D12_RESOURCE_DIMENSION_TEXTURE2D;
+      bbDesc.Flags = D3D12_RESOURCE_FLAG_ALLOW_RENDER_TARGET;
+      bbDesc.Format = backbufferFmt;
+      bbDesc.Height = screenHeight;
+      bbDesc.Layout = D3D12_TEXTURE_LAYOUT_UNKNOWN;
+      bbDesc.MipLevels = 1;
+      bbDesc.SampleDesc.Count = 1;
+      bbDesc.SampleDesc.Quality = 0;
+      bbDesc.Width = screenWidth;
+
+      D3D12_HEAP_PROPERTIES heapProps;
+      heapProps.Type = D3D12_HEAP_TYPE_DEFAULT;
+      heapProps.CPUPageProperty = D3D12_CPU_PAGE_PROPERTY_UNKNOWN;
+      heapProps.MemoryPoolPreference = D3D12_MEMORY_POOL_UNKNOWN;
+      heapProps.CreationNodeMask = 1;
+      heapProps.VisibleNodeMask = 1;
+
+      CHECK_HR(dev->CreateCommittedResource(&heapProps, D3D12_HEAP_FLAG_NONE, &bbDesc,
+                                            D3D12_RESOURCE_STATE_PRESENT, NULL,
+                                            __uuidof(ID3D12Resource), (void **)&bbTex[0]));
+      CHECK_HR(dev->CreateCommittedResource(&heapProps, D3D12_HEAP_FLAG_NONE, &bbDesc,
+                                            D3D12_RESOURCE_STATE_PRESENT, NULL,
+                                            __uuidof(ID3D12Resource), (void **)&bbTex[1]));
+    }
   }
 
   dev->CreateFence(0, D3D12_FENCE_FLAG_NONE, __uuidof(ID3D12Fence), (void **)&m_GPUSyncFence);
@@ -376,37 +432,6 @@ GraphicsWindow *D3D12GraphicsTest::MakeWindow(int width, int height, const char
   return new Win32Window(width, height, title);
 }
 
-bool D3D12GraphicsTest::IsSupported()
-{
-  static bool checked = false, result = false;
-
-  if(checked)
-    return result;
-
-  checked = true;
-
-  HMODULE d3d12 = LoadLibraryA("d3d12.dll");
-  HMODULE d3dcompiler = LoadLibraryA("d3dcompiler_47.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_46.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_45.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_44.dll");
-  if(!d3dcompiler)
-    d3dcompiler = LoadLibraryA("d3dcompiler_43.dll");
-
-  if(d3d12)
-    FreeLibrary(d3d12);
-  if(d3dcompiler)
-    FreeLibrary(d3dcompiler);
-
-  if(d3d12 && d3dcompiler)
-    result = true;
-
-  return result;
-}
-
 void D3D12GraphicsTest::Shutdown()
 {
   GPUSync();
@@ -492,7 +517,25 @@ void D3D12GraphicsTest::GPUSync()
 
 void D3D12GraphicsTest::Present()
 {
-  swap->Present(0, 0);
+  if(swap)
+  {
+    swap->Present(0, 0);
+  }
+  else
+  {
+    ID3D12CommandQueueDownlevelPtr downlevel = queue;
+
+    ID3D12GraphicsCommandListPtr cmd = GetCommandBuffer();
+    Reset(cmd);
+
+    downlevel->Present(cmd, bbTex[1 - texIdx], ((Win32Window *)mainWindow)->wnd,
+                       D3D12_DOWNLEVEL_PRESENT_FLAG_NONE);
+
+    m_GPUSyncCounter++;
+    queue->Signal(m_GPUSyncFence, m_GPUSyncCounter);
+
+    pendingCommandBuffers.push_back(std::make_pair(cmd, m_GPUSyncFence));
+  }
 
   for(auto it = pendingCommandBuffers.begin(); it != pendingCommandBuffers.end();)
   {
@@ -716,6 +759,26 @@ void D3D12GraphicsTest::SetBufferData(ID3D12ResourcePtr buffer, D3D12_RESOURCE_S
   m_Alloc->Reset();
 }
 
+void D3D12GraphicsTest::pushMarker(ID3D12GraphicsCommandListPtr cmd, const std::string &name)
+{
+  // D3D debug layer spams un-mutable errors if we call this. Don't do anything unless renderdoc is
+  // attached
+  if(rdoc)
+    cmd->BeginEvent(1, name.data(), (UINT)name.size());
+}
+
+void D3D12GraphicsTest::setMarker(ID3D12GraphicsCommandListPtr cmd, const std::string &name)
+{
+  if(rdoc)
+    cmd->SetMarker(1, name.data(), (UINT)name.size());
+}
+
+void D3D12GraphicsTest::popMarker(ID3D12GraphicsCommandListPtr cmd)
+{
+  if(rdoc)
+    cmd->EndEvent();
+}
+
 void D3D12GraphicsTest::ResourceBarrier(ID3D12GraphicsCommandListPtr cmd, ID3D12ResourcePtr res,
                                         D3D12_RESOURCE_STATES before, D3D12_RESOURCE_STATES after)
 {
@@ -1044,4 +1107,4 @@ ID3D12RootSignaturePtr D3D12GraphicsTest::MakeSig(const std::vector<D3D12_ROOT_P
   CHECK_HR(dev->CreateRootSignature(0, blob->GetBufferPointer(), blob->GetBufferSize(),
                                     __uuidof(ID3D12RootSignature), (void **)&ret));
   return ret;
-}
\ No newline at end of file
+}
