@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2018-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -46,10 +46,10 @@ struct D3D12GraphicsTest : public GraphicsTest
 {
   static const TestAPI API = TestAPI::D3D12;
 
-  bool Init(int argc, char **argv);
+  void Prepare(int argc, char **argv);
+  bool Init();
   void Shutdown();
   GraphicsWindow *MakeWindow(int width, int height, const char *title);
-  bool IsSupported();
 
   enum BufType
   {
@@ -131,6 +131,10 @@ struct D3D12GraphicsTest : public GraphicsTest
     return ret;
   }
 
+  void pushMarker(ID3D12GraphicsCommandListPtr cmd, const std::string &name);
+  void setMarker(ID3D12GraphicsCommandListPtr cmd, const std::string &name);
+  void popMarker(ID3D12GraphicsCommandListPtr cmd);
+
   void ResourceBarrier(ID3D12GraphicsCommandListPtr cmd, ID3D12ResourcePtr res,
                        D3D12_RESOURCE_STATES before, D3D12_RESOURCE_STATES after);
   void ResourceBarrier(ID3D12ResourcePtr res, D3D12_RESOURCE_STATES before,
@@ -181,7 +185,8 @@ struct D3D12GraphicsTest : public GraphicsTest
   ID3D12ResourcePtr bbTex[2];
   uint32_t texIdx = 0;
 
-  IDXGIFactory4Ptr m_Factory;
+  bool gpuva = false;
+  IDXGIFactory1Ptr m_Factory;
 
   ID3D12DebugPtr d3d12Debug;
   ID3D12InfoQueuePtr infoqueue;
@@ -206,4 +211,4 @@ struct D3D12GraphicsTest : public GraphicsTest
 
   std::vector<ID3D12GraphicsCommandListPtr> freeCommandBuffers;
   std::vector<std::pair<ID3D12GraphicsCommandListPtr, UINT64>> pendingCommandBuffers;
-};
\ No newline at end of file
+};
