@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2018-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -24,8 +24,22 @@
 
 #pragma once
 
+#include <comdef.h>
 #include <string>
+#include "dx/official/dxgi1_4.h"
 
 extern std::string D3DFullscreenQuadVertex;
 extern std::string D3DDefaultVertex;
-extern std::string D3DDefaultPixel;
\ No newline at end of file
+extern std::string D3DDefaultPixel;
+
+#define COM_SMARTPTR(classname) _COM_SMARTPTR_TYPEDEF(classname, __uuidof(classname))
+
+COM_SMARTPTR(IDXGISwapChain);
+COM_SMARTPTR(IDXGISwapChain1);
+COM_SMARTPTR(IDXGIFactory);
+COM_SMARTPTR(IDXGIFactory1);
+COM_SMARTPTR(IDXGIFactory4);
+COM_SMARTPTR(IDXGIAdapter);
+COM_SMARTPTR(IDXGISurface);
+
+IDXGIAdapterPtr ChooseD3DAdapter(IDXGIFactoryPtr factory, int argc, char **argv, bool &warp);
