@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "gl_test.h"
 
-struct GL_Buffer_Updates : OpenGLGraphicsTest
+RD_TEST(GL_Buffer_Updates, OpenGLGraphicsTest)
 {
   static constexpr const char *Description =
       "Test of buffer updates, both buffers that are updated regularly and get marked as "
@@ -55,15 +55,18 @@ layout(binding = 0, std140) uniform constsbuf
 
 void main()
 {
-	Color = col;
+  if(col == vec4(0.0, 1.0, 0.0, 1.0))
+    Color = col;
+  else
+    Color = vec4(1.0, 0.0, 0.0, 1.0);
 }
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     GLuint vao = MakeVAO();
@@ -74,30 +77,34 @@ void main()
     Vec4f green(0.0f, 1.0f, 0.0f, 1.0f);
 
 // clang-format explodes on these for some reason
-#define TEST_CASES()                            \
-  BUFFER_TEST(BufferDataImmutable)              \
-  BUFFER_TEST(BufferStorageImmutable)           \
-  BUFFER_TEST(BufferDataOrphanedOnce)           \
-  BUFFER_TEST(BufferDataOrphanedMany)           \
-  BUFFER_TEST(BufferDataOrphanedPerFrame)       \
-  BUFFER_TEST(BufferDataUpdatedOnce)            \
-  BUFFER_TEST(BufferDataUpdatedMany)            \
-  BUFFER_TEST(BufferDataUpdatedPerFrame)        \
-  BUFFER_TEST(BufferStorageUpdatedOnce)         \
-  BUFFER_TEST(BufferStorageUpdatedMany)         \
-  BUFFER_TEST(BufferStorageUpdatedPerFrame)     \
-  BUFFER_TEST(SingleMapBufferReadback)          \
-  BUFFER_TEST(SingleMapBufferRangeReadback)     \
-  BUFFER_TEST(CoherentMapBufferRangeReadback)   \
-  BUFFER_TEST(CleanBufferMapWriteInvalidate)    \
-  BUFFER_TEST(CleanBufferMapWriteNonInvalidate) \
-  BUFFER_TEST(DirtyBufferMapWriteInvalidate)    \
-  BUFFER_TEST(DirtyBufferMapWriteNonInvalidate) \
-  BUFFER_TEST(CleanBufferMapFlushExplicit)      \
-  BUFFER_TEST(DirtyBufferMapFlushExplicit)      \
-  BUFFER_TEST(CoherentMapWrite)                 \
-  BUFFER_TEST(NonCoherentMapFlush)              \
-  BUFFER_TEST(OffsetMapWrite)                   \
+#define TEST_CASES()                             \
+  BUFFER_TEST(BufferDataImmutable)               \
+  BUFFER_TEST(BufferStorageImmutable)            \
+  BUFFER_TEST(BufferDataOrphanedOnce)            \
+  BUFFER_TEST(BufferDataOrphanedMany)            \
+  BUFFER_TEST(BufferDataOrphanedPerFrame)        \
+  BUFFER_TEST(BufferDataUpdatedOnce)             \
+  BUFFER_TEST(BufferDataUpdatedMany)             \
+  BUFFER_TEST(BufferDataUpdatedPerFrame)         \
+  BUFFER_TEST(BufferStorageUpdatedOnce)          \
+  BUFFER_TEST(BufferStorageUpdatedMany)          \
+  BUFFER_TEST(BufferStorageUpdatedPerFrame)      \
+  BUFFER_TEST(SingleMapBufferReadback)           \
+  BUFFER_TEST(SingleMapBufferRangeReadback)      \
+  BUFFER_TEST(CoherentMapBufferRangeReadback)    \
+  BUFFER_TEST(CleanBufferMapWriteInvalidate)     \
+  BUFFER_TEST(CleanBufferMapWriteNonInvalidate)  \
+  BUFFER_TEST(DirtyBufferMapWriteInvalidate)     \
+  BUFFER_TEST(DirtyBufferMapWriteNonInvalidate)  \
+  BUFFER_TEST(CleanBufferMapFlushExplicit)       \
+  BUFFER_TEST(DirtyBufferMapFlushExplicit)       \
+  BUFFER_TEST(CoherentMapWrite)                  \
+  BUFFER_TEST(CoherentMapWriteInvalidateRange)   \
+  BUFFER_TEST(CoherentMapWriteInvalidateBuffer)  \
+  BUFFER_TEST(CoherentMapWriteUnsynchronised)    \
+  BUFFER_TEST(NonCoherentMapFlush)               \
+  BUFFER_TEST(NonCoherentMapFlushUnsynchronised) \
+  BUFFER_TEST(OffsetMapWrite)                    \
   BUFFER_TEST(OffsetMapFlush)
 
 #undef BUFFER_TEST
@@ -195,7 +202,8 @@ void main()
       glBufferSubData(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f), &red);
 
     glBindBuffer(GL_UNIFORM_BUFFER, buffers[DirtyBufferMapFlushExplicit]);
-    glBufferStorage(GL_UNIFORM_BUFFER, sizeof(Vec4f), &red, GL_MAP_WRITE_BIT);
+    glBufferStorage(GL_UNIFORM_BUFFER, sizeof(Vec4f), &red,
+                    GL_MAP_WRITE_BIT | GL_DYNAMIC_STORAGE_BIT);
     for(int i = 0; i < 100; i++)
       glBufferSubData(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f), &red);
 
@@ -207,6 +215,30 @@ void main()
         (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f),
                                   GL_MAP_COHERENT_BIT | GL_MAP_PERSISTENT_BIT | GL_MAP_WRITE_BIT);
 
+    glBindBuffer(GL_UNIFORM_BUFFER, buffers[CoherentMapWriteInvalidateRange]);
+    glBufferStorage(GL_UNIFORM_BUFFER, sizeof(Vec4f), &red,
+                    GL_MAP_WRITE_BIT | GL_MAP_PERSISTENT_BIT | GL_MAP_COHERENT_BIT);
+
+    ptrs[CoherentMapWriteInvalidateRange] = (Vec4f *)glMapBufferRange(
+        GL_UNIFORM_BUFFER, 0, sizeof(Vec4f), GL_MAP_COHERENT_BIT | GL_MAP_PERSISTENT_BIT |
+                                                 GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_RANGE_BIT);
+
+    glBindBuffer(GL_UNIFORM_BUFFER, buffers[CoherentMapWriteInvalidateBuffer]);
+    glBufferStorage(GL_UNIFORM_BUFFER, sizeof(Vec4f), &red,
+                    GL_MAP_WRITE_BIT | GL_MAP_PERSISTENT_BIT | GL_MAP_COHERENT_BIT);
+
+    ptrs[CoherentMapWriteInvalidateBuffer] = (Vec4f *)glMapBufferRange(
+        GL_UNIFORM_BUFFER, 0, sizeof(Vec4f), GL_MAP_COHERENT_BIT | GL_MAP_PERSISTENT_BIT |
+                                                 GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT);
+
+    glBindBuffer(GL_UNIFORM_BUFFER, buffers[CoherentMapWriteUnsynchronised]);
+    glBufferStorage(GL_UNIFORM_BUFFER, sizeof(Vec4f), &red,
+                    GL_MAP_WRITE_BIT | GL_MAP_PERSISTENT_BIT | GL_MAP_COHERENT_BIT);
+
+    ptrs[CoherentMapWriteUnsynchronised] = (Vec4f *)glMapBufferRange(
+        GL_UNIFORM_BUFFER, 0, sizeof(Vec4f),
+        GL_MAP_COHERENT_BIT | GL_MAP_PERSISTENT_BIT | GL_MAP_WRITE_BIT | GL_MAP_UNSYNCHRONIZED_BIT);
+
     glBindBuffer(GL_UNIFORM_BUFFER, buffers[NonCoherentMapFlush]);
     glBufferStorage(GL_UNIFORM_BUFFER, sizeof(Vec4f), &red, GL_MAP_WRITE_BIT | GL_MAP_PERSISTENT_BIT);
 
@@ -214,6 +246,14 @@ void main()
         GL_UNIFORM_BUFFER, 0, sizeof(Vec4f),
         GL_MAP_PERSISTENT_BIT | GL_MAP_WRITE_BIT | GL_MAP_FLUSH_EXPLICIT_BIT);
 
+    glBindBuffer(GL_UNIFORM_BUFFER, buffers[NonCoherentMapFlushUnsynchronised]);
+    glBufferStorage(GL_UNIFORM_BUFFER, sizeof(Vec4f), &red, GL_MAP_WRITE_BIT | GL_MAP_PERSISTENT_BIT);
+
+    ptrs[NonCoherentMapFlushUnsynchronised] =
+        (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f),
+                                  GL_MAP_PERSISTENT_BIT | GL_MAP_WRITE_BIT |
+                                      GL_MAP_FLUSH_EXPLICIT_BIT | GL_MAP_UNSYNCHRONIZED_BIT);
+
     glBindBuffer(GL_UNIFORM_BUFFER, buffers[OffsetMapWrite]);
     glBufferStorage(GL_UNIFORM_BUFFER, sizeof(Vec4f), &cyan,
                     GL_MAP_WRITE_BIT | GL_DYNAMIC_STORAGE_BIT);
@@ -235,7 +275,6 @@ void main()
     glBufferStorage(GL_UNIFORM_BUFFER, sizeof(Vec4f), &red, 0);
 
     GLuint program = MakeProgram(vertex, pixel);
-    glObjectLabel(GL_PROGRAM, program, -1, "Full program");
 
     while(Running())
     {
@@ -278,15 +317,19 @@ void main()
 
       ptr = (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f),
                                       GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT);
-      memcpy(ptr, &green, sizeof(Vec4f));
+      if(ptr)
+        memcpy(ptr, &green, sizeof(Vec4f));
       glUnmapBuffer(GL_UNIFORM_BUFFER);
 
       glBindBuffer(GL_UNIFORM_BUFFER, buffers[CleanBufferMapWriteNonInvalidate]);
       glBufferData(GL_UNIFORM_BUFFER, sizeof(Vec4f), &red, GL_DYNAMIC_DRAW);
 
       ptr = (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f), GL_MAP_WRITE_BIT);
-      ptr->x = 0.0f;
-      ptr->y = 1.0f;
+      if(ptr)
+      {
+        ptr->x = 0.0f;
+        ptr->y = 1.0f;
+      }
       glUnmapBuffer(GL_UNIFORM_BUFFER);
 
       glBindBuffer(GL_UNIFORM_BUFFER, buffers[DirtyBufferMapWriteInvalidate]);
@@ -294,15 +337,19 @@ void main()
 
       ptr = (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f),
                                       GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT);
-      memcpy(ptr, &green, sizeof(Vec4f));
+      if(ptr)
+        memcpy(ptr, &green, sizeof(Vec4f));
       glUnmapBuffer(GL_UNIFORM_BUFFER);
 
       glBindBuffer(GL_UNIFORM_BUFFER, buffers[DirtyBufferMapWriteNonInvalidate]);
       glBufferSubData(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f), &red);
 
       ptr = (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f), GL_MAP_WRITE_BIT);
-      ptr->x = 0.0f;
-      ptr->y = 1.0f;
+      if(ptr)
+      {
+        ptr->x = 0.0f;
+        ptr->y = 1.0f;
+      }
       glUnmapBuffer(GL_UNIFORM_BUFFER);
 
       glBindBuffer(GL_UNIFORM_BUFFER, buffers[CleanBufferMapFlushExplicit]);
@@ -310,31 +357,53 @@ void main()
 
       ptr = (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f),
                                       GL_MAP_WRITE_BIT | GL_MAP_FLUSH_EXPLICIT_BIT);
-      ptr->x = 0.0f;
-      ptr->y = 1.0f;
+      if(ptr)
+      {
+        ptr->x = 0.0f;
+        ptr->y = 1.0f;
+      }
       glFlushMappedBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(float) * 2);
       glUnmapBuffer(GL_UNIFORM_BUFFER);
 
       glBindBuffer(GL_UNIFORM_BUFFER, buffers[DirtyBufferMapFlushExplicit]);
       ptr = (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f),
                                       GL_MAP_WRITE_BIT | GL_MAP_FLUSH_EXPLICIT_BIT);
-      ptr->x = 0.0f;
-      ptr->y = 1.0f;
+      if(ptr)
+      {
+        ptr->x = 0.0f;
+        ptr->y = 1.0f;
+      }
       glFlushMappedBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(float) * 2);
       glUnmapBuffer(GL_UNIFORM_BUFFER);
 
-      memcpy(ptrs[CoherentMapWrite], &red, sizeof(Vec4f));
+      if(ptrs[CoherentMapWrite])
+        memcpy(ptrs[CoherentMapWrite], &red, sizeof(Vec4f));
+      if(ptrs[CoherentMapWriteInvalidateRange])
+        memcpy(ptrs[CoherentMapWriteInvalidateRange], &red, sizeof(Vec4f));
+      if(ptrs[CoherentMapWriteInvalidateBuffer])
+        memcpy(ptrs[CoherentMapWriteInvalidateBuffer], &red, sizeof(Vec4f));
+      if(ptrs[CoherentMapWriteUnsynchronised])
+        memcpy(ptrs[CoherentMapWriteUnsynchronised], &red, sizeof(Vec4f));
 
       glBindBuffer(GL_UNIFORM_BUFFER, buffers[NonCoherentMapFlush]);
-      memcpy(ptrs[NonCoherentMapFlush], &red, sizeof(Vec4f));
+      if(ptrs[NonCoherentMapFlush])
+        memcpy(ptrs[NonCoherentMapFlush], &red, sizeof(Vec4f));
       glFlushMappedBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(float) * 4);
 
+      glBindBuffer(GL_UNIFORM_BUFFER, buffers[NonCoherentMapFlushUnsynchronised]);
+      if(ptrs[NonCoherentMapFlushUnsynchronised])
+        memcpy(ptrs[NonCoherentMapFlushUnsynchronised], &red, sizeof(Vec4f));
+      glFlushMappedBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(float) * 4);
+
+      glMemoryBarrier(GL_BUFFER_UPDATE_BARRIER_BIT);
+
       glBindBuffer(GL_UNIFORM_BUFFER, buffers[OffsetMapWrite]);
       glBufferSubData(GL_UNIFORM_BUFFER, 0, sizeof(Vec4f), &cyan);
 
       ptr = (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, sizeof(float) * 2, sizeof(float),
                                       GL_MAP_WRITE_BIT);
-      ptr->x = 0.0f;
+      if(ptr)
+        ptr->x = 0.0f;
       glUnmapBuffer(GL_UNIFORM_BUFFER);
 
       glBindBuffer(GL_UNIFORM_BUFFER, buffers[OffsetMapFlush]);
@@ -342,7 +411,8 @@ void main()
 
       ptr = (Vec4f *)glMapBufferRange(GL_UNIFORM_BUFFER, sizeof(float) * 2, sizeof(float),
                                       GL_MAP_WRITE_BIT | GL_MAP_FLUSH_EXPLICIT_BIT);
-      ptr->x = 0.0f;
+      if(ptr)
+        ptr->x = 0.0f;
       glFlushMappedBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(float));
       glUnmapBuffer(GL_UNIFORM_BUFFER);
 
@@ -359,16 +429,33 @@ void main()
           glDebugMessageInsert(GL_DEBUG_SOURCE_APPLICATION, GL_DEBUG_TYPE_MARKER, 0,
                                GL_DEBUG_SEVERITY_HIGH, -1, TestNames[buf]);
 
-          if(buf == CoherentMapWrite)
-            memcpy(ptrs[CoherentMapWrite], &green, sizeof(Vec4f));
+          if(buf == CoherentMapWrite || buf == CoherentMapWriteInvalidateRange ||
+             buf == CoherentMapWriteInvalidateBuffer || buf == CoherentMapWriteUnsynchronised)
+          {
+            if(ptrs[buf])
+              memcpy(ptrs[buf], &green, sizeof(Vec4f));
+
+            if(buf == CoherentMapWriteUnsynchronised)
+              glMemoryBarrier(GL_BUFFER_UPDATE_BARRIER_BIT);
+          }
 
           if(buf == NonCoherentMapFlush)
           {
             glBindBuffer(GL_UNIFORM_BUFFER, buffers[NonCoherentMapFlush]);
-            memcpy(ptrs[NonCoherentMapFlush], &green, sizeof(Vec4f));
+            if(ptrs[NonCoherentMapFlush])
+              memcpy(ptrs[NonCoherentMapFlush], &green, sizeof(Vec4f));
             glFlushMappedBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(float) * 4);
           }
 
+          if(buf == NonCoherentMapFlushUnsynchronised)
+          {
+            glBindBuffer(GL_UNIFORM_BUFFER, buffers[NonCoherentMapFlushUnsynchronised]);
+            if(ptrs[NonCoherentMapFlushUnsynchronised])
+              memcpy(ptrs[NonCoherentMapFlushUnsynchronised], &green, sizeof(Vec4f));
+            glFlushMappedBufferRange(GL_UNIFORM_BUFFER, 0, sizeof(float) * 4);
+            glMemoryBarrier(GL_BUFFER_UPDATE_BARRIER_BIT);
+          }
+
           if(buf == SingleMapBufferReadback)
           {
             glBindBuffer(GL_COPY_READ_BUFFER, buffers[buf]);
@@ -435,4 +522,4 @@ void main()
   }
 };
 
-REGISTER_TEST(GL_Buffer_Updates);
\ No newline at end of file
+REGISTER_TEST();
