@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "gl_test.h"
 
-struct GL_CBuffer_Zoo : OpenGLGraphicsTest
+RD_TEST(GL_CBuffer_Zoo, OpenGLGraphicsTest)
 {
   static constexpr const char *Description =
       "Tests every kind of constant that can be in a cbuffer to make sure it's decoded "
@@ -71,6 +71,19 @@ struct vec3_1 { vec3 a; float b; };
 
 struct nested { vec3_1 a; vec4 b[4]; vec3_1 c[4]; };
 
+struct float2_struct { float x; float y; };
+
+struct nested_with_padding
+{
+  float a;                              // 0, <1, 2, 3>
+  vec4 b;                               // {4, 5, 6, 7}
+  float c;                              // 8, <9, 10, 11>
+  vec3 d[4];                            // [0]: {12, 13, 14}, <15>
+                                        // [1]: {16, 17, 18}, <19>
+                                        // [2]: {20, 21, 22}, <23>
+                                        // [3]: {24, 25, 26}, <27>
+};
+
 layout(binding = 0, std140) uniform constsbuf
 {
   // dummy* entries are just to 'reset' packing to avoid pollution between tests
@@ -257,14 +270,26 @@ layout(binding = 0, std140) uniform constsbuf
                                           //         <435, 439>
                                           // }
 
-  vec4 test;                              // {440, 441, 442, 443}
+  nested_with_padding ak[2];              // 440 - 467, 468 - 495
+
+  vec4 dummy12;                           // forces no trailing overlap with ak
+
+  float al;                               // {500}, <501, 502, 503>
+
+  // struct is always float4 aligned, can't be packed with al
+  float2_struct am;                       // {504, 505}, <506, 507>
+
+  // struct doesn't allow trailing things into padding
+  float an;                               // {506}
+
+  vec4 test;                              // {512, 513, 514, 515}
 
   // because GL has worse handling of multidimensional arrays than other APIs, we add an extra test
   // here with more than 2 dimensions
 
-  vec4 multiarray2[4][3][2];              // [0][0][0] = {444, 445, 446, 447}
-                                          // [0][0][1] = {448, 449, 450, 451}
-                                          // [0][1][0] = {452, ..., ..., ...}
+  vec4 multiarray2[4][3][2];              // [0][0][0] = {516, 517, 518, ...}
+                                          // [0][0][1] = {..., ..., ..., ...}
+                                          // [0][1][0] = {..., ..., ..., ...}
                                           // [0][1][1] = {..., ..., ..., ...}
                                           // [0][2][0] = {..., ..., ..., ...}
                                           // [0][2][1] = {..., ..., ..., ...}
@@ -303,7 +328,7 @@ void main()
   blah += n[0] + o[0] + p.x;
   blah += q[0].x + r[0].x + s[0].x + t[0].x + u[0].x + v[0].x + w[0].x + x[0].x + y[0].x + z;
   blah += multiarray[0][0].x + ac[0][0] + ad[0][0] + ae[0][0][0] + af[0][0][0];
-  blah += A.z + B.x + C.y + D[0][1] + E[2] + F[1][0][1].y;
+  blah += A.z + B.x + C.y + D[0][1] + E[2] + F[1][0][0].y + F[1][0][1].y;
   blah += G[0].a.b + G[1].a.b + G[1].b[3].w + G[1].c[3].a.y;
   blah *= vertIn.uv.z;
   Color = blah + test + vec4(0.1f, 0.0f, 0.0f, 0.0f);
@@ -311,10 +336,10 @@ void main()
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     GLuint vao = MakeVAO();
@@ -334,11 +359,10 @@ void main()
     glEnableVertexAttribArray(2);
 
     GLuint program = MakeProgram(common + vertex, common + pixel);
-    glObjectLabel(GL_PROGRAM, program, -1, "Full program");
 
-    Vec4f cbufferdata[684];
+    Vec4f cbufferdata[1024];
 
-    for(int i = 0; i < 684; i++)
+    for(int i = 0; i < 1024; i++)
       cbufferdata[i] = Vec4f(float(i * 4 + 0), float(i * 4 + 1), float(i * 4 + 2), float(i * 4 + 3));
 
     GLuint cb = MakeBuffer();
@@ -561,4 +585,4 @@ void main()
   }
 };
 
-REGISTER_TEST(GL_CBuffer_Zoo);
\ No newline at end of file
+REGISTER_TEST();
