@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "gl_test.h"
 
-struct DepthStencil_FBO : OpenGLGraphicsTest
+RD_TEST(GL_DepthStencil_FBO, OpenGLGraphicsTest)
 {
   static constexpr const char *Description =
       "Creates a depth-stencil FBO and writes both depth and stencil to it";
@@ -101,10 +101,10 @@ void main()
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     GLuint vao = MakeVAO();
@@ -196,4 +196,4 @@ void main()
   }
 };
 
-REGISTER_TEST(DepthStencil_FBO);
\ No newline at end of file
+REGISTER_TEST();
