@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "gl_test.h"
 
-struct Runtime_Bind_Prog_To_Pipe : OpenGLGraphicsTest
+RD_TEST(GL_Runtime_Bind_Prog_To_Pipe, OpenGLGraphicsTest)
 {
   static constexpr const char *Description =
       "Creates a single program pipeline and binds different programs to it mid-frame";
@@ -84,10 +84,10 @@ void main()
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     GLuint vao = MakeVAO();
@@ -141,4 +141,4 @@ void main()
   }
 };
 
-REGISTER_TEST(Runtime_Bind_Prog_To_Pipe);
\ No newline at end of file
+REGISTER_TEST();
