@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "gl_test.h"
 
-struct SPIRV_Shader : OpenGLGraphicsTest
+RD_TEST(GL_SPIRV_Shader, OpenGLGraphicsTest)
 {
   static constexpr const char *Description = "Draws using a SPIR-V shader pipeline.";
 
@@ -41,7 +41,7 @@ layout(location = 2) out vec2 oUV;
 
 layout(location = 2) uniform vec4 offset;
 layout(location = 8) uniform vec4 scale;
-layout(location = 13) uniform vec4 UVscroll;
+layout(location = 13) uniform vec2 UVscroll;
 
 void main()
 {
@@ -71,21 +71,27 @@ void main()
 }
 
 )EOSHADER";
+  void Prepare(int argc, char **argv)
+  {
+    OpenGLGraphicsTest::Prepare(argc, argv);
 
-  int main(int argc, char **argv)
+    if(!Avail.empty())
+      return;
+
+    if(!SpvCompilationSupported())
+      Avail = InternalSpvCompiler() ? "Internal SPIR-V compiler did not initialise"
+                                    : "Couldn't find 'glslc' or 'glslangValidator' in PATH - "
+                                      "required for SPIR-V compilation";
+  }
+
+  int main()
   {
     debugDevice = true;
 
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
-    if(!SpvCompilationSupported())
-    {
-      TEST_ERROR("Can't run SPIR-V test without glslc in PATH");
-      return 2;
-    }
-
     GLuint vao = MakeVAO();
     glBindVertexArray(vao);
 
@@ -134,7 +140,6 @@ void main()
     glBindBufferBase(GL_UNIFORM_BUFFER, 1, fsbuf);
 
     GLuint glslprogram = MakeProgram(vertex, pixel);
-    glObjectLabel(GL_PROGRAM, glslprogram, -1, "Full program");
 
     GLuint spirvprogram = MakeProgram();
 
@@ -222,6 +227,21 @@ void main()
       float col[] = {0.4f, 0.5f, 0.6f, 1.0f};
       glClearBufferfv(GL_COLOR, 0, col);
 
+      GLsizei w = GLsizei(screenWidth) >> 1;
+      GLsizei h = GLsizei(screenHeight) >> 1;
+
+      glBindVertexArray(vao);
+
+      glViewport(0, 0, w, h);
+
+      glUseProgram(glslprogram);
+      glDrawArrays(GL_TRIANGLES, 0, 3);
+
+      glViewport(w, 0, w, h);
+
+      glUseProgram(spirvprogram);
+      glDrawArrays(GL_TRIANGLES, 0, 3);
+
       vsdata.UVscroll.x += 0.01f;
       vsdata.UVscroll.y += 0.02f;
 
@@ -236,20 +256,18 @@ void main()
         // UVscroll location 13
         glUniform4fv(2, 1, &vsdata.offset.x);
         glUniform4fv(8, 1, &vsdata.scale.x);
-        glUniform4fv(13, 1, &vsdata.UVscroll.x);
+        glUniform2fv(13, 1, &vsdata.UVscroll.x);
 
         // tint location 7
         glUniform4fv(7, 1, &fsdata.x);
       }
 
-      glBindVertexArray(vao);
-
-      glViewport(0, 0, GLsizei(screenWidth) >> 1, GLsizei(screenHeight));
+      glViewport(0, h, w, h);
 
       glUseProgram(glslprogram);
       glDrawArrays(GL_TRIANGLES, 0, 3);
 
-      glViewport(GLsizei(screenWidth) >> 1, 0, GLsizei(screenWidth) >> 1, GLsizei(screenHeight));
+      glViewport(w, h, w, h);
 
       glUseProgram(spirvprogram);
       glDrawArrays(GL_TRIANGLES, 0, 3);
@@ -261,4 +279,4 @@ void main()
   }
 };
 
-REGISTER_TEST(SPIRV_Shader);
\ No newline at end of file
+REGISTER_TEST();
