@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2015-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -34,9 +34,9 @@ struct OpenGLGraphicsTest : public GraphicsTest
 {
   static const TestAPI API = TestAPI::OpenGL;
 
-  bool Init(int argc, char **argv);
+  void Prepare(int argc, char **argv);
+  bool Init();
   void Shutdown();
-  bool IsSupported();
   GraphicsWindow *MakeWindow(int width, int height, const char *title);
   void *MakeContext(GraphicsWindow *win, void *share);
   void DestroyContext(void *ctx);
@@ -44,7 +44,8 @@ struct OpenGLGraphicsTest : public GraphicsTest
 
   void PostInit();
 
-  GLuint MakeProgram(std::string vertSrc, std::string fragSrc, bool sep = false);
+  GLuint MakeProgram(std::string vertSrc, std::string fragSrc, std::string geomSrc = "");
+  GLuint MakeProgram(std::string compSrc);
   GLuint MakeProgram();
   GLuint MakePipeline();
   GLuint MakeBuffer();
@@ -52,6 +53,10 @@ struct OpenGLGraphicsTest : public GraphicsTest
   GLuint MakeVAO();
   GLuint MakeFBO();
 
+  void pushMarker(const std::string &name);
+  void setMarker(const std::string &name);
+  void popMarker();
+
   bool Running();
   void Present(GraphicsWindow *window);
   void Present() { Present(mainWindow); }
@@ -68,4 +73,4 @@ struct OpenGLGraphicsTest : public GraphicsTest
   {
     std::vector<GLuint> bufs, texs, progs, pipes, vaos, fbos;
   } managedResources;
-};
\ No newline at end of file
+};
