@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2018-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -29,10 +29,28 @@
 
 #include "../linux/linux_window.h"
 
-bool OpenGLGraphicsTest::Init(int argc, char **argv)
+void OpenGLGraphicsTest::Prepare(int argc, char **argv)
 {
-  // parse parameters here to override parameters
-  GraphicsTest::Init(argc, argv);
+  GraphicsTest::Prepare(argc, argv);
+
+  static bool prepared = false;
+  static void *libGL = NULL;
+
+  if(!prepared)
+  {
+    prepared = true;
+
+    libGL = dlopen("libGL.so", RTLD_GLOBAL | RTLD_NOW);
+  }
+
+  if(!libGL)
+    Avail = "libGL.so is not available";
+}
+
+bool OpenGLGraphicsTest::Init()
+{
+  if(!GraphicsTest::Init())
+    return false;
 
   X11Window::Init();
 
@@ -60,28 +78,11 @@ bool OpenGLGraphicsTest::Init(int argc, char **argv)
     return false;
   }
 
-  glEnable(GL_FRAMEBUFFER_SRGB);
+  PostInit();
 
   return true;
 }
 
-bool OpenGLGraphicsTest::IsSupported()
-{
-  static bool checked = false, result = false;
-
-  if(checked)
-    return result;
-
-  checked = true;
-
-  void *GL = dlopen("libGL.so", RTLD_GLOBAL | RTLD_NOW);
-
-  if(GL)
-    result = true;
-
-  return result;
-}
-
 GraphicsWindow *OpenGLGraphicsTest::MakeWindow(int width, int height, const char *title)
 {
   return new X11Window(width, height, title);
@@ -133,6 +134,8 @@ void *OpenGLGraphicsTest::MakeContext(GraphicsWindow *win, void *share)
                              8,
                              GLX_BLUE_SIZE,
                              8,
+                             GLX_ALPHA_SIZE,
+                             8,
                              GLX_DOUBLEBUFFER,
                              True,
                              GLX_FRAMEBUFFER_SRGB_CAPABLE_ARB,
@@ -212,4 +215,4 @@ void OpenGLGraphicsTest::Present(GraphicsWindow *window)
   X11Window *x11win = (X11Window *)window;
 
   glXSwapBuffers(x11win->xlib.display, x11win->xlib.window);
-}
\ No newline at end of file
+}
