@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2018-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -32,24 +32,41 @@ typedef BOOL(WINAPI *PFN_wglMakeCurrent)(HDC, HGLRC);
 typedef HGLRC(WINAPI *PFN_wglCreateContext)(HDC);
 typedef BOOL(WINAPI *PFN_wglDeleteContext)(HGLRC);
 
-bool OpenGLGraphicsTest::Init(int argc, char **argv)
+namespace
 {
-  // parse parameters here to override parameters
-  GraphicsTest::Init(argc, argv);
+PFN_wglMakeCurrent makeCurrent = NULL;
+PFN_wglCreateContext createContext = NULL;
+PFN_wglDeleteContext deleteContext = NULL;
+};
 
-  HMODULE opengl = LoadLibraryA("opengl32.dll");
+void OpenGLGraphicsTest::Prepare(int argc, char **argv)
+{
+  GraphicsTest::Prepare(argc, argv);
 
-  if(!opengl)
+  static bool prepared = false;
+
+  if(!prepared)
   {
-    TEST_ERROR("Couldn't load opengl32");
-    return false;
+    prepared = true;
+
+    HMODULE opengl = LoadLibraryA("opengl32.dll");
+
+    if(opengl)
+    {
+      makeCurrent = (PFN_wglMakeCurrent)GetProcAddress(opengl, "wglMakeCurrent");
+      createContext = (PFN_wglCreateContext)GetProcAddress(opengl, "wglCreateContext");
+      deleteContext = (PFN_wglDeleteContext)GetProcAddress(opengl, "wglDeleteContext");
+    }
   }
 
-  PFN_wglMakeCurrent makeCurrent = (PFN_wglMakeCurrent)GetProcAddress(opengl, "wglMakeCurrent");
-  PFN_wglCreateContext makeContext =
-      (PFN_wglCreateContext)GetProcAddress(opengl, "wglCreateContext");
-  PFN_wglDeleteContext deleteContext =
-      (PFN_wglDeleteContext)GetProcAddress(opengl, "wglDeleteContext");
+  if(!makeCurrent)
+    Avail = "opengl32.dll is not available";
+}
+
+bool OpenGLGraphicsTest::Init()
+{
+  if(!GraphicsTest::Init())
+    return false;
 
   mainWindow = new Win32Window(screenWidth, screenHeight, screenTitle);
 
@@ -72,11 +89,12 @@ bool OpenGLGraphicsTest::Init(int argc, char **argv)
   pfd.cColorBits = 24;
   pfd.cDepthBits = 0;
   pfd.cStencilBits = 0;
+  pfd.cAlphaBits = 8;
 
   int pf = ::ChoosePixelFormat(dc, &pfd);
   ::SetPixelFormat(dc, pf, &pfd);
 
-  HGLRC rc = makeContext(dc);
+  HGLRC rc = createContext(dc);
 
   makeCurrent(dc, rc);
 
@@ -105,32 +123,11 @@ bool OpenGLGraphicsTest::Init(int argc, char **argv)
     return false;
   }
 
-  glEnable(GL_FRAMEBUFFER_SRGB);
+  PostInit();
 
   return true;
 }
 
-bool OpenGLGraphicsTest::IsSupported()
-{
-  static bool checked = false, result = false;
-
-  if(checked)
-    return result;
-
-  checked = true;
-
-  HMODULE opengl = LoadLibraryA("opengl32.dll");
-
-  if(opengl)
-  {
-    result = true;
-
-    FreeLibrary(opengl);
-  }
-
-  return result;
-}
-
 GraphicsWindow *OpenGLGraphicsTest::MakeWindow(int width, int height, const char *title)
 {
   if(!GLAD_WGL_ARB_pixel_format)
@@ -255,9 +252,6 @@ void OpenGLGraphicsTest::DestroyContext(void *ctx)
   if(ctx == NULL)
     return;
 
-  PFN_wglDeleteContext deleteContext =
-      (PFN_wglDeleteContext)GetProcAddress(GetModuleHandleA("opengl32.dll"), "wglDeleteContext");
-
   deleteContext((HGLRC)ctx);
 }
 
@@ -267,9 +261,6 @@ void OpenGLGraphicsTest::ActivateContext(GraphicsWindow *win, void *ctx)
 
   HDC dc = GetDC(win32win->wnd);
 
-  PFN_wglMakeCurrent makeCurrent =
-      (PFN_wglMakeCurrent)GetProcAddress(GetModuleHandleA("opengl32.dll"), "wglMakeCurrent");
-
   makeCurrent(dc, (HGLRC)ctx);
 
   ReleaseDC(win32win->wnd, dc);
@@ -284,4 +275,4 @@ void OpenGLGraphicsTest::Present(GraphicsWindow *window)
   SwapBuffers(dc);
 
   ReleaseDC(win32win->wnd, dc);
-}
\ No newline at end of file
+}
