@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2015-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "gl_test.h"
 
-struct VAO_0 : OpenGLGraphicsTest
+RD_TEST(GL_VAO_0, OpenGLGraphicsTest)
 {
   static constexpr const char *Description = "Uses VAO 0 (i.e. never binds a VAO)";
 
@@ -72,12 +72,12 @@ void main()
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  int main()
   {
     coreProfile = false;
 
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     uint32_t idxs[3] = {0, 1, 2};
@@ -101,13 +101,19 @@ void main()
     glBufferStorage(GL_ELEMENT_ARRAY_BUFFER, sizeof(uint32_t) * 3, idxs, 0);
 
     GLuint program = MakeProgram(common + vertex, common + pixel);
-    glObjectLabel(GL_PROGRAM, program, -1, "Full program");
 
     while(Running())
     {
       float col[] = {0.4f, 0.5f, 0.6f, 1.0f};
       glClearBufferfv(GL_COLOR, 0, col);
 
+      // try to delete VAO 0. Should do nothing
+      GLuint zero = 0;
+      glDeleteVertexArrays(1, &zero);
+
+      // same with FBO 0
+      glDeleteFramebuffers(1, &zero);
+
       glUseProgram(program);
 
       // use both buffers
@@ -151,4 +157,4 @@ void main()
   }
 };
 
-REGISTER_TEST(VAO_0);
\ No newline at end of file
+REGISTER_TEST();
