@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2016-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -183,11 +183,31 @@ void LoadXPM(const char **XPM, Texture &tex)
 #define HAVE_SHADERC 0
 #endif
 
+// on VS2015 define __std_reverse_trivially_swappable_8 so we can link against shaderc_combined.lib
+// built on newer VS
+#if defined(_MSC_VER) && _MSC_VER <= 1900
+extern "C" __declspec(noalias) void __cdecl __std_reverse_trivially_swappable_8(void *a,
+                                                                                void *b) noexcept
+{
+  unsigned long long temp;
+  static_assert(sizeof(temp) == 8, "wrong basic type");
+  memcpy(&temp, a, sizeof(temp));
+  memcpy(a, b, sizeof(temp));
+  memcpy(b, &temp, sizeof(temp));
+}
+#endif
+
 // this define toggles on/off using the linked shaderc. This can be useful if e.g. on windows the
 // shaderc in VULKAN_SDK is broken.
 #define USE_LINKED_SHADERC (1 && HAVE_SHADERC)
 
 static shaderc_compiler_t shaderc = NULL;
+static std::string externalCompiler;
+
+bool InternalSpvCompiler()
+{
+  return bool(USE_LINKED_SHADERC);
+}
 
 bool SpvCompilationSupported()
 {
@@ -201,20 +221,30 @@ bool SpvCompilationSupported()
   if(shaderc)
     return true;
 
-  FILE *pipe = popen("glslc" EXECUTABLE_SUFFIX " --help", "r");
+  for(std::string compiler : {"glslc", "glslangValidator"})
+  {
+    FILE *pipe = popen((compiler + EXECUTABLE_SUFFIX " --version 2>&1").c_str(), "r");
 
-  if(!pipe)
-    return false;
+    if(!pipe)
+      continue;
 
-  msleep(250);
+    msleep(500);
 
-  int code = pclose(pipe);
+    int code = pclose(pipe);
 
-  return WEXITSTATUS(code) == 0;
+    if(WEXITSTATUS(code) == 0)
+    {
+      externalCompiler = compiler;
+      return true;
+    }
+  }
+
+  return false;
 }
 
 std::vector<uint32_t> CompileShaderToSpv(const std::string &source_text, SPIRVTarget target,
-                                         ShaderLang lang, ShaderStage stage, const char *entry_point)
+                                         ShaderLang lang, ShaderStage stage, const char *entry_point,
+                                         const std::map<std::string, std::string> &macros)
 {
   std::vector<uint32_t> ret;
 
@@ -245,6 +275,13 @@ std::vector<uint32_t> CompileShaderToSpv(const std::string &source_text, SPIRVTa
 
     if(target == SPIRVTarget::opengl)
       shaderc_compile_options_set_target_env(opts, shaderc_target_env_opengl, 0);
+    else if(target == SPIRVTarget::vulkan11)
+      shaderc_compile_options_set_target_env(opts, shaderc_target_env_vulkan,
+                                             shaderc_env_version_vulkan_1_1);
+
+    for(auto it : macros)
+      shaderc_compile_options_add_macro_definition(opts, it.first.c_str(), it.first.length(),
+                                                   it.second.c_str(), it.second.length());
 
     shaderc_compilation_result_t res = shaderc_compile_into_spv(
         shaderc, source_text.c_str(), source_text.size(), shader_kind, "inshader", entry_point, opts);
@@ -278,35 +315,81 @@ std::vector<uint32_t> CompileShaderToSpv(const std::string &source_text, SPIRVTa
 #endif
   }
 
-  std::string command_line = "glslc" EXECUTABLE_SUFFIX " -g -O0";
-
-  command_line += " -fentry-point=";
-  command_line += entry_point;
-
-  if(lang == ShaderLang::glsl)
-    command_line += " -x glsl";
-  else if(lang == ShaderLang::hlsl)
-    command_line += " -x hlsl";
-
-  switch(stage)
-  {
-    case ShaderStage::vert: command_line += " -fshader-stage=vert"; break;
-    case ShaderStage::frag: command_line += " -fshader-stage=frag"; break;
-    case ShaderStage::tesscontrol: command_line += " -fshader-stage=tesscontrol"; break;
-    case ShaderStage::tesseval: command_line += " -fshader-stage=tesseval"; break;
-    case ShaderStage::geom: command_line += " -fshader-stage=geom"; break;
-    case ShaderStage::comp: command_line += " -fshader-stage=comp"; break;
-  }
+  std::string command_line;
 
   char infile[MAX_PATH] = {};
   char outfile[MAX_PATH] = {};
-  tmpnam(infile);
-  tmpnam(outfile);
+  get_tmpnam(infile);
+  get_tmpnam(outfile);
 
-  command_line += " -o ";
-  command_line += outfile;
-  command_line += " ";
-  command_line += infile;
+  if(externalCompiler == "glslc")
+  {
+    command_line = "glslc" EXECUTABLE_SUFFIX " -g -O0";
+    command_line += " -fentry-point=";
+    command_line += entry_point;
+
+    if(lang == ShaderLang::glsl)
+      command_line += " -x glsl";
+    else if(lang == ShaderLang::hlsl)
+      command_line += " -x hlsl";
+
+    for(auto it : macros)
+      command_line += " -D" + it.first + "=" + it.second;
+
+    switch(stage)
+    {
+      case ShaderStage::vert: command_line += " -fshader-stage=vert"; break;
+      case ShaderStage::frag: command_line += " -fshader-stage=frag"; break;
+      case ShaderStage::tesscontrol: command_line += " -fshader-stage=tesscontrol"; break;
+      case ShaderStage::tesseval: command_line += " -fshader-stage=tesseval"; break;
+      case ShaderStage::geom: command_line += " -fshader-stage=geom"; break;
+      case ShaderStage::comp: command_line += " -fshader-stage=comp"; break;
+    }
+
+    if(target == SPIRVTarget::opengl)
+      command_line += " --target-env=opengl";
+    else if(target == SPIRVTarget::vulkan11)
+      command_line += " --target-env=vulkan1.1";
+
+    command_line += " -o ";
+    command_line += outfile;
+    command_line += " ";
+    command_line += infile;
+  }
+  else if(externalCompiler == "glslangValidator")
+  {
+    command_line = "glslangValidator" EXECUTABLE_SUFFIX " -g ";
+    command_line += " --entry-point ";
+    command_line += entry_point;
+
+    if(lang == ShaderLang::hlsl)
+      command_line += " -D";
+
+    for(auto it : macros)
+      command_line += " -D" + it.first + "=" + it.second;
+
+    switch(stage)
+    {
+      case ShaderStage::vert: command_line += " -S vert"; break;
+      case ShaderStage::frag: command_line += " -S frag"; break;
+      case ShaderStage::tesscontrol: command_line += " -S tesscontrol"; break;
+      case ShaderStage::tesseval: command_line += " -S tesseval"; break;
+      case ShaderStage::geom: command_line += " -S geom"; break;
+      case ShaderStage::comp: command_line += " -S comp"; break;
+    }
+
+    if(target == SPIRVTarget::opengl)
+      command_line += " -G --target-env opengl";
+    else if(target == SPIRVTarget::vulkan11)
+      command_line += " -V --target-env vulkan1.1";
+    else if(target == SPIRVTarget::vulkan)
+      command_line += " -V --target-env vulkan1.0";
+
+    command_line += " -o ";
+    command_line += outfile;
+    command_line += " ";
+    command_line += infile;
+  }
 
   FILE *f = fopen(infile, "wb");
   if(f)
@@ -319,7 +402,7 @@ std::vector<uint32_t> CompileShaderToSpv(const std::string &source_text, SPIRVTa
 
   if(!pipe)
   {
-    TEST_ERROR("Couldn't run shaderc to compile shaders.");
+    TEST_ERROR("Couldn't run %s to compile shaders.", externalCompiler.c_str());
     return ret;
   }
 
@@ -329,7 +412,7 @@ std::vector<uint32_t> CompileShaderToSpv(const std::string &source_text, SPIRVTa
 
   if(code != 0)
   {
-    TEST_ERROR("Invoking shaderc failed: %s.", command_line.c_str());
+    TEST_ERROR("Invoking %s failed: %s.", externalCompiler.c_str(), command_line.c_str());
     return ret;
   }
 
@@ -349,9 +432,21 @@ std::vector<uint32_t> CompileShaderToSpv(const std::string &source_text, SPIRVTa
   return ret;
 }
 
-bool GraphicsTest::Init(int argc, char **argv)
+int GraphicsTest::maxFrameCount = -1;
+std::string GraphicsTest::dataRoot;
+int GraphicsTest::screenWidth = 400;
+int GraphicsTest::screenHeight = 300;
+bool GraphicsTest::debugDevice = false;
+
+void GraphicsTest::Prepare(int argc, char **argv)
 {
-  srand(0U);
+  static bool prepared = false;
+
+  // nothing to do per-test if we've already prepared
+  if(prepared)
+    return;
+
+  prepared = true;
 
   dataRoot = GetEnvVar("RENDERDOC_DEMOS_DATA");
 
@@ -372,6 +467,26 @@ bool GraphicsTest::Init(int argc, char **argv)
       maxFrameCount = atoi(argv[i + 1]);
     }
 
+    if(i + 1 < argc && (!strcmp(argv[i], "--width") || !strcmp(argv[i], "-w")))
+    {
+      screenWidth = atoi(argv[i + 1]);
+
+      if(screenWidth < 1)
+        screenWidth = 1;
+      if(screenWidth > 7680)
+        screenWidth = 7680;
+    }
+
+    if(i + 1 < argc && (!strcmp(argv[i], "--height") || !strcmp(argv[i], "-h")))
+    {
+      screenHeight = atoi(argv[i + 1]);
+
+      if(screenHeight < 1)
+        screenHeight = 1;
+      if(screenHeight > 4320)
+        screenHeight = 4320;
+    }
+
     if(i + 1 < argc && !strcmp(argv[i], "--data"))
     {
       dataRoot = argv[i + 1];
@@ -380,29 +495,32 @@ bool GraphicsTest::Init(int argc, char **argv)
       dataRoot += "/";
     }
   }
+}
+
+bool GraphicsTest::Init()
+{
+  srand(0U);
+
+  pRENDERDOC_GetAPI RENDERDOC_GetAPI = NULL;
 
 #if defined(WIN32)
-
   HMODULE mod = GetModuleHandleA("renderdoc.dll");
   if(mod)
-  {
-    pRENDERDOC_GetAPI RENDERDOC_GetAPI = (pRENDERDOC_GetAPI)GetProcAddress(mod, "RENDERDOC_GetAPI");
+    RENDERDOC_GetAPI = (pRENDERDOC_GetAPI)GetProcAddress(mod, "RENDERDOC_GetAPI");
+#else
+  void *mod = dlopen("librenderdoc.so", RTLD_NOW | RTLD_NOLOAD);
+  if(mod)
+    RENDERDOC_GetAPI = (pRENDERDOC_GetAPI)dlsym(mod, "RENDERDOC_GetAPI");
+#endif
 
-    int ret = RENDERDOC_GetAPI(eRENDERDOC_API_Version_1_4_0, (void **)&rdoc);
+  if(RENDERDOC_GetAPI)
+  {
+    int ret = RENDERDOC_GetAPI(eRENDERDOC_API_Version_1_0_0, (void **)&rdoc);
 
     if(ret != 1)
       rdoc = NULL;
-
-    ret = RENDERDOC_GetAPI(eRENDERDOC_API_Version_1_0_0, (void **)&rdoc100);
-
-    if(ret != 1)
-      rdoc100 = NULL;
   }
 
-#else
-
-#endif
-
   return true;
 }
 
