@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2015-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,7 @@
 #include <stdint.h>
 #include <string.h>
 #include <algorithm>
+#include <map>
 #include <string>
 #include <vector>
 
@@ -41,10 +42,21 @@
 
 typedef uint8_t byte;
 
+#define STRINGIZE2(a) #a
+#define STRINGIZE(a) STRINGIZE2(a)
+
+#define CONCAT2(a, b) a##b
+#define CONCAT(a, b) CONCAT2(a, b)
+
+#define PCI_VENDOR_NV 0x10DE
+#define PCI_VENDOR_AMD 0x1002
+#define PCI_VENDOR_INTEL 0x8086
+
 enum class SPIRVTarget
 {
   opengl,
-  vulkan
+  vulkan,
+  vulkan11,
 };
 enum class ShaderLang
 {
@@ -61,9 +73,11 @@ enum class ShaderStage
   comp
 };
 
+bool InternalSpvCompiler();
 bool SpvCompilationSupported();
 std::vector<uint32_t> CompileShaderToSpv(const std::string &source_text, SPIRVTarget target,
-                                         ShaderLang lang, ShaderStage stage, const char *entry_point);
+                                         ShaderLang lang, ShaderStage stage, const char *entry_point,
+                                         const std::map<std::string, std::string> &macros = {});
 
 struct Vec2f
 {
@@ -107,6 +121,8 @@ struct Vec4f
     z = Z;
     w = W;
   }
+  bool operator==(const Vec4f &o) { return x == o.x && y == o.y && z == o.z && w == o.w; }
+  bool operator!=(const Vec4f &o) { return !(*this == o); }
   float x, y, z, w;
 };
 
@@ -152,29 +168,32 @@ struct GraphicsTest
 {
   virtual ~GraphicsTest() {}
   virtual GraphicsWindow *MakeWindow(int width, int height, const char *title) { return NULL; }
-  virtual int main(int argc, char **argv) { return 9; }
-  virtual bool IsSupported() { return false; }
-  virtual bool Init(int argc, char **argv);
+  virtual int main() { return 9; }
+  virtual void Prepare(int argc, char **argv);
+  virtual void SetName(const char *name) { screenTitle = name; }
+  virtual bool Init();
   virtual void Shutdown();
 
+  std::string Avail;
+
   std::string GetDataPath(const std::string &filename);
 
   bool FrameLimit();
 
-  std::string dataRoot;
+  int curFrame = -1;
 
-  int curFrame = 0;
-  int maxFrameCount = -1;
-
-  int screenWidth = 400;
-  int screenHeight = 300;
   const char *screenTitle = "RenderDoc test program";
-  bool fullscreen = false;
-  bool debugDevice = false;
+
   bool headless = false;
 
-  RENDERDOC_API_1_0_0 *rdoc100 = NULL;
-  RENDERDOC_API_1_4_0 *rdoc = NULL;
+  RENDERDOC_API_1_0_0 *rdoc = NULL;
+
+  // shared parameters
+  static int maxFrameCount;
+  static std::string dataRoot;
+  static int screenWidth;
+  static int screenHeight;
+  static bool debugDevice;
 };
 
 enum class TestAPI
@@ -186,6 +205,20 @@ enum class TestAPI
   Count,
 };
 
+inline const char *APIName(TestAPI API)
+{
+  switch(API)
+  {
+    case TestAPI::D3D11: return "D3D11";
+    case TestAPI::Vulkan: return "Vulkan";
+    case TestAPI::OpenGL: return "OpenGL";
+    case TestAPI::D3D12: return "D3D12";
+    case TestAPI::Count: break;
+  }
+
+  return "???";
+}
+
 struct TestMetadata
 {
   TestAPI API;
@@ -193,27 +226,8 @@ struct TestMetadata
   const char *Description;
   GraphicsTest *test;
 
-  std::string QualifiedName() const
-  {
-    std::string ret = APIName();
-    ret += "::";
-    ret += Name;
-    return ret;
-  }
-
-  const char *APIName() const
-  {
-    switch(API)
-    {
-      case TestAPI::D3D11: return "D3D11";
-      case TestAPI::Vulkan: return "VK";
-      case TestAPI::OpenGL: return "GL";
-      case TestAPI::D3D12: return "D3D12";
-    }
-
-    return "???";
-  }
-
+  bool IsAvailable() const { return test->Avail.empty(); }
+  const char *AvailMessage() const { return test->Avail.c_str(); }
   bool operator<(const TestMetadata &o)
   {
     if(API != o.API)
@@ -229,25 +243,33 @@ struct TestMetadata
 
 void RegisterTest(TestMetadata test);
 
-#define REGISTER_TEST(TestName)                 \
-  namespace                                     \
-  {                                             \
-  struct TestRegistration                       \
-  {                                             \
-    TestName m_impl;                            \
-    TestRegistration()                          \
-    {                                           \
-      TestMetadata test;                        \
-      test.API = TestName::API;                 \
-      test.Name = #TestName;                    \
-      test.Description = TestName::Description; \
-      test.test = &m_impl;                      \
-      if(m_impl.IsSupported())                  \
-        RegisterTest(test);                     \
-    }                                           \
-  };                                            \
-  };                                            \
-  static TestRegistration Anon##__LINE__;
+#define RD_TEST(Test, Parent)             \
+  struct Test;                            \
+  int CONCAT(unique_, Test);              \
+  typedef Test CurrentTest;               \
+  namespace                               \
+  {                                       \
+  constexpr const char *TestName = #Test; \
+  };                                      \
+  struct Test : Parent
+
+#define REGISTER_TEST()                            \
+  namespace                                        \
+  {                                                \
+  struct TestRegistration                          \
+  {                                                \
+    CurrentTest m_impl;                            \
+    TestRegistration()                             \
+    {                                              \
+      TestMetadata test;                           \
+      test.API = CurrentTest::API;                 \
+      test.Name = TestName;                        \
+      test.Description = CurrentTest::Description; \
+      test.test = &m_impl;                         \
+      RegisterTest(test);                          \
+    }                                              \
+  } Anon##__LINE__;                                \
+  };
 
 std::string GetCWD();
 std::string GetEnvVar(const char *var);
@@ -313,3 +335,62 @@ void DebugPrint(const char *fmt, ...);
     DEBUG_BREAK();                                                                   \
     exit(0);                                                                         \
   } while(0)
+
+namespace TextureZoo
+{
+enum class DataType
+{
+  Float,
+  UNorm,
+  SNorm,
+  UInt,
+  SInt,
+  Count,
+};
+
+enum class TextureType
+{
+  Unknown,
+  Regular,
+  R9G9B9E5,
+  G4R4,
+  A4R4G4B4,
+  R4G4B4A4,
+  R5G6B5,
+  R5G5B5A1,
+  A1R5G5B5,
+  RGB10A2,
+  BC1,
+  BC2,
+  BC3,
+  BC4,
+  BC5,
+  BC6,
+  BC7,
+};
+
+static const uint32_t texWidth = 8;
+static const uint32_t texHeight = 8;
+static const uint32_t texDepth = 10;
+static const uint32_t texMips = 3;
+static const uint32_t texSlices = 2;
+static const uint32_t texSamples = 2;
+
+struct TexConfig
+{
+  TextureType type;
+  uint32_t componentCount;
+  uint32_t componentBytes;
+  DataType data;
+};
+
+struct TexData
+{
+  std::vector<byte> byteData;
+  uint32_t rowPitch = 0;
+  uint32_t slicePitch = 0;
+};
+
+void MakeData(TexData &data, const TexConfig &cfg, Vec4i dimensions, uint32_t mip, uint32_t slice);
+
+};    // namespace TextureZoo
