@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,14 +39,12 @@
 #endif
 
 #define BUFIDX 15
+#define INDEX3 4
 #define INDEX1 49
 #define INDEX2 381
 #define NONUNIFORMIDX 20
 
-#define STRINGISE2(a) #a
-#define STRINGISE(a) STRINGISE2(a)
-
-struct VK_Descriptor_Indexing : VulkanGraphicsTest
+RD_TEST(VK_Descriptor_Indexing, VulkanGraphicsTest)
 {
   static constexpr const char *Description =
       "Draws a triangle using descriptor indexing with large descriptor sets.";
@@ -148,7 +146,7 @@ layout(binding = 0, std430) buffer inbuftype {
   tex_ref inrefs[];
 } inbuf[];
 
-layout(binding = 1) uniform sampler2D tex1[)EOSHADER" STRINGISE(DESC_ARRAY1_SIZE) R"EOSHADER(];
+layout(binding = 1) uniform sampler2D tex1[)EOSHADER" STRINGIZE(DESC_ARRAY1_SIZE) R"EOSHADER(];
 layout(binding = 2) uniform sampler2D tex2[];
 
 void add_color(sampler2D tex)
@@ -156,21 +154,21 @@ void add_color(sampler2D tex)
   Color *= (vec4(0.25f) + texture(tex, vertIn.uv.xy));
 }
 
-void add_indirect_color2(sampler2D texs[)EOSHADER" STRINGISE(DESC_ARRAY1_SIZE) R"EOSHADER(], uint idx)
+void add_indirect_color2(sampler2D texs[)EOSHADER" STRINGIZE(DESC_ARRAY1_SIZE) R"EOSHADER(], uint idx)
 {
   add_color(texs[idx]);
 }
 
 void add_indirect_color(int dummy,
-                        sampler2D texs[)EOSHADER" STRINGISE(DESC_ARRAY1_SIZE) R"EOSHADER(], tex_ref t)
+                        sampler2D texs[)EOSHADER" STRINGIZE(DESC_ARRAY1_SIZE) R"EOSHADER(], tex_ref t)
 {
   // second array-param function call
   add_indirect_color2(texs, t.idx);
 }
 
 void dispatch_indirect_color(int dummy1,
-                             sampler2D texA[)EOSHADER" STRINGISE(DESC_ARRAY1_SIZE) R"EOSHADER(],
-                             sampler2D texB[)EOSHADER" STRINGISE(DESC_ARRAY1_SIZE) R"EOSHADER(],
+                             sampler2D texA[)EOSHADER" STRINGIZE(DESC_ARRAY1_SIZE) R"EOSHADER(],
+                             sampler2D texB[)EOSHADER" STRINGIZE(DESC_ARRAY1_SIZE) R"EOSHADER(],
                              float dummy2, tex_ref t)
 {
   if(t.binding == 0)
@@ -185,12 +183,20 @@ void dispatch_indirect_color(int dummy1,
   }
 }
 
+void add_parameterless()
+{
+  // use array directly without it being a function parameter
+  Color += 0.1f * texture(tex1[)EOSHADER" STRINGIZE(INDEX3) R"EOSHADER(], vertIn.uv.xy);
+}
+
 void main()
 {
   if(vertIn.uv.y < 0.2f)
   {
     // nonuniform dynamic index
     Color = texture(tex1[nonuniformEXT(int(vertIn.col.w+0.5f))], vertIn.uv.xy);
+
+    add_parameterless();
   }
   else
   {
@@ -214,28 +220,55 @@ void main()
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  void Prepare(int argc, char **argv)
   {
     devExts.push_back(VK_EXT_DESCRIPTOR_INDEXING_EXTENSION_NAME);
 
     // dependencies of VK_EXT_descriptor_indexing
     devExts.push_back(VK_KHR_MAINTENANCE3_EXTENSION_NAME);
-    instExts.push_back(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
 
     features.fragmentStoresAndAtomics = VK_TRUE;
 
-    VkPhysicalDeviceDescriptorIndexingFeaturesEXT descIndexing = {
+    VulkanGraphicsTest::Prepare(argc, argv);
+
+    if(!Avail.empty())
+      return;
+
+    VkPhysicalDeviceProperties props;
+    vkGetPhysicalDeviceProperties(phys, &props);
+
+    // lazy - we could reduce this limit to a couple by not using combined image samplers
+    if(props.limits.maxDescriptorSetSamplers < DESC_ARRAY1_SIZE + DESC_ARRAY2_SIZE)
+      Avail = "maxDescriptorSetSamplers " + std::to_string(props.limits.maxDescriptorSetSamplers) +
+              " is insufficient";
+    else if(props.limits.maxDescriptorSetSampledImages < DESC_ARRAY1_SIZE + DESC_ARRAY2_SIZE)
+      Avail = "maxDescriptorSetSampledImages " +
+              std::to_string(props.limits.maxDescriptorSetSampledImages) + " is insufficient";
+
+    if(!Avail.empty())
+      return;
+
+    static VkPhysicalDeviceDescriptorIndexingFeaturesEXT descIndexing = {
         VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_INDEXING_FEATURES_EXT,
     };
 
-    descIndexing.descriptorBindingPartiallyBound = VK_TRUE;
-    descIndexing.runtimeDescriptorArray = VK_TRUE;
-    descIndexing.shaderSampledImageArrayNonUniformIndexing = VK_TRUE;
+    getPhysFeatures2(&descIndexing);
+
+    if(!descIndexing.descriptorBindingPartiallyBound)
+      Avail = "Descriptor indexing feature 'descriptorBindingPartiallyBound' not available";
+    else if(!descIndexing.runtimeDescriptorArray)
+      Avail = "Descriptor indexing feature 'runtimeDescriptorArray' not available";
+    else if(!descIndexing.shaderSampledImageArrayNonUniformIndexing)
+      Avail =
+          "Descriptor indexing feature 'shaderSampledImageArrayNonUniformIndexing' not available";
 
     devInfoNext = &descIndexing;
+  }
 
+  int main()
+  {
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     VkDescriptorSetLayoutBindingFlagsCreateInfoEXT descFlags = {
@@ -280,7 +313,7 @@ void main()
     vkh::GraphicsPipelineCreateInfo pipeCreateInfo;
 
     pipeCreateInfo.layout = layout;
-    pipeCreateInfo.renderPass = swapRenderPass;
+    pipeCreateInfo.renderPass = mainWindow->rp;
 
     pipeCreateInfo.vertexInputState.vertexBindingDescriptions = {vkh::vertexBind(0, DefaultA2V)};
     pipeCreateInfo.vertexInputState.vertexAttributeDescriptions = {
@@ -308,17 +341,16 @@ void main()
         {Vec3f(0.5f, -0.5f, 0.0f), Vec4f(0.0f, 0.0f, 1.0f, right), Vec2f(1.0f, 0.0f)},
     };
 
-    AllocatedBuffer vb(allocator,
-                       vkh::BufferCreateInfo(sizeof(tri), VK_BUFFER_USAGE_VERTEX_BUFFER_BIT |
-                                                              VK_BUFFER_USAGE_TRANSFER_DST_BIT),
+    AllocatedBuffer vb(this, vkh::BufferCreateInfo(sizeof(tri), VK_BUFFER_USAGE_VERTEX_BUFFER_BIT |
+                                                                    VK_BUFFER_USAGE_TRANSFER_DST_BIT),
                        VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_CPU_TO_GPU}));
 
     vb.upload(tri);
 
-    AllocatedImage img(allocator, vkh::ImageCreateInfo(
-                                      4, 4, 0, VK_FORMAT_R32G32B32A32_SFLOAT,
-                                      VK_IMAGE_USAGE_TRANSFER_DST_BIT | VK_IMAGE_USAGE_SAMPLED_BIT),
-                       VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_GPU_ONLY}));
+    AllocatedImage img(
+        this, vkh::ImageCreateInfo(4, 4, 0, VK_FORMAT_R32G32B32A32_SFLOAT,
+                                   VK_IMAGE_USAGE_TRANSFER_DST_BIT | VK_IMAGE_USAGE_SAMPLED_BIT),
+        VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_GPU_ONLY}));
 
     setName(img.image, "Colour Tex");
 
@@ -330,17 +362,17 @@ void main()
       pixels[i] = RANDF(0.2f, 1.0f);
 
     AllocatedBuffer uploadBuf(
-        allocator, vkh::BufferCreateInfo(sizeof(pixels), VK_BUFFER_USAGE_TRANSFER_SRC_BIT),
+        this, vkh::BufferCreateInfo(sizeof(pixels), VK_BUFFER_USAGE_TRANSFER_SRC_BIT),
         VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_CPU_TO_GPU}));
 
     uploadBuf.upload(pixels);
 
     // create an image with black contents for all the indices we aren't using
 
-    AllocatedImage badimg(allocator, vkh::ImageCreateInfo(4, 4, 0, VK_FORMAT_R32G32B32A32_SFLOAT,
-                                                          VK_IMAGE_USAGE_TRANSFER_DST_BIT |
-                                                              VK_IMAGE_USAGE_SAMPLED_BIT),
-                          VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_GPU_ONLY}));
+    AllocatedImage badimg(
+        this, vkh::ImageCreateInfo(4, 4, 0, VK_FORMAT_R32G32B32A32_SFLOAT,
+                                   VK_IMAGE_USAGE_TRANSFER_DST_BIT | VK_IMAGE_USAGE_SAMPLED_BIT),
+        VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_GPU_ONLY}));
 
     setName(badimg.image, "Black Tex");
 
@@ -447,7 +479,7 @@ void main()
 
     vkh::updateDescriptorSets(device, ups);
 
-    AllocatedBuffer ssbo(allocator,
+    AllocatedBuffer ssbo(this,
                          vkh::BufferCreateInfo(1024 * 1024, VK_BUFFER_USAGE_STORAGE_BUFFER_BIT |
                                                                 VK_BUFFER_USAGE_TRANSFER_DST_BIT),
                          VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_GPU_ONLY}));
@@ -463,6 +495,11 @@ void main()
     vkh::updateDescriptorSets(
         device,
         {
+            vkh::WriteDescriptorSet(
+                descset[0], 1, INDEX3, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER,
+                {
+                    vkh::DescriptorImageInfo(imgview, VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL, sampler),
+                }),
             vkh::WriteDescriptorSet(
                 descset[0], 1, INDEX1, VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER,
                 {
@@ -531,7 +568,7 @@ void main()
                                              ssbo.buffer)});
 
       vkCmdBeginRenderPass(
-          cmd, vkh::RenderPassBeginInfo(swapRenderPass, swapFramebuffers[swapIndex], scissor),
+          cmd, vkh::RenderPassBeginInfo(mainWindow->rp, mainWindow->GetFB(), mainWindow->scissor),
           VK_SUBPASS_CONTENTS_INLINE);
 
       vkCmdBindPipeline(cmd, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe);
@@ -544,8 +581,8 @@ void main()
       // bind the actual one
       vkCmdBindDescriptorSets(cmd, VK_PIPELINE_BIND_POINT_GRAPHICS, layout, 0, 1, &descset[0], 0,
                               NULL);
-      vkCmdSetViewport(cmd, 0, 1, &viewport);
-      vkCmdSetScissor(cmd, 0, 1, &scissor);
+      vkCmdSetViewport(cmd, 0, 1, &mainWindow->viewport);
+      vkCmdSetScissor(cmd, 0, 1, &mainWindow->scissor);
       vkh::cmdBindVertexBuffers(cmd, 0, {vb.buffer}, {0});
       idx = {BUFIDX, 0, 0, 0};
       vkCmdPushConstants(cmd, layout, VK_SHADER_STAGE_FRAGMENT_BIT | VK_SHADER_STAGE_COMPUTE_BIT, 0,
@@ -563,6 +600,8 @@ void main()
       Present();
     }
 
+    vkDeviceWaitIdle(device);
+
     vkDestroyDescriptorPool(device, descpool, NULL);
     vkDestroySampler(device, sampler, NULL);
 
@@ -570,4 +609,4 @@ void main()
   }
 };
 
-REGISTER_TEST(VK_Descriptor_Indexing);
\ No newline at end of file
+REGISTER_TEST();
