@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "vk_test.h"
 
-struct VK_Discard_Rectangles : VulkanGraphicsTest
+RD_TEST(VK_Discard_Rectangles, VulkanGraphicsTest)
 {
   static constexpr const char *Description =
       "Draws a large number of triangles using VK_EXT_discard_rectangles discard rectangles to "
@@ -74,13 +74,17 @@ void main()
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  void Prepare(int argc, char **argv)
   {
-    instExts.push_back(VK_KHR_GET_PHYSICAL_DEVICE_PROPERTIES_2_EXTENSION_NAME);
     devExts.push_back(VK_EXT_DISCARD_RECTANGLES_EXTENSION_NAME);
 
+    VulkanGraphicsTest::Prepare(argc, argv);
+  }
+
+  int main()
+  {
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     VkPhysicalDeviceDiscardRectanglePropertiesEXT discardProps = {
@@ -88,8 +92,8 @@ void main()
 
     vkGetPhysicalDeviceProperties2KHR(phys, vkh::PhysicalDeviceProperties2KHR().next(&discardProps));
 
-    const int32_t w = (int32_t)scissor.extent.width;
-    const int32_t h = (int32_t)scissor.extent.height;
+    const int32_t w = (int32_t)mainWindow->scissor.extent.width;
+    const int32_t h = (int32_t)mainWindow->scissor.extent.height;
 
     VkRect2D discardRects[] = {
         // TL eye
@@ -114,7 +118,7 @@ void main()
     vkh::GraphicsPipelineCreateInfo pipeCreateInfo;
 
     pipeCreateInfo.layout = layout;
-    pipeCreateInfo.renderPass = swapRenderPass;
+    pipeCreateInfo.renderPass = mainWindow->rp;
 
     pipeCreateInfo.vertexInputState.vertexBindingDescriptions = {vkh::vertexBind(0, DefaultA2V)};
     pipeCreateInfo.vertexInputState.vertexAttributeDescriptions = {
@@ -151,7 +155,7 @@ void main()
       trispam[i].uv = Vec2f(0.0f, 0.0f);
     }
 
-    AllocatedBuffer vb(allocator,
+    AllocatedBuffer vb(this,
                        vkh::BufferCreateInfo(sizeof(trispam), VK_BUFFER_USAGE_VERTEX_BUFFER_BIT |
                                                                   VK_BUFFER_USAGE_TRANSFER_DST_BIT),
                        VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_CPU_TO_GPU}));
@@ -172,16 +176,16 @@ void main()
                            vkh::ImageSubresourceRange());
 
       vkCmdBeginRenderPass(
-          cmd, vkh::RenderPassBeginInfo(swapRenderPass, swapFramebuffers[swapIndex], scissor),
+          cmd, vkh::RenderPassBeginInfo(mainWindow->rp, mainWindow->GetFB(), mainWindow->scissor),
           VK_SUBPASS_CONTENTS_INLINE);
 
-      VkViewport view = viewport;
+      VkViewport view = mainWindow->viewport;
       view.width /= 2.0f;
 
       vkCmdBindPipeline(cmd, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe);
       vkCmdSetDiscardRectangleEXT(cmd, 0, ARRAY_COUNT(discardRects), discardRects);
       vkCmdSetViewport(cmd, 0, 1, &view);
-      vkCmdSetScissor(cmd, 0, 1, &scissor);
+      vkCmdSetScissor(cmd, 0, 1, &mainWindow->scissor);
       vkh::cmdBindVertexBuffers(cmd, 0, {vb.buffer}, {0});
       vkCmdDraw(cmd, 3000, 1, 0, 0);
 
@@ -206,4 +210,4 @@ void main()
   }
 };
 
-REGISTER_TEST(VK_Discard_Rectangles);
\ No newline at end of file
+REGISTER_TEST();
