@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2018-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -155,7 +155,7 @@ GraphicsPipelineCreateInfo::GraphicsPipelineCreateInfo()
 
   rasterizationState = VkPipelineRasterizationStateCreateInfo();
   rasterizationState.sType = VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO;
-  rasterizationState.depthClampEnable = VK_TRUE;
+  rasterizationState.depthClampEnable = VK_FALSE;
   rasterizationState.polygonMode = VK_POLYGON_MODE_FILL;
   rasterizationState.cullMode = VK_CULL_MODE_NONE;
   rasterizationState.frontFace = VK_FRONT_FACE_CLOCKWISE;
@@ -167,6 +167,16 @@ GraphicsPipelineCreateInfo::GraphicsPipelineCreateInfo()
 
   depthStencilState = VkPipelineDepthStencilStateCreateInfo();
   depthStencilState.sType = VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO;
+  depthStencilState.depthCompareOp = VK_COMPARE_OP_LESS_OR_EQUAL;
+  depthStencilState.minDepthBounds = 0.0f;
+  depthStencilState.maxDepthBounds = 1.0f;
+  depthStencilState.front.compareMask = 0xff;
+  depthStencilState.front.writeMask = 0xff;
+  depthStencilState.front.compareOp = VK_COMPARE_OP_EQUAL;
+  depthStencilState.front.passOp = VK_STENCIL_OP_REPLACE;
+  depthStencilState.front.failOp = VK_STENCIL_OP_KEEP;
+  depthStencilState.front.depthFailOp = VK_STENCIL_OP_KEEP;
+  depthStencilState.back = depthStencilState.front;
 
   colorBlendState.attachments.push_back({
       // blendEnable
@@ -208,4 +218,4 @@ void GraphicsPipelineCreateInfo::bake()
   dynamicState.dynamicStateCount = (uint32_t)dynamicState.dynamicStates.size();
 }
 
-};    // namespace vkh
\ No newline at end of file
+};    // namespace vkh
