@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "vk_test.h"
 
-struct VK_Overlay_Test : VulkanGraphicsTest
+RD_TEST(VK_Overlay_Test, VulkanGraphicsTest)
 {
   static constexpr const char *Description =
       "Makes a couple of draws that show off all the overlays in some way";
@@ -73,17 +73,22 @@ void main()
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  int main()
   {
+    optDevExts.push_back(VK_KHR_MAINTENANCE1_EXTENSION_NAME);
+
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
+    bool KHR_maintenance1 = std::find(devExts.begin(), devExts.end(),
+                                      VK_KHR_MAINTENANCE1_EXTENSION_NAME) != devExts.end();
+
     VkPipelineLayout layout = createPipelineLayout(vkh::PipelineLayoutCreateInfo());
 
     // note that the Y position values are inverted for vulkan 1.0 viewport convention, relative to
     // all other APIs
-    const DefaultA2V VBData[] = {
+    DefaultA2V VBData[] = {
         // this triangle occludes in depth
         {Vec3f(-0.5f, 0.5f, 0.0f), Vec4f(0.0f, 0.0f, 1.0f, 1.0f), Vec2f(0.0f, 0.0f)},
         {Vec3f(-0.5f, 0.0f, 0.0f), Vec4f(0.0f, 0.0f, 1.0f, 1.0f), Vec2f(0.0f, 1.0f)},
@@ -142,7 +147,14 @@ void main()
         {Vec3f(0.025f, -0.7f, 0.5f), Vec4f(1.0f, 0.5f, 1.0f, 1.0f), Vec2f(1.0f, 0.0f)},
     };
 
-    AllocatedBuffer vb(allocator,
+    // negate y if we're using negative viewport height
+    if(KHR_maintenance1)
+    {
+      for(DefaultA2V &v : VBData)
+        v.pos.y = -v.pos.y;
+    }
+
+    AllocatedBuffer vb(this,
                        vkh::BufferCreateInfo(sizeof(VBData), VK_BUFFER_USAGE_VERTEX_BUFFER_BIT |
                                                                  VK_BUFFER_USAGE_TRANSFER_DST_BIT),
                        VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_CPU_TO_GPU}));
@@ -150,10 +162,10 @@ void main()
     vb.upload(VBData);
 
     // create depth-stencil image
-    AllocatedImage depthimg(allocator,
-                            vkh::ImageCreateInfo(scissor.extent.width, scissor.extent.height, 0,
-                                                 VK_FORMAT_D32_SFLOAT_S8_UINT,
-                                                 VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT),
+    AllocatedImage depthimg(this, vkh::ImageCreateInfo(mainWindow->scissor.extent.width,
+                                                       mainWindow->scissor.extent.height, 0,
+                                                       VK_FORMAT_D32_SFLOAT_S8_UINT,
+                                                       VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT),
                             VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_GPU_ONLY}));
 
     VkImageView dsvview = createImageView(vkh::ImageViewCreateInfo(
@@ -163,8 +175,8 @@ void main()
     // create renderpass using the DS image
     vkh::RenderPassCreator renderPassCreateInfo;
 
-    renderPassCreateInfo.attachments.push_back(
-        vkh::AttachmentDescription(swapFormat, VK_IMAGE_LAYOUT_GENERAL, VK_IMAGE_LAYOUT_GENERAL));
+    renderPassCreateInfo.attachments.push_back(vkh::AttachmentDescription(
+        mainWindow->format, VK_IMAGE_LAYOUT_GENERAL, VK_IMAGE_LAYOUT_GENERAL));
     renderPassCreateInfo.attachments.push_back(vkh::AttachmentDescription(
         VK_FORMAT_D32_SFLOAT_S8_UINT, VK_IMAGE_LAYOUT_UNDEFINED, VK_IMAGE_LAYOUT_GENERAL,
         VK_ATTACHMENT_LOAD_OP_CLEAR, VK_ATTACHMENT_STORE_OP_DONT_CARE, VK_SAMPLE_COUNT_1_BIT,
@@ -177,11 +189,11 @@ void main()
 
     // create framebuffers using swapchain images and DS image
     std::vector<VkFramebuffer> fbs;
-    fbs.resize(swapImageViews.size());
+    fbs.resize(mainWindow->GetCount());
 
-    for(size_t i = 0; i < swapImageViews.size(); i++)
-      fbs[i] = createFramebuffer(
-          vkh::FramebufferCreateInfo(renderPass, {swapImageViews[i], dsvview}, scissor.extent));
+    for(size_t i = 0; i < mainWindow->GetCount(); i++)
+      fbs[i] = createFramebuffer(vkh::FramebufferCreateInfo(
+          renderPass, {mainWindow->GetView(i), dsvview}, mainWindow->scissor.extent));
 
     // create PSO
     vkh::GraphicsPipelineCreateInfo pipeCreateInfo;
@@ -236,23 +248,31 @@ void main()
       VkImage swapimg =
           StartUsingBackbuffer(cmd, VK_ACCESS_TRANSFER_WRITE_BIT, VK_IMAGE_LAYOUT_GENERAL);
 
-      VkViewport v = viewport;
+      VkViewport v = mainWindow->viewport;
       v.x += 10.0f;
       v.y += 10.0f;
       v.width -= 20.0f;
       v.height -= 20.0f;
+
+      // if we're using KHR_maintenance1, check that negative viewport height is handled
+      if(KHR_maintenance1)
+      {
+        v.y += v.height;
+        v.height = -v.height;
+      }
+
       vkCmdSetViewport(cmd, 0, 1, &v);
-      vkCmdSetScissor(cmd, 0, 1, &scissor);
+      vkCmdSetScissor(cmd, 0, 1, &mainWindow->scissor);
       vkh::cmdBindVertexBuffers(cmd, 0, {vb.buffer}, {0});
 
       vkCmdClearColorImage(cmd, swapimg, VK_IMAGE_LAYOUT_GENERAL,
                            vkh::ClearColorValue(0.4f, 0.5f, 0.6f, 1.0f), 1,
                            vkh::ImageSubresourceRange());
 
-      vkCmdBeginRenderPass(cmd,
-                           vkh::RenderPassBeginInfo(renderPass, fbs[swapIndex], scissor,
-                                                    {vkh::ClearValue(), vkh::ClearValue(1.0f, 0)}),
-                           VK_SUBPASS_CONTENTS_INLINE);
+      vkCmdBeginRenderPass(
+          cmd, vkh::RenderPassBeginInfo(renderPass, fbs[mainWindow->imgIndex], mainWindow->scissor,
+                                        {vkh::ClearValue(), vkh::ClearValue(1.0f, 0)}),
+          VK_SUBPASS_CONTENTS_INLINE);
 
       // draw the setup triangles
 
@@ -286,4 +306,4 @@ void main()
   }
 };
 
-REGISTER_TEST(VK_Overlay_Test);
\ No newline at end of file
+REGISTER_TEST();
