@@ -1,7 +1,7 @@
 /******************************************************************************
  * The MIT License (MIT)
  *
- * Copyright (c) 2018-2019 Baldur Karlsson
+ * Copyright (c) 2019-2020 Baldur Karlsson
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,7 +24,7 @@
 
 #include "vk_test.h"
 
-struct Secondary_CmdBuf : VulkanGraphicsTest
+RD_TEST(VK_Secondary_CmdBuf, VulkanGraphicsTest)
 {
   static constexpr const char *Description = "Draw using secondary command buffers";
 
@@ -72,18 +72,18 @@ void main()
 
 )EOSHADER";
 
-  int main(int argc, char **argv)
+  int main()
   {
     // initialise, create window, create context, etc
-    if(!Init(argc, argv))
+    if(!Init())
       return 3;
 
     VkPipelineLayout layout = createPipelineLayout(vkh::PipelineLayoutCreateInfo());
 
-    VkRect2D size = scissor;
+    VkRect2D size = mainWindow->scissor;
 
     AllocatedImage img(
-        allocator,
+        this,
         vkh::ImageCreateInfo(size.extent.width, size.extent.height, 0, VK_FORMAT_R8G8B8A8_SRGB,
                              VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT | VK_IMAGE_USAGE_TRANSFER_SRC_BIT),
         VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_GPU_ONLY}));
@@ -146,7 +146,7 @@ void main()
         {Vec3f(0.75f, -0.5f, 0.0f), Vec4f(0.0f, 0.0f, 1.0f, 1.0f), Vec2f(1.0f, 0.0f)},
     };
 
-    AllocatedBuffer vb(allocator,
+    AllocatedBuffer vb(this,
                        vkh::BufferCreateInfo(sizeof(TwoTris), VK_BUFFER_USAGE_VERTEX_BUFFER_BIT |
                                                                   VK_BUFFER_USAGE_TRANSFER_DST_BIT),
                        VmaAllocationCreateInfo({0, VMA_MEMORY_USAGE_CPU_TO_GPU}));
@@ -162,7 +162,7 @@ void main()
                                             vkh::CommandBufferInheritanceInfo(renderPass, 1)));
 
       vkCmdBindPipeline(cmd2, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe1);
-      vkCmdSetViewport(cmd2, 0, 1, &viewport);
+      vkCmdSetViewport(cmd2, 0, 1, &mainWindow->viewport);
       vkCmdSetScissor(cmd2, 0, 1, &size);
       vkh::cmdBindVertexBuffers(cmd2, 0, {vb.buffer}, {sizeof(DefaultA2V) * 3});
 
@@ -188,7 +188,7 @@ void main()
                            VK_SUBPASS_CONTENTS_INLINE);
 
       vkCmdBindPipeline(cmd, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe0);
-      vkCmdSetViewport(cmd, 0, 1, &viewport);
+      vkCmdSetViewport(cmd, 0, 1, &mainWindow->viewport);
       vkCmdSetScissor(cmd, 0, 1, &size);
       vkh::cmdBindVertexBuffers(cmd, 0, {vb.buffer}, {0});
 
@@ -218,8 +218,8 @@ void main()
       region.srcOffsets[1].x = size.extent.width;
       region.srcOffsets[1].y = size.extent.height;
       region.srcOffsets[1].z = 1;
-      region.dstOffsets[1].x = scissor.extent.width;
-      region.dstOffsets[1].y = scissor.extent.height;
+      region.dstOffsets[1].x = mainWindow->scissor.extent.width;
+      region.dstOffsets[1].y = mainWindow->scissor.extent.height;
       region.dstOffsets[1].z = 1;
 
       vkCmdBlitImage(cmd, img.image, VK_IMAGE_LAYOUT_GENERAL, swapimg, VK_IMAGE_LAYOUT_GENERAL, 1,
@@ -238,4 +238,4 @@ void main()
   }
 };
 
-REGISTER_TEST(Secondary_CmdBuf);
\ No newline at end of file
+REGISTER_TEST();
