@@ -1,7 +1,7 @@
 /******************************************************************************
 * The MIT License (MIT)
 *
-* Copyright (c) 2018-2019 Baldur Karlsson
+* Copyright (c) 2019-2020 Baldur Karlsson
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
@@ -25,6 +25,7 @@
 #pragma once
 
 #include <functional>
+#include <mutex>
 #include <set>
 #include <vector>
 #include "../test_common.h"
@@ -32,35 +33,21 @@
 #include "vk_helpers.h"
 #include "vk_test.h"
 
+struct VulkanGraphicsTest;
+
 struct AllocatedBuffer
 {
-  VmaAllocator allocator;
-  VkBuffer buffer;
-  VmaAllocation alloc;
+  VulkanGraphicsTest *test = NULL;
+  VmaAllocator allocator = NULL;
+  VkBuffer buffer = VK_NULL_HANDLE;
+  VmaAllocation alloc = {};
 
   AllocatedBuffer() {}
-  AllocatedBuffer(VmaAllocator allocator, const VkBufferCreateInfo &bufInfo,
-                  const VmaAllocationCreateInfo &allocInfo)
-  {
-    create(allocator, bufInfo, allocInfo);
-  }
-  AllocatedBuffer(const AllocatedBuffer &) = delete;
-  AllocatedBuffer &operator=(const AllocatedBuffer &) = delete;
+  AllocatedBuffer(VulkanGraphicsTest *test, const VkBufferCreateInfo &bufInfo,
+                  const VmaAllocationCreateInfo &allocInfo);
 
-  void create(VmaAllocator vma, const VkBufferCreateInfo &bufInfo,
-              const VmaAllocationCreateInfo &allocInfo)
-  {
-    allocator = vma;
-    VkBuffer buf;
-    vmaCreateBuffer(allocator, &bufInfo, &allocInfo, &buf, &alloc, NULL);
-    buffer = VkBuffer(buf);
-  }
+  void free();
 
-  ~AllocatedBuffer()
-  {
-    if(buffer != VK_NULL_HANDLE)
-      vmaDestroyBuffer(allocator, (VkBuffer)buffer, alloc);
-  }
   template <typename T, size_t N>
   void upload(const T (&data)[N])
   {
@@ -91,33 +78,16 @@ struct AllocatedBuffer
 
 struct AllocatedImage
 {
-  VmaAllocator allocator;
-  VkImage image;
-  VmaAllocation alloc;
+  VulkanGraphicsTest *test = NULL;
+  VmaAllocator allocator = NULL;
+  VkImage image = VK_NULL_HANDLE;
+  VmaAllocation alloc = {};
 
   AllocatedImage() {}
-  AllocatedImage(VmaAllocator allocator, const VkImageCreateInfo &imgInfo,
-                 const VmaAllocationCreateInfo &allocInfo)
-  {
-    create(allocator, imgInfo, allocInfo);
-  }
-  AllocatedImage(const AllocatedImage &) = delete;
-  AllocatedImage &operator=(const AllocatedImage &) = delete;
+  AllocatedImage(VulkanGraphicsTest *test, const VkImageCreateInfo &imgInfo,
+                 const VmaAllocationCreateInfo &allocInfo);
 
-  void create(VmaAllocator vma, const VkImageCreateInfo &imgInfo,
-              const VmaAllocationCreateInfo &allocInfo)
-  {
-    allocator = vma;
-    VkImage img;
-    vmaCreateImage(allocator, &imgInfo, &allocInfo, &img, &alloc, NULL);
-    image = VkImage(img);
-  }
-
-  ~AllocatedImage()
-  {
-    if(image != VK_NULL_HANDLE)
-      vmaDestroyImage(allocator, (VkImage)image, alloc);
-  }
+  void free();
 };
 
 #define CHECK_VKR(cmd)                                                               \
@@ -134,29 +104,98 @@ struct AllocatedImage
     }                                                                                \
   } while(0);
 
+struct VulkanGraphicsTest;
+
+struct VulkanWindow : public GraphicsWindow
+{
+  VkFormat format;
+  uint32_t imgIndex = 0;
+  VkRenderPass rp = VK_NULL_HANDLE;
+  VkViewport viewport;
+  VkRect2D scissor;
+
+  VulkanWindow(VulkanGraphicsTest *test, GraphicsWindow *win);
+  virtual ~VulkanWindow();
+  void Shutdown();
+
+  size_t GetCount() { return imgs.size(); }
+  VkImage GetImage(size_t idx = ~0U)
+  {
+    if(idx == ~0U)
+      idx = imgIndex;
+    return imgs[idx];
+  }
+  VkImageView GetView(size_t idx = ~0U)
+  {
+    if(idx == ~0U)
+      idx = imgIndex;
+    return imgviews[idx];
+  }
+  VkFramebuffer GetFB(size_t idx = ~0U)
+  {
+    if(idx == ~0U)
+      idx = imgIndex;
+    return fbs[idx];
+  }
+  bool Initialised() { return swap != VK_NULL_HANDLE; }
+  VkCommandBuffer GetCommandBuffer(VkCommandBufferLevel level);
+  void Submit(int index, int totalSubmits, const std::vector<VkCommandBuffer> &cmds,
+              const std::vector<VkCommandBuffer> &seccmds, VkQueue q);
+  void Present(VkQueue q);
+  void Acquire();
+
+  // forward GraphicsWindow functions to internal window
+  void Resize(int width, int height) { m_Win->Resize(width, height); }
+  bool Update() { return m_Win->Update(); }
+private:
+  bool CreateSwapchain();
+  void DestroySwapchain();
+
+  VkSurfaceKHR surface = VK_NULL_HANDLE;
+  VkSwapchainKHR swap = VK_NULL_HANDLE;
+  std::vector<VkImage> imgs;
+  std::vector<VkImageView> imgviews;
+  VkSemaphore renderStartSemaphore = VK_NULL_HANDLE, renderEndSemaphore = VK_NULL_HANDLE;
+  std::vector<VkFramebuffer> fbs;
+
+  VkCommandPool cmdPool;
+  std::set<VkFence> fences;
+
+  std::vector<VkCommandBuffer> freeCommandBuffers[VK_COMMAND_BUFFER_LEVEL_RANGE_SIZE];
+  std::vector<std::pair<VkCommandBuffer, VkFence>>
+      pendingCommandBuffers[VK_COMMAND_BUFFER_LEVEL_RANGE_SIZE];
+
+  GraphicsWindow *m_Win;
+  VulkanGraphicsTest *m_Test;
+};
+
 struct VulkanGraphicsTest : public GraphicsTest
 {
   static const TestAPI API = TestAPI::Vulkan;
 
   VulkanGraphicsTest();
 
-  bool Init(int argc, char **argv);
+  void Prepare(int argc, char **argv);
+  bool Init();
   void Shutdown();
-  bool IsSupported();
-  GraphicsWindow *MakeWindow(int width, int height, const char *title);
-  VkResult CreateSurface(GraphicsWindow *win, VkSurfaceKHR *outSurf);
+  VulkanWindow *MakeWindow(int width, int height, const char *title);
 
   bool Running();
-  VkImage StartUsingBackbuffer(VkCommandBuffer cmd, VkAccessFlags nextUse, VkImageLayout layout);
-  void FinishUsingBackbuffer(VkCommandBuffer cmd, VkAccessFlags prevUse, VkImageLayout layout);
+  VkImage StartUsingBackbuffer(VkCommandBuffer cmd, VkAccessFlags nextUse, VkImageLayout layout,
+                               VulkanWindow *window = NULL);
+  void FinishUsingBackbuffer(VkCommandBuffer cmd, VkAccessFlags prevUse, VkImageLayout layout,
+                             VulkanWindow *window = NULL);
   void Submit(int index, int totalSubmits, const std::vector<VkCommandBuffer> &cmds,
-              const std::vector<VkCommandBuffer> &seccmds = {});
-  void Present();
+              const std::vector<VkCommandBuffer> &seccmds = {}, VulkanWindow *window = NULL,
+              VkQueue q = VK_NULL_HANDLE);
+  void Present(VulkanWindow *window = NULL, VkQueue q = VK_NULL_HANDLE);
 
-  VkPipelineShaderStageCreateInfo CompileShaderModule(const std::string &source_text,
-                                                      ShaderLang lang, ShaderStage stage,
-                                                      const char *entry_point = "main");
-  VkCommandBuffer GetCommandBuffer(VkCommandBufferLevel level = VK_COMMAND_BUFFER_LEVEL_PRIMARY);
+  VkPipelineShaderStageCreateInfo CompileShaderModule(
+      const std::string &source_text, ShaderLang lang, ShaderStage stage,
+      const char *entry_point = "main", const std::map<std::string, std::string> &macros = {},
+      SPIRVTarget target = SPIRVTarget::vulkan);
+  VkCommandBuffer GetCommandBuffer(VkCommandBufferLevel level = VK_COMMAND_BUFFER_LEVEL_PRIMARY,
+                                   VulkanWindow *window = NULL);
 
   void setName(VkObjectType objType, uint64_t obj, const std::string &name);
   void pushMarker(VkCommandBuffer cmd, const std::string &name);
@@ -176,48 +215,41 @@ struct VulkanGraphicsTest : public GraphicsTest
   VkPipelineLayout createPipelineLayout(const VkPipelineLayoutCreateInfo *info);
   VkDescriptorSetLayout createDescriptorSetLayout(const VkDescriptorSetLayoutCreateInfo *info);
 
-  void resize();
-  void onResize(std::function<void()> callback) { resizeCallbacks.push_back(callback); }
-  bool createSwap();
-  void destroySwap();
-  void acquireImage();
+  void getPhysFeatures2(void *nextStruct);
+
+  std::mutex mutex;
+
+  // instance version
+  uint32_t instVersion;
+  // device version
+  uint32_t devVersion;
 
   // requested features
   VkPhysicalDeviceFeatures features = {};
 
-  // required extensions before Init(), enabled extensions after Init()
+  // enabled instance extensions
   std::vector<const char *> instExts;
+
+  // required extensions before Init(), enabled extensions after Init()
   std::vector<const char *> devExts;
 
+  // optional extensions, will be added to devExts if supported (allows fallback paths)
+  std::vector<const char *> optDevExts;
+
   // a custom struct to pass to vkDeviceCreateInfo::pNext
   const void *devInfoNext = NULL;
 
-  // optional extensions
-  std::vector<const char *> optInstExts;
-  std::vector<const char *> optDevExts;
-
   // core objects
   VkInstance instance;
   VkPhysicalDevice phys;
   VkDevice device;
-  uint32_t queueFamilyIndex;
+  uint32_t queueFamilyIndex = ~0U;
+  uint32_t queueCount;
   VkQueue queue;
-
-  // swapchain stuff
-  VkSurfaceKHR surface = VK_NULL_HANDLE;
-  VkSwapchainKHR swap;
-  VkFormat swapFormat;
-  std::vector<VkImage> swapImages;
-  std::vector<VkImageView> swapImageViews;
-  uint32_t swapIndex = 0;
-  VkSemaphore renderStartSemaphore, renderEndSemaphore;
-  std::vector<std::function<void()>> resizeCallbacks;
-  VkRenderPass swapRenderPass;
-  std::vector<VkFramebuffer> swapFramebuffers;
+  VkPhysicalDeviceProperties physProperties;
 
   // utilities
   VkDebugUtilsMessengerEXT debugUtilsMessenger;
-  VkCommandPool cmdPool;
 
   // tracking object lifetimes
   std::vector<VkShaderModule> shaders;
@@ -230,18 +262,16 @@ struct VulkanGraphicsTest : public GraphicsTest
   std::vector<VkPipelineLayout> pipelayouts;
   std::vector<VkDescriptorSetLayout> setlayouts;
 
-  VkViewport viewport;
-  VkRect2D scissor;
+  std::map<VkImage, VmaAllocation> imageAllocs;
+  std::map<VkBuffer, VmaAllocation> bufferAllocs;
 
-  GraphicsWindow *mainWindow = NULL;
-
-  // internal bookkeeping
-  std::set<VkFence> fences;
-
-  std::vector<VkCommandBuffer> freeCommandBuffers[VK_COMMAND_BUFFER_LEVEL_RANGE_SIZE];
-  std::vector<std::pair<VkCommandBuffer, VkFence>>
-      pendingCommandBuffers[VK_COMMAND_BUFFER_LEVEL_RANGE_SIZE];
+  VulkanWindow *mainWindow = NULL;
 
   // VMA
   VmaAllocator allocator = VK_NULL_HANDLE;
-};
\ No newline at end of file
+
+private:
+  static bool prepared_vk;
+
+  GraphicsWindow *MakePlatformWindow(int width, int height, const char *title);
+};
