@@ -141,7 +141,19 @@ def _run_test(testclass, failedcases: list):
                            .format(test_run.returncode))
 
 
-def run_tests(test_include: str, test_exclude: str, in_process: bool, slow_tests: bool):
+def fetch_tests():
+    output = subprocess.run([util.get_demos_binary(), '--list-raw'], stdout=subprocess.PIPE).stdout
+
+    # Skip the header, grab all the remaining lines
+    tests = str(output, 'utf-8').splitlines()[1:]
+
+    # Split the TSV values and store
+    split_tests = [ test.split('\t') for test in tests ]
+
+    return { x[0]: (x[1] == 'True', x[2]) for x in split_tests }
+
+
+def run_tests(test_include: str, test_exclude: str, in_process: bool, slow_tests: bool, debugger: bool):
     start_time = time.time()
 
     rd.InitGlobalEnv(rd.GlobalEnvironment(), [])
@@ -189,6 +201,8 @@ def run_tests(test_include: str, test_exclude: str, in_process: bool, slow_tests
 
     log.comment("driver={}".format(driver))
 
+    log.print("Demos running from {}".format(util.get_demos_binary()))
+
     layerInfo = rd.VulkanLayerRegistrationInfo()
     if rd.NeedVulkanLayerRegistration(layerInfo):
         log.print("Vulkan layer needs to be registered: {}".format(str(layerInfo.flags)))
@@ -226,6 +240,8 @@ def run_tests(test_include: str, test_exclude: str, in_process: bool, slow_tests
 
     os.environ['RENDERDOC_DEMOS_DATA'] = util.get_data_path('demos')
 
+    testcase.TestCase.set_test_list(fetch_tests())
+
     testcases = get_tests()
 
     include_regexp = re.compile(test_include, re.IGNORECASE)
@@ -252,9 +268,12 @@ def run_tests(test_include: str, test_exclude: str, in_process: bool, slow_tests
     for testclass in testcases:
         name = testclass.__name__
 
-        if ((testclass.platform != '' and testclass.platform != plat) or
-                (testclass.platform_version != 0 and testclass.platform_version > ver)):
-            log.print("Skipping {} as it's not supported on this platform '{} version {}'".format(name, plat, ver))
+        instance = testclass()
+
+        supported,unsupported_reason = instance.check_support()
+
+        if not supported:
+            log.print("Skipping {} as {}".format(name, unsupported_reason))
             skippedcases.append(testclass)
             continue
 
@@ -278,15 +297,20 @@ def run_tests(test_include: str, test_exclude: str, in_process: bool, slow_tests
 
         util.set_current_test(name)
 
-        try:
+        def do(debugMode):
             if in_process:
-                instance = testclass()
-                instance.invoketest()
+                instance.invoketest(debugMode)
             else:
                 _run_test(testclass, failedcases)
-        except Exception as ex:
-            log.failure(ex)
-            failedcases.append(testclass)
+
+        if debugger:
+            do(True)
+        else:
+            try:
+                do(False)
+            except Exception as ex:
+                log.failure(ex)
+                failedcases.append(testclass)
 
         log.end_test(name)
 
@@ -317,6 +341,23 @@ def vulkan_register():
     rd.UpdateVulkanLayerRegistration(True)
 
 
+def launch_remote_server():
+    # Fork the interpreter to run the test, in case it crashes we can catch it.
+    # We can re-run with the same parameters
+    args = sys.argv.copy()
+    args.insert(0, sys.executable)
+
+    # Add parameter to run the remote server itself
+    args.append('--internal_remote_server')
+
+    subprocess.Popen(args)
+    return
+
+
+def become_remote_server():
+    rd.BecomeRemoteServer('localhost', None, None)
+
+
 def internal_run_test(test_name):
     testcases = get_tests()
 
@@ -332,7 +373,7 @@ def internal_run_test(test_name):
 
             try:
                 instance = testclass()
-                instance.invoketest()
+                instance.invoketest(False)
                 suceeded = True
             except Exception as ex:
                 log.failure(ex)
