@@ -2,9 +2,11 @@ import os
 import traceback
 import copy
 import re
+import time
 import renderdoc as rd
 from . import util
 from . import analyse
+from . import capture
 from .logging import log, TestFailureException
 
 
@@ -126,8 +128,22 @@ class ConstantBufferChecker:
 
 class TestCase:
     slow_test = False
-    platform = ''
-    platform_version = 0
+    demos_test_name = ''
+    demos_frame_cap = 5
+    _test_list = {}
+
+    @staticmethod
+    def set_test_list(tests):
+        TestCase._test_list = tests
+
+    def check_support(self):
+        if self.demos_test_name != '':
+            if self.demos_test_name not in TestCase._test_list:
+                return False,'Test {} not in compiled tests'.format(self.demos_test_name)
+            return TestCase._test_list[self.demos_test_name]
+
+        # Otherwise assume we can run - child tests can override if they want to do some other check
+        return True,""
 
     def __init__(self):
         self.capture_filename = ""
@@ -153,6 +169,24 @@ class TestCase:
             else:
                 raise TestFailureException('Assertion Failure: {}'.format(msg))
 
+    def get_replay_options(self):
+        """
+        Method to overload if you want to override the replay options used.
+
+        :return: The renderdoc.ReplayOptions to use.
+        """
+
+        return rd.ReplayOptions()
+
+    def get_capture_options(self):
+        """
+        Method to overload if you want to override the capture options used.
+
+        :return: The renderdoc.CaptureOptions to use.
+        """
+
+        return rd.CaptureOptions()
+
     def get_capture(self):
         """
         Method to overload if not implementing a run(), using the default run which
@@ -161,6 +195,10 @@ class TestCase:
         :return: The path to the capture to open. If in a temporary path, it will be
           deleted if the test completes.
         """
+
+        if self.demos_test_name != '':
+            return capture.run_and_capture(util.get_demos_binary(), self.demos_test_name, self.demos_frame_cap, opts=self.get_capture_options())
+
         raise NotImplementedError("If run() is not implemented in a test, then"
                                   "get_capture() and check_capture() must be.")
 
@@ -202,6 +240,29 @@ class TestCase:
 
         return self._find_draw(name, start_event, self.controller.GetDrawcalls())
 
+    def get_vsin(self, draw: rd.DrawcallDescription, first_index: int=0, num_indices: int=0, instance: int=0, view: int=0):
+        ib: rd.BoundVBuffer = self.controller.GetPipelineState().GetIBuffer()
+
+        if num_indices == 0:
+            num_indices = draw.numIndices
+        else:
+            num_indices = min(num_indices, draw.numIndices)
+
+        mesh = rd.MeshFormat()
+        mesh.numIndices = num_indices
+        mesh.indexByteOffset = ib.byteOffset + draw.indexOffset * draw.indexByteWidth
+        mesh.indexByteStride = draw.indexByteWidth
+        mesh.indexResourceId = ib.resourceId
+        mesh.baseVertex = draw.baseVertex
+
+        attrs = analyse.get_vsin_attrs(self.controller, mesh)
+
+        first_index = min(first_index, draw.numIndices-1)
+
+        indices = analyse.fetch_indices(self.controller, mesh, 0, first_index, num_indices)
+
+        return analyse.decode_mesh_data(self.controller, indices, attrs, 0)
+
     def get_postvs(self, data_stage: rd.MeshDataStage, first_index: int=0, num_indices: int=0, instance: int=0, view: int=0):
         mesh: rd.MeshFormat = self.controller.GetPostVSData(instance, view, data_stage)
 
@@ -225,19 +286,36 @@ class TestCase:
         for idx in mesh_ref:
             ref = mesh_ref[idx]
             if idx >= len(mesh_data):
-                raise TestFailureException('PostVS data doesn\'t have expected element {}'.format(idx))
+                raise TestFailureException('Mesh data doesn\'t have expected element {}'.format(idx))
 
             data = mesh_data[idx]
 
             for key in ref:
                 if key not in data:
-                    raise TestFailureException('PostVS data[{}] doesn\'t contain data {} as expected. Data is: {}'.format(idx, key, list(data.keys())))
+                    raise TestFailureException('Mesh data[{}] doesn\'t contain data {} as expected. Data is: {}'.format(idx, key, list(data.keys())))
 
                 if not util.value_compare(ref[key], data[key]):
-                    raise TestFailureException('PostVS data[{}] \'{}\': {} is not as expected: {}'.format(idx, key, data[key], ref[key]))
+                    raise TestFailureException('Mesh data[{}] \'{}\': {} is not as expected: {}'.format(idx, key, data[key], ref[key]))
 
         log.success("Mesh data is identical to reference")
 
+    def check_pixel_value(self, tex: rd.ResourceId, x, y, value, eps=util.FLT_EPSILON):
+        tex_details = self.get_texture(tex)
+        res_details = self.get_resource(tex)
+
+        if type(x) is float:
+            x = int(tex_details.width * x)
+        if type(y) is float:
+            y = int(tex_details.height * y)
+
+        picked: rd.PixelValue = self.controller.PickPixel(tex, x, y, rd.Subresource(0, 0, 0), rd.CompType.Typeless)
+
+        if not util.value_compare(picked.floatValue, value, eps):
+            raise TestFailureException(
+                "Picked value {} at {},{} doesn't match expectation of {}".format(picked.floatValue, x, y, value))
+
+        log.success("Picked value at {},{} in {} is as expected".format(x, y, res_details.name))
+
     def run(self):
         self.capture_filename = self.get_capture()
 
@@ -245,16 +323,23 @@ class TestCase:
 
         log.print("Loading capture")
 
-        self.controller = analyse.open_capture(self.capture_filename)
+        self.controller = analyse.open_capture(self.capture_filename, opts=self.get_replay_options())
 
         log.print("Checking capture")
 
         self.check_capture()
 
-        self.controller.Shutdown()
+        if self.controller is not None:
+            self.controller.Shutdown()
 
-    def invoketest(self):
+    def invoketest(self, debugMode):
+        start_time = time.time()
         self.run()
+        duration = time.time() - start_time
+        minutes = int(duration / 60) % 60
+        seconds = round(duration % 60)
+        log.print("Test ran in {:02}:{:02}".format(minutes, seconds))
+        self.debugMode = debugMode
 
     def get_first_draw(self):
         first_draw: rd.DrawcallDescription = self.controller.GetDrawcalls()[0]
@@ -274,6 +359,16 @@ class TestCase:
 
         return None
 
+    def get_resource(self, id: rd.ResourceId):
+        resources = self.controller.GetResources()
+
+        for r in resources:
+            r: rd.ResourceDescription
+            if r.resourceId == id:
+                return r
+
+        return None
+
     def get_last_draw(self):
         last_draw: rd.DrawcallDescription = self.controller.GetDrawcalls()[-1]
 
