@@ -3,10 +3,7 @@ import rdtest
 
 
 class D3D11_Simple_Triangle(rdtest.TestCase):
-    platform = 'win32'
-
-    def get_capture(self):
-        return rdtest.run_and_capture("demos_x64", "D3D11_Simple_Triangle", 5)
+    demos_test_name = 'D3D11_Simple_Triangle'
 
     def check_capture(self):
         self.check_final_backbuffer()
