@@ -3,11 +3,7 @@ import renderdoc as rd
 
 
 class D3D12_CBuffer_Zoo(rdtest.TestCase):
-    platform = 'win32'
-    platform_version = 10
-
-    def get_capture(self):
-        return rdtest.run_and_capture("demos_x64", "D3D12_CBuffer_Zoo", 5)
+    demos_test_name = 'D3D12_CBuffer_Zoo'
 
     def check_capture(self):
         draw = self.find_draw("Draw")
@@ -16,20 +12,36 @@ class D3D12_CBuffer_Zoo(rdtest.TestCase):
 
         self.controller.SetFrameEvent(draw.eventId, False)
 
-        # Make an output so we can pick pixels
-        out: rd.ReplayOutput = self.controller.CreateOutput(rd.CreateHeadlessWindowingData(), rd.ReplayOutputType.Texture)
-
-        self.check(out is not None)
-
-        out.SetDimensions(100, 100)
-
         pipe: rd.PipeState = self.controller.GetPipelineState()
 
         stage = rd.ShaderStage.Pixel
+
+        refl: rd.ShaderReflection = pipe.GetShaderReflection(stage)
+        mapping: rd.ShaderBindpointMapping = pipe.GetBindpointMapping(stage)
+
+        # Make sure we have three constant buffers - b0 normal, b1 root constants, and space9999999:b0
+        binds = [
+            (0, 0),
+            (0, 1),
+            (999999999, 0),
+        ]
+
+        if len(refl.constantBlocks) != len(mapping.constantBlocks) or len(refl.constantBlocks) != len(binds):
+            raise rdtest.TestFailureException(
+                "Expected {}} constant buffers, only got {} {}".format(len(binds), len(refl.constantBlocks),
+                                                                       len(refl.constantBlocks)))
+
+        for b in range(0, len(binds)):
+            if binds[b][0] != mapping.constantBlocks[b].bindset or binds[b][1] != mapping.constantBlocks[b].bind:
+                raise rdtest.TestFailureException(
+                    "Unexpected cb[{}] mapping: set {} bind {}".format(b, mapping.constantBlocks[b].bindset,
+                                                                       mapping.constantBlocks[b].bind))
+
         cbuf: rd.BoundCBuffer = pipe.GetConstantBuffer(stage, 0, 0)
 
         var_check = rdtest.ConstantBufferChecker(
-            self.controller.GetCBufferVariableContents(pipe.GetShader(stage),
+            self.controller.GetCBufferVariableContents(pipe.GetGraphicsPipelineObject(),
+                                                       pipe.GetShader(stage),
                                                        pipe.GetShaderEntryPoint(stage), 0,
                                                        cbuf.resourceId, cbuf.byteOffset))
 
@@ -310,31 +322,79 @@ class D3D12_CBuffer_Zoo(rdtest.TestCase):
                                                                  433.0, 437.0]),
         })
 
+        # struct nested_with_padding
+        # {
+        #   float a; // float3 padding
+        #   float4 b;
+        #   float c; // float3 padding
+        #   float3 d[4]; // float padding after each one
+        # };
+        # nested_with_padding ak[2];
+        var_check.check('ak').rows(0).cols(0).arraySize(2).members({
+            # ak[0]
+            0: lambda s: s.rows(0).cols(0).structSize(4).members({
+                'a': lambda y: y.rows(1).cols(1).value([440.0]),
+                'b': lambda y: y.rows(1).cols(4).value([444.0, 445.0, 446.0, 447.0]),
+                'c': lambda y: y.rows(1).cols(1).value([448.0]),
+                'd': lambda x: x.rows(0).cols(0).arraySize(4).members({
+                    0: lambda z: z.rows(1).cols(3).value([452.0, 453.0, 454.0]),
+                    1: lambda z: z.rows(1).cols(3).value([456.0, 457.0, 458.0]),
+                    2: lambda z: z.rows(1).cols(3).value([460.0, 461.0, 462.0]),
+                    3: lambda z: z.rows(1).cols(3).value([464.0, 465.0, 466.0]),
+                }),
+            }),
+            # ak[1]
+            1: lambda s: s.rows(0).cols(0).structSize(4).members({
+                'a': lambda y: y.rows(1).cols(1).value([468.0]),
+                'b': lambda y: y.rows(1).cols(4).value([472.0, 473.0, 474.0, 475.0]),
+                'c': lambda y: y.rows(1).cols(1).value([476.0]),
+                'd': lambda x: x.rows(0).cols(0).arraySize(4).members({
+                    0: lambda z: z.rows(1).cols(3).value([480.0, 481.0, 482.0]),
+                    1: lambda z: z.rows(1).cols(3).value([484.0, 485.0, 486.0]),
+                    2: lambda z: z.rows(1).cols(3).value([488.0, 489.0, 490.0]),
+                    3: lambda z: z.rows(1).cols(3).value([492.0, 493.0, 494.0]),
+                }),
+            }),
+        })
+
+        # float2 dummy13;
+        var_check.check('dummy13')
+
+        # float al;
+        var_check.check('al').rows(1).cols(1).value([500.0])
+
+        # struct float2_struct
+        # {
+        #   float x, y;
+        # };
+        # float2_struct am;
+        var_check.check('am').rows(0).cols(0).members({
+            'x': lambda y: y.rows(1).cols(1).value([504.0]),
+            'y': lambda y: y.rows(1).cols(1).value([505.0]),
+        })
+
+        # float an;
+        var_check.check('an').rows(1).cols(1).value([506.0])
+
+        # float4 gldummy4;
+        var_check.check('gldummy4')
+
         # float4 test;
-        var_check.check('test').rows(1).cols(4).value([440.0, 441.0, 442.0, 443.0])
+        var_check.check('test').rows(1).cols(4).value([512.0, 513.0, 514.0, 515.0])
 
         var_check.done()
 
         rdtest.log.success("CBuffer variables are as expected")
 
-        tex = rd.TextureDisplay()
-        tex.resourceId = pipe.GetOutputTargets()[0].resourceId
-        out.SetTextureDisplay(tex)
-
-        texdetails = self.get_texture(tex.resourceId)
-
-        picked: rd.PixelValue = out.PickPixel(tex.resourceId, False,
-                                              int(texdetails.width / 2), int(texdetails.height / 2), 0, 0, 0)
-
-        if not rdtest.value_compare(picked.floatValue, [440.1, 441.0, 442.0, 443.0]):
-            raise rdtest.TestFailureException("Picked value {} doesn't match expectation".format(picked.floatValue))
+        self.check_pixel_value(pipe.GetOutputTargets()[0].resourceId, 0.5, 0.5, [512.1, 513.0, 514.0, 515.0])
 
         rdtest.log.success("Picked value is as expected")
 
         cbuf: rd.BoundCBuffer = pipe.GetConstantBuffer(stage, 1, 0)
 
         var_check = rdtest.ConstantBufferChecker(
-            self.controller.GetCBufferVariableContents(pipe.GetShader(stage),
+            self.controller.GetCBufferVariableContents(pipe.GetGraphicsPipelineObject(),
+                                                       pipe.GetShader(stage),
                                                        pipe.GetShaderEntryPoint(stage), 1,
                                                        cbuf.resourceId, cbuf.byteOffset))
 
@@ -360,4 +420,13 @@ class D3D12_CBuffer_Zoo(rdtest.TestCase):
 
         rdtest.log.success("Root signature variables are as expected")
 
-        out.Shutdown()
+        cbuf: rd.BoundCBuffer = pipe.GetConstantBuffer(stage, 2, 0)
+
+        var_check = rdtest.ConstantBufferChecker(
+            self.controller.GetCBufferVariableContents(pipe.GetGraphicsPipelineObject(),
+                                                       pipe.GetShader(stage),
+                                                       pipe.GetShaderEntryPoint(stage), 2,
+                                                       cbuf.resourceId, cbuf.byteOffset))
+
+        # float4 huge_val;
+        var_check.check('huge_val').rows(1).cols(4).value([64.0, 65.0, 66.0, 67.0])
