@@ -3,8 +3,7 @@ import renderdoc as rd
 
 
 class GL_CBuffer_Zoo(rdtest.TestCase):
-    def get_capture(self):
-        return rdtest.run_and_capture("demos_x64", "GL_CBuffer_Zoo", 5)
+    demos_test_name = 'GL_CBuffer_Zoo'
 
     def check_capture(self):
         draw = self.find_draw("Draw")
@@ -13,20 +12,14 @@ class GL_CBuffer_Zoo(rdtest.TestCase):
 
         self.controller.SetFrameEvent(draw.eventId, False)
 
-        # Make an output so we can pick pixels
-        out: rd.ReplayOutput = self.controller.CreateOutput(rd.CreateHeadlessWindowingData(), rd.ReplayOutputType.Texture)
-
-        self.check(out is not None)
-
-        out.SetDimensions(100, 100)
-
         pipe: rd.PipeState = self.controller.GetPipelineState()
 
         stage = rd.ShaderStage.Pixel
         cbuf: rd.BoundCBuffer = pipe.GetConstantBuffer(stage, 0, 0)
 
         var_check = rdtest.ConstantBufferChecker(
-            self.controller.GetCBufferVariableContents(pipe.GetShader(stage),
+            self.controller.GetCBufferVariableContents(pipe.GetGraphicsPipelineObject(),
+                                                       pipe.GetShader(stage),
                                                        pipe.GetShaderEntryPoint(stage), 0,
                                                        cbuf.resourceId, cbuf.byteOffset))
 
@@ -292,11 +285,65 @@ class GL_CBuffer_Zoo(rdtest.TestCase):
                                                                  433.0, 437.0]),
         })
 
-        # vec4 test;
-        var_check.check('test').rows(1).cols(4).value([440.0, 441.0, 442.0, 443.0])
+        # struct nested_with_padding
+        # {
+        #   float a; // vec3 padding
+        #   vec4 b;
+        #   float c; // vec3 padding
+        #   vec3 d[4]; // float padding after each one
+        # };
+        # nested_with_padding ak[2];
+        var_check.check('ak').rows(0).cols(0).arraySize(2).members({
+            # ak[0]
+            0: lambda s: s.rows(0).cols(0).structSize(4).members({
+                'a': lambda y: y.rows(1).cols(1).value([440.0]),
+                'b': lambda y: y.rows(1).cols(4).value([444.0, 445.0, 446.0, 447.0]),
+                'c': lambda y: y.rows(1).cols(1).value([448.0]),
+                'd': lambda x: x.rows(0).cols(0).arraySize(4).members({
+                    0: lambda z: z.rows(1).cols(3).value([452.0, 453.0, 454.0]),
+                    1: lambda z: z.rows(1).cols(3).value([456.0, 457.0, 458.0]),
+                    2: lambda z: z.rows(1).cols(3).value([460.0, 461.0, 462.0]),
+                    3: lambda z: z.rows(1).cols(3).value([464.0, 465.0, 466.0]),
+                }),
+            }),
+            # ak[1]
+            1: lambda s: s.rows(0).cols(0).structSize(4).members({
+                'a': lambda y: y.rows(1).cols(1).value([468.0]),
+                'b': lambda y: y.rows(1).cols(4).value([472.0, 473.0, 474.0, 475.0]),
+                'c': lambda y: y.rows(1).cols(1).value([476.0]),
+                'd': lambda x: x.rows(0).cols(0).arraySize(4).members({
+                    0: lambda z: z.rows(1).cols(3).value([480.0, 481.0, 482.0]),
+                    1: lambda z: z.rows(1).cols(3).value([484.0, 485.0, 486.0]),
+                    2: lambda z: z.rows(1).cols(3).value([488.0, 489.0, 490.0]),
+                    3: lambda z: z.rows(1).cols(3).value([492.0, 493.0, 494.0]),
+                }),
+            }),
+        })
+
+        # float2 dummy12;
+        var_check.check('dummy12')
+
+        # float al;
+        var_check.check('al').rows(1).cols(1).value([500.0])
+
+        # struct float2_struct
+        # {
+        #   float x, y;
+        # };
+        # float2_struct am;
+        var_check.check('am').rows(0).cols(0).members({
+            'x': lambda y: y.rows(1).cols(1).value([504.0]),
+            'y': lambda y: y.rows(1).cols(1).value([505.0]),
+        })
+
+        # float an;
+        var_check.check('an').rows(1).cols(1).value([508.0])
+
+        # float4 test;
+        var_check.check('test').rows(1).cols(4).value([512.0, 513.0, 514.0, 515.0])
 
         # to save duplicating if this array changes, we calculate out from the start, as the array is tightly packed
-        base = 444.0
+        base = 516.0
 
         exp_vals = lambda wi,yi,xi: [base + wi * 24.0 + yi * 8.0 + xi * 4.0 + c * 1.0 for c in range(0,4)]
 
@@ -364,24 +411,15 @@ class GL_CBuffer_Zoo(rdtest.TestCase):
 
         rdtest.log.success("CBuffer variables are as expected")
 
-        tex = rd.TextureDisplay()
-        tex.resourceId = pipe.GetOutputTargets()[0].resourceId
-        out.SetTextureDisplay(tex)
-
-        texdetails = self.get_texture(tex.resourceId)
-
-        picked: rd.PixelValue = out.PickPixel(tex.resourceId, False,
-                                              int(texdetails.width / 2), int(texdetails.height / 2), 0, 0, 0)
-
-        if not rdtest.value_compare(picked.floatValue, [440.1, 441.0, 442.0, 443.0]):
-            raise rdtest.TestFailureException("Picked value {} doesn't match expectation".format(picked.floatValue))
+        self.check_pixel_value(pipe.GetOutputTargets()[0].resourceId, 0.5, 0.5, [512.1, 513.0, 514.0, 515.0])
 
         rdtest.log.success("Picked value is as expected")
 
         cbuf: rd.BoundCBuffer = pipe.GetConstantBuffer(stage, 1, 0)
 
         var_check = rdtest.ConstantBufferChecker(
-            self.controller.GetCBufferVariableContents(pipe.GetShader(stage),
+            self.controller.GetCBufferVariableContents(pipe.GetGraphicsPipelineObject(),
+                                                       pipe.GetShader(stage),
                                                        pipe.GetShaderEntryPoint(stage), 1,
                                                        cbuf.resourceId, cbuf.byteOffset))
 
@@ -412,13 +450,38 @@ class GL_CBuffer_Zoo(rdtest.TestCase):
 
         # vec4 F[3][2][2];
         # Multidimensional arrays are represented as structs with N members
-        # Due to lacking reflection, we only access F[1][0][1] so that's the only one present
-        var_check.check('F').cols(0).rows(0).structSize(1).members({
-            '[1]': lambda x: x.cols(0).rows(0).structSize(1).members({
+        # Due to lacking reflection, we skip structSize() checks, but check everything else if it's present (if it's
+        # not, it will be skipped)
+        var_check.check('F').cols(0).rows(0).members({
+            '[0]': lambda x: x.cols(0).rows(0).members({
+                '[0]': lambda x: x.cols(0).rows(0).arraySize(2).members({
+                    0: lambda x: x.cols(4).rows(1).value([190.0, 200.0, 210.0, 220.0]),
+                    1: lambda x: x.cols(4).rows(1).value([230.0, 240.0, 250.0, 260.0]),
+                }),
+                '[1]': lambda x: x.cols(0).rows(0).arraySize(2).members({
+                    0: lambda x: x.cols(4).rows(1).value([270.0, 280.0, 290.0, 300.0]),
+                    1: lambda x: x.cols(4).rows(1).value([310.0, 320.0, 330.0, 340.0]),
+                }),
+            }),
+            '[1]': lambda x: x.cols(0).rows(0).members({
                 '[0]': lambda x: x.cols(0).rows(0).arraySize(2).members({
                     0: lambda x: x.cols(4).rows(1).value([350.0, 360.0, 370.0, 380.0]),
                     1: lambda x: x.cols(4).rows(1).value([390.0, 400.0, 410.0, 420.0]),
                 }),
+                '[1]': lambda x: x.cols(0).rows(0).arraySize(2).members({
+                    0: lambda x: x.cols(4).rows(1).value([430.0, 440.0, 450.0, 460.0]),
+                    1: lambda x: x.cols(4).rows(1).value([470.0, 480.0, 490.0, 500.0]),
+                }),
+            }),
+            '[2]': lambda x: x.cols(0).rows(0).members({
+                '[0]': lambda x: x.cols(0).rows(0).arraySize(2).members({
+                    0: lambda x: x.cols(4).rows(1).value([510.0, 520.0, 530.0, 540.0]),
+                    1: lambda x: x.cols(4).rows(1).value([550.0, 560.0, 570.0, 580.0]),
+                }),
+                '[1]': lambda x: x.cols(0).rows(0).arraySize(2).members({
+                    0: lambda x: x.cols(4).rows(1).value([590.0, 600.0, 610.0, 620.0]),
+                    1: lambda x: x.cols(4).rows(1).value([630.0, 640.0, 650.0, 660.0]),
+                }),
             }),
         })
 
@@ -430,7 +493,8 @@ class GL_CBuffer_Zoo(rdtest.TestCase):
         var_check.check('G').cols(0).rows(0).arraySize(2).members({
             # G[0]
             0: lambda s: s.cols(0).rows(0).structSize(3).members({
-                'a': lambda x: x.cols(0).rows(0).structSize(1).members({
+                'a': lambda x: x.cols(0).rows(0).members({
+                    'a': lambda y: y.cols(3).rows(1).value([680.0, 690.0, 700.0]),
                     'b': lambda y: y.cols(1).rows(1).value([710.0]),
                 }),
                 'b': lambda x: x.cols(0).rows(0).arraySize(4).members({
@@ -440,23 +504,28 @@ class GL_CBuffer_Zoo(rdtest.TestCase):
                     3: lambda y: y.cols(4).rows(1),
                 }),
                 'c': lambda x: x.cols(0).rows(0).arraySize(4).members({
-                    0: lambda y: y.cols(0).rows(0).structSize(1).members({
+                    0: lambda y: y.cols(0).rows(0).members({
                         'a': lambda z: z.cols(3).rows(1),
+                        'b': lambda z: z.cols(1).rows(1),
                     }),
-                    1: lambda y: y.cols(0).rows(0).structSize(1).members({
+                    1: lambda y: y.cols(0).rows(0).members({
                         'a': lambda z: z.cols(3).rows(1),
+                        'b': lambda z: z.cols(1).rows(1),
                     }),
-                    2: lambda y: y.cols(0).rows(0).structSize(1).members({
+                    2: lambda y: y.cols(0).rows(0).members({
                         'a': lambda z: z.cols(3).rows(1),
+                        'b': lambda z: z.cols(1).rows(1),
                     }),
-                    3: lambda y: y.cols(0).rows(0).structSize(1).members({
+                    3: lambda y: y.cols(0).rows(0).members({
                         'a': lambda z: z.cols(3).rows(1),
+                        'b': lambda z: z.cols(1).rows(1),
                     }),
                 }),
             }),
             # G[1]
             1: lambda s: s.cols(0).rows(0).structSize(3).members({
-                'a': lambda x: x.cols(0).rows(0).structSize(1).members({
+                'a': lambda x: x.cols(0).rows(0).members({
+                    'a': lambda y: y.cols(3).rows(1).value([1040.0, 1050.0, 1060.0]),
                     'b': lambda y: y.cols(1).rows(1).value([1070.0]),
                 }),
                 'b': lambda x: x.cols(0).rows(0).arraySize(4).members({
@@ -466,17 +535,21 @@ class GL_CBuffer_Zoo(rdtest.TestCase):
                     3: lambda y: y.cols(4).rows(1).value([1200.0, 1210.0, 1220.0, 1230.0]),
                 }),
                 'c': lambda x: x.cols(0).rows(0).arraySize(4).members({
-                    0: lambda y: y.cols(0).rows(0).structSize(1).members({
+                    0: lambda y: y.cols(0).rows(0).members({
                         'a': lambda z: z.cols(3).rows(1),
+                        'b': lambda z: z.cols(1).rows(1),
                     }),
-                    1: lambda y: y.cols(0).rows(0).structSize(1).members({
+                    1: lambda y: y.cols(0).rows(0).members({
                         'a': lambda z: z.cols(3).rows(1),
+                        'b': lambda z: z.cols(1).rows(1),
                     }),
-                    2: lambda y: y.cols(0).rows(0).structSize(1).members({
+                    2: lambda y: y.cols(0).rows(0).members({
                         'a': lambda z: z.cols(3).rows(1),
+                        'b': lambda z: z.cols(1).rows(1),
                     }),
-                    3: lambda y: y.cols(0).rows(0).structSize(1).members({
+                    3: lambda y: y.cols(0).rows(0).members({
                         'a': lambda z: z.cols(3).rows(1).value([1360.0, 1370.0, 1380.0]),
+                        'b': lambda z: z.cols(1).rows(1),
                     }),
                 }),
             }),
@@ -485,5 +558,3 @@ class GL_CBuffer_Zoo(rdtest.TestCase):
         var_check.done()
 
         rdtest.log.success("Bare uniform variables are as expected")
-
-        out.Shutdown()
