@@ -121,7 +121,10 @@ class Iter_Test(rdtest.TestCase):
         target = rd.ResourceId.Null()
 
         if len(pipe.GetOutputTargets()) > 0:
-            target = pipe.GetOutputTargets()[0].resourceId
+            valid_targets = [o.resourceId for o in pipe.GetOutputTargets() if o.resourceId != rd.ResourceId.Null()]
+            rdtest.log.print("Valid targets at {} are {}".format(draw.eventId, valid_targets))
+            if len(valid_targets) > 0:
+                target = valid_targets[int(random.random()*len(valid_targets))]
 
         if target == rd.ResourceId.Null():
             target = pipe.GetDepthTarget().resourceId
@@ -132,7 +135,7 @@ class Iter_Test(rdtest.TestCase):
 
         rdtest.log.print("Fetching history for %d,%d on target %s" % (x, y, str(target)))
 
-        history = self.controller.PixelHistory(target, x, y, 0, 0, 0xffffffff, rd.CompType.Typeless)
+        history = self.controller.PixelHistory(target, x, y, rd.Subresource(0, 0, 0), rd.CompType.Typeless)
 
         rdtest.log.success("Pixel %d,%d has %d history events" % (x, y, len(history)))
 
@@ -153,26 +156,39 @@ class Iter_Test(rdtest.TestCase):
 
             if mod.sampleMasked or mod.backfaceCulled or mod.depthClipped or mod.viewClipped or mod.scissorClipped or mod.depthTestFailed or mod.stencilTestFailed:
                 rdtest.log.print("This hit failed, looking for one that passed....")
+                lastmod = None
                 continue
 
             break
 
         if lastmod is not None:
-            rdtest.log.print("Debugging pixel {},{} @ {}".format(x, y, lastmod.eventId))
+            rdtest.log.print("Debugging pixel {},{} @ {}, primitive {}".format(x, y, lastmod.eventId, lastmod.primitiveID))
             self.controller.SetFrameEvent(lastmod.eventId, True)
-            trace = self.controller.DebugPixel(x, y, 0xffffffff, 0xffffffff)
 
-            rdtest.log.success('Successfully debugged pixel in {} cycles'.format(len(trace.states)))
+            trace = self.controller.DebugPixel(x, y, 0, lastmod.primitiveID)
+
+            if draw.outputs[0] == rd.ResourceId.Null():
+                rdtest.log.success('Successfully debugged pixel in {} cycles, skipping result check due to no output'.format(len(trace.states)))
+            elif draw.numInstances == 1:
+                lastState: rd.ShaderDebugState = trace.states[-1]
+
+                output_index = [o.resourceId for o in self.controller.GetPipelineState().GetOutputTargets()].index(target)
+                rdtest.log.print("At event {} the target is index {}".format(lastmod.eventId, output_index))
+
+                debugged: rd.ShaderVariable = lastState.outputs[output_index]
+
+                debuggedValue = [debugged.value.f.x, debugged.value.f.y, debugged.value.f.z, debugged.value.f.w]
+
+                if not rdtest.value_compare(lastmod.shaderOut.col.floatValue, [debugged.value.f.x, debugged.value.f.y, debugged.value.f.z, debugged.value.f.w]):
+                    raise rdtest.TestFailureException("Debugged value {}: {} doesn't match history shader output {}".format(debugged.name, debuggedValue, lastmod.shaderOut.col.floatValue))
+
+                rdtest.log.success('Successfully debugged pixel in {} cycles, result matches'.format(len(trace.states)))
+            else:
+                rdtest.log.success('Successfully debugged pixel in {} cycles, skipping result check due to instancing'.format(len(trace.states)))
 
             self.controller.SetFrameEvent(draw.eventId, True)
 
-    def iter_test(self, path):
-        try:
-            self.controller = rdtest.open_capture(path)
-        except RuntimeError as err:
-            rdtest.log.print("Skipping. Can't open {}: {}".format(path, err))
-            return
-
+    def iter_test(self):
         # Handy tweaks when running locally to disable certain things
 
         action_chance = 0.1     # Chance of doing anything at all
@@ -181,9 +197,9 @@ class Iter_Test(rdtest.TestCase):
         do_pixel_debug = 1.0    # Chance of doing pixel history at the current event and debugging a pixel (if valid)
 
         actions = {
-            'Image Save': {'chance': 0.25, 'func': self.image_save},
-            'Vertex Debug': {'chance': 1.0, 'func': self.vert_debug},
-            'Pixel History & Debug': {'chance': 1.0, 'func': self.pixel_debug},
+            'Image Save': {'chance': do_image_save, 'func': self.image_save},
+            'Vertex Debug': {'chance': do_vert_debug, 'func': self.vert_debug},
+            'Pixel History & Debug': {'chance': do_pixel_debug, 'func': self.pixel_debug},
         }
 
         # To choose an action, if we're going to do one, we take random in range(0, choice_max) then check each action
@@ -217,8 +233,6 @@ class Iter_Test(rdtest.TestCase):
 
             draw = draw.next
 
-        self.controller.Shutdown()
-
     def run(self):
         dir_path = self.get_ref_path('', extra=True)
 
@@ -231,8 +245,27 @@ class Iter_Test(rdtest.TestCase):
 
             rdtest.log.print('Iterating {}'.format(file.name))
 
-            self.iter_test(file.path)
+            try:
+                self.controller = rdtest.open_capture(file.path)
+            except RuntimeError as err:
+                rdtest.log.print("Skipping. Can't open {}: {}".format(file.path, err))
+                continue
+
+            self.iter_test()
+
+            self.controller.Shutdown()
 
             rdtest.log.success("Iterated {}".format(file.name))
 
         rdtest.log.success("Iterated all files")
+
+    # Useful for calling from within the UI
+    def run_external(self, controller: rd.ReplayController):
+        self.controller = controller
+
+        self.iter_test()
+
+
+def run_locally(r):
+    test = Iter_Test()
+    test.run_external(r)
\ No newline at end of file
