@@ -3,18 +3,44 @@ import renderdoc as rd
 
 
 class VK_Overlay_Test(rdtest.TestCase):
-    def get_capture(self):
-        return rdtest.run_and_capture("demos_x64", "VK_Overlay_Test", 5)
+    demos_test_name = 'VK_Overlay_Test'
+    
+    def check_clearbeforedraw_depth(self, out, depthId):
+        # Test ClearBeforeDraw with a depth target
+        tex = rd.TextureDisplay()
+        tex.overlay = rd.DebugOverlay.ClearBeforeDraw
+        tex.resourceId = depthId
+
+        out.SetTextureDisplay(tex)
+        out.GetDebugOverlayTexID() # Called to refresh the overlay
+        
+        overlay = rd.DebugOverlay.ClearBeforeDraw
+        test_name = str(overlay) + '.Depth'
+
+        overlay_path = rdtest.get_tmp_path(test_name + '.png')
+        ref_path = self.get_ref_path(test_name + '.png')
+
+        save_data = rd.TextureSave()
+        save_data.resourceId = depthId
+        save_data.destType = rd.FileType.PNG
+        save_data.channelExtract = 0
+
+        tolerance = 2
+
+        self.controller.SaveTexture(save_data, overlay_path)
+
+        if not rdtest.png_compare(overlay_path, ref_path, tolerance):
+            raise rdtest.TestFailureException("Reference and output image differ for overlay {}".format(test_name), overlay_path, ref_path)
+
+        rdtest.log.success("Reference and output image are identical for {}".format(test_name))
 
     def check_capture(self):
         self.check_final_backbuffer()
 
-        out: rd.ReplayOutput = self.controller.CreateOutput(rd.CreateHeadlessWindowingData(), rd.ReplayOutputType.Texture)
+        out: rd.ReplayOutput = self.controller.CreateOutput(rd.CreateHeadlessWindowingData(100, 100), rd.ReplayOutputType.Texture)
 
         self.check(out is not None)
 
-        out.SetDimensions(100, 100)
-
         test_marker: rd.DrawcallDescription = self.find_draw("Test")
 
         self.controller.SetFrameEvent(test_marker.next.eventId, True)
@@ -94,4 +120,6 @@ class VK_Overlay_Test(rdtest.TestCase):
 
         rdtest.log.success("Reference and output image are identical for stencil")
 
+        self.check_clearbeforedraw_depth(out, pipe.GetDepthTarget().resourceId)
+
         out.Shutdown()
