@@ -1,18 +1,18 @@
 #!/bin/sh
 
-# ignore "error" codes in the env script below
-set +e
-
-. /opt/qt56/bin/qt56-env.sh 
-
 set -e
 
-# Switch to the gcc version we want
+# Switch to the gcc/clang version we want
 if [ $CC == "gcc" ]; then
-	export CC=gcc-6;
-	export CXX=g++-6;
+	export CC=gcc-5;
+	export CXX=g++-5;
+else
+	export CC=clang-3.8;
+	export CXX=clang++-3.8;
 fi
 
+export QT_SELECT=qt5
+
 mkdir build
 cd build
 if [[ "$RELEASE_BUILD" == "1" ]]; then
